"use client";

    import { useState, useEffect, useRef, useCallback, useMemo } from "react";
import Stomp from "stompjs";
import SockJS from "sockjs-client";
import { ILiveComment } from "@/shared/types/live";
import { useGetCommentsLive } from "@/shared/api/live";
import { LiveCommentsContext } from "./live-comments-context";
import { WEB_SOCKET_URL } from "@/config-global";

type Props = {
    children: React.ReactNode;
    liveId: string;
};

const MAX_COMMENTS = 100;
const MAX_PINNED_COMMENTS = 3;
const MAX_RECONNECT_ATTEMPTS = 5;
const INITIAL_RECONNECT_DELAY = 1000;

export const LiveCommentsProvider = ({ children, liveId }: Props) => {
    const [comments, setComments] = useState<ILiveComment[]>([]);
    const [pinnedComments, setPinnedComments] = useState<ILiveComment[]>([]);

    
    const { commentsData } = useGetCommentsLive(liveId);
    
    const clientRef = useRef<Stomp.Client | null>(null);
    const reconnectAttempts = useRef(0);
    const reconnectDelay = useRef(INITIAL_RECONNECT_DELAY);
    const reconnectTimeoutRef = useRef<NodeJS.Timeout | null>(null);
    const subscriptionRef = useRef<{ unsubscribe: () => void } | null>(null);
    const mountedRef = useRef(true);

    useEffect(() => {
        console.log("commentsData");
    }, [commentsData]);

    // Memoize initial comments processing
    const initialCommentsData = useMemo(() => {

        if (!commentsData) return { initialComments: [], initialPinned: [] };

        const initialComments: ILiveComment[] = [];
        const initialPinned: ILiveComment[] = [];

        for (const comment of commentsData) {
            comment.pinned ? initialPinned.push(comment) : initialComments.push(comment);
        }

        return {
            initialComments: initialComments.slice(-MAX_COMMENTS),
            initialPinned,
        };
    }, [commentsData]);

    // Initialize state with memoized data
    useEffect(() => {
        setComments(initialCommentsData.initialComments);
        setPinnedComments(initialCommentsData.initialPinned);
    }, [initialCommentsData]);

    // WebSocket message handler
    const handleMessage = useCallback((message: Stomp.Message) => {
        const comment: ILiveComment = JSON.parse(message.body);
        switch (comment.type) {
            case "DELETE":
                setComments(prev => prev.filter(c => c.id != comment.id));
                setPinnedComments(prev => prev.filter(c => c.id != comment.id));
                break;
            case "CHAT":
                setComments(prev => [...prev, comment].slice(-MAX_COMMENTS));
                break;
            case "PIN":
                setComments((prevComments) => {
                    const commentToPin = prevComments.find((c) => c.id === comment.id);
                    if (commentToPin) {
                        setPinnedComments((prevPinned) => [...prevPinned, { ...commentToPin, pinned: true }].slice(-MAX_PINNED_COMMENTS));
                        return prevComments.filter((c) => c.id !== comment.id).slice(-MAX_COMMENTS);
                    }
                    return prevComments.slice(-MAX_COMMENTS);
                });
                break;
            case "UNPIN":
                setPinnedComments((prevPinned) => {
                    const commentToUnpin = prevPinned.find((c) => c.id == comment.id);
                    if (commentToUnpin) {
                        setComments((prevComments) => [...prevComments, { ...commentToUnpin, pinned: false }]);
                        return prevPinned.filter((c) => c.id != comment.id);
                    }
                    return prevPinned;
                });
                break;
            default:
                break
                
        }
        
    }, []);

    // WebSocket management
    const connect = useCallback(() => {
        if (!mountedRef.current || clientRef.current?.connected) return;

        const socket = new SockJS(WEB_SOCKET_URL || "http://localhost:8082/ws");
        const client = Stomp.over(socket);
        clientRef.current = client;

        client.connect({}, () => {
            if (!mountedRef.current) return;
            reconnectAttempts.current = 0;
            reconnectDelay.current = INITIAL_RECONNECT_DELAY;
            
            subscriptionRef.current = client.subscribe(
                `/topic/messages/${liveId}`,
                handleMessage
            );
        }, () => {
            clientRef.current = null;
            handleDisconnect();
        });

        socket.onclose = () => {
            clientRef.current = null;
            handleDisconnect();
        };
    }, [liveId, handleMessage]);

    const handleDisconnect = useCallback(() => {
        if (!mountedRef.current || reconnectAttempts.current >= MAX_RECONNECT_ATTEMPTS) return;

        const delay = reconnectDelay.current;
        reconnectTimeoutRef.current = setTimeout(() => {
            if (!mountedRef.current) return;
            reconnectAttempts.current += 1;
            reconnectDelay.current *= 2;
            connect();
        }, delay);
    }, [connect]);

    // Lifecycle management
    useEffect(() => {
        mountedRef.current = true;
        connect();

        return () => {
            mountedRef.current = false;
            
            if (reconnectTimeoutRef.current) {
                clearTimeout(reconnectTimeoutRef.current);
            }
            
            if (clientRef.current?.connected) {
                subscriptionRef.current?.unsubscribe();
                clientRef.current.disconnect(() => {
                    console.log("Disconnected from WebSocket");
                });
            }
            
            clientRef.current = null;
            reconnectAttempts.current = 0;
            reconnectDelay.current = INITIAL_RECONNECT_DELAY;
        };
    }, [connect]);

    return (
        <LiveCommentsContext.Provider value={{
            comments,
            pinnedComments
        }}>
            {children}
        </LiveCommentsContext.Provider>
    );
};