"use client";

import { useState, useEffect, useReducer } from "react";
import { LiveProductsContext } from "./live-products-context";
import { ILiveProduct, LiveProductStatus } from "@/shared/types/live";

import { addProductToLive, presentProductLive, hideProductLive, getProductsLive, deleteProductLive, pushProductLive, addTimeStampProduct, deleteTimeStampProduct, updateTimeStampProduct } from "@/shared/api/live";
// ----------------------------------------------------------------------

type Props = {
    children: React.ReactNode;
    liveId: string;
};


export const LiveProductsProvider = ({ children, liveId }: Props) => {
    const [products, setProducts] = useState<ILiveProduct[]>([]);
    const [productsLoading, setProductsLoading] = useState<boolean>(true);
    const [productsError, setProductsError] = useState<string | null>(null);


    useEffect(() => {
        getProductsLive(liveId)
            .then((data) => {
                setProducts(data);
            })
            .catch((error) => {
                setProductsError(error);

            })
            .finally(() => {
                setProductsLoading(false);
            });

    }, [liveId]);

    const addProductToLiveFunction = (product: ILiveProduct, setLoading?: (loading: boolean) => void) => {
        setLoading && setLoading(true);
        addProductToLive(liveId, product)
            .then((data) => {
                setProducts(prevProducts => [...prevProducts, {...data, status: LiveProductStatus.NOT_PRESENTED, timeStampProduct: []}]);
            })
            .catch((error) => {
                setProductsError(error);
            })
            .finally(() => {
                setLoading && setLoading(false);
            });
    }


    const deleteProductFromLiveFunction = (productId: string, setLoading?: (loading: boolean) => void) => {
        setLoading && setLoading(true);
        deleteProductLive(productId,liveId)
            .then((data) => {
                setProducts(prevProducts => prevProducts.filter(product => product.id != productId));
            })
            .catch((error) => {
                setProductsError(error);
            })
            .finally(() => {
                setLoading && setLoading(false);
            });
    }

    const presentProductInLiveFunction = (productId: string, liveId: string, startTime: number, setLoading?: (loading: boolean) => void) => {
        setLoading && setLoading(true);
        presentProductLive(productId, liveId, startTime)
            .then((data) => {
                setProducts(prevProducts => prevProducts.map(product => product.id == productId ? { ...product, isPresent: true, status: LiveProductStatus.IN_PRESENTATION } : product));
            })
            .catch((error) => {
                setProductsError(error);
            })
            .finally(() => {
                setLoading && setLoading(false);
            });
    }

    const pushProductToLiveFunction = (productId: string, liveId: string, startTime: number, setLoading?: (loading: boolean) => void) => {
        setLoading && setLoading(true);
        pushProductLive(productId, startTime)
            .then((data) => {
                setProducts(prevProducts => {
                    const index = prevProducts.findIndex((product) => product.id === productId);
                    if (index > -1 && index < prevProducts.length) {
                        prevProducts[index] = { ...prevProducts[index], timeStampProduct: [...prevProducts[index].timeStampProduct, data], status: LiveProductStatus.IN_PRESENTATION };
                    }
                    return [...prevProducts];
                });
            })
            .catch((error) => {
                setProductsError(error);
            })
            .finally(() => {
                setLoading && setLoading(false);
            });
    }

    const hideProductFromLiveFunction = (productId: string , endTime: number, setLoading?: (loading: boolean) => void) => {
        setLoading && setLoading(true);
        hideProductLive(productId, endTime)
            .then((data) => {
                setProducts(prevProducts => {
                    const productIndex = prevProducts.findIndex((product) => product.id === productId);
                    if (productIndex > -1 && productIndex < prevProducts.length) {
                        prevProducts[productIndex].status = LiveProductStatus.PRESENTED
                        const timeStampIndex = prevProducts[productIndex].timeStampProduct.findIndex((timeStamp) => timeStamp.id === data.id);
                        if (timeStampIndex > -1) {
                            prevProducts[productIndex].timeStampProduct[timeStampIndex] = data;
                        } else {
                            prevProducts[productIndex].timeStampProduct.push(data);
                        }

                    }
                    return [...prevProducts];
                });
            })
            .catch((error) => {
                setProductsError(error);
            })
            .finally(() => {
                setLoading && setLoading(false);
            });
    }


    const addTimeStampProductFunction = (productId: string, startTime: number, endTime: number, setLoading?: (loading: boolean) => void) => {
        setLoading && setLoading(true);
        addTimeStampProduct(productId, startTime, endTime)
            .then((data) => {
                setProducts(prevProducts => {
                    const index = prevProducts.findIndex((product) => product.id === productId);
                    if (index > -1 && index < prevProducts.length) {
                        const indexTimeStamp = prevProducts[index].timeStampProduct.findIndex((timeStamp) => timeStamp.id === data.id);
                        if (indexTimeStamp > -1) {
                            prevProducts[index].timeStampProduct[indexTimeStamp] = data;
                        } else {
                            prevProducts[index].timeStampProduct.push(data);
                        }
                    }
                    return [...prevProducts];
                });
            })
            .catch((error) => {
                setProductsError(error);
            })
            .finally(() => {
                setLoading && setLoading(false);
            });
    }

    const deleteTimeStampProductFunction = (productId: string, timeStampId: string, setLoading?: (loading: boolean) => void) => {
        setLoading && setLoading(true);
        deleteTimeStampProduct(timeStampId)
            .then((data) => {
                setProducts(prevProducts => prevProducts.map(product => product.id == productId ? { ...product, timeStampProduct: product.timeStampProduct.filter(timeStamp => timeStamp.id !== timeStampId) } : product));
            })
            .catch((error) => {
                setProductsError(error);
            })
            .finally(() => {
                setLoading && setLoading(false);
            });
    }

    const updateTimeStampProductFunction = (productId: string, id: string, startTime: number, endTime: number, setLoading?: (loading: boolean) => void) => {
        setLoading && setLoading(true);
        updateTimeStampProduct(id, startTime, endTime)
            .then((data) => {
                setProducts(prevProducts => prevProducts.map(product => product.id == productId ? { ...product, ...product } : product));
            })
            .catch((error) => {
                setProductsError(error);
            })
            .finally(() => {
                setLoading && setLoading(false);
            });
    }







    return <LiveProductsContext.Provider value={{ products, productsLoading, productsError, addProductToLiveFunction, deleteProductFromLiveFunction, presentProductInLiveFunction, pushProductToLiveFunction, hideProductFromLiveFunction, addTimeStampProductFunction, deleteTimeStampProductFunction, updateTimeStampProductFunction }}>
        {children}
    </LiveProductsContext.Provider>;
};
