import useSWR, { mutate } from "swr";
import { useMemo } from "react";
import axiosInstance, { endpoints, fetcher } from "./server";
import axios, { AxiosRequestConfig, AxiosResponse } from "axios";
import { IAttribute, ITerm } from "../types/attribut";
import { IProductSessionItem } from "../types/product";

const options = {
  revalidateIfStale: true,
  revalidateOnFocus: true,
  revalidateOnReconnect: true,
  revalidateOnMount: true,
};

////////////-----Attribute API-----//////////////
////////////////////////////////////////////

export function useGetAttributs() {
  const URL = [endpoints.stock_management.Attributs.getAll()];
  const { data, error, isLoading, isValidating, mutate } = useSWR<IAttribute[]>(
    URL,
    fetcher
  );

  const memoizedValue = useMemo(
    () => ({
      AttributesData: (data as IAttribute[]) || [],
      AttributesLoading: isLoading,
      AttributesError: error,
      AttributesValidating: isValidating,
      AttributesMutate: mutate,
    }),

    [data, error]
  );
  return memoizedValue;
}

export function useGetAttribute(attributId: string) {
  const URL = [endpoints.stock_management.Attributs.getById(attributId)];
  const { data, error, isLoading, isValidating } = useSWR<IAttribute>(
    URL,
    fetcher
  );

  const memoizedValue = useMemo(
    () => ({
      attributData: (data as IAttribute) || null,
      attributLoading: isLoading,
      attributError: error,
      attributValidating: isValidating,
    }),

    [data, error]
  );
  return memoizedValue;
}

export async function useAddAttribute(attribute: IAttribute) {
  try {
    const URL = endpoints.stock_management.Attributs.add();
    const response: AxiosResponse<IAttribute> = await axiosInstance.post(
      URL,
      attribute
    );
    return response;
  } catch (error) {
    console.error("Erreur lors de l'ajout de l'attribue:", error);
    throw error;
  }
}

export async function useUpdateAttribut(
  attribute: IAttribute,
  attributId: string
) {
  try {
    const URL = endpoints.stock_management.Attributs.edit(attributId);
    const response: AxiosResponse<IAttribute> = await axiosInstance.put(
      URL,
      attribute
    );
    return response;
  } catch (error) {
    console.error("Erreur lors de la mise à jour de l'attribue:", error);
    throw error;
  }
}

export async function useDeleteAttribut(attributId: string) {
  try {
    const URL = endpoints.stock_management.Attributs.delete(attributId);
    const response = await axiosInstance.delete(URL);
    return response;
  } catch (error) {
    console.error("Erreur lors de la suppression de l'attribue:", error);
    throw error;
  }
}

export async function useDeleteAttributs(attributIds: string[]) {
  try {
    const URL = endpoints.stock_management.Attributs.deleteBunch();
    const response = await axiosInstance.delete(URL, {
      data: attributIds,
    });
    return response;
  } catch (error) {
    console.error("Erreur lors de la suppression des attribues:", error);
    throw error;
  }
}
/////////////////////// Get Terms by Attribute /////////////////////////

export function useGetTermsByAttributeId(attributeId: string) {
  const URL = `/api/stock/attributs/${attributeId}/terms`;
  const { data, error, isLoading, isValidating } = useSWR<ITerm[]>(
    URL,
    fetcher
  );

  const memoizedValue = useMemo(
    () => ({
      termsData: (data || []).map((term) => ({
        id: term.id,
        nom: term.nom,
        name: term.name,
        slug: term.slug,
      })),
      termsLoading: isLoading,
      termsError: error,
      termsValidating: isValidating,
    }),
    [data, error]
  );

  return memoizedValue;
}

///////////////////// Get Product with variation /////////////////

export function useAllGetProductsWithVariation() {
  const URL = "/api/stock/products/ProductWithvariations";

  const { data, isLoading, error, isValidating } = useSWR<
    IProductSessionItem[]
  >(URL, fetcher, {
    revalidateOnFocus: false,
    revalidateOnReconnect: false,
  });

  const memoizedValue = useMemo(
    () => ({
      products: data || [],
      productsLoading: isLoading,
      productsError: error,
      productsValidating: isValidating,
      productsEmpty: !isLoading && (!data || data.length === 0),
      revalidate: () => mutate(URL), // Allow manual revalidation
    }),
    [data, error, isLoading, isValidating]
  );

  return memoizedValue;
}
