import { endpoints, axiosInstance } from "./server";
import useSWR , { mutate } from 'swr';
import { ICartClientPage, IItemCart, SearchClientCartParams} from "../types/cart";
import { CartCountConfig } from "../types/CartCountConfig";


export default axiosInstance;

export const useGetCart = (clientId: string) => {
  const { data, error, isLoading, mutate } = useSWR(
    clientId ? endpoints.cart.getCart(clientId) : null,
    fetcher
  );

  return {
    cart: data,
    isLoading,
    error,
    mutate,
  };
};


export const useDeleteCartItem = async (clientId: string, itemCartId: string) => {
  try {
    await axiosInstance.delete(endpoints.cart.deleteItem, {
      params: { clientId, itemCartId },
    });

    await mutate(endpoints.cart.getCart(clientId), (currentCart) => {
      if (!currentCart) return null; 
    
      const updatedItems = currentCart.items.filter((item: IItemCart) => item.id.toString() !== itemCartId);
    
      const updatedCart = {
        ...currentCart,
        items: updatedItems,
        totalPrice: updatedItems.reduce((acc: number, item: IItemCart) => acc + item.totalPrice, 0),
      };
    
      if (updatedCart.items.length === 0) {
        updatedCart.totalPrice = 0;
      }
    
      return updatedCart;
    }, false);
    

    return true;
  } catch (error) {
    console.error('Erreur lors de la suppression de l\'élément du panier :', error);
    throw error;
  }
};

const fetcher = (url: string) => axiosInstance.get(url).then((res) => res.data);

export const saveCartCountConfig = async (config: CartCountConfig) => {
  try {
    const response = await axiosInstance.post(endpoints.cart.saveOrUpdate, config);
    return response.data;
  } catch (error) {
    console.error('Erreur lors de l\'enregistrement de la configuration du compte du panier :', error);
    throw error;
  }
};

export const getCartCountConfig = async (): Promise<CartCountConfig> => {
  try {
    const response = await axiosInstance.get('/api/cart/cart-count-config');
    return response.data;
  } catch (error) {
    console.error('Erreur lors de la récupération de la configuration du compte du panier :', error);
    throw error;
  }
};

export function fetchPaginatedClientCarts({
  keyword = '',
  productKeyword='',
  page = 0,
  pageSize = 10,
}: SearchClientCartParams) {
  const filterParams = new URLSearchParams({
    keyword: keyword || '',
    productKeyword:productKeyword|| '',
    page: page.toString(),
    pageSize: pageSize.toString(),
  });

  const url = `${endpoints.cart.getClientCarts}?${filterParams.toString()}`;
  const { data, error, isValidating, mutate } = useSWR<ICartClientPage>(url, fetcher);

  return {
    carts: data?.content || [],              
    totalCarts: data?.totalElements || 0,    
    totalPages: data?.totalPages || 0,       
    pageNumber: data?.pageable.pageNumber || 0, 
    pageSize: data?.pageable.pageSize || 0,  
    loading: !data && !error,                
    isError: error,                         
    isValidating,                           
    revalidate: mutate,                     
  };

}
