import useSWR from "swr";
import { useMemo } from "react";
import axiosInstance, { endpoints, fetcher } from "./server";
import axios, { AxiosResponse } from "axios";
import { ICategory } from "../types/category";



const options = {
  revalidateIfStale: true,
  revalidateOnFocus: true,
  revalidateOnReconnect: true,
  revalidateOnMount: true,
};

////////////-----category API-----//////////////
////////////////////////////////////////////

export function useGetCategories() {
  const URL = [endpoints.stock_management.categories.getAll()];
  const { data, error, isLoading, isValidating, mutate } = useSWR<ICategory[]>(
    URL,
    fetcher
  );

  const memoizedValue = useMemo(
    () => ({
      categoriesData: (data as ICategory[]) || [],
      categoriesLoading: isLoading,
      categoriesError: error,
      categoriesValidating: isValidating,
      mutateCategories: mutate,
    }),

    [data, error]
  );
  return memoizedValue;
}

export function useGetCategory(categoryId: string) {
  const URL = endpoints.stock_management.categories.getById(categoryId);
  const { data, isLoading, isValidating, error, mutate } = useSWR<ICategory>(
    URL,
    fetcher
  );

  const memoizedValue = useMemo(
    () => ({
      categoryData: (data as ICategory) || null,
      categoryLoading: isLoading,
      categoryError: error,
      categoryValidating: isValidating,
      categoryMutate: mutate,
    }),

    [data, error]
  );
  return memoizedValue;
}

export async function useAddCategory(category: ICategory) {
  try {
    const URL = endpoints.stock_management.categories.add();
    const response: AxiosResponse<ICategory> = await axiosInstance.post(
      URL,
      category
    );
  } catch (error) {
    console.error("Erreur lors de l'ajout de la catégorie:", error);
    throw error;
  }
}


export async function useUpdateCategory(
  category: ICategory,
  categoryId: string
) {
  try {
    const URL = endpoints.stock_management.categories.edit(categoryId);
    const response: AxiosResponse<ICategory> = await axiosInstance.put(
      URL,
      category
    );

    return response;
  } catch (error) {
    console.error("Erreur lors de la mise à jour de la catégorie:", error);
    throw error;
  }
}

export async function useDeleteCategory(categoryId: string) {
  try {
    const URL = endpoints.stock_management.categories.delete(categoryId);
    const response = await axiosInstance.delete(URL);
    return response;
  } catch (error) {
    console.error("Erreur lors de la suppression de la catégorie:", error);
    throw error;
  }
}

export async function useDeleteCategories(categoryIds: string[]) {
  try {
    const URL = endpoints.stock_management.categories.deleteBunch();
    const response = await axiosInstance.delete(URL, {
      data: categoryIds,
    });
    return response;
  } catch (error) {
    console.error("Erreur lors de la suppression des catégories:", error);
    throw error;
  }
}
export async function useImportCategories(file: File): Promise<Map<string, boolean>> {
  try {
    const URL = endpoints.stock_management.categories.importCategory();
    const formData = new FormData();
    formData.append("file", file);
    const response: AxiosResponse<Map<string, boolean>> = await axiosInstance.post(URL, formData, {
      headers: {
        "Content-Type": "multipart/form-data",
      },
    });
    return response.data;
  } catch (error) {
    console.error("Erreur lors de l'importation des catégories:", error);
    throw error;
  }
}

export async function useDeleteAfterUpdate(categoryId: string , defaultCategoryId: string) {
  try {
    const URL = endpoints.stock_management.categories.deleteAfterUpdate(categoryId, defaultCategoryId);
    const response = await axiosInstance.delete(URL);
    return response;
  } catch (error) {
    console.error("Erreur lors de la suppression de la catégorie:", error);
    throw error;
  }
}
