import { IChatConversations } from '../types/chat-bot';
import { useMemo } from 'react';
import useSWR, { mutate } from 'swr';

import { ITiketItem } from '../types/ticket';
import axios, { fetcherchatbot , endpoints } from './server';
import { isValidToken } from '@/utils/token';

// ----------------------------------------------------------------------

const swrOptions = {
  revalidateIfStale: true,     
  revalidateOnFocus: false,    
  revalidateOnReconnect: true, 
};


// ----------------------------------------------------------------------

type ConversationsData = {
  conversations: IChatConversations[];
};

export function useGetConversations(supportId: string) {
  const url = endpoints.chatbot.chat.conversations(supportId);

  const { data, isLoading, error, isValidating } = useSWR<ConversationsData>(url,fetcherchatbot,swrOptions);

  const memoizedValue = useMemo(
    () => ({
      conversations: data?.conversations,
      conversationsLoading: isLoading,
      conversationsError: error,
      conversationsValidating: isValidating,
      conversationsEmpty: !isLoading && !data?.conversations.length,
      refetchConversations: () => mutate(url),
    }),
    [data?.conversations, error, isLoading, isValidating]
  );

  return memoizedValue;
}

// ----------------------------------------------------------------------
export function useGetConversation(conversationId: string) {
  const url = endpoints.chatbot.chat.conversation(conversationId);

  const { data, isLoading, error, isValidating } = useSWR<IChatConversations>(url,fetcherchatbot,swrOptions);

  const memoizedValue = useMemo(
    () => ({
      conversation: data,
      conversationLoading: isLoading,
      conversationError: error,
      conversationValidating: isValidating,
      conversationEmpty: !isLoading && !data,
      refetchConversation: () => mutate(url),
    }),
    [data, error, isLoading, isValidating]
  );

  return memoizedValue;
}

// ---------------------------------------------------------------------
export function useGetTicketByConversation(conversationId: string) {
  const url = endpoints.chatbot.chat.ticket(conversationId);

  const { data, isLoading, error, isValidating } = useSWR<ITiketItem>(url,fetcherchatbot,swrOptions);

  const memoizedValue = useMemo(
    () => ({
      ticket: data,
      ticketLoading: isLoading,
      ticketError: error,
      ticketValidating: isValidating,
      ticketEmpty: !isLoading && !data,
      refetchTicket: () => mutate(url),
    }),
    [data, error, isLoading, isValidating]
  );

  return memoizedValue;
}

// ----------------------------------------------------------------------

export function useGetAllUsers(shouldFetch = false) {
  const { data, error, isLoading } = useSWR(
    shouldFetch ? endpoints.chatbot.user.list : null, 
    fetcherchatbot,
    swrOptions
  );
  return {
    users: data || [],
    usersLoading: isLoading,
    usersError: error,
  };
}

// ----------------------------------------------------------------------
export function useGetAllConversation() {
  const url = endpoints.chatbot.chat.all;

  const { data, isLoading, error, isValidating } = useSWR(url,fetcherchatbot,swrOptions);

  const memoizedValue = useMemo(
    () => ({
      conversations: data?.conversations,
      conversationsLoading: isLoading,
      conversationsError: error,
      conversationsValidating: isValidating,
      conversationsEmpty: !isLoading && !data?.conversations.length,
      refetchConversations: () => mutate(url),
    }),
    [data, error, isLoading, isValidating]
  );

  return memoizedValue;
}

// ----------------------------------------------------------------------
export function useConversations(userIdToFetch:string) {
  const url = userIdToFetch 
    ? endpoints.chatbot.chat.conversations(userIdToFetch)
    : endpoints.chatbot.chat.all;
  const { data, isLoading, error, mutate } = useSWR(url, fetcherchatbot, swrOptions);

  return {
    conversations: data?.conversations || [],
    conversationsLoading: isLoading,
    refetchConversations: mutate,
  };
}
// ----------------------------------------------------------------------
export const getCurrentChatUser = async (chatBotToken: string) => {
  try {
    if (!chatBotToken || !isValidToken(chatBotToken)) {
      console.error('Invalid token');
      return null;
    }
    
    const res = await axios.get(endpoints.chatbot.auth.currentUser, {
      headers: {
        Authorization: `Bearer ${chatBotToken}`
      }
    });

    return res.data;
  } catch (error) {
    console.error('Error getting current chatbot user:', error);
    return null;
  }
};
