import useSWR from 'swr';
import axiosInstance from './server'; 
import { ChartConfig } from '../types/statistics';
import { ChartData } from './saleSessionStatistics';




export type LiveDTO = {
  id: number;
  title: string;
};

function toIsoString(date: Date) {
  return date.toISOString().split('.')[0]; 
}


export function useGetLivesInRange(startDate?: Date, endDate?: Date) {
  const shouldFetch = !!startDate && !!endDate;

  const params: Record<string, string> = {};
  if (startDate) params.startDate = toIsoString(startDate);
  if (endDate) params.endDate = toIsoString(endDate);

  const { data, error } = useSWR<LiveDTO[]>(
    shouldFetch ? ['/api/stream/stats/live-by-date', params] : null,
    async ([url, params]) => {
      const res = await axiosInstance.get<LiveDTO[]>(url, { params });
      return res.data; 
    }
  );

  return {
    livesData: data,
    isLoading: !error && !data && shouldFetch,
    isError: error,
  };
}

  
export function useLivesCount(startDate?: Date, endDate?: Date) {
    const shouldFetch = !!startDate && !!endDate;
  

    const { data, error } = useSWR(
      shouldFetch ? ['/api/stream/stats/count-live-by-date', startDate, endDate] : null,
      async ([url, sDate, eDate]) => {
        const params = {
          startDate: toIsoString(sDate),
          endDate: toIsoString(eDate),
        };
        const response = await axiosInstance.get<number>(url, { params });
        return response.data; 
      }
    );
  
    return {
      livesCount: data,
      isLoading: !error && !data && shouldFetch,
      isError: error,
    };
}

function buildQueryParams(liveIds: number[]) {
    return liveIds.map((id) => `liveIds=${id}`).join('&');
  }
  
  export function useSumViewers(selectedLiveIds: number[]) {
    
    const shouldFetch = selectedLiveIds.length > 0;
    const queryString = buildQueryParams(selectedLiveIds);
  
    const { data, error } = useSWR<number>(
      shouldFetch ? `/api/stream/stats/viewers-sum?${queryString}` : null,
      (url: string) => axiosInstance.get<number>(url).then((r) => r.data)
    );
  
    return {
      sumViewers: data,                         
      isLoadingSumViewers: !error && !data && shouldFetch,
      isErrorSumViewers: !!error,
    };
}



export function useActiveClientsWithLiveItems(startDate?: Date, endDate?: Date) {
    const shouldFetch = !!startDate && !!endDate;
  

    const { data, error } = useSWR(
      shouldFetch ? ['/api/cart/statistics/active-clients/live', startDate, endDate] : null,
      async ([url, sDate, eDate]) => {
        const params = {
          startDate: toIsoString(sDate),
          endDate: toIsoString(eDate),
        };
        const response = await axiosInstance.get<number>(url, { params });
        return response.data; 
      }
    );
  
    return {
    activeClientsCount: data,
      isLoading: !error && !data && shouldFetch,
      isError: error,
    };
}

export function useActiveClientsCompare(startDate?: Date, endDate?: Date) {
    const shouldFetch = Boolean(startDate && endDate);
  
    const { data, error } = useSWR<ChartConfig>(
      shouldFetch ? ["activeClientsCompare", startDate, endDate] : null,
      async () => {
        const params = {
          startDate: toIsoString(startDate!),
          endDate: toIsoString(endDate!),
        };
        const res = await axiosInstance.get<ChartConfig>(
          "/api/cart/statistics/active-clients-compare",
          { params }
        );
        return res.data;
      }
    );
  
    return {
      data,                          
      isLoading: !error && !data && shouldFetch,
      isError: !!error,
    };
  }


  export const fetchBasketComparisonChartDataGrossRevenue = async (startDate: Date, endDate: Date, sources: string[]) => {
    const params = {
      startDate: startDate.toISOString(),
      endDate: endDate.toISOString(),
      sources: sources.join(","),
    };
  
    try {
      const response = await axiosInstance.get<ChartData>(
        '/api/orders/statistics/basket-comparison-chart-gross',
        { params }
      );
      return response.data;
    } catch (error) {
      console.error("Erreur lors de la récupération des données du graphique de comparaison du panier :", error);
      throw error;
    }
  };


  export function sumShippingPrice(startDate?: Date, endDate?: Date) {
    const shouldFetch = !!startDate && !!endDate;
  

    const { data, error } = useSWR(
      shouldFetch ? ['/api/orders/statistics/shipping-price/total', startDate, endDate] : null,
      async ([url, sDate, eDate]) => {
        const params = {
          startDate: toIsoString(sDate),
          endDate: toIsoString(eDate),
        };
        const response = await axiosInstance.get<number>(url, { params });
        return response.data; 
      }
    );
  
    return {
      sumShippingPriceData: data,
      isLoading: !error && !data && shouldFetch,
      isError: error,
    };
}


export function useGrossRevenueBySource(startDate?: Date, endDate?: Date, sources: string[] = []) {
  const shouldFetch = !!startDate && !!endDate;

  const { data, error } = useSWR(
    shouldFetch ? ['/api/orders/statistics/gross-revenue/by-source', startDate, endDate, sources] : null,
    async ([url, sDate, eDate, srcs]) => {
      const params = {
        startDate: toIsoString(sDate),
        endDate: toIsoString(eDate),
        sources: srcs.join(','),
      };
      const response = await axiosInstance.get(url, { params });
      return response.data; 
    }
  );

  return {
    grossRevenueBySourceData: data,
    isLoading: !error && !data && shouldFetch,
    isError: error,
  };
}