import type { IHelpArticleAddUpdate, IHelpCollectionAddUpdate, IHelpGetItem, IHelpItem } from '../types/help';
import useSWR, { mutate } from 'swr';
import { useMemo } from 'react';
import { fetcherchatbot , endpoints } from './server';

// ----------------------------------------------------------------------

export function extractCollection(data: { name: string; description: string; articles: { question: string; answer: string }[] }) {
  const collection: IHelpCollectionAddUpdate = {
    name: data.name,
    description: data.description,
  };

  const articles: IHelpArticleAddUpdate[] = data.articles.map((article) => ({
    id: '',
    question: article.question,
    answer: article.answer,
  }));

  return { collection, articles };
}

export function makeCollection(collection: { name: string; description: string; } , articles: IHelpArticleAddUpdate[]) {
  const collectionData: IHelpGetItem = {
    name: collection.name,
    description: collection.description,
    articles: articles,
  };
  return collectionData;
}

// ----------------------------------------------------------------------

const swrOptions = {
  revalidateIfStale: true,     
  revalidateOnFocus: false,    
  revalidateOnReconnect: true, 
};

// -------------------------------COLLECTIONS-------------------------------
// ----------------------------------------------------------------------

type HelpsData = {
  collections: IHelpItem[]; 
};

export function useGetHelps() {
  const url = endpoints.chatbot.help.list;

  const { data, isLoading, error, isValidating } = useSWR<HelpsData>(url, fetcherchatbot, swrOptions);

  const memoizedValue = useMemo(
    () => ({
      helps: data?.collections || [],
      helpsLoading: isLoading,
      helpsError: error,
      helpsValidating: isValidating,
      helpsEmpty: !isLoading && !data?.collections.length,
      refetchHelps: () => mutate(url),
    }),
    [data, error, isLoading, isValidating]
  );

  return memoizedValue;
}

// ----------------------------------------------------------------------

export function useGetHelp(collectionId: string) {
  const url = endpoints.chatbot.help.details(collectionId);

  const { data, isLoading, error, isValidating } = useSWR<IHelpItem>(url, fetcherchatbot, swrOptions);

  const memoizedValue = useMemo(
    () => ({
      help: data,
      helpLoading: isLoading,
      helpError: error,
      helpValidating: isValidating,
      helpEmpty: !isLoading && !data,
      refetchHelp: () => mutate(url),
    }),
    [data, error, isLoading, isValidating]
  );

  return memoizedValue;
}

// ----------------------------------------------------------------------

export const addCollection = async (postData: any) => {
  const url = endpoints.chatbot.help.add;
  const collectionData = await fetcherchatbot([url, {}], 'post', postData);
  mutate(endpoints.chatbot.help.list);
  return {collectionData};
}

// ----------------------------------------------------------------------

export const deleteCollection = async (collectionId: string) => {
  const url_articles = endpoints.chatbot.help.article.list(collectionId);
  const articles = await fetcherchatbot(url_articles, 'get');
  await Promise.all(
    articles.map(async (article: any) => {
      const url = endpoints.chatbot.help.article.delete(collectionId, article.id);
      await fetcherchatbot([url, {}], 'delete');
    })
  );
  const url = endpoints.chatbot.help.delete(collectionId);
  await fetcherchatbot([url, {}], 'delete');
  mutate(endpoints.chatbot.help.list);
}

// ----------------------------------------------------------------------
export function updateCollection(collectionId: string, data: IHelpCollectionAddUpdate) {
  const url = endpoints.chatbot.help.update(collectionId);
  mutate(endpoints.chatbot.help.list);
  return fetcherchatbot([url, {}], 'put', data);
}

// ------------------------------ARTICLES---------------------------------
// ----------------------------------------------------------------------

export function useGetArticles(collectionId: string) {
  const url = endpoints.chatbot.help.article.list(collectionId);

  const { data, isLoading, error, isValidating } = useSWR<IHelpArticleAddUpdate[]>(url, fetcherchatbot, swrOptions);

  const memoizedValue = useMemo(
    () => ({
      articles: data,
      articlesLoading: isLoading,
      articlesError: error,
      articlesValidating: isValidating,
      articlesEmpty: !isLoading && !data?.length,
      refetchArticles: () => mutate(url),
    }),
    [data, error, isLoading, isValidating]
  );

  return memoizedValue;
}

// ----------------------------------------------------------------------
export function useGetArticle(collectionId: string, articleId: string) {
  const url = endpoints.chatbot.help.article.details(collectionId, articleId);

  const { data, isLoading, error, isValidating } = useSWR<IHelpArticleAddUpdate>(url, fetcherchatbot, swrOptions);

  const memoizedValue = useMemo(
    () => ({
      article: data,
      articleLoading: isLoading,
      articleError: error,
      articleValidating: isValidating,
      articleEmpty: !isLoading && !data,
      refetchArticle: () => mutate(url),
    }),
    [data, error, isLoading, isValidating]
  );

  return memoizedValue;
}

// ----------------------------------------------------------------------

export const addArticle = async (collectionId: string, postData: any) => {
  const url = endpoints.chatbot.help.article.add(collectionId);
  const articleData= await fetcherchatbot([url, {}], 'post', postData);
  mutate(endpoints.chatbot.help.article.list(collectionId));
  return {articleData};
}

// ----------------------------------------------------------------------
export const deleteArticle = async (collectionId: string, articleId: string) => {
  const url = endpoints.chatbot.help.article.delete(collectionId, articleId);
  await fetcherchatbot([url, {}], 'delete');
  mutate(endpoints.chatbot.help.article.list(collectionId));
}

// ----------------------------------------------------------------------
export const updateArticle = async (collectionId: string , articleId: string, data: IHelpArticleAddUpdate) => {
  const url = endpoints.chatbot.help.article.update(collectionId, articleId);
  mutate(endpoints.chatbot.help.article.list(collectionId));
  return fetcherchatbot([url, {}], 'put', data);
}

export async function getCollectionWithArticles(collectionId: string) {
  try {
    const collectionUrl = endpoints.chatbot.help.details(collectionId);
    const articlesUrl = endpoints.chatbot.help.article.list(collectionId);

    // Fetch both collection and articles in parallel
    const [collection, articles] = await Promise.all([
      fetcherchatbot(collectionUrl, 'get'),
      fetcherchatbot(articlesUrl, 'get')
    ]);

    // Combine collection and its articles into one object
    return {
      ...collection,
      articles,
    };
  } catch (error) {
    console.error("Error fetching collection with articles:", error);
    throw error;
  }
}
