import { AllLivesAnalytics, ILive, ILiveComment, ILiveItem, ILiveProduct, ILiveStatistic, ILiveStatisticsSummary, IProductListItem, LiveProductStatus, LiveStatus, TimeStampProduct } from "@/shared/types/live";
import { generateStatisticsForLives } from "../_mock";
import { useMemo } from 'react';
import axiosInstance, { endpoints, fetcher } from "./server";
import useSWR, { mutate} from 'swr';
import { tr } from "date-fns/locale";
import axios from 'axios';


const options = {
    revalidateIfStale: false,
    revalidateOnFocus: true,
    revalidateOnReconnect: false,
};





export async function addLive(data: Record<string, any>, image: File) {
    const formData = new FormData();
    formData.append('image', image);
    formData.append('data', new Blob([JSON.stringify(data)], { type: 'application/json' }));

    const response = await axiosInstance.post<ILiveItem>(endpoints.live.create, formData, {
        headers: {
            'Content-Type': 'multipart/form-data'
        }
    });

    const newData = response.data;
    console.log(newData);
    
    const livesUrl = localStorage.getItem('livesUrl');
    if (livesUrl) {
        await mutate(
            livesUrl,
            (livesResponse: { content: ILiveItem[] } | undefined = undefined) => {
                if (livesResponse && livesResponse.content) {
                    livesResponse.content.push(newData);
                }
                return livesResponse;
            },
            true
        );
    }

    await mutate([endpoints.live.statusCount], (statusCount: Record<LiveStatus, number> | undefined) => {
        if (statusCount) {
            statusCount[LiveStatus.COMING] = statusCount[LiveStatus.COMING] + 1;
            statusCount[LiveStatus.ALL] = statusCount[LiveStatus.ALL] + 1;
        }
        return statusCount;
    }, true);
    return newData;
}


export async function editLive(liveId: string, data: Record<string, any>, image?: File | null) {
    try {
        const formData = new FormData();
        if (image) {
            formData.append('image', image);
        }
        formData.append('data', new Blob([JSON.stringify(data)], { type: 'application/json' }));

        const response = await axiosInstance.put(endpoints.live.update(liveId), formData, {
            headers: {
                'Content-Type': 'multipart/form-data'
            }
        });

        const newData = response.data;
        
        mutate([localStorage.getItem('livesUrl')], (lives: ILiveItem[] = []) => {
            const index = lives.findIndex((live) => live.id === liveId);
            if (index > -1) {
                lives[index] = { ...lives[index], ...newData };
            }
            return [...lives];
        }, true);

        mutate(
            [endpoints.live.get(liveId)],
            (live: ILive | null = null) => {
                if (live) {
                    live = { ...live, ...newData };
                }
                return live;
            },
            false
        );

        return newData;
    } catch (error) {
        console.error('Error editing live:', error);
        throw error;
    }
}




export async function deleteLive(liveId: string) {
    await axiosInstance.delete(endpoints.live.delete(liveId));
    
    await mutate([localStorage.getItem('livesUrl')],
        (livesResponse: { content: ILiveItem[] } | undefined = undefined) => {
            if (livesResponse && livesResponse.content) {
                const index = livesResponse.content.findIndex((live) => live.id === liveId);
                if (index > -1) {
                    livesResponse.content.splice(index, 1);
                }
            }
            return livesResponse;
        },
        true
    );

    await mutate([endpoints.live.statusCount], undefined, true);
}


export async function archiveLive(liveId: string) {
    await axiosInstance.post(endpoints.live.archive(liveId));
    
    mutate([localStorage.getItem('livesUrl')], 
        (livesResponse: { content: ILiveItem[] } | undefined = undefined) => {
            if (livesResponse && livesResponse.content) {
                const index = livesResponse.content.findIndex((live) => live.id === liveId);
                if (index > -1) {
                    livesResponse.content[index].status = LiveStatus.ARCHIVED;
                }
            }
            return livesResponse;
        }, 
        true
    );

    mutate(
        [endpoints.live.get(liveId)],
        (live: ILive | null = null) => {
            if (live) {
                live.status = LiveStatus.ARCHIVED;
            }
            return live;
        },
        true
    );

    await mutate([endpoints.live.statusCount], undefined, true);
}



export async function restoreLive(liveId: string) {
    await axiosInstance.post(endpoints.live.restore(liveId));
    
    mutate([localStorage.getItem('livesUrl')], 
        (livesResponse: { content: ILiveItem[] } | undefined = undefined) => {
            if (livesResponse && livesResponse.content) {
                const index = livesResponse.content.findIndex((live) => live.id === liveId);
                if (index > -1) {
                    livesResponse.content[index].status = LiveStatus.REVIEW;
                }
            }
            return livesResponse;
        }, 
        true
    );

    mutate(
        [endpoints.live.get(liveId)],
        (live: ILive | null = null) => {
            if (live) {
                live.status = LiveStatus.REVIEW;
            }
            return live;
        },
        true
    );

    await mutate([endpoints.live.statusCount], (statusCount: Record<LiveStatus, number> | undefined) => {
        if (statusCount) {
            statusCount[LiveStatus.ARCHIVED] = statusCount[LiveStatus.ARCHIVED] - 1;
            statusCount[LiveStatus.REVIEW] = statusCount[LiveStatus.REVIEW] + 1;
        }
        return statusCount;
    }, true);
}



export async function publishLive(liveId: string) {
    await axiosInstance.post(endpoints.live.publish(liveId));
    
    mutate(
        [endpoints.live.get(liveId)],
        (live: ILive | null = null) => {
            if (live) {
                live.status = LiveStatus.REPLAY;
            }
            return live;
        },
        false
    );

    mutate(
        [localStorage.getItem('livesUrl')],
        (livesResponse: { content: ILiveItem[] } | undefined = undefined) => {
            if (livesResponse && livesResponse.content) {
                const index = livesResponse.content.findIndex((live) => live.id === liveId);
                if (index > -1) {
                    livesResponse.content[index].status = LiveStatus.REPLAY;
                }
            }
            return livesResponse;
        },
        true
    );

    await mutate([endpoints.live.statusCount], (statusCount: Record<LiveStatus, number> | undefined) => {
        if (statusCount) {
            statusCount[LiveStatus.REVIEW] = statusCount[LiveStatus.REVIEW] - 1;
            statusCount[LiveStatus.REPLAY] = statusCount[LiveStatus.REPLAY] + 1;
        }
        return statusCount;
    }, true);
}


export function useGetLives(page: number, size: number, sort: string, sortBy: string, status: string, search: string) {
    const URL = [endpoints.live.all(page, size, sort, sortBy, status, search)];
    localStorage.setItem('livesUrl', endpoints.live.all(page, size, sort, sortBy, status, search));
    const { data, isLoading, error, isValidating } = useSWR<{
        content: ILiveItem[],
        totalElements: number,
        totalPages: number,
        size: number,
        number: number,
        first: boolean,
        last: boolean,
        numberOfElements: number,
        empty: boolean
    }>(URL, fetcher, options);

    return useMemo(
        () => ({
            livesData: data?.content || [],
            totalElements: data?.totalElements || 0,
            totalPages: data?.totalPages || 0,
            currentPage: data?.number || 0,
            pageSize: data?.size || 10,
            isFirstPage: data?.first || true,
            isLastPage: data?.last || true,
            numberOfElements: data?.numberOfElements || 0,
            isEmpty: data?.empty || true,
            livesLoading: isLoading,
            livesError: error,
            livesValidating: isValidating,
        }),
        [data, error, isLoading, isValidating]
    );
}

export function useGetLive(liveId: string) {
    const URL = [endpoints.live.get(liveId)];
    const { data, isLoading, error } = useSWR(URL, fetcher, options);
    return { liveData: data as ILive, liveIsLoading: isLoading, liveError: error };
}


export  function useGetLiveStatusCount() {
    const URL = [endpoints.live.statusCount];
    const { data, isLoading, error } = useSWR<Record<LiveStatus, number>>(URL, fetcher, options);
    return useMemo(() => ({
        liveStatusCountData: data ,
        liveStatusCountLoading: isLoading,
        liveStatusCountError: error,
    }), [data, error, isLoading]);
}

/////////////////////////////////Live Statistics API//////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

export async function getRealTimeStats(id: string): Promise<ILiveStatistic> {
    const response = await axiosInstance.post<ILiveStatistic>(endpoints.stats.getStreamStats(id));
    return response.data;
}


export async function getLiveStatistics(liveId: string) {
    const response = await axiosInstance.get<ILiveStatisticsSummary>(endpoints.stats.getStreamSummary(liveId));
    return response.data;
}


export async function getLiveStatisticsForAllLives() {
    const response = await axiosInstance.get<AllLivesAnalytics>(endpoints.stats.getAllLivesSummary);
    return response.data;
}

export function useGetLivesStatistic(startDate: Date, endDate: Date, lives: string[]) {
    return useMemo(
        () => ({
            statsLoading: false,
            statsError: false,
            statsValidating: false,
            statsData: generateStatisticsForLives(2),
        }),
        [startDate, endDate, lives]
    );
}

/////////////////////////////////Live Comment API/////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

export async function addCommentLive(commentData: ILiveComment) {
    const response = await axiosInstance.post(endpoints.comments.add, commentData);
    return response.data as ILiveComment;
}


export function useGetCommentsLive(liveId: string) {
    const URL = endpoints.comments.get(liveId);

    const { data, isLoading, error, isValidating } = useSWR<ILiveComment[]>(
        liveId ? URL : null, // Only fetch if liveId is provided
        fetcher,
        {
            revalidateOnFocus: false, 
            revalidateOnReconnect: true,
            revalidateOnMount: true,
            refreshInterval: 0,
        }
    );

    return useMemo(
        () => ({
            commentsData: data || [],
            commentsLoading: isLoading,
            commentsError: error,
            commentsValidating: isValidating,
            mutate,
        }),
        [data, isLoading, error, isValidating] // Include mutate in the dependency array
    );
}


export async function exportCommentsLive(liveId: string) {
    const response = await axiosInstance.get<ILiveComment[]>(endpoints.comments.export(liveId));
    return response.data;
}

export async function deleteCommentLive(commentId: string, liveId: string) {
    await axiosInstance.delete(endpoints.comments.delete(commentId, liveId));
}

export async function pinCommentLive(commentId: string, liveId: string) {
    await axiosInstance.post(endpoints.comments.pin(commentId, liveId));
}


export async function unpinCommentLive(commentId: string, liveId: string) {
    await axiosInstance.post(endpoints.comments.unpin(commentId, liveId));
}


export async function blockUserLive(userId: string, liveId: string) {
    await axiosInstance.post(endpoints.comments.blockUser(userId, liveId));
}


/////////////////////////////////Live State API////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

export async function stopLive(liveId: string) {
    await axiosInstance.post(endpoints.live.stop(liveId));
    
    mutate(
        [endpoints.live.get(liveId)],
        (live: ILive | null = null) => {
            if (live) {
                live.status = LiveStatus.REVIEW;
            }
            return live;
        },
        true
    );

    mutate(
        [localStorage.getItem('livesUrl')],
        (livesResponse: { content: ILiveItem[] } | undefined = undefined) => {
            if (livesResponse && livesResponse.content) {
                const index = livesResponse.content.findIndex((live) => live.id === liveId);
                if (index > -1) {
                    livesResponse.content[index].status = LiveStatus.REVIEW;
                }
            }
            return livesResponse;
        },
        true
    );
}

export async function startLive(liveId: string) {
    await axiosInstance.post(endpoints.live.start(liveId));
    
    mutate(
        [endpoints.live.get(liveId)],
        (live: ILive | null = null) => {
            if (live) {
                live.status = LiveStatus.ONGOING;
                live.startedAt = new Date();
            }
            return live;
        },
        true
    );

    mutate(
        [localStorage.getItem('livesUrl')],
        (livesResponse: { content: ILiveItem[] } | undefined = undefined) => {
            if (livesResponse && livesResponse.content) {
                const index = livesResponse.content.findIndex((live) => live.id === liveId);
                if (index > -1) {
                    livesResponse.content[index].status = LiveStatus.ONGOING;
                }
            }
            return livesResponse;
        },
        true
    );
}

/////////////////////////////////Live Product API////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////


export const useGetProductsLive = (liveId: string) => {
    const URL = [endpoints.liveProducts.all(liveId)];
    const { data, isLoading, error } = useSWR<ILiveProduct[]>(URL, fetcher, options);
    return { products: data || [], productsLoading: isLoading, productsError: error };
}

export async function searchProductLive(sku: string) {
    return await axiosInstance.get(endpoints.product.search(sku));
}

export async function addProductToLive(liveId: string, product: ILiveProduct) {
    const response = await axiosInstance.post<ILiveProduct>(endpoints.liveProducts.add(liveId), product);
    return response.data; 
}



export async function deleteProductLive(liveProductId: string, liveId: string) {
    await axiosInstance.delete(endpoints.liveProducts.delete(liveId, liveProductId));
}


export async function getProductsLive(liveId: string) {
    const response = await axiosInstance.get<ILiveProduct[]>(endpoints.liveProducts.all(liveId));
    return response.data;
}


export async function addTimeStampProduct(productId: string, startTime: number, endTime: number) {
    const response = await axiosInstance.post<TimeStampProduct>(endpoints.liveProducts.addTimeStamp(productId, startTime, endTime));
    return response.data;
}

export async function deleteTimeStampProduct(id: string) {
    await axiosInstance.delete(endpoints.liveProducts.deleteTimeStamp(id));
   
}



export async function presentProductLive(productId: string, liveId: string, startTime: number) {
    await axiosInstance.post(endpoints.liveProducts.present(liveId, productId, startTime));
}


export async function pushProductLive(productId: string, startTime: number) {
    const response = await axiosInstance.post<TimeStampProduct>(endpoints.liveProducts.push(productId, startTime));
    return response.data;
}

export async function hideProductLive(productId: string, endTime: number) {
  
    const response = await axiosInstance.post<TimeStampProduct>(endpoints.liveProducts.hide(productId, endTime));
    return response.data;
}


export async function updateTimeStampProduct(id: string, startTime: number, endTime: number) {
    const response = await axiosInstance.put<TimeStampProduct>(endpoints.liveProducts.updateTimeStamp(id, startTime, endTime));
    return response.data;
}


export function useSearchProductsLive(page: number, size: number, keyword: string) {
    const URL = `${endpoints.Addons.PromoCode.getProductsPublishedWebsite(page, size, keyword)}`;
    const { data, error, isValidating } = useSWR(URL, fetcher);

    const productsData = useMemo(() => {
        return (data?.content as IProductListItem[]) || [];
    }, [data?.content]);

    const memoizedValue = useMemo(
        () => ({
            productsData: productsData,
            productsLoading: !data && !error,
            productsError: error,
            productsValidating: isValidating,
            productsEmpty: productsData.length === 0,
            totalPages: data?.totalPages || 1,
            currentPage: page,
        }),
        [productsData, error, isValidating, data, page]
    );

    return memoizedValue;
}

export async function searchProduct(keyword: string) {
    // Normalize the keyword (trim and lowercase)
    const normalizedKeyword = keyword.trim().toLowerCase();
    if (!normalizedKeyword) {
        return [];
    }
    
    // Setup retry logic
    const maxRetries = 2;
    let retries = 0;
    let lastError: unknown = null;
    
    // Try with retry logic
    while (retries <= maxRetries) {
        try {
            const URL = `${endpoints.Addons.PromoCode.getProductsPublishedWebsite(0, 10, normalizedKeyword)}`;
            
            // Make the request without cache headers
            const response = await axiosInstance.get(URL);
            
            // Validate response format before returning
            const data = response.data?.content;
            if (!Array.isArray(data)) {
                console.warn('Invalid product search response format:', response.data);
                return [];
            }
            
            return data as IProductListItem[];
        } catch (error: unknown) {
            lastError = error;
            // Only retry on network errors or 5xx server errors
            const isNetworkError = error instanceof Error && error.message?.includes('Network Error');
            const isServerError = axios.isAxiosError(error) && error.response?.status !== undefined && error.response.status >= 500;
            
            if (isNetworkError || isServerError) {
                retries++;
                // Exponential backoff delay
                const delay = Math.pow(2, retries) * 300;
                await new Promise(resolve => setTimeout(resolve, delay));
            } else {
                // Don't retry for client errors
                break;
            }
        }
    }
    
    // Log the error
    console.error('Product search failed after retries:', lastError);
    
    // Throw the error to be handled by the caller
    if (lastError instanceof Error) {
        throw lastError;
    } else {
        throw new Error('Failed to search products');
    }
}

