import { useMemo } from "react";
import useSWR, { mutate } from "swr";
import axiosInstance, { endpoints, fetcher } from "./server";
import { IOrderPage, Order, OrderStatus } from "@/contexts/types/main-order";
import { GATEWAY_API_URL } from "@/config-global";
import { formatDateToString } from "@/utils/format-time";


const options = {
  revalidateIfStale: false,
  revalidateOnFocus: false,
  revalidateOnReconnect: false,
};

///////////////////-----------Order Management----------///////////////////////
//////////////////////////////////////////////////////////////////////////////
export function getAllOrders() {
  const URL = [endpoints.order.getAll];
  const { data, error, isLoading, isValidating, mutate } = useSWR<Order[]>(URL, fetcher, options);

  const memoizedValue = useMemo(
    () => ({
      ordersData: (data as Order[]) || [],
      ordersError: error,
      ordersLoading: isLoading,
      ordersValidating: isValidating,
      mutateOrders: mutate,
    }),
    [data, error]
  );

  return memoizedValue;
}

export function getOrderById(orderId: string) {
  const { data, error, isLoading, isValidating, mutate } = useSWR<Order>(
    orderId ? endpoints.order.getById(orderId) : null,
    fetcher,
    options
  );

  const memoizedValue = useMemo(
    () => ({
      orderData: data,
      orderError: error,
      orderLoading: isLoading,
      orderValidating: isValidating,
      mutateOrder: mutate,
    }),
    [data, error]
  );

  return memoizedValue;
}

export async function updateOrderStatus(orderId: string, userId: string, orderStatus: OrderStatus) {
  try {
    const url = `${endpoints.order.updateOrderStatus}`;
    const response = await axiosInstance.put(url, null, {
      params: {
        orderId,
        userId,
        orderStatus,
      },
    });
    return response.data;
  } catch (error) {
    console.error('Erreur lors de la mise à jour du statut de la commande', error);
    throw error;
  }
}

export async function useDeleteOrderList(ordersIDs: string[]): Promise<number> {
  try {
    const response = await axiosInstance.delete(endpoints.order.deleteList, {
      params: { ordersIDs: ordersIDs.join(',') },
    });
    await mutate(endpoints.order.getAll);
    return response.status;
  } catch (error) {
    console.error("Error deleting order:", error);
    throw error;
  }
}


export async function useDeleteOrder(id: string) {
  const response = await axiosInstance.delete(
    endpoints.order.deleteOne(id)
  );
  mutate(endpoints.order.getAll);
  return response.status;
}

export function useSearchOrders({
  keyword = "",
  status = "",
  page = 0,
  size = 10,
  sortBy = "id",
  sortDir = "ASC",
  startDate = null,
  endDate = null,
}: {
  keyword?: string;
  status?: string;
  page?: number;
  size?: number;
  sortBy?: string;
  sortDir?: string;
  startDate?: Date | null;
  endDate?: Date | null;
}) {
  const filterParams = new URLSearchParams({
    keyword: keyword || "",
    status: status || "",
    page: page.toString(),
    size: size.toString(),
    sortBy: sortBy || "",
    sortDir: sortDir || "",
    startDate: startDate?.toString() || "",
    endDate: endDate?.toString() || "",

  });

  const url = `${endpoints.order.filter}?${filterParams.toString()}`;

  const { data, error, isValidating, mutate } = useSWR<IOrderPage>(url, fetcher);

  return useMemo(
    () => ({
      orders: data?.content || [],
      totalOrders: data?.totalElements || 0,
      totalPages: data?.totalPages || 0,
      pageNumber: data?.pageable.pageNumber || 0,
      pageSize: data?.pageable.pageSize || 0,
      loading: !data && !error,
      error,
      isValidating,
      revalidate: mutate,
    }),
    [data, error, isValidating, mutate]
  );
}
export function useOrderStatusCounts({
  keyword = "",
  status = "",
  startDate = null,
  endDate = null,
}: {
  keyword?: string;
  status?: string;
  startDate?: Date | null;
  endDate?: Date | null;
}) {
  const params = {
    keyword,
    status,
    startDate: startDate ? startDate.toString() : "",
    endDate: endDate ? endDate.toString() : "",
  };

  const url = `${endpoints.order.statusCount}?${new URLSearchParams(params).toString()}`;

  const { data, error } = useSWR(url, async () => {
    try {
      const response = await axiosInstance.get(url);
      return response.data;
    } catch (err) {
      console.error("Error fetching order status counts:", err);
      throw err;
    }
  });

  return useMemo(
    () => ({
      orderStatusCounts: data || [],
      orderStatusError: error,
    }),
    [data, error]
  );
}


export const exportOrders = (startDate: Date | null, endDate: Date | null, orderStatus: string | null) => {

  axiosInstance.get(endpoints.order.exportByDateRange, { responseType: 'blob', params: { startDate, endDate, orderStatus } })
    .then((response) => {
      const link = document.createElement("a");
      link.href = window.URL.createObjectURL(response.data);
      link.download = `orders_${formatDateToString(new Date())}.csv`;
      link.click();
    })
    .catch((error) => {
      console.error("Error exporting orders:", error);
      throw error;
    });
}

export const exportOrdersProducts = (startDate: Date | null, endDate: Date | null, orderStatus: string | null) => {

  axiosInstance.get(endpoints.order.exportordersProduct, { responseType: 'blob', params: { startDate, endDate, orderStatus } })
    .then((response) => {
      const link = document.createElement("a");
      link.href = window.URL.createObjectURL(response.data);
      link.download = `orders_products_${formatDateToString(new Date())}.csv`;
      link.click();
    })
    .catch((error) => {
      console.error("Error exporting orders:", error);
      throw error;
    });
}


