import { useMemo } from "react";
import useSWR from "swr";
import { _orders, userId } from "../_mock";
import axiosInstance, { endpoints, fetcher } from "./server";
import { mutate } from "swr/_internal";
import { IMedia } from "../types/media";
import { AxiosResponse } from "axios";


const options = {
  revalidateIfStale: false,
  revalidateOnFocus: true,
  revalidateOnReconnect: false,
};

///////////////////-----------Media Management----------///////////////////////
//////////////////////////////////////////////////////////////////////////////
export function useGetMedias() {
  const URL = [endpoints.media.getAll];
  const { data, error, isLoading, isValidating, mutate } = useSWR<IMedia[]>(URL, fetcher, options);

  const memoizedValue = useMemo(
    () => ({
      mediasData: (data as IMedia[]) || [],
      mediasError: error,
      mediasLoading: isLoading,
      mediasValidating: isValidating,
      mutateMedias: mutate,
    }),
    [data, error]
  );

  return memoizedValue;
}

export function useGetMedia(mediaId: string) {
  const { data, error, isLoading, isValidating, mutate } = useSWR<IMedia>(
    mediaId ? endpoints.media.getById(mediaId) : null,
    fetcher,
    options
  );

  const memoizedValue = useMemo(
    () => ({
      mediaData: data,
      mediaError: error,
      mediaLoading: isLoading,
      mediaValidating: isValidating,
      mutateMedia: mutate,
    }),
    [data, error]
  );

  return memoizedValue;
}

export const useUploadMedias = async (userId: string, medias: (File | string)[]): Promise<void> => {
  try {
    let url = endpoints.media.add;
    const formData = new FormData();
    formData.append("userId", userId);
    medias.forEach(file => formData.append('medias', file));

    const response: AxiosResponse = await axiosInstance.post(url, formData, {
      headers: {
        "Content-Type": "multipart/form-data",
      },
    });

    mutate(endpoints.media.getAll);
  } catch (error) {
    throw error;
  }
};

export async function useUpdateMedia(
  userId: string,
  id: string,
  updatedMedia: any,
  file?: File,
) {
  const formData = new FormData();

  formData.append("userId", userId);
  formData.append("title", updatedMedia.title);
  formData.append("description", updatedMedia.description);

  if (file) {
    formData.append("file", file);
  }

  const response = await axiosInstance.put(endpoints.media.edit(id), formData, {
    headers: {
      "Content-Type": "multipart/form-data",
    },
  });

  mutate(endpoints.media.getAll);

  const updatedData: IMedia = response.data;
  return updatedData;
}

export async function useDeleteMedia(mediaIDs: string[]): Promise<number> {
  try {
    const response = await axiosInstance.delete(endpoints.media.delete, {
      data: mediaIDs,
    });
    await mutate(endpoints.media.getAll);
    return response.status;
  } catch (error) {
    // Handle the error appropriately
    console.error("Error deleting media:", error);
    throw error;
  }
}

export const useDownloadMedia = async (
  mediaId: string,
  mediaName: string
): Promise<void> => {
  try {
    const url1 = `${endpoints.media.download}?mediaId=${mediaId}`;
    const response = await axiosInstance.get(url1, {
      responseType: "blob",
    });

    const contentType = response.headers["content-type"];

    let fileNameToDownload = mediaName;

    if (!/\.[^.]+$/.test(mediaName)) {
      fileNameToDownload += ".png";
    }

    const blob = new Blob([response.data], { type: contentType });

    const url = window.URL.createObjectURL(blob);
    const link = document.createElement("a");
    link.href = url;
    link.setAttribute("download", fileNameToDownload);
    document.body.appendChild(link);
    link.click();
    document.body.removeChild(link);
    window.URL.revokeObjectURL(url);
  } catch (error) {
    throw error;
  }
};

export const downloadMediaFile = async (mediaId: string): Promise<void> => {
  try {
    const url = `${endpoints.media.download}?mediaId=${mediaId}`;
    
    const response = await axiosInstance.get(url, {
      responseType: "blob",
    });

    const contentType = response.headers["content-type"];

    const fileExtension = contentType.split("/")[1] || "png";
    const fileNameToDownload = `downloaded_media.${fileExtension}`;

    const blob = new Blob([response.data], { type: contentType });

    const downloadUrl = window.URL.createObjectURL(blob);
    const link = document.createElement("a");
    link.href = downloadUrl;
    link.setAttribute("download", fileNameToDownload);
    document.body.appendChild(link);
    link.click();
    
    document.body.removeChild(link);
    window.URL.revokeObjectURL(downloadUrl);
  } catch (error) {
    throw error;
  }
};
