import useSWR, { mutate } from "swr";
import {useMemo } from "react";
import {
  IProductFilterParams,
  IProductItem,
  IProductListItem,
  IProductPage,
  IProductSessionItem,
  IProductSessionSerchTerm,
  ProductDetailsDTO,
  ProductType,
  ProductWithCustomResponse
} from "@/shared/types/product";
import  axiosInstance, { fetcher, endpoints, postFetcher } from "./server";
import { IVariationItem } from "@/contexts/types/main-order";
import { IMigration } from "../types/migration";

const options = {
  revalidateIfStale: false,
  revalidateOnFocus: false,
  revalidateOnReconnect: false,
};

//--------------------Add Product -----------------------------------
export async function useAddSimpleProduct(
  data: IProductItem
): Promise<IProductItem> {
  const formData = {
    ...data,
    categoryIds: data.categoryIds,
    tagIds: data.tagIds,
    Variations: [], //should be empty
  };

  try {
    const response = await axiosInstance.post("/api/stock/products", formData);
    const newData: IProductItem = response.data;


    return newData;
  } catch (error) {
    console.error("Error adding product:", error);
    throw error;
  }
}


export async function addVariableProduct(
  data: IProductItem
): Promise<IProductItem> {
  const formData = {
    ...data,
    categoryIds: data.categoryIds,
    tagIds: data.tagIds,
    variations: data.variations,
  };

  try {
    const response = await axiosInstance.post("/api/stock/products", formData);
    return response.data;
  } catch (error) {
    console.error("Error adding variable product:", error);
    throw error;
  }
}
// --------------------- Get Category ----------------

export function useGetHierarchicalCategories() {
  const { data, error, mutate } = useSWR("/api/stock/categories/hierarchical", fetcher);

  return {
    categories: data,
    isLoading: !error && !data,
    isError: error,

  };
}
// ---------------------------------- Get Product ------------------------------------
export const searchProductWithIndex = async (
  searchKey: string
): Promise<ProductWithCustomResponse[]> => {
  try {
    const response = await axiosInstance.get<ProductWithCustomResponse[]>(
      endpoints.product.searchProductWithIndex(searchKey)
    );

    return response.data;
  } catch (error) {
    throw error;
  }
};

export function useAllGetProducts() {
  const URL = "/api/stock/products/list";

  const { data, isLoading, error, isValidating, mutate } = useSWR<IProductListItem[]>(
    URL,
    fetcher,
    options
  );

  const memoizedValue = useMemo(
    () => ({
      products: data || [],
      productsLoading: isLoading,
      productsError: error,
      productsValidating: isValidating,
      productsEmpty: !isLoading && (!data || data.length === 0),
      revalidate: () => mutate(), // Allow manual revalidation
    }),
    [data, error, isLoading, isValidating]
  );

  return memoizedValue;
}

export function useAllGetProductsWithVariation(searchData: IProductSessionSerchTerm | null) {
  const shouldFetch = searchData && searchData.searchTerm && searchData.searchTerm.length >= 3;

  const URL = "/api/stock/products/ProductWithVariations";

  const { data, isLoading, error, isValidating, mutate } = useSWR<IProductSessionItem[]>(
    shouldFetch ? [URL, searchData] : null,
    ([url, requestData]) => postFetcher(url, requestData)
  );

  const memoizedValue = useMemo(
    () => ({
      products: data || [],
      productsLoading: isLoading || false,
      productsError: error || null,
      productsValidating: isValidating || false,
      productsEmpty: !isLoading && (!data || data.length === 0),
      revalidate: () => {
        if (shouldFetch) mutate();
      },
      mutate,
    }),
    [data, error, isLoading, isValidating, shouldFetch, mutate]
  );

  return memoizedValue;
}


// // ----------------------------------------------------------------------

export function useGetProduct(productId: string) {
  const URL = `/api/stock/products/id/${productId}`;

  const { data, error, mutate } = useSWR<ProductDetailsDTO>(URL, fetcher);

  return {
    product: data,
    productLoading: !error && !data,
    productError: error,
    mutateProduct: mutate,
  };
}



// -------------------- Update Product -----------------------------------

export async function updateProduct(
  productId: string,
  data: IProductItem
): Promise<IProductItem> {
  try {
    const response = await axiosInstance.put(
      `/api/stock/products/edit/${productId}`,
      data
    );
    return response.data;
  } catch (error) {
    console.error("Error updating product:", error);
    throw error;
  }
}


///////////////////////////////////

export function useUpdateProduct(productId: string | number) {
  const URL = `/api/stock/products/${productId}`;

  const { data, error, mutate } = useSWR<IProductItem>(URL, fetcher, options);

  const memoizedValue = useMemo(
    () => ({
      product: data,
      productLoading: !error && !data,
      productError: error,
      productMutate: mutate,
    }),
    [data, error]
  );
  return memoizedValue;
}
////////////////////// DELETE PRODUCT///////////////////

export async function useDeleteProduct(id: string) {
  try {
    const URL = `/api/stock/products/delete/${id}`;
    const response = await axiosInstance.delete(URL);
    await mutate("/api/stock/products/list");
    return response;
  } catch (error) {
    console.error("Erreur lors de la suppression du produit:", error);
    throw error;
  }
}

export async function useDeleteProducts(ids: string[]) {
  try {
    const URL = "/api/stock/products/delete-bulk";
    const response = await axiosInstance.delete(URL, {
      data: ids,
    });
    await mutate("/api/stock/products/list");
    return response;
  } catch (error) {
    console.error("Erreur lors de la suppression des Produits :", error);
    throw error;
  }
}

export function useGetProductsByIds(productIds: string[]) {
  const baseUrl = endpoints.product.allProductsByIDs;
  const URL = `${baseUrl}?productIDs=${productIds.join(",")}`;
  const { data, isLoading, error, isValidating, mutate } = useSWR<
    IProductListItem[]
  >(URL, fetcher);

  const memoizedValue = useMemo(
    () => ({
      products: data || [],
      productsLoading: isLoading,
      productsError: error,
      productsValidating: isValidating,
      productsEmpty: !isLoading && (!data || data.length === 0),
      mutateProducts: mutate,
    }),
    [data, error, isLoading, isValidating]
  );

  return memoizedValue;
}

export function useGetVariationsByIds(variationsIDs: string[]) {
  const baseUrl = endpoints.product.allVariationsByIDs;
  const URL = `${baseUrl}?variationsIDs=${variationsIDs.join(",")}`;
  const { data, isLoading, error, isValidating, mutate } = useSWR<
    IVariationItem[]
  >(URL, fetcher);

  const memoizedValue = useMemo(
    () => ({
      variations: data || [],
      variationsLoading: isLoading,
      variationsError: error,
      variationsValidating: isValidating,
      variationsEmpty: !isLoading && (!data || data.length === 0),
      mutateVariations: mutate,
    }),
    [data, error, isLoading, isValidating]
  );

  return memoizedValue;
}

// --------------------- Duplicate Product ----------------

export async function duplicateProduct(productId: string, newUgs: string): Promise<IProductItem> {
  try {
    const response = await axiosInstance.post(`/api/stock/products/duplicate/${productId}?newUgs=${encodeURIComponent(newUgs)}`);
    const newProduct: IProductItem = response.data;


    return newProduct;
  } catch (error) {
    console.error("Error duplicating product:", error);
    throw error;
  }
}


// ------------------- Product Migration -----------------------------

export function useGetMigrationProducts() {
  const { data, error, mutate } = useSWR<IMigration[]>('/api/stock/migration/products', fetcher);

  return {
    migrationProducts: data,
    loading: !error && !data,
    error,
    mutate,
  };
}

export async function confirmMigration(products: IMigration[]) {
  try {
    const response = await axiosInstance.post('/api/stock/migration/confirm', products);
    return response.data;
  } catch (error) {
    console.error('Error confirming migration:', error);
    throw error;
  }
}


export async function triggerMigration() {
  try {
    const response = await axiosInstance.post('/api/stock/migration/trigger');
    return response.data;
  } catch (error: any) {
    console.error('Error triggering migration:', error);
    throw error;
  }
}

//----------------- Update Product or Variation Price ----------------------


export async function updateProductPrice(productId: any, newPrice: any) {
  try {
    const response = await axiosInstance.put(
      `/api/stock/products/${productId}/price`,
      null,
      {
        params: { newPrice },
      }
    );

    // Update the SWR cache directly
    mutate(
      "/api/stock/products/ProductWithVariations",
      (data: IProductSessionItem[] = []) => {
        return data.map((product) =>
          product.id === productId
            ? { ...product, regularPrice: parseFloat(newPrice) }
            : product
        );
      },
      false // Do not revalidate; use the updated data
    );

    return response.data;
  } catch (error) {
    console.error("Error updating product price:", error);
    throw error;
  }
}


export async function updateVariationPrice(variationId: any, newPrice: any) {
  try {
    const response = await axiosInstance.put(
      `/api/stock/products/variations/${variationId}/price`,
      null,
      {
        params: { newPrice },
      }
    );

    // Update the SWR cache directly
    mutate(
      "/api/stock/products/ProductWithVariations",
      (data: IProductSessionItem[] = []) => {
        return data.map((product) => {
          if (product.productType === ProductType.VARIABLE && product.variations) {
            const updatedVariations = product.variations.map((variation) =>
              variation.id === variationId
                ? { ...variation, regularPrice: parseFloat(newPrice) }
                : variation
            );
            return { ...product, variations: updatedVariations };
          }
          return product;
        });
      },
      false // Do not revalidate; use the updated data
    );

    return response.data;
  } catch (error) {
    console.error("Error updating variation price:", error);
    throw error;
  }
}


export async function checkUgsUnique(ugs: string): Promise<boolean> {
  try {
    const response = await axiosInstance.get('/api/stock/products/check-ugs', {
      params: { ugs },
    });
    return response.data.unique;
  } catch (error) {
    console.error('Error checking UGS uniqueness:', error);
    // Optionally, handle the error as needed. Here, we assume it's not unique.
    return false;
  }

}

export const exportProductDetailsToCsv = (product: ProductDetailsDTO, filename: string) => {
  const attributeNames = new Set<string>();

  // Collecte des noms des attributs pour les produits de type VARIABLE
  if (product.productType === 'VARIABLE' && product.variations) {
    product.variations.forEach(variation => {
      variation.attributes?.forEach(attr => {
        attributeNames.add(attr.attributeName || '-');
      });
    });
  }

  const exportData = [];

  // Gestion des variations (produits de type VARIABLE)
  if (product.productType === 'VARIABLE' && product.variations) {
    product.variations.forEach(variation => {
      const attributeValues: { [key: string]: string } = {};
      attributeNames.forEach(name => {
        attributeValues[name] = '-'; // Valeur par défaut pour les attributs non trouvés
      });

      variation.attributes?.forEach(attr => {
        attributeValues[attr.attributeName || '-'] = attr.attributeValue || '-';
      });

      exportData.push({
        'Nom du produit': product.name || '-',
        'UGS': variation.ugs || '-',
        'Prix de vente': variation.regularPrice?.toString() || '-',
        ...attributeValues,
        'Fournisseur': product.supplierName || '-',
        'Référence fournisseur': product.supplierRef || '-',
        'Prix d\'achat': product.purchasePrice?.toString() || '-',
      });
    });
  } else {
    // Gestion des produits simples
    const attributeValues: { [key: string]: string } = {};
    attributeNames.forEach(name => {
      attributeValues[name] = '-';
    });

    exportData.push({
      'Nom du produit': product.name || '-',
      'UGS': product.ugs || '-',
      'Prix de vente': product.regularPrice?.toString() || '-',
      ...attributeValues,
      'Fournisseur': product.supplierName || '-',
      'Référence fournisseur': product.supplierRef || '-',
      'Prix d\'achat': product.purchasePrice?.toString() || '-',
    });
  }

  // Génération du contenu CSV
  const csvContent = exportData.map(row =>
    Object.values(row)
      .map(value => `"${value}"`)
      .join(',')
  ).join('\n');

  const header = Object.keys(exportData[0]).join(',') + '\n';
  const blob = new Blob([header + csvContent], { type: 'text/csv;charset=utf-8;' });
  const link = document.createElement('a');

  if (navigator && 'msSaveBlob' in navigator) {
    (navigator as any).msSaveBlob(blob, filename);
  } else {
    link.href = URL.createObjectURL(blob);
    link.setAttribute('download', filename);
    document.body.appendChild(link);
    link.click();
    document.body.removeChild(link);
  }
};
 export function useFilteredProducts(filters: IProductFilterParams) {
  const {
    category,
    name,
    priceMin,
    priceMax,
    page,
    size,
    sortBy,
    direction,
    isDraft,
    isPublishLive,
    isPublishWeb,
    isInStockWeb,
    isInStockLive,
    keyword,
    startDate,
    endDate
  } = filters;

  const filterParams = {
    category: category || '',
    name: name || '',
    priceMin: priceMin != null ? priceMin.toString() : '',
    priceMax: priceMax != null ? priceMax.toString() : '',
    page: page.toString(),
    size: size.toString(),
    isDraft: isDraft?.toString() || '',
    isPublishLive: isPublishLive?.toString() || '',
    isPublishWeb: isPublishWeb?.toString() || '',
    isInStockWeb: isInStockWeb?.toString() || '',
    isInStockLive: isInStockLive?.toString() || '',
    keyword: keyword || '',
    startDate: startDate == null ? '' : startDate.toString(),
    endDate: endDate == null ? '' : endDate.toString(),
  };

   let queryParams = new URLSearchParams(filterParams);

  if (sortBy && Array.isArray(sortBy)) {
    sortBy.forEach((field) => {
      queryParams.append('sortBy', field);
    });
  }

  if (direction && Array.isArray(direction)) {
    direction.forEach((dir) => {
      queryParams.append('direction', dir);
    });
  }

  const url = `/api/stock/products?${queryParams.toString()}`;
  const { data, error, isValidating, mutate } = useSWR<IProductPage>(url, fetcher);

  return {
    products: data?.content || [],
    totalProducts: data?.totalElements || 0,
    totalPages: data?.totalPages || 0,
    pageNumber:data?.pageable.pageNumber||0,
    pageSize:data?.pageable.pageSize||0,
    loading: !data && !error,
    isError: error,
    isValidating,
    revalidate: mutate,
  };
}

export async function downloadProductZebraFile(productId: string) {
  try {
      const response = await axiosInstance.get(endpoints.product.downloadProductZebraFile(productId), {
          responseType: "blob",
      });

      if (response.status !== 200) {
          const errorMessage = `Erreur de téléchargement : ${response.statusText}`;
          throw new Error(errorMessage);
      }

      const blob = response.data;

      const url = window.URL.createObjectURL(blob);
      const a = document.createElement('a');
      a.href = url;
      a.download = `zebra_file_${productId}_${new Date()}.csv`;
      document.body.appendChild(a);
      a.click();
      a.remove();
      window.URL.revokeObjectURL(url);
  } catch (error) {
      console.error("Erreur lors du téléchargement du fichier :", error);
  }
}

export const getProductDetailsByUGS = async (ugs: string) => {
  if (!ugs) return { emplacement: '', tags: [] };
  try {
    const response = await axiosInstance.get(endpoints.product.getTagsAndEmplacementByUgs(ugs));
    const { tagNames, emplacement } = response.data;
    return {
      emplacement: emplacement || '',
      tags: tagNames || [],
    };
  } catch (error) {
    console.error(`Erreur lors de la récupération des détails pour l'UGS: ${ugs}`, error);
    return { emplacement: '', tags: [] };
  }
};

