import useSWR from 'swr';
import axiosInstance from './server'; 
import { ChartData } from '../types/statistics';

// Helper pour formater les paramètres de requête
function getParams(startDate?: Date, endDate?: Date, publicationSource?: string) {
  const params: Record<string, string> = {};
  if (startDate) params.startDate = startDate.toISOString();
  if (endDate) params.endDate = endDate.toISOString();
  if (publicationSource) params.publicationSource = publicationSource;
  return params;
}

// Hook pour récupérer le nombre de produits créés
export const useProductCount = (startDate?: Date, endDate?: Date, publicationSource?: string) => {
  const params = getParams(startDate, endDate, publicationSource);
  const { data, error } = useSWR(
    ['/api/stock/statistic/product-count', params],
    async ([url, params]) => {
      const response = await axiosInstance.get<number>(url, { params });
      return response.data;
    }
  );

  return {
    productCount: data,
    isLoading: !error && !data,
    isError: error,
  };
};

// Hook pour récupérer le nombre d'articles réceptionnés
export const useReceivedProductsCount = (startDate?: Date, endDate?: Date) => {
  const params = getParams(startDate, endDate);
  const { data, error } = useSWR(
    ['/api/stock/statistic/received-products-count', params],
    async ([url, params]) => {
      const response = await axiosInstance.get<number>(url, { params });
      return response.data;
    }
  );

  return {
    receivedProductsCount: data,
    isLoading: !error && !data,
    isError: error,
  };
};

// Hook pour récupérer le nombre d'articles à collecter
export const useItemsToCollectCount = (startDate?: Date, endDate?: Date) => {
  const params = getParams(startDate, endDate);
  const { data, error } = useSWR(
    ['/api/stock/statistic/items-to-collect-count', params],
    async ([url, params]) => {
      const response = await axiosInstance.get<number>(url, { params });
      return response.data;
    }
  );

  return {
    itemsToCollectCount: data,
    isLoading: !error && !data,
    isError: error,
  };
};

// Hook pour récupérer le montant total d'achat
export const useTotalPurchaseAmount = (startDate?: Date, endDate?: Date) => {
  const params = getParams(startDate, endDate);
  const { data, error } = useSWR(
    ['/api/stock/statistic/total-purchase-amount', params],
    async ([url, params]) => {
      const response = await axiosInstance.get<number>(url, { params });
      return response.data;
    }
  );

  return {
    totalPurchaseAmount: data,
    isLoading: !error && !data,
    isError: error,
  };
};

// Hook pour récupérer les données du graphique d'articles réceptionnés
export const useArticlesReceptionsChartData = (startDate: Date, endDate: Date) => {
  const params = {
    startDate: startDate.toISOString().split('T')[0],
    endDate: endDate.toISOString().split('T')[0],
  };

  const { data, error } = useSWR(
    ['/api/stock/statistic/articles-receptions', params],
    async ([url, params]) => {
      const response = await axiosInstance.get<ChartData>(url, { params });
      return response.data;
    }
  );

  return {
    articlesReceptionsChartData: data,
    isLoading: !error && !data,
    isError: error,
  };
};

// Hook pour récupérer le nombre d'articles livrés (commandes finalisées)
export const useFinishedOrderLinesCount = (startDate?: Date, endDate?: Date, sources?: string[]) => {
  const params = {
    startDate: startDate?.toISOString(),
    endDate: endDate?.toISOString(),
    sources: sources?.join(","),
  };
  const { data, error } = useSWR(
    ['/api/orders/statistics/count-finished-order-lines', params],
    async ([url, params]) => {
      const response = await axiosInstance.get<number>(url, { params });
      return response.data;
    }
  );

  return {
    finishedOrderLinesCount: data,
    isLoading: !error && !data,
    isError: error,
  };
};

// Hook pour récupérer le nombre d'articles remboursés
export const useRefundedArticlesCount = (startDate?: Date, endDate?: Date, sources?: string[]) => {
  const params = {
    startDate: startDate?.toISOString(),
    endDate: endDate?.toISOString(),
    sources: sources?.join(","),
  };
  const { data, error } = useSWR(
    ['/api/orders/statistics/refunded-articles', params],
    async ([url, params]) => {
      const response = await axiosInstance.get<number>(url, { params });
      return response.data;
    }
  );

  return {
    refundedArticlesCount: data,
    isLoading: !error && !data,
    isError: error,
  };
};


