import useSWR  from 'swr';
import { useMemo } from 'react';
import axiosInstance , {fetcher, endpoints } from '@/shared/api/server';
import { mutate } from 'swr';
import { ProductPromoCode, PromoCodeItem } from '@/contexts/types/promoCode';

// ----------------------------------------------------------------------

export function getAllPromoCodes() {
  const URL = endpoints.Addons.PromoCode.getAll;
  const { data, error, isValidating, mutate } = useSWR(URL, fetcher);
  const promoCodes = useMemo(() => {
    return (data?.promoCodes as PromoCodeItem[]) || [];
  }, [data?.promoCodes]);

  const memoizedValue = useMemo(
    () => ({
      promoCodes:  data || [],
      promoCodesLoading: !data && !error,
      promoCodesError: error,
      promoCodesValidating: isValidating,
      promoCodesEmpty: promoCodes.length === 0,
      mutatePromoCodes: mutate,
    }),
    [promoCodes, error, isValidating, data]
  );

  return memoizedValue;
}

export function getPromoCodeById(promoCodeId: string) {
  const { data, error, isLoading, isValidating, mutate } = useSWR<PromoCodeItem>(
      promoCodeId ? `${endpoints.Addons.PromoCode.getById(promoCodeId)}` : null,
      fetcher
  );

  const memoizedValue = useMemo(() => ({
      promoCodeData: data as PromoCodeItem,
      promoCodeError: error,
      promoCodeLoading: isLoading,
      promoCodeValidating: isValidating,
      mutatePromoCode: mutate,
  }), [data, error, isLoading, isValidating]);

  return memoizedValue;
}

export async function checkPromoCodeExists(code: string) {
  const response = await axiosInstance.get(endpoints.Addons.PromoCode.checkPromoCodeExists(code));
  return response.data;
}

export async function checkPromoCodeIfUsed(code: string) {
  const response = await axiosInstance.get(endpoints.Addons.PromoCode.checkPromoCodeIfUsed(code));
  return response.data;
}

export async function addPromoCode(data: PromoCodeItem) {
  const response = await axiosInstance.post(
    endpoints.Addons.PromoCode.add,
    data
  );
  const newData: PromoCodeItem = response.data;
  mutate(
    [endpoints.Addons.PromoCode.getAll],
    (promoCodes: PromoCodeItem[] = []) => [...promoCodes, newData],
    true
  );
  return newData;
}

export async function updatePromoCode(
  id: string,
  updatedPromoCode: PromoCodeItem
) {
  const response = await axiosInstance.put(
    endpoints.Addons.PromoCode.update(id),
    updatedPromoCode
  );
  const updatedData: PromoCodeItem = response.data; 
  mutate(
    [endpoints.Addons.PromoCode.getAll],
    (promoCodes: PromoCodeItem[] = []) =>
      promoCodes.map((promoCode) =>
        promoCode.id === id ? updatedData : promoCode
      ),
    true
  );
  return response.data as PromoCodeItem;
}

export async function deletePromoCodeById(id: string) {
  const response = await axiosInstance.delete(
    endpoints.Addons.PromoCode.delete(id)
  );
  mutate(endpoints.Addons.PromoCode.getAll);
  return response.status;
}

export async function deletePromoCodesByIDs(promoCodesIds: string[]): Promise<number> {
  try {
    const response = await axiosInstance.delete(endpoints.Addons.PromoCode.deleteByIds, {
      params: { promoCodesIds: promoCodesIds.join(',') },
    });
    await mutate(endpoints.Addons.PromoCode.getAll);
    return response.status;
  } catch (error) {
    console.error("Error deleting promo codes:", error);
    throw error;
  }
}

export function getPaginatedProductsPromoCodes(page: number, size: number, keyword: string) {
  const URL = `${endpoints.Addons.PromoCode.getAllProductSummaryWithKeyword(page, size, keyword)}`;
  const { data, error, isValidating, mutate } = useSWR(URL, fetcher);

  const promoCodesData = useMemo(() => {
    return (data?.content as ProductPromoCode[]) || [];
  }, [data?.content]);

  const memoizedValue = useMemo(
    () => ({
      promoCodesData: promoCodesData,
      promoCodesLoading: !data && !error,
      promoCodesError: error,
      promoCodesValidating: isValidating,
      promoCodesEmpty: promoCodesData.length === 0,
      totalPages: data?.totalPages || 1,
      currentPage: page,
      mutatePromoCodes: mutate,
    }),
    [promoCodesData, error, isValidating, data, page]
  );

  return memoizedValue;
}

