import { CreateRefundOrderRequest, RefundMethod, RefundOrderItem } from "@/contexts/types/refundOrder";
import axiosInstance, { endpoints, fetcher } from "./server";
import useSWR, { mutate } from "swr";
import { useMemo } from "react";
import { Order } from "@/contexts/types/main-order";
import { Resource } from "i18next";


export async function useCreateRefundOrder(data: CreateRefundOrderRequest) {

    const response = await axiosInstance.post(
        endpoints.order.refundOrder.createRefundOrder,
        data
    );

    const newData: Order = response.data;
    mutate(
        [endpoints.order.getAll],
        (order: Order[] = []) => [...order, newData],
        true
    );

    return newData;
}

export async function useGetAllRefundOrders(
    refundMethods: RefundMethod[],
    keyword: string,
    startDate: Date | null,
    endDate: Date | null,
    page: number,
    pageSize: number
) {

    const refundMethodsQuery = refundMethods.length > 0 ? `&refundMethods=${encodeURIComponent(refundMethods.join(','))}` : '';
    const keywordQuery = keyword ? `&keyword=${encodeURIComponent(keyword)}` : '';
    const startDateQuery = startDate ? `&startDate=${encodeURIComponent(startDate.toISOString())}` : '';
    const endDateQuery = endDate ? `&endDate=${encodeURIComponent(endDate.toISOString())}` : '';
    const URL = `${endpoints.order.refundOrder.getAllRefundOrders}?page=${page}&pageSize=${pageSize}` +
        refundMethodsQuery + keywordQuery + startDateQuery + endDateQuery;

    const { data, error, isLoading, isValidating, mutate } = useSWR(URL, fetcher);

    const refundOrdersResponse = useMemo(() => data?.content || [], [data?.content]);
    const totalPages = useMemo(() => data?.totalPages || 1, [data?.totalPages]);

    return {
        refundOrdersResponse,
        refundOrdersError: error,
        refundOrdersLoading: isLoading,
        refundOrdersValidating: isValidating,
        refundOrdersEmpty: refundOrdersResponse.length === 0,
        totalPages,
        currentPage: page,
        mutateRefundOrders: mutate,
    };
}

export function getRefundOrdersByOrderId(orderId: string) {
    const { data, error, isLoading, isValidating, mutate } = useSWR<RefundOrderItem[]>(
        endpoints.order.refundOrder.getRefundOrdersByOrderId(orderId),
        fetcher
    );

    const memoizedValue = useMemo(
        () => ({
            refundOrdersData: data,
            refundOrdersError: error,
            refundOrdersLoading: isLoading,
            refundOrdersValidating: isValidating,
            mutateRefundOrders: mutate,
        }),
        [data, error]
    );

    return memoizedValue;
}

export const exportRefundOrders = (
  refundMethods: RefundMethod[],
  keyword: string,
  startDate: Date | null,
  endDate: Date | null
) => {
  const params = {
    refundMethods: refundMethods.length > 0 ? refundMethods.join(',') : undefined,
    keyword: keyword || undefined,
    startDate: startDate ? startDate.toISOString() : undefined,
    endDate: endDate ? endDate.toISOString() : undefined,
  };

  axiosInstance
    .get(endpoints.order.refundOrder.exportRefundOrders, {
      responseType: 'blob',
      params,
    })
    .then((response) => {
      const link = document.createElement('a');
      link.href = window.URL.createObjectURL(response.data);
      link.download = `refund_orders_${new Date().toISOString().split('T')[0]}.csv`;
      link.click();
    })
    .catch((error) => {
      console.error('Error exporting refund orders:', error);
      throw error;
    });
};
