import { useMemo } from "react";
import useSWR, { mutate } from "swr";
import axiosInstance, { endpoints, fetcher } from "./server";
import { ConfirmSaleOrderRequest, ISaleOrderPage, SaleOrder, SearchSaleOrderParams } from "@/contexts/types/saleOrder";
import { SupplierOrder } from "@/contexts/types/supplierOrder";


export function getAllSaleOrders() {
    const URL = endpoints.saleOrder.getAllSaleOrders;
    const { data, error, isLoading, isValidating, mutate } = useSWR<SaleOrder[]>(URL, fetcher);

    const memoizedValue = useMemo(() => ({
        saleOrdersData: data as SaleOrder[] || [],
        saleOrdersError: error,
        saleOrdersLoading: isLoading,
        saleOrdersValidating: isValidating,
        mutateSaleOrders: mutate,
    }), [data, error, isLoading, isValidating]);

    return memoizedValue;
}

export function getSaleOrderById(saleOrderId: string | number | undefined) {
    const { data, error, isLoading, isValidating, mutate } = useSWR<SaleOrder>(
        saleOrderId ? `${endpoints.saleOrder.getSaleOrderById}/${saleOrderId}` : null,
        fetcher
    );

    const memoizedValue = useMemo(() => ({
        saleOrderData: data as SaleOrder,
        saleOrderError: error,
        saleOrderLoading: isLoading,
        saleOrderValidating: isValidating,
        mutateSaleOrder: mutate,
    }), [data, error, isLoading, isValidating]);

    return memoizedValue;
}

export async function confirmSaleOrderCreation(data: ConfirmSaleOrderRequest) {
    const payload: ConfirmSaleOrderRequest = {
        saleOrderId: data.saleOrderId,
        confirmedBy: data.confirmedBy,
        deliveryDate: data.deliveryDate,
        supplierId: data.supplierId,
        driverId: data.driverId,
        orderItems: data.orderItems,
    };

    try {
        const response = await axiosInstance.post(endpoints.saleOrder.confirm, payload);
        const newData: SupplierOrder = response.data;
        return newData;
    } catch (error) {
        console.error("Erreur lors de la confirmation de la commande de vente:", error);
        throw error;
    }
}

export const generateSaleOrder = async (id: string) => {
    try {
        const body = {
            startDate: getTodayAtTwoAM(),
            endDate: getCurrentDate(),
        };
        const response = await axiosInstance.post(endpoints.saleOrder.generate(id), body);
        return response.data;
    } catch (error) {
        throw new Error('Failed to generate sale order');
    }
};

export const deleteSaleOrder = async (id: string) => {
    try {
        const response = await axiosInstance.delete(endpoints.saleOrder.delete(id));
        return response.data;
    } catch (error) {
        throw new Error('Failed to delete sale order');
    }
};

export const deleteSaleOrderItem = async (id: string) => {
    try {
        const response = await axiosInstance.delete(endpoints.saleOrder.deleteItem(id));
        return response.data;
    } catch (error) {
        throw new Error('Failed to delete the product');
    }
};

export const deleteSaleOrders = async (saleOrderIds: string[]) => {
    try {
        const response = await axiosInstance.delete(endpoints.saleOrder.bulkDelete, {
            data: saleOrderIds
        });
        return response.data;
    } catch (error) {
        throw new Error('Failed to delete sale orders');
    }
};

export const formatDateForLocalDateTime = (date: Date) => {
    const offsetInMs = date.getTimezoneOffset() * 60000;
    const localTime = new Date(date.getTime() - offsetInMs);
    return localTime.toISOString().slice(0, 19);
};

const getPreviousDayAtTwoAM = () => {
    const now = new Date();
    const previousDay = new Date(now);
    previousDay.setDate(now.getDate() - 1);
    previousDay.setHours(2, 0, 0, 0);
    return formatDateForLocalDateTime(previousDay);
};

const getTodayAtTwoAM = () => {
    const now = new Date();
    const todayAtTwoAM = new Date(now);
    todayAtTwoAM.setHours(2, 0, 0, 0);

    if (now.getTime() >= todayAtTwoAM.getTime()) {
        return formatDateForLocalDateTime(todayAtTwoAM);
    } else {
        return getPreviousDayAtTwoAM();
    }
};

const getCurrentDate = () => {
    return formatDateForLocalDateTime(new Date());
};


export function useSearchSaleOrders({
    name = "",
    page = 0,
    size = 10,
    creationDate = null,
    sortBy = "createdAt",
    sortDir = "asc",
  }: SearchSaleOrderParams) {
    const filterParams = new URLSearchParams({
      name: name || "",
      creationDate: creationDate ? creationDate.toString() : "",
      page: page.toString(),
      size: size.toString(),
      sortBy: sortBy || "",
      sortDir: sortDir || "",
    });
  
    const url = `${endpoints.saleOrder.getSaleOrderPage}?${filterParams.toString()}`;
    const { data, error, isValidating, mutate } = useSWR<ISaleOrderPage>(url, fetcher);
  
    return {
      saleOrders: data?.content || [],
      totalSaleOrders: data?.totalElements || 0,
      totalPages: data?.totalPages || 0,
      pageNumber: data?.pageable.pageNumber || 0,
      pageSize: data?.pageable.pageSize || 0,
      loading: !data && !error,
      isError: error,
      isValidating,
      revalidate: mutate,
    };
  }
