import { endpoints, axiosInstance, fetcher } from "./server";
import useSWR, { mutate, useSWRConfig } from 'swr';
import { useMemo } from 'react';
import { IUserItem, IUnregistredClientItem, ILiveItem, ISessionOrder, IClientPage, SearchClientParams, SearchSaleSessionParams, ISaleSessionPage, IUnregistredClientPage } from '../types/saleSession';
import { AxiosResponse } from "axios";

export function useGetClients() {
  const URL = endpoints.salleSession.getClients;

  const { data, error, isLoading, isValidating } = useSWR<IUserItem[]>(URL, fetcher, {
    revalidateOnFocus: false,
    revalidateOnReconnect: false,
  });

  const memoizedValue = useMemo(
    () => ({
      clients: data || [],
      clientsLoading: isLoading,
      clientsError: error,
      clientsValidating: isValidating,
      clientsEmpty: !isLoading && (!data || data.length === 0),
      revalidate: () => mutate(URL),
    }),
    [data, error, isLoading, isValidating]
  );

  return memoizedValue;
}
export function useUpdateClient() {
  const updateClient = async (id: string, client: IUserItem) => {
    try {
      const response = await axiosInstance.put(endpoints.salleSession.updateClient(id), client);

      await mutate(endpoints.salleSession.getClients);
      return response.data;
    } catch (error) {
      console.error('Failed to update client', error);
      throw error;
    }
  };

  const memoizedValue = useMemo(
    () => ({
      updateClient,
    }),
    []
  );

  return memoizedValue;
}

export function useBanClient() {
  const banClient = async (clientId: string) => {
    try {
      const response = await axiosInstance.put(endpoints.salleSession.banClient(clientId));


      await mutate(endpoints.salleSession.getClients);
      return response.data;
    } catch (error) {
      console.error('Failed to ban client', error);
      throw error;
    }
  };

  const memoizedValue = useMemo(
    () => ({
      banClient,
    }),
    []
  );

  return memoizedValue;
}

export function useUnbanClient() {
  const unbanClient = async (clientId: string) => {
    try {
      const response = await axiosInstance.put(endpoints.salleSession.unbanClient(clientId));
      await mutate(endpoints.salleSession.getClients);
      return response.data;
    } catch (error) {
      console.error('Failed to unban client', error);
      throw error;
    }
  };

  const memoizedValue = useMemo(
    () => ({
      unbanClient,
    }),
    []
  );

  return memoizedValue;
}


export function useGetUnregistredClients() {
  const URL = endpoints.salleSession.getUnregistredClients;

  const { data, error, isLoading, isValidating } = useSWR<IUnregistredClientItem[]>(URL, fetcher, {
    revalidateOnFocus: false,
    revalidateOnReconnect: false,
  });

  const memoizedValue = useMemo(
    () => ({
      clients: data || [],
      clientsLoading: isLoading,
      clientsError: error,
      clientsValidating: isValidating,
      clientsEmpty: !isLoading && (!data || data.length === 0),
      revalidate: () => mutate(URL),
    }),
    [data, error, isLoading, isValidating]
  );

  return memoizedValue;
}


export function useCreateUnregisteredClient() {
  const createClient = async (pseudo: string) => {
    try {

      const response = await axiosInstance.post(endpoints.salleSession.addUnregistredClient, null, {
        params: {
          pseudo: pseudo, // Passing the pseudo as a query parameter
        }
      });

      await mutate(endpoints.salleSession.getUnregistredClients);
      return response.data;
    } catch (error) {
      console.error('Failed to create unregistered client', error);
      throw error;
    }
  };

  const memoizedValue = useMemo(
    () => ({
      createClient,
    }),
    []
  );

  return memoizedValue;
}

export function useAssociateAndDeleteClient() {
  const associateAndDeleteClient = async (unregisteredClientId: string, newUserId: string) => {
    try {
      const response = await axiosInstance.post(endpoints.salleSession.associateAndDelete, null, {
        params: {
          unregisteredClientId,
          newUserId,
        },
      });

      await mutate(endpoints.salleSession.getUnregistredClients);
      return response.data;
    } catch (error) {
      console.error("Failed to associate and delete client", error);
      throw error;
    }
  };

  return { associateAndDeleteClient };
}

export function useAddSaleSession() {
  const addSaleSession = async (saleSessionData: any) => {
    try {
      const response = await axiosInstance.post(endpoints.salleSession.addSaleSession, saleSessionData);

      // Met à jour la liste des sessions après l'ajout
      await mutate(endpoints.salleSession.getSessions);
      return response.data;
    } catch (error) {
      console.error('Failed to add sale session', error);
      throw error;
    }
  };

  const memoizedAddSaleSession = useMemo(() => ({ addSaleSession }), [addSaleSession]);

  return memoizedAddSaleSession;
}

export function useGetSaleSessions() {
  const URL = endpoints.salleSession.getSessions;

  const { data, error, isLoading, isValidating, mutate } = useSWR<ILiveItem[]>(URL, fetcher, {
    revalidateOnFocus: false,
    revalidateOnReconnect: false,
  });

  const memoizedValue = useMemo(
    () => ({
      sessions: data || [],
      sessionsLoading: isLoading,
      sessionsError: error,
      sessionsValidating: isValidating,
      sessionsEmpty: !isLoading && (!data || data.length === 0),
      mutate: mutate,
    }),
    [data, error, isLoading, isValidating]
  );

  return memoizedValue;
}

export function useGetSaleSessionsRange() {
  const URL = endpoints.salleSession.dateRangeSessions;

  const { data, error, isLoading, isValidating, mutate } = useSWR<ILiveItem[]>(URL, fetcher, {
    revalidateOnFocus: false,
    revalidateOnReconnect: false,
  });

  const memoizedValue = useMemo(
    () => ({
      sessions: data || [],
      sessionsLoading: isLoading,
      sessionsError: error,
      sessionsValidating: isValidating,
      sessionsEmpty: !isLoading && (!data || data.length === 0),
      mutate: mutate,
    }),
    [data, error, isLoading, isValidating]
  );

  return memoizedValue;
}


export function useGetSaleSession(id: string) {
  const URL = endpoints.salleSession.getSaleSessionById(id)
  const { data, isLoading, isValidating, error, mutate } = useSWR<ILiveItem>(
    URL,
    fetcher
  );

  const memoizedValue = useMemo(
    () => ({
      SaleSessionData: (data as ILiveItem) || null,
      SaleSessionLoading: isLoading,
      SaleSessionError: error,
      SaleSessionValidating: isValidating,
      SaleSessionMutate: mutate,
    }),

    [data, error]
  );
  return memoizedValue;
}

export async function useUpdateExpirationDate(
  date: string,
  saleSessionId: string
) {
  try {
    // Conversion de la chaîne de date en un objet Date
    const parsedDate = new Date(date);

    // Formater la date dans le format attendu par LocalDateTime : "yyyy-MM-dd'T'HH:mm:ss"
    const formattedDate = parsedDate.toISOString().slice(0, 19); // "YYYY-MM-DDTHH:MM:SS"

    const URL = endpoints.salleSession.modifieExpirationDate(saleSessionId);
    const response: AxiosResponse<ILiveItem> = await axiosInstance.put(
      URL,
      { expirationDate: formattedDate },  // Assurez-vous d'envoyer un objet JSON
      {
        headers: {
          "Content-Type": "application/json",  // Spécifiez le type de contenu
        },
      }
    );

    return response;
  } catch (error) {
    console.error("Erreur lors de la mise à jour de la date d'expiration de la session:", error);
    throw error;
  }
}


// Add product to registered cart
export function useAddProductToCartFromLive() {
  const addProductToCartFromLive = async (sessionRequest: any) => {
    try {
      const response = await axiosInstance.post(endpoints.salleSession.addProductToCart, sessionRequest);

      await mutate(endpoints.salleSession.getOrdersBySessionId);
      await mutate("/api/stock/products/ProductWithVariations");

      return response.data;
    } catch (error: any) {
      console.error("Erreur lors de l'ajout du produit au panier:", error);
      throw error;
    }
  };

  const memoizedValue = useMemo(() => ({ addProductToCartFromLive }), []);

  return memoizedValue;
}

// Add product to unregistered cart
export function useAddProductToUnregisteredCartFromLive() {
  const addProductToUnregisteredCartFromLive = async (sessionRequest: any) => {
    try {
      const response = await axiosInstance.post(endpoints.salleSession.addProductToUnregisteredCart, sessionRequest);

      await mutate(endpoints.salleSession.getOrdersBySessionId);
      await mutate("/api/stock/products/ProductWithVariations");

      return response.data;
    } catch (error: any) {
      console.error("Erreur lors de l'ajout du produit au panier:", error);
      throw error;
    }
  };

  const memoizedValue = useMemo(() => ({ addProductToUnregisteredCartFromLive }), []);

  return memoizedValue;
}
export function useGetAllSessionOrders() {
  const URL = endpoints.salleSession.getAllSessionOrders;

  const { data, error, isLoading, isValidating } = useSWR<ISessionOrder[]>(URL, fetcher, {
    revalidateOnFocus: false,
    revalidateOnReconnect: false,
  });

  const memoizedValue = useMemo(
    () => ({
      orders: data || [],
      ordersLoading: isLoading,
      ordersError: error,
      ordersValidating: isValidating,
      ordersEmpty: !isLoading && (!data || data.length === 0),
      revalidate: () => mutate(URL),
    }),
    [data, error, isLoading, isValidating]
  );

  return memoizedValue;
}
export function useGetOrdersBySessionId(sessionId: string) {
  const URL = endpoints.salleSession.getOrdersBySessionId(sessionId);

  const { data, error, isLoading, isValidating } = useSWR<ISessionOrder[]>(URL, fetcher, {
    revalidateOnFocus: false,
    revalidateOnReconnect: false,
  });

  const memoizedValue = useMemo(
    () => ({
      orders: data || [],
      ordersLoading: isLoading,
      ordersError: error,
      ordersValidating: isValidating,
      ordersEmpty: !isLoading && (!data || data.length === 0),
      revalidate: () => mutate(URL),
    }),
    [data, error, isLoading, isValidating]
  );

  return memoizedValue;
}
export function useUpdateOrderQuantity() {
  const updateOrderQuantity = async (orderId: string, quantity: number) => {
    try {
      const response = await axiosInstance.put(
        endpoints.salleSession.updateOrderQuantity(orderId),
        null,
        { params: { quantity } }
      );
      await mutate(endpoints.salleSession.getOrdersBySessionId);
      return response.data;
    } catch (error: any) {
      console.error("Failed to update order quantity", error);
      throw error;
    }
  };

  const memoizedValue = useMemo(() => ({ updateOrderQuantity }), []);

  return memoizedValue;
}
export function useDeleteOrder() {
  const deleteOrder = async (orderId: string) => {
    try {
      const response = await axiosInstance.delete(endpoints.salleSession.deleteOrder(orderId));
      await mutate(endpoints.salleSession.getOrdersBySessionId);
      return response.data;
    } catch (error) {
      console.error("Failed to delete order", error);
      throw error;
    }
  };

  const memoizedValue = useMemo(() => ({ deleteOrder }), []);

  return memoizedValue;
}

export const deleteUnregisteredClientById = async (id: number) => {
  try {
    const response = await axiosInstance.delete(`${endpoints.salleSession.deleteUnregisteredClientById}/${id}`);
    return response;
  } catch (error) {
    throw error;
  }
};
export const restoreSaleSession = async (id: number) => {
  try {
    const response = await axiosInstance.put(`${endpoints.salleSession.restoreSaleSession(id)}`);
    return response;
  } catch (error) {
    throw error;
  }
};
export function useUpdateSaleSession() {
  const { mutate } = useSWRConfig();

  const updateSaleSession = async (sessionId: string, data: { name: string; description: string }) => {
    try {
      const URL = endpoints.salleSession.updateDetails(sessionId);
      const response: AxiosResponse<ILiveItem> = await axiosInstance.put(URL, data, {
        headers: {
          'Content-Type': 'application/json',
        },
      });

      await mutate(endpoints.salleSession.getSessions, true);
      return response.data;
    } catch (error) {
      console.error('Failed to update sale session', error);
      throw error;
    }
  };

  return useMemo(
    () => ({
      updateSaleSession,
    }),
    []
  );
}
export const deleteSaleSessionById = async (id: number) => {

  try {
    const token = localStorage.getItem('token');
    const response = await axiosInstance.delete(endpoints.salleSession.deleteSaleSessionById(id), {
      headers: {
        Authorization: `Bearer ${token}`,
      },
    });
    await mutate(endpoints.salleSession.getSessions);
    return response;
  } catch (error) {
    console.error("Failed to delete sale session", error);
    throw error;
  }
};


export function useClientsCountsByStatus({
  keyword = "",
  status = ""
}: {
  keyword?: string;
  status?: string;
}) {
  const params = {
    keyword,
    status
  };

  const url = `${endpoints.user.clientStatusCount}?${new URLSearchParams(
    params
  ).toString()}`;

  const { data, error } = useSWR(url, async () => {
    try {
      const response = await axiosInstance.get(endpoints.user.clientStatusCount, {
        params,
      });
      return response.data;
    } catch (err) {
      console.error("Error fetching user counts by status:", err);
      throw err;
    }
  });

  return useMemo(
    () => ({
      clientCounts: data || [],
      clientCountsError: error,
    }),
    [data, error]
  );
}


export function useSearchClient({
  keyword = "",
  status = "",
  page = 0,
  size = 10,
  sortBy = "createdAt",
  sortDir = "ASC",
}: SearchClientParams) {
  const filterParams = new URLSearchParams({
    keyword: keyword || "",
    status: status || "",
    page: page.toString(),
    size: size.toString(),
    sortBy: sortBy || "",
    sortDir: sortDir || "",
  });

  const url = `${endpoints.user.getClients}?${filterParams.toString()}`;
  const { data, error, isValidating, mutate } = useSWR<IClientPage>(url, fetcher);

  return {
    clients: data?.content || [],
    totalClients: data?.totalElements || 0,
    totalPages: data?.totalPages || 0,
    pageNumber: data?.pageable.pageNumber || 0,
    pageSize: data?.pageable.pageSize || 0,
    loading: !data && !error,
    isError: error,
    isValidating,
    revalidate: mutate,
  };
}


export function useSearchSaleSession({
  name = "",
  type = "",
  page = 0,
  size = 10,
  startDate = null,
  endDate = null,
  sortBy = "id",
  sortDir = "ASC",
}: SearchSaleSessionParams) {
  const filterParams = new URLSearchParams({
    name: name || "",
    type: type || "",
    page: page.toString(),
    size: size.toString(),
    sortBy: sortBy || "",
    sortDir: sortDir || "",
    startDate: startDate?.toString() || "",
    endDate: endDate?.toString() || "",
  });
  const url = `${endpoints.salleSession.getSeleSessionPage}?${filterParams.toString()}`;
  const { data, error, isValidating, mutate } = useSWR<ISaleSessionPage>(url, fetcher);

  return {
    saleSessions: data?.content || [],
    totalSaleSessions: data?.totalElements || 0,
    totalPages: data?.totalPages || 0,
    pageNumber: data?.pageable.pageNumber || 0,
    pageSize: data?.pageable.pageSize || 0,
    loading: !data && !error,
    isError: error,
    isValidating,
    revalidatePage: mutate,
  };
}


export function useSaleSessionCountsByType({
  name = "",
  type = "",
  startDate = null,
  endDate = null,
}: {
  name?: string;
  type?: string;
  startDate?: Date|null;
  endDate?: Date|null;
}) {
  const params = {
    name,
    type,
    startDate: startDate ? startDate.toString() : "",
    endDate: endDate ? endDate.toString() : "",
  };

  const url = `${endpoints.user.statusCount}?${new URLSearchParams(
    params
  ).toString()}`;

  const { data, error } = useSWR(url, async () => {
    try {
      const response = await axiosInstance.get(endpoints.salleSession.getTypeCount, {
        params, 
      });
      return response.data;
    } catch (err) {
      console.error("Error fetching user counts by status:", err);
      throw err;
    }
  });

  return useMemo(
    () => ({
      saleSessionTypeCount: data || [],
      saleSessionTypeCountError: error,
    }),
    [data, error]
  );
}


export function useSearchDeletedSaleSession({
  name = "",
  type = "",
  page = 0,
  size = 10,
  startDate = null,
  endDate = null,
  sortBy = "id",
  sortDir = "ASC",
}: SearchSaleSessionParams) {
  const filterParams = new URLSearchParams({
    name: name || "",
    type: type || "",
    page: page.toString(),
    size: size.toString(),
    sortBy: sortBy || "",
    sortDir: sortDir || "",
    startDate: startDate?.toString() || "",
    endDate: endDate?.toString() || "",
  });
  const url = `${endpoints.salleSession.getDeletedSeleSessionPage}?${filterParams.toString()}`;
  const { data, error, isValidating, mutate } = useSWR<ISaleSessionPage>(url, fetcher);

  return {
    saleSessions: data?.content || [],
    totalSaleSessions: data?.totalElements || 0,
    totalPages: data?.totalPages || 0,
    pageNumber: data?.pageable.pageNumber || 0,
    pageSize: data?.pageable.pageSize || 0,
    loading: !data && !error,
    isError: error,
    isValidating,
    revalidatePage: mutate,
  };
}


export function useDeletedSaleSessionCountsByType({
  name = "",
  type = "",
  startDate = null,
  endDate = null,
}: {
  name?: string;
  type?: string;
  startDate?: Date|null;
  endDate?: Date|null;
}) {
  const params = {
    name,
    type,
    startDate: startDate ? startDate.toString() : "",
    endDate: endDate ? endDate.toString() : "",
  };

  const url = `${endpoints.user.statusCount}?${new URLSearchParams(
    params
  ).toString()}`;

  const { data, error } = useSWR(url, async () => {
    try {
      const response = await axiosInstance.get(endpoints.salleSession.getDeletedTypeCount, {
        params, 
      });
      return response.data;
    } catch (err) {
      console.error("Error fetching user counts by status:", err);
      throw err;
    }
  });

  return useMemo(
    () => ({
      saleSessionTypeCount: data || [],
      saleSessionTypeCountError: error,
    }),
    [data, error]
  );
}


export function useSearchUnregistredClient({
  pseudo = "",
  page = 0,
  size = 10,
  sortBy = "id",
  sortDir = "DESC",
}: {
  pseudo?: string;
  page?: Number;
  size?: Number;
  sortBy?: string;
  sortDir?: string;
}) {
  const filterParams = new URLSearchParams({
    pseudo: pseudo || "",
    page: page.toString(),
    size: size.toString(),
    sortBy: sortBy || "",
    sortDir: sortDir || "",
  });
  const url = `${endpoints.salleSession.getUnregisteredClientPage}?${filterParams.toString()}`;
  const { data, error, isValidating, mutate } = useSWR<IUnregistredClientPage>(url, fetcher);

  return {
    unregistredClients: data?.content || [],
    totalUnregistredClients: data?.totalElements || 0,
    totalPages: data?.totalPages || 0,
    pageNumber: data?.pageable.pageNumber || 0,
    pageSize: data?.pageable.pageSize || 0,
    loading: !data && !error,
    isError: error,
    isValidating,
    revalidate: mutate,
  };
}