import { RefundReason } from '@/contexts/types/refundOrder';
import axiosInstance, { endpoints } from './server';


export interface SeriesDataDTO {
  name: string;
  type: string;
  fill: string;
  data: number[];
}

export interface ChartDataDTO {
  labels: string[];
  series: SeriesDataDTO[];
}

/////////////////////


export interface DataPoint {
  name: string;
  data: number[];
}

export type RefundReasonType = {
  [RefundReason.CUSTOMER_RETURN]: number;
  [RefundReason.STOCK_ERROR]: number;
  [RefundReason.DEFECT]: number
}

export interface Series {
  type: string;
  data: DataPoint[];
}

export interface ChartData {
  categories: string[];
  series: Series[];
}

export const fetchAddToCartTrends = async (params: {
  source?: string;
  startDate?: string;
  endDate?: string;
}) => {
  try {
    const response = await axiosInstance.get<ChartDataDTO>(
      '/api/cart/statistics/add-to-cart-trends',
      {
        params,
        paramsSerializer: (params) => new URLSearchParams(params).toString(),
      }
    );
    return response.data;
  } catch (error) {
    console.error("Erreur lors de la récupération des tendances d'ajout au panier :", error);
    throw error;
  }
};


export const getTheoreticalRevenue = async (params: {
  sources?: string[];
  startDate?: string;
  endDate?: string;
}) => {
  const { data } = await axiosInstance.get<number>(
    '/api/cart/statistics/theoretical-revenue',
    {
      params,
      paramsSerializer: (params) =>
        new URLSearchParams(params).toString(),
    }
  );
  return data;
};

export const fetchTopSellingProducts = async (startDate: Date, endDate: Date, sources: string[]) => {
  const params = {
    startDate: startDate.toISOString(),
    endDate: endDate.toISOString(),
    sources: sources.join(","),
  };

  try {
    const response = await axiosInstance.get('/api/orders/statistics/top-selling-products', { params });
    return response.data;
  } catch (error) {
    console.error("Erreur lors de la récupération des produits les plus vendus :", error);
    throw error;
  }
};



export const fetchRealRevenue = async (startDate: Date, endDate: Date, sources: string[]) => {
  const params = {
    startDate: startDate.toISOString(),
    endDate: endDate.toISOString(),
    sources: sources.join(","),
  };
  
  try {
    const response = await axiosInstance.get('/api/orders/statistics/real-revenue', { params });
    return response.data; 
  } catch (error) {
    console.error("Erreur lors de la récupération du chiffre d'affaire réel :", error);
    throw error;
  }
};

export const fetchRealRevenueStats = async (startDate: Date, endDate: Date, sources: string[], shippingMethods?: string[], filterStatus?: string) => {
  const params = {
    startDate: startDate.toISOString(),
    endDate: endDate.toISOString(),
    sources: sources.join(","),
    filterStatus: filterStatus,
    shipping: shippingMethods?.join(",")
  };

  try {
    const response = await axiosInstance.get('/api/orders/statistics/real-revenue-stats', { params });
    return response.data;
  } catch (error) {
    console.error("Erreur lors de la récupération du chiffre d'affaire réel :", error);
    throw error;
  }
};

export const fetchRealRefundRevenue = async (startDate: Date, endDate: Date, sources: string[], filterStatus: string, shippingMethods:string[]) => {
  const params = {
    startDate: startDate.toISOString(),
    endDate: endDate.toISOString(),
    sources: sources.join(","),
    status: filterStatus,
    shipping: shippingMethods?.join(",")
  };

  try {
    const response = await axiosInstance.get('/api/orders/statistics/real-refund-revenue', { params });
    return response.data;
  } catch (error) {
    console.error("Erreur lors de la récupération du chiffre d'affaire réel :", error);
    throw error;
  }
};

export const fetchRefundReasons = async (startDate: Date, endDate: Date, sources: string[]) => {
  const params = {
    startDate: startDate.toISOString(),
    endDate: endDate.toISOString(),
    sources: sources.join(",")
  };

  try {
    const response = await axiosInstance.get('/api/orders/statistics/refund-count-by-reason', { params });
    return response.data;
  } catch (error) {
    console.error("Erreur lors de la récupération du chiffre d'affaire réel :", error);
    throw error;
  }
};


export const fetchTotalCarts = async (startDate: Date, endDate: Date, sources: string[]) => {
  const params = {
    startDate: startDate.toISOString(),
    endDate: endDate.toISOString(),
    sources: sources.join(","),
  };
  try {
    const response = await axiosInstance.get<number>('/api/cart/statistics/cart-count', { params });
    return response.data;
  } catch (error) {
    console.error("Erreur lors de la récupération du nombre total de paniers :", error);
    throw error;
  }
};

export const fetchTotalOrders = async (startDate: Date, endDate: Date, sources: string[]) => {
  const params = {
    startDate: startDate.toISOString(),
    endDate: endDate.toISOString(),
    sources: sources.join(","),
  };
  try {
    const response = await axiosInstance.get<number>('/api/orders/statistics/order-count', { params });
    return response.data;
  } catch (error) {
    console.error("Erreur lors de la récupération du nombre total de commandes :", error);
    throw error;
  }
};

export const fetchTotalOrdersStats = async (startDate: Date, endDate: Date, sources: string[], shippins: string[]) => {
  const params = {
    startDate: startDate.toISOString(),
    endDate: endDate.toISOString(),
    sources: sources.join(","),
    shippins: shippins.join(",")
  };
  try {
    const response = await axiosInstance.get<number>('/api/orders/statistics/order-count-stats', { params });
    return response.data;
  } catch (error) {
    console.error("Erreur lors de la récupération du nombre total de commandes :", error);
    throw error;
  }
};

export const fetchNetSalesData = async (startDate: Date, endDate: Date, sources: string[]) => {
  const params = {
    startDate: startDate.toISOString(),
    endDate: endDate.toISOString(),
    sources: sources.join(","),
  };
  try {
    const response = await axiosInstance.get('/api/orders/statistics/net-sales', { params });
    return response.data;
  } catch (error) {
    console.error("Erreur lors de la récupération des ventes nettes :", error);
    throw error;
  }
};

export const fetchRefundsStatsData = async (startDate: Date, endDate: Date) => {
  const params = {
    startDate: startDate.toISOString(),
    endDate: endDate.toISOString(),
  };
  try {
    const response = await axiosInstance.get('/api/orders/statistics/refunds-stats', { params });
    return response.data;
  } catch (error) {
    console.error("Erreur lors de la récupération des ventes nettes :", error);
    throw error;
  }
};

export const fetchOrdersStatsData = async (startDate: Date, endDate: Date) => {
  const params = {
    startDate: startDate.toISOString(),
    endDate: endDate.toISOString(),
  };
  try {
    const response = await axiosInstance.get('/api/orders/statistics/orders/stats', { params });
    return response.data;
  } catch (error) {
    console.error("Erreur lors de la récupération des ventes nettes :", error);
    throw error;
  }
};

export const fetchBasketComparisonChartData = async (startDate: Date, endDate: Date, sources: string[]) => {
  const params = {
    startDate: startDate.toISOString(),
    endDate: endDate.toISOString(),
    sources: sources.join(","),
  };

  try {
    const response = await axiosInstance.get<ChartData>(
      '/api/orders/statistics/basket-comparison-chart',
      { params }
    );
    return response.data;
  } catch (error) {
    console.error("Erreur lors de la récupération des données du graphique de comparaison du panier :", error);
    throw error;
  }
};

export interface ProductStatisticsDTO {
  id: number;
  name: string;
  imagePath: string;
  category: string;
  quantitySold: number;
  totalRevenue: number;
  productType: string;
}
