import { GATEWAY_API_URL } from "@/config-global";
import { getRequestAuthorization } from "@/utils/encryption";
import axios, { AxiosRequestConfig, CancelTokenSource } from "axios";

export const axiosInstance = axios.create({
  baseURL: GATEWAY_API_URL
});

axiosInstance.interceptors.request.use(
  (config: any) => {
    config.headers = {
      ...config.headers,
      requestAuthorization: getRequestAuthorization()
    };
    return config;
  },
  (error) => {
    return error;
  }
);

axiosInstance.interceptors.response.use(
  (res) => res,
  (error) =>
    Promise.reject(
      (error.response && error.response.data) || "Something went wrong"
    )
);

export default axiosInstance;

// -----------------------------CHAT-BOT-----------------------------------------
export const chatbotAxiosInstance = axios.create({
  baseURL: GATEWAY_API_URL,
});

chatbotAxiosInstance.interceptors.request.use(
  (config) => {
    const chatBotToken = localStorage.getItem("chatbotToken");
    if (chatBotToken) {
      config.headers.Authorization = `Bearer ${chatBotToken}`;
    }
    return config;
  },
  (error) => Promise.reject(error)
);

chatbotAxiosInstance.interceptors.response.use(
  (res) => res,
  (error) =>
    Promise.reject(
      (error.response && error.response.data) || "Something went wrong"
    )
);

export const fetcher = async (args: string | [string, AxiosRequestConfig]) => {
  const [url, config] = Array.isArray(args) ? args : [args];
  const res = await axiosInstance.get(url, { ...config });
  return res.data;
};

export const postFetcher = async (args: string | [string, any], requestData: any, cancelToken?: CancelTokenSource) => {
  const [url, config] = Array.isArray(args) ? args : [args];
  const res = await axiosInstance.post(url, requestData, {
    ...config,
    cancelToken: cancelToken ? cancelToken.token : undefined,  // Ajout du cancel token
  });
  return res.data;
};

export const fetcherchatbot = async (
  args: string | [string, AxiosRequestConfig],
  method: 'get' | 'post' | 'put' | 'delete' | 'patch',
  data?: any
) => {
  try {
    const [url, config] = Array.isArray(args) ? args : [args];
    const axiosConfig = { ...config, method };

    if (method === 'post' || method === 'put' || method === 'patch') {
      axiosConfig.data = data;
    }
    const res = await chatbotAxiosInstance(url, axiosConfig);

    return res.data;
  } catch (error) {
    console.error('Failed to fetch:', error);
    throw error;
  }
};


// Exemple d'utilisation pour annuler les requêtes précédentes
export const createCancelToken = (): CancelTokenSource => {
  return axios.CancelToken.source();
};


export const endpoints = {
  comments: {
    add: "/api/stream/chat/message",
    get: (liveId: string) => `/api/stream/chat/live?streamId=${liveId}`,
    delete: (commentId: string, liveId: string) =>
      `/api/stream/chat/live/delete?streamId=${liveId}&messageId=${commentId}`,
    pin: (commentId: string, liveId: string) =>
      `/api/stream/chat/live/pin?streamId=${liveId}&messageId=${commentId}`,
    unpin: (commentId: string, liveId: string) =>
      `/api/stream/chat/live/unpin?streamId=${liveId}&messageId=${commentId}`,
    blockUser: (senderName: string, liveId: string) =>
      `/api/stream/chat/block-user?streamId=${liveId}&senderName=${senderName}`,
    export: (liveId: string) => `/api/stream/chat/live/export?streamId=${liveId}`,
  },
  notification: {
    startLiveStreamNotification: () =>
      `/api/stream/notify/startLiveStream`,
  },

  referralSettings: {
    updateOrCreate: "/api/addons/referral-settings/updateOrCreate",
    get: "/api/addons/referral-settings",
  },
  referralShareSettings: {
    updateOrCreate: "/api/addons/referral-share-settings/updateOrCreate",
    get: "/api/addons/referral-share-settings",
  },

  referralPromoCodes: {
    getAll: "/api/addons/referral/",
  },
  referral: {
    getAll: "/api/addons/referral/referrer",
    totalReferredClients: "/api/addons/referral/total-referrer",
    countReferral: "/api/orders/statistics/count-referral"
  },
  product: {
    all: (page: number, size: number) =>
      `/api/stream/products/all?page=${page}&size=${size}`,
    searchProductWithIndex :(searchKey: string) => "/api/stock/products/searchProductByIndex?searchKey="+searchKey,
    search: (sku: string) => `/api/stream/products/search?sku=${sku}`,
    allProductsByIDs: "/api/stock/products/simple-product/retrieve-ids",
    allVariationsByIDs: "/api/stock/variations/variation/retrieve-ids",
    variation: (id: string) => `/api/stock/variations/${id}`,
    export: "/api/stock/products/export",
    getTagsAndEmplacementByUgs: (ugs: string) => `/api/stock/products/tags-and-emplacement-by-ugs?ugs="${ugs}`,
    downloadProductZebraFile: (productId: string) => `/api/stock/products/download-product-zebra-file/${productId}`,
  },
  live: {
    all: (page: number, size: number, sort: string, sortBy: string, status: string, search: string) => 
      `/api/stream/live/all?search=${search}&page=${page}&size=${size}&sortDir=${sort}&sortBy=${sortBy}&status=${status}`,
    allNoFilter: '/api/stream/live/all',
    serverTime: "/api/stream/live/server-time",
    client: "/api/stream/live/client",
    get: (liveId: string) => `/api/stream/live?liveId=${liveId}`,
    create: "/api/stream/live/create",
    update: (liveId: string) => `/api/stream/live/update?liveId=${liveId}`,
    start: (liveId: string) => `/api/stream/live/start?liveId=${liveId}`,
    stop: (liveId: string) => `/api/stream/live/stop?liveId=${liveId}`,
    publish: (liveId: string) => `/api/stream/live/publish?liveId=${liveId}`,
    delete: (liveId: string) => `/api/stream/live/delete?liveId=${liveId}`,
    archive: (liveId: string) => `/api/stream/live/archive?liveId=${liveId}`,
    restore: (liveId: string) => `/api/stream/live/restore?liveId=${liveId}`,
    ongoing: "/api/stream/live/ongoing",
    statusCount: "/api/stream/live/status-counts",
  },
  cards: {
    add: (userId: string, productId: string) =>
      `/cards/add?userId=${userId}&productId=${productId}`,
    all: (userId: string) => `/api/stream/cards/all?userId=${userId}`,
  },
  liveProducts: {
    presented: (liveId: string) => `/api/stream/live-products/presented?liveId=${liveId}`,
    all: (liveId: string) => `/api/stream/live-products/all?liveId=${liveId}`,
    add: (liveId: string) =>
      `/api/stream/live-products/add?liveId=${liveId}`,
    delete: (liveId: string, liveProductId: string) =>
      `/api/stream/live-products/delete?liveId=${liveId}&liveProductId=${liveProductId}`,
    present: (liveId: string, liveProductId: string, startTime: number) =>
      `/api/stream/live-products/present?liveId=${liveId}&liveProductId=${liveProductId}&startTime=${startTime}`,
    push: (liveProductId: string, startTime: number) =>
      `/api/stream/live-products/push?liveProductId=${liveProductId}&startTime=${startTime}`,
    hide: (liveProductId: string, endTime: number) =>
      `/api/stream/live-products/hide?liveProductId=${liveProductId}&endTime=${endTime}`,
    addTimeStamp: (liveProductId: string, startTime: number, endTime: number) =>
      `/api/stream/live-products/add-time-stamp?liveProductId=${liveProductId}&startTime=${startTime}&endTime=${endTime}`,
    deleteTimeStamp: (id: string) =>
      `/api/stream/live-products/delete-time-stamp?id=${id}`,
    updateTimeStamp: (id: string, startTime: number, endTime: number) =>
      `/api/stream/live-products/update-time-stamp?id=${id}&startTime=${startTime}&endTime=${endTime}`,
  },
  stats: {
    getStats: `/api/stream/stats`,
    getStreamStats: (liveId: string) => `/api/stream/stats?liveId=${liveId}`,
    getStreamSummary: (liveId: string) => `/api/stream/stats/summary?liveId=${liveId}`,
    getAllLivesSummary: `/api/stream/stats/summary/all`,
  },
  shipping: {
    shippingClass: {
      getAll: `/api/shipping/class`,
      getById: (shippingClassId: string) =>
        `/api/shipping/class/${shippingClassId}`,
      add: "/api/shipping/class",
      edit: (shippingClassId: string) =>
        `/api/shipping/class/${shippingClassId}`,
      delete: (shippingClassId: string) =>
        `/api/shipping/class/${shippingClassId}`,
    },
    shippingRules: {
      getShippingRules: `/api/shipping/rules/retrieve-shippingRules`,
      saveShippingRules: `/api/shipping/rules/save-shippingRules`,
    },
    shippingMethod: {
      getAll: `/api/shipping/method`,
      getById: (shippingMethodId: string) =>
        `/api/shipping/method/${shippingMethodId}`,
      editActive: (shippingMethodId: string) =>
        `/api/shipping/method/edit-active/${shippingMethodId}`,
      delete: (shippingMethodId: string) =>
        `/api/shipping/method/${shippingMethodId}`,
      freeShipping: {
        getAll: `/api/shipping/method/free-shipping`,
        getById: (shippingMethodId: string) =>
          `/api/shipping/method/free-shipping/${shippingMethodId}`,
        add: "/api/shipping/method/free-shipping",
        edit: (shippingMethodId: string) =>
          `/api/shipping/method/free-shipping/${shippingMethodId}`,
      },
      flatRate: {
        getAll: `/api/shipping/method/flat-rate`,
        getById: (shippingMethodId: string) =>
          `/api/shipping/method/flat-rate/${shippingMethodId}`,
        add: "/api/shipping/method/flat-rate",
        edit: (shippingMethodId: string) =>
          `/api/shipping/method/flat-rate/${shippingMethodId}`,
      },
      pointOfSale: {
        getAll: `/api/shipping/method/point-of-sale`,
        getById: (shippingMethodId: string) =>
          `/api/shipping/method/point-of-sale/${shippingMethodId}`,
        add: "/api/shipping/method/point-of-sale",
        edit: (shippingMethodId: string) =>
          `/api/shipping/method/point-of-sale/${shippingMethodId}`,
      },
      colissimo: {
        getAll: `/api/shipping/method/colissimo`,
        getById: (shippingMethodId: string) =>
          `/api/shipping/method/colissimo/${shippingMethodId}`,
        add: "/api/shipping/method/colissimo",
        edit: (shippingMethodId: string) =>
          `/api/shipping/method/colissimo/${shippingMethodId}`,
      },
      mondialRelay: {
        getAll: `/api/shipping/method/mondial-relay`,
        getById: (shippingMethodId: string) =>
          `/api/shipping/method/mondial-relay/${shippingMethodId}`,
        add: "/api/shipping/method/mondial-relay",
        edit: (shippingMethodId: string) =>
          `/api/shipping/method/mondial-relay/${shippingMethodId}`,
      },
    },
    shippingArea: {
      getAll: `/api/shipping/area`,
      getById: (shippingAreaId: string) =>
        `/api/shipping/area/${shippingAreaId}`,
      add: "/api/shipping/area",
      edit: (shippingAreaId: string) => `/api/shipping/area/${shippingAreaId}`,
      delete: (shippingAreaId: string) =>
        `/api/shipping/area/${shippingAreaId}`,
    },
    shipments: {
      getAll: `/api/shipping/shipment/all_shipments`,
      getById: (shipmentId: string) =>
        `/api/shipping/shipment/${shipmentId}`,
    },
  },
  media: {
    getAll: `/api/medias/all-medias`,
    getById: (mediaId: string) => `/api/medias/${mediaId}`,
    add: "/api/medias/upload-medias",
    edit: (mediaId: string) => `/api/medias/update/${mediaId}`,
    delete: `/api/medias/delete`,
    download: `/api/medias/download`,
  },
  order: {
    getAll: "/api/orders/all-orders",
    getById: (orderId: string) => `/api/orders/${orderId}`,
    exportByDateRange: `/api/orders/export-orders`,
    exportordersProduct: `/api/orders/export-orders-products`,
    updateOrderStatus: `/api/orders/update-status`,
    deleteOne: (orderId: string) => `/api/orders/${orderId}`,
    deleteNote: (noteId: number) => `/api/orders/note/delete/${noteId}`,
    addOrder:  `/api/orders/addProductFromBo`,
    getOrderHistory:(orderId : number, size : number, page : number) =>`/api/orders/orderHistory/${orderId}?page=${page}&size=${size}&sort=createdAt,desc` ,
    getlastOrderHistory:(orderId : number) =>`/api/orders/lastOrderHistory/${orderId}` ,
    getOrderToUpdate:(orderId : number) =>`/api/orders/getOrderToUpdate/${orderId}` ,
    updateOrder:`/api/orders/updateOrder` ,
    deleteList: `/api/orders/delete-orders`,
    refundOrder: {
      getAllRefundOrders: `/api/orders/refund-order`,
      exportRefundOrders: `/api/orders/refund-order/export-refunded-orders`,
      createRefundOrder: `/api/orders/refund-order`,
      getRefundOrdersByOrderId: (orderId: string) => `/api/orders/refund-order/fetch-by-order-id/${orderId}`,
    },
    filter:"/api/orders/filter",
    statusCount : 'api/orders/status-count',
  },
  user: {
    login: "/api/sync/user/login/backoffice",
    searchAndFetchRandomClients :(searchKey: string)=> "/api/sync/user/searchAndFetchRandomClients?searchKey="+searchKey,
    register: "api/sync/user/register",
    add: "api/sync/user/addUser",
    getAll: "api/sync/user/",
    addAdresss:"/api/sync/user/address/addAddress",
    updateAdress:"/api/sync/user/address/updateAddress",
    getUserById: (id: number) => `api/sync/user/${id}`,
    findUserById: (id: number) => `api/sync/user/findUserById/${id}`,
    getByEmail: (email: string) => `api/sync/user/getByEmail/${email}`,
    changingLogingActivity: "api/sync/user/changeLoginActivity",
    updateUser: `api/sync/user/updateUser`,
    updateClient: `api/sync/user/updateClient`,
    changeUserStatus: "api/sync/user/changeUserStatus",
    updateUserPassword: "api/sync/user/updatePassword",
    getAllRoles: "api/sync/user/roles/",
    addRole: "api/sync/user/roles/addRole",
    getClientsByIDs: (clientsIDs: number[]) => `api/sync/user/retrieve-clients-by-ids?clientsIDs=${clientsIDs.join(',')}`,
    getUsersByIDs: (usersIDs: number[]) => `api/sync/user/retrieve-users-by-ids?usersIDs=${usersIDs.join(',')}`,
    export: '/api/sync/user/export',
    searchUsers: '/api/sync/user/filter',
    statusCount : 'api/sync/user/status-counts',
    getClients  : 'api/sync/user/client/search',
    clientStatusCount : 'api/sync/user/clients/status-count',
  },
  stock_management: {
    products: {
      add: "/api/stock/products",
      list: "/api/stock/products/list",
      details: (productId: string) => `/api/stock/products/id/${productId}`,
      SaleSessionResearch: () => "/api/stock/products/ProductWithvariations",
      variations: (productId: string) => `/api/stock/variations/product/${productId}`,
    },
    categories: {
      getAll: () => "/api/stock/categories",
      importCategory: () => "/api/stock/categories/import",
      getById: (categoryId: string) => `/api/stock/categories/${categoryId}`,
      add: () => "/api/stock/categories/add-categorie",
      edit: (categoryId: string) => `/api/stock/categories/edit/${categoryId}`,
      delete: (categoryId: string) =>
        `/api/stock/categories/delete/${categoryId}`,
      deleteBunch: () => "/api/stock/categories/delete-bulk",
      deleteAfterUpdate: (categoryId: string , defaultCategoryId: string) => `/api/stock/categories/${categoryId}/delete-category-after-product/${defaultCategoryId}`
    },
    tags: {
      getAll: () => "/api/stock/tags",
      getById: (tagId: string) => `/api/stock/tags/${tagId}`,
      add: () => "/api/stock/tags/add-tag",
      edit: (tagId: string) => `/api/stock/tags/edit/${tagId}`,
      delete: (tagId: string) => `/api/stock/tags/delete/${tagId}`,
      deleteBunch: () => "/api/stock/tags/delete-bulk",
    },
    Attributs: {
      getAll: () => "/api/stock/attributs",
      getById: (attributId: string) => `/api/stock/attributs/${attributId}`,
      add: () => "/api/stock/attributs/add-attribut",
      edit: (attributId: string) => `/api/stock/attributs/edit/${attributId}`,
      delete: (attributId: string) =>
        `/api/stock/attributs/delete/${attributId}`,
      deleteBunch: () => "/api/stock/attributs/delete-bulk",
    },
    termColor: {
      getAllByAttribut: (attributId: string) =>
        `/api/stock/termColors/attribute/${attributId}`,
      getById: (colorId: string) => `/api/stock/termColors/${colorId}`,
      add: () => "/api/stock/termColors/add-color",
      edit: (colorId: string) => `/api/stock/termColors/edit/${colorId}`,
      delete: (colorId: string , attributId: string) => `/api/stock/termColors/${attributId}/delete/${colorId}`,
      deleteBunch: (attributId: string) => `/api/stock/termLabels/${attributId}/delete-bulk`,
    },
    termLabel: {
      getAllByAttribut: (attributId: string) =>
        `/api/stock/termLabels/attribute/${attributId}`,
      getById: (labelId: string) => `/api/stock/termLabels/${labelId}`,
      add: () => "/api/stock/termLabels/add-label",
      edit: (labelId: string) => `/api/stock/termLabels/edit/${labelId}`,
      delete: (labelId: string , attributId: string) => `/api/stock/termLabels/${attributId}/delete/${labelId}`,
      deleteBunch: (attributId: string) => `/api/stock/termLabels/${attributId}/delete-bulk`,
    },
    termChoix: {
      getAllByAttribut: (attributId: string) =>
        `/api/stock/termChoix/attribute/${attributId}`,
      getById: (choixId: string) => `/api/stock/termChoix/${choixId}`,
      add: () => "/api/stock/termChoix/add-choix",
      edit: (choixId: string) => `/api/stock/termChoix/edit/${choixId}`,
      delete: (choixId: string , attributId: string) => `/api/stock/termChoix/${attributId}/delete/${choixId}`,
      deleteBunch: (attributId: string) => `/api/stock/termLabels/${attributId}/delete-bulk`,
      checkSlugExists: (slug: string , attributid: string) => `api/stock/termLabels/exists?slug=${encodeURIComponent(slug)}&attributeId=${encodeURIComponent(attributid)}`,

    },
    termImage: {
      getAllByAttribut: (attributId: string) =>
        `/api/stock/termImage/attribute/${attributId}`,
      getById: (imageId: string) => `/api/stock/termImage/${imageId}`,
      add: () => "/api/stock/termImage/add-image",
      edit: (imageId: string) => `/api/stock/termImage/edit/${imageId}`,
      delete: (imageId: string  , attributId: string) => `/api/stock/termImage/${attributId}/delete/${imageId}`,
      deleteBunch: (attributId: string) => `/api/stock/termLabels/${attributId}/delete-bulk`,
    },
  },
  payment: {
    methods: {
      all: "/api/payment/config",
      type: (paymentMethod: string) => `/api/payment/config/${paymentMethod}`,
      changeStatus: (paymentMethod: string) =>
        `/api/payment/method/state/${paymentMethod}`,
    },
    bankTransferConfig: {
      get: "/api/payment/config/bank_transfer",
      edit: "/api/payment/config/bank_transfer",
    },
    creditCardConfig: {
      get: "/api/payment/config/credit_card",
      edit: "/api/payment/config/credit_card",
    },
    paypalConfig: {
      get: "/api/payment/config/paypal",
      edit: "/api/payment/config/paypal",
      getConnexionParams: "/api/paypal/config",
      updateConnexionParams: "/api/paypal/configure"
    },
    payPlug: {
      getConfig: "/api/payment/config/payplug",
      updateConfig: "/api/payment/config/payplug",
      fetchPaymentConfig: "/api/payment/payplug/config",
      savePaymentConfig: "/api/payment/payplug/config",
      getPayment: (paymentId: string) => `/api/payment/payplug/payments/${paymentId}`,
      createRefund: (paymentId: string) => `/api/payment/payplug/payments/${paymentId}/refunds`,
      getRefund: (paymentId: string, refundId: string) => `/api/payment/payplug/payments/${paymentId}/refunds/${refundId}`,
    },
  },
  supplierOrder: {
    getAll: "/api/stock/supplier-order/all",
    getExportSuppliers : "/api/stock/supplier-order/export_suppliers",
    getById: (id: string) => `/api/stock/supplier-order/${id}`,
    getOrderDetails: (id: string) => `/api/stock/supplier-order/${id}/details`,
    getByDriver: (id: string) => `/api/stock/supplier-order/driver/${id}`,
    new: (userId: string) => `/api/stock/supplier-order/new/${userId}`,
    addComment: (id: string) => `/api/stock/supplier-order/${id}/comment`,
    updateStatus: (id: String) => `/api/stock/supplier-order/${id}/status`,
    edit: `/api/stock/supplier-order/update`,
    updateSupplierOrderStatus: `/api/stock/supplier-order/update-supplier-order-status`,
    updateItemCollectionStatus: `/api/stock/supplier-order/update-item-collection-status`,
    updateItemReceiptStatus: `/api/stock/supplier-order/update-item-receipt-status`,
    updateAllStatus: `/api/stock/supplier-order/update-all-status`,
    delete: (id: string) => `/api/stock/supplier-order/${id}`,
    bulkDelete: "/api/stock/supplier-order/bulk-delete",
    deleteItem: (id: string) => `/api/stock/supplier-order/items/${id}`,
    downloadSupplierOrderZebraFile: (id: string) => `/api/stock/supplier-order/download-order-zebra-file/${id}`,
    downloadSupplierOrderItemZebraFile: (id: string) => `/api/stock/supplier-order/download-order-item-zebra-file/${id}`,
    declareDiscrepancy: "/api/stock/supplier-order/declare-discrepancy",
    getAllProductSummaryWithKeyword: (page: number, size: number, keyword: string) => `/api/stock/products/paginated_products_summary?page=${page}&size=${size}&keyword=${keyword}`,
    getSupplierOrderPage:"/api/stock/supplier-order",
    statusCount : '/api/stock/supplier-order/status',
  },
  salleSession: {
    getClients: `/api/sync/user/clients`,
    updateClient: (id: string) => `/api/sync/user/clients/${id}`,
    banClient: (clientId: string) => `/api/sync/user/clients/ban/${clientId}`,
    unbanClient: (clientId: string) => `/api/sync/user/clients/unban/${clientId}`,
    getUnregistredClients: `/api/sync/user/unregistered-client/`,
    addUnregistredClient: `/api/sync/user/unregistered-client/add`,
    associateAndDelete: "/api/sync/user/unregistered-client/associate-unregistered-client",
    addSaleSession: `/api/cart/add-saleSession`,
    getSessions: `/api/cart/saleSessions`,
    dateRangeSessions: `/api/cart/rangeSaleSessions`,
    addProductToCart: `/api/cart/addProductToCartFromLive`,
    addProductToUnregisteredCart: `/api/cart/addProductToUnregisteredCartFromLive`,
    getOrdersBySessionId: (sessionId: string) => `/api/cart/ordersSession/get-orders/${sessionId}`,
    getAllSessionOrders: `/api/cart/ordersSession/get-orders/all`,
    updateOrderQuantity: (orderId: string) => `/api/cart/ordersSession/update-quantity/${orderId}`,
    deleteOrder: (orderId: string) => `/api/cart/ordersSession/delete-order/${orderId}`,
    getSaleSessionById: (saleSessionId: string) => `/api/cart/SaleSession/${saleSessionId}`,
    modifieExpirationDate: (saleSessionId: string) => `api/cart/${saleSessionId}/expiration-date`,
    deleteUnregisteredClientById: `/api/sync/user/unregistered-client/deleteUnregisteredClient`,
    updateDetails: (saleSessionId: string) => `api/cart/${saleSessionId}/details`,
    deleteSaleSessionById: (id: number) => `/api/cart/deleteSession/${id}`,
    restoreSaleSession: (id: number) => `/api/cart/restock/${id}`,
    getSeleSessionPage: "/api/cart/page",
    getTypeCount:"/api/cart/count-type",
    getDeletedSeleSessionPage: "/api/cart/deleted/page",
    getDeletedTypeCount:"/api/cart/deleted/count-type",
    getUnregisteredClientPage:"/api/sync/user/unregistered-client/page"
  },
  supplier: {
    getAllSupplier: "/api/sync/user/supplier",
    getSupplierDetails: (id: string) => `/api/sync/user/supplier/${id}`,
    addSupplier: "/api/sync/user/supplier/add",
    deleteSupplier: (id: number) => `/api/sync/user/supplier/delete/${id}`,
    updateSupplier: "/api/sync/user/supplier/update",
    archiveSupplier: (id: number, isArchived: boolean) => `/api/sync/user/supplier/${id}/${isArchived}`,
    getArchivedSuppliers: "/api/sync/user/supplier/archived",
    getNotArchivedSuppliers: "/api/sync/user/supplier/notArchived",
  },
  driver: {
    getAllDrivers: "api/sync/user/driver",
    getDriverById: (id: string) => `api/sync/user/driver/${id}`,
  },
  saleOrder: {
    getAllSaleOrders: "/api/stock/sale-orders",
    getSaleOrderById: (id: string) => `/api/stock/sale-orders/${id}`,
    generate: (id: string) => `/api/stock/sale-orders/generate/${id}`,
    confirm: "/api/stock/sale-orders/confirm",
    delete: (id: string) => `/api/stock/sale-orders/${id}`,
    bulkDelete: "/api/stock/sale-orders/bulk-delete",
    deleteItem: (id: string) => `/api/stock/sale-orders/item/${id}`,
    getSaleOrderPage : "/api/stock/sale-orders/page",
  },
  cart: {
    getCart: (clientId: string) => `/api/cart/find/${clientId}`,
    deleteItem: "/api/cart/delete-itemBO",
    saveOrUpdate: "api/cart/cart-count-config",
    getClientCarts:"/api/cart",
  },
  Reviews: {
    Config: {
      getConfig: () => `/api/addons/configurations/1`,
      editConfig: () => `/api/addons/configurations/1`
    }
  },
  Addons: {
    GiftCart: {
      add: "/api/addons/giftcart",
      addGiftcardAndSendMail: "/api/addons/giftcart/createGiftcardAndSendMail",
      getAll: "/api/addons/giftcart",
      delete: (id: number) => `/api/addons/giftcart/delete/${id}`,
      update: (id: number) => `/api/addons/giftcart/edit/${id}`,
      duplicate: (id: number) => `/api/addons/giftcart/${id}/duplicate`,
      getById: (id: string) => `/api/addons/giftcart/${id}`,
      sendEmail: (id: number) => `/api/addons/giftcart/${id}/send-email`,
    },
    PromoCode: {
      getAll: "/api/addons/promoCode",
      getById: (id: string) => `/api/addons/promoCode/${id}`,
      checkPromoCodeExists: (code: string) => `/api/addons/promoCode/check-promoCode-exists/${code}`,
      checkPromoCodeIfUsed: (code: string) => `/api/addons/promoCode/check-promoCode-used/${code}`,
      add: "/api/addons/promoCode/add-promoCode",
      update: (id: string) => `/api/addons/promoCode/update-promoCode/${id}`,
      delete: (id: string) => `/api/addons/promoCode/delete-promoCode/${id}`,
      deleteByIds: "/api/addons/promoCode/delete-promoCodes-by-ids",
      getAllProductSummaryWithKeyword: (page: number, size: number, keyword: string) => `/api/stock/products/paginated_products_summary?page=${page}&size=${size}&keyword=${keyword}`,
      getProductsPublishedWebsite: (page: number, size: number, keyword: string) => `/api/stock/products/paginated_products_summary?page=${page}&size=${size}&keyword=${keyword}&publishWeb=true`,
    },
    Vouchers: {
      add: "/api/addons/vouchers",
      getAll: "/api/addons/vouchers",
      delete: (id: number) => `/api/addons/vouchers/delete/${id}`,
      update: (id: number) => `/api/addons/vouchers/edit/${id}`,
      duplicate: (id: number) => `/api/addons/vouchers/${id}/duplicate`,
      getById: (id: string) => `/api/addons/vouchers/${id}`,
      sendEmail: (id: number) => `/api/addons/vouchers/${id}/send-email`,

    }
  },
  chatbot : {
    auth: {
      currentUser: '/api/chatbot/users/current',
    },
    faq: {
      list: '/api/chatbot/faq/all',
      add: '/api/chatbot/faq/create',
      delete : '/api/chatbot/faq/delete',
      update: '/api/chatbot/faq/update',
      details: '/api/chatbot/faq',
    },
    help: {
      list: '/api/chatbot/help/collections/all',
      add: '/api/chatbot/help/collections/create',
      update: (collectionId: string) => `/api/chatbot/help/collections/update/${collectionId}`,
      delete: (collectionId: string) => `/api/chatbot/help/collections/delete/${collectionId}`,
      details: (collectionId: string) => `/api/chatbot/help/collections/${collectionId}`,
      article : {
        list: (collectionId: string) => `/api/chatbot/help/collections/${collectionId}/articles/all`,
        add: (collectionId: string) => `/api/chatbot/help/collections/${collectionId}/articles/create`,
        update: (collectionId: string, articleId: string) => `/api/chatbot/help/collections/${collectionId}/articles/update/${articleId}`,
        delete: (collectionId: string, articleId: string) => `/api/chatbot/help/collections/${collectionId}/articles/delete/${articleId}`,
        details: (collectionId: string, articleId: string) => `/api/chatbot/help/collections/${collectionId}/articles/${articleId}`,
      }
    },
    openai: {
      list :`/api/ai/documents/all`,
      add: `/api/ai/documents/ingest`,
      delete: (documentId: string) => `/api/ai/documents/${documentId}/delete`,
      update: (openaiId: string) => `/api/chatbot/openai/conf/${openaiId}/update`,
      details: (openaiId: string) => `/api/chatbot/openai/conf/${openaiId}`,
    },
    ticket: {
      details : (ticketId: string) => `/api/chatbot/tickets/${ticketId}`,
      list: '/api/chatbot/tickets/all',
      support: (supportId: string) => `/api/chatbot/tickets/support/${supportId}/all`,
      update: (ticketId: string) => `/api/chatbot/tickets/update/${ticketId}`,
    },
    support : {
      list: '/api/chatbot/users/support/all',
      details: (userId: string) => `/api/chatbot/users/${userId}`,
      conversation : '/api/chatbot/support/conversation',
    },
    chat : {
      all: '/api/chatbot/tickets/conversations/all',
      ticket: (conversation_id : string) => `/api/chatbot/tickets/${conversation_id}/conversation`,
      conversations: (support_id: string) => `/api/chatbot/tickets/conversations/${support_id}/all`,
      conversation: (conversation_id: string) => `/api/chatbot/conversations/${conversation_id}`,
    },
    user: {
      list: '/api/chatbot/users/all',
      details: (userId: string) => `/api/chatbot/users/${userId}`,
    }
  }
};
