import {
  Colissimo,
  FlatRate,
  FreeShipping,
  IShippingAreaItem,
  MondialRelay,
  PointOfSale,
  ShippingClass,
  ShippingMethod,
  ShippingMethodType,
  ShippingRules,
} from "@/contexts/types/Shipping";
import { useMemo } from "react";
import useSWR from "swr";
import { _orders } from "../_mock";
import axiosInstance, { endpoints, fetcher } from "./server";
import { mutate } from "swr/_internal";

const options = {
  revalidateIfStale: false,
  revalidateOnFocus: false,
  revalidateOnReconnect: false,
};

///////////////////-----------ShippingClass----------///////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
export function useGetShippingClasses() {
  const URL = [endpoints.shipping.shippingClass.getAll];
  const { data, error, isLoading, isValidating, mutate } = useSWR<ShippingClass[]>(URL, fetcher, options);

  const memoizedValue = useMemo(
    () => ({
      shippingClassData: (data as ShippingClass[]) || [],
      shippingClassError: error,
      shippingClassLoading: isLoading,
      shippingClassValidating: isValidating,
      mutateShippingClasses: mutate,
    }),
    [data, error]
  );

  return memoizedValue;
}

export function useGetShippingClass(shippingClassId: string) {
  const { data, error, isLoading, isValidating, mutate } = useSWR<ShippingClass>(
    shippingClassId
      ? endpoints.shipping.shippingClass.getById(shippingClassId)
      : null,
    fetcher
  );

  const memoizedValue = useMemo(
    () => ({
      shippingClass: data,
      shippingClassError: error,
      shippingClassLoading: isLoading,
      shippingClassValidating: isValidating,
      mutateShippingClass: mutate,
    }),
    [data, error]
  );

  return memoizedValue;
}

export async function useAddShippingClass(data: ShippingClass) {
  const formData = {
    name: data.name,
    code: data.code,
    description: data.description,
  };
  const response = await axiosInstance.post(
    endpoints.shipping.shippingClass.add,
    formData
  );
  const newData: ShippingClass = response.data;
  mutate(
    [endpoints.shipping.shippingClass.getAll],
    (shippingClasses: ShippingClass[] = []) => [...shippingClasses, newData],
    true
  );
  return newData;
}

export async function useUpdateShippingClass(
  id: string,
  updatedShippingClass: ShippingClass
) {
  const response = await axiosInstance.put(
    endpoints.shipping.shippingClass.edit(id),
    updatedShippingClass
  );
  const updatedData: ShippingClass = response.data;
  mutate(
    [endpoints.shipping.shippingClass.getAll],
    (shippingClasses: ShippingClass[] = []) =>
      shippingClasses.map((shippingClass) =>
        shippingClass.id === id ? updatedData : shippingClass
      ),
    true
  );
  return response.data as ShippingClass;
}

export async function useDeleteShippingClass(id: string) {
  const response = await axiosInstance.delete(
    endpoints.shipping.shippingClass.delete(id)
  );
  mutate(endpoints.shipping.shippingClass.getAll);
  return response.status;
}

///////////////////-----------ShippingRules----------///////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
export function useGetShippingRules() {
  const { data, error, isLoading, isValidating, mutate } = useSWR<ShippingRules>(
    endpoints.shipping.shippingRules.getShippingRules,
    fetcher
  );

  const memoizedValue = useMemo(
    () => ({
      shippingRulesData: data,
      shippingRulesError: error,
      shippingRulesLoading: isLoading,
      shippingRulesValidating: isValidating,
      mutateShippingRules: mutate,
    }),
    [data, error]
  );

  return memoizedValue;
}

export async function useSaveShippingRules(
  updatedShippingRules: ShippingRules
) {
  const response = await axiosInstance.post(
    endpoints.shipping.shippingRules.saveShippingRules,
    updatedShippingRules
  );
  const updatedData: ShippingRules = response.data;
  mutate(endpoints.shipping.shippingRules.getShippingRules, updatedData, true);
  return response.data as ShippingRules;
}

///////////////////-----------ShippingMethod----------///////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
export function useGetShippingMethods() {
  const URL = [endpoints.shipping.shippingMethod.getAll];
  const { data, error, isLoading, isValidating, mutate } = useSWR<ShippingMethod[]>(URL, fetcher, options);

  const memoizedValue = useMemo(
    () => ({
      shippingMethodData: (data as ShippingMethod[]) || [],
      shippingMethodError: error,
      shippingMethodLoading: isLoading,
      shippingMethodValidating: isValidating,
      mutateShippingMethods: mutate,
    }),
    [data, error]
  );

  return memoizedValue;
}

export function useGetShippingMethod(shippingMethodId: string) {
  const { data, error, isLoading, isValidating, mutate } = useSWR<ShippingMethod>(
    shippingMethodId
      ? endpoints.shipping.shippingMethod.getById(shippingMethodId)
      : null,
    fetcher
  );

  const memoizedValue = useMemo(
    () => ({
      shippingMethod: data,
      shippingMethodError: error,
      shippingMethodLoading: isLoading,
      shippingMethodValidating: isValidating,
      mutateShippingMethod: mutate,
    }),
    [data, error]
  );
  if (data) {
    data.type = ShippingMethodType.FreeShipping;
  }

  return memoizedValue;
}

export async function useDeleteShippingMethod(id: string) {
  const response = await axiosInstance.delete(
    endpoints.shipping.shippingMethod.delete(id)
  );
  mutate(endpoints.shipping.shippingMethod.getAll);
  return response.status;
}

export async function useUpdateShippingMethodActive(
  id: string,
  active: boolean
) {
  const response = await axiosInstance.put(
    endpoints.shipping.shippingMethod.editActive(id),
    active
  );
  const updatedData: ShippingMethod = response.data;
  mutate(endpoints.shipping.shippingMethod.getById(id), updatedData, true);
  return response.data as ShippingMethod;
}

///-------FreeShipping-------///
export async function useAddFreeShipping(data: FreeShipping) {
  const formData = {
    name: data.name,
    description: data.description,
    active: data.active,
    ordre: 5,
    requires: data.requires,
    minOrderAmount: data.minOrderAmount,
    applyMinOrderRuleBeforePromoCodeDiscount:
      data.applyMinOrderRuleBeforePromoCodeDiscount,
  };
  const response = await axiosInstance.post(
    endpoints.shipping.shippingMethod.freeShipping.add,
    formData
  );
  const newData: FreeShipping = response.data;
  mutate(
    [endpoints.shipping.shippingMethod.getAll],
    (shippingMethods: ShippingMethod[] = []) => [...shippingMethods, newData],
    true
  );
  console.log(newData);
  return newData;
}

export async function useUpdateFreeShipping(
  id: string,
  updatedShippingMethod: FreeShipping
) {
  const response = await axiosInstance.put(
    endpoints.shipping.shippingMethod.freeShipping.edit(id),
    updatedShippingMethod
  );
  const updatedData: ShippingMethod = response.data;
  mutate(
    endpoints.shipping.shippingMethod.getAll,
    (shippingMethods = []) => [
      ...shippingMethods.filter(
        (shippingMethod: ShippingMethod) => shippingMethod.id !== id
      ),
      updatedData,
    ],
    false
  );
  console.log(response.data);
  return response.data as FreeShipping;
}

export function useGetAllFreeShipping() {
  const URL = [endpoints.shipping.shippingMethod.getAll];
  const { data, error, isLoading, isValidating, mutate } = useSWR<FreeShipping[]>(URL, fetcher, options);

  const memoizedValue = useMemo(
    () => ({
      shippingMethodData: (data as FreeShipping[]) || [],
      shippingMethodError: error,
      shippingMethodLoading: isLoading,
      shippingMethodValidating: isValidating,
      mutateShippingMethods: mutate,
    }),
    [data, error]
  );

  return memoizedValue;
}

export function useGetFreeShipping(shippingMethodId: string) {
  const { data, error, isLoading, isValidating, mutate } = useSWR<FreeShipping>(
    shippingMethodId
      ? endpoints.shipping.shippingMethod.freeShipping.getById(shippingMethodId)
      : null,
    fetcher
  );

  const memoizedValue = useMemo(
    () => ({
      shippingMethod: data,
      shippingMethodError: error,
      shippingMethodLoading: isLoading,
      shippingMethodValidating: isValidating,
      mutateShippingMethod: mutate,
    }),
    [data, error]
  );

  return memoizedValue;
}

///-------FlatRate-------///
export async function useAddFlatRate(data: FlatRate) {
  const formData = {
    name: data.name,
    description: data.description,
    active: data.active,
    ordre: 5,
    tvaStatut: data.tvaStatut,
    cost: data.cost,
    costHeavyShippingClass: data.costHeavyShippingClass,
    costNoShippingClass: data.costNoShippingClass,
    calculationType: data.calculationType,
  };
  const response = await axiosInstance.post(
    endpoints.shipping.shippingMethod.flatRate.add,
    formData
  );
  const newData: FlatRate = response.data;
  mutate(
    [endpoints.shipping.shippingMethod.getAll],
    (shippingMethods: ShippingMethod[] = []) => [...shippingMethods, newData],
    true
  );
  console.log(newData);
  return newData;
}

export async function useUpdateFlatRate(
  id: string,
  updatedShippingMethod: FlatRate
) {
  const response = await axiosInstance.put(
    endpoints.shipping.shippingMethod.flatRate.edit(id),
    updatedShippingMethod
  );
  const updatedData: ShippingMethod = response.data;
  mutate(
    [endpoints.shipping.shippingMethod.getAll],
    (shippingMethods: ShippingMethod[] = []) =>
      shippingMethods.map((shippingMethod) =>
        shippingMethod.id === id ? updatedData : shippingMethod
      ),
    true
  );
  console.log(response.data);
  return response.data as FlatRate;
}

export function useGetAllFlatRate() {
  const URL = [endpoints.shipping.shippingMethod.flatRate.getAll];
  const { data, error, isLoading, isValidating, mutate } = useSWR<FlatRate[]>(URL, fetcher, options);

  const memoizedValue = useMemo(
    () => ({
      shippingMethodData: (data as FlatRate[]) || [],
      shippingMethodError: error,
      shippingMethodLoading: isLoading,
      shippingMethodValidating: isValidating,
      mutateShippingMethods: mutate,
    }),
    [data, error]
  );

  return memoizedValue;
}

export function useGetFlatRate(shippingMethodId: string) {
  const { data, error, isLoading, isValidating, mutate } = useSWR<FlatRate>(
    shippingMethodId
      ? endpoints.shipping.shippingMethod.flatRate.getById(shippingMethodId)
      : null,
    fetcher
  );

  const memoizedValue = useMemo(
    () => ({
      shippingMethod: data,
      shippingMethodError: error,
      shippingMethodLoading: isLoading,
      shippingMethodValidating: isValidating,
      mutateShippingMethod: mutate,
    }),
    [data, error]
  );

  return memoizedValue;
}

///-------PointOfSale-------///
export async function useAddPointOfSale(data: PointOfSale) {
  const formData = {
    name: data.name,
    description: data.description,
    active: data.active,
    ordre: 5,
    tvaStatut: data.tvaStatut,
    cost: data.cost,
  };
  const response = await axiosInstance.post(
    endpoints.shipping.shippingMethod.pointOfSale.add,
    formData
  );
  const newData: PointOfSale = response.data;
  mutate(
    [endpoints.shipping.shippingMethod.getAll],
    (shippingMethods: ShippingMethod[] = []) => [...shippingMethods, newData],
    true
  );
  console.log(newData);
  return newData;
}

export async function useUpdatePointOfSale(
  id: string,
  updatedShippingMethod: PointOfSale
) {
  const response = await axiosInstance.put(
    endpoints.shipping.shippingMethod.pointOfSale.edit(id),
    updatedShippingMethod
  );
  const updatedData: ShippingMethod = response.data;
  mutate(
    [endpoints.shipping.shippingMethod.getAll],
    (shippingMethods: ShippingMethod[] = []) =>
      shippingMethods.map((shippingMethod) =>
        shippingMethod.id === id ? updatedData : shippingMethod
      ),
    true
  );
  console.log(response.data);
  return response.data as PointOfSale;
}

export function useGetAllPointOfSale() {
  const URL = [endpoints.shipping.shippingMethod.pointOfSale.getAll];
  const { data, error, isLoading, isValidating, mutate } = useSWR<PointOfSale[]>(URL, fetcher, options);

  const memoizedValue = useMemo(
    () => ({
      shippingMethodData: (data as PointOfSale[]) || [],
      shippingMethodError: error,
      shippingMethodLoading: isLoading,
      shippingMethodValidating: isValidating,
      mutateShippingMethods: mutate,
    }),
    [data, error]
  );

  return memoizedValue;
}

export function useGetPointOfSale(shippingMethodId: string) {
  const { data, error, isLoading, isValidating, mutate } = useSWR<PointOfSale>(
    shippingMethodId
      ? endpoints.shipping.shippingMethod.pointOfSale.getById(shippingMethodId)
      : null,
    fetcher
  );

  const memoizedValue = useMemo(
    () => ({
      shippingMethod: data,
      shippingMethodError: error,
      shippingMethodLoading: isLoading,
      shippingMethodValidating: isValidating,
      mutateShippingMethod: mutate,
    }),
    [data, error]
  );

  return memoizedValue;
}

///-------Colissimo-------///
export async function useAddColissimo(data: Colissimo, signature: boolean) {
  const formData = {
    name: data.name,
    description: data.description,
    active: data.active,
    ordre: 0,
    signature: signature,
    alwaysFree: data.alwaysFree,
    nameFreeShipping: data.nameFreeShipping,
    shippingClassExcludedList: data.shippingClassExcludedList,
    freeShippingClassList: data.freeShippingClassList,
    freeIfOneProductHaveFreeShippingClass:
      data.freeIfOneProductHaveFreeShippingClass,
    deliveryCostColissimoList: data.deliveryCostColissimoList,
    deliveryReductionColissimoList: data.deliveryReductionColissimoList,
  };
  const response = await axiosInstance.post(
    endpoints.shipping.shippingMethod.colissimo.add,
    formData
  );
  const newData: Colissimo = response.data;
  mutate(
    [endpoints.shipping.shippingMethod.getAll],
    (shippingMethods: ShippingMethod[] = []) => [...shippingMethods, newData],
    true
  );
  console.log(newData);
  return newData;
}

export async function useUpdateColissimo(
  id: string,
  updatedShippingMethod: Colissimo
) {
  const response = await axiosInstance.put(
    endpoints.shipping.shippingMethod.colissimo.edit(id),
    updatedShippingMethod
  );
  const updatedData: ShippingMethod = response.data;
  mutate(
    [endpoints.shipping.shippingMethod.getAll],
    (shippingMethods: ShippingMethod[] = []) =>
      shippingMethods.map((shippingMethod) =>
        shippingMethod.id === id ? updatedData : shippingMethod
      ),
    true
  );
  console.log(response.data);
  return response.data as Colissimo;
}

export function useGetAllColissimo() {
  const URL = [endpoints.shipping.shippingMethod.colissimo.getAll];
  const { data, error, isLoading, isValidating, mutate } = useSWR<Colissimo[]>(URL, fetcher, options);

  const memoizedValue = useMemo(
    () => ({
      shippingMethodData: (data as Colissimo[]) || [],
      shippingMethodError: error,
      shippingMethodLoading: isLoading,
      shippingMethodValidating: isValidating,
      mutateShippingMethods: mutate,
    }),
    [data, error]
  );

  return memoizedValue;
}

export function useGetColissimo(shippingMethodId: string) {
  const { data, error, isLoading, isValidating, mutate } = useSWR<Colissimo>(
    shippingMethodId
      ? endpoints.shipping.shippingMethod.colissimo.getById(shippingMethodId)
      : null,
    fetcher
  );

  const memoizedValue = useMemo(
    () => ({
      shippingMethod: data,
      shippingMethodError: error,
      shippingMethodLoading: isLoading,
      shippingMethodValidating: isValidating,
      mutateShippingMethod: mutate,
    }),
    [data, error]
  );

  return memoizedValue;
}

///-------MondialRelay-------///
export async function useAddMondialRelay(data: MondialRelay) {
  const formData = {
    name: data.name,
    description: data.description,
    active: data.active,
    ordre: 0,
    amountFreeShipping: data.amountFreeShipping,
    nameFreeShipping: data.nameFreeShipping,
    mondialRelayMode: data.mondialRelayMode,
    deliveryCostMondialRelayList: data.deliveryCostMondialRelayList,
  };
  const response = await axiosInstance.post(
    endpoints.shipping.shippingMethod.mondialRelay.add,
    formData
  );
  const newData: MondialRelay = response.data;
  mutate(
    [endpoints.shipping.shippingMethod.getAll],
    (shippingMethods: ShippingMethod[] = []) => [...shippingMethods, newData],
    true
  );
  console.log(newData);
  return newData;
}

export async function useUpdateMondialRelay(
  id: string,
  updatedShippingMethod: MondialRelay
) {
  const response = await axiosInstance.put(
    endpoints.shipping.shippingMethod.mondialRelay.edit(id),
    updatedShippingMethod
  );
  const updatedData: ShippingMethod = response.data;
  mutate(
    [endpoints.shipping.shippingMethod.getAll],
    (shippingMethods: ShippingMethod[] = []) =>
      shippingMethods.map((shippingMethod) =>
        shippingMethod.id === id ? updatedData : shippingMethod
      ),
    true
  );
  console.log(response.data);
  return response.data as MondialRelay;
}

export function useGetAllMondialRelay() {
  const URL = [endpoints.shipping.shippingMethod.mondialRelay.getAll];
  const { data, error, isLoading, isValidating, mutate } = useSWR<MondialRelay[]>(URL, fetcher, options);

  const memoizedValue = useMemo(
    () => ({
      shippingMethodData: (data as MondialRelay[]) || [],
      shippingMethodError: error,
      shippingMethodLoading: isLoading,
      shippingMethodValidating: isValidating,
      mutateShippingMethods: mutate,
    }),
    [data, error]
  );

  return memoizedValue;
}

export function useGetMondialRelay(shippingMethodId: string) {
  const { data, error, isLoading, isValidating, mutate } = useSWR<MondialRelay>(
    shippingMethodId
      ? endpoints.shipping.shippingMethod.mondialRelay.getById(shippingMethodId)
      : null,
    fetcher
  );

  const memoizedValue = useMemo(
    () => ({
      shippingMethod: data,
      shippingMethodError: error,
      shippingMethodLoading: isLoading,
      shippingMethodValidating: isValidating,
      mutateShippingMethod: mutate,
    }),
    [data, error]
  );

  return memoizedValue;
}

///////////////////-----------ShippingArea----------///////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
export function useGetAllShippingArea() {
  const URL = [endpoints.shipping.shippingArea.getAll];
  const { data, error, isLoading, isValidating, mutate } = useSWR<IShippingAreaItem[]>(URL, fetcher, options);

  const memoizedValue = useMemo(
    () => ({
      shippingAreaData: (data as IShippingAreaItem[]) || [],
      shippingAreaError: error,
      shippingAreaLoading: isLoading,
      shippingAreaValidating: isValidating,
      mutateShippingAreas: mutate,
    }),
    [data, error]
  );

  return memoizedValue;
}

export function useGetShippingArea(shippingAreaId: string) {
  const { data, error, isLoading, isValidating, mutate } = useSWR<IShippingAreaItem>(
    shippingAreaId
      ? endpoints.shipping.shippingArea.getById(shippingAreaId)
      : null,
    fetcher
  );

  const memoizedValue = useMemo(
    () => ({
      shippingArea: data,
      shippingAreaError: error,
      shippingAreaLoading: isLoading,
      shippingAreaValidating: isValidating,
      mutateShippingArea: mutate,
    }),
    [data, error]
  );

  return memoizedValue;
}

export async function useAddShippingArea(data: IShippingAreaItem) {
  console.log("ship method api : " + data.shippingMethodList);
  const formData = {
    name: data.name,
    regionList: data.regionList,
    shippingMethodList: data.shippingMethodList,
  };
  const response = await axiosInstance.post(
    endpoints.shipping.shippingArea.add,
    formData
  );
  const newData: IShippingAreaItem = response.data;
  mutate(
    [endpoints.shipping.shippingArea.getAll],
    (shippingAreas: IShippingAreaItem[] = []) => [...shippingAreas, newData],
    true
  );
  return newData;
}

export async function useUpdateShippingArea(
  id: string,
  updatedShippingArea: IShippingAreaItem
) {
  const response = await axiosInstance.put(
    endpoints.shipping.shippingArea.edit(id),
    updatedShippingArea
  );
  const updatedData: IShippingAreaItem = response.data;
  mutate(
    [endpoints.shipping.shippingArea.getAll],
    (shippingAreas: IShippingAreaItem[] = []) =>
      shippingAreas.map((shippingArea) =>
        shippingArea.id === id ? updatedData : shippingArea
      ),
    true
  );
  return response.data as IShippingAreaItem;
}

export async function useDeleteShippingArea(id: string) {
  const response = await axiosInstance.delete(
    endpoints.shipping.shippingArea.delete(id)
  );
  mutate(endpoints.shipping.shippingArea.getAll);
  return response.status;
}
