import useSWR, { mutate } from "swr";
import axiosInstance, { endpoints, fetcher } from "./server";
import { SaveSupplierItemRequest, SaveSupplierRequest, SupplierDTO} from "@/contexts/types/supplierOrder";
import { useMemo } from "react";


export const ConcatFullAddress = (supplier: SupplierDTO): string => {
  if (!supplier) return 'N/A';

  const { companyStreet, companyCity, companyCountry } = supplier;

  return [
    companyStreet || '',
    companyCity || '',
    companyCountry || '',
  ]
    .filter((part) => part.trim() !== '') 
    .join(', ') || 'N/A'; 
};


export function useGetAllSuppliers() {
  const URL = [endpoints.supplier.getAllSupplier];
  const { data, error, isLoading, isValidating, mutate } = useSWR<SupplierDTO[]>(URL, fetcher);

  const memoizedValue = useMemo(
    () => ({
      suppliersData: (data as SupplierDTO[]) || [],
      suppliersError: error,
      suppliersLoading: isLoading,
      suppliersValidating: isValidating,
      mutate,
    }),
    [data, error]
  );

  return memoizedValue;
}

export function useGetSupplierById(supplierId: string) {
  const { data, error, isLoading, isValidating, mutate } = useSWR<SupplierDTO>(
    supplierId
      ? endpoints.supplier.getSupplierDetails(supplierId)
      : null,
    fetcher
  );

  const memoizedValue = useMemo(
    () => ({
      supplier: data,
      supplierError: error,
      supplierLoading: isLoading,
      supplierValidating: isValidating,
      mutateSupplier: mutate,
    }),
    [data, error]
  );

  return memoizedValue;
}

export async function useAddSupplier(supplierDto: SaveSupplierRequest) {
  try {
    const response = await axiosInstance.post(endpoints.supplier.addSupplier, supplierDto);
    return response.data;
  } catch (error) {
    console.error('Error adding supplier:', error);
    throw error; 
  }
}
export async function useUpdateSupplier(supplierDto: SupplierDTO) {
  try {
    const response = await axiosInstance.put(endpoints.supplier.updateSupplier, supplierDto);
    return response;
  } catch (error) {
    console.error('Error adding supplier:', error);
    throw error; 
  }
}
 export const deleteSupplier = async (id:number) => {
  try {
    const response = await axiosInstance.delete(endpoints.supplier.deleteSupplier(id));
    return response;
  } catch (error) {
    console.error('Error adding supplier:', error);
    throw error; 
  }
  };
  export const archiveSupplier = async (id: number, isArchived: boolean) => {
    try {
      const response = await axiosInstance.put(endpoints.supplier.archiveSupplier(id,isArchived), {});
      return response;
    } catch (error) {
      console.error('Error archiving supplier:', error);
      throw error; 
    }
  };
  
  export function useGetArchivedSuppliers() {
    const { data, error, isLoading, isValidating, mutate } = useSWR(
      endpoints.supplier.getArchivedSuppliers,
      async (url) => {
        const response = await axiosInstance.get(url);
        return response.data;
      }
    );
  
    const memoizedValue = useMemo(
      () => ({
      suppliersData: (data as SupplierDTO[]) || [],
      suppliersError: error,
      suppliersLoading: isLoading,
      suppliersValidating: isValidating,
      mutate,
      }),
      [data, error, isLoading, isValidating, mutate]
    );
  
    return memoizedValue;
  };
  export function useGetNotArchivedSuppliers() {
    const { data, error, isLoading, isValidating, mutate } = useSWR(
      endpoints.supplier.getNotArchivedSuppliers,
      async (url) => {
        const response = await axiosInstance.get(url);
        return response.data;
      }
    );
  
    const memoizedValue = useMemo(
      () => ({
      suppliersData: (data as SupplierDTO[]) || [],
      suppliersError: error,
      suppliersLoading: isLoading,
      suppliersValidating: isValidating,
      mutate,
      }),
      [data, error, isLoading, isValidating, mutate]
    );
  
    return memoizedValue;
  };
  
  