import { useMemo } from "react";
import useSWR from "swr";
import axiosInstance, { endpoints, fetcher } from "./server";
import { DiscrepancyDeclaration, ISupplierOrderPage, ReceiptStatus, SaveSupplierOrderRequest, SearchSupplierOrderParams, SupplierOrder, SupplierOrderProduct, SupplierOrderStatus, UpdateAllStatuses, UpdateCollectionStatuses, UpdateReceiptStatuses, UpdateSupplierOrderStatus } from "@/contexts/types/supplierOrder";
import { Driver } from "@/contexts/types/driver";
import { ISupplierOrderItem } from "../types/supplierOrder";


export function getAllSupplierOrders() {
    const URL = endpoints.supplierOrder.getAll;
    const { data, error, isLoading, isValidating, mutate } = useSWR<SupplierOrder[]>(URL, fetcher);

    const memoizedValue = useMemo(() => ({
        supplierOrdersData: data as SupplierOrder[] || [],
        supplierOrdersError: error,
        supplierOrdersLoading: isLoading,
        supplierOrdersValidating: isValidating,
        mutateSupplierOrders: mutate,
    }), [data, error, isLoading, isValidating]);

    return memoizedValue;
}

export function getSupplierOrderById(supplierOrderId: string) {
    const { data, error, isLoading, isValidating, mutate } = useSWR<SupplierOrder>(
        supplierOrderId ? `${endpoints.supplierOrder.getById(supplierOrderId)}` : null,
        fetcher
    );

    const memoizedValue = useMemo(() => ({
        supplierOrderData: data as SupplierOrder,
        supplierOrderError: error,
        supplierOrderLoading: isLoading,
        supplierOrderValidating: isValidating,
        mutateSupplierOrder: mutate,
    }), [data, error, isLoading, isValidating]);

    return memoizedValue;
}

export function getAllSupplierOrdersByDriver(driverId: string) {

    const { data, error, isLoading, isValidating, mutate } = useSWR<SupplierOrder[]>(
        driverId ? `${endpoints.supplierOrder.getByDriver(driverId)}` : null,
        fetcher
    );

    const memoizedValue = useMemo(() => ({
        supplierOrdersData: data as SupplierOrder[] || [],
        supplierOrdersError: error,
        supplierOrdersLoading: isLoading,
        supplierOrdersValidating: isValidating,
        mutateSupplierOrders: mutate,
    }), [data, error, isLoading, isValidating]);

    return memoizedValue;
}

export async function createSupplierOrder(userId: string, data: SaveSupplierOrderRequest) {
    try {
        const url = endpoints.supplierOrder.new(userId);
        const response = await axiosInstance.post(url, data);
        return response.data;
    } catch (error) {
        console.error("Failed to create supplier order:", error);
        throw error;
    }
}

export async function updateSupplierOrder(data: SaveSupplierOrderRequest) {
    try {
        const response = await axiosInstance.put(endpoints.supplierOrder.edit, data);
        const updatedData: SupplierOrder = response.data;
        return updatedData;
    } catch (error) {
        console.error("Failed to update supplier order:", error);
        throw error;
    }
}

export const updateSupplierOrderStatus = async (
    dto: UpdateSupplierOrderStatus
): Promise<any> => {
    try {
        const response = await axiosInstance.put(endpoints.supplierOrder.updateSupplierOrderStatus, dto);
        if (response.status !== 200) {
            throw new Error('Failed to update supplier order status');
        }
        return response.data;
    } catch (error) {
        console.error('Error updating supplier order status:', error);
        throw error;
    }
};

export const updateSupplierOrderItemCollectionStatus = async (
    dto: UpdateCollectionStatuses
): Promise<any> => {
    try {
        const response = await axiosInstance.put(endpoints.supplierOrder.updateItemCollectionStatus, dto);
        if (response.status !== 200) {
            throw new Error('Failed to update item collection statuses');
        }
        return response.data;
    } catch (error) {
        console.error('Error updating item collection statuses:', error);
        throw error;
    }
};

export const updateSupplierOrderItemReceiptStatus = async (
    receiptStatus: ReceiptStatus,
    supplierOrderItemId: string
): Promise<any> => {
    try {
        const response = await axiosInstance.put(
            `${endpoints.supplierOrder.updateItemReceiptStatus}?receiptStatus=${receiptStatus}&supplierOrderItemId=${supplierOrderItemId}`
        );
        if (response.status !== 200) {
            throw new Error('Failed to update item receipt statuses');
        }
        return response.data;
    } catch (error) {
        console.error('Error updating item receipt statuses:', error);
        throw error;
    }
};

export const updateAllSupplierOrderStatuses = async (
    dto: UpdateAllStatuses
): Promise<any> => {
    try {
        const response = await axiosInstance.put(endpoints.supplierOrder.updateAllStatus, dto);
        if (response.status !== 200) {
            throw new Error('Failed to update supplier order and item statuses');
        }
        return response.data;
    } catch (error) {
        console.error('Error updating supplier order and item statuses:', error);
        throw error;
    }
};

export function getAllProductSummaryWithKeyword(page: number, size: number, keyword: string) {
    const URL = `${endpoints.supplierOrder.getAllProductSummaryWithKeyword(page, size, keyword)}`;
    const { data, error, isValidating, mutate } = useSWR(URL, fetcher);

    const productsData = useMemo(() => {
        return (data?.content as SupplierOrderProduct[]) || [];
    }, [data?.content]);

    const memoizedValue = useMemo(
        () => ({
            productsData: productsData,
            productsLoading: !data && !error,
            productsError: error,
            productsValidating: isValidating,
            productsEmpty: productsData.length === 0,
            totalPages: data?.totalPages || 1,
            currentPage: page,
            mutatePromoCodes: mutate,
        }),
        [productsData, error, isValidating, data, page]
    );

    return memoizedValue;
}

export async function exportSuppliers(startDate: Date | null, endDate: Date| null, product: string, driverEmail: string, supplierEmail: string,status:string) {
    try {
        const params = {
            startDate: startDate?.toISOString(),
            endDate: endDate?.toISOString(),
            product,
            driverEmail,
            supplierEmail,
            status
        };

        const response = await axiosInstance.get(endpoints.supplierOrder.getExportSuppliers, {
            params: params,
            responseType: 'blob',
        });

        if (response.status !== 200) {
            const errorMessage = `Erreur de téléchargement : ${response.statusText}`;
            throw new Error(errorMessage);
        }

        const blob = response.data;
        const url = window.URL.createObjectURL(blob);
        const a = document.createElement('a');
        a.href = url;
        a.download = `supplier_orders_export_${new Date().toISOString()}.csv`;
        document.body.appendChild(a);
        a.click();
        a.remove();
        window.URL.revokeObjectURL(url);
    } catch (error) {
        console.error("Erreur lors de l'exportation des commandes fournisseurs :", error);
    }
}


export async function downloadSupplierOrderZebraFile(supplierOrderId: string) {
    try {
        const response = await axiosInstance.get(endpoints.supplierOrder.downloadSupplierOrderZebraFile(supplierOrderId), {
            responseType: "blob",
        });

        if (response.status !== 200) {
            const errorMessage = `Erreur de téléchargement : ${response.statusText}`;
            throw new Error(errorMessage);
        }

        const blob = response.data;

        const url = window.URL.createObjectURL(blob);
        const a = document.createElement('a');
        a.href = url;
        a.download = `zebra_file_${supplierOrderId}_${new Date()}.csv`;
        document.body.appendChild(a);
        a.click();
        a.remove();
        window.URL.revokeObjectURL(url);
    } catch (error) {
        console.error("Erreur lors du téléchargement du fichier :", error);
    }
}

export async function saveDiscrepancyDeclaration(dto: DiscrepancyDeclaration): Promise<string> {
    try {
        const response = await axiosInstance.post(endpoints.supplierOrder.declareDiscrepancy, dto);
        return response.data;
    } catch (error) {
        console.error("Error saving discrepancy declaration:", error);
        throw error;
    }
}

export function getAllDrivers() {
    const URL = [endpoints.driver.getAllDrivers];
    const { data, error, isLoading, isValidating, mutate } = useSWR<Driver[]>(URL, fetcher);

    const memoizedValue = useMemo(
        () => ({
            driversData: (data as Driver[]) || [],
            driversError: error,
            driversLoading: isLoading,
            driversValidating: isValidating,
            mutateDrivers: mutate,
        }),
        [data, error]
    );

    return memoizedValue;
}


export async function deleteSupplierOrder(orderId: any) {
    try {

        const response = await axiosInstance.delete(endpoints.supplierOrder.delete(orderId));

        return response.status === 204;

    } catch (error) {
        console.error('Error deleting supplier order:', error);
        return false;
    }
}

export async function deleteMultipleSupplierOrders(orderIds: string[]) {
    try {

        const response = await axiosInstance.delete(endpoints.supplierOrder.bulkDelete, { data: orderIds });

        return response.status === 204;

    } catch (error) {
        console.error('Error deleting multiple supplier orders:', error);
        return false;
    }
}

export const deleteItemFromSupplierOrder = async (id: any) => {
    try {
        const response = await axiosInstance.delete(endpoints.supplierOrder.deleteItem(id));
        return response.status === 204;
    } catch (error) {
        return false;
    }
}




/////////////////////

export const addOrderComment = async (orderId: string, comment: string): Promise<void> => {
    try {
        const response = await axiosInstance.post(endpoints.supplierOrder.addComment(orderId), comment, {
            headers: {
                'Content-Type': 'text/plain',
            },
        });

        if (response.status !== 200) {
            throw new Error('Failed to add comment');
        }
    } catch (error) {
        console.error('Error adding comment:', error);
        throw error;
    }
};

export const getAllSuppliers = async () => {
    try {
        const response = await axiosInstance.get(endpoints.supplier.getAllSupplier);
        if (response.status === 200) {
            return response.data;
        } else {
            return null;
        }
    } catch (error) {
        throw new Error('Failed to fetch suppliers');
    }
}

export async function getSupplierById(supplierId: number) {
    try {
        const response = await axiosInstance.get(endpoints.supplier.getSupplierDetails(supplierId.toString()));
        if (response.status === 200) {
            return response.data;
        } else {
            return null;
        }

    } catch (error) {
        console.error("Error fetching supplier by id:", error);
        return null;
    }
}


export function useSearchSupplierOrder({
    product = "",
    status= "",
    driverEmail = "",
    supplierEmail = "",
    page = 0,
    size = 10,
    startDate = null,
    endDate = null,
  }: SearchSupplierOrderParams) {
    const filterParams = new URLSearchParams({
      product: product || "",
      driverEmail: driverEmail || "",
      supplierEmail: supplierEmail || "",
      status: status || "",
      page: page.toString(),
      size: size.toString(),
    });
  
    if (startDate) {
      filterParams.append("startDate", startDate.toString());
    }
    if (endDate) {
      filterParams.append("endDate", endDate.toString());
    }
  
    const url = `${endpoints.supplierOrder.getSupplierOrderPage}?${filterParams.toString()}`;
    const { data, error, isValidating, mutate } = useSWR<ISupplierOrderPage>(url, fetcher);
  
    return {
      supplierOrders: data?.content || [],
      totalSupplierOrders: data?.totalElements || 0,
      totalPages: data?.totalPages || 0,
      pageNumber: data?.pageable.pageNumber || 0,
      pageSize: data?.pageable.pageSize || 0,
      loading: !data && !error,
      isError: error,
      isValidating,
      revalidate: mutate,
    };
  }


  export function useSupplierOrderCountsByStatus({
    product = "",
    driverEmail = "",
    supplierEmail = "",
    startDate = null,
    endDate = null,
  }: {
    product?: string;
    driverEmail?: string;
    supplierEmail?: string;
    startDate?: Date | null;
    endDate?: Date | null;
  }) {
    const params = {
      product,
      driverEmail,
      supplierEmail,
      startDate: startDate ? startDate.toString() : "",
      endDate: endDate ? endDate.toString() : "",
    };
  
    const url = `${endpoints.supplierOrder.statusCount}?${new URLSearchParams(
      params
    ).toString()}`;
  
    const { data, error } = useSWR(url, async () => {
      try {
        const response = await axiosInstance.get(endpoints.supplierOrder.statusCount, {
          params,
        });
        return response.data;
      } catch (err) {
        console.error("Error fetching user counts by status:", err);
        throw err;
      }
    });
  
    return useMemo(
      () => ({
        supplierOrderCounts: data || [],
        supplierOrderCountsError: error,
      }),
      [data, error]
    );
  }
