import useSWR from "swr";
import { useMemo } from "react";
import axiosInstance, { endpoints, fetcher } from "./server";
import axios, { AxiosRequestConfig, AxiosResponse } from "axios";
import { IColor, IColorRequest, ILabel, IChoice, IImage } from "../types/term";



export function useGetColorsByAttribut(attributid: string) {
  const URL = [
    endpoints.stock_management.termColor.getAllByAttribut(attributid),
  ];
  const { data, error, isLoading, isValidating, mutate } = useSWR<IColor[]>(
    URL,
    fetcher
  );

  const memoizedValue = useMemo(
    () => ({
      ColorsData: (data as IColor[]) || [],
      ColorsLoading: isLoading,
      ColorsError: error,
      ColorsValidating: isValidating,
      ColorsMutate: mutate,
    }),

    [data, error]
  );
  return memoizedValue;
}

export function useGetColor(colorId: string) {
  const URL = [endpoints.stock_management.termColor.getById(colorId)];
  const { data, error, isLoading, isValidating, mutate } = useSWR<IColor>(
    URL,
    fetcher
  );

  const memoizedValue = useMemo(
    () => ({
      ColorData: (data as IColor) || null,
      ColorLoading: isLoading,
      ColorError: error,
      ColorValidating: isValidating,
      ColorMutate: mutate,
    }),

    [data, error]
  );
  return memoizedValue;
}

export async function useAddColor(color: IColorRequest) {
  try {
    const URL = endpoints.stock_management.termColor.add();
    const response: AxiosResponse<IColorRequest> = await axiosInstance.post(
      URL,
      color
    );
  } catch (error) {
    console.error("Erreur lors de l'ajout de la couleur:", error);
    throw error;
  }
}
export async function useUpdateColor(color: IColorRequest, colorId: string) {
  try {
    const URL = endpoints.stock_management.termColor.edit(colorId);
    const response: AxiosResponse<IColorRequest> = await axiosInstance.put(
      URL,
      color
    );
    return response;
  } catch (error) {
    console.error("Erreur lors de la mise à jour de la couleur:", error);
    throw error;
  }
}

export async function useDeleteColor(colorId: string , attributId: string) {
  try {
    const URL = endpoints.stock_management.termColor.delete(colorId, attributId);
    const response = await axiosInstance.delete(URL);
    return response;
  } catch (error) {
    console.error("Erreur lors de la suppression de la couleur:", error);
    throw error;
  }
}

export async function useDeleteColors(attributId: string ,colorIds: string[]) {
  try {
    const URL = endpoints.stock_management.termColor.deleteBunch(attributId);
    const response = await axiosInstance.delete(URL, {
      data: colorIds,
    });
    return response;
  } catch (error) {
    console.error("Erreur lors de la suppression de la couleur:", error);
    throw error;
  }
}

export function useGetLabelsByAttribut(attributid: string) {
  const URL = [
    endpoints.stock_management.termLabel.getAllByAttribut(attributid),
  ];
  const { data, error, isLoading, isValidating, mutate } = useSWR<ILabel[]>(
    URL,
    fetcher
  );

  const memoizedValue = useMemo(
    () => ({
      labelsData: (data as ILabel[]) || [],
      labelsLoading: isLoading,
      labelsError: error,
      labelsValidating: isValidating,
      labelsMutate: mutate,
    }),

    [data, error]
  );
  return memoizedValue;
}

export function useGetLabel(labelId: string) {
  const URL = [endpoints.stock_management.termLabel.getById(labelId)];
  const { data, error, isLoading, isValidating, mutate } = useSWR<ILabel>(
    URL,
    fetcher
  );

  const memoizedValue = useMemo(
    () => ({
      labelData: (data as ILabel) || null,
      labelLoading: isLoading,
      labelError: error,
      labelValidating: isValidating,
      labelMutate: mutate,
    }),

    [data, error]
  );
  return memoizedValue;
}

export async function useAddLabel(label: ILabel) {
  try {
    const URL = endpoints.stock_management.termLabel.add();
    const response: AxiosResponse<ILabel> = await axiosInstance.post(
      URL,
      label
    );
    console.log(response);
  } catch (error) {
    console.error("Erreur lors de l'ajout du label:", error);
    throw error;
  }
}
export async function useUpdateLabel(label: ILabel, labelId: string) {
  try {
    const URL = endpoints.stock_management.termLabel.edit(labelId);
    const response: AxiosResponse<ILabel> = await axiosInstance.put(URL, label);
    return response;
  } catch (error) {
    console.error("Erreur lors de la mise à jour du Label:", error);
    throw error;
  }
}

export async function useDeleteLabel(labelId: string, attributId: string) {
  try {
    const URL = endpoints.stock_management.termLabel.delete(labelId, attributId);
    const response = await axiosInstance.delete(URL);
    return response;
  } catch (error) {
    console.error("Erreur lors de la suppression du label:", error);
    throw error;
  }
}

export async function useDeleteLabels(attributId: string ,labelIds: string[]) {
  try {
    const URL = endpoints.stock_management.termLabel.deleteBunch(attributId);
    const response = await axiosInstance.delete(URL, {
      data: labelIds,
    });
    return response;
  } catch (error) {
    console.error("Erreur lors de la suppression des labels:", error);
    throw error;
  }
}

export function useGetChoicesByAttribut(attributid: string) {
  const URL = [
    endpoints.stock_management.termChoix.getAllByAttribut(attributid),
  ];
  const { data, error, isLoading, isValidating, mutate } = useSWR<IChoice[]>(
    URL,
    fetcher
  );

  const memoizedValue = useMemo(
    () => ({
      ChoicesData: (data as IChoice[]) || [],
      ChoicesLoading: isLoading,
      ChoicesError: error,
      ChoicesValidating: isValidating,
      ChoicesMutate: mutate,
    }),

    [data, error]
  );
  return memoizedValue;
}

export function useGetChoice(choiceId: string) {
  const URL = [endpoints.stock_management.termChoix.getById(choiceId)];
  const { data, error, isLoading, isValidating, mutate } = useSWR<IChoice>(
    URL,
    fetcher
  );

  const memoizedValue = useMemo(
    () => ({
      choiceData: (data as IChoice) || null,
      choiceLoading: isLoading,
      choiceError: error,
      choiceValidating: isValidating,
      choiceMutate: mutate,
    }),

    [data, error]
  );
  return memoizedValue;
}

export async function useAddChoice(choix: IChoice) {
  try {
    const URL = endpoints.stock_management.termChoix.add();
    const response: AxiosResponse<IChoice> = await axiosInstance.post(
      URL,
      choix
    );
  } catch (error) {
    console.error("Erreur lors de l'ajout du choix:", error);
    throw error;
  }
}
export async function useUpdateChoice(choix: IChoice, choiceId: string) {
  try {
    const URL = endpoints.stock_management.termChoix.edit(choiceId);
    const response: AxiosResponse<IChoice> = await axiosInstance.put(
      URL,
      choix
    );
    return response;
  } catch (error) {
    console.error("Erreur lors de la mise à jour du choix:", error);
    throw error;
  }
}

export async function useDeleteChoice(choiceId: string, attributId: string) {
  try {
    const URL = endpoints.stock_management.termChoix.delete(choiceId, attributId);
    const response = await axiosInstance.delete(URL);
    return response;
  } catch (error) {
    console.error("Erreur lors de la suppression du choix:", error);
    throw error;
  }
}

export async function useDeleteChoices(attributId: string ,choiceIds: string[]) {
  try {
    const URL = endpoints.stock_management.termChoix.deleteBunch(attributId)
    const response = await axiosInstance.delete(URL, {
      data: choiceIds,
    });
    return response;
  } catch (error) {
    console.error("Erreur lors de la suppression des choix:", error);
    throw error;
  }
}

export function useGetImageByAttribut(attributid: string) {
  const URL = [
    endpoints.stock_management.termImage.getAllByAttribut(attributid),
  ];
  const { data, error, isLoading, isValidating, mutate } = useSWR<IImage[]>(
    URL,
    fetcher
  );

  const memoizedValue = useMemo(
    () => ({
      ImagesData: (data as IImage[]) || [],
      ImagesLoading: isLoading,
      ImagesError: error,
      ImagesValidating: isValidating,
      ImagesMutate: mutate,
    }),

    [data, error]
  );
  return memoizedValue;
}

export function useGetImage(imageId: string) {
  const URL = [endpoints.stock_management.termImage.getById(imageId)];
  const { data, error, isLoading, isValidating, mutate } = useSWR<IImage>(
    URL,
    fetcher
  );

  const memoizedValue = useMemo(
    () => ({
      ImageData: (data as IImage) || null,
      ImageLoading: isLoading,
      Error: error,
      ImageValidating: isValidating,
      ImageMutate: mutate,
    }),

    [data, error]
  );
  return memoizedValue;
}

export async function useAddImage(image: IImage) {
  try {
    const URL = endpoints.stock_management.termImage.add();
    const response: AxiosResponse<IImage> = await axiosInstance.post(
      URL,
      image
    );
  } catch (error) {
    console.error("Erreur lors de l'ajoute de l'image:", error);
    throw error;
  }
}
export async function useUpdateImage(image: IImage, imageId: string) {
  try {
    const URL = endpoints.stock_management.termImage.edit(imageId);
    const response: AxiosResponse<IImage> = await axiosInstance.put(URL, image);
    return response;
  } catch (error) {
    console.error("Erreur lors de la mise à jour du terme image:", error);
    throw error;
  }
}

export async function useDeleteImage(imageId: string, attributId: string) {
  try {
    const URL = endpoints.stock_management.termImage.delete(imageId, attributId);
    const response = await axiosInstance.delete(URL);
    return response;
  } catch (error) {
    console.error("Erreur lors de la suppression du terme image:", error);
    throw error;
  }
}

export async function useDeleteImages(attributId: string , imageIds: string[]) {
  try {
    const URL = endpoints.stock_management.termImage.deleteBunch(attributId)
    const response = await axiosInstance.delete(URL, {
      data: imageIds,
    });
    return response;
  } catch (error) {
    console.error("Erreur lors de la suppression des termes images:", error);
    throw error;
  }
}


export const checkSlugExistenceChoice = async (slug: string, attributid: string): Promise<boolean> => {
  if (!slug || !attributid) return false;

  try {
 
    const URL = `api/stock/termChoix/exists?slug=${slug}&attributeId=${attributid}`;
    
    const response = await axiosInstance.get(URL); 

    return response.data === true; 
  } catch (error) {
    console.error("Erreur lors de la vérification du slug :", error);
    return false; 
  }
};

export const checkSlugExistenceColor = async (slug: string, attributid: string): Promise<boolean> => {
  if (!slug || !attributid) return false;

  try {
    const URL = `api/stock/termColors/exists?slug=${slug}&attributeId=${attributid}`;
    
    const response = await axiosInstance.get(URL);

    
    return response.data === true; 
  } catch (error) {
    console.error("Erreur lors de la vérification du slug :", error);
    return false; 
  }
};

export const checkSlugExistenceLabel = async (slug: string, attributid: string): Promise<boolean> => {
  if (!slug || !attributid) return false;

  try {
    const URL = `api/stock/termLabels/exists?slug=${slug}&attributeId=${attributid}`;
    
    const response = await axiosInstance.get(URL); 

   
    return response.data === true;
  } catch (error) {
    console.error("Erreur lors de la vérification du slug :", error);
    return false; 
  }
};


export const checkSlugExistenceImage = async (slug: string, attributid: string): Promise<boolean> => {
  if (!slug || !attributid) return false;

  try {
 
    const URL = `api/stock/termImage/exists?slug=${slug}&attributeId=${attributid}`;
    
    const response = await axiosInstance.get(URL);  

  
    return response.data === true; 
  } catch (error) {
    console.error("Erreur lors de la vérification du slug :", error);
    return false;
  }
};