import axios, { AxiosRequestConfig } from "axios";
import { useMemo } from "react";
import useSWR from "swr";
import {
  AuthenticationRequest,
  ClientWithCustomResponse,
  IAddress,
  IAddressItem,
  IAllUsersItem,
  IClientItem,
  IRole,
  IUser,
  IUserPage,
  RequestChangeUserStatus,
  RequestLoginActivity,
  SearchUsersParams,
  UpdatePasswordRequest,
  UserCountByStatus,
} from "../types/user";
import axiosInstance, { endpoints, fetcher } from "./server";
import {  SaveAddressRequest } from "../types/address";
import { getRequestAuthorization } from "@/utils/encryption";

export default axiosInstance;
export function useGetUsers() {
  const URL = [endpoints.user.getAll];
  const { data, error } = useSWR<IAllUsersItem>(URL, fetcher);

  const memoizedValue = useMemo(
    () => ({
      userData: (data as IAllUsersItem) || [],
      userError: error,
    }),
    [data, error]
  );
  return memoizedValue;
}

export function useGetRoles() {
  const URL = [endpoints.user.getAllRoles];
  const { data, error } = useSWR<IRole[]>(URL, fetcher);
  const memoizedValue = useMemo(
    () => ({
      roleData: (data as IRole[]) || [],
      roleError: error,
    }),
    [data, error]
  );
  return memoizedValue;
}
export const searchAndFetchRandomClients = async (
  searchKey: string
): Promise<ClientWithCustomResponse[]> => {
  try {
    const response = await axiosInstance.get<ClientWithCustomResponse[]>(
      endpoints.user.searchAndFetchRandomClients(searchKey)
    );

    return response.data;
  } catch (error) {
    throw error;
  }
};
export const UserByEmail = async (email: string) => {
  try {
    const response = await axiosInstance.get(endpoints.user.getByEmail(email));
    return response;
  } catch (error) {
    throw error;
  }
};
export const findUserById = async (id: number) => {
  try {
    const response = await axiosInstance.get(endpoints.user.findUserById(id));
    return response;
  } catch (error) {
    throw error;
  }
};
export function useGetUserById(id: number) {
  const URL = [endpoints.user.getUserById(id)];
  const { data, error } = useSWR<{ client?: IClientItem; user?: IUser }>(
    URL,
    fetcher
  );

  const memoizedValue = useMemo(() => {
    if (data?.client) {
      return {
        userData: data.client || null,
        userError: error,
      };
    }

    if (data?.user) {
      return {
        userData: data.user || null,
        userError: error,
      };
    }

    return {
      userData: null,
      userError: error,
    };
  }, [data, error]);
  return memoizedValue;
}

export const loginUser = async (authData: AuthenticationRequest) => {
  try {
    const response = await axiosInstance.post(endpoints.user.login, authData);
    return response;
  } catch (error) {
    throw error;
  }
};
export const addRole = async (name: string) => {
  try {
    const response = await axiosInstance.post(endpoints.user.addRole, { name });
    return response;
  } catch (error) {
    throw error;
  }
};
export const UpdateUserPassword = async (
  updatePassword: UpdatePasswordRequest
) => {
  try {
    const response = await axiosInstance.post(
      endpoints.user.updateUserPassword,
      updatePassword
    );
    return response;
  } catch (error) {
    throw error;
  }
};
export const ChangingLogingActivity = async (
  requestLoginActivity: RequestLoginActivity
) => {
  try {
    const response = await axiosInstance.post(
      endpoints.user.changingLogingActivity,
      requestLoginActivity
    );
    return response;
  } catch (error) {
    throw error;
  }
};
export const changeUserStatus = async (
  requestChangeUser: RequestChangeUserStatus
) => {
  try {
    const response = await axiosInstance.post(
      endpoints.user.changeUserStatus,
      requestChangeUser
    );
    return response;
  } catch (error) {
    throw error;
  }
};

export const addAdressToUser = async (adresse: SaveAddressRequest) => {
  try {
    const response = await axiosInstance.post(
      endpoints.user.addAdresss,
      adresse
    );
    return response;
  } catch (error) {
    throw error;
  }
};
export const updateAdressToUser = async (adresse: IAddress) => {
  try {
    const response = await axiosInstance.put(
      endpoints.user.updateAdress,
      adresse
    );
    return response;
  } catch (error) {
    throw error;
  }
};
export const saveUser = async (userdtoJson: string, imageFile?: File) => {
  try {
    const formData = new FormData();
    formData.append("user", userdtoJson);
    if (imageFile) {
      formData.append("imageFile", imageFile);
    }
    const response = await axiosInstance.post(endpoints.user.add, formData, {
      headers: {
        "Content-Type": "multipart/form-data",
      },
    });
    return response.data;
  } catch (error) {
    throw error;
  }
};

export const updateUser = async (userDtoJson: string, imageFile?: File) => {
  try {
    const formData = new FormData();
    if (imageFile) {
      formData.append("imageFile", imageFile);
    }
    formData.append("userDto", userDtoJson);
    const response = await axiosInstance.put(
      endpoints.user.updateUser,
      formData,
      {
        headers: {
          "Content-Type": "multipart/form-data",
        },
      }
    );

    return response;
  } catch (error) {
    throw error;
  }
};
export const updateClient = async (
  clientDTO: IClientItem,
  imageFile?: File
) => {
  try {
    const formData = new FormData();

    // Convert clientDTO to Blob and specify its content type as application/json
    const clientDTOBlob = new Blob([JSON.stringify(clientDTO)], {
      type: "application/json",
    });

    // Append the clientDTO as a Blob to the formData
    formData.append("clientDTO", clientDTOBlob);

    // Append the image file if it exists
    if (imageFile) {
      formData.append("imageFile", imageFile);
    }

    const response = await axiosInstance.put(
      endpoints.user.updateClient,
      formData,
      {
        headers: {
          "Content-Type": "multipart/form-data",
        },
      }
    );

    return response;
  } catch (error) {
    throw error;
  }
};

export async function useGetClientsByIDs(clientsIDs: number[]) {
  const URL = [endpoints.user.getClientsByIDs(clientsIDs)];
  const { data, error, isLoading, isValidating, mutate } = useSWR<
    IClientItem[]
  >(URL, fetcher);

  const memoizedValue = useMemo(
    () => ({
      clientsData: (data as IClientItem[]) || [],
      clientsError: error,
      clientsLoading: isLoading,
      clientsValidating: isValidating,
      mutateClients: mutate,
    }),
    [data, error]
  );

  return memoizedValue;
}

export async function useGetUsersByIDs(usersIDs: number[]) {
  const URL = [endpoints.user.getUsersByIDs(usersIDs)];
  const { data, error, isLoading, isValidating, mutate } = useSWR<IUser[]>(
    URL,
    fetcher
  );

  const memoizedValue = useMemo(
    () => ({
      usersData: (data as IUser[]) || [],
      usersError: error,
      usersLoading: isLoading,
      usersValidating: isValidating,
      mutateUsers: mutate,
    }),
    [data, error]
  );
  return memoizedValue;
}

export function useSearchUsers({
  keyword = "",
  status = "",
  roles = [],
  page = 0,
  size = 10,
  startDate = "",
  endDate = "",
  tags = [],
  sortBy = "createdAt",
  sortDir = "ASC",
}: SearchUsersParams) {
  const filterParams = new URLSearchParams({
    keyword: keyword || "",
    status: status || "",
    page: page.toString(),
    size: size.toString(),
    sortBy: sortBy || "",
    sortDir: sortDir || "",
  });

  if (startDate) {
    filterParams.append("startDate", startDate);
  }
  if (endDate) {
    filterParams.append("endDate", endDate);
  }

  if (roles.length) {
    filterParams.append("roles", roles.join(","));
  }
  if (tags.length) {
    filterParams.append("tags", tags.join(","));
  }

  const url = `${endpoints.user.searchUsers}?${filterParams.toString()}`;
  const { data, error, isValidating, mutate } = useSWR<IUserPage>(url, fetcher);

  return {
    users: data?.content || [],
    totalUsers: data?.totalElements || 0,
    totalPages: data?.totalPages || 0,
    pageNumber: data?.pageable.pageNumber || 0,
    pageSize: data?.pageable.pageSize || 0,
    loading: !data && !error,
    isError: error,
    isValidating,
    revalidate: mutate,
  };
}

export function useUserCountsByStatus({
  keyword = "",
  roles = [],
  tags = [],
  startDate = "",
  endDate = "",
}: {
  keyword?: string;
  roles?: string[];
  tags?: string[];
  startDate?: string;
  endDate?: string;
}) {
  const params = {
    keyword,
    roles: roles.join(","),
    tags: tags.join(","),
    startDate,
    endDate
  };

  const url = `${endpoints.user.statusCount}?${new URLSearchParams(
    params
  ).toString()}`;

  const { data, error } = useSWR(url, async () => {
    try {
      const response = await axiosInstance.get(endpoints.user.statusCount, {
        params, // Pass params directly to Axios
      });
      return response.data;
    } catch (err) {
      console.error("Error fetching user counts by status:", err);
      throw err;
    }
  });

  return useMemo(
    () => ({
      userCounts: data || [],
      userCountsError: error,
    }),
    [data, error]
  );
}

export const exportUsers = async (
  filters: SearchUsersParams
): Promise<Blob> => {
  try {
    const exportParams = {
      keyword: filters.keyword || undefined,
      status: filters.status || undefined,
      roles: filters.roles?.length ? filters.roles.join(",") : undefined,
      tags: filters.tags?.length ? filters.tags.join(",") : undefined,
      startDate: filters.startDate || undefined,
      endDate: filters.endDate || undefined,
      sortBy: filters.sortBy || undefined,
      sortDir: filters.sortDir || undefined,
    };
    const queryString = new URLSearchParams(
      Object.entries(exportParams).reduce((acc, [key, value]) => {
        if (value !== null && value !== undefined && value !== "") {
          acc[key] = value.toString();
        }
        return acc;
      }, {} as Record<string, string>)
    ).toString();
    const response = await axiosInstance.get(
      `${endpoints.user.export}?${queryString}`,
      {
        headers: {
          "Content-Type": "application/json",
        },
      }
    );
    return new Blob([response.data], { type: "text/csv" });
  } catch (error) {
    throw error;
  }
};
