import { Controller, useFormContext } from 'react-hook-form';
import FormHelperText from '@mui/material/FormHelperText';
import { Upload, UploadBox, UploadProps, UploadAvatar } from '../upload';

interface Props extends Omit<UploadProps, 'file'>{
  name: string;
  maxSize?: number;
  onDrop?: (acceptedFiles: File[]) => void;
  helperText?: React.ReactNode;
  multiple?: boolean;
}

// ----------------------------------------------------------------------

export function RHFUploadAvatar({ name, maxSize = 3145728, onDrop, helperText, ...other }: Props) {
  const { control, setValue } = useFormContext();

  return (
    <Controller
      name={name}
      control={control}
      render={({ field, fieldState: { error } }) => (
        <div>
          <UploadAvatar
            error={!!error}
            file={field.value}
            maxSize={maxSize}
            onDrop={(acceptedFiles) => {
              if (acceptedFiles.length > 0) {
                const file = acceptedFiles[0];
                const newFile = Object.assign(file, {
                  preview: URL.createObjectURL(file),
                });
                setValue(name, newFile.preview, { shouldValidate: true });
                if (onDrop) {
                  onDrop(acceptedFiles);
                }
              }
            }}
            {...other}
          />
          {!!error && (
            <FormHelperText error sx={{ px: 2, textAlign: 'center' }}>
              {error.message}
            </FormHelperText>
          )}
          {helperText && !error && (
            <FormHelperText sx={{ px: 2, textAlign: 'center' }}>
              {helperText}
            </FormHelperText>
          )}
        </div>
      )}
    />
  ); 
}
// ----------------------------------------------------------------------

export function RHFUploadBox({ name, ...other }: Props) {
  const { control } = useFormContext();

  return (
    <Controller
      name={name}
      control={control}
      render={({ field, fieldState: { error } }) => (
        <UploadBox files={field.value} error={!!error} {...other} />
      )}
    />
  );
}

// ----------------------------------------------------------------------

export function RHFUpload({ name, multiple, helperText, ...other }: Props) {
  const { control } = useFormContext();

  return (
    <Controller
      name={name}
      control={control}
      render={({ field, fieldState: { error } }) =>
        multiple ? (
          <Upload
            multiple
            accept={{ 'image/': [] }}
            files={field.value}
            error={!!error}
            helperText={
              (!!error || helperText) && (
                <FormHelperText error={!!error} sx={{ px: 2 }}>
                  {error ? error?.message : helperText}
                </FormHelperText>
              )
            }
            {...other}
          />
        ) : (
          <Upload
            accept={{ 'image/': [] }}
            file={field.value}
            error={!!error}
            helperText={
              (!!error || helperText) && (
                <FormHelperText error={!!error} sx={{ px: 2 }}>
                  {error ? error?.message : helperText}
                </FormHelperText>
              )
            }
            {...other}
          />
        )
      }
    />
  );
}
