import * as Yup from "yup";
import { useForm } from "react-hook-form";
import { yupResolver } from "@hookform/resolvers/yup";
import Card from "@mui/material/Card";
import Stack from "@mui/material/Stack";
import IconButton from "@mui/material/IconButton";
import LoadingButton from "@mui/lab/LoadingButton";
import InputAdornment from "@mui/material/InputAdornment";
import Alert from "@mui/material/Alert";
import Collapse from "@mui/material/Collapse";
import { useBoolean } from "@/hooks/use-boolean";
import Iconify from "@/components/iconify";
import FormProvider, { RHFTextField } from "@/components/hook-form";
import { UpdateUserPassword, UserByEmail } from "@/shared/api/user";
import { IUser, UpdatePasswordRequest } from "@/shared/types/user";
import { useEffect, useState } from "react";

// ----------------------------------------------------------------------

export default function AccountChangePassword() {
  const password = useBoolean();

  const [showSuccessAlert, setShowSuccessAlert] = useState(false);
  const [showErrorAlert, setShowErrorAlert] = useState(false);
  const [showOldPasswordError, setShowOldPasswordError] = useState(false);

  const ChangePassWordSchema = Yup.object().shape({
    oldPassword: Yup.string().required("Ancien mot de passe requis"),
    newPassword: Yup.string()
      .required("Nouveau mot de passe requis")
      .min(6, "Le mot de passe doit contenir au moins 6 caractères")
      .test(
        "no-match",
        "Le nouveau mot de passe doit être différent de l'ancien mot de passe",
        (value, { parent }) => value !== parent.oldPassword
      ),
    confirmNewPassword: Yup.string()
      .required("La confirmation du mot de passe est requise")
      .oneOf([Yup.ref("newPassword")], "Les mots de passe doivent correspondre"),
  });

  const defaultValues = {
    oldPassword: "",
    newPassword: "",
    confirmNewPassword: "",
  };

  const methods = useForm({
    resolver: yupResolver(ChangePassWordSchema),
    defaultValues,
  });

  const {
    reset,
    handleSubmit,
    formState: { isSubmitting },
  } = methods;

  const [email, setEmail] = useState<string | null>(null);

  useEffect(() => {
    const storedEmail = localStorage.getItem("email");
    setEmail(storedEmail);
  }, []);

  const handleAlertClose = () => {
    setShowSuccessAlert(false);
    setShowErrorAlert(false);
    setShowOldPasswordError(false);
  };

  useEffect(() => {
    if (showSuccessAlert || showErrorAlert || showOldPasswordError) {
      const timer = setTimeout(() => {
        handleAlertClose();
      }, 2000);
      return () => clearTimeout(timer);
    }
  }, [showSuccessAlert, showErrorAlert, showOldPasswordError]);

  const onSubmit = handleSubmit(async (data) => {
    try {
      if(email){
      const response = await UserByEmail(email);
      const user : IUser = response.data;
      if (user && data.oldPassword === user.password) {
        const updatePasswordRequest: UpdatePasswordRequest = {
          email: email || "",
          password: data.newPassword,
        };
        const response = await UpdateUserPassword(updatePasswordRequest);
        if (response.status === 200) {
          setShowSuccessAlert(true);
          setShowErrorAlert(false);
          reset();
        }
      } else {
        setShowOldPasswordError(true);
        setShowSuccessAlert(false);
      }
      }
    } catch (error) {
      console.error(error);
      setShowErrorAlert(true);
      setShowSuccessAlert(false);
    }
  });

  return (
    <FormProvider methods={methods} onSubmit={onSubmit}>
      <Stack component={Card} spacing={1} sx={{ p: 2 }}>
        <Collapse in={showSuccessAlert}>
          <Alert severity="success">
            Mise à jour réussie !
          </Alert>
        </Collapse>
        <Collapse in={showErrorAlert}>
          <Alert severity="error">
            Échec de la mise à jour !
          </Alert>
        </Collapse>
        <Collapse in={showOldPasswordError}>
          <Alert severity="error">
            Ancien mot de passe incorrect !
          </Alert>
        </Collapse>
        <RHFTextField
          name="oldPassword"
          type={password.value ? "text" : "password"}
          label="Ancien mot de passe"
          InputProps={{
            endAdornment: (
              <InputAdornment position="end">
                <IconButton onClick={password.onToggle} edge="end">
                  <Iconify
                    icon={
                      password.value
                        ? "solar:eye-bold"
                        : "solar:eye-closed-bold"
                    }
                  />
                </IconButton>
              </InputAdornment>
            ),
          }}
        />
        <RHFTextField
          name="newPassword"
          label="Nouveau mot de passe"
          type={password.value ? "text" : "password"}
          InputProps={{
            endAdornment: (
              <InputAdornment position="end">
                <IconButton onClick={password.onToggle} edge="end">
                  <Iconify
                    icon={
                      password.value
                        ? "solar:eye-bold"
                        : "solar:eye-closed-bold"
                    }
                  />
                </IconButton>
              </InputAdornment>
            ),
          }}
        />

        <RHFTextField
          name="confirmNewPassword"
          type={password.value ? "text" : "password"}
          label="Confirmer le nouveau mot de passe"
          InputProps={{
            endAdornment: (
              <InputAdornment position="end">
                <IconButton onClick={password.onToggle} edge="end">
                  <Iconify
                    icon={
                      password.value
                        ? "solar:eye-bold"
                        : "solar:eye-closed-bold"
                    }
                  />
                </IconButton>
              </InputAdornment>
            ),
          }}
        />

        <LoadingButton
          type="submit"
          variant="contained"
          loading={isSubmitting}
          sx={{ ml: "auto" }}
        >
          Enregistrer les modifications
        </LoadingButton>
      </Stack>
    </FormProvider>
  );
}
