import React, { useCallback, useMemo, useState,useEffect } from "react";
import { Snackbar, Alert } from "@mui/material";
import {
  Card,
  CardContent,
  Typography,
  Table,
  TableBody,
  TableCell,
  TableContainer,
  TableRow,
  Avatar,
  ListItemText,
  Box,
  Grid,
  Stack,
  Divider,
  Button,
  Chip,
  TextField,
  InputAdornment,
  Dialog,
  DialogTitle,
  DialogContent,
  DialogActions,
} from "@mui/material";
import CloseIcon from "@mui/icons-material/Close";
import {
  IProductSessionItem,
  AttributeType,
  ProductType,
} from "@/shared/types/product";

import { useTable, TableHeadCustom } from "@/shared/components/table";
import VariationTableToolbar from "./Variation-table-toolbar";
import VariationTableFiltersResult from "./Variation-table-filters-result";
import { warning } from "@/theme/palette";
import { updateProductPrice, updateVariationPrice } from "@/shared/api/product"; 
import { ClearUrl } from "@/utils/cleanImageUrl";
// Composant pour extraire l'URL de l'image


type Props = {
  product: IProductSessionItem;
  onSelect: (
    productId: any,
    quantity: number,
    type: ProductType,
    name: string
  ) => void;
  onProductUpdate: (updatedProduct: IProductSessionItem) => void; // Nouvelle prop
};


export type Filters = {
  nameOrUGS: string;
  termColor: string[];
  termLabel: string[];
  termChoice: string[];
};

const defaultFilters: Filters = {
  nameOrUGS: "",
  termColor: [],
  termLabel: [],
  termChoice: [],
};

export default function ProductVariationsCard({
  product,
  onSelect,
  onProductUpdate,
}: Props) {
  const [productData, setProductData] = useState(product);
    const [filters, setFilters] = useState(defaultFilters); 
  const [quantities, setQuantities] = useState<{
    [key: string]: number | null;
  }>({}); // Quantités des variations

  const [open, setOpen] = useState(false);
  const [newPrice, setNewPrice] = useState("");
  const [selectedRow, setSelectedRow] = useState<string | null>(null);
  const [selectedProductId, setSelectedProductId] = useState<any | null>(null);
  const [selectedQuantity, setSelectedQuantity] = useState<number>(0);
  const table = useTable(); // Utilisation du hook pour la table
  // Fonction pour récupérer les noms des attributs dynamiques

  const [snackbarOpen, setSnackbarOpen] = useState(false);
  const [snackbarMessage, setSnackbarMessage] = useState("");
  const [snackbarSeverity, setSnackbarSeverity] = useState<
    "success" | "error"
  >("success");
  const [openBulkPriceDialog, setOpenBulkPriceDialog] = useState(false);


  useEffect(() => {
    setProductData(product);
    setQuantities({});
    setSelectedProductId(null);
    setSelectedQuantity(0);
  }, [product]);
  
  const handleSavePrice = async () => {
    try {
      if (productData.productType === ProductType.SIMPLE) {
        await updateProductPrice(productData.id, parseFloat(newPrice));
        // Mettre à jour le prix localement
        setProductData((prevProduct) => {
          const updatedProduct = {
            ...prevProduct,
            regularPrice: parseFloat(newPrice),
          };
          onProductUpdate(updatedProduct); // Appeler la fonction de rappel
          return updatedProduct;
        });
      } else if (
        productData.productType === ProductType.VARIABLE &&
        selectedRow
      ) {
        await updateVariationPrice(selectedRow, parseFloat(newPrice));
        // Mettre à jour le prix de la variation localement
        setProductData((prevProduct) => {
          const updatedVariations = prevProduct.variations.map((variation) =>
            variation.id.toString() === selectedRow
              ? { ...variation, regularPrice: parseFloat(newPrice) }
              : variation
          );
          const updatedProduct = { ...prevProduct, variations: updatedVariations };
          onProductUpdate(updatedProduct); // Appeler la fonction de rappel
          return updatedProduct;
        });
      }
      handleClosePriceDialog();
      // Afficher un message de succès
      setSnackbarMessage("Le prix a été mis à jour avec succès.");
      setSnackbarSeverity("success");
      setSnackbarOpen(true);
    } catch (error) {
      console.error("Failed to update price:", error);
      // Afficher un message d'erreur
      setSnackbarMessage("Échec de la mise à jour du prix.");
      setSnackbarSeverity("error");
      setSnackbarOpen(true);
    }
  };

  const handleSaveBulkPrice = async () => {
    // Type guard to ensure we only proceed with variable products
    if (product.productType !== ProductType.VARIABLE) {
      console.error("Bulk price update is only for variable products.");
      return;
    }
  
    try {
      const newPriceValue = parseFloat(newPrice);
      if (isNaN(newPriceValue)) {
        throw new Error("Le prix doit être un nombre valide.");
      }
  
      // dataFiltered is now guaranteed to be IVariationItem[]
      const updatedVariations = dataFiltered.map((variation) => ({
        ...variation,
        regularPrice: newPriceValue,
      }));
  
      // Update variations via API (if applicable)
      await Promise.all(
        updatedVariations.map((variation) =>
          updateVariationPrice(variation.id.toString(), newPriceValue)
        )
      );
  
      // Update local state
      setProductData((prevProduct) => {
        const newVariations = prevProduct.variations.map((variation) => {
          const updatedVariation = updatedVariations.find(
            (uv) => uv.id === variation.id
          );
          return updatedVariation
            ? { ...variation, regularPrice: newPriceValue }
            : variation;
        });
        const updatedProduct = { ...prevProduct, variations: newVariations };
        onProductUpdate(updatedProduct); // Notify parent of the update
        return updatedProduct;
      });
  
      // Reset UI state
      setOpenBulkPriceDialog(false);
      setNewPrice("");
      setSnackbarMessage("Tous les prix ont été mis à jour avec succès.");
      setSnackbarSeverity("success");
      setSnackbarOpen(true);
    } catch (error) {
      console.error("Failed to update bulk prices:", error);
      setSnackbarMessage("Échec de la mise à jour des prix.");
      setSnackbarSeverity("error");
      setSnackbarOpen(true);
    }
  };
  
  
  const attributeNames = useMemo(() => {
    let colorName = "";
    let labelName = "";
    let choiceName = "";

    if (
      product.productType === ProductType.VARIABLE &&
      Array.isArray(product.variations)
    ) {
      product.variations.forEach((variation) => {
        variation.attributes.forEach((attribute) => {
          if (
            attribute.type === AttributeType.COLOR 
          ) {
            colorName = attribute.attributeName;
          }
          if (
            attribute.type === AttributeType.LABEL
          ) {
            labelName = attribute.attributeName;
          }
          if (
            attribute.type === AttributeType.CHOICE
          ) {
            choiceName =attribute.attributeName;
          }
        });
      });
    }

    return { color: colorName, label: labelName, choice: choiceName };
  }, [product.variations, product.productType]);

  // Collect all unique attribute names
  const allAttributeNames = useMemo(() => {
    if (product.productType !== ProductType.VARIABLE || !product.variations) {
      return [];
    }

    const namesSet = new Set<string>();
    product.variations.forEach((variation) => {
      variation.attributes.forEach((attr) => {
        // Only add non-empty attributeName
        if (attr.attributeName && attr.attributeName.trim() !== "") {
          namesSet.add(attr.attributeName);
        }
      });
    });
    return Array.from(namesSet);
  }, [product.productType, product.variations]);

  // Build table headers dynamically from all attribute names
  const getDynamicTableHead = useCallback(() => {
    const tableHead = [
      { id: "Produit", label: "Produit", width: 250 },
      { id: "id", label: "ID", width: 80 },
      { id: "ugs", label: "UGS", width: 80 },
    ];

    // Add a column for each unique attribute name
    allAttributeNames.forEach((attrName) => {
      tableHead.push({ id: attrName, label: attrName, width: 80 });
    });

    tableHead.push({
      id: "Quantité en stock live",
      label: "Quantité en stock live",
      width: 80,
    });
    tableHead.push({
      id: "Quantité sélectionnée",
      label: "Quantité sélectionnée",
      width: 80,
    });
    tableHead.push({ id: "prix", label: "Prix", width: 80 });
    tableHead.push({ id: "action", label: "Action", width: 120 });

    return tableHead;
  }, [allAttributeNames]);

  const handleFilters = useCallback(
    (name: string, value: any) => {
      table.onResetPage();
      setFilters((prev) => ({
        ...prev,
        [name]: value,
      }));
    },
    [table]
  );

  // Réinitialiser les filtres
  const handleResetFilters = useCallback(() => {
    setFilters(defaultFilters);
  }, []);
  // Gestion de la recherche et du filtrage
  const dataFiltered = useMemo(() => {
    let filteredVariations = product.variations || []; 

    if (filters.termColor.length > 0) {
      filteredVariations = filteredVariations.filter((variation) =>
        filters.termColor.includes(
          variation.attributes.find((attr) => attr.type === AttributeType.COLOR)
            ?.attributeValue ?? ""
        )
      );
    }

    if (filters.termLabel.length > 0) {
      filteredVariations = filteredVariations.filter((variation) =>
        filters.termLabel.includes(
          variation.attributes.find((attr) => attr.type === AttributeType.LABEL)
            ?.attributeValue ?? ""
        )
      );
    }

    if (filters.termChoice.length > 0) {
      filteredVariations = filteredVariations.filter((variation) =>
        filters.termChoice.includes(
          variation.attributes.find(
            (attr) => attr.type === AttributeType.CHOICE
          )?.attributeValue ?? ""
        )
      );
    }

    if (filters.nameOrUGS.trim() !== "") {
      filteredVariations = filteredVariations.filter(
        (variation) =>
          variation.name
            .toLowerCase()
            .includes(filters.nameOrUGS.toLowerCase()) ||
          variation.ugs.toLowerCase().includes(filters.nameOrUGS.toLowerCase())
      );
    }

    // Tri des variations en fonction de table.order et table.orderBy
    filteredVariations.sort((a, b) => {
      const orderBy = table.orderBy;

      let valueA;
      let valueB;

      switch (orderBy) {
        case 'Produit':
          valueA = a.name || '';
          valueB = b.name || '';
          break;
        case 'id':
          valueA = a.id || 0;
          valueB = b.id || 0;
          break;
        case 'ugs':
          valueA = a.ugs || '';
          valueB = b.ugs || '';
          break;
        default:
          valueA = '';
          valueB = '';
      }

      if (typeof valueA === 'number' && typeof valueB === 'number') {
        return table.order === 'asc' ? valueA - valueB : valueB - valueA;
      } else {
        return table.order === 'asc'
          ? valueA.toString().localeCompare(valueB.toString())
          : valueB.toString().localeCompare(valueA.toString());
      }
    });

    if (product.productType === ProductType.SIMPLE) {
      return [
        {
          id: product.id,
          name: product.name,
          ugs: product.ugs,
          stockLiveConfig: {
            quantity: product.stockLiveConfig.quantity,
            isPreOrderAuthorized: product.stockLiveConfig.isPreOrderAuthorized,
          },
          stockWebConfig: {
            quantity: product.stockWebConfig.quantity,
            isPreOrderAuthorized: product.stockWebConfig.isPreOrderAuthorized,
          },
          regularPrice: product.regularPrice || 0,
          attributes: [],
        },
      ];
    }

    return filteredVariations;
  }, [filters, product, table.order, table.orderBy]);

  const handleOpenPriceDialog = (variationId: string) => {
    setSelectedRow(variationId);
    setOpen(true);
  };

  const handleClosePriceDialog = () => {
    setSelectedRow(null);
    setOpen(false);
  };

  const handlePriceChange = (e: any) => setNewPrice(e.target.value);
  
  
  const handleIncreaseQuantity = (variationId: string, maxQuantity: number) => {
    console.log(quantities[variationId])
    setSelectedProductId(variationId);
    setQuantities((prevQuantities) => {
      const newQuantities = Object.keys(prevQuantities).reduce((acc, id) => {
      acc[id] = id === variationId
        ? (prevQuantities[id] || 0) + 1
        : 0;
      return acc;
    }, {} as { [key: string]: number });

      if (!(variationId in prevQuantities)) {
      newQuantities[variationId] = 1;
     }
     setSelectedQuantity(newQuantities[variationId]);
     handleSelectProduct(variationId,newQuantities[variationId],getProductNameById(variationId));
    return newQuantities;
  });
    
  };
  
  
  const handleDecreaseQuantity = (variationId: string) => {
    setSelectedProductId(variationId);
    const selectedQuantity = Math.max((quantities[variationId] || 0) - 1, 0); 
    setQuantities((prevQuantities) => {
      const newQuantities = Object.keys(prevQuantities).reduce((acc, id) => {
        acc[id] = id === variationId ? selectedQuantity : 0;
        return acc;
      }, {} as { [key: string]: number });
      setSelectedQuantity(newQuantities[variationId]);
      handleSelectProduct(variationId,newQuantities[variationId],getProductNameById(variationId));
      return newQuantities;
    });

  };

  const getProductNameById = (id: any): string => {
    if(product.productType==ProductType.SIMPLE){
      return product.name;
    }
    const variation = product.variations.find((p) => p.id === id); 
    return variation ? variation.name : 'Unknown Product';
 };
  
  const handleSelectProduct = (productId: any,quantity:number,name:string) => {
        onSelect(productId,quantity,product.productType,name);
    
  };
  

  useEffect(() => {


    if (product.productType === ProductType.VARIABLE && product.variations) {
      const initialQuantities = product.variations.reduce((acc: { [key: string]: number }, variation) => {
        acc[variation.id] =1; 
        return acc;
      }, {});
      setQuantities(initialQuantities);
    } else if (product.productType === ProductType.SIMPLE) {
      setQuantities({ [product.id]:  1 }); 
    }

  }, [product]);
  return (
    <Grid xs={12}>
      <Card>
        <Stack
          direction="column"
          divider={<Divider />}
          spacing={2}
          sx={{ ml: 2 }}
        >
          <Box sx={{ display: "flex", flexDirection: "row" }}>
            <Typography variant="subtitle1" gutterBottom mt={2}>
              Produit sélectionné
            </Typography>
            
          </Box>

          <Box sx={{ display: "flex", flexDirection: "row", pl: "40px" }}>
            <Avatar
              alt={product.name}
              src={ClearUrl(product.mainImagePath)}
              variant="square" // ou 'circular', 'rounded'
              sx={{
                width: 96,
                height: 96,
                objectFit: "contain", // Contenir l'image sans étirer
                borderRadius: 2, // Pour un léger arrondi
              }}
            />

            <ListItemText
              primary={
                <Box
                  component="div"
                  sx={{ typography: "body2", color: "text.disabled" }}
                >
                  {product.name}
                </Box>
              }
              sx={{
                display: "flex",
                flexDirection: "column",
                justifyContent: "center",
                pl: 2,
              }}
            />
          </Box>
        </Stack>

        {product.productType === ProductType.VARIABLE  &&
          (
            <>
              <VariationTableToolbar
                filters={filters}
                onFilters={handleFilters}
                onResetFilters={handleResetFilters}
                attributeNames={attributeNames}
                termColorOptions={
                  product.productType === ProductType.VARIABLE
                    ? product.variations
                        .map(
                          (variation) =>
                            variation.attributes.find(
                              (attr) => attr.type === AttributeType.COLOR
                            )?.attributeValue || ""
                        )
                        .filter((value) => value.trim() !== "")
                    : []
                }
                termLabelOptions={
                  product.productType === ProductType.VARIABLE
                    ? product.variations
                        .map(
                          (variation) =>
                            variation.attributes.find(
                              (attr) => attr.type === AttributeType.LABEL
                            )?.attributeValue || ""
                        )
                        .filter((value) => value.trim() !== "")
                    : []
                }
                termChoiceOptions={
                  product.productType === ProductType.VARIABLE
                    ? product.variations
                        .map(
                          (variation) =>
                            variation.attributes.find(
                              (attr) => attr.type === AttributeType.CHOICE
                            )?.attributeValue || ""
                        )
                        .filter((value) => value.trim() !== "")
                    : []
                }
                onBulkPriceClick={() => setOpenBulkPriceDialog(true)}
              />

              <VariationTableFiltersResult
                filters={filters}
                onFilters={handleFilters}
                onResetFilters={handleResetFilters}
                results={dataFiltered.length}
                sx={{ pl: "40px" }}
              />
            </>
          )}

        <CardContent>
          <TableContainer sx={{ ml: 2, mr: 3 }}>
            <Table size="medium" sx={{ minWidth: 960 }}>
              <TableHeadCustom
                order={table.order}
                orderBy={table.orderBy}
                headLabel={getDynamicTableHead()}
                rowCount={dataFiltered.length}
                numSelected={table.selected.length}
                onSort={table.onSort}
              />
              <TableBody>
                {dataFiltered.map((variation) => (
                  <TableRow  key={variation.id}
                  onClick={() => {
                    setSelectedProductId(variation.id);
                    setQuantities((prevQuantities) => {
                      const newQuantities = Object.keys(prevQuantities).reduce((acc, id) => {
                        acc[id] = id === variation.id.toString()
                          ? (prevQuantities[id] || 1) 
                          : 1;
                        return acc;
                      }, {} as { [key: string]: number });
                      handleSelectProduct(variation.id,newQuantities[variation.id],getProductNameById(variation.id));
                      return newQuantities;
                    });
                  }}
                  sx={{
                    backgroundColor:
                    selectedProductId === variation.id ? "#E0E0E0" : "inherit", 
                    cursor: "pointer", 
                   }}
                 >
                    <TableCell>
                      <Stack direction="row">
                        <Avatar
                          alt={variation.name}
                          src={
                            variation.attributes.find(
                              (attribute) =>
                                attribute.type === AttributeType.IMAGE
                            )?.attributeValue ||
                            ClearUrl(product.mainImagePath)
                          }
                          variant="rounded" 
                          sx={{
                            width: 64,
                            height: 64,
                            mr: 2,
                            objectFit: "cover", 
                          }}
                        />

                        <ListItemText
                          primary={
                            <Box
                              component="div"
                              sx={{
                                typography: "body2",
                                color: "text.disabled",
                              }}
                            >
                              {variation.name}
                            </Box>
                          }
                          sx={{
                            display: "flex",
                            flexDirection: "column",
                            justifyContent: "center",
                          }}
                        />
                      </Stack>
                      <Stack direction="column">
                        {variation.stockLiveConfig.quantity > 0 ? (
                          <Chip
                            label="en stock"
                            size="small"
                            sx={{
                              backgroundColor: "#e8f5e9", // Vert clair pour en stock
                              color: "#2e7d32", // Vert foncé pour le texte
                              mt: 0.5,
                              maxWidth: "70px",
                              overflow: "hidden",
                              textOverflow: "ellipsis",
                            }}
                          />
                        ) : (
                          <Chip
                            label="rupture de stock"
                            size="small"
                            sx={{
                              backgroundColor: "#ffebee", // Rouge clair pour rupture de stock
                              color: "#d32f2f", // Rouge foncé pour le texte
                              mt: 0.5,
                              maxWidth: "130px",
                              overflow: "hidden",
                              textOverflow: "ellipsis",
                            }}
                          />
                        )}

                        {variation.stockLiveConfig.isPreOrderAuthorized==="AUTHORIZE" ? (
                          <Chip
                            size="small"
                            label="réapprovisionnement autorisé"
                            sx={{
                              backgroundColor: "#f3e5f5",
                              color: "#4a148c",
                              mt: 0.5,
                              maxWidth: "200px",
                              overflow: "hidden",
                              textOverflow: "ellipsis",
                            }}
                          />
                        ) : (
                          <Chip
                            label="réapprovisionnement non autorisé"
                            size="small"
                            sx={{
                              backgroundColor: warning, // Rouge clair pour rupture de stock
                              color: "000", // Rouge foncé pour le texte
                              mt: 0.5,
                              maxWidth: "230px",
                              overflow: "hidden",
                              textOverflow: "ellipsis",
                            }}
                          />
                        )}
                      </Stack>
                    </TableCell>
                    <TableCell>{variation.id}</TableCell>
                    <TableCell>{variation.ugs}</TableCell>
                    {/* Dynamically display columns for each attribute name */}
                    {allAttributeNames.map((attrName) => {
                      const matchingAttr = variation.attributes.find(
                        (attr) => attr.attributeName === attrName
                      );
                      return (
                        <TableCell key={attrName}>
                          {matchingAttr?.attributeValue ?? ""}
                        </TableCell>
                      );
                    })}
                    <TableCell>{variation.stockLiveConfig.quantity}</TableCell>


                    <TableCell>
                      <Box display="flex" alignItems="center">
                        <Button
                          onClick={() =>
                            handleDecreaseQuantity(variation.id.toString())
                          }
                        >
                          -
                        </Button>
                        <TextField
                           value={quantities[variation.id] || 0}
                           onChange={(e) => {
                            const newValue = Number(e.target.value);
                  
                              setQuantities((prevQuantities) => ({
                                ...prevQuantities,
                                [variation.id]: newValue, 
                              }));
                            
                          }}
                          
                          type="number"
                          inputProps={{
                            min: 0,
                            max: variation.stockLiveConfig.quantity,
                          }}
                          sx={{ width: "50px", mx: 1 }}
                        />
                        <Button
                          onClick={() =>
                            handleIncreaseQuantity(
                              variation.id.toString(),
                              variation.stockLiveConfig.quantity ?? 1
                            )
                          }
                        >
                          +
                        </Button>
                      </Box>
                    </TableCell>

                    <TableCell>{variation.regularPrice} €</TableCell>
                    <TableCell>
                      <Button
                        variant="outlined"
                        onClick={() =>
                          handleOpenPriceDialog(variation.id.toString())
                        }
                        sx={{ mr: 2 }}
                      >
                        Modifier le prix
                      </Button>
                    </TableCell>
                  </TableRow>
                ))}
              </TableBody>
            </Table>
          </TableContainer>
        </CardContent>

        {/* Dialog pour modifier le prix */}
        <Dialog
          open={open}
          onClose={handleClosePriceDialog}
          fullWidth
          maxWidth="xs"
        >
          <DialogTitle>Modifier le Prix</DialogTitle>
          <DialogContent>
            <TextField
              autoFocus
              margin="normal"
              id="newPrice"
              label="Nouveau Prix (€)"
              type="number"
              fullWidth
              variant="outlined"
              value={newPrice}
              onChange={handlePriceChange}
              InputProps={{
                startAdornment: (
                  <InputAdornment position="start">€</InputAdornment>
                ),
              }}
            />
          </DialogContent>
          <DialogActions>
            <Button
              onClick={handleClosePriceDialog}
              color="secondary"
              variant="outlined"
            >
              Annuler
            </Button>
            <Button
              onClick={handleSavePrice}
              color="primary"
              variant="contained"
            >
              Enregistrer
            </Button>
          </DialogActions>
        </Dialog>
        <Dialog
          open={openBulkPriceDialog}
          onClose={() => setOpenBulkPriceDialog(false)}
          fullWidth
          maxWidth="xs"
        >
          <DialogTitle>Modifier tous les prix</DialogTitle>
          <DialogContent>
            <TextField
              autoFocus
              margin="normal"
              id="newBulkPrice"
              label="Nouveau Prix pour toutes les variations (€)"
              type="number"
              fullWidth
              variant="outlined"
              value={newPrice}
              onChange={handlePriceChange}
              InputProps={{
                startAdornment: (
                  <InputAdornment position="start">€</InputAdornment>
                ),
              }}
            />
          </DialogContent>
          <DialogActions>
            <Button
              onClick={() => setOpenBulkPriceDialog(false)}
              color="secondary"
              variant="outlined"
            >
              Annuler
            </Button>
            <Button
              onClick={handleSaveBulkPrice} // Define this function next
              color="primary"
              variant="contained"
            >
              Enregistrer
            </Button>
          </DialogActions>
        </Dialog>
      </Card>
      <Snackbar
        open={snackbarOpen}
        autoHideDuration={6000}
        onClose={() => setSnackbarOpen(false)}
        anchorOrigin={{ vertical: 'top', horizontal: 'right' }} 
      >
        <Alert
          onClose={() => setSnackbarOpen(false)}
          severity={snackbarSeverity}
          sx={{ width: "100%" }}
        >
          {snackbarMessage}
        </Alert>
      </Snackbar>
    </Grid>
    
  );
}
