import React, { useCallback } from "react";
import Stack from "@mui/material/Stack";
import MenuItem from "@mui/material/MenuItem";
import Checkbox from "@mui/material/Checkbox";
import TextField from "@mui/material/TextField";
import InputLabel from "@mui/material/InputLabel";
import FormControl from "@mui/material/FormControl";
import OutlinedInput from "@mui/material/OutlinedInput";
import Select, { SelectChangeEvent } from "@mui/material/Select";
import InputAdornment from "@mui/material/InputAdornment";
import Iconify from "@/shared/components/iconify";
import { Filters } from "./Product-Variation";
import { AttributeType, IVariationAttribute } from "@/shared/types/product";
import { Button } from "@mui/material";

type VariationTableToolbarProps = {
  filters: Filters;
  onFilters: (name: string, value: any) => void;
  onResetFilters: () => void;
  termColorOptions: string[];
  termLabelOptions: string[];
  termChoiceOptions: string[];
  attributeNames: {
    color: string; // Nom pour l'attribut de couleur
    label: string; // Nom pour l'attribut de label
    choice: string; // Nom pour l'attribut de choix
  };
  onBulkPriceClick: () => void; // New prop to handle bulk price button click
};

export default function VariationTableToolbar({
  filters,
  onFilters,
  onResetFilters,
  termColorOptions,
  termLabelOptions,
  termChoiceOptions,
  attributeNames, // Noms dynamiques des attributs
  onBulkPriceClick, // New prop to handle bulk price button click
}: VariationTableToolbarProps) {
  const handleFilterNameOrUGS = useCallback(
    (event: React.ChangeEvent<HTMLInputElement>) => {
      const value = event.target.value;
      onFilters('nameOrUGS', value);
    },
    [onFilters]
  );

  const handleFilterColor = useCallback(
    (event: SelectChangeEvent<string[]>) => {
      onFilters(
        "termColor",
        typeof event.target.value === "string"
          ? event.target.value.split(",")
          : event.target.value
      );
    },
    [onFilters]
  );

  const handleFilterLabel = useCallback(
    (event: SelectChangeEvent<string[]>) => {
      onFilters(
        "termLabel",
        typeof event.target.value === "string"
          ? event.target.value.split(",")
          : event.target.value
      );
    },
    [onFilters]
  );

  const handleFilterChoice = useCallback(
    (event: SelectChangeEvent<string[]>) => {
      onFilters(
        "termChoice",
        typeof event.target.value === "string"
          ? event.target.value.split(",")
          : event.target.value
      );
    },
    [onFilters]
  );

  // Filtrer les options pour éviter les chaînes vides
  // Rendre les options de termes uniques
  const validTermColorOptions = Array.from(
    new Set(termColorOptions.filter((option) => option.trim() !== ""))
  );
  const validTermLabelOptions = Array.from(
    new Set(termLabelOptions.filter((option) => option.trim() !== ""))
  );
  const validTermChoiceOptions = Array.from(
    new Set(termChoiceOptions.filter((option) => option.trim() !== ""))
  );

  return (
    <Stack spacing={2} direction={{ xs: "column", md: "row" }} sx={{ p: 2.5  , pl: 5 }}>
      {/* Recherche par nom ou UGS */}        
        <TextField
          sx={{ width: "90%" }}
          value={filters.nameOrUGS}
          onChange={handleFilterNameOrUGS}
          placeholder="Rechercher par nom ou UGS..."
          InputProps={{
            startAdornment: (
          <InputAdornment position="start">
            <Iconify icon="eva:search-fill" sx={{ color: "text.disabled" }} />
          </InputAdornment>
            ),
          }}
        />
        <Button
          variant="outlined"
          onClick={onBulkPriceClick}
          sx={{ width: "10%" }}
        >
            Modifier toutes les prix
        </Button>
      {/* Sélection de couleur - afficher uniquement si des options valides existent */}
      {validTermColorOptions.length > 0 && (
        <FormControl sx={{ width: { xs: 1, md: 200 } }}>
          <InputLabel>{attributeNames.color}</InputLabel>{" "}
          {/* Nom dynamique de l'attribut */}
          <Select
            multiple
            value={filters.termColor}
            onChange={handleFilterColor}
            input={<OutlinedInput label={attributeNames.color} />}
            renderValue={(selected: string[]) => selected.join(", ")}
          >
            {validTermColorOptions.map((option) => (
              <MenuItem key={option} value={option}>
                <Checkbox
                  size="small"
                  checked={filters.termColor.includes(option)}
                />
                {option}
              </MenuItem>
            ))}
          </Select>
        </FormControl>
      )}

      {/* Sélection de label - afficher uniquement si des options valides existent */}
      {validTermLabelOptions.length > 0 && (
        <FormControl sx={{ width: { xs: 1, md: 200 } }}>
          <InputLabel>{attributeNames.label}</InputLabel>{" "}
          {/* Nom dynamique de l'attribut */}
          <Select
            multiple
            value={filters.termLabel}
            onChange={handleFilterLabel}
            input={<OutlinedInput label={attributeNames.label} />}
            renderValue={(selected: string[]) => selected.join(", ")}
          >
            {validTermLabelOptions.map((option) => (
              <MenuItem key={option} value={option}>
                <Checkbox
                  size="small"
                  checked={filters.termLabel.includes(option)}
                />
                {option}
              </MenuItem>
            ))}
          </Select>
        </FormControl>
      )}

      {/* Sélection de choix - afficher uniquement si des options valides existent */}
      {validTermChoiceOptions.length > 0 && (
        <FormControl sx={{ width: { xs: 1, md: 200 } }}>
          <InputLabel>{attributeNames.choice}</InputLabel>{" "}
          {/* Nom dynamique de l'attribut */}
          <Select
            multiple
            value={filters.termChoice}
            onChange={handleFilterChoice}
            input={<OutlinedInput label={attributeNames.choice} />}
            renderValue={(selected: string[]) => selected.join(", ")}
          >
            {validTermChoiceOptions.map((option) => (
              <MenuItem key={option} value={option}>
                <Checkbox
                  size="small"
                  checked={filters.termChoice.includes(option)}
                />
                {option}
              </MenuItem>
            ))}
          </Select>
        </FormControl>
      )}
    </Stack>
  );
}
