"use client";
import React, { useState, useEffect } from "react";
import dayjs from "dayjs";
import utc from "dayjs/plugin/utc";
import timezone from "dayjs/plugin/timezone";
import {
  Card,
  CardContent,
  Typography,
  Button,
  Grid,
  Stack,
  CircularProgress,
  Snackbar, // Importation de Snackbar
  Alert, // Importation de Alert pour l'intérieur du Snackbar
  Dialog,
  DialogTitle,
  DialogContent,
  DialogActions,
} from "@mui/material";
import { useForm, FormProvider } from "react-hook-form";
import { fDate, fTime } from "@/utils/format-time";
import {
  useGetSaleSession,
  useUpdateExpirationDate,
} from "@/shared/api/saleSession";
import ProductListDialog from "./add-products";
import ProductVariationsCard from "./AllVariation/Product-Variation";
import AllUsersView from "./all-users/all-users-view";
import AllOrderView from "./all-Orders/all-orders-view";
import AllNewUsersView from "./all-NewUsers/All-NewUsers-view";
import AnalyticsWidgetSummary from "./statistics-widget-summary";
import CustomBreadcrumbs from "@/components/custom-breadcrumbs";
import { paths } from "@/routes/paths";
import { IProductListItem, IProductSessionItem, ProductType } from "@/shared/types/product";
import { RHFMobileDateTimePicker } from "@/shared/components/hook-form/rhf-date-picker";
import { useAddProductToCartFromLive, useAddProductToUnregisteredCartFromLive } from "@/shared/api/saleSession";
import { useSnackbar } from "@/components/snackbar";
import { IVariation } from "@/shared/types/invoice";
import { useSearchParams } from 'next/navigation';






type Props = {
  liveId: string;
};
interface ProductDetails {
  productId: string | null;
  quantity: number;
  productType: "SIMPLE" | "VARIABLE" | null;
}
dayjs.extend(utc);
dayjs.extend(timezone);

const convertUTCToLocalTime = (utcDate: string) => {
  const localTimeZone = Intl.DateTimeFormat().resolvedOptions().timeZone;
  return dayjs.utc(utcDate).tz(localTimeZone).format('YYYY-MM-DD HH:mm');
};



export default function LiveCard({ liveId }: Props) {
  const searchParams = useSearchParams();
  const name = searchParams.get("name");

  const { enqueueSnackbar } = useSnackbar();
  const [snackbarMessage, setSnackbarMessage] = useState<string | null>(null);
  const { SaleSessionData, SaleSessionLoading, SaleSessionMutate } =
    useGetSaleSession(liveId);
  const { addProductToCartFromLive } = useAddProductToCartFromLive();
  const { addProductToUnregisteredCartFromLive } = useAddProductToUnregisteredCartFromLive();
  const [isExpired, setIsExpired] = useState(false);
  const [dialogOpen, setDialogOpen] = useState(false);
  const [selectedProduct, setSelectedProduct] =
    useState<IProductSessionItem | null>(null);


  const [openSnackbar, setOpenSnackbar] = useState(false); // Pour ouvrir/fermer le Snackbar
  const [snackbarSeverity, setSnackbarSeverity] = useState<"success" | "error">("success"); // Définir le type de Snackbar
  const [useWebStockForLive, setUseWebStockForLive] = useState<null | boolean>(null);
  const [showWebStockDialog, setShowWebStockDialog] = useState(false);

  const [selectedProductId, setSelectedProductId] = useState<any>(null);
  const [selectedQuantity, setSelectedQuantity] = useState<any>(0);
  const [selectedProductType, setSelectedProductType] = useState<ProductType | null>(null);
  const [selectedProductName, setSelectedProductName] = useState<string>('');

  const [selectedClientId, setSelectedClientId] = useState<any>(null);
  const [selectedClientEmail, setSelectedClientEmail] = useState<string>('');
  const [isRegistered, setIsRegistered] = useState(false);


  const handleProductUpdate = (updatedProduct: IProductSessionItem) => {
    setSelectedProduct(updatedProduct);
  };





  // Setup the form with default values for expiration date
  const methods = useForm({
    defaultValues: {
      expired: "", // Default value will be updated plus tard avec les données récupérées
    },
  });

  const { setValue } = methods;

  useEffect(() => {
    if (SaleSessionData) {
      const expirationDate = convertUTCToLocalTime(SaleSessionData.expirationDate.toString());
      setValue("expired", expirationDate);

      // Vérifier si la session est expirée
      const expirationDayjs = dayjs(expirationDate);
      if (expirationDayjs.isBefore(dayjs())) {
        setIsExpired(true);
      } else {
        setIsExpired(false);
      }
    }
  }, [SaleSessionData, setValue]);


  const handleOpenDialog = () => {
    setDialogOpen(true);
  };

  const handleCloseDialog = () => {
    setDialogOpen(false);
  };

  const handleProductSelect = (selectedProduct: IProductSessionItem) => {
    setSelectedProduct(selectedProduct);
    setDialogOpen(false);

    // Réinitialiser les états liés au produit sélectionné
    setSelectedProductId(null);
    setSelectedQuantity(0);
    setSelectedProductType(null);
    setSelectedProductName('');
  };

  const handleUpdateExpirationDate = async () => {
    const expirationDate = methods.getValues("expired"); // Récupérer la nouvelle date d'expiration
    if (expirationDate) {
      // Convertir la date locale en UTC
      const utcExpirationDate = dayjs(expirationDate).tz(Intl.DateTimeFormat().resolvedOptions().timeZone).utc().format();

      try {
        await useUpdateExpirationDate(utcExpirationDate, liveId); // Appeler le hook pour mettre à jour la date
        console.log(utcExpirationDate); // La date en UTC
        SaleSessionMutate(); // Mettre à jour les données après modification
        setSnackbarMessage("Date d'expiration modifiée avec succès.");
        setSnackbarSeverity("success"); // Définir la sévérité sur succès
        setOpenSnackbar(true); // Ouvrir le Snackbar de succès
      } catch (error) {
        setSnackbarMessage("La modification de la date d'expiration a échoué.");
        setSnackbarSeverity("error"); // Définir la sévérité sur erreur
        setOpenSnackbar(true); // Ouvrir le Snackbar d'erreur
      }
    }
  };



  const handleCloseSnackbar = () => {
    setOpenSnackbar(false); // Fermer le Snackbar
  };

  const renderDetails = () => (
    <Grid xs={12}>
      <Grid container spacing={3}>
        <Grid item xs={12} sm={6} md={4}>
          <AnalyticsWidgetSummary
            title="Nombre de produits vendus"
            total={SaleSessionData?.orderProductsNumber || 0}
            icon={<img alt="icon" src="/assets/icons/glass/ic_glass_bag.png" />}
            onClick={ () => console.log(SaleSessionData)}
          />
        </Grid>

        <Grid item xs={12} sm={6} md={4}>
          <AnalyticsWidgetSummary
            title="Nombre des clients"
            total={SaleSessionData?.clientsNumber || 0}
            icon={
              <img alt="icon" src="/assets/icons/glass/ic_glass_users.png" />
            }
          />
        </Grid>

        <Grid item xs={12} sm={6} md={4}>
          <AnalyticsWidgetSummary
            title="Total vente"
            total={SaleSessionData?.totalSales || 0}
            icon={<img alt="icon" src="/assets/icons/glass/ic_glass_buy.png" />}
            showEuroSymbol={true}
          />
        </Grid>

        <Grid item xs={12} sm={6} md={4}>
          <AnalyticsWidgetSummary
            title="Produits consulté"
            total={SaleSessionData?.consultedProductsNumber || 0}
            icon={
              <img alt="icon" src="/assets/icons/glass/ic_glass_message.png" />
            }
          />
        </Grid>

        <Grid item xs={12} sm={6} md={4}>
          <AnalyticsWidgetSummary
            title="Produits supprimé"
            total={SaleSessionData?.deletedProductsNumber || 0}
            icon={
              <img alt="icon" src="/assets/icons/glass/ic_glass_live.png" />
            }
          />
        </Grid>

        <Grid item xs={12} sm={6} md={4}>
          <AnalyticsWidgetSummary
            title="Produits payé"
            total={SaleSessionData?.paidProductsNumber || 0}
            icon={<img alt="icon" src="/assets/icons/glass/ic_glass_buy.png" />}
          />
        </Grid>
      </Grid>

      <Card style={{ margin: "20px 0" }}>
        <CardContent>
          <Typography variant="subtitle2" color="textSecondary">
            Date de création:{" "}
            {fDate(SaleSessionData.startedDate) +
              "  " +
              fTime(SaleSessionData.startedDate).toString()}
          </Typography>
          <Typography variant="subtitle2" color="textSecondary">
            Dernière mise à jour:{" "}
            {fDate(
              SaleSessionData.lastModification || SaleSessionData.startedDate
            ) +
              "  " +
              fTime(
                SaleSessionData.lastModification || SaleSessionData.startedDate
              ).toString()}
          </Typography>

          <Typography variant="subtitle2" color="textSecondary">
            Description: {SaleSessionData.description}
          </Typography>
          <Typography variant="subtitle2" color="textSecondary">
            Date d'expiration :
          </Typography>

          <div
            style={{
              display: "flex",
              alignItems: "center",
              marginTop: "10px",
            }}
          >
            <FormProvider {...methods}>
              <div
                style={{
                  display: "flex",
                  alignItems: "center",
                  marginTop: "10px",
                  gap: "10px",
                }}
              >
                {/* Affichage du date picker */}
                <RHFMobileDateTimePicker
                  name="expired"
                  label="Date d'expiration"
                />
                <Button
                  variant="contained"
                  color="primary"
                  onClick={handleUpdateExpirationDate}
                >
                  Modifier
                </Button>
              </div>
            </FormProvider>

          </div>
          {isExpired && (
            <Typography variant="subtitle1" color="error" sx={{ mt: 2, ml: 3 }}>
              ⚠️ La session a expiré.
            </Typography>
          )}

        </CardContent>

        <CardContent style={{ textAlign: "center" }}>
          <Button
            variant="contained"
            color="primary"
            onClick={handleOpenDialog}
          >
            Rechercher un produit
          </Button>
        </CardContent>
        <ProductListDialog
          open={dialogOpen}
          onClose={handleCloseDialog}
          onSelect={handleProductSelect}
        />
      </Card>
    </Grid>
  );


  const handleSnackbarClose = () => {
    setOpenSnackbar(false);
  };

  const handleSelectProduct = (id: any, quantity: number, type: ProductType, name: string) => {
    setSelectedProductId(id);
    setSelectedQuantity(quantity);
    setSelectedProductType(type);
    setSelectedProductName(name);

  };

  const getSelectedProduct = () => {
    return selectedProduct?.productType === ProductType.SIMPLE
      ? selectedProduct
      : selectedProduct?.variations.find((p) => p.id === selectedProductId);
  };

  const handleAddToCart = async (clientId: string, isRegisteredFlag: boolean, clientEmail: string) => {
    setSelectedClientId(clientId);
    setIsRegistered(isRegisteredFlag);
    setSelectedClientEmail(clientEmail);

    if (!selectedProductId || selectedQuantity === 0) {
      setSnackbarMessage("Veuillez sélectionner un produit et une quantité.");
      setSnackbarSeverity("error");
      setOpenSnackbar(true);
      return;
    }

    const product = getSelectedProduct();

    if (!product) {
      enqueueSnackbar("Produit introuvable.", { variant: "error" });
      return;
    }

    const liveStockQuantity = product?.stockLiveConfig?.quantity ?? 0;
    const webStockQuantity = product?.stockWebConfig?.quantity ?? 0;

    if (liveStockQuantity <= 0 || selectedQuantity > liveStockQuantity) {
      if (webStockQuantity >= selectedQuantity) {
        setShowWebStockDialog(true);
        return;
      }
    }
    await addToCart(false, isRegisteredFlag, clientId, clientEmail);

  };

  const addToCart = async (
    useWebStockForLiveOverride: boolean,
    isRegistered: boolean,
    clientId: string,
    clientEmail: string
  ) => {
    const product = getSelectedProduct();

    if (!product) {
      enqueueSnackbar("Produit introuvable.", { variant: "error" });
      return;
    }

    // Extract the UGS (SKU) from the product
    const ugs = product.ugs // Adjust according to your product object's structure

    const sessionRequest = {
      clientId,
      productId: selectedProductId,
      sessionId: liveId,
      quantity: selectedQuantity,
      type: selectedProductType,
      useWebStockForLive: useWebStockForLiveOverride,
      ugs: ugs, // Include the UGS in the request
    };

    enqueueSnackbar(
      `${selectedProductName} en cours d'ajout pour ${clientEmail}`,
      { variant: "info" }
    );

    try {
      if (isRegistered) {
        await addProductToCartFromLive(sessionRequest);
      } else {
        await addProductToUnregisteredCartFromLive(sessionRequest);
      }

      // Update product stock quantities
      if (useWebStockForLiveOverride) {
        product.stockWebConfig.quantity -= selectedQuantity;
      } else {
        product.stockLiveConfig.quantity -= selectedQuantity;
      }

      setSelectedProduct((prevProduct) => prevProduct ? {
        ...prevProduct,
        ...(prevProduct.productType === ProductType.SIMPLE ? {} : {
          variations: prevProduct.variations.map(variation =>
            variation.id === product.id ? { ...variation } : variation
          )
        }),
      } : null);

      enqueueSnackbar(
        `${selectedProductName} ajouté avec succès pour ${clientEmail}`,
        { variant: "success" }
      );
    } catch (error: any) {
      let errorMessage = `Échec de l'ajout de ${selectedProductName} au panier`;
      if (typeof error === 'string' && error.includes("Product is out of stock")) {
        errorMessage = `${selectedProductName} est en rupture de stock`;
      } else if (typeof error === 'string' && error.includes("Sale session has expired for session id")) {
        errorMessage = "La session de vente a expiré";
      }
      enqueueSnackbar(errorMessage, { variant: "error" });
    }
  };

  const cancelWebStockUsage = () => {
    setShowWebStockDialog(false);
    setUseWebStockForLive(false);
    addToCart(false, isRegistered, selectedClientId, selectedClientEmail);
  };
  const confirmUseWebStock = () => {
    setUseWebStockForLive(true);
    setShowWebStockDialog(false)
    addToCart(true, isRegistered, selectedClientId, selectedClientEmail);
  };

  return (
    <div>
      <CustomBreadcrumbs
        heading={`Panier du live : ${name ? name : "No Name"}`}
        links={[
          {
            name: "Tableau de bord",
            href: paths.dashboard.root,
          },
          {
            name: "Tous les Panier",
            href: paths.dashboard.admin.cart.root,
          },
          {
            name: "Paniers",
          },
        ]}
        sx={{
          mb: { xs: 3, md: 5 },
        }}
      />
      <Stack
        direction="column"
        justifyContent="flex-start"
        alignItems="stretch"
        spacing={4}
      >
        {SaleSessionLoading ? (
          <div style={{ textAlign: "center", margin: "20px 0" }}>
            <CircularProgress />
            <Typography>Chargement des statistiques...</Typography>
          </div>
        ) : (
          renderDetails()
        )}
        {selectedProduct ? (
          <ProductVariationsCard
            product={selectedProduct}
            onSelect={handleSelectProduct}
            onProductUpdate={handleProductUpdate} // Pass the callback function
          />
        ) : (
          <Typography variant="h6" color="textSecondary" align="center">
            Aucun produit sélectionné.
          </Typography>
        )}

        <AllUsersView onAddToCart={handleAddToCart} />
        <AllNewUsersView onAddToCart={handleAddToCart} />
        <AllOrderView liveId={liveId} />
        {/* Snackbar pour afficher les messages de succès ou d'erreur */}

        <Snackbar
          open={openSnackbar}
          autoHideDuration={5000}
          onClose={handleSnackbarClose}
        >
          <Alert
            onClose={handleSnackbarClose}
            severity={snackbarSeverity}
            sx={{ width: '100%' }}
          >
            {snackbarMessage}
          </Alert>
        </Snackbar>
      </Stack>
      {/* Pop-in pour confirmation d'utilisation du stock web */}
      <Dialog open={showWebStockDialog} onClose={cancelWebStockUsage}>
        <DialogTitle>Utiliser le stock web</DialogTitle>
        <DialogContent>
          <Typography>
            Il n'y a pas assez de stock en live. Souhaitez-vous utiliser le stock web pour ajouter ce produit à votre panier ?
          </Typography>
        </DialogContent>
        <DialogActions>
          <Button onClick={cancelWebStockUsage} color="secondary">
            Annuler
          </Button>
          <Button onClick={confirmUseWebStock} color="primary" variant="contained">
            Confirmer
          </Button>
        </DialogActions>
      </Dialog>
    </div>
  );
}
