import React, { useState, useEffect } from 'react';
import {
  FormControlLabel,
  Typography,
  Card,
  CardContent,
  Switch,
  Box,
  Button,
  Snackbar,
} from '@mui/material';
import { saveCartCountConfig, getCartCountConfig } from '@/shared/api/cart';
import { logger } from '@/utils/logger';

function CountdownOptions() {
  const [switchState, setSwitchState] = useState({
    product: false,
    cart: false,
    none: true,
  });
  const [showSnackbar, setShowSnackbar] = useState(false);

  useEffect(() => {
    const fetchConfig = async () => {
      try {
        const config = await getCartCountConfig(); 
        setSwitchState({
          product: config.showCountItem,
          cart: config.showCountCart,
          none: config.dontShowCountCart,
        });
      } catch (error) {
        console.error('Erreur lors de la récupération de la configuration :', error);
      }
    };

    fetchConfig();
  }, []);

  const handleSwitchChange =
    (name: string) => (event: React.ChangeEvent<HTMLInputElement>) => {
      setSwitchState({
        product: name === 'product' ? event.target.checked : false,
        cart: name === 'cart' ? event.target.checked : false,
        none: name === 'none' ? event.target.checked : false,
      });
    };

  const handleSave = async () => {
    logger('Choix enregistré :', switchState);
    try {
      await saveCartCountConfig({
        showCountCart: switchState.cart,
        showCountItem: switchState.product,
        dontShowCountCart: switchState.none,
      });
      setShowSnackbar(true);
    } catch (error) {
      console.error('Erreur lors de l\'enregistrement de la configuration :', error);
    }
  };

  const handleCloseSnackbar = () => {
    setShowSnackbar(false);
  };

  return (
    <Card>
      <CardContent>
        <Typography variant="h6">Sélectionner une Configuration</Typography>
        <Box display="flex" flexDirection="column">
          <FormControlLabel
            control={
              <Switch
                checked={switchState.product}
                onChange={handleSwitchChange('product')}
                color="primary"
                disabled={switchState.cart || switchState.none}
                aria-label="Appliquer le compte à rebours sur chaque produit"
              />
            }
            label="Appliquer le compte à rebours sur chaque produit"
          />

          <FormControlLabel
            control={
              <Switch
                checked={switchState.cart}
                onChange={handleSwitchChange('cart')}
                color="primary"
                disabled={switchState.product || switchState.none}
                aria-label="Appliquer le compte à rebours sur tout le panier"
              />
            }
            label="Appliquer le compte à rebours sur tout le panier"
          />

          <FormControlLabel
            control={
              <Switch
                checked={switchState.none}
                onChange={handleSwitchChange('none')}
                color="primary"
                disabled={switchState.product || switchState.cart}
                aria-label="Ne pas appliquer le compte à rebours"
              />
            }
            label="Ne pas appliquer le compte à rebours"
          />

          <Box display="flex" justifyContent="center" mt={2}>
            <Button
              variant="contained"
              style={{
                backgroundColor: 'black',
                color: 'white',
                maxWidth: '300px',
                width: '100%',
              }}
              onClick={handleSave}
            >
              Enregistrer le choix
            </Button>
          </Box>
        </Box>

    
        <Snackbar
          open={showSnackbar}
          autoHideDuration={3000}
          onClose={handleCloseSnackbar}
          message="Choix enregistré avec succès"
        />
      </CardContent>
    </Card>
  );
}

export default CountdownOptions;