"use client";

import { useState, useEffect, useCallback } from "react";
import Card from "@mui/material/Card";
import Table from "@mui/material/Table";
import Container from "@mui/material/Container";
import TableBody from "@mui/material/TableBody";
import TableContainer from "@mui/material/TableContainer";
import { paths } from "../../../../routes/paths";
import { useRouter } from "../../../../hooks/use-router";


import Scrollbar from "../../../components/scrollbar";
import { useSnackbar } from "../../../../components/snackbar";
import { useSettingsContext } from "../../../components/settings";
import CustomBreadcrumbs from "../../../../components/custom-breadcrumbs";
import {
  useTable,
  emptyRows,
  TableNoData,
  TableEmptyRows,
  TableHeadCustom,
  TableSelectedAction,
  TablePaginationCustom,
} from "../../../components/table";

import { IUserCartTableFilters, IUserCartTableFilterValue, IUserItem } from "@/shared/types/saleSession";
import UserCartTableRow from "./UserCart-table-row";
import UserCartTableToolbar from "./UserCart-table-toolbar";
import UserCartTableFiltersResult from "./UserCart-table-filters-result";
import {  fetchPaginatedClientCarts } from "@/shared/api/cart";
import { ICartDTO } from "@/shared/types/cart";
import { isEqual } from "lodash";
import { CircularProgress, TableCell, TableRow } from "@mui/material";


// ----------------------------------------------------------------------

const TABLE_HEAD = [
  { id: "email", label: "Utilisateur", width: '30%' },
  { id: "phoneNumber", label: "N° téléphone", width: '20%' ,align: "center"},
  { id: "totalQuantity", label: "Nombre Produits", width: '20%' },
  { id: "totalAmount", label: "Prix Total", width: '20%' },
  { id: "", width: '10%' },
];

const defaultFilters: IUserCartTableFilters = {
  keyword: "",
  productKeyword:""
};

// ----------------------------------------------------------------------

export default function UserCartListView() {
  const { enqueueSnackbar } = useSnackbar();

  const table = useTable();

  const settings = useSettingsContext();

  const router = useRouter();

  const [filters, setFilters] = useState(defaultFilters);

    const {
    carts,
    totalCarts,
    totalPages,
    pageNumber,
    pageSize,
    loading,
    isError,
    revalidate,
  } = fetchPaginatedClientCarts({  
    keyword: filters.keyword,
    productKeyword:filters.productKeyword,
    page: table.page,
    pageSize: table.rowsPerPage, });
  const [tableData, setTableData] = useState<ICartDTO[]>([]);


  useEffect(() => {
    if (carts && carts.length > 0) {
      setTableData(carts);
    }
  }, [carts]);
  const denseHeight = table.dense ? 56 : 56 + 20;
  
  const canReset = !isEqual(defaultFilters, filters);


  const notFound = totalCarts === 0 && canReset;

  const handleFilters = useCallback(
    (name: string, value: IUserCartTableFilterValue) => {
      table.onResetPage();
      setFilters((prevState) => ({
        ...prevState,
        [name]: value,
      }));
    },
    [table]
  );

  useEffect(() => {
    if(filters.keyword||
      filters.productKeyword
    ){
     revalidate();
    }
   }, [filters,revalidate])

  const handleResetFilters = useCallback(() => {
    setFilters(defaultFilters);
  }, []);

  const handleDeleteRow = useCallback(
    (uid: string) => {
      const deleteRow = tableData.filter((row) => row.clientDTO.uid.toString() !== uid);

      enqueueSnackbar("Suppression réussie!");

      setTableData(deleteRow);

      table.onUpdatePageDeleteRow(pageSize);
    },
    [pageSize, enqueueSnackbar, table, tableData]
  );
  const refresh = () => {
    revalidate()
  };
  
  
  const handleViewRow = useCallback(
    (uid: string) => {
      router.push(paths.dashboard.admin.order.details(uid));
    },
    [router]
  );
 

  return (
    <>
      <Container maxWidth={settings.themeStretch ? false : "lg"}>
        <CustomBreadcrumbs
          heading="Liste des paniers clients"
          links={[
            {
              name: "Tableau de bord",
              href: paths.dashboard.root,
            },
            {
              name: "Paniers",
              href: paths.dashboard.admin.order.all_orders,
            },
            {
              name: "Liste des Paniers",
            },
          ]}
          sx={{
            mb: { xs: 3, md: 5 },
          }}
        />

        <Card>
          <UserCartTableToolbar
            filters={filters}
            onFilters={handleFilters}
            //
          />

          {canReset && (
            <UserCartTableFiltersResult
              filters={filters}
              onFilters={handleFilters}
              //
              onResetFilters={handleResetFilters}
              //
              results={totalCarts}
              sx={{ p: 2.5, pt: 0 }}
            />
          )}

          <TableContainer sx={{ position: "relative", overflow: "unset" }}>
            <TableSelectedAction
              dense={table.dense}
              numSelected={table.selected.length}
              rowCount={totalCarts}
              onSelectAllRows={(checked) =>
                table.onSelectAllRows(
                  checked,
                  tableData.map((row) => row.clientDTO.uid.toString())
                )
              }
             
            />

            <Scrollbar>
              <Table
                size={table.dense ? "small" : "medium"}
                sx={{ minWidth: 960 }}
              >
                <TableHeadCustom
                  order={table.order}
                  orderBy={table.orderBy}
                  headLabel={TABLE_HEAD}
                  rowCount={totalCarts}
                  numSelected={table.selected.length}
                  onSort={table.onSort}
                  
                />

                <TableBody>
              {loading ? (
                    <TableRow>
                      <TableCell
                        colSpan={TABLE_HEAD.length}
                        style={{ textAlign: "center", padding: "20px" }}
                      >
                        <CircularProgress />
                      </TableCell>
                    </TableRow>
                  ) : (
                    <>
                      {notFound ? (
                        <TableNoData notFound={notFound} />
                      ) : (
                        tableData.map((row) => (
                          <UserCartTableRow
                        key={row.clientDTO.uid}
                        row={row}
                        selected={table.selected.includes(row.clientDTO.uid.toString())}
                        onSelectRow={() => table.onSelectRow(row.clientDTO.uid.toString())}
                        onDeleteRow={() => handleDeleteRow(row.clientDTO.uid.toString())}
                        onViewRow={() => handleViewRow(row.clientDTO.uid.toString())}
                        revalidate={refresh}
                      />
                        ))
                      )}

                      <TableEmptyRows
                        height={denseHeight}
                        emptyRows={emptyRows(
                          table.page,
                          table.rowsPerPage,
                          totalCarts
                        )}
                      />
                    </>
                  )}
                </TableBody>
              </Table>
            </Scrollbar>
          </TableContainer>

            <TablePaginationCustom
            count={totalCarts}
            page={pageNumber}
            rowsPerPage={pageSize}
            onPageChange={table.onChangePage}
            onRowsPerPageChange={table.onChangeRowsPerPage}
            dense={table.dense}
            onChangeDense={table.onChangeDense}
          />
        </Card>
      </Container>

      
    </>
  );
}


