import React, { useEffect, useState } from "react";
import Box from "@mui/material/Box";
import Paper from "@mui/material/Paper";
import Stack from "@mui/material/Stack";
import Button from "@mui/material/Button";
import Collapse from "@mui/material/Collapse";
import TableRow from "@mui/material/TableRow";
import Checkbox from "@mui/material/Checkbox";
import TableCell from "@mui/material/TableCell";
import IconButton from "@mui/material/IconButton";
import ListItemText from "@mui/material/ListItemText";
import Chip from "@mui/material/Chip";
import { Avatar, Snackbar, Alert } from "@mui/material"; // Ajout de Snackbar et Alert
import { fCurrency } from "../../../../utils/format-number";
import Iconify from "../../../components/iconify";
import { ConfirmDialog } from "../../../components/custom-dialog";
import { usePopover } from "../../../components/custom-popover";
import { useBoolean } from "../../../../hooks/use-boolean";
import { useGetCart, useDeleteCartItem } from "@/shared/api/cart";
import {
  ICart,
  ICartDTO,
  IClientCartDTO,
  IDetailsProductCartDTO,
  IItemCart,
  IItemCartStatus,
} from "@/shared/types/cart";
import { IUserItem } from "@/shared/types/saleSession";
import Loading from "@/app/loading";
import LoadingButton from "@mui/lab/LoadingButton";
import { mutate } from "swr";

// ----------------------------------------------------------------------

type Props = {
  row: ICartDTO;
  selected: boolean;
  onViewRow: VoidFunction;
  onSelectRow: VoidFunction;
  onDeleteRow: VoidFunction;
  revalidate: () => void;
};

export default function UserCartTableRow({
  row: client,
  selected,
  onViewRow,
  onSelectRow,
  onDeleteRow,
  revalidate,
}: Props) {
  const [isDeleting, setIsDeleting] = useState(false);
  const confirm = useBoolean();
  const collapse = useBoolean();
  const popover = usePopover();
  const [itemIdToDelete, setItemIdToDelete] = useState<string | null>(null);

  const [openSnackbar, setOpenSnackbar] = useState(false);
  const [snackbarMessage, setSnackbarMessage] = useState("");

  const [productDetails, setProductDetails] = useState(
    client.productDetailsCartDTOS
  );

  const handleOpenConfirmDialog = (itemId: string) => {
    setItemIdToDelete(itemId);
    confirm.onTrue();
  };

  const handleCloseConfirmDialog = () => {
    setItemIdToDelete(null);
    confirm.onFalse();
  };

  const handleConfirmDelete = async () => {
    if (itemIdToDelete) {
      await handleDeleteItem(itemIdToDelete);
    }
    handleCloseConfirmDialog();
  };

  const handleDeleteIconClick = (itemId: string) => {
    handleOpenConfirmDialog(itemId);
  };
  const handleDeleteItem = async (itemId: string) => {
    try {
      setIsDeleting(true);
      const updatedProductDetails = client.productDetailsCartDTOS.map(
        (item) => {
          if (item.itemId.toString() === itemId) {
            return { ...item, status: "DELETED_BO" };
          }
          return item;
        }
      );
      await useDeleteCartItem(client.id.toString(), itemId);

      revalidate();
      setProductDetails(updatedProductDetails);
      setIsDeleting(false);
      setSnackbarMessage("L'élément a été supprimé avec succès !");
      setOpenSnackbar(true);
      confirm.onFalse();
    } catch (error) {
      console.error("Erreur lors de la suppression de l'élément:", error);
      setIsDeleting(false);
      setSnackbarMessage("Une erreur est survenue lors de la suppression.");
      setOpenSnackbar(true);
    }
  };

  const renderPrimary = (
    <TableRow hover selected={selected}>
      <TableCell sx={{ display: "flex", alignItems: "center" }}>
        <ListItemText
          primary={`${client.clientDTO.firstName} ${client.clientDTO.lastName}`}
          secondary={client.clientDTO.email}
          primaryTypographyProps={{ typography: "body2" }}
          secondaryTypographyProps={{
            component: "span",
            color: "text.disabled",
          }}
        />
      </TableCell>
      <TableCell align="center"> {client.clientDTO.phoneNumber} </TableCell>
      <TableCell align="center"> {client.itemsCount} </TableCell>
      <TableCell> {fCurrency(client.subTotalPrice)} </TableCell>

      <TableCell align="right" sx={{ px: 1, whiteSpace: "nowrap" }}>
        <IconButton
          color={collapse.value ? "inherit" : "default"}
          onClick={collapse.onToggle}
          sx={{
            ...(collapse.value && {
              bgcolor: "action.hover",
            }),
          }}
        >
          <Iconify icon="eva:arrow-ios-downward-fill" />
        </IconButton>
      </TableCell>
    </TableRow>
  );

  const renderSecondary = (
    <TableRow
      sx={{
        ...(client.productDetailsCartDTOS.some(
          (item: IDetailsProductCartDTO) =>
            item.status === "DELETED_BO" || item.status === "DELETED_SITE"
        ) && {
          pointerEvents: "none",
        }),
      }}
    >
      <TableCell sx={{ p: 1, border: "none" }} colSpan={9}>
        <Collapse
          in={collapse.value}
          timeout="auto"
          unmountOnExit
          sx={{ bgcolor: "background.neutral" }}
        >
          <Stack component={Paper} sx={{ m: 1.5 }}>
            {client.productDetailsCartDTOS.map(
              (item: IDetailsProductCartDTO) => {
                const isDeleted =
                  item.status === "DELETED_BO" ||
                  item.status === "DELETED_SITE";

                return (
                  <Stack
                    key={item.productCartDTOS.productId}
                    direction="row"
                    alignItems="center"
                    sx={{
                      p: (theme) => theme.spacing(1.5, 2, 1.5, 1.5),
                      "&:not(:last-of-type)": {
                        borderBottom: (theme) =>
                          `solid 2px ${theme.palette.background.neutral}`,
                      },
                      backgroundColor: isDeleted ? "#e4e1f6" : "transparent",
                      pointerEvents: isDeleted ? "none" : "auto",
                    }}
                  >
                    <Avatar
                      src={
                        item.productCartDTOS?.mainImagePath
                          ?.replace("Optional[", "")
                          .replace("]", "") || " "
                      }
                      variant="rounded"
                      sx={{ width: 48, height: 48, mr: 2 }}
                    />
                    <ListItemText
                      primary={`${item.productCartDTOS?.name}${
                        item.productCartDTOS?.ugs
                          ? " - " + item.productCartDTOS?.ugs
                          : ""
                      }`}
                      secondary={
                        <>
                          <Chip
                            size="small"
                            label={item.source === "LIVE" ? "Live" : "Site"}
                            sx={{
                              backgroundColor:
                                item.source === "WEB" ? "#00b8d9" : "#22c55e",
                              color: "white",
                            }}
                          />

                          <Box sx={{ marginTop: 1 }}>
                            {item.source === "LIVE"
                              ? `Session de Vente: #${item.sessionOrderId}`
                              : " "}
                          </Box>
                        </>
                      }
                      primaryTypographyProps={{ typography: "body2" }}
                      secondaryTypographyProps={{
                        component: "span",
                        color: "text.disabled",
                        mt: 0.5,
                      }}
                    />
                    <Stack direction="row" alignItems="center" spacing={6}>
                      {item.productCartDTOS.promoPrice !== 0 || null ? (
                        <Box sx={{ width: 110, textAlign: "right" }}>
                          {fCurrency(item.productCartDTOS.promoPrice)}
                        </Box>
                      ) : (
                        <Box sx={{ width: 110, textAlign: "right" }}>
                          {fCurrency(item.productCartDTOS.regularPrice)}
                        </Box>
                      )}
                      <Box>x{item.quantity}</Box>
                      <Box sx={{ width: 110, textAlign: "right" }}>
                        {fCurrency(item.totalPrice)}
                      </Box>

                      <IconButton
                        color="error"
                        onClick={() =>
                          handleDeleteIconClick(item.itemId.toString())
                        }
                        disabled={isDeleted || isDeleting} // Désactive le bouton si l'élément est supprimé ou en cours de suppression
                      >
                        <Iconify icon="solar:trash-bin-trash-bold" />
                      </IconButton>
                    </Stack>
                  </Stack>
                );
              }
            )}
          </Stack>
        </Collapse>
      </TableCell>
    </TableRow>
  );

  return (
    <>
      {renderPrimary}
      {renderSecondary}

      {/* ConfirmDialog pour confirmer la suppression */}
      <ConfirmDialog
        open={confirm.value}
        onClose={handleCloseConfirmDialog}
        title="Confirmer la suppression"
        content="Êtes-vous sûr de vouloir supprimer cet article de votre panier ?"
        action={
          <LoadingButton
            variant="contained"
            color="error"
            onClick={handleConfirmDelete}
            loading={isDeleting}
          >
            Supprimer
          </LoadingButton>
        }
      />

      {/* Snackbar pour afficher les messages */}
      <Snackbar
        open={openSnackbar}
        autoHideDuration={3000}
        onClose={() => setOpenSnackbar(false)}
        anchorOrigin={{
          vertical: "top",
          horizontal: "right",
        }}
      >
        <Alert onClose={() => setOpenSnackbar(false)} severity="success">
          {snackbarMessage}
        </Alert>
      </Snackbar>
    </>
  );
}
