import React, { useState, useCallback, useEffect } from "react";
import Dialog from "@mui/material/Dialog";
import TextField from "@mui/material/TextField";
import Stack from "@mui/material/Stack";
import Button from "@mui/material/Button";
import InputAdornment from "@mui/material/InputAdornment";
import List from "@mui/material/List";
import ListItem from "@mui/material/ListItem";
import ListItemAvatar from "@mui/material/ListItemAvatar";
import Avatar from "@mui/material/Avatar";
import Card from "@mui/material/Card";
import CircularProgress from "@mui/material/CircularProgress";
import { useAllGetProductsWithVariation } from "@/shared/api/product";
import { IProductSessionItem } from "@/shared/types/product";
import Scrollbar from "@/components/scrollbar";
import { Typography, Box, ListItemText } from "@mui/material"; 
import debounce from 'lodash/debounce';
import { ClearUrl } from "@/utils/cleanImageUrl";



type Props = {
  open: boolean;
  onClose: () => void;
  onSelect: (product: IProductSessionItem) => void;
};

export default function ProductListDialog({ open, onClose, onSelect }: Props) {
  const [searchQuery, setSearchQuery] = useState("");
  const [debouncedQuery, setDebouncedQuery] = useState("");

  // Handle search query change and debounce
  const handleSearchQueryChange = useCallback(
    (event: React.ChangeEvent<HTMLInputElement>) => {
      const value = event.target.value;
      setSearchQuery(value);
    },
    []
  );

  // Debounce the search input based on conditions
  useEffect(() => {
    const debounceHandler = debounce(() => {
      if (searchQuery.length >= 3) {
        setDebouncedQuery(searchQuery);
      } else {
        setDebouncedQuery(""); // Ne pas rechercher si moins de 3 caractères
      }
    }, 500); // Délais de 500ms pour le debounce

    debounceHandler();

    return () => {
      debounceHandler.cancel();
    };
  }, [searchQuery]);
  
   

  useEffect(() => {
    if (!open) {
      setSearchQuery(""); // Réinitialiser la recherche lorsque le dialogue est fermé
      setDebouncedQuery(""); // Réinitialiser le terme de recherche débouncé
    }
  }, [open]); 

 
  // Use the debounced search term to fetch products
  const { products, productsLoading, productsError } = useAllGetProductsWithVariation(
    debouncedQuery ? { searchTerm: debouncedQuery } : null
  );
  // Fonction pour sélectionner un produit
  const handleProductSelect = useCallback(
    (product: IProductSessionItem) => {
      onSelect(product);
      onClose(); // Réinitialiser la recherche après la sélection
    },
    [onClose, onSelect]
  );

  return (
    <Dialog
      fullWidth
      maxWidth="xs"
      sx={{ 
        width: "700px", 
        margin: "auto", 
      }} 
      open={open}
      onClose={onClose}
    >
      <Box
        sx={{
          height: "800px", 
          display: "flex",
          flexDirection: "column",
          p: "20px",
          overflow: "hidden",
        }}
      >
        <TextField
          value={searchQuery}
          onChange={handleSearchQueryChange}
          placeholder="Rechercher un nom de produit ou un UGS..."
          InputProps={{
            startAdornment: <InputAdornment position="start">🔍</InputAdornment>,
          }}
        />

        {/* Handle loading and error cases */}
        {productsLoading ? (
          <Stack
            sx={{
              alignItems: "center",
              justifyContent: "center",
              height: "550px",
            }}
          >
            <CircularProgress />
            <Typography variant="body2" sx={{ mt: 2 }}>
              Chargement des produits...
            </Typography>
          </Stack>
        ) : productsError ? (
          <Typography variant="body2" sx={{ mt: 2, color: 'red', textAlign: 'center' }}>
            Une erreur s'est produite lors de la récupération des produits. Veuillez réessayer plus tard.
          </Typography>
        ) : (
          <>
            {products.length === 0 ? (
              <Typography variant="body2" sx={{ mt: 2, textAlign: 'center' }}>
                Aucun produit trouvé.
              </Typography>
            ) : (
              <Scrollbar sx={{ height: "600px", bgcolor: "background.neutral", p: "5px" }}>
                <List>
                  {products.map((product) => (
                    <Card variant="outlined" key={product.id}>
                      <ListItem onClick={() => handleProductSelect(product)}>
                        <ListItemAvatar>
                          <Avatar
                            variant="rounded"
                            alt={product.name}
                            sx={{ height: 100, width: 100 }}
                            src={(product.mainImagePath)}
                          />
                        </ListItemAvatar>
                        <ListItemText
                          primary={product.name}
                          secondary={`ID: ${product.id} | Quantité: ${product.stockLiveConfig.quantity} | Créé le: ${new Date(
                            product.createdAt
                          ).toLocaleDateString()}`}
                        />
                      </ListItem>
                    </Card>
                  ))}
                </List>
              </Scrollbar>
            )}
          </>
        )}

        <Button fullWidth onClick={onClose} sx={{ height: "50px" }}>
          Annuler
        </Button>
      </Box>
    </Dialog>
  );
}
