"use client";

import isEqual from "lodash/isEqual";
import { useState, useCallback, useEffect } from "react";

import Card from "@mui/material/Card";
import Table from "@mui/material/Table";
import TableBody from "@mui/material/TableBody";
import TableContainer from "@mui/material/TableContainer";
import { _User } from "@/shared/_mock/_cartLive";
import Scrollbar from "@/components/scrollbar";

import {
  useTable,
  emptyRows,
  TableNoData,
  TableEmptyRows,
  TableHeadCustom,
  TableSelectedAction,
  TablePaginationCustom,
} from "@/shared/components/table";
import {
  IUserTableFilters,
  IUserTableFilterValue,
} from "@/shared/_mock/_cartLive";

import UserTableRow from "./NewUser-table-row";
import UserTableToolbar from "./NewUser-table-toolbar";
import UserTableFiltersResult from "./NewUser-table-filters-result";
import { Box, CircularProgress, Divider, Grid, Stack, TableCell, TableRow, Typography } from "@mui/material";
import { useSearchUnregistredClient } from '@/shared/api/saleSession';
import { IUnregistredClientItem } from '@/shared/types/saleSession';
// ----------------------------------------------------------------------

const TABLE_HEAD = [
  { id: "cart", label: "Panier" },
  { id: "pseudo", label: "Nom d'utilisateur" },
  { id: "creationDate", label: "Date de création", width: 180 },
  {label: "Action", width: 88 },
];

const defaultFilters: IUserTableFilters = {
  name: "",
  status: "all",
};
type AllUsersViewProps = {
  onAddToCart: (clientId: string,isRegistred:boolean,clientEmail:string) => void;

};

// ----------------------------------------------------------------------


export default function AllNewUsersView({ onAddToCart }: AllUsersViewProps) {
  const table = useTable();
  const [tableData, setTableData] = useState<IUnregistredClientItem[]>([]);
  const [filters, setFilters] = useState(defaultFilters);


  const {
    unregistredClients,
    totalUnregistredClients,
    totalPages,
    pageNumber,
    pageSize,
    loading,
    revalidate,
  } = useSearchUnregistredClient({
    pseudo: filters.name,
    page: table.page,
    size: table.rowsPerPage,
    sortBy: table.orderBy,
    sortDir: table.order,
  });

  useEffect(() => {
    if (unregistredClients && unregistredClients.length > 0) {
      setTableData(unregistredClients);
    }
  }, [unregistredClients, table.page, table.rowsPerPage]);


   useEffect(() => {
     if (
       filters.name
     ) {
       revalidate();
     }
   }, [filters, revalidate]);
 
   const dataInPage = tableData.slice(
     table.page * table.rowsPerPage,
     table.page * table.rowsPerPage + table.rowsPerPage
   );
 
   const denseHeight = table.dense ? 56 : 56 + 20;
 
   const canReset = !isEqual(defaultFilters, filters);
 
   const notFound = totalUnregistredClients === 0 && canReset;


   const handleFilters = useCallback(
     (name: string, value: IUserTableFilterValue) => {
       table.onResetPage();
       setFilters((prevState) => ({
         ...prevState,
         [name]: value,
       }));
     },
     [table]
   );
  const handleResetFilters = useCallback(() => {
    setFilters(defaultFilters);
  }, []);

  return (
    <>
      <Grid xs={12}>
        <Card>
          <Stack
            direction="column"
            divider={<Divider orientation="horizontal" />}
            spacing={2}
            sx={{ ml: 2 }}
          >
            <Box
              sx={{
                display: "flex",
                flexDirection: "row",
              }}
            >
              <Typography variant="subtitle1" gutterBottom mt={2}>
                Liste des clients non inscrits
              </Typography>
            </Box>
          </Stack>
          <UserTableToolbar
            filters={filters}
            onFilters={handleFilters}
            //
          />

          {canReset && (
            <UserTableFiltersResult
              filters={filters}
              onFilters={handleFilters}
              //
              onResetFilters={handleResetFilters}
              //
              results={totalUnregistredClients}
              sx={{ p: 2.5, pt: 0 }}
            />
          )}

          <TableContainer sx={{ position: "relative", overflow: "unset" }}>
            <TableSelectedAction
              dense={table.dense}
              numSelected={table.selected.length}
              rowCount={totalUnregistredClients}
              onSelectAllRows={(checked) =>
                table.onSelectAllRows(
                  checked,
                  tableData.map((row: IUnregistredClientItem) => row.id)
                )
              }
            />

            <Scrollbar>
              <Table
                size={table.dense ? "small" : "medium"}
                sx={{ minWidth: 960 }}
              >
                <TableHeadCustom
                  order={table.order}
                  orderBy={table.orderBy}
                  headLabel={TABLE_HEAD}
                  rowCount={totalUnregistredClients}
                  numSelected={table.selected.length}
                  onSort={table.onSort}
                />

<TableBody>
                  {loading ? (
                    <TableRow>
                      <TableCell
                        colSpan={TABLE_HEAD.length}
                        style={{ textAlign: "center", padding: "20px" }}
                      >
                        <CircularProgress />
                      </TableCell>
                    </TableRow>
                  ) : (
                    <>
                      {notFound ? (
                        <TableNoData notFound={notFound} />
                      ) : (
                        tableData.map((row) => (
                          <UserTableRow
                          key={row.id}
                          row={row}
                          onAddToCart={onAddToCart} 
                        />
                        ))
                      )}

                      <TableEmptyRows
                        height={denseHeight}
                        emptyRows={emptyRows(
                          table.page,
                          table.rowsPerPage,
                          totalUnregistredClients
                        )}
                      />
                    </>
                  )}
                </TableBody>
              </Table>
            </Scrollbar>
          </TableContainer>

          <TablePaginationCustom
           count={totalUnregistredClients}
           page={pageNumber}
           rowsPerPage={pageSize}
            onPageChange={table.onChangePage}
            onRowsPerPageChange={table.onChangeRowsPerPage}
            //
            dense={table.dense}
            onChangeDense={table.onChangeDense}
          />
        </Card>
      </Grid>
    </>
  );
}

// ----------------------------------------------------------------------

