import Tooltip from "@mui/material/Tooltip";
import TableRow from "@mui/material/TableRow";
import TableCell from "@mui/material/TableCell";
import IconButton from "@mui/material/IconButton";
import Iconify from "@/components/iconify";
import { _User } from "@/shared/_mock/_cartLive";
import DeleteIcon from "@mui/icons-material/Delete";
import { IUnregistredClientItem, IUserItem } from "@/shared/types/saleSession";
import {
  useGetClients,
  useAssociateAndDeleteClient,
  useAddProductToUnregisteredCartFromLive,
  deleteUnregisteredClientById,
  useGetUnregistredClients,
} from "@/shared/api/saleSession";
import {
  Avatar,
  Autocomplete,
  Box,
  Button,
  Dialog,
  DialogActions,
  DialogContent,
  DialogTitle,
  ListItemText,
  TextField,
} from "@mui/material";
import ShoppingCartIcon from "@mui/icons-material/ShoppingCart";
import { useState } from "react";
import { fDate, fTime } from "../../../../utils/format-time";
import { useSnackbar } from "@/components/snackbar";

// ----------------------------------------------------------------------

type Props = {
  row: any;
  onAddToCart: (
    clientId: string,
    isRegistred: boolean,
    clientEmail: string
  ) => void;
};

export default function UserTableRow({ row, onAddToCart }: Props) {
  const name = row.pseudo;
  const createdAt = row.creationDate;
  const { enqueueSnackbar } = useSnackbar();
  const { clients } = useGetClients(); 
  const { revalidate } = useGetUnregistredClients();
  const { associateAndDeleteClient } = useAssociateAndDeleteClient();
  const { addProductToUnregisteredCartFromLive } =
    useAddProductToUnregisteredCartFromLive();

  const [openDialog, setOpenDialog] = useState(false);
  const [selectedAccount, setSelectedAccount] = useState<IUserItem | null>(
    null
  );

  const handleOpenDialog = () => {
    setOpenDialog(true);
  };

  const handleCloseDialog = () => {
    setOpenDialog(false);
  };

  const handleAccountSelect = (account: any) => {
    setSelectedAccount(account);
  };

  const handleAssociateButtonClick = async () => {
    if (selectedAccount) {
      try {
        await associateAndDeleteClient(row.id, selectedAccount.uid);
        setOpenDialog(false);
      } catch (error) {
        console.error(
          "Error associating and deleting the unregistered client:",
          error
        );
      }
    }
  };

  const handleAddToCart = () => {
    onAddToCart(row.id, false, name);
  };

  const handleDelete = async () => {
    try {
      const response = await deleteUnregisteredClientById(row.id);
      if (response.status === 204) { 
        enqueueSnackbar("Client deleted successfully", { variant: "success" });
        revalidate();
      } else {
        enqueueSnackbar("Failed to delete the client", { variant: "error" });
      }
    } catch (error) {
      console.error("Failed to delete client:", error);
      enqueueSnackbar("An error occurred", { variant: "error" });
    }
  };
  return (
    <TableRow>
      <TableCell>
        <Tooltip title="Ajouter au panier">
          <IconButton sx={{ color: "#00e676" }} onClick={handleAddToCart}>
            <ShoppingCartIcon />
          </IconButton>
        </Tooltip>
      </TableCell>
      <TableCell sx={{ whiteSpace: "nowrap" }}>{name}</TableCell>
      <TableCell>
        <ListItemText
          primary={fDate(createdAt)}
          secondary={fTime(createdAt)}
          primaryTypographyProps={{ typography: "body2", noWrap: true }}
          secondaryTypographyProps={{
            mt: 0.5,
            component: "span",
            typography: "caption",
          }}
        />
      </TableCell>

      <TableCell align="right" sx={{ px: 1, whiteSpace: "nowrap" }}>
        <>
          <Tooltip title="Associer au Compte" placement="top" arrow>
            <IconButton color="default" onClick={handleOpenDialog}>
              <Iconify icon="solar:pen-bold" />
            </IconButton>
          </Tooltip>
          <Tooltip title="Supprimer le Client" placement="top" arrow>
            <IconButton color="error" onClick={() => handleDelete()}>
              <DeleteIcon />
            </IconButton>
          </Tooltip>
        </>
      </TableCell>

      <Dialog
        open={openDialog}
        onClose={handleCloseDialog}
        fullWidth
        maxWidth="sm"
      >
        <DialogTitle>Sélectionnez un Compte Utilisateur</DialogTitle>
        <DialogContent>
          <Autocomplete
            options={clients}
            getOptionLabel={(option) =>
              `${option.firstName} ${option.lastName} (${option.email})`
            }
            isOptionEqualToValue={(option, value) => option.id === value.id}
            renderOption={(props, option) => (
              <Box
                component="li"
                {...props}
                sx={{
                  display: "flex",
                  alignItems: "center",
                  "& > img": { mr: 2, flexShrink: 0 },
                }}
              >
                <Avatar
                  alt={option.firstName ?? ""}
                  src={option.imageUrl ?? ""}
                  sx={{ mr: 2 }}
                />
                <span>
                  {option.firstName} {option.lastName} [{option.email}]
                </span>
              </Box>
            )}
            onChange={(event, newValue) => setSelectedAccount(newValue)}
            renderInput={(params) => (
              <TextField
                {...params}
                label="Compte Utilisateur"
                variant="outlined"
                margin="normal"
              />
            )}
          />
        </DialogContent>
        <DialogActions>
          <Button onClick={handleCloseDialog} color="secondary">
            Annuler
          </Button>
          <Button
            onClick={handleAssociateButtonClick}
            color="primary"
            variant="contained"
            disabled={!selectedAccount}
          >
            Associer
          </Button> 
        </DialogActions>
      </Dialog>
    </TableRow>
  );
}
