"use client";

import isEqual from "lodash/isEqual";
import { useState, useCallback, useMemo,useEffect } from "react";
import Tab from "@mui/material/Tab";
import Tabs from "@mui/material/Tabs";
import Card from "@mui/material/Card";
import Table from "@mui/material/Table";
import { alpha } from "@mui/material/styles";
import TableBody from "@mui/material/TableBody";
import TableContainer from "@mui/material/TableContainer";
import {Order_STATUS_OPTIONS, IOrder } from "@/shared/types/saleSession";
import Label from "@/shared/components/label";
import Scrollbar from "@/components/scrollbar";

import {
  useTable,
  emptyRows,
  TableNoData,
  getComparator,
  TableEmptyRows,
  TableHeadCustom,
  TableSelectedAction,
  TablePaginationCustom,
} from "@/shared/components/table";
import {
 
  OrderTableFilterValue,
  OrderTableFilters,
} from "@/shared/_mock/_cartLive";

import { Box, Divider, Grid, Stack, Typography } from "@mui/material";
import { useGetOrdersBySessionId } from "@/shared/api/saleSession"; 
import OrderTableToolbar from "./Order-table-toolbar";
import OrderTableFiltersResult from "./Order-table-filters-result";
import OrderTableRow from "./Order-table-row";

// ----------------------------------------------------------------------

const STATUS_OPTIONS = [
  { value: "all", label: "Tous" },
  ...Order_STATUS_OPTIONS,
];

const TABLE_HEAD = [
  { id: "nameid", label: "Nom d'utilisateur" },
  { id: "name", label: "name", width: 180 },
  { id: "product", label: "Nom Produit", width: 180 },
  { id: "UGS", label: "UGS" },
  { id: "quantity", label: "Quantité" },
  { id: "CreatedAt", label: "Date de création", width: 180 },
  { id: "price", label: "Prix" },
  { id: "statusP", label: "Status Panier", width: 100 },
  { id: "statusC", label: "Status Commande", width: 100 },
  { id: "CreatedAt", label: "Date dernière Modification ", width: 180 },
  { id: "", width: 88 },
];

const defaultFilters: OrderTableFilters = {
  name: "",
  statutPanier: "all",
  email : "",
};
type AllUsersViewProps = {
  liveId: string;
};

// ----------------------------------------------------------------------

export default function AllOrderView({ liveId }: AllUsersViewProps) {

  const table = useTable({
    defaultOrder: 'desc',
    defaultOrderBy: 'CreatedAt',
  });

  const { orders, ordersLoading, ordersError, revalidate } = useGetOrdersBySessionId(liveId);

  const [tableData, setTableData] = useState<IOrder[]>([]);

  const [filters, setFilters] = useState(defaultFilters);
  const memoizedApplyFilter = useMemo(() => applyFilter, []);

  const dataFiltered = memoizedApplyFilter({
    inputData: tableData,
    comparator: getComparator(table.order, table.orderBy),
    filters,
  });

  const dataInPage = dataFiltered.slice(
    table.page * table.rowsPerPage,
    table.page * table.rowsPerPage + table.rowsPerPage
  );

  const denseHeight = table.dense ? 56 : 56 + 20;

  const canReset = !isEqual(defaultFilters, filters);

  
  const notFound = (!dataFiltered.length && canReset) || !dataFiltered.length;

  const handleFilters = useCallback(
    (name: string, value: OrderTableFilterValue) => {
      table.onResetPage();
      setTableData([]); 
      setFilters((prevState) => ({
        ...prevState,
        [name]: value,
      }));
    },
    [table]
  );
  const handleResetFilters = useCallback(() => {
    setTableData([]); 
    setFilters(defaultFilters);
  }, []);

  const handleFilterStatus = useCallback(
    (event: React.SyntheticEvent, newValue: string) => {
     
      handleFilters("statutPanier", newValue);
    },
    [handleFilters]
  );

  useEffect(() => {
    if (orders) {
    
      const transformedOrders = orders.map((order) => ({
        orderId:order.id.toString(),
        id:order.client?.uid?.toString() || order.client?.id?.toString() || "Unknown ID",
        name: order.client?.firstName && order.client?.lastName
          ? `${order.client.firstName} ${order.client.lastName}`
          : order.client?.pseudo || '',
        email: order.client?.email || order.client?.pseudo || '',
        title: order.itemCartDTO.product ? order.itemCartDTO.product.name : '',
        UGS: order.itemCartDTO.product ? order.itemCartDTO.product.ugs : '',
        quantity: order.itemCartDTO.quantity,
        pseudo: order.client ? order.client.pseudo : '', 
        CreatedAt: order.createdDate,
        EditDate: order.lastModification,
        price: order.price,
        statutPanier: order.itemCartDTO.status,
        statutOrder: order.status,
      }));
  
      setTableData(transformedOrders);
    }
  }, [orders,filters]);
  
  const getStatutPanierLabel = (value:string) => {
    const statusOption = Order_STATUS_OPTIONS.find(option => option.value === value);
    return statusOption ? statusOption.label : value;
  };
  
  return (
    <>
      <Grid xs={12}>
        <Card>
          <Stack
            direction="column"
            divider={<Divider orientation="horizontal" />}
            spacing={2}
            sx={{ ml: 2 }}
          >
            <Box
              sx={{
                display: "flex",
                flexDirection: "row",
              }}
            >
              <Typography variant="subtitle1" gutterBottom mt={2}>
                Commandes
              </Typography>
            </Box>
          </Stack>
          <Tabs
            value={filters.statutPanier}
            onChange={handleFilterStatus}
            sx={{
              px: 2.5,
              boxShadow: (theme) =>
                `inset 0 -2px 0 0 ${alpha(theme.palette.grey[500], 0.08)}`,
            }}
          >
            {STATUS_OPTIONS.map((tab) => (
              <Tab
                key={tab.value}
                iconPosition="end"
                value={tab.value}
                label={tab.label}
                icon={
                  <Label
                    variant={
                      ((tab.value === "all" ||
                        tab.value === filters.statutPanier) &&
                        "filled") ||
                      "soft"
                    }
                    color={
                      (tab.value === "CONSULTED" && "success") ||
                      (tab.value === "NOT_CONSULTED" && "warning") ||
                      (tab.value === "DELETED_BO" && "error") ||
                      (tab.value === "DELETED_SITE" && "error") ||
                      "default"
                    }
                  >
                    {tab.value === "all"
                     ? tableData.length
                       : tableData.filter(
                            (user) => user.statutPanier === tab.value
                             ).length}
                  </Label>
                }
              />
            ))}
          </Tabs>

          <OrderTableToolbar
            filters={filters}
            onFilters={handleFilters}
            //
          />

          {canReset && (
            <OrderTableFiltersResult
            filters={{
              ...filters,
              statutPanier: getStatutPanierLabel(filters.statutPanier),
            }}
              onFilters={handleFilters}
              //
              onResetFilters={handleResetFilters}
              //
              results={dataFiltered.length}
              sx={{ p: 2.5, pt: 0 }}
            />
          )}

          <TableContainer sx={{ position: "relative", overflow: "unset" }}>
            <TableSelectedAction
              dense={table.dense}
              numSelected={table.selected.length}
              rowCount={dataFiltered.length}
              onSelectAllRows={(checked) =>
                table.onSelectAllRows(
                  checked,
                  dataFiltered.map((row) => row.id)
                )
              }
            />

            <Scrollbar>
              <Table
                size={table.dense ? "small" : "medium"}
                sx={{ minWidth: 960 }}
              >
                <TableHeadCustom
                  order={table.order}
                  orderBy={table.orderBy}
                  headLabel={TABLE_HEAD}
                  rowCount={dataFiltered.length}
                  numSelected={table.selected.length}
                  onSort={table.onSort}
                  onSelectAllRows={(checked) =>
                    table.onSelectAllRows(
                      checked,
                      dataFiltered.map((row) => row.id)
                    )
                  }
                />

                <TableBody>
                  {dataFiltered
                    .slice(
                      table.page * table.rowsPerPage,
                      table.page * table.rowsPerPage + table.rowsPerPage
                    )
                    .map((row) => (
                      <OrderTableRow
                        key={row.id}
                        row={row}
                        selected={table.selected.includes(row.id)}
                      />
                    ))}

                  <TableEmptyRows
                    height={denseHeight}
                    emptyRows={emptyRows(
                      table.page,
                      table.rowsPerPage,
                      dataFiltered.length
                    )}
                  />

                  <TableNoData notFound={notFound} />
                </TableBody>
              </Table>
            </Scrollbar>
          </TableContainer>

          <TablePaginationCustom
            count={dataFiltered.length}
            page={table.page}
            rowsPerPage={table.rowsPerPage}
            onPageChange={table.onChangePage}
            onRowsPerPageChange={table.onChangeRowsPerPage}
            //
            dense={table.dense}
            onChangeDense={table.onChangeDense}
          />
        </Card>
      </Grid>
    </>
  );
}

// ----------------------------------------------------------------------

function applyFilter({
  inputData,
  comparator,
  filters,
}: {
  inputData: IOrder[];
  comparator: (a: any, b: any) => number;
  filters: OrderTableFilters;
}) {
  const { name,statutPanier ,email} = filters;
  const stabilizedThis = inputData.map((el, index) => [el, index] as const);

  stabilizedThis.sort((a, b) => {
    const order = comparator(a[0], b[0]);
    if (order !== 0) return order;
    return a[1] - b[1];
  });

  inputData = stabilizedThis.map((el) => el[0]);

  if (name) {
    inputData = inputData.filter(
      (order) =>
        order.name?.toLowerCase().includes(name.toLowerCase()) ||
        order.email?.toLowerCase().includes(name.toLowerCase()) ||
        order.pseudo?.toLowerCase().includes(name.toLowerCase()) ||
        (order.UGS as string)?.toLowerCase().includes(name.toLowerCase()) ||
        (order.title as string)?.toLowerCase().includes(name.toLowerCase())
    );
  }

  if (statutPanier !== "all") {
    inputData = inputData.filter((order) => order.statutPanier === statutPanier);
  }



  return inputData;
}
