import * as Yup from "yup";
import { useEffect } from "react";
import { useForm } from "react-hook-form";
import { yupResolver } from "@hookform/resolvers/yup";
import Dialog from "@mui/material/Dialog";
import DialogTitle from "@mui/material/DialogTitle";
import DialogContent from "@mui/material/DialogContent";
import DialogActions from "@mui/material/DialogActions";
import Button from "@mui/material/Button";
import LoadingButton from "@mui/lab/LoadingButton";
import { TextField } from "@mui/material";
import { useSnackbar } from "@/components/snackbar";
import { useUpdateOrderQuantity } from "@/shared/api/saleSession";

// ----------------------------------------------------------------------

type Props = {
  open: boolean;
  onClose: VoidFunction;
  orderId: string;
  initialQuantity: number;
};

export default function QuantityEditForm({ open, onClose, orderId, initialQuantity }: Props) {
  const { enqueueSnackbar } = useSnackbar();
  const { updateOrderQuantity } = useUpdateOrderQuantity();

  const validationSchema = Yup.object().shape({
    quantity: Yup.number().min(1, "La quantité doit être supérieure à 0").required("La quantité est requise"),
  });

  const defaultValues = {
    quantity: initialQuantity,
  };

  const methods = useForm({
    resolver: yupResolver(validationSchema),
    defaultValues,
  });

  const {
    reset,
    handleSubmit,
    formState: { isSubmitting },
  } = methods;

  const onSubmit = handleSubmit(async (data) => {
    try {
      await updateOrderQuantity(orderId, data.quantity);
      enqueueSnackbar("Quantité mise à jour avec succès", { variant: "success" });
      reset();
      onClose();
    } catch (error: any) {
      let errorMessage = "Échec de la mise à jour de la quantité"; 
      if (typeof error === 'string'  && error.includes("Le produit est en rupture de stock.")) {
        errorMessage = "Le produit est en rupture de stock";
      }else if (typeof error === 'string' && error.includes("Sale session has expired for session id")) {
        errorMessage = "La session de vente a expiré";
      } 
      enqueueSnackbar(errorMessage, { variant: "error" });
    }
  });

  useEffect(() => {

    reset({ quantity: initialQuantity });
  }, [initialQuantity, reset, open]);

  return (
    <Dialog fullWidth maxWidth="xs" open={open} onClose={onClose}>
      <DialogTitle>Modifier la quantité</DialogTitle>

      <DialogContent>
        <TextField
          fullWidth
          label="Quantité"
          type="number"
          {...methods.register("quantity")}
          defaultValue={initialQuantity}
          sx={{ marginTop: 2 }}
        />
      </DialogContent>

      <DialogActions>
        <Button onClick={onClose}>Annuler</Button>
        <LoadingButton
          variant="contained"
          onClick={onSubmit}
          loading={isSubmitting}
        >
          Mettre à jour
        </LoadingButton>
      </DialogActions>
    </Dialog>
  );
}
