"use client";
import { useEffect } from "react";
import isEqual from "lodash/isEqual";
import { useState, useCallback } from "react";

import Tab from "@mui/material/Tab";
import Tabs from "@mui/material/Tabs";
import Card from "@mui/material/Card";
import Table from "@mui/material/Table";

import { alpha } from "@mui/material/styles";

import TableBody from "@mui/material/TableBody";
import TableContainer from "@mui/material/TableContainer";

import { USER_STATUS_OPTIONS } from "@/shared/types/saleSession";

import Label from "@/shared/components/label";

import Scrollbar from "@/components/scrollbar";

import {
  useTable,
  emptyRows,
  TableNoData,
  TableEmptyRows,
  TableHeadCustom,
  TableSelectedAction,
  TablePaginationCustom,
} from "@/shared/components/table";
import {
  IClientTableFilters,
  IUserTableFilterValue,
} from "@/shared/_mock/_cartLive";
import { IUserItem } from "@/shared/types/saleSession";

import UserTableRow from "./user-table-row";
import UserTableToolbar from "./user-table-toolbar";
import UserTableFiltersResult from "./user-table-filters-result";
import { Box, CircularProgress, Divider, Grid, Stack, TableCell, TableRow, Typography } from "@mui/material";
import { useClientsCountsByStatus, useSearchClient } from "@/shared/api/saleSession";

// ----------------------------------------------------------------------

const STATUS_OPTIONS = [{ value: "", label: "Tous" }, ...USER_STATUS_OPTIONS];

const TABLE_HEAD = [
  { id: "cart", label: "Panier" },
  { id: "email", label: "Nom d'utilisateur" },
  { id: "pseudo", label: "Pseudo", width: 180 },
  { id: "firstName", label: "Prenom", width: 180 },
  { id: "lastName", label: "Nom", width: 180 },
  { id: "createdAt", label: "Date de création", width: 180 },
  { id: "email", label: "Email", width: 180 },
  { id: "status", label: "Status", width: 100 },
  { id: "", width: 88 },
];

const defaultFilters: IClientTableFilters = {
  keyword: "",
  status:""
};
type AllUsersViewProps = {
  onAddToCart: (clientId: string, isRegistred: boolean, clientEmail: string) => void;
};

// ----------------------------------------------------------------------

export default function AllUsersView({ onAddToCart }: AllUsersViewProps) {
  const table = useTable();

  const [tableData, setTableData] = useState<IUserItem[]>([]);

  const [filters, setFilters] = useState(defaultFilters);
  const {
    clients,
    totalClients,
    totalPages,
    pageNumber,
    pageSize,
    loading,
    revalidate,
  } = useSearchClient({
    keyword: filters.keyword,
    status:filters.status,
    page: table.page,
    size: table.rowsPerPage,
    sortBy: table.orderBy,
    sortDir: table.order,
  });

  const { clientCounts, clientCountsError } = useClientsCountsByStatus(filters);
  const [statusCounts, setStatusCounts] = useState<{ [key: string]: number }>({
    APPROVED: 0,
    PENDING: 0,
    BANNED: 0,
  });


  useEffect(() => {
    if (clients && clients.length > 0) {
      setTableData(clients);
    }
  }, [clients]);

  const handleFilters = useCallback(
    (name: string, value: IUserTableFilterValue) => {
      table.onResetPage();
      setFilters((prevState) => ({
        ...prevState,
        [name]: value,
      }));
    },
    [table]
  );

  useEffect(() => {
    if (
      filters.keyword ||
      filters.status
    ) {
      revalidate();
    }
  }, [filters, revalidate]);

  useEffect(() => {
    if (clientCountsError) {
      console.error("Error fetching status counts:", clientCountsError);
      return;
    }
    if (Array.isArray(clientCounts)) {
      const statusObj = clientCounts.reduce<{ [key: string]: number }>(
        (acc, [status, count]) => {
          acc[status] = count;
          return acc;
        },
        {}
      );

      setStatusCounts(statusObj);
    } else {
      console.error(
        "Expected counts to be an array, but received:",
        clientCounts
      );
    }
  }, [filters, clientCounts, clientCountsError]);


  const denseHeight = table.dense ? 56 : 56 + 20;

  const canReset = !isEqual(defaultFilters, filters);

  const notFound = totalClients === 0 && canReset;
  const handleResetFilters = useCallback(() => {
    setFilters(defaultFilters);
  }, []);


  return (
    <>
      <Grid xs={12}>
        <Card>
          <Stack
            direction="column"
            divider={<Divider orientation="horizontal" />}
            spacing={2}
            sx={{ ml: 2 }}
          >
            <Box
              sx={{
                display: "flex",
                flexDirection: "row",
              }}
            >
              <Typography variant="subtitle1" gutterBottom mt={2}>
                Liste des Clients
              </Typography>
            </Box>
          </Stack>
          {loading ? (
            <Grid
              style={{
                display: "flex",
                justifyContent: "center",
                alignItems: "center",
              }}
            >
              <CircularProgress />
            </Grid>
          ) : (
            <Tabs
              value={filters.status}
              onChange={(event, newValue) => handleFilters("status", newValue)}
              sx={{
                px: 2.5,
                boxShadow: (theme) =>
                  `inset 0 -2px 0 0 ${alpha(theme.palette.grey[500], 0.08)}`,
              }}
            >
              {STATUS_OPTIONS.map((tab) => {
                const count =
                  statusCounts[tab.value as keyof typeof statusCounts] || 0;
                return (
                  <Tab
                    key={tab.value}
                    iconPosition="end"
                    value={tab.value}
                    label={tab.label}
                    icon={
                      <Label
                        variant={
                          ((tab.value === "" ||
                            tab.value === filters.status) &&
                            "filled") ||
                          "soft"
                        }
                        color={
                          (tab.value === "APPROVED" && "success") ||
                          (tab.value === "PENDING" && "warning") ||
                          (tab.value === "BANNED" && "error") ||
                          "default"
                        }
                      >
                        {tab.value === ""
                          ? Object.values(statusCounts).reduce(
                            (acc, count) => acc + (Number(count) || 0)
                            , 0)
                          : count}
                      </Label>
                    }
                  />
                );
              })}
            </Tabs>
          )}

            <UserTableToolbar
                   filters={filters}
                   onFilters={handleFilters}
                 />

              {canReset ? (
                    <UserTableFiltersResult
                      filters={filters}
                      onFilters={handleFilters}
                      onResetFilters={handleResetFilters}
                      results={totalClients}
                      sx={{ p: 2.5, pt: 0 }}
                    />
                  ): null}

          <TableContainer sx={{ position: "relative", overflow: "unset" }}>
          <TableSelectedAction
              dense={table.dense}
              numSelected={table.selected.length}
              rowCount={totalClients}
              onSelectAllRows={(checked) =>
                table.onSelectAllRows(
                  checked,
                  tableData.map((row) => row.id)
                )
              }
            />

            <Scrollbar>
              <Table
                size={table.dense ? "small" : "medium"}
                sx={{ minWidth: 960 }}
              >
                <TableHeadCustom
                  order={table.order}
                  orderBy={table.orderBy}
                  headLabel={TABLE_HEAD}
                  rowCount={totalClients}
                  numSelected={table.selected.length}
                  onSort={table.onSort}
                />

<TableBody>
                  {loading ? (
                    <TableRow>
                      <TableCell
                        colSpan={TABLE_HEAD.length}
                        style={{ textAlign: "center", padding: "20px" }}
                      >
                        <CircularProgress />
                      </TableCell>
                    </TableRow>
                  ) : (
                    <>
                      {notFound ? (
                        <TableNoData notFound={notFound} />
                      ) : (
                        tableData.map((row) => (
                          <UserTableRow
                          key={row.id}
                          row={row}
                          onAddToCart={onAddToCart}
                        />
                        ))
                      )}

                      <TableEmptyRows
                        height={denseHeight}
                        emptyRows={emptyRows(
                          table.page,
                          table.rowsPerPage,
                          totalClients
                        )}
                      />
                    </>
                  )}
                </TableBody>
              </Table>
            </Scrollbar>
          </TableContainer>

    
          <TablePaginationCustom
            count={totalClients}
            page={pageNumber}
            rowsPerPage={pageSize}
            onPageChange={table.onChangePage}
            onRowsPerPageChange={table.onChangeRowsPerPage}
            dense={table.dense}
            onChangeDense={table.onChangeDense}
          />
        </Card>
      </Grid>
    </>
  );
}


