import { useCallback, useState } from "react";

import Box from "@mui/material/Box";
import Chip from "@mui/material/Chip";
import Paper from "@mui/material/Paper";
import Button from "@mui/material/Button";
import Stack, { StackProps } from "@mui/material/Stack";
import Iconify from "@/shared/components/iconify";
import Snackbar from "@mui/material/Snackbar";
import Alert from "@mui/material/Alert";

import {
  IClientTableFilters,
  IUserTableFilterValue,
} from "@/shared/_mock/_cartLive";
import {
  Dialog,
  DialogActions,
  DialogContent,
  DialogContentText,
  DialogTitle,
  TextField
} from "@mui/material";
import { useCreateUnregisteredClient } from "@/shared/api/saleSession";
import { translateClientStatus } from "@/utils/UserStatusTranslate";

// ----------------------------------------------------------------------

type Props = StackProps & {
  filters: IClientTableFilters;
  onFilters: (name: string, value: IUserTableFilterValue) => void;
  onResetFilters: VoidFunction;
  results: number;
};

export default function UserTableFiltersResult({
  filters,
  onFilters,
  onResetFilters,
  results,
  ...other
}: Props) {
  const { createClient } = useCreateUnregisteredClient();

  const [open, setOpen] = useState(false);
  const [username, setUsername] = useState("");
  const [snackbar, setSnackbar] = useState<{ open: boolean; message: string; severity: "success" | "error" }>({ open: false, message: "", severity: "success" });

  const handleCreateUserClick = useCallback(() => {
    setOpen(true);
  }, []);

  const handleCreateUser = useCallback(async () => {
    try {
      const newUser = await createClient(username);
      setOpen(false);
      setSnackbar({ open: true, message: "Utilisateur ajouté avec succès.", severity: "success" });
    } catch (error) {
      console.error(error);
        setSnackbar({ open: true, message: "Le nom d'utilisateur doit être unique.", severity: "error" });
    }
  }, [username, createClient]);

  const handleClose = useCallback(() => {
    setOpen(false);
  }, []);

  const handleRemoveKeyword = useCallback(() => {
    onFilters("keyword", "");
  }, [onFilters]);

  const handleRemoveStatus = useCallback(() => {
    onFilters("status", "");
  }, [onFilters]);

  const handleSnackbarClose = useCallback(() => {
    setSnackbar({ ...snackbar, open: false });
  }, [snackbar]);

  const isStatusFilter = filters.status !== "";

  return (
    <Stack spacing={1.5} {...other}>
      <Box sx={{ typography: "body2" }}>
        <strong>{results}</strong>
        <Box component="span" sx={{ color: "text.secondary", ml: 0.25 }}>
          résultats trouvés
        </Box>
      </Box>

      <Stack
        flexGrow={1}
        spacing={1}
        direction="row"
        flexWrap="wrap"
        alignItems="center"
      >
        {!!filters.keyword && (
          <Block label="Mot-clé :">
            <Chip
              label={filters.keyword}
              size="small"
              onDelete={handleRemoveKeyword}
            />
          </Block>
        )}

        {isStatusFilter && (
          <Block label="Statut :">
            <Chip
              label={
                filters.status
                  ? translateClientStatus(filters.status)
                  : "status indéfini"
              }
              size="small"
              onDelete={filters.status ? handleRemoveStatus : undefined}
            />
          </Block>
        )}

        <Button
          color="error"
          onClick={onResetFilters}
          startIcon={<Iconify icon="solar:trash-bin-trash-bold" />}
        >
          Vider
        </Button>

        <Button
          color="primary"
          onClick={handleCreateUserClick}
          startIcon={<Iconify icon="bi:person-plus" />}
        >
          Créer
        </Button>
      </Stack>
      <Dialog open={open} onClose={handleClose}>
        <DialogTitle>Création d'un utilisateur</DialogTitle>
        <DialogContent>
          <DialogContentText>
            Entrez un nom d'utilisateur pour le nouveau compte :
          </DialogContentText>
          <TextField
            autoFocus
            margin="dense"
            id="username"
            label="Nom d'utilisateur"
            type="text"
            fullWidth
            variant="outlined"
            value={username}
            onChange={(e) => setUsername(e.target.value)}
          />
        </DialogContent>
        <DialogActions>
          <Button onClick={handleClose} color="secondary">
            Annuler
          </Button>
          <Button onClick={handleCreateUser} color="primary">
            Confirmer
          </Button>
        </DialogActions>
      </Dialog>
      <Snackbar
        open={snackbar.open}
        autoHideDuration={6000}
        onClose={handleSnackbarClose}
      >
        <Alert onClose={handleSnackbarClose} severity={snackbar.severity} sx={{ width: '100%' }}>
          {snackbar.message}
        </Alert>
      </Snackbar>
    </Stack>
  );
}

// ----------------------------------------------------------------------

type BlockProps = StackProps & {
  label: string;
};

function Block({ label, children, sx, ...other }: BlockProps) {
  return (
    <Stack
      component={Paper}
      variant="outlined"
      spacing={1}
      direction="row"
      sx={{
        p: 1,
        borderRadius: 1,
        overflow: "hidden",
        borderStyle: "dashed",
        ...sx,
      }}
      {...other}
    >
      <Box component="span" sx={{ typography: "subtitle2" }}>
        {label}
      </Box>

      <Stack spacing={1} direction="row" flexWrap="wrap">
        {children}
      </Stack>
    </Stack>
  );
}
