import { ApexOptions } from "apexcharts";
import { useState, useCallback } from "react";

import Box from "@mui/material/Box";
import MenuItem from "@mui/material/MenuItem";
import ButtonBase from "@mui/material/ButtonBase";
import CardHeader from "@mui/material/CardHeader";
import Card, { CardProps } from "@mui/material/Card";

import Iconify from "@/components/iconify";
import Chart, { useChart } from "@/components/chart";
import CustomPopover, { usePopover } from "../../../components/custom-popover";

interface Props extends CardProps {
  title?: string;
  subheader?: string;
  chart: {
    categories?: string[];
    colors?: string[];
    series: {
      type: string;
      data: {
        name: string;
        data: number[];
      }[];
    }[];
    options?: ApexOptions;
  };
}

export default function BasketAmountChart({
  title,
  subheader,
  chart,
  ...other
}: Props) {
  const { categories, colors, series, options } = chart;

  const popover = usePopover();

  const [selectedComparison, setSelectedComparison] = useState("Mois en cours");

  const chartOptions = useChart({
    colors,
    stroke: {
      show: true,
      width: 2,
      curve: 'smooth',
    },
    xaxis: {
      categories,
    },
    tooltip: {
      y: {
        formatter: (value: number, { seriesIndex, w }: any) => {
          if (seriesIndex === 2) { 
            return `Écart : €${value}`;
          }
          return `€${value}`;
        },
      },
    },
    ...options,
  });

  const handleChangeComparison = useCallback(
    (newValue: string) => {
      popover.onClose();
      setSelectedComparison(newValue);
    },
    [popover]
  );

  const theoreticalData =
    series
      .find((item) => item.type === selectedComparison)
      ?.data.find((d) => d.name === 'Montant théorique')?.data || [];

  const realData =
    series
      .find((item) => item.type === selectedComparison)
      ?.data.find((d) => d.name === 'Montant réel')?.data || [];

  const ecartData = theoreticalData.map((value, index) => value - (realData[index] || 0));

  const filteredSeries = [
    {
      name: 'Montant théorique',
      data: theoreticalData,
    },
    {
      name: 'Montant réel',
      data: realData,
    },
    {
      name: 'Écart',
      data: ecartData,
    },
  ];

  return (
    <>
      <Card {...other}>
        <CardHeader
          title={title}
          subheader={subheader}
          action={
            <ButtonBase
              onClick={popover.onOpen}
              sx={{
                pl: 1,
                py: 0.5,
                pr: 0.5,
                borderRadius: 1,
                typography: "subtitle2",
                bgcolor: "background.neutral",
              }}
            >
              Comparer à
              <Iconify
                width={16}
                icon={
                  popover.open
                    ? "eva:arrow-ios-upward-fill"
                    : "eva:arrow-ios-downward-fill"
                }
                sx={{ ml: 0.5 }}
              />
            </ButtonBase>
          }
        />

        <Box sx={{ mt: 3, mx: 3 }}>
          <Chart
            dir="ltr"
            type="line"
            series={filteredSeries}
            options={chartOptions}
            width="100%"
            height={364}
          />
        </Box>
      </Card>

      <CustomPopover
        open={popover.open}
        onClose={popover.onClose}
        sx={{ width: 140 }}
      >
        {["Mois en cours", "Période précédente", "Année précédente"].map(
          (option) => (
            <MenuItem
              key={option}
              selected={option === selectedComparison}
              onClick={() => handleChangeComparison(option)}
            >
              {option}
            </MenuItem>
          )
        )}
      </CustomPopover>
    </>
  );
}
