import React, { useState, useEffect } from "react";
import {
  Card,
  SelectChangeEvent,
  Select,
  Checkbox,
  MenuItem,
  FormControl,
  InputLabel,
  OutlinedInput,
  ListItemText,
} from "@mui/material";
import CustomDateRangePicker from "@/shared/components/custom-date-range-picker/custom-date-range-picker";

import Iconify from "@/components/iconify";
import { fDate } from "@/utils/format-time";

type Props = {
  startDate: Date;
  endDate: Date;
  setSelectedOrders: (selectedOrders: string[]) => void;
  selectedOrders: string[];
  onChangeStartDate: (startDate: Date) => void;
  onChangeEndDate: (endDate: Date) => void;
};

export default function OrderFilters({
  startDate,
  endDate,
  onChangeEndDate,
  onChangeStartDate,
  setSelectedOrders,
  selectedOrders,
}: Props) {
  const [isDateRangeOpen, setIsDateRangeOpen] = useState(false);
  const [selectAll, setSelectAll] = useState(true);
  const OrderSource = ["LIVE", "WEB"];

  useEffect(() => {
    setSelectAll(selectedOrders.length === OrderSource.length);
  }, [selectedOrders]);

  useEffect(() => {
    setSelectedOrders([...OrderSource]);
  }, []);

  const closeDateRange = () => {
    setIsDateRangeOpen(false);
  };

  const openDateRange = () => {
    setIsDateRangeOpen(true);
  };

  const handleChangeOrders = (
    event: SelectChangeEvent<typeof selectedOrders>
  ) => {
    const {
      target: { value },
    } = event;
    if (value.includes("all")) {
      if (selectAll) {
        setSelectedOrders([]);
        setSelectAll(false);
      } else {
        setSelectedOrders([...OrderSource]);
        setSelectAll(true);
      }
    } else {
      setSelectedOrders(typeof value === "string" ? value.split(",") : value);
    }
  };

  return (
    <Card
      sx={{
        minHeight: "auto",
        display: "flex",
        alignItems: "center",
        p: 3,
        gap: 2,
      }}
    >
      <Card
        onClick={openDateRange}
        variant="outlined"
        sx={{
          height: 50,
          width: 500,
          cursor: "pointer",
          borderRadius: 1,
          px: 10,
          display: "flex",
          boxShadow: "none",
          justifyContent: "space-between",
          alignItems: "center",
          border: "1px solid #e0e0e0",
          "&:hover": {
            backgroundColor: "#f5f5f5",
          },
        }}
      >
        <Iconify sx={{ mr: 3 }} icon="solar:calendar-date-bold" />
        <ListItemText
          secondary={`${fDate(startDate)} - ${fDate(endDate)}`}
          primaryTypographyProps={{
            variant: "body2",
            color: "text.primary",
            mb: 0,
          }}
        />
      </Card>
      <CustomDateRangePicker
        open={isDateRangeOpen}
        onClose={closeDateRange}
        startDate={startDate}
        endDate={endDate}
        onChangeEndDate={(newValue: Date | null) => onChangeEndDate(newValue!)}
        onChangeStartDate={(newValue: Date | null) =>
          onChangeStartDate(newValue!)
        }
      />
      <FormControl sx={{ m: 1, width: 500 }}>
        <InputLabel>Source</InputLabel>
        <Select
          multiple
          value={selectedOrders}
          onChange={handleChangeOrders}
          input={<OutlinedInput sx={{ height: 50 }} label="Source" />}
          renderValue={(selected) => selected.join(", ")}
        >
          <MenuItem value="all">
            <Checkbox checked={selectAll} />
            Select All
          </MenuItem>
          {OrderSource.map((origine) => (
            <MenuItem key={origine} value={origine}>
              <Checkbox checked={selectedOrders.includes(origine)} />
              {origine}
            </MenuItem>
          ))}
        </Select>
      </FormControl>
    </Card>
  );
}
