import type { IFAQItem } from '@/shared/types/faq';

import { z as zod } from 'zod';
import { useForm } from 'react-hook-form';
import { zodResolver } from '@hookform/resolvers/zod';
import { useMemo, useEffect} from 'react';

import Card from '@mui/material/Card';
import Stack from '@mui/material/Stack';
import Divider from '@mui/material/Divider';
import CardHeader from '@mui/material/CardHeader';
import LoadingButton from '@mui/lab/LoadingButton';

import { paths } from '@/routes/paths';
import { useRouter } from 'next/navigation';

import { useAddFAQ , useUpdateFAQ } from '@/shared/api/faq';
import toast from 'react-hot-toast';
import { Field } from '@/shared/components/hook-form';
import FormProvider from '@/shared/components/hook-form';

// ----------------------------------------------------------------------

export const FAQ_CATEGORY_OPTIONS = [
  { value: 'BILLING', label: 'Facturation' },
  { value: 'TECHNICAL_SUPPORT', label: 'Assistance technique' },
  { value: 'GENERAL', label: 'Général' },
  { value: 'PRODUCT_INFO', label: 'Informations sur le produit' },
]

export type NewProductSchemaType = zod.infer<typeof NewProductSchema>;

export const NewProductSchema = zod.object({
  question: zod.string().min(1, { message: 'Question Est Requis!' }),
  answer: zod.string().min(1, { message: 'Réponse Est Requis!' }),
  category: zod.string().min(1, { message: 'Category Est Requis!' }),
});

// ----------------------------------------------------------------------

type Props = {
  currentFAQ?: IFAQItem | null;
};
 
export function FAQNewEditForm({ currentFAQ }: Props) {

  const { addFAQ } = useAddFAQ();
  const { updateFAQ } = useUpdateFAQ(currentFAQ?.id || '');

  const router = useRouter();

  const defaultValues = useMemo(
    () => ({
      question: currentFAQ?.question || '',
      answer: currentFAQ?.answer || '',
      category: currentFAQ?.category || '',
    }),
    [currentFAQ]
  );

  const methods = useForm<NewProductSchemaType>({
    resolver: zodResolver(NewProductSchema),
    defaultValues,
  });

  const {
    reset,
    handleSubmit,
    formState: { isSubmitting },
  } = methods;


  useEffect(() => {
    if (currentFAQ) {
      reset(defaultValues);
    }
  }, [currentFAQ, defaultValues, reset]);

  const onSubmit = handleSubmit(async (data) => {
    try {
      await new Promise((resolve) => setTimeout(resolve, 500));
      currentFAQ ? await updateFAQ(data) : await addFAQ(data);
      reset();
      // Await the toast display before navigating
    await toast.promise(
      Promise.resolve(),
      {
        loading: currentFAQ ? 'Enregistrement...' : 'Création...',
        success: currentFAQ ? 'Modifications enregistrées' : 'FAQ créée',
        error: 'Erreur lors de la mise à jour de la FAQ',
      }
    );
      router.push(paths.dashboard.chatbot.faq.root);
    } catch (error) {
      console.error(error);
    }
  });

  const renderDetails = (
    <Card>
      <CardHeader title="Détails" subheader="Ajouter une nouvelle FAQ , remplissez les champs ci-dessous"sx={{ p: 3 }} />
      <Divider />
      <Stack spacing={3} sx={{ p: 3 }}>
        <Field.Text name="question" label="Question de la FAQ" />
        <Field.Editor name="answer" placeholder="Entrer la réponse..." />
        <Field.Select native name="category" label="Catégorie" InputLabelProps={{ shrink: true }}>
            {FAQ_CATEGORY_OPTIONS.map((category) => 
              <option key={category.value} value={category.value}>{category.label}</option>
            )}

          </Field.Select>
      </Stack>
    </Card>
  );

  const renderActions = (
    <Stack spacing={3} direction="row" alignItems="center" flexWrap="wrap">
      <LoadingButton type="submit" variant="contained" size="large" loading={isSubmitting}>
        {!currentFAQ ? 'Créer FAQ' : 'Enregistrer'}
      </LoadingButton>
    </Stack>
  );

  return (
    <FormProvider methods={methods} onSubmit={onSubmit}>
      <Stack spacing={{ xs: 3, md: 5 }} sx={{ mx: 'auto', maxWidth: { xs: 720, xl: 880 } }}>
        {renderDetails}
        {renderActions}
      </Stack>
    </FormProvider>
  );
}
