'use client';

import type { IFAQItem } from '@/shared/types/faq';
import type {
  GridColDef,
  GridRowSelectionModel,
  GridColumnVisibilityModel,
} from '@mui/x-data-grid'; 

import { useState, useEffect, useCallback } from 'react';

import Card from '@mui/material/Card';
import Stack from '@mui/material/Stack';
import Button from '@mui/material/Button';
import {
  DataGrid,
  gridClasses,
  GridActionsCellItem,
  GridToolbarContainer,
  GridToolbarQuickFilter,
} from '@mui/x-data-grid';

import { paths } from '@/routes/paths';
import { useRouter } from 'next/navigation';
import RouterLink from '@/routes/router-link';

import { useBoolean } from '@/hooks';

import { useGetFaqs , useDeleteFAQ } from '@/shared/api/faq';
import Container from "@mui/material/Container";

import toast from 'react-hot-toast';
import Iconify from '@/shared/components/iconify';
import EmptyContent from '@/shared/components/empty-content';
import ConfirmDialog from '@/shared/components/confirm-dialog';
import CustomBreadcrumbs from '@/shared/components/custom-breadcrumbs';

import {
  RenderCellFAQQuestion,
  RenderCellFAQAnswer,
  RenderCellCreatedAt,
  RenderCellUpdatedAt,
  RenderCellCategory,
} from '../faq-table-row';

// ----------------------------------------------------------------------

const HIDE_COLUMNS = { category: false };
const HIDE_COLUMNS_TOGGLABLE = ['category', 'actions'];

// ----------------------------------------------------------------------

export function FAQListView() {
  const confirmRows = useBoolean();

  const router = useRouter();
  const { faqs, faqsLoading} = useGetFaqs();

  const { deleteFAQ } = useDeleteFAQ();

  const [tableData, setTableData] = useState<IFAQItem[]>([]);
  const [selectedRowIds, setSelectedRowIds] = useState<GridRowSelectionModel>([]);
  const [filterButtonEl, setFilterButtonEl] = useState<HTMLButtonElement | null>(null);
  const [columnVisibilityModel, setColumnVisibilityModel] = useState<GridColumnVisibilityModel>(HIDE_COLUMNS);
  const [confirmDeleteRowId, setConfirmDeleteRowId] = useState<string | null>(null); // State to manage row ID for individual deletion

  useEffect(() => {
    if (faqs.length) {
      setTableData(faqs);
    }
  }, [faqs]);

  const handleDeleteRow = useCallback(
    (id: string) => {
      setConfirmDeleteRowId(id); // Set row ID for confirmation
    },
    []
  );

  const confirmDeleteRow = async () => {
    if (confirmDeleteRowId) {
      await deleteFAQ(confirmDeleteRowId);
      setConfirmDeleteRowId(null); 
      toast.success('Suppression réussie!');
    }
  };
  
  const handleDeleteRows = useCallback(() => {
    tableData.filter((row) => {
      if (selectedRowIds.includes(row.id)) {
        deleteFAQ(row.id);
      }
    });
    toast.success('Suppression réussie!');
  }, [selectedRowIds, tableData]);
  

  const cancelDeleteRow = () => {
    setConfirmDeleteRowId(null);
    toast('Suppression annulée');
  };

  const handleEditRow = useCallback(
    (id: string) => {
      router.push(paths.dashboard.chatbot.faq.edit(id));
    },
    [router]
  );

  const CustomToolbarCallback = useCallback(
    () => (
      <CustomToolbar
        selectedRowIds={selectedRowIds}
        setFilterButtonEl={setFilterButtonEl}
        filteredResults={tableData.length}
        onOpenConfirmDeleteRows={confirmRows.onTrue}
      />
    ),
    [selectedRowIds]
  );

  const columns: GridColDef[] = [
    {
      field: 'question',
      headerName: 'Question',
      flex: 1,
      minWidth: 150,
      hideable: false,
      renderCell: (params) => <RenderCellFAQQuestion params={params} />,
    },
    {
      field: 'answer',
      headerName: 'Réponse',
      flex: 1,
      minWidth: 150,
      hideable: false,
      renderCell: (params) => <RenderCellFAQAnswer params={params} />,
    },
    {
      field: 'createdAt',
      headerName: 'Créé le',
      width: 160,
      renderCell: (params) => <RenderCellCreatedAt params={params} />,
    },
    {
      field: 'updatedAt',
      headerName: 'Mis à jour le',
      width: 160,
      renderCell: (params) => <RenderCellUpdatedAt params={params} />,
    },
    { 
      field: 'category_name', 
      headerName: 'Catégorie', 
      width: 220, 
      renderCell: (params) => <RenderCellCategory params={params} />,
    },
    {
      type: 'actions',
      field: 'actions',
      headerName: ' ',
      align: 'right',
      headerAlign: 'right',
      width: 80,
      sortable: false,
      filterable: false,
      disableColumnMenu: true,
      getActions: (params) => {
        if (params.row.category === "DELETED") {
          return [];
        }
        return [
          <GridActionsCellItem
            showInMenu
            icon={<Iconify icon="solar:pen-bold" />}
            label="Modifier"
            onClick={() => handleEditRow(params.row.id)}
          />,
          <GridActionsCellItem
            showInMenu
            icon={<Iconify icon="solar:trash-bin-trash-bold" />}
            label="Supprimer"
            onClick={() => handleDeleteRow(params.row.id)}
            sx={{ color: 'error.main' }}
          />,
        ];
      },
    }
    
  ];

  const getTogglableColumns = () =>
    columns.filter((column) => !HIDE_COLUMNS_TOGGLABLE.includes(column.field)).map((column) => column.field);

  return (


    <>
      <Container 
        maxWidth="xl"
        sx={{ flexGrow: 1, display: 'flex', flexDirection: 'column' }}>
        <CustomBreadcrumbs
          heading="Liste FAQ"
          links={[{ name: 'Tableau de bord' }, { name: 'Liste FAQ' }]}
          action={
            <Button
              component={RouterLink}
              href={paths.dashboard.chatbot.faq.new}
              variant="contained"
              startIcon={<Iconify icon="mingcute:add-line" />}
            >
              Nouvelle FAQ
            </Button>
          }
          sx={{ mb: { xs: 5, md: 5 } }}
        />

        <Card
          sx={{
            flexGrow: { md: 2 },
            display: { md: 'flex' },
            height: { xs: 800, md: 2 },
            width: { md: 1 },
            flexDirection: { md: 'column' },
          }}
        >
          <DataGrid
            checkboxSelection
            disableRowSelectionOnClick
            rows={tableData}
            columns={columns}
            loading={faqsLoading}
            getRowHeight={() => 'auto'}
            pageSizeOptions={[5, 10, 25]}
            localeText={{
              toolbarQuickFilterPlaceholder: 'Rechercher...',
              toolbarColumns: "Colonnes",
              toolbarColumnsLabel: "Chercher",
              toolbarFilters: "Filtrer",
              toolbarExport: "Exporter",
              filterPanelOperator: 'Opérateurs',
              filterPanelColumns: 'Colonnes',
              filterPanelInputLabel: 'Valeur',
              filterPanelInputPlaceholder: 'Valeur du filtre',
              filterOperatorContains: 'Contient',
              filterOperatorEquals: 'Égal à',
              filterOperatorStartsWith: 'Commence par',
              filterOperatorEndsWith: 'Finit par',
              filterOperatorIsEmpty: 'Est vide',
              filterOperatorIsNotEmpty: 'N\'est pas vide',
              filterOperatorIsAnyOf: 'Est l\'un de',
              columnMenuLabel: 'Menu',
              columnMenuShowColumns: 'Afficher les colonnes',
              columnMenuManageColumns: 'Gérer les colonnes',
              columnMenuFilter: 'Filtrer',
              columnMenuHideColumn: 'Masquer la colonne',
              columnMenuUnsort: 'Annuler le tri',
              columnMenuSortAsc: 'Tri croissant',
              columnMenuSortDesc: 'Tri décroissant'
            }}
            initialState={{
              pagination: { paginationModel: { pageSize: 10 } },
              sorting: { sortModel: [{ field: 'updatedAt', sort: 'desc' }] },
            }}
            onRowSelectionModelChange={(newSelectionModel) => setSelectedRowIds(newSelectionModel)}
            columnVisibilityModel={columnVisibilityModel}
            onColumnVisibilityModelChange={(newModel) => setColumnVisibilityModel(newModel)}
            slots={{
              toolbar: CustomToolbarCallback,
              noRowsOverlay: () => <EmptyContent />,
              noResultsOverlay: () => <EmptyContent title="Non Trouvé" />,
            }}
            slotProps={{
              panel: { anchorEl: filterButtonEl },
              toolbar: { setFilterButtonEl },
            }}
            sx={{ [`& .${gridClasses.cell}`]: { alignItems: 'center', display: 'inline-flex' } }}
          />
        </Card>
      </Container>

      {/* Confirm dialog for row deletion */}
      <ConfirmDialog
        open={Boolean(confirmDeleteRowId)}
        onClose={cancelDeleteRow}
        title="Supprimer"
        content="Êtes-vous sûr de vouloir supprimer cet élément?"
        action={
          <Button variant="contained" color="error" onClick={confirmDeleteRow}>
            Oui, Supprimer
          </Button>
        }
      />

      {/* Confirm dialog for bulk deletion */}
      <ConfirmDialog
        open={confirmRows.value}
        onClose={confirmRows.onFalse}
        title="Supprimer"
        content={
          <>
            Êtes-vous sûr de vouloir supprimer les
            <strong> {selectedRowIds.length} </strong> éléments sélectionnés?
          </>
        }
        action={
          <Button
            variant="contained"
            color="error"
            onClick={() => {
              handleDeleteRows();
              confirmRows.onFalse();
            }}
          >
            Oui, Supprimer
          </Button>
        }
      />
    </>
  );
}

// ----------------------------------------------------------------------

interface CustomToolbarProps {
  filteredResults: number;
  selectedRowIds: GridRowSelectionModel;
  onOpenConfirmDeleteRows: () => void;
  setFilterButtonEl: React.Dispatch<React.SetStateAction<HTMLButtonElement | null>>;
}

function CustomToolbar({
  selectedRowIds,
  onOpenConfirmDeleteRows,
}: CustomToolbarProps) {
  return (
    <>
      <GridToolbarContainer>
        <GridToolbarQuickFilter />
        <Stack spacing={1} flexGrow={1} direction="row" alignItems="center" justifyContent="flex-end">
          {!!selectedRowIds.length && (
            <Button
              size="small"
              color="error"
              startIcon={<Iconify icon="solar:trash-bin-trash-bold" />}
              onClick={onOpenConfirmDeleteRows}
            >
              Supprimer ({selectedRowIds.length})
            </Button>
          )}
        </Stack>
      </GridToolbarContainer>
    </>
  );
}
