'use client';

import type { IFile } from '@/shared/types/file';
import { useState, useCallback, useEffect, useMemo } from 'react';
import Button from '@mui/material/Button';
import Dialog from '@mui/material/Dialog';
import DialogContent from '@mui/material/DialogContent';
import Card from '@mui/material/Card';
import CardHeader from '@mui/material/CardHeader';
import Divider from '@mui/material/Divider';
import Stack from '@mui/material/Stack';
import Box from '@mui/material/Box';
import Typography from '@mui/material/Typography';
import List from '@mui/material/List';
import ListItem from '@mui/material/ListItem';
import ListItemText from '@mui/material/ListItemText';
import ListItemSecondaryAction from '@mui/material/ListItemSecondaryAction';
import IconButton from '@mui/material/IconButton';
import TextField from '@mui/material/TextField';
import { CircularProgress, DialogTitle } from '@mui/material';

import { useBoolean } from '@/hooks';
import Container from "@mui/material/Container";

import toast from 'react-hot-toast';
import { useTable } from '@/shared/components/table';
import ConfirmDialog from '@/shared/components/confirm-dialog';
import CustomBreadcrumbs from '@/shared/components/custom-breadcrumbs';
import { FileManagerTable } from '../file-manager-table';
import Iconify from '@/shared/components/iconify';
import { useDeleteDocuments , useGetFiles } from '@/shared/api/file';
import { z as zod } from 'zod';
import { zodResolver } from '@hookform/resolvers/zod';
import { addInsightDocument } from '@/shared/api/openai';

// Dummy imports for Form components (adjust to your implementation)
import LoadingButton from '@mui/lab/LoadingButton';
import FormProvider from '@/shared/components/hook-form';
import { useForm } from 'react-hook-form';
import UploadBox from '@/shared/components/upload/upload-box';

export const FileUploadSchema = zod.object({
  files: zod.array(zod.any()).min(1, 'Au moins un fichier est requis'),
});

// ----------------------------------------------------------------------

export function FileManagerView() {
  const table = useTable({ defaultRowsPerPage: 10 });
  const confirm = useBoolean();
  const { documents, documentsLoading } = useGetFiles();
  const { deleteDocuments } = useDeleteDocuments();
  const [tableData, setTableData] = useState<IFile[]>([]);
  const [openModal, setOpenModal] = useState(false);
  const [files, setFiles] = useState<(File | string)[]>([]);
  const [isFileSubmit, setIsFileSubmit] = useState(false);
  const [searchQuery, setSearchQuery] = useState('');

  // Second form (file upload)
  const fileMethods = useForm<{ files: (File | string)[] }>({
    resolver: zodResolver(FileUploadSchema),
    defaultValues: { files: [] },
  });

  const { handleSubmit: handleFileSubmit, setValue } = fileMethods;

  const handleDrop = useCallback(
    (acceptedFiles: File[]) => {
      setFiles((prevFiles) => [...prevFiles, ...acceptedFiles]);
      setValue('files', [...files, ...acceptedFiles]); 
    },
    [files, setValue]
  );

  const handleRemoveFile = (file: File | string) => {
    setFiles((prevFiles) => prevFiles.filter((f) => f !== file));
    setValue('files', files.filter((f) => f !== file)); 
  };

  const onFileSubmit = handleFileSubmit(async (data) => {
    try {
      setIsFileSubmit(true);
      await new Promise((resolve) => setTimeout(resolve, 500));
      console.log(data);
      await Promise.all(data.files.map((file) => addInsightDocument(file as File)));
      setIsFileSubmit(false);
      setFiles([]);
      toast.success('Base de Donnée Vector a été mis a jour avec succès');
    } catch (error) {
      setIsFileSubmit(false);
      console.error(error);
      toast.error('Failed to upload files');
    }
  });

  const handleDeleteItem = useCallback(
    async (id: string) => {
      await deleteDocuments(id);
      const deleteRow = tableData.filter((row) => row.id !== id);
      toast.success('Base de Données supprimée avec succès!');
      setTableData(deleteRow);
    },
    [tableData, deleteDocuments]
  );

  const handleDeleteItems = useCallback(() => {
    const deleteIds = table.selected;
    deleteIds.forEach(async (id) => {
      await deleteDocuments(id);
    });
    const deleteRows = tableData.filter((row) => !table.selected.includes(row.id));
    toast.success('Base de Données supprimée avec succès!');
    setTableData(deleteRows);
  }, [table, tableData, deleteDocuments]);

  useEffect(() => {
    if (documents) setTableData(documents);
  }, [documents]);

  // Compute filtered data based on search query
  const filteredData = useMemo(() => {
    if (!searchQuery) return tableData;
    return tableData.filter((file) =>
      file.source.toLowerCase().includes(searchQuery.toLowerCase())
    );
  }, [searchQuery, tableData]);

  const renderFile = (
    <Card>
      <CardHeader
        title="Configuration De Model"
        subheader="Ajouter les fichier .txt pour les fichier RAG"
        sx={{ mb: 3 }}
      />
      <Divider />
      <Stack spacing={3} sx={{ p: 3 }}>
        <Stack spacing={1.5}>
          <UploadBox
            onDrop={handleDrop}
            placeholder={
              <Box
                sx={{
                  gap: 0.5,
                  display: 'flex',
                  alignItems: 'center',
                  color: 'text.disabled',
                  flexDirection: 'column',
                }}
              >
                <Iconify icon="eva:cloud-upload-fill" width={40} />
                <Typography variant="body2">Importer Un Fichier</Typography>
              </Box>
            }
            sx={{
              py: 2.5,
              width: 'auto',
              height: 'auto',
              borderRadius: 1.5,
            }}
          />
          <List>
            {files.map((file, index) => (
              <ListItem key={index}>
                <ListItemText primary={typeof file === 'string' ? file : file.name} />
                <ListItemSecondaryAction>
                  <IconButton edge="end" aria-label="delete" onClick={() => handleRemoveFile(file)}>
                    <Iconify icon="eva:close-fill" width={20} height={20} />
                  </IconButton>
                </ListItemSecondaryAction>
              </ListItem>
            ))}
          </List>
        </Stack>
      </Stack>
    </Card>
  );

  return (
    <>
      <Container>
        <CustomBreadcrumbs
          heading="Base de Données Vecteur"
          links={[
            { name: 'OpenAI API' },
            { name: 'Base de Données Vecteur' }
          ]}
          action={
            <Button
              variant="contained"
              startIcon={<Iconify icon="mingcute:add-line" />}
              onClick={() => setOpenModal(true)}
            >
              Nouvelle Base de Données
            </Button>
          }
          sx={{ mb: { xs: 3, md: 5 } }}
        />
        <Box sx={{ mb: 2 }}>
          <TextField
            value={searchQuery}
            onChange={(e) => setSearchQuery(e.target.value)}
            placeholder="Rechercher..."
            variant="outlined"
            fullWidth
          />
        </Box>
        {documentsLoading ? (
          <div
            style={{
              display: 'flex',
              flexDirection: 'column',
              alignItems: 'center',
              justifyContent: 'center',
              minHeight: '200px'
            }}
          >
            <CircularProgress size={40} />
            <div style={{ marginTop: '16px', fontSize: '16px', color: '#555' }}>
              Chargement...
            </div>
          </div>
        ) : (
          <FileManagerTable
            table={table}
            dataFiltered={filteredData}
            onDeleteRow={handleDeleteItem}
            onOpenConfirm={confirm.onTrue}
          />
        )}
      </Container>

      <ConfirmDialog
        open={confirm.value}
        onClose={confirm.onFalse}
        title="Supprimer"
        content={
          <>
            Êtes-vous sûr de vouloir supprimer <strong>{table.selected.length}</strong> éléments ?
          </>
        }
        action={
          <Button
            variant="contained"
            color="error"
            onClick={() => {
              handleDeleteItems();
              confirm.onFalse();
            }}
          >
            Supprimer
          </Button>
        }
      />
      <Dialog open={openModal} onClose={() => setOpenModal(false)} fullWidth maxWidth="md">
        <DialogTitle sx={{ p: 3 }}>Ajouter une nouvelle base de données</DialogTitle>
        <DialogContent sx={{ p: 3 }}>
          <FormProvider methods={fileMethods} onSubmit={onFileSubmit}>
            <Stack spacing={{ xs: 3, md: 5 }} sx={{ mx: 'auto', maxWidth: { xs: 720, xl: 1200 } }}>
              {renderFile}
              <Stack spacing={3} direction="row" alignItems="center" flexWrap="wrap">
                <LoadingButton type="submit" variant="contained" size="large" loading={isFileSubmit}>
                  Début de l'entraînement
                </LoadingButton>
              </Stack>
            </Stack>
          </FormProvider>
        </DialogContent>
      </Dialog>
    </>
  );
}