import React, { useState } from 'react';
import type { IHelpItem } from '@/shared/types/help';

import Box from '@mui/material/Box';
import Link from '@mui/material/Link';
import Card from '@mui/material/Card';
import Stack from '@mui/material/Stack'; 
import MenuList from '@mui/material/MenuList';
import MenuItem from '@mui/material/MenuItem';
import { useTheme } from '@mui/material/styles';
import IconButton from '@mui/material/IconButton';
import Typography from '@mui/material/Typography';
import Dialog from '@mui/material/Dialog';
import DialogActions from '@mui/material/DialogActions';
import DialogContent from '@mui/material/DialogContent';
import DialogContentText from '@mui/material/DialogContentText';
import DialogTitle from '@mui/material/DialogTitle';
import Button from '@mui/material/Button';

import { paths } from '@/routes/paths';
import { useRouter } from 'next/navigation';
import RouterLink from '@/routes/router-link';

import toast from 'react-hot-toast';
import { fDate } from '@/utils/format-time';

import Label from '@/shared/components/label';
import Iconify from '@/components/iconify';
import { usePopover  } from '@/shared/components/custom-popover';
import CustomPopover from '@/shared/components/custom-popover';
import { deleteCollection } from '@/shared/api/help';

// ----------------------------------------------------------------------

// Utility function to limit text to a maximum number of lines with ellipsis
function maxLine({ line }: { line: number }) {
  return {
    overflow: 'hidden',
    display: '-webkit-box',
    WebkitLineClamp: line,
    WebkitBoxOrient: 'vertical',
  };
}

type Props = {
  collection: IHelpItem;
};

export function CollectionItemHorizontal({ collection }: Props) {
  const theme = useTheme();

  const popover = usePopover();

  const router = useRouter();

  const [openDialog, setOpenDialog] = useState(false);

  const {
    id,
    name,
    createdAt,
    description,
    articleCount
  } = collection;

  const hundleCancel = () => {
    setOpenDialog(false);
    toast.error('Suppression annulée');
  }

  const handleDelete = (id: string) => {
    deleteCollection(id);
    setOpenDialog(false);
    toast.success('Collection supprimée avec succès');
    router.push(paths.dashboard.chatbot.help.root);
  };

  return (
    <>
      <Card sx={{ display: 'flex' }}>
        <Stack spacing={2} sx={{ p: theme.spacing(3, 3, 2, 3) }}>
          <Box display="flex" alignItems="center" justifyContent="space-between" sx={{ mb: 2 }}>
            <Label variant="soft" color={'info'}>
              {fDate(createdAt)}
            </Label>
            <Box component="span" sx={{ typography: 'caption', color: 'text.disabled' }}>
              {articleCount} Article
            </Box>
          </Box>

          <Stack spacing={1} flexGrow={1}>
            <Link
              component={RouterLink}
              href={paths.dashboard.chatbot.help.details(id)}
              color="inherit"
              variant="subtitle2"
              sx={{ ...maxLine({ line: 2 }) }}
            >
              {name}
            </Link>

            <Typography variant="body2" sx={{ ...maxLine({ line: 2 }), color: 'text.secondary' }}>
              {description}
            </Typography>
          </Stack>

          <Box display="flex" alignItems="center">
            <IconButton color={popover.open ? 'inherit' : 'default'} onClick={popover.onOpen}>
              <Iconify icon="eva:more-horizontal-fill" />
            </IconButton>
          </Box>
        </Stack>
      </Card>

      <CustomPopover
        open={popover.open}
        anchorEl={popover.open}
        onClose={popover.onClose}
      >
        <MenuList>
          <MenuItem
            onClick={() => {
              popover.onClose();
              router.push(paths.dashboard.chatbot.help.details(id));
            }}
          >
            <Iconify icon="solar:eye-bold" />
            Voir Les Détails
          </MenuItem>
          <MenuItem
            onClick={() => {
              popover.onClose();
              router.push(paths.dashboard.chatbot.help.edit(id));
            }}
          >
            <Iconify icon="solar:pen-bold" />
            Modifier
          </MenuItem>

          <MenuItem
            onClick={() => {
              popover.onClose();
              setOpenDialog(true);
            }}
            sx={{ color: 'error.main' }}
          >
            <Iconify icon="solar:trash-bin-trash-bold" />
            Supprimer
          </MenuItem>
        </MenuList>
      </CustomPopover>

      <Dialog
        open={openDialog}
        onClose={() => setOpenDialog(false)}
        aria-labelledby="alert-dialog-title"
        aria-describedby="alert-dialog-description"
      >
        <DialogTitle id="alert-dialog-title">Confirmer la suppression</DialogTitle>
        <DialogContent>
          <DialogContentText id="alert-dialog-description">
            Êtes-vous sûr de vouloir supprimer cette collection ?
          </DialogContentText>
        </DialogContent>
        <DialogActions>
          <Button onClick={() => hundleCancel()} color="primary">
            Annuler
          </Button>
          <Button onClick={() => handleDelete(id)} color="primary" autoFocus>
            Supprimer
          </Button>
        </DialogActions>
      </Dialog>
    </>
  );
}
