import { z as zod } from 'zod';
import { useForm } from 'react-hook-form';
import { zodResolver } from '@hookform/resolvers/zod';
import { useEffect, useState } from 'react';
import {
  Box,
  Card,
  Stack,
  Divider,
  CardHeader,
  TextField,
  Chip,
  Dialog,
  DialogActions,
  DialogContent,
  DialogContentText,
  DialogTitle,
  Button,
  Typography,
} from '@mui/material';
import LoadingButton from '@mui/lab/LoadingButton';
import toast from 'react-hot-toast';
import { paths } from '@/routes/paths';
import { useRouter } from 'next/navigation';
import {
  addCollection,
  addArticle,
  updateCollection,
  updateArticle,
  deleteArticle,
  getCollectionWithArticles,
} from '@/shared/api/help';

import { IHelpGetItem } from '@/shared/types/help';
import Iconify from '@/components/iconify';
import { FormProvider } from 'react-hook-form';
import { Field } from '@/shared/components/hook-form';

// Schema for form validation
const NewPostSchema = zod.object({
  name: zod.string().min(1, { message: 'Nom de la Collection est requis!' }),
  description: zod.string().min(1, { message: 'Description de la Collection est requis!' }),
});

// Props type definition
type Props = {
  currentCollection?: IHelpGetItem;
  collectionId?: string;
};

// Article interface for type safety
interface Article {
  id: string;
  question: string;
  answer: string;
  modified?: boolean;
}

export function CollectionNewEditForm({ currentCollection, collectionId }: Props) {
  const router = useRouter();

  // State declarations with proper typing
  const [existingArticles, setExistingArticles] = useState<Article[]>([]);
  const [newArticles, setNewArticles] = useState<Article[]>([]);
  const [deletedArticles, setDeletedArticles] = useState<string[]>([]);
  const [isCollectionReady, setIsCollectionReady] = useState(!!currentCollection);
  const [articleToDelete, setArticleToDelete] = useState<string | null>(null);

  // Form setup with react-hook-form
  const methods = useForm({
    mode: 'onSubmit',
    resolver: zodResolver(NewPostSchema),
    defaultValues: {
      name: currentCollection?.name || '',
      description: currentCollection?.description || '',
    },
  });

  const {
    handleSubmit,
    formState: { isSubmitting },
    watch,
    reset,
  } = methods;
  const name = watch('name');
  const description = watch('description');

  // Update collection readiness when name and description are set
  useEffect(() => {
    if (name && description) setIsCollectionReady(true);
  }, [name, description]);

  // Load existing articles and convert IDs to strings
  useEffect(() => {
    if (currentCollection) {
      setExistingArticles(
        currentCollection.articles?.map((article) => ({
          ...article,
          id: String(article.id), // Ensure id is always a string
          modified: false,
        })) || []
      );
      reset({ name: currentCollection.name, description: currentCollection.description });
    }
  }, [currentCollection, reset]);

  const collectionUpdated = name !== currentCollection?.name || description !== currentCollection?.description;

  // Add a new temporary article
  const handleAddArticle = () => {
    const newArticle: Article = { id: `temp-${Date.now()}`, question: '', answer: '' };
    setNewArticles([...newArticles, newArticle]);
  };

  // Update an article's question or answer
  const handleUpdateArticle = (id: string, field: 'question' | 'answer', value: string) => {
    if (id.startsWith('temp-')) {
      setNewArticles((prev) =>
        prev.map((article) => (article.id === id ? { ...article, [field]: value } : article))
      );
    } else {
      setExistingArticles((prev) =>
        prev.map((article) =>
          article.id === id ? { ...article, [field]: value, modified: true } : article
        )
      );
    }
  };

  // Show confirmation dialog before deleting an article
  const handleDeleteArticle = (id: string) => {
    setArticleToDelete(id);
  };

  // Confirm deletion of an article
  const confirmDeleteArticle = () => {
    if (articleToDelete) {
      if (articleToDelete.startsWith('temp-')) {
        setNewArticles(newArticles.filter((article) => article.id !== articleToDelete));
      } else {
        setDeletedArticles([...deletedArticles, articleToDelete]);
        setExistingArticles(existingArticles.filter((article) => article.id !== articleToDelete));
      }
      setArticleToDelete(null);
    }
  };

  // Cancel deletion of an article
  const cancelDeleteArticle = () => {
    setArticleToDelete(null);
  };

  // Fetch updated collection data after modifications
  const fetchUpdatedCollection = async () => {
    if (!collectionId) {
      throw new Error('Collection ID is required');
    }
    const updatedCollection = await getCollectionWithArticles(collectionId);
    setExistingArticles(
      updatedCollection.articles?.map((article: any) => ({
        ...article,
        id: String(article.id), // Ensure id is a string
        modified: false,
      })) || []
    );
    reset({ name: updatedCollection.name, description: updatedCollection.description });
  };

  // Handle form submission
  const onSubmit = async (data: any) => {
    try {
      let collectionData = { collectionData: { id: collectionId } };

      // Create or update collection
      if (!currentCollection) {
        collectionData = await addCollection({ name: data.name, description: data.description });
        setIsCollectionReady(true);
      } else if (collectionUpdated && collectionId) {
        collectionData = await updateCollection(collectionId, { name: data.name, description: data.description });
      }

      const collectionIdToUse = collectionData.collectionData?.id || collectionId;
      if (!collectionIdToUse) {
        throw new Error('Collection ID is required');
      }

      // Process new articles
      const addTasks = newArticles.map((article) => {
        const { id, ...articleData } = article;
        return addArticle(collectionIdToUse, articleData);
      });

      // Process updated articles
      const updateTasks = existingArticles
        .filter((a) => a.modified)
        .map((article) =>
          updateArticle(collectionIdToUse, article.id, { id: article.id, question: article.question, answer: article.answer })
        );

      // Process deleted articles
      const deleteTasks = deletedArticles.map((id) => deleteArticle(collectionIdToUse, id));

      // Execute all tasks concurrently
      await Promise.all([...addTasks, ...updateTasks, ...deleteTasks]);

      // Refresh collection data if updates were made
      if (updateTasks.length) await fetchUpdatedCollection();

      toast.success(currentCollection ? 'Mis à jour avec Succès' : 'Collection créée');
      router.push(paths.dashboard.chatbot.help.root);
    } catch (error) {
      console.error('Error during submission:', error);
    }
  };

  return (
    <FormProvider {...methods}>
      <form onSubmit={handleSubmit(onSubmit)}>
        <Stack spacing={5} sx={{ mx: 'auto', maxWidth: { xs: 720, xl: 880 } }}>
          {/* Collection Details Card */}
          <Card>
            <CardHeader title="Collection" subheader="Entrer les détails de la collection..." sx={{ mb: 3 }} />
            <Divider />
            <Stack spacing={3} sx={{ p: 3 }}>
              <TextField label="Nom de la Collection" fullWidth required {...methods.register('name')} />
              <TextField
                label="Description de la Collection"
                fullWidth
                multiline
                rows={3}
                {...methods.register('description')}
              />
            </Stack>
          </Card>

          {/* Articles Card */}
          {isCollectionReady && (
            <Card>
              <CardHeader title="Articles" subheader="Ajouter des articles à la collection..." sx={{ mb: 2 }} />
              <Divider />
              <Stack spacing={2} sx={{ p: 2 }}>
                {[...existingArticles, ...newArticles].map((article, index) => (
                  <Stack key={article.id} spacing={2} sx={{ mb: 2, border: '1px solid #eee', p: 2, borderRadius: 1 }}>
                    <Stack direction="row" alignItems="center" spacing={1}>
                      <Chip label={`Article ${index + 1}`} variant="outlined" size="medium" />
                      <Iconify icon="solar:trash-bin-trash-bold" color="error" onClick={() => handleDeleteArticle(article.id)} />
                    </Stack>
                    <TextField
                      label="Question"
                      variant="outlined"
                      fullWidth
                      multiline
                      rows={2}
                      value={article.question}
                      onChange={(e) => handleUpdateArticle(article.id, 'question', e.target.value)}
                    />
                    <Field.Editor
                      name={`answer-${article.id}`}
                      value={article.answer}
                      onChange={(html: string) => handleUpdateArticle(article.id, 'answer', html)}
                      sx={{ maxHeight: 200 }}
                      placeholder="Entrer la réponse..."
                    />
                  </Stack>
                ))}
                <LoadingButton type="button" variant="contained" size="large" onClick={handleAddArticle}>
                  Ajouter un Article
                </LoadingButton>
              </Stack>
            </Card>
          )}

          {/* Submit Button */}
          <Box display="flex" alignItems="center" justifyContent="flex-end">
            <LoadingButton type="submit" variant="contained" size="large" loading={isSubmitting} sx={{ mt: 3 }}>
              {currentCollection ? 'Mettre à Jour' : 'Créer une Collection'}
            </LoadingButton>
          </Box>
        </Stack>

        {/* Confirmation Dialog */}
        <Dialog
          open={!!articleToDelete}
          onClose={cancelDeleteArticle}
          aria-labelledby="alert-dialog-title"
          aria-describedby="alert-dialog-description"
        >
          <DialogTitle id="alert-dialog-title">{"Confirmer la suppression"}</DialogTitle>
          <DialogContent>
            <DialogContentText id="alert-dialog-description">
              Êtes-vous sûr de vouloir supprimer cet article ?
            </DialogContentText>
          </DialogContent>
          <DialogActions>
            <Button onClick={cancelDeleteArticle} color="primary">
              Annuler
            </Button>
            <Button onClick={confirmDeleteArticle} color="primary" autoFocus>
              Supprimer
            </Button>
          </DialogActions>
        </Dialog>
      </form>
    </FormProvider>
  );
}