import { CircularProgress } from '@mui/material';
import React, { useState } from 'react';
import Stack from '@mui/material/Stack';
import { IconButton, Dialog, DialogTitle, DialogContent, DialogActions, Button, Select, MenuItem } from '@mui/material';
import { MenuList } from '@mui/material';
import { MenuItem as MuiMenuItem } from '@mui/material';

import { useGetConversation } from '@/shared/api/chat-bot';
import { usePopover } from '@/shared/components/custom-popover';
import Iconify from '@/components/iconify';
import { useUpdateTicket , useGetSupportUsers } from '@/shared/api/ticket';
import { useGetTicketByConversation } from '@/shared/api/chat-bot';
import toast from 'react-hot-toast';
import { useRouter } from 'next/navigation';
import { ISupportAgent } from '@/shared/types/user';
import CustomPopover from '@/shared/components/custom-popover';

type Props = {
  conversationId: string;
};

export function ChatHeader({ conversationId }: Props) {
  const { conversation, conversationLoading } = useGetConversation(conversationId);
  const { ticket, ticketLoading } = useGetTicketByConversation(conversationId);
  const { updateTicket } = useUpdateTicket();
  const { support, supportLoading } = useGetSupportUsers();
  const router = useRouter();

  const categories: { [key: string]: string } = {
    IN_PROGRESS: 'En cours',
    RESOLVED: 'Résolu',
    WAITING: 'En attente',
    OPEN: 'Ouvert',
    REASSIGNED: 'Réassigné',
    REJECTEED: 'Rejeté'
  };

  const popover = usePopover();
  const [openReassignDialog, setOpenReassignDialog] = useState(false);
  const [selectedSupportAgent, setSelectedSupportAgent] = useState<string>('');

  const handleReassign = async () => {
    if (ticket && selectedSupportAgent) {
      await updateTicket(ticket.id, {
        ...ticket,
        support_id: selectedSupportAgent,
        ticketstatus: 'IN_PROGRESS'
      });
      setOpenReassignDialog(false);
      toast.success('Ticket a été réassigné avec succès');
      router.push('/dashboard/ticket/');
    }
  };

  return (
    <>
      <div className="chat-header">
        {conversationLoading ? (
          <div className="chat-header__loading">
            <CircularProgress />
          </div>
        ) : (
          <div className="chat-header__content">
            <div className="chat-header__content__title">
              {conversation?.title}
            </div>
          </div>
        )}
      </div>
      {conversation ? (
        <>
          <Stack direction="row" flexGrow={1} justifyContent="flex-end" spacing={1}>
            {/* Button to open reassign dialog */}
            <IconButton onClick={() => setOpenReassignDialog(true)}>
              <Iconify icon="eva:refresh-fill" />
            </IconButton>
            <IconButton onClick={popover.onOpen}>
              <Iconify icon="eva:more-vertical-fill" />
            </IconButton>
          </Stack>
          <CustomPopover open={popover.open} anchorEl={popover.open} onClose={popover.onClose}>
            <MenuList>
              <MuiMenuItem disabled>
                <strong>Mettre à jour le statut du ticket</strong>
              </MuiMenuItem>
              {Object.entries(categories).map(([key, value]) => (
                <MuiMenuItem
                  key={key}
                  onClick={async () => {
                    if (ticket) {
                      await updateTicket(ticket.id, { 
                        ...ticket,
                        ticketstatus: key 
                      });
                    }
                    toast.success(`Statut du ticket mis à jour: ${value}`);
                    popover.onClose();
                  }}
                >
                  {value}
                </MuiMenuItem>
              ))}
            </MenuList>
          </CustomPopover>
          {/* Reassign Dialog */}
          <Dialog open={openReassignDialog} onClose={() => setOpenReassignDialog(false)} maxWidth="sm" fullWidth>
            <DialogTitle>Réassigner le ticket</DialogTitle>
            <DialogContent>
              {supportLoading ? (
                <CircularProgress />
              ) : (
                <Select
                  fullWidth
                  value={selectedSupportAgent}
                  onChange={(e) => setSelectedSupportAgent(e.target.value)}
                  displayEmpty
                >
                  <MenuItem value="">
                    <em>Choisir un agent de support</em>
                  </MenuItem>
                  {support.map((agent:ISupportAgent) => (
                    <MenuItem key={agent.id} value={agent.id}>
                      {agent.firstName} {agent.lastName}
                    </MenuItem>
                  ))}
                </Select>
              )}
            </DialogContent>
            <DialogActions>
              <Button onClick={() => setOpenReassignDialog(false)}>Annuler</Button>
              <Button onClick={handleReassign} disabled={!selectedSupportAgent}>
                Accepter
              </Button>
            </DialogActions>
          </Dialog>
        </>
      ) : null}
    </>
  );
}
