"use client";
import React, { useCallback } from "react";
import { IChatConversations } from "@/shared/types/chat-bot";
import {
  Box,
  List,
  ListItemAvatar,
  ListItemButton,
  ListItemText,
  Avatar,
  Typography,
  useTheme,
  Stack,
  Badge,
} from "@mui/material";
import TextField from "@mui/material/TextField";
import InputAdornment from "@mui/material/InputAdornment";
import IconButton from "@mui/material/IconButton";
import { ChatNavAccount } from "./chat-nav-account";

import { fToNow } from "@/utils/format-time";
import Iconify from "@/shared/components/iconify";
import Scrollbar from "@/shared/components/scrollbar";
import { useCollapseNav , useResponsive } from "@/hooks";
import { useGetUserById } from "@/shared/api/ticket";


type Props = {
  users: any;
  user: any;
  conversations: IChatConversations[];
  onClickConversation: (id: string) => void;
  selectedConversationId: string;
  collapseNav: ReturnType<typeof useCollapseNav>;
  ticketFilter: string;
  onChangeTicketFilter: (value: string) => void;
};

const NAV_COLLAPSE_WIDTH = 90;

type ChatConversationItemProps = {
  chat: IChatConversations;
  selectedConversationId: string;
  onClickConversation: (id: string) => void;
};

function ChatConversationItem({
  chat,
  selectedConversationId,
  onClickConversation,
}: ChatConversationItemProps) {
  const { user } = useGetUserById(chat.user_id);
  const displayName = user
    ? `${user.firstName} ${user.lastName}`
    : chat.title;

  return (
    <Box component="li" sx={{ display: "flex" }}>
      <ListItemButton
        selected={selectedConversationId === chat.id}
        onClick={() => onClickConversation(chat.id)}
        sx={{
          py: 1.5,
          px: 2.5,
          gap: 1,
          ...(selectedConversationId === chat.id && { bgcolor: "action.selected" }),
        }}
      >
        <Badge
          color="error"
          overlap="circular"
          variant="offline"
          anchorOrigin={{ vertical: "bottom", horizontal: "right" }}
        >
          <ListItemAvatar>
            <Avatar alt={displayName} src="/static/mock-images/avatars/avatar_1.jpg" />
          </ListItemAvatar>
        </Badge>
        <ListItemText
          primary={displayName}
          primaryTypographyProps={{ noWrap: true, component: "span", variant: "subtitle2" }}
          secondary={chat.messages[chat.messages.length - 1].content}
          secondaryTypographyProps={{
            noWrap: true,
            component: "span",
            variant: "body2",
            color: "text.secondary",
          }}
        />
        <Stack alignItems="flex-end" sx={{ alignSelf: "stretch" }}>
          <Typography
            noWrap
            variant="body2"
            component="span"
            sx={{ mb: 1.5, fontSize: 12, color: "text.disabled" }}
          >
            {fToNow(chat.messages[chat.messages.length - 1].timestamp)}
          </Typography>
        </Stack>
      </ListItemButton>
    </Box>
  );
}

export function ChatList({
  users,
  user,
  conversations,
  onClickConversation,
  selectedConversationId,
  collapseNav,
  ticketFilter,
  onChangeTicketFilter,
}: Props) {
  const { onCloseMobile, collapseDesktop, onCollapseDesktop } = collapseNav;
  const theme = useTheme();

  const [searchContacts, setSearchContacts] = React.useState({ query: "" });

  const handleSearchContacts = (query: string) => {
    setSearchContacts({ ...searchContacts, query });
  };

  const filteredConversations = conversations.filter((chat) =>
    chat.title.toLowerCase().includes(searchContacts.query.toLowerCase())
  );

  const mdUp = useResponsive("up", "md");

  const handleToggleNav = useCallback(() => {
    if (mdUp) {
      onCollapseDesktop();
    } else {
      onCloseMobile();
    }
  }, [mdUp, onCloseMobile, onCollapseDesktop]);

  return (
  <Stack
    sx={{
      minHeight: 0,
      flex: 1,
      width: collapseDesktop ? NAV_COLLAPSE_WIDTH : 350,
      display: { xs: "none", md: "flex" },
      flexDirection: "column",
      transition: theme.transitions.create(["width"], {
        duration: theme.transitions.duration.shorter,
      }),
      ...(collapseDesktop && { width: NAV_COLLAPSE_WIDTH }),
      position: 'relative', // Add this to make positioning more predictable
    }}
  >
    <Stack direction="row" alignItems="center" justifyContent="center" sx={{ p: 2.5, pb: 0, flexShrink: 0 }}>
      {!collapseDesktop && (
        <>
          <ChatNavAccount
            user={user}
            users={users}
            ticketFilter={ticketFilter}
            onChangeTicketFilter={onChangeTicketFilter}
          />
          <Box sx={{ flexGrow: 1 }} />
        </>
      )}
      <IconButton onClick={handleToggleNav}>
        <Iconify
          icon={collapseDesktop ? "eva:arrow-ios-forward-fill" : "eva:arrow-ios-back-fill"}
        />
      </IconButton>
    </Stack>
    
    {!collapseDesktop && (
      <TextField
        fullWidth
        value={searchContacts.query}
        onChange={(event) => handleSearchContacts(event.target.value)}
        placeholder="Rechercher un Contact ..."
        InputProps={{
          startAdornment: (
            <InputAdornment position="start">
              <Iconify icon="eva:search-fill" sx={{ color: "text.disabled" }} />
            </InputAdornment>
          ),
        }}
        sx={{ mt: 2.5, p: 2.5, pt: 0, flexShrink: 0 }}
      />
    )}
    
    {/* Modified Scrollbar with absolute position to prevent double scrollbars */}
    <Box sx={{ 
      flex: 1, 
      position: 'relative',  // This is important
      overflow: 'hidden',    // Hide any overflow at this level
    }}>
      <Scrollbar 
        sx={{ 
          position: 'absolute', 
          inset: 0,            // Take up all the space
          overflowX: "hidden"  // Disable horizontal scrolling
        }}
      >
        <List>
          {filteredConversations.map((chat) => (
            <ChatConversationItem
              key={chat.id}
              chat={chat}
              selectedConversationId={selectedConversationId}
              onClickConversation={onClickConversation}
            />
          ))}
        </List>
      </Scrollbar>
    </Box>
  </Stack>
);
}