'use client';

import React, { useState, useEffect, useCallback } from 'react';
import Container from '@mui/material/Container';
import { CircularProgress, Stack } from '@mui/material';

import CustomBreadcrumbs from '@/shared/components/custom-breadcrumbs';
import { useCollapseNav } from '@/hooks';
import { useWebSocket } from '@/hooks/use-web-socket';
import {
  useGetAllUsers,
  getCurrentChatUser,
  useConversations, // Import the new custom Hook
} from '@/shared/api/chat-bot';
import { ChatList } from '../chat-list';
import { ChatWindow } from '../chat-window';
import { ChatHeader } from '../chat-header';
import { ChatMessageInput } from '../chat-message-input';
import { ChatLayout } from '../chat-layout';
import { ChatRoom } from '../chat-room';
import { IMessageSocketItem } from '@/shared/types/chat-bot';
import { ChatUser } from '@/shared/types/user';
import { ChatEmptyState } from '../chat-empty-state';

export function ChatView() {
  const [userData, setUserData] = useState<ChatUser | null>(null);

  useEffect(() => {
    async function fetchUser() {
      const token = localStorage.getItem('chatbotToken');
      if (token) {
        const data = await getCurrentChatUser(token);
        setUserData(data);
      }
    }
    fetchUser();
  }, []);

  const userId = userData?.id;
  const isAdminOrLeader = userData?.role === 'ADMIN' || userData?.role === 'LEADER';

  const { users } = useGetAllUsers(isAdminOrLeader);

  const [selectedConversationId, setSelectedConversationId] = useState<string | null>(null);
  const [ticketFilter, setTicketFilter] = useState('me');
  const [previousTicketFilter, setPreviousTicketFilter] = useState('me');

  const userIdToFetch =
    ticketFilter === 'all' ? null : ticketFilter === 'me' ? userId : ticketFilter;

  // Use the custom Hook unconditionally
  const { conversations, conversationsLoading, refetchConversations } = useConversations(userIdToFetch);
  
  // Reset selected conversation when ticket filter changes
  useEffect(() => {
    if (ticketFilter !== previousTicketFilter) {
      setSelectedConversationId(null);
      setPreviousTicketFilter(ticketFilter);
    }
  }, [ticketFilter, previousTicketFilter]);

  const sortedConversations = conversations?.sort((a:any, b:any) => {
    const aLatestTimestamp = a.messages?.length
      ? new Date(a.messages[a.messages.length - 1].timestamp).getTime()
      : new Date(a.createdAt).getTime();
    const bLatestTimestamp = b.messages?.length
      ? new Date(b.messages[b.messages.length - 1].timestamp).getTime()
      : new Date(b.createdAt).getTime();
    return bLatestTimestamp - aLatestTimestamp;
  });

  const { messages, sendMessage } = useWebSocket(userId);
  const conversationsNav = useCollapseNav();

  useEffect(() => {
    if (messages?.length > 0) {
      refetchConversations();
    }
  }, [messages, refetchConversations]);

  const handleSendMessage = useCallback(
    async (message:IMessageSocketItem) => {
      sendMessage(message);
    },
    [sendMessage]
  );

  return (
    <Container
      maxWidth={false}
      sx={{ display: 'flex', flex: '1 1 auto', flexDirection: 'column' }}
    >
      <ChatLayout
        sx={{
          minHeight: 0,
          flex: '1 1 0',
          borderRadius: 1.3,
          position: 'relative',
          bgcolor: 'background.paper',
          boxShadow: (theme) => theme.customShadows.card,
        }}
        slots={{
          nav: (
            <>
              {conversationsLoading ? (
                <CircularProgress />
              ) : (
                <ChatList
                  user={userData}
                  users={users}
                  conversations={sortedConversations || []}
                  onClickConversation={(id: string) => setSelectedConversationId(id)}
                  selectedConversationId={selectedConversationId || ''}
                  collapseNav={conversationsNav}
                  ticketFilter={ticketFilter}
                  onChangeTicketFilter={setTicketFilter}
                />
              )}
            </>
          ),
        header: selectedConversationId ? (
          <ChatHeader conversationId={selectedConversationId} />
        ) : null,
          main: (
            <Stack direction="column" spacing={2} sx={{ height: '100%' }}>
              {selectedConversationId ? (
                <>
                  <ChatWindow
                    conversationId={selectedConversationId}
                    socketMessages={messages}
                    key={`${selectedConversationId}-${ticketFilter}`}
                  />
                  <ChatMessageInput
                    conversationId={selectedConversationId}
                    onSendMessage={handleSendMessage}
                    userid={userId}
                    ticketFilter={ticketFilter}
                  />
                </>
              ) : (
                <ChatEmptyState
                  title="Bonjour!"
                  description="Merci de sélectionner une conversation pour commencer à discuter."
                />
              )}
            </Stack>
          ),
          details: selectedConversationId && (
            <ChatRoom
              user={userData}
              collapseNav={conversationsNav}
              conversationId={selectedConversationId}
            />
          ),
        }}
      />
    </Container>
  );
}