import { z as zod } from 'zod';
import { useForm } from 'react-hook-form';
import { zodResolver } from '@hookform/resolvers/zod';
import { useMemo, useEffect} from 'react';

import Card from '@mui/material/Card';
import Stack from '@mui/material/Stack';
import Divider from '@mui/material/Divider';
import CardHeader from '@mui/material/CardHeader';
import LoadingButton from '@mui/lab/LoadingButton';

import { paths } from '@/routes/paths';
import { useRouter } from 'next/navigation';
import { updateOpenAIConf } from '@/shared/api/openai';

import toast from 'react-hot-toast';
import FormProvider , {Field} from '@/shared/components/hook-form';
import { IOpenAiItem } from '@/shared/types/openai';
import { OPENAI_MODEl_OPTIONS , OPENAI_STATUS_OPTIONS} from '@/shared/_mock/_openai';

// ----------------------------------------------------------------------

export type NewOpenAiSchemaType = zod.infer<typeof NewOpenAISchema>;

export const NewOpenAISchema = zod.object({
  name: zod.string().nonempty('Nom De Configuration est requis'),
  apiKey: zod.string().nonempty('API Key est requis'),
  status: zod.string().nonempty('Status est requis'),
});


// ----------------------------------------------------------------------

type Props = {
  currentOpenAi?: IOpenAiItem | null;
};

export function OpenAINewEditForm({ currentOpenAi }: Props) {
  const router = useRouter();

  // First form (OpenAI config)
  const defaultValues = useMemo(
    () => ({
      name: currentOpenAi?.name || '',
      apiKey: currentOpenAi?.apiKey || '',
      status: currentOpenAi?.status ? String(currentOpenAi.status) : '',
    }),
    [currentOpenAi]
  );

  const configMethods = useForm<NewOpenAiSchemaType>({
    resolver: zodResolver(NewOpenAISchema),
    defaultValues,
  });

  const {
    reset,
    handleSubmit: handleConfigSubmit,
    formState: { isSubmitting },
  } = configMethods;

  useEffect(() => {
    if (currentOpenAi) {
      reset(defaultValues);
    }
  }, [currentOpenAi, defaultValues, reset]);

  const onConfigSubmit = handleConfigSubmit(async (data) => {
    try {
      await new Promise((resolve) => setTimeout(resolve, 500));
      await updateOpenAIConf(currentOpenAi?.id || '', { ...data, status: data.status === 'true' });
      reset();
      router.push(paths.dashboard.chatbot.openai.root);
      toast.success(currentOpenAi ? 'Configuration Modifié' : 'Configuration Ajouté');
    } catch (error) {
      console.error(error);
    }
  });

  const renderDetails = (
    <Card>
      <CardHeader
        title="Configuration OpenAI"
        subheader="Ajouter ou Modifier Configuration OpenAI"
        sx={{ mb: 3 }}
      />
      <Divider />
      <Stack spacing={3} sx={{ p: 3 }}>
        <Field.Select native name="name" label="Nom De la Model" InputLabelProps={{ shrink: true }}>
          {OPENAI_MODEl_OPTIONS.map((category) => (
            <option key={category.value} value={category.value}>
              {category.label}
            </option>
          ))}
        </Field.Select>
        <Field.Text type="password" name="apiKey" label="API Key" />
        <Field.Select native name="status" label="Status" InputLabelProps={{ shrink: true }}>
          {OPENAI_STATUS_OPTIONS.map((category) => (
            <option key={category.value} value={category.value}>
              {category.label}
            </option>
          ))}
        </Field.Select>
      </Stack>
    </Card>
  );

  return (
      <FormProvider methods={configMethods} onSubmit={onConfigSubmit}>
        <Stack spacing={{ xs: 3, md: 5 }} sx={{ mb: 5, mx: 'auto', maxWidth: { xs: 720, xl: 1200 } }}>
          {renderDetails}
          <Stack spacing={3} direction="row" alignItems="center" flexWrap="wrap">
            <LoadingButton type="submit" variant="contained" size="large" loading={isSubmitting}>
              {!currentOpenAi ? 'Créer Configuration' : 'Modifier Configuration'}
            </LoadingButton>
          </Stack>
        </Stack>
      </FormProvider>
  );
}