'use client';

import type {
  GridColDef,
  GridRowSelectionModel,
  GridColumnVisibilityModel,
} from '@mui/x-data-grid';

import { useState, useEffect, useCallback } from 'react';

import Card from '@mui/material/Card';
import Stack from '@mui/material/Stack';
import Button from '@mui/material/Button';
import Dialog from '@mui/material/Dialog';
import DialogActions from '@mui/material/DialogActions';
import DialogContent from '@mui/material/DialogContent';
import DialogContentText from '@mui/material/DialogContentText';
import DialogTitle from '@mui/material/DialogTitle';
import {
  DataGrid,
  gridClasses,
  GridActionsCellItem,
  GridToolbarContainer,
  GridToolbarQuickFilter,
} from '@mui/x-data-grid';

import { paths } from '@/routes/paths';
import { useRouter } from '@/hooks';
import { useBoolean } from '@/hooks';
import Container from "@mui/material/Container";

import toast from 'react-hot-toast';
import Iconify from '@/components/iconify';
import EmptyContent from '@/shared/components/empty-content';
import CustomBreadcrumbs from '@/shared/components/custom-breadcrumbs';

import { 
  useGetTicketBySupportId,
  useUpdateTicket
} from '@/shared/api/ticket';

import {
  RenderCellTicketTitle,
  RenderCellCreatedAt,
  RenderCellUpdatedAt,
  RenderCellStatus,
  RenderCellAssignedAt
} from '../support-table-row';

import { ITiketItem } from '@/shared/types/ticket';
import { ChatUser } from '@/shared/types/user';
import { getCurrentChatUser } from '@/shared/api/chat-bot';


const HIDE_COLUMNS = { category: false };
const HIDE_COLUMNS_TOGGLABLE = ['category', 'actions'];

// ----------------------------------------------------------------------

export function SupportTicketListView() {
  const confirmRows = useBoolean();

  const [userData, setUserData] = useState<ChatUser | null>(null);
  useEffect(() => {
      async function fetchUser() {
        const token = localStorage.getItem('chatbotToken');
        if (token) {
          const data = await getCurrentChatUser(token);
          setUserData(data);
        }
      }
      fetchUser();
  }, []);

  const user = userData || null;
  const { tickets, ticketsLoading } = useGetTicketBySupportId(user?.id ?? '');
  const { updateTicket } = useUpdateTicket();
  const router = useRouter();
  const [tableData, setTableData] = useState<ITiketItem[]>([]);
  const [selectedRowIds, setSelectedRowIds] = useState<GridRowSelectionModel>([]);
  const [filterButtonEl, setFilterButtonEl] = useState<HTMLButtonElement | null>(null);
  const [columnVisibilityModel, setColumnVisibilityModel] = useState<GridColumnVisibilityModel>(HIDE_COLUMNS);
  const [openDialog, setOpenDialog] = useState(false);
  const [selectedTicketId, setSelectedTicketId] = useState<string | null>(null);

  useEffect(() => {
    if (tickets.length) {
      setTableData(tickets);
    }
  }, [tickets]);

  const handleAcceptTicket = useCallback(
    (id: string) => {
      setSelectedTicketId(id);
      setOpenDialog(true);
    },
    []
  );

  const handleReassigneTicket = useCallback(
    async (id: string) => {
      const ticketToUpdate = tableData.find((ticket) => ticket.id === id);
      if (ticketToUpdate) {
        const updatedTicket = { 
          ...ticketToUpdate, 
          support_id: null,
          ticketstatus: "REASSIGNED" 
        };
        const response =await updateTicket(id, updatedTicket);
        console.log(response);
        toast.success('Ticket a été réassigné');
      }
    },
    [tableData, updateTicket]
  );
  
  

  const handleConfirmAcceptTicket = useCallback(
    async () => {
      if (selectedTicketId) {
        const ticketToUpdate = tableData.find((ticket) => ticket.id === selectedTicketId);
        if(ticketToUpdate) {
          await updateTicket(selectedTicketId, { ...ticketToUpdate, ticketstatus: "IN_PROGRESS" });
          toast.success('Ticket A été accepté');
          router.push(paths.dashboard.chatbot.support.conversation);
        }
        setOpenDialog(false);
        setSelectedTicketId(null);
      }
    },
    [selectedTicketId, tableData, updateTicket]
  );

  const CustomToolbarCallback = useCallback(
    () => (
      <CustomToolbar
        selectedRowIds={selectedRowIds}
        setFilterButtonEl={setFilterButtonEl}
        filteredResults={tableData.length}
        onOpenConfirmDeleteRows={confirmRows.onTrue}
      />
    ),
    [selectedRowIds]
  );

  const columns: GridColDef[] = [
    {
      field: 'title',
      headerName: 'Titre',
      width: 520,
      renderCell: (params) => <RenderCellTicketTitle params={params} />,
    },
    {
      field: 'createdAt',
      headerName: 'Créé le',
      width: 200,
      renderCell: (params) => <RenderCellCreatedAt params={params} />,
    },
    {
      field: 'updatedAt',
      headerName: 'Mis à jour le',
      width: 200,
      renderCell: (params) => <RenderCellUpdatedAt params={params} />,
    },
    {
      field: 'assignedAt',
      headerName: 'Assigné le',
      width: 200,
      renderCell: (params) => <RenderCellAssignedAt params={params} />,
    },
    { 
      field: 'ticketstatus', 
      headerName: 'Statut',
      width: 150, 
      renderCell: (params) => <RenderCellStatus params={params} />,
    },
    {
      type: 'actions',
      field: 'actions',
      headerName: ' ',
      align: 'right',
      headerAlign: 'right',
      width: 80,
      sortable: false,
      filterable: false,
      disableColumnMenu: true,
      getActions: (params) => {
        if (params.row.ticketstatus !== 'OPEN' && params.row.ticketstatus !== 'REASSIGNED') {
          return [];
        }
        return [
          <GridActionsCellItem
            showInMenu
            icon={<Iconify icon="solar:archive-minimalistic-bold-duotone" />}
            label="Accepter"
            onClick={() => handleAcceptTicket(params.row.id)}
          />,
          <GridActionsCellItem
            showInMenu
            icon={<Iconify icon="solar:archive-minimalistic-bold-duotone" />}
            label="Reassigner"
            onClick={() => handleReassigneTicket(params.row.id)}
          />,
        ];
      },
    }
    
  ];

  const getTogglableColumns = () =>
    columns.filter((column) => !HIDE_COLUMNS_TOGGLABLE.includes(column.field)).map((column) => column.field);

  return (
    <>
      <Container sx={{ flexGrow: 1, display: 'flex', flexDirection: 'column' }}>
        <CustomBreadcrumbs
          heading="Liste des tickets"
          links={
            [
              { name: 'Tableau de bord' }, 
              { name: 'Liste des ticketss' , href : paths.dashboard.chatbot.support.root}
            ]
          }
          sx={{ mb: { xs: 5, md: 5 } }}
        />

        <Card
          sx={{
            flexGrow: { md: 2 },
            display: { md: 'flex' },
            height: { xs: 800, md: 2 },
            width: { md: 1 },
            flexDirection: { md: 'column' },
          }}
        >
          <DataGrid
            disableRowSelectionOnClick
            localeText={{
              toolbarQuickFilterPlaceholder: 'Rechercher...',
              toolbarColumns: "Colonnes",
              toolbarColumnsLabel: "Chercher",
              toolbarFilters: "Filtrer",
              toolbarExport: "Exporter",
              filterPanelOperator: 'Opérateurs',
              filterPanelColumns: 'Colonnes',
              filterPanelInputLabel: 'Valeur',
              filterPanelInputPlaceholder: 'Valeur du filtre',
              filterOperatorContains: 'Contient',
              filterOperatorEquals: 'Égal à',
              filterOperatorStartsWith: 'Commence par',
              filterOperatorEndsWith: 'Finit par',
              filterOperatorIsEmpty: 'Est vide',
              filterOperatorIsNotEmpty: 'N\'est pas vide',
              filterOperatorIsAnyOf: 'Est l\'un de',
              columnMenuLabel: 'Menu',
              columnMenuShowColumns: 'Afficher les colonnes',
              columnMenuManageColumns: 'Gérer les colonnes',
              columnMenuFilter: 'Filtrer',
              columnMenuHideColumn: 'Masquer la colonne',
              columnMenuUnsort: 'Annuler le tri',
              columnMenuSortAsc: 'Tri croissant',
              columnMenuSortDesc: 'Tri décroissant'
            }}
            rows={tableData}
            columns={columns}
            loading={ticketsLoading}
            getRowHeight={() => 'auto'}
            pageSizeOptions={[5, 10, 25]}
            initialState={{
              pagination: { paginationModel: { pageSize: 10 } },
              sorting: { sortModel: [{ field: 'updatedAt', sort: 'desc' }] },
            }}
            onRowSelectionModelChange={(newSelectionModel) => setSelectedRowIds(newSelectionModel)}
            columnVisibilityModel={columnVisibilityModel}
            onColumnVisibilityModelChange={(newModel) => setColumnVisibilityModel(newModel)}
            slots={{
              toolbar: CustomToolbarCallback,
              noRowsOverlay: () => <EmptyContent />,
              noResultsOverlay: () => <EmptyContent title="Non Trouvé" />,
            }}
            slotProps={{
              panel: { anchorEl: filterButtonEl },
              toolbar: { setFilterButtonEl },
            }}
            sx={{ [`& .${gridClasses.cell}`]: { alignItems: 'center', display: 'inline-flex' } }}
          />
        </Card>
      </Container>

      <Dialog
        open={openDialog}
        onClose={() => setOpenDialog(false)}
      >
        <DialogTitle>Confirmer l'acceptation</DialogTitle>
        <DialogContent>
          <DialogContentText>
            Êtes-vous sûr de vouloir accepter ce ticket ? Vous pouvez pas le rejeter après tout.
          </DialogContentText>
        </DialogContent>
        <DialogActions>
          <Button onClick={() => setOpenDialog(false)} color="primary">
            Annuler
          </Button>
          <Button onClick={handleConfirmAcceptTicket} color="primary" autoFocus>
            Accepter
          </Button>
        </DialogActions>
      </Dialog>
    </>
  );
}

// ----------------------------------------------------------------------

interface CustomToolbarProps {
  filteredResults: number;
  selectedRowIds: GridRowSelectionModel;
  onOpenConfirmDeleteRows: () => void;
  setFilterButtonEl: React.Dispatch<React.SetStateAction<HTMLButtonElement | null>>;
}

function CustomToolbar({
  selectedRowIds,
  onOpenConfirmDeleteRows,
}: CustomToolbarProps) {
  return (
    <>
      <GridToolbarContainer>
        <GridToolbarQuickFilter />
        <Stack spacing={1} flexGrow={1} direction="row" alignItems="center" justifyContent="flex-end">
          {!!selectedRowIds.length ? (
            <Button
              size="small"
              color="error"
              startIcon={<Iconify icon="solar:trash-bin-trash-bold" />}
              onClick={onOpenConfirmDeleteRows}
            >
              Supprimer ({selectedRowIds.length})
            </Button>
          ): null}
        </Stack>
      </GridToolbarContainer>
    </>
  );
}
