import type { GridCellParams } from '@mui/x-data-grid';

import Box from '@mui/material/Box';
import Link from '@mui/material/Link';
import Stack from '@mui/material/Stack';
import ListItemText from '@mui/material/ListItemText';
import { fTime , fDate } from '@/utils/format-time';
import React from 'react';
import { Menu, MenuItem } from '@mui/material';
import Label from '@/shared/components/label';
import { useGetUserById } from '@/shared/api/ticket';
import { useRouter } from 'next/navigation';
import { paths } from '@/routes/paths';
import toast from 'react-hot-toast';
import { useUpdateTicket } from '@/shared/api/ticket';


// ----------------------------------------------------------------------

type ParamsProps = {
  params: GridCellParams;
};

export function RenderCellStatus({ params }: ParamsProps) {
  const { updateTicket } = useUpdateTicket();

  const categories: { [key: string]: string } = {
    OPEN: 'Ouvert',
    IN_PROGRESS: 'En cours',
    RESOLVED: 'Résolu',
    REJECTEED: 'Rejeté',
    WAITING: 'En attente',
    REASSIGNED: 'Réassigné',
  };

  const [anchorEl, setAnchorEl] = React.useState<null | HTMLElement>(null);

  const handleOpenMenu = (event: React.MouseEvent<HTMLSpanElement>) => {
    setAnchorEl(event.currentTarget);
  };

  const handleCloseMenu = () => {
    setAnchorEl(null);
  };

  const handleStatusChange = async (statusKey: string) => {
    await updateTicket(params.row.id, {
      ...params.row,
      ticketstatus: statusKey,
    });
    toast.success(`Statut du ticket mis à jour: ${categories[statusKey]}`);
    handleCloseMenu();
  };

  return (
    <>
      <Label
        variant="soft"
        color={
          (params.row.ticketstatus === 'OPEN' && 'info') ||
          (params.row.ticketstatus === 'IN_PROGRESS' && 'warning') ||
          (params.row.ticketstatus === 'WAITING' && 'success') ||
          (params.row.ticketstatus === 'REASSIGNED' && 'warning') ||
          (params.row.ticketstatus === 'RESOLVED' && 'primary') ||
          (params.row.ticketstatus === 'REJECTEED' && 'error') ||
          'default'
        }
        sx={{ 
          width: '100%', 
          maxWidth: '220px',
          cursor: 'pointer',
          textAlign: 'center',
          overflow: 'hidden',
          textOverflow: 'ellipsis',
          whiteSpace: 'nowrap',
          px: 1
        }}
        onClick={handleOpenMenu}
      >
        {categories[params.row.ticketstatus]}
      </Label>
      <Menu
        anchorEl={anchorEl}
        open={Boolean(anchorEl)}
        onClose={handleCloseMenu}
        anchorOrigin={{ vertical: 'bottom', horizontal: 'left' }}
        transformOrigin={{ vertical: 'top', horizontal: 'left' }}
        PaperProps={{
          sx: {
            maxWidth: '100%',
            width: { xs: '180px', sm: '220px' }
          }
        }}
      >
        {Object.entries(categories).map(([key, value]) => (
          <MenuItem 
            key={key} 
            onClick={() => handleStatusChange(key)}
            sx={{
              overflow: 'hidden',
              textOverflow: 'ellipsis',
              whiteSpace: 'nowrap'
            }}
          >
            {value}
          </MenuItem>
        ))}
      </Menu>
    </>
  );
}

export function RenderCellCreatedAt({ params }: ParamsProps) {
  return (
    <Stack spacing={0.5}>
      <Box component="span">{fDate(params.row.createdAt)}</Box>
      <Box component="span" sx={{ typography: 'caption', color: 'text.secondary' }}>
        {fTime(params.row.createdAt)}
      </Box>
    </Stack>
  );
}

export function RenderCellUpdatedAt({ params }: ParamsProps) {
  return (
    <Stack spacing={0.5}>
      <Box component="span">{fDate(params.row.updatedAt)}</Box>
      <Box component="span" sx={{ typography: 'caption', color: 'text.secondary' }}>
        {fTime(params.row.updatedAt)}
      </Box>
    </Stack>
  );
}

export function RenderCellAssignedAt({ params }: ParamsProps) {
  return (
    <>
      {params.row.support_id ? (
        <Stack spacing={0.5}>
          <Box component="span">{fDate(params.row.updatedAt)}</Box>
          <Box component="span" sx={{ typography: 'caption', color: 'text.secondary' }}>
            {fTime(params.row.updatedAt)}
          </Box>
        </Stack>
      ) : (
        <Stack spacing={0.5}>
          <Box component="span">Non assigné</Box>
        </Stack>
      )}
    </>
  );
}

export function RenderCellTicketTitle({ params }: ParamsProps) {
  const router = useRouter();

  return (
    <Stack
      direction="row"
      alignItems="center"
      sx={{
        py: { xs: 1, sm: 2 },
        width: "100%",
        flexGrow: 1,
      }}
    >
      <ListItemText
        disableTypography
        primary={
          <Link
            noWrap
            color="inherit"
            onClick={() =>
              router.push(
                paths.dashboard.chatbot.ticket.view(params.row.id)
              )
            }
            sx={{
              cursor: "pointer",
              typography: { xs: "body2", sm: "subtitle2" },
              overflow: "hidden",
              textOverflow: "ellipsis",
              whiteSpace: "nowrap",
              display: "block",
              maxWidth: { xs: 120, sm: 200, md: 300 },
            }}
          >
            {params.row.title}
          </Link>
        }
        sx={{
          display: "flex",
          flexDirection: "column",
          px: { xs: 1, sm: 4 },
          width: "100%",
        }}
      />
    </Stack>
  );
}

export function RenderCellTicketAsigned({params,}: ParamsProps) {
  const { user } = params.row.support_id ? useGetUserById(params.row.support_id) : { user: null };
  return (
    <Stack direction="row" alignItems="center" sx={{ py: 2, width: 1 }}>
      <ListItemText
        disableTypography
        primary={
          <Link
            noWrap
            color="inherit"
            variant="subtitle2"
            sx={{ cursor: 'pointer' }}
          >
            {params.row.support_id ? user?.firstName + ' ' + user?.lastName : 'Non assigné'}
          </Link>
        }
        sx={{ display: 'flex', flexDirection: 'column' }}
      />
    </Stack>
  );
}