"use client";
import React, { useRef, useState, useEffect } from "react";
import {
    Box,
    CircularProgress,
    Typography,
    Card,
    CardContent,
    CardHeader,
    Divider,
    Avatar,
    Grid,
    Chip,
    Menu,
    MenuItem,
    Paper,
    Tooltip,
    Badge,
    IconButton,
    LinearProgress,
    useTheme
} from "@mui/material";

import {
    Timeline,
    TimelineItem,
    TimelineSeparator,
    TimelineConnector,
    TimelineContent,
    TimelineDot,
    TimelineOppositeContent
} from '@mui/lab';

import { 
    AccessTime as AccessTimeIcon,
    PersonOutline as PersonIcon,
    Flag as PriorityIcon,
    Category as CategoryIcon,
    Update as UpdateIcon,
    Create as CreateIcon,
    Chat as ChatIcon,
    MoreVert as MoreVertIcon,
    Email as EmailIcon,
    Assignment as AssignmentIcon
} from "@mui/icons-material";
import { ChatEmptyState } from "../../newchat/chat-empty-state";
import { ChatMessageItem } from "../../newchat/chat-message-item";
import { useGetTicketById, useUpdateTicket } from "@/shared/api/ticket";
import { useGetConversation } from "@/shared/api/chat-bot";
import Scrollbar from "@/shared/components/scrollbar";
import toast from "react-hot-toast";
import { IMessageItem } from "@/shared/types/chat-bot";
import { m } from "framer-motion";
import { CheckCircleIcon } from "lucide-react";


interface TicketHistoryDataViewProps {
    ticket_id: string;
}

export function TicketHistoryDataView({ ticket_id }: TicketHistoryDataViewProps) {
    const { ticket, ticketLoading } = useGetTicketById(ticket_id);
    const { updateTicket } = useUpdateTicket();
    const { conversation, conversationLoading } = useGetConversation(ticket?.conversation_id);
    console.log('conversation', conversation);
    const scrollbarsRef = useRef(null);
    const theme = useTheme();
    const [isScrolledToBottom, setIsScrolledToBottom] = useState(true);

    // Define status categories
    const categories: { [key: string]: { label: string, color: string, bgColor: string } } = {
        OPEN: { label: "Ouvert", color: "#2196f3", bgColor: "rgba(33, 150, 243, 0.1)" },
        IN_PROGRESS: { label: "En cours", color: "#ff9800", bgColor: "rgba(255, 152, 0, 0.1)" },
        RESOLVED: { label: "Résolu", color: "#4caf50", bgColor: "rgba(76, 175, 80, 0.1)" },
        REJECTEED: { label: "Rejeté", color: "#f44336", bgColor: "rgba(244, 67, 54, 0.1)" },
        WAITING: { label: "En attente", color: "#9c27b0", bgColor: "rgba(156, 39, 176, 0.1)" },
        REASSIGNED: { label: "Réassigné", color: "#ff5722", bgColor: "rgba(255, 87, 34, 0.1)" }
    };

    // For changing ticket status
    const [anchorEl, setAnchorEl] = useState<null | HTMLElement>(null);

    const handleOpenMenu = (event: React.MouseEvent<HTMLDivElement>) => {
        setAnchorEl(event.currentTarget);
    };

    const handleCloseMenu = () => {
        setAnchorEl(null);
    };

    const handleStatusChange = async (statusKey: string) => {
        if (ticket) {
            await updateTicket(ticket.id, {
                ...ticket,
                ticketstatus: statusKey
            });
            toast.success(`Statut du ticket mis à jour: ${categories[statusKey].label}`);
        }
        handleCloseMenu();
    };

    const getStatusProgress = (status: string): number => {
        const progressMap: { [key: string]: number } = {
            OPEN: 20,
            WAITING: 40,
            IN_PROGRESS: 60,
            REASSIGNED: 70,
            RESOLVED: 100,
            REJECTEED: 100
        };
        return progressMap[status] || 0;
    };

    const priorityLevels = ['Faible', 'Moyenne', 'Haute', 'Urgente'];
    const priorityLevel = ticket?.priority || 1;

    if (ticketLoading) {
        return (
            <Box sx={{ 
                display: "flex", 
                flexDirection: "column",
                justifyContent: "center", 
                alignItems: "center", 
                height: "100vh" 
            }}>
                <CircularProgress size={60} thickness={4} />
                <Typography variant="h6" sx={{ mt: 2 }}>
                    Chargement des détails du ticket...
                </Typography>
            </Box>
        );
    }

    return (
        <Box component={m.div} 
            initial={{ opacity: 0 }}
            animate={{ opacity: 1 }}
            transition={{ duration: 0.5 }}
            sx={{ 
                p: 3, 
                minHeight: "100vh",
                background: `linear-gradient(to bottom, ${theme.palette.background.default}, ${theme.palette.grey[50]})` 
            }}
        >
            <Grid container spacing={3}>
                <Grid item xs={12} md={8}>
                    <Card 
                        component={m.div} 
                        initial={{ y: 20 }}
                        animate={{ y: 0 }}
                        transition={{ delay: 0.1 }}
                        sx={{ 
                            mb: 3, 
                            borderRadius: 2,
                            boxShadow: '0 8px 24px rgba(0,0,0,0.1)',
                            overflow: 'visible'
                        }}
                    >
                        <CardHeader
                            avatar={
                                <Avatar 
                                    sx={{ 
                                        bgcolor: theme.palette.primary.main, 
                                        width: 64, 
                                        height: 64, 
                                        fontSize: 28,
                                        boxShadow: '0 4px 12px rgba(0,0,0,0.15)'
                                    }}
                                >
                                    {ticket?.title?.charAt(0) || "T"}
                                </Avatar>
                            }
                            title={
                                <Typography variant="h4" fontWeight="bold">
                                    {ticket?.title}
                                </Typography>
                            }
                            subheader={
                                <Box sx={{ mt: 1, display: 'flex', alignItems: 'center' }}>
                                    <Tooltip title="ID du ticket">
                                        <Chip 
                                            icon={<AssignmentIcon />}
                                            label={`#${ticket_id.substring(0, 8)}`}
                                            size="small"
                                            sx={{ mr: 1, fontFamily: 'monospace' }}
                                        />
                                    </Tooltip>
                                    <Tooltip title="Statut actuel">
                                        <Chip
                                            label={categories[ticket?.ticketstatus || "OPEN"].label}
                                            onClick={handleOpenMenu}
                                            sx={{ 
                                                cursor: "pointer",
                                                bgcolor: categories[ticket?.ticketstatus || "OPEN"].bgColor,
                                                color: categories[ticket?.ticketstatus || "OPEN"].color,
                                                fontWeight: 'bold',
                                                '&:hover': {
                                                    boxShadow: '0 2px 8px rgba(0,0,0,0.15)'
                                                }
                                            }}
                                        />
                                    </Tooltip>
                                </Box>
                            }
                        
                        />
                        <Menu
                            anchorEl={anchorEl}
                            open={Boolean(anchorEl)}
                            onClose={handleCloseMenu}
                            anchorOrigin={{ vertical: "bottom", horizontal: "left" }}
                            transformOrigin={{ vertical: "top", horizontal: "left" }}
                            slotProps={{
                                paper: {
                                    elevation: 3,
                                    sx: { borderRadius: 2, minWidth: 200 }
                                }
                            }}
                        >
                            {Object.entries(categories).map(([key, value]) => (
                                <MenuItem 
                                    key={key} 
                                    onClick={() => handleStatusChange(key)}
                                    sx={{ 
                                        py: 1.5,
                                        '&:hover': { 
                                            bgcolor: value.bgColor,
                                            color: value.color 
                                        }
                                    }}
                                >
                                    <Box sx={{ 
                                        width: 12, 
                                        height: 12, 
                                        borderRadius: '50%', 
                                        bgcolor: value.color,
                                        mr: 1.5
                                    }} />
                                    {value.label}
                                </MenuItem>
                            ))}
                        </Menu>
                        <CardContent>
                            <Box sx={{ mb: 3 }}>
                                <LinearProgress 
                                    variant="determinate" 
                                    value={getStatusProgress(ticket?.ticketstatus || "OPEN")} 
                                    sx={{ 
                                        height: 8, 
                                        borderRadius: 4,
                                        bgcolor: 'rgba(0,0,0,0.05)',
                                        '& .MuiLinearProgress-bar': {
                                            bgcolor: categories[ticket?.ticketstatus || "OPEN"].color
                                        }
                                    }}
                                />
                                <Typography variant="caption" sx={{ mt: 0.5, display: 'block', textAlign: 'right' }}>
                                    {getStatusProgress(ticket?.ticketstatus || "OPEN")}% complet
                                </Typography>
                            </Box>

                            <Grid container spacing={3}>
                                <Grid item xs={12} md={6}>
                                    <Paper 
                                        elevation={0} 
                                        sx={{ 
                                            p: 2, 
                                            bgcolor: 'rgba(0,0,0,0.02)', 
                                            borderRadius: 2,
                                            height: '100%'
                                        }}
                                    >
                                        <Typography variant="subtitle2" color="textSecondary">
                                            <AccessTimeIcon fontSize="small" sx={{ verticalAlign: 'middle', mr: 1 }} />
                                            Chronologie
                                        </Typography>
                                        <Box sx={{ mt: 1.5 }}>
                                            <Box sx={{ display: 'flex', alignItems: 'center', mb: 1 }}>
                                                <CreateIcon fontSize="small" color="primary" sx={{ mr: 1 }} />
                                                <Typography variant="body2">
                                                    <strong>Créé le:</strong> {new Date(ticket?.createdAt).toLocaleString()}
                                                </Typography>
                                            </Box>
                                            <Box sx={{ display: 'flex', alignItems: 'center' }}>
                                                <UpdateIcon fontSize="small" color="primary" sx={{ mr: 1 }} />
                                                <Typography variant="body2">
                                                    <strong>Mis à jour le:</strong> {new Date(ticket?.updatedAt).toLocaleString()}
                                                </Typography>
                                            </Box>
                                            {ticket?.resolvedAt && (
                                                <Box sx={{ display: 'flex', alignItems: 'center', mt: 1 }}>
                                                    <CheckCircleIcon fontSize="small" color="success" />
                                                    <Typography variant="body2">
                                                        <strong>Résolu le:</strong> {new Date(ticket?.resolvedAt).toLocaleString()}
                                                    </Typography>
                                                </Box>
                                            )}
                                        </Box>
                                    </Paper>
                                </Grid>
                                <Grid item xs={12} md={6}>
                                    <Paper 
                                        elevation={0} 
                                        sx={{ 
                                            p: 2, 
                                            bgcolor: 'rgba(0,0,0,0.02)', 
                                            borderRadius: 2,
                                            height: '100%'
                                        }}
                                    >
                                        <Typography variant="subtitle2" color="textSecondary">
                                            <CategoryIcon fontSize="small" sx={{ verticalAlign: 'middle', mr: 1 }} />
                                            Informations
                                        </Typography>
                                        <Box sx={{ mt: 1.5 }}>
                                            <Box sx={{ display: 'flex', alignItems: 'center', mb: 1 }}>
                                                <PersonIcon fontSize="small" color="primary" sx={{ mr: 1 }} />
                                                <Typography variant="body2">
                                                    <strong>Demandeur:</strong> {ticket?.requester_email || "Non spécifié"}
                                                </Typography>
                                            </Box>
                                            <Box sx={{ display: 'flex', alignItems: 'center', mb: 1 }}>
                                                <PersonIcon fontSize="small" color="primary" sx={{ mr: 1 }} />
                                                <Typography variant="body2">
                                                    <strong>Assigné à:</strong> {ticket?.assignee_email || "Non assigné"}
                                                </Typography>
                                            </Box>
                                            <Box sx={{ display: 'flex', alignItems: 'center' }}>
                                                <PriorityIcon fontSize="small" color={
                                                    priorityLevel === 4 ? "error" : 
                                                    priorityLevel === 3 ? "warning" : 
                                                    priorityLevel === 2 ? "info" : "success"
                                                } sx={{ mr: 1 }} />
                                                <Typography variant="body2">
                                                    <strong>Priorité:</strong>{" "}
                                                    <Chip
                                                        label={priorityLevels[priorityLevel-1]}
                                                        size="small"
                                                        sx={{
                                                            bgcolor: priorityLevel === 4 ? "error.light" : 
                                                                priorityLevel === 3 ? "warning.light" : 
                                                                priorityLevel === 2 ? "info.light" : "success.light",
                                                            color: priorityLevel === 4 ? "error.dark" : 
                                                                priorityLevel === 3 ? "warning.dark" : 
                                                                priorityLevel === 2 ? "info.dark" : "success.dark",
                                                        }}
                                                    />
                                                </Typography>
                                            </Box>
                                        </Box>
                                    </Paper>
                                </Grid>
                            </Grid>

                            {ticket?.description && (
                                <Paper 
                                    elevation={0}
                                    sx={{ 
                                        p: 2, 
                                        mt: 3, 
                                        bgcolor: 'rgba(0,0,0,0.02)', 
                                        borderRadius: 2,
                                        borderLeft: `4px solid ${theme.palette.primary.main}`
                                    }}
                                >
                                    <Typography variant="subtitle2" color="textSecondary" gutterBottom>
                                        Description du problème
                                    </Typography>
                                    <Typography variant="body1">
                                        {ticket.description}
                                    </Typography>
                                </Paper>
                            )}
                        </CardContent>
                    </Card>

                    <Card 
                        component={m.div} 
                        initial={{ y: 20, opacity: 0 }}
                        animate={{ y: 0, opacity: 1 }}
                        transition={{ delay: 0.2 }}
                        sx={{ 
                            borderRadius: 2,
                            boxShadow: '0 8px 24px rgba(0,0,0,0.1)',
                            overflow: 'hidden'
                        }}
                    >
                        <CardHeader
                            avatar={
                                <Badge 
                                    badgeContent={conversation?.messages?.length || 0} 
                                    color="primary"
                                    max={99}
                                >
                                    <Avatar sx={{ bgcolor: theme.palette.info.main }}>
                                        <ChatIcon />
                                    </Avatar>
                                </Badge>
                            }
                            title={
                                <Typography variant="h6">
                                    Conversation
                                </Typography>
                            }
                            subheader={
                                conversation?.messages?.length 
                                    ? `${conversation.messages.length} message${conversation.messages.length > 1 ? 's' : ''}`
                                    : "Aucun message"
                            }
                        />
                        <Divider />
                        <Box sx={{ height: 500, position: 'relative' }}>
                            {conversationLoading && (
                                <LinearProgress sx={{ position: 'absolute', top: 0, left: 0, right: 0 }} />
                            )}
                            <Scrollbar
                                ref={scrollbarsRef}
                                sx={{ 
                                    px: 3, 
                                    pt: 2, 
                                    pb: 2, 
                                    height: '100%',
                                    bgcolor: theme.palette.background.default
                                }}
                                onScroll={(e: any) => {
                                    const { scrollTop, scrollHeight, clientHeight } = e.target;
                                    const isBottom = scrollHeight - scrollTop - clientHeight < 10;
                                    setIsScrolledToBottom(isBottom);
                                }}
                            >
                                {conversationLoading ? (
                                    <Box
                                        sx={{
                                            display: "flex",
                                            justifyContent: "center",
                                            alignItems: "center",
                                            height: "100%"
                                        }}
                                    >
                                        <CircularProgress />
                                    </Box>
                                ) : conversation?.messages?.length ? (
                                    conversation.messages.map((message: IMessageItem, index: number) => (
                                        <ChatMessageItem 
                                            key={index} 
                                            message={message} 
                                        />
                                    ))
                                ) : (
                                    <ChatEmptyState
                                        title="Aucune conversation"
                                        description="Il n'y a pas encore de messages dans cette conversation."
                                        imgUrl="/assets/icons/empty/ic-content.svg"
                                    />
                                )}
                            </Scrollbar>
                        </Box>
                    </Card>
                </Grid>
                
                <Grid item xs={12} md={4}>
                    <Card 
                        component={m.div}
                        initial={{ x: 20, opacity: 0 }}
                        animate={{ x: 0, opacity: 1 }}
                        transition={{ delay: 0.3 }}
                        sx={{ 
                            borderRadius: 2,
                            boxShadow: '0 8px 24px rgba(0,0,0,0.1)',
                            position: 'sticky',
                            top: 24
                        }}
                    >
                        <CardHeader
                            title="Historique du ticket"
                            avatar={
                                <Avatar sx={{ bgcolor: theme.palette.success.main }}>
                                    <AccessTimeIcon />
                                </Avatar>
                            }
                        />
                        <Divider />
                        <CardContent sx={{ p: 0 }}>
                            <Timeline position="right" sx={{ p: 0, m: 0 }}>
                                <TimelineItem>
                                    <TimelineOppositeContent color="text.secondary" sx={{ maxWidth: 120 }}>
                                        {new Date(ticket?.createdAt).toLocaleDateString()}
                                    </TimelineOppositeContent>
                                    <TimelineSeparator>
                                        <TimelineDot color="primary">
                                            <CreateIcon fontSize="small" />
                                        </TimelineDot>
                                        <TimelineConnector />
                                    </TimelineSeparator>
                                    <TimelineContent>
                                        <Typography variant="body2" component="span">
                                            <strong>Ticket créé</strong>
                                        </Typography>
                                        <Typography variant="caption" display="block">
                                            {new Date(ticket?.createdAt).toLocaleTimeString()}
                                        </Typography>
                                    </TimelineContent>
                                </TimelineItem>
                                
                                {ticket?.status_history?.map((history: any, index: number) => (
                                    <TimelineItem key={index}>
                                        <TimelineOppositeContent color="text.secondary" sx={{ maxWidth: 120 }}>
                                            {new Date(history.date).toLocaleDateString()}
                                        </TimelineOppositeContent>
                                        <TimelineSeparator>
                                            <TimelineDot 
                                                sx={{ 
                                                    bgcolor: categories[history.status]?.color || 'grey'
                                                }}
                                            />
                                            <TimelineConnector />
                                        </TimelineSeparator>
                                        <TimelineContent>
                                            <Typography variant="body2" component="span">
                                                <strong>Statut changé en {categories[history.status]?.label}</strong>
                                            </Typography>
                                            <Typography variant="caption" display="block">
                                                {new Date(history.date).toLocaleTimeString()}
                                            </Typography>
                                        </TimelineContent>
                                    </TimelineItem>
                                )) || (
                                    <TimelineItem>
                                        <TimelineOppositeContent color="text.secondary" sx={{ maxWidth: 120 }}>
                                            {new Date(ticket?.updatedAt).toLocaleDateString()}
                                        </TimelineOppositeContent>
                                        <TimelineSeparator>
                                            <TimelineDot color="info">
                                                <UpdateIcon fontSize="small" />
                                            </TimelineDot>
                                            <TimelineConnector />
                                        </TimelineSeparator>
                                        <TimelineContent>
                                            <Typography variant="body2" component="span">
                                                <strong>Statut actuel: {categories[ticket?.ticketstatus || "OPEN"].label}</strong>
                                            </Typography>
                                            <Typography variant="caption" display="block">
                                                {new Date(ticket?.updatedAt).toLocaleTimeString()}
                                            </Typography>
                                        </TimelineContent>
                                    </TimelineItem>
                                )}
                                
                                {ticket?.resolvedAt && (
                                    <TimelineItem>
                                        <TimelineOppositeContent color="text.secondary" sx={{ maxWidth: 120 }}>
                                            {new Date(ticket.resolvedAt).toLocaleDateString()}
                                        </TimelineOppositeContent>
                                        <TimelineSeparator>
                                            <TimelineDot color="success" />
                                        </TimelineSeparator>
                                        <TimelineContent>
                                            <Typography variant="body2" component="span">
                                                <strong>Ticket résolu</strong>
                                            </Typography>
                                            <Typography variant="caption" display="block">
                                                {new Date(ticket.resolvedAt).toLocaleTimeString()}
                                            </Typography>
                                        </TimelineContent>
                                    </TimelineItem>
                                )}
                            </Timeline>
                        </CardContent>
                    </Card>
                    
                    {ticket?.related_tickets && ticket.related_tickets.length > 0 && (
                        <Card 
                            component={m.div}
                            initial={{ x: 20, opacity: 0 }}
                            animate={{ x: 0, opacity: 1 }}
                            transition={{ delay: 0.4 }}
                            sx={{ 
                                mt: 3, 
                                borderRadius: 2,
                                boxShadow: '0 8px 24px rgba(0,0,0,0.1)'
                            }}
                        >
                            <CardHeader
                                title="Tickets associés"
                                avatar={
                                    <Avatar sx={{ bgcolor: theme.palette.warning.main }}>
                                        <EmailIcon />
                                    </Avatar>
                                }
                            />
                            <Divider />
                            <CardContent>
                                {ticket.related_tickets.map((relatedTicket: any, index: number) => (
                                    <Box 
                                        key={index}
                                        sx={{ 
                                            p: 1.5, 
                                            mb: 1, 
                                            borderRadius: 1,
                                            bgcolor: 'background.paper',
                                            boxShadow: '0 2px 8px rgba(0,0,0,0.05)',
                                            '&:hover': {
                                                bgcolor: 'action.hover'
                                            }
                                        }}
                                    >
                                        <Typography variant="subtitle2">
                                            {relatedTicket.title}
                                        </Typography>
                                        <Box sx={{ display: 'flex', alignItems: 'center', mt: 1 }}>
                                            <Chip
                                                size="small"
                                                label={categories[relatedTicket.status]?.label || "Inconnu"}
                                                sx={{ 
                                                    bgcolor: categories[relatedTicket.status]?.bgColor || 'grey.100',
                                                    color: categories[relatedTicket.status]?.color || 'grey.700',
                                                    mr: 1
                                                }}
                                            />
                                            <Typography variant="caption" color="text.secondary">
                                                #{relatedTicket.id.substring(0, 8)}
                                            </Typography>
                                        </Box>
                                    </Box>
                                ))}
                            </CardContent>
                        </Card>
                    )}
                </Grid>
            </Grid>
        </Box>
    );
}