'use client';

import type {
  GridColDef,
  GridRowSelectionModel,
  GridColumnVisibilityModel, 
} from '@mui/x-data-grid';

import { useState, useEffect, useCallback } from 'react';

import Card from '@mui/material/Card';
import Stack from '@mui/material/Stack';
import Button from '@mui/material/Button';
import Dialog from '@mui/material/Dialog';
import DialogActions from '@mui/material/DialogActions';
import DialogContent from '@mui/material/DialogContent';
import DialogTitle from '@mui/material/DialogTitle';
import MenuItem from '@mui/material/MenuItem';
import Select from '@mui/material/Select';

import {
  DataGrid,
  gridClasses,
  GridActionsCellItem,
  GridToolbarContainer,
  GridToolbarQuickFilter,
  GridToolbarFilterButton
} from '@mui/x-data-grid';

import { useBoolean } from '@/hooks';
import Container from "@mui/material/Container";
import { ChatUser, ISupportAgent } from '@/shared/types/user';

import toast from 'react-hot-toast';
import Iconify from '@/components/iconify';
import EmptyContent from '@/shared/components/empty-content';
import CustomBreadcrumbs from '@/shared/components/custom-breadcrumbs';
import { paths } from '@/routes/paths';
import { useRouter } from '@/hooks';

import {
  RenderCellTicketTitle,
  RenderCellTicketAsigned,
  RenderCellCreatedAt,
  RenderCellUpdatedAt,
  RenderCellStatus,
  RenderCellAssignedAt
} from '../ticket-table-row';

import { 
  useGetTickets,
  useGetSupportUsers,
  useUpdateTicket
} from '@/shared/api/ticket';
import { ITiketItem } from '@/shared/types/ticket';
import { useWebSocket } from '@/hooks/use-web-socket';
import { TicketStatistics } from '../ticket-status-statistic';
import { getCurrentChatUser } from '@/shared/api/chat-bot';

const HIDE_COLUMNS = { category: false }; 
const HIDE_COLUMNS_TOGGLABLE = ['category', 'actions'];

// ----------------------------------------------------------------------

export function TicketListView() {
  const confirmRows = useBoolean();

  const [userData, setUserData] = useState<ChatUser | null>(null);

  useEffect(() => {
      async function fetchUser() {
        const token = localStorage.getItem('chatbotToken');
        if (token) {
          const data = await getCurrentChatUser(token);
          setUserData(data);
        }
      }
      fetchUser();
    }, []);

  const user  = userData || {};
  const userid = user?.id;

  const { tickets, ticketsLoading } = useGetTickets();
  const { support, supportLoading } = useGetSupportUsers();
  const { updateTicket } = useUpdateTicket();
  
  const router = useRouter();

  const { messages, sendMessage } = useWebSocket(userid);

  const [tableData, setTableData] = useState<ITiketItem[]>([]);
  const [selectedRowIds, setSelectedRowIds] = useState<GridRowSelectionModel>([]);
  const [filterButtonEl, setFilterButtonEl] = useState<HTMLButtonElement | null>(null);
  const [columnVisibilityModel, setColumnVisibilityModel] = useState<GridColumnVisibilityModel>(HIDE_COLUMNS);
  const [openDialog, setOpenDialog] = useState(false);
  const [selectedSupportAgent, setSelectedSupportAgent] = useState<string>('');
  const [currentTicketId, setCurrentTicketId] = useState<string | null>(null);
  
  useEffect(() => {
    if (tickets.length) {
      setTableData(tickets);
    }
  }, [tickets]);

  const handleEditRow = useCallback(
    (id: string) => {
      setCurrentTicketId(id);
      setOpenDialog(true);
    },
    []
  );

  const handleAssign = async () => {
      if (currentTicketId) {
        const ticketToUpdate = tableData.find(ticket => ticket.id === currentTicketId);
        if (ticketToUpdate) {
           await updateTicket(currentTicketId, { ...ticketToUpdate, support_id: selectedSupportAgent });
        }
        setOpenDialog(false);
        toast.success('Ticket a été assigné avec succès');
      }
    };
  
  const handleAssignToLeader = async (TicketId: string) => {
    if (TicketId) {
      const ticketToUpdate = tableData.find(ticket => ticket.id === TicketId);
      if (ticketToUpdate && user) {
          const response = await updateTicket(TicketId, { 
            ...ticketToUpdate, 
            ticketstatus: 'IN_PROGRESS',
            support_id: user.id });
      }
      setOpenDialog(false);
      toast.success('Ticket a été assigné avec succès');
      router.push(paths.dashboard.chatbot.support.conversation);
    }
  };

  const handleRejectRow = async (TicketId: string) => {
    if (TicketId) {
      const ticketToUpdate = tableData.find(ticket => ticket.id === TicketId);
      if (ticketToUpdate) {
          await updateTicket(TicketId, { 
            ...ticketToUpdate, 
            support_id: user?.id,
            ticketstatus: 'REJECTEED'
          });

          sendMessage({
            ticketId: TicketId || "",
            senderId: user?.id || "",
            content: "Votre ticket a été rejeté , veuillez contacter le support pour plus d'informations",
            owner: "ai",
          });

          await updateTicket(TicketId, {
            ...ticketToUpdate,
            support_id: null,
            ticketstatus: 'REJECTEED',
          });
      }
      toast.success('Ticket a été rejeté avec succès');
    }
  };

  const CustomToolbarCallback = useCallback(
    () => (
      <CustomToolbar
        setFilterButtonEl={setFilterButtonEl}
        data={tableData}
      />
    ),
    [selectedRowIds, tableData]
  );

  const columns: GridColDef[] = [
    {
      field: 'title',
      headerName: 'Titre',
      flex: 1,
      minWidth: 200,
      renderCell: (params) => <RenderCellTicketTitle params={params} />,
    },
    { 
      field: 'ticketstatus', 
      headerName: 'Statut',
      flex: 0.8,
      minWidth: 120, 
      renderCell: (params) => <RenderCellStatus params={params} />,
    },
    {
      field: 'createdAt',
      headerName: 'Date de création',
      flex: 0.8,
      minWidth: 150,
      renderCell: (params) => <RenderCellCreatedAt params={params} />,
    },
    { 
      field: 'support_id', 
      headerName: 'Assigné à',
      flex: 0.8,
      minWidth: 150, 
      renderCell: (params) => <RenderCellTicketAsigned params={params} />,
    },
    {
      field: 'assignedAt',
      headerName: "Date d'assignation",
      flex: 0.8,
      minWidth: 150,
      renderCell: (params) => <RenderCellAssignedAt params={params} />,
    },
    {
      field: 'updatedAt',
      headerName: 'Dernière mise à jour',
      flex: 0.8,
      minWidth: 150,
      renderCell: (params) => <RenderCellUpdatedAt params={params} />,
    },
    {
      type: 'actions',
      field: 'actions',
      headerName: ' ',
      align: 'right',
      headerAlign: 'right',
      width: 80, 
      sortable: false,
      filterable: false,
      disableColumnMenu: true,
      getActions: (params) => {
        if (params.row.ticketstatus !== 'OPEN' && params.row.ticketstatus !== 'REASSIGNED') {
          return [];
        }
        return [
          <GridActionsCellItem
            showInMenu
            icon={<Iconify icon="solar:document-add-broken" />}
            label="Assigner"
            onClick={() => handleEditRow(params.row.id)}
          />,
          <GridActionsCellItem
            showInMenu
            icon={<Iconify icon="solar:document-add-broken" />}
            label="Assigner à moi"
            onClick={() => handleAssignToLeader(params.row.id)}
          />,
          <GridActionsCellItem
            showInMenu
            icon={<Iconify icon="solar:document-add-broken" />}
            label="Rejeté"
            onClick={() => handleRejectRow(params.row.id)}
          />
        ];
      },
    }
  ];

  return (
      <Container
        maxWidth="xl"
        sx={{ flexGrow: 1, display: 'flex', flexDirection: 'column' }}>
        <CustomBreadcrumbs
          heading="Voir Tous Les Tickets"
          links={[{ name: 'Tableau de bord' }, { name: 'Voir Tous Les Tickets' }]}
          sx={{ mb: { xs: 5, md: 5 } }}
        />

        <Card
          sx={{
            flexGrow: { md: 2 },
            display: { md: 'flex' },
            height: { xs: 800, md: 2 }, 
            width: { md: 1 },
            flexDirection: { md: 'column' },
          }}
        >
          <DataGrid
            disableRowSelectionOnClick
            rows={tableData}
            columns={columns}
            loading={ticketsLoading}
            getRowHeight={() => 'auto'}
            pageSizeOptions={[5, 10, 25]}
            initialState={{
              pagination: { paginationModel: { pageSize: 10 } },
              sorting: { sortModel: [{ field: 'updatedAt', sort: 'desc' }] },
            }}
            localeText={{
              toolbarQuickFilterPlaceholder: 'Rechercher...',
              toolbarColumns: "Colonnes",
              toolbarColumnsLabel: "Chercher",
              toolbarFilters: "Filtrer",
              toolbarExport: "Exporter",
              filterPanelOperator: 'Opérateurs',
              filterPanelColumns: 'Colonnes',
              filterPanelInputLabel: 'Valeur',
              filterPanelInputPlaceholder: 'Valeur du filtre',
              filterOperatorContains: 'Contient',
              filterOperatorEquals: 'Égal à',
              filterOperatorStartsWith: 'Commence par',
              filterOperatorEndsWith: 'Finit par',
              filterOperatorIsEmpty: 'Est vide',
              filterOperatorIsNotEmpty: 'N\'est pas vide',
              filterOperatorIsAnyOf: 'Est l\'un de',
              columnMenuLabel: 'Menu',
              columnMenuShowColumns: 'Afficher les colonnes',
              columnMenuManageColumns: 'Gérer les colonnes',
              columnMenuFilter: 'Filtrer',
              columnMenuHideColumn: 'Masquer la colonne',
              columnMenuUnsort: 'Annuler le tri',
              columnMenuSortAsc: 'Tri croissant',
              columnMenuSortDesc: 'Tri décroissant'
            }}
            onRowSelectionModelChange={(newSelectionModel) => setSelectedRowIds(newSelectionModel)}
            columnVisibilityModel={columnVisibilityModel}
            onColumnVisibilityModelChange={(newModel) => setColumnVisibilityModel(newModel)}
            slots={{
              toolbar: CustomToolbarCallback,
              noRowsOverlay: () => <EmptyContent />,
              noResultsOverlay: () => <EmptyContent title="Non Trouvé" />,
            }}
            slotProps={{
              panel: { anchorEl: filterButtonEl },
              toolbar: { setFilterButtonEl },
            }}
            sx={{ [`& .${gridClasses.cell}`]: { alignItems: 'center', display: 'inline-flex' } }}
          />
        </Card>

        <Dialog open={openDialog} onClose={() => setOpenDialog(false)} maxWidth="sm" fullWidth>
          <DialogTitle>Assigner un agent de support</DialogTitle>
          <DialogContent>
            <Select
              fullWidth
              value={selectedSupportAgent}
              onChange={(e) => setSelectedSupportAgent(e.target.value)}
            >
              {support.map((agent: ISupportAgent) => (
                <MenuItem key={agent.id} value={agent.id}>
                  {agent.firstName} {agent.lastName}
                </MenuItem>
              ))}
            </Select>
          </DialogContent>
          <DialogActions>
            <Button onClick={() => setOpenDialog(false)}>Annuler</Button>
            <Button onClick={handleAssign} disabled={!selectedSupportAgent}>Accepter</Button>
          </DialogActions>
        </Dialog>
      </Container>
  );
}

// ----------------------------------------------------------------------

interface CustomToolbarProps {
  setFilterButtonEl: React.Dispatch<React.SetStateAction<HTMLButtonElement | null>>;
  data: ITiketItem[];
}

function CustomToolbar({
  setFilterButtonEl,
  data,
}: CustomToolbarProps) {
  return (
    <GridToolbarContainer>
      <GridToolbarQuickFilter />
      <GridToolbarFilterButton ref={setFilterButtonEl} />
      <TicketStatistics data={data} />
    </GridToolbarContainer>
  );
}