import React, { useCallback, useMemo } from "react";
import Stack from "@mui/material/Stack";
import TextField from "@mui/material/TextField";
import InputAdornment from "@mui/material/InputAdornment";
import Iconify from "../../../../components/iconify";
import { DatePicker } from '@mui/x-date-pickers/DatePicker';
import { ISupplierOrderTableToDriverFilters, ISupplierOrderTableToDriverFilterValue, SupplierDTO } from '@/contexts/types/supplierOrder';
import { Checkbox, InputLabel, Select, ListItemText, MenuItem, FormControl, SelectChangeEvent, OutlinedInput, Grid } from "@mui/material";


type Props = {
  filters: ISupplierOrderTableToDriverFilters;
  onFilters: (name: string, value: ISupplierOrderTableToDriverFilterValue) => void;
  dateError: boolean;
  selectedDate: Date | null;
  setSelectedDate: (date: Date | null) => void;
  supplierOrdersList: SupplierDTO[];
};

export default function SupplierOrderTableToolbar({
  filters,
  onFilters,
  dateError,
  selectedDate,
  setSelectedDate,
  supplierOrdersList,
}: Props) {
  const handleFilterName = useCallback(
    (event: React.ChangeEvent<HTMLInputElement>) => {
      onFilters("name", event.target.value);
    },
    [onFilters]
  );

  const handleFilterDate = useCallback(
    (newValue: Date | null) => {
      setSelectedDate(newValue);
      onFilters("deliveryDate", newValue);
    },
    [onFilters, setSelectedDate]
  );

  const supplierOptions = useMemo(() => {
    const suppliers = Array.from(new Set(
      supplierOrdersList.map(order => order.supplierName || 'Fournisseur non trouvé')  // Ensure that undefined values are handled
    ));
    return suppliers.map(supplier => ({ value: supplier, label: supplier }));
  }, [supplierOrdersList]);

  const isAllSelected = supplierOptions.length > 0 && filters.supplier.length === supplierOptions.length;

  const handleSelectAllClick = useCallback(() => {
    const newValue = isAllSelected ? [] : supplierOptions.map(option => option.value);
    onFilters("supplier", newValue);
  }, [isAllSelected, supplierOptions, onFilters]);

  const handleFilterSupplier = (event: SelectChangeEvent<string[]>) => {
    const value = event.target.value as string[];
      if (value.includes("select-all")) {
      const newValue = isAllSelected ? [] : supplierOptions.map((option) => option.value);
      onFilters("supplier", newValue);
    } else {
      onFilters("supplier", value);
    }
  };
  

  return (
    <Stack
      spacing={2}
      alignItems={{ xs: "flex-end", md: "center" }}
      direction={{
        xs: "column",
        md: "row",
      }}
      sx={{
        p: 2.5,
        pr: { xs: 2.5, md: 1 },
      }}
    >
      <Grid container spacing={2} alignItems="center">
        <Grid item xs={12} md={3}>
          <FormControl fullWidth>
            <InputLabel>Fournisseur</InputLabel>
            <Select
              multiple
              value={filters.supplier}
              onChange={handleFilterSupplier}
              input={<OutlinedInput label="Fournisseur" />}
              renderValue={(selected) => (selected as string[]).join(', ')}
            >
              <MenuItem onClick={handleSelectAllClick} value="select-all">
                <Checkbox
                  checked={isAllSelected}
                  indeterminate={filters.supplier.length > 0 && filters.supplier.length < supplierOptions.length}
                />
                <ListItemText primary="Selectionner Tous" />
              </MenuItem>
              {supplierOptions.map((option) => (
                <MenuItem key={option.value} value={option.value}>
                  <Checkbox checked={filters.supplier.includes(option.value)} />
                  <ListItemText primary={option.label} />
                </MenuItem>
              ))}
            </Select>
          </FormControl>
        </Grid>

        <Grid item xs={12} md={6}>
          <TextField
            fullWidth
            value={filters.name}
            onChange={handleFilterName}
            placeholder="Chercher un fournisseur ou un numéro de commande..."
            InputProps={{
              startAdornment: (
                <InputAdornment position="start">
                  <Iconify
                    icon="eva:search-fill"
                    sx={{ color: "text.disabled" }}
                  />
                </InputAdornment>
              ),
            }}
          />
        </Grid>

        <Grid item xs={12} md={3}>
          <DatePicker
            label="Filtrer par Date de livraison"
            value={selectedDate}
            onChange={handleFilterDate}
            slotProps={{
              textField: {
                fullWidth: true,
                error: dateError,
                helperText: dateError && "Invalid date range",
              },
            }}
          />
        </Grid>
      </Grid>
    </Stack>
  );
}

