import { useCallback } from 'react';
import Box from '@mui/material/Box';
import Chip from '@mui/material/Chip';
import Paper from '@mui/material/Paper';
import Button from '@mui/material/Button';
import Stack, { StackProps } from '@mui/material/Stack';
import Iconify from '../../../../components/iconify';
import { shortDateLabel } from '../../../../components/custom-date-range-picker';
import { ISupplierOrderTableToDriverFilters, ISupplierOrderTableToDriverFilterValue, SupplierOrderStatus, supplierOrderStatusLabels } from '@/contexts/types/supplierOrder';

// ----------------------------------------------------------------------

type Props = StackProps & {
  filters: ISupplierOrderTableToDriverFilters;
  onFilters: (name: string, value: ISupplierOrderTableToDriverFilterValue) => void;
  onResetFilters: VoidFunction;
  results: number;
};

export default function OrderTableFiltersResult({
  filters,
  onFilters,
  onResetFilters,
  results,
  ...other
}: Props) {
  const shortLabel = shortDateLabel(filters.startDate, filters.endDate);

  const handleRemoveKeyword = useCallback(() => {
    onFilters('name', '');
  }, [onFilters]);

  const handleRemoveStatus = useCallback(() => {
    onFilters('status', 'select-all');
  }, [onFilters]);

  const handleRemoveDate = useCallback(() => {
    onFilters('startDate', null);
    onFilters('endDate', null);
  }, [onFilters]);

  const handleRemoveDeliveryDate = useCallback(() => {
    onFilters('deliveryDate', null);
  }, [onFilters]);

  const handleRemoveSupplier = useCallback(() => {
    onFilters('supplier', []);
  }, [onFilters]);

  return (
    <Stack spacing={1.5} {...other}>
      {(results > 0 || filters.supplier.length > 0 || filters.status !== 'select-all' || filters.startDate || filters.endDate || filters.deliveryDate || filters.name) && (
        <Box sx={{ typography: 'body2' }}>
          <strong>{results}</strong>
          <Box component="span" sx={{ color: 'text.secondary', ml: 0.25 }}>
            results found
          </Box>
        </Box>
      )}

      <Stack flexGrow={1} spacing={1} direction="row" flexWrap="wrap" alignItems="center">

        {filters.supplier.length > 0 && (
          <Block label="Supplier:">
            <Chip size="small" label={filters.supplier.join(', ')} onDelete={handleRemoveSupplier} />
          </Block>
        )}

        {filters.status !== 'select-all' && (
          <Block label="Status:">
            <Chip
              size="small"
              label={supplierOrderStatusLabels[filters.status as SupplierOrderStatus]}
              onDelete={handleRemoveStatus}
            />
          </Block>
        )}

        {filters.startDate && filters.endDate && (
          <Block label="Date:">
            <Chip size="small" label={shortLabel} onDelete={handleRemoveDate} />
          </Block>
        )}

        {filters.deliveryDate && (
          <Block label="Date de livraison:">
            <Chip
              size="small"
              label={filters.deliveryDate ? filters.deliveryDate.toDateString() : "No Date Selected"}
              onDelete={filters.deliveryDate ? handleRemoveDeliveryDate : undefined}
            />
          </Block>
        )}

        {!!filters.name && (
          <Block label="Mot-clé:">
            <Chip label={filters.name} size="small" onDelete={handleRemoveKeyword} />
          </Block>
        )}

        {(results > 0 || filters.supplier.length > 0 || filters.status !== 'select-all' || filters.startDate || filters.endDate || filters.deliveryDate || filters.name) && (
          <Button
            color="error"
            onClick={onResetFilters}
            startIcon={<Iconify icon="solar:trash-bin-trash-bold" />}
          >
            Effacer
          </Button>
        )}
      </Stack>
    </Stack>
  );
}

// ----------------------------------------------------------------------

type BlockProps = StackProps & {
  label: string;
};

function Block({ label, children, sx, ...other }: BlockProps) {
  return (
    <Stack
      component={Paper}
      variant="outlined"
      spacing={1}
      direction="row"
      sx={{
        p: 1,
        borderRadius: 1,
        overflow: 'hidden',
        borderStyle: 'dashed',
        ...sx,
      }}
      {...other}
    >
      <Box component="span" sx={{ typography: 'subtitle2' }}>
        {label}
      </Box>

      <Stack spacing={1} direction="row" flexWrap="wrap">
        {children}
      </Stack>
    </Stack>
  );
}
