import React, { useState, useEffect } from 'react';
import Box from '@mui/material/Box';
import Card from '@mui/material/Card';
import IconButton from '@mui/material/IconButton';
import Typography from '@mui/material/Typography';
import MenuItem from '@mui/material/MenuItem';
import Select, { SelectChangeEvent } from '@mui/material/Select';
import Iconify from '../../../../components/iconify';
import { CollectionStatus, collectionStatusLabels, SupplierOrderItem, SupplierOrderStatus } from '@/contexts/types/supplierOrder';
import CardHeader from '@mui/material/CardHeader';
import CardContent from '@mui/material/CardContent';
import InputLabel from '@mui/material/InputLabel';
import FormControl from '@mui/material/FormControl';
import { Collapse, Grid } from '@mui/material';

type Props = {
  items: SupplierOrderItem[];
  status: SupplierOrderStatus;
  onChangeCollectionStatusItem: (itemId: string, newStatus: CollectionStatus) => void;
};

export default function OrderDetailsItems({ items, status, onChangeCollectionStatusItem }: Props) {
  const [selectedSupplierOrderStatus, setSelectedSupplierOrderStatus] = useState<SupplierOrderStatus>();
  const [itemsList, setItemsList] = useState<SupplierOrderItem[]>([]);
  const [expandedItems, setExpandedItems] = useState<{ [key: string]: boolean }>({});

  useEffect(() => {
    setItemsList(items);
  }, [items]);

  useEffect(() => {
    setSelectedSupplierOrderStatus(status);
  }, [status]);

  const handleToggle = (itemId: string) => {
    setExpandedItems((prev) => ({
      ...prev,
      [itemId]: !prev[itemId],
    }));
  };

  const onChangeCollectionStatus = (event: SelectChangeEvent<CollectionStatus>, itemId: string) => {
    const newStatus = event.target.value as CollectionStatus;
    setItemsList((prevItems) =>
      prevItems.map((item) =>
        item.id === itemId ? { ...item, collectionStatus: newStatus } : item
      )
    );
    onChangeCollectionStatusItem(itemId, newStatus);
  };


  return (
    <>
      {itemsList && itemsList.length > 0 && itemsList.map((item) => (
        <Card sx={{ mb: 2 }}>
          <CardHeader
            title={`${item.productId} | ${item.productName}`}
            action={
              <IconButton onClick={() => handleToggle(item.id)}>
                {expandedItems[item.id] ? <Iconify icon="eva:arrow-up-fill" /> : <Iconify icon="eva:arrow-down-fill" />}
              </IconButton>
            }
          />

          {selectedSupplierOrderStatus === SupplierOrderStatus.DELIVERED ? (
            <CardContent>
              <FormControl fullWidth variant="outlined" sx={{ mb: 2 }}>
                <InputLabel>Status de collecte</InputLabel>
                <Select
                  defaultValue={item.collectionStatus}
                  value={item.collectionStatus}
                  onChange={(event) => onChangeCollectionStatus(event as SelectChangeEvent<CollectionStatus>, item.id)}
                  label="Status de collecte"
                >
                  {Object.values(CollectionStatus).map((status) => (
                    <MenuItem key={status} value={status}>
                      {collectionStatusLabels[status]}
                    </MenuItem>
                  ))}
                </Select>
              </FormControl>
            </CardContent>
          ) : (
            <CardContent sx={{ p: 1 }}></CardContent>
          )}
          
          <Collapse in={expandedItems[item.id]} timeout="auto" unmountOnExit>
            <CardContent>
              <Grid container spacing={2}>
                <Grid item xs={12} sm={6} md={4}>
                  <Box display="flex" alignItems="center">
                    <Typography variant="body2" sx={{ fontWeight: 'bold', mr: 1 }}>
                      UGS:
                    </Typography>
                    <Typography variant="body2">{item.productUgs}</Typography>
                  </Box>
                </Grid>
                <Grid item xs={12} sm={6} md={4}>
                  <Box display="flex" alignItems="center">
                    <Iconify icon="ic:round-price" sx={{ mr: 1, color: 'warning.main' }} />
                    <Typography variant="body2" sx={{ fontWeight: 'bold', mr: 1 }}>
                      Quantité:
                    </Typography>
                    <Typography variant="body2">{item.quantity}</Typography>
                  </Box>
                </Grid>
                <Grid item xs={12} sm={6} md={4}>
                  <Box display="flex" alignItems="center">
                    <Iconify icon="ic:round-price" sx={{ mr: 1, color: 'success.main' }} />
                    <Typography variant="body2" sx={{ fontWeight: 'bold', mr: 1 }}>
                      Prix unit:
                    </Typography>
                    <Typography variant="body2">€{item.productPrice.toFixed(2)}</Typography>
                  </Box>
                </Grid>
                {item.comment && (
                  <Grid item xs={12} sm={6} md={8}>
                    <Box display="flex" alignItems="center">
                      <Iconify icon="ic:round-comment" sx={{ mr: 1, color: 'info.main' }} />
                      <Typography variant="body2" sx={{ fontWeight: 'bold', mr: 1 }}>
                        Commentaire:
                      </Typography>
                      <Typography variant="body2">{item.comment}</Typography>
                    </Box>
                  </Grid>
                )}
                <Grid item xs={12} sm={12} md={12}>
                  <Box display="flex" justifyContent="flex-end">
                    <Iconify icon="ic:round-price" sx={{ mr: 1, mt: 3, color: 'success.main' }} />
                    <Typography variant="body2" sx={{ fontWeight: 'bold', mr: 1 }}>
                      Sous total:
                    </Typography>
                    <Typography variant="body2">
                      €{item.totalPrice.toFixed(2)}
                    </Typography>
                  </Box>
                </Grid>
              </Grid>
            </CardContent>
          </Collapse>
        </Card>
      ))}
    </>
  );
}
