import { useState, useEffect } from 'react';
import Stack from '@mui/material/Stack';
import Button from '@mui/material/Button';
import MenuItem from '@mui/material/MenuItem';
import IconButton from '@mui/material/IconButton';
import Typography from '@mui/material/Typography';
import { fDateTime } from '../../../../../utils/format-time';
import Label from '../../../../components/label';
import Iconify from '../../../../components/iconify';
import CustomPopover, { usePopover } from '../../../../components/custom-popover';
import { SupplierOrderStatus, supplierOrderStatusLabels, UpdateCollectionStatusList } from '@/contexts/types/supplierOrder';
import { useSnackbar } from 'notistack';

const mapStatus = (status: string): SupplierOrderStatus => {
  switch (status) {
    case "A_COLLECTER":
      return SupplierOrderStatus.TO_BE_COLLECTED;
    case "RECEPTIONNEE":
      return SupplierOrderStatus.RECEIVED;
    case "LIVREE":
      return SupplierOrderStatus.DELIVERED;
    default:
      return status as SupplierOrderStatus;
  }
};

type Props = {
  status: SupplierOrderStatus;
  backLink: string;
  supplierOrderId: string;
  createdAt: Date;
  deliveryDate: Date;
  onChangeStatus: (newValue: SupplierOrderStatus) => void;
  onMutate: () => void;
  collectionStatusList: UpdateCollectionStatusList[];
  onUpdateOrderStatus: () => void;
  onUpdateItemCollectionStatus: () => void;
  onUpdateAllOrderStatus: () => void;
  setAreStatusesSaved: (newValue: boolean) => void;
};

export default function OrderDetailsToolbar({
  status,
  backLink,
  createdAt,
  deliveryDate,
  supplierOrderId,
  onChangeStatus,
  onMutate,
  collectionStatusList,
  onUpdateOrderStatus,
  onUpdateItemCollectionStatus,
  onUpdateAllOrderStatus,
  setAreStatusesSaved
}: Props) {
  const popover = usePopover();
  const { enqueueSnackbar } = useSnackbar();
  const [mappedStatus, setMappedStatus] = useState<SupplierOrderStatus>();
  const [selectedStatus, setSelectedStatus] = useState<SupplierOrderStatus>();

  useEffect(() => {
    const mappedDefaultStatus = mapStatus(status);
    setMappedStatus(mappedDefaultStatus);
    setSelectedStatus(mappedDefaultStatus);
    setAreStatusesSaved(true);
  }, [status]);

  const handleStatusChange = (newStatus: SupplierOrderStatus) => {
    setSelectedStatus(newStatus);
    onChangeStatus(newStatus);
    if (newStatus !== mappedStatus) {
      setAreStatusesSaved(false);
    }
    popover.onClose();
  };

  const handleConfirmClick = async () => {
    try {
      if (noStatusChanged()) {
        enqueueSnackbar("Aucun status n'a été modifié !", { variant: "error" });
        return;
      }

      if (shouldUpdateOrderAndItems()) {
        onUpdateAllOrderStatus();
        enqueueSnackbar("Status de la commande et des produits mis à jour avec succès !", { variant: "success" });
      } else if (shouldUpdateOrderOnly()) {
        onUpdateOrderStatus();
        enqueueSnackbar("Status de la commande mis à jour avec succès !", { variant: "success" });
      } else if (shouldUpdateItemsOnly()) {
        onUpdateItemCollectionStatus();
        enqueueSnackbar("Status des produits mis à jour avec succès !", { variant: "success" });
      } else if (shouldRevertOrderAndItems()) {
        onUpdateOrderStatus();
        enqueueSnackbar("Status de la commande et des produits réinitialisés avec succès !", { variant: "success" });
      }

      setMappedStatus(selectedStatus);
      setAreStatusesSaved(true);
      onMutate();
    } catch (error) {
      enqueueSnackbar("Failed to update order status:", { variant: "error" });
    }
  };

  const noStatusChanged = () => collectionStatusList.length <= 0 && selectedStatus === mappedStatus;
  const shouldUpdateOrderOnly = () => selectedStatus !== mappedStatus && collectionStatusList.length <= 0;
  const shouldUpdateItemsOnly = () => selectedStatus == SupplierOrderStatus.DELIVERED && collectionStatusList.length > 0;
  const shouldUpdateOrderAndItems = () => selectedStatus !== mappedStatus && 
    selectedStatus == SupplierOrderStatus.DELIVERED && collectionStatusList.length > 0;
  const shouldRevertOrderAndItems = () => selectedStatus !== mappedStatus &&
    (selectedStatus === SupplierOrderStatus.TO_BE_COLLECTED ||
      selectedStatus === SupplierOrderStatus.RECEIVED) &&
    collectionStatusList.length > 0;

  return (
    <>
      <Stack
        spacing={3}
        direction={{ xs: 'column', md: 'row' }}
        sx={{
          mb: { xs: 3, md: 5 },
        }}
      >
        <Stack spacing={1} direction="row" alignItems="flex-start">
          <IconButton href={backLink}>
            <Iconify icon="eva:arrow-ios-back-fill" />
          </IconButton>

          <Stack spacing={0.5}>
            <Stack spacing={1} direction="row" alignItems="center">
              <Typography variant="h4"> Order {supplierOrderId} </Typography>
              <Label
                variant="soft"
                color={
                  (mappedStatus === SupplierOrderStatus.DELIVERED && 'success') ||
                  (mappedStatus === SupplierOrderStatus.TO_BE_COLLECTED && 'warning') ||
                  (mappedStatus === SupplierOrderStatus.RECEIVED && 'info') ||
                  'default'
                }
              >
                {mappedStatus && supplierOrderStatusLabels[mappedStatus]}
              </Label>
            </Stack>

            <Typography variant="body2" sx={{ color: 'text.disabled' }}>
              Date de la Création  : {fDateTime(createdAt)}
            </Typography>
            <Typography variant="body2" sx={{ color: 'text.disabled' }}>
              Date de la Livraison : {fDateTime(deliveryDate)}
            </Typography>
          </Stack>
        </Stack>

        <Stack
          flexGrow={1}
          spacing={1.5}
          direction="row"
          alignItems="center"
          justifyContent="flex-end"
        >
          <Button
            color="inherit"
            variant="outlined"
            endIcon={<Iconify icon="eva:arrow-ios-downward-fill" />}
            onClick={popover.onOpen}
            sx={{ textTransform: 'capitalize' }}
          >
            {selectedStatus && supplierOrderStatusLabels[selectedStatus]}
          </Button>

          <Button
            color="inherit"
            variant="outlined"
            startIcon={<Iconify icon="solar:printer-minimalistic-bold" />}
          >
            Print
          </Button>
          <Button
            color="inherit"
            variant="contained"
            onClick={handleConfirmClick}
            disabled={selectedStatus === mappedStatus && collectionStatusList.length <= 0}
          >
            Valider
          </Button>
        </Stack>
      </Stack>

      <CustomPopover open={popover.open} onClose={popover.onClose} arrow="top-right" sx={{ width: 140 }}>
        {Object.values(SupplierOrderStatus).map((status) => (
          <MenuItem
            key={status}
            selected={status === selectedStatus}
            onClick={() => handleStatusChange(status)}
          >
            {supplierOrderStatusLabels[status]}
          </MenuItem>
        ))}
      </CustomPopover>
    </>
  );
}
