'use client';

import { useState, useCallback, useEffect } from 'react';
import Stack from '@mui/material/Stack';
import Container from '@mui/material/Container';
import Grid from '@mui/material/Unstable_Grid2';
import { useSettingsContext } from '../../../../components/settings';
import OrderDetailsInfo from '../details/order-details-info';
import OrderDetailsItems from '../details/order-details-item';
import OrderDetailsToolbar from '../details/order-details-toolbar';
import OrderComment from '../details/order-comment';
import { paths } from '../../../../../routes/paths';
import { useAuthContext } from '@/hooks';
import { getSupplierOrderById, updateSupplierOrderStatus, updateSupplierOrderItemCollectionStatus, updateAllSupplierOrderStatuses, getAllSupplierOrders } from '@/shared/api/supplierOrders';
import OrderStatusHistory from '../details/order-status-history';
import { UserByEmail } from '@/shared/api/user';
import { IUser } from '@/shared/types/user';
import { SnackbarProvider } from 'notistack';
import { CollectionStatus, SupplierOrderItem, SupplierOrderStatus, UpdateAllStatuses, UpdateCollectionStatuses, UpdateCollectionStatusList, UpdateSupplierOrderStatus } from '@/contexts/types/supplierOrder';
import { SupplierOrder } from '@/contexts/types/supplierOrder';
import { Alert, CircularProgress } from '@mui/material';



type Props = {
  id: string;
};

export default function OrderDetailsView({ id }: Props) {
  const settings = useSettingsContext();
  const { user } = useAuthContext();
  const [supplierOrder, setSupplierOrder] = useState<SupplierOrder>();
  const { supplierOrderData, supplierOrderError, supplierOrderLoading, mutateSupplierOrder } = getSupplierOrderById(id);
  const [supplierOrderStatus, setSupplierOrderStatus] = useState<SupplierOrderStatus>();
  const [supplierOrderItems, setSupplierOrderItems] = useState<SupplierOrderItem[]>();
  const [collectionStatusList, setCollectionStatusList] = useState<UpdateCollectionStatusList[]>([]);
  const [userId, setUserId] = useState<string>("");
  const [areStatusesSaved, setAreStatusesSaved] = useState(true);
  const { mutateSupplierOrders } = getAllSupplierOrders();

  const fetchUserData = async () => {
    const clientEmail = localStorage.getItem("email");
    const response = await UserByEmail(clientEmail!);
    const userData: IUser = response.data;
    if (userData)
      setUserId(userData.id)
  }

  useEffect(() => {
    fetchUserData();
  }, []);

  useEffect(() => {
    if (supplierOrderData) {
      setSupplierOrder(supplierOrderData);
      setSupplierOrderStatus(supplierOrderData.supplierOrderStatus);
      setSupplierOrderItems(supplierOrderData?.supplierOrderItems);
    }
  }, [supplierOrderData]);

  const handleChangeSupplierOrderStatus = useCallback((newValue: SupplierOrderStatus) => {
    setSupplierOrderStatus(newValue);
  }, []);

  const onChangeCollectionStatusItem = (supplierOrderItemId: string, collectionStatus: CollectionStatus) => {
    if (supplierOrderStatus == SupplierOrderStatus.DELIVERED) {
      setCollectionStatusList((prevList) => {
        const itemIndex = prevList.findIndex(item => item.supplierOrderItemId === supplierOrderItemId);

        if (itemIndex !== -1) {
          const updatedList = [...prevList];
          updatedList[itemIndex].collectionStatus = collectionStatus;
          return updatedList;
        } else {
          return [...prevList, { supplierOrderItemId, collectionStatus }];
        }
      });
    }
  };

  const onUpdateOrderStatus = async () => {
    const formData: UpdateSupplierOrderStatus = {
      userId: userId,
      supplierOrderId: supplierOrder?.id!,
      supplierOrderStatus: supplierOrderStatus!
    };
    await updateSupplierOrderStatus(formData);
    mutateSupplierOrder();
    mutateSupplierOrders();
  };

  const onUpdateItemCollectionStatus = async () => {
    const formData: UpdateCollectionStatuses = {
      supplierOrderId: supplierOrder?.id!,
      updateCollectionStatusList: collectionStatusList
    };
    await updateSupplierOrderItemCollectionStatus(formData);
    mutateSupplierOrder();
    mutateSupplierOrders();
  };

  const onUpdateAllOrderStatus = async () => {
    const formData: UpdateAllStatuses = {
      userId: userId,
      supplierOrderId: supplierOrder?.id!,
      supplierOrderStatus: supplierOrderStatus!,
      itemCollectionStatuses: collectionStatusList
    };
    await updateAllSupplierOrderStatuses(formData);
    mutateSupplierOrder();
    mutateSupplierOrders();
  };

  const backLink = user?.role === 'admin'
    ? paths.dashboard.admin.order.all_orders
    : paths.dashboard.driver.listSupplierOrder;

  if (supplierOrderLoading) {
    return <Grid container justifyContent="center" alignItems="center" height="100vh">
      <CircularProgress />
    </Grid>;
  }

  if (supplierOrderError || !supplierOrderData) {
    return <Grid container justifyContent="center" alignItems="center" height="100vh">
      <Alert severity="warning">Problème lors de la récupération de la commande. Veuillez réessayer plus tard.</Alert>
    </Grid>;
  }

  return (
    <SnackbarProvider>
      <Container maxWidth={settings.themeStretch ? false : 'lg'}>
        <OrderDetailsToolbar
          backLink={backLink}
          supplierOrderId={supplierOrder?.id!}
          createdAt={supplierOrder?.createdAt!}
          deliveryDate={supplierOrder?.deliveryDate!}
          status={supplierOrder?.supplierOrderStatus!}
          onChangeStatus={handleChangeSupplierOrderStatus}
          onMutate={mutateSupplierOrder}
          collectionStatusList={collectionStatusList}
          onUpdateOrderStatus={onUpdateOrderStatus}
          onUpdateItemCollectionStatus={onUpdateItemCollectionStatus}
          onUpdateAllOrderStatus={onUpdateAllOrderStatus}
          setAreStatusesSaved={setAreStatusesSaved}
        />

        <Grid container spacing={3}>
          <Grid xs={12} md={7}>
            <Stack spacing={3} direction={{ xs: 'column-reverse', md: 'column' }}>
              <OrderDetailsItems
                items={supplierOrderItems!}
                status={supplierOrderStatus!}
                onChangeCollectionStatusItem={onChangeCollectionStatusItem}
              />
            </Stack>
          </Grid>

          <Grid xs={12} md={5}>
            <Stack spacing={3}>
              <OrderDetailsInfo
                supplierId={supplierOrder?.supplierId!}
              />
              <OrderComment 
                orderId={supplierOrderData.id}
                comment={supplierOrderData.comment!}
                onMutate={mutateSupplierOrder}
                areStatusesSaved={areStatusesSaved}
              />
              <OrderStatusHistory
                history={supplierOrderData?.supplierOrderHistories!} 
              />
            </Stack>
          </Grid>

        </Grid>
      </Container>
    </SnackbarProvider>
  );
}
