'use client';

import { useEffect, useState, useCallback, useMemo } from "react";
import Tab from "@mui/material/Tab";
import Tabs from "@mui/material/Tabs";
import Card from "@mui/material/Card";
import Table from "@mui/material/Table";
import { alpha } from "@mui/material/styles";
import Container from "@mui/material/Container";
import TableBody from "@mui/material/TableBody";
import TableContainer from "@mui/material/TableContainer";
import {
  Grid,
  useTheme,
  CircularProgress,
  Alert
} from "@mui/material";
import { paths } from "../../../../../routes/paths";
import { useRouter } from "../../../../../hooks/use-router";
import { useBoolean } from "../../../../../hooks/use-boolean";
import { isAfter } from "../../../../../utils/format-time";
import Label from "../../../../components/label";
import Scrollbar from "../../../../components/scrollbar";
import { useSnackbar } from "../../../../components/snackbar";
import { useSettingsContext } from "../../../../components/settings";
import CustomBreadcrumbs from "../../../../components/custom-breadcrumbs";
import {
  useTable,
  emptyRows,
  TableNoData,
  getComparator,
  TableEmptyRows,
  TableHeadCustom,
  TablePaginationCustom
} from "../../../../components/table";
import SupplierOrderTableRow from "../all-orders/order-table-row";
import OrderTableToolbar from "../all-orders/SupplierOrderTableToolbar";
import OrderTableFiltersResult from "../all-orders/order-table-filters-result";
import { UserByEmail } from "@/shared/api/user";
import { IUser } from "@/shared/types/user";
import { ISupplierOrderTableToDriverFilters, ISupplierOrderTableToDriverFilterValue, SupplierDTO, SupplierOrder, SupplierOrderStatus, supplierOrderStatusLabels } from "@/contexts/types/supplierOrder";
import { getAllSupplierOrdersByDriver } from "@/shared/api/supplierOrders";
import { useGetAllSuppliers } from "@/shared/api/supplier";

// ----------------------------------------------------------------------

const TABLE_HEAD = [
  { id: "id", label: "Commande", minWidth: 120 },
  { id: "supplierName", label: "Fournisseur", minWidth: 200 },
  { id: "createdAt", label: "Date de Création", minWidth: 140 },
  { id: "deliveryDate", label: "Date de Livraison", minWidth: 140 },
  { id: "supplierFullAddress", label: "Adresse de Livraison", minWidth: 300 },
  { id: "supplierOrderStatus", label: "Status", minWidth: 100 },
  { id: "", label: "", minWidth: 50, align: 'right' },
];

const defaultFilters: ISupplierOrderTableToDriverFilters = {
  name: "",
  status: "all",
  supplier: [],
  startDate: null,
  endDate: null,
  deliveryDate: null,
};

export default function SupplierOrderListDriverView() {
  const { enqueueSnackbar } = useSnackbar();

  const table = useTable({ defaultOrderBy: "id", defaultOrder: "desc" });

  const theme = useTheme();

  const settings = useSettingsContext();

  const router = useRouter();

  const [userId, setUserId] = useState<string>("");

  const fetchUserData = async () => {
    const clientEmail = localStorage.getItem("email");
    const response = await UserByEmail(clientEmail!);
    const userData: IUser = response.data;
    if (userData)
      setUserId(userData.id)
  }

  useEffect(() => {
    fetchUserData();
  }, []);

  const { supplierOrdersData, supplierOrdersError, supplierOrdersLoading, mutateSupplierOrders } = getAllSupplierOrdersByDriver(userId);
  const [supplierOrdersList, setSupplierOrdersList] = useState<SupplierOrder[]>([]);
  const [suppliersList, setSuppliersList] = useState<SupplierDTO[]>([]);
  const { suppliersData } = useGetAllSuppliers();

  useEffect(() => {
    if (suppliersData && suppliersData.length > 0) {
      const suppliersDetails: SupplierDTO[] = suppliersData;
      setSuppliersList(suppliersDetails);
    }
  }, [suppliersData]);

  useEffect(() => {
    setSupplierOrdersList(supplierOrdersData);
  }, [supplierOrdersData]);
  const ConcatFullAddress = (supplier: SupplierDTO): string => {
    return `${supplier.companyStreet}, ${supplier.companyCity}, ${supplier.companyCountry}`;
  };
  
  useEffect(() => {
    const loadSupplierOrders = async () => {
      try {
        const supplierNameMap: { [key: string]: string } = suppliersList.reduce((map, supplier) => {
          map[supplier.id] = supplier.supplierName;
          return map;
        }, {} as { [key: string]: string });
        const supplierPhoneNumberMap: { [key: string]: string } = suppliersList.reduce((map, supplier) => {
          map[supplier.id] = supplier.supplierPhoneNumber;
          return map;
        }, {} as { [key: string]: string });
        const supplierFullAddressMap: { [key: string]: string } = suppliersList.reduce((map, supplier) => {
          map[supplier.id] = ConcatFullAddress(supplier);
          return map;
        }, {} as { [key: string]: string });

        const updatedSupplierOrders = supplierOrdersData.map(order => {
          const supplierName = supplierNameMap[order.supplierId as string];
          const supplierPhoneNumber = supplierPhoneNumberMap[order.supplierId as string];
          const supplierFullAddress = supplierFullAddressMap[order.supplierId as string];
          return {
            ...order,
            supplierName: supplierName || 'Fournisseur non trouvé',
            supplierFullAddress: supplierFullAddress || 'Adresse non trouvée',
            supplierPhoneNumber: supplierPhoneNumber || 'Numéro de téléphone non trouvé',
          };
        });

        setSupplierOrdersList(updatedSupplierOrders);
      } catch (error) {
        enqueueSnackbar('Failed to fetch supplier orders', { variant: 'error' });
      }
    };

    loadSupplierOrders();
  }, [supplierOrdersData, suppliersList]);

  const [filters, setFilters] = useState<ISupplierOrderTableToDriverFilters>(defaultFilters);
  const [selectedDate, setSelectedDate] = useState<Date | null>(null);
  const dateError = isAfter(filters.startDate, filters.endDate);

  useEffect(() => {
    if (supplierOrdersError) {
      enqueueSnackbar("Error fetching data", { variant: "error" });
    }
  }, [supplierOrdersError, enqueueSnackbar]);

  const dataFiltered = useMemo(() => applyFilter({
    inputData: supplierOrdersList,
    comparator: getComparator(table.order, table.orderBy),
    filters,
    dateError,
  }), [supplierOrdersList, table.order, table.orderBy, filters, dateError]);

  const dataInPage = dataFiltered.slice(
    table.page * table.rowsPerPage,
    table.page * table.rowsPerPage + table.rowsPerPage
  );

  const denseHeight = table.dense ? 56 : 76;

  const canReset =
    !!filters.name ||
    filters.status !== "all" ||
    filters.supplier.length > 0 ||
    (!!filters.startDate && !!filters.endDate) ||
    filters.deliveryDate !== null ||
    filters.supplier.length > 0;

  const notFound = (!dataFiltered.length && canReset) || !dataFiltered.length;

  const getSupplierOrdersLength = (status: SupplierOrderStatus) =>
    supplierOrdersList.filter((item) => item.supplierOrderStatus === status).length;

  const TABS = [
    { value: "all", label: "Tous", color: "default", count: supplierOrdersList.length },
    {
      value: SupplierOrderStatus.TO_BE_COLLECTED,
      label: supplierOrderStatusLabels[SupplierOrderStatus.TO_BE_COLLECTED],
      color: "info",
      count: getSupplierOrdersLength(SupplierOrderStatus.TO_BE_COLLECTED),
    },
    {
      value: SupplierOrderStatus.DELIVERED,
      label: supplierOrderStatusLabels[SupplierOrderStatus.DELIVERED],
      color: "success",
      count: getSupplierOrdersLength(SupplierOrderStatus.DELIVERED),
    },
    {
      value: SupplierOrderStatus.RECEIVED,
      label: supplierOrderStatusLabels[SupplierOrderStatus.RECEIVED],
      color: "warning",
      count: getSupplierOrdersLength(SupplierOrderStatus.RECEIVED),
    },
  ] as const;

  const handleFilters = useCallback(
    (name: string, value: ISupplierOrderTableToDriverFilterValue) => {
      table.onResetPage();
      setFilters((prevState) => ({
        ...prevState,
        [name]: value,
      }));
    },
    [table]
  );

  const handleResetFilters = useCallback(() => {
    setFilters(defaultFilters);
    setSelectedDate(null);
  }, []);

  const handleDeleteRow = useCallback(
    (id: string) => {
      const deleteRow = supplierOrdersList.filter((row) => row.id !== id);
      enqueueSnackbar("Delete success!");
      setSupplierOrdersList(deleteRow);
      table.onUpdatePageDeleteRow(dataInPage.length);
    },
    [dataInPage.length, enqueueSnackbar, supplierOrdersList]
  );

  const handleViewRow = useCallback(
    (id: string) => {
      router.push(paths.dashboard.driver.details(id));
    },
    [router]
  );

  const handleFilterStatus = useCallback(
    (event: React.SyntheticEvent, newValue: string) => {
      handleFilters("status", newValue);
    },
    [handleFilters]
  );

  if (supplierOrdersLoading) {
    return <Grid container justifyContent="center" alignItems="center" height="100vh">
      <CircularProgress />
    </Grid>;
  }

  if (supplierOrdersError) {
    return <Grid container justifyContent="center" alignItems="center" height="100vh">
      <Alert severity="warning">Problème lors de la récupération des commandes. Veuillez réessayer plus tard.</Alert>
    </Grid>;
  }

  return (
    <>
      <Container maxWidth={settings.themeStretch ? false : "lg"}>
        <CustomBreadcrumbs
          heading="Liste des commandes Fournisseurs"
          links={[
            {
              name: "Tableau de bord",
              href: paths.dashboard.driver.root,
            },
            {
              name: "Liste des commandes",
              href: paths.dashboard.driver.listSupplierOrder,
            },
            {
              name: "Liste des Fournisseurs",
              href: paths.dashboard.driver.supplier,
            },
          ]}
          sx={{
            mb: { xs: 3, md: 5 },
          }}
        />

        <Card>
          <Tabs
            value={filters.status}
            onChange={handleFilterStatus}
            sx={{
              px: 2.5,
              boxShadow: `inset 0 -2px 0 0 ${alpha(
                theme.palette.grey[500],
                0.08
              )}`,
            }}
          >
            {TABS.map((tab) => (
              <Tab
                key={tab.value}
                value={tab.value}
                label={tab.label}
                iconPosition="end"
                icon={
                  <Label
                    variant={
                      ((tab.value === "all" || tab.value === filters.status) &&
                        "filled") ||
                      "soft"
                    }
                    color={tab.color}
                  >
                    {tab.count}
                  </Label>
                }
              />
            ))}
          </Tabs>

          <OrderTableToolbar
            filters={filters}
            onFilters={handleFilters}
            dateError={dateError}
            selectedDate={selectedDate}
            setSelectedDate={setSelectedDate}
            supplierOrdersList={suppliersData}
          />

          {canReset && (
            <OrderTableFiltersResult
              filters={filters}
              onFilters={handleFilters}
              onResetFilters={handleResetFilters}
              results={dataFiltered.length}
              sx={{ p: 2.5, pt: 0 }}
            />
          )}

          <TableContainer sx={{ position: "relative", overflow: "unset" }}>
            <Scrollbar>
              <Table
                size={table.dense ? "small" : "medium"}
                sx={{ minWidth: 960 }}
              >
                <TableHeadCustom
                  order={table.order}
                  orderBy={table.orderBy}
                  headLabel={TABLE_HEAD}
                  rowCount={dataFiltered.length}
                  numSelected={table.selected.length}
                  onSort={table.onSort}
                />


                <TableBody>
                  {dataInPage.map((row) => (
                    <SupplierOrderTableRow
                      key={row.id}
                      row={row}
                      selected={table.selected.includes(row.id)}
                      onSelectRow={() => table.onSelectRow(row.id)}
                      onDeleteRow={() => handleDeleteRow(row.id)}
                      onViewRow={() => handleViewRow(row.id)}
                    />
                  ))}

                  <TableEmptyRows
                    height={denseHeight}
                    emptyRows={emptyRows(
                      table.page,
                      table.rowsPerPage,
                      dataFiltered.length
                    )}
                  />

                  <TableNoData notFound={notFound} />
                </TableBody>
              </Table>
            </Scrollbar>
          </TableContainer>

          <TablePaginationCustom
            count={dataFiltered.length}
            page={table.page}
            rowsPerPage={table.rowsPerPage}
            onPageChange={table.onChangePage}
            onRowsPerPageChange={table.onChangeRowsPerPage}
            dense={table.dense}
            onChangeDense={table.onChangeDense}
          />
        </Card>
      </Container>
    </>
  );
}

// ----------------------------------------------------------------------

function applyFilter({
  inputData,
  comparator,
  filters,
  dateError,
}: {
  inputData: SupplierOrder[];
  comparator: (a: any, b: any) => number;
  filters: ISupplierOrderTableToDriverFilters;
  dateError: boolean;
}) {
  const { name, status, supplier, deliveryDate } = filters;

  const stabilizedThis = inputData.map((el, index) => [el, index] as const);

  stabilizedThis.sort((a, b) => {
    const order = comparator(a[0], b[0]);
    if (order !== 0) return order;
    return a[1] - b[1];
  });

  inputData = stabilizedThis.map((el) => el[0]);

  if (name) {
    inputData = inputData.filter(
      (order) =>
        order.supplierName?.toLowerCase().indexOf(name.toLowerCase()) !== -1
    );
  }

  if (status !== "all") {
    inputData = inputData.filter((supplierOrder) => supplierOrder.supplierOrderStatus === status);
  } else {
    inputData = inputData;
  }

  if (supplier.length) {
    inputData = inputData.filter((order) => supplier.includes(order.supplierName || 'N/A'));
  }

  if (deliveryDate) {
    inputData = inputData.filter((order) => new Date(order.deliveryDate!).toDateString() === deliveryDate?.toDateString());
  }

  return inputData;
}
