import React, { useCallback, useMemo } from "react";
import Stack from "@mui/material/Stack";
import TextField from "@mui/material/TextField";
import InputAdornment from "@mui/material/InputAdornment";
import Iconify from "../../../../components/iconify";
import { ISupplierTableFilters, ISupplierTableFilterValue } from "../../../../types/supplier";
import { Checkbox, Grid, SelectChangeEvent } from "@mui/material";
import { ListItemText, MenuItem } from "@mui/material";
import { OutlinedInput, InputLabel, Select } from "@mui/material";
import { FormControl } from "@mui/material";
import { SupplierDTO } from "@/contexts/types/supplierOrder";

// ----------------------------------------------------------------------

type Props = {
  filters: ISupplierTableFilters;
  onFilters: (name: string, value: ISupplierTableFilterValue) => void;
  suppliersList: SupplierDTO[];
};

export default function SupplierTableToolbar({
  filters,
  onFilters,
  suppliersList,
}: Props) {
  const handleFilterName = useCallback(
    (event: React.ChangeEvent<HTMLInputElement>) => {
      onFilters("name", event.target.value);
    },
    [onFilters]
  );

  const supplierOptions = useMemo(() => {
    return suppliersList.map(supplier => ({ value: supplier.supplierName + ' ' + supplier.supplierLastName, label: supplier.supplierName + ' ' + supplier.supplierLastName }));
  }, [suppliersList]);

  const isAllSelected = supplierOptions.length > 0 && filters.selectedSuppliers.length === supplierOptions.length;

  const handleSelectAllClick = useCallback(() => {
    const newValue = isAllSelected ? [] : supplierOptions.map(option => option.value);
    onFilters("selectedSuppliers", newValue);
  }, [isAllSelected, supplierOptions, onFilters]);

  const handleFilterSupplier = (event: SelectChangeEvent<string[]>) => {
    const value = event.target.value as string[];
    if (value.includes("select-all")) {
      const newValue = isAllSelected ? [] : supplierOptions.map((option) => option.value);
      onFilters("selectedSuppliers", newValue);
    } else {
      onFilters("selectedSuppliers", value);
    }
  };

  return (
    <>
      <Stack
        spacing={2}
        alignItems={{ xs: "flex-end", md: "center" }}
        direction={{
          xs: "column",
          md: "row",
        }}
        sx={{
          p: 2.5,
          pr: { xs: 2.5, md: 1 },
        }}
      >
        <Grid container spacing={2} alignItems="center">
          <Grid item xs={12} md={5}>
            <FormControl fullWidth>
              <InputLabel>Fournisseur</InputLabel>
              <Select
                multiple
                value={filters.selectedSuppliers}
                onChange={handleFilterSupplier}
                input={<OutlinedInput label="Fournisseur" />}
                renderValue={(selected) => (selected as string[]).join(', ')}
              >
                <MenuItem onClick={handleSelectAllClick} value="select-all">
                  <Checkbox
                    checked={isAllSelected}
                    indeterminate={filters.selectedSuppliers.length > 0 && filters.selectedSuppliers.length < supplierOptions.length}
                  />
                  <ListItemText primary="Select All" />
                </MenuItem>
                {supplierOptions.map((option) => (
                  <MenuItem key={option.value} value={option.value}>
                    <Checkbox checked={filters.selectedSuppliers.includes(option.value)} />
                    <ListItemText primary={option.label} />
                  </MenuItem>
                ))}
              </Select>
            </FormControl>
          </Grid>

          <Grid item xs={12} md={7}>
            <TextField
              fullWidth
              value={filters.name}
              onChange={handleFilterName}
              placeholder="Chercher par nom ou adresse..."
              InputProps={{
                startAdornment: (
                  <InputAdornment position="start">
                    <Iconify
                      icon="eva:search-fill"
                      sx={{ color: "text.disabled" }}
                    />
                  </InputAdornment>
                ),
              }}
            />
          </Grid>
        </Grid>
      </Stack>
    </>
  );
}
