"use client";

import { useState, useCallback, useMemo } from "react";
import Card from "@mui/material/Card";
import Table from "@mui/material/Table";
import Container from "@mui/material/Container";
import TableBody from "@mui/material/TableBody";
import TableContainer from "@mui/material/TableContainer";
import { paths } from "../../../../../routes/paths";
import { useRouter } from "../../../../../hooks/use-router";
import Scrollbar from "../../../../components/scrollbar";
import { useSnackbar } from "../../../../components/snackbar";
import { useSettingsContext } from "../../../../components/settings";
import CustomBreadcrumbs from "../../../../components/custom-breadcrumbs";
import {
  useTable,
  emptyRows,
  TableNoData,
  getComparator,
  TableEmptyRows,
  TableHeadCustom,
  TablePaginationCustom,
} from "../../../../components/table";
import {
  ISupplierTableFilters,
} from "../../../../types/supplier";
import SupplierTableRow from "../all-supplier/SupplierTableRow";
import SupplierTableToolbar from "../all-supplier/SupplierTableToolbar";
import SupplierTableFiltersResult from "../all-supplier/SupplierTableFiltersResult";
import { archiveSupplier, deleteSupplier, useGetNotArchivedSuppliers } from "@/shared/api/supplier";
import { SupplierDTO } from "@/contexts/types/supplierOrder";
import { Alert, Button, Snackbar } from "@mui/material";
import { getTokenInfo } from "@/utils/token";
import Iconify from "@/components/iconify";
import RouterLink from "@/routes/router-link";


const TABLE_HEAD = [
  { id: "name", label: "Nom", minWidth: 120 },
  { id: "address", label: "Adresse", minWidth: 200 },
  { id: "phone", label: "Numéro de Contact", minWidth: 140 },
  { id: "", label: "", minWidth: 50, align: "right" },
];

const defaultFilters: ISupplierTableFilters = {
  name: "",
  selectedSuppliers: [],
};


export default function SupplierListView() {
  const { enqueueSnackbar } = useSnackbar();

  const table = useTable({ defaultOrderBy: "id", defaultOrder: "desc" });

  const settings = useSettingsContext();

  const router = useRouter();
  const [snackbarOpen, setSnackbarOpen] = useState(false);
  const [snackbarMessage, setSnackbarMessage] = useState("");
  const [snackbarSeverity, setSnackbarSeverity] = useState<"success" | "error">(
    "success"
  );

  const { suppliersData, suppliersError, mutate} = useGetNotArchivedSuppliers();

  const [filters, setFilters] = useState(defaultFilters);

  const dataFiltered = useMemo(
    () =>
      applyFilter({
        inputData: suppliersData,
        comparator: getComparator(table.order, table.orderBy),
        filters,
      }),
    [suppliersData, table.order, table.orderBy, filters]
  );

  const dataInPage = dataFiltered.slice(
    table.page * table.rowsPerPage,
    table.page * table.rowsPerPage + table.rowsPerPage
  );

  const denseHeight = table.dense ? 56 : 76;

  const canReset = !!filters.name || filters.selectedSuppliers.length > 0;

  const notFound = (!dataFiltered.length && canReset) || !dataFiltered.length;

  const handleFilters = useCallback(
    (name: string, value: string | string[]) => {
      table.onResetPage();
      setFilters((prevState) => ({
        ...prevState,
        [name]: value,
      }));
    },
    [table]
  );
  const token = localStorage.getItem("token");
  const storedUserInfo = token ? getTokenInfo(token) : null;
  const isNotDriver = !storedUserInfo?.realm_access?.roles?.includes("DRIVER");


  const handleResetFilters = useCallback(() => {
    setFilters(defaultFilters);
  }, []);

  const handleViewRow = useCallback(
    (id: string) => {
      router.push(paths.dashboard.driver.supplierDetails(id));
    },
    [router]
  );
  const handleEditRow = useCallback(
    (id: number) => {
      router.push(paths.dashboard.admin.supplier.edit(String(id)));
    },
    [router]
  );
  const handleArchiveRow = useCallback(
    async (id: number) => {
      try {
        const response = await archiveSupplier(id, true);
        if (response.status === 200) {
          setSnackbarMessage("Fournisseur archivé avec succès");
          setSnackbarSeverity("success");
          setSnackbarOpen(true);
          mutate();
        } else {
          setSnackbarMessage("Échec de l'archivage du fournisseur");
          setSnackbarSeverity("error");
          setSnackbarOpen(true);
        }
      } catch (error) {
        setSnackbarMessage("Échec de l'archivage du fournisseur");
        setSnackbarSeverity("error");
        setSnackbarOpen(true);
        console.error(error);
      }
    },
    [mutate]
  );

  const handleDeleteRow = useCallback(
    async (id: number) => {
      try {
        const response = await deleteSupplier(id);
        if (response.status === 200) {
          setSnackbarMessage("Fournisseur supprimé avec succès");
          setSnackbarSeverity("success");
          setSnackbarOpen(true);
          mutate();
        } else {
          setSnackbarMessage("Échec de la suppression du fournisseur");
          setSnackbarSeverity("error");
          setSnackbarOpen(true);
        }
      } catch (error) {
        setSnackbarMessage("Échec de la suppression du fournisseur");
        setSnackbarSeverity("error");
        setSnackbarOpen(true);
        console.error(error);
      }
    },
    [enqueueSnackbar, table, mutate]
  );


  if (suppliersError) {
    enqueueSnackbar("Failed to fetch suppliers", { variant: "error" });
    return <div>Error loading suppliers</div>;
  }

  return (
    <>
      <Container maxWidth={settings.themeStretch ? false : "lg"}>
        <CustomBreadcrumbs
          heading="Liste des fournisseurs"
          links={[
            {
              name: "Tableau de bord",
              href: paths.dashboard.driver.root,
            },
            {
              name: "Liste des fournisseurs",
              href: paths.dashboard.driver.supplier,
            },
          ]}
          action={
            <Button
              component={RouterLink}
              href={paths.dashboard.admin.supplier.new}
              variant="contained"
              startIcon={<Iconify icon="mingcute:add-line" />}
            >
              Nouveau fournisseur
            </Button>
          }
          sx={{
            mb: { xs: 3, md: 5 },
          }}
        />

        <Card>
          <SupplierTableToolbar
            filters={filters}
            onFilters={handleFilters}
            suppliersList={suppliersData}
          />

          {canReset && (
            <SupplierTableFiltersResult
              filters={filters}
              onFilters={handleFilters}
              onResetFilters={handleResetFilters}
              results={dataFiltered.length}
              sx={{ p: 2.5, pt: 0 }}
            />
          )}

          <TableContainer sx={{ position: "relative", overflow: "unset" }}>
            <Scrollbar>
              <Table
                size={table.dense ? "small" : "medium"}
                sx={{ minWidth: 960 }}
              >
                <TableHeadCustom
                  order={table.order}
                  orderBy={table.orderBy}
                  headLabel={TABLE_HEAD}
                  rowCount={dataFiltered.length}
                  numSelected={table.selected.length}
                  onSort={table.onSort}
                />

                <TableBody>
                  {dataInPage.map((row) => (
                    <SupplierTableRow
                      key={row.id}
                      row={{
                        ...row,
                        id: Number(row.id),
                      }}
                      selected={table.selected.includes(String(row.id))}
                      onSelectRow={() => table.onSelectRow(String(row.id))}
                      onViewRow={() => handleViewRow(String(row.id))}
                      onEditRow={() => handleEditRow(Number(row.id))}
                      onDeleteRow={() => handleDeleteRow(Number(row.id))}
                      onArchiveRow={() => handleArchiveRow(Number(row.id))}
                      isNotDriver={isNotDriver}
                    />
                  ))}

                  <TableEmptyRows
                    height={denseHeight}
                    emptyRows={emptyRows(
                      table.page,
                      table.rowsPerPage,
                      dataFiltered.length
                    )}
                  />

                  <TableNoData notFound={notFound} />
                </TableBody>
              </Table>
            </Scrollbar>
          </TableContainer>

          <TablePaginationCustom
            count={dataFiltered.length}
            page={table.page}
            rowsPerPage={table.rowsPerPage}
            onPageChange={table.onChangePage}
            onRowsPerPageChange={table.onChangeRowsPerPage}
            dense={table.dense}
            onChangeDense={table.onChangeDense}
          />
        </Card>
      </Container>
      <Snackbar
        open={snackbarOpen}
        autoHideDuration={3000}
        onClose={() => setSnackbarOpen(false)}
        anchorOrigin={{ vertical: "top", horizontal: "right" }}
      >
        <Alert
          onClose={() => setSnackbarOpen(false)}
          severity={snackbarSeverity}
        >
          {snackbarMessage}
        </Alert>
      </Snackbar>
    </>
  );
}

// ----------------------------------------------------------------------

function applyFilter({
  inputData,
  comparator,
  filters,
}: {
  inputData: SupplierDTO[];
  comparator: (a: any, b: any) => number;
  filters: ISupplierTableFilters;
}) {
  const { name, selectedSuppliers } = filters;

  const stabilizedThis = inputData.map((el, index) => [el, index] as const);

  stabilizedThis.sort((a, b) => {
    const order = comparator(a[0], b[0]);
    if (order !== 0) return order;
    return a[1] - b[1];
  });

  inputData = stabilizedThis.map((el) => el[0]);

  if (name) {
    inputData = inputData.filter(
      (supplier) =>
        supplier.supplierName.toLowerCase().indexOf(name.toLowerCase()) !==
          -1 ||
        supplier.supplierLastName.toLowerCase().indexOf(name.toLowerCase()) !==
          -1 ||
        supplier.companyStreet.toLowerCase().indexOf(name.toLowerCase()) !==
          -1 ||
        supplier.companyCity.toLowerCase().indexOf(name.toLowerCase()) !== -1 ||
        supplier.companyCountry.toLowerCase().indexOf(name.toLowerCase()) !== -1
    );
  }

  if (selectedSuppliers.length) {
    inputData = inputData.filter((supplier) =>
      selectedSuppliers.includes(
        supplier.supplierName + " " + supplier.supplierLastName
      )
    );
  }

  return inputData;
}
