'use client';
import React, { useState } from 'react';
import { useFormik } from 'formik';
import * as Yup from 'yup';
import Typography from '@mui/material/Typography';
import FormControl from '@mui/material/FormControl';
import TextField from '@mui/material/TextField';
import Button from '@mui/material/Button';
import { IGiftCard } from '@/shared/types/giftcard';
import {useAddGiftCardAndSendEmail } from '@/shared/api/giftcard';
import CustomBreadcrumbs from '@/components/custom-breadcrumbs';
import { paths } from '@/routes/paths';
import Container from '@mui/material/Container';
import { useSettingsContext } from '@/components/settings';
import { Box, Card, Checkbox, FormControlLabel } from '@mui/material';
import CustomAlert from '@/shared/components/CustomAlert';
import { useRouter } from "@/hooks";


interface GiftCardFormValues {
  solde: number;
  dateExpiration: string;
  emailDestinataire: string;
  emailBeneficiaire: string;
  message: string;
  paid: boolean;
}

export default function AddGiftCard() {
  const settings = useSettingsContext();
  const router = useRouter();
  const [alert, setAlert] = useState<{ message: string; type: 'success' | 'error' } | null>(null);

  const validationSchema = Yup.object({
    solde: Yup.number()
      .required('Le solde est requis')
      .min(10, 'Le solde doit être d\'au moins 10 €'),
    dateExpiration: Yup.date()
      .required('La date d\'expiration est requise')
      .min(new Date(), 'La date d\'expiration doit être future'),
    emailDestinataire: Yup.string()
      .email('Format d\'email invalide')
      .required('L\'email du destinataire est requis'),
    emailBeneficiaire: Yup.string()
      .email('Format d\'email invalide')
      .required('L\'email du bénéficiaire est requis'),
    message: Yup.string().max(500, 'Le message ne doit pas dépasser 500 caractères'),
    paid: Yup.boolean(),
  });

  const formik = useFormik<GiftCardFormValues>({
    initialValues: {
      solde: 10,
      dateExpiration: '',
      emailDestinataire: '',
      emailBeneficiaire: '',
      message: '',
      paid: true,
    },
    validationSchema,
    onSubmit: async (values) => {
      try {
        const dateExpirationWithTime = `${values.dateExpiration}T00:00:00`;

        const dataToSend: IGiftCard = {
          solde: values.solde,
          dateexpiration: dateExpirationWithTime,
          emaildestinataire: values.emailDestinataire,
          emailbeneficiaire: values.emailBeneficiaire,
          message: values.message,
          paid: values.paid,
        };

        await useAddGiftCardAndSendEmail(dataToSend);
        formik.resetForm();
        setAlert({
          message: 'Carte cadeau créée avec succès !',
          type: 'success',
        });

        router.push(paths.dashboard.admin.giftcard.root);
      } catch (error) {
        setAlert({
          message: 'Erreur lors de la création de la carte cadeau',
          type: 'error',
        });
      }
    },
  });

  return (
    <>
      {alert && (
        <CustomAlert
          message={alert.message}
          type={alert.type}
          onClose={() => setAlert(null)}
          style={{ top: '20px', right: '20px', zIndex: 3000 }}
        />
      )}
      <Container
        maxWidth={settings.themeStretch ? false : "xl"}
        sx={{
          flexGrow: 1,
          display: "flex",
          flexDirection: "column",
          justifyContent: "flex-start",
          alignItems: "flex-start",
          padding: 3,
          gap: 2,
        }}
      >
        <CustomBreadcrumbs
          heading="Ajouter carte cadeau"
          links={[
            { name: "Tableau de bord", href: paths.dashboard.root },
            {
              name: "Carte Cadeaux",
              href: paths.dashboard.admin.giftcard.all_giftcard,
            },
            { name: "Ajouter" },
          ]}
          rollbackLink={paths.dashboard.admin.giftcard.all_giftcard}
          sx={{
            mb: 2,
          }}
        />

        <Card
          variant="outlined"
          sx={{
            padding: 3,
            maxWidth: 550,
            width: "100%", // S'adapte aux petits écrans
            display: "flex-start",
            flexDirection: "column", // Organise le contenu verticalement
            gap: 2, // Espacement entre les champs du formulaire
          }}
        >
          <Typography variant="h5" gutterBottom>
            Créer une carte cadeau
          </Typography>

          <Box component="form" onSubmit={formik.handleSubmit} sx={{ width: "100%" }}>
          <FormControl fullWidth>

                <TextField
                  label="Solde de la carte cadeau (€)"
                  name="solde"
                  type="number"
                  value={formik.values.solde}
                  onChange={formik.handleChange}
                  onBlur={formik.handleBlur}
                  error={formik.touched.solde && Boolean(formik.errors.solde)}
                  helperText={formik.touched.solde && formik.errors.solde}
                  fullWidth
                  margin="normal"
                  inputProps={{ min: 10 }}
                />

                <TextField
                  label="Date d’expiration"
                  name="dateExpiration"
                  type="date"
                  value={formik.values.dateExpiration}
                  onChange={formik.handleChange}
                  onBlur={formik.handleBlur}
                  error={formik.touched.dateExpiration && Boolean(formik.errors.dateExpiration)}
                  helperText={formik.touched.dateExpiration && formik.errors.dateExpiration}
                  fullWidth
                  margin="normal"
                  InputLabelProps={{
                    shrink: true,
                  }}
                />

                <TextField
                  label="Email du destinataire"
                  name="emailDestinataire"
                  type="email"
                  value={formik.values.emailDestinataire}
                  onChange={formik.handleChange}
                  onBlur={formik.handleBlur}
                  error={formik.touched.emailDestinataire && Boolean(formik.errors.emailDestinataire)}
                  helperText={formik.touched.emailDestinataire && formik.errors.emailDestinataire}
                  fullWidth
                  margin="normal"
                />

                <TextField
                  label="Email du bénéficiaire"
                  name="emailBeneficiaire"
                  type="email"
                  value={formik.values.emailBeneficiaire}
                  onChange={formik.handleChange}
                  onBlur={formik.handleBlur}
                  error={formik.touched.emailBeneficiaire && Boolean(formik.errors.emailBeneficiaire)}
                  helperText={formik.touched.emailBeneficiaire && formik.errors.emailBeneficiaire}
                  fullWidth
                  margin="normal"
                />

                <TextField
                  label="Message (optionnel)"
                  name="message"
                  multiline
                  rows={4}
                  value={formik.values.message}
                  onChange={formik.handleChange}
                  onBlur={formik.handleBlur}
                  error={formik.touched.message && Boolean(formik.errors.message)}
                  helperText={formik.touched.message && formik.errors.message}
                  fullWidth
                  margin="normal"
                />
                <FormControlLabel
                  control={
                    <Checkbox
                      name="paid"
                      checked={formik.values.paid}
                      onChange={formik.handleChange}
                      color="primary"
                    />
                  }
                  label="Carte cadeau prête à être utilisée "
                  sx={{ mt: 2 }}
                />

                <Button type="submit" variant="contained" color="primary" fullWidth style={{ marginTop: '20px' }}>
                  Enregistrer
                </Button>
              </FormControl>

          </Box>
        </Card>
      </Container>
    </>
  );
}
