'use client';

import React, { useState, useEffect } from 'react';
import { useFormik } from 'formik';
import * as Yup from 'yup';
import Typography from '@mui/material/Typography';
import FormControl from '@mui/material/FormControl';
import TextField from '@mui/material/TextField';
import Button from '@mui/material/Button';
import { IGiftCard } from '@/shared/types/giftcard';

import { paths } from '@/routes/paths';
import { useSettingsContext } from '@/components/settings';
import { Box, Card, Checkbox, FormControlLabel } from '@mui/material';
import CustomAlert from '@/shared/components/CustomAlert';
import { useRouter } from "@/hooks";
import { updateGiftCard, useGetGiftCardById } from '@/shared/api/giftcard';
import { logger } from '@/utils/logger';

interface GiftCardFormValues {
  solde: number;
  dateExpiration: string;
  emailDestinataire: string;
  emailBeneficiaire: string;
  message: string;
  paid: boolean;
}


export default function EditGiftCard({ giftcardId }: { giftcardId: number }) {
  const settings = useSettingsContext();
  const router = useRouter();
  const [alert, setAlert] = useState<{ message: string; type: 'success' | 'error' } | null>(null);

  const { giftcard, isLoading, isError } = useGetGiftCardById(giftcardId.toString());

  const validationSchema = Yup.object({
    solde: Yup.number()
      .required('Le solde est requis')
      .min(10, 'Le solde doit être d\'au moins 10 €')
      .max(500, 'Le solde ne doit pas dépasser 500 €'),
    dateExpiration: Yup.date()
      .required('La date d\'expiration est requise')
      .min(new Date(), 'La date d\'expiration doit être future'),
    emailDestinataire: Yup.string()
      .email('Format d\'email invalide')
      .required('L\'email du destinataire est requis'),
    emailBeneficiaire: Yup.string()
      .email('Format d\'email invalide')
      .required('L\'email du bénéficiaire est requis'),
    message: Yup.string().max(500, 'Le message ne doit pas dépasser 500 caractères'),
    paid: Yup.boolean(),
  });

  const formik = useFormik<GiftCardFormValues>({
    initialValues: {
      solde: giftcard?.solde || 10,
      dateExpiration: giftcard ? giftcard.dateexpiration.split('T')[0] : '',
      emailDestinataire: giftcard?.emaildestinataire || '',
      emailBeneficiaire: giftcard?.emailbeneficiaire || '',
      message: giftcard?.message || '',
      paid: giftcard?.paid || true,
    },
    enableReinitialize: true,
    validationSchema,
    onSubmit: async (values) => {
      try {
        const dateExpirationWithTime = `${values.dateExpiration}T00:00:00`;

        const dataToSend: IGiftCard = {
          solde: values.solde,
          dateexpiration: dateExpirationWithTime,
          emaildestinataire: values.emailDestinataire,
          emailbeneficiaire: values.emailBeneficiaire,
          message: values.message,
        };

        const updatedGiftCard = await updateGiftCard(giftcardId, dataToSend);
        logger('Carte cadeau mise à jour:', updatedGiftCard);
        formik.resetForm();
        setAlert({
          message: 'Carte cadeau mise à jour avec succès !',
          type: 'success',
        });

        router.push(paths.dashboard.admin.giftcard.root);

      } catch (error) {
        setAlert({
          message: 'Erreur lors de la mise à jour de la carte cadeau',
          type: 'error',
        });
      }
    },
  });

  if (isLoading) return <Typography>Chargement...</Typography>;
  if (isError) return <Typography>Erreur lors du chargement des données.</Typography>;
  if (!giftcard) return <Typography>Carte cadeau non trouvée.</Typography>;

  return (
    <>
      {alert && (
        <CustomAlert
          message={alert.message}
          type={alert.type}
          onClose={() => setAlert(null)}
          style={{ top: '20px', right: '20px', zIndex: 3000 }}
        />
      )}


      <Card variant="outlined" sx={{ padding: 2, maxWidth: 550, ml: 0 }}>
        <Box sx={{ maxWidth: 500 }}>
          <form onSubmit={formik.handleSubmit}>
            <FormControl fullWidth>
              <Typography variant="h5" gutterBottom>
                Modifier une carte cadeau
              </Typography>
              <TextField
                label="Solde de la carte cadeau (€)"
                name="solde"
                type="number"
                value={formik.values.solde}
                onChange={formik.handleChange}
                onBlur={formik.handleBlur}
                error={formik.touched.solde && Boolean(formik.errors.solde)}
                helperText={formik.touched.solde && formik.errors.solde}
                fullWidth
                margin="normal"
                inputProps={{ min: 10 }}
              />

              <TextField
                label="Date d’expiration"
                name="dateExpiration"
                type="date"
                value={formik.values.dateExpiration}
                onChange={formik.handleChange}
                onBlur={formik.handleBlur}
                error={formik.touched.dateExpiration && Boolean(formik.errors.dateExpiration)}
                helperText={formik.touched.dateExpiration && formik.errors.dateExpiration}
                fullWidth
                margin="normal"
                InputLabelProps={{
                  shrink: true,
                }}
              />

              <TextField
                label="Email du destinataire"
                name="emailDestinataire"
                type="email"
                value={formik.values.emailDestinataire}
                onChange={formik.handleChange}
                onBlur={formik.handleBlur}
                error={formik.touched.emailDestinataire && Boolean(formik.errors.emailDestinataire)}
                helperText={formik.touched.emailDestinataire && formik.errors.emailDestinataire}
                fullWidth
                margin="normal"
              />

              <TextField
                label="Email du bénéficiaire"
                name="emailBeneficiaire"
                type="email"
                value={formik.values.emailBeneficiaire}
                onChange={formik.handleChange}
                onBlur={formik.handleBlur}
                error={formik.touched.emailBeneficiaire && Boolean(formik.errors.emailBeneficiaire)}
                helperText={formik.touched.emailBeneficiaire && formik.errors.emailBeneficiaire}
                fullWidth
                margin="normal"
              />

              <TextField
                label="Message (optionnel)"
                name="message"
                multiline
                rows={4}
                value={formik.values.message}
                onChange={formik.handleChange}
                onBlur={formik.handleBlur}
                error={formik.touched.message && Boolean(formik.errors.message)}
                helperText={formik.touched.message && formik.errors.message}
                fullWidth
                margin="normal"
              />

              <FormControlLabel
                control={
                  <Checkbox
                    name="paid"
                    checked={formik.values.paid}
                    onChange={formik.handleChange}
                    color="primary"
                  />
                }
                label="Carte cadeau prête à être utilisée "
                sx={{ mt: 2 }}
              />


              <Button type="submit" variant="contained" color="primary" fullWidth style={{ marginTop: '20px' }}>
                Enregistrer les Modifications
              </Button>
            </FormControl>
          </form>
        </Box>
      </Card>
    </>
  );
}
