import React, { useState, useEffect, useMemo } from "react";
import {
  Card,
  SelectChangeEvent,
  Select,
  Checkbox,
  MenuItem,
  FormControl,
  InputLabel,
  OutlinedInput,
  ListItemText,
} from "@mui/material";
import CustomDateRangePicker from "@/shared/components/custom-date-range-picker/custom-date-range-picker";
import { useGetLivesInRange, LiveDTO } from "@/shared/api/chiffreLiveStatistics";
import Iconify from "@/components/iconify";
import { fDate } from "@/utils/format-time";

type Props = {
  startDate: Date;
  endDate: Date;
  selectedLiveIds: number[];                 // <-- On va stocker et renvoyer des IDs
  setSelectedLiveIds: (ids: number[]) => void;
  onChangeStartDate: (startDate: Date) => void;
  onChangeEndDate: (endDate: Date) => void;
};

export default function OrderFilters({
  startDate,
  endDate,
  onChangeEndDate,
  onChangeStartDate,
  selectedLiveIds,
  setSelectedLiveIds,
}: Props) {
  const [isDateRangeOpen, setIsDateRangeOpen] = useState(false);
  const [selectAll, setSelectAll] = useState(true);

  // On récupère la liste des lives (id, title)
  const { livesData, isLoading, isError } = useGetLivesInRange(startDate, endDate);

  // On mémorise le tableau => plus facile d'y accéder pour la select
  const allLives: LiveDTO[] = useMemo(() => livesData ?? [], [livesData]);

  // A) Pré-sélection automatique => si vous voulez tout cocher par défaut
  useEffect(() => {
    if (allLives.length > 0) {
      setSelectedLiveIds(allLives.map((l) => l.id));
    } else {
      setSelectedLiveIds([]);
    }
  }, [allLives]);

  // B) Select All
  useEffect(() => {
    setSelectAll(selectedLiveIds.length === allLives.length);
  }, [selectedLiveIds, allLives]);

  // Ouvrir / fermer le DateRange
  const closeDateRange = () => setIsDateRangeOpen(false);
  const openDateRange = () => setIsDateRangeOpen(true);

  // Gestion du changement de sélection
  const handleChangeLives = (event: SelectChangeEvent<number[]>) => {
    const { value } = event.target;  // array d'IDs
    if (value.includes(-1 as never)) {
      // -1 pour Select All
      if (selectAll) {
        setSelectedLiveIds([]);
        setSelectAll(false);
      } else {
        setSelectedLiveIds(allLives.map((l) => l.id));
        setSelectAll(true);
      }
    } else {
      // Sinon, on applique la sélection
      setSelectedLiveIds(typeof value === "string"
        ? value.split(",").map(Number)
        : value
      );
    }
  };

  return (
    <Card
      sx={{
        minHeight: "auto",
        display: "flex",
        alignItems: "center",
        p: 3,
        gap: 2,
      }}
    >
      {/* Card pour le date picker */}
      <Card
        onClick={openDateRange}
        variant="outlined"
        sx={{
          height: 50,
          width: 500,
          cursor: "pointer",
          borderRadius: 1,
          px: 10,
          display: "flex",
          boxShadow: "none",
          justifyContent: "space-between",
          alignItems: "center",
          border: "1px solid #e0e0e0",
          "&:hover": {
            backgroundColor: "#f5f5f5",
          },
        }}
      >
        <Iconify sx={{ mr: 3 }} icon="solar:calendar-date-bold" />
        <ListItemText
          secondary={`${fDate(startDate)} - ${fDate(endDate)}`}
          primaryTypographyProps={{
            variant: "body2",
            color: "text.primary",
            mb: 0,
          }}
        />
      </Card>

      <CustomDateRangePicker
        open={isDateRangeOpen}
        onClose={closeDateRange}
        startDate={startDate}
        endDate={endDate}
        onChangeEndDate={(newValue) => onChangeEndDate(newValue!)}
        onChangeStartDate={(newValue) => onChangeStartDate(newValue!)}
      />

      {/* Multi-select pour les lives */}
      <FormControl sx={{ m: 1, width: 500 }}>
        <InputLabel>Lives</InputLabel>
        <Select<number[]>
          multiple
          value={selectedLiveIds}
          onChange={handleChangeLives}
          input={<OutlinedInput sx={{ height: 50 }} label="Lives" />}
          renderValue={(selected) => {
            // on affiche les titres des IDs sélectionnés
            const selectedTitles = allLives
              .filter((l) => selected.includes(l.id))
              .map((l) => l.title);
            return selectedTitles.join(", ");
          }}
        >
          {/* "Select All" */}
          <MenuItem value={-1 as never}>
            <Checkbox checked={selectAll} />
            Select All
          </MenuItem>
          {/* Les lives retournés */}
          {allLives.map((live) => (
            <MenuItem key={live.id} value={live.id as never}>
              <Checkbox checked={selectedLiveIds.includes(live.id)} />
              {live.title}
            </MenuItem>
          ))}
        </Select>
      </FormControl>
    </Card>
  );
}
