"use client";
import Container from "@mui/material/Container";
import { paths } from "@/routes/paths";
import { useSettingsContext } from "@/shared/components/settings";
import CustomBreadcrumbs from "@/components/custom-breadcrumbs";
import { Grid } from "@mui/material";
import { useEffect, useState } from "react";
import AppWidgetSummary from "./statistics-widget-summary";
import { useTheme } from "@mui/material/styles";
import AnalyticsItemsReceived from "./ReceivedItemsRate";

import ProductStatisticsFilters from "./statistics-filters-Product";

import {
  useProductCount,
  useReceivedProductsCount,
  useItemsToCollectCount,
  useTotalPurchaseAmount,
  useArticlesReceptionsChartData,
  useFinishedOrderLinesCount,
  useRefundedArticlesCount,
} from "@/shared/api/productStatistics"
import { fetchRefundReasons, RefundReasonType } from "@/shared/api/saleSessionStatistics";
import AnalyticsRefundReason from "../../order/statistics/analytics-refund-reason";
import { enqueueSnackbar } from "@/shared/components/snackbar";

export default function AllProductStatisticsView() {
  const settings = useSettingsContext();
  const theme = useTheme();

  const [startDate, setStartDate] = useState<Date>(() => {
    const oneMonthAgo = new Date();
    oneMonthAgo.setMonth(oneMonthAgo.getMonth() - 1);
    return oneMonthAgo;
  });

  const [endDate, setEndDate] = useState<Date>(new Date());
  const [selectedOrders, setSelectedOrders] = useState<string[]>([]);
  const [selectedPublicationSource, setSelectedPublicationSource] = useState<string>("ALL");

  const { productCount, isLoading: loadingProductCount } = useProductCount(startDate, endDate, selectedPublicationSource);
  const { receivedProductsCount, isLoading: loadingReceived } = useReceivedProductsCount(startDate, endDate);
  const { totalPurchaseAmount, isLoading: loadingPurchaseAmount } = useTotalPurchaseAmount(startDate, endDate);
  const { itemsToCollectCount, isLoading: loadingItemsToCollect } = useItemsToCollectCount(startDate, endDate);
  const { articlesReceptionsChartData, isLoading: loadingChart } = useArticlesReceptionsChartData(startDate, endDate);

  const { finishedOrderLinesCount, isLoading: loadingFinishedOrderLines } = useFinishedOrderLinesCount(startDate, endDate, selectedOrders);
  const { refundedArticlesCount, isLoading: loadingRefundedArticles } = useRefundedArticlesCount(startDate, endDate, selectedOrders);
  const [RefundReasonData, setRefundReason] = useState<RefundReasonType>({
    "STOCK_ERROR": 0,
    "CUSTOMER_RETURN": 0,
    "DEFECT": 0
  });


  const statsFiltersProps = {
    startDate,
    endDate,
    onChangeStartDate: (date: Date) => setStartDate(date),
    onChangeEndDate: (date: Date) => setEndDate(date),
    selectedOrders,
    setSelectedOrders,
    selectedPublicationSource,
    setSelectedPublicationSource,
  };

  const productCountValue = loadingProductCount ? 0 : (productCount ?? 0);
  const receivedProductsValue = loadingReceived ? 0 : (receivedProductsCount ?? 0);
  const totalPurchaseAmountValue = loadingPurchaseAmount ? 0 : (totalPurchaseAmount ?? 0);
  const itemsToCollectValue = loadingItemsToCollect ? 0 : (itemsToCollectCount ?? 0);
  const finishedOrderLinesValue = loadingFinishedOrderLines ? 0 : (finishedOrderLinesCount ?? 0);
  const refundedArticlesValue = loadingRefundedArticles ? 0 : (refundedArticlesCount ?? 0);

  useEffect(() => {
    fetchRefundReasons(startDate, endDate, selectedOrders).then((data) => {
      setRefundReason(data);
    }).catch(() => {
      enqueueSnackbar("error loading Refund Reasons");
    });
  }, [startDate, endDate, selectedOrders]);

  return (
    <Container
      sx={{ gap: 3, display: "flex", flexDirection: "column" }}
      maxWidth={settings.themeStretch ? false : "lg"}
    >
      <CustomBreadcrumbs
        heading="Statistique Produits"
        links={[
          {
            name: "Tableau de bord",
            href: paths.dashboard.root,
          },
          {
            name: "Produit",
            href: paths.dashboard.admin.productStatistics,
          },
          {
            name: "Statistiques",
          },
        ]}
        sx={{
          mb: { xs: 3, md: 5 },
        }}
      />

      <ProductStatisticsFilters {...statsFiltersProps} />

      <Grid container spacing={3}>
        <Grid item xs={12} sm={6} md={4}>
          <AppWidgetSummary
            title="Nombre de références créés"
            total={productCountValue}
            color={theme.palette.error.light}
          />
        </Grid>

        <Grid item xs={12} sm={6} md={4}>
          <AppWidgetSummary
            title="Nombre d’articles réceptionnés"
            total={receivedProductsValue}
            color={theme.palette.info.dark}
          />
        </Grid>

        <Grid item xs={12} sm={6} md={4}>
          <AppWidgetSummary
            title="Montant total des commandes d’achat"
            total={totalPurchaseAmountValue}
            color={theme.palette.error.light}
          />
        </Grid>

        <Grid item xs={12} sm={6} md={4}>
          <AppWidgetSummary
            title="Nombre d’articles à collecter"
            total={itemsToCollectValue}
            color={theme.palette.info.dark}
          />
        </Grid>

        {/* Désormais connectées au backend */}
        <Grid item xs={12} sm={6} md={4}>
          <AppWidgetSummary
            title="Nombre d’articles livrés"
            total={finishedOrderLinesValue}
            color={theme.palette.error.light}
          />
        </Grid>

        <Grid item xs={12} sm={6} md={4}>
          <AppWidgetSummary
            title="Nombre d’articles remboursés"
            total={refundedArticlesValue}
            color={theme.palette.info.dark}
          />
        </Grid>
      </Grid>

      <Grid container spacing={1}>
        <Grid item xs={4} md={4} lg={4}>
          <AnalyticsRefundReason
            title="Taux de remboursement par raisons"
            chart={{
              colors: ["#4CAF50", "#FFC107", "#FF5722"],
              series: [
                { label: "Retour client", value: RefundReasonData.CUSTOMER_RETURN },
                { label: "Erreur de stock", value: RefundReasonData.STOCK_ERROR },
                { label: "Défaut", value: RefundReasonData.DEFECT },
              ],
            }}
          />
        </Grid>

        <Grid item xs={8} md={6} lg={8}>
          {(!loadingChart && articlesReceptionsChartData && articlesReceptionsChartData.chart.series.length > 0) ? (
            <AnalyticsItemsReceived
              title={articlesReceptionsChartData.title}
              chart={articlesReceptionsChartData.chart}
            />
          ) : (
            <AnalyticsItemsReceived
              title="Nombre d’articles réceptionnés"
              chart={{
                labels: [],
                series: [
                  {
                    name: "Mois en cours",
                    type: "line",
                    fill: "solid",
                    data: []
                  }
                ],
              }}
            />
          )}
        </Grid>
      </Grid>
    </Container>
  );
}
