import React, { useState, useEffect } from "react";
import {
  Card,
  SelectChangeEvent,
  Select,
  Checkbox,
  MenuItem,
  FormControl,
  InputLabel,
  OutlinedInput,
  ListItemText,
} from "@mui/material";
import CustomDateRangePicker from "@/shared/components/custom-date-range-picker/custom-date-range-picker";

import Iconify from "@/components/iconify";
import { fDate } from "@/utils/format-time";

type Props = {
  startDate: Date;
  endDate: Date;
  setSelectedOrders: (selectedOrders: string[]) => void;
  selectedOrders: string[];
  onChangeStartDate: (startDate: Date) => void;
  onChangeEndDate: (endDate: Date) => void;
  selectedPublicationSource: string;
  setSelectedPublicationSource: (selectedPublicationSource: string) => void;
};

export default function ProductStatisticsFilters({
  startDate,
  endDate,
  onChangeEndDate,
  onChangeStartDate,
  setSelectedOrders,
  selectedOrders,
  selectedPublicationSource,
  setSelectedPublicationSource,
}: Props) {
  const [isDateRangeOpen, setIsDateRangeOpen] = useState(false);
  const [selectAllOrders, setSelectAllOrders] = useState(true);

  const OrderSource = ["LIVE", "WEB"];
  const PublicationSource = ["WEB", "LIVE", "BOTH", "ALL"];

  useEffect(() => {
    setSelectAllOrders(selectedOrders.length === OrderSource.length);
  }, [selectedOrders]);

  useEffect(() => {
    setSelectedOrders([...OrderSource]);
    setSelectedPublicationSource("ALL");
  }, []);

  const closeDateRange = () => {
    setIsDateRangeOpen(false);
  };

  const openDateRange = () => {
    setIsDateRangeOpen(true);
  };

  const handleChangeOrders = (event: SelectChangeEvent<typeof selectedOrders>) => {
    const {
      target: { value },
    } = event;
    if (value.includes("all")) {
      if (selectAllOrders) {
        setSelectedOrders([]);
        setSelectAllOrders(false);
      } else {
        setSelectedOrders([...OrderSource]);
        setSelectAllOrders(true);
      }
    } else {
      setSelectedOrders(typeof value === "string" ? value.split(",") : value);
    }
  };

  const handleChangePublicationSource = (event: SelectChangeEvent) => {
    setSelectedPublicationSource(event.target.value as string);
  };

  return (
    <Card
      sx={{
        minHeight: "auto",
        display: "flex",
        alignItems: "center",
        p: 3,
        gap: 2,
      }}
    >
      <Card
        onClick={openDateRange}
        variant="outlined"
        sx={{
          height: 50,
          width: 300,
          cursor: "pointer",
          borderRadius: 1,
          px: 2,
          display: "flex",
          boxShadow: "none",
          justifyContent: "space-between",
          alignItems: "center",
          border: "1px solid #e0e0e0",
          "&:hover": {
            backgroundColor: "#f5f5f5",
          },
        }}
      >
        <Iconify sx={{ mr: 1 }} icon="solar:calendar-date-bold" />
        <ListItemText
          secondary={`${fDate(startDate)} - ${fDate(endDate)}`}
          primaryTypographyProps={{
            variant: "body2",
            color: "text.primary",
            mb: 0,
          }}
        />
      </Card>
      <CustomDateRangePicker
        open={isDateRangeOpen}
        onClose={closeDateRange}
        startDate={startDate}
        endDate={endDate}
        onChangeEndDate={(newValue: Date | null) => onChangeEndDate(newValue!)}
        onChangeStartDate={(newValue: Date | null) => onChangeStartDate(newValue!)}
      />

      {/* Order Source Filter */}
      <FormControl sx={{ m: 1, width: 250 }}>
        <InputLabel>Order Source</InputLabel>
        <Select
          multiple
          value={selectedOrders}
          onChange={handleChangeOrders}
          input={<OutlinedInput sx={{ height: 50 }} label="Order Source" />}
          renderValue={(selected) => selected.join(", ")}
        >
          <MenuItem value="all">
            <Checkbox checked={selectAllOrders} />
            Select All
          </MenuItem>
          {OrderSource.map((source) => (
            <MenuItem key={source} value={source}>
              <Checkbox checked={selectedOrders.includes(source)} />
              {source}
            </MenuItem>
          ))}
        </Select>
      </FormControl>

      {/* Publication Source Filter */}
      <FormControl sx={{ m: 1, width: 250 }}>
        <InputLabel>Publication Source</InputLabel>
        <Select
          value={selectedPublicationSource}
          onChange={handleChangePublicationSource}
          input={<OutlinedInput sx={{ height: 50 }} label="Publication Source" />}
        >
          {PublicationSource.map((source) => (
            <MenuItem key={source} value={source}>
              {source}
            </MenuItem>
          ))}
        </Select>
      </FormControl>
    </Card>
  );
}
