import React, { useState, useEffect } from "react";
import {
  Card,
  SelectChangeEvent,
  Select,
  Checkbox,
  MenuItem,
  FormControl,
  InputLabel,
  OutlinedInput,
  ListItemText,
} from "@mui/material";
import CustomDateRangePicker from "@/shared/components/custom-date-range-picker/custom-date-range-picker";

import { _orders } from "@/shared/_mock";
import Iconify from "@/components/iconify";
import { fDate } from "@/utils/format-time";
import { ShippingMethod } from "@/shared/types/order";

type Props = {
  startDate: Date;
  endDate: Date;
  setSelectedOrders: (selectedOrders: string[]) => void;
  setSelectedShipping: (selectedOrders: string[]) => void;
  selectedOrders: string[];
  onChangeStartDate: (startDate: Date) => void;
  onChangeEndDate: (endDate: Date) => void;
  onChangeFilterStatus: (filterStatus: string) => void;
  filterStatus: string;
  selectedShipping: string[];
};

export default function OrderFilters({
  startDate,
  endDate,
  onChangeEndDate,
  onChangeStartDate,
  setSelectedOrders,
  setSelectedShipping,
  onChangeFilterStatus,
  selectedOrders,
  filterStatus,
  selectedShipping,
}: Props) {
  const [isDateRangeOpen, setIsDateRangeOpen] = useState(false);
  const [selectAll, setSelectAll] = useState(true);
  const [selectAllShipping, setSelectAllShipping] = useState(true);
  const OrderSource = ["LIVE", "WEB"];
  const [filterByRefund, setFilterByRefund] = useState("");
  const [compareOptions, setCompareOptions] = useState({
    previousPeriod: false,
    previousYear: false,
  });
  enum filterType {
    date = "date",
    period = "period"
  }
  const [localFilterStatus, setLocalFilterStatus] = useState<string>(
    filterType.period.toString()
  );

  const handleChange = (event: any) => {
    const { value } = event.target;
    setLocalFilterStatus(value);
    setFilterByRefund(value);
    setCompareOptions({
      previousPeriod: value === "period",
      previousYear: value === "date",
    });
    onChangeFilterStatus(value);
  };


  useEffect(() => {
    setSelectAll(selectedOrders.length === OrderSource.length);
  }, [selectedOrders]);


  useEffect(() => {
    setSelectedOrders([...OrderSource]);
    setSelectedShipping([]);
  }, []);

  const closeDateRange = () => {
    setIsDateRangeOpen(false);
  };

  const openDateRange = () => {
    setIsDateRangeOpen(true);
  };
  const handleChangeOrders = (
    event: SelectChangeEvent<typeof selectedOrders>
  ) => {
    const {
      target: { value },
    } = event;
    if (value.includes("all")) {
      if (selectAll) {
        setSelectedOrders([]);
        setSelectAll(false);
      } else {
        setSelectedOrders(OrderSource.slice(0, 2));
        setSelectAll(true);
      }
    } else {
      setSelectedOrders(typeof value === "string" ? value.split(",") : value);
    }
  };

  const handleChangeShipping = (
    event: SelectChangeEvent<typeof selectedShipping>
  ) => {
    const {
      target: { value },
    } = event;

    if (value.includes("all")) {
      if (selectAllShipping) {
        setSelectedShipping([]);
        setSelectAllShipping(false);
      } else {
        setSelectedShipping(Object.values(ShippingMethod));
        setSelectAllShipping(true);
      }
    } else {
      setSelectedShipping(typeof value === "string" ? value.split(",") : value);
    }
  };

  return (
    <Card
      sx={{ minHeight: "auto", display: "flex", alignItems: "center", px: 2 }}
    >
      <Card
        onClick={openDateRange}
        variant="outlined"
        sx={{
          height: 50,
          cursor: "pointer",
          borderRadius: 1,
          p: 3,
          display: "flex",
          boxShadow: "none",
          justifyContent: "space-between",
          alignItems: "center",
        }}
      >
        <Iconify sx={{ mr: 3 }} icon="solar:calendar-date-bold" />
        <ListItemText
          secondary={`${fDate(startDate)} - ${fDate(endDate)}`}
          primaryTypographyProps={{
            typography: "body2",
            color: "text.primary",
            mb: 0.5,
          }}
        />
      </Card>
      <CustomDateRangePicker
        open={isDateRangeOpen}
        onClose={closeDateRange}
        startDate={startDate}
        endDate={endDate}
        onChangeEndDate={(newValue: Date | null) => onChangeEndDate(newValue!)}
        onChangeStartDate={(newValue: Date | null) =>
          onChangeStartDate(newValue!)
        }
      />
      <FormControl sx={{ m: 1, width: 300 }}>
        <InputLabel>Source</InputLabel>
        <Select
          multiple
          value={selectedOrders}
          onChange={handleChangeOrders}
          input={<OutlinedInput sx={{ height: 50 }} label="Source" />}
          renderValue={(selected) => selected.join(", ")}
        >
          <MenuItem value="all">
            <Checkbox checked={selectAll} />
            Select All
          </MenuItem>
          {OrderSource.map((origine) => (
            <MenuItem key={origine} value={origine}>
              <Checkbox checked={selectedOrders.includes(origine)} />
              {origine}
            </MenuItem>
          ))}
        </Select>
      </FormControl>


      <FormControl sx={{ m: 1, width: 300 }}>
        <InputLabel>Expédition</InputLabel>
        <Select
          multiple
          value={selectedShipping}
          onChange={handleChangeShipping}
          input={<OutlinedInput sx={{ height: 50 }} label="Shipping" />}
          renderValue={(selected) => selected.join(", ")}
        >
          <MenuItem value="all">
            <Checkbox checked={selectAllShipping} />
            Select All
          </MenuItem>
          {Object.values(ShippingMethod).map((method) => (
            <MenuItem key={method} value={method}>
              <Checkbox
                checked={selectedShipping.includes(method)}
              />
              {method}
            </MenuItem>
          ))}
        </Select>
      </FormControl>

      <FormControl sx={{ m: 1, width: 300 }}>
        <InputLabel>Remboursement</InputLabel>
        <Select
          value={localFilterStatus}
          onChange={handleChange}
          input={<OutlinedInput sx={{ height: 50 }} label="Remboursement" />}
        >
          <MenuItem value="period">selon la période en cours</MenuItem>
          <MenuItem value="date">selon la date de la commande</MenuItem>
        </Select>
      </FormControl>
    </Card>
  );
}
