"use client";

import * as Yup from "yup";
import {
  Controller,
  useForm,
  UseFormReturn,
  FormProvider as Form,
} from "react-hook-form";
import { yupResolver } from "@hookform/resolvers/yup";
import { useMemo, useState } from "react";
import Chip from "@mui/material/Chip";
import Grid from "@mui/material/Unstable_Grid2";
import Card from "@mui/material/Card";
import Stack from "@mui/material/Stack";
import CardHeader from "@mui/material/CardHeader";
import Typography from "@mui/material/Typography";
import { TimePicker } from "@mui/x-date-pickers/TimePicker";
import { renderTimeViewClock } from "@mui/x-date-pickers/timeViewRenderers";
import { DatePicker } from "@mui/x-date-pickers/DatePicker";
import LoadingButton from "@mui/lab/LoadingButton";
import { Container } from "@mui/material";

import { useResponsive } from "@/hooks/use-responsive";
import { useSnackbar } from "@/components/snackbar";
import FormProvider, {
  RHFTextField,
  RHFAutocomplete,
} from "@/components/hook-form";
import { _tags } from "@/shared/_mock";
import OptionItem from "./add-option-item";
import { VideoFormatsField } from "./video-format";
import AddProduct from "./add-products";
import CustomBreadcrumbs from "@/components/custom-breadcrumbs";
import { paths } from "@/routes/paths";
import { useSettingsContext } from "@/shared/components/settings";
import { ILiveProduct } from "@/shared/types/live";
import { addLive } from "@/shared/api/live";
import { useRouter } from "next/navigation";
import ImageCrop from "@/components/image-crop";
import useAsync from "@/hooks/use-async";
import { add } from "lodash";
import { sendNotificationToUserAfterLiveCreation } from "@/shared/api/notif";
import { TokenPayload } from "@/shared/types/user";
import { getTokenInfo } from "@/utils/token";
import { getToken } from "firebase/messaging";
import { generateToken } from "@/utils/firebase";

export default function AddLiveView() {
  const mdUp = useResponsive("up", "md");
  const { enqueueSnackbar } = useSnackbar();
  const settings = useSettingsContext();
  const router = useRouter();

  const NewLiveSchema = Yup.object().shape({
    title: Yup.string().required("Title is required"),
    chatName: Yup.string().required("Moderator is required"),
    description: Yup.string().required("Description is required"),
    startDate: Yup.date().nullable(),
    keyWord: Yup.array().of(Yup.string()).required("Tags are required"),
    staging: Yup.boolean().required("Staging is required"),
    enableChat: Yup.boolean().required("Chat is required"),
    externalVideo: Yup.boolean().required(),
    enableLive: Yup.boolean().required("Cache is required"),
    videoFormat: Yup.string().required("Video format is required"),
  });

  const defaultValues = useMemo(
    () => ({
      title: "",
      chatName: "",
      description: "",
      startDate: null,
      keyWord: [],
      staging: false,
      enableChat: false,
      enableLive: false,
      externalVideo: true,
      videoFormat: "portrait",
    }),
    []
  );

  const methods = useForm({
    resolver: yupResolver(NewLiveSchema),
    defaultValues,
  });

  const {
    watch,
    control,
    setValue,
    reset,
    handleSubmit,
    formState: { isSubmitting },
  } = methods;

  const [products, setProducts] = useState<ILiveProduct[]>([]);
  const [image, setImage] = useState<File | null>(null);

  const values = watch();

  const addProduct = (product: ILiveProduct) => {
    setProducts([...products, product]);
  };

  const removeProduct = (product: ILiveProduct) => {
    setProducts(products.filter((p) => p.id !== product.id));
  };



  const [adding, setAdding] = useState(false);

  const onSubmit = handleSubmit((data) => {
    if (adding) return;
    const formData = {
      ...data,
      products
    };
    if (image) {
      setAdding(true);
      addLive(formData, image)
        .then(async (response) => {
          enqueueSnackbar("Live créé avec succès", { variant: "success" });
          reset();
          setProducts([]);
          setImage(null);
          setAdding(false);
          router.push(paths.dashboard.admin.live.info(response.id));
        })
        .catch((err) => {
          enqueueSnackbar(err.message, { variant: "error" });
          console.error(err);
          setAdding(false);
        });
    } else {
      enqueueSnackbar("Veuillez ajouter une image de couverture", {
        variant: "error",
      });
    }
  });

  const renderDetails = (
    <>
      {mdUp && (
        <Grid md={4}>
          <Typography variant="h6" sx={{ mb: 0.5 }}>
            Details
          </Typography>
          <Typography variant="body2" sx={{ color: "text.secondary" }}>
            Définissez le cadre de votre live : titre, modérateur, description
            et horaire pour une présentation complète
          </Typography>
        </Grid>
      )}

      <Grid xs={12} md={8}>
        <Card>
          {!mdUp && <CardHeader title="Details" />}
          <Stack spacing={3} sx={{ p: 3 }}>
            <RHFTextField name="title" label="titre du Live" />
            <RHFTextField
              name="description"
              label="Description"
              placeholder="Description..."
              multiline
              rows={4}
            />
            <RHFTextField
              name="chatName"
              label="Nom affiché dans le chat"
              multiline
            />
            <Stack spacing={2} direction={{ xs: "column", sm: "row" }}>
              <Controller
                name="startDate"
                control={control}
                render={({ field, fieldState: { error } }) => (
                  <DatePicker
                    label="Date de debut"
                    value={field.value}
                    onChange={(newValue) => field.onChange(newValue)}
                    slotProps={{
                      textField: {
                        fullWidth: true,
                        error: !!error,
                        helperText: error?.message,
                      },
                    }}
                  />
                )}
              />
              <Controller
                name="startDate"
                control={control}
                render={({ field, fieldState: { error } }) => (
                  <TimePicker
                    label="heure de debut"
                    value={field.value}
                    viewRenderers={{
                      hours: renderTimeViewClock,
                      minutes: renderTimeViewClock,
                      seconds: renderTimeViewClock,
                    }}
                    onChange={(newValue) => field.onChange(newValue)}
                    slotProps={{
                      textField: {
                        fullWidth: true,
                        error: !!error,
                        helperText: error?.message,
                      },
                    }}
                  />
                )}
              />
            </Stack>
            <RHFAutocomplete
              name="keyWord"
              label="mots clés"
              placeholder="+ mot clé"
              multiple
              freeSolo
              disableCloseOnSelect
              options={_tags}
              getOptionLabel={(option) => option}
              renderOption={(props, option) => (
                <li {...props} key={option}>
                  {option}
                </li>
              )}
              renderTags={(selected, getTagProps) =>
                selected.map((option, index) => (
                  <Chip
                    {...getTagProps({ index })}
                    key={option}
                    label={option}
                    size="small"
                    color="info"
                    variant="soft"
                  />
                ))
              }
            />
          </Stack>
        </Card>
      </Grid>
    </>
  );

  const optionsItems = [
    {
      title: "Source de vidéo externe",
      description:
        "Génère un lien pour votre logiciel de diffusion. Le lien sera disponible une fois le live crée",
      enabled: values.externalVideo,
      onToggle: () => setValue("externalVideo", !values.externalVideo),
    },
    {
      title: "Staging",
      description: "Ce live sera uniquement visible sur votre site de staging",
      enabled: values.staging,
      onToggle: () => setValue("staging", !values.staging),
    },
    {
      title: "Désactiver le chat",
      description: "Le chat sera accessible aux modérateur uniquement",
      enabled: values.enableChat,
      onToggle: () => setValue("enableChat", !values.enableChat),
    },
    {
      title: "Cacher ce live",
      description: "Ce live ne sera pas visible sur votre site",
      enabled: values.enableLive,
      onToggle: () => setValue("enableLive", !values.enableLive),
    },
  ];

  const renderOptions = (
    <>
      {mdUp && (
        <Grid xs={12} md={4}>
          <Typography variant="h6" sx={{ mb: 0.5 }}>
            Options
          </Typography>
          <Typography variant="body2" sx={{ color: "text.secondary" }}>
            Ajustez les paramètres de votre live : liens de diffusion,
            visibilité et interactions, pour un contrôle total.
          </Typography>
        </Grid>
      )}

      <Grid xs={12} md={8}>
        <Card>
          {!mdUp && <CardHeader title="Options" />}
          <Grid container spacing={2} sx={{ p: 3 }}>
            {optionsItems.map((item, index) => (
              <Grid key={index} xs={12} sm={6}>
                <OptionItem
                  title={item.title}
                  description={item.description}
                  onToggle={item.onToggle}
                  enabled={item.enabled}
                />
              </Grid>
            ))}
          </Grid>
        </Card>
      </Grid>
    </>
  );

  const renderProducts = (
    <>
      {mdUp && (
        <Grid md={4}>
          <Typography variant="h6" sx={{ mb: 0.5 }}>
            Produits
          </Typography>
          <Typography variant="body2" sx={{ color: "text.secondary" }}>
            Enrichissez votre live avec des produits, enregistrez et gérez la
            diffusion pour une expérience enrichie
          </Typography>
        </Grid>
      )}

      <Grid xs={12} md={8}>
        <Card>
          {!mdUp && <CardHeader title="Products" />}
          <Stack spacing={2} sx={{ p: 3 }}>
            <AddProduct
              onAdd={addProduct}
              onRemove={removeProduct}
              title="Produits"
              selectedItems={products}
            />
          </Stack>
        </Card>
      </Grid>
    </>
  );

  const renderVisuels = (
    <>
      {mdUp && (
        <Grid xs={12} md={4}>
          <Typography variant="h6" sx={{ mb: 0.5 }}>
            Visuels
          </Typography>
          <Typography variant="body2" sx={{ color: "text.secondary" }}>
            Personnalisez l'esthétique de votre live avec une photo de
            couverture et le format vidéo parfait.
          </Typography>
        </Grid>
      )}

      <Grid xs={12} md={8}>
        <Card sx={{ paddingBottom: 5 }}>
          {!mdUp && <CardHeader title="Visuel" />}
          <Stack spacing={2} sx={{ p: 3 }}>
            <Typography variant="subtitle1">Format vidéo</Typography>
            <Controller
              name="videoFormat"
              control={control}
              render={({ field }) => <VideoFormatsField field={field} />}
            />
          </Stack>
          <Stack spacing={2} sx={{ p: 3, paddingBottom: 10 }}>
            <Typography variant="subtitle1">Photo de couverture</Typography>
            <ImageCrop editedImage={image} setEditedImage={setImage} />
          </Stack>
        </Card>
      </Grid>
    </>
  );


  const renderActions = (
    <>
      {mdUp && <Grid md={4} />}
      <Grid
        xs={12}
        md={8}
        sx={{ display: "flex", justifyContent: "end", py: 4 }}
      >
        <LoadingButton
          type="submit"
          variant="contained"
          size="large"
          loading={adding}
          disabled={adding}
          sx={{ ml: 2 }}
        >
          Enregistrer
        </LoadingButton>
      </Grid>
    </>
  );

  return (
    <Container maxWidth={settings.themeStretch ? false : "lg"}>
      <CustomBreadcrumbs
        heading="Créer un nouveau live"
        rollbackLink={paths.dashboard.admin.live.all_lives}
        links={[
          {
            name: "Tableau de bord",
            href: paths.dashboard.root,
          },
          {
            name: "Lives",
            href: paths.dashboard.admin.live.all_lives,
          },
          {
            name: "création des lives",
          },
        ]}
        sx={{ mb: { xs: 3, md: 5 } }}
      />
      <FormProvider onSubmit={onSubmit} methods={methods}>
        <Grid container spacing={3}>
          {renderDetails}
          {renderOptions}
          {renderVisuels}
          {renderProducts}
          {renderActions}
        </Grid>
      </FormProvider>
    </Container>
  );
}



