// PaginatedProductsDialog.tsx

import React, { useEffect, useState, useCallback, ReactNode } from 'react';
import {
  Card,
  CardContent,
  CardMedia,
  Typography,
  Box,
  Button,
  Dialog,
  Pagination,
  CircularProgress,
  Grid,
  MenuItem,
  Select,
  FormControl,
  TextField,
  Checkbox,
  FormControlLabel,
  SelectChangeEvent,
  Avatar,
  List,
  ListItem,
  ListItemAvatar,
  ListItemText,
  styled,
} from '@mui/material';
import { debounce } from 'lodash';
import { getPaginatedProductsPromoCodes } from '@/shared/api/promoCode';
import { IProductListItem } from '@/shared/types/live';
import { DEFAULT_IMAGE_URL } from '@/config-global';
import { ILiveProduct, LiveProductStatus } from '@/shared/types/live';
import { useSearchProductsLive } from '@/shared/api/live';

const StyledList = styled(List)({
  overflowY: 'auto',
  maxHeight: 'calc(100% - 64px)',
  '&::-webkit-scrollbar': {
    width: '10px',
  },
  '&::-webkit-scrollbar-track': {
    boxShadow: 'inset 0 0 5px grey',
    borderRadius: '10px',
  },
  '&::-webkit-scrollbar-thumb': {
    background: 'darkgrey',
    borderRadius: '10px',
  },
  '&::-webkit-scrollbar-thumb:hover': {
    background: 'transparent',
  }
});


type ProductCardProps = {
  product: ILiveProduct;
  isChecked: boolean;
  handleToggle: (product: ILiveProduct) => void;
}

const ProductCard = ({ product, isChecked, handleToggle }: ProductCardProps) => (
  <Card variant="outlined" key={product.id} sx={{ marginBottom: 2 }}>
    <ListItem
      secondaryAction={
        <Checkbox
          edge="end"
          checked={isChecked}
          onChange={() => handleToggle(product)}
          inputProps={{ 'aria-labelledby': `checkbox-list-secondary-label-${product.id}` }}
        />
      }
      disablePadding
    >
      <ListItemAvatar>
        <Avatar variant="rounded" alt={product.title } sx={{ height: 100, width: 100 }} src={product.image} />
      </ListItemAvatar>
      <ListItem sx={{ display: 'flex', textOverflow: '', flexDirection: 'column', alignItems: 'start', gap: 1 }}>
        <ListItemText primary={product.title } />
        <ListItemText secondary={product.sku} />
        {product.price && <ListItemText primary={`Prix: ${product.price}`} />}
      </ListItem>
    </ListItem>
  </Card>
);


interface PaginatedProductsDialogProps {
  onAdd: (products: ILiveProduct) => void;
  onRemove: (products: ILiveProduct) => void;
  title: string;
  selectedItems: ILiveProduct[];
}

const PaginatedProductsDialog: React.FC<PaginatedProductsDialogProps> = ({
  onAdd,
  onRemove,
  title,
  selectedItems = [],
}) => {
  const [open, setOpen] = useState(false);
  const [page, setPage] = useState(1); // Start from 1 for better UX
  const [size, setSize] = useState(12);
  const [keyword, setKeyword] = useState('');

  // Fetch Data
  const { productsData, productsLoading, totalPages } = useSearchProductsLive(
    page - 1,
    size,
    keyword
  );
  // Handlers
  const handlePageChange = (_: React.ChangeEvent<unknown>, value: number) => {
    setPage(value);
  };

  const handleSizeChange = (newSize: number) => {
    setSize(newSize);
    setPage(1);
  };

  const debouncedKeywordChange = useCallback(
    debounce((value: string) => {
      setKeyword(value.length >= 3 ? value : '');
      setPage(1);
    }, 300),
    []
  );

  const handleKeywordChange = (value: string) => {
    debouncedKeywordChange(value);
  };

  const handleOpen = () => setOpen(true);
  const handleClose = () => {
    setOpen(false);
  };

  // Selection Handlers
  const handleSelect = (product: IProductListItem) => {
    if (selectedItems.map((item) => item.productId).includes(product.id.toString())) {
      onRemove({
        id: product.id.toString(),
        title: product.name,
        sku: product.ugs,
        productId: product.id.toString(),
        image: product.mainImagePath,
        price: product.regularPrice as number,
        isPresented: false,
        status: LiveProductStatus.NOT_PRESENTED,
        timeStampProduct: []
      });
    } else {
      onAdd({
        id: product.id.toString(),
        title: product.name,
        sku: product.ugs,
        productId: product.id.toString(),
        image: product.mainImagePath,
        price: product.regularPrice as number,
        isPresented: false,
        status: LiveProductStatus.NOT_PRESENTED,
        timeStampProduct: []
      });
    }
  };


  // Fallback image handler
  const getImageSrc = (path: string) => path || DEFAULT_IMAGE_URL;

  return (
    <>
      <Box>
        <Button variant="outlined" onClick={handleOpen} fullWidth>
          {title}
        </Button>
        <Dialog open={open} onClose={handleClose} maxWidth="md" fullWidth>
          {/* Dialog Title */}
          <Typography variant="h6" align="center" mt={2}>
            {title}
          </Typography>

          {/* Search and Filter */}
          <SearchAndFilter
            size={size}
            onSizeChange={handleSizeChange}
            onKeywordChange={handleKeywordChange}
          />

          {/* Content Area */}
          <Box p={2}>
            {productsLoading ? (
              <LoadingIndicator />
            ) : (
              <>
                {productsData.length > 0 ? (
                  <ProductsGrid
                    items={productsData}
                    selectedItems={selectedItems}
                    onSelect={handleSelect}
                    getImageSrc={getImageSrc}
                  />
                ) : (
                  <EmptyState message="Aucun produit trouvé" />
                )}
                <Pagination
                  count={totalPages}
                  page={page}
                  onChange={handlePageChange}
                  color="primary"
                  sx={{ display: 'flex', justifyContent: 'center', mt: 2 }}
                />
              </>
            )}
          </Box>

        </Dialog>
      </Box>
      <StyledList dense sx={{
        overflowY: 'auto',
        maxHeight: 'calc(100% - 64px)',
        p: 2
      }}>
        {selectedItems.map((product) => {
          return (
            <ProductCard key={product.id} product={product} isChecked={true} handleToggle={onRemove} />
          );
        })}
      </StyledList>
    </>
  );
};

export default PaginatedProductsDialog;

/* ---------------------------- Sub-Components Below ---------------------------- */

/**
 * SearchAndFilter Component
 * Handles the search input and size selection.
 */
interface SearchAndFilterProps {
  size: number;
  onSizeChange: (newSize: number) => void;
  onKeywordChange: (value: string) => void;
}

const SearchAndFilter: React.FC<SearchAndFilterProps> = ({
  size,
  onSizeChange,
  onKeywordChange,
}) => {
  const handleSizeSelect = (event: React.ChangeEvent<{ value: unknown }>) => {
    onSizeChange(event.target.value as number);
  };

  const handleSearchInput = (event: React.ChangeEvent<HTMLInputElement>) => {
    onKeywordChange(event.target.value.trim());
    console.log(event.target.value.trim());
  };

  return (
    <Box display="flex" justifyContent="space-between" p={2}>
      <TextField
        label="Rechercher"
        fullWidth
        onChange={handleSearchInput}
        sx={{ flex: 1, mr: 2 }}
      />
      <FormControl sx={{ minWidth: 80 }}>
        <Select value={size} onChange={handleSizeSelect as (event: SelectChangeEvent<number>, child: ReactNode) => void}>
          {[12, 24, 36].map((option) => (
            <MenuItem key={option} value={option}>
              {option}
            </MenuItem>
          ))}
        </Select>
      </FormControl>
    </Box>
  );
};

/**
 * ProductsGrid Component
 * Renders the grid of products using ProductCard.
 */
interface ProductsGridProps {
  items: IProductListItem[];
  selectedItems: ILiveProduct[];
  onSelect: (product: IProductListItem) => void;
  getImageSrc: (path: string) => string | undefined;
}

const ProductsGrid: React.FC<ProductsGridProps> = ({
  items,
  selectedItems,
  onSelect,
  getImageSrc,
}) => {
  return (
    <Grid container spacing={2}>
      {items.map((item) => (
        <Grid item xs={12} sm={6} md={4} key={item.id}>
          <PopupProductCard
            item={item}
            isSelected={selectedItems.some((selected) => selected.productId == item.id)}
            onSelect={onSelect}
            getImageSrc={getImageSrc as (path: string) => string}
          />
        </Grid>
      ))}
    </Grid>
  );
};

/**
 * ProductCard Component
 * Displays individual product details with a checkbox.
 */
interface PopupProductCardProps {
  item: IProductListItem;
  isSelected: boolean;
  onSelect: (product: IProductListItem) => void;
  getImageSrc: (path: string) => string;
}

const PopupProductCard: React.FC<PopupProductCardProps> = ({
  item,
  isSelected,
  onSelect,
  getImageSrc,
}) => {
  const handleCheckboxChange = () => {
    onSelect(item);
  };

  const handleImageError = (e: React.SyntheticEvent<HTMLImageElement, Event>) => {
    e.currentTarget.src = DEFAULT_IMAGE_URL as string;
  };

  return (
    <Card sx={{ position: 'relative', height: '100%', display: 'flex', flexDirection: 'column' }}>
      {/* Checkbox Overlay */}
      <FormControlLabel
        control={
          <Checkbox
            checked={isSelected}
            onChange={handleCheckboxChange}
            color="primary"
            sx={{ position: 'absolute', top: 8, left: 8, backgroundColor: 'rgba(255,255,255,0.7)', borderRadius: '50%' }}
          />
        }
        label=""
      />
      {/* Product Image */}
      <CardMedia
        component="img"
        height="140"
        src={getImageSrc(item.mainImagePath)}
        alt={item.name}
        onError={handleImageError}
      />
      {/* Product Details */}
      <CardContent>
        <Typography variant="h6">{item.name}</Typography>
        <Typography color="text.secondary">{item.ugs}</Typography>
      </CardContent>
    </Card>
  );
};

/**
 * LoadingIndicator Component
 * Shows a loading spinner.
 */
const LoadingIndicator: React.FC = () => (
  <Grid container justifyContent="center">
    <CircularProgress />
  </Grid>
);

/**
 * EmptyState Component
 * Displays a message when no products are found.
 */
interface EmptyStateProps {
  message: string;
}

const EmptyState: React.FC<EmptyStateProps> = ({ message }) => (
  <Grid item xs={12} display="flex" justifyContent="center" my={2}>
    <Typography color="text.secondary">{message}</Typography>
  </Grid>
);
