"use client";

import * as Yup from 'yup';
import { Controller, useForm } from 'react-hook-form';
import { yupResolver } from '@hookform/resolvers/yup';
import Chip from '@mui/material/Chip';
import Grid from '@mui/material/Unstable_Grid2';
import Card from '@mui/material/Card';
import Stack from '@mui/material/Stack';
import CardHeader from '@mui/material/CardHeader';
import Typography from '@mui/material/Typography';

import { useResponsive } from '@/hooks/use-responsive';
import { useSnackbar } from '@/components/snackbar';
import FormProvider, { RHFTextField, RHFAutocomplete } from '@/components/hook-form';
import { _tags } from '@/shared/_mock';
import { TimePicker } from '@mui/x-date-pickers/TimePicker';
import { DatePicker } from '@mui/x-date-pickers/DatePicker';
import OptionItem from './add-option-item';
import LoadingButton from '@mui/lab/LoadingButton';
import { VideoFormatsField } from './video-format';
import CustomBreadcrumbs from '@/components/custom-breadcrumbs';
import { paths } from '@/routes/paths';
import { useSettingsContext } from '@/shared/components/settings';
import { Container } from '@mui/material';
import { ILive, ILiveProduct, LiveStatus } from '@/shared/types/live';

import { editLive, useGetProductsLive } from '@/shared/api/live';
import { useEffect, useMemo, useState } from 'react';
import { useLiveData } from '@/contexts/live-details';
import CopyButton from '@/components/copy-button';
import ImageCrop from '@/components/image-crop/index';

export default function EditLiveView() {
    const liveData = useLiveData();
    const mdUp = useResponsive('up', 'md');
    const { enqueueSnackbar } = useSnackbar();
    const settings = useSettingsContext();
    const [editLoding, setEditLoading] = useState(false);

    const NewLiveSchema = Yup.object().shape({
        title: Yup.string().required('Title is required'),
        chatName: Yup.string().required('Moderator is required'),
        description: Yup.string().required('Description is required'),
        startDate: Yup.date().required('Start date is required'),
        keyWord: Yup.array().of(Yup.string()).required('Tags are required'),
        staging: Yup.boolean().required('Staging is required'),
        enableChat: Yup.boolean().required('Chat is required'),
        externalVideo: Yup.boolean().required(),
        enableLive: Yup.boolean().required('Cache is required'),
        videoFormat: Yup.string().required('Video format is required'),
    });

    const { products, productsLoading } = useGetProductsLive(liveData.id);
    const [image, setImage] = useState<File | null>(null);
    const [liveProducts, setLiveProducts] = useState<ILiveProduct[]>(products || []);

    useEffect(() => {
        setLiveProducts(products || []);
    }, [productsLoading]);

    const defaultValues = useMemo(() => ({
        title: liveData.title,
        chatName: liveData.chatName,
        description: liveData.description,
        startDate: new Date(liveData.startDate),
        keyWord: liveData.keyWord,
        staging: liveData.staging,
        enableChat: liveData.enableChat,
        enableLive: liveData.enableLive,
        externalVideo: true,
        videoFormat: liveData.videoFormat,
    }), [liveData]);

    const methods = useForm({
        resolver: yupResolver(NewLiveSchema),
        defaultValues,
    });

    const {
        watch,
        control,
        setValue,
        handleSubmit,
        formState: { isSubmitting },
    } = methods;

    const values = watch();



    const onSubmit = handleSubmit((data) => {
        if (editLoding) return;
        setEditLoading(true);
        const formData = {
            ...liveData,
            ...data
        };

        editLive(liveData.id, formData, image)
            .then(() => {
                enqueueSnackbar('live modifié avec succés', { variant: 'success' });
                setImage(null);
                setEditLoading(false);
            })
            .catch((err) => {
                enqueueSnackbar(err.message, { variant: 'error' });
                console.error(err);
                setEditLoading(false);
            });
    });

    const renderConnextionDetails = (
        <>
            {mdUp && (
                <Grid md={4}>
                    <Typography variant="h6" sx={{ mb: 0.5 }}>
                        Connexion
                    </Typography>
                    <Typography variant="body2" sx={{ color: 'text.secondary' }}>
                        Définissez le cadre de votre live : titre, modérateur, description et horaire pour une présentation complète
                    </Typography>
                </Grid>
            )}

            <Grid xs={12} md={8}>
                <Card>
                    {!mdUp && <CardHeader title="Connexion" />}
                    <Grid container spacing={2} sx={{ p: 3 }}>
                        <Grid xs={12} sm={6}>
                            <Card sx={{ p: 2 }}>
                                <Typography variant="subtitle1" gutterBottom>
                                    Clé de diffusion
                                </Typography>
                                <Typography variant="body2" gutterBottom>
                                    {liveData.keysWork?.keyStream}
                                    <CopyButton sx={{ position: 'absolute', right: 3, top:1 }} value={liveData.keysWork?.keyStream as string} />
                                </Typography>
                            </Card>
                        </Grid>
                        <Grid xs={12} sm={6}>
                            <Card sx={{ p: 2 }}>
                                <Typography variant="subtitle1" gutterBottom>
                                    adresse du serveur rtmp
                                </Typography>
                                <Typography  variant="body2" gutterBottom>
                                    {liveData.keysWork?.rtmpServerAddress}
                                    <CopyButton sx={{ position: 'absolute', right: 3, top:1 }} value={liveData.keysWork?.rtmpServerAddress as string} />
                                </Typography>
                            </Card>
                        </Grid>
                        <Grid xs={12} sm={6}>
                            <Card sx={{ p: 2 }}>
                                <Typography variant="subtitle1" gutterBottom>
                                    adresse du serveur hls
                                </Typography>
                                <Typography variant="body2" gutterBottom>
                                    {liveData.keysWork?.hlsServerAddress}
                                    <CopyButton sx={{ position: 'absolute', right: 3, top:1 }} value={liveData.keysWork?.hlsServerAddress as string} />
                                </Typography>
                            </Card>
                        </Grid>
                    </Grid>
                </Card>
            </Grid>
        </>
    );

    const renderDetails = (
        <>
            {mdUp && (
                <Grid md={4}>
                    <Typography variant="h6" sx={{ mb: 0.5 }}>
                        Details
                    </Typography>
                    <Typography variant="body2" sx={{ color: 'text.secondary' }}>
                        Définissez le cadre de votre live : titre, modérateur, description et horaire pour une présentation complète
                    </Typography>
                </Grid>
            )}

            <Grid xs={12} md={8}>
                <Card>
                    {!mdUp && <CardHeader title="Details" />}
                    <Stack spacing={3} sx={{ p: 3 }}>
                        <RHFTextField name="title" label="titre du Live"/>
                        <RHFTextField name="description" label="Description" placeholder="Description..." multiline rows={4}  />
                        <RHFTextField name="chatName" label="Nom affiché dans le chat" multiline  />
                        <Stack spacing={2} direction={{ xs: 'column', sm: 'row' }}>
                            <Controller
                                name="startDate"
                                control={control}
                                render={({ field, fieldState: { error } }) => (
                                    <DatePicker
                                        label="Date de debut"
                                        value={field.value}
                                        onChange={(newValue) => {
                                            field.onChange(newValue);
                                        }}
                                        slotProps={{
                                            textField: {
                                                fullWidth: true,
                                                error: !!error,
                                                helperText: error?.message,
                                            },
                                        }}
                                        
                                    />
                                )}
                            />
                            <Controller
                                name="startDate"
                                control={control}
                                render={({ field, fieldState: { error } }) => (
                                    <TimePicker
                                        label="heure de debut"
                                        value={field.value}
                                        onChange={(newValue) => {
                                            field.onChange(newValue);
                                        }}
                                        slotProps={{
                                            textField: {
                                                fullWidth: true,
                                                error: !!error,
                                                helperText: error?.message,
                                            },
                                        }}
                                        
                                    />
                                )}
                            />
                        </Stack>
                        <RHFAutocomplete
                            name="keyWord"
                            label="mots clés"
                            placeholder="+ mot clé"
                            multiple
                            freeSolo
                            disableCloseOnSelect
                            options={_tags.map((option) => option)}
                            getOptionLabel={(option) => option}
                            renderOption={(props, option) => (
                                <li {...props} key={option}>
                                    {option}
                                </li>
                            )}
                            renderTags={(selected, getTagProps) =>
                                selected.map((option, index) => (
                                    <Chip
                                        {...getTagProps({ index })}
                                        key={option}
                                        label={option}
                                        size="small"
                                        color="info"
                                        variant="soft"
                                    />
                                ))
                            }
                            
                        />
                    </Stack>
                </Card>
            </Grid>
        </>
    );

    const optionsItems = [
        {
            title: "Source de vidéo externe",
            description: "Génère un lien pour votre logiciel de diffusion. Le lien sera disponible une fois le live crée",
            enabled: values.externalVideo,
            onToggle: () => setValue('externalVideo', !values.externalVideo as never),
        },
        {
            title: "Staging",
            description: "Ce live sera uniquement visible sur votre site de staging",
            enabled: values.staging,
            onToggle: () => setValue("staging", !values.staging as never),
        },
        {
            title: "Désactiver le chat",
            description: "Le chat sera accessible aux modérateur uniquement",
            enabled: values.enableChat,
            onToggle: () => setValue('enableChat', !values.enableChat as never),
        },
        {
            title: "Cacher ce live",
            description: "Ce live ne sera pas visible sur votre site",
            enabled: values.enableLive,
            onToggle: () => setValue('enableLive', !values.enableLive as never),
        },
    ];

    const renderOptions = (
        <>
            {mdUp && (
                <Grid xs={12} md={4}>
                    <Typography variant="h6" sx={{ mb: 0.5 }}>
                        Options
                    </Typography>
                    <Typography variant="body2" sx={{ color: 'text.secondary' }}>
                        Ajustez les paramètres de votre live : liens de diffusion, visibilité et interactions, pour un contrôle total.
                    </Typography>
                </Grid>
            )}
            <Grid xs={12} md={8}>
                <Card>
                    {!mdUp && <CardHeader title="Options" />}
                    <Grid container spacing={2} sx={{ p: 3 }}>
                        {optionsItems.map((item, index) => (
                            <Grid key={index} xs={12} sm={6}>
                                <OptionItem
                                    title={item.title}
                                    description={item.description}
                                    onToggle={item.onToggle}
                                    enabled={item.enabled}
                                />
                            </Grid>
                        ))}
                    </Grid>
                </Card>
            </Grid>
        </>
    );

    const renderVisuels = (
        <>
            {mdUp && (
                <Grid xs={12} md={4}>
                    <Typography variant="h6" sx={{ mb: 0.5 }}>
                        Visuels
                    </Typography>
                    <Typography variant="body2" sx={{ color: 'text.secondary' }}>
                        Personnalisez l'esthétique de votre live avec une photo de couverture et le format vidéo parfait.
                    </Typography>
                </Grid>
            )}
            <Grid xs={12} md={8}>
                <Card>
                    {!mdUp && <CardHeader title="Visuel" />}
                    <Stack spacing={2} sx={{ p: 3 }}>
                        <Typography variant="subtitle1">Format vidéo</Typography>
                        <Controller
                            name="videoFormat"
                            control={control}
                            render={({ field }) => (
                                <VideoFormatsField field={field}  />
                            )}
                        />
                    </Stack>
                    <Stack spacing={2} sx={{ p: 3 }}>
                        <Typography variant="subtitle1">Photo de couverture</Typography>
                        <ImageCrop imageToEdit={liveData.image as string}  editedImage={image} setEditedImage={setImage}/>
                    </Stack>
                </Card>
            </Grid>
        </>
    );

    const renderActions = (
        <>
            {mdUp && <Grid md={4} />}
            <Grid xs={12} md={8} sx={{ display: 'flex', justifyContent: 'end', py: 4 }}>
*
                    <LoadingButton
                        type="submit"
                        variant="contained"
                        size="large"
                        loading={editLoding}
                        disabled={editLoding}
                        sx={{ ml: 2 }}
                    >
                        Enregistrer
                    </LoadingButton>
            </Grid>
        </>
    );

    return (
        <Container maxWidth={settings.themeStretch ? false : 'lg'}>
            <CustomBreadcrumbs
                heading={`Modifier live #${liveData.id}`}
                rollbackLink={paths.dashboard.admin.live.info(liveData.id)}
                links={[
                    { name: 'Tableau de bord', href: paths.dashboard.root },
                    { name: 'Lives', href: paths.dashboard.admin.live.all_lives },
                    { name: `Modifier live #${liveData.id}` },
                ]}
                sx={{ mb: { xs: 3, md: 5 } }}
            />
            <FormProvider methods={methods} onSubmit={onSubmit}>
                <Grid container spacing={3}>
                    {renderConnextionDetails}
                    {renderDetails}
                    {renderOptions}
                    {renderVisuels}
                    {renderActions}
                </Grid>
            </FormProvider>
        </Container>
    );
}
