import Avatar from '@mui/material/Avatar';
import MenuItem from '@mui/material/MenuItem';
import TableRow from '@mui/material/TableRow';
import TableCell from '@mui/material/TableCell';
import IconButton from '@mui/material/IconButton';
import ListItemText from '@mui/material/ListItemText';
import { ConfirmDialog } from "@/shared/components/custom-dialog";
import { useSnackbar } from '@/components/snackbar';

import { useBoolean } from '@/hooks';
import { fDate, fTime } from '@/utils/format-time';

import Label from '@/shared/components/label';
import Iconify from '@/shared/components/iconify';
import CustomPopover, { usePopover } from '@/shared/components/custom-popover';
import { ILiveItem, LiveStatus } from '@/shared/types/live';
import { archiveLive, deleteLive, publishLive, restoreLive } from '@/shared/api/live';
import LoadingButton from '@mui/lab/LoadingButton';
import useAsync from '@/hooks/use-async';

type Props = {
  row: ILiveItem;
  selected: boolean;
  onViewRow: VoidFunction;
  onDetails: VoidFunction;
  onEditRow: VoidFunction;
  onViewStats: VoidFunction;
};

export default function LiveTableRow({
  row,
  onViewRow,
  onDetails,
  onEditRow,
  onViewStats
}: Props) {
  const { title, description, startDate, status, image } = row;
  const popover = usePopover();
  const openDetails = useBoolean();
  const deleteConfirm = useBoolean();
  const publishConfirm = useBoolean();
  const archiveConfirm = useBoolean();
  const restoreConfirm = useBoolean();
  const { enqueueSnackbar } = useSnackbar();

  const asyncDeleteLive = useAsync(async () => {
    await deleteLive(row.id);
  });

  const asyncPublishLive = useAsync(async () => {
    await publishLive(row.id);
  });

  const asyncArchiveLive = useAsync(async () => {
    await archiveLive(row.id);
  });

  const asyncRestoreLive = useAsync(async () => {
    await restoreLive(row.id);
  });

  const handleDelete = () => {
    asyncDeleteLive.execute()
      .then(() => {
        deleteConfirm.onFalse();
        enqueueSnackbar('live supprimé avec succès', { variant: 'success' });
      })
      .catch(() => {
        enqueueSnackbar('erreur lors de la suppression', { variant: 'error' });
      });
  };

  const handlePublish = () => {
    asyncPublishLive.execute()
      .then(() => {
        publishConfirm.onFalse();
        enqueueSnackbar('live publié avec succès', { variant: 'success' });
      })
      .catch(() => {
        enqueueSnackbar('erreur lors de la publication', { variant: 'error' });
      });
  };

  const handleArchive = () => {
    asyncArchiveLive.execute()
      .then(() => {
        archiveConfirm.onFalse();
        enqueueSnackbar('live archivé avec succès', { variant: 'success' });
      })
      .catch(() => {
        enqueueSnackbar('erreur lors de l\'archivation', { variant: 'error' });
      });
  };

  const handleUnarchive = () => {
    asyncRestoreLive.execute()
      .then(() => {
        restoreConfirm.onFalse();
        enqueueSnackbar('live désarchivé avec succès', { variant: 'success' });
      })
      .catch(() => {
        enqueueSnackbar('erreur lors de la désarchivation', { variant: 'error' });
      });
  };

  return (
    <>
      <TableRow hover>
        <TableCell sx={{ cursor: 'pointer' }} onClick={onDetails}>
          <Avatar
            alt={title}
            src={image}
            variant="rounded"
            sx={{ width: 64, height: 64, mr: 2 }}
          />
        </TableCell>

        <TableCell sx={{ cursor: 'pointer' }} onClick={onDetails}>
          {title}
        </TableCell>

        <TableCell sx={{ cursor: 'pointer' }} onClick={onDetails}>
          <ListItemText
            primary={fDate(startDate)}
            primaryTypographyProps={{ typography: 'body2', noWrap: true }}
            secondaryTypographyProps={{
              mt: 0.5,
              component: 'span',
              typography: 'caption',
            }}
          />
        </TableCell>

        <TableCell sx={{ cursor: 'pointer' }} onClick={onDetails}>
          <ListItemText
            primary={fTime(startDate)}
            primaryTypographyProps={{ typography: 'body2', noWrap: true }}
            secondaryTypographyProps={{
              mt: 0.5,
              component: 'span',
              typography: 'caption',
            }}
          />
        </TableCell>

        <TableCell sx={{ cursor: 'pointer' }} onClick={onDetails}>{description}</TableCell>
        <TableCell sx={{ cursor: 'pointer' }} onClick={onDetails}>
          <Label
            variant="soft"
            color={
              (status == LiveStatus.REPLAY && 'success') ||
              (status == LiveStatus.ONGOING && 'warning') ||
              (status == LiveStatus.COMING && 'error') ||
              (status == LiveStatus.COMING && 'primary') ||
              (status == LiveStatus.REVIEW && 'info') || 'default'
            }
          >
            {
              (status == LiveStatus.REPLAY && 'rediffusion') ||
              (status == LiveStatus.ONGOING && 'en cours') ||
              (status == LiveStatus.COMING && 'à venir') ||
              (status == LiveStatus.REVIEW && 'en revue') ||
              (status == LiveStatus.ARCHIVED && 'archivé')
            }
          </Label>
        </TableCell>

        <TableCell align="right" sx={{ px: 1 }}>
          <IconButton color={popover.open ? 'inherit' : 'default'} onClick={popover.onOpen}>
            <Iconify icon="eva:more-vertical-fill" />
          </IconButton>
        </TableCell>
      </TableRow>

      <CustomPopover
        open={popover.open}
        onClose={popover.onClose}
        arrow="right-top"
        sx={{ width: 160 }}
      >
        <MenuItem
          onClick={() => {
            onViewRow();
            openDetails.onTrue();
            popover.onClose();
          }}
        >
          <Iconify icon="solar:eye-bold" />
          voir
        </MenuItem>
        <MenuItem
          onClick={() => {
            onDetails();
            openDetails.onTrue();
            popover.onClose();
          }}
        >
          <Iconify icon="gg:details-more" />
          détails
        </MenuItem>
        {status === LiveStatus.REVIEW && (
          <MenuItem
            onClick={() => {
              popover.onClose();
              publishConfirm.onTrue();
            }}
            style={{ outline: 'none', color: 'blue' }}
          >
            <Iconify icon="ic:baseline-publish" />
            publier
          </MenuItem>
        )}
        <MenuItem
          onClick={() => {
            onEditRow();
            popover.onClose();
          }}
          style={{ outline: 'none' }}
        >
          <Iconify icon="solar:pen-bold" />
          Modifier
        </MenuItem>
        {row.status !== LiveStatus.COMING && (
          <MenuItem
            onClick={() => {
              onViewStats();
              popover.onClose();
            }}
            style={{ outline: 'none' }}
          >
            <Iconify icon="icomoon-free:stats-bars" />
            Stats
          </MenuItem>
        )}
        <MenuItem
          onClick={() => {
            popover.onClose();
            deleteConfirm.onTrue();
          }}
          style={{ outline: 'none', color: 'red' }}
        >
          <Iconify icon="ic:round-delete" />
          Supprimer
        </MenuItem>

        {(status === LiveStatus.REPLAY || status === LiveStatus.REVIEW) && (
          <MenuItem
            onClick={() => {
              popover.onClose();
              archiveConfirm.onTrue();
            }}
            style={{ outline: 'none', color: 'orange' }}
          >
            <Iconify icon="system-uicons:archive" />
            archiver
          </MenuItem>
        )}

        {status === LiveStatus.ARCHIVED && (
          <MenuItem
            onClick={() => {
              popover.onClose();
              restoreConfirm.onTrue();
            }}
            style={{ outline: 'none', color: 'orange' }}
          >
            <Iconify icon="system-uicons:archive" />
            restorer
          </MenuItem>
        )}
      </CustomPopover>
      <ConfirmDialog
        open={deleteConfirm.value}
        onClose={deleteConfirm.onFalse}
        title="Supprimer"
        content="Voulez-vous vraiment supprimer ce live?"
        action={
          <LoadingButton loading={asyncDeleteLive.loading} variant="contained" color="error" onClick={handleDelete}>
            Supprimer
          </LoadingButton>
        }
      />
      <ConfirmDialog
        open={publishConfirm.value}
        onClose={publishConfirm.onFalse}
        title="publier"
        content="Voulez-vous vraiment publier ce live?"
        action={
          <LoadingButton loading={asyncPublishLive.loading} variant="contained" color="info" onClick={handlePublish}>
            publier
          </LoadingButton>
        }
      />
      <ConfirmDialog
        open={archiveConfirm.value}
        onClose={archiveConfirm.onFalse}
        title="archiver"
        content="Voulez-vous vraiment archiver ce live?"
        action={
          <LoadingButton loading={asyncArchiveLive.loading} variant="contained" color="warning" onClick={handleArchive}>
            archiver
          </LoadingButton>
        }
      />

      <ConfirmDialog
        open={restoreConfirm.value}
        onClose={restoreConfirm.onFalse}
        title="restorer"
        content="Voulez-vous vraiment restorer ce live?"
        action={
          <LoadingButton loading={asyncRestoreLive.loading} variant="contained" color="warning" onClick={handleUnarchive}>
            restorer
          </LoadingButton>
        }
      />
    </>
  );
}
