"use client";

import { useState, useCallback, useMemo } from 'react';

import Tab from '@mui/material/Tab';
import Tabs from '@mui/material/Tabs';
import Card from '@mui/material/Card';
import Table from '@mui/material/Table';
import Stack from '@mui/material/Stack';
import Button from '@mui/material/Button';
import Container from '@mui/material/Container';
import TableBody from '@mui/material/TableBody';
import { alpha, useTheme } from '@mui/material/styles';
import TableContainer from '@mui/material/TableContainer';
import LocalizationProvider from '@/shared/locales/localization-provider';
import { paths } from '@/routes/paths';
import { useRouter } from '@/hooks';
import RouterLink from '@/routes/router-link';
import { _lives } from '@/shared/_mock';
import Label, { LabelColor } from '@/shared/components/label';
import Iconify from '@/shared/components/iconify';
import Scrollbar from '@/shared/components/scrollbar';
import { useSettingsContext } from '@/shared/components/settings';
import CustomBreadcrumbs from '@/components/custom-breadcrumbs';
import {
  useTable,
  emptyRows,
  TableNoData,
  TablePaginationCustom,
  TableEmptyRows,
} from '@/shared/components/table';
import { ILiveTableFilter, ILiveTableFilterValues } from '@/shared/types/live';
import { LiveStatus } from '@/shared/types/live';
import LiveTableRow from './live-table-row';
import LiveSearch from './live-search';
import TableHead from './live-table-head';
import { useGetLives, useGetLiveStatusCount } from '@/shared/api/live';
import { TableRow } from '@mui/material';

const defaultFilters: ILiveTableFilter = {
  status: 'ALL',
  startDate: null,
  tags: [],
};

export default function AllLivesView() {
  const theme = useTheme();
  const settings = useSettingsContext();
  const router = useRouter();
  const table = useTable({ defaultOrderBy: 'startDate', defaultOrder: 'desc' });
  const [filters, setFilters] = useState(defaultFilters);
  const [searchQuery, setSearchQuery] = useState('');
  const { 
    livesData, 
    totalElements, 
    currentPage, 
    pageSize, 
    livesLoading 
  } = useGetLives(
    table.page, 
    table.rowsPerPage, 
    table.order.toLowerCase(), 
    table.orderBy, 
    filters.status, 
    searchQuery
  );

  const denseHeight = table.dense ? 56 : 56 + 20;

  const canReset = filters.status !== 'ALL' || !!filters.tags.length || (!!filters.startDate);

  const notFound = (!livesData.length && canReset) || !livesData.length;

  const { liveStatusCountData, liveStatusCountLoading, liveStatusCountError } = useGetLiveStatusCount();
  
  console.log(liveStatusCountData)

  const TABS = useMemo(() => [
    { value: 'ALL', label: 'Tout', color: 'default', count: liveStatusCountData?.[LiveStatus.ALL] || 0 },
    { value: LiveStatus.COMING, label: "à venir", color: 'error', count: liveStatusCountData?.[LiveStatus.COMING] || 0 },
    { value: LiveStatus.ONGOING, label: "En cours", color: 'warning', count: liveStatusCountData?.[LiveStatus.ONGOING] || 0 },
    { value: LiveStatus.REVIEW, label: "En revue", color: 'info', count: liveStatusCountData?.[LiveStatus.REVIEW] || 0 },
    { value: LiveStatus.REPLAY, label: "Rediffusion", color: 'success', count: liveStatusCountData?.[LiveStatus.REPLAY] || 0 },
    { value: LiveStatus.ARCHIVED, label: "Archivé", color: 'default', count: liveStatusCountData?.[LiveStatus.ARCHIVED] || 0 },
  ], [liveStatusCountData, livesData, liveStatusCountLoading, liveStatusCountError]);

  const handleFilters = useCallback(
    (name: string, value: ILiveTableFilterValues) => {
      table.onResetPage();
      setFilters((prevState) => ({
        ...prevState,
        [name]: value,
      }));
    },
    [table]
  );


  const handleEditRow = useCallback(
    (id: string) => {
      router.push(paths.dashboard.admin.live.edit(id));
    },
    [router]
  );

  const handleViewStats = useCallback(
    (id: string) => {
      router.push(paths.dashboard.admin.live.statistics.single(id));
    },
    [router]
  );

  const handleViewInfo = useCallback(
    (id: string) => {
      router.push(paths.dashboard.admin.live.info(id));
    },
    [router]
  );

  const handleFilterStatus = useCallback(
    (event: React.SyntheticEvent, newValue: string) => {
      handleFilters('status', newValue);
    },
    [handleFilters]
  );

  const handleViewRow = useCallback(
    (id: string) => {
      router.push(paths.dashboard.admin.live.details(id));
    },
    [router]
  );

  const renderSearch = (
    <Stack
      spacing={3}
      justifyContent="space-between"
      sx={{ mb: { xs: 3, md: 5 } }}
      width={'100%'}
      alignItems={'flex-end'}
      direction={'row'}
    >
      
      <LiveSearch
        onSearch={setSearchQuery}
      />
      <Button
        component={RouterLink}
        href={paths.dashboard.admin.live.new}
        variant="contained"
        startIcon={<Iconify icon="mingcute:add-line" />}
      >
        Nouveau live
      </Button>
    </Stack>
  );


  return (
    <>
      <LocalizationProvider>
        <Container maxWidth={settings.themeStretch ? false : 'lg'}>
          <CustomBreadcrumbs
            heading="Tout les lives"
            links={[
              { name: 'Tableau de bord', href: paths.dashboard.root },
              { name: 'Lives', href: paths.dashboard.admin.live.all_lives },
              { name: 'Tout les lives' },
            ]}
            sx={{ mb: { xs: 3, md: 5 } }}
          />
          {renderSearch}

          <Card>
            <Tabs
              value={filters.status}
              onChange={handleFilterStatus}
              sx={{ px: 2.5, boxShadow: `inset 0 -2px 0 0 ${alpha(theme.palette.grey[500], 0.08)}` }}
            >
              {TABS.map((tab) => (
                <Tab
                  key={tab.value}
                  value={tab.value}
                  label={tab.label}
                  iconPosition="end"
                  icon={
                    <Label
                      variant={(tab.value === 'ALL' || tab.value === filters.status) && 'filled' || 'soft'}
                      color={tab.color as LabelColor}
                    >
                      {tab.count}
                    </Label>
                  }
                />
              ))}
            </Tabs>

            <TableContainer sx={{ position: 'relative', overflow: 'unset' }}>
              <Scrollbar>
                <Table size={table.dense ? 'small' : 'medium'} sx={{ minWidth: 800 }}>
                  <TableHead order={table.order} orderBy={table.orderBy} onSort={table.onSort} />
                  <TableBody>
                    {livesLoading ? (
                      <TableRow>
                        <TableEmptyRows height={denseHeight} emptyRows={5} />
                      </TableRow>
                    ) : (
                      <>
                        {livesData.map((row) => (
                          <LiveTableRow
                            key={row.id}
                            row={row}
                            onDetails={() => handleViewInfo(row.id)}
                            selected={table.selected.includes(row.id)}
                            onViewRow={() => handleViewRow(row.id)}
                            onViewStats={() => handleViewStats(row.id)}
                            onEditRow={() => handleEditRow(row.id)}
                          />
                        ))}
                        <TableNoData notFound={notFound} />
                      </>
                    )}
                  </TableBody>
                </Table>
              </Scrollbar>
            </TableContainer>

            <TablePaginationCustom
              count={totalElements}
              page={table.page}
              rowsPerPage={table.rowsPerPage}
              onPageChange={table.onChangePage}
              onRowsPerPageChange={table.onChangeRowsPerPage}
              dense={table.dense}
              onChangeDense={table.onChangeDense}
            />
          </Card>
        </Container>
      </LocalizationProvider>
    </>
  );
}
