"use client";

import * as Yup from 'yup';
import { Controller, useForm } from 'react-hook-form';
import { yupResolver } from '@hookform/resolvers/yup';
import Chip from '@mui/material/Chip';
import Grid from '@mui/material/Unstable_Grid2';
import Card from '@mui/material/Card';
import Stack from '@mui/material/Stack';
import CardHeader from '@mui/material/CardHeader';
import Typography from '@mui/material/Typography';
import { useResponsive } from '@/hooks/use-responsive';
import { useSnackbar } from '@/components/snackbar';
import FormProvider, { RHFTextField, RHFAutocomplete } from '@/components/hook-form';
import { _tags } from '@/shared/_mock';
import { TimePicker } from '@mui/x-date-pickers/TimePicker';
import { DatePicker } from '@mui/x-date-pickers/DatePicker';
import OptionItem from './add-option-item';
import { VideoFormatsField } from './video-format';
import CustomBreadcrumbs from '@/components/custom-breadcrumbs';
import { paths } from '@/routes/paths';
import { useSettingsContext } from '@/shared/components/settings';
import { Avatar, Button, Container, List, ListItem, ListItemAvatar, ListItemText, Paper } from '@mui/material';
import { ILiveProduct, LiveStatus } from '@/shared/types/live';
import { deleteLive, editLive, useGetProductsLive, publishLive, restoreLive, archiveLive } from '@/shared/api/live';
import { useCallback, useEffect, useMemo, useState } from 'react';
import { useLiveData } from '@/contexts/live-details';
import LoadingButton from '@mui/lab/LoadingButton';
import { useRouter } from 'next/navigation';
import Iconify from '@/components/iconify';
import Label from "@/shared/components/label";
import { styled } from '@mui/material/styles';
import CopyButton from '@/components/copy-button';
import { useBoolean } from '@/hooks';
import { ConfirmDialog } from '@/shared/components/custom-dialog';
import useAsync from '@/hooks/use-async';
import { MYDRESSIN_CLIENT_URL } from '@/config-global';

const StyledList = styled(List)({
    overflowY: 'auto',
    maxHeight: 'calc(100% - 64px)',
    '&::-webkit-scrollbar': {
        width: '10px',
    },
    '&::-webkit-scrollbar-track': {
        boxShadow: 'inset 0 0 5px grey',
        borderRadius: '10px',
    },
    '&::-webkit-scrollbar-thumb': {
        background: 'darkgrey',
        borderRadius: '10px',
    },
    '&::-webkit-scrollbar-thumb:hover': {
        background: 'transparent',
    }
});


const ProductCard = ({ product }: any) => (
    <Card variant="outlined" key={product.id} sx={{ marginBottom: 2 }}>
        <ListItem>
            <ListItemAvatar>
                <Avatar variant="rounded" alt={product.title} sx={{ height: 100, width: 100 }} src={product.image} />
            </ListItemAvatar>
            <ListItem sx={{ display: 'flex', textOverflow: '', flexDirection: 'column', alignItems: 'start', gap: 1 }}>
                <ListItemText primary={product.title} />
                <ListItemText secondary={product.id} />
                <ListItemText primary={`Prix: ${product.price}`} />
            </ListItem>
        </ListItem>
    </Card>
);

export default function LiveDetailsView() {
    const liveData = useLiveData();
    const { status } = liveData;
    const mdUp = useResponsive('up', 'md');
    const { enqueueSnackbar } = useSnackbar();
    const settings = useSettingsContext();

    const NewLiveSchema = Yup.object().shape({
        title: Yup.string().required('Title is required'),
        chatName: Yup.string().required('Moderator is required'),
        description: Yup.string().required('Description is required'),
        startDate: Yup.date().required('Start date is required'),
        keyWord: Yup.array().min(2).of(Yup.string()).required('Tags are required'),
        staging: Yup.boolean().required('Staging is required'),
        enableChat: Yup.boolean().required('Chat is required'),
        externalVideo: Yup.boolean().required(),
        enableLive: Yup.boolean().required('Cache is required'),
        videoFormat: Yup.string().required('Video format is required'),
    });

    const { products, productsLoading } = useGetProductsLive(liveData.id);
    const [image, setImage] = useState<File | null>(null);
    const [liveProducts, setLiveProducts] = useState<ILiveProduct[]>(products || []);

    useEffect(() => {
        setLiveProducts(products || []);
    }, [productsLoading]);

    const defaultValues = useMemo(() => ({
        title: liveData.title,
        chatName: liveData.chatName,
        description: liveData.description,
        startDate: new Date(liveData.startDate),
        keyWord: liveData.keyWord,
        staging: liveData.staging,
        enableChat: liveData.enableChat,
        enableLive: liveData.enableLive,
        externalVideo: true,
        videoFormat: liveData.videoFormat,
    }), [liveData]);

    const deleteConfirm = useBoolean();
    const publishConfirm = useBoolean();
    const archiveConfirm = useBoolean();
    const restoreConfirm = useBoolean();

    const asyncDeleteLive = useAsync(async () => {
        await deleteLive(liveData.id);
        deleteConfirm.onFalse();
        router.push(paths.dashboard.admin.live.all_lives);
    });

    const asyncPublishLive = useAsync(async () => {
        await publishLive(liveData.id);
    });

    const asyncArchiveLive = useAsync(async () => {
        await archiveLive(liveData.id);
    });

    const asyncRestoreLive = useAsync(async () => {
        await restoreLive(liveData.id);
    });

    const handleDelete = () => {
        asyncDeleteLive.execute()
            .then(() => {
                enqueueSnackbar('Live supprimé avec succès', { variant: 'success' });
            })
            .catch(() => {
                enqueueSnackbar('Erreur lors de la suppression', { variant: 'error' });
            });
    };

    const handlePublishLive = () => {
        asyncPublishLive.execute()
            .then(() => {
                enqueueSnackbar('Le live a été publié', { variant: 'success' });
            })
            .catch((err) => {
                enqueueSnackbar(err.message, { variant: 'error' });
            })
            .finally(() => {
                publishConfirm.onFalse();
            });
    };

    const handleArchiveLive = () => {
        asyncArchiveLive.execute()
            .then(() => {
                enqueueSnackbar('Le live a été archivé', { variant: 'success' });
            })
            .catch((err) => {
                enqueueSnackbar(err.message, { variant: 'error' });
            })
            .finally(() => {
                archiveConfirm.onFalse();
            });
    };

    const handleRestoreLive = () => {
        asyncRestoreLive.execute()
            .then(() => {
                enqueueSnackbar('Le live a été restauré', { variant: 'success' });
            })
            .catch((err) => {
                enqueueSnackbar(err.message, { variant: 'error' });
            })
            .finally(() => {
                restoreConfirm.onFalse();
            });
    };

    const methods = useForm({
        resolver: yupResolver(NewLiveSchema),
        defaultValues,
    });

    const {
        watch,
        control,
        setValue,
        handleSubmit,
        formState: { isSubmitting },
    } = methods;

    const values = watch();

    const addProduct = (product: ILiveProduct) => {
        setLiveProducts((prev) => [...prev, product]);
    };

    const removeProduct = (product: ILiveProduct) => {
        setLiveProducts((prev) => prev.filter((p) => p.id !== product.id));
    };

    const router = useRouter();

    const handleEditRow = useCallback(
        (id: string) => {
            router.push(paths.dashboard.admin.live.edit(id));
        },
        [router]
    );

    const handleViewStats = useCallback(
        (id: string) => {
            router.push(paths.dashboard.admin.live.statistics.single(id));
        },
        [router]
    );

    const handleViewInfo = useCallback(
        (id: string) => {
            router.push(paths.dashboard.admin.live.details(id));
        },
        [router]
    );

    const onSubmit = handleSubmit((data) => {
        const formData = {
            ...data,
            image: image as File,
            staging: false,
            products: products.map((product: ILiveProduct) => product.id)
        };
        editLive(liveData.id, formData)
            .then(() => {
                enqueueSnackbar('Live mis à jour avec succès', { variant: 'success' });
                setImage(null);
            })
            .catch((err) => {
                enqueueSnackbar(err.message, { variant: 'error' });
                console.error(err);
            });
    });

    const renderConnextionDetails = (
        <>
            {mdUp && (
                <Grid md={4}>
                    <Typography variant="h6" sx={{ mb: 0.5 }}>
                        Connexion
                    </Typography>
                    <Typography variant="body2" sx={{ color: 'text.secondary' }}>
                        Définissez le cadre de votre live : titre, modérateur, description et horaire pour une présentation complète
                    </Typography>
                </Grid>
            )}

            <Grid xs={12} md={8}>
                <Card>
                    {!mdUp && <CardHeader title="Connexion" />}
                    <Grid container spacing={2} sx={{ p: 3 }}>
                        <Grid xs={12} sm={6}>
                            <Card sx={{ p: 2 }}>
                                <Typography variant="subtitle1" gutterBottom>
                                    Clé de diffusion
                                </Typography>
                                <Typography variant="body2" gutterBottom>
                                    {liveData.keysWork?.keyStream}
                                    <CopyButton sx={{ position: 'absolute', right: 3, top: 1 }} value={liveData.keysWork?.keyStream as string} />
                                </Typography>
                            </Card>
                        </Grid>
                        <Grid xs={12} sm={6}>
                            <Card sx={{ p: 2 }}>
                                <Typography variant="subtitle1" gutterBottom>
                                    adresse du serveur rtmp
                                </Typography>
                                <Typography variant="body2" gutterBottom>
                                    {liveData.keysWork?.rtmpServerAddress}
                                    <CopyButton sx={{ position: 'absolute', right: 3, top: 1 }} value={liveData.keysWork?.rtmpServerAddress as string} />
                                </Typography>
                            </Card>
                        </Grid>
                        <Grid xs={12} sm={6}>
                            <Card sx={{ p: 2 }}>
                                <Typography variant="subtitle1" gutterBottom>
                                    lien de la diffusion
                                </Typography>
                                <Typography variant="body2" gutterBottom>
                                    {MYDRESSIN_CLIENT_URL + 'lives?liveId=' + liveData.id}
                                    <CopyButton sx={{ position: 'absolute', right: 3, top: 1 }} value={MYDRESSIN_CLIENT_URL + 'lives?liveId=' + liveData.id} />
                                </Typography>
                            </Card>
                        </Grid>
                    </Grid>
                </Card>
            </Grid>
        </>
    );

    const renderDetails = (
        <>
            {mdUp && (
                <Grid md={4}>
                    <Typography variant="h6" sx={{ mb: 0.5 }}>
                        Details
                    </Typography>
                    <Typography variant="body2" sx={{ color: 'text.secondary' }}>
                        Définissez le cadre de votre live : titre, modérateur, description et horaire pour une présentation complète
                    </Typography>
                </Grid>
            )}

            <Grid xs={12} md={8}>
                <Card>
                    {!mdUp && <CardHeader title="Details" />}
                    <Stack spacing={3} sx={{ p: 3 }}>
                        <RHFTextField name="title" label="titre du Live" disabled={true} />
                        <RHFTextField name="description" label="Description" placeholder="Description..." multiline rows={4} disabled={true} />
                        <RHFTextField name="chatName" label="Nom affiché dans le chat" multiline disabled={true} />
                        <Stack spacing={2} direction={{ xs: 'column', sm: 'row' }}>
                            <Controller
                                name="startDate"
                                control={control}
                                render={({ field, fieldState: { error } }) => (
                                    <DatePicker
                                        label="Date de debut"
                                        value={field.value}
                                        onChange={(newValue) => {
                                            field.onChange(newValue);
                                        }}
                                        slotProps={{
                                            textField: {
                                                fullWidth: true,
                                                error: !!error,
                                                helperText: error?.message,
                                            },
                                        }}
                                        disabled={true}
                                    />
                                )}
                            />
                            <Controller
                                name="startDate"
                                control={control}
                                render={({ field, fieldState: { error } }) => (
                                    <TimePicker
                                        label="heure de debut"
                                        value={field.value}
                                        onChange={(newValue) => {
                                            field.onChange(newValue);
                                        }}
                                        slotProps={{
                                            textField: {
                                                fullWidth: true,
                                                error: !!error,
                                                helperText: error?.message,
                                            },
                                        }}
                                        disabled={true}
                                    />
                                )}
                            />
                        </Stack>
                        <RHFAutocomplete
                            name="keyWord"
                            label="mots clés"
                            placeholder="+ mot clé"
                            multiple
                            freeSolo
                            disableCloseOnSelect
                            options={_tags.map((option) => option)}
                            getOptionLabel={(option) => option}
                            renderOption={(props, option) => (
                                <li {...props} key={option}>
                                    {option}
                                </li>
                            )}
                            renderTags={(selected, getTagProps) =>
                                selected.map((option, index) => (
                                    <Chip
                                        {...getTagProps({ index })}
                                        key={option}
                                        label={option}
                                        size="small"
                                        color="info"
                                        variant="soft"
                                    />
                                ))
                            }
                            disabled={true}
                        />
                    </Stack>
                </Card>
            </Grid>
        </>
    );

    const optionsItems = [
        {
            title: "Source de vidéo externe",
            description: "Génère un lien pour votre logiciel de diffusion. Le lien sera disponible une fois le live crée",
            enabled: values.externalVideo,
            onToggle: () => setValue('externalVideo', !values.externalVideo as never),
            disabled: liveData.status !== LiveStatus.COMING,
        },
        {
            title: "Staging",
            description: "Ce live sera uniquement visible sur votre site de staging",
            enabled: values.staging,
            onToggle: () => setValue("staging", !values.staging as never),
            disabled: liveData.status !== LiveStatus.COMING,
        },
        {
            title: "Désactiver le chat",
            description: "Le chat sera accessible aux modérateur uniquement",
            enabled: values.enableChat,
            onToggle: () => setValue('enableChat', !values.enableChat as never),
            disabled: liveData.status !== LiveStatus.COMING,
        },
        {
            title: "Cacher ce live",
            description: "Ce live ne sera pas visible sur votre site",
            enabled: values.enableLive,
            onToggle: () => setValue('enableLive', !values.enableLive as never),
            disabled: liveData.status !== LiveStatus.COMING,
        },
    ];

    const renderOptions = (
        <>
            {mdUp && (
                <Grid xs={12} md={4}>
                    <Typography variant="h6" sx={{ mb: 0.5 }}>
                        Options
                    </Typography>
                    <Typography variant="body2" sx={{ color: 'text.secondary' }}>
                        Ajustez les paramètres de votre live : liens de diffusion, visibilité et interactions, pour un contrôle total.
                    </Typography>
                </Grid>
            )}
            <Grid xs={12} md={8}>
                <Card>
                    {!mdUp && <CardHeader title="Options" />}
                    <Grid container spacing={2} sx={{ p: 3 }}>
                        {optionsItems.map((item, index) => (
                            <Grid key={index} xs={12} sm={6}>
                                <OptionItem
                                    title={item.title}
                                    description={item.description}
                                    onToggle={item.onToggle}
                                    enabled={item.enabled}
                                    disabled={item.disabled}
                                />
                            </Grid>
                        ))}
                    </Grid>
                </Card>
            </Grid>
        </>
    );

    const renderProducts = (
        <>
            {mdUp && (
                <Grid md={4}>
                    <Typography variant="h6" sx={{ mb: 0.5 }}>
                        Produits
                    </Typography>
                    <Typography variant="body2" sx={{ color: 'text.secondary' }}>
                        Enrichissez votre live avec des produits, enregistrez et gérez la diffusion pour une expérience enrichie
                    </Typography>
                </Grid>
            )}
            <Grid xs={12} md={8}>
                <Card>
                    {!mdUp && <CardHeader title="Products" />}
                    <Stack spacing={2} sx={{ p: 3 }}>
                        <StyledList dense sx={{
                            overflowY: 'auto',
                            maxHeight: 'calc(100% - 64px)',
                            p: 2
                        }}>
                            {liveProducts.map((product) => {
                                return (
                                    <ProductCard key={product.id} product={product} />
                                );
                            })}
                        </StyledList>
                    </Stack>
                </Card>
            </Grid>
        </>
    );

    const renderVisuels = (
        <>
            {mdUp && (
                <Grid xs={12} md={4}>
                    <Typography variant="h6" sx={{ mb: 0.5 }}>
                        Visuels
                    </Typography>
                    <Typography variant="body2" sx={{ color: 'text.secondary' }}>
                        Personnalisez l'esthétique de votre live avec une photo de couverture et le format vidéo parfait.
                    </Typography>
                </Grid>
            )}
            <Grid xs={12} md={8}>
                <Card>
                    {!mdUp && <CardHeader title="Visuel" />}
                    <Stack spacing={2} sx={{ p: 3 }}>
                        <Typography variant="subtitle1">Format vidéo</Typography>
                        <Controller
                            name="videoFormat"
                            control={control}
                            render={({ field }) => (
                                <VideoFormatsField field={field} disabled={true} />
                            )}
                        />
                    </Stack>
                    <Stack spacing={2} sx={{ p: 3 }}>
                        <Typography variant="subtitle1">Photo de couverture</Typography>
                        <img src={liveData.image as string} style={{ maxHeight: 200, maxWidth: 150, borderRadius: 10 }} />
                    </Stack>
                </Card>
            </Grid>
        </>
    );

    const renderActions = (
        <Grid xs={12} md={4} sx={{ display: 'flex', justifyContent: 'space-between', alignItems: 'center', gap: 2, py: 4 }}>
            <Label
                sx={{ textTransform: 'capitalize', height: 40, fontSize: 16, fontWeight: 600, padding: '0 16px' }}
                variant="soft"
                color={
                    (status == LiveStatus.REPLAY && 'success') ||
                    (status == LiveStatus.ONGOING && 'warning') ||
                    (status == LiveStatus.COMING && 'error') ||
                    (status == LiveStatus.REVIEW && 'info') || 'default'
                }
            >
                {
                    (status == LiveStatus.REPLAY && 'rediffusion') ||
                    (status == LiveStatus.ONGOING && 'en cours') ||
                    (status == LiveStatus.COMING && 'à venir') ||
                    (status == LiveStatus.ARCHIVED && 'archivé') ||
                    (status == LiveStatus.REVIEW && 'en revue')
                }
            </Label>
            <Grid xs={12} md={8} sx={{ display: 'flex', justifyContent: 'end', py: 4 }}>
                <LoadingButton
                    variant="contained"
                    size="large"
                    loading={isSubmitting}
                    sx={{ ml: 2 }}
                    onClick={() => handleViewInfo(liveData.id)}
                >
                    <Iconify sx={{ mr: 2 }} icon="solar:eye-bold" />
                    voir
                </LoadingButton>
                <LoadingButton
                    variant="contained"
                    size="large"
                    loading={isSubmitting}
                    sx={{ ml: 2 }}
                    onClick={() => handleEditRow(liveData.id)}
                >
                    <Iconify sx={{ mr: 2 }} icon="solar:pen-bold" />
                    Modifier
                </LoadingButton>
                {liveData.status !== LiveStatus.COMING && (
                    <LoadingButton
                        variant="contained"
                        size="large"
                        loading={isSubmitting}
                        sx={{ ml: 2 }}
                        onClick={() => handleViewStats(liveData.id)}
                    >
                        <Iconify sx={{ mr: 2 }} icon="icomoon-free:stats-bars" />
                        Statistiques
                    </LoadingButton>
                )}
                {liveData.status === LiveStatus.REVIEW && (
                    <LoadingButton
                        variant="contained"
                        size="large"
                        onClick={publishConfirm.onTrue}
                        sx={{ ml: 2, backgroundColor: 'blue' }}
                    >
                        <Iconify sx={{ mr: 2 }} icon="ic:baseline-publish" />
                        Publier
                    </LoadingButton>
                )}
                {(liveData.status === LiveStatus.REVIEW || liveData.status === LiveStatus.REPLAY) && (
                    <LoadingButton
                        variant="contained"
                        size="large"
                        color='warning'
                        onClick={archiveConfirm.onTrue}
                        sx={{ ml: 2 }}
                    >
                        <Iconify sx={{ mr: 2 }} icon="dashicons:archive" />
                        archiver
                    </LoadingButton>
                )}
                {liveData.status === LiveStatus.ARCHIVED && (
                    <LoadingButton
                        variant="contained"
                        size="large"
                        color='warning'
                        onClick={restoreConfirm.onTrue}
                        sx={{ ml: 2 }}
                    >
                        <Iconify sx={{ mr: 2 }} icon="mdi:archive-off" />
                        restorer
                    </LoadingButton>
                )}
                <LoadingButton
                    type="submit"
                    variant="contained"
                    size="large"
                    onClick={deleteConfirm.onTrue}
                    sx={{ ml: 2, backgroundColor: 'red' }}
                >
                    <Iconify sx={{ mr: 2 }} icon="material-symbols:delete" />
                    Supprimer
                </LoadingButton>
            </Grid>
        </Grid>
    );

    return (
        <Container maxWidth={settings.themeStretch ? false : 'lg'}>
            <CustomBreadcrumbs
                heading={`Détails du live #${liveData.id}`}
                rollbackLink={paths.dashboard.admin.live.all_lives}
                links={[
                    { name: 'Tableau de bord', href: paths.dashboard.root },
                    { name: 'Lives', href: paths.dashboard.admin.live.all_lives },
                    { name: `Détails du live #${liveData.id}` },
                ]}
                sx={{ mb: { xs: 3, md: 5 } }}
            />
            {renderActions}
            <FormProvider methods={methods} onSubmit={onSubmit}>
                <Grid container spacing={3}>
                    {renderConnextionDetails}
                    {renderDetails}
                    {renderOptions}
                    {renderVisuels}
                    {renderProducts}
                </Grid>
            </FormProvider>
            <ConfirmDialog
                open={deleteConfirm.value}
                onClose={deleteConfirm.onFalse}
                title="Supprimer"
                content="Voulez-vous vraiment supprimer ce live?"
                action={
                    <LoadingButton loading={asyncDeleteLive.loading} variant="contained" color="error" onClick={handleDelete}>
                        Supprimer
                    </LoadingButton>
                }
            />
            <ConfirmDialog
                open={publishConfirm.value}
                onClose={publishConfirm.onFalse}
                title="Publier"
                content="Voulez-vous publier le live?"
                action={
                    <LoadingButton loading={asyncPublishLive.loading} variant="contained" color="info" onClick={handlePublishLive}>
                        Publier
                    </LoadingButton>
                }
            />

            <ConfirmDialog
                open={archiveConfirm.value}
                onClose={archiveConfirm.onFalse}
                title="Archiver"
                content="Voulez-vous archiver le live?"
                action={
                    <LoadingButton loading={asyncArchiveLive.loading} variant="contained" color="warning" onClick={handleArchiveLive}>
                        Archiver
                    </LoadingButton>
                }
            />

            <ConfirmDialog
                open={restoreConfirm.value}
                onClose={restoreConfirm.onFalse}
                title="Restorer"
                content="Voulez-vous restorer le live?"
                action={
                    <LoadingButton loading={asyncRestoreLive.loading} variant="contained" color="warning" onClick={handleRestoreLive}>
                        Restorer
                    </LoadingButton>
                }
            />
        </Container>
    );
}
