"use client";
import isEqual from "lodash/isEqual";
import { useState, useCallback, useEffect } from "react";
import * as Yup from "yup";

import Tab from "@mui/material/Tab";
import Tabs from "@mui/material/Tabs";
import Card from "@mui/material/Card";
import Table from "@mui/material/Table";
import Button from "@mui/material/Button";
import { alpha } from "@mui/material/styles";
import Container from "@mui/material/Container";
import TableBody from "@mui/material/TableBody";
import TableContainer from "@mui/material/TableContainer";

import { paths } from "@/routes/paths";

import { useBoolean } from "@/hooks/use-boolean";

import Label from "../../../components/label";
import Iconify from "@/components/iconify";
import Scrollbar from "@/components/scrollbar";
import { useSnackbar } from "@/components/snackbar";
import { ConfirmDialog } from "../../../components/custom-dialog";
import { useSettingsContext } from "@/components/settings";
import CustomBreadcrumbs from "@/components/custom-breadcrumbs";
import {
  useTable,
  emptyRows,
  TableNoData,
  TableEmptyRows,
  TableHeadCustom,
  TableSelectedAction,
  TablePaginationCustom,
} from "@/shared/components/table";

import {
  ILiveItem,
  LiveTyple,
  SalesSessionTableFilters,
  SalesSessionTableFilterValue,
} from "@/shared/types/saleSession";

import SalesSessionTableRow from "./SalesSession-table-row";
import SalesSessionTableToolbar from "./SalesSession-table-toolbar";
import SalesSessionTableFiltersResult from "./SalesSession-table-filters-result";
import {
  Autocomplete,
  Box,
  Dialog,
  DialogActions,
  DialogContent,
  DialogTitle,
  FormControl,
  InputLabel,
  MenuItem,
  Select,
  TextField,
  FormHelperText,
  TableRow,
  CircularProgress,
  TableCell,
  Grid,
} from "@mui/material";
import {
  deleteSaleSessionById,
  useAddSaleSession,
  useSaleSessionCountsByType,
  useSearchSaleSession,
  useUpdateSaleSession,
} from "@/shared/api/saleSession";
import { useGetLives } from "@/shared/api/live";
import { getTokenInfo } from "@/utils/token";
import LoadingButton from "@mui/lab/LoadingButton";
import EditSalesSessionDialog from "./EditSalesSessionDialog";
import SessionOrdersProcessor from "./ExportSaleSessions";
// ----------------------------------------------------------------------

const Type_OPTIONS = [{ value: "", label: "Tous" }, ...LiveTyple];

const TABLE_HEAD = [
  { id: "id", label: "id", width: 50 },
  { id: "name", label: "Live", width: 60 },
  { id: "startedDate", label: "Date création ", width: 100 },
  { id: "lastModification", label: "Dernière mise a jour", width: 100 },
  { id: "orderProductsNumber", label: "produits Vendu", width: 40 },
  { id: "clientsNumber", label: "Nombre Client", width: 40 },
  { id: "totalSales", label: "Total Vente", width: 40 },
  { id: "expirationDate", label: "Date d'expiration", width: 80 },
  { id: "description", label: "Description", width: 100 },
  { id: "Type", label: "Type Live", width: 80 },
  { id: "", label: "Action", width: 20 },
];



interface FormErrors {
  name?: string;
  startedDate?: string;
  expirationDate?: string;
  type?: string;
  live?: string;
}

// ----------------------------------------------------------------------

export default function AllSalesSessionView() {
  const table = useTable();
  const { enqueueSnackbar } = useSnackbar();
  const { addSaleSession } = useAddSaleSession();
  const { livesData } = useGetLives(1, 10, "asc", "startDate", "", "");
  const [formData, setFormData] = useState<ILiveItem>({} as ILiveItem);
  const [formErrors, setFormErrors] = useState<FormErrors>({});
  const [openEditDialog, setOpenEditDialog] = useState(false);
  const [openExportDialog, setOpenExportDialog] = useState(false);
  const [selectedRow, setSelectedRow] = useState<ILiveItem | null>(null);
  const { updateSaleSession } = useUpdateSaleSession();
  const [tableData, setTableData] = useState<ILiveItem[]>([]);
  const [tableDataExport, setTableDataExport] = useState<ILiveItem[]>([]);

  const [typeCount, setTypeCount] = useState<{ [key: string]: number }>({
    Live: 0,
    Demande_MP: 0,
    Article_a_controle: 0,
    Autre: 0,
  });

  const defaultFilters: SalesSessionTableFilters = {
    name: "",
    type: "",
    startDate: null,
    endDate: null
  };

  const defaultFiltersLocalStorage: SalesSessionTableFilters = localStorage.getItem("lastSaleSessionsFilters")
    ?
    JSON.parse(localStorage.getItem("lastSaleSessionsFilters")!)
    :
    {
      name: "",
      type: "",
      startDate: null,
      endDate: null
    };
  const [filters, setFilters] = useState(defaultFiltersLocalStorage);


  useEffect(() => {
    localStorage.removeItem("lastSaleSessionsFilters")
    localStorage.setItem("lastSaleSessionsFilters", JSON.stringify(filters));
  }, [filters]);


  const {
    saleSessions,
    totalSaleSessions,
    totalPages,
    pageNumber,
    pageSize,
    loading,
    revalidatePage,
  } = useSearchSaleSession({
    name: filters.name,
    type: filters.type,
    startDate: filters.startDate,
    endDate: filters.endDate,
    page: table.page,
    size: table.rowsPerPage,
    sortBy: "startedDate",
    sortDir: "desc",
  });
  const { saleSessionTypeCount, saleSessionTypeCountError } = useSaleSessionCountsByType(filters);

  useEffect(() => {
    if (saleSessions && saleSessions.length > 0) {
      const resultToExport = table.selected.length != 0 ? saleSessions.filter(session => table.selected.includes(session.id)) : saleSessions;
      setTableDataExport(resultToExport);
      setTableData(saleSessions);
    }
  }, [saleSessions, table.page, table.rowsPerPage, table.selected]);


  useEffect(() => {
    if (saleSessionTypeCountError) {
      console.error("Error fetching status counts:", saleSessionTypeCountError);
      return;
    }
    if (Array.isArray(saleSessionTypeCount)) {
      const statusObj = saleSessionTypeCount.reduce<{ [key: string]: number }>(
        (acc, [status, count]) => {
          acc[status] = count;
          return acc;
        },
        {}
      );

      setTypeCount(statusObj);
    } else {
      console.error(
        "Expected counts to be an array, but received:",
        saleSessionTypeCount
      );
    }
  }, [filters, saleSessionTypeCount, saleSessionTypeCountError]);


  useEffect(() => {
    if (
      filters.name ||
      filters.type ||
      filters.startDate ||
      filters.endDate
    ) {
      revalidatePage();
    }
  }, [filters, revalidatePage]);



  const handleFormChange = (
    e: React.ChangeEvent<
      HTMLInputElement | { name?: string | undefined; value: unknown }
    >
  ) => {
    const { name, value } = e.target;
    setFormData({
      ...formData,
      [name as string]: value,
    });
    setFormErrors({
      ...formErrors,
      [name as string]: undefined,
    });
  };



  const settings = useSettingsContext();

  const isConfirm = useBoolean();


  const dataInPage = tableData.slice(
    table.page * table.rowsPerPage,
    table.page * table.rowsPerPage + table.rowsPerPage
  );

  const denseHeight = table.dense ? 56 : 56 + 20;

  const canReset = !isEqual(defaultFilters, filters);

  const notFound = totalSaleSessions === 0 && canReset;



  const handleFilters = useCallback(
    (title: string, value: SalesSessionTableFilterValue) => {
      table.onResetPage();
      setFilters((prevState: any) => ({
        ...prevState,
        [title]: value,
      }));
    },
    [table]
  );
  const handleResetFilters = useCallback(() => {
    setFilters(defaultFilters);
  }, []);

  const handleDeleteRow = useCallback(
    (id: string) => {
      const deleteRow = tableData.filter((row) => row.id !== id);
      enqueueSnackbar("Session de vente supprimée avec succès !");
      setTableData(deleteRow);
      table.onUpdatePageDeleteRow(dataInPage.length);
    },
    [dataInPage.length, enqueueSnackbar, table, tableData]
  );

  const [isDeleting, setIsDeleting] = useState(false);
  const handleDeleteRows = useCallback(async () => {
    setIsDeleting(true);
    try {
      await Promise.all(
        table.selected.map((id) => deleteSaleSessionById(Number(id)))
      );

      const deleteRows = tableData.filter(
        (row) => !table.selected.includes(row.id)
      );

      setTableData(deleteRows);

      table.onUpdatePageDeleteRows({
        totalRowsInPage: dataInPage.length,
        totalRowsFiltered: totalSaleSessions,
      });

      enqueueSnackbar("Sessions supprimées avec succès!", {
        variant: "success",
      });
    } catch (error) {
      console.error("Erreur lors de la suppression des sessions", error);
      enqueueSnackbar("Erreur lors de la suppression des sessions", {
        variant: "error",
      });
    } finally {
      setIsDeleting(false);
      isConfirm.onFalse();
    }
  }, [
    totalSaleSessions,
    dataInPage.length,
    enqueueSnackbar,
    table,
    tableData,
  ]);

  const handleEditRow = useCallback((row: ILiveItem) => {
    setSelectedRow(row);
    setOpenEditDialog(true);
  }, []);

  const handleFiltertype = useCallback(
    (event: React.SyntheticEvent, newValue: string) => {
      handleFilters("type", newValue);
    },
    [handleFilters]
  );

  const [openAddDialog, setOpenAddDialog] = useState(false);
  const [selectedType, setSelectedType] = useState("");

  const handleOpenAddDialog = () => {
    setOpenAddDialog(true);
  };

  const handleCloseAddDialog = () => {
    setOpenAddDialog(false);
    setFormErrors({});
  };

  const validationSchema = Yup.object().shape({
    name: Yup.string().required("Le nom est requis"),
    startedDate: Yup.date().required("La date de début est requise"),
    expirationDate: Yup.date()
      .required("La date d'expiration est requise")
      .min(
        Yup.ref("startedDate"),
        "La date d'expiration doit être après la date de début"
      ),
    type: Yup.string().required("Le type de live est requis"),

    description: Yup.string(),
  });

  const handleAddSaleSession = async () => {
    try {
      await validationSchema.validate(formData, { abortEarly: false });

      await addSaleSession(formData);
      enqueueSnackbar("Session de vente ajoutée avec succès", {
        variant: "success",
      });
      handleCloseAddDialog();
    } catch (error: any) {
      if (error instanceof Yup.ValidationError) {
        const errors: FormErrors = {};
        error.inner.forEach((err) => {
          if (err.path) {
            errors[err.path as keyof FormErrors] = err.message;
          }
        });
        setFormErrors(errors);
        enqueueSnackbar("Veuillez corriger les erreurs du formulaire", {
          variant: "error",
        });
      } else {
        enqueueSnackbar("Erreur lors de l'ajout de la session de vente", {
          variant: "error",
        });
      }
    }
  };



  useEffect(() => {
    const { startedDate, expirationDate } = getCurrentDateAndExpiration();
    setFormData({
      ...formData,
      startedDate,
      expirationDate,
    });
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);

  const handleCreationDateChange = (e: React.ChangeEvent<HTMLInputElement>) => {
    const creationDateString = e.target.value;
    const creationDate = new Date(creationDateString);
    const now = new Date();
    if (creationDate < now) {
      enqueueSnackbar(
        "Veuillez sélectionner une date de début valide à partir d'aujourd'hui.",
        {
          variant: "error",
        }
      );
      return;
    }
    const expirationDate = calculateExpirationDate(creationDate);

    setFormData({
      ...formData,
      startedDate: creationDate,
      expirationDate,
    });
  };

  const handleExpirationDateChange = (
    e: React.ChangeEvent<HTMLInputElement>
  ) => {
    const expirationDateString = e.target.value;
    const expirationDate = new Date(expirationDateString);

    if (formData.startedDate && expirationDate <= formData.startedDate) {
      enqueueSnackbar(
        "La date d'expiration doit être après la date de début. Veuillez la corriger.",
        { variant: "warning" }
      );
      return;
    }

    setFormData({
      ...formData,
      expirationDate,
    });
  };

  const getCurrentDateAndExpiration = () => {
    const now = new Date();

    const expirationDate = new Date(now);

    expirationDate.setDate(expirationDate.getDate() + 5);
    expirationDate.setHours(0, 0, 0, 0);

    return {
      startedDate: now,
      expirationDate,
    };
  };

  const calculateExpirationDate = (creationDate: Date): Date => {
    const expiration = new Date(creationDate);
    expiration.setDate(expiration.getDate() + 5);
    expiration.setHours(0, 0, 0, 0);
    return expiration;
  };

  const adjustToLocalTimezone = (date: Date): string => {
    const userTimezoneOffset = date.getTimezoneOffset() * 60000;
    const localDate = new Date(date.getTime() - userTimezoneOffset);
    return localDate.toISOString().slice(0, 16);
  };

  const getTypeLabel = (value: string) => {
    const typeOption = LiveTyple.find((option) => option.value === value);
    return typeOption ? typeOption.label : value;
  };

  const handleModifySaleSession = async (updatedData: ILiveItem) => {
    try {
      await updateSaleSession(updatedData.id, {
        name: updatedData.name,
        description: updatedData.description,
      });

      enqueueSnackbar("Session de vente modifiée avec succès", {
        variant: "success",
      });

      // Optionnel : Vous pouvez également mettre à jour localement si nécessaire
      setTableData((prevData) =>
        prevData.map((session) =>
          session.id === updatedData.id ? updatedData : session
        )
      );
    } catch (error) {
      enqueueSnackbar("Erreur lors de la modification de la session de vente", {
        variant: "error",
      });
    }
  };
  const token = localStorage.getItem("token");
  const storedUserInfo = token ? getTokenInfo(token) : null;
  const isAdmin =
    storedUserInfo?.realm_access?.roles?.includes("ADMIN") ?? false;

  function updateSessionsID(sessions: ILiveItem[]) {
    const updatedSessions = sessions.map((session, index) => ({
      ...session,
      id: `'${sessions[index].id}'`
    }));
    return updatedSessions;
  }

  return (
    <>
      <Container maxWidth={settings.themeStretch ? false : "xl"}>
        <CustomBreadcrumbs
          heading="Liste session de vente"
          links={[
            { name: "Tableau de bord", href: paths.dashboard.root },
            { name: "Vente", href: paths.dashboard.admin.live.saleSession },
            { name: "Listes" },
          ]}
          action={
            <Button
              variant="contained"
              startIcon={<Iconify icon="mingcute:add-line" />}
              onClick={handleOpenAddDialog}
            >
              Ajouter
            </Button>
          }
          sx={{
            mb: { xs: 3, md: 5 },
          }}
        />

        <SessionOrdersProcessor open={openExportDialog} onClose={() => setOpenExportDialog(false)} saleSessions={tableDataExport} />

        <Dialog
          open={openAddDialog}
          onClose={handleCloseAddDialog}
          fullWidth
          maxWidth="sm"
        >
          <DialogTitle>{"Ajouter une session de vente"}</DialogTitle>
          <DialogContent>
            <FormControl
              fullWidth
              margin="normal"
              error={Boolean(formErrors.type)}
            >
              <InputLabel id="type-live-label">Type de Live</InputLabel>
              <Select
                labelId="type-live-label"
                value={selectedType}
                onChange={(e) => {
                  setFormData({
                    ...formData,
                    type: e.target.value,
                  });
                  setSelectedType(e.target.value);
                }}
                label="Type de Live"
              >
                {LiveTyple.map((option) => (
                  <MenuItem key={option.value} value={option.value}>
                    {option.label}
                  </MenuItem>
                ))}
              </Select>
              {formErrors.type ? (
                <FormHelperText>{formErrors.type}</FormHelperText>
              ) : (
                <FormHelperText>Please select a valid option</FormHelperText>
              )}
            </FormControl>
            {selectedType === "Live" && (
              <Autocomplete
                id="live"
                autoHighlight
                options={livesData}
                getOptionLabel={(option) => option.title || option.id}
                isOptionEqualToValue={(option, value) =>
                  option.title === value.title || option.id === value.id
                }
                onChange={(event, newValue) => {
                  setFormData({
                    ...formData,
                    live: newValue?.id!.toString(),
                  });
                  setFormErrors({
                    ...formErrors,
                    live: undefined,
                  });
                }}
                renderOption={(props, option) => (
                  <Box
                    component="li"
                    sx={{ display: "flex", alignItems: "center" }}
                    {...props}
                  >
                    <img
                      loading="lazy"
                      width="40"
                      src={option.image}
                      alt=""
                      style={{ marginRight: 8 }}
                    />
                    <span>{option.title}</span>
                  </Box>
                )}
                renderInput={(params) => (
                  <TextField
                    {...params}
                    label="Choisir un live"
                    InputLabelProps={{ shrink: true }}
                    variant="outlined"
                    margin="normal"
                    error={Boolean(formErrors.live)}
                    helperText={formErrors.live}
                  />
                )}
                sx={{ width: "100%", marginTop: 2 }}
              />
            )}

            <TextField
              fullWidth
              margin="normal"
              label="Nom"
              variant="outlined"
              name="name"
              value={formData.name || ""}
              onChange={handleFormChange}
              error={Boolean(formErrors.name)}
              helperText={formErrors.name}
            />
            <TextField
              fullWidth
              margin="normal"
              label="Date début"
              type="datetime-local"
              InputLabelProps={{ shrink: true }}
              variant="outlined"
              name="startedDate"
              value={
                formData.startedDate
                  ? adjustToLocalTimezone(new Date(formData.startedDate))
                  : ""
              }
              onChange={handleCreationDateChange}
              error={Boolean(formErrors.startedDate)}
              helperText={formErrors.startedDate}
            />

            <TextField
              fullWidth
              margin="normal"
              label="Date d'expiration"
              type="datetime-local"
              InputLabelProps={{ shrink: true }}
              variant="outlined"
              name="expirationDate"
              value={
                formData.expirationDate
                  ? adjustToLocalTimezone(new Date(formData.expirationDate))
                  : ""
              }
              onChange={handleExpirationDateChange}
              error={Boolean(formErrors.expirationDate)}
              helperText={formErrors.expirationDate}
            />

            <TextField
              fullWidth
              margin="normal"
              label="Description"
              multiline
              rows={4}
              variant="outlined"
              name="description"
              value={formData.description || ""}
              onChange={handleFormChange}
            />
          </DialogContent>
          <DialogActions>
            <Button onClick={handleCloseAddDialog} color="secondary">
              Annuler
            </Button>
            <Button
              color="primary"
              variant="contained"
              onClick={handleAddSaleSession}
            >
              Ajouter
            </Button>
          </DialogActions>
        </Dialog>



        <EditSalesSessionDialog
          open={openEditDialog}
          onClose={() => setOpenEditDialog(false)}
          initialData={selectedRow}
          onModify={handleModifySaleSession}
        />

        <Card>
          <Box
            sx={{
              display: 'flex',
              alignItems: 'center',
              justifyContent: 'space-between', // Space between Tabs and Button
              px: 2.5,
              boxShadow: (theme) =>
                `inset 0 -2px 0 0 ${alpha(theme.palette.grey[500], 0.08)}`,
            }}
          >
            {loading ? (
              <Grid
                style={{
                  display: "flex",
                  justifyContent: "center",
                  alignItems: "center",
                }}
              >
                <CircularProgress />
              </Grid>
            ) : (
              <Tabs
                value={filters.type}
                onChange={(event, newValue) => handleFilters("type", newValue)}
                sx={{
                  px: 2.5,
                  boxShadow: (theme) =>
                    `inset 0 -2px 0 0 ${alpha(theme.palette.grey[500], 0.08)}`,
                }}
              >
                {Type_OPTIONS.map((tab) => {
                  const count =
                    typeCount[tab.value as keyof typeof typeCount] || 0;
                  return (
                    <Tab
                      key={tab.value}
                      iconPosition="end"
                      value={tab.value}
                      label={tab.label}
                      icon={
                        <Label
                          variant={
                            ((tab.value === "all" ||
                              tab.value === filters.type) &&
                              "filled") ||
                            "soft"
                          }
                          color={
                            (tab.value === "Live" && "success") ||
                            (tab.value === "Demande_MP" && "warning") ||
                            (tab.value === "Article_a_controle" && "error") ||
                            (tab.value === "Autre" && "default") ||
                            "default"
                          }
                        >
                          {tab.value === ""
                            ? Object.values(typeCount).reduce(
                              (acc, count) => acc + Number(count),
                              0
                            )
                            : count}
                        </Label>
                      }
                    />
                  );
                })}
              </Tabs>
            )}

            <Button
              variant="contained"
              onClick={() => setOpenExportDialog(true)}
            >
              Exporter
            </Button>
          </Box>

          <SalesSessionTableToolbar
            filters={{
              ...filters,
              type: getTypeLabel(filters.type),
            }}
            onFilters={handleFilters}
          />

          {canReset && (
            <SalesSessionTableFiltersResult
              filters={{
                ...filters,
                type: getTypeLabel(filters.type),
              }}
              onFilters={handleFilters}
              //
              onResetFilters={handleResetFilters}
              //
              results={totalSaleSessions}
              sx={{ p: 2.5, pt: 0 }}
            />
          )}

          <TableContainer sx={{ position: "relative", overflow: "unset" }}>
            <TableSelectedAction
              dense={table.dense}
              numSelected={table.selected.length}
              rowCount={totalSaleSessions}
              onSelectAllRows={(checked) =>
                table.onSelectAllRows(
                  checked,
                  tableData.map((row) => row.id)
                )
              }
              onClickTrash={() => {
                isConfirm.onTrue();
              }}
            />

            <Scrollbar>
              <Table
                size={table.dense ? "small" : "medium"}
                sx={{ minWidth: 960 }}
              >
                <TableHeadCustom
                  order={table.order}
                  orderBy={table.orderBy}
                  headLabel={TABLE_HEAD}
                  rowCount={totalSaleSessions}
                  numSelected={table.selected.length}
                  onSort={table.onSort}
                  onSelectAllRows={
                    isAdmin
                      ? (checked) =>
                        table.onSelectAllRows(
                          checked,
                          tableData.map((row) => row.id)
                        )
                      : undefined
                  }
                />
                <TableBody>
                  {loading ? (
                    <TableRow>
                      <TableCell
                        colSpan={TABLE_HEAD.length}
                        style={{ textAlign: "center", padding: "20px" }}
                      >
                        <CircularProgress />
                      </TableCell>
                    </TableRow>
                  ) : (
                    <>
                      {notFound ? (
                        <TableNoData notFound={notFound} />
                      ) : (
                        tableData.map((row) => (
                          <SalesSessionTableRow
                            key={row.id}
                            row={row}
                            selected={table.selected.includes(row.id)}
                            onSelectRow={() => table.onSelectRow(row.id)}
                            onDeleteRow={() => handleDeleteRow(row.id)}
                            onEditRow={() => handleEditRow(row)}
                            isAdmin={isAdmin}
                          />
                        ))
                      )}

                      <TableEmptyRows
                        height={denseHeight}
                        emptyRows={emptyRows(
                          table.page,
                          table.rowsPerPage,
                          totalSaleSessions
                        )}
                      />
                    </>
                  )}
                </TableBody>


              </Table>
            </Scrollbar>
          </TableContainer>

          <TablePaginationCustom
            count={totalSaleSessions}
            page={pageNumber}
            rowsPerPage={pageSize}
            onPageChange={table.onChangePage}
            onRowsPerPageChange={table.onChangeRowsPerPage}
            dense={table.dense}
            onChangeDense={table.onChangeDense}
          />

        </Card>
      </Container>

      <ConfirmDialog
        open={isConfirm.value}
        onClose={isConfirm.onFalse}
        title="Supprimer les sessions"
        content={
          <>
            Êtes-vous sûr de vouloir supprimer{" "}
            <strong>{table.selected.length}</strong> session(s) ?
          </>
        }
        action={
          <LoadingButton
            loading={isDeleting}
            variant="contained"
            color="error"
            onClick={handleDeleteRows}
          >
            Supprimer
          </LoadingButton>
        }
      />
    </>
  );
}

// ----------------------------------------------------------------------

