"use client";

import React, { useState, useCallback } from "react";
import { ILiveItem, ISessionOrder } from "@/shared/types/saleSession";
import { endpoints, axiosInstance } from "@/shared/api/server";
import { useGetSaleSessions } from "@/shared/api/saleSession";
import Papa from "papaparse";
import { saveAs } from "file-saver";

import {
  Dialog,
  DialogTitle,
  DialogContent,
  DialogActions,
  Button,
  LinearProgress,
  Typography,
} from "@mui/material";
import { getProductDetailsByUGS } from "@/shared/api/product";

interface ExportDialogProps {
  open: boolean;
  onClose: () => void;
  saleSessions: ILiveItem[];
}

const SessionOrdersProcessor: React.FC<ExportDialogProps> = ({
  open,
  onClose,
  saleSessions,
}) => {
  const [processedData, setProcessedData] = useState<any[]>([]);
  const [exporting, setExporting] = useState(false);
  const [progress, setProgress] = useState(0);
  const [exportComplete, setExportComplete] = useState(false);

  const getOrdersBySessionID = async (id: string) => {
    try {
      const response = await axiosInstance.get(
        endpoints.salleSession.getOrdersBySessionId(id)
      );
      return Array.isArray(response.data) ? response.data : [];
    } catch (err) {
      console.error("Error getting orders for session", id, err);
      return [];
    }
  };

  const processOrders = async (orders: any[], session: ILiveItem) => {
    if (!Array.isArray(orders)) {
      console.error("Orders is not an array for session:", session.id);
      return [];
    }

    const processed = await Promise.all(
      orders.map(async (order) => {
        const ugs = order.itemCartDTO?.product?.ugs;
        const { emplacement, tags } = await getProductDetailsByUGS(ugs);

        return {
          idUtilisateur: order.client?.uid,
          idCommande: order.id,
          idSessionVente: session.id,
          typeLive: session.type,
          prenom: order.client?.firstName,
          nom: order.client?.lastName,
          email: order.client?.email,
          telephone: order.client?.phoneNumber,
          nomUtilisateur: order.client?.pseudo,
          nomProduit: order.itemCartDTO?.product?.name,
          UGS: ugs,
          quantite: order.itemCartDTO?.quantity,
          dateCreationProduitCommande: order.createdDate,
          prixProduit: order.price / (order.itemCartDTO?.quantity || 1),
          montant: order.price,
          statutPanier: order.itemCartDTO?.status,
          statutCommande: order.status,
          derniereModification:
            order.lastModification || session.lastModification,
          dateExpiration: session.expirationDate,
          emplacement,
          tags: tags.join(", "),
        };
      })
    );

    return processed;
  };

  const handleExport = useCallback(async () => {
    if (!saleSessions || saleSessions.length === 0) return;

    setExporting(true);
    setProgress(0);
    setProcessedData([]);

    const allProcessedOrders: any[] = [];
    const uniqueOrders = new Map();

    let processedSessionCount = 0;

    for (let i = 0; i < saleSessions.length; i += 15) {
      const batch = saleSessions.slice(i, i + 15);

      for (const session of batch) {
        try {
          const orders = await getOrdersBySessionID(session.id);
          const processedOrders = await processOrders(orders, session);
          allProcessedOrders.push(...processedOrders);

          setProcessedData((prevData) => {
            const newData = [...prevData, ...processedOrders];
            newData.forEach((order) => {
              uniqueOrders.set(order.idCommande, order);
            });
            return Array.from(uniqueOrders.values());
          });
        } catch (err) {
          console.error("Erreur lors du traitement de la session", session.id, err);
        }

        processedSessionCount++;

        setProgress(Math.round((processedSessionCount / saleSessions.length) * 100));

        // Ajoute une petite pause pour forcer le rendu React à chaque incrément
        await new Promise((resolve) => setTimeout(resolve, 50));
      }

      await new Promise((resolve) => setTimeout(resolve, 1000)); // pause batch
    }

    setExporting(false);
    setExportComplete(true);
    setProgress(100);
  }, [saleSessions]);


  const downloadCSV = useCallback((csvData: any[]) => {
    const csv = Papa.unparse(csvData);
    const blob = new Blob([csv], { type: "text/csv;charset=utf-8;" });
    saveAs(
      blob,
      "sale_Sessions" + new Date().toISOString().split("T")[0] + ".csv"
    );
  }, []);

  const handleClose = useCallback(() => {
    onClose();
    setProcessedData([]);
    setProgress(0);
    setExportComplete(false);
    setExporting(false);
  }, [onClose]);

  return (
    <Dialog open={open} onClose={handleClose} maxWidth="md" fullWidth>
      <DialogTitle>Exporter les sessions de ventes</DialogTitle>
      <DialogContent>
        {exporting ? (
          <>
            <Typography>Export en cours... {Math.round(progress)}%</Typography>
            <LinearProgress variant="determinate" value={progress} />
          </>
        ) : !saleSessions || saleSessions.length === 0 ? (
          <Typography>Aucune session pour exporter.</Typography>
        ) : exportComplete ? (
          <>
            <Typography>
              Nombre des lignes à exporter : {processedData.length}
            </Typography>
            <Button
              onClick={() => downloadCSV(processedData)}
              variant="contained"
              color="primary"
            >
              Télécharger CSV
            </Button>
          </>
        ) : null}
      </DialogContent>
      <DialogActions>
        <Button onClick={handleClose}>Annuler</Button>
        <Button
          onClick={handleExport}
          variant="contained"
          color="primary"
          disabled={exporting || !saleSessions || saleSessions.length === 0}
        >
          Commencer l'export
        </Button>
      </DialogActions>
    </Dialog>
  );
};

export default SessionOrdersProcessor;
