import Button from "@mui/material/Button";
import MenuItem from "@mui/material/MenuItem";
import TableRow from "@mui/material/TableRow";
import Checkbox from "@mui/material/Checkbox";
import TableCell from "@mui/material/TableCell";
import IconButton from "@mui/material/IconButton";
import ListItemText from "@mui/material/ListItemText";
import EditIcon from '@mui/icons-material/Edit';
import { Tooltip } from "@mui/material";
import { useBoolean } from "@/hooks/use-boolean";
import Label from "../../../components/label";
import Iconify from "@/components/iconify";
import { ConfirmDialog } from "../../../components/custom-dialog";
import CustomPopover, { usePopover } from "../../../components/custom-popover";
import { _livesInf } from "@/shared/_mock/_cartLive";
import { ILiveItem, LiveTyple } from "@/shared/types/saleSession";
import { fDate, fTime } from "@/utils/format-time";
import ShoppingCartIcon from "@mui/icons-material/ShoppingCart";
import { useCallback, useState } from "react";
import { useRouter } from "@/hooks";
import { paths } from "@/routes/paths";
import { getTokenInfo } from "@/utils/token";
import { deleteSaleSessionById } from "@/shared/api/saleSession";
import LoadingButton from "@mui/lab/LoadingButton";

type Props = {
  selected: boolean;
  onEditRow: VoidFunction;
  row: ILiveItem;
  onSelectRow: VoidFunction;
  onDeleteRow: VoidFunction;
  isAdmin: boolean;
};

export default function SalesSessionTableRow({
  row,
  selected,
  onEditRow,
  onSelectRow,
  onDeleteRow,
  isAdmin,
}: Props) {
  const {
    id,
    name,
    startedDate,
    lastModification,
    totalSales,
    clientsNumber,
    paidProductsNumber,
    orderProductsNumber,
    expirationDate,
    type,
    description,
  } = row;

  const confirm = useBoolean();
  const popover = usePopover();
  const router = useRouter();

  const handelPanier = useCallback(() => {
    const nameUrl = name.replace(/ /g, "+");
    router.push(paths.dashboard.admin.live.panier(id, nameUrl));
  }, [router, id, name]);
  const [isDeleting, setIsDeleting] = useState(false);
  const handleDelete = async () => {
    setIsDeleting(true);
    try {
      await deleteSaleSessionById(Number(id));
      onDeleteRow();
      confirm.onFalse();
    } catch (error) {
      console.error("Failed to delete sale session", error);
    } finally {
      setIsDeleting(false);
    }
  };
  const getTypeLabel = (value: string) => {
    const typeOption = LiveTyple.find((option) => option.value === value);
    return typeOption ? typeOption.label : value;
  };

  return (
    <>
      <TableRow hover selected={selected} onClick={handelPanier}>
        {isAdmin && (
          <TableCell padding="checkbox">
            <Checkbox
              checked={selected}
              onClick={(e) => {
                e.stopPropagation();
                onSelectRow();
              }}
            />
          </TableCell>
        )}

        <TableCell sx={{ whiteSpace: "nowrap" }}>{id}</TableCell>
        <TableCell sx={{ whiteSpace: "nowrap" }}>{name}</TableCell>
        <TableCell>
          <ListItemText
            primary={fDate(startedDate)}
            secondary={fTime(startedDate)}
            primaryTypographyProps={{ typography: "body2", noWrap: true }}
            secondaryTypographyProps={{
              mt: 0.5,
              component: "span",
              typography: "caption",
            }}
          />
        </TableCell>
        <TableCell>
          <ListItemText
            primary={fDate(lastModification)}
            secondary={fTime(lastModification)}
            primaryTypographyProps={{ typography: "body2", noWrap: true }}
            secondaryTypographyProps={{
              mt: 0.5,
              component: "span",
              typography: "caption",
            }}
          />
        </TableCell>

        <TableCell sx={{ whiteSpace: "nowrap" }}>
          {orderProductsNumber}
        </TableCell>
        <TableCell sx={{ whiteSpace: "nowrap" }}>{clientsNumber}</TableCell>
        <TableCell sx={{ whiteSpace: "nowrap" }}>{totalSales}</TableCell>
        <TableCell>
          <ListItemText
            primary={fDate(expirationDate)}
            secondary={fTime(expirationDate)}
            primaryTypographyProps={{ typography: "body2", noWrap: true }}
            secondaryTypographyProps={{
              mt: 0.5,
              component: "span",
              typography: "caption",
            }}
          />
        </TableCell>
        <TableCell sx={{ whiteSpace: "nowrap" }}>{description}</TableCell>
        <TableCell>
          <Label
            variant="soft"
            color={
              (type === "Live" && "success") ||
              (type === "Demande_MP" && "warning") ||
              (type === "Article_a_controle" && "error") ||
              (type === "Autre" && "default") ||
              "default"
            }
          >
            {getTypeLabel(type)}
          </Label>
        </TableCell>

        <TableCell
          align="right"
          sx={{ px: 1, whiteSpace: "nowrap" }}
          onClick={(e) => e.stopPropagation()}
        >
          <Tooltip title="Panier">
            <IconButton sx={{ color: "#00e676" }} onClick={handelPanier}>
              <ShoppingCartIcon />
            </IconButton>
          </Tooltip>
          <Tooltip title="Modifier">
          <IconButton sx={{ color: "primary.main" }} onClick={onEditRow}>
            <EditIcon />
          </IconButton>
        </Tooltip>
          {isAdmin && (
            <Tooltip title="Supprimer">
              <IconButton sx={{ color: "error.main" }} onClick={confirm.onTrue}>
                <Iconify icon="mdi:trash-can-outline" />
              </IconButton>
            </Tooltip>
          )}
        </TableCell>
        
      </TableRow>

      <ConfirmDialog
        open={confirm.value}
        onClose={confirm.onFalse}
        title="Confirmer la suppression"
        content="Êtes-vous sûr de vouloir supprimer cette session de vente ?"
        action={
          <LoadingButton
            loading={isDeleting}
            variant="contained"
            color="error"
            onClick={handleDelete}
          >
            Supprimer
          </LoadingButton>
        }
      />
    </>
  );
}
