
import React, { useState, useEffect } from 'react';
import {
  Dialog,
  DialogTitle,
  DialogContent,
  DialogActions,
  Button,
  TextField,
  FormHelperText,
} from '@mui/material';
import * as Yup from 'yup';
import { useSnackbar } from '@/components/snackbar';
import { ILiveItem } from '@/shared/types/saleSession';

interface EditSalesSessionDialogProps {
  open: boolean;
  onClose: () => void;
  initialData: ILiveItem | null;
  onModify: (updatedData: ILiveItem) => void;
}

interface FormErrors {
  name?: string;
  description?: string;
}

const validationSchema = Yup.object().shape({
  name: Yup.string().required('Le nom est requis'),
});

const EditSalesSessionDialog: React.FC<EditSalesSessionDialogProps> = ({
  open,
  onClose,
  initialData,
  onModify,
}) => {
  const { enqueueSnackbar } = useSnackbar();
  const [formData, setFormData] = useState<{ name: string; description: string }>({
    name: '',
    description: '',
  });
  const [formErrors, setFormErrors] = useState<FormErrors>({});

  useEffect(() => {
    if (initialData) {
      setFormData({
        name: initialData.name,
        description: initialData.description,
      });
    }
  }, [initialData]);

  const handleChange = (
    e: React.ChangeEvent<HTMLInputElement | { name?: string; value: unknown }>
  ) => {
    const { name, value } = e.target;
    setFormData((prev) => ({
      ...prev,
      [name as string]: value,
    }));
    setFormErrors((prev) => ({
      ...prev,
      [name as string]: undefined,
    }));
  };

  const handleModify = async () => {
    try {
      await validationSchema.validate(formData, { abortEarly: false });
      if (initialData) {
        onModify({
          ...initialData,
          name: formData.name,
          description: formData.description,
        });
        enqueueSnackbar('Session de vente modifiée avec succès', { variant: 'success' });
        onClose();
      }
    } catch (error: any) {
      if (error instanceof Yup.ValidationError) {
        const errors: FormErrors = {};
        error.inner.forEach((err) => {
          if (err.path) {
            errors[err.path as keyof FormErrors] = err.message;
          }
        });
        setFormErrors(errors);
        enqueueSnackbar('Veuillez corriger les erreurs du formulaire', { variant: 'error' });
      } else {
        enqueueSnackbar("Erreur lors de la modification de la session de vente", {
          variant: 'error',
        });
      }
    }
  };
  useEffect(() => {
    if (initialData) {
      setFormData({
        name: initialData.name,
        description: initialData.description,
      });
    }
  }, [initialData]);
  

  const handleCancel = () => {
    onClose();
    setFormErrors({});
  };

  return (
    <Dialog open={open} onClose={handleCancel} fullWidth maxWidth="sm">
      <DialogTitle>Modifier la session de vente</DialogTitle>
      <DialogContent>
        <TextField
          fullWidth
          margin="normal"
          label="Nom"
          variant="outlined"
          name="name"
          value={formData.name}
          onChange={handleChange}
          error={Boolean(formErrors.name)}
          helperText={formErrors.name}
        />
        <TextField
          fullWidth
          margin="normal"
          label="Description"
          variant="outlined"
          name="description"
          value={formData.description}
          onChange={handleChange}
          error={Boolean(formErrors.description)}
          helperText={formErrors.description}
          multiline
          rows={4}
        />
      </DialogContent>
      <DialogActions>
        <Button onClick={handleCancel} color="secondary">
          Annuler
        </Button>
        <Button onClick={handleModify} color="primary" variant="contained">
          Modifier
        </Button>
      </DialogActions>
    </Dialog>
  );
};

export default EditSalesSessionDialog;
