import { useCallback, useState } from "react";

import Stack from "@mui/material/Stack";
import MenuItem from "@mui/material/MenuItem";
import TextField from "@mui/material/TextField";
import IconButton from "@mui/material/IconButton";
import InputAdornment from "@mui/material/InputAdornment";

import Iconify from "../../../components/iconify";
import CustomPopover, { usePopover } from "../../../components/custom-popover";
import { SalesSessionTableFilters, SalesSessionTableFilterValue } from "@/shared/types/saleSession";
import CustomDateRangePicker from "@/shared/components/custom-date-range-picker";
import { fDate, formatDateToString } from "@/utils/format-time";
import { FormControl, Card, ListItemText } from "@mui/material";


// ----------------------------------------------------------------------

type Props = {
  filters: SalesSessionTableFilters;
  onFilters: (name: string, value: SalesSessionTableFilterValue) => void;
  //
};

export default function SalesSessionTableToolbar({
  filters,
  onFilters,
}: //

Props) {
  const popover = usePopover();

  const handleFilterName = useCallback(
    (event: React.ChangeEvent<HTMLInputElement>) => {
      onFilters("name", event.target.value);
    },
    [onFilters]
  );

 const [isDateRangeOpen, setIsDateRangeOpen] = useState(false);
  const [startDate, setStartDate] = useState<Date | null>(null);
  const [endDate, setEndDate] = useState<Date | null>(null);

  const openDateRange = useCallback(() => {
    setIsDateRangeOpen(true);
  }, []);

  const closeDateRange = useCallback(() => {
    setIsDateRangeOpen(false);
  }, []);

  const onChangeStartDate = (newStartDate: Date | null) => {
    setStartDate(newStartDate);
    if (newStartDate) {
      const formattedDate = formatDateToString(newStartDate);
      onFilters("startDate", formattedDate);
    }
  };

  const onChangeEndDate = (newEndDate: Date | null) => {
    setEndDate(newEndDate);
    if (newEndDate) {
      onFilters("endDate", formatDateToString(newEndDate));
    }
  };
  

  return (
    <>
      <Stack
        spacing={2}
        alignItems={{ xs: "flex-end", md: "center" }}
        direction={{
          xs: "column",
          md: "row",
        }}
        sx={{
          p: 2.5,
          pr: { xs: 2.5, md: 1 },
        }}
      >
        <Stack
          direction="row"
          alignItems="center"
          spacing={2}
          flexGrow={1}
          sx={{ width: 1 }}
        >
          <TextField
            fullWidth
            value={filters.name}
            onChange={handleFilterName}
            placeholder="Recherche..."
            InputProps={{
              startAdornment: (
                <InputAdornment position="start">
                  <Iconify
                    icon="eva:search-fill"
                    sx={{ color: "text.disabled" }}
                  />
                </InputAdornment>
              ),
            }}
          />

          
        </Stack>
        <FormControl
          sx={{
            flexShrink: 0,
            width: { xs: 1, md: 350 },
          }}
        >
          <Card
            onClick={openDateRange}
            variant="outlined"
            sx={{
              height: 55,
              cursor: "pointer",
              borderRadius: 1,
              p: 3,
              display: "flex",
              boxShadow: "none",
              justifyContent: "space-between",
              alignItems: "center",
            }}
          >
            <Iconify sx={{ mr: 3 }} icon="solar:calendar-date-bold" />
            <ListItemText
              secondary={
                filters.startDate && filters.endDate
                  ? `${fDate(filters.startDate)} - ${fDate(filters.endDate)}`
                  : filters.startDate
                  ? `${fDate(filters.startDate)} - sélectionner la date de fin`
                  : filters.endDate
                  ? `sélectionner la date de début - ${fDate(filters.endDate)}`
                  : "Sélectionner une plage de dates"
              }
              primaryTypographyProps={{
                typography: "body2",
                color: "text.primary",
                mb: 0.5,
              }}
            />
          </Card>

          <CustomDateRangePicker
            open={isDateRangeOpen}
            onClose={closeDateRange}
            startDate={startDate}
            endDate={endDate}
            onChangeStartDate={onChangeStartDate}
            onChangeEndDate={onChangeEndDate}
          />
        </FormControl>



      </Stack>

      
    </>
  );
}
