import { useState } from 'react';

import { useResponsive } from '@/hooks';
import { InputEmoji } from '@/shared/components/input-emoji';
import Iconify from '@/components/iconify';
import { addCommentLive, unpinCommentLive } from '@/shared/api/live';
import { Card, IconButton, Paper, Typography, useTheme } from '@mui/material';
import { ILiveComment } from '@/shared/types/live';
import { useLiveData } from '@/contexts/live-details';
import useLiveComments from '@/contexts/live-comments/use-live-comments';


type Props = {
    parentComment: ILiveComment | null;
    changeParentComment: (comment: ILiveComment | null) => void
};

export default function CommentsInput({
   parentComment, changeParentComment
}: Props) {
    const [comment, setComment] = useState('');
    const liveData = useLiveData();
    const { pinnedComments } = useLiveComments();

    const theme = useTheme();


    const handleSendComment = async () => {
        try {
            if (comment !== '') {
                await addCommentLive({
                    content: comment,
                    type: "CHAT",
                    admin: true,
                    senderName: liveData.chatName || "mydressin" ,
                    parent: parentComment ? parentComment : undefined,
                    liveStream: liveData.id
                });
                setComment('');
                changeParentComment(null);
            }

        } catch (error) {
            console.error(error);
        }
    };

    return (
        <Paper sx={{
            zIndex: 2,
            flexShrink: 0,
            display: 'flex',
            flexDirection: 'column',
            alignItems: 'center',
            justifyContent: 'center',
            width: {md: '98%', xs: '100%'},
            gap: 0,
            left: '50%',
            background: 'transparent',
            borderRadius: '40px',
            position: 'absolute',
            transform: 'translateX(-50%)',
            bottom: {md: 1, xs:-40},
        }}>
            {pinnedComments.length != 0 && <Paper style={{
                display: 'flex',
                flexDirection: 'column',
                background: theme.palette.background.default,
                gap:3,
                alignItems: 'center',
                justifyContent: 'center',
                borderRadius: '10px',
                width: '100%',
                padding: 10,
            }}>
                {pinnedComments.map((comment: ILiveComment) => (
                    <Card sx={{ position: "relative", width: "100%", display: "flex", alignItems: "center", wordWrap: 'break-word', borderRadius: "40px", gap: 2, paddingX: 1, }}>
                        <Typography variant="body2" sx={{ color: 'gray' }}>
                            {comment?.senderName}:
                        </Typography>
                        <Typography variant="body1" >
                            {comment?.content}
                        </Typography>
                        <IconButton onClick={async () => await unpinCommentLive(comment.id as string, liveData.id)} sx={{ position: "absolute", right: 0, padding: 1 }} size="small">
                            <Iconify style={{ color: 'gray', marginBottom: 2 }} width={20} icon="ic:baseline-pin-off" />
                        </IconButton>
                    </Card>
                ))}
            </Paper>}

            <Paper style={{
                width: "100%",
                background: theme.palette.background.default,
                borderRadius: '10px',
            }}>
                {parentComment && <Paper style={{
                    display: 'flex',
                    alignItems: 'center',
                    justifyContent: 'start',
                    gap: 4,
                    width: '100%',
                    padding: 10,
                    marginTop: 2,
                    borderRadius: 2,
                    background: 'transparent',
                }}>
                    <Card sx={{ position: "relative", width: "100%", display: "flex", alignItems: "center", wordWrap: 'break-word', borderRadius: "40px", gap: 2, paddingX: 1, }}>
                        <Typography variant="body2" sx={{ color: 'gray' }}>
                            {parentComment?.senderName}:
                        </Typography>
                        <Typography variant="body1" >
                            {parentComment?.content}
                        </Typography>
                        <IconButton onClick={() => changeParentComment(null)} sx={{ position: "absolute", right: 0, padding: 1 }} size="small">
                            <Iconify style={{ color: 'gray', marginBottom: 2 }} width={20} icon="ic:sharp-cancel" />
                        </IconButton>
                    </Card>

                </Paper>}

                <Paper style={{
                    display: 'flex',
                    alignItems: 'center',
                    justifyContent: 'center',
                    width: '100%',
                    padding: 0,
                    background: 'transparent',
                }}>
                    <InputEmoji
                        value={comment}
                        onChange={(value) => setComment(value)}
                        placeholder="Type a message"
                        height={20}
                        width={"100%"}
                        key="comment"
                        onEnter={handleSendComment}
                        keepOpened={true}
                    />

                    <Iconify onClick={handleSendComment} style={{ height: 30, width: 30, color: 'gray', marginBottom: 2 }} icon="iconamoon:send-fill" />
                </Paper>
            </Paper>
        </Paper>
    );
}
