import Stack from '@mui/material/Stack';
import IconButton from '@mui/material/IconButton';
import Typography from '@mui/material/Typography';
import Iconify from '@/components/iconify';
import { ILiveComment } from '@/shared/types/live';
import { Card, useTheme, alpha } from '@mui/material';
import { useResponsive } from '@/hooks';
import { blockUserLive, deleteCommentLive, pinCommentLive } from '@/shared/api/live';
import { useSnackbar } from '@/components/snackbar';

type Props = {
  comment: ILiveComment;
  changeParentComment: (comment: ILiveComment) => void;
};

export default function LiveCommentItem({ comment, changeParentComment }: Props) {
  const { content, senderName, parent, admin } = comment;
  const theme = useTheme();
  const mdUp = useResponsive('up', 'md');
  const bgStyle = mdUp ? {
    backgroundColor: !admin ? alpha(theme.palette['primary'].main, 0.2) : "black",
    color: !admin ? "black" : "white",
    width: "100%",
    p: .5,
    fontSize: theme.typography.pxToRem(12),
  } : {
    fontSize: theme.typography.pxToRem(12),
    color: 'white',
    borderColor: 'gray',
    textDecorationColor: 'gray',
    width: 200,
    p: .5,
  };

  const {enqueueSnackbar} = useSnackbar();

  const handleBlockUser = async () => {
    try {
      await blockUserLive(comment.senderName as string, comment.liveStream);
      enqueueSnackbar('utilisateur bloqué', { variant: 'success' });
    } catch (error) {
      enqueueSnackbar('Erreur lors du blocage de l\'utilisateur', { variant: 'error' });
    }
  }

  const renderInfo = (
    <Typography
      noWrap
      variant="caption"
      sx={{
        mr: 'auto',
        color: 'grey'
      }}
    >
      {senderName}
    </Typography>
  );

  const renderBody = (
    <Card
      sx={{
        borderRadius: 1,
        ...bgStyle,
        height: 'auto',
        maxWidth: '100%',
        wordWrap: 'break-word'
      }}
      key={comment.id}
    >
      {parent && (
        <Stack
          direction="row"
          alignItems="center"
          sx={{
            borderLeft: '2px solid',
            borderRight: '2px solid',
            borderColor: 'primary.main',
            borderRadius: 1,
            pl: 1,
            mb: 1,
            width: '100%',
            flexWrap: 'wrap',
          }}
        >
          <Typography
            variant="caption"
            sx={{ color: 'primary.main', flexShrink: 0 }}
          >
            {parent.senderName}
          </Typography>
          <Typography
            variant="caption"
            sx={{
              color: 'text.secondary',
              marginLeft: 1,
              overflow: 'hidden',
              textOverflow: 'ellipsis',
              whiteSpace: 'normal',
              flexGrow: 1,
              wordBreak: 'break-word',
            }}
          >
            {parent.content}
          </Typography>
        </Stack>
      )}
      <Typography variant="body2">
        {content}
      </Typography>
    </Card>
  );

  const renderActions = (
    <Stack
      direction="row"
      className="message-actions"
      sx={{
        pt: 0.5,
        opacity: 0,
        bottom: '100%',
        right: 1,
        position: 'absolute',
        transition: (theme) =>
          theme.transitions.create(['opacity'], {
            duration: theme.transitions.duration.shorter,
          }),
      }}
    >
     { !admin && <IconButton onClick={handleBlockUser} size="small">
        <Iconify icon="mdi:block" width={16} />
      </IconButton>}
      <IconButton onClick={() => changeParentComment(comment)} size="small">
        <Iconify icon="solar:reply-bold" width={16} />
      </IconButton>
      <IconButton onClick={() => deleteCommentLive(comment.id as string, comment.liveStream)} size="small">
        <Iconify icon="material-symbols:delete" width={16} />
      </IconButton>
      <IconButton onClick={() => pinCommentLive(comment.id as string, comment.liveStream)} size="small">
        <Iconify icon="mdi:pin" width={16} />
      </IconButton>
    </Stack>
  );

  return (
    <Stack direction="column" justifyContent={'unset'} sx={{ mb: 2 }}>
      <Stack alignItems="flex-end">
        {renderInfo}
      </Stack>
      <Stack
        direction="row"
        alignItems="flex-end"
        sx={{
          position: 'relative',
          '&:hover': {
            '& .message-actions': {
              opacity: 1,
            },
          },
        }}
      >
        {renderBody}
        {renderActions}
      </Stack>
    </Stack>
  );
}
