import { useCallback, useEffect, useState } from "react";
import { IconButton, Paper, Stack, TextField, Typography } from "@mui/material";
import LoadingButton from '@mui/lab/LoadingButton';
import Iconify from "@/components/iconify";
import Label from "@/shared/components/label";
import { useBoolean, useResponsive, useRouter } from "@/hooks";
import { paths } from "@/routes/paths";
import RouterLink from "@/routes/router-link";
import { fDateTime } from "@/utils/format-time";
import { LiveStatus } from "@/shared/types/live";
import { publishLive, startLive, stopLive } from "@/shared/api/live";
import { useGetLiveNumberOfviewers } from "@/contexts/live-stats";
import { ConfirmDialog } from "@/shared/components/custom-dialog";
import { useSnackbar } from "@/components/snackbar";
import useAsync from "@/hooks/use-async";
import Countdown from "./countdow";
import { useLiveData } from "@/contexts/live-details";


export default function HeadSection() {
    const { id: liveId, startedAt, status, startDate } = useLiveData();
    const smUp = useResponsive('up', 'sm');
    const [elapsedTime, setElapsedTime] = useState('00:00:00');
    const numberOfViewers = useGetLiveNumberOfviewers();
    const { enqueueSnackbar } = useSnackbar();

    useEffect(() => {
        let timer: NodeJS.Timeout;
        if (status === LiveStatus.ONGOING && startedAt) {
            timer = setInterval(() => {
                const now = new Date();
                const nowUTC = Date.UTC(
                    now.getUTCFullYear(),
                    now.getUTCMonth(),
                    now.getUTCDate(),
                    now.getUTCHours(),
                    now.getUTCMinutes(),
                    now.getUTCSeconds()
                );

                const start = new Date(`${startedAt}Z`); // Add 'Z' to force UTC interpretation
                const startUTC = Date.UTC(
                    start.getUTCFullYear(),
                    start.getUTCMonth(),
                    start.getUTCDate(),
                    start.getUTCHours(),
                    start.getUTCMinutes(),
                    start.getUTCSeconds()
                );

                const diff = nowUTC - startUTC;  // Time difference in milliseconds

                // Convert the time difference into hours, minutes, and seconds
                const hours = Math.floor(diff / 3600000);
                const minutes = Math.floor((diff % 3600000) / 60000);
                const seconds = Math.floor((diff % 60000) / 1000);

                // Update the elapsed time
                setElapsedTime(
                    `${hours.toString().padStart(2, '0')}:${minutes.toString().padStart(2, '0')}:${seconds.toString().padStart(2, '0')}`
                );
            }, 1000);
        }

        return () => {
            if (timer) clearInterval(timer);
        };
    }, [status, startedAt]);



    const startConfirm = useBoolean();
    const stopConfirm = useBoolean();
    const publishConfirm = useBoolean();

    const router = useRouter();
    const navigateToLives = useCallback(() => {
        router.push(paths.dashboard.admin.live.info(liveId));
    }, [router]);

    const asyncStartLive = useAsync(async () => {
        await startLive(liveId);
    });

    const asyncStopLive = useAsync(async () => {
        await stopLive(liveId);
    });

    const asyncPublishLive = useAsync(async () => {
        await publishLive(liveId);
    });

    const handleStartLive = async () => {
        try {
            await asyncStartLive.execute();
            enqueueSnackbar('Le live a démarré', { variant: 'success' });
        } catch (error) {
            enqueueSnackbar(error as string, { variant: 'error' });
        } finally {
            startConfirm.onFalse();
        }
    };


    const handleStopLive = async () => {
        try {
            await asyncStopLive.execute();
            enqueueSnackbar('Le live a été arrêté', { variant: 'success' });
        } catch (error) {
            enqueueSnackbar(error as string, { variant: 'error' });
        } finally {
            stopConfirm.onFalse();
        }
    };

    const handlePublishLive = async () => {
        try {
            await asyncPublishLive.execute();
            enqueueSnackbar('Le live a été publié', { variant: 'success' });
        } catch (error) {
            enqueueSnackbar(error as string, { variant: 'error' });
        } finally {
            publishConfirm.onFalse();
        }
    };


    return (
        <>
            <Paper
                sx={{
                    position: 'relative',
                    display: 'flex',
                    flexDirection: { xs: "column", lg: "row" },
                    mb: 4,
                    justifyContent: 'space-between',
                    alignItems: { xs: "start", lg: "center" },
                    gap: 2,
                    width: "100%",
                    bgcolor: "transparent",
                }}
            >
                <Stack width={300} direction="row" alignItems="flex-start">
                    <IconButton onClick={navigateToLives}>
                        <Iconify icon="eva:arrow-ios-back-fill" />
                    </IconButton>

                    <Stack spacing={0.5}>
                        <Stack spacing={1} direction="row" alignItems="center">
                            <Typography variant="h4">Live #{liveId}</Typography>
                            <Label variant="soft" color="success">
                                {status === LiveStatus.REPLAY && 'Rediffusion'}
                                {status === LiveStatus.REVIEW && 'En revue'}
                                {status === LiveStatus.COMING && 'à venir'}
                                {status === LiveStatus.ONGOING && 'En cours'}
                                {status === LiveStatus.ARCHIVED && 'Archivé'}
                            </Label>
                        </Stack>

                        <Typography variant="body2" sx={{ color: 'text.disabled' }}>
                            {fDateTime(startDate && new Date(startDate))}
                        </Typography>
                    </Stack>
                </Stack>

                <Stack
                    spacing={3}
                    justifyContent={{ xs: 'center', md: 'flex-end' }}
                    alignItems="center"
                    direction="row"
                    height="100%"
                >
                    {status === LiveStatus.ONGOING && (
                        <>
                            <TextField
                                label="Spectateur"
                                size="small"
                                variant="outlined"
                                sx={{ maxWidth: 100 }}
                                value={numberOfViewers}
                            />
                            {smUp && (
                                <TextField
                                    label="Ajout au panier"
                                    size="small"
                                    variant="outlined"
                                    sx={{ maxWidth: 110 }}
                                    value={0}
                                />
                            )}

                            <TextField
                                value={elapsedTime}
                                label="Durée"
                                size="small"
                                variant="outlined"
                                sx={{ maxWidth: 100 }}
                                InputLabelProps={{ shrink: true }}
                                inputProps={{ readOnly: true }}
                            />
                        </>
                    )}
                    {status === LiveStatus.COMING && (
                        <>
                            <Countdown startDate={startDate ? new Date(startDate) : new Date()} />
                            <LoadingButton
                                onClick={startConfirm.onTrue}
                                variant="contained"
                                startIcon={<Iconify icon="material-symbols:not-started-rounded" />}
                                loading={asyncStartLive.loading}
                            >
                                {smUp && "Démarrer"}
                            </LoadingButton>
                        </>
                    )}
                    {status === LiveStatus.ONGOING && (
                        <LoadingButton
                            onClick={stopConfirm.onTrue}
                            variant="contained"
                            sx={{ color: 'error.main' }}
                            startIcon={<Iconify icon="fluent:record-stop-24-regular" />}
                            loading={asyncStopLive.loading}
                        >
                            {smUp && "Arrêter"}
                        </LoadingButton>
                    )}
                    {status === LiveStatus.REVIEW && (
                        <LoadingButton
                            onClick={publishConfirm.onTrue}
                            variant="contained"
                            color="info"
                            startIcon={<Iconify icon="ic:round-publish" />}
                            loading={asyncPublishLive.loading}
                        >
                            {smUp && "Publier"}
                        </LoadingButton>
                    )}
                    <LoadingButton
                        href={paths.dashboard.admin.live.root}
                        LinkComponent={RouterLink}
                        variant="contained"
                        startIcon={<Iconify icon="material-symbols-light:text-select-move-back-word" />}
                    >
                        {smUp && "Quitter"}
                    </LoadingButton>
                </Stack>
            </Paper>
            <ConfirmDialog
                open={stopConfirm.value}
                onClose={stopConfirm.onFalse}
                title="Arrêter"
                content="Voulez-vous arrêter le live?"
                action={
                    <LoadingButton variant="contained" color="error" onClick={handleStopLive} loading={asyncStopLive.loading}>
                        Arrêter
                    </LoadingButton>
                }
            />
            <ConfirmDialog
                open={startConfirm.value}
                onClose={startConfirm.onFalse}
                title="Démarrer"
                content="Voulez-vous démarrer le live?"
                action={
                    <LoadingButton variant="contained" color="success" onClick={handleStartLive} loading={asyncStartLive.loading}>
                        Démarrer
                    </LoadingButton>
                }
            />

            <ConfirmDialog
                open={publishConfirm.value}
                onClose={publishConfirm.onFalse}
                title="Démarrer"
                content="Voulez-vous publier le live?"
                action={
                    <LoadingButton variant="contained" color="info" onClick={handlePublishLive} loading={asyncPublishLive.loading}>
                        Publier
                    </LoadingButton>
                }
            />
        </>
    );
}
