"use client";

import React, { useState, useCallback, useEffect } from 'react';
import {
    Avatar,
    Box,
    Card,
    Grid,
    IconButton,
    ListItemText,
    MenuItem,
    useTheme,
} from '@mui/material';
import { TimeField } from '@mui/x-date-pickers';
import Iconify from '@/components/iconify';
import {
    calculateSeconds,
    convertDateToUTC,
    differenceInSeconds,
    secondsToTime,
} from '@/utils/format-time';
import { useLiveData } from '@/contexts/live-details';
import { LiveProductStatus, LiveStatus, TimeStampProduct } from '@/shared/types/live';
import { useSnackbar } from '@/components/snackbar';
import EditIcon from '@mui/icons-material/Edit';
import DeleteIcon from '@mui/icons-material/Delete';
import useAsync from '@/hooks/use-async';
import LoadingButton from '@mui/lab/LoadingButton';
import CustomPopover, { usePopover } from '@/shared/components/custom-popover';
import useLiveProducts from '@/contexts/live-products/use-live-products';


type ProductCardProps = {
    liveId: string;
    id: string;
    title: string;
    status: LiveProductStatus;
    sku: string;
    image: string;
    price: number;
    timeStampProduct: TimeStampProduct[];
};

type TimeStampProductFiledProps = {
    liveProductId: string;
    liveId: string;
    id?: string;
    initialStartTime?: number;
    initialEndTime?: number;
    status: LiveProductStatus;
};

const TimeStampProductFiled = ({ id, initialStartTime, initialEndTime, status, liveId, liveProductId }: TimeStampProductFiledProps) => {
    const [startTime, setStartTime] = useState<number >(initialStartTime || 0);
    const [endTime, setEndTime] = useState<number >(initialEndTime || 0);
    const [modify, setModify] = useState<boolean>(false);
    const [loading,setLoading] = useState<boolean>(false);
    const popover = usePopover();
    const { deleteTimeStampProductFunction, updateTimeStampProductFunction, addTimeStampProductFunction } = useLiveProducts();


    const handleDeleteTimeStampProduct = useCallback(() => {
        deleteTimeStampProductFunction(liveProductId,id as string, setLoading);
    }, [deleteTimeStampProductFunction, id, liveId, liveProductId]);

    const handleUpdateTimeStampProduct = () => {
        updateTimeStampProductFunction(liveProductId,id as string, startTime , endTime, (loading:boolean) => {
            setLoading(loading);
            setModify(loading);
        });
    };

    const handleAddTimeStampProduct = () => {
        addTimeStampProductFunction(liveProductId, startTime , endTime , setLoading);
    };


    return (
        <Box sx={{ display: 'flex', width: '100%', justifyContent: 'start', gap: 1 }}>
            <TimeField
                value={startTime && secondsToTime(startTime)}
                onChange={(date) => setStartTime(calculateSeconds(date as Date))}
                disabled={id != undefined && !modify}
                fullWidth

                size="small"
                label="Début"
                InputLabelProps={{
                    shrink: true,
                    style: { fontSize: '0.8rem', }
                }}
                inputProps={{

                    step: 1,
                    style: { fontSize: '0.8rem', textAlign: 'center', height: 10, width:200 }
                }}
                sx={{ width: 200 }}
                ampm={false}
                format="HH:mm:ss"
            />

            <TimeField
                fullWidth
                size="small"
                label="Fin"
                onChange={(date) => setEndTime(calculateSeconds(date as Date))}
                value={endTime && secondsToTime(endTime)}
                disabled={id != undefined && !modify}
                InputLabelProps={{
                    shrink: true,
                    style: { fontSize: '0.8rem' },
                }}
                inputProps={{
                    step: 1,
                    style: { fontSize: '0.8rem', textAlign: 'center', height: 10 , width:200 }
                }}
                sx={{ width: 200 }}
                ampm={false}
                format="HH:mm:ss"
            />

            <CustomPopover
                open={popover.open}
                onClose={popover.onClose}
                arrow="right-top"
                sx={{ width: 160 }}
            >
                <MenuItem
                    onClick={() => {
                        setModify(true);
                        popover.onClose();

                    }}

                >
                    <LoadingButton loading={loading}  >
                        <EditIcon /> modifier
                    </LoadingButton>
                </MenuItem>
                <MenuItem
                    onClick={() => {
                        handleDeleteTimeStampProduct();
                    }}

                >
                    <LoadingButton loading={loading}  >
                        <DeleteIcon /> supprimer
                    </LoadingButton>
                </MenuItem>
            </CustomPopover>


            {id != undefined ? (
                <>
                    {modify ? (
                        <LoadingButton loading={loading} onClick={handleUpdateTimeStampProduct} disabled={loading}>
                            <Iconify icon="dashicons:saved" />
                        </LoadingButton>
                    ) : (
                        <IconButton color={popover.open ? 'inherit' : 'default'} onClick={popover.onOpen}>
                            <Iconify icon="eva:more-vertical-fill" />
                        </IconButton>
                    )}
                </>
            ) : (
                <IconButton  onClick={handleAddTimeStampProduct} disabled={loading}>
                    <Iconify icon="carbon:add-filled" />
                </IconButton>
            )}
        </Box>
    );
};

const ProductCard: React.FC<ProductCardProps> = ({
    liveId,
    id,
    title,
    status,
    sku,
    image,
    timeStampProduct = [],
}) => {
    const theme = useTheme();
    const { enqueueSnackbar } = useSnackbar();
    const { startedAt, status: liveStatus } = useLiveData();
    const [loading, setLoading] = useState(false);
    const { presentProductInLiveFunction, pushProductToLiveFunction, hideProductFromLiveFunction ,deleteProductFromLiveFunction} = useLiveProducts();

    const handlePushProduct = useCallback(() => {
        if (status === LiveProductStatus.NOT_PRESENTED) {
            presentProductInLiveFunction(id, liveId, differenceInSeconds(convertDateToUTC(new Date()), new Date(startedAt as Date)), setLoading);
        } else if (status === LiveProductStatus.PRESENTED) {
            pushProductToLiveFunction(id, liveId, differenceInSeconds(convertDateToUTC(new Date()), new Date(startedAt as Date)), setLoading);
        } else {
            enqueueSnackbar('Produit déjà présenté', { variant: 'warning' });
        }
    }, [presentProductInLiveFunction, pushProductToLiveFunction, id, liveId, status, startedAt]);

    const handleHideProduct = useCallback(() => {
        hideProductFromLiveFunction(id,  differenceInSeconds(convertDateToUTC(new Date()), new Date(startedAt as Date)), setLoading);
    }, [hideProductFromLiveFunction, id, liveId]);


    const handleDeleteProduct = useCallback(() => {
        deleteProductFromLiveFunction(id,setLoading);
    }, [deleteProductFromLiveFunction, id]);

    return (
        <Card variant="outlined" sx={{ marginBottom: 2, bgcolor: theme.palette.background.neutral }}>
            <Grid container>
                <Grid item xs={4}>
                    <Avatar
                        variant="rounded"
                        sx={{ height: '100%', width: '100%' }}
                        alt={title}
                        src={image}
                    />
                </Grid>
                <Grid item xs={8} sx={{ p: 1, gap: 0, display: 'flex', flexDirection: 'column' }} padding={2}>
                    <ListItemText
                        primary={title}
                        secondary={sku}
                        primaryTypographyProps={{ typography: 'body2', noWrap: true }}
                        secondaryTypographyProps={{
                            mt: 0.5,
                            component: 'span',
                            typography: 'caption',
                        }}
                    />
                    <Box sx={{ display: 'flex', width: '100%', justifyContent: 'space-between', gap: 1, mb: 2 }}>
                        {liveStatus === LiveStatus.ONGOING && (
                            <>
                                {(status === LiveProductStatus.NOT_PRESENTED || status === LiveProductStatus.PRESENTED) && (
                                    <LoadingButton loading={loading} variant="contained" fullWidth onClick={handlePushProduct} disabled={loading}>
                                        Pousser
                                    </LoadingButton>
                                )}
                                {status === LiveProductStatus.IN_PRESENTATION && (
                                    <LoadingButton loading={loading} variant="contained" color="info" fullWidth onClick={handleHideProduct} disabled={loading}>
                                        Cacher
                                    </LoadingButton>
                                )}
                            </>
                        )}
                        <LoadingButton
                            onClick={handleDeleteProduct}
                            variant="contained"
                            fullWidth
                            startIcon={<Iconify icon="ic:round-delete" />}
                            disabled={loading}
                            loading={loading}

                        >
                            Supprimer
                        </LoadingButton>
                    </Box>
                    {(liveStatus === LiveStatus.REVIEW || liveStatus === LiveStatus.REPLAY) && (
                        <>
                            {timeStampProduct.map((timeStamp) => (
                                <TimeStampProductFiled
                                    key={timeStamp.id}
                                    id={timeStamp.id}
                                    initialStartTime={timeStamp.startTime}
                                    initialEndTime={timeStamp.endTime}
                                    status={status}
                                    liveId={liveId}
                                    liveProductId={id}
                                />
                            ))}
                            <TimeStampProductFiled
                                key={-1}
                                status={status}
                                liveId={liveId}
                                liveProductId={id}
                            />
                        </>
                    )}
                </Grid>
            </Grid>
        </Card>
    );
};

export default ProductCard;
