// components/ProductsSearch.tsx
import React, { useMemo, useCallback, useState, useEffect } from 'react';
import {
    Box,
    TextField,
    InputAdornment,
    Button,
    Autocomplete,
    Grid,
    ListItemText,
    Avatar,
    SxProps,
    Theme,
    CircularProgress,
    IconButton,
    Chip,
    Stack,
} from '@mui/material';
import { autocompleteClasses } from '@mui/material/Autocomplete';
import Iconify from '@/shared/components/iconify';
import SearchNotFound from '@/shared/components/search-not-found';
import Label from '@/shared/components/label';
import { ILiveProduct, IProductListItem, LiveProductStatus } from '@/shared/types/live';
import useLiveProducts from '@/contexts/live-products/use-live-products';

type Props = {
    query: string;
    results: IProductListItem[];
    onSearch: (inputValue: string) => void;
    loading: boolean;
    sx?: SxProps<Theme>;
    minSearchChars?: number;
};

// Maximum number of search history items to store
const MAX_SEARCH_HISTORY = 5;

const ProductsSearch: React.FC<Props> = ({
    query,
    results,
    onSearch,
    loading,
    sx,
    minSearchChars = 3,
}) => {
    const { addProductToLiveFunction, products } = useLiveProducts();
    const [searchHistory, setSearchHistory] = useState<string[]>([]);
    
    // Log when props change
    useEffect(() => {
        console.log('LiveSearch component received results:', results);
        console.log('Current query:', query);
        console.log('Loading state:', loading);
    }, [results, query, loading]);
    
    // Save search to history (in-memory only)
    const saveSearchToHistory = useCallback((searchTerm: string) => {
        if (!searchTerm || searchTerm.trim().length < minSearchChars) return;
        
        setSearchHistory(prevHistory => {
            const normalizedTerm = searchTerm.trim();
            // Remove duplicates and add new term at the beginning
            const newHistory = [
                normalizedTerm, 
                ...prevHistory.filter(term => term !== normalizedTerm)
            ].slice(0, MAX_SEARCH_HISTORY);
            
            return newHistory;
        });
    }, [minSearchChars]);

    // When a user selects something and there are results, add to history
    useEffect(() => {
        if (results.length > 0 && query.trim().length >= minSearchChars) {
            saveSearchToHistory(query);
        }
    }, [results, query, minSearchChars, saveSearchToHistory]);

    const isProductInList = useCallback(
        (product: IProductListItem) => products.some((p) => p.sku === product.ugs),
        [products]
    );

    const handleAdd = useCallback(
        (event: React.MouseEvent, product: IProductListItem) => {
            event.stopPropagation();
            addProductToLiveFunction({
                id: product.id,
                title: product.name,
                productId: product.id,
                sku: product.ugs,
                image: product.mainImagePath,
                status: LiveProductStatus.NOT_PRESENTED,
                price: product.regularPrice || 0,
                timeStampProduct: [],
                isPresented: false,
            });
        },
        [addProductToLiveFunction]
    );

    const handleHistoryItemClick = useCallback((term: string) => {
        onSearch(term);
    }, [onSearch]);

    const handleClearHistory = useCallback(() => {
        setSearchHistory([]);
    }, []);

    const renderOption = useCallback(
        (props: React.HTMLAttributes<HTMLLIElement>, product: IProductListItem) => {
            console.log('Rendering option for product:', product);
            const isAdded = isProductInList(product);
            return (
                <Box
                    component="li"
                    {...props}
                    sx={{ display: 'flex', alignItems: 'center', justifyContent: 'space-between', width: '100%' }}
                >
                    <Box sx={{ display: 'flex', alignItems: 'center' }}>
                        <Avatar
                            variant="square"
                            sx={{ height: 40, width: 40, mr: 2 }}
                            alt={product.name}
                            src={product.mainImagePath}
                        />
                        <ListItemText
                            primary={product.name}
                            secondary={product.ugs}
                            primaryTypographyProps={{ variant: 'body2', noWrap: true }}
                            secondaryTypographyProps={{ variant: 'caption', color: 'text.secondary' }}
                        />
                    </Box>
                    {isAdded ? (
                        <Label
                            sx={{ height: 30, fontSize: 12, fontWeight: 600, padding: '0 16px' }}
                            color="success"
                            variant="soft"
                        >
                            Ajouté
                        </Label>
                    ) : (
                        <Button
                            variant="outlined"
                            size="small"
                            onClick={(event) => handleAdd(event, product)}
                        >
                            Ajouter
                        </Button>
                    )}
                </Box>
            );
        },
        [isProductInList, handleAdd]
    );
    // Placeholder message based on character count
    const placeholderText = useMemo(() => {
        return `Rechercher produits par UGS (min. ${minSearchChars} caractères)...`;
    }, [minSearchChars]);

    const noOptions = useMemo(
        () => <SearchNotFound query={query} sx={{ bgcolor: 'unset' }} />,
        [query]
    );

    return (
        <Box sx={{ width: '100%', ...sx }}>
            <Autocomplete
                inputValue={query}
                loading={loading}
                autoHighlight
                popupIcon={null}
                options={results}
                getOptionKey={(option) => option.id}
                onInputChange={(event, newValue) => onSearch(newValue)}
                getOptionLabel={(option) => option.name}
                noOptionsText={loading ? 'Chargement...' : noOptions}
                isOptionEqualToValue={(option, value) => option.id === value.id}
                filterOptions={(x) => x}
                blurOnSelect
                clearOnBlur={false}
                disableListWrap
                slotProps={{
                    popper: {
                        placement: 'bottom-start',  
                        sx: { minWidth: 320 },
                    },  
                    paper: {
                        sx: {
                            [` .${autocompleteClasses.option}`]: {
                                pl: 0.75,
                            },
                        },
                    },
                }}
                renderInput={(params) => (
                    <TextField
                        {...params}
                        placeholder={placeholderText}
                        InputProps={{
                            ...params.InputProps,
                            startAdornment: (
                                <InputAdornment position="start">
                                    <Iconify icon="eva:search-fill" sx={{ ml: 1, color: 'text.disabled' }} />
                                </InputAdornment>
                            ),
                            endAdornment: (
                                <InputAdornment position="end">
                                    {loading ? <CircularProgress size={20} /> : null}
                                    {params.InputProps.endAdornment}
                                </InputAdornment>
                            ),
                        }}
                    />
                )}
                renderOption={renderOption}
            />
            
            {/* Search History Section */}
            {searchHistory.length > 0 && (
                <Box sx={{ mt: 1, p: 1 }}>
                    <Box sx={{ display: 'flex', justifyContent: 'space-between', alignItems: 'center', mb: 1 }}>
                        <Box sx={{ fontSize: '0.75rem', color: 'text.secondary' }}>
                            Recherches récentes
                        </Box>
                        <IconButton 
                            size="small" 
                            onClick={handleClearHistory}
                            sx={{ p: 0.5 }}
                        >
                            <Iconify icon="eva:trash-2-outline" width={14} />
                        </IconButton>
                    </Box>
                    <Stack direction="row" spacing={1} flexWrap="wrap" useFlexGap>
                        {searchHistory.map((term, index) => (
                            <Chip
                                key={index}
                                label={term}
                                size="small"
                                onClick={() => handleHistoryItemClick(term)}
                                sx={{ mb: 0.5 }}
                            />
                        ))}
                    </Stack>
                </Box>
            )}
        </Box>
    );
};

export default React.memo(ProductsSearch);
