import { useResponsive } from "@/hooks/use-responsive";
import { Alert, Box, Button, Card, Grid, Modal, Typography } from "@mui/material";
import ProductCard from "./details-product-card";
import Scrollbar from "@/shared/components/scrollbar";
import useScroll from "@/hooks/use-scroll";
import { useState, useEffect, useCallback, useRef, useMemo } from "react";
import { searchProduct } from "@/shared/api/live";
import { ILiveProduct, IProductListItem } from "@/shared/types/live";
import LiveSearch from "./details-products-search";
import useLiveProducts from "@/contexts/live-products/use-live-products";
import { SplashScreen } from "@/shared/components/loading-screen";
import { useLiveData } from "@/contexts/live-details";

// Custom hook for debouncing
function useDebounce<T>(value: T, delay: number): T {
    const [debouncedValue, setDebouncedValue] = useState<T>(value);
  
    useEffect(() => {
        console.log('Debounce hook received new value, setting timeout');
        const timer = setTimeout(() => {
            console.log('Debounce timeout completed, updating value');
            setDebouncedValue(value);
        }, delay);
  
        return () => {
            console.log('Cleaning up previous debounce timeout');
            clearTimeout(timer);
        };
    }, [value, delay]);
  
    return debouncedValue;
}

export default function ProductsSection() {
    const lgUp = useResponsive('up', 'lg');

    const { products, productsLoading , productsError} = useLiveProducts();
    const liveData = useLiveData();
 
    const { endRef: productsEndRef } = useScroll(products);
    const [open, setOpen] = useState(false);
    const [loading, setLoading] = useState(false);
    const [query, setQuery] = useState("");
    const [searchResults, setSearchResults] = useState<IProductListItem[]>([]);
    const [searchError, setSearchError] = useState<string | null>(null);
    
    // Minimum characters before search is triggered
    const MIN_SEARCH_CHARS = 3;
    
    // Use the custom debounce hook
    const debouncedQuery = useDebounce(query, 400);

    function handleClose() {
        setOpen(false);
    }

    function handleOpen() {
        setOpen(true);
    }

    function handleSearch(inputValue: string) {
        console.log('Search input received:', inputValue);
        setQuery(inputValue);
        setSearchError(null);
    }

    // Improved search function
    const performSearch = useCallback((searchQuery: string) => {
        console.log('Performing search with query:', searchQuery);
        
        // Reset results if query is too short
        if (searchQuery.trim().length < MIN_SEARCH_CHARS) {
            console.log('Query too short, minimum required:', MIN_SEARCH_CHARS);
            setSearchResults([]);
            return;
        }

        // Normalize the query
        const normalizedQuery = searchQuery.trim().toLowerCase();
        console.log('Normalized query:', normalizedQuery);
        
        setLoading(true);
        setSearchError(null);

        console.log('Calling search API with query:', normalizedQuery);
        searchProduct(normalizedQuery)
            .then((data) => {
                console.log('Search API response:', data);
                console.log('Number of results:', data.length);
                setSearchResults(data);
            })
            .catch((error) => { 
                console.error('Search API error:', error);
                setSearchError('Erreur lors de la recherche de produits');
                setSearchResults([]);
            })
            .finally(() => {
                console.log('Search request completed');
                setLoading(false);
            });
    }, []);

    // Effect to trigger search when debounced query changes
    useEffect(() => {
        console.log('Debounced query changed:', debouncedQuery);
        performSearch(debouncedQuery);
    }, [debouncedQuery, performSearch]);

    // Get the appropriate message for search status
    const searchStatusMessage = useMemo(() => {
        if (searchError) {
            return <Alert severity="error">{searchError}</Alert>;
        }
        if (query.trim().length >= MIN_SEARCH_CHARS && !loading && searchResults.length === 0) {
            return <Typography variant="body2" sx={{ p: 2, textAlign: 'center', color: 'text.secondary' }}>
                Aucun produit trouvé pour "{query}"
            </Typography>;
        }
        if (query.trim().length > 0 && query.trim().length < MIN_SEARCH_CHARS) {
            return <Typography variant="body2" sx={{ p: 1, textAlign: 'center', color: 'text.secondary' }}>
                Saisissez au moins {MIN_SEARCH_CHARS} caractères pour rechercher
            </Typography>;
        }
        return null;
    }, [searchError, query, loading, searchResults.length]);

    if (productsLoading) {
        return <SplashScreen />;
    }

    else if (productsError) {
        return <Alert severity="error">Erreur lors de la récupération des produits</Alert>;
    }

    return (
        <>
            {lgUp ? (
                <Grid item lg={4} md={6} xs={12}>
                    <LiveSearch
                        query={query}
                        results={searchResults}
                        onSearch={handleSearch}
                        loading={loading}
                        sx={{ marginBottom: { xs: 0, lg: 2 } }}
                        minSearchChars={MIN_SEARCH_CHARS}
                    />
                    {searchStatusMessage}
                    <Card sx={{ height: 652 }}>
                        <Scrollbar ref={productsEndRef} sx={{ px: 2, py: 5, height: 1 }}>
                            {products.map((product: ILiveProduct) => (
                                <ProductCard key={product.id} liveId={liveData.id} {...product}  id={product.id}/>
                            ))}
                        </Scrollbar>
                    </Card>
                </Grid>
            ) : (
                <>
                    <Button variant="outlined" onClick={handleOpen} sx={{ position: 'absolute', color: 'white', zIndex: 6, top: 20, left: 20 }}>
                        Produits
                    </Button>
                    <Modal
                        open={open}
                        onClose={handleClose}
                        aria-labelledby="modal-modal-title"
                        aria-describedby="modal-modal-description"
                    >
                        <Card sx={{
                            position: 'absolute',
                            top: '50%',
                            left: '50%',
                            transform: 'translate(-50%, -50%)',
                            height: '80vh',
                            minWidth: 400,
                            overflow: 'hidden',
                            display: 'flex',
                            flexDirection: 'column',
                            boxShadow: 24,
                        }}>
                            <LiveSearch
                                query={query}
                                results={searchResults}
                                onSearch={handleSearch}
                                loading={loading}
                                minSearchChars={MIN_SEARCH_CHARS}
                            />
                            {searchStatusMessage}
                            <Scrollbar ref={productsEndRef} sx={{ px: 2, py: 5, height: 1, overflowY: 'auto' }}>
                                <Box>
                                    {products.map((product: ILiveProduct) => (
                                        <ProductCard key={product.id} liveId={liveData.id} {...product} />
                                    ))}
                                </Box>
                            </Scrollbar>
                        </Card>
                    </Modal>
                </>
            )}
        </>
    );
}
