import { useState, useEffect, useCallback, useMemo } from 'react';
import Stack from '@mui/material/Stack';
import Grid from '@mui/material/Grid';
import { useSettingsContext } from '@/shared/components/settings';
import Container from '@mui/material/Container';
import FormProvider, {
    RHFUpload,
} from "@/shared/components/hook-form";
import { Alert, IconButton, Snackbar, Typography } from '@mui/material';
import * as Yup from "yup";
import { useForm } from "react-hook-form";
import { yupResolver } from "@hookform/resolvers/yup";
import { fData } from "@/utils/format-number";
import { useGetMedias, useUploadMedias } from '@/shared/api/media';
import { UserByEmail } from '@/shared/api/user';
import { IUser } from '@/shared/types/user';
import { CloseIcon } from '@/shared/components/lightbox';

interface Props {
    onCreate?: VoidFunction;
    mediaName?: string;
    onChangeMediaName?: (event: React.ChangeEvent<HTMLInputElement>) => void;
    onUploadChangeTab: () => void;
}

export default function MultiMediaUploadView({
    onCreate,
    mediaName,
    onChangeMediaName,
    onUploadChangeTab,
    ...other
}: Props) {
    const [userId, setUserId] = useState<string>("");

    const fetchUserData = async () => {
        const clientEmail = localStorage.getItem("email");
        const response = await UserByEmail(clientEmail!);
        const userData: IUser = response.data;
        if (userData)
            setUserId(userData.id)
    }

    useEffect(() => {
        fetchUserData();
    }, []);

    const [medias, setMedias] = useState<(File | string)[]>([]);

    const settings = useSettingsContext();

    const { mutateMedias } = useGetMedias();

    const MediaSchema = Yup.object().shape({
        url: Yup.mixed<any>().nullable().required("Média requis"),
    });

    const defaultValues = useMemo(() => ({
        url: null,
    }),
        []
    );

    const methods = useForm({
        resolver: yupResolver(MediaSchema),
        defaultValues,
    });

    const {
        reset,
        handleSubmit,
    } = methods;

    useEffect(() => {
        if (!open) {
            setMedias([]);
        }
    }, [open]);

    const [showMessage, setShowMessage] = useState(false);

    const handleCloseMessage = () => {
        setShowMessage(false);
    };

    const handleDrop = useCallback(
        (acceptedFiles: File[]) => {
            const validFormats = ['image/png', 'image/jpeg', 'image/jpg'];
            let hasInvalidFile = false;

            const newFiles = acceptedFiles.filter((file) => {
                if (!validFormats.includes(file.type)) {
                    hasInvalidFile = true;
                    return false;
                }
                return true;
            }).map((file) =>
                Object.assign(file, {
                    preview: URL.createObjectURL(file),
                })
            );

            if (hasInvalidFile) {
                setShowMessage(true);
            }

            setMedias([...medias, ...newFiles]);
        },
        [medias]
    );

    const handleUpload = async () => {
        try {
            await useUploadMedias(userId, medias);
            if (onCreate) {
                onCreate();
            }
            mutateMedias();
            setMedias([]);
            setOpenSaveAlert(true);
        } catch (error) {
            setOpenErrorAlert(true);
            console.info('Une erreur s\'est produite lors de l\'upload');
            console.info('Upload error:', error);
        }
    };

    const handleRemoveMedia = (inputMedia: File | string) => {
        const filtered = medias.filter((media) => media !== inputMedia);
        setMedias(filtered);
    };

    const handleRemoveAllMedias = () => {
        setMedias([]);
    };

    const [openSaveAlert, setOpenSaveAlert] = useState(false);
    const handleCloseSaveAlert = (event?: React.SyntheticEvent | Event, reason?: string) => {
        if (reason === 'clickaway') {
            return;
        }
        setOpenSaveAlert(false);
    };

    const [openErrorAlert, setOpenErrorAlert] = useState(false);
    const handleCloseErrorAlert = (event?: React.SyntheticEvent | Event, reason?: string) => {
        if (reason === 'clickaway') {
            return;
        }
        setOpenErrorAlert(false);
    };

    return (
        <Container maxWidth={settings.themeStretch ? false : 'lg'} sx={{ height: '70vh', maxHeight: '80vh', overflowY: 'auto' }}>
            <FormProvider methods={methods}>
                <Stack maxWidth={'lg'} {...other} >
                    <Stack direction="row" alignItems="center" justifyContent="space-between">
                        <Typography variant="h4">Téléverser un média</Typography>
                    </Stack>
                    
                    {showMessage && (
                        <Alert
                            severity="info"
                            action={
                                <IconButton
                                    aria-label="close"
                                    color="inherit"
                                    size="small"
                                    onClick={handleCloseMessage}
                                >
                                    <CloseIcon fontSize="inherit" />
                                </IconButton>
                            }
                            sx={{ mb: 2 }}
                        >
                            Format d'image non valide. Seuls les images PNG, JPG et JPEG sont autorisés.
                        </Alert>
                    )}

                    <Stack justifyContent={'center'} >
                        <Grid xs={12} md={12}>
                            <Stack sx={{ pt: 1, pb: 0, border: 'none' }}>
                                <RHFUpload
                                    multiple
                                    thumbnail
                                    files={medias}
                                    name="images"
                                    maxSize={3145728}
                                    onDrop={handleDrop}
                                    onRemove={handleRemoveMedia}
                                    onRemoveAll={handleRemoveAllMedias}
                                    onUpload={handleUpload}
                                    helperText={
                                        <Typography
                                            variant="caption"
                                            sx={{
                                                mt: 3,
                                                mx: "auto",
                                                display: "block",
                                                textAlign: "center",
                                                color: "text.disabled",
                                            }}
                                        >
                                            Formats autorisés : *.jpeg, *.jpg, *.png
                                            <br /> taille maximale de {fData(3145728)}
                                        </Typography>
                                    }
                                />
                            </Stack>
                        </Grid>
                    </Stack>
                </Stack>
            </FormProvider>
            <Snackbar open={openSaveAlert} autoHideDuration={3000}
                onClose={handleCloseSaveAlert} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
                <Alert
                    onClose={handleCloseSaveAlert}
                    severity="success"
                    variant="filled"
                    sx={{ width: '100%' }}
                >
                    L'upload est fait avec succès
                </Alert>
            </Snackbar>

            <Snackbar open={openErrorAlert} autoHideDuration={3000}
                onClose={handleCloseErrorAlert} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
                <Alert
                    onClose={handleCloseErrorAlert}
                    severity="error"
                    variant="filled"
                    sx={{ width: '100%' }}
                >
                    Échec
                </Alert>
            </Snackbar>
        </Container>
    );
}
