import { useState, useEffect, useCallback, useMemo } from 'react';
import Stack from '@mui/material/Stack';
import Button from '@mui/material/Button';
import Box from '@mui/material/Box';
import Iconify from '@/components/iconify';
import { Alert, Container, Grid, IconButton, Snackbar, Typography } from '@mui/material';
import * as Yup from "yup";
import { useForm } from "react-hook-form";
import { yupResolver } from "@hookform/resolvers/yup";
import { fData } from "@/utils/format-number";
import FormProvider from '@/components/hook-form/form-provider';
import { useGetMedias, useUploadMedias } from '@/shared/api/media';
import { RHFUpload } from '@/shared/components/hook-form';
import { useSettingsContext } from '@/components/settings';
import { UserByEmail } from '@/shared/api/user';
import { IUser } from '@/shared/types/user';
import { CloseIcon } from '@/shared/components/lightbox';

interface Props {
    onCreate?: VoidFunction;
    mediaName?: string;
    onChangeMediaName?: (event: React.ChangeEvent<HTMLInputElement>) => void;
    onUploadChangeTab: () => void;
}

export default function SingleMediaUploadView({
    onCreate,
    mediaName,
    onChangeMediaName,
    onUploadChangeTab,
    ...other
}: Props) {
    const [userId, setUserId] = useState<string>("");

    const fetchUserData = async () => {
        const clientEmail = localStorage.getItem("email");
        const response = await UserByEmail(clientEmail!);
        const userData: IUser = response.data;
        if (userData)
            setUserId(userData.id)
    }

    useEffect(() => {
        fetchUserData();
    }, []);

    const [medias, setMedias] = useState<(File | string)[]>([]);
    const [previewImage, setPreviewImage] = useState<string | null>(null);
    const { mutateMedias } = useGetMedias();

    const MediaSchema = Yup.object().shape({
        url: Yup.mixed<any>().nullable().required("Média requis"),
    });

    const defaultValues = useMemo(() => ({
        url: null,
    }), []);

    const methods = useForm({
        resolver: yupResolver(MediaSchema),
        defaultValues,
    });

    const settings = useSettingsContext();

    const {
        formState: { isSubmitting },
    } = methods;

    useEffect(() => {
        setMedias([]);
        setPreviewImage(null);
    }, []);

    const [showMessage, setShowMessage] = useState(false);
    const handleCloseMessage = () => {
        setShowMessage(false);
    };

    const [alertMessage, setAlertMessage] = useState<string>("");

    const handleDrop = useCallback(
        (acceptedFiles: File[]) => {
            const validFormats = ['image/png', 'image/jpeg', 'image/jpg'];
            let hasInvalidFile = false;

            const newFiles = acceptedFiles.filter((file) => {
                if (!validFormats.includes(file.type)) {
                    hasInvalidFile = true;
                    return false;
                }
                return true;
            }).map((file) =>
                Object.assign(file, {
                    preview: URL.createObjectURL(file),
                })
            );

            if (hasInvalidFile) {
                setShowMessage(true);
            }

            setMedias((prevMedias) => {
                const updatedMedias = [...prevMedias, ...newFiles];
                return updatedMedias;
            });

            if (newFiles.length > 0) {
                setPreviewImage(newFiles[0].preview);
            } else {
                setPreviewImage(null);
            }
        },
        [medias]
    );

    const handleUpload = async () => {
        if (medias.length < 1) {
            setAlertMessage("Veuillez sélectionner une image");
            setOpenErrorAlert(true);
            return;
        }
        try {
            await useUploadMedias(userId, medias);
            if (onCreate) {
                onCreate();
            }
            mutateMedias();
            setMedias([]);
            setAlertMessage("L'upload est fait avec succès");
            setOpenSaveAlert(true);
        } catch (error) {
            setAlertMessage("Échec de l'upload");
            setOpenErrorAlert(true);
        }
    };

    const handleRemoveMedia = (inputMedia: File | string) => {
        const filtered = medias.filter((media) => media !== inputMedia);
        setMedias(filtered);
        setPreviewImage(null);
    };

    const handleRemoveAllMedias = () => {
        setMedias([]);
        setPreviewImage(null);
    };

    const [openSaveAlert, setOpenSaveAlert] = useState(false);
    const handleCloseSaveAlert = (event?: React.SyntheticEvent | Event, reason?: string) => {
        if (reason === 'clickaway') {
            return;
        }
        setOpenSaveAlert(false);
    };

    const [openErrorAlert, setOpenErrorAlert] = useState(false);
    const handleCloseErrorAlert = (event?: React.SyntheticEvent | Event, reason?: string) => {
        if (reason === 'clickaway') {
            return;
        }
        setOpenErrorAlert(false);
    };

    return (
        <>
            <Container maxWidth={settings.themeStretch ? false : 'lg'} sx={{ height: '70vh', maxHeight: '80vh', overflowY: 'auto' }}>
                <FormProvider methods={methods}>
                    <Stack maxWidth={'lg'} {...other} >
                        <Stack direction="row" alignItems="center" justifyContent="space-between">
                            <Typography variant="h4">Téléverser un média</Typography>
                        </Stack>

                        {showMessage && (
                            <Alert
                                severity="info"
                                action={
                                    <IconButton
                                        aria-label="close"
                                        color="inherit"
                                        size="small"
                                        onClick={handleCloseMessage}
                                    >
                                        <CloseIcon fontSize="inherit" />
                                    </IconButton>
                                }
                                sx={{ mb: 2 }}
                            >
                                Format d'image non valide. Seuls les images PNG, JPG et JPEG sont autorisés.
                            </Alert>
                        )}


                        <Stack justifyContent={'center'} >
                            <Grid xs={12} md={12}>
                                <Stack sx={{ pt: 1, pb: 0, border: 'none' }}>
                                    <RHFUpload
                                        thumbnail
                                        files={medias}
                                        name="images"
                                        maxSize={3145728}
                                        onDrop={handleDrop}
                                        onRemove={handleRemoveMedia}
                                        onRemoveAll={handleRemoveAllMedias}
                                        onUpload={handleUpload}
                                        helperText={
                                            <Typography
                                                variant="caption"
                                                sx={{
                                                    mt: 3,
                                                    mx: "auto",
                                                    display: "block",
                                                    textAlign: "center",
                                                    color: "text.disabled",
                                                }}
                                            >
                                                Formats autorisés : *.jpeg, *.jpg, *.png
                                                <br /> taille maximale de {fData(3145728)}
                                            </Typography>
                                        }
                                    />
                                </Stack>
                            </Grid>
                        </Stack>
                    </Stack>
                </FormProvider>
            </Container>
            {previewImage && (
                <Stack direction="row">
                    <Box
                        sx={{
                            width: '80px',
                            height: '80px',
                            borderRadius: '8px',
                            overflow: 'hidden',
                            position: 'relative',
                            backgroundColor: 'white',
                            boxShadow: '0px 4px 10px rgba(0, 0, 0, 0.1)',
                        }}
                    >
                        <Box
                            component="div"
                            sx={{
                                width: '100%',
                                height: '100%',
                                backgroundImage: `url(${previewImage})`,
                                backgroundSize: 'cover',
                                backgroundPosition: 'center',
                            }}
                        />
                    </Box>
                </Stack>
            )}

            {previewImage && (
                <Stack sx={{ mt: 1 }}>
                    <Stack direction='row' width={'200px'}>
                        <Button size="small" variant="outlined" color="inherit" onClick={handleRemoveAllMedias} sx={{ width: '50%' }}>
                            Remove
                        </Button>
                        <Button
                            type="submit"
                            size="small"
                            variant="contained"
                            startIcon={<Iconify icon="eva:cloud-upload-fill" />}
                            sx={{ width: '50%', ml: 1 }}
                            disabled={isSubmitting}
                            onClick={handleUpload}
                        >
                            Upload
                        </Button>
                    </Stack>
                </Stack>
            )}

            <Snackbar open={openSaveAlert} autoHideDuration={3000}
                onClose={handleCloseSaveAlert} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
                <Alert
                    onClose={handleCloseSaveAlert}
                    severity="success"
                    variant="filled"
                    sx={{ width: '100%' }}
                >
                    {alertMessage}
                </Alert>
            </Snackbar>

            <Snackbar open={openErrorAlert} autoHideDuration={3000}
                onClose={handleCloseErrorAlert} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
                <Alert
                    onClose={handleCloseErrorAlert}
                    severity="error"
                    variant="filled"
                    sx={{ width: '100%' }}
                >
                    {alertMessage}
                </Alert>
            </Snackbar>
        </>
    );
}
