import { useState, useEffect } from 'react';
import Button from '@mui/material/Button';
import Divider from '@mui/material/Divider';
import MenuItem from '@mui/material/MenuItem';
import Box from '@mui/material/Box';
import Grid from '@mui/material/Grid';
import Checkbox from '@mui/material/Checkbox';
import IconButton from '@mui/material/IconButton';
import Typography from '@mui/material/Typography';
import ListItemText from '@mui/material/ListItemText';
import { alpha, useTheme } from '@mui/material/styles';
import TableRow, { tableRowClasses } from '@mui/material/TableRow';
import TableCell, { tableCellClasses } from '@mui/material/TableCell';
import { useBoolean } from '@/hooks/use-boolean';
import { useDoubleClick } from '@/hooks/use-double-click';
import { fDate, fTime } from '@/utils/format-time';
import Iconify from '@/components/iconify';
import { ConfirmDialog } from '@/shared/components/custom-dialog';
import CustomPopover, { usePopover } from '@/shared/components/custom-popover';
import { IMedia } from '@/shared/types/media';
import { Download, Edit } from '@mui/icons-material';
import MediaManagerFileDetails from './media-manager-file-details';
import { useGetUserById } from '@/shared/api/user';
import { IUser } from '@/shared/types/user';

// ----------------------------------------------------------------------

type Props = {
  row: IMedia;
  selected: boolean;
  onSelectRow: VoidFunction;
  onEditRow: VoidFunction;
  onDeleteRow: VoidFunction;
  onDownloadRow: VoidFunction;
};

export default function MediaManagerTableRow({
  row,
  selected,
  onSelectRow,
  onEditRow,
  onDeleteRow,
  onDownloadRow
}: Props) {
  const theme = useTheme();

  const { title, description, name, userId, modifiedAt, url } = row;

  const details = useBoolean();

  const confirm = useBoolean();

  const popover = usePopover();

  const [userName, setUserName] = useState<string>("");

  const fetchedUserData = useGetUserById(Number(userId));

  useEffect(() => {
    const userData: IUser | null = fetchedUserData.userData;
    if (userData)
      setUserName(userData.firstName + " " + userData.lastName)
  }, [userId, row]);

  const [imageUrl, setImageUrl] = useState('');

  useEffect(() => {
    const extractUrl = (url: string) => {
      const match = url.match(/Optional\[(.*)\]/);
      return match ? match[1] : url;
    };

    setImageUrl(extractUrl(url));
  }, [row])

  const handleClick = useDoubleClick({
    click: () => {
      details.onTrue();
    },
    doubleClick: () => console.info('DOUBLE CLICK'),
  });

  const defaultStyles = {
    borderTop: `solid 1px ${alpha(theme.palette.grey[500], 0.16)}`,
    borderBottom: `solid 1px ${alpha(theme.palette.grey[500], 0.16)}`,
    '&:first-of-type': {
      borderTopLeftRadius: 16,
      borderBottomLeftRadius: 16,
      borderLeft: `solid 1px ${alpha(theme.palette.grey[500], 0.16)}`,
    },
    '&:last-of-type': {
      borderTopRightRadius: 16,
      borderBottomRightRadius: 16,
      borderRight: `solid 1px ${alpha(theme.palette.grey[500], 0.16)}`,
    },
  };

  return (
    <>
      <TableRow
        selected={selected}
        sx={{
          borderRadius: 2,
          [`&.${tableRowClasses.selected}, &:hover`]: {
            backgroundColor: 'background.paper',
            boxShadow: theme.customShadows.z20,
            transition: theme.transitions.create(['background-color', 'box-shadow'], {
              duration: theme.transitions.duration.shortest,
            }),
            '&:hover': {
              backgroundColor: 'background.paper',
              boxShadow: theme.customShadows.z20,
            },
          },
          [`& .${tableCellClasses.root}`]: {
            ...defaultStyles,
          },
          ...(details.value && {
            [`& .${tableCellClasses.root}`]: {
              ...defaultStyles,
            },
          }),
        }}
      >
        <TableCell padding="checkbox">
          <Checkbox
            checked={selected}
            onDoubleClick={() => console.info('ON DOUBLE CLICK')}
            onClick={onSelectRow}
          />
        </TableCell>

        <TableCell onClick={handleClick} sx={{ whiteSpace: 'nowrap' }}>
          <Grid container alignItems="center" spacing={1} direction={"row"}>
            <Grid item md={2}>
              <Box component="img" src={imageUrl}
                sx={{
                  objectFit: 'cover',
                  objectPosition: 'center',
                  borderRadius: '8px',
                }}
              />
            </Grid>
            <Grid item md={10} direction={"column"}>
              <Typography variant="subtitle1" ml={2} sx={{ whiteSpace: 'wrap', fontWeight: 'bold' }}>{title}</Typography>
              <Typography variant="body1" ml={2} sx={{ whiteSpace: 'wrap' }}>{description}</Typography>
            </Grid>
          </Grid>
        </TableCell>

        <TableCell onClick={handleClick} sx={{ whiteSpace: 'nowrap' }}>
          {name}
        </TableCell>

        <TableCell onClick={handleClick} sx={{ whiteSpace: 'nowrap' }}>
          {userName || "Nom de l'utilisateur non trouvé"}
        </TableCell>

        <TableCell onClick={handleClick} sx={{ whiteSpace: 'nowrap' }}>
          <ListItemText
            primary={fDate(modifiedAt)}
            secondary={fTime(modifiedAt)}
            primaryTypographyProps={{ typography: 'body2' }}
            secondaryTypographyProps={{
              mt: 0.5,
              component: 'span',
              typography: 'caption',
            }}
          />
        </TableCell>

        <TableCell
          align="right"
          sx={{
            px: 1,
            whiteSpace: 'nowrap',
          }}
        >
          <IconButton color={popover.open ? 'inherit' : 'default'} onClick={popover.onOpen}>
            <Iconify icon="eva:more-vertical-fill" />
          </IconButton>
        </TableCell>
      </TableRow>

      <CustomPopover
        open={popover.open}
        onClose={popover.onClose}
        arrow="right-top"
        sx={{ width: 160 }}
      >

        <MenuItem
          onClick={() => {
            onDownloadRow();
            popover.onClose();
          }}
        >
          <Download />
          Télécharger
        </MenuItem>

        <Divider sx={{ borderStyle: 'dashed' }} />
        <MenuItem
          onClick={() => {
            onEditRow();
            popover.onClose();
          }}
        >
          <Edit />
          Modifier
        </MenuItem>

        <MenuItem
          onClick={() => {
            popover.onClose();
            confirm.onTrue();
          }}
          sx={{ color: 'error.main' }}
        >
          <Iconify icon="solar:trash-bin-trash-bold" />
          Delete
        </MenuItem>
      </CustomPopover>

      <MediaManagerFileDetails
        item={row}
        open={details.value}
        onClose={details.onFalse}
        onEdit={onEditRow}
        onDelete={() => {
          popover.onClose();
          confirm.onTrue();
        }}
        onDownload={onDownloadRow}
      />

      <ConfirmDialog
        open={confirm.value}
        onClose={confirm.onFalse}
        title="Delete"
        content="Are you sure want to delete?"
        action={
          <Button variant="contained" color="error" onClick={onDeleteRow}>
            Delete
          </Button>
        }
      />
    </>
  );
}
