"use client";

import { useState, useEffect, useMemo } from 'react';
import * as Yup from 'yup';
import { useForm } from 'react-hook-form';
import { yupResolver } from '@hookform/resolvers/yup';
import { useResponsive } from '@/hooks/use-responsive';
import { useSnackbar } from '@/components/snackbar';
import FormProvider, {
  RHFTextField,
} from "@/shared/components/hook-form";
import { paths } from '@/routes/paths';
import LoadingButton from '@mui/lab/LoadingButton';
import { useRouter } from '@/hooks';
import Box from '@mui/material/Box';
import Stack from '@mui/material/Stack';
import Grid from '@mui/material/Grid';
import Button from '@mui/material/Button';
import Divider from '@mui/material/Divider';
import Typography from '@mui/material/Typography';
import { fData } from '@/utils/format-number';
import { fDateTime } from '@/utils/format-time';
import Scrollbar from '@/components/scrollbar';
import CustomBreadcrumbs from "@/shared/components/custom-breadcrumbs";
import EditImageModal from '../crop-image/EditImageModal';
import { useGetMedia, useGetMedias, useUpdateMedia } from '@/shared/api/media';
import { Alert, Snackbar } from '@mui/material';
import { IUser } from '@/shared/types/user';
import { UserByEmail } from '@/shared/api/user';
import { LoadingScreen } from '@/shared/components/loading-screen';

type Props = {
  id: string;
};

export default function EditMediaView({ id }: Props) {
  const { mediaData, mediaLoading, mediaError, mutateMedia } = useGetMedia(id);
  const router = useRouter();
  const { enqueueSnackbar } = useSnackbar();
  const { mutateMedias } = useGetMedias();

  const MediaSchema = Yup.object().shape({
    title: Yup.string().required('Titre est requis'),
    description: Yup.string().required('Description est requise'),
    url: Yup.mixed()
      .nullable()
      .test('is-valid-image', 'Image must be a File or a URL string', value =>
        typeof value === 'string' || (value instanceof File)
      ),
  });

  const defaultValues = useMemo(() => ({
    title: mediaData?.title || '',
    name: mediaData?.name || '',
    size: mediaData?.size || '',
    url: mediaData?.url || '',
    type: mediaData?.fileType || '',
    modifiedAt: mediaData?.modifiedAt || '',
    createdAt: mediaData?.createdAt || '',
    author: mediaData?.userId || '',
    description: mediaData?.description || '',
  }), [mediaData]);

  const methods = useForm({
    resolver: yupResolver(MediaSchema),
    defaultValues,
  });

  const {
    reset,
    watch,
    handleSubmit,
    formState: { isSubmitting },
  } = methods;

  const [imageUrl, setImageUrl] = useState('');
  const [editedImage, setEditedImage] = useState<File | undefined>(undefined);
  const [userId, setUserId] = useState<string>("");

  useEffect(() => {
    const extractUrl = (url: string) => {
      const match = url.match(/Optional\[(.*)\]/);
      return match ? match[1] : url;
    };
    if (mediaData) {
      setImageUrl(extractUrl(mediaData?.url));
    }
  }, [mediaData]);

  useEffect(() => {
    if (mediaData) {
      reset(defaultValues);
    }
  }, [mediaData, defaultValues, reset]);

  useEffect(() => {
    fetchUserData();
  }, [id])

  const fetchUserData = async () => {
    const clientEmail = localStorage.getItem("email");
    const response = await UserByEmail(clientEmail!);
    const userData: IUser = response.data;
    if (userData)
      setUserId(userData.id)
  }

  const [openSaveAlert, setOpenSaveAlert] = useState(false);
  const handleCloseSaveAlert = (event?: React.SyntheticEvent | Event, reason?: string) => {
    if (reason === 'clickaway') {
      return;
    }
    setOpenSaveAlert(false);
  };

  const [openErrorAlert, setOpenErrorAlert] = useState(false);
  const handleCloseErrorAlert = (event?: React.SyntheticEvent | Event, reason?: string) => {
    if (reason === 'clickaway') {
      return;
    }
    setOpenErrorAlert(false);
  };

  const onSubmit = handleSubmit(async (data) => {
    try {
      const updatedMedia = await useUpdateMedia(userId, id, data, editedImage);
      mutateMedias();
      mutateMedia(updatedMedia, false);
      router.push(paths.dashboard.admin.media.root);
      setOpenSaveAlert(true);
    } catch (error) {
      setOpenErrorAlert(true);
      console.error(error);
      enqueueSnackbar('Failed to edit this media.', { variant: 'error' });
    }
  });

  const [open, setOpen] = useState<boolean>(false);

  const handleOpen = () => {
    setOpen(true);
  };

  const handleClose = () => {
    setOpen(false);
  };

  if (mediaLoading) {
    return (
      <Grid container spacing={3} sx={{ justifyContent: 'center', alignItems: 'center', height: '100vh' }}>
        <LoadingScreen />
      </Grid>
    );
  }

  if (mediaError) {
    return (
      <Grid container spacing={3} sx={{ justifyContent: 'center', alignItems: 'center', height: '100vh' }}>
        <Alert severity="error">Problème lors de la récupération des données du média. Veuillez réessayer plus tard.</Alert>
      </Grid>
    );
  }

  const renderDetails = (
    <>
      <Scrollbar sx={{ height: 1 }}>
        <Grid container spacing={2} sx={{ mt: 2 }}>
          <Grid item xs={6} sx={{ ml: 3, mr: '5%' }}>
            <CustomBreadcrumbs
              heading="Modifier média"
              links={[
                { name: 'Tableau de bord', href: paths.dashboard.root },
                { name: 'Médiathèque', href: paths.dashboard.admin.media.root },
                { name: 'Modifier' },
              ]}
              sx={{ mb: { xs: 3, md: 5 } }}
            />
            <Stack alignItems="center" justifyContent="space-between" sx={{ p: 2.5 }}>
              <RHFTextField name="title" label="Titre du média" />
              <RHFTextField name="description" label="Description du média" sx={{ mt: 2 }} multiline rows={4} />
              <Button variant="outlined" onClick={handleOpen} sx={{ width: '100%', mt: 2 }}>Modifier image</Button>
              <EditImageModal open={open} onClose={handleClose} image={mediaData} setEditedImage={setEditedImage} />
            </Stack>
            <Grid sx={{ display: 'flex', justifyContent: 'left' }}>
              <LoadingButton
                type="submit"
                variant="contained"
                size="large"
                loading={isSubmitting}
                sx={{ ml: 2 }}
              >Enregistrer</LoadingButton>
            </Grid>
          </Grid>
          <Grid item xs={5} sx={{ mt: "120px" }}>
            <Stack
              spacing={2.5}
              justifyContent="center"
              sx={{ p: 2.5, bgcolor: 'background.neutral' }}
              direction={"row"}
            >
              <Grid item xs={12}>
                <Box
                  component="img"
                  src={editedImage ? URL.createObjectURL(editedImage) : imageUrl}
                  sx={{
                    objectFit: 'contain',
                    objectPosition: 'center',
                    borderRadius: '8px',
                  }}
                />
              </Grid>
              <Grid xs={12} md={9}>
                <Stack sx={{ typography: 'caption', textTransform: 'capitalize', mt: 2 }}>
                  {defaultValues.title}
                </Stack>
                <Stack sx={{ typography: 'caption', textTransform: 'capitalize' }}>
                  {defaultValues.description}
                </Stack>
                <Divider sx={{ borderStyle: 'dashed' }} />
                <Typography variant="subtitle1" sx={{ wordBreak: 'break-all', mb: 2 }}>
                  {defaultValues.name}
                </Typography>
                <Stack spacing={1.5}>
                  <Stack direction="row" sx={{ typography: 'caption', textTransform: 'capitalize' }}>
                    <Box component="span" sx={{ width: 90, color: 'text.secondary', mr: 2 }}>
                      Modifié le :
                    </Box>
                    {fDateTime(defaultValues.modifiedAt)}
                  </Stack>
                  <Stack direction="row" sx={{ typography: 'caption', textTransform: 'capitalize' }}>
                    <Box component="span" sx={{ width: 90, color: 'text.secondary', mr: 2 }}>
                      Téléversé par :
                    </Box>
                    Michel
                  </Stack>
                  <Stack direction="row" sx={{ typography: 'caption', textTransform: 'capitalize' }}>
                    <Box component="span" sx={{ width: 90, color: 'text.secondary', mr: 2 }}>
                      Téléversé le:
                    </Box>
                    {fDateTime(defaultValues.createdAt)}
                  </Stack>
                  <Stack direction="row" sx={{ typography: 'caption', textTransform: 'capitalize' }}>
                    <Box component="span" sx={{ width: 90, color: 'text.secondary', mr: 2 }}>
                      Taille :
                    </Box>
                    {fData(defaultValues.size)}
                  </Stack>
                </Stack>
              </Grid>
            </Stack>
          </Grid>
        </Grid>
      </Scrollbar>
      <Snackbar open={openSaveAlert} autoHideDuration={2000}
        onClose={handleCloseSaveAlert} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
        <Alert
          onClose={handleCloseSaveAlert}
          severity="success"
          variant="filled"
          sx={{ width: '100%' }}
        >
          Media modifié avec succès
        </Alert>
      </Snackbar>
      <Snackbar open={openErrorAlert} autoHideDuration={3000}
        onClose={handleCloseErrorAlert} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
        <Alert
          onClose={handleCloseErrorAlert}
          severity="error"
          variant="filled"
          sx={{ width: '100%' }}
        >
          Échec
        </Alert>
      </Snackbar>
    </>
  );

  return (
    <FormProvider methods={methods} onSubmit={onSubmit}>
      <Grid container spacing={3}>
        {renderDetails}
      </Grid>
    </FormProvider>
  );
}
