'use client';

import { useState, useCallback, useEffect } from 'react';
import Stack from '@mui/material/Stack';
import Button from '@mui/material/Button';
import Container from '@mui/material/Container';
import Typography from '@mui/material/Typography';
import ToggleButton from '@mui/material/ToggleButton';
import ToggleButtonGroup from '@mui/material/ToggleButtonGroup';
import { useBoolean } from '@/hooks/use-boolean';
import { isAfter, isBetween } from '@/utils/format-time';
import Iconify from '@/components/iconify';
import { useSnackbar } from '@/shared/components/snackbar';
import EmptyContent from '@/shared/components/empty-content';
import { ConfirmDialog } from '@/shared/components/custom-dialog';
import { useSettingsContext } from '@/shared/components/settings';
import { useTable, getComparator } from '@/shared/components/table';
import { useRouter } from '@/hooks';
import { paths } from '@/routes/paths';
import { IMedia, IMediaFilters, IMediaFilterValue } from '@/shared/types/media';
import FileManagerFilters from '../media-manager-filters';
import FileManagerFiltersResult from '../media-manager-filters-result';
import NewMediaDialog from './media-new-dialog';
import { useGetMedias, useDeleteMedia, downloadMediaFile } from '@/shared/api/media';
import { Alert, Grid, Snackbar } from '@mui/material';
import MediaManagerGridView from '../list-medias/media-manager-grid-view';
import MediaManagerTable from '../list-medias/media-manager-table';
import { LoadingScreen } from '@/shared/components/loading-screen';

const defaultFilters: IMediaFilters = {
  name: '',
  startDate: null,
  endDate: null,
};

// ----------------------------------------------------------------------

export default function MediaManagerView() {
  const { enqueueSnackbar } = useSnackbar();

  const router = useRouter();

  const table = useTable({ defaultRowsPerPage: 10 });

  const settings = useSettingsContext();

  const openDateRange = useBoolean();

  const confirm = useBoolean();

  const upload = useBoolean();

  const [view, setView] = useState('list');

  const { mediasData, mediasLoading, mediasError, mutateMedias } = useGetMedias();

  const [tableData, setTableData] = useState<IMedia[]>(mediasData);

  const [filters, setFilters] = useState(defaultFilters);

  const dateError = isAfter(filters.startDate, filters.endDate);

  useEffect(() => {
    setTableData(mediasData);
  }, [mediasData]);

  const dataFiltered = applyFilter({
    inputData: tableData,
    comparator: getComparator(table.order, table.orderBy),
    filters,
    dateError,
  });

  const dataInPage = dataFiltered.slice(
    table.page * table.rowsPerPage,
    table.page * table.rowsPerPage + table.rowsPerPage
  );

  const canReset =
    !!filters.name || (!!filters.startDate && !!filters.endDate);

  const notFound = (!dataFiltered.length && canReset) || !dataFiltered.length;

  const handleChangeView = useCallback(
    (event: React.MouseEvent<HTMLElement>, newView: string | null) => {
      if (newView !== null) {
        setView(newView);
      }
    },
    []
  );

  const handleFilters = useCallback(
    (name: string, value: IMediaFilterValue) => {
      table.onResetPage();
      setFilters((prevState) => ({
        ...prevState,
        [name]: value,
      }));
    },
    [table]
  );

  const handleResetFilters = useCallback(() => {
    setFilters(defaultFilters);
  }, []);

  const handleEditItem = useCallback(
    (id: string) => {
      router.push(paths.dashboard.admin.media.edit(id));
    },
    [router]
  );

  const [openDeleteAlert, setOpenDeleteAlert] = useState(false);
  const handleCloseDeleteAlert = (event?: React.SyntheticEvent | Event, reason?: string) => {
    if (reason === 'clickaway') {
      return;
    }
    setOpenDeleteAlert(false);
  };

  const [openErrorAlert, setOpenErrorAlert] = useState(false);
  const handleCloseErrorAlert = (event?: React.SyntheticEvent | Event, reason?: string) => {
      if (reason === 'clickaway') {
          return;
      }
      setOpenErrorAlert(false);
  };

  const handleDeleteItem = useCallback(async (id: string) => {
    try {
      const status = await useDeleteMedia([id]);
      if (status === 200) {
        const deleteRow = tableData.filter((row) => row.id !== id);
        enqueueSnackbar('Delete success!');
        setTableData(deleteRow);
        table.onUpdatePageDeleteRow(dataInPage.length);
        mutateMedias();
      } else {
        setOpenErrorAlert(true);
        enqueueSnackbar('Failed to delete!', { variant: 'error' });
      }
    } catch (error) {
      setOpenErrorAlert(true);
      enqueueSnackbar('Failed to delete!', { variant: 'error' });
      console.error('Error deleting media:', error);
    }
  },
    [dataInPage.length, enqueueSnackbar, table, tableData]
  );

  const handleDeleteItems = useCallback(async () => {
    try {
      const mediasToDelete: string[] = table.selected.map((id) => {
        const media = tableData.find((row) => row.id === id);
        return media ? media.id : null;
      }).filter((id): id is string => id !== null);

      if (mediasToDelete.length === 0) {
        enqueueSnackbar('Failed to delete: No valid id found!', { variant: 'error' });
        return;
      }

      const status = await useDeleteMedia(mediasToDelete);

      if (status === 200) {
        const deleteRows = tableData.filter((row) => !table.selected.includes(row.id));
        enqueueSnackbar('Delete success!');
        setTableData(deleteRows);
        table.onUpdatePageDeleteRows({
          totalRowsInPage: dataInPage.length,
          totalRowsFiltered: dataFiltered.length,
        });
        mutateMedias();
      } else {
        enqueueSnackbar('Failed to delete!', { variant: 'error' });
      }
    } catch (error) {
      enqueueSnackbar('Failed to delete!', { variant: 'error' });
      console.error('Error deleting media:', error);
    }
  }, [dataFiltered.length, dataInPage.length, enqueueSnackbar, table, tableData]);

  const handleDownloadItem = useCallback(async (id: string) => {
    try {
      await downloadMediaFile(id);
      enqueueSnackbar('Download success!');
    } catch (error) {
      enqueueSnackbar('Failed to delete!', { variant: 'error' });
    }
  },
    [dataInPage.length, enqueueSnackbar, table, tableData]
  );

  if (mediasLoading) {
    return (
      <Grid container spacing={3} sx={{ justifyContent: 'center', alignItems: 'center', height: '100vh' }}>
        <LoadingScreen />
      </Grid>
    );
  }

  if (mediasError) {
    return (
      <Grid container spacing={3} sx={{ justifyContent: 'center', alignItems: 'center', height: '100vh' }}>
        <Alert severity="error">Problème lors de la récupération des données des médias. Veuillez réessayer plus tard.</Alert>
      </Grid>
    );
  }

  const renderFilters = (
    <Stack
      spacing={2}
      direction={{ xs: 'column', md: 'row' }}
      alignItems={{ xs: 'flex-end', md: 'center' }}
    >
      <FileManagerFilters
        openDateRange={openDateRange.value}
        onCloseDateRange={openDateRange.onFalse}
        onOpenDateRange={openDateRange.onTrue}
        //
        filters={filters}
        onFilters={handleFilters}
        //
        dateError={dateError}
      />

      <ToggleButtonGroup size="small" value={view} exclusive onChange={handleChangeView}>
        <ToggleButton value="list">
          <Iconify icon="solar:list-bold" />
        </ToggleButton>

        <ToggleButton value="grid">
          <Iconify icon="mingcute:dot-grid-fill" />
        </ToggleButton>
      </ToggleButtonGroup>
    </Stack>
  );

  const renderResults = (
    <FileManagerFiltersResult
      filters={filters}
      onResetFilters={handleResetFilters}
      //
      canReset={canReset}
      onFilters={handleFilters}
      //
      results={dataFiltered.length}
    />
  );

  return (
    <>
      <Container maxWidth={settings.themeStretch ? false : 'lg'}>
        <Stack direction="row" alignItems="center" justifyContent="space-between">
          <Typography variant="h4">Médiathèque</Typography>
          <Button
            variant="contained"
            color="primary"
            startIcon={<Iconify icon="eva:cloud-upload-fill" />}
            onClick={upload.onTrue}
          >
            Ajouter
          </Button>
        </Stack>

        <Stack
          spacing={2.5}
          sx={{
            my: { xs: 3, md: 5 },
          }}
        >
          {renderFilters}

          {canReset && renderResults}
        </Stack>

        {notFound ? (
          <EmptyContent
            filled
            title="Aucune donnée"
            sx={{
              py: 10,
            }}
          />
        ) : (
          <>
            {view === 'list' ? (
              <MediaManagerTable
                table={table}
                dataFiltered={dataFiltered}
                onEditRow={handleEditItem}
                onDeleteRow={handleDeleteItem}
                onDownloadRow={handleDownloadItem}
                notFound={notFound}
                onOpenConfirm={confirm.onTrue}
              />
            ) : (
              <MediaManagerGridView
                table={table}
                dataFiltered={dataFiltered}
                onEditItem={handleEditItem}
                onDownloadItem={handleDownloadItem}
                onDeleteItem={handleDeleteItem}
                onOpenConfirm={confirm.onTrue}
              />
            )}
          </>
        )}
      </Container>

      <NewMediaDialog open={upload.value} onClose={upload.onFalse} />

      <ConfirmDialog
        open={confirm.value}
        onClose={confirm.onFalse}
        title="Delete"
        content={
          <>
            Etes-vous sûr de vouloir supprimer <strong> {table.selected.length} </strong> éléments?
          </>
        }
        action={
          <Button
            variant="contained"
            color="error"
            onClick={() => {
              handleDeleteItems();
              confirm.onFalse();
              setOpenDeleteAlert(true);
            }}
          >
            Delete
          </Button>
        }
      />
      <Snackbar open={openDeleteAlert} autoHideDuration={3500}
        onClose={handleCloseDeleteAlert} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
        <Alert
          onClose={handleCloseDeleteAlert}
          severity="success"
          variant="filled"
          sx={{ width: '100%' }}
        >
          Médias supprimer avec succès
        </Alert>
      </Snackbar>
      <Snackbar open={openErrorAlert} autoHideDuration={3000}
        onClose={handleCloseErrorAlert} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
        <Alert
          onClose={handleCloseErrorAlert}
          severity="error"
          variant="filled"
          sx={{ width: '100%' }}
        >
          Échec
        </Alert>
      </Snackbar>
    </>
  );
}

// ----------------------------------------------------------------------

function applyFilter({
  inputData,
  comparator,
  filters,
  dateError,
}: {
  inputData: IMedia[];
  comparator: (a: any, b: any) => number;
  filters: IMediaFilters;
  dateError: boolean;
}) {
  const { name, startDate, endDate } = filters;

  const stabilizedThis = inputData.map((el, index) => [el, index] as const);

  stabilizedThis.sort((a, b) => {
    const order = comparator(a[0], b[0]);
    if (order !== 0) return order;
    return a[1] - b[1];
  });

  inputData = stabilizedThis.map((el) => el[0]);

  if (name) {
    inputData = inputData.filter(
      (file) => file.name.toLowerCase().indexOf(name.toLowerCase()) !== -1
    );
  }

  if (!dateError) {
    if (startDate && endDate) {
      inputData = inputData.filter((file) => isBetween(file.createdAt, startDate, endDate));
    }
  }

  return inputData;
}
