import { useState, useEffect, useCallback } from 'react';
import Stack from '@mui/material/Stack';
import Button from '@mui/material/Button';
import TextField from '@mui/material/TextField';
import DialogTitle from '@mui/material/DialogTitle';
import DialogActions from '@mui/material/DialogActions';
import DialogContent from '@mui/material/DialogContent';
import Dialog, { DialogProps } from '@mui/material/Dialog';
import Iconify from '@/components/iconify';
import { Upload } from '@/shared/components/upload';
import { useGetMedias, useUploadMedias } from '@/shared/api/media';
import { Alert, IconButton, Snackbar } from '@mui/material';
import {  UserByEmail } from '@/shared/api/user';
import { IUser } from '@/shared/types/user';
import { CloseIcon } from '@/shared/components/lightbox';

interface Props extends DialogProps {
  title?: string;
  //
  onCreate?: VoidFunction;
  onUpdate?: VoidFunction;
  //
  mediaName?: string;
  onChangeMediaName?: (event: React.ChangeEvent<HTMLInputElement>) => void;
  //
  open: boolean;
  onClose: VoidFunction;
}

export default function NewMediaDialog({
  title = 'Upload Média',
  open,
  onClose,
  //
  onCreate,
  onUpdate,
  //
  mediaName,
  onChangeMediaName,
  ...other
}: Props) {
  const [medias, setMedias] = useState<(File | string)[]>([]);
  const [userId, setUserId] = useState<string>("");

  const fetchUserData = async () => {
    const clientEmail = localStorage.getItem("email");
    const response = await UserByEmail(clientEmail!);
    const userData: IUser = response.data;
    if (userData)
      setUserId(userData.id)
  }

  useEffect(() => {
    fetchUserData();
  }, []);

  const { mutateMedias } = useGetMedias();

  useEffect(() => {
    if (!open) {
      setMedias([]);
    }
  }, [open]);

  const [showMessage, setShowMessage] = useState(false);

  const handleCloseMessage = () => {
    setShowMessage(false);
  };

  const handleDrop = useCallback(
    (acceptedFiles: File[]) => {
      const validFormats = ['image/png', 'image/jpeg', 'image/jpg'];
      let hasInvalidFile = false;

      const newFiles = acceptedFiles.filter((file) => {
        if (!validFormats.includes(file.type)) {
          hasInvalidFile = true;
          return false;
        }
        return true;
      }).map((file) =>
        Object.assign(file, {
          preview: URL.createObjectURL(file),
        })
      );

      if (hasInvalidFile) {
        setShowMessage(true);
      }

      setMedias([...medias, ...newFiles]);
    },
    [medias]
  );

  const [openSaveAlert, setOpenSaveAlert] = useState(false);
  const handleCloseSaveAlert = (event?: React.SyntheticEvent | Event, reason?: string) => {
    if (reason === 'clickaway') {
      return;
    }
    setOpenSaveAlert(false);
  };

  const [openErrorAlert, setOpenErrorAlert] = useState(false);
  const handleCloseErrorAlert = (event?: React.SyntheticEvent | Event, reason?: string) => {
    if (reason === 'clickaway') {
      return;
    }
    setOpenErrorAlert(false);
  };

  const handleUpload = async () => {
    onClose();
    try {
      await useUploadMedias(userId, medias);
      if (onCreate) {
        onCreate();
      }
      mutateMedias();
      console.info('Upload réussi');
    } catch (error) {
      console.info('Une erreur s\'est produite lors de l\'upload');
      console.info('Upload error:', error);
    }
  };

  const handleRemoveMedia = (inputMedia: File | string) => {
    const filtered = medias.filter((media) => media !== inputMedia);
    setMedias(filtered);
  };

  const handleRemoveAllMedias = () => {
    setMedias([]);
  };

  return (
    <>
      <Dialog fullWidth maxWidth="sm" open={open} onClose={onClose} {...other}>
        <DialogTitle sx={{ p: (theme) => theme.spacing(3, 3, 2, 3) }}> {title} </DialogTitle>

        {showMessage && (
          <Alert
            severity="info"
            action={
              <IconButton
                aria-label="close"
                color="inherit"
                size="small"
                onClick={handleCloseMessage}
              >
                <CloseIcon fontSize="inherit" />
              </IconButton>
            }
            sx={{ mb: 2 }}
          >
            Format d'image non valide. Seuls les images PNG, JPG et JPEG sont autorisés.
          </Alert>
        )}

        <DialogContent dividers sx={{ pt: 1, pb: 0, border: 'none' }}>
          {(onCreate || onUpdate) && (
            <TextField
              fullWidth
              label="Nom du média"
              value={mediaName}
              onChange={onChangeMediaName}
              sx={{ mb: 3 }}
            />
          )}

          <Upload multiple files={medias} onDrop={handleDrop} onRemove={handleRemoveMedia} />
        </DialogContent>

        <DialogActions>
          <Button
            variant="contained"
            startIcon={<Iconify icon="eva:cloud-upload-fill" />}
            onClick={handleUpload}
          >
            Enregistrer
          </Button>

          {!!medias.length && (
            <Button variant="outlined" color="inherit" onClick={handleRemoveAllMedias}>
              Retirer tout
            </Button>
          )}

          {(onCreate || onUpdate) && (
            <Stack direction="row" justifyContent="flex-end" flexGrow={1}>
              <Button variant="soft" onClick={onCreate || onUpdate}>
                {onUpdate ? 'Save' : 'Create'}
              </Button>
            </Stack>
          )}
        </DialogActions>
      </Dialog>
      <Snackbar open={openSaveAlert} autoHideDuration={2000}
        onClose={handleCloseSaveAlert} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
        <Alert
          onClose={handleCloseSaveAlert}
          severity="success"
          variant="filled"
          sx={{ width: '100%' }}
        >
          Medias uploader avec succès
        </Alert>
      </Snackbar>
      <Snackbar open={openErrorAlert} autoHideDuration={3000}
        onClose={handleCloseErrorAlert} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
        <Alert
          onClose={handleCloseErrorAlert}
          severity="error"
          variant="filled"
          sx={{ width: '100%' }}
        >
          Échec
        </Alert>
      </Snackbar>
    </>
  );
}
