"use client";
import React, { useEffect, useRef, useState } from "react";
import {
  Grid,
  Card,
  CardHeader,
  Stack,
  Typography,
  TextField,
  Autocomplete,
  IconButton,
  TableContainer,
  Table,
  TableHead,
  TableRow,
  TableCell,
  TableBody,
  Paper,
  Button,
  Tooltip,
  Alert,
  Box,
  CircularProgress,
  Pagination,
  Snackbar,
} from "@mui/material";
import {
  OrderStatus,
  OrderToUpdateResponse,
  UpdateNoteRequest,
} from "../../../types/order";
import LoadingButton from "@mui/lab/LoadingButton";
import CustomBreadcrumbs from "@/components/custom-breadcrumbs/custom-breadcrumbs";
import { paths } from "@/routes/paths";
import { LocalizationProvider } from "@mui/x-date-pickers";
import { useForm, SubmitHandler, Controller } from "react-hook-form";
import { yupResolver } from "@hookform/resolvers/yup";
import * as Yup from "yup";
import Iconify from "@/components/iconify";
import {
  getStatusInFrench,
  IOrderStatus,
  OrderStatusHistoryDTO,
} from "@/shared/types/order";
import DeleteOutlineIcon from "@mui/icons-material/DeleteOutline";
import {
  deleteNote,
  getOrderHistory,
  getOrderToUpdate,
  updateOrder,
} from "@/shared/api/order";
import {
  NoteType,
  noteTypeLabels,
  order_options,
} from "@/contexts/types/main-order";
import AddressForm from "../UpdateOrderAddressForm";
import { AddressType, IAddressItem } from "@/shared/types/user";

interface FormValues {}
type OrderEditViewProps = {
  id: string;
};

export default function OrderEditView({ id }: OrderEditViewProps) {
  const motif = ["En rupture du stock", "Retour client"];

  const [history, setHistory] = useState<OrderStatusHistoryDTO[]>([]);
  const [error, setError] = useState<string | null>(null);
  const [currentPage, setCurrentPage] = useState(1);
  const [pageSize] = useState(2);
  const [numberOfElements, setNumberOfElements] = useState(0);
  const [loading, setLoading] = useState<boolean>(true);

  const [notes, setNotes] = useState<UpdateNoteRequest[]>([]);

  // adress traitement
  const [billingAddress, setBillingAddress] = useState<IAddressItem>({
    firstName: "",
    lastName: "",
    companyName: "",
    email: "",
    phoneNumber: 0,
    street: "",
    home: "",
    city: "",
    zipCode: "",
    country: "",
    addressType: AddressType.BILLING,
  });

  const handleBillingChange = (updatedAddress: IAddressItem) => {
    setBillingAddress(updatedAddress);
  };

  const [shippingAddress, setShippingAddress] = useState<IAddressItem>({
    firstName: "",
    lastName: "",
    companyName: "",
    email: "",
    phoneNumber: 0,
    street: "",
    home: "",
    city: "",
    zipCode: "",
    country: "",
    addressType: AddressType.SHIPPING,
  });

  const handleShippingChange = (updatedAddress: IAddressItem) => {
    setShippingAddress(updatedAddress);
  };
  const [order, setOrder] = useState<OrderToUpdateResponse | null>(null);
  const [statutSelected, setStatutSelected] = useState<OrderStatus>(
    order?.orderStatus!
  );

  const fetchOrder = async () => {
    try {
      const fetchedOrder = await getOrderToUpdate(Number(id));
      setOrder(fetchedOrder);
      setNotes(fetchedOrder.note);
      setShippingAddress(fetchedOrder.shippingAddress);
      setBillingAddress(fetchedOrder.billingAddress);
    } catch (error: any) {
      setError(
        error.message ||
          "Erreur lors du chargement des informations de la commande"
      );
    }
  };

  const fetchOrderHistory = async () => {
    try {
      setLoading(true);
      const data = await getOrderHistory(Number(id), pageSize, currentPage - 1);
      setNumberOfElements(data.totalElements);
      setHistory(data.content);
    } catch (err: any) {
      setError("Failed to fetch order history.");
    } finally {
      setLoading(false);
    }
  };
  useEffect(() => {
    fetchOrderHistory();
  }, [id, currentPage]);

  useEffect(() => {
    fetchOrder();
  }, [id]);

  const addNote = () => {
    const newNote: UpdateNoteRequest = {
      id: 0,
      note: "",
      noteType: NoteType.PRIVATE,
    };
    setNotes((prev) => [...prev, newNote]);
  };

  const updateNote = (
    id: number,
    field: keyof UpdateNoteRequest,
    value: string
  ) => {
    setNotes((prev) =>
      prev.map((note) => (note.id === id ? { ...note, [field]: value } : note))
    );
  };

  const removeNote = async (id: number, index: number) => {
    setNotes((prev) => {
      const newNotes = [...prev];
      newNotes.splice(index, 1);
      return newNotes;
    });

    try {
      await deleteNote(id);
    } catch (error) {
      console.error("Error removing note:", error);
    }
  };
  //

  // Formater la date pour l'affichage
  const formatDate = (date: Date) => {
    return (
      new Date(date).toLocaleDateString("fr-FR", {
        year: "2-digit",
        month: "numeric",
        day: "numeric",
      }) +
      " " +
      new Date(date).toLocaleTimeString("fr-FR")
    );
  };

  const [snackbarOpen, setSnackbarOpen] = useState(false);
  const [snackbarMessage, setSnackbarMessage] = useState("");
  const [snackbarSeverity, setSnackbarSeverity] = useState<
    "success" | "warning" | "error"
  >("success");

  const handleSnackbarClose = () => {
    setSnackbarOpen(false);
  };

  const validationSchema = Yup.object().shape({
    etat: Yup.string().required("État est requis"),
    client: Yup.string().required("Client est requis"),
  });
  const {
    control,
    formState: { errors },
  } = useForm({ resolver: yupResolver(validationSchema) });

  const [updateLoading, setUpdateLoading] = useState(false);
  const handleModifierCommande = async () => {
    setUpdateLoading(true);

    try {
      const data: OrderToUpdateResponse = {
        id: Number(id),
        createdAt: order?.createdAt || new Date().toISOString(),
        orderStatus: statutSelected || order?.orderStatus! || null,
        clientFullName: order?.clientFullName || "Nom inconnu",
        note: notes || "",
        shippingAddress: shippingAddress || "Adresse non renseignée",
        billingAddress: billingAddress || "Adresse non renseignée",
        updateOrderLineRequests: order?.updateOrderLineRequests || [],
        total: order?.total || 0,
        subTotal: order?.subTotal || 0,
        userWhoUpdate: localStorage.getItem("userId")
          ? parseInt(localStorage.getItem("userId")!, 10)
          : 0,
      };

      const response = await updateOrder(data);

      if (response.status === 200) {
        setSnackbarMessage("Commande mise à jour avec succès.");
        setSnackbarSeverity("success");
        fetchOrder();
        fetchOrderHistory();
      } else {
        setSnackbarMessage("Une erreur est survenue lors de la mise à jour.");
        setSnackbarSeverity("warning");
      }
    } catch (error) {
      setSnackbarMessage("Erreur lors de la modification de la commande.");
      setSnackbarSeverity("error");
    } finally {
      setUpdateLoading(false);
      setSnackbarOpen(true);
    }
  };

  return (
    <>
      <LocalizationProvider>
        <CustomBreadcrumbs
          heading="Modifier une commande"
          links={[
            {
              name: "Tableau de bord",
              href: paths.dashboard.root,
            },
            {
              name: "Commandes",
              href: paths.dashboard.admin.order.all_orders,
            },
            {
              name: "Modifier une commande",
            },
          ]}
          sx={{
            mb: { xs: 3, md: 5 },
          }}
        />
        <Grid style={{
            pointerEvents:
              order?.orderStatus === OrderStatus.DELETED ? "none" : "auto",
            opacity: order?.orderStatus === OrderStatus.DELETED ? 0.7 : 1,
          }}>
        <Grid
          container
          spacing={2}
          sx={{ padding: 2 }}
        >
          <Grid item xs={12} md={8}>
            <Card>
              <CardHeader title={`Details - N° ${id}`} />
              <Stack spacing={3} sx={{ p: 3 }}>
                <TextField
                  name="creationDateTime"
                  label="Date de création"
                  type="datetime-local"
                  fullWidth
                  value={
                    order?.createdAt
                      ? new Date(order.createdAt).toISOString().slice(0, 16)
                      : ""
                  }
                  InputLabelProps={{
                    shrink: true,
                  }}
                  disabled
                />
                <Controller
                  name="etat"
                  control={control}
                  defaultValue={order?.orderStatus}
                  render={({ field }) => (
                    <Autocomplete
                      {...field}
                      id="combo-box-demo"
                      options={Object.keys(order_options) as OrderStatus[]}
                      getOptionLabel={(option) =>
                        order_options[option as keyof typeof order_options] ||
                        ""
                      }
                      disableClearable
                      fullWidth
                      value={field.value || order?.orderStatus || ""}
                      onChange={(_, value) => {
                        setStatutSelected(value as OrderStatus);
                      }}
                      renderInput={(params) => (
                        <TextField
                          {...params}
                          label="Etat"
                          error={!!errors.etat}
                          helperText={errors.etat?.message}
                        />
                      )}
                    />
                  )}
                />

                <Controller
                  name="client"
                  control={control}
                  disabled
                  render={({ field }) => (
                    <TextField
                      {...field}
                      id="client-textfield"
                      label="Client"
                      fullWidth
                      value={order?.clientFullName}
                      error={!!errors.client}
                      helperText={errors.client?.message}
                      InputLabelProps={{
                        shrink: true,
                      }}
                    />
                  )}
                />

                <Grid container spacing={2} alignItems="center">
                  <Grid item xs={12} md={6}>
                    <AddressForm
                      type="billing"
                      address={billingAddress}
                      onChange={handleBillingChange}
                    />
                  </Grid>

                  <Grid item xs={12} md={6}>
                    <AddressForm
                      type="shipping"
                      address={shippingAddress}
                      onChange={handleShippingChange}
                    />
                  </Grid>
                </Grid>
              </Stack>
            </Card>
          </Grid>
          <Grid item xs={12} md={4}>
            <Card>
              <CardHeader title="Notes de commandes" />
              <Stack spacing={3} sx={{ p: 3 }}>
                {notes.map((note, index) => (
                  <Stack key={note.id} spacing={2}>
                    <TextField
                      label={`Note ${index + 1}`}
                      multiline
                      rows={4}
                      fullWidth
                      value={note.note}
                      onChange={(e) =>
                        updateNote(note.id, "note", e.target.value)
                      }
                    />
                    <Grid
                      container
                      spacing={2}
                      key={note.id}
                      alignItems="center"
                    >
                      <Grid item xs={10}>
                        <Autocomplete
                          id={`type-select-${note.id}`}
                          options={Object.values(NoteType)}
                          getOptionLabel={(option) =>
                            noteTypeLabels[option as NoteType]
                          }
                          value={note.noteType || NoteType.PRIVATE}
                          onChange={(_, value) =>
                            updateNote(note.id, "noteType", value || "")
                          }
                          renderInput={(params) => (
                            <TextField
                              {...params}
                              label="Type de Note"
                              fullWidth
                            />
                          )}
                        />
                      </Grid>

                      <Grid item xs={2}>
                        <IconButton
                          color="error"
                          onClick={() => removeNote(note.id, index)}
                          aria-label="Supprimer cette note"
                        >
                          <DeleteOutlineIcon />
                        </IconButton>
                      </Grid>
                    </Grid>
                  </Stack>
                ))}
                <Button variant="contained" onClick={addNote}>
                  Ajouter une note
                </Button>
              </Stack>
            </Card>
          </Grid>
        </Grid>
        <Grid container spacing={2} sx={{ padding: 2 }}>
          <Grid item xs={12} md={8}>
            <Card>
              <Stack spacing={3} sx={{ p: 3 }}>
                <TableContainer component={Paper}>
                  <Table sx={{ minWidth: 650 }} aria-label="simple table">
                    <TableHead>
                      <TableRow>
                        <TableCell>Article</TableCell>
                        <TableCell align="right">&nbsp;&nbsp;Coût</TableCell>
                        <TableCell align="right">Qte</TableCell>
                        <TableCell align="right">Total</TableCell>
                      </TableRow>
                    </TableHead>
                    <TableBody>
                      {order?.updateOrderLineRequests.map((item, index) => (
                        <TableRow key={index}>
                          <TableCell>{item.productName}</TableCell>
                          <TableCell align="right">{item.unitPrice}€</TableCell>
                          <TableCell align="right">{item.qts}</TableCell>
                          <TableCell align="right">
                            {item?.unitPrice * item?.qts}€
                          </TableCell>
                        </TableRow>
                      ))}
                    </TableBody>
                  </Table>
                </TableContainer>

                <Typography variant="subtitle2" align="right">
                  Sous-total des articles: {order?.total}€
                </Typography>
                <Typography variant="subtitle2" align="right">
                  Total de la commande : {order?.subTotal}€
                </Typography>
              </Stack>
            </Card>
          </Grid>
          <Grid item xs={12} md={4}>
            <Card sx={{ marginBottom: 2 }}>
              <>
                <CardHeader title="Historique des modifications" />
                {error ? (
                  <Typography color="error">{error}</Typography>
                ) : loading ? (
                  <Box
                    sx={{ display: "flex", justifyContent: "center", my: 3 }}
                  >
                    <CircularProgress />
                  </Box>
                ) : (
                  <>
                    <TableContainer sx={{ maxHeight: 400 }}>
                      <Table stickyHeader>
                        <TableBody>
                          <TableRow>
                            <TableCell
                              sx={{ width: "25%", fontSize: "0.85rem" }}
                            >
                              <strong>Date</strong>
                            </TableCell>

                            <TableCell
                              sx={{ width: "35%", fontSize: "0.85rem" }}
                            >
                              <strong>Nouveau statut</strong>
                            </TableCell>
                            <TableCell
                              sx={{ width: "35%", fontSize: "0.85rem" }}
                            >
                              <strong>Ancien Statut</strong>
                            </TableCell>
                          </TableRow>
                          {history.length > 0 ? (
                            history.map((entry, index) => (
                              <TableRow key={index}>
                                <TableCell
                                  sx={{
                                    width: "25%",
                                    fontSize: "0.85rem",
                                    overflow: "hidden",
                                    textOverflow: "ellipsis",
                                    whiteSpace: "nowrap",
                                  }}
                                >
                                  {formatDate(entry?.createdAt)}
                                </TableCell>
                                <TableCell
                                  sx={{
                                    width: "35%",
                                    fontSize: "0.85rem",
                                    fontWeight: "bold",
                                    overflow: "hidden",
                                    textOverflow: "ellipsis",
                                    whiteSpace: "nowrap",
                                  }}
                                >
                                  {getStatusInFrench(
                                    entry?.oldOrderStatus as OrderStatus
                                  )}
                                </TableCell>
                                <TableCell
                                  sx={{
                                    width: "35%",
                                    fontSize: "0.85rem",
                                    fontWeight: "bold",
                                    overflow: "hidden",
                                    textOverflow: "ellipsis",
                                    whiteSpace: "nowrap",
                                  }}
                                >
                                  {getStatusInFrench(
                                    entry?.newOrderStatus as OrderStatus
                                  )}
                                </TableCell>
                              </TableRow>
                            ))
                          ) : (
                            <TableRow>
                              <TableCell colSpan={3}>
                                <Typography align="center">
                                  Aucun historique trouvé.
                                </Typography>
                              </TableCell>
                            </TableRow>
                          )}
                        </TableBody>
                      </Table>
                    </TableContainer>

                    <Pagination
                      count={Math.max(
                        1,
                        pageSize > 0
                          ? Math.ceil(numberOfElements / pageSize)
                          : 1
                      )}
                      shape="rounded"
                      page={currentPage}
                      onChange={(_, value) => setCurrentPage(value)}
                      size="small"
                      disabled={Math.ceil(numberOfElements / pageSize) === 0}
                      sx={{
                        mt: 2,
                        display: "flex",
                        justifyContent: "center",
                        "& .MuiPaginationItem-root": {
                          fontSize: "1rem",
                          padding: "4px 8px",
                          mb: "4px",
                        },
                        "& .Mui-disabled": {
                          opacity: 0.5,
                        },
                      }}
                    />
                  </>
                )}
              </>
            </Card>
          </Grid>
        </Grid>
        </Grid>
        <Grid container spacing={3}>
          <Grid item xs={12} md={8}>
            <Box sx={{ display: "flex", justifyContent: "flex-end", mt: 2 }}>
              <LoadingButton
                onClick={handleModifierCommande}
                loading={updateLoading}
                variant="contained"
                disabled={order?.orderStatus === OrderStatus.DELETED}
              >
                Modifier la commande
              </LoadingButton>
            </Box>
          </Grid>
        </Grid>
        <Snackbar
          open={snackbarOpen}
          autoHideDuration={6000}
          onClose={handleSnackbarClose}
          anchorOrigin={{ vertical: "top", horizontal: "right" }}
        >
          <Alert onClose={handleSnackbarClose} severity={snackbarSeverity}>
            {snackbarMessage}
          </Alert>
        </Snackbar>
      </LocalizationProvider>
    </>
  );
}
