import Box from "@mui/material/Box";
import Paper from "@mui/material/Paper";
import Stack from "@mui/material/Stack";
import Button from "@mui/material/Button";
import Collapse from "@mui/material/Collapse";
import MenuItem from "@mui/material/MenuItem";
import TableRow from "@mui/material/TableRow";
import Checkbox from "@mui/material/Checkbox";
import TableCell from "@mui/material/TableCell";
import IconButton from "@mui/material/IconButton";
import ListItemText from "@mui/material/ListItemText";
import { useBoolean } from "../../../../hooks/use-boolean";
import { fDate, fTime } from "../../../../utils/format-time";
import Label from "../../../components/label";
import Iconify from "../../../components/iconify";
import { ConfirmDialog } from "../../../components/custom-dialog";
import CustomPopover, { usePopover } from "../../../components/custom-popover";
import {
  Order,
  order_options,
  OrderLine,
  OrderStatus,
  orderStatusLabels,
  OrderWithClientDetails,
} from "@/contexts/types/main-order";
import { useGetUserById } from "@/shared/api/user";
import { IClientItem } from "@/shared/types/user";
import { useEffect, useState } from "react";
import {
  useGetProductsByIds,
  useGetVariationsByIds,
} from "@/shared/api/product";
import {
  Alert,
  Avatar,
  Dialog,
  DialogActions,
  DialogContent,
  DialogTitle,
  FormControl,
  Grid,
  InputLabel,
  Select,
  Snackbar,
} from "@mui/material";
import { ProductType } from "@/shared/types/product";
import { DEFAULT_IMAGE_URL } from "@/config-global";
import giftCardImage from "@/shared/assets/giftcard.png";
import { updateOrderStatus } from "@/shared/api/main-order";

type Props = {
  row: OrderWithClientDetails;
  selected: boolean;
  onViewRow: VoidFunction;
  onSelectRow: VoidFunction;
  onTrashOrder: VoidFunction;
  onDeleteRow: VoidFunction;
  onEditRow: VoidFunction;
  onRestoreOrder: VoidFunction;
  onChangeStatut: (orderId: string, userId: string, statut: OrderStatus) => void;
};

export default function OrderTableRow({
  row,
  selected,
  onViewRow,
  onSelectRow,
  onTrashOrder,
  onDeleteRow,
  onEditRow,
  onRestoreOrder,
  onChangeStatut,
}: Props) {
  const {
    orderLines,
    status,
    id,
    createdAt,
    clientId,
    total,
    clientFullName,
    clientEmail,
  } = row;

  const confirmCorbeille = useBoolean();
  const confirmDelete = useBoolean();
  const confirmRestore = useBoolean();

  const collapse = useBoolean();

  const popover = usePopover();

  const [totalQuantityOrder, setTotalQuantityOrder] = useState<number>(0);

  useEffect(() => {
    const totalQuantity = calculateTotalQuantity(row);
    setTotalQuantityOrder(totalQuantity);
  }, [row]);

  function calculateTotalQuantity(order: Order): number {
    return order.orderLines.reduce((total, line) => total + line.quantity, 0);
  }

  const [parentProductsImagePath, setParentProductsImagePath] = useState<{
    [key: string]: string;
  }>({});

  const simpleProductIds = row.orderLines
    .filter((orderLine) => orderLine.productType === "SIMPLE")
    .map((orderLine) => orderLine.productId);

  const variationIds = row.orderLines
    .filter((orderLine) => orderLine.productType === "VARIABLE")
    .map((orderLine) => orderLine.productId);

  const { products: simpleProducts, productsLoading: simpleProductsLoading } =
    useGetProductsByIds(simpleProductIds);
  const {
    variations: variationProducts,
    variationsLoading: variationProductsLoading,
  } = useGetVariationsByIds(variationIds);

  const parentProductIds = variationProducts
    .map((variation) => variation.productId)
    .filter((id): id is number => id !== undefined)
    .map((id) => id.toString());

  const { products: parentProducts, productsLoading: parentProductsLoading } =
    useGetProductsByIds(parentProductIds);

  const getFinalImagePath = (orderLine: OrderLine): string => {
    if (orderLine.productType === ProductType.SIMPLE) {
      return extractUrl(orderLine.productImagePath);
    }

    if (orderLine.productType === ProductType.VARIABLE) {
      const parentImagePath = parentProductsImagePath[orderLine.productId];
      return valideVariationUrl(orderLine.productImagePath, parentImagePath);
    }

    if (orderLine.productType === ProductType.GIFT_CARD) {
      return giftCardImage.src;
    }

    return "https://mydressin-rec.s3.eu-west-3.amazonaws.com/Image-not-found.png";
  };

  const [openDialog, setOpenDialog] = useState(false);
  const [statut, setStatut] = useState<OrderStatus>(status);
  const handleOpenDialog = () => {
    if(status !== OrderStatus.DELETED){
      setOpenDialog(true);
    }
  };

  const handleCloseDialog = () => {
    setOpenDialog(false);
  };

  const handleChangeStatut = async () => {
    try {
      const userId = localStorage.getItem("userId") || "";
      onChangeStatut(id, userId, statut);
    } catch (error) {
      console.error("Erreur lors de la mise à jour du statut :", error);
    } finally {
      handleCloseDialog();
    }
  };

  const renderPrimary = (
    <TableRow hover selected={selected}>
      <TableCell padding="checkbox">
        <Checkbox checked={selected} onClick={onSelectRow} />
      </TableCell>

      <TableCell>
        <Box
          onClick={onViewRow}
          sx={{
            cursor: "pointer",
            "&:hover": {
              textDecoration: "underline",
            },
          }}
        >
          {id}
        </Box>
      </TableCell>

      <TableCell sx={{ display: "flex", alignItems: "center" }}>
        <ListItemText
          primary={clientFullName}
          secondary={clientEmail}
          primaryTypographyProps={{ typography: "body2" }}
          secondaryTypographyProps={{
            component: "span",
            color: "text.disabled",
          }}
        />
      </TableCell>

      <TableCell>
        <ListItemText
          primary={fDate(createdAt)}
          secondary={fTime(createdAt)}
          primaryTypographyProps={{ typography: "body2", noWrap: true }}
          secondaryTypographyProps={{
            mt: 0.5,
            component: "span",
            typography: "caption",
          }}
        />
      </TableCell>

      <TableCell align="center"> {totalQuantityOrder} </TableCell>

      <TableCell> {total + " €"}</TableCell>

      <TableCell>
        <Label
          onClick={handleOpenDialog}
          variant="soft"
          color={
            (status === OrderStatus.FINISHED && "success") ||
            (status === OrderStatus.FAILED && "warning") ||
            (status === OrderStatus.ON_HOLD && "info") ||
            (status === OrderStatus.IN_PROGRESS && "primary") ||
            (status === OrderStatus.CANCELED && "error") ||
            "default"
          }
          sx={{
            ...(status === OrderStatus.REFUNDED && {
              backgroundColor: "lightblue",
              color: "darkblue",
            }),
            ...(status === OrderStatus.DELETED && {
              backgroundColor: "pink",
              color: "darkred",
            }),
          }}
        >
          {status === OrderStatus.DELETED
            ? "Corbeille"
            : orderStatusLabels[status]}
        </Label>
      </TableCell>

      <TableCell align="right" sx={{ px: 1, whiteSpace: "nowrap" }}>
        <IconButton
          color={collapse.value ? "inherit" : "default"}
          onClick={collapse.onToggle}
          sx={{
            ...(collapse.value && {
              bgcolor: "action.hover",
            }),
          }}
        >
          <Iconify icon="eva:arrow-ios-downward-fill" />
        </IconButton>

        <IconButton
          color={popover.open ? "inherit" : "default"}
          onClick={popover.onOpen}
        >
          <Iconify icon="eva:more-vertical-fill" />
        </IconButton>
      </TableCell>
    </TableRow>
  );

  const renderSecondary = (
    <TableRow>
      <TableCell sx={{ p: 1, border: "none" }} colSpan={9}>
        <Collapse
          in={collapse.value}
          timeout="auto"
          unmountOnExit
          sx={{ bgcolor: "background.neutral" }}
        >
          <Stack component={Paper} sx={{ m: 1.5 }}>
            {orderLines.map((orderLine) => (
              <Stack
                key={orderLine.id}
                direction="row"
                alignItems="center"
                sx={{
                  p: (theme) => theme.spacing(1.5, 2, 1.5, 1.5),
                  "&:not(:last-of-type)": {
                    borderBottom: (theme) =>
                      `solid 2px ${theme.palette.background.neutral}`,
                  },
                }}
              >
                <Avatar
                  alt={
                    orderLine.productType === ProductType.GIFT_CARD
                      ? "Carte cadeau"
                      : orderLine.productName
                  }
                  src={getFinalImagePath(orderLine)}
                  variant="rounded"
                  sx={{ width: 64, height: 64, mr: 2 }}
                />

                <ListItemText
                  primary={
                    orderLine.productType === ProductType.GIFT_CARD
                      ? "Carte cadeau"
                      : orderLine.productName || ""
                  }
                  secondary={
                    orderLine.productType === ProductType.GIFT_CARD
                      ? ""
                      : orderLine.productUgs || ""
                  }
                  primaryTypographyProps={{
                    typography: "body2",
                  }}
                  secondaryTypographyProps={{
                    component: "span",
                    color: "text.disabled",
                    mt: 0.5,
                  }}
                />

                <Label
                  variant="filled"
                  color={orderLine.source === "LIVE" ? "success" : "info"}
                  sx={{ mx: 8, padding: 1.5 }}
                >
                  {orderLine.source === "LIVE" ? "Live" : "Site"}
                </Label>

                <Box sx={{ ml: "auto" }}>x{orderLine.quantity}</Box>

                <Box sx={{ width: 110, textAlign: "right" }}>
                  {orderLine.totalPrice + " €"}
                </Box>
              </Stack>
            ))}
          </Stack>
        </Collapse>
      </TableCell>
    </TableRow>
  );

  return (
    <>
      {renderPrimary}

      {renderSecondary}

      <CustomPopover
        open={popover.open}
        onClose={popover.onClose}
        arrow="right-top"
        sx={{ width: 200 }}
      >
        <MenuItem
          onClick={() => {
            onViewRow();
            popover.onClose();
          }}
        >
          <Iconify icon="solar:eye-bold" />
          Visualiser
        </MenuItem>
        {status !== OrderStatus.DELETED && (
          <MenuItem
            onClick={() => {
              onEditRow();
              popover.onClose();
            }}
          >
            <Iconify icon="solar:pen-bold" />
            Modifier
          </MenuItem>
        )}
        {status === OrderStatus.DELETED && (
          <MenuItem
            onClick={() => {
              confirmRestore.onTrue();
              popover.onClose();
            }}
            sx={{ color: "primary.main" }}
          >
            <Iconify icon="eva:refresh-fill" sx={{ mr: 2 }} />
            Restaurer
          </MenuItem>
        )}
        {status == OrderStatus.DELETED && (
          <MenuItem
            onClick={() => {
              confirmDelete.onTrue();
              popover.onClose();
            }}
            sx={{ color: "error.main" }}
          >
            <Iconify icon="solar:trash-bin-trash-bold" />
            Supprimer
          </MenuItem>
        )}
        {status !== OrderStatus.DELETED && (
          <MenuItem
            onClick={() => {
              confirmCorbeille.onTrue();
              popover.onClose();
            }}
            sx={{ color: "error.main" }}
          >
            <Iconify icon="solar:trash-bin-trash-bold" />
            Mettre à la corbeille
          </MenuItem>
        )}
      </CustomPopover>
      <ConfirmDialog
        open={confirmCorbeille.value}
        onClose={confirmCorbeille.onFalse}
        title="Mettre à la corbeille"
        content="Etes-vous sûr de vouloir le mettre à la corbeille?"
        action={
          <Button
            variant="contained"
            color="error"
            onClick={() => {
              onTrashOrder();
              confirmCorbeille.onFalse();
            }}
          >
            Mettre à la corbeille
          </Button>
        }
      />

      <ConfirmDialog
        open={confirmDelete.value}
        onClose={confirmDelete.onFalse}
        title="Supprimer"
        content="Etes-vous sûr de vouloir supprimer cette commande?"
        action={
          <Button variant="contained" color="error" onClick={onDeleteRow}>
            Supprimer
          </Button>
        }
      />
      <ConfirmDialog
        open={confirmRestore.value}
        onClose={confirmRestore.onFalse}
        title="Confirmer la restauration"
        content="Êtes-vous sûr de vouloir restaurer cette commande ?"
        action={
          <Button
            variant="contained"
            color="success"
            onClick={() => {
              onRestoreOrder();
              confirmRestore.onFalse();
            }}
          >
            Restaurer
          </Button>
        }
      />
      <Dialog
        open={openDialog}
        onClose={handleCloseDialog}
        maxWidth="sm"
        fullWidth
      >
        <DialogTitle>Modifier le Statut</DialogTitle>
        <DialogContent>
          <Box sx={{ width: "400px", margin: "auto", marginTop: "12px" }}>
            <Grid container spacing={2}>
              <Grid item xs={12}>
                <FormControl fullWidth>
                  <Select
                    value={statut}
                    onChange={(e) => setStatut(e.target.value as OrderStatus)}
                  >
                    {Object.values(OrderStatus).map((status) => (
                      <MenuItem key={status} value={status}>
                        {order_options[status]}
                      </MenuItem>
                    ))}
                  </Select>
                </FormControl>
              </Grid>
            </Grid>
          </Box>
        </DialogContent>
        <DialogActions>
          <Button onClick={handleCloseDialog}>Annuler</Button>
          <Button color="primary" onClick={handleChangeStatut}>
            Enregistrer
          </Button>
        </DialogActions>
      </Dialog>
    </>
  );
}

const extractUrl = (
  url: string | { preview: string } | null | undefined
): string => {
  const defaultImageUrl = DEFAULT_IMAGE_URL ?? "";
  if (url == null) {
    return defaultImageUrl;
  }
  if (typeof url === "string") {
    const trimmedUrl = url.trim();
    const cleanedUrl = trimmedUrl.replace(/^Optional\[(.*)\]$/, "$1").trim();
    const urlPattern = /^(https?:\/\/)/;
    if (cleanedUrl && urlPattern.test(cleanedUrl)) {
      return cleanedUrl;
    }
    return defaultImageUrl;
  }
  if (typeof url === "object" && url !== null && "preview" in url) {
    return extractUrl(url.preview);
  }
  return defaultImageUrl;
};

const extractUrls = (
  urls: (string | { preview: string } | null | undefined)[]
): string[] => {
  return urls.map(extractUrl);
};

const valideVariationUrl = (
  firstUrl: string | { preview: string } | null | undefined,
  secondUrl: string | { preview: string } | null | undefined
): string => {
  const defaultImageUrl = DEFAULT_IMAGE_URL;
  const firstUrlResult = extractUrl(firstUrl);
  if (firstUrlResult !== defaultImageUrl) {
    return firstUrlResult;
  }
  const secondUrlResult = extractUrl(secondUrl);
  if (secondUrlResult !== defaultImageUrl) {
    return secondUrlResult;
  }
  return defaultImageUrl;
};
