import React, { useCallback, useState } from "react";
import Stack from "@mui/material/Stack";
import TextField from "@mui/material/TextField";
import InputAdornment from "@mui/material/InputAdornment";
import Iconify from "../../../components/iconify";
import {IOrderTableFilters,IOrderTableFilterValue} from "../../../types/order";
import { Button } from "@mui/material";
import CustomDateRangePicker from "@/shared/components/custom-date-range-picker";

type Props = {
  filters: IOrderTableFilters;
  onFilters: (name: string, value: IOrderTableFilterValue) => void;
};

export default function OrderTableToolbar({
  filters,
  onFilters,
}: Props) {

  const handleFilterName = useCallback(
    (event: React.ChangeEvent<HTMLInputElement>) => {
      onFilters("keyword", event.target.value);
    },
    [onFilters]
  );

  const [startDate, setStartDate] = useState<Date | null>(new Date());
  const [endDate, setEndDate] = useState<Date | null>(new Date());

  const [isDateRangeOpen, setIsDateRangeOpen] = useState(false);

  const formatDateToLocalDateTimeStart = (date: Date): string => {
    return date.toISOString().split('T')[0] + "T00:00:00";
  };
  const formatDateToLocalDateTimeEnd = (date: Date): string => {
    return date.toISOString().split('T')[0] + "T23:59:59";
  };

  const onChangeStartDate = (newStartDate: Date | null) => {
    setStartDate(newStartDate);
    if (newStartDate) {
      const formattedDate = formatDateToLocalDateTimeStart(newStartDate);
      onFilters("startDate", formattedDate);
    }
  };

  const onChangeEndDate = (newEndDate: Date | null) => {
    setEndDate(newEndDate);
    if (newEndDate) {
      onFilters("endDate", formatDateToLocalDateTimeEnd(newEndDate));
    }
  };

  const closeDateRange = () => {
    setIsDateRangeOpen(!isDateRangeOpen);
  }


  return (
    <>
      <Stack
        spacing={2}
        alignItems={{ xs: "flex-end", md: "center" }}
        direction={{
          xs: "column",
          md: "row",
        }}
        sx={{
          p: 2.5,
          pr: { xs: 2.5, md: 1 },
        }}
      >
        <Stack
          direction="row"
          alignItems="center"
          spacing={2}
          flexGrow={1}
          sx={{ width: 1 }}
        >
          <TextField
            sx={{ width: '75%' }}
            value={filters.keyword}
            onChange={handleFilterName}
            placeholder="Chercher par nom du client ou numéro de commande..."
            InputProps={{
              startAdornment: (
                <InputAdornment position="start">
                  <Iconify
                    icon="eva:search-fill"
                    sx={{ color: "text.disabled" }}
                  />
                </InputAdornment>
              ),
            }}
          />
          <Button
            sx={{
              width: {
                xs: '100%', sm: '50%', md: '25%'
              },
              height: {
                xs: '45px', sm: '50px', md: '55px'
              },
              whiteSpace: 'nowrap',
              overflow: 'hidden',
              textOverflow: 'ellipsis',
              fontSize: {
                xs: '12px', sm: '14px'
              },
              padding: {
                xs: '8px', sm: '12px', md: '16px'
              }
            }}
            variant="outlined"
            startIcon={
              <Iconify
                icon="carbon:calendar"
                sx={{
                  mr: { xs: 0.5, sm: 1 },
                  fontSize: {
                    xs: '18px',
                    sm: '20px',
                    md: '24px'
                  }
                }}
              />
            }
            onClick={() => setIsDateRangeOpen(true)}
          >
            {startDate?.toLocaleDateString()} - {endDate?.toLocaleDateString()}
          </Button>
        </Stack>
      </Stack>
      <CustomDateRangePicker
        open={isDateRangeOpen}
        onClose={closeDateRange}
        startDate={startDate}
        endDate={endDate}
        onChangeEndDate={(newValue: Date | null) => {
          if (newValue) {
            onChangeEndDate(newValue);
          }
        }}
        onChangeStartDate={(newValue: Date | null) => {
          if (newValue) {
            onChangeStartDate(newValue);
          }
        }}
      />
    </>
  );
}
