import React, { useEffect, useState } from "react";
import {
  Stack,
  Grid,
  Typography,
  TextField,
  Snackbar,
  Alert,
  Autocomplete,
  Tooltip,
  IconButton,
} from "@mui/material";
import LoadingButton from "@mui/lab/LoadingButton";
import {
  AddressType,
  ClientWithCustomResponse,
  IAddress,
  IAddressItem,
} from "@/shared/types/user";
import { SaveAddressRequest } from "@/shared/types/address";
import { addAdressToUser, updateAdressToUser } from "@/shared/api/user";
import * as Yup from "yup";
import { countries } from "@/shared/assets/data";
import SwitchLeftRoundedIcon from "@mui/icons-material/SwitchLeftRounded";

const billingAddressSchema = Yup.object().shape({
  billingAddressFirstName: Yup.string().required("Le prénom est obligatoire"),
  billingAddressLastName: Yup.string().required("Le nom est obligatoire"),
  billingAddressCompanyName: Yup.string().optional(),
  billingAddressPhoneNumber: Yup.string().required(
    "Le numéro de téléphone est obligatoire"
  ),
  billingAddressEmail: Yup.string()
    .email("Adresse e-mail invalide")
    .required("L'adresse e-mail est obligatoire"),
  billingAddressStreet: Yup.string().required("La rue est obligatoire"),
  billingAddressHome: Yup.string().optional(),
  billingAddressCity: Yup.string().required("La ville est obligatoire"),
  billingAddressZipCode: Yup.string().required(
    "Le code postal est obligatoire"
  ),
  billingAddressCountry: Yup.string().required("Le pays est obligatoire"),
});
const shippingAddressSchema = Yup.object().shape({
  shippingAddressFirstName: Yup.string().required("Le prénom est obligatoire"),
  shippingAddressLastName: Yup.string().required("Le nom est obligatoire"),
  shippingAddressCompanyName: Yup.string().optional(),
  shippingAddressPhoneNumber: Yup.string().required(
    "Le numéro de téléphone est obligatoire"
  ),
  shippingAddressEmail: Yup.string()
    .email("Adresse e-mail invalide")
    .required("L'adresse e-mail est obligatoire"),
  shippingAddressStreet: Yup.string().required("La rue est obligatoire"),
  shippingAddressHome: Yup.string().optional(),
  shippingAddressCity: Yup.string().required("La ville est obligatoire"),
  shippingAddressZipCode: Yup.string().required(
    "Le code postal est obligatoire"
  ),
  shippingAddressCountry: Yup.string().required("Le pays est obligatoire"),
});

interface AddressDetailsCardProps {
  selectedClient: ClientWithCustomResponse | null;
  onAddressSubmitted: (
    billingId: number | null,
    shippingId: number | null
  ) => void;
}

const AddressDetailsCard: React.FC<AddressDetailsCardProps> = ({
  selectedClient,
  onAddressSubmitted,
}) => {
  const [snackbarOpen, setSnackbarOpen] = useState(false);
  const [snackbarMessage, setSnackbarMessage] = useState("");
  const [snackbarSeverity, setSnackbarSeverity] = useState<
    "success" | "warning"
  >("success");

  const handleSnackbarClose = () => {
    setSnackbarOpen(false);
  };

  const [errors, setErrors] = useState<Record<string, string>>({});
  const [billingLoading, setBillingLoading] = useState(false);
  const [shippingLoading, setShippingLoading] = useState(false);
  const [shouldUpdateShipping, setShouldUpdateShipping] = useState(false);
  const [shouldUpdateBilling, setShouldUpdateBilling] = useState(false);
  const [billingId, setBillingId] = useState<number>(0);
  const [shippingId, setShippingId] = useState<number>(0);
  const [canSwitchfromShipping, setCanSwitchfromShipping] = useState(false);
  const [canSwitchfromBilling, setCanSwitchfromBilling] = useState(false);

  const [billingAddress, setBillingAddress] = useState<SaveAddressRequest>({
    clientUid: 0,
    firstName: "",
    lastName: "",
    companyName: "",
    phoneNumber: "0",
    email: "",
    street: "",
    home: "",
    city: "",
    zipCode: "",
    country: "",
    addressType: AddressType.BILLING,
  });
  const [shippingAddress, setShippingAddress] = useState<SaveAddressRequest>({
    clientUid: 0,
    firstName: "",
    lastName: "",
    companyName: "",
    phoneNumber: "",
    email: "",
    street: "",
    home: "",
    city: "",
    zipCode: "",
    country: "",
    addressType: AddressType.SHIPPING,
  });

  useEffect(() => {
    if (selectedClient?.shippingAddress?.id) {
      setShippingId(selectedClient?.shippingAddress?.id);
      setCanSwitchfromShipping(true)
      setShouldUpdateShipping(true);
    }
    if (selectedClient?.billingAddress?.id) {
      setBillingId(selectedClient?.billingAddress?.id);
      setCanSwitchfromBilling(true)
      setShouldUpdateBilling(true);
    }
    if (selectedClient) {
      setShippingAddress({
        clientUid: selectedClient?.uid ?? 0,
        firstName: selectedClient.shippingAddress?.firstName || "",
        lastName: selectedClient.shippingAddress?.lastName || "",
        companyName: selectedClient.shippingAddress?.companyName || "",
        phoneNumber:
          selectedClient.shippingAddress?.phoneNumber?.toString() || "",
        email: selectedClient.shippingAddress?.email || "",
        street: selectedClient.shippingAddress?.street || "",
        home: selectedClient.shippingAddress?.home || "",
        city: selectedClient.shippingAddress?.city || "",
        zipCode: selectedClient.shippingAddress?.zipCode || "",
        country: selectedClient.shippingAddress?.country || "",
        addressType: AddressType.SHIPPING,
      });

      setBillingAddress({
        clientUid: selectedClient?.uid ?? 0,
        firstName: selectedClient.billingAddress?.firstName || "",
        lastName: selectedClient.billingAddress?.lastName || "",
        companyName: selectedClient.billingAddress?.companyName || "",
        phoneNumber:
          selectedClient.billingAddress?.phoneNumber?.toString() || "",
        email: selectedClient.billingAddress?.email || "",
        street: selectedClient.billingAddress?.street || "",
        home: selectedClient.billingAddress?.home || "",
        city: selectedClient.billingAddress?.city || "",
        zipCode: selectedClient.billingAddress?.zipCode || "",
        country: selectedClient.billingAddress?.country || "",
        addressType: AddressType.BILLING,
      });
    }
  }, [selectedClient]);

  const billingAddressFieldNameToFormPropertyMap: {
    [key: string]: keyof SaveAddressRequest;
  } = {
    billingAddressFirstName: "firstName",
    billingAddressLastName: "lastName",
    billingAddressCompanyName: "companyName",
    billingAddressPhoneNumber: "phoneNumber",
    billingAddressEmail: "email",
    billingAddressStreet: "street",
    billingAddressHome: "home",
    billingAddressCity: "city",
    billingAddressZipCode: "zipCode",
    billingAddressCountry: "country",
  };

  const billingAddresshandleInputChange = (
    e: React.ChangeEvent<HTMLInputElement | HTMLTextAreaElement>
  ) => {
    const { name, value } = e.target;
    const formProperty = billingAddressFieldNameToFormPropertyMap[name];
    if (formProperty) {
      setBillingAddress((prev) => ({
        ...prev,
        [formProperty]: value,
      }));
      setErrors((prevErrors) => ({ ...prevErrors, [name]: "" }));
    }
  };

  const shippingAddressFieldNameToFormPropertyMap: {
    [key: string]: keyof SaveAddressRequest;
  } = {
    shippingAddressFirstName: "firstName",
    shippingAddressLastName: "lastName",
    shippingAddressCompanyName: "companyName",
    shippingAddressPhoneNumber: "phoneNumber",
    shippingAddressEmail: "email",
    shippingAddressStreet: "street",
    shippingAddressHome: "home",
    shippingAddressCity: "city",
    shippingAddressZipCode: "zipCode",
    shippingAddressCountry: "country",
  };

  const shippingAddressHandleInputChange = (
    e: React.ChangeEvent<HTMLInputElement | HTMLTextAreaElement>
  ) => {
    const { name, value } = e.target;
    const formProperty = shippingAddressFieldNameToFormPropertyMap[name];
    if (formProperty) {
      setShippingAddress((prev) => ({
        ...prev,
        [formProperty]: value,
      }));
    }
  };

  const [selectedBillingCountry, setSelectedBillingCountry] = useState<
    string | null
  >(null);
  const [selectedShippingCountry, setSelectedShippingCountry] = useState<
    string | null
  >(null);

  useEffect(() => {
    setSelectedBillingCountry(
      selectedClient?.billingAddress?.country || countries[0].code
    );
    setSelectedShippingCountry(
      selectedClient?.shippingAddress?.country || countries[0].code
    );
  }, [selectedClient]);

  const handleSubmitBilling = async () => {
    setBillingLoading(true);
    try {
      const billingValidate = {
        billingAddressFirstName: billingAddress.firstName,
        billingAddressLastName: billingAddress.lastName,
        billingAddressCompanyName: billingAddress.companyName,
        billingAddressPhoneNumber: billingAddress.phoneNumber,
        billingAddressEmail: billingAddress.email,
        billingAddressStreet: billingAddress.street,
        billingAddressHome: billingAddress.home,
        billingAddressCity: billingAddress.city,
        billingAddressZipCode: billingAddress.zipCode,
        billingAddressCountry: billingAddress.country,
      };

      await billingAddressSchema.validate(billingValidate, {
        abortEarly: false,
      });
      if (shouldUpdateBilling) {
        const updateAdressData: IAddress = {
          id: billingId,
          firstName: billingAddress.firstName,
          lastName: billingAddress.lastName,
          companyName: billingAddress.companyName,
          email: billingAddress.email,
          phoneNumber: billingAddress.phoneNumber,
          street: billingAddress.street,
          home: billingAddress.home,
          city: billingAddress.city,
          zipCode: billingAddress.zipCode,
          mainAddress:true,
          country: billingAddress.country,
          addressType: AddressType.BILLING,
        };
        const response = await updateAdressToUser(updateAdressData);

        if (response.status === 200) {
          setSnackbarOpen(true);
          setSnackbarMessage("Adresse modifié avec succès!");
          setSnackbarSeverity("success");
          onAddressSubmitted(response.data.id, null);
          setBillingId(response.data.id);
        } else {
          setSnackbarOpen(true);
          setSnackbarMessage("Problème avec l'ajout de l'adresse.");
          setSnackbarSeverity("warning");
        }
      } else {
        const response = await addAdressToUser(billingAddress);

        if (response.status === 200) {
          setSnackbarOpen(true);
          setSnackbarMessage("Adresse ajoutée avec succès!");
          setSnackbarSeverity("success");
          onAddressSubmitted(response.data.id, null);
          setShouldUpdateBilling(true);
          setBillingId(response.data.id);
          setCanSwitchfromBilling(true)
        } else {
          setSnackbarOpen(true);
          setSnackbarMessage("Problème avec l'ajout de l'adresse.");
          setSnackbarSeverity("warning");
        }
      }
    } catch (error) {
      if (error instanceof Yup.ValidationError) {
        const validationErrors: Record<string, string> = {};
        error.inner.forEach((err) => {
          if (err.path) validationErrors[err.path] = err.message;
        });
        setErrors(validationErrors);
      } else {
        console.error("Erreur lors de la soumission :", error);
      }
    } finally {
      setBillingLoading(false);
    }
  };
  const handleSubmitShipping = async () => {
    setShippingLoading(true);
    try {
      const shippingValidate = {
        shippingAddressFirstName: shippingAddress.firstName,
        shippingAddressLastName: shippingAddress.lastName,
        shippingAddressCompanyName: shippingAddress.companyName,
        shippingAddressPhoneNumber: shippingAddress.phoneNumber,
        shippingAddressEmail: shippingAddress.email,
        shippingAddressStreet: shippingAddress.street,
        shippingAddressHome: shippingAddress.home,
        shippingAddressCity: shippingAddress.city,
        shippingAddressZipCode: shippingAddress.zipCode,
        shippingAddressCountry: shippingAddress.country,
      };

      await shippingAddressSchema.validate(shippingValidate, {
        abortEarly: false,
      });
      if (shouldUpdateShipping) {
        const updateAdressData: IAddress = {
          id: shippingId,
          firstName: shippingAddress.firstName,
          lastName: shippingAddress.lastName,
          companyName: shippingAddress.companyName,
          email: shippingAddress.email,
          phoneNumber: shippingAddress.phoneNumber,
          street: shippingAddress.street,
          home: shippingAddress.home,
          city: shippingAddress.city,
          zipCode: shippingAddress.zipCode,
          mainAddress:true,
          country: shippingAddress.country,
          addressType: AddressType.SHIPPING,
        };
        const response = await updateAdressToUser(updateAdressData);
        if (response.status === 200) {
          setSnackbarOpen(true);
          setSnackbarMessage("Adresse modifié avec succès!");
          setSnackbarSeverity("success");
          onAddressSubmitted(null, response.data.id);
          setShippingId(response.data.id);
          setShouldUpdateShipping(true);
        } else {
          setSnackbarOpen(true);
          setSnackbarMessage("Problème avec l'ajout de l'adresse.");
          setSnackbarSeverity("warning");
        }
      } else {
        const response = await addAdressToUser(shippingAddress);
        if (response.status === 200) {
          setSnackbarOpen(true);
          setSnackbarMessage("Adresse ajoutée avec succès!");
          setSnackbarSeverity("success");
          onAddressSubmitted(null, response.data.id);
          setShippingId(response.data.id);
          setShouldUpdateShipping(true);
          setCanSwitchfromShipping(true)
        } else {
          setSnackbarOpen(true);
          setSnackbarMessage("Problème avec l'ajout de l'adresse.");
          setSnackbarSeverity("warning");
        }
      }
    } catch (error) {
      if (error instanceof Yup.ValidationError) {
        const validationErrors: Record<string, string> = {};
        error.inner.forEach((err) => {
          if (err.path) validationErrors[err.path] = err.message;
        });
        setErrors(validationErrors);
      } else {
        console.error("Erreur lors de la soumission :", error);
      }
    } finally {
      setShippingLoading(false);
    }
  };

  const handleSwitchAddresses = (type: "switchBilling" | "switchShipping") => {
    if (type === "switchBilling") {
      setBillingAddress({
        ...shippingAddress,
        addressType: AddressType.BILLING,
      });
    } else if (type === "switchShipping") {
      setShippingAddress({
        ...billingAddress,
        addressType: AddressType.SHIPPING,
      });
    }
  };


  return (
    <>
      <Stack spacing={3} sx={{ p: 3 }}>
        <Grid container spacing={2}>
          <Grid item xs={6}>
            <Grid container spacing={2}>
              <Grid container item xs={12}>
                <Grid item xs={6}>
                  <Typography variant="h6" gutterBottom>
                    Facturation
                  </Typography>
                </Grid>

                <Grid item xs={6}>
                  <Tooltip
                    title="Expidier à la meme adresse de livraison"
                    placement="top"
                  >
                    <IconButton
                      onClick={() => handleSwitchAddresses("switchBilling")}
                      disabled={!canSwitchfromShipping}
                      sx={{
                        color: "primary.main",
                        "&:hover": {
                          color: "primary.dark",
                        },
                      }}
                    >
                      <SwitchLeftRoundedIcon />
                    </IconButton>
                  </Tooltip>
                </Grid>
              </Grid>
              {selectedClient ? (
                <>
                  <Grid item xs={6}>
                    <TextField
                      name="billingAddressFirstName"
                      label="Nom"
                      onChange={billingAddresshandleInputChange}
                      fullWidth
                      type="text"
                      value={billingAddress?.firstName}
                      error={!!errors.billingAddressFirstName}
                      helperText={errors.billingAddressFirstName}
                      InputLabelProps={{
                        shrink: true,
                      }}
                    />
                  </Grid>
                  <Grid item xs={6}>
                    <TextField
                      name="billingAddressLastName"
                      label="Prénom"
                      onChange={billingAddresshandleInputChange}
                      fullWidth
                      type="text"
                      value={billingAddress?.lastName}
                      error={!!errors.billingAddressLastName}
                      helperText={errors.billingAddressLastName}
                      InputLabelProps={{
                        shrink: true,
                      }}
                    />
                  </Grid>
                  <Grid item xs={6}>
                    <TextField
                      name="billingAddressCompanyName"
                      label="Nom de l'entreprise"
                      fullWidth
                      type="text"
                      onChange={billingAddresshandleInputChange}
                      value={billingAddress?.companyName}
                      error={!!errors.billingAddressCompanyName}
                      helperText={errors.billingAddressCompanyName}
                      InputLabelProps={{
                        shrink: true,
                      }}
                    />
                  </Grid>
                  <Grid item xs={6}>
                    <TextField
                      name="billingAddressEmail"
                      label="E-mail"
                      fullWidth
                      onChange={billingAddresshandleInputChange}
                      type="email"
                      value={billingAddress?.email}
                      error={!!errors.billingAddressEmail}
                      helperText={errors.billingAddressEmail}
                      InputLabelProps={{
                        shrink: true,
                      }}
                    />
                  </Grid>
                  <Grid item xs={6}>
                    <TextField
                      name="billingAddressPhoneNumber"
                      label="Numéro de téléphone"
                      fullWidth
                      onChange={billingAddresshandleInputChange}
                      type="text"
                      value={billingAddress?.phoneNumber}
                      error={!!errors.billingAddressPhoneNumber}
                      helperText={errors.billingAddressPhoneNumber}
                      InputLabelProps={{
                        shrink: true,
                      }}
                    />
                  </Grid>
                  <Grid item xs={6}>
                    <TextField
                      name="billingAddressStreet"
                      label="Rue"
                      fullWidth
                      onChange={billingAddresshandleInputChange}
                      type="text"
                      value={billingAddress?.street}
                      error={!!errors.billingAddressStreet}
                      helperText={errors.billingAddressStreet}
                      InputLabelProps={{
                        shrink: true,
                      }}
                    />
                  </Grid>
                  <Grid item xs={6}>
                    <TextField
                      name="billingAddressHome"
                      label="Maison"
                      fullWidth
                      onChange={billingAddresshandleInputChange}
                      type="text"
                      value={billingAddress?.home}
                      error={!!errors.billingAddressHome}
                      helperText={errors.billingAddressHome}
                      InputLabelProps={{
                        shrink: true,
                      }}
                    />
                  </Grid>
                  <Grid item xs={6}>
                    <TextField
                      name="billingAddressCity"
                      label="Ville"
                      fullWidth
                      onChange={billingAddresshandleInputChange}
                      type="text"
                      value={billingAddress?.city}
                      error={!!errors.billingAddressCity}
                      helperText={errors.billingAddressCity}
                      InputLabelProps={{
                        shrink: true,
                      }}
                    />
                  </Grid>
                  <Grid item xs={6}>
                    <TextField
                      name="billingAddressZipCode"
                      label="Code postal"
                      fullWidth
                      onChange={billingAddresshandleInputChange}
                      type="text"
                      value={billingAddress?.zipCode}
                      error={!!errors.billingAddressZipCode}
                      helperText={errors.billingAddressZipCode}
                      InputLabelProps={{
                        shrink: true,
                      }}
                    />
                  </Grid>
                  <Grid item xs={6}>
                    <Autocomplete
                      options={countries.map((country) => country.code)}
                      getOptionLabel={(option) =>
                        countries.find((country) => country.code === option)
                          ?.label ?? ""
                      }
                      value={
                        selectedBillingCountry || billingAddress?.country || ""
                      }
                      onChange={(event, newValue) => {
                        billingAddresshandleInputChange({
                          target: {
                            name: "billingAddressCountry",
                            value: newValue || "",
                          },
                        } as React.ChangeEvent<HTMLInputElement | HTMLTextAreaElement>);

                        setSelectedBillingCountry(newValue || "");
                      }}
                      renderInput={(params) => (
                        <TextField
                          {...params}
                          label="Pays"
                          error={!!errors.billingAddressCountry}
                          helperText={errors.billingAddressCountry}
                          InputLabelProps={{
                            shrink: true,
                          }}
                        />
                      )}
                      isOptionEqualToValue={(option, value) => option === value}
                    />
                  </Grid>

                  <Grid item xs={12}>
                    <LoadingButton
                      onClick={handleSubmitBilling}
                      variant="contained"
                      fullWidth
                      loading={billingLoading}
                    >
                       Soumettre
                    </LoadingButton>
                  </Grid>
                </>
              ) : (
                <Grid item xs={12}>
                  <Typography>Veuillez sélectionner un client.</Typography>
                </Grid>
              )}
            </Grid>
          </Grid>

          <Grid item xs={6}>
            <Grid container spacing={2}>
              <Grid container item xs={12}>
                <Grid item xs={4}>
                  <Typography variant="h6" gutterBottom>
                    Livraison
                  </Typography>
                </Grid>
                <Grid item xs={8}>
                  <Tooltip
                    title="Expidier à la meme adresse de Facturation"
                    placement="top"
                  >
                    <IconButton
                      onClick={() => handleSwitchAddresses("switchShipping")}
                      sx={{
                        color: "primary.main",
                        "&:hover": {
                          color: "primary.dark",
                        },
                      }}
                      disabled={!canSwitchfromBilling}
                    >
                      <SwitchLeftRoundedIcon />
                    </IconButton>
                  </Tooltip>
                </Grid>
              </Grid>
              {selectedClient ? (
                <>
                  <Grid item xs={6}>
                    <TextField
                      name="shippingAddressFirstName"
                      label="Nom"
                      onChange={shippingAddressHandleInputChange}
                      fullWidth
                      type="text"
                      value={shippingAddress?.firstName}
                      error={!!errors.shippingAddressFirstName}
                      helperText={errors.shippingAddressFirstName}
                      InputLabelProps={{
                        shrink: true,
                      }}
                    />
                  </Grid>
                  <Grid item xs={6}>
                    <TextField
                      name="shippingAddressLastName"
                      label="Prénom"
                      onChange={shippingAddressHandleInputChange}
                      fullWidth
                      type="text"
                      value={shippingAddress?.lastName}
                      error={!!errors.shippingAddressLastName}
                      helperText={errors.shippingAddressLastName}
                      InputLabelProps={{
                        shrink: true,
                      }}
                    />
                  </Grid>
                  <Grid item xs={6}>
                    <TextField
                      name="shippingAddressCompanyName"
                      label="Nom de l'entreprise"
                      fullWidth
                      type="text"
                      onChange={shippingAddressHandleInputChange}
                      value={shippingAddress?.companyName}
                      error={!!errors.shippingAddressCompanyName}
                      helperText={errors.shippingAddressCompanyName}
                      InputLabelProps={{
                        shrink: true,
                      }}
                    />
                  </Grid>
                  <Grid item xs={6}>
                    <TextField
                      name="shippingAddressEmail"
                      label="E-mail"
                      fullWidth
                      onChange={shippingAddressHandleInputChange}
                      type="email"
                      value={shippingAddress?.email}
                      error={!!errors.shippingAddressEmail}
                      helperText={errors.shippingAddressEmail}
                      InputLabelProps={{
                        shrink: true,
                      }}
                    />
                  </Grid>
                  <Grid item xs={6}>
                    <TextField
                      name="shippingAddressPhoneNumber"
                      label="Numéro de téléphone"
                      fullWidth
                      onChange={shippingAddressHandleInputChange}
                      type="text"
                      value={shippingAddress?.phoneNumber}
                      error={!!errors.shippingAddressPhoneNumber}
                      helperText={errors.shippingAddressPhoneNumber}
                      InputLabelProps={{
                        shrink: true,
                      }}
                    />
                  </Grid>
                  <Grid item xs={6}>
                    <TextField
                      name="shippingAddressStreet"
                      label="Rue"
                      fullWidth
                      onChange={shippingAddressHandleInputChange}
                      type="text"
                      value={shippingAddress?.street}
                      error={!!errors.shippingAddressStreet}
                      helperText={errors.shippingAddressStreet}
                      InputLabelProps={{
                        shrink: true,
                      }}
                    />
                  </Grid>
                  <Grid item xs={6}>
                    <TextField
                      name="shippingAddressHome"
                      label="Maison"
                      fullWidth
                      onChange={shippingAddressHandleInputChange}
                      type="text"
                      value={shippingAddress?.home}
                      error={!!errors.shippingAddressHome}
                      helperText={errors.shippingAddressHome}
                      InputLabelProps={{
                        shrink: true,
                      }}
                    />
                  </Grid>
                  <Grid item xs={6}>
                    <TextField
                      name="shippingAddressCity"
                      label="Ville"
                      fullWidth
                      onChange={shippingAddressHandleInputChange}
                      type="text"
                      value={shippingAddress?.city}
                      error={!!errors.shippingAddressCity}
                      helperText={errors.shippingAddressCity}
                      InputLabelProps={{
                        shrink: true,
                      }}
                    />
                  </Grid>
                  <Grid item xs={6}>
                    <TextField
                      name="shippingAddressZipCode"
                      label="Code postal"
                      fullWidth
                      onChange={shippingAddressHandleInputChange}
                      type="text"
                      value={shippingAddress?.zipCode}
                      error={!!errors.shippingAddressZipCode}
                      helperText={errors.shippingAddressZipCode}
                      InputLabelProps={{
                        shrink: true,
                      }}
                    />
                  </Grid>
                  <Grid item xs={6}>
                    <Autocomplete
                      options={countries.map((country) => country.code)}
                      getOptionLabel={(option) =>
                        countries.find((country) => country.code === option)
                          ?.label ?? ""
                      }
                      value={
                        selectedShippingCountry ||
                        shippingAddress?.country ||
                        ""
                      }
                      onChange={(event, newValue) => {
                        shippingAddressHandleInputChange({
                          target: {
                            name: "shippingAddressCountry",
                            value: newValue || "",
                          },
                        } as React.ChangeEvent<HTMLInputElement | HTMLTextAreaElement>);

                        setSelectedShippingCountry(newValue || "");
                      }}
                      renderInput={(params) => (
                        <TextField
                          {...params}
                          label="Pays"
                          error={!!errors.shippingAddressCountry}
                          helperText={errors.shippingAddressCountry}
                          InputLabelProps={{
                            shrink: true,
                          }}
                        />
                      )}
                      isOptionEqualToValue={(option, value) => option === value}
                    />
                  </Grid>

                  <Grid item xs={12}>
                    <LoadingButton
                      onClick={handleSubmitShipping}
                      variant="contained"
                      fullWidth
                      loading={shippingLoading}
                    >
                     Soumettre
                    </LoadingButton>
                  </Grid>
                </>
              ) : (
                <Grid item xs={12}>
                  <Typography>Veuillez sélectionner un client.</Typography>
                </Grid>
              )}
            </Grid>
          </Grid>
        </Grid>
      </Stack>

      <Snackbar
        open={snackbarOpen}
        autoHideDuration={6000}
        onClose={handleSnackbarClose}
        anchorOrigin={{ vertical: "top", horizontal: "right" }}
      >
        <Alert onClose={handleSnackbarClose} severity={snackbarSeverity}>
          {snackbarMessage}
        </Alert>
      </Snackbar>
    </>
  );
};

export default AddressDetailsCard;
