import Box from "@mui/material/Box";
import Card from "@mui/material/Card";
import Stack from "@mui/material/Stack";
import Divider from "@mui/material/Divider";
import CardHeader from "@mui/material/CardHeader";
import Typography from "@mui/material/Typography";
import { CardContent, Chip, Grid, Pagination, Table, TableBody, TableCell, TableContainer, TableRow } from "@mui/material";
import CircularProgress from "@mui/material/CircularProgress";
import { OrderStatusHistoryDTO, getStatusInFrench, OrderStatus } from "../../../types/order";
import { findUserById } from "@/shared/api/user";
import { useCallback, useEffect, useState } from "react";
import { noteTypeLabels, Order } from "@/contexts/types/main-order";
import { IClientItem } from "@/shared/types/user";
import { ShippingMethodType } from "@/contexts/types/Shipping";
import { getOrderHistory } from "@/shared/api/order";

type Props = {
  order: Order;
};

export default function OrderDetailsInfo({ order }: Props) {
  const [history, setHistory] = useState<OrderStatusHistoryDTO[]>([]);
  const [error, setError] = useState<string | null>(null);
  const [currentPage, setCurrentPage] = useState(1);
  const [pageSize] = useState(2);
  const [numberOfElements, setNumberOfElements] = useState(0);
  const [loading, setLoading] = useState<boolean>(true);

  useEffect(() => {
    const fetchOrderHistory = async () => {
      try {
        setLoading(true);
        const data = await getOrderHistory(
          Number(order?.id),
          pageSize,
          currentPage - 1
        );
        setNumberOfElements(data.totalElements);
        setHistory(data.content);
      } catch (err: any) {
        setError("Failed to fetch order history.");
      } finally {
        setLoading(false);
      }
    };

    fetchOrderHistory();
  }, [order?.id, currentPage]);

  const [clientData, setClientData] = useState<IClientItem | null>(null);

  const fetchUser = useCallback(async () => {
    try {
      const response = await findUserById(Number(order?.clientId));
      setClientData(response.data);
    } catch (error) {
      console.error("Erreur lors de la récupération de l'utilisateur :", error);
    }
  }, [order?.clientId]);
  useEffect(() => {
    fetchUser();
  }, [fetchUser]);

  function getShippingMethodNameInFrench(shippingMethod: string): string {
    switch (shippingMethod) {
      case ShippingMethodType.PointOfSale:
        return "Point de vente";
      case ShippingMethodType.FlatRate:
        return "Forfait";
      case ShippingMethodType.FreeShipping:
        return "Livraison gratuite";
      case ShippingMethodType.ColissimoWithSignature:
        return "Colissimo avec signature";
      case ShippingMethodType.ColissimoWithoutSignature:
        return "Colissimo sans signature";
      case ShippingMethodType.MondialRelay:
        return "Mondial Relay - InPost";
      default:
        return "Méthode de livraison inconnue";
    }
  }

  const formatDate = (date: Date) => {
    return (
      new Date(date).toLocaleDateString("fr-FR", {
        year: "2-digit",
        month: "numeric",
        day: "numeric",
      }) +
      " " +
      new Date(date).toLocaleTimeString("fr-FR")
    );
  };

  const renderHistory = (
    <Grid className="no-print">
      <CardHeader title="Historique des modifications" />
      {error ? (
        <Typography color="error">{error}</Typography>
      ) : loading ? (
        <Box sx={{ display: "flex", justifyContent: "center", my: 3 }}>
          <CircularProgress />
        </Box>
      ) : (
        <>
          <TableContainer sx={{ maxHeight: 400 }}>
            <Table stickyHeader>
              <TableBody>
                <TableRow>
                  <TableCell sx={{ width: "25%", fontSize: "0.85rem" }}>
                    <strong>Date</strong>
                  </TableCell>

                  <TableCell sx={{ width: "35%", fontSize: "0.85rem" }}>
                    <strong>Ancien Statut</strong>
                  </TableCell>

                  <TableCell sx={{ width: "35%", fontSize: "0.85rem" }}>
                    <strong> Nouveau statut</strong>
                  </TableCell>

                  <TableCell sx={{ width: "25%", fontSize: "0.85rem" }}>
                    <strong>Changé par</strong>
                  </TableCell>
                </TableRow>
                {history?.length > 0 ? (
                  history?.map((entry, index) => (
                    <TableRow key={index}>
                      <TableCell
                        sx={{
                          width: "25%",
                          fontSize: "0.85rem",
                          overflow: "hidden",
                          textOverflow: "ellipsis",
                          whiteSpace: "nowrap",
                        }}
                      >
                        {formatDate(entry?.createdAt)}
                      </TableCell>
                      <TableCell
                        sx={{
                          width: "35%",
                          fontSize: "0.85rem",
                          fontWeight: "bold",
                          overflow: "hidden",
                          textOverflow: "ellipsis",
                          whiteSpace: "nowrap",
                        }}
                      >
                        {getStatusInFrench(
                          entry?.oldOrderStatus as OrderStatus
                        )}
                      </TableCell>
                      <TableCell
                        sx={{
                          width: "35%",
                          fontSize: "0.85rem",
                          fontWeight: "bold",
                          overflow: "hidden",
                          textOverflow: "ellipsis",
                          whiteSpace: "nowrap",
                        }}
                      >
                        {getStatusInFrench(
                          entry?.newOrderStatus as OrderStatus
                        )}
                      </TableCell>
                      <TableCell
                        sx={{
                          width: "30%",
                          fontSize: "0.85rem",
                          fontWeight: "bold",
                          overflow: "hidden",
                          textOverflow: "ellipsis",
                          whiteSpace: "nowrap",
                        }}
                      >
                        {entry.userFullName}
                      </TableCell>
                    </TableRow>
                  ))
                ) : (
                  <TableRow>
                    <TableCell colSpan={3}>
                      <Typography align="center">
                        Aucun historique trouvé.
                      </Typography>
                    </TableCell>
                  </TableRow>
                )}
              </TableBody>
            </Table>
          </TableContainer>

          <Pagination
            count={Math.max(1, Math.ceil(numberOfElements / pageSize))}
            shape="rounded"
            page={currentPage}
            onChange={(_, value) => setCurrentPage(value)}
            size="small"
            disabled={Math.ceil(numberOfElements / pageSize) === 0}
            sx={{
              mt: 2,
              display: "flex",
              justifyContent: "center",
              "& .MuiPaginationItem-root": {
                fontSize: "1rem",
                padding: "4px 8px",
                mb: "4px",
              },
              "& .Mui-disabled": {
                opacity: 0.5,
              },
            }}
          />
        </>
      )}
    </Grid>
  );

  const [page, setPage] = useState<number>(1);
  const notesPerPage = 2;

  const handlePageChange = (_: React.ChangeEvent<unknown>, value: number) => {
    setPage(value);
  };

  const sortedNotes = order?.notes
    ? order.notes.sort(
      (a, b) =>
        new Date(b.createdAt).getTime() - new Date(a.createdAt).getTime()
    )
    : [];

  const paginatedNotes = sortedNotes.slice(
    (page - 1) * notesPerPage,
    page * notesPerPage
  );
  const totalPages = Math.ceil(sortedNotes.length / notesPerPage);
  const renderNote = (
    <Grid className="no-print">
      <CardHeader title="Notes de la commande" />
      {order?.notes && order.notes.length > 0 ? (
        <CardContent>
          <Stack spacing={2}>
            {paginatedNotes.map((note) => (
              <Box
                key={note.id}
                sx={{
                  border: 1,
                  borderColor: "divider",
                  borderRadius: 2,
                  p: 2,
                }}
              >
                <Stack spacing={1}>
                  <Chip
                    label={`Note #${note.id}`}
                    color="primary"
                    size="small"
                    sx={{ alignSelf: "flex-start" }}
                  />
                  <Typography variant="body1" color="text.primary">
                    <strong>Note:</strong> {note.content}
                  </Typography>
                  <Typography variant="body2" color="text.secondary">
                    <strong>Type:</strong> {noteTypeLabels[note.noteType]}
                  </Typography>
                </Stack>
              </Box>
            ))}
          </Stack>
        </CardContent>
      ) : (
        <Box
          sx={{
            p: 3,
            typography: "body2",
            textAlign: "center",
          }}
        >
          <Typography align="center">Cette commande n'a aucune note</Typography>
        </Box>
      )}
      <Pagination
        count={Math.max(1, Math.ceil(sortedNotes.length / notesPerPage))}
        page={page}
        size="small"
        shape="rounded"
        onChange={handlePageChange}
        disabled={totalPages === 0}
        getItemAriaLabel={(type, page) =>
          type === "page"
            ? `PageNote ${page}`
            : type === "next"
              ? "PageNote suivante"
              : type === "previous"
                ? "PageNote précédente"
                : ""
        }
        sx={{
          mt: 2,
          display: "flex",
          justifyContent: "center",
          "& .MuiPaginationItem-root": {
            fontSize: "1rem",
            padding: "4px 8px",
            mb: "4px",
          },
          "& .Mui-disabled": {
            opacity: 0.5,
          },
        }}
      />
    </Grid>
  );

  const renderCustomer = (
    <>
      <CardHeader title="Client" />
      <Stack
        spacing={1.5}
        sx={{ p: 3, typography: "body2" }}
        alignItems="flex-start"
      >
        <Stack direction="row">
          <Box
            component="span"
            sx={{ color: "text.secondary", width: 120, flexShrink: 0 }}
          >
            Nom complet:
          </Box>
          {clientData?.firstName + " " + clientData?.lastName}
        </Stack>
        <Stack direction="row">
          <Box
            component="span"
            sx={{ color: "text.secondary", width: 120, flexShrink: 0 }}
          >
            E-mail:
          </Box>
          <Box>{clientData?.email}</Box>
        </Stack>
        <Stack direction="row">
          <Box
            component="span"
            sx={{ color: "text.secondary", width: 120, flexShrink: 0 }}
          >
            Téléphone:
          </Box>
          <Box>{clientData?.phoneNumber}</Box>
        </Stack>
      </Stack>
    </>
  );

  const renderBilling = (
    <Grid className="no-print">
      <CardHeader title="Adresse de Facturation" />
      {order?.billingAddress ? (
        <Stack spacing={1.5} sx={{ p: 3, typography: "body2" }}>
          <Stack direction="row">
            <Box
              component="span"
              sx={{ color: "text.secondary", width: 120, flexShrink: 0 }}
            >
              Adresse
            </Box>
            {`${order.billingAddress.street || ""} ${order.billingAddress.home || ""
              }`}
          </Stack>
          <Stack direction="row">
            <Box
              component="span"
              sx={{ color: "text.secondary", width: 120, flexShrink: 0 }}
            >
              Ville
            </Box>
            {order.billingAddress.city || ""}
          </Stack>
          <Stack direction="row" alignItems="center">
            <Box
              component="span"
              sx={{ color: "text.secondary", width: 120, flexShrink: 0 }}
            >
              Code Postal
            </Box>
            {order.billingAddress.zipCode || ""}
          </Stack>
          <Stack direction="row" alignItems="center">
            <Box
              component="span"
              sx={{ color: "text.secondary", width: 120, flexShrink: 0 }}
            >
              Pays
            </Box>
            {order.billingAddress.country || ""}
          </Stack>
        </Stack>
      ) : (
        <Box sx={{ p: 3, typography: "body2", color: "text.secondary" }}>
          Commande n'a pas d'adresse de facturation
        </Box>
      )}
    </Grid>
  );

  const renderShipping = (
    <Grid className="no-print">
      <CardHeader title="Adresse de Livraison" />
      {order?.shippingAddress ? (
        <Stack spacing={1.5} sx={{ p: 3, typography: "body2" }}>
          <Stack direction="row">
            <Box
              component="span"
              sx={{ color: "text.secondary", width: 120, flexShrink: 0 }}
            >
              Adresse
            </Box>
            {`${order.shippingAddress.street || ""} ${order.shippingAddress.home || ""
              }`}
          </Stack>
          <Stack direction="row">
            <Box
              component="span"
              sx={{ color: "text.secondary", width: 120, flexShrink: 0 }}
            >
              Ville
            </Box>
            {order.shippingAddress.city || ""}
          </Stack>
          <Stack direction="row" alignItems="center">
            <Box
              component="span"
              sx={{ color: "text.secondary", width: 120, flexShrink: 0 }}
            >
              Code Postal
            </Box>
            {order.shippingAddress.zipCode || ""}
          </Stack>
          <Stack direction="row" alignItems="center">
            <Box
              component="span"
              sx={{ color: "text.secondary", width: 120, flexShrink: 0 }}
            >
              Pays
            </Box>
            {order.shippingAddress.country || ""}
          </Stack>
        </Stack>
      ) : (
        <Box sx={{ p: 3, typography: "body2", color: "text.secondary" }}>
          Commande n'a pas d'adresse de livraison
        </Box>
      )}

      <Stack spacing={1.5} sx={{ p: 3, typography: "body2" }} className="no-print">
        <Stack direction="row">
          <Box
            component="span"
            sx={{ color: "text.secondary", width: 118, flexShrink: 0 }}
          >
            Mode d’expédition
          </Box>
          {getShippingMethodNameInFrench(order.shippingMethodType) || ""}
        </Stack>

        <Typography
          variant="subtitle1"
          sx={{
            fontWeight: "bold",
            textTransform: "uppercase",
            color: "primary.main",
            mt: 2,
            mb: 1,
          }}
        >
          Point de retrait du Mondial Relay
        </Typography>

        <Stack direction="row">
          <Box
            component="span"
            sx={{ color: "text.secondary", width: 118, flexShrink: 0 }}
          >
            Nom
          </Box>
          {order?.relayPointName}
        </Stack>
        <Stack direction="row">
          <Box
            component="span"
            sx={{ color: "text.secondary", width: 118, flexShrink: 0 }}
          >
            Addresse 1
          </Box>
          {order?.relayPointAddress1}
        </Stack>
        <Stack direction="row">
          <Box
            component="span"
            sx={{ color: "text.secondary", width: 118, flexShrink: 0 }}
          >
            Addresse 2
          </Box>
          {order?.relayPointAddress2}
        </Stack>
        <Stack direction="row">
          <Box
            component="span"
            sx={{ color: "text.secondary", width: 118, flexShrink: 0 }}
          >
            Code postal
          </Box>
          {order?.relayPointZipCode}
        </Stack>
        <Stack direction="row">
          <Box
            component="span"
            sx={{ color: "text.secondary", width: 118, flexShrink: 0 }}
          >
            Ville
          </Box>
          {order?.relayPointCity}
        </Stack>
        <Stack direction="row">
          <Box
            component="span"
            sx={{ color: "text.secondary", width: 118, flexShrink: 0 }}
          >
            Pays
          </Box>
          {order?.relayPointCountry}
        </Stack>
      </Stack>
    </Grid>
  );

  return (
    <Card>
      {renderHistory}

      <Divider sx={{ borderStyle: "dashed" }} />
      {renderCustomer}

      <Divider sx={{ borderStyle: "dashed" }} />
      {renderShipping}

      <Divider sx={{ borderStyle: "dashed" }} />
      {renderBilling}

      <Divider sx={{ borderStyle: "dashed" }} />
      {renderNote}
    </Card>
  );
}
