import Stack from "@mui/material/Stack";
import Button from "@mui/material/Button";
import MenuItem from "@mui/material/MenuItem";
import IconButton from "@mui/material/IconButton";
import Typography from "@mui/material/Typography";
import { fDateTime } from "../../../../utils/format-time";
import Label from "../../../components/label";
import Iconify from "../../../components/iconify";
import CustomPopover, { usePopover } from "../../../components/custom-popover";
import RouterLink from "@/routes/router-link";
import {
  Order,
  OrderStatus,
  orderStatusLabels,
  OrderStatusList,
} from "@/contexts/types/main-order";
import { useCallback, useEffect, useState } from "react";
import { getAllOrders, updateOrderStatus } from "@/shared/api/main-order";
import { Alert, Card, Snackbar } from "@mui/material";
import { getPaymentMethodNameInFrench } from "@/shared/types/payment";
import { paths } from "@/routes/paths";

type Props = {
  order: Order;
  onMutate: () => void;
  printRef?: React.RefObject<HTMLDivElement>;
};

export default function OrderDetailsToolbar({
  order,
  onMutate,
  printRef,
}: Props) {
  const popover = usePopover();

  const [userId, setUserId] = useState("");

  useEffect(() => {
    fetchUserData();
  }, [order]);

  const fetchUserData = async () => {
    const userEmail = localStorage.getItem("email");
    const userId = localStorage.getItem("userId");
    if (userEmail && userId) setUserId(userId);
  };

  const statusOptions: { value: OrderStatus; label: string }[] =
    OrderStatusList.map((status: OrderStatus) => ({
      value: status,
      label: orderStatusLabels[status],
    }));

  const [status, setStatus] = useState(order?.status);

  const [confirmedStatus, setConfirmedStatus] = useState<OrderStatus>(
    order?.status
  );

  const handleChangeStatus = useCallback((newValue: OrderStatus) => {
    setConfirmedStatus(newValue);
  }, []);

  useEffect(() => {
    setStatus(order?.status);
  }, [order]);

  const handleSubmitStatus = async () => {
    try {
      await updateOrderStatus(order.id, userId, confirmedStatus)
        .then(() => {
          setConfirmedStatus(confirmedStatus);
          setStatus(confirmedStatus);
          onMutate();
          setOpenSaveAlert(true);
        })
        .catch((error) => {
          setOpenErrorAlert(true);
        });
    } catch (error) {
      console.error("Erreur lors de la mise à jour du statut", error);
      alert("Une erreur est survenue lors de la mise à jour.");
    }
  };

  const [openSaveAlert, setOpenSaveAlert] = useState(false);
  const handleCloseSaveAlert = (
    event?: React.SyntheticEvent | Event,
    reason?: string
  ) => {
    if (reason === "clickaway") {
      return;
    }
    setOpenSaveAlert(false);
  };

  const [openErrorAlert, setOpenErrorAlert] = useState(false);
  const handleCloseErrorAlert = (
    event?: React.SyntheticEvent | Event,
    reason?: string
  ) => {
    if (reason === "clickaway") {
      return;
    }
    setOpenErrorAlert(false);
  };

  return (
    <>
      <Stack
        spacing={3}
        direction={{ xs: "column", md: "row" }}
        sx={{
          mb: { xs: 3, md: 5 },
        }}
      >
        <Stack spacing={1} direction="row" alignItems="flex-start">
          <IconButton
            component={RouterLink}
            href={paths.dashboard.admin.order.all_orders}
          >
            <Iconify icon="eva:arrow-ios-back-fill" />
          </IconButton>

          <Stack spacing={0.5}>
            <Stack spacing={1} direction="row" alignItems="center">
              <Typography variant="h4"> Order {order?.id} </Typography>
              <Label
                variant="soft"
                color={
                  (status === OrderStatus.FINISHED && "success") ||
                  (status === OrderStatus.FAILED && "warning") ||
                  (status === OrderStatus.ON_HOLD && "info") ||
                  (status === OrderStatus.IN_PROGRESS && "primary") ||
                  (status === OrderStatus.CANCELED && "error") ||
                  (status === OrderStatus.REFUNDED && "info") ||
                  (status === OrderStatus.DELETED && "error") ||
                  "default"
                }
              >
                {orderStatusLabels[status]}
              </Label>
            </Stack>

            <Typography variant="body2" sx={{ color: "text.disabled" }}>
              {fDateTime(order?.createdAt)}
            </Typography>
          </Stack>
        </Stack>

        <Stack
          flexGrow={1}
          spacing={1.5}
          direction="row"
          alignItems="center"
          justifyContent="flex-end"
        >
          <Button
            color="inherit"
            variant="outlined"
            endIcon={<Iconify icon="eva:arrow-ios-downward-fill" />}
            onClick={popover.onOpen}
            sx={{ textTransform: "capitalize" }}
            className="no-print"
          >
            {orderStatusLabels[confirmedStatus]}
          </Button>
          <Button
            color="inherit"
            variant="contained"
            disabled={status == OrderStatus.DELETED}
            onClick={handleSubmitStatus}
            className="no-print"
          >
            Appliqué
          </Button>
        </Stack>
      </Stack>

      <Card sx={{ minHeight: "60px", marginBottom: 2 }}>
        <Stack
          direction="row"
          spacing={1}
          alignItems="center"
          justifyContent="flex-start"
          sx={{ marginLeft: 2 }}
        >
          <Typography variant="body2" sx={{ marginTop: 2 }} fontWeight="bold">
            Paiement via:
          </Typography>
          <Typography variant="body2" sx={{ marginTop: 2 }}>
            {getPaymentMethodNameInFrench(order?.paymentMethodType)}
          </Typography>
          <Typography
            variant="body2"
            sx={{ marginTop: 2, marginLeft: 15 }}
            fontWeight="bold"
          >
            Date de création:
          </Typography>
          <Typography variant="body2" sx={{ marginTop: 2 }}>
            {fDateTime(order?.createdAt)}
          </Typography>
        </Stack>
      </Card>

      <CustomPopover
        open={popover.open}
        onClose={popover.onClose}
        arrow="top-right"
        defaultValue={status}
        sx={{ minWidth: "auto", maxWidth: "100%" }}
      >
        {statusOptions?.map((option) => (
          <MenuItem
            key={option?.value}
            selected={option?.value === status}
            onClick={() => {
              popover.onClose();
              handleChangeStatus(option?.value);
            }}
          >
            {option?.label}
          </MenuItem>
        ))}
      </CustomPopover>
      <Snackbar
        open={openSaveAlert}
        autoHideDuration={3000}
        onClose={handleCloseSaveAlert}
        anchorOrigin={{ vertical: "top", horizontal: "right" }}
      >
        <Alert
          onClose={handleCloseSaveAlert}
          severity="success"
          variant="filled"
          sx={{ width: "100%" }}
        >
          Status modifié avec succès
        </Alert>
      </Snackbar>
      <Snackbar
        open={openErrorAlert}
        autoHideDuration={3000}
        onClose={handleCloseErrorAlert}
        anchorOrigin={{ vertical: "top", horizontal: "right" }}
      >
        <Alert
          onClose={handleCloseErrorAlert}
          severity="error"
          variant="filled"
          sx={{ width: "100%" }}
        >
          Échec
        </Alert>
      </Snackbar>
    </>
  );
}
