import React, { useState } from "react";
import {
  Card,
  Grid,
  Stack,
  TextField,
  IconButton,
} from "@mui/material";
import Autocomplete from "@mui/material/Autocomplete";
import AddIcon from "@mui/icons-material/Add";
import DeleteIcon from "@mui/icons-material/Delete";
import LoadingButton from "@mui/lab/LoadingButton";
import { searchProductWithIndex } from "@/shared/api/product";
import { ProductWithCustomResponse } from "@/shared/types/product";

export interface Entry {
  product: ProductWithCustomResponse | null;
  quantity?: number;
}

interface ProductFormProps {
  onSubmit: (entries: Entry[]) => void;
  initialEntries?: Entry[];
}

export default function ProductForm({
  onSubmit,
  initialEntries = [],
}: ProductFormProps) {
  const [entries, setEntries] = useState<Entry[]>(
    initialEntries.length > 0
      ? initialEntries
      : [{ product: null, quantity: 1 }]
  );
  const [productOptions, setProductOptions] = useState<
    ProductWithCustomResponse[]
  >([]);
  const [loading, setLoading] = useState<boolean>(false);

  const fetchProducts = async (searchKey: string) => {
    if (!searchKey) return;
    setLoading(true);
    try {
      const products = await searchProductWithIndex(searchKey);
      setProductOptions(products);
    } catch (error) {
      console.error("Erreur lors du chargement des produits", error);
    } finally {
      setLoading(false);
    }
  };

  const handleAddEntry = () => {
    setEntries([...entries, { product: null, quantity: 1 }]);
    setProductOptions([]);
  };

  const handleRemoveEntry = (index: number) => {
    const newEntries = [...entries];
    newEntries.splice(index, 1);
    setEntries(newEntries);
  };

  const handleInputChange = (
    index: number,
    field: keyof Entry,
    value: string | number | ProductWithCustomResponse | null
  ) => {
    const newEntries = [...entries];
    newEntries[index][field] = value as never;
    setEntries(newEntries);
  };

  const handleProductChange = (
    index: number,
    selectedProduct: ProductWithCustomResponse | null
  ) => {
    handleInputChange(index, "product", selectedProduct);
    setProductOptions([]);
  };


  const handleSubmit = async () => {
    try {
      const filteredEntries = entries.filter(
        (entry) => entry.product !== null
      );

      onSubmit(filteredEntries);
    } catch (error) {
      console.error("Erreur lors de la soumission", error);
    }
  };


  return (
    <>
      <Card>
        <Stack spacing={3} sx={{ p: 3 }}>
          {entries.map((entry, index) => (
            <Grid container spacing={2} alignItems="center" key={index}>
              <Grid item xs={6}>
                <Autocomplete
                  value={entry.product}
                  onInputChange={(event, newInputValue) => {
                    fetchProducts(newInputValue);
                  }}
                  onChange={(event, newValue) =>
                    handleProductChange(index, newValue)
                  }
                  options={productOptions}
                  getOptionLabel={(option) =>
                    `${option.productName} - ${option.ugs}`
                  }
                  fullWidth
                  loading={loading}
                  renderInput={(params) => (
                    <TextField {...params} label="Produit" fullWidth />
                  )}
                />
              </Grid>
              <Grid item xs={3}>
                <TextField
                  value={entry.quantity || ""}
                  onChange={(event) =>
                    handleInputChange(
                      index,
                      "quantity",
                      Number(event.target.value)
                    )
                  }
                  fullWidth
                  label="Quantité"
                  type="number"
                  InputLabelProps={{
                        shrink: true,
                      }}
                />
              </Grid>
              <Grid item xs={2} container justifyContent="flex-end">
                <IconButton onClick={handleAddEntry}>
                  <AddIcon />
                </IconButton>
              </Grid>
              {(index > 0 || entries.length>1) && (
                <Grid item xs={1} container justifyContent="flex-end">
                  <IconButton onClick={() => handleRemoveEntry(index)}>
                    <DeleteIcon />
                  </IconButton>
                </Grid>
              )}
            </Grid>
          ))}
        </Stack>
        <LoadingButton
          type="button"
          variant="contained"
          fullWidth
          onClick={handleSubmit}
        >
          Soumettre
        </LoadingButton>
      </Card>
    </>
  );
}
