"use client";

import { useState, useCallback, useEffect } from "react";
import Tab from "@mui/material/Tab";
import Tabs from "@mui/material/Tabs";
import Card from "@mui/material/Card";
import Table from "@mui/material/Table";
import Stack from "@mui/material/Stack";
import Container from "@mui/material/Container";
import TableBody from "@mui/material/TableBody";
import { alpha, useTheme } from "@mui/material/styles";
import TableContainer from "@mui/material/TableContainer";
import { paths } from "@/routes/paths";
import { isAfter } from "@/utils/format-time";
import Label from "@/shared/components/label";
import Scrollbar from "@/components/scrollbar";
import { useSettingsContext } from "@/components/settings";
import CustomBreadcrumbs from "@/components/custom-breadcrumbs";
import {
  useTable,
  emptyRows,
  TableNoData,
  TableEmptyRows,
  TableHeadCustom,
  TableSelectedAction,
} from "@/shared/components/table";
import { CircularProgress } from '@mui/material';
import { getRefundMethodColor, IRefundOrderTableFilters, IRefundOrderTableFilterValue, RefundMethod, refundMethodLabels, RefundOrderItem } from '@/contexts/types/refundOrder';
import RefundOrderTableToolbar from './refund-order-table-toolbar';
import RefundOrderTableRow from './refund-order-table-row';
import { useGetAllRefundOrders } from '@/shared/api/refundOrders';
import RefundOrderTableFiltersResult from './refund-order-table-filters-result';
import CustomPagination from '../../order-components/CustomPagination';

// ----------------------------------------------------------------------

const TABLE_HEAD = [
  { id: "id", label: "N°" },
  { id: "orderId", label: "N° de commande" },
  { id: "clientName", label: "Client" },
  { id: "userName", label: "Utilisateur" },
  { id: "createdAt", label: "Création" },
  { id: "totalAmountToRefund", label: "Montant" },
  { id: "totalQuantityRefund", label: "Quantité", align: "center" },
  { id: "refundMethod", label: "Méthode" },
  { id: "refundReason", label: "Raison" },
];

const defaultFilters: IRefundOrderTableFilters = {
  keyword: "",
  refundMethods: [],
  startDate: null,
  endDate: null,
  page: 0,
  pageSize: 10,
};

// ----------------------------------------------------------------------

export default function RefundOrderListView() {
  const theme = useTheme();
  
  const settings = useSettingsContext();
  
  const table = useTable({ defaultOrderBy: "id", defaultOrder: "desc" });
  
  const [tableData, setTableData] = useState<RefundOrderItem[]>([]);
  const defaultFiltersLocalStorage: IRefundOrderTableFilters = localStorage.getItem("lastRefundListFilters")
    ?
    JSON.parse(localStorage.getItem("lastRefundListFilters")!)
    :
    {
      keyword: "",
      refundMethods: [],
      startDate: null,
      endDate: null,
      page: 0,
      pageSize: 10,
    };
    const getFilterFromLocalStorage = (): IRefundOrderTableFilters => {
      try {
        const savedFilters = localStorage.getItem("lastRefundListFilters");
        if (!savedFilters) return defaultFilters;
        
        const parsed = JSON.parse(savedFilters);
        
        return {
          ...parsed,
          startDate: parsed.startDate ? new Date(parsed.startDate) : null,
          endDate: parsed.endDate ? new Date(parsed.endDate) : null
        };
      } catch (error) {
        console.error("Error loading filters:", error);
        return defaultFilters;
      }
    };

  const [filters, setFilters] = useState(getFilterFromLocalStorage());

  const dateError = isAfter(filters.startDate, filters.endDate);

  const [loading, setLoading] = useState(false);

  const [page, setPage] = useState(0);
  const [rowsPerPage, setRowsPerPage] = useState(10);
  const [totalPages, setTotalPages] = useState(0);
  
  useEffect(() => {
    localStorage.removeItem("lastRefundListFilters");
    localStorage.setItem("lastRefundListFilters", JSON.stringify(filters));
  }, [filters]);


  useGetAllRefundOrders(filters.refundMethods, filters.keyword, filters.startDate, filters.endDate, page, rowsPerPage).then((response) => {
    setTableData(response.refundOrdersResponse);
    setTotalPages(response.totalPages);
  });



  const denseHeight = table.dense ? 56 : 56 + 20;

  const canReset =
    !!filters.keyword ||
    !!filters.refundMethods.length ||
    (!!filters.startDate && !!filters.endDate);

  const notFound = (!tableData.length && canReset) || !tableData.length;

  const getRefundOrdersLength = (refundMethod: RefundMethod) =>
    tableData.filter((item) => item.refundMethod === refundMethod).length;

  const TABS = [
    { value: [], label: "Tous", color: "default", count: tableData.length },
    {
      value: [RefundMethod.CREDIT_CARD],
      label: refundMethodLabels[RefundMethod.CREDIT_CARD],
      color: getRefundMethodColor(RefundMethod.CREDIT_CARD),
      count: getRefundOrdersLength(RefundMethod.CREDIT_CARD),
    },
    {
      value: [RefundMethod.PAYPAL],
      label: refundMethodLabels[RefundMethod.PAYPAL],
      color: getRefundMethodColor(RefundMethod.PAYPAL),
      count: getRefundOrdersLength(RefundMethod.PAYPAL),
    },
    {
      value: [RefundMethod.MANUAL],
      label: refundMethodLabels[RefundMethod.MANUAL],
      color: getRefundMethodColor(RefundMethod.MANUAL),
      count: getRefundOrdersLength(RefundMethod.MANUAL),
    },
  ] as const;

  const handleFilters = useCallback(
    (name: string, value: IRefundOrderTableFilterValue) => {
      table.onResetPage();
      setPage(0);
      setFilters((prevState) => ({
        ...prevState,
        [name]: value,
      }));
    },
    []
  );

  const handleResetFilters = useCallback(() => {
    setFilters(defaultFilters);
    setPage(0);
  }, []);

  const handleFilterRefundMethod = useCallback(
    (event: React.SyntheticEvent, newValue: string) => {
      handleFilters("refundMethods", newValue);
      setPage(0);
    },
    [handleFilters]
  );

  function arraysEqual(a: any[], b: any[]) {
    if (a.length !== b.length) return false;
    return a.every((value, index) => value === b[index]);
  }

  return (
    <>
      <Container maxWidth={settings.themeStretch ? false : "lg"}>
        <CustomBreadcrumbs
          heading="Liste des remboursements"
          links={[
            {
              name: "Tableau de bord",
              href: paths.dashboard.root,
            },
            {
              name: "Liste des remboursements",
              href: paths.dashboard.admin.order.refund.root,
            }
          ]}
          sx={{
            mb: { xs: 3, md: 5 },
          }}
        />

        <Card>
          <Tabs
            value={filters.refundMethods}
            onChange={handleFilterRefundMethod}
            sx={{
              px: 2.5,
              boxShadow: `inset 0 -2px 0 0 ${alpha(
                theme.palette.grey[500],
                0.08
              )}`,
            }}
          >
            {TABS.map((tab, index) => (
              <Tab
                key={index}
                value={tab.value}
                label={tab.label}
                iconPosition="end"
                icon={
                  <Label
                    variant={
                      (arraysEqual(tab.value.slice(), filters.refundMethods.slice()) || arraysEqual(tab.value.slice(), [])) ? "filled" : "soft"
                    }
                    color={tab.color}
                  >
                    {tab.count}
                  </Label>
                }
              />
            ))}
          </Tabs>

          <RefundOrderTableToolbar
            filters={filters}
            onFilters={handleFilters}
            dateError={dateError}
          />

          {canReset && (
            <RefundOrderTableFiltersResult
              filters={filters}
              onFilters={handleFilters}
              //
              onResetFilters={handleResetFilters}
              //
              results={tableData.length}
              sx={{ p: 2.5, pt: 0 }}
            />
          )}

          <TableContainer sx={{ position: "relative", overflow: "unset" }}>
            <TableSelectedAction
              dense={table.dense}
              numSelected={table.selected.length}
              rowCount={tableData.length}
              onSelectAllRows={(checked) => {
                table.onSelectAllRows(
                  checked,
                  tableData.map((row) => row.id.toString())
                );
              }}
            />
            {
              loading ? (
                <Stack alignItems="center" justifyContent="center" sx={{ py: 5 }}>
                  <CircularProgress />
                </Stack>
              ) : (
                <Scrollbar>
                  <Table
                    size={table.dense ? "small" : "medium"}
                    sx={{ minWidth: 800 }}
                  >
                    <TableHeadCustom
                      order={table.order}
                      orderBy={table.orderBy}
                      headLabel={TABLE_HEAD}
                      rowCount={tableData.length}
                      numSelected={table.selected.length}
                      onSort={table.onSort}
                      onSelectAllRows={(checked) =>
                        table.onSelectAllRows(
                          checked,
                          tableData.map((row) => row.id.toString())
                        )
                      }
                    />

                    <TableBody>
                      {tableData
                        .map((row) => (
                          <RefundOrderTableRow
                            key={row.id}
                            row={row}
                            selected={table.selected.includes(row.id.toString())}
                            onSelectRow={() => table.onSelectRow(row.id.toString())}
                          />
                        ))}

                      <TableEmptyRows
                        height={denseHeight}
                        emptyRows={emptyRows(
                          table.page,
                          table.rowsPerPage,
                          tableData.length
                        )}
                      />

                      <TableNoData notFound={notFound} />
                    </TableBody>
                  </Table>
                </Scrollbar>
              )
            }

          </TableContainer>

          <CustomPagination
            count={tableData?.length}
            page={page}
            rowsPerPage={rowsPerPage}
            setPage={setPage}
            setRowsPerPage={setRowsPerPage}
            rowsPerPageOptions={[10, 25, 50]}
            totalPages={totalPages}
          />

        </Card>
      </Container>
    </>
  );
}
