import { useCallback } from "react";
import Box from "@mui/material/Box";
import Chip from "@mui/material/Chip";
import Paper from "@mui/material/Paper";
import Button from "@mui/material/Button";
import Stack, { StackProps } from "@mui/material/Stack";
import Iconify from "@/shared/components/iconify";
import { shortDateLabel } from "@/shared/components/custom-date-range-picker";
import { IRefundOrderTableFilters, IRefundOrderTableFilterValue, refundMethodLabels } from "@/contexts/types/refundOrder";


type Props = StackProps & {
  filters: IRefundOrderTableFilters;
  onFilters: (name: string, value: IRefundOrderTableFilterValue) => void;
  onResetFilters: VoidFunction;
  results: number;
};

export default function RefundOrderTableFiltersResult({
  filters,
  onFilters,
  onResetFilters,
  results,
  ...other
}: Props) {
  const shortLabel = shortDateLabel(filters.startDate, filters.endDate);

  const handleRemoveKeyword = useCallback(() => {
    onFilters("keyword", "");
  }, [onFilters]);

  const handleRemoveRefundMethod = useCallback(() => {
    onFilters("refundMethods", []);
  }, [onFilters]);

  const handleRemoveDate = useCallback(() => {
    onFilters("startDate", null);
    onFilters("endDate", null);
  }, [onFilters]);

  return (
    <Stack spacing={1.5} {...other}>
      <Box sx={{ typography: "body2" }}>
        <strong>{results}</strong>
        <Box component="span" sx={{ color: "text.secondary", ml: 0.25 }}>
          Résultat trouvé
        </Box>
      </Box>

      <Stack
        flexGrow={1}
        spacing={1}
        direction="row"
        flexWrap="wrap"
        alignItems="center"
      >
        {filters.refundMethods.length > 0 ? (
          <Block label="Méthode de remboursement:">
            <Chip
              size="small"
              label={refundMethodLabels[filters.refundMethods.map(method => method)[0]]}
              onDelete={handleRemoveRefundMethod}
            />
          </Block>
        ) : null}

        {filters.startDate && filters.endDate ? (
          <Block label="Date:">
            <Chip size="small" label={shortLabel} onDelete={handleRemoveDate} />
          </Block>
        ) : null}

        {filters.keyword ? (
          <Block label="Mot-clé:">
            <Chip
              label={filters.keyword}
              size="small"
              onDelete={handleRemoveKeyword}
            />
          </Block>
        ) : null}

        <Button
          color="error"
          onClick={onResetFilters}
          startIcon={<Iconify icon="solar:trash-bin-trash-bold" />}
        >
          Supprimer
        </Button>
      </Stack>
    </Stack>
  );
}

// ----------------------------------------------------------------------

type BlockProps = StackProps & {
  label: string;
};

function Block({ label, children, sx, ...other }: BlockProps) {
  return (
    <Stack
      component={Paper}
      variant="outlined"
      spacing={1}
      direction="row"
      sx={{
        p: 1,
        borderRadius: 1,
        overflow: "hidden",
        borderStyle: "dashed",
        ...sx,
      }}
      {...other}
    >
      <Box component="span" sx={{ typography: "subtitle2" }}>
        {label}
      </Box>

      <Stack spacing={1} direction="row" flexWrap="wrap">
        {children}
      </Stack>
    </Stack>
  );
}
