import { useCallback, useState } from "react";
import Stack from "@mui/material/Stack";
import TextField from "@mui/material/TextField";
import InputAdornment from "@mui/material/InputAdornment";
import { formHelperTextClasses } from "@mui/material/FormHelperText";
import Iconify from "@/shared/components/iconify";
import { usePopover } from "@/shared/components/custom-popover";
import {
  IRefundOrderTableFilters,
  IRefundOrderTableFilterValue,
} from "@/contexts/types/refundOrder";
import { DateTimePicker } from "@mui/x-date-pickers";
import {
  Button,
  Dialog,
  DialogActions,
  DialogContent,
  DialogTitle,
} from "@mui/material";
import { exportRefundOrders } from "@/shared/api/refundOrders";

type Props = {
  filters: IRefundOrderTableFilters;
  onFilters: (name: string, value: IRefundOrderTableFilterValue) => void;
  dateError: boolean;
};

export default function RefundOrderTableToolbar({
  filters,
  onFilters,
  dateError,
}: Props) {
  const handleFilterKeyword = useCallback(
    (event: React.ChangeEvent<HTMLInputElement>) => {
      onFilters("keyword", event.target.value);
    },
    [onFilters]
  );

  const handleFilterStartDate = useCallback(
    (newValue: Date | null) => {
      newValue = newValue
        ? new Date(
            new Date(newValue).getTime() -
              new Date(newValue).getTimezoneOffset() * 60000
          )
        : null;
      onFilters("startDate", newValue);
    },
    [onFilters]
  );

  const handleFilterEndDate = useCallback(
    (newValue: Date | null) => {
      newValue = newValue
        ? new Date(
            new Date(newValue).getTime() -
              new Date(newValue).getTimezoneOffset() * 60000
          )
        : null;
      onFilters("endDate", newValue);
    },
    [onFilters]
  );
  const [openConfirmExport, setOpenConfirmExport] = useState(false);

  const handleExportClick = useCallback(() => {
    setOpenConfirmExport(true);
  }, []);

  const handleConfirmExport = useCallback(() => {
    setOpenConfirmExport(false);
    exportRefundOrders(
      filters.refundMethods || [],
      filters.keyword || "",
      filters.startDate || null,
      filters.endDate || null
    );
  }, [filters]);

  const handleCancelExport = useCallback(() => {
    setOpenConfirmExport(false);
  }, []);

  return (
    <>
      <Stack
        spacing={2}
        alignItems={{ xs: "flex-end", md: "center" }}
        direction={{
          xs: "column",
          md: "row",
        }}
        sx={{
          p: 2.5,
          pr: { xs: 2.5, md: 1 },
        }}
      >
        <DateTimePicker
          label="Date début"
          value={filters.startDate}
          onChange={handleFilterStartDate}
          format="yyyy/MM/dd HH:mm:ss"
          slotProps={{ textField: { fullWidth: true } }}
          sx={{
            maxWidth: { md: 180 },
          }}
        />

        <DateTimePicker
          label="Date fin"
          value={filters.endDate}
          onChange={handleFilterEndDate}
          format="yyyy/MM/dd HH:mm:ss"
          slotProps={{
            textField: {
              fullWidth: true,
              error: dateError,
              helperText:
                dateError && "Date fin doit être supérieure à la date début",
            },
          }}
          sx={{
            maxWidth: { md: 180 },
            [`& .${formHelperTextClasses.root}`]: {
              position: { md: "absolute" },
              bottom: { md: -40 },
            },
          }}
        />

        <Stack
          direction="row"
          alignItems="center"
          spacing={2}
          flexGrow={1}
          sx={{ width: 1 }}
        >
          <TextField
            fullWidth
            value={filters.keyword}
            onChange={handleFilterKeyword}
            placeholder="Chercher par numéro de commande, nom du client ou d'employé et montant remboursé."
            InputProps={{
              startAdornment: (
                <InputAdornment position="start">
                  <Iconify
                    icon="eva:search-fill"
                    sx={{ color: "text.disabled" }}
                  />
                </InputAdornment>
              ),
            }}
          />
        </Stack>

        {filters.endDate && filters.startDate ? (
          <Button
            variant="soft"
            color="primary"
            style={{ marginLeft: "auto", width: "150px", height: "50px" }}
            onClick={handleExportClick}
          >
            Exporter
          </Button>
        ) : (
          <Button
            variant="soft"
            color="primary"
            style={{ marginLeft: "auto", width: "150px", height: "50px" }}
            disabled
          >
            Exporter
          </Button>
        )}
      </Stack>

      <Dialog open={openConfirmExport} onClose={handleCancelExport}>
        <DialogTitle>Confirmer l'export</DialogTitle>
        <DialogContent>
          Voulez-vous vraiment exporter les commandes de remboursement avec les
          filtres actuels ?
          <br />
          Cela générera un fichier CSV contenant toutes les données
          correspondant à vos critères.
        </DialogContent>
        <DialogActions>
          <Button onClick={handleCancelExport} color="secondary">
            Annuler
          </Button>
          <Button
            onClick={handleConfirmExport}
            color="primary"
            variant="contained"
          >
            Exporter
          </Button>
        </DialogActions>
      </Dialog>
    </>
  );
}
