import Stack from '@mui/material/Stack';
import ListItemText from '@mui/material/ListItemText';
import { fCurrency } from '../../../../../utils/format-number';
import Scrollbar from '../../../../components/scrollbar';
import { Order, OrderLine } from '@/contexts/types/main-order';
import { RefundReason, RefundMethod, refundMethodLabels, refundReasonLabels, RefundOrderItem } from '@/contexts/types/refundOrder';
import { useEffect, useState } from 'react';
import { useGetProductsByIds, useGetVariationsByIds } from '@/shared/api/product';
import { ProductType } from '@/shared/types/product';
import { DEFAULT_IMAGE_URL } from '@/config-global';
import { CreateRefundOrderRequest, RefundOrderLineRequest } from '@/contexts/types/refundOrder';
import { useCreateRefundOrder } from '@/shared/api/refundOrders';
import { UserByEmail } from '@/shared/api/user';
import { IUser } from '@/shared/types/user';
import { Alert, Avatar, Box, Button, Card, CardHeader, Checkbox, CircularProgress, Grid, IconButton, Snackbar, TextField, Typography } from '@mui/material';
import { Add, Remove } from '@mui/icons-material';
import { PaymentMethodType } from '@/contexts/types/payment';


type Props = {
  order: Order;
  isRefund: boolean;
  handleDisableRefund: () => void;
  mutateOrder: () => void;
};

export default function OrderDetailsItemsRefund({ order, isRefund, handleDisableRefund, mutateOrder }: Props) {

  const [originalSimpleAndVariableOrderLines, setOriginalSimpleAndVariableOrderLines] = useState<OrderLine[]>([]);
  const [totalPriceOriginalSimpleAndVariableOrderLines, setTotalPriceOriginalSimpleAndVariableOrderLines] = useState<number>(0);
  const [partiallyOrNotRefundedOrderLines, setPartiallyOrNotRefundedOrderLines] = useState<OrderLine[]>([]);
  const [fullyRefundedItems, setFullyRefundedItems] = useState<OrderLine[]>([]);
  const [linesToRefund, setLinesToRefund] = useState<RefundOrderLineRequest[]>([]);
  const [sousTotalWithSimpleAndVariableProducts, setSousTotalWithSimpleAndVariableProducts] = useState<number>(0);
  const [shippingPriceToRefund, setShippingPriceToRefund] = useState<number>(0);
  const [selectedItems, setSelectedItems] = useState<string[]>([]);
  const [totalPriceHT, setTotalPriceHT] = useState<number>(0);
  const [tva, setTva] = useState<number>(0);
  const [totalPriceTTCToRefund, setTotalPriceTTCToRefund] = useState<number>(0);
  const [refundMethod, setRefundMethod] = useState<RefundMethod>(RefundMethod.MANUAL);
  const [refundReason, setRefundReason] = useState<RefundReason>(RefundReason.CUSTOMER_RETURN);
  const [totalRefundAndPendingRefundAmount, setTotalRefundAndPendingRefundAmount] = useState<number>(0);
  const [userId, setUserId] = useState<string>("");
  const [isLoading, setIsLoading] = useState<boolean>(false);

  const totalPriceAllGiftCardItems = order?.orderLines?.filter(line => line?.productType === ProductType.GIFT_CARD).reduce((acc, line) => acc + line?.totalPrice, 0);


  //******************fetch user data*********************
  const fetchUserData = async () => {
    const clientEmail = localStorage.getItem("email");
    const response = await UserByEmail(clientEmail!);
    const userData: IUser = response.data;
    if (userData)
      setUserId(userData.id)
  }
  useEffect(() => {
    fetchUserData();
  }, []);


  //******************alert*********************
  const [openSuccessAlert, setOpenSuccessAlert] = useState(false);
  const [openErrorAlert, setOpenErrorAlert] = useState(false);
  const [alertMessage, setAlertMessage] = useState("");


  //******************refund order form features*********************
  useEffect(() => {
    if (order) {
      const simpleAndVariableOrderLines = order?.orderLines?.filter(line =>
        line?.productType === ProductType.SIMPLE || line?.productType === ProductType.VARIABLE
      ) || [];
      const totalPriceSimpleAndVariableOrderLines = simpleAndVariableOrderLines.reduce(
        (total, line) => total + (line?.totalPrice ?? 0),
        0
      );
      setTotalPriceOriginalSimpleAndVariableOrderLines(totalPriceSimpleAndVariableOrderLines);
      setOriginalSimpleAndVariableOrderLines(simpleAndVariableOrderLines);
      const { fullyRefundedItems, partiallyOrNotRefundedItems } = splitFullyAndPartiallyRefundedOrderLines(simpleAndVariableOrderLines);
      setFullyRefundedItems(fullyRefundedItems);
      setPartiallyOrNotRefundedOrderLines(partiallyOrNotRefundedItems);
      setSousTotalWithSimpleAndVariableProducts(partiallyOrNotRefundedItems?.reduce((acc, line) => acc + line?.totalPrice, 0) || 0);
    }
  }, [order]);

  useEffect(() => {
    setSelectedItems(partiallyOrNotRefundedOrderLines.length > 0 ? [partiallyOrNotRefundedOrderLines[0].id] : []);
    setLinesToRefund(partiallyOrNotRefundedOrderLines?.map((line) => ({
      amountToRefund: line?.totalPrice,
      quantityToRefund: line?.quantity,
      orderLineId: line?.id,
    })));
    setSousTotalWithSimpleAndVariableProducts(partiallyOrNotRefundedOrderLines.length > 0 ? partiallyOrNotRefundedOrderLines[0]?.totalPrice : 0);
  }, [partiallyOrNotRefundedOrderLines]);

  useEffect(() => {
    const { isOriginalAmountsEqualPartialOrNotRefundedAmounts,
      isOriginalQuantitiesEqualPartialOrNotRefundedQuantities,
      totalPendingAndRefundAmount } = evaluateRefundCompletionStatus(linesToRefund, partiallyOrNotRefundedOrderLines);
    setTotalRefundAndPendingRefundAmount(totalPendingAndRefundAmount);

    if (isOriginalAmountsEqualPartialOrNotRefundedAmounts && isOriginalQuantitiesEqualPartialOrNotRefundedQuantities && partiallyOrNotRefundedOrderLines.length > 0) {
      setShippingPriceToRefund(order?.shippingPrice || 0);
    } else if (!isOriginalAmountsEqualPartialOrNotRefundedAmounts && !isOriginalQuantitiesEqualPartialOrNotRefundedQuantities) {
      setShippingPriceToRefund(0);
    }
  }, [sousTotalWithSimpleAndVariableProducts]);

  useEffect(() => {
    recalculateAllPrices();
  }, [sousTotalWithSimpleAndVariableProducts, shippingPriceToRefund]);

  const recalculateAllPrices = () => {
    const TVA_RATE = 0.20;

    const totalTTC = (sousTotalWithSimpleAndVariableProducts + shippingPriceToRefund || 0);
    const totalHT = totalTTC / (1 + TVA_RATE);
    const tva = totalHT * TVA_RATE;

    setTotalPriceHT(totalHT);
    setTva(tva);
    setTotalPriceTTCToRefund(totalTTC);
  };

  const clickQuantityChange = (orderLineId: string, action: 'increment' | 'decrement') => {
    const refundOrderLineQuantity = linesToRefund?.find(line => line?.orderLineId === orderLineId)?.quantityToRefund;
    const orderLineQuantity = originalSimpleAndVariableOrderLines?.find(line => line?.id === orderLineId)?.quantity || 0;
    let refundedQuantity = 0;
    order?.refundOrders?.forEach((refundOrder) => {
      refundOrder.refundOrderLines.forEach((refundLine) => {
        if (refundLine.orderLineId === orderLineId) {
          refundedQuantity += refundLine.quantityToRefund;
        }
      });
    });

    const maxQuantityToRefund = orderLineQuantity - refundedQuantity;

    if (refundOrderLineQuantity && orderLineQuantity && action === 'increment' && refundOrderLineQuantity + 1 > maxQuantityToRefund) {
      setAlertMessage('La quantité du remboursement de chaque produit ne peut être supérieure à la quantité du produit');
      setOpenErrorAlert(true);
      return;
    }
    const newLinesToRefund = linesToRefund?.map(line => {
      if (line?.orderLineId === orderLineId) {
        const unitPrice = line?.amountToRefund / line?.quantityToRefund;
        const newQuantity = action === 'increment' ? line?.quantityToRefund + 1 : Math.max(1, line?.quantityToRefund - 1);
        const newTotalPrice = unitPrice * newQuantity;

        return {
          ...line,
          quantityToRefund: newQuantity,
          amountToRefund: newTotalPrice,
        };
      }
      return line;
    }).filter(line => selectedItems.includes(line?.orderLineId));

    setLinesToRefund(newLinesToRefund);
    setSousTotalWithSimpleAndVariableProducts(newLinesToRefund?.reduce((acc, line) => acc + line?.amountToRefund, 0));
  };

  const handleQuantityItemChange = (orderLineId: string, newQuantity: number) => {
    const refundOrderLineQuantity = linesToRefund?.find(line => line?.orderLineId === orderLineId)?.quantityToRefund;
    const orderLineQuantity = originalSimpleAndVariableOrderLines?.find(line => line?.id === orderLineId)?.quantity || 0;
    let refundedQuantity = 0;
    order?.refundOrders?.forEach((refundOrder) => {
      refundOrder.refundOrderLines.forEach((refundLine) => {
        if (refundLine.orderLineId === orderLineId) {
          refundedQuantity += refundLine.quantityToRefund;
        }
      });
    });

    const maxQuantityToRefund = orderLineQuantity - refundedQuantity;

    if (refundOrderLineQuantity && orderLineQuantity && newQuantity > 0 && newQuantity > orderLineQuantity) {
      setAlertMessage('La quantité du remboursement de chaque produit ne peut être supérieure à la quantité du produit acheté');
      setOpenErrorAlert(true);
      return;
    }

    if (newQuantity > 0 && orderLineQuantity && refundOrderLineQuantity && newQuantity <= maxQuantityToRefund) {
      const newLinesToRefund = linesToRefund?.map(line => {
        const unitPrice = line?.amountToRefund / line?.quantityToRefund;
        const newTotalPrice = unitPrice * newQuantity;

        if (line?.orderLineId === orderLineId) {
          return {
            ...line,
            quantityToRefund: newQuantity,
            amountToRefund: newTotalPrice,
          };
        }
        return line;
      }).filter(line => selectedItems.includes(line?.orderLineId));

      setLinesToRefund(newLinesToRefund);
      setSousTotalWithSimpleAndVariableProducts(newLinesToRefund?.reduce((acc, line) => acc + line?.amountToRefund, 0));
    }
  };

  const handleTotalPriceChange = (newTotalPrice: number) => {
    setTotalPriceTTCToRefund(newTotalPrice);
  };

  const handleCheckboxChange = (orderLineId: string, checked: boolean) => {
    const newSelectedItems = checked
      ? [...selectedItems, orderLineId]
      : selectedItems.filter(id => id !== orderLineId);

    setSelectedItems(newSelectedItems);

    const newLinesToRefund = partiallyOrNotRefundedOrderLines
      .filter(line => newSelectedItems.includes(line.id))
      .map(line => ({
        amountToRefund: line.totalPrice,
        quantityToRefund: line.quantity,
        orderLineId: line.id,
      }));

    setLinesToRefund(newLinesToRefund);
    setSousTotalWithSimpleAndVariableProducts(newLinesToRefund.reduce((acc, line) => acc + line.amountToRefund, 0));
  };


  const splitFullyAndPartiallyRefundedOrderLines = (simpleAndVariableOrderLines: OrderLine[]) => {
    const refundedAmounts = new Map<string, number>();
    const refundedQuantities = new Map<string, number>();

    order?.refundOrders?.forEach((refundOrder) => {
      refundOrder.refundOrderLines.forEach((refundLine) => {
        refundedAmounts.set(
          refundLine.orderLineId,
          (refundedAmounts.get(refundLine.orderLineId) || 0) + refundLine.amountToRefund
        );
        refundedQuantities.set(
          refundLine.orderLineId,
          (refundedQuantities.get(refundLine.orderLineId) || 0) + refundLine.quantityToRefund
        );
      });
    });

    const fullyRefundedItems: OrderLine[] = [];
    const partiallyOrNotRefundedItems: OrderLine[] = [];

    simpleAndVariableOrderLines.forEach((orderLine) => {
      const totalRefundedPrice = refundedAmounts.get(orderLine.id) || 0;
      const totalRefundedQuantity = refundedQuantities.get(orderLine.id) || 0;

      const isFullyRefunded =
        totalRefundedPrice >= orderLine.totalPrice || totalRefundedQuantity >= orderLine.quantity;

      if (isFullyRefunded) {
        fullyRefundedItems.push(orderLine);
      } else {
        partiallyOrNotRefundedItems.push({
          ...orderLine,
          totalPrice: orderLine.totalPrice - totalRefundedPrice,
          quantity: orderLine.quantity - totalRefundedQuantity,
        });
      }
    });

    return {
      fullyRefundedItems,
      partiallyOrNotRefundedItems
    };
  };

  const evaluateRefundCompletionStatus = (linesToRefund: RefundOrderLineRequest[], partiallyOrNotRefundedOrderLines: OrderLine[]) => {
    const originalAmounts = new Map<string, number>();
    const originalQuantities = new Map<string, number>();
    const refundedAmounts = new Map<string, number>();
    const refundedQuantities = new Map<string, number>();

    originalSimpleAndVariableOrderLines.forEach((ol) => {
      originalAmounts.set(ol.id, ol.totalPrice);
      originalQuantities.set(ol.id, ol.quantity);
    });

    let totalAlreadyRefundedAmount = 0;

    order?.refundOrders?.forEach((refundOrder) => {
      refundOrder.refundOrderLines.forEach((refundLine) => {
        refundedAmounts.set(
          refundLine.orderLineId,
          (refundedAmounts.get(refundLine.orderLineId) || 0) + refundLine.amountToRefund
        );
        refundedQuantities.set(
          refundLine.orderLineId,
          (refundedQuantities.get(refundLine.orderLineId) || 0) + refundLine.quantityToRefund
        );
        totalAlreadyRefundedAmount += refundLine.amountToRefund;
      });
    });

    linesToRefund.forEach((l) => {
      refundedAmounts.set(
        l.orderLineId,
        (refundedAmounts.get(l.orderLineId) || 0) + l.amountToRefund
      );
      refundedQuantities.set(
        l.orderLineId,
        (refundedQuantities.get(l.orderLineId) || 0) + l.quantityToRefund
      );
    });

    const allPartialOrNotRefundedIdsInSelectedItems = partiallyOrNotRefundedOrderLines.every((partialLine) =>
      selectedItems.includes(partialLine.id)
    );

    const isOriginalAmountsEqualPartialOrNotRefundedAmounts = allPartialOrNotRefundedIdsInSelectedItems && Array.from(originalAmounts.keys()).every((key) => {
      const originalAmount = originalAmounts.get(key) || 0;
      const refundedAmount = refundedAmounts.get(key) || 0;
      return Math.abs(originalAmount - refundedAmount) < 0.01;
    });

    const isOriginalQuantitiesEqualPartialOrNotRefundedQuantities = allPartialOrNotRefundedIdsInSelectedItems && Array.from(originalQuantities.keys()).every((key) => {
      const originalQuantity = originalQuantities.get(key) || 0;
      const refundedQuantity = refundedQuantities.get(key) || 0;
      return originalQuantity === refundedQuantity;
    });

    const totalPendingAndRefundAmount = Array.from(refundedAmounts.values()).reduce(
      (sum, refundedAmount) => sum + refundedAmount,
      0
    );

    return {
      isOriginalAmountsEqualPartialOrNotRefundedAmounts,
      isOriginalQuantitiesEqualPartialOrNotRefundedQuantities,
      totalPendingAndRefundAmount,
      totalAlreadyRefundedAmount
    };
  };

  //*******************products image********************/
  const [parentProductsImagePath, setParentProductsImagePath] = useState<{ [key: string]: string }>({});

  const simpleProductIds = order?.orderLines
    ?.filter(orderLine => orderLine?.productType === ProductType.SIMPLE)
    ?.map(orderLine => orderLine?.productId);

  const variationIds = order?.orderLines
    ?.filter(orderLine => orderLine?.productType === ProductType.VARIABLE)
    ?.map(orderLine => orderLine?.productId);

  const { products: simpleProducts, productsLoading: simpleProductsLoading } = useGetProductsByIds(simpleProductIds);
  const { variations: variationProducts, variationsLoading: variationProductsLoading } = useGetVariationsByIds(variationIds);

  const parentProductIds = variationProducts
    ?.map(variation => variation?.productId)
    ?.filter((id): id is number => id !== undefined)
    ?.map(id => id.toString());

  const { products: parentProducts, productsLoading: parentProductsLoading } = useGetProductsByIds(parentProductIds);

  useEffect(() => {
    const fetchParentProductsImagePaths = () => {
      const imagePathMap = variationProducts.reduce((acc, variation) => {
        if (variation?.id && variation?.productId) {
          const parentProduct = parentProducts?.find(p => p?.id === variation?.productId);
          if (parentProduct) {
            acc[variation?.id] = parentProduct?.mainImagePath;
          }
        }
        return acc;
      }, {} as { [key: string]: string });
      setParentProductsImagePath(imagePathMap);
    };
    if (!parentProductsLoading && variationProducts && variationProducts.length > 0) {
      fetchParentProductsImagePaths();
    }
  }, [order, simpleProducts, variationProducts, simpleProductsLoading, variationProductsLoading, parentProductsLoading, parentProducts]);

  const getFinalImagePath = (orderLine: OrderLine) => {
    if (orderLine?.productType === ProductType.SIMPLE) {
      return extractUrl(orderLine?.productImagePath);
    }
    if (orderLine?.productType === ProductType.VARIABLE) {
      const parentImagePath = parentProductsImagePath[orderLine?.productId];
      const imagePath = valideVariationUrl(orderLine?.productImagePath, parentImagePath);
      return imagePath;
    }
    return DEFAULT_IMAGE_URL;
  };

  //*******************Save refund order*******************/
  const handleSaveRefund = async () => {
    setIsLoading(true);

    const productsRefundLength = linesToRefund?.filter(line => selectedItems.includes(line?.orderLineId)).length;

    if (productsRefundLength === 0) {
      setAlertMessage('Aucun produit sélectionné pour le remboursement');
      setOpenErrorAlert(true);
      setIsLoading(false);
      return;
    }

    if (order.paymentMethodType === PaymentMethodType.PAYPLUG && refundMethod !== RefundMethod.PAYPLUG && refundMethod !== RefundMethod.MANUAL) {
      setAlertMessage('Le remboursement PayPlug ne peut être effectué que par PayPlug ou Manuel.');
      setOpenErrorAlert(true);
      setIsLoading(false);
      return;
    }
    if (order.paymentMethodType === PaymentMethodType.PAYPAL &&
      refundMethod !== RefundMethod.PAYPAL && refundMethod !== RefundMethod.MANUAL) {
      setAlertMessage('Le remboursement PayPlug ne peut être effectué que par PayPal ou Manuel.');
      setOpenErrorAlert(true);
      setIsLoading(false);
      return;
    }
    if (order.paymentMethodType === PaymentMethodType.CREDIT_CARD &&
      refundMethod !== RefundMethod.CREDIT_CARD && refundMethod !== RefundMethod.MANUAL) {
      setAlertMessage('Le remboursement PayPlug ne peut être effectué que par PayPal ou Manuel.');
      setOpenErrorAlert(true);
      setIsLoading(false);
      return;
    }
    if (order.paymentMethodType !== PaymentMethodType.PAYPLUG && refundMethod === RefundMethod.PAYPLUG) {
      setAlertMessage('Le remboursement PayPlug ne peut être effectué que pour les commandes payées avec PayPlug.');
      setOpenErrorAlert(true);
      setIsLoading(false);
      return;
    }
    if (order.paymentMethodType !== PaymentMethodType.PAYPAL && refundMethod === RefundMethod.PAYPAL) {
      setAlertMessage('Le remboursement PayPal ne peut être effectué que pour les commandes payées avec PayPal.');
      setOpenErrorAlert(true);
      setIsLoading(false);
      return;
    }
    if (order.paymentMethodType !== PaymentMethodType.CREDIT_CARD && refundMethod === RefundMethod.CREDIT_CARD) {
      setAlertMessage('Le remboursement Carte Bancaire ne peut être effectué que pour les commandes payées avec Carte Bancaire.');
      setOpenErrorAlert(true);
      setIsLoading(false);
      return;
    }

    linesToRefund?.filter(line => selectedItems.includes(line?.orderLineId)).forEach(line => {
      if (line?.amountToRefund <= 0) {
        setAlertMessage('Le montant du remboursement de chaque produit ne peut être inférieur ou égal à 0');
        setOpenErrorAlert(true);
        setIsLoading(false);
        return;
      }
      if (line?.quantityToRefund <= 0) {
        setAlertMessage('La quantité du remboursement de chaque produit ne peut être inférieure ou égale à 0');
        setOpenErrorAlert(true);
        setIsLoading(false);
        return;
      }
    });

    if (totalPriceTTCToRefund <= 0) {
      setAlertMessage("Montant total du remboursement invalide");
      setOpenErrorAlert(true);
      setIsLoading(false);
      return;
    }

    const { isOriginalAmountsEqualPartialOrNotRefundedAmounts, isOriginalQuantitiesEqualPartialOrNotRefundedQuantities,
      totalPendingAndRefundAmount, totalAlreadyRefundedAmount } = evaluateRefundCompletionStatus(linesToRefund, partiallyOrNotRefundedOrderLines);

    if (isOriginalQuantitiesEqualPartialOrNotRefundedQuantities && isOriginalAmountsEqualPartialOrNotRefundedAmounts) {
      if ((totalPendingAndRefundAmount + order.shippingPrice) < order.total) {

        console.log(isOriginalAmountsEqualPartialOrNotRefundedAmounts + ' ' +
          isOriginalQuantitiesEqualPartialOrNotRefundedQuantities + ' ' +
          ((totalPendingAndRefundAmount + order.shippingPrice) < order.total) + ' ' +
          (totalPendingAndRefundAmount + order.shippingPrice) + ' ' +
          order.total
        );

        setAlertMessage('Le montant total du remboursement ne peut être inférieur au total de la commande');
        setOpenErrorAlert(true);
        setIsLoading(false);
        return;
      }
      if ((totalAlreadyRefundedAmount + totalPriceTTCToRefund) < (totalPendingAndRefundAmount + order.shippingPrice)) {
        setAlertMessage('Le montant total du remboursement ne peut être inférieur au total de la commande');
        setOpenErrorAlert(true);
        setIsLoading(false);
        return;
      }
    }

    if ((refundMethod === RefundMethod.PAYPLUG || refundMethod === RefundMethod.PAYPAL || refundMethod === RefundMethod.CREDIT_CARD) && 
          (totalRefundAndPendingRefundAmount + order.shippingPrice) > order.total) {
      setAlertMessage("Le remboursement via PayPlug ou PayPal ou Carte Bancaire ne peut pas dépasser le total de la commande.");
      setOpenErrorAlert(true);
      setIsLoading(false);
      return;
    }

    const request: CreateRefundOrderRequest = {
      userId: userId,
      clientId: order?.clientId,
      refundOrderLines: linesToRefund.filter(line => selectedItems.includes(line?.orderLineId)),
      shippingAmountToRefund: shippingPriceToRefund,
      totalAmountToRefund: totalPriceTTCToRefund,
      refundMethod: refundMethod,
      refundReason: refundReason,
      orderId: order?.id,
    }

    useCreateRefundOrder(request).then(() => {
      setAlertMessage('Rembourssement enregistré');
      setOpenSuccessAlert(true);
      setTimeout(() => {
        handleDisableRefund();
      }, 1500);
      setIsLoading(false);
      mutateOrder();
      window.location.reload();
    }).catch(() => {
      setAlertMessage('Erreur lors de l\'enregistrement du remboursement');
      setOpenErrorAlert(true);
      setIsLoading(false);
    });
  };

  const renderPriceDetails = (
    <Stack
      spacing={2}
      alignItems="flex-end"
      sx={{
        my: 3,
        textAlign: 'right',
        typography: 'body2',
      }}
    >
      {[
        {
          label: "Sous Total des produits (TTC)",
          value: fCurrency(sousTotalWithSimpleAndVariableProducts || 0),
        },
        {
          label: "Montant total des cartes cadeaux achetées (TTC)",
          value: fCurrency(totalPriceAllGiftCardItems || 0),
        },
        {
          label: "Frais d'expédition à rembourser (TTC)",
          value: fCurrency(shippingPriceToRefund || 0),
        },
        { label: "Total HT", value: fCurrency(totalPriceHT || 0) },
        { label: "TVA", value: fCurrency(tva || 0) },
        {
          label: "Total TTC",
          value: fCurrency(totalPriceTTCToRefund || 0),
          input: true,
          onChange: handleTotalPriceChange,
          inputValue: totalPriceTTCToRefund,
        },
      ].map((item, index) => (
        <Stack
          key={index}
          direction="row"
          alignItems="center"
          sx={{
            justifyContent: 'flex-end',
            width: '100%',
          }}
        >
          <Box
            sx={{
              width: 300,
              textAlign: 'right',
              color: 'text.secondary',
            }}
          >
            {item.label}
          </Box>

          <Box
            sx={{
              width: 160,
              textAlign: 'right',
              typography: item.input ? 'body2' : 'subtitle2',
            }}
          >
            {item.input ? (
              <TextField
                variant="outlined"
                size="small"
                sx={{ width: '80px' }}
                type="number"
                value={item.inputValue}
                onChange={(e) => item.onChange(parseFloat(e.target.value))}
              />
            ) : (
              item.value
            )}
          </Box>
        </Stack>
      ))}
    </Stack>
  );


  return (
    <>
      <Card sx={{ mt: 3 }}>
        <CardHeader title="Créer un nouveau remboursement" />
        <Stack
          sx={{
            px: 3,
          }}
        >
          <Scrollbar>
            <Stack
              direction="row"
              alignItems="center"
              sx={{
                py: 2,
                borderBottom: (theme) => `solid 1px ${theme.palette.divider}`,
                fontWeight: 'bold',
                textAlign: 'center',
              }}
            >
              <Box sx={{ flexGrow: 1, textAlign: 'left' }}>Produit</Box>
              <Box sx={{ width: 160, textAlign: 'left' }}>Quantité</Box>
              <Box sx={{ width: 160, textAlign: 'left' }}>Prix Total</Box>
              <Box sx={{ width: 100, textAlign: 'left' }}>Réductions</Box>
            </Stack>

            {fullyRefundedItems.map((orderLine) => (
              <Stack
                key={orderLine.id}
                direction="row"
                alignItems="center"
                sx={{
                  py: 3,
                  minWidth: 640,
                  borderBottom: (theme) => `dashed 2px ${theme.palette.background.neutral}`,
                  backgroundColor: '#fdf9fe'
                }}
              >

                <Avatar
                  alt={orderLine.productName}
                  src={getFinalImagePath(orderLine)}
                  variant="rounded"
                  sx={{ width: 64, height: 64, mr: 2, ml: 2 }}
                />

                <ListItemText
                  primary={orderLine.productName || 'Carte cadeau'}
                  secondary={orderLine.productUgs || ''}
                  primaryTypographyProps={{
                    typography: 'body2',
                  }}
                  secondaryTypographyProps={{
                    component: 'span',
                    color: 'text.disabled',
                    mt: 0.5,
                  }}
                />

                <Box sx={{ width: 160, display: 'flex', alignItems: 'left', flexDirection: "column" }}>
                  <Box sx={{ display: 'flex', alignItems: 'center', marginBottom: 1, justifyContent: 'center' }}>
                    <Typography variant="subtitle2">{`x ${orderLine?.quantity}`}</Typography>
                  </Box>
                </Box>

                <Box sx={{ width: 160, display: 'flex', alignItems: 'left', flexDirection: "column" }}>
                  <Box sx={{ display: 'flex', alignItems: 'center', marginBottom: 1, justifyContent: 'center' }}>
                    <Typography variant="subtitle2">{fCurrency(orderLine?.totalPrice)}</Typography>
                  </Box>
                </Box>
                <Box sx={{ width: 100, textAlign: 'center', typography: 'subtitle2' }}>
                  {fCurrency(orderLine?.totalPromoCodeDiscounts || 0)}
                </Box>
              </Stack>
            ))}

            {partiallyOrNotRefundedOrderLines.map((orderLine) => (
              <Stack
                key={orderLine.id}
                direction="row"
                alignItems="center"
                sx={{
                  py: 3,
                  minWidth: 640,
                  borderBottom: (theme) => `dashed 2px ${theme.palette.background.neutral}`,
                }}
              >

                <Checkbox
                  checked={selectedItems.includes(orderLine?.id)}
                  onChange={(e) => handleCheckboxChange(orderLine?.id, e.target.checked)}
                />

                <Avatar
                  alt={orderLine.productName}
                  src={getFinalImagePath(orderLine)}
                  variant="rounded"
                  sx={{ width: 64, height: 64, mr: 2 }}
                />

                <ListItemText
                  primary={orderLine.productName || 'Carte cadeau'}
                  secondary={orderLine.productUgs || ''}
                  primaryTypographyProps={{
                    typography: 'body2',
                  }}
                  secondaryTypographyProps={{
                    component: 'span',
                    color: 'text.disabled',
                    mt: 0.5,
                  }}
                />

                <Box sx={{ width: 160, display: 'flex', alignItems: 'left', flexDirection: "column" }}>
                  <Box sx={{ display: 'flex', alignItems: 'center', marginBottom: 1, justifyContent: 'center' }}>
                    <Typography variant="subtitle2">{`x ${orderLine?.quantity}`}</Typography>
                  </Box>
                  <Box sx={{ display: 'flex', alignItems: 'center' }}>
                    <IconButton onClick={() => clickQuantityChange(orderLine?.id, 'decrement')} size="small">
                      <Remove />
                    </IconButton>
                    <TextField
                      variant="outlined"
                      value={linesToRefund?.find(line => line?.orderLineId === orderLine?.id)?.quantityToRefund || 1}
                      size="small"
                      sx={{ width: 80, mx: 1 }}
                      type="number"
                      inputProps={{
                        min: 1,
                        max: orderLine?.quantity,
                      }}
                      onChange={(e) => handleQuantityItemChange(orderLine?.id, parseInt(e.target.value))}
                    />
                    <IconButton onClick={() => clickQuantityChange(orderLine?.id, 'increment')} size="small">
                      <Add />
                    </IconButton>
                  </Box>
                </Box>

                <Box sx={{ width: 160, display: 'flex', alignItems: 'left', flexDirection: "column" }}>
                  <Box sx={{ display: 'flex', alignItems: 'center', marginBottom: 1, justifyContent: 'center' }}>
                    <Typography variant="subtitle2" color={"green"}>{`= ${fCurrency(orderLine?.totalPrice)}`}</Typography>
                  </Box>
                  <Box sx={{ display: 'flex', alignItems: 'center', justifyContent: 'center' }}>
                    <Typography variant="subtitle2">{`${fCurrency(linesToRefund?.find(line => line?.orderLineId === orderLine?.id)?.amountToRefund || orderLine?.totalPrice)}`}</Typography>
                  </Box>
                </Box>
                <Box sx={{ width: 100, textAlign: 'center', typography: 'subtitle2' }}>
                  {fCurrency(orderLine?.totalPromoCodeDiscounts || 0)}
                </Box>
              </Stack>
            ))}
          </Scrollbar>

          {renderPriceDetails}

          <Grid container spacing={2} sx={{ my: 3 }} justifyContent="flex-end">
            <Grid item xs={3} md={3} lg={3}>
              <TextField
                select
                label="Sélectionner une raison"
                fullWidth
                size="small"
                sx={{ mb: 2 }}
                value={refundReason}
                onChange={(e) => setRefundReason(e.target.value as RefundReason)}
                SelectProps={{
                  native: true,
                }}
              >
                {Object.values(RefundReason).map((reason) => (
                  <option key={reason} value={reason}>
                    {refundReasonLabels[reason]}
                  </option>
                ))}
              </TextField>
              <TextField
                select
                label="Sélectionner la méthode"
                fullWidth
                size="small"
                value={refundMethod}
                onChange={(e) => setRefundMethod(e.target.value as RefundMethod)}
                SelectProps={{
                  native: true,
                }}
              >
                {Object.values(RefundMethod)
                  .filter(
                    (method) =>
                      method === RefundMethod.MANUAL ||
                      (method === RefundMethod.PAYPLUG && order?.paymentMethodType === PaymentMethodType.PAYPLUG) ||
                      (method === RefundMethod.PAYPAL && order?.paymentMethodType === PaymentMethodType.PAYPAL) ||
                      (method === RefundMethod.CREDIT_CARD && order?.paymentMethodType === PaymentMethodType.CREDIT_CARD)
                  )
                  .map((method) => (
                    <option key={method} value={method}>
                      {refundMethodLabels[method]}
                    </option>
                  ))}
              </TextField>
            </Grid>
          </Grid>
        </Stack>
      </Card>

      <Snackbar open={openSuccessAlert} autoHideDuration={3000}
        onClose={() => setOpenSuccessAlert(false)} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
        <Alert
          onClose={() => setOpenSuccessAlert(false)}
          severity="success"
          variant="filled"
          sx={{ width: '100%' }}
        >
          {alertMessage}
        </Alert>
      </Snackbar>
      <Snackbar open={openErrorAlert} autoHideDuration={3000}
        onClose={() => setOpenErrorAlert(false)} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
        <Alert
          onClose={() => setOpenErrorAlert(false)}
          severity="error"
          variant="filled"
          sx={{ width: '100%' }}
        >
          {alertMessage}
        </Alert>
      </Snackbar>

      <Stack spacing={1} direction="row" justifyContent="flex-end">
        {isRefund ? (
          <>
            <Button variant="outlined" color="primary" sx={{ mt: 1 }} onClick={handleDisableRefund}>
              Annuler
            </Button>
            <Button variant="contained" color="primary" sx={{ mt: 1 }} onClick={handleSaveRefund} disabled={isLoading}>
              {isLoading ? <CircularProgress size={24} color="inherit" /> : "Terminer"}
            </Button>
          </>
        ) : null}
      </Stack>
    </>
  );
}


const extractUrl = (url: string | { preview: string } | null | undefined): string => {
  const defaultImageUrl = DEFAULT_IMAGE_URL ?? '';
  if (url == null) {
    return defaultImageUrl;
  }
  if (typeof url === 'string') {
    const trimmedUrl = url.trim();
    const cleanedUrl = trimmedUrl.replace(/^Optional\[(.*)\]$/, '$1').trim();
    const urlPattern = /^(https?:\/\/)/;
    if (cleanedUrl && urlPattern.test(cleanedUrl)) {
      return cleanedUrl;
    }
    return defaultImageUrl;
  }
  if (typeof url === 'object' && url !== null && 'preview' in url) {
    return extractUrl(url.preview);
  }
  return defaultImageUrl;
};

const extractUrls = (urls: (string | { preview: string } | null | undefined)[]): string[] => {
  return urls.map(extractUrl);
};

const valideVariationUrl = (
  firstUrl: string | { preview: string } | null | undefined,
  secondUrl: string | { preview: string } | null | undefined
): string => {
  const defaultImageUrl = DEFAULT_IMAGE_URL;
  const firstUrlResult = extractUrl(firstUrl);
  if (firstUrlResult !== defaultImageUrl) {
    return firstUrlResult;
  }
  const secondUrlResult = extractUrl(secondUrl);
  if (secondUrlResult !== defaultImageUrl) {
    return secondUrlResult;
  }
  return defaultImageUrl;
};
