import Box from '@mui/material/Box';
import Card from '@mui/material/Card';
import Stack from '@mui/material/Stack';
import ListItemText from '@mui/material/ListItemText';
import { fCurrency } from '../../../../../utils/format-number';
import { Order, OrderLine } from '@/contexts/types/main-order';
import { useEffect, useState } from 'react';
import { useGetProductsByIds, useGetVariationsByIds } from '@/shared/api/product';
import { ProductType } from '@/shared/types/product';
import { Avatar, CardHeader, TableBody, TableContainer, Table, TableCell, TableHead, TableRow } from '@mui/material';
import { DEFAULT_IMAGE_URL } from '@/config-global';


type Props = {
  order: Order;
};

export default function OriginOrderDetailsItems({ order }: Props) {

  const [parentProductsImagePath, setParentProductsImagePath] = useState<{ [key: string]: string }>({});

  const simpleProductIds = order?.orderLines
    ?.filter(orderLine => orderLine?.productType === 'SIMPLE')
    ?.map(orderLine => orderLine?.productId);

  const variationIds = order?.orderLines
    ?.filter(orderLine => orderLine?.productType === 'VARIABLE')
    ?.map(orderLine => orderLine?.productId);

  const { products: simpleProducts, productsLoading: simpleProductsLoading } = useGetProductsByIds(simpleProductIds);
  const { variations: variationProducts, variationsLoading: variationProductsLoading } = useGetVariationsByIds(variationIds);

  const parentProductIds = variationProducts
    ?.map(variation => variation?.productId)
    ?.filter((id): id is number => id !== undefined)
    ?.map(id => id.toString());

  const { products: parentProducts, productsLoading: parentProductsLoading } = useGetProductsByIds(parentProductIds);

  useEffect(() => {
    const fetchParentProductsImagePaths = () => {
      const imagePathMap = variationProducts?.reduce((acc, variation) => {
        if (variation?.id && variation?.productId) {
          const parentProduct = parentProducts?.find(p => p?.id === variation?.productId);
          if (parentProduct) {
            acc[variation?.id] = parentProduct?.mainImagePath || 'Default Path';
          }
        }
        return acc;
      }, {} as { [key: string]: string });

      setParentProductsImagePath(imagePathMap);
    };

    if (!parentProductsLoading && variationProducts.length > 0) {
      fetchParentProductsImagePaths();
    }
  }, [order, simpleProducts, variationProducts, simpleProductsLoading, variationProductsLoading, parentProductsLoading, parentProducts]);

  const getFinalImagePath = (orderLine: OrderLine) => {
    if (orderLine?.productType === ProductType.SIMPLE) {
      return extractUrl(orderLine?.productImagePath);
    }

    if (orderLine?.productType === ProductType.VARIABLE) {
      const parentImagePath = parentProductsImagePath[orderLine?.productId];
      const imagePath = valideVariationUrl(orderLine?.productImagePath, parentImagePath);
      return imagePath;
    }

    return DEFAULT_IMAGE_URL;
  };

  const renderTotal = (
    <Stack
      spacing={2}
      alignItems="flex-end"
      sx={{ my: 3, textAlign: 'right', typography: 'body2' }}
    >
      <Stack direction="row">
        <Box sx={{ color: 'text.secondary' }}>Sous Total (TTC)</Box>
        <Box sx={{ width: 160, typography: 'subtitle2' }}>
          {fCurrency(order?.sousTotal || 0)}
        </Box>
      </Stack>

      <Stack direction="row">
        <Box sx={{ color: 'text.secondary' }}>Remise Total des codes promo</Box>
        <Box sx={{ width: 160, typography: 'subtitle2' }}>
          {fCurrency(order?.totalPromoCodeDiscounts || 0)}
        </Box>
      </Stack>

      <Stack direction="row">
        <Box sx={{ color: 'text.secondary' }}>Montant payer avec cartes cadeau</Box>
        <Box sx={{ width: 160, typography: 'subtitle2' }}>
          {fCurrency(order?.giftCardBalanceUsed || 0)}
        </Box>
      </Stack>

      <Stack direction="row">
        <Box sx={{ color: 'text.secondary' }}>Montant payer avec bons d'achat</Box>
        <Box sx={{ width: 160, typography: 'subtitle2' }}>
          {fCurrency(order?.voucherBalanceUsed || 0)}
        </Box>
      </Stack>

      <Stack direction="row">
        <Box sx={{ color: 'text.secondary' }}>Frais d'expédition (TTC)</Box>
        <Box sx={{ width: 160, typography: 'subtitle2' }}>
          {fCurrency(order?.shippingPrice || 0)}
        </Box>
      </Stack>

      <Stack direction="row">
        <Box sx={{ color: 'text.secondary' }}>Total HT</Box>
        <Box sx={{ width: 160, typography: 'subtitle2' }}>
          {fCurrency(order?.totalPriceHT || 0)}
        </Box>
      </Stack>

      <Stack direction="row">
        <Box sx={{ color: 'text.secondary' }}>TVA</Box>
        <Box sx={{ width: 160, typography: 'subtitle2' }}>
          {fCurrency(order?.tva || 0)}
        </Box>
      </Stack>

      <Stack direction="row" sx={{ typography: 'subtitle1' }}>
        <Box>Total TTC</Box>
        <Box sx={{ width: 160 }}>{fCurrency(order?.total || 0)}</Box>
      </Stack>
    </Stack>
  );


  return (
    <>
      <Card>
        <CardHeader title="Commande" />
        <Stack
          sx={{
            px: 3,
          }}
        >

          <TableContainer>
            <Table>
              <TableHead>
                <TableCell>N°</TableCell>
                <TableCell>Produit</TableCell>
                <TableCell>Quantité</TableCell>
                <TableCell>Prix total</TableCell>
              </TableHead>
              <TableBody>
                {order?.orderLines?.map((orderLine) => (
                  <TableRow>
                    <TableCell>{orderLine?.id}</TableCell>
                    <TableCell>
                      <Avatar
                        alt={orderLine?.productName}
                        src={getFinalImagePath(orderLine)}
                        variant="rounded"
                        sx={{ width: 64, height: 64, mr: 2 }}
                      />
                      <ListItemText
                        primary={orderLine?.productName || (orderLine?.productType === ProductType.GIFT_CARD ? 'Carte cadeau' : '')}
                        secondary={orderLine?.productUgs || ''}
                        primaryTypographyProps={{
                          typography: 'body2',
                        }}
                        secondaryTypographyProps={{
                          component: 'span',
                          color: 'text.disabled',
                          mt: 0.5,
                        }}
                      />
                    </TableCell>
                    <TableCell>x{orderLine?.quantity}</TableCell>
                    <TableCell>{fCurrency(orderLine?.totalPrice.toFixed(2) || 0)}</TableCell>
                  </TableRow>
                ))}
              </TableBody>
            </Table>
          </TableContainer>
          {renderTotal}
        </Stack>
      </Card>
    </>
  );
}


const extractUrl = (url: string | { preview: string } | null | undefined): string => {
  const defaultImageUrl = DEFAULT_IMAGE_URL ?? '';
  if (url == null) {
    return defaultImageUrl;
  }
  if (typeof url === 'string') {
    const trimmedUrl = url.trim();
    const cleanedUrl = trimmedUrl.replace(/^Optional\[(.*)\]$/, '$1').trim();
    const urlPattern = /^(https?:\/\/)/;
    if (cleanedUrl && urlPattern.test(cleanedUrl)) {
      return cleanedUrl;
    }
    return defaultImageUrl;
  }
  if (typeof url === 'object' && url !== null && 'preview' in url) {
    return extractUrl(url.preview);
  }
  return defaultImageUrl;
};

const extractUrls = (urls: (string | { preview: string } | null | undefined)[]): string[] => {
  return urls.map(extractUrl);
};

const valideVariationUrl = (
  firstUrl: string | { preview: string } | null | undefined,
  secondUrl: string | { preview: string } | null | undefined
): string => {
  const defaultImageUrl = DEFAULT_IMAGE_URL;
  const firstUrlResult = extractUrl(firstUrl);
  if (firstUrlResult !== defaultImageUrl) {
    return firstUrlResult;
  }
  const secondUrlResult = extractUrl(secondUrl);
  if (secondUrlResult !== defaultImageUrl) {
    return secondUrlResult;
  }
  return defaultImageUrl;
};