"use client";

import { useChart } from "@/components/chart";
import Container from "@mui/material/Container";
import { paths } from "@/routes/paths";
import { useSettingsContext } from "@/shared/components/settings";
import CustomBreadcrumbs from "@/components/custom-breadcrumbs";
import { Grid } from "@mui/material";
import { useEffect, useMemo, useState } from "react";
import OrderFilters from "./statistics-filters";
import OrderChartReturns from "./order-chart-returns";
import OrderMontantChart from "./order-chart";
import OrderCommandChart from "./OrderCommandChart";
import AnalyticsRefundReason from "./analytics-refund-reason";
import AppWidgetSummary from "./statistics-widget-summary";
import { useTheme } from "@mui/material/styles";
import { fetchRealRefundRevenue, RefundReasonType, fetchRefundReasons, fetchNetSalesData, fetchRefundsStatsData, fetchOrdersStatsData, fetchRealRevenueStats, fetchTotalOrdersStats } from "@/shared/api/saleSessionStatistics";
import { useSnackbar } from "notistack";
import { ShippingMethod } from "@/shared/types/order";

export default function AllStatisticsView() {
  const settings = useSettingsContext();
  const [startDate, setStartDate] = useState<Date>(new Date(new Date().setDate(new Date().getDate() - 7)));
  const [endDate, setEndDate] = useState<Date>(new Date());
  const [selectedOrders, setSelectedOrders] = useState<string[]>([]);
  const [selectedShipping, setSelectedShipping] = useState<string[]>([ShippingMethod.ColissimoAvecSignature, ShippingMethod.ColissimoSansSignature]);
  const [Turnover, setTurnover] = useState<number>(0);
  const [netSalesData, setNetSalesData] = useState<{
    categories?: string[];
    series: {
      type: string;
      data: { name: string; data: number[] }[];
    }[];
  } | null>(null);
  const [refundsStatsData, setRefundsStats] = useState<{
    categories?: string[];
    series: {
      type: string;
      data: { name: string; data: number[] }[];
    }[];
  } | null>(null);
  const [OrderStats, setOrderStats] = useState<{
    categories?: string[];
    series: {
      type: string;
      data: { name: string; data: number[] }[];
    }[];
  } | null>(null);
  const [totalOrders, setTotalOrders] = useState<number>(0);
  const [totalRefunds, setTotalRefunds] = useState<number>(0);
  const [filterStatus, setFilterLStatus] = useState<string>("period");
  const [RefundReasonData, setRefundReason] = useState<RefundReasonType>({
    "STOCK_ERROR": 0,
    "CUSTOMER_RETURN": 0,
    "DEFECT": 0
  });



  const theme = useTheme();
  const { enqueueSnackbar } = useSnackbar();
  const statsFiltersProps = {
    startDate,
    endDate,
    onChangeStartDate: (startDate: Date) => {
      setStartDate(startDate);
    },
    onChangeEndDate: (endDate: Date) => {
      setEndDate(endDate);
    },
    selectedOrders,
    setSelectedOrders,
    setSelectedShipping,
    onChangeFilterStatus: (filterStatus: string) => {
      setFilterLStatus(filterStatus);
    },
    filterStatus,
    selectedShipping
  };

  useEffect(() => {

    fetchTotalOrdersStats(startDate, endDate, selectedOrders, selectedShipping).then((data) => {
      setTotalOrders(data);
    }).catch(() => { enqueueSnackbar("error loading Total Orders"); });

    fetchRealRevenueStats(startDate, endDate, selectedOrders, selectedShipping, filterStatus).then((data) => {
      setTurnover(data);
    }).catch(() => {
      enqueueSnackbar("error loading Turnover");
    });

    fetchRealRefundRevenue(startDate, endDate, selectedOrders, filterStatus, selectedShipping).then((data) => {
      setTotalRefunds(data);
    }).catch(() => {
      enqueueSnackbar("error loading Total Refunds");
    });

    fetchRefundReasons(startDate, endDate, selectedOrders).then((data) => {
      setRefundReason(data);
    }).catch(() => {
      enqueueSnackbar("error loading Refund Reasons");
    });

    fetchNetSalesData(startDate, endDate, selectedOrders).then((data) => {
      setNetSalesData(data);
    }).catch(() => {
      enqueueSnackbar("error loading Net Sales");
    });

    fetchRefundsStatsData(startDate, endDate).then((data) => {
      setRefundsStats(data);
    }).catch(() => {
      enqueueSnackbar("error loading Refunds Stats");
    });

    fetchOrdersStatsData(startDate, endDate).then((data) => {
      setOrderStats(data);
    }).catch(() => {
      enqueueSnackbar("error loading Refunds Stats");
    });
  }, [startDate, endDate, selectedOrders, filterStatus, selectedShipping]);

  const maxChartX = useMemo(() => {
    const allItem = [
      ...(OrderStats?.series[0].data?.map((d) => d.data)?.[0] || []),
      ...(OrderStats?.series[1].data?.map((d) => d.data)?.[0] || []),
      ...(OrderStats?.series[2].data?.map((d) => d.data)?.[0] || []),
    ];

    return Math.max(
      ...allItem
    )
  }, [OrderStats]);

  return (
    <Container
      sx={{ gap: 3, display: "flex", flexDirection: "column" }}
      maxWidth={settings.themeStretch ? false : "lg"}
    >
      <CustomBreadcrumbs
        heading="Statistiques des commandes"
        links={[
          {
            name: "Tableau de bord",
            href: paths.dashboard.root,
          },
          {
            name: "Commandes",
            href: paths.dashboard.admin.order.all_orders,
          },
          {
            name: "Statistiques",
          },
        ]}
        rollbackLink={paths.dashboard.admin.order.all_orders}

        sx={{
          mb: { xs: 3, md: 5 },
        }}
      />

      <OrderFilters {...statsFiltersProps} />
      <Grid container spacing={3}>
        <Grid item xs={12} sm={6} md={4}>
          <AppWidgetSummary
            title="Chiffre d'affaires"
            total={Turnover}
            color={theme.palette.error.light}
            showEuroSymbol={true}
          />
        </Grid>

        <Grid item xs={12} sm={6} md={4}>
          <AppWidgetSummary
            title="Total des commandes"
            total={totalOrders}
            color={theme.palette.info.dark}
          />
        </Grid>

        <Grid item xs={12} sm={6} md={4}>
          <AppWidgetSummary
            title="Remboursements totaux"
            total={totalRefunds}
            color={theme.palette.primary.main}
          />
        </Grid>
      </Grid>
      <Grid container spacing={1}>
        <Grid item xs={4} md={4} lg={4}>
          <AnalyticsRefundReason
            title="Taux de remboursement par raisons"
            chart={{
              colors: ["#4CAF50", "#FFC107", "#FF5722"],
              series: [
                { label: "Retour client", value: RefundReasonData.CUSTOMER_RETURN },
                { label: "Erreur de stock", value: RefundReasonData.STOCK_ERROR },
                { label: "Défaut", value: RefundReasonData.DEFECT },
              ],
            }}
          />
        </Grid>
        <Grid item xs={8} md={6} lg={8}>
          <OrderCommandChart
            maxChartX={maxChartX}
            title="Historique des commandes"
            chart={{
              labels: OrderStats?.categories || [],
              series: [
                {
                  name: "Période précédente",
                  type: "line",
                  fill: "solid",
                  data: OrderStats?.series[0].data?.map((d) => d.data)?.[0] || [0, 0, 0, 0, 0, 0, 0, 0],
                },
                {
                  name: "Année précédente",
                  type: "line",
                  fill: "solid",
                  data: OrderStats?.series[1].data?.map((d) => d.data)?.[0] || [0, 0, 0, 0, 0, 0, 0, 0],
                },
                {
                  name: "Mois en cours",
                  type: "line",
                  fill: "solid",
                  data: OrderStats?.series[2].data?.map((d) => d.data)?.[0] || [0, 0, 0, 0, 0, 0, 0, 0],
                },
              ],
            }}
          />
        </Grid>
      </Grid>
      <OrderMontantChart
        title="Montant total des ventes"
        chart={{
          categories: netSalesData?.categories || [],
          series: netSalesData?.series || [],
          options: {
            yaxis: {
              title: {
                text: "Mantant",
              },
              min: 0,
              max: 10000,
              labels: {
                formatter: (value: number) => {
                  if (value >= 1000) {
                    return `${(value / 1000).toFixed(0)}K€`;
                  } else {
                    return `${value}€`;
                  }
                },
              },
            },
          },
        }}
      />
      <OrderChartReturns
        title="Retours"
        chart={{
          categories: refundsStatsData?.categories || [],
          series: refundsStatsData?.series || [],
          options: {
            yaxis: {
              title: {
                text: "Mantant",
              },
              min: 0,
              max: 10000,
              labels: {
                formatter: (value: number) => {
                  if (value >= 1000) {
                    return `${(value / 1000).toFixed(0)}K€`;
                  } else {
                    return `${value}€`;
                  }
                },
              },
            },
          },
        }}
      />
    </Container>
  );
}
