import React, { useState, useEffect } from "react";
import {
  Card,
  SelectChangeEvent,
  Select,
  Checkbox,
  MenuItem,
  FormControl,
  InputLabel,
  OutlinedInput,
  ListItemText,
} from "@mui/material";
import CustomDateRangePicker from "@/shared/components/custom-date-range-picker/custom-date-range-picker";

import { _orders } from "@/shared/_mock";
import Iconify from "@/components/iconify";
import { fDate } from "@/utils/format-time";
import { ShippingMethod } from "@/shared/types/order";

type Props = {
  startDate: Date;
  endDate: Date;
  setSelectedOrders: (selectedOrders: string[]) => void;
  setSelectedShipping: (selectedOrders: string[]) => void;
  selectedOrders: string[];
  onChangeStartDate: (startDate: Date) => void;
  onChangeEndDate: (endDate: Date) => void;
  onChangeFilterStatus: (filterStatus: string) => void;
  filterStatus: string;
  selectedShipping: string[];
};

export default function OrderFilters({
  startDate,
  endDate,
  onChangeEndDate,
  onChangeStartDate,
  setSelectedOrders,
  setSelectedShipping,
  onChangeFilterStatus,
  selectedOrders,
  filterStatus,
  selectedShipping,
}: Props) {
  const [isDateRangeOpen, setIsDateRangeOpen] = useState(false);
  const [selectAll, setSelectAll] = useState(true);
  const [selectedShippingKeys, setSelectedShippingKeys] = useState<string[]>([]);
  const [selectAllShipping, setSelectAllShipping] = useState(false);
  const OrderSource = ["LIVE", "WEB"];
  const [filterByRefund, setFilterByRefund] = useState("");
  const [compareOptions, setCompareOptions] = useState({
    previousPeriod: false,
    previousYear: false,
  });
  enum filterType {
    date = "date",
    period = "period"
  }
  const [localFilterStatus, setLocalFilterStatus] = useState<string>(
    filterType.period.toString()
  );

  const handleChange = (event: any) => {
    const { value } = event.target;
    setLocalFilterStatus(value);
    setFilterByRefund(value);
    setCompareOptions({
      previousPeriod: value === "period",
      previousYear: value === "date",
    });
    onChangeFilterStatus(value);
  };

  function getKeyByValue(value: string): keyof typeof ShippingMethod | undefined {
    const entries = Object.entries(ShippingMethod);
    const found = entries.find(([_, val]) => val === value);
    return found ? found[0] as keyof typeof ShippingMethod : undefined;
  }

  useEffect(() => {
    setSelectAll(selectedOrders.length === OrderSource.length);
  }, [selectedOrders]);


  useEffect(() => {
    setSelectedOrders([...OrderSource]);
    setSelectedShipping(Object.values(ShippingMethod));
    setSelectedShippingKeys(Object.keys(ShippingMethod));
  }, []);

  const closeDateRange = () => {
    setIsDateRangeOpen(false);
  };

  const openDateRange = () => {
    setIsDateRangeOpen(true);
  };
  const handleChangeOrders = (
    event: SelectChangeEvent<typeof selectedOrders>
  ) => {
    const {
      target: { value },
    } = event;
    if (value.includes("all")) {
      if (selectAll) {
        setSelectedOrders([]);
        setSelectAll(false);
      } else {
        setSelectedOrders(OrderSource.slice(0, 2));
        setSelectAll(true);
      }
    } else {
      setSelectedOrders(typeof value === "string" ? value.split(",") : value);
    }
  };

  const handleChangeShipping = (
    event: SelectChangeEvent<typeof selectedShipping>
  ) => {
    const {
      target: { value },
    } = event;
    if (value.includes("all")) {
      if (selectAllShipping) {
        setSelectedShipping([]);
        setSelectedShippingKeys([]);
        setSelectAllShipping(false);
      } else {
        setSelectedShipping(Object.values(ShippingMethod));
        setSelectedShippingKeys(Object.keys(ShippingMethod));
        setSelectAllShipping(true);
      }
    } else {
      const newValues = typeof value === "string" ? value.split(",") : value;
      setSelectedShipping(newValues);
      const keys = newValues.map(val => getKeyByValue(val))
        .filter((key): key is keyof typeof ShippingMethod => key !== undefined);
      setSelectedShippingKeys(keys);
    }
  };

  return (
    <Card
      sx={{ minHeight: "auto", display: "flex", alignItems: "center", px: 2 }}
    >
      <Card
        onClick={openDateRange}
        variant="outlined"
        sx={{
          height: 50,
          cursor: "pointer",
          borderRadius: 1,
          p: 3,
          display: "flex",
          boxShadow: "none",
          justifyContent: "space-between",
          alignItems: "center",
        }}
      >
        <Iconify sx={{ mr: 3 }} icon="solar:calendar-date-bold" />
        <ListItemText
          secondary={`${fDate(startDate)} - ${fDate(endDate)}`}
          primaryTypographyProps={{
            typography: "body2",
            color: "text.primary",
            mb: 0.5,
          }}
        />
      </Card>
      <CustomDateRangePicker
        open={isDateRangeOpen}
        onClose={closeDateRange}
        startDate={startDate}
        endDate={endDate}
        onChangeEndDate={(newValue: Date | null) => onChangeEndDate(newValue!)}
        onChangeStartDate={(newValue: Date | null) =>
          onChangeStartDate(newValue!)
        }
      />
      <FormControl sx={{ m: 1, width: 300 }}>
        <InputLabel>Source</InputLabel>
        <Select
          multiple
          value={selectedOrders}
          onChange={handleChangeOrders}
          input={<OutlinedInput sx={{ height: 50 }} label="Source" />}
          renderValue={(selected) => selected.join(", ")}
        >
          <MenuItem value="all">
            <Checkbox checked={selectAll} />
            Sélectionner tout
          </MenuItem>
          {OrderSource.map((origine) => (
            <MenuItem key={origine} value={origine}>
              <Checkbox checked={selectedOrders.includes(origine)} />
              {origine}
            </MenuItem>
          ))}
        </Select>
      </FormControl>


      <FormControl sx={{ m: 1, width: 300 }}>
        <InputLabel>Expédition</InputLabel>
        <Select
          multiple
          value={selectedShipping}
          onChange={handleChangeShipping}
          input={<OutlinedInput sx={{ height: 50 }} label="Shipping" />}
          renderValue={(selected) => {
            return selected
              .map(value => getKeyByValue(value))
              .filter(key => key !== undefined)
              .join(", ");
          }}
        >
          <MenuItem value="all">
            <Checkbox checked={selectAllShipping} />
            Sélectionner tout
          </MenuItem>
          {Object.keys(ShippingMethod).map((key) => (
            <MenuItem key={key} value={ShippingMethod[key as keyof typeof ShippingMethod]}>
              <Checkbox
                checked={selectedShipping.includes(ShippingMethod[key as keyof typeof ShippingMethod])}
              />
              {key}
            </MenuItem>
          ))}
        </Select>
      </FormControl>

      <FormControl sx={{ m: 1, width: 300 }}>
        <InputLabel>Remboursement</InputLabel>
        <Select
          value={localFilterStatus}
          onChange={handleChange}
          input={<OutlinedInput sx={{ height: 50 }} label="Remboursement" />}
        >
          <MenuItem value="period">selon la période en cours</MenuItem>
          <MenuItem value="date">selon la date de la commande</MenuItem>
        </Select>
      </FormControl>
    </Card>
  );
}
