"use client";

import { useState, useEffect, useRef } from "react";
import Stack from "@mui/material/Stack";
import Container from "@mui/material/Container";
import Grid from "@mui/material/Unstable_Grid2";
import { useSettingsContext } from "../../../components/settings";
import OrderDetailsInfo from "../order-details/order-details-info";
import OrderDetailsItems from "../order-details/order-details-item";
import OrderDetailsToolbar from "../order-details/order-details-toolbar";
import { getOrderById } from "@/shared/api/main-order";
import { Order, OrderStatus } from "@/contexts/types/main-order";
import { Alert, Button } from "@mui/material";
import OrderDetailsItemRefund from "../refund-order/create-refund-order/order-details-item-refund";
import OrderRefundDetails from "../order-details/order-details-refund";
import LoadingScreen from "@/shared/components/loading-screen/loading-screen";
import { ProductType } from "@/shared/types/product";
import PrintIcon from "@mui/icons-material/Print";
type Props = {
  orderId: string;
};

export default function OrderDetailsView({ orderId }: Props) {
  const { orderData, orderError, orderLoading, mutateOrder } =
    getOrderById(orderId);
  const [selectedOrder, setSelectedOrder] = useState<Order>();
  const [activeRefund, setActiveRefund] = useState<boolean>(false);

  useEffect(() => {
    if (orderData) {
      setSelectedOrder(orderData);
    }
  }, [orderData]);

  const hasSimpleOrVariableProduct =
    orderData?.orderLines?.some(
      (line) =>
        line?.productType === ProductType.SIMPLE ||
        line?.productType === ProductType.VARIABLE
    ) || false;

  const settings = useSettingsContext();

  const handleEnableRefund = () => {
    setActiveRefund(true);
    setTimeout(scrollToTarget, 100);
  };

  const handleDisableRefund = () => {
    setActiveRefund(false);
  };

  const getProductName = (orderLineId: string): string | undefined => {
    return orderData?.orderLines?.find((line) => line.id === orderLineId)
      ?.productName;
  };

  const getProductImage = (orderLineId: string): string | undefined => {
    return orderData?.orderLines?.find((line) => line.id === orderLineId)
      ?.productImagePath;
  };

  const targetRef = useRef<HTMLDivElement>(null);

  const scrollToTarget = () => {
    if (targetRef.current) {
      targetRef.current.scrollIntoView({ behavior: "smooth" });
    }
  };
  const printRef1 = useRef<HTMLDivElement>(null);
  const printRef2 = useRef<HTMLDivElement>(null);
  const printRef3 = useRef<HTMLDivElement>(null);
  const handlePrint = () => {
    if (
      printRef1.current &&
      printRef2.current &&
      printRef3.current
    ) {
      const printWindow = window.open(" ", "_blank");

      if (printWindow) {
        const styles = Array.from(document.styleSheets)
          .map((styleSheet: any) => {
            try {
              return Array.from(styleSheet.cssRules)
                .map((rule: any) => rule.cssText)
                .join("");
            } catch (e) {
              return "";
            }
          })
          .join("");

        const printContent = `
          <div>
            ${printRef1.current.outerHTML}
            ${printRef3.current.outerHTML}
            ${printRef2.current.outerHTML}
          </div>

        `;

        printWindow.document.write(`
          <html>
            <head>
              <title>mydressin</title>
              <style>
                body { font-family: Arial, sans-serif; padding: 20px; }
                ${styles}
                @media print {
                  .no-print {
                    display: none !important;
                  }
                }
              </style>
            </head>
            <body>
              ${printContent}
            </body>
          </html>
        `);

        printWindow.document.close();
        printWindow.focus();

        setTimeout(() => {
          printWindow.print();
          printWindow.close();
        }, 500);
      }
    }
  };

  if (orderLoading) {
    return (
      <Container maxWidth={settings.themeStretch ? false : "lg"}>
        <LoadingScreen />;
      </Container>
    );
  }

  if (orderError) {
    return (
      <Alert severity="error">
        Problème lors de la récupération des données de la commande. Veuillez
        réessayer plus tard.
      </Alert>
    );
  }

  return (
    <>
      {selectedOrder && (
        <Container maxWidth={settings.themeStretch ? false : "lg"}>
          <div ref={printRef1}>
            <OrderDetailsToolbar order={selectedOrder} onMutate={mutateOrder} />
          </div>
          <Grid container spacing={3}>
            <Grid ref={printRef2} xs={12} md={8}>
              <Stack
                spacing={3}
                direction={{ xs: "column-reverse", md: "column" }}
              >
                <OrderDetailsItems order={selectedOrder} />
              </Stack>
              <Stack
                spacing={1}
                direction="row"
                justifyContent="flex-end"
                sx={{ mt: 3, mb: 3 }}
              >
                {!activeRefund ? (
                  <Button
                    className="no-print"
                    variant="contained"
                    color="primary"
                    sx={{ mt: 1 }}
                    onClick={handleEnableRefund}
                    disabled={
                      selectedOrder?.refunded ||
                      selectedOrder?.status === OrderStatus.DELETED ||
                      !hasSimpleOrVariableProduct
                    }
                  >
                    Remboursement
                  </Button>
                ) : null}
              </Stack>
            </Grid>
            <Grid ref={printRef3} xs={12} md={4} >
              <OrderDetailsInfo order={selectedOrder} />
            </Grid>
          </Grid>

          <Grid container spacing={3} className="no-print">
            <Grid xs={12} md={12}>
              {selectedOrder &&
              selectedOrder.refundOrders &&
              selectedOrder.refundOrders.length > 0 ? (
                <OrderRefundDetails
                  orderId={orderId}
                  getProductName={getProductName}
                  getProductImage={getProductImage}
                />
              ) : null}
              <Stack
                spacing={3}
                direction={{ xs: "column-reverse", md: "column" }}
                sx={{ mt: 3 }}
                ref={targetRef}
              >
                {activeRefund ? (
                  <OrderDetailsItemRefund
                    order={selectedOrder}
                    isRefund={activeRefund}
                    handleDisableRefund={handleDisableRefund}
                    mutateOrder={mutateOrder}
                  />
                ) : null}
              </Stack>
            </Grid>
          </Grid>
          <Button
            startIcon={<PrintIcon />}
            variant="contained"
            color="primary"
            onClick={handlePrint}
          >
            Imprimer
          </Button>
        </Container>
      )}
    </>
  );
}
