"use client";

import { useState, useCallback, useEffect } from "react";
import Tab from "@mui/material/Tab";
import Tabs from "@mui/material/Tabs";
import Card from "@mui/material/Card";
import Table from "@mui/material/Table";
import Button from "@mui/material/Button";
import { alpha } from "@mui/material/styles";
import Container from "@mui/material/Container";
import TableBody from "@mui/material/TableBody";
import TableContainer from "@mui/material/TableContainer";
import { paths } from "../../../../routes/paths";
import { useRouter } from "../../../../hooks/use-router";
import Label from "../../../components/label";
import Scrollbar from "../../../components/scrollbar";
import { useSnackbar } from "../../../../components/snackbar";
import { useSettingsContext } from "../../../components/settings";
import CustomBreadcrumbs from "../../../../components/custom-breadcrumbs";
import { useTable, emptyRows, TableNoData, TableEmptyRows, TableHeadCustom, TableSelectedAction, TablePaginationCustom, } from "../../../components/table";
import { IOrderTableFilters, IOrderTableFilterValue } from "../../../types/order";
import OrderTableRow from "../all-orders/order-table-row";
import OrderTableToolbar from "../all-orders/order-table-toolbar";
import OrderTableFiltersResult from "../all-orders/order-table-filters-result";
import { Alert, CircularProgress, Grid, Snackbar, TableCell, TableRow } from "@mui/material";
import { exportOrders, exportOrdersProducts, updateOrderStatus, useDeleteOrder, useDeleteOrderList, useOrderStatusCounts, useSearchOrders } from "@/shared/api/main-order";
import { OrderStatus, orderStatusLabels, OrderStatusList, OrderWithClientDetails, } from "@/contexts/types/main-order";
import { getlastOrderHistory } from "@/shared/api/order";

interface IOrderStatusOption {
  value: string;
  label: string;
}
const STATUS_OPTIONS: IOrderStatusOption[] = [
  { value: "", label: "Tous" },
  ...OrderStatusList.map((status) => ({
    value: status,
    label:
      orderStatusLabels[status] === "Supprimé"
        ? "Corbeille"
        : orderStatusLabels[status],
  })),
];

const TABLE_HEAD = [
  { id: "id", label: "N° de commande", width: 200 },
  { id: "clientFullName", label: "Client" },
  { id: "createdAt", label: "Date", width: 140 },
  { id: "", label: "Articles", width: 120, align: "center" },
  { id: "", label: "Prix", width: 140 },
  { id: "status", label: "Status", width: 110 },
  { id: "", width: 88 },
];


export default function OrderListView() {
  const { enqueueSnackbar } = useSnackbar();
  const table = useTable({ defaultOrderBy: "id", defaultOrder: "desc" });

  const settings = useSettingsContext();

  const router = useRouter();

  const [statusCounts, setStatusCounts] = useState<{ [key: string]: number }>({
    FAILED: 0,
    ON_HOLD: 0,
    IN_PROGRESS: 0,
    FINISHED: 0,
    CANCELED: 0,
    REFUNDED: 0,
    DELETED: 0,
  });


    const defaultFilters: IOrderTableFilters = localStorage.getItem("lastOrdersFilters")
    ?
    JSON.parse(localStorage.getItem("lastOrdersFilters")!)
    :
    {
      keyword: "",
      status: "",
      startDate: null,
      endDate: null,
    };

    const defaultEmptyState :IOrderTableFilters = {
      keyword: "",
      status: "",
      startDate: null,
      endDate: null,
    };

    const [filters, setFilters] = useState(defaultFilters);
  
  
    useEffect(() => {
      localStorage.removeItem("lastOrdersFilters")
      localStorage.setItem("lastOrdersFilters", JSON.stringify(filters));
    }, [filters]);

  const {
    orders,
    totalOrders,
    totalPages,
    pageNumber,
    pageSize,
    loading,
    revalidate,
  } = useSearchOrders({
    keyword: filters.keyword,
    status: filters.status,
    startDate: filters.startDate,
    endDate: filters.endDate,
    page: table.page,
    size: table.rowsPerPage,
    sortBy: table.orderBy,
    sortDir: table.order,

  });
  const [tableData, setTableData] = useState<OrderWithClientDetails[]>([]);
  useEffect(() => {
    if (orders) {
      setTableData(orders);
    }
  }, [orders]);
  const { orderStatusCounts, orderStatusError } = useOrderStatusCounts(filters);

  const dataInPage = tableData.slice(
    table.page * table.rowsPerPage,
    table.page * table.rowsPerPage + table.rowsPerPage
  );

  const denseHeight = table.dense ? 56 : 56 + 20;

  const canReset =
    !!filters.keyword ||
    filters.status !== "" ||
    (!!filters.startDate && !!filters.endDate);

  const notFound = (!totalOrders && canReset) || !totalOrders;

  const handleFilters = useCallback(
    (name: string, value: IOrderTableFilterValue) => {
      table.onResetPage();
      setFilters((prevState) => ({
        ...prevState,
        [name]: value,
      }));
    },
    [table]
  );

  const [snackbarOpen, setSnackbarOpen] = useState(false);
  const [snackbarMessage, setSnackbarMessage] = useState("");
  const [snackbarSeverity, setSnackbarSeverity] = useState<
    "success" | "warning" | "error"
  >("success");

  const handleSnackbarClose = () => {
    setSnackbarOpen(false);
  };
  const handleResetFilters = useCallback(() => {
    setFilters(defaultEmptyState);
  }, []);

  useEffect(() => {
    if (orderStatusError) {
      console.error("Error fetching status counts:", orderStatusError);
      return;
    }
    if (Array.isArray(orderStatusCounts)) {
      const statusObj = orderStatusCounts.reduce<{ [key: string]: number }>(
        (acc, [count, status]) => {
          acc[status] = count;
          return acc;
        },
        {}
      );
      setStatusCounts(statusObj);
    } else {
      console.error(
        "Expected counts to be an array, but received:",
        orderStatusCounts
      );
    }
  }, [filters, orderStatusCounts, orderStatusError]);

  const handleDeleteRow = useCallback(
    async (id: string) => {
      try {
        const status = await useDeleteOrder(id);
        if (status === 200) {
          const deleteRow = tableData.filter((row) => row.id !== id);
          setSnackbarMessage("Suppression réussie !");
          setSnackbarSeverity("success");
          setTableData(deleteRow);
          table.onUpdatePageDeleteRow(dataInPage.length);
          revalidate();
        } else {
          setSnackbarMessage("Échec de la suppression !");
          setSnackbarSeverity("warning");
        }
      } catch (error) {
        setSnackbarMessage(
          "Une erreur s'est produite lors de la suppression !"
        );
        setSnackbarSeverity("warning");
        console.error("Erreur lors de la suppression de la commande :", error);
      } finally {
        setSnackbarOpen(true);
      }
    },
    [dataInPage.length, enqueueSnackbar, table, tableData]
  );
  const handleChangeStatut = async (orderId: string, userId: string, statut: OrderStatus) => {
    try {
      await updateOrderStatus(orderId, userId, statut);
      setSnackbarMessage("Statut mis à jour avec succès !");
      setSnackbarSeverity("success");
      setSnackbarOpen(true);
      revalidate();
    } catch (error) {
      setSnackbarMessage("Erreur lors de la mise à jour du statut.");
      setSnackbarSeverity("error");
      setSnackbarOpen(true);
      console.error("Erreur lors de la mise à jour du statut :", error);
    }
  };


  const handleTrashOrder = useCallback(
    async (id: string) => {
      try {
        const userId = localStorage.getItem("userId")
          ? parseInt(localStorage.getItem("userId")!, 10)
          : 0;
        const status = await updateOrderStatus(
          id,
          String(userId),
          OrderStatus.DELETED
        );

        if (status) {
          setSnackbarMessage(
            "La commande a été déplacée dans la corbeille avec succès!"
          );
          setSnackbarSeverity("success");
          revalidate();
        } else {
          setSnackbarMessage("Échec du déplacement dans la corbeille!");
          setSnackbarSeverity("warning");
        }
      } catch (error) {
        setSnackbarMessage("Échec du déplacement dans la corbeille!");
        setSnackbarSeverity("warning");
        console.error("Erreur lors du déplacement de la commande:", error);
      } finally {
        setSnackbarOpen(true);
      }
    },
    [revalidate]
  );

  const restoreOrder = useCallback(
    async (id: string) => {
      try {
        const userId = localStorage.getItem("userId")
          ? parseInt(localStorage.getItem("userId")!, 10)
          : 0;
        const response = await getlastOrderHistory(Number(id));
        if (response.status == 200) {
          const status: OrderStatus = response.data.oldOrderStatus;
          const res = await updateOrderStatus(id, String(userId), status);

          if (res) {
            setSnackbarMessage("La commande a été restaurée avec succès !");
            setSnackbarSeverity("success");
            revalidate();
          } else {
            setSnackbarMessage("Échec de la restauration de la commande !");
            setSnackbarSeverity("warning");
          }
        }
      } catch (error) {
        setSnackbarMessage(
          "Une erreur est survenue lors de la restauration de la commande."
        );
        setSnackbarSeverity("error");
        console.error("Erreur lors de la restauration de la commande:", error);
      } finally {
        setSnackbarOpen(true);
      }
    },
    [revalidate]
  );

  const handleDeleteRows = useCallback(async () => {
    try {
      const ordersToDelete: string[] = table.selected
        .map((id) => {
          const order = tableData.find((row) => row.id === id);
          return order ? order.id : null;
        })
        .filter((id): id is string => id !== null);
      if (ordersToDelete.length === 0) {
        enqueueSnackbar("Failed to delete: No valid id found!", {
          variant: "error",
        });
        return;
      }
      const status = await useDeleteOrderList(ordersToDelete);
      if (status === 200) {
        setSnackbarMessage("Delete success!");
        setSnackbarSeverity("success");
        const deleteRows = tableData.filter(
          (row) => !table.selected.includes(row.id)
        );
        setTableData(deleteRows);
        table.onUpdatePageDeleteRows({
          totalRowsInPage: dataInPage.length,
          totalRowsFiltered: totalOrders,
        });
        revalidate();
      } else {
        setSnackbarMessage("Failed to delete!");
        setSnackbarSeverity("warning");
      }
    } catch (error) {
      setSnackbarMessage("Failed to delete!");
      setSnackbarSeverity("warning");
      console.error("Error deleting order:", error);
    } finally {
      setSnackbarOpen(true);
    }
  }, [
    totalOrders,
    dataInPage.length,
    enqueueSnackbar,
    table,
    tableData,
  ]);

  const handleViewRow = useCallback(
    (id: string) => {
      router.push(paths.dashboard.admin.order.details(id));
    },
    [router]
  );
  const handleEditRow = useCallback(
    (id: string) => {
      router.push(paths.dashboard.admin.order.edit(id));
    },
    [router]
  );
  const handleAddOrderClick = () => {
    router.push(paths.dashboard.admin.order.create);
  };

  const handleFilterStatus = useCallback(
    (event: React.SyntheticEvent, newValue: string) => {
      handleFilters("status", newValue);
    },
    [handleFilters]
  );
  
  const downloadOrdersCSV = () => {
      exportOrders(filters.startDate, filters.endDate, filters.status);
    };
    const downloadProductsCSV = () => {
      exportOrdersProducts(filters.startDate, filters.endDate, filters.status);
    };
  

  return (
    <>
      <Container maxWidth={settings.themeStretch ? false : "lg"}>
        <CustomBreadcrumbs
          heading="Liste des commandes"
          links={[
            {
              name: "Tableau de bord",
              href: paths.dashboard.root,
            },
            {
              name: "Commandes",
              href: paths.dashboard.admin.order.all_orders,
            },
            {
              name: "Liste des commandes",
            },
          ]}
          sx={{
            mb: { xs: 3, md: 5 },
          }}
          action={
            <Button variant="contained" onClick={handleAddOrderClick}>
              Ajouter une commande
            </Button>
          }
        />
<Grid
  container
  justifyContent="flex-end"
  style={{ marginBottom: "16px" }}
>
  <Button 
    variant="contained" 
    onClick={downloadOrdersCSV} 
    style={{ marginRight: "8px" }} // Adds space between buttons
  >
    Exporter les commandes
  </Button>
  
  <Button variant="contained" onClick={downloadProductsCSV}>
    Exporter les produits
  </Button>
</Grid>


        <Card>

          <Tabs
            value={filters.status}
            onChange={(event, newValue) => handleFilters("status", newValue)}
            sx={{
              px: 2.5,
              boxShadow: (theme) =>
                `inset 0 -2px 0 0 ${alpha(theme.palette.grey[500], 0.08)}`,
            }}
          >
            {STATUS_OPTIONS.map((tab) => {
              const count =
                statusCounts[tab.value as keyof typeof statusCounts] || 0;
              return (
                <Tab
                  key={tab.value}
                  iconPosition="end"
                  value={tab.value}
                  label={tab.label}
                  icon={
                    <Label
                      variant={
                        ((tab.value === "all" ||
                          tab.value === filters.status) &&
                          "filled") ||
                        "soft"
                      }
                      color={
                        (tab.value === OrderStatus.FINISHED && "success") ||
                        (tab.value === OrderStatus.FAILED && "warning") ||
                        (tab.value === OrderStatus.ON_HOLD && "info") ||
                        (tab.value === OrderStatus.IN_PROGRESS && "primary") ||
                        (tab.value === OrderStatus.CANCELED && "error") ||
                        "default"
                      }
                      sx={{
                        ...(tab.value === OrderStatus.REFUNDED && {
                          backgroundColor: "lightblue",
                          color: "darkblue",
                        }),
                        ...(tab.value === OrderStatus.DELETED && {
                          backgroundColor: "pink",
                          color: "darkred",
                        }),
                      }}
                    >
                      {tab.value === ""
                        ? Object.values(statusCounts).reduce(
                          (acc, count) => acc + count,
                          0
                        )
                        : count}
                    </Label>
                  }
                />
              );
            })}
          </Tabs>
          <OrderTableToolbar
            filters={filters}
            onFilters={handleFilters}
          />

          {canReset && (
            <OrderTableFiltersResult
              filters={filters}
              onFilters={handleFilters}
              //
              onResetFilters={handleResetFilters}
              //
              results={totalOrders}
              sx={{ p: 2.5, pt: 0 }}
            />
          )}

          <TableContainer sx={{ position: "relative", overflow: "unset" }}>
            <TableSelectedAction
              dense={table.dense}
              numSelected={table.selected.length}
              rowCount={totalOrders}
              onSelectAllRows={(checked) =>
                table.onSelectAllRows(
                  checked,
                  tableData.map((row) => row.id)
                )
              }
            />

            <Scrollbar>
              <Table
                size={table.dense ? "small" : "medium"}
                sx={{ minWidth: 960 }}
              >
                <TableHeadCustom
                  order={table.order}
                  orderBy={table.orderBy}
                  headLabel={TABLE_HEAD}
                  rowCount={totalOrders}
                  numSelected={table.selected.length}
                  onSort={table.onSort}
                  onSelectAllRows={(checked) =>
                    table.onSelectAllRows(
                      checked,
                      tableData.map((row) => row.id)
                    )
                  }
                />

                <TableBody>
                  {loading ? (
                    <TableRow>
                      <TableCell
                        colSpan={TABLE_HEAD.length}
                        style={{ textAlign: "center", padding: "20px" }}
                      >
                        <CircularProgress />
                      </TableCell>
                    </TableRow>
                  ) : (
                    <>
                      {notFound ? (
                        <TableNoData notFound={notFound} />
                      ) : (
                        tableData
                          .map((row) => (
                            <OrderTableRow
                              key={row.id}
                              row={row}
                              selected={table.selected.includes(row.id)}
                              onSelectRow={() => table.onSelectRow(row.id)}
                              onDeleteRow={() => handleDeleteRow(row.id)}
                              onTrashOrder={() => handleTrashOrder(row.id)}
                              onViewRow={() => handleViewRow(row.id)}
                              onEditRow={() => handleEditRow(row.id)}
                              onRestoreOrder={() => {
                                restoreOrder(row.id);
                              }}
                              onChangeStatut={(orderId, userId, statut) => handleChangeStatut(orderId, userId, statut)}
                            />
                          ))
                      )}
                      <TableEmptyRows
                        height={denseHeight}
                        emptyRows={emptyRows(
                          table.page,
                          table.rowsPerPage,
                          totalOrders
                        )}
                      />
                    </>
                  )}
                </TableBody>
              </Table>
            </Scrollbar>
          </TableContainer>

          <TablePaginationCustom
            count={totalOrders}
            page={pageNumber}
            rowsPerPage={pageSize}
            onPageChange={table.onChangePage}
            onRowsPerPageChange={table.onChangeRowsPerPage}
            dense={table.dense}
            onChangeDense={table.onChangeDense}
          />
        </Card>
      </Container>
      <Snackbar
        open={snackbarOpen}
        autoHideDuration={6000}
        onClose={handleSnackbarClose}
        anchorOrigin={{ vertical: "top", horizontal: "right" }}
      >
        <Alert onClose={handleSnackbarClose} severity={snackbarSeverity}>
          {snackbarMessage}
        </Alert>
      </Snackbar>
    </>
  );
}


