'use client';

import { useState, useEffect } from 'react';
import Stack from '@mui/material/Stack';
import Container from '@mui/material/Container';
import Grid from '@mui/material/Unstable_Grid2';
import { paths } from '../../../../routes/paths';
import { useSettingsContext } from '../../../components/settings';
import { getOrderById } from '@/shared/api/main-order';
import { Order } from '@/contexts/types/main-order';
import { CardContent, CardHeader, Card, Typography, IconButton, Alert } from '@mui/material';
import { useGetUserById } from '@/shared/api/user';
import { IClientItem } from '@/shared/types/user';
import OriginOrderDetailsItems from '../refund-order/refund-details/origin-order-details-item';
import RefundOrderDetailsItems from '../refund-order/refund-details/refund-order-details-item';
import Iconify from '@/shared/components/iconify';
import RouterLink from '@/routes/router-link';
import LoadingScreen from '@/shared/components/loading-screen/loading-screen';
import { RefundOrderItem } from '@/contexts/types/refundOrder';
import { getRefundOrdersByOrderId } from '@/shared/api/refundOrders';

type Props = {
  orderId: string;
};

export default function RefundOrderDetailsView({ orderId }: Props) {

  const { orderData, orderError, orderLoading } = getOrderById(orderId);
  const { refundOrdersData, refundOrdersError, refundOrdersLoading } = getRefundOrdersByOrderId(orderId);
  const [selectedOrder, setSelectedOrder] = useState<Order>();
  const [selectedRefundOrders, setSelectedRefundOrders] = useState<RefundOrderItem[]>([]);

  const { userData } = useGetUserById(Number(orderData?.clientId));
  const [clientData, setClientData] = useState<IClientItem | null>(null);

  useEffect(() => {
    if (userData && userData?.roles?.name === "CLIENT") {
      setClientData(userData as IClientItem);
    }
  }, [userData]);

  useEffect(() => {
    if (orderData) {
      setSelectedOrder(orderData);
    }
  }, [orderData]);

  useEffect(() => {
    if (refundOrdersData) {
      setSelectedRefundOrders(refundOrdersData);
    }
  }, [refundOrdersData]);
  
  const getProductName = (orderLineId: string): string | undefined => {
    return orderData?.orderLines?.find(line => line.id === orderLineId)?.productName;
  };
  
  const getProductImage = (orderLineId: string): string | undefined => {
    return orderData?.orderLines?.find(line => line.id === orderLineId)?.productImagePath;
  };

  const settings = useSettingsContext();

  if (orderLoading || refundOrdersLoading) {
    return (
      <Grid container spacing={3}>
        <Grid xs={12} sx={{ display: 'flex', justifyContent: 'center', alignItems: 'center', height: '100vh' }}>
          <LoadingScreen />
        </Grid>
      </Grid>
    );
  }
  
  if (orderError || refundOrdersError) {
    return (
      <Grid container spacing={3}>
        <Grid xs={12} sx={{ display: 'flex', justifyContent: 'center', alignItems: 'center', height: '100vh' }}>
          <Alert severity="warning">Problème lors de la récupération des données. Veuillez réessayer plus tard.</Alert>
        </Grid>
      </Grid>
    );
  }

  return (
    <>
      <Container maxWidth={settings.themeStretch ? false : 'lg'}>
        <Stack spacing={1} direction="row" alignItems="flex-start" sx={{ mb: 3 }}>
          <IconButton component={RouterLink} href={paths.dashboard.admin.order.refund.root}>
            <Iconify icon="eva:arrow-ios-back-fill" />
          </IconButton>

          <Stack spacing={0.5}>
            <Stack spacing={1} direction="row" alignItems="center">
              <Typography variant="h4"> Remboursement de la commande: N° {orderData?.id} </Typography>
            </Stack>
          </Stack>
        </Stack>

        {selectedOrder && selectedRefundOrders && clientData ? (
          <>
            <Card sx={{ mb: 3 }}>
              <CardHeader title="Client" />
              <CardContent>
                <Stack direction="row">
                  <Typography variant="body1" sx={{ mr: 6 }}>{clientData?.firstName} {clientData?.lastName}</Typography>
                  <Typography variant="body1" sx={{ mr: 6 }}>{clientData?.email}</Typography>
                  <Typography variant="body1">{clientData?.phoneNumber}</Typography>
                </Stack>
              </CardContent>
            </Card>
            <Grid container spacing={3}>
              <Grid xs={12} md={7}>
                <OriginOrderDetailsItems order={selectedOrder!} />
              </Grid>
              <Grid xs={12} md={5}>
                <RefundOrderDetailsItems selectedRefundOrders={selectedRefundOrders!} getProductName={getProductName} getProductImage={getProductImage} />
              </Grid>
            </Grid>
          </>
        ) : <></>}
      </Container >
    </>
  );
}
