import React from "react";
import {
  Table,
  TableBody,
  TableCell,
  TableContainer,
  TableHead,
  TableRow,
  Paper,
  Button,
  IconButton,
  TextField,
  TableFooter,
} from "@mui/material";
import { DragDropContext, Draggable, Droppable } from "react-beautiful-dnd";
import { Delete, Edit, Add } from "@mui/icons-material";

export interface IBankAccount {
  id?: number;
  accountName: string;
  bankName: string;
  accountNumber: string;
  iban: string;
  swift: string;
  currency?: string;
  order: number;
}

type Props = {
  updateBankAccounts: (bankAccounts: IBankAccount[]) => void;
  bankAccounts: IBankAccount[];
};

const BankAccountsTable: React.FC<Props> = ({ updateBankAccounts, bankAccounts }) => {
  const [editAccountId, setEditAccountId] = React.useState<number | null>(null);
  const [editedAccount, setEditedAccount] = React.useState<IBankAccount | null>(null);
  const [newAccount, setNewAccount] = React.useState<IBankAccount>({
    id: undefined,
    accountName: "",
    bankName: "",
    accountNumber: "",
    iban: "",
    swift: "",
    currency: "",
    order: bankAccounts.length,
  });


  const handleDragEnd = (result: any) => {
    if (!result.destination) return;

    const updatedAccounts = Array.from(bankAccounts);
    const [movedAccount] = updatedAccounts.splice(result.source.index, 1);
    updatedAccounts.splice(result.destination.index, 0, movedAccount);

    // Update the order based on the new position
    updatedAccounts.forEach((account, index) => {
      account.order = index;
    });

    updateBankAccounts(updatedAccounts);
  };

  const handleEditClick = (account: IBankAccount) => {
    setEditAccountId(account.id || null);
    setEditedAccount(account);
  };

  const handleSaveClick = () => {
    if (editedAccount) {
      const updatedAccounts = bankAccounts.map((account) =>
        account.id === editedAccount.id ? editedAccount : account
      );
      updateBankAccounts(updatedAccounts);
      setEditAccountId(null);
      setEditedAccount(null);
    }
  };

  const handleDeleteClick = (id: number | undefined) => {
    if (id !== undefined) {
      const updatedAccounts = bankAccounts.filter((account) => account.id !== id);
      updateBankAccounts(updatedAccounts);
    }
  };

  const handleAddAccount = () => {
    const accountToAdd = { ...newAccount};
    const updatedAccounts = [...bankAccounts, accountToAdd];
    updateBankAccounts(updatedAccounts);
    setNewAccount({
      id: undefined,
      accountName: "",
      bankName: "",
      accountNumber: "",
      iban: "",
      swift: "",
      currency: "",
      order: updatedAccounts.length,
    });
  };

  return (
    <>
      <DragDropContext onDragEnd={handleDragEnd}>
        <Droppable droppableId="bankAccounts">
          {(provided) => (
            <TableContainer component={Paper} {...provided.droppableProps} ref={provided.innerRef}>
              <Table>
                <TableHead>
                  <TableRow>
                    <TableCell>Nom du compte</TableCell>
                    <TableCell>Nom de la banque</TableCell>
                    <TableCell>Numéro de compte</TableCell>
                    <TableCell>IBAN</TableCell>
                    <TableCell>SWIFT</TableCell>
                    <TableCell>Devise</TableCell>
                    <TableCell>Actions</TableCell>
                  </TableRow>
                </TableHead>
                <TableBody>
                  {bankAccounts.map((account, index) => (
                    <Draggable key={account.id} draggableId={String(account.id)} index={index}>
                      {(provided) => (
                        <TableRow
                          ref={provided.innerRef}
                          {...provided.draggableProps}
                          {...provided.dragHandleProps}
                        >
                          {editAccountId === account.id ? (
                            <>
                              <TableCell>
                                <TextField
                                  value={editedAccount?.accountName || ""}
                                  onChange={(e) =>
                                    setEditedAccount({
                                      ...editedAccount!,
                                      accountName: e.target.value,
                                    })
                                  }
                                  size="small"
                                />
                              </TableCell>
                              <TableCell>
                                <TextField
                                  value={editedAccount?.bankName || ""}
                                  onChange={(e) =>
                                    setEditedAccount({
                                      ...editedAccount!,
                                      bankName: e.target.value,
                                    })
                                  }
                                  size="small"
                                />
                              </TableCell>
                              <TableCell>
                                <TextField
                                  value={editedAccount?.accountNumber || ""}
                                  onChange={(e) =>
                                    setEditedAccount({
                                      ...editedAccount!,
                                      accountNumber: e.target.value,
                                    })
                                  }
                                  size="small"
                                />
                              </TableCell>
                              <TableCell>
                                <TextField
                                  value={editedAccount?.iban || ""}
                                  onChange={(e) =>
                                    setEditedAccount({
                                      ...editedAccount!,
                                      iban: e.target.value,
                                    })
                                  }
                                  size="small"
                                />
                              </TableCell>
                              <TableCell>
                                <TextField
                                  value={editedAccount?.swift || ""}
                                  onChange={(e) =>
                                    setEditedAccount({
                                      ...editedAccount!,
                                      swift: e.target.value,
                                    })
                                  }
                                  size="small"
                                />
                              </TableCell>
                              <TableCell>
                                <TextField
                                  value={editedAccount?.currency || ""}
                                  onChange={(e) =>
                                    setEditedAccount({
                                      ...editedAccount!,
                                      currency: e.target.value,
                                    })
                                  }
                                  size="small"
                                />
                              </TableCell>
                              <TableCell>
                                <Button onClick={handleSaveClick}>Enregistrer</Button>
                              </TableCell>
                            </>
                          ) : (
                            <>
                              <TableCell>{account.accountName}</TableCell>
                              <TableCell>{account.bankName}</TableCell>
                              <TableCell>{account.accountNumber}</TableCell>
                              <TableCell>{account.iban}</TableCell>
                              <TableCell>{account.swift}</TableCell>
                              <TableCell>{account.currency}</TableCell>
                              <TableCell>
                                <IconButton onClick={() => handleEditClick(account)}>
                                  <Edit />
                                </IconButton>
                                <IconButton onClick={() => handleDeleteClick(account.id)}>
                                  <Delete />
                                </IconButton>
                              </TableCell>
                            </>
                          )}
                        </TableRow>
                      )}
                    </Draggable>
                  ))}
                  {provided.placeholder}
                </TableBody>
              </Table>
            </TableContainer>
          )}
        </Droppable>
      </DragDropContext>
      <Paper style={{ padding: "8px", marginTop: "16px" }}>
        <h3>Ajouter un compte bancaire</h3>
        <Table>
          <TableBody>
            <TableRow>
              <TableCell style={{ padding: "4px" }}>
                <TextField
                  label="Nom du compte"
                  value={newAccount.accountName}
                  onChange={(e) =>
                    setNewAccount({ ...newAccount, accountName: e.target.value })
                  }
                  size="small"
                  margin="dense"
                />
              </TableCell>
              <TableCell style={{ padding: "4px" }}>
                <TextField
                  label="Nom de la banque"
                  value={newAccount.bankName}
                  onChange={(e) =>
                    setNewAccount({ ...newAccount, bankName: e.target.value })
                  }
                  size="small"
                  margin="dense"
                />
              </TableCell>
              <TableCell style={{ padding: "4px" }}>
                <TextField
                  label="Numéro de compte"
                  value={newAccount.accountNumber}
                  onChange={(e) =>
                    setNewAccount({ ...newAccount, accountNumber: e.target.value })
                  }
                  size="small"
                  margin="dense"
                />
              </TableCell>
              <TableCell style={{ padding: "4px" }}>
                <TextField
                  label="IBAN"
                  value={newAccount.iban}
                  onChange={(e) => setNewAccount({ ...newAccount, iban: e.target.value })}
                  size="small"
                  margin="dense"
                />
              </TableCell>
              <TableCell style={{ padding: "4px" }}>
                <TextField
                  label="SWIFT"
                  value={newAccount.swift}
                  onChange={(e) => setNewAccount({ ...newAccount, swift: e.target.value })}
                  size="small"
                  margin="dense"
                />
              </TableCell>
              <TableCell style={{ padding: "4px" }}>
                <TextField
                  label="Devise"
                  value={newAccount.currency}
                  onChange={(e) =>
                    setNewAccount({ ...newAccount, currency: e.target.value })
                  }
                  size="small"
                  margin="dense"
                />
              </TableCell>
              <TableCell style={{ padding: "4px" }}>
                <Button
                  variant="contained"
                  color="primary"
                  startIcon={<Add />}
                  onClick={handleAddAccount}
                  size="small"
                >
                  Ajouter
                </Button>
              </TableCell>
            </TableRow>
          </TableBody>
        </Table>
      </Paper>
    </>
  );
};

export default BankAccountsTable;
