'use client';

import React, { useEffect, useState } from 'react';
import {
    Card,
    CardHeader,
    Grid,
    Stack,
    Typography,
    CircularProgress,
    Alert
} from '@mui/material';
import * as Yup from 'yup';
import { yupResolver } from '@hookform/resolvers/yup';
import { useForm, useWatch } from 'react-hook-form';
import FormProvider, { RHFSwitch, RHFTextField } from '@/components/hook-form';
import LoadingButton from '@mui/lab/LoadingButton';
import BankAccountsTable from './account-table';
import { updateBankTrasnferConf, useGetBankTranferConfig } from '@/shared/api/payment';
import { IBankAccount } from '@/shared/types/payment';
import { useResponsive } from '@/hooks';
import { useSnackbar } from '@/components/snackbar';

export default function BankTransferConfView() {
    const lgUp = useResponsive('up', 'lg');

    const validationSchema = Yup.object().shape({
        active: Yup.boolean().required(),
        title: Yup.string().required('Title is required'),
        description: Yup.string().required('Description is required'),
        instructions: Yup.string().required('Instructions are required'),
    });

    const { enqueueSnackbar } = useSnackbar();
    const { bankTranferConfigData, bankTranferConfigError, bankTranferConfigLoading } = useGetBankTranferConfig();

    const methods = useForm({
        resolver: yupResolver(validationSchema),
        defaultValues: {
            active: false,
            title: '',
            description: '',
            instructions: '',
        }
    });

    const { setValue, handleSubmit, formState: { isSubmitting } } = methods;
    const [bankAccounts, setBankAccounts] = useState<IBankAccount[]>([]);

    useEffect(() => {
        if (bankTranferConfigData) {
            setValue('active', bankTranferConfigData.active);
            setValue('title', bankTranferConfigData.title);
            setValue('description', bankTranferConfigData.description);
            setValue('instructions', bankTranferConfigData.instructions);
            setBankAccounts(bankTranferConfigData.bankAccounts || []);
        }
    }, [bankTranferConfigData, setValue]);

    const updateBankAccounts = (updatedBankAccounts: IBankAccount[]) => {
        setBankAccounts(updatedBankAccounts);
    };

    const onSubmit = handleSubmit(async (data) => {
        try {
            await updateBankTrasnferConf({
                ...data,
                bankAccounts,
                type: 'bank_transfer',
                id: bankTranferConfigData?.id || '',
                order: 1
            });
            enqueueSnackbar('La configuration du virement bancaire a été mise à jour avec succès', { variant: 'success' });
        } catch (error) {
            enqueueSnackbar('Échec de la mise à jour de la configuration du virement bancaire. Veuillez réessayer plus tard.', { variant: 'error' });
        }
    });

    if (bankTranferConfigLoading) {
        return (
            <Grid container justifyContent="center" alignItems="center" style={{ minHeight: '100vh' }}>
                <CircularProgress />
            </Grid>
        );
    }

    if (bankTranferConfigError) {
        return (
            <Grid container justifyContent="center" alignItems="center" style={{ minHeight: '100vh' }}>
                <Alert severity="error">Failed to load bank transfer configuration. Please try again later.</Alert>
            </Grid>
        );
    }

    return (
        <FormProvider methods={methods} onSubmit={onSubmit}>
            <Grid container spacing={3}>
                {lgUp && (
                    <Grid item md={4} paddingY={2}>
                        <Typography variant="h6" sx={{ mb: 0.5 }}>
                            Mode Virement Bancaire
                        </Typography>
                        <Typography variant="body2" sx={{ color: 'text.secondary' }}>
                            Acceptez les paiements en personne avec BACS. Aussi connu sous le nom de virement bancaire.
                        </Typography>
                    </Grid>
                )}
                <Grid item xs={12} lg={8} paddingY={2}>
                    <Card>
                        {!lgUp && <CardHeader title="Le Mode Live" />}
                        <Stack spacing={3} sx={{ p: 3 }}>
                            <Grid container spacing={3}>
                                <Grid item xs={12} sm={6}>
                                    <Card sx={{ p: 2, minHeight: { lg: 170 } }} variant="outlined">
                                        <Typography variant="subtitle1" gutterBottom>
                                            Activer/Désactiver
                                        </Typography>
                                        <Typography variant="body2" gutterBottom sx={{ color: 'text.secondary' }}>
                                            Activer le mode de paiement par carte bancaire
                                        </Typography>
                                        <RHFSwitch label="Activer" name="active" />
                                    </Card>
                                </Grid>
                                <Grid item xs={12} sm={6}>
                                    <Card sx={{ p: 2 }} variant="outlined">
                                        <Typography variant="subtitle1" gutterBottom>
                                            Titre
                                        </Typography>
                                        <Typography variant="body2" gutterBottom sx={{ color: 'text.secondary' }}>
                                            Contrôler le titre que l'utilisateur voit lors du paiement
                                        </Typography>
                                        <RHFTextField label="Titre" name="title" placeholder="Titre" />
                                    </Card>
                                </Grid>
                                <Grid item xs={12}>
                                    <Card sx={{ p: 2 }} variant="outlined">
                                        <Typography variant="subtitle1" gutterBottom>
                                            Description
                                        </Typography>
                                        <Typography variant="body2" gutterBottom sx={{ color: 'text.secondary' }}>
                                            Méthode de paiement que le client verra lors du passage de la commande
                                        </Typography>
                                        <RHFTextField label="Description" name="description" placeholder="Description" />
                                    </Card>
                                </Grid>
                                <Grid item xs={12}>
                                    <Card sx={{ p: 2 }} variant="outlined">
                                        <Typography variant="subtitle1" gutterBottom>
                                            Instructions
                                        </Typography>
                                        <Typography variant="body2" gutterBottom>
                                            Instructions qui seront ajoutées à la page de remerciement et aux emails
                                        </Typography>
                                        <RHFTextField label="Instructions" name="instructions" placeholder="Instructions" />
                                    </Card>
                                </Grid>
                                <Grid item xs={12}>
                                    <Card sx={{ p: 2 }} variant="outlined">
                                        <Typography variant="subtitle1" gutterBottom>
                                            Détails du Compte
                                        </Typography>
                                        <BankAccountsTable bankAccounts={bankAccounts} updateBankAccounts={updateBankAccounts} />
                                    </Card>
                                </Grid>
                            </Grid>
                        </Stack>
                    </Card>
                </Grid>
                <Grid item xs={12} md={8} sx={{ display: 'flex', justifyContent: 'end', py: 2 }}>
                    <LoadingButton
                        type="submit"
                        variant="contained"
                        size="large"
                        loading={isSubmitting}
                        sx={{ ml: 2 }}
                    >
                        Enregistrer
                    </LoadingButton>
                </Grid>
            </Grid>
        </FormProvider>
    );
}
