'use client';

import React, { useEffect } from 'react';
import {
  Card,
  CardHeader,
  Chip,
  Grid,
  MenuItem,
  Select,
  Stack,
  Typography,
  CircularProgress,
  Alert
} from '@mui/material';
import { useResponsive } from '@/hooks';
import * as Yup from 'yup';
import { yupResolver } from '@hookform/resolvers/yup';
import { useForm } from 'react-hook-form';
import FormProvider, { RHFAutocomplete, RHFSwitch, RHFTextField } from '@/components/hook-form';
import { _tags } from '@/shared/_mock';
import LoadingButton from '@mui/lab/LoadingButton';
import { useGetCreditCardConfig, updateCreditCardConfig } from '@/shared/api/payment';
import { ICreditCardConfig } from '@/shared/types/payment';
import { useSnackbar } from '@/components/snackbar';

export default function AdvancedCardProcessingForm() {
  const mdUp = useResponsive('up', 'md');

  // Validation schema using Yup
  const creditCardConfSchema = Yup.object<ICreditCardConfig>().shape({
    active: Yup.boolean().required(),
    title: Yup.string().required(),
    logos: Yup.array().of(Yup.string()).required(),
    disabledCreditCards: Yup.array().of(Yup.string()).required(),
    vaulting: Yup.boolean().required(),
    contingencyFor3DSecure: Yup.string().required()
  });

  const { enqueueSnackbar } = useSnackbar();
  // Fetch configuration data
  const { creditCardConfigData, creditCardConfigError, creditCardConfigLoading } = useGetCreditCardConfig();

  const methods = useForm({
    resolver: yupResolver(creditCardConfSchema),
    defaultValues: {
      active: false,
      title: '',
      logos: [],
      disabledCreditCards: [],
      vaulting: false,
      contingencyFor3DSecure: 'REQUIRED'
    }
  });

  const { setValue, handleSubmit, formState: { isSubmitting } } = methods;

  // Set form values when data is fetched successfully
  useEffect(() => {
    if (creditCardConfigData) {
      // Ensure that each field is correctly populated when data is available
      setValue('active', creditCardConfigData.active || false);
      setValue('title', creditCardConfigData.title || '');
      setValue('logos', creditCardConfigData.logos || []);
      setValue('disabledCreditCards', creditCardConfigData.disabledCreditCards || []);
      setValue('vaulting', creditCardConfigData.vaulting || false);
      setValue('contingencyFor3DSecure', creditCardConfigData.contingencyFor3DSecure || 'REQUIRED');
    }
  }, [creditCardConfigData, setValue]);

  const onSubmit = handleSubmit(async (data) => {
    try {
      await updateCreditCardConfig({
        ...creditCardConfigData,
        ...data,
      } as ICreditCardConfig);
      enqueueSnackbar('La configuration du mode carte bancaire a été mise à jour avec succès', { variant: 'success' });
        } catch (error) {
            enqueueSnackbar('Échec de la mise à jour de la configuration mode carte bancaire. Veuillez réessayer plus tard.', { variant: 'error' });
        }
  });

  // Loading state
  if (creditCardConfigLoading) {
    return (
      <Grid container justifyContent="center" alignItems="center" style={{ minHeight: '100vh' }}>
        <CircularProgress />
      </Grid>
    );
  }

  // Error state
  else if (creditCardConfigError) {
    return (
      <Grid container justifyContent="center" alignItems="center" style={{ minHeight: '100vh' }}>
        <Alert severity="error">Erreur lors du chargement de la configuration. Veuillez réessayer plus tard.</Alert>
      </Grid>
    );
  }

  const renderForm = (
    <>
      {mdUp && (
        <Grid md={4} paddingY={2}>
          <Typography variant="h6" sx={{ mb: 0.5 }}>
            Mode Carte Bancaire
          </Typography>
          <Typography variant="body2" sx={{ color: 'text.secondary' }}>
            Acceptez les cartes de débit et de crédit, ainsi que les méthodes de paiement locales avec la solution PayPal.
          </Typography>
        </Grid>
      )}
      <Grid xs={12} md={8} paddingY={2}>
        <Card>
          {!mdUp && <CardHeader title="Mode Carte Bancaire" />}
          <Stack spacing={3} sx={{ p: 3 }}>
            <Grid container spacing={3}>
              <Grid item xs={12} sm={6}>
                <Card sx={{ p: 2, minHeight: { sm: 170 } }} variant='outlined'>
                  <Typography variant="subtitle1" gutterBottom>
                    Activer/Désactiver
                  </Typography>
                  <Typography variant="body2" gutterBottom>
                    Activer le mode de paiement par carte bancaire
                  </Typography>
                  <RHFSwitch label={undefined} name="active" />
                </Card>
              </Grid>
              <Grid item xs={12} sm={6}>
                <Card sx={{ p: 2 }} variant='outlined'>
                  <Typography variant="subtitle1" gutterBottom>
                    Titre
                  </Typography>
                  <Typography variant="body2" gutterBottom>
                    Contrôler le titre que l'utilisateur voit lors du paiement
                  </Typography>
                  <RHFTextField label={undefined} name="title" placeholder="Titre" />
                </Card>
              </Grid>
              <Grid item xs={12}>
                <Card sx={{ p: 2 }} variant='outlined'>
                  <Typography variant="subtitle1" gutterBottom>
                    Désactiver des cartes de crédit spécifiques
                  </Typography>
                  <RHFAutocomplete
                    name="disabledCreditCards"
                    label="Cartes de Crédit"
                    placeholder="+ Carte de Crédit"
                    multiple
                    freeSolo
                    disableCloseOnSelect
                    options={_tags.map((option) => option)}
                    getOptionLabel={(option) => option}
                    renderOption={(props, option) => (
                      <li {...props} key={option}>
                        {option}
                      </li>
                    )}
                    renderTags={(selected, getTagProps) =>
                      selected.map((option, index) => (
                        <Chip
                          {...getTagProps({ index })}
                          key={option}
                          label={option}
                          size="small"
                          color="info"
                          variant="soft"
                        />
                      ))
                    }
                  />
                </Card>
              </Grid>
              <Grid item xs={12}>
                <Card sx={{ p: 2 }} variant='outlined'>
                  <Typography variant="subtitle1" gutterBottom>
                    Afficher le logo des cartes de crédit suivantes
                  </Typography>
                  <RHFAutocomplete
                    name="logos"
                    label="Logos"
                    placeholder="+ Logo"
                    multiple
                    freeSolo
                    disableCloseOnSelect
                    options={_tags.map((option) => option)}
                    getOptionLabel={(option) => option}
                    renderOption={(props, option) => (
                      <li {...props} key={option}>
                        {option}
                      </li>
                    )}
                    renderTags={(selected, getTagProps) =>
                      selected.map((option, index) => (
                        <Chip
                          {...getTagProps({ index })}
                          key={option}
                          label={option}
                          size="small"
                          color="info"
                          variant="soft"
                        />
                      ))
                    }
                  />
                </Card>
              </Grid>
              <Grid item xs={12} sm={6}>
                <Card sx={{ p: 2, minHeight: { sm: 170 } }} variant='outlined'>
                  <Typography variant="subtitle1" gutterBottom>
                    Vaulting
                  </Typography>
                  <Typography variant="body2" gutterBottom>
                    Stocker en toute sécurité les cartes de crédit de vos clients pour une expérience fluide
                  </Typography>
                  <RHFSwitch label={undefined} name="vaulting" />
                </Card>
              </Grid>
              <Grid item xs={12} sm={6}>
                <Card sx={{ p: 2 }} variant='outlined'>
                  <Typography variant="subtitle1" gutterBottom>
                    3D Secure
                  </Typography>
                  <Typography variant="body2" gutterBottom>
                    Le 3D Secure offre une vérification renforcée pour plus de sécurité
                  </Typography>
                  <Select
                    name="contingencyFor3DSecure"
                    label={undefined}
                    fullWidth
                    defaultValue="REQUIRED"
                  >
                    <MenuItem value="REQUIRED">3D Secure requis</MenuItem>
                    <MenuItem value="ALWAYS">Toujours utiliser 3D Secure</MenuItem>
                    <MenuItem value="NEVER">Ne jamais utiliser 3D Secure</MenuItem>
                  </Select>
                </Card>
              </Grid>
            </Grid>
          </Stack>
        </Card>
      </Grid>
    </>
  );

  const renderActions = (
    <>
      {mdUp && <Grid md={4} />}
      <Grid xs={12} md={8} sx={{ display: 'flex', justifyContent: 'end', py: 2 }}>
        <LoadingButton
          type="submit"
          variant="contained"
          size="large"
          loading={isSubmitting}
          sx={{ ml: 2 }}
        >
          Enregistrer
        </LoadingButton>
      </Grid>
    </>
  );

  return (
    <FormProvider methods={methods} onSubmit={onSubmit}>
      <Grid container spacing={3}>
        {renderForm}
        {renderActions}
      </Grid>
    </FormProvider>
  );
}
