"use client";

import * as Yup from 'yup';
import { useEffect } from 'react';
import { useForm } from 'react-hook-form';
import { yupResolver } from '@hookform/resolvers/yup';
import Grid from '@mui/material/Grid';
import Card from '@mui/material/Card';
import Stack from '@mui/material/Stack';
import Typography from '@mui/material/Typography';
import LoadingButton from '@mui/lab/LoadingButton';
import CardHeader from '@mui/material/CardHeader';
import { useResponsive } from '@/hooks/use-responsive';
import FormProvider, { RHFTextField, RHFSwitch, RHFSelect } from '@/components/hook-form';
import { useGetPaypalConnexion, updatePaypalConnexion } from '@/shared/api/payment';
import { useSnackbar } from '@/components/snackbar';
import { IPaypalConnexionConf } from '@/shared/types/payment';
import { MenuItem } from '@mui/material';
import Label from '@/shared/components/label';


// Validation schema for the form
const paypalConnectionSchema = Yup.object().shape({
    mode: Yup.string(),
    connected: Yup.boolean().required(),
    liveEmail: Yup.string().email('Please enter a valid email').required('Email is required'),
    liveMerchantId: Yup.string(),
    liveClientId: Yup.string().required('Client ID is required'),
    liveSecretKey: Yup.string().required('Secret Key is required'),
    sandboxEmail: Yup.string(),
    sandboxMerchantId: Yup.string(),
    sandboxClientId: Yup.string(),
    sandboxSecretKey: Yup.string(),
});

export default function PaypalConnexionView() {
    const mdUp = useResponsive('up', 'md');
    const { enqueueSnackbar } = useSnackbar();

    // Fetch PayPal configuration data
    const { paypalConnexionData, paypalConnexionError, paypalConnexionLoading } = useGetPaypalConnexion();

    const methods = useForm({
        resolver: yupResolver(paypalConnectionSchema),
        defaultValues: {
            mode: 'LIVE',
            connected: true,
            liveEmail: '',
            liveMerchantId: '',
            liveClientId: '',
            liveSecretKey: '',
            sandboxEmail: '',
            sandboxMerchantId: '',
            sandboxClientId: '',
            sandboxSecretKey: ''
        }
    });

    const { handleSubmit, setValue, formState: { isSubmitting }, reset } = methods;

    // When the data is fetched from the server, update the form with the server data
    useEffect(() => {
        if (paypalConnexionData) {
            reset({
                mode: paypalConnexionData.mode,
                connected: paypalConnexionData.connected,
                liveEmail: paypalConnexionData.liveEmail,
                liveMerchantId: paypalConnexionData.liveMerchantId,
                liveClientId: paypalConnexionData.liveClientId,
                liveSecretKey: paypalConnexionData.liveSecretKey,
                sandboxEmail: paypalConnexionData.sandboxEmail,
                sandboxMerchantId: paypalConnexionData.sandboxMerchantId,
                sandboxClientId: paypalConnexionData.sandboxClientId,
                sandboxSecretKey: paypalConnexionData.sandboxSecretKey,
            });
        }
    }, [paypalConnexionData, reset]);

    // Handle form submission
    const onSubmit = handleSubmit(async (data) => {
        try {
            await updatePaypalConnexion({ ...paypalConnexionData, ...data } as IPaypalConnexionConf);
            enqueueSnackbar('PayPal configuration updated successfully', { variant: 'success' });
        } catch (error) {
            enqueueSnackbar('Failed to update PayPal configuration', { variant: 'error' });
        }
    });

    if (paypalConnexionLoading) {
        return <div>Loading...</div>;
    }

    if (paypalConnexionError) {
        return <div>Error fetching PayPal configuration</div>;
    }

    return (
        <FormProvider methods={methods} onSubmit={onSubmit}>
            <Grid container spacing={3}>
                <ConnexionMode mdUp={mdUp} connected={paypalConnexionData?.connected as boolean} />
                <LiveMode mdUp={mdUp} />
                <SandboxMode mdUp={mdUp} />
                <Actions mdUp={mdUp} isSubmitting={isSubmitting} />
            </Grid>
        </FormProvider>
    );
}

// Components for rendering different parts of the form
function ConnexionMode({ mdUp, connected }: { mdUp: boolean; connected: boolean }) {
    return (
        <>
            {mdUp && (
                <Grid md={4} paddingY={4}>
                    <Typography variant="h6" sx={{ mb: 0.5 }}>
                        Mode de connexion
                    </Typography>
                    <Typography variant="body2" sx={{ color: 'text.secondary' }}>
                        Connecter et choisir le mode de connexion PayPal
                    </Typography>
                </Grid>
            )}
            <Grid xs={12} md={8} paddingY={2}>
                <Card>
                    {!mdUp && <CardHeader title="Mode de connexion" />}
                    <Stack spacing={3} sx={{ p: 3 }}>
                        <Grid container spacing={2}>
                            <Grid item xs={12} sm={6}>
                                <Card sx={{ p: 2 }} variant="outlined">
                                    <Typography variant="subtitle1" gutterBottom>
                                        Status de Connexion
                                    </Typography>
                                    <Typography variant="body2" gutterBottom>
                                        Cliquez pour réinitialiser les identifiants actuels et utiliser un autre compte
                                    </Typography>
                                    <Label
                                        variant="soft"
                                        color={connected ? 'success' : 'error'}
                                    >
                                        {connected ? 'Connecté' : 'Déconnecté'}
                                    </Label>
                                </Card>
                            </Grid>
                            <Grid item xs={12} sm={6}>
                                <Card sx={{ p: 2, minHeight:{ sm: 170 } }} variant='outlined'>
                                    <Typography variant="subtitle1" gutterBottom>
                                        Mode de connexion
                                    </Typography>
                                    <Typography variant="body2" gutterBottom>
                                        Choisir le mode de connexion live ou sandbox
                                    </Typography>
                                    <RHFSelect name="mode" label="mode" fullWidth>
                                        <MenuItem value="LIVE">live</MenuItem>
                                        <MenuItem value="SANDBOX">sandbox</MenuItem>
                                    </RHFSelect>
                                </Card>
                            </Grid>
                        </Grid>
                    </Stack>
                </Card>
            </Grid>
        </>
    );
}

function LiveMode({ mdUp }: { mdUp: boolean }) {
    return (
        <>
            {mdUp && (
                <Grid md={4} paddingY={2}>
                    <Typography variant="h6" sx={{ mb: 0.5 }}>
                        Mode live
                    </Typography>
                    <Typography variant="body2" sx={{ color: 'text.secondary' }}>
                        Définir les paramètres de connexion en mode live
                    </Typography>
                </Grid>
            )}
            <Grid xs={12} md={8} paddingY={2}>
                <Card>
                    {!mdUp && <CardHeader title="Mode live" />}
                    <Stack spacing={3} sx={{ p: 3 }}>
                        <Grid container spacing={2}>
                            <Grid item xs={12} md={6}>
                                <RHFTextField name="liveEmail" label="Adresse e-mail" />
                            </Grid>
                            <Grid item xs={12} md={6}>
                                <RHFTextField name="liveMerchantId" label="ID de marchand" />
                            </Grid>
                            <Grid item xs={12} md={6}>
                                <RHFTextField name="liveClientId" label="ID client" />
                            </Grid>
                            <Grid item xs={12} md={6}>
                                <RHFTextField type="password" name="liveSecretKey" label="Clé secrète" />
                            </Grid>
                        </Grid>
                    </Stack>
                </Card>
            </Grid>
        </>
    );
}

function SandboxMode({ mdUp }: { mdUp: boolean }) {
    return (
        <>
            {mdUp && (
                <Grid md={4} paddingY={2}>
                    <Typography variant="h6" sx={{ mb: 0.5 }}>
                        Mode sandbox
                    </Typography>
                    <Typography variant="body2" sx={{ color: 'text.secondary' }}>
                        Définir les paramètres de connexion en mode sandbox
                    </Typography>
                </Grid>
            )}
            <Grid xs={12} md={8} paddingY={2}>
                <Card>
                    {!mdUp && <CardHeader title="Mode sandbox" />}
                    <Stack spacing={3} sx={{ p: 3 }}>
                        <Grid container spacing={2}>
                            <Grid item xs={12} md={6}>
                                <RHFTextField name="sandboxEmail" label="Adresse e-mail" />
                            </Grid>
                            <Grid item xs={12} md={6}>
                                <RHFTextField name="sandboxMerchantId" label="ID de marchand" />
                            </Grid>
                            <Grid item xs={12} md={6}>
                                <RHFTextField name="sandboxClientId" label="ID client" />
                            </Grid>
                            <Grid item xs={12} md={6}>
                                <RHFTextField type="password" name="sandboxSecretKey" label="Clé secrète" />
                            </Grid>
                        </Grid>
                    </Stack>
                </Card>
            </Grid>
        </>
    );
}

function Actions({ mdUp, isSubmitting }: { mdUp: boolean; isSubmitting: boolean }) {
    return (
        <>
            {mdUp && <Grid md={4} />}
            <Grid xs={12} md={8} sx={{ display: 'flex', justifyContent: 'end', py: 2 }}>
                <LoadingButton
                    type="submit"
                    variant="contained"
                    size="large"
                    loading={isSubmitting}
                    sx={{ ml: 2 }}
                >
                    Enregistrer
                </LoadingButton>
            </Grid>
        </>
    );
}
