'use client';

import React, { useEffect } from 'react';
import {
  Card,
  CardHeader,
  Grid,
  Stack,
  Typography,
  CircularProgress,
  Alert,
} from '@mui/material';
import { useResponsive } from '@/hooks';
import * as Yup from 'yup';
import { yupResolver } from '@hookform/resolvers/yup';
import { useForm } from 'react-hook-form';
import FormProvider, { RHFSwitch, RHFTextField } from '@/components/hook-form';
import LoadingButton from '@mui/lab/LoadingButton';
import { updatePayPlugConfig, useGetPayPlugConfig } from '@/shared/api/payment';
import { useSnackbar } from '@/components/snackbar';
import { IPayPlugConfig } from '@/shared/types/payment';
import { PaymentMethodType } from '@/contexts/types/payment';

export default function PayPlugConfigForm() {
  const mdUp = useResponsive('up', 'md');

  const payPlugConfigSchema = Yup.object().shape({
    active: Yup.boolean().required(),
    title: Yup.string().required(),
    force3ds: Yup.boolean().required()
  });

  const { enqueueSnackbar } = useSnackbar();
  const { payPlugConfigData, payPlugConfigError, payPlugConfigLoading } = useGetPayPlugConfig();

  const methods = useForm({
    resolver: yupResolver(payPlugConfigSchema),
    defaultValues: {
      active: false,
      title: '',
      force3ds: false
    }
  });

  const { setValue, handleSubmit, formState: { isSubmitting } } = methods;

  useEffect(() => {
    if (payPlugConfigData) {
      setValue('active', payPlugConfigData.active);
      setValue('title', payPlugConfigData.title);
      setValue('force3ds', payPlugConfigData.force3ds);
    }
  }, [payPlugConfigData, setValue]);

  const onSubmit = handleSubmit(async (data) => {
    try {
      const payPlugConfig: IPayPlugConfig = {
        active: data.active,  
        title: data.title,
        force3ds: data.force3ds,
        id: payPlugConfigData?.id || '',
        order: payPlugConfigData?.order || 0,
        type: PaymentMethodType.PAYPLUG,
        description: payPlugConfigData?.description || ''
      }
      await updatePayPlugConfig(payPlugConfig);
      enqueueSnackbar('La configuration PayPlug a été mise à jour avec succès', { variant: 'success' });
    } catch (error) {
      enqueueSnackbar('Échec de la mise à jour de la configuration PayPlug. Veuillez réessayer plus tard.', { variant: 'error' });
    }
  });

  // État de chargement
  if (payPlugConfigLoading) {
    return (
      <Grid container justifyContent="center" alignItems="center" style={{ minHeight: '100vh' }}>
        <CircularProgress />
      </Grid>
    );
  }

  // État d'erreur
  if (payPlugConfigError) {
    return (
      <Grid container justifyContent="center" alignItems="center" style={{ minHeight: '100vh' }}>
        <Alert severity="error">Erreur lors du chargement de la configuration. Veuillez réessayer plus tard.</Alert>
      </Grid>
    );
  }

  const renderForm = (
    <>
      {mdUp ? (
        <Grid md={4} paddingY={2}>
          <Typography variant="h6" sx={{ mb: 0.5 }}>
            Paiement avec PayPlug
          </Typography>
        </Grid>
      ) : (
        <></>
      )}
      <Grid xs={12} md={8} paddingY={2}>
        <Card>
          {!mdUp ? <CardHeader title="Mode PayPlug" /> : <></>}
          <Stack spacing={3} sx={{ p: 3 }}>
            <Grid container spacing={3}>
              <Grid item xs={12} sm={6}>
                <Card sx={{ p: 2, minHeight: { sm: 170 } }} variant='outlined'>
                  <Typography variant="subtitle1" gutterBottom>
                    Activer/Désactiver
                  </Typography>
                  <Typography variant="body2" gutterBottom>
                    Activer le mode de paiement PayPlug
                  </Typography>
                  <RHFSwitch label="Activer" name="active" />
                </Card>
              </Grid>
              <Grid item xs={12} sm={6}>
                <Card sx={{ p: 2 }} variant='outlined'>
                  <Typography variant="subtitle1" gutterBottom>
                    Titre
                  </Typography>
                  <Typography variant="body2" gutterBottom sx={{ mb: 2 }}>
                    Contrôler le titre que l'utilisateur voit lors du paiement
                  </Typography>
                  <RHFTextField label="Titre" name="title" placeholder="Titre" />
                </Card>
              </Grid>
              <Grid item xs={12} sm={6}>
                <Card sx={{ p: 2 }} variant='outlined'>
                  <Typography variant="subtitle1" gutterBottom>
                    3D Secure
                  </Typography>
                  <Typography variant="body2" gutterBottom>
                    Le 3D Secure offre une vérification renforcée pour plus de sécurité
                  </Typography>
                  <RHFSwitch label="Forcer 3D Secure" name="force3ds" />
                </Card>
              </Grid>
            </Grid>
          </Stack>
        </Card>
      </Grid>
    </>
  );

  const renderActions = (
    <>
      {mdUp ? <Grid md={4} /> : <></>}
      <Grid xs={12} md={8} sx={{ display: 'flex', justifyContent: 'end', py: 2 }}>
        <LoadingButton
          type="submit"
          variant="contained"
          size="large"
          loading={isSubmitting}
          sx={{ ml: 2 }}
        >
          Enregistrer
        </LoadingButton>
      </Grid>
    </>
  );

  return (
    <FormProvider methods={methods} onSubmit={onSubmit}>
      <Grid container spacing={3}>
        {renderForm}
        {renderActions}
      </Grid>
    </FormProvider>
  );
}
