'use client';

import React, { useEffect } from 'react';
import {
  Card,
  CardHeader,
  Grid,
  Stack,
  Typography,
  CircularProgress,
  Alert,
  Chip
} from '@mui/material';
import { useResponsive } from '@/hooks';
import * as Yup from 'yup';
import { yupResolver } from '@hookform/resolvers/yup';
import { useForm } from 'react-hook-form';
import FormProvider, { RHFAutocomplete, RHFTextField } from '@/components/hook-form';
import LoadingButton from '@mui/lab/LoadingButton';
import { updatePayPlugPaymentConfig, useGetPayPlugPaymentConfig } from '@/shared/api/payment';
import { useSnackbar } from '@/components/snackbar';
import { PayPlugMode, PayPlugModeLabels } from '@/shared/types/payment';

export default function PayPlugPaymentConfigForm() {
  const mdUp = useResponsive('up', 'md');

  const payPlugPaymentConfigSchema = Yup.object().shape({
    liveSecretKey: Yup.string().required(),
    testSecretKey: Yup.string().required(),
    payPlugMode: Yup.mixed().oneOf(Object.values(PayPlugMode)).required(),
  });

  const { enqueueSnackbar } = useSnackbar();
  const { payPlugPaymentConfigData, payPlugPaymentConfigError, payPlugPaymentConfigLoading } = useGetPayPlugPaymentConfig();

  const methods = useForm({
    resolver: yupResolver(payPlugPaymentConfigSchema),
    defaultValues: {
      liveSecretKey: '',
      testSecretKey: '',
      payPlugMode: PayPlugMode.SANDBOX
    }
  });

  const { setValue, handleSubmit, formState: { isSubmitting } } = methods;

  useEffect(() => {
    if (payPlugPaymentConfigData) {
      setValue('liveSecretKey', payPlugPaymentConfigData.liveSecretKey);
      setValue('testSecretKey', payPlugPaymentConfigData.testSecretKey);
      setValue('payPlugMode', payPlugPaymentConfigData.payPlugMode);
    }
  }, [payPlugPaymentConfigData, setValue]);

  const onSubmit = handleSubmit(async (data) => {
    try {
      const payPlugPaymentConfig = {
        liveSecretKey: data.liveSecretKey,
        testSecretKey: data.testSecretKey,
        payPlugMode: data.payPlugMode as PayPlugMode
      }
      await updatePayPlugPaymentConfig(payPlugPaymentConfig);
      enqueueSnackbar('La configuration de paiement PayPlug a été mise à jour avec succès', { variant: 'success' });
    } catch (error) {
      enqueueSnackbar('Échec de la mise à jour de la configuration de paiement PayPlug. Veuillez réessayer plus tard.', { variant: 'error' });
    }
  });

  if (payPlugPaymentConfigLoading) {
    return (
      <Grid container justifyContent="center" alignItems="center" style={{ minHeight: '100vh' }}>
        <CircularProgress />
      </Grid>
    );
  }

  if (payPlugPaymentConfigError) {
    return (
      <Grid container justifyContent="center" alignItems="center" style={{ minHeight: '100vh' }}>
        <Alert severity="error">Erreur lors du chargement de la configuration de paiement PayPlug. Veuillez réessayer plus tard.</Alert>
      </Grid>
    );
  }

  const renderForm = (
    <>
      {mdUp ? (
        <Grid md={4} paddingY={2}>
          <Typography variant="h6" sx={{ mb: 0.5 }}>
            Configuration de paiement PayPlug
          </Typography>
        </Grid>
      ) : (
        <></>
      )}
      <Grid xs={12} md={8} paddingY={2}>
        <Card>
          {!mdUp ? <CardHeader title="Configuration externe du Paiement via PayPlug" /> : <></>}
          <Stack spacing={3} sx={{ p: 3 }}>
            <Grid container spacing={3}>
              <Grid item xs={12} sm={6}>
                <Card sx={{ p: 2 }} variant='outlined'>
                  <Typography variant="subtitle1" gutterBottom>
                    Live Secret Key
                  </Typography>
                  <Typography variant="body2" gutterBottom sx={{ mb: 2 }}>
                    La clé secrète de production de PayPlug
                  </Typography>
                  <RHFTextField label="Live Secret Key" name="liveSecretKey" placeholder="Clé secrète de production" type="password" />
                </Card>
              </Grid>
              <Grid item xs={12} sm={6}>
                <Card sx={{ p: 2 }} variant='outlined'>
                  <Typography variant="subtitle1" gutterBottom>
                    Sandbox Secret Key
                  </Typography>
                  <Typography variant="body2" gutterBottom sx={{ mb: 2 }}>
                    La clé secrète de test de PayPlug
                  </Typography>
                  <RHFTextField label="Sandbox Secret Key" name="testSecretKey" placeholder="Clé secrète de test" type="password" />
                </Card>
              </Grid>
              <Grid item xs={12} sm={6}>
                <Card sx={{ p: 2 }} variant='outlined'>
                  <Typography variant="subtitle1" gutterBottom>
                    Mode de paiement
                  </Typography>
                  <Typography variant="body2" gutterBottom sx={{ mb: 2 }}>
                    Le mode de paiement PayPlug
                  </Typography>
                  <RHFAutocomplete
                    name="payPlugMode"
                    label="Mode de paiement"
                    placeholder="Mode de paiement"
                    disableClearable
                    options={Object.values(PayPlugMode)}
                    getOptionLabel={(option) => PayPlugModeLabels[option as PayPlugMode]}
                    onChange={(event, value, reason) => setValue('payPlugMode', value as PayPlugMode)}
                    renderOption={(props, option) => (
                      <li {...props} key={option}>
                        {PayPlugModeLabels[option as PayPlugMode]}
                      </li>
                    )}
                    renderTags={(selected, getTagProps) =>
                      selected.map((option, index) => (
                        <Chip
                          {...getTagProps({ index })}
                          key={option}
                          label={PayPlugModeLabels[option as PayPlugMode]}
                          size="small"
                          color="success"
                          variant="soft"
                        />
                      ))
                    }
                  />
                </Card>
              </Grid>
            </Grid>
          </Stack>
        </Card>
      </Grid>
    </>
  );

  const renderActions = (
    <>
      {mdUp ? <Grid md={4} /> : <></>}
      <Grid xs={12} md={8} sx={{ display: 'flex', justifyContent: 'end', py: 2 }}>
        <LoadingButton
          type="submit"
          variant="contained"
          size="large"
          loading={isSubmitting}
          sx={{ ml: 2 }}
        >
          Enregistrer
        </LoadingButton>
      </Grid>
    </>
  );

  return (
    <FormProvider methods={methods} onSubmit={onSubmit}>
      <Grid container spacing={3}>
        {renderForm}
        {renderActions}
      </Grid>
    </FormProvider>
  );
}
