'use client';

import React, { useEffect } from 'react';
import {
  Card,
  CardHeader,
  Grid,
  MenuItem,
  Select,
  Stack,
  Typography,
  CircularProgress,
  Alert
} from '@mui/material';
import { useResponsive } from '@/hooks';
import * as Yup from 'yup';
import { yupResolver } from '@hookform/resolvers/yup';
import { useForm } from 'react-hook-form';
import FormProvider, { RHFSwitch, RHFTextField, RHFSelect } from '@/components/hook-form';
import LoadingButton from '@mui/lab/LoadingButton';
import { updatePaypalConfig, useGetPaypalConfig } from '@/shared/api/payment';
import { useSnackbar } from '@/components/snackbar';
import { IPayPalConfig } from '@/shared/types/payment';

export default function PaypalConfigForm() {
  const mdUp = useResponsive('up', 'md');

  const paypalConfigSchema = Yup.object().shape({
    active: Yup.boolean().required(),
    intent: Yup.string().required(),
    title: Yup.string().required(),
    contingencyFor3DSecure: Yup.string().required()
  });

  const { enqueueSnackbar } = useSnackbar();
  const { paypalConfigData, paypalConfigError, paypalConfigLoading } = useGetPaypalConfig();

  const methods = useForm({
    resolver: yupResolver(paypalConfigSchema),
    defaultValues: {
      active: false,
      intent: 'CAPTURE',
      title: '',
      contingencyFor3DSecure: 'REQUIRED'
    }
  });

  const { setValue, handleSubmit, formState: { isSubmitting } } = methods;

  useEffect(() => {
    if (paypalConfigData) {
      setValue('active', paypalConfigData.active);
      setValue('intent', paypalConfigData.intent);
      setValue('title', paypalConfigData.title);
      setValue('contingencyFor3DSecure', paypalConfigData.contingencyFor3DSecure);
    }
  }, [paypalConfigData, setValue]);

  const onSubmit = handleSubmit(async (data) => {
    try {
      await updatePaypalConfig({ ...paypalConfigData, ...data } as IPayPalConfig);
      enqueueSnackbar('La configuration PayPal a été mise à jour avec succès', { variant: 'success' });
    } catch (error) {
      enqueueSnackbar('Échec de la mise à jour de la configuration PayPal. Veuillez réessayer plus tard.', { variant: 'error' });
    }
  });

  // État de chargement
  if (paypalConfigLoading) {
    return (
      <Grid container justifyContent="center" alignItems="center" style={{ minHeight: '100vh' }}>
        <CircularProgress />
      </Grid>
    );
  }

  // État d'erreur
  if (paypalConfigError) {
    return (
      <Grid container justifyContent="center" alignItems="center" style={{ minHeight: '100vh' }}>
        <Alert severity="error">Erreur lors du chargement de la configuration. Veuillez réessayer plus tard.</Alert>
      </Grid>
    );
  }

  const renderForm = (
    <>
      {mdUp && (
        <Grid md={4} paddingY={2}>
          <Typography variant="h6" sx={{ mb: 0.5 }}>
            Paiement standard : PayPal
          </Typography>
          <Typography variant="body2" sx={{ color: 'text.secondary' }}>
            Acceptez les paiements par solde PayPal.
          </Typography>
        </Grid>
      )}
      <Grid xs={12} md={8} paddingY={2}>
        <Card>
          {!mdUp && <CardHeader title="Mode PayPal" />}
          <Stack spacing={3} sx={{ p: 3 }}>
            <Grid container spacing={3}>
              <Grid item xs={12} sm={6}>
                <Card sx={{ p: 2, minHeight: { sm: 170 } }} variant='outlined'>
                  <Typography variant="subtitle1" gutterBottom>
                    Activer/Désactiver
                  </Typography>
                  <Typography variant="body2" gutterBottom>
                    Activer le mode de paiement PayPal
                  </Typography>
                  <RHFSwitch label="Activer" name="active" />
                </Card>
              </Grid>
              <Grid item xs={12} sm={6}>
                <Card sx={{ p: 2 }} variant='outlined'>
                  <Typography variant="subtitle1" gutterBottom>
                    Titre
                  </Typography>
                  <Typography variant="body2" gutterBottom>
                    Contrôler le titre que l'utilisateur voit lors du paiement
                  </Typography>
                  <RHFTextField label="Titre" name="title" placeholder="Titre" />
                </Card>
              </Grid>
              <Grid item xs={12} sm={6}>
                <Card sx={{ p: 2, minHeight: { sm: 170 } }} variant='outlined'>
                  <Typography variant="subtitle1" gutterBottom>
                    Intent
                  </Typography>
                  <Typography variant="body2" gutterBottom>
                    Capturer le paiement immédiatement ou autoriser un paiement pour une commande après la création de la commande
                  </Typography>
                  <RHFSelect name="intent" label="Intent" fullWidth>
                    <MenuItem value="CAPTURE">Capturer</MenuItem>
                    <MenuItem value="AUTHORIZE">Autoriser</MenuItem>
                  </RHFSelect>
                </Card>
              </Grid>
              <Grid item xs={12} sm={6}>
                <Card sx={{ p: 2 }} variant='outlined'>
                  <Typography variant="subtitle1" gutterBottom>
                    3D Secure
                  </Typography>
                  <Typography variant="body2" gutterBottom>
                    Le 3D Secure offre une vérification renforcée pour plus de sécurité
                  </Typography>
                  <RHFSelect name="contingencyFor3DSecure" label="3D Secure" fullWidth>
                    <MenuItem value="REQUIRED">3D Secure requis</MenuItem>
                    <MenuItem value="ALWAYS">Toujours utiliser 3D Secure</MenuItem>
                    <MenuItem value="NEVER">Ne jamais utiliser 3D Secure</MenuItem>
                  </RHFSelect>
                </Card>
              </Grid>
            </Grid>
          </Stack>
        </Card>
      </Grid>
    </>
  );

  const renderActions = (
    <>
      {mdUp && <Grid md={4} />}
      <Grid xs={12} md={8} sx={{ display: 'flex', justifyContent: 'end', py: 2 }}>
        <LoadingButton
          type="submit"
          variant="contained"
          size="large"
          loading={isSubmitting}
          sx={{ ml: 2 }}
        >
          Enregistrer
        </LoadingButton>
      </Grid>
    </>
  );

  return (
    <FormProvider methods={methods} onSubmit={onSubmit}>
      <Grid container spacing={3}>
        {renderForm}
        {renderActions}
      </Grid>
    </FormProvider>
  );
}
