'use client';

import { useState, useCallback, useEffect } from 'react';

import Card from '@mui/material/Card';
import Table from '@mui/material/Table';
import Grid from '@mui/material/Grid';
import Container from '@mui/material/Container';
import TableBody from '@mui/material/TableBody';
import TableCell from '@mui/material/TableCell';
import TableHead from '@mui/material/TableHead';
import TableContainer from '@mui/material/TableContainer';
import { Reorder } from 'framer-motion';

import LocalizationProvider from '@/shared/locales/localization-provider';
import { paths } from '@/routes/paths';
import { useRouter } from '@/hooks';
import Scrollbar from '@/shared/components/scrollbar';
import CustomBreadcrumbs from '@/components/custom-breadcrumbs';
import { useSettingsContext } from '@/shared/components/settings';
import {
  useTable,
  emptyRows,
  TableNoData,
  TablePaginationCustom,
  TableEmptyRows,
} from '@/shared/components/table';
import PaymentMethodTableRow from './methods-table-row';
import { IPaymentMethod } from '@/shared/types/payment';
import { useGetPaymentMethods } from '@/shared/api/payment';
import CircularProgress from '@mui/material/CircularProgress';
import Alert from '@mui/material/Alert';

export default function PaymentMethodsView() {
  const settings = useSettingsContext();
  const router = useRouter();
  const table = useTable({ defaultOrderBy: 'title' });

  const { paymentMethodsData, paymentMethodsError, paymentMethodsLoading } = useGetPaymentMethods();
  const [tableData, setTableData] = useState<IPaymentMethod[]>([]);

  const denseHeight = table.dense ? 56 : 76;

  const notFound = !tableData.length;

  useEffect(() => {
    if (paymentMethodsData) {
      setTableData(paymentMethodsData);
    }
  }, [paymentMethodsData]);

  

  const handleReorder = useCallback(
    (newOrder: number[]) => {
      const reorderedData = newOrder.map(index => tableData[index]);
      setTableData(reorderedData);
    },
    [tableData]
  );

  if (paymentMethodsLoading) {
    return (
      <Container maxWidth={settings.themeStretch ? false : 'lg'}>
        <Grid container justifyContent="center" alignItems="center" style={{ minHeight: '100vh' }}>
          <CircularProgress />
        </Grid>
      </Container>
    );
  }

  if (paymentMethodsError) {
    return (
      <Container maxWidth={settings.themeStretch ? false : 'lg'}>
        <Alert severity="error">Failed to load payment methods. Please try again later.</Alert>
      </Container>
    );
  }

  return (
    <LocalizationProvider >
      <Container maxWidth={settings.themeStretch ? false : 'lg'}>
        <Grid container alignItems="center" justifyContent="space-between">
          <Grid item>
            <CustomBreadcrumbs
              heading="Méthodes de paiements"
              links={[
                {
                  name: 'dashboard',
                  href: paths.dashboard.root,
                },
                {
                  name: "Paiement",
                  href: paths.dashboard.admin.payment.root,
                }
              ]}
              sx={{
                mb: { xs: 3, md: 5 },
              }}
            />
          </Grid>
        </Grid>

        <Card>
          <TableContainer sx={{ position: 'relative', overflow: 'unset', maxWidth: '1420px' }}>
            <TableHead  >
              <TableCell sx={{ width: '300px' }} align='center' >Nom</TableCell>
              <TableCell sx={{ width: '300px', pl: '50px' }}  align='center'  >Activé</TableCell>
              <TableCell sx={{ width: '520px' }} align='center'>Description</TableCell>
              <TableCell sx={{ width: '300px' }} align='center' >Actions</TableCell>
            </TableHead>

            <Scrollbar>
              <Table size={table.dense ? 'small' : 'medium'} sx={{ minWidth: 800 }}>
                <TableBody>
                  <Reorder.Group
                    axis="y" // Vertical reordering
                    values={tableData.map((_, index) => index)}
                    onReorder={handleReorder}
                  >
                    {tableData
                      .slice(
                        table.page * table.rowsPerPage,
                        table.page * table.rowsPerPage + table.rowsPerPage
                      )
                      .map((row, index) => (
                        <Reorder.Item key={row.id} value={index}>
                          <PaymentMethodTableRow
                            key={row.id}
                            row={row}
                          />
                        </Reorder.Item>
                      ))}
                  </Reorder.Group>
                  <TableEmptyRows
                    height={denseHeight}
                    emptyRows={emptyRows(table.page, table.rowsPerPage, tableData.length)}
                  />

                  <TableNoData notFound={notFound} />
                </TableBody>
              </Table>
            </Scrollbar>
          </TableContainer>
          <TablePaginationCustom
            count={tableData.length}
            page={table.page}
            rowsPerPage={table.rowsPerPage}
            onPageChange={table.onChangePage}
            onRowsPerPageChange={table.onChangeRowsPerPage}
            dense={table.dense}
            onChangeDense={table.onChangeDense}
          />
        </Card>
      </Container>
    </LocalizationProvider>
  );
}
