import { useState } from "react";
import { useFormik } from "formik";
import * as Yup from "yup";
import TextField from "@mui/material/TextField";
import Button from "@mui/material/Button";
import Stack from "@mui/material/Stack";
import Typography from "@mui/material/Typography";
import Card from "@mui/material/Card";
import { useAddColor, useGetColorsByAttribut, checkSlugExistenceColor } from "@/shared/api/terms";
import { IColorRequest } from "@/shared/types/term";
import { Alert, Snackbar } from "@mui/material";

//------------------------------------------------------
export default function AddTermeColorForm({
  attributid,
}: {
  attributid: string;
}) {
  const { ColorsMutate } = useGetColorsByAttribut(attributid);
  const [openSnackbar, setOpenSnackbar] = useState<boolean>(false);
  const [snackbarMessage, setSnackbarMessage] = useState<string>("");
  const [showSuccessMessage, setShowSuccessMessage] = useState(false);

  const handleCloseSnackbar = () => {
    setOpenSnackbar(false);
  };

  const formik = useFormik({
    initialValues: {
      nom: "",
      slug: "",
      description: "",
      color: "#F0FFF0",
    },
    validationSchema: Yup.object({
      nom: Yup.string()
        .matches(/^[a-zA-Z0-9]+(?:[- ][a-zA-Z0-9]+)*$/, "Nom ne doit contenir que des lettres, des chiffres et des tirets.")
        .required("Nom est requis"),
      slug: Yup.string()
        .matches(
          /^[a-zA-Z0-9]+(?:[a-zA-Z0-9- ]*[a-zA-Z0-9]+)*$/,
          "Le slug doit commencer par une lettre ou un chiffre, peut contenir des tirets, et ne doit pas contenir uniquement des caractères spéciaux."
        )
        .required("Slug est requis")
        .test("slug-unique", "Ce slug existe déjà.", async (slug) => {
          if (slug) {
            const exists = await checkSlugExistenceColor(slug, attributid);
            return !exists; // Retourne `false` si le slug existe déjà, déclenchant l'erreur
          }
          return true; // Retourne `true` si le champ `slug` est vide (ou autre condition que vous souhaitez)
        }),
      color: Yup.string().required("Veuillez sélectionner une couleur."),
    }),
    onSubmit: async (values) => {
      const colorRequest: IColorRequest = {
        nom: values.nom,
        slug: values.slug,
        description: values.description,
        color: values.color,
        attributeId: attributid,
      };

      try {
        await useAddColor(colorRequest);
        ColorsMutate();
        setShowSuccessMessage(true);
        const timeoutId = setTimeout(() => {
          setShowSuccessMessage(false);
          clearTimeout(timeoutId);
        }, 5000);
      } catch (error) {
        let errorMessage;
        if (
          typeof error === "string" &&
          error.includes("A Color with this name already exists")
        ) {
          errorMessage = "Le slug doit également être unique.";
        } else if (
          error instanceof Error &&
          error.message.includes("A Color with this name already exists")
        ) {
          errorMessage = "Le slug doit également être unique.";
        }
        setSnackbarMessage(errorMessage ?? "La couleur n'a pas pu être créée.");
        setOpenSnackbar(true);
      } finally {
        const timeoutId = setTimeout(() => {
          setShowSuccessMessage(false);
          clearTimeout(timeoutId);
        }, 5000);
      }
    },
  });

  return (
    <Card variant="outlined">
      <form onSubmit={formik.handleSubmit}>
        <Stack spacing={1.5} sx={{ p: 3 }}>
          <Typography variant="subtitle2">Ajouter le Terme</Typography>

          <TextField
            name="nom"
            label="Nom"
            InputLabelProps={{ shrink: true }}
            value={formik.values.nom}
            onChange={formik.handleChange}
            onBlur={formik.handleBlur}
            error={formik.touched.nom && Boolean(formik.errors.nom)}
            helperText={formik.touched.nom && formik.errors.nom}
          />

          <TextField
            label="Slug"
            name="slug"
            InputLabelProps={{ shrink: true }}
            value={formik.values.slug}
            onChange={formik.handleChange}
            onBlur={formik.handleBlur}
            error={formik.touched.slug && Boolean(formik.errors.slug)}
            helperText={formik.touched.slug && formik.errors.slug}
          />

          <TextField
            name="description"
            label="Description"
            multiline
            rows={4}
            InputLabelProps={{ shrink: true }}
            value={formik.values.description}
            onChange={formik.handleChange}
            onBlur={formik.handleBlur}
            error={formik.touched.description && Boolean(formik.errors.description)}
            helperText={formik.touched.description && formik.errors.description}
          />

          <TextField
            type="color"
            name="color"
            value={formik.values.color}
            onChange={formik.handleChange}
            onBlur={formik.handleBlur}
          />

          <Button
            type="submit"
            variant="contained"
            color="primary"
          >
            Ajouter
          </Button>
        </Stack>
        <Snackbar
          open={openSnackbar}
          autoHideDuration={6000}
          onClose={handleCloseSnackbar}
        >
          <Alert onClose={handleCloseSnackbar} severity="error">
            {snackbarMessage}
          </Alert>
        </Snackbar>
        {showSuccessMessage && (
          <Alert
            severity="success"
            onClose={() => setShowSuccessMessage(false)}
          >
            la couleur a été ajoutée avec succès
          </Alert>
        )}
      </form>
    </Card>
  );
}
