import { useEffect, useState } from "react";
import TextField from "@mui/material/TextField";
import Button from "@mui/material/Button";
import Stack from "@mui/material/Stack";
import Typography from "@mui/material/Typography";
import Card from "@mui/material/Card";
import { useGetColor, useGetColorsByAttribut, useUpdateColor, checkSlugExistenceColor } from "@/shared/api/terms";
import { IColorRequest } from "@/shared/types/term";
import { Alert, Snackbar } from "@mui/material";
import * as yup from "yup"; // Import de yup

//------------------------------------------------------
export default function EditTermColorForm({
  attributid,
  colorId,
}: {
  attributid: string;
  colorId: string;
}) {
  const { ColorsMutate } = useGetColorsByAttribut(attributid);
  const { ColorData } = useGetColor(colorId);

  const [nom, setNom] = useState<string>("");
  const [slug, setSlug] = useState<string>("");
  const [description, setDescription] = useState<string>("");
  const [color, setColor] = useState("#F0FFF0");

  const formSetter = () => {
    setNom(ColorData.nom);
    setSlug(ColorData.slug);
    setDescription(ColorData.description);
    setColor(ColorData.color);
  };

  useEffect(() => {
    if (ColorData) {
      formSetter();
    }
  }, [ColorData]);

  // States for Snackbar
  const [openSnackbar, setOpenSnackbar] = useState<boolean>(false);
  const [snackbarMessage, setSnackbarMessage] = useState<string>("");
  const [snackbarSeverity, setSnackbarSeverity] = useState<"error" | "success">("success");

  const handleCloseSnackbar = () => {
    setOpenSnackbar(false);
  };

  const validationSchema = yup.object({
    nom: yup
      .string()
      .matches(/^[a-zA-Z0-9]+(?:[- ][a-zA-Z0-9]+)*$/, "Le nom ne doit contenir que des lettres, des chiffres et des tirets.")
      .required("Le nom est requis."),
    slug: yup
      .string()
      .matches(/^[a-zA-Z0-9]+(?:[a-zA-Z0-9- ]*[a-zA-Z0-9]+)*$/, "Le slug doit commencer par une lettre ou un chiffre, peut contenir des tirets.")
      .required("Le slug est requis()")
      .test("unique-slug", "Ce slug existe déjà.", async (value) => {
        if (value !== ColorData.slug) { // Si le slug a été modifié
          const exists = await checkSlugExistenceColor(value, attributid);
          return !exists; // Retourne true si le slug n'existe pas, false s'il existe déjà
        }
        return true; // Si le slug n'a pas été modifié, on ne vérifie pas l'unicité
      }),
    description: yup.string().optional(),
    color: yup.string().required("Veuillez sélectionner une couleur."),
  });

  const validateForm = async (): Promise<boolean> => {
    try {
      await validationSchema.validate({ nom, slug, description, color }, { abortEarly: false });
      return true;
    } catch (err) {
      if (err instanceof yup.ValidationError) {
        setSnackbarMessage(err.errors[0]); // Affiche la première erreur
        setSnackbarSeverity("error"); // Met la couleur en rouge pour l'erreur
        setOpenSnackbar(true);
      }
      return false;
    }
  };

  // Vérification du slug au moment du blur
  const handleSlugBlur = async (e: React.FocusEvent<HTMLInputElement>) => {
    setSlug(e.target.value);
  };

  const handleOnSubmit = async (event: React.FormEvent<HTMLFormElement>) => {
    event.preventDefault();
    const isValid = await validateForm();
    if (!isValid) {
      return;
    }
    const colorRequest: IColorRequest = {
      nom,
      slug,
      description,
      color,
      attributeId: attributid,
    };

    try {
      await useUpdateColor(colorRequest, colorId);
      ColorsMutate();
      setSnackbarMessage("La couleur a été modifiée avec succès");
      setSnackbarSeverity("success"); // Change la couleur en vert pour succès
      setOpenSnackbar(true);
    } catch (error) {
      let errorMessage;
      if (
        typeof error === "string" &&
        error.includes("A Color with this name already exists")
      ) {
        errorMessage = "Le slug doit également être unique.";
      } else if (
        error instanceof Error &&
        error.message.includes("A Color with this name already exists")
      ) {
        errorMessage = "Le slug doit également être unique.";
      }
      setSnackbarMessage(errorMessage ?? "La couleur n'a pas pu être modifiée.");
      setSnackbarSeverity("error"); // Met la couleur en rouge en cas d'erreur
      setOpenSnackbar(true);
    }
  };

  return (
    <Card variant="outlined">
      <form onSubmit={handleOnSubmit}>
        <Stack spacing={1.5} sx={{ p: 3 }}>
          <Typography variant="subtitle2">Modifier le Terme</Typography>

          <TextField
            name="nom"
            label="Nom"
            InputLabelProps={{ shrink: true }}
            value={nom}
            onChange={(e) => setNom(e.target.value)}
          />

          <TextField
            label="Slug"
            name="slug"
            InputLabelProps={{ shrink: true }}
            value={slug}
            onChange={(e) => setSlug(e.target.value)}
            onBlur={handleSlugBlur} // Vérification du slug lors du blur
          />

          <TextField
            name="description"
            label="Description"
            multiline
            rows={4}
            InputLabelProps={{ shrink: true }}
            value={description}
            onChange={(e) => setDescription(e.target.value)}
          />

          <TextField
            id="outlined-valeur-code"
            type="color"
            label="Couleur"
            variant="outlined"
            InputLabelProps={{ shrink: true }}
            value={color}
            style={{ marginBottom: "40px" }}
            onChange={(e) => setColor(e.target.value)}
          />
          <Button type="submit" variant="contained" color="primary">
            Modifier
          </Button>
        </Stack>
        <Snackbar open={openSnackbar} autoHideDuration={6000} onClose={handleCloseSnackbar}>
          <Alert onClose={handleCloseSnackbar} severity={snackbarSeverity}>
            {snackbarMessage}
          </Alert>
        </Snackbar>
      </form>
    </Card>
  );
}
