"use client";

import { useState, useCallback } from "react";
import Container from "@mui/material/Container";
import Stack from "@mui/material/Stack";
import Grid from "@mui/material/Grid";
import Button from "@mui/material/Button";
import Iconify from "@/shared/components/iconify";
import { paths } from "@/routes/paths";
import CustomBreadcrumbs from "@/shared/components/custom-breadcrumbs";
import Card from "@mui/material/Card";
import { useBoolean } from "@/hooks/use-boolean";
import ConfirmDialog from "@/shared/components/confirm-dialog";
import AddTermeImage from "./form-termeImage";
import { useRouter } from "@/hooks";
import { useDeleteImage, useDeleteImages, useGetImageByAttribut } from "@/shared/api/terms";

import {
  DataGrid,
  GridColDef,
  GridActionsCellItem,
  GridToolbarQuickFilter,
  GridToolbarExport,
  GridToolbarContainer,
  GridRowSelectionModel,
  GridToolbarFilterButton,
} from "@mui/x-data-grid";

import { Alert, Avatar, CircularProgress, Typography } from "@mui/material";
import { Description } from "@mui/icons-material";

//----------------------------------------------------------------------------------------
export default function TermeImage({ attributid }: { attributid: string }) {
  const [selectedRowIds, setSelectedRowIds] = useState<GridRowSelectionModel>([]);
  const [errorMessage, setErrorMessage] = useState<string | null>(null);
  const [successMessage, setSuccessMessage] = useState<string | null>(null);
  const [rowToDelete, setRowToDelete] = useState<number | null>(null);

  const { ImagesData, ImagesLoading, ImagesError, ImagesMutate } = useGetImageByAttribut(attributid);
  const router = useRouter();
  const handleEditRow = useCallback(
    (id: string) => {
      router.push(paths.dashboard.admin.product.termeImage.editImage(attributid, id));
    },
    [router]
  );
  const confirmRows = useBoolean();
  const confirmSingleRow = useBoolean();

  const handleRetry = async () => {
    try {
      await ImagesMutate();
    } catch (err) {
      setErrorMessage("Erreur lors du rechargement des termes Images.");
      setTimeout(() => {
        setErrorMessage(null);
      }, 5000);
    }
  };

  const handleDeleteRow = async (id: number) => {
    try {
      const response = await useDeleteImage(id.toString(), attributid);
      ImagesMutate();

      if (response.status === 204) {
        setSelectedRowIds([]);
        setSuccessMessage("Image supprimée avec succès.");
      }
    } catch (error) {
      setErrorMessage("Impossible de supprimer une terme liée à un produit");
    } finally {
      const timeoutId = setTimeout(() => {
        setSuccessMessage(null);
        setErrorMessage(null);
        clearTimeout(timeoutId);
      }, 5000);
    }
  };

  const handleDeleteRows = async () => {
    try {
      if (selectedRowIds.length > 0) {
        const idsToDelete = selectedRowIds.map((id) => id.toString());
        const response = await useDeleteImages(attributid, idsToDelete);
        setSelectedRowIds([]);
        ImagesMutate();

        if (response.status === 200) {
          setSuccessMessage("Images(s) supprimée(s) avec succès.");
        }
      }
    } catch (error) {
      setErrorMessage("Impossible de supprimer les termes liée à un produit");
    } finally {
      const timeoutId = setTimeout(() => {
        setSuccessMessage(null);
        setErrorMessage(null);
        clearTimeout(timeoutId);
      }, 5000);
    }
  };

  const columns: GridColDef[] = [
    {
      field: "Preview",
      headerName: "Preview",
      flex: 1,
      renderCell: (params) => (
        <Avatar
          src={params.row.Preview}
          variant="rounded"
          sx={{ width: 64, height: 64, mr: 2 }}
        />
      ),
    },
    {
      field: "name",
      headerName: "Nom",
      flex: 1,
      editable: true,
    },
    {
      field: "slug",
      headerName: "Slug",
      flex: 1,
      editable: true,
    },
    {
      field: "description",
      headerName: "Description",
      flex: 1,
      editable: true,
    },
    {
      field: "total",
      headerName: "Total",
      flex: 1,
      editable: true,
    },
    {
      type: "actions",
      field: "actions",
      headerName: " ",
      align: "right",
      headerAlign: "right",
      width: 60,
      sortable: false,
      filterable: false,
      disableColumnMenu: true,
      getActions: (params) => [
        <GridActionsCellItem
          showInMenu
          icon={<Iconify icon="solar:pen-bold" />}
          label="Modifier"
          onClick={() => handleEditRow(params.row.id)}
        />,

        <GridActionsCellItem
          showInMenu
          icon={<Iconify icon="solar:trash-bin-trash-bold" />}
          label="Supprimer"
          onClick={() => {
            setRowToDelete(params.row.id);
            confirmSingleRow.onTrue();
          }}
          sx={{ color: "error.main" }}
        />,
      ],
    },
  ];

  const SortedImageData = ImagesData.sort((a, b) => {
    return new Date(b.createdAt ?? 0).getTime() - new Date(a.createdAt ?? 0).getTime();
  });

  const rows = SortedImageData.map((image) => ({
    id: image.id,
    Preview: image.image,
    name: image.nom,
    slug: image.slug,
    description: image.description,
    total: image.total,
  }));

  return (
    <Container
      maxWidth="xl"
      sx={{
        flexGrow: 1,
        display: "flex",
        flexDirection: "column",
      }}
    >
      <CustomBreadcrumbs
        heading="Attributs"
        links={[
          { name: "Tableau de bord", href: paths.dashboard.root },
          {
            name: "Product",
            href: paths.dashboard.admin.product.root,
          },
          { name: "Attribut", href: paths.dashboard.admin.product.attribut },
          {
            name: "Images",
            href: paths.dashboard.admin.product.termeImage.root(attributid),
          },
        ]}
        sx={{
          mb: {
            xs: 3,
            md: 5,
          },
        }}
      />

      <Grid container spacing={3}>
        <Grid item xs={4}>
          <AddTermeImage attributid={attributid} />
        </Grid>

        <Grid item xs={8}>
          {ImagesLoading ? (
            <Stack
              direction="row"
              spacing={2}
              justifyContent="center"
              alignItems="center"
              sx={{ height: "100%", width: "100%" }}
            >
              <CircularProgress />
              <Typography>Chargement des termes Images...</Typography>
            </Stack>
          ) : ImagesError ? (
            <Container>
              <Alert severity="error" sx={{ mb: 2 }}>
                Impossible de charger la liste des termes images
              </Alert>
              <Button variant="contained" color="primary" onClick={handleRetry}>
                Réessayer
              </Button>
            </Container>
          ) : (
            <Card variant="outlined">
              <DataGrid
                checkboxSelection
                autoHeight
                getRowHeight={() => "auto"}
                rows={rows}
                columns={columns}
                disableColumnSelector
                disableDensitySelector
                pageSizeOptions={[5, 10, 25]}
                initialState={{
                  pagination: {
                    paginationModel: { pageSize: 10 },
                  },
                }}
                onRowSelectionModelChange={(newSelectionModel) => {
                  setSelectedRowIds(newSelectionModel);
                }}
                slots={{
                  toolbar: () => (
                    <GridToolbarContainer>
                      <GridToolbarFilterButton />
                      <GridToolbarExport />
                      <GridToolbarQuickFilter />
                      <Stack
                        spacing={1}
                        flexGrow={1}
                        direction="row"
                        alignItems="center"
                        justifyContent="flex-end"
                      >
                        {!!selectedRowIds.length && (
                          <Stack>
                            <Button
                              size="small"
                              color="error"
                              startIcon={<Iconify icon="solar:trash-bin-trash-bold" />}
                              onClick={confirmRows.onTrue}
                            >
                              Supprimer ({selectedRowIds.length})
                            </Button>
                          </Stack>
                        )}
                      </Stack>
                    </GridToolbarContainer>
                  ),
                }}
                slotProps={{
                  toolbar: {
                    showQuickFilter: true,
                  },
                }}
              />
              {errorMessage && (
                <Alert severity="error" onClose={() => setErrorMessage(null)}>
                  {errorMessage}
                </Alert>
              )}
              {successMessage && (
                <Alert severity="success" onClose={() => setSuccessMessage(null)}>
                  {successMessage}
                </Alert>
              )}
            </Card>
          )}
          <ConfirmDialog
            open={confirmRows.value}
            onClose={confirmRows.onFalse}
            title="Supprimer"
            content={
              <>
                Êtes-vous sûr de vouloir supprimer <strong> {selectedRowIds.length} </strong> items?
              </>
            }
            action={
              <Button
                variant="contained"
                color="error"
                onClick={() => {
                  handleDeleteRows();
                  confirmRows.onFalse();
                }}
              >
                Supprimer
              </Button>
            }
          />
          <ConfirmDialog
            open={confirmSingleRow.value}
            onClose={confirmSingleRow.onFalse}
            title="Supprimer"
            content={
              <>
                Êtes-vous sûr de vouloir supprimer cet item?
              </>
            }
            action={
              <Button
                variant="contained"
                color="error"
                onClick={() => {
                  if (rowToDelete !== null) {
                    handleDeleteRow(rowToDelete);
                    confirmSingleRow.onFalse();
                  }
                }}
              >
                Supprimer
              </Button>
            }
          />
        </Grid>
      </Grid>
    </Container>
  );
}
