"use client";
import { useEffect, useState } from "react";
import TextField from "@mui/material/TextField";
import Button from "@mui/material/Button";
import Stack from "@mui/material/Stack";
import Typography from "@mui/material/Typography";
import Card from "@mui/material/Card";
import { AccessLevel, FileType, IMedia, Visibility } from "@/shared/types/media";
import { Alert, Grid, IconButton, Snackbar } from "@mui/material";
import {
  useGetImageByAttribut,
  useUpdateImage,
  useGetImage,
  checkSlugExistenceImage,
} from "@/shared/api/terms";
import { IImage } from "@/shared/types/term";
import CloseIcon from "@mui/icons-material/Close";
import AddMediaButton from "../../media/Add-media-button";
import * as yup from "yup";

function extractUrl(str: string | null) {
  if (!str) {
    return null;
  }
  const match = str.match(/\[(.*?)\]/);
  return match ? match[1] : str; // Returns full string if no brackets found
}

export default function EditTermeImageForm({
  attributid,
  imageId,
}: {
  attributid: string;
  imageId: string;
}) {

  const { ImagesMutate } = useGetImageByAttribut(attributid);
  const { ImageData, ImageMutate } = useGetImage(imageId);
  const [returnedMultiMedia, setReturnedMultiMedia] = useState<IMedia[]>([]);

  const [nom, setNom] = useState<string>("");
  const [slug, setSlug] = useState<string>("");
  const [description, setDescription] = useState<string>("");
  const [initialImageLoaded, setInitialImageLoaded] = useState<boolean>(false);
  const [isEditing, setIsEditing] = useState<boolean>(false);

  const formSetter = () => {
    setNom(ImageData.nom);
    setSlug(ImageData.slug);
    setDescription(ImageData.description);
    if (ImageData.image && !initialImageLoaded) {
      setReturnedMultiMedia([{
        id: "",
        title: "",
        description: "",
        name: "",
        createdAt: "",
        modifiedAt: "",
        size: 0,
        userId: "",
        fileType: FileType.IMAGE,
        visibility: Visibility.PUBLIC,
        permissions: AccessLevel.READ,
        path: "",
        uniqueId: "",
        url: ImageData.image,
      }]);
      setInitialImageLoaded(true);
    }
  };

  useEffect(() => {

    if (ImageData) {
      formSetter();
    }
  }, [ImageData]);

  const [openSnackbar, setOpenSnackbar] = useState<boolean>(false);
  const [snackbarMessage, setSnackbarMessage] = useState<string>("");
  const [showSuccessMessage, setShowSuccessMessage] = useState(false);

  const handleCloseSnackbar = () => {
    setOpenSnackbar(false);
  };




  const validationSchema = yup.object({
    nom: yup
      .string()
      .matches(/^[a-zA-Z0-9]+(?:[- ][a-zA-Z0-9]+)*$/, "Le nom ne doit contenir que des lettres, des chiffres et des tirets.")
      .required("Le nom est requis."),
    slug: yup
      .string()
      .matches(/^[a-zA-Z0-9]+(?:[a-zA-Z0-9- ]*[a-zA-Z0-9]+)*$/, "Le slug doit commencer par une lettre ou un chiffre, peut contenir des tirets.")
      .required("Le slug est requis()")
      .test("unique-slug", "Ce slug existe déjà.", async (value) => {
        if (value !== ImageData.slug) { // Si le slug a été modifié
          const exists = await checkSlugExistenceImage(value, attributid);
          return !exists; // Retourne true si le slug n'existe pas, false si il existe déjà
        }
        return true; // Si le slug n'a pas été modifié, on ne vérifie pas l'unicité
      }),
    description: yup.string().optional(),
  });

  const validateForm = async (): Promise<boolean> => {
    try {
      await validationSchema.validate({ nom, slug, description}, { abortEarly: false });
      return true;
    } catch (err) {
      if (err instanceof yup.ValidationError) {
        setSnackbarMessage(err.errors[0]); // Affiche la première erreur
        setOpenSnackbar(true);
      }
      return false;
    }
  };

  const handleOnSubmit = async (event: React.FormEvent<HTMLFormElement>) => {
    event.preventDefault();
    const isValid = await validateForm();
    if (!isValid) {
      return;
    }
    const imageURL = returnedMultiMedia.length > 0 ? returnedMultiMedia[0].url : null;

    const imageRequest: IImage = {
      nom,
      slug,
      description,
      image: extractUrl(imageURL),
      attributeId: attributid,
    };

    try {
      await useUpdateImage(imageRequest, imageId);
      ImagesMutate();
      ImageMutate()
      setShowSuccessMessage(true);
      const timeoutId = setTimeout(() => {
        setShowSuccessMessage(false);
        clearTimeout(timeoutId);
      }, 5000);
    } catch (error) {
      let errorMessage;
      if (
        typeof error === "string" &&
        error.includes("An Image with this name already exists")
      ) {
        errorMessage = "Le slug doit également être unique.";
      } else if (
        error instanceof Error &&
        error.message.includes("An Image with this name already exists")
      ) {
        errorMessage = "Le slug doit également être unique.";
      }
      setSnackbarMessage(errorMessage ?? "L'image n'a pas pu être modifiée.");
      setOpenSnackbar(true);
    }
  };

  return (
    <Card variant="outlined">
      <form onSubmit={handleOnSubmit}>
        <Stack spacing={1.5} sx={{ p: 3 }}>
          <Typography variant="subtitle2">Modifier le Terme</Typography>

          <TextField
            name="nom"
            label="Nom *"
            InputLabelProps={{ shrink: true }}
            value={nom}
            onChange={(e) => setNom(e.target.value)}
          />

          <TextField
            label="Slug"
            name="slug"
            InputLabelProps={{ shrink: true }}
            value={slug}
            onChange={(e) => setSlug(e.target.value)}
          />

          <TextField
            name="description"
            label="Description *"
            multiline
            rows={4}
            InputLabelProps={{ shrink: true }}
            value={description}
            onChange={(e) => setDescription(e.target.value)}
          />

          <Stack spacing={1.5}>
            <Grid container spacing={1} mt={1}>
              <AddMediaButton
                variant="outlined"
                color="success"
                size="medium"
                sx={{ mt: 1 }}
                nameBtn="Modifier l'image"
                returnedMultiMedia={returnedMultiMedia}
                setReturnedSingleMedia={(media) => {
                  setReturnedMultiMedia(media);
                  setIsEditing(true); // Activer le mode édition lorsque l'utilisateur clique
                }}
              />
            </Grid>
            {returnedMultiMedia.length > 0 && !isEditing && (
              <Grid md={12} xs={12} style={{ display: 'flex', flexWrap: 'wrap' }}>
                <Typography
                  component="span"
                  style={{
                    fontSize: '15px',
                    color: 'text.secondary',
                    marginTop: 4,
                    width: '100%'
                  }}
                >
                  {returnedMultiMedia.length} Média sélectionné
                </Typography>
                {returnedMultiMedia.map((media) => (
                  <Grid key={media.id} style={{ margin: '10px', position: 'relative' }}>
                    <img src={extractUrl(media.url) ?? undefined} alt="image" style={{ width: '100px', height: '100px', objectFit: 'contain' }} />
                    <IconButton onClick={() => setReturnedMultiMedia([])} style={{ position: 'absolute', top: 0, right: 0 }}>
                      <CloseIcon />
                    </IconButton>
                  </Grid>
                ))}
              </Grid>
            )}
          </Stack>

          <Button type="submit" variant="contained" color="primary">
            Modifier
          </Button>
        </Stack>
        <Snackbar
          open={openSnackbar}
          autoHideDuration={6000}
          onClose={handleCloseSnackbar}
        >
          <Alert onClose={handleCloseSnackbar} severity="error">
            {snackbarMessage}
          </Alert>
        </Snackbar>
        {showSuccessMessage && (
          <Alert
            severity="success"
            onClose={() => setShowSuccessMessage(false)}
          >
            L'image a été modifiée avec succès
          </Alert>
        )}
      </form>
    </Card>
  );
}
