"use client";
import { useEffect, useState } from "react";
import { useFormik } from "formik";
import * as Yup from "yup";
import TextField from "@mui/material/TextField";
import Button from "@mui/material/Button";
import Stack from "@mui/material/Stack";
import Typography from "@mui/material/Typography";
import Avatar from "@mui/material/Avatar";
import Card from "@mui/material/Card";
import { IImage } from "@/shared/types/term";
import { Alert, Grid, Snackbar } from "@mui/material";
import { useAddImage, useGetImageByAttribut, checkSlugExistenceImage } from "@/shared/api/terms";
import AddMediaButton from "../../media/Add-media-button";
import { IMedia } from "@/shared/types/media";

function extractUrl(str: string | null) {
  if (!str) {
    return null;
  }
  const match = str.match(/\[(.*?)\]/);
  return match ? match[1] : null;
}

export default function AddTermeImage({ attributid }: { attributid: string }) {
  const { ImagesMutate } = useGetImageByAttribut(attributid);

  const [returnedMultiMedia, setReturnedMultiMedia] = useState<IMedia[] | undefined>(undefined);

  const [openSnackbar, setOpenSnackbar] = useState<boolean>(false);
  const [snackbarMessage, setSnackbarMessage] = useState<string>("");
  const [showSuccessMessage, setShowSuccessMessage] = useState(false);

  const handleCloseSnackbar = () => {
    setOpenSnackbar(false);
  };

  const validationSchema = Yup.object({
    nom: Yup.string()
      .matches(
        /^[a-zA-Z0-9]+(?:[- ][a-zA-Z0-9]+)*$/,
        "Nom ne doit contenir que des lettres, des chiffres et des tirets."
      )
      .required("Nom est requis"),
    slug: Yup.string()
      .matches(
        /^[a-zA-Z0-9]+(?:[a-zA-Z0-9- ]*[a-zA-Z0-9]+)*$/,
        "Le slug doit commencer par une lettre ou un chiffre, peut contenir des tirets, et ne doit pas contenir uniquement des caractères spéciaux."
      )
      .required("Slug est requis")
      .test("slug-unique", "Ce slug existe déjà.", async (slug) => {
        if (slug) {
          const exists = await checkSlugExistenceImage(slug, attributid);
          return !exists; 
        }
        return true; 
      }),
    image: Yup.string().required("Veuillez sélectionner une image."),
  });

  const formik = useFormik({
    initialValues: {
      nom: "",
      slug: "",
      description: "",
      image: "",
    },
    validationSchema: validationSchema,
    onSubmit: async (values, { resetForm }) => {
      const imageRequest: IImage = {
        nom: values.nom,
        slug: values.slug,
        description: values.description,
        image: values.image,
        attributeId: attributid,
      };

      try {
        await useAddImage(imageRequest);
        ImagesMutate();
        setShowSuccessMessage(true);
        resetForm();
        setReturnedMultiMedia(undefined);
        setTimeout(() => {
          setShowSuccessMessage(false);
        }, 5000);
      } catch (error) {
        let errorMessage;
        if (
          typeof error === "string" &&
          error.includes("An Image with this name already exists")
        ) {
          errorMessage = "Le slug doit également être unique.";
        } else if (
          error instanceof Error &&
          error.message.includes("An Image with this name already exists")
        ) {
          errorMessage = "Le slug doit également être unique.";
        }
        setSnackbarMessage(errorMessage ?? "L'image n'a pas pu être créée.");
        setOpenSnackbar(true);
      }
    },
  });

  const handleSlugBlur = async (e: React.FocusEvent<HTMLInputElement>) => {
    formik.handleBlur(e);
    const slug = e.target.value;
    const exists = await checkSlugExistenceImage(slug, attributid);
    if (exists) {
      formik.setFieldError("slug", "Ce slug existe déjà.");
    }
  };

  useEffect(() => {
    if (returnedMultiMedia && returnedMultiMedia.length > 0) {
      const imageURL = extractUrl(returnedMultiMedia[0].url);
      formik.setFieldValue("image", imageURL);
    } else {
      formik.setFieldValue("image", "");
    }
  }, [returnedMultiMedia]);

  return (
    <Card variant="outlined">
      <form onSubmit={formik.handleSubmit}>
        <Stack spacing={1.5} sx={{ p: 3 }}>
          <Typography variant="subtitle2">Ajouter le Terme</Typography>

          <TextField
            name="nom"
            label="Nom *"
            InputLabelProps={{ shrink: true }}
            value={formik.values.nom}
            onChange={formik.handleChange}
            onBlur={formik.handleBlur}
            error={formik.touched.nom && Boolean(formik.errors.nom)}
            helperText={formik.touched.nom && formik.errors.nom}
          />

          <TextField
            name="slug"
            label="Slug"
            InputLabelProps={{ shrink: true }}
            value={formik.values.slug}
            onChange={formik.handleChange}
            onBlur={handleSlugBlur} // Vérification du slug lors du blur
            error={formik.touched.slug && Boolean(formik.errors.slug)}
            helperText={formik.touched.slug && formik.errors.slug}
          />

          <TextField
            name="description"
            label="Description *"
            multiline
            rows={4}
            InputLabelProps={{ shrink: true }}
            value={formik.values.description}
            onChange={formik.handleChange}
            onBlur={formik.handleBlur}
            error={formik.touched.description && Boolean(formik.errors.description)}
            helperText={formik.touched.description && formik.errors.description}
          />

          <Stack spacing={1.5}>
            <Grid container spacing={1} mt={1}>
              <AddMediaButton
                variant="outlined"
                color="success"
                size="medium"
                sx={{ mt: 1 }}
                returnedMultiMedia={returnedMultiMedia}
                setReturnedSingleMedia={setReturnedMultiMedia}
              />
            </Grid>
            {returnedMultiMedia && returnedMultiMedia.length > 0 && (
              <Avatar
                src={extractUrl(returnedMultiMedia[0].url) || ""}
                alt="Selected Image"
                variant="rounded"
                sx={{ width: 100, height: 100, mt: 2 }}
              />
            )}
            {/* Display image selection error */}
            {formik.touched.image && formik.errors.image && (
              <Typography color="error" variant="body2">
                {formik.errors.image}
              </Typography>
            )}
          </Stack>

          <Button type="submit" variant="contained" color="primary">
            Ajouter
          </Button>
        </Stack>
        <Snackbar
          open={openSnackbar}
          autoHideDuration={6000}
          onClose={handleCloseSnackbar}
        >
          <Alert onClose={handleCloseSnackbar} severity="error">
            {snackbarMessage}
          </Alert>
        </Snackbar>
        {showSuccessMessage && (
          <Alert
            severity="success"
            onClose={() => setShowSuccessMessage(false)}
          >
            L'image a été ajoutée avec succès
          </Alert>
        )}
      </form>
    </Card>
  );
}