"use client";
import React, { useState } from "react";
import {
  TextField,
  Autocomplete,
  Button,
  Stack,
  Typography,
  Card,
  Snackbar,
  Alert,
  Grid,
} from "@mui/material";
import * as Yup from "yup";
import { ICategory, ICategorySelected } from "@/shared/types/category";
import { useAddCategory } from "@/shared/api/categorie";
import { IMedia } from "@/shared/types/media";
import { extractUrl } from "@/utils/cleanImageUrl";
import AddMediaButton from "../../media/Add-media-button";

// Mapping for display types
const displayTypeMapping: { [key: string]: string } = {
  "Valeur par défaut": "DEFAULT_VALUE",
  Produits: "PRODUCT",
  "Sous-Catégorie": "SUBCATEGORY",
  "Les deux (Produits et Sous-Catégorie)": "BOTH",
};

// Validation schema using Yup
const validationSchema = Yup.object().shape({
  nom: Yup.string()
    .matches(
      /^[a-zA-Z0-9]+(?:[- ][a-zA-Z0-9]+)*$/,
      "Nom ne doit contenir que des lettres, des chiffres et des tirets."
    )
    .required("Le nom est requis"),
  slug: Yup.string()
    .matches(
      /^[a-zA-Z0-9]+(?:[a-zA-Z0-9- ]*[a-zA-Z0-9]+)*$/,
      "Le slug doit commencer par une lettre ou un chiffre, peut contenir des tirets, et ne doit pas contenir uniquement des caractères spéciaux."
    )
    .required("Le slug est requis"),
  parentId: Yup.number()
    .nullable()
    .required("La catégorie parente est requise"),
});

interface ProductCategoryAddFormProps {
  categories: ICategory[];
  mutate: () => void;
}

const ProductCategoryAddForm: React.FC<ProductCategoryAddFormProps> = ({
  categories,
  mutate,
}) => {
  // Local Snackbar state
  const [openSnackbar, setOpenSnackbar] = useState<boolean>(false);
  const [snackbarMessage, setSnackbarMessage] = useState<string>("");

  // Form state
  const [formData, setFormData] = useState<{
    nom: string;
    slug: string;
    parentId: number | null;
    image: string | null;
  }>({
    nom: "",
    slug: "",
    parentId: null,
    image: null,
  });

  const [formErrors, setFormErrors] = useState<{
    nom?: string;
    slug?: string;
    parentId?: string;
  }>({});

  const [returnedMultiMedia, setReturnedMultiMedia] = useState<IMedia[] | undefined>([]);

  // Prepare parent categories options
  const parentCategories = [
    { id: "0", nom: "Aucun" },
    ...categories.map((cat) => ({ id: cat.id, nom: cat.nom })),
  ];

  // Handler for text input changes
  const handleChange = (
    e: React.ChangeEvent<HTMLInputElement | HTMLTextAreaElement>
  ) => {
    const { name, value } = e.target;
    setFormData((prev) => ({
      ...prev,
      [name]: value,
    }));
    setFormErrors((prev) => ({
      ...prev,
      [name]: undefined,
    }));
  };

  // Handler for category selection changes
  const handleCategoryChange = (
    event: React.SyntheticEvent<Element, Event>,
    value: { id: string | undefined; nom: string } | null,
    reason: any,
    details?: any
  ) => {
    if (value) {
      setFormData((prev) => ({
        ...prev,
        parentId: value?.id === "0" ? null : value?.id ? parseInt(value.id, 10) : null,
      }));
      setFormErrors((prev) => ({
        ...prev,
        parentId: undefined,
      }));
    } else {
      setFormData((prev) => ({
        ...prev,
        parentId: null,
      }));
    }
  };

  // Form submission handler
  const handleSubmit = async (event: React.FormEvent<HTMLFormElement>) => {
    event.preventDefault();

    try {
      // Validate form data
      await validationSchema.validate(formData, { abortEarly: false });

      const apiDisplayType = displayTypeMapping["Valeur par défaut"]; // Since typeAffichage is removed

      let imageURL = null;
      if (returnedMultiMedia && returnedMultiMedia.length > 0) {
        imageURL = returnedMultiMedia[0].url;
      }

      const categoryRequest: ICategory = {
        nom: formData.nom,
        slug: formData.slug,
        description: " ",
        image: extractUrl(imageURL),
        typeAffichage: apiDisplayType,
        parentId: formData.parentId,
      };

      // Add category via API
      await useAddCategory(categoryRequest);
      mutate();
      setSnackbarMessage("Catégorie ajoutée avec succès");
      setOpenSnackbar(true);
      // Reset form
      setFormData({
        nom: "",
        slug: "",
        parentId: null,
        image: null,
      });
      setReturnedMultiMedia([]);
      setFormErrors({});
    } catch (error: any) {
      console.error("errpr" ,error);
      if (error instanceof Yup.ValidationError) {
        const errors: { nom?: string; slug?: string; parentId?: string } = {};
        error.inner.forEach((err) => {
          if (err.path) {
            errors[err.path as keyof typeof errors] = err.message;
          }
        });
        setFormErrors(errors);
        setSnackbarMessage("Veuillez corriger les erreurs du formulaire");
        setOpenSnackbar(true);
      } else {
        let errorMessage = "La catégorie n'a pas pu être créée.";
        if (
          (typeof error === "string" &&
            error.includes("A category with this name already exists")) ||
          (error instanceof Error &&
            error.message.includes("A category with this name already exists"))
        ) {
          errorMessage = "Le slug doit également être unique.";
        }
        setSnackbarMessage(errorMessage);
        setOpenSnackbar(true);
      }
    }
  };

  // Handler to close the Snackbar
  const handleCloseSnackbar = (
    event?: React.SyntheticEvent | Event,
    reason?: string
  ) => {
    if (reason === "clickaway") {
      return;
    }
    setOpenSnackbar(false);
  };

  return (
    <Card variant="outlined">
      <form onSubmit={handleSubmit}>
        <Stack spacing={2} sx={{ p: 3 }}>
          <Typography variant="h6">Ajouter une Catégorie</Typography>

          <TextField
            name="nom"
            label="Nom *"
            InputLabelProps={{ shrink: true }}
            value={formData.nom}
            onChange={handleChange}
            error={Boolean(formErrors.nom)}
            helperText={formErrors.nom}
            fullWidth
          />

          <TextField
            name="slug"
            label="Slug *"
            InputLabelProps={{ shrink: true }}
            value={formData.slug}
            onChange={handleChange}
            error={Boolean(formErrors.slug)}
            helperText={formErrors.slug}
            fullWidth
          />

          <Autocomplete
            id="parent-category"
            autoHighlight
            options={parentCategories}
            getOptionLabel={(option) => option.nom}
            isOptionEqualToValue={(option, value) => option.id === value.id}
            onChange={handleCategoryChange}
            renderInput={(params) => (
              <TextField
                {...params}
                label="Catégorie parente *"
                InputLabelProps={{ shrink: true }}
                error={Boolean(formErrors.parentId)}
                helperText={formErrors.parentId}
              />
            )}
            renderOption={(props, option) => (
              <li {...props} key={option.id}>
                {option.nom}
              </li>
            )}
          />

          {/* Media Upload */}
          <Grid container spacing={1}>
            <AddMediaButton
              variant="outlined"
              color="primary"
              size="medium"
              returnedMultiMedia={returnedMultiMedia}
              setReturnedSingleMedia={setReturnedMultiMedia}
            />
          </Grid>

          {/* Submit Button */}
          <Button type="submit" variant="contained" color="primary">
            Ajouter
          </Button>
        </Stack>
      </form>

      {/* Snackbar for Notifications */}
      <Snackbar
        open={openSnackbar}
        autoHideDuration={6000}
        onClose={handleCloseSnackbar}
      >
        <Alert
          onClose={handleCloseSnackbar}
          severity={
            snackbarMessage === "Catégorie ajoutée avec succès"
              ? "success"
              : "error"
          }
          sx={{ width: "100%" }}
        >
          {snackbarMessage}
        </Alert>
      </Snackbar>
    </Card>
  );
};

export default ProductCategoryAddForm;
