import { useState, useEffect } from "react";
import TextField from "@mui/material/TextField";
import Autocomplete from "@mui/material/Autocomplete";
import Button from "@mui/material/Button";
import Stack from "@mui/material/Stack";
import Typography from "@mui/material/Typography";
import Card from "@mui/material/Card";
import Snackbar from "@mui/material/Snackbar";
import Alert from "@mui/material/Alert";
import { ICategory, ICategorySelected } from "@/shared/types/category";
import {
  useUpdateCategory,
  useGetCategory,
  useGetCategories,
} from "@/shared/api/categorie";
import { Grid, IconButton } from "@mui/material";
import { CloseIcon } from "yet-another-react-lightbox";
import {
  AccessLevel,
  FileType,
  IMedia,
  Visibility,
} from "@/shared/types/media";
import AddMediaButton from "../../media/Add-media-button";

function extractUrl(str: string | null) {
  if (!str) {
    return null;
  }
  const match = str.match(/\[(.*?)\]/);
  return match ? match[1] : str;
}

const displayTypeMapping: { [key: string]: string } = {
  "Valeur par défaut": "DEFAULT_VALUE",
  Produits: "PRODUCT",
  "Sous-Catégorie": "SUBCATEGORY",
  "Les deux (Produits et Sous-Catégorie)": "BOTH",
};

export default function ProductCategoryEditForm({
  categories,
  id,
}: {
  categories: ICategory[];
  id: string;
}) {
  const { categoryData, categoryMutate } = useGetCategory(id);
  const { mutateCategories } = useGetCategories();
  const [selectedCategory, setSelectedCategory] =
    useState<ICategorySelected | null>(null);
  const [selectedDisplayType, setSelectedDisplayType] = useState<string | null>(
    null
  );
  const [formValues, setFormValues] = useState({
    nom: "",
    slug: "",
    description: "",
  });
  const [showSuccessMessage, setShowSuccessMessage] = useState(false);
  const [returnedMultiMedia, setReturnedMultiMedia] = useState<IMedia[]>([]);
  const [initialImageLoaded, setInitialImageLoaded] = useState<boolean>(false);
  const [isEditing, setIsEditing] = useState<boolean>(false);
  const [openSnackbar, setOpenSnackbar] = useState<boolean>(false);
  const [snackbarMessage, setSnackbarMessage] = useState<string>("");
  const parentCategories = [
    { id: "0", nom: "Aucun" },
    ...categories
      .filter((cat) => cat.id != id)
      .map((cat) => ({ id: cat.id, nom: cat.nom })),
  ];

  useEffect(() => {
    if (categoryData) {
      setFormValues((prevValues) => ({
        nom: prevValues.nom || categoryData.nom,
        slug: prevValues.slug || categoryData.slug,
        description: prevValues.description || categoryData.description,
      }));

      // Initialize display type only if it hasn't been set yet
      if (!selectedDisplayType) {
        const displayTypeKey = Object.keys(displayTypeMapping).find(
          (key) => displayTypeMapping[key] === categoryData.typeAffichage
        );
        setSelectedDisplayType(displayTypeKey || null);
      }

      // Initialize the parent category from parentCategories
      if (!selectedCategory) {
        if (categoryData.parentId) {
          const parentCategory = parentCategories.find(
            (cat) => parseInt(cat.id || "0") === categoryData.parentId
          );
          if (parentCategory) {
            setSelectedCategory({
              id: parentCategory.id ?? "0",
              nom: parentCategory.nom,
            });
          }
        } else {
          // If parentId is null, select "Aucun"
          setSelectedCategory({ id: "0", nom: "Aucun" });
        }
      }

      // Initialize image if not loaded yet
      if (categoryData.image && !initialImageLoaded) {
        setReturnedMultiMedia([
          {
            id: "",
            title: "",
            description: "",
            name: "",
            createdAt: "",
            modifiedAt: "",
            size: 0,
            userId: "",
            fileType: FileType.IMAGE,
            visibility: Visibility.PUBLIC,
            permissions: AccessLevel.READ,
            path: "",
            uniqueId: "",
            url: categoryData.image,
          },
        ]);
        setInitialImageLoaded(true);
      }
    }
  }, [categoryData, initialImageLoaded]); // Removed `parentCategories` from dependencies

  const handleInputChange = (event: React.ChangeEvent<HTMLInputElement>) => {
    setFormValues({ ...formValues, [event.target.name]: event.target.value });
  };

  const regex = /^[a-zA-Z0-9]+(?:[- ][a-zA-Z0-9]+)*$/;
  const slugRegex = /^[a-zA-Z0-9]+(?:[a-zA-Z0-9- ]*[a-zA-Z0-9]+)*$/;
  const validateForm = (): boolean => {
    if (!formValues.nom || !formValues.description || !selectedDisplayType) {
      setSnackbarMessage("Tous les champs requis doivent être remplis.");
      setOpenSnackbar(true);
      return false;
    }

    if (!regex.test(formValues.nom)) {
      setSnackbarMessage(
        "Nom ne doit contenir que des lettres, des chiffres et des tirets."
      );
      setOpenSnackbar(true);
      return false;
    }

    if (!slugRegex.test(formValues.slug) && formValues.slug !== "") {
      setSnackbarMessage(
        "Le slug doit commencer par une lettre ou un chiffre, peut contenir des tirets, et ne doit pas contenir uniquement des caractères spéciaux."
      );
      setOpenSnackbar(true);
      return false;
    }

    if (
      !selectedCategory ||
      !parentCategories.some((cat) => cat.id === selectedCategory.id)
    ) {
      setSnackbarMessage("Veuillez sélectionner une catégorie parente valide.");
      setOpenSnackbar(true);
      return false;
    }

    if (!Object.keys(displayTypeMapping).includes(selectedDisplayType)) {
      setSnackbarMessage("Veuillez sélectionner un type d'affichage valide.");
      setOpenSnackbar(true);
      return false;
    }

    return true;
  };

  const handleSubmit = async (event: React.FormEvent<HTMLFormElement>) => {
    event.preventDefault();

    if (!validateForm()) {
      return;
    }

    const apiDisplayType = displayTypeMapping[selectedDisplayType || ""];
    if (!apiDisplayType) {
      setSnackbarMessage("Type d'affichage non valide.");
      setOpenSnackbar(true);
      return;
    }
    const imageURL =
      returnedMultiMedia.length > 0 ? returnedMultiMedia[0].url : null;

    const categoryRequest: ICategory = {
      nom: formValues.nom,
      slug: formValues.slug,
      description: "",
      image: extractUrl(imageURL),
      typeAffichage: "",
      parentId:
        selectedCategory?.id === "0" || selectedCategory === null
          ? null
          : parseInt(selectedCategory.id, 10),
    };


    try {
      await useUpdateCategory(categoryRequest, id);
      mutateCategories();
      categoryMutate();
      setShowSuccessMessage(true);
      setTimeout(() => {
        setShowSuccessMessage(false);
      }, 5000);
    } catch (error) {
      let errorMessage = "La catégorie n'a pas pu être modifée.";
      if (
        typeof error === "string" &&
        error.includes("A category with this name already exists")
      ) {
        errorMessage = "Le slug doit également être unique.";
      } else if (
        error instanceof Error &&
        error.message.includes("A category with this name already exists")
      ) {
        errorMessage = "Le slug doit également être unique.";
      } else {
        errorMessage =
          error instanceof Error
            ? error.message
            : typeof error === "string"
            ? error
            : "nous avons pas pu modifer cette catégorie";
      }
      setSnackbarMessage(errorMessage);
      setOpenSnackbar(true);
    }
  };

  const handleCloseSnackbar = () => {
    setOpenSnackbar(false);
  };

  const typeAffichage = [
    "Valeur par défaut",
    "Produits",
    "Sous-Catégorie",
    "Les deux (Produits et Sous-Catégorie)",
  ];

  return (
    <Card variant="outlined">
      <form onSubmit={handleSubmit}>
        <Stack spacing={1.5} sx={{ p: 3 }}>
          <Typography variant="subtitle2">Modifier une Catégorie</Typography>

          <TextField
            name="nom"
            label="Nom *"
            value={formValues.nom}
            onChange={handleInputChange}
            InputLabelProps={{ shrink: true }}
          />

          <TextField
            name="slug"
            label="Slug"
            value={formValues.slug}
            onChange={handleInputChange}
            InputLabelProps={{ shrink: true }}
          />

          <Autocomplete
            id="Pcategory"
            autoHighlight
            options={parentCategories}
            getOptionLabel={(option) => option.nom}
            isOptionEqualToValue={(option, value) => option.id === value.id}
            value={selectedCategory}
            onChange={(event, newValue) => {
              if (newValue) {
                setSelectedCategory(
                  newValue.id
                    ? { id: newValue.id, nom: newValue.nom }
                    : { id: "", nom: newValue.nom }
                );
              } else {
                setSelectedCategory(null);
              }
            }}
            renderInput={(params) => (
              <TextField
                {...params}
                label="Catégorie parente *"
                InputLabelProps={{ shrink: true }}
              />
            )}
            renderOption={(props, option) => (
              <li {...props} key={option.id}>
                {option.nom}
              </li>
            )}
          />

          <Stack spacing={1.5}>
            <Grid container spacing={1} mt={1}>
              <AddMediaButton
                variant="outlined"
                color="success"
                size="medium"
                sx={{ mt: 1 }}
                nameBtn="Modifier l'image"
                returnedMultiMedia={returnedMultiMedia}
                setReturnedSingleMedia={(media) => {
                  setReturnedMultiMedia(media);
                  setIsEditing(true);
                }}
              />
            </Grid>
            {returnedMultiMedia.length > 0 && !isEditing && (
              <Grid
                md={12}
                xs={12}
                style={{ display: "flex", flexWrap: "wrap" }}
              >
                <Typography
                  component="span"
                  style={{
                    fontSize: "15px",
                    color: "text.secondary",
                    marginTop: 4,
                    width: "100%",
                  }}
                >
                  {returnedMultiMedia.length} Média sélectionné
                </Typography>
                {returnedMultiMedia.map((media) => (
                  <Grid
                    key={media.id}
                    style={{ margin: "10px", position: "relative" }}
                  >
                    <img
                      src={extractUrl(media.url) ?? undefined}
                      alt="image"
                      style={{
                        width: "100px",
                        height: "100px",
                        objectFit: "contain",
                      }}
                    />
                    <IconButton
                      onClick={() => setReturnedMultiMedia([])}
                      style={{ position: "absolute", top: 0, right: 0 }}
                    >
                      <CloseIcon />
                    </IconButton>
                  </Grid>
                ))}
              </Grid>
            )}
          </Stack>

          <Button type="submit" variant="contained" color="primary">
            Modifier
          </Button>
        </Stack>
        {showSuccessMessage && (
          <Alert
            severity="success"
            onClose={() => setShowSuccessMessage(false)}
          >
            Catégorie modifiée avec succès
          </Alert>
        )}
      </form>

      <Snackbar
        open={openSnackbar}
        autoHideDuration={6000}
        onClose={handleCloseSnackbar}
      >
        <Alert onClose={handleCloseSnackbar} severity="error">
          {snackbarMessage}
        </Alert>
      </Snackbar>
    </Card>
  );
}
