import React, { useCallback, useState } from "react";
import { useDropzone } from "react-dropzone";
import {
  Typography,
  Box,
  Card,
  CardContent,
  CardHeader,
  Button,
  CircularProgress,
  Alert,
  AlertTitle,
  Snackbar,
} from "@mui/material";
import CloudUploadIcon from "@mui/icons-material/CloudUpload";
import { useImportCategories } from "@/shared/api/categorie";

const ImportCategory: React.FC = () => {
  const [file, setFile] = useState<File | null>(null);
  const [errorMessage, setErrorMessage] = useState<string | null>(null);
  const [importResult, setImportResult] = useState<Map<string, boolean> | null>(
    null
  );
  const [loading, setLoading] = useState<boolean>(false);
  const [showSnackbar, setShowSnackbar] = useState<boolean>(false); // State for Snackbar visibility
  const [alertSeverity, setAlertSeverity] = useState<
    "success" | "warning" | "error"
  >("success");
  const [alertMessage, setAlertMessage] = useState<string>("");

  const onDrop = useCallback((acceptedFiles: File[]) => {
    const csvFile = acceptedFiles[0];
    if (csvFile && csvFile.type === "text/csv") {
      setFile(csvFile);
      setErrorMessage(null);
    } else {
      setErrorMessage("Veuillez sélectionner un fichier CSV.");
    }
  }, []);

  const { getRootProps, getInputProps } = useDropzone({
    onDrop,
    accept: { "text/csv": [".csv"] },
  });

  const handleSave = async () => {
    if (file) {
      setLoading(true);
      try {
        const result = await useImportCategories(file);
        setImportResult(result);

        // Determine alert message based on result
        const failedCount = Object.values(result).filter(
          (success) => !success
        ).length;

        if (failedCount > 0) {
          setAlertSeverity("warning");
          setAlertMessage(
            `${failedCount} catégories non enregistrées (existent déjà).`
          );
        } else {
          setAlertSeverity("success");
          setAlertMessage(
            "Toutes les catégories ont été enregistrées avec succès."
          );
        }
        setShowSnackbar(true); // Show Snackbar with result

        console.log("Résultat de l'importation :", result);
      } catch (error) {
        console.error("Erreur lors de l'importation du fichier :", error);
        setAlertSeverity("error");
        setAlertMessage(
          "Une erreur est survenue lors de l'importation des catégories."
        );
        setShowSnackbar(true); // Show error Snackbar
      } finally {
        setLoading(false); // Stop loading
      }
    }
  };

  return (
    <Card sx={{ maxWidth: 400, margin: "auto" }}>
      <CardHeader
        title={
          <Typography
            sx={{ fontSize: "1rem", fontWeight: "bold", textAlign: "left" }}
          >
            Importer les catégories
          </Typography>
        }
      />
      <CardContent>
        <Box
          {...getRootProps()}
          sx={{
            border: "2px dashed gray",
            padding: "0px",
            textAlign: "center",
            borderRadius: "5px",
            cursor: "pointer",
            marginBottom: "5px",
            marginTop: "-12px",
          }}
        >
          <input {...getInputProps()} />
          <CloudUploadIcon fontSize="medium" sx={{ color: "gray" }} />

          {file ? (
            <Typography variant="subtitle2" sx={{ color: "gray" }}>
              Fichier sélectionné : {file.name}
            </Typography>
          ) : (
            <Typography variant="subtitle2" sx={{ color: "gray" }}>
              Glissez-déposez un fichier CSV ici ou cliquez pour le sélectionner
            </Typography>
          )}

          {errorMessage && (
            <Typography color="error" variant="body2" mt={2}>
              {errorMessage}
            </Typography>
          )}
        </Box>
        {file && (
          <Box mt={2} textAlign="center">
            <Button
              variant="contained"
              color="primary"
              onClick={handleSave}
              disabled={loading}
              startIcon={loading && <CircularProgress size={20} />}
            >
              {loading ? "Importation en cours..." : "Importer"}
            </Button>
          </Box>
        )}

        <Snackbar
          open={showSnackbar}
          autoHideDuration={6000}
          onClose={() => setShowSnackbar(false)}
          anchorOrigin={{ vertical: "top", horizontal: "center" }}
        >
          <Alert
            severity={alertSeverity}
            onClose={() => setShowSnackbar(false)}
          >
            <AlertTitle>
              {alertSeverity === "error"
                ? "Erreur"
                : alertSeverity === "warning"
                ? "Attention"
                : "Succès"}
            </AlertTitle>
            {alertMessage}
          </Alert>
        </Snackbar>
      </CardContent>
    </Card>
  );
};

export default ImportCategory;
