"use client";

import { useState, useCallback, useRef, useEffect } from "react";
import Container from "@mui/material/Container";
import Stack from "@mui/material/Stack";
import Grid from "@mui/material/Grid";
import Button from "@mui/material/Button";
import Iconify from "@/shared/components/iconify";
import { paths } from "@/routes/paths";
import CustomBreadcrumbs from "@/shared/components/custom-breadcrumbs";
import Card from "@mui/material/Card";
import { useBoolean } from "@/hooks/use-boolean";
import ConfirmDialog from "@/shared/components/confirm-dialog";
import ProductCategoryAddForm from "./category-add-form";
import { useRouter } from "@/hooks";
import { 
  useGetCategories, 
  useUpdateCategory, 
  useDeleteAfterUpdate
} from "@/shared/api/categorie";

import {
  DataGrid,
  GridColDef,
  GridActionsCellItem,
  GridToolbarQuickFilter,
  GridToolbarExport,
  GridToolbarContainer,
  GridRowSelectionModel,
  GridToolbarFilterButton,
} from "@mui/x-data-grid";

import { 
  Avatar, 
  CircularProgress, 
  Typography, 
  Alert, 
  Dialog, 
  DialogTitle, 
  DialogContent, 
  DialogActions, 
  Select, 
  MenuItem 
} from "@mui/material";
import { makeStyles } from "@mui/styles";
import LoadingButton from "@mui/lab/LoadingButton";

const useStyles = makeStyles({
  defaultCategoryRow: {
    backgroundColor: "#ffebee", // light red background
  },
});

export default function ProductCategory() {
  const classes = useStyles();
  const { categoriesData, categoriesLoading, categoriesError, mutateCategories } = useGetCategories();

  // Sort categoriesData by created_at date
  const sortedCategoriesData = categoriesData.sort((a, b) => new Date(b.createdAt ?? 0).getTime() - new Date(a.createdAt ?? 0).getTime());

  const rows = sortedCategoriesData.map((category) => ({
    id: category.id,
    Preview: category.image,
    name: category.nom,
    slug: category.slug,
    description: category.description,
    total: category.total,
    isdefaultcategory: category.isdefaultcategory,
    createdAt: category.createdAt,
  }));

  const [selectedRowIds, setSelectedRowIds] = useState<GridRowSelectionModel>([]);
  const [successMessage, setSuccessMessage] = useState<string | null>(null);
  const [errorMessage, setErrorMessage] = useState<string | null>(null);
  const [rowToDelete, setRowToDelete] = useState<number | null>(null);
  const [defaultCategoryId, setDefaultCategoryId] = useState<number | "">("");
  const [showDefaultCategorySelect, setShowDefaultCategorySelect] = useState<boolean>(false);

  const router = useRouter();
  const handleEditRow = useCallback(
    (id: string) => {
      router.push(paths.dashboard.admin.product.editCategory(id));
    },
    [router]
  );
  const confirmRows = useBoolean();

  const handleRetry = async () => {
    try {
      mutateCategories();
    } catch (err) {
      console.error(err);
    }
  };

  const handleDeleteRows = async () => {
    try {
      if (selectedRowIds.length > 0) {
        const idsToDelete = selectedRowIds.map(id => id.toString());
        const defaultCategories = categoriesData.filter(category => category.isdefaultcategory);

        if (defaultCategories.length === 0) {
          setErrorMessage("Aucune catégorie par défaut trouvée.");
          return;
        }

        if (defaultCategories.length > 1) {
          setShowDefaultCategorySelect(true);
          return;
        }

        if (defaultCategories[0]?.id !== undefined) {
          await reassignProductsAndDeleteCategories(idsToDelete, Number(defaultCategories[0].id));
        } else {
          setErrorMessage("ID de la catégorie par défaut non valide.");
        }
      }
    } catch (error: unknown) {
      let errorMessage = "Erreur lors de la suppression des catégories.";

      if (typeof error === 'string') {
        errorMessage = error;
      }

      setErrorMessage(errorMessage);
      const timeoutId = setTimeout(() => {
        setErrorMessage(null);
        clearTimeout(timeoutId);
      }, 5000);
    }
  };

  const reassignProductsAndDeleteCategories = async (categoryIds: string[], defaultCategoryId: number) => {
    for (const categoryId of categoryIds){
      const response = await useDeleteAfterUpdate(categoryId , defaultCategoryId.toString());
    }
    mutateCategories();
    setSelectedRowIds([]);
    setSuccessMessage("Catégorie(s) supprimée(s) avec succès.");
    const timeoutId = setTimeout(() => {
      setSuccessMessage(null);
      clearTimeout(timeoutId);
    }, 5000);
  };

  const handleDefaultRow = async (id: number) => {
    try {
      const categoryToUpdate = categoriesData.find(category => Number(category.id) === id);
      if (categoryToUpdate) {
        const response = await useUpdateCategory({
          ...categoryToUpdate,
          isdefaultcategory: !categoryToUpdate.isdefaultcategory,
        }, id.toString());
        if (response.status === 200) {
          setSuccessMessage("Catégorie par défaut mise à jour avec succès.");
        }
      }
      mutateCategories();
    } catch (error: unknown) {
      let errorMessage = "Erreur lors de la mise à jour de la catégorie par défaut.";

      if (typeof error === 'string') {
        errorMessage = error;
      }

      setErrorMessage(errorMessage);
    } finally {
      const timeoutId = setTimeout(() => {
        setErrorMessage(null);
        clearTimeout(timeoutId);
      }, 5000);
    }
  };

  const handleDeleteRow = async (id: number) => {
    const categoryToDelete = categoriesData.find(category => Number(category.id) === id);
    if (categoryToDelete?.isdefaultcategory) {
      setErrorMessage("Impossible de supprimer une catégorie par défaut.");
      const timeoutId = setTimeout(() => {
        setErrorMessage(null);
        clearTimeout(timeoutId);
      }, 5000);
      return;
    }
    setRowToDelete(id);
  };

  const confirmDeleteRow = async () => {
    if (rowToDelete !== null) {
      try {
        const defaultCategories = categoriesData.filter(category => category.isdefaultcategory);

        if (defaultCategories.length === 0) {
          setErrorMessage("Aucune catégorie par défaut trouvée.");
          return;
        }

        if (defaultCategories.length > 1) {
          setShowDefaultCategorySelect(true);
          return;
        }

        if (defaultCategories[0]?.id !== undefined) {
          await reassignProductsAndDeleteCategories([rowToDelete.toString()], Number(defaultCategories[0].id));
        } else {
          setErrorMessage("ID de la catégorie par défaut non valide.");
        }
      } catch (error: unknown) {
        let errorMessage = "Erreur lors de la suppression de la catégorie.";

        if (typeof error === 'string') {
          errorMessage = error;
        }

        setErrorMessage(errorMessage);
      } finally {
        const timeoutId = setTimeout(() => {
          setErrorMessage(null);
          clearTimeout(timeoutId);
        }, 5000);
      }
    }
    setRowToDelete(null);
  };

  const columns: GridColDef[] = [
    {
      field: "image",
      headerName: "Image",
      flex: 1,
      renderCell: (params) => (
        <Avatar
          src={params.row.Preview}
          variant="rounded"
          sx={{ width: 64, height: 64, mr: 2 }}
        />
      ),
    },
    {
      field: "name",
      headerName: "Nom",
      flex: 1,
      editable: true,
    },
    {
      field: "slug",
      headerName: "Slug",
      flex: 1,
      editable: true,
    },
    {
      field: "total",
      headerName: "Total",
      flex: 1,
      editable: true,
    },
    {
      field: "createdAt",
      headerName: "Créé le",
      flex: 1,
      editable: true,
    },
    {
      type: "actions",
      field: "actions",
      headerName: " ",
      align: "right",
      headerAlign: "right",
      width: 60,
      sortable: false,
      filterable: false,
      disableColumnMenu: true,
      getActions: (params) => [
        <GridActionsCellItem
          showInMenu
          icon={<Iconify icon="solar:pen-bold" />}
          label="modifier"
          onClick={() => handleEditRow(params.row.id)}
        />,
        <GridActionsCellItem
          showInMenu
          icon={<Iconify icon="solar:trash-bin-trash-bold" />}
          label="Supprimer"
          onClick={() => handleDeleteRow(params.row.id)}
          sx={{ color: "error.main" }}
        />,
        <GridActionsCellItem
          showInMenu
          onClick={() => handleDefaultRow(params.row.id)}
          icon={<Iconify icon={params.row.isdefaultcategory ? "solar:check-bold" : "solar:circle-bold"} />}
          label={params.row.isdefaultcategory ? "Annuler par défaut" : "Définir par défaut"}
        />,
      ],
    },
  ];

  const mainContentRef = useRef<HTMLDivElement>(null);
  const isDialogOpen = showDefaultCategorySelect || confirmRows.value || rowToDelete !== null;

  useEffect(() => {
    if (mainContentRef.current) {
      if (isDialogOpen) {
        mainContentRef.current.setAttribute('inert', '');
      } else {
        mainContentRef.current.removeAttribute('inert');
      }
    }
  }, [isDialogOpen]);

  return (
    <Container
      maxWidth="xl"
      ref={mainContentRef}
      sx={{
        flexGrow: 1,
        display: "flex",
        flexDirection: "column",
      }}
    >
      <CustomBreadcrumbs
        heading="Catégories"
        links={[
          { name: "Tableau de bord", href: paths.dashboard.root },
          { name: "Product", href: paths.dashboard.admin.product.root },
          { name: "Catégorie" },
        ]}
        sx={{
          mb: {
            xs: 3,
            md: 5,
          },
        }}
      />

      <Grid container spacing={5}>
        <Grid item xs={4}>
          <ProductCategoryAddForm categories={categoriesData} mutate={mutateCategories} />
        </Grid>

        <Grid item xs={8}>
          {categoriesLoading ? (
            <Stack
              direction="row"
              spacing={2}
              justifyContent="center"
              alignItems="center"
              sx={{ height: '100%', width: '100%' }}
            >
              <CircularProgress />
              <Typography>Chargement des catégories...</Typography>
            </Stack>
          ) : categoriesError ? (
            <Container>
              <Alert severity="error" sx={{ mb: 2 }}>
                Impossible de charger la liste des catégories
              </Alert>
              <Button variant="contained" color="primary" onClick={handleRetry}>
                Réessayer
              </Button>
            </Container>
          ) : (
            <Card variant="outlined">
              <DataGrid
                checkboxSelection
                autoHeight
                getRowHeight={() => "auto"}
                rows={rows}
                columns={columns}
                disableColumnSelector
                disableDensitySelector
                pageSizeOptions={[5, 10, 25]}
                initialState={{
                  pagination: {
                    paginationModel: { pageSize: 10 },
                  },
                }}
                onRowSelectionModelChange={(newSelectionModel) => {
                  setSelectedRowIds(newSelectionModel);
                }}
                getRowClassName={(params) =>
                  params.row.isdefaultcategory ? classes.defaultCategoryRow : ""
                }
                slots={{
                  toolbar: () => (
                    <GridToolbarContainer>
                      <GridToolbarFilterButton />
                      <GridToolbarExport />
                      <GridToolbarQuickFilter placeholder="Recherche..."/>
                      <Stack
                        spacing={1}
                        flexGrow={1}
                        direction="row"
                        alignItems="center"
                        justifyContent="flex-end"
                      >
                        {!!selectedRowIds.length && (
                          <Stack>
                            <Button
                              size="small"
                              color="error"
                              startIcon={
                                <Iconify icon="solar:trash-bin-trash-bold" />
                              }
                              onClick={confirmRows.onTrue}
                            >
                              Supprimer ({selectedRowIds.length})
                            </Button>
                            <Button
                              size="small"
                              variant="outlined"
                              color="primary"
                              startIcon={
                                <Iconify icon="heroicons-outline:pencil-alt" />
                              }
                              disabled
                            >
                              Modifier ({selectedRowIds.length})
                            </Button>
                          </Stack>
                        )}
                      </Stack>
                    </GridToolbarContainer>
                  ),
                }}
                slotProps={{
                  toolbar: {
                    showQuickFilter: true,
                  },
                }}
              />
              {errorMessage && (
                <Alert severity="error" onClose={() => setErrorMessage(null)}>
                  {errorMessage}
                </Alert>
              )}

              {successMessage && (
                <Alert severity="success" onClose={() => setSuccessMessage(null)}>
                  {successMessage}
                </Alert>
              )}
            </Card>
          )}
          <ConfirmDialog
            open={confirmRows.value}
            onClose={confirmRows.onFalse}
            title="Supprimer"
            content={
              <>
                Êtes-vous sûr de vouloir supprimer{" "}
                <strong> {selectedRowIds.length} </strong> items?
              </>
            }
            action={
              <Button
                variant="contained"
                color="error"
                onClick={() => {
                  handleDeleteRows();
                  confirmRows.onFalse();
                }}
              >
                Supprimer
              </Button>
            }
          />
          <ConfirmDialog
              open={rowToDelete !== null}
              onClose={() => setRowToDelete(null)}
              title="Supprimer"
              content={
                <>
                  Êtes-vous sûr de vouloir supprimer cet item?
                </>
              }
              action={
                <LoadingButton
                  loading={categoriesLoading}
                  variant="contained"
                  color="error"
                  onClick={confirmDeleteRow}
                >
                  Supprimer
                </LoadingButton>
              }
            />

          <Dialog open={showDefaultCategorySelect} onClose={() => setShowDefaultCategorySelect(false)}>
            <DialogTitle>Sélectionner la catégorie par défaut</DialogTitle>
            <DialogContent>
              <Select
                value={defaultCategoryId}
                onChange={(e) => setDefaultCategoryId(e.target.value === "" ? "" : Number(e.target.value))}
                fullWidth
                displayEmpty
              >
                <MenuItem value="">
                  <em>None</em>
                </MenuItem>
                {categoriesData.filter(category => category.isdefaultcategory).map(category => (
                  <MenuItem key={category.id} value={category.id}>
                    {category.nom}
                  </MenuItem>
                ))}
              </Select>
            </DialogContent>
            <DialogActions>
              <Button onClick={() => setShowDefaultCategorySelect(false)} color="primary">
                Annuler
              </Button>
              <Button
                onClick={async () => {
                  if (defaultCategoryId !== "" && defaultCategoryId !== null) {
                    await reassignProductsAndDeleteCategories(
                      rowToDelete !== null ? [rowToDelete.toString()] : selectedRowIds.map(id => id.toString()),
                      Number(defaultCategoryId)
                    );
                    setShowDefaultCategorySelect(false);
                  } else {
                    setErrorMessage("Veuillez sélectionner une catégorie par défaut.");
                  }
                }}
                color="primary"
                disabled={defaultCategoryId === ""}
              >
                Confirmer
              </Button>
            </DialogActions>
          </Dialog>
        </Grid>
      </Grid>
    </Container>
  );
}
