"use client";

import { useState } from "react";
import Container from "@mui/material/Container";
import Stack from "@mui/material/Stack";
import Grid from "@mui/material/Grid";
import Button from "@mui/material/Button";
import Iconify from "@/shared/components/iconify";
import { paths } from "@/routes/paths";
import CustomBreadcrumbs from "@/shared/components/custom-breadcrumbs";
import Card from "@mui/material/Card";
import { useBoolean } from "@/hooks/use-boolean";
import ConfirmDialog from "@/shared/components/confirm-dialog";
import { useRouter } from "@/hooks";
import { useCallback } from "react";
import {
  DataGrid,
  GridColDef,
  GridActionsCellItem,
  GridToolbarQuickFilter,
  GridToolbarExport,
  GridToolbarContainer,
  GridRowSelectionModel,
  GridToolbarFilterButton,
} from "@mui/x-data-grid";
import { ILabel } from "@/shared/types/term";
import { Alert, CircularProgress, Typography } from "@mui/material";
import CustomDataGridToolbar from "../../add-edit-ConfigurationIColorTerme/GridToolbarContainer";
import AddTermeLabelForm from "./termeLabel-add-form";
import { useDeleteLabel, useDeleteLabels, useGetLabelsByAttribut } from "@/shared/api/terms";

export default function TermeLabel({ attributid }: { attributid: string }) {
  const [selectedRowIds, setSelectedRowIds] = useState<GridRowSelectionModel>([]);
  const { labelsData, labelsLoading, labelsError, labelsMutate } = useGetLabelsByAttribut(attributid);
  const router = useRouter();
  const handleEditRow = useCallback(
    (id: string) => {
      router.push(paths.dashboard.admin.product.termeLabel.editLabel(attributid, id));
    },
    [router]
  );
  const confirmRows = useBoolean();
  const confirmSingleRow = useBoolean();
  const [tableData, setTableData] = useState<ILabel[]>();
  const [errorMessage, setErrorMessage] = useState<string | null>(null);
  const [successMessage, setSuccessMessage] = useState<string | null>(null);
  const [rowToDelete, setRowToDelete] = useState<number | null>(null);

  const handleRetry = async () => {
    try {
      await labelsMutate();
    } catch (err) {
      setErrorMessage("Erreur lors du rechargement des attributs.");
      setTimeout(() => {
        setErrorMessage(null);
      }, 5000);
    }
  };

  const handleDeleteRow = async (id: number) => {
    setRowToDelete(id);
    confirmSingleRow.onTrue();
  };

  const confirmDeleteRow = async () => {
    if (rowToDelete !== null) {
      try {
        const response = await useDeleteLabel(rowToDelete.toString(), attributid);
        labelsMutate();

        if (response.status === 204) {
          setSelectedRowIds([]);
          setSuccessMessage("Label supprimée avec succès.");
        }
      } catch (error) {
        setErrorMessage("Impossible de supprimer une terme liée à un produit");
      } finally {
        const timeoutId = setTimeout(() => {
          setSuccessMessage(null);
          setErrorMessage(null);
          clearTimeout(timeoutId);
        }, 5000);
      }
    }
    confirmSingleRow.onFalse();
  };

  const handleDeleteRows = async () => {
    try {
      if (selectedRowIds.length > 0) {
        const idsToDelete = selectedRowIds.map((id) => id.toString());
        const response = await useDeleteLabels(attributid, idsToDelete);
        labelsMutate();

        if (response.status === 200) {
          setSelectedRowIds([]);
          setSuccessMessage("Label(s) supprimée(s) avec succès.");
        }
      }
    } catch (error) {
      setErrorMessage("Impossible de supprimer les terme liée à un produit");
    } finally {
      const timeoutId = setTimeout(() => {
        setSuccessMessage(null);
        setErrorMessage(null);
        clearTimeout(timeoutId);
      }, 5000);
    }
  };

  const columns: GridColDef[] = [
    {
      field: "name",
      headerName: "Nom",
      flex: 1,
      editable: true,
    },
    {
      field: "slug",
      headerName: "Slug",
      flex: 1,
      editable: true,
    },
    {
      field: "description",
      headerName: "Description",
      flex: 1,
      editable: true,
    },
    {
      field: "total",
      headerName: "Total",
      flex: 1,
      editable: true,
    },
    {
      field: "createdAt",
      headerName: "Créé le",
      flex: 1,
      editable: true,
    },
    {
      type: "actions",
      field: "actions",
      headerName: " ",
      align: "right",
      headerAlign: "right",
      width: 60,
      sortable: false,
      filterable: false,
      disableColumnMenu: true,
      getActions: (params) => [
        <GridActionsCellItem
          showInMenu
          icon={<Iconify icon="solar:pen-bold" />}
          label="modifier"
          onClick={() => handleEditRow(params.row.id)}
        />,
        <GridActionsCellItem
          showInMenu
          icon={<Iconify icon="solar:trash-bin-trash-bold" />}
          label="Supprimer"
          onClick={() => handleDeleteRow(params.row.id)}
          sx={{ color: "error.main" }}
        />,
        <GridActionsCellItem
          showInMenu
          icon={<Iconify icon="bi:eye" />}
          label="voir"
        />,
      ],
    },
  ];

  const SortedLabelData = labelsData.sort((a, b) => {
    return new Date(b.createdAt ?? 0).getTime() - new Date(a.createdAt ?? 0).getTime(); 
  });

  const rows = SortedLabelData.map((label) => ({
    id: label.id,
    name: label.name,
    slug: label.slug,
    description: label.description,
    total: label.total,
    createdAt: label.createdAt,
  }));

  return (
    <Container maxWidth="xl" sx={{ flexGrow: 1, display: "flex", flexDirection: "column" }}>
      <CustomBreadcrumbs
        heading="Attributs"
        links={[
          { name: "Tableau de bord", href: paths.dashboard.root },
          {
            name: "Product",
            href: paths.dashboard.admin.product.root,
          },
          { name: "Attribut", href: paths.dashboard.admin.product.attribut },
          {
            name: "Labels",
            href: paths.dashboard.admin.product.termeLabel.root(attributid),
          },
        ]}
        sx={{
          mb: {
            xs: 3,
            md: 5,
          },
        }}
      />

      <Grid container spacing={3}>
        <Grid item xs={4}>
          <AddTermeLabelForm attributid={attributid} />
        </Grid>

        <Grid item xs={8}>
          {labelsLoading ? (
            <Stack
              direction="row"
              spacing={2}
              justifyContent="center"
              alignItems="center"
              sx={{ height: "100%", width: "100%" }}
            >
              <CircularProgress />
              <Typography>Chargement des Labels...</Typography>
            </Stack>
          ) : labelsError ? (
            <Container>
              <Alert severity="error" sx={{ mb: 2 }}>
                Impossible de charger la liste des labels
              </Alert>
              <Button variant="contained" color="primary" onClick={handleRetry}>
                Réessayer
              </Button>
            </Container>
          ) : (
            <Card variant="outlined">
              <DataGrid
                checkboxSelection
                autoHeight
                getRowHeight={() => "auto"}
                rows={rows}
                columns={columns}
                disableColumnSelector
                disableDensitySelector
                pageSizeOptions={[5, 10, 25]}
                initialState={{
                  pagination: {
                    paginationModel: { pageSize: 10 },
                  },
                }}
                onRowSelectionModelChange={(newSelectionModel) => {
                  setSelectedRowIds(newSelectionModel);
                }}
                slots={{
                  toolbar: () => (
                    <CustomDataGridToolbar
                      selectedRowIds={selectedRowIds.map(String)}
                      onDeleteRows={() => confirmRows.onTrue()}
                    />
                  ),
                }}
                slotProps={{
                  toolbar: {
                    showQuickFilter: true,
                  },
                }}
              />
              {errorMessage && (
                <Alert severity="error" onClose={() => setErrorMessage(null)}>
                  {errorMessage}
                </Alert>
              )}
              {successMessage && (
                <Alert severity="success" onClose={() => setSuccessMessage(null)}>
                  {successMessage}
                </Alert>
              )}
            </Card>
          )}
          <ConfirmDialog
            open={confirmRows.value}
            onClose={confirmRows.onFalse}
            title="Supprimer"
            content={
              <>
                Êtes-vous sûr de vouloir supprimer <strong>{selectedRowIds.length}</strong>
                {selectedRowIds.length > 1 ? " labels" : " label"} ?
              </>
            }
            action={
              <Button
                variant="contained"
                color="error"
                onClick={() => {
                  handleDeleteRows();
                  confirmRows.onFalse();
                }}
              >
                Supprimer
              </Button>
            }
          />
          <ConfirmDialog
            open={confirmSingleRow.value}
            onClose={confirmSingleRow.onFalse}
            title="Supprimer"
            content="Êtes-vous sûr de vouloir supprimer ce label ?"
            action={
              <Button
                variant="contained"
                color="error"
                onClick={confirmDeleteRow}
              >
                Supprimer
              </Button>
            }
          />
        </Grid>
      </Grid>
    </Container>
  );
}