"use client";
import { useEffect, useState } from "react";
import TextField from "@mui/material/TextField";
import Button from "@mui/material/Button";
import Stack from "@mui/material/Stack";
import Typography from "@mui/material/Typography";
import Card from "@mui/material/Card";
import { ILabel } from "@/shared/types/term";
import { Alert, Snackbar } from "@mui/material";
import { useGetLabel, useGetLabelsByAttribut, useUpdateLabel, checkSlugExistenceLabel } from "@/shared/api/terms";
import * as yup from "yup"; // Import de yup

//------------------------------------------------------
export default function EditTermLabForm({ attributid, labelId }: { attributid: string; labelId: string }) {
  const { labelsMutate } = useGetLabelsByAttribut(attributid);
  const { labelData } = useGetLabel(labelId);

  const [name, setName] = useState<string>("");
  const [slug, setSlug] = useState<string>("");
  const [description, setDescription] = useState<string>("");

  const formSetter = () => {
    setName(labelData.name);
    setSlug(labelData.slug);
    setDescription(labelData.description);
  };

  useEffect(() => {
    if (labelData) {
      formSetter();
    }
  }, [labelData]);

  // States for Snackbar
  const [openSnackbar, setOpenSnackbar] = useState<boolean>(false);
  const [snackbarMessage, setSnackbarMessage] = useState<string>("");
  const [showSuccessMessage, setShowSuccessMessage] = useState(false);

  const handleCloseSnackbar = () => {
    setOpenSnackbar(false);
  };

  // Définir le schéma de validation avec YUP
  const validationSchema = yup.object({
    name: yup
      .string()
      .matches(/^[a-zA-Z0-9]+(?:[- ][a-zA-Z0-9]+)*$/, "Le nom ne doit contenir que des lettres, des chiffres et des tirets.")
      .required("Le nom est requis."),
    slug: yup
      .string()
      .matches(/^[a-zA-Z0-9]+(?:[a-zA-Z0-9- ]*[a-zA-Z0-9]+)*$/, "Le slug doit commencer par une lettre ou un chiffre, peut contenir des tirets.")
      .required("Le slug est requis.")
      .test("unique-slug", "Ce slug existe déjà.", async (value) => {
                    if (value !== labelData.slug) { 
                      const exists = await checkSlugExistenceLabel(value, attributid);
                      return !exists; 
                    }
                    return true; 
                  }),
    description: yup.string().optional(),
  });

  const validateForm = async (): Promise<boolean> => {
    try {
      await validationSchema.validate({ name, slug, description }, { abortEarly: false });
      return true;
    } catch (err) {
      if (err instanceof yup.ValidationError) {
        setSnackbarMessage(err.errors[0]); // Affiche la première erreur
        setOpenSnackbar(true);
      }
      return false;
    }
  };

  // Vérification du slug au moment du blur
  const handleSlugBlur = async (e: React.FocusEvent<HTMLInputElement>) => {
    setSlug(e.target.value);
  };

  const handleOnSubmit = async (event: React.FormEvent<HTMLFormElement>) => {
    event.preventDefault();
    const isValid = await validateForm();
    if (!isValid) return;

    const labelRequest: ILabel = {
      name,
      slug,
      description,
      attributeId: attributid,
    };

    try {
      await useUpdateLabel(labelRequest, labelId);
      labelsMutate();
      setShowSuccessMessage(true);
      const timeoutId = setTimeout(() => {
        setShowSuccessMessage(false);
        clearTimeout(timeoutId);
      }, 5000);
    } catch (error) {
      let errorMessage;
      if (
        typeof error === "string" &&
        error.includes("A Label with this name already exists")
      ) {
        errorMessage = "Le slug doit également être unique.";
      } else if (
        error instanceof Error &&
        error.message.includes("A Label tag with this name already exists")
      ) {
        errorMessage = "Le slug doit également être unique.";
      }
      setSnackbarMessage(errorMessage ?? "Le label n'a pas pu être modifié.");
      setOpenSnackbar(true);
    }
  };

  return (
    <Card variant="outlined">
      <form onSubmit={handleOnSubmit}>
        <Stack spacing={1.5} sx={{ p: 3 }}>
          <Typography variant="subtitle2">Modifier le Terme</Typography>

          <TextField
            name="name"
            label="Nom *"
            InputLabelProps={{ shrink: true }}
            value={name}
            onChange={(e) => setName(e.target.value)}
          />

          <TextField
            name="slug"
            label="Slug"
            InputLabelProps={{ shrink: true }}
            value={slug}
            onChange={(e) => setSlug(e.target.value)}
            onBlur={handleSlugBlur} // Vérification du slug lors du blur
          />

          <TextField
            name="description"
            label="Description *"
            multiline
            rows={4}
            InputLabelProps={{ shrink: true }}
            value={description}
            onChange={(e) => setDescription(e.target.value)}
          />

          <Button type="submit" variant="contained" color="primary">
            Modifier
          </Button>
        </Stack>
        <Snackbar open={openSnackbar} autoHideDuration={6000} onClose={handleCloseSnackbar}>
          <Alert onClose={handleCloseSnackbar} severity="error">
            {snackbarMessage}
          </Alert>
        </Snackbar>
        {showSuccessMessage && (
          <Alert severity="success" onClose={() => setShowSuccessMessage(false)}>
            Le label a été modifié avec succès
          </Alert>
        )}
      </form>
    </Card>
  );
}
