"use client";

import React, { useState } from "react";
import {
  TextField,
  Button,
  Stack,
  Typography,
  Card,
  Snackbar,
  Alert,
} from "@mui/material";
import Autocomplete from "@mui/material/Autocomplete";
import { useAddAttribute, useGetAttributs } from "@/shared/api/attribut";
import { IAttribute, TrieOption, TypeOption } from "@/shared/types/attribut";
import * as Yup from "yup";

// Utility functions to handle enum mappings
const getTrieOptionKey = (
  value: string | null
): keyof typeof TrieOption | null => {
  if (value === null) return null;
  const key = Object.keys(TrieOption).find(
    (key) => TrieOption[key as keyof typeof TrieOption] === value
  );
  return key ? (key as keyof typeof TrieOption) : null;
};

const getTypeOptionKey = (
  value: string | null
): keyof typeof TypeOption | null => {
  if (value === null) return null;
  const key = Object.keys(TypeOption).find(
    (key) => TypeOption[key as keyof typeof TypeOption] === value
  );
  return key ? (key as keyof typeof TypeOption) : null;
};

const getTrieOptionValue = (
  key: keyof typeof TrieOption | null
): TrieOption | null => {
  return key ? TrieOption[key] : null;
};

const getTypeOptionValue = (
  key: keyof typeof TypeOption | null
): TypeOption | null => {
  return key ? TypeOption[key] : null;
};

// Yup validation schema
const validationSchema = Yup.object().shape({
  nom: Yup.string()
    .matches(
      /^[a-zA-Z0-9]+(?:[- ][a-zA-Z0-9]+)*$/,
      "Le nom ne doit contenir que des lettres, des chiffres et des tirets."
    )
    .required("Le nom est requis"),
  identifiant: Yup.string()
    .matches(
      /^[a-zA-Z0-9]+(?:[a-zA-Z0-9- ]*[a-zA-Z0-9]+)*$/,
      "L'identifiant doit commencer par une lettre ou un chiffre, peut contenir des tirets, et ne doit pas contenir uniquement des caractères spéciaux."
    )
    .required("L'identifiant est requis"),
  type: Yup.string()
    .required("Veuillez sélectionner un type."),
});

const ProductAttributAddForm: React.FC = () => {
  const { AttributesMutate } = useGetAttributs(); // Used to reload attributes after creation

  // Centralized form data state
  const [formData, setFormData] = useState<{
    nom: string;
    identifiant: string;
    activeArchive: boolean;
    defaultSort: keyof typeof TrieOption | null;
    type: keyof typeof TypeOption | null;
  }>({
    nom: "",
    identifiant: "",
    activeArchive: false,
    defaultSort: null,
    type: null,
  });

  // Centralized form errors state
  const [formErrors, setFormErrors] = useState<{
    nom?: string;
    identifiant?: string;
    type?: string;
  }>({});

  // Snackbar state
  const [openSnackbar, setOpenSnackbar] = useState<boolean>(false);
  const [snackbarMessage, setSnackbarMessage] = useState<string>("");
  const [snackbarSeverity, setSnackbarSeverity] = useState<
    "success" | "error" | "warning" | "info"
  >("success");

  // Handler for text input changes
  const handleChange = (
    event: React.ChangeEvent<HTMLInputElement | HTMLTextAreaElement>
  ) => {
    const { name, value, type, checked } = event.target as HTMLInputElement;
    setFormData((prevState) => ({
      ...prevState,
      [name]: type === "checkbox" ? checked : value,
    }));
    // Clear specific field error when user modifies the input
    setFormErrors((prevState) => ({
      ...prevState,
      [name]: undefined,
    }));
  };

  // Handler for Autocomplete changes
  const handleAutocompleteChange = (
    name: "defaultSort" | "type",
    value: TrieOption | TypeOption | null
  ) => {
    setFormData((prevState) => ({
      ...prevState,
      [name]:
        value &&
        (name === "defaultSort"
          ? getTrieOptionKey(value as string)
          : getTypeOptionKey(value as string)) !== null
          ? (name === "defaultSort"
              ? getTrieOptionKey(value as string)
              : getTypeOptionKey(value as string))
          : null,
    }));
    // Clear specific field error when user modifies the input
    setFormErrors((prevState) => ({
      ...prevState,
      [name]: undefined,
    }));
  };

  // Form submission handler
  const handleSubmit = async (event: React.FormEvent<HTMLFormElement>) => {
    event.preventDefault();

    try {
      // Validate form data using Yup
      await validationSchema.validate(formData, { abortEarly: false });

      // Prepare tag request object with defaultSort and type as per enum keys
      const submitPayload: IAttribute = {
        ...formData,
        defaultSort: formData.defaultSort ? formData.defaultSort : null,
        type: formData.type ? formData.type : null,
      };

      // Add attribute via API
      await useAddAttribute(submitPayload);
      // Reload attributes list
      await AttributesMutate();
      // Show success message
      setSnackbarMessage("L'attribut a été créé avec succès.");
      setSnackbarSeverity("success");
      setOpenSnackbar(true);
      // Reset form fields
      setFormData({
        nom: "",
        identifiant: "",
        activeArchive: false,
        defaultSort: null,
        type: null,
      });
      setFormErrors({});
    } catch (error: any) {
      if (error instanceof Yup.ValidationError) {
        // Handle validation errors
        const errors: { nom?: string; identifiant?: string; type?: string } = {};
        error.inner.forEach((err) => {
          if (err.path) {
            errors[err.path as keyof typeof errors] = err.message;
          }
        });
        setFormErrors(errors);
        // Show validation error message
        setSnackbarMessage("Veuillez corriger les erreurs du formulaire.");
        setSnackbarSeverity("error");
        setOpenSnackbar(true);
      } else {
        // Handle API or other errors
        let errorMessage = "L'attribut n'a pas pu être créé.";
        if (
          (typeof error === "string" &&
            error.includes("An attribute with this name already exists")) ||
          (error instanceof Error &&
            error.message.includes("An attribute with this name already exists"))
        ) {
          errorMessage = "Le slug doit également être unique.";
        }
        setSnackbarMessage(errorMessage);
        setSnackbarSeverity("error");
        setOpenSnackbar(true);
      }
    }
  };

  // Handler to close the Snackbar
  const handleCloseSnackbar = (
    event?: React.SyntheticEvent | Event,
    reason?: string
  ) => {
    if (reason === "clickaway") {
      return;
    }
    setOpenSnackbar(false);
  };

  return (
    <Card variant="outlined">
      <form onSubmit={handleSubmit}>
        <Stack spacing={2} sx={{ p: 3 }}>
          <Typography variant="h6">Ajouter un Attribut</Typography>

          <TextField
            name="nom"
            label="Nom *"
            InputLabelProps={{ shrink: true }}
            value={formData.nom}
            onChange={handleChange}
            error={Boolean(formErrors.nom)}
            helperText={formErrors.nom}
            fullWidth
          />

          <TextField
            name="identifiant"
            label="Identifiant *"
            InputLabelProps={{ shrink: true }}
            value={formData.identifiant}
            onChange={handleChange}
            error={Boolean(formErrors.identifiant)}
            helperText={formErrors.identifiant}
            fullWidth
          />

          <Autocomplete
            id="type"
            autoHighlight
            options={Object.values(TypeOption)}
            value={getTypeOptionValue(formData.type)}
            onChange={(event, value) =>
              handleAutocompleteChange("type", value as TypeOption | null)
            }
            isOptionEqualToValue={(option, value) => option === value}
            renderInput={(params) => (
              <TextField
                {...params}
                label="Type *"
                InputLabelProps={{ shrink: true }}
                error={Boolean(formErrors.type)}
                helperText={formErrors.type}
                fullWidth
              />
            )}
          />
          
          <Button type="submit" variant="contained" color="primary">
            Enregistrer
          </Button>
        </Stack>

        {/* Snackbar for Notifications */}
        <Snackbar
          open={openSnackbar}
          autoHideDuration={6000}
          onClose={handleCloseSnackbar}
          anchorOrigin={{ vertical: "bottom", horizontal: "center" }}
        >
          <Alert
            onClose={handleCloseSnackbar}
            severity={snackbarSeverity}
            sx={{ width: "100%" }}
          >
            {snackbarMessage}
          </Alert>
        </Snackbar>
      </form>
    </Card>
  );
}

export default ProductAttributAddForm;
