"use client";

import TextField from "@mui/material/TextField";
import Button from "@mui/material/Button";
import Stack from "@mui/material/Stack";
import Typography from "@mui/material/Typography";
import Card from "@mui/material/Card";
import Checkbox from "@mui/material/Checkbox";
import FormControlLabel from "@mui/material/FormControlLabel";
import Autocomplete from "@mui/material/Autocomplete";
import CircularProgress from "@mui/material/CircularProgress";
import { useEffect, useState } from "react";
import {
  useGetAttribute,
  useGetAttributs,
  useUpdateAttribut,
} from "@/shared/api/attribut";
import { IAttribute, TrieOption, TypeOption } from "@/shared/types/attribut";
import { Alert, Snackbar } from "@mui/material";

// Fonction pour obtenir la clé d'une valeur de l'énumération TrieOption
const getTrieOptionKey = (
  value: string | null
): keyof typeof TrieOption | null => {
  if (value === null) return null;
  const key = Object.keys(TrieOption).find(
    (key) => TrieOption[key as keyof typeof TrieOption] === value
  );
  return key ? (key as keyof typeof TrieOption) : null;
};

// Fonction pour obtenir la clé d'une valeur de l'énumération TypeOption
const getTypeOptionKey = (
  value: string | null
): keyof typeof TypeOption | null => {
  if (value === null) return null;
  const key = Object.keys(TypeOption).find(
    (key) => TypeOption[key as keyof typeof TypeOption] === value
  );
  return key ? (key as keyof typeof TypeOption) : null;
};

// Fonction pour obtenir la valeur d'une clé d'option de tri
const getTrieOptionValue = (key: keyof typeof TrieOption): string => {
  return TrieOption[key];
};

// Fonction pour obtenir la valeur d'une clé d'option de type
const getTypeOptionValue = (key: keyof typeof TypeOption): string => {
  return TypeOption[key];
};

// Fonction pour obtenir la clé d'une valeur de l'énumération
const getKeyFromValue = (enumObj: any, value: string): string | undefined => {
  return Object.keys(enumObj).find((key) => enumObj[key] === value);
};

export default function ProductAttributEditForm({ id }: { id: string }) {
  const { attributData, attributLoading } = useGetAttribute(id);
  const { AttributesMutate } = useGetAttributs();

  const [showSuccessMessage, setShowSuccessMessage] = useState(false);

  // States for Snackbar
  const [openSnackbar, setOpenSnackbar] = useState<boolean>(false);
  const [snackbarMessage, setSnackbarMessage] = useState<string>("");

  const [formValue, setFormValue] = useState({
    nom: "",
    identifiant: "",
    activeArchive: false,
    defaultSort: null as keyof typeof TrieOption | null,
    type: null as keyof typeof TypeOption | null,
  });

  const formSetter = () => {
    setFormValue({
      nom: attributData.nom || "",
      identifiant: attributData.identifiant || "",
      activeArchive: attributData.activeArchive || false,
      defaultSort: attributData.defaultSort,
      type: attributData.type,
    });
  };
  useEffect(() => {
    if (attributData) {
      console.log("Attribut Data:", attributData);
      formSetter();
    }
  }, [attributData]);
  console.log(formValue?.defaultSort?TrieOption[formValue.defaultSort]:null)

  const handleChange = (event: React.ChangeEvent<HTMLInputElement>) => {
    const { name, value, type, checked } = event.target;
    setFormValue((prevState) => ({
      ...prevState,
      [name]: type === "checkbox" ? checked : value,
    }));
  };

  const handleAutocompleteChange = (
    name: "defaultSort" | "type",
    value: TrieOption | TypeOption | null
  ) => {
    setFormValue((prevState) => ({
      ...prevState,
      [name]: value
        ? (name === "defaultSort"
            ? getTrieOptionKey(value as string)
            : getTypeOptionKey(value as string)) || null
        : null,
    }));
  };

  const handleCloseSnackbar = () => {
    setOpenSnackbar(false);
  };

  const regex = /^[a-zA-Z0-9]+(?:[- ][a-zA-Z0-9]+)*$/;
  const slugRegex = /^[a-zA-Z0-9]+(?:[a-zA-Z0-9- ]*[a-zA-Z0-9]+)*$/;
  const validateForm = (): boolean => {
    if (!formValue.nom) {
      setSnackbarMessage("Tous les champs requis doivent être remplis.");
      setOpenSnackbar(true);
      return false;
    }

    if (!regex.test(formValue.nom)) {
      setSnackbarMessage(
        "le nom ne doit contenir que des lettres, des chiffres et des tirets."
      );
      setOpenSnackbar(true);
      return false;
    }

    if (
      !slugRegex.test(formValue.identifiant) &&
      formValue.identifiant !== ""
    ) {
      setSnackbarMessage(
        "L'identifiant doit commencer par une lettre ou un chiffre, peut contenir des tirets, et ne doit pas contenir uniquement des caractères spéciaux."
      );
      setOpenSnackbar(true);
      return false;
    }
    if (!formValue.type) {
      setSnackbarMessage("Veuillez sélectionner un type.");
      setOpenSnackbar(true);
      return false;
    }
    if (!formValue.defaultSort) {
      setSnackbarMessage("Veuillez sélectionner un tri par défault.");
      setOpenSnackbar(true);
      return false;
    }
    return true;
  };

  const handleSubmit = async (event: React.FormEvent<HTMLFormElement>) => {
    event.preventDefault();

    if (!validateForm()) {
      return;
    }

    // Convertir les valeurs sélectionnées en clés d'énumération pour l'envoi
    const submitPayload: IAttribute = {
      ...formValue,
      defaultSort: formValue.defaultSort ? formValue.defaultSort : null,
      type: formValue.type ? formValue.type : null,
    };
    console.log(submitPayload)

    try {
      await useUpdateAttribut(submitPayload, id);
      await AttributesMutate();
      setSnackbarMessage("L'attribut a été modifié avec succès.");
      setShowSuccessMessage(true);
    } catch (error) {
      let errorMessage ;
    if (typeof error === 'string' && error.includes("An attribute with this name already exists")) {
      errorMessage = "Le slug doit également être unique.";
    } else if (error instanceof Error && error.message.includes("An attribute with this name already exists")) {
      errorMessage = "Le slug doit également être unique.";
    }
    setSnackbarMessage(errorMessage??"L'attribut n'a pas pu être modifié.");
    
      
    } finally {
      setOpenSnackbar(true);
      const timeoutId = setTimeout(()=>{
        handleCloseSnackbar()
      },5000)
    
    }
  };

  return (
    <Card variant="outlined">
      {attributLoading ? (
        <Stack
          direction="row"
          spacing={2}
          justifyContent="center"
          alignItems="center"
          sx={{ height: "100%", width: "100%" }}
        >
          <CircularProgress />
          <Typography>Chargement de l'attribut...</Typography>
        </Stack>
      ) : (
        <form onSubmit={handleSubmit}>
          <Stack spacing={1.5} sx={{ p: 3 }}>
            <Typography variant="subtitle2">Attribut</Typography>

            <TextField
              name="nom"
              label="Nom"
              InputLabelProps={{ shrink: true }}
              value={formValue.nom}
              onChange={handleChange}
            />

            <TextField
              label="Identifiant"
              name="identifiant"
              value={formValue.identifiant}
              InputLabelProps={{ shrink: true }}
              onChange={handleChange}
            />

            <Autocomplete
              id="type"
              autoHighlight
              options={Object.values(TypeOption)}
              value={attributData?.type?TypeOption[attributData.type]:null} // Modifié pour utiliser getTypeOptionValue
              disabled={true} 
              isOptionEqualToValue={(option, value) => option === value}
              renderInput={(params) => (
                <TextField
                  {...params}
                  label="Type"
                  InputLabelProps={{ shrink: true }}
                />
              )}
            />

            <Button type="submit" variant="contained" color="primary">
              Enregistrer
            </Button>
          </Stack>
          <Snackbar
            open={openSnackbar}
            autoHideDuration={6000}
            onClose={handleCloseSnackbar}
          >
            <Alert
              onClose={handleCloseSnackbar}
              severity={showSuccessMessage ? "success" : "error"}
            >
              {snackbarMessage}
            </Alert>
          </Snackbar>
        </form>
      )}
    </Card>
  );
}
