"use client";

import { useState, useCallback } from "react";
import Container from "@mui/material/Container";
import Stack from "@mui/material/Stack";
import Grid from "@mui/material/Grid";
import Button from "@mui/material/Button";
import Iconify from "@/shared/components/iconify";
import { paths } from "@/routes/paths";
import CustomBreadcrumbs from "@/shared/components/custom-breadcrumbs";
import Card from "@mui/material/Card";
import ConfirmDialog from "@/shared/components/confirm-dialog";
import ProductAttributAddForm from "./attribut-add-form";
import SettingsIcon from "@mui/icons-material/Settings";
import { useBoolean, useRouter } from "@/hooks";
import {
  DataGrid,
  GridColDef,
  GridActionsCellItem,
  GridToolbarQuickFilter,
  GridToolbarExport,
  GridToolbarContainer,
  GridRowSelectionModel,
  GridToolbarFilterButton,
} from "@mui/x-data-grid";
import { useGetAttributs, useDeleteAttribut, useDeleteAttributs } from "@/shared/api/attribut";
import { CircularProgress, Typography, Alert, Tooltip, IconButton } from "@mui/material";
import { TrieOption, TypeOption } from "@/shared/types/attribut";


export default function ProductAttribut() {
  const { AttributesData, AttributesLoading, AttributesError, AttributesMutate } = useGetAttributs();
  const [selectedRowIds, setSelectedRowIds] = useState<GridRowSelectionModel>([]);
  const [successMessage, setSuccessMessage] = useState<string | null>(null);
  const [errorMessage, setErrorMessage] = useState<string | null>(null);
  const router = useRouter();
  const confirmRows = useBoolean();

  const handleEditRow = useCallback(
    (id: string) => {
      router.push(paths.dashboard.admin.product.editAttribut(id));
    },
    [router]
  );

  const handleRetry = async () => {
    try {
      await AttributesMutate();
    } catch (err) {
      setErrorMessage("Erreur lors du rechargement des attributs.");
      setTimeout(() => {
        setErrorMessage(null);
      }, 5000); 
    }
  };

  const handleDeleteRows = async () =>  {

    try {
      if (selectedRowIds.length > 0) {
        const idsToDelete = selectedRowIds.map(id=>id.toString())
        const response = await useDeleteAttributs(idsToDelete);
       
        await AttributesMutate();

        if (response.status === 200) {
          setSelectedRowIds([]);
          setSuccessMessage("Attributs supprimés avec succès.");
           
        }
      }
    } catch (error) {
      setErrorMessage("Erreur lors de la suppression des attributs.");
    } finally {
      setTimeout(() => {
        setSuccessMessage(null);
        setErrorMessage(null);
      }, 5000); 
    }
  };

  
  const handleConfigureTerms = (type: string, id: string) => {
    switch (type) {
      case "CHOICE":
        router.push(paths.dashboard.admin.product.termeChoix.root(id));
        break;
      case "IMAGE":
        router.push(paths.dashboard.admin.product.termeImage.root(id));
        break;
      case "COLOR":
        router.push(paths.dashboard.admin.product.termeColor.root(id));
        break;
      case "LABEL":
        router.push(paths.dashboard.admin.product.termeLabel.root(id));
        break;
      default:
        break;
    }
  };

  const handleDeleteRow = async (id: string) => {
    try {
      const response = await useDeleteAttribut(id);
      await AttributesMutate();
     

      if (response.status === 200) {
        setSelectedRowIds([]);
        setSuccessMessage("Attribut supprimé avec succès.");
         
      }
    } catch (error) {
      if ((error as any).message  && (error as any).message.includes("constraint")) {
        setErrorMessage("Impossible de supprimer une attrribut liée à un produit.");
      } else {
        setErrorMessage("Erreur lors de la suppression des attrribut.");
      }
    } finally {
      setTimeout(() => {
        setSuccessMessage(null);
        setErrorMessage(null);
      }, 5000); 
    }
  };

  const columns: GridColDef[] = [
    { field: "name", headerName: "Nom", flex: 1, editable: true },
    { field: "identifiant", headerName: "Identifiant", flex: 1, editable: true },
    {
      field: "type",
      headerName: "Type",
      flex: 1, 
      editable: true,
      renderCell: (params) => TypeOption[params.value as keyof typeof TypeOption],
    },
    // {
    //   field: "trie",
    //   headerName: "Trié par",
    //   flex: 1,
    //   editable: true,
    //   renderCell: (params) => TrieOption[params.value as keyof typeof TrieOption],
    // },
    {
      field: "terme",
      headerName: "Termes",
      flex: 1,
      width: 60,
      align: "center",
      editable: true,
      renderCell: (params) => (
        <Tooltip title="Configurer les termes">
          <IconButton onClick={() => {
            handleConfigureTerms(params.row.type as TypeOption, params.row.id)}}>
            <SettingsIcon />
          </IconButton>
        </Tooltip>
      ),
    },
    {
      field: "createdAt",
      headerName: "Créé le",
      flex: 1,
      editable: true,
    },
    {
      type: "actions",
      field: "actions",
      headerName: " ",
      align: "right",
      headerAlign: "right",
      width: 60,
      sortable: false,
      filterable: false,
      disableColumnMenu: true,
      getActions: (params) => [
        <GridActionsCellItem
          showInMenu
          icon={<Iconify icon="solar:pen-bold" />}
          label="modifier"
          onClick={() => handleEditRow(params.row.id)}
        />,
        <GridActionsCellItem
          showInMenu
          icon={<Iconify icon="solar:trash-bin-trash-bold" />}
          label="Supprimer"
          onClick={() => handleDeleteRow(params.row.id)}
          sx={{ color: "error.main" }}
        />,
      ],
    },
  ];

  const rows = AttributesData.map((attribut) => ({
    id: attribut.id,
    name: attribut.nom,
    identifiant: attribut.identifiant,
    type: attribut.type as TypeOption,
    trie: attribut.defaultSort as TrieOption,
    terme: attribut.activeArchive,
    createdAt: attribut.createdAt,
  })).sort((a, b) => new Date(b.createdAt ?? 0).getTime() - new Date(a.createdAt ?? 0).getTime());

  return (
    <Container
      maxWidth="xl"
      sx={{
        flexGrow: 1,
        display: "flex",
        flexDirection: "column",
      }}
    >
      <CustomBreadcrumbs
        heading="Attributs"
        links={[
          { name: "Tableau de bord", href: paths.dashboard.root },
          { name: "Produits", href: paths.dashboard.admin.product.root },
          { name: "Attributs" },
        ]}
        sx={{ mb: { xs: 3, md: 5 } }}
      />

      <Grid container spacing={5}>
        <Grid item xs={3}>
          <ProductAttributAddForm />
        </Grid>

        <Grid item xs={9}>
          {AttributesLoading ? (
            <Stack
              direction="row"
              spacing={2}
              justifyContent="center"
              alignItems="center"
              sx={{ height: '100%', width: '100%' }}
            >
              <CircularProgress />
              <Typography>Chargement des attributs...</Typography>
            </Stack>
          ) : AttributesError ? (
            <Container>
              <Alert severity="error" sx={{ mb: 2 }}>
                Impossible de charger la liste des attributs
              </Alert>
              <Button variant="contained" color="primary" onClick={handleRetry}>
                Réessayer
              </Button>
            </Container>
          ) : (
            <Card variant="outlined">
              <DataGrid
                checkboxSelection
                autoHeight
                getRowHeight={() => "auto"}
                rows={rows}
                columns={columns}
                disableColumnSelector
                disableDensitySelector
                disableRowSelectionOnClick
                disableColumnFilter
                onRowSelectionModelChange={(newSelectionModel) => {
                  setSelectedRowIds(newSelectionModel);
                }}
                localeText={{
                  toolbarColumns: "Colonnes", 
                  toolbarColumnsLabel:"chercher ",
                  toolbarFilters: "Filtrer",
                  toolbarExport: "Exporter",
                }}
                slots={{
                  toolbar: () => (
                    <GridToolbarContainer>
                      <GridToolbarFilterButton />
                      <GridToolbarExport />
                      <GridToolbarQuickFilter  placeholder="Recherche..."/>
                      <Stack
                        spacing={1}
                        flexGrow={1}
                        direction="row"
                        alignItems="center"
                        justifyContent="flex-end"
                      >
                        {!!selectedRowIds.length && (
                          <Button
                            size="small"
                            color="error"
                            startIcon={<Iconify icon="solar:trash-bin-trash-bold" />}
                            onClick={confirmRows.onTrue}
                          >
                            Supprimer ({selectedRowIds.length})
                          </Button>
                        )}
                      </Stack>
                    </GridToolbarContainer>
                  ),
                }}
                slotProps={{
                  toolbar: {
                    showQuickFilter: true,
                  },
                }}
              />
              {errorMessage && (
                <Alert severity="error" onClose={() => setErrorMessage(null)}>
                  {errorMessage}
                </Alert>
              )}
              
              {successMessage && (
                <Alert severity="success" onClose={() => setSuccessMessage(null)}>
                  {successMessage}
                </Alert>
              )}
            </Card>
          )}
        </Grid>
      </Grid>

      <ConfirmDialog
        open={confirmRows.value}
        onClose={confirmRows.onFalse}
        action={
          <Button
            variant="contained"
            color="error"
            onClick={() => {
              handleDeleteRows();
              confirmRows.onFalse();
            }}
          >
            Supprimer
          </Button>
        }
        title="Confirmation"
        content={`Êtes-vous sûr de vouloir supprimer ${selectedRowIds.length} attributs ?`}
      />
    </Container>
  );
}
