"use client";
import { useState } from "react";
import { useFormik } from "formik";
import * as Yup from "yup";
import TextField from "@mui/material/TextField";
import Button from "@mui/material/Button";
import Stack from "@mui/material/Stack";
import Typography from "@mui/material/Typography";
import Card from "@mui/material/Card";
import { IChoice } from "@/shared/types/term";
import { Alert, Snackbar } from "@mui/material";
import { useAddChoice, useGetChoicesByAttribut, checkSlugExistenceChoice } from "@/shared/api/terms";

export default function TermChoixAddForm({
  attributid,
}: {
  attributid: string;
}) {
  const { ChoicesMutate } = useGetChoicesByAttribut(attributid);

  // États pour Snackbar et message d'erreur
  const [openSnackbar, setOpenSnackbar] = useState<boolean>(false);
  const [snackbarMessage, setSnackbarMessage] = useState<string>("");
  const [showSuccessMessage, setShowSuccessMessage] = useState(false);

  const handleCloseSnackbar = () => {
    setOpenSnackbar(false);
  };

  const validationSchema = Yup.object({
    nom: Yup.string()
      .matches(
        /^[a-zA-Z0-9]+(?:[- ][a-zA-Z0-9]+)*$/,
        "Le nom ne doit contenir que des lettres, des chiffres et des tirets."
      )
      .required("Le nom est requis."),
    slug: Yup.string()
      .matches(
        /^[a-zA-Z0-9]+(?:[a-zA-Z0-9- ]*[a-zA-Z0-9]+)*$/,
        "Le slug doit commencer par une lettre ou un chiffre, peut contenir des tirets, et ne doit pas contenir uniquement des caractères spéciaux."
      )
      .required("Le slug est requis.")
      .test("slug-unique", "Ce slug existe déjà.", async (slug) => {
        if (slug) {
          const exists = await checkSlugExistenceChoice(slug, attributid);
          return !exists; // Retourne `false` si le slug existe déjà, ce qui déclenche l'erreur
        }
        return true; // Retourne `true` si le champ `slug` est vide ou valide
      }),
    description: Yup.string().optional(),
  });

  const formik = useFormik({
    initialValues: {
      nom: "",
      slug: "",
      description: "",
    },
    validationSchema: validationSchema,
    onSubmit: async (values, { resetForm }) => {
      const choiceRequest: IChoice = {
        nom: values.nom,
        slug: values.slug,
        description: values.description,
        attributeId: attributid,
      };

      try {
        await useAddChoice(choiceRequest);
        ChoicesMutate();
        setShowSuccessMessage(true);
        resetForm();
        setTimeout(() => {
          setShowSuccessMessage(false);
        }, 5000);
      } catch (error) {
        let errorMessage;
        if (
          typeof error === "string" &&
          error.includes("A Choice with this name already exists")
        ) {
          errorMessage = "Le slug doit également être unique.";
        } else if (
          error instanceof Error &&
          error.message.includes("A Choice tag with this name already exists")
        ) {
          errorMessage = "Le slug doit également être unique.";
        }
        setSnackbarMessage(
          errorMessage ?? "Le terme choix n'a pas pu être créé."
        );
        setOpenSnackbar(true);
      }
    },
  });

  const handleSlugBlur = async (e: React.FocusEvent<HTMLInputElement>) => {
    formik.handleBlur(e);
    const slug = e.target.value;
    const exists = await checkSlugExistenceChoice(slug, attributid);
    if (exists) {
      formik.setFieldError("slug", "Ce slug existe déjà.");
    }
  };

  return (
    <Card variant="outlined">
      <form onSubmit={formik.handleSubmit}>
        <Stack spacing={1.5} sx={{ p: 3 }}>
          <Typography variant="subtitle2">Ajouter le Terme</Typography>

          <TextField
            name="nom"
            label="Nom *"
            InputLabelProps={{ shrink: true }}
            value={formik.values.nom}
            onChange={formik.handleChange}
            onBlur={formik.handleBlur}
            error={formik.touched.nom && Boolean(formik.errors.nom)}
            helperText={formik.touched.nom && formik.errors.nom}
          />

          <TextField
            name="slug"
            label="Slug"
            InputLabelProps={{ shrink: true }}
            value={formik.values.slug}
            onChange={formik.handleChange}
            onBlur={formik.handleBlur} // Utilisation de la méthode intégrée
            error={formik.touched.slug && Boolean(formik.errors.slug)}
            helperText={formik.touched.slug && formik.errors.slug}
          />

          <TextField
            name="description"
            label="Description *"
            multiline
            rows={4}
            InputLabelProps={{ shrink: true }}
            value={formik.values.description}
            onChange={formik.handleChange}
            onBlur={formik.handleBlur}
            error={formik.touched.description && Boolean(formik.errors.description)}
            helperText={formik.touched.description && formik.errors.description}
          />

          <Button
            type="submit"
            variant="contained"
            color="primary"
            disabled={formik.isSubmitting}
          >
            {formik.isSubmitting ? "Ajout en cours..." : "Ajouter"}
          </Button>
        </Stack>

        <Snackbar
          open={openSnackbar}
          autoHideDuration={6000}
          onClose={handleCloseSnackbar}
        >
          <Alert onClose={handleCloseSnackbar} severity="error">
            {snackbarMessage}
          </Alert>
        </Snackbar>
        {showSuccessMessage && (
          <Alert
            severity="success"
            onClose={() => setShowSuccessMessage(false)}
          >
            Le choix a été ajouté avec succès
          </Alert>
        )}
      </form>
    </Card>
  );
}
