"use client";
import { useEffect, useState } from "react";
import TextField from "@mui/material/TextField";
import Button from "@mui/material/Button";
import Stack from "@mui/material/Stack";
import Typography from "@mui/material/Typography";
import Card from "@mui/material/Card";
import { IChoice } from "@/shared/types/term";
import { Alert, Snackbar } from "@mui/material";
import { useUpdateChoice, useGetChoicesByAttribut, useGetChoice, checkSlugExistenceChoice } from "@/shared/api/terms";
import * as yup from "yup"; // Import de yup

//------------------------------------------------------
export default function TermChoixEditForm({
  attributid,
  choiceId,
}: {
  attributid: string;
  choiceId: string;
}) {
  const { ChoicesMutate } = useGetChoicesByAttribut(attributid);
  const { choiceData } = useGetChoice(choiceId);

  const [nom, setNom] = useState<string>("");
  const [slug, setSlug] = useState<string>("");
  const [description, setDescription] = useState<string>("");

  const formSetter = () => {
    setNom(choiceData.nom);
    setSlug(choiceData.slug);
    setDescription(choiceData.description);
  };

  useEffect(() => {
    if (choiceData) {
      formSetter();
    }
  }, [choiceData]);

  // States for Snackbar
  const [openSnackbar, setOpenSnackbar] = useState<boolean>(false);
  const [snackbarMessage, setSnackbarMessage] = useState<string>("");
  const [showSuccessMessage, setShowSuccessMessage] = useState(false);

  const handleCloseSnackbar = () => {
    setOpenSnackbar(false);
  };

  // Définir le schéma de validation avec YUP
  const validationSchema = yup.object({
    nom: yup
      .string()
      .matches(/^[a-zA-Z0-9]+(?:[- ][a-zA-Z0-9]+)*$/, "Le nom ne doit contenir que des lettres, des chiffres et des tirets.")
      .required("Le nom est requis."),
    slug: yup
      .string()
      .matches(/^[a-zA-Z0-9]+(?:[a-zA-Z0-9- ]*[a-zA-Z0-9]+)*$/, "Le slug doit commencer par une lettre ou un chiffre, peut contenir des tirets.")
      .required("Le slug est requis.")
      .test("unique-slug", "Ce slug existe déjà.", async (value) => {
              if (value !== choiceData.slug) { // Si le slug a été modifié
                const exists = await checkSlugExistenceChoice(value, attributid);
                return !exists; // Retourne true si le slug n'existe pas, false s'il existe déjà
              }
              return true; // Si le slug n'a pas été modifié, on ne vérifie pas l'unicité
            }),
    description: yup.string().optional(),
  });

  const validateForm = async (): Promise<boolean> => {
    try {
      await validationSchema.validate({ nom, slug, description }, { abortEarly: false });
      return true;
    } catch (err) {
      if (err instanceof yup.ValidationError) {
        setSnackbarMessage(err.errors[0]); // Affiche la première erreur
        setOpenSnackbar(true);
      }
      return false;
    }
  };

  // Vérification du slug au moment du blur
  const handleSlugBlur = async (e: React.FocusEvent<HTMLInputElement>) => {
    setSlug(e.target.value);
  };

  const handleOnSubmit = async (event: React.FormEvent<HTMLFormElement>) => {
    event.preventDefault();
    const isValid = await validateForm();
    if (!isValid) return;

    const choiceRequest: IChoice = {
      nom,
      slug,
      description,
      attributeId: attributid,
    };

    try {
      await useUpdateChoice(choiceRequest, choiceId);
      ChoicesMutate();
      setShowSuccessMessage(true);
      const timeoutId = setTimeout(() => {
        setShowSuccessMessage(false);
        clearTimeout(timeoutId);
      }, 5000);
    } catch (error) {
      let errorMessage;
      if (
        typeof error === "string" &&
        error.includes("A Choice with this name already exists")
      ) {
        errorMessage = "Le slug doit également être unique.";
      } else if (
        error instanceof Error &&
        error.message.includes("A Choice with this name already exists")
      ) {
        errorMessage = "Le slug doit également être unique.";
      }
      setSnackbarMessage(errorMessage ?? "Le terme choix n'a pas pu être modifié.");
      setOpenSnackbar(true);
    }
  };

  return (
    <Card variant="outlined">
      <form onSubmit={handleOnSubmit}>
        <Stack spacing={1.5} sx={{ p: 3 }}>
          <Typography variant="subtitle2">Modifier le Terme</Typography>

          <TextField
            name="nom"
            label="Nom "
            InputLabelProps={{ shrink: true }}
            value={nom}
            onChange={(e) => setNom(e.target.value)}
          />

          <TextField
            label="Slug "
            name="slug"
            InputLabelProps={{ shrink: true }}
            value={slug}
            onChange={(e) => setSlug(e.target.value)}
            onBlur={handleSlugBlur} // Vérification du slug lors du blur
          />

          <TextField
            name="description"
            label="Description "
            multiline
            rows={4}
            InputLabelProps={{ shrink: true }}
            value={description}
            onChange={(e) => setDescription(e.target.value)}
          />

          <Button type="submit" variant="contained" color="primary">
            Modifier
          </Button>
        </Stack>
        <Snackbar open={openSnackbar} autoHideDuration={6000} onClose={handleCloseSnackbar}>
          <Alert onClose={handleCloseSnackbar} severity="error">
            {snackbarMessage}
          </Alert>
        </Snackbar>
       {showSuccessMessage ? (
        <Alert severity="success" onClose={() => setShowSuccessMessage(false)}>
          Le choix a été modifié avec succès
        </Alert>
      ) : (
        <p></p> 
      )}
      </form>
    </Card>
  );
}
