"use client";

import { useState, useCallback } from "react";
import Container from "@mui/material/Container";
import Stack from "@mui/material/Stack";
import Grid from "@mui/material/Grid";
import Button from "@mui/material/Button";
import Iconify from "@/shared/components/iconify";
import { paths } from "@/routes/paths";
import CustomBreadcrumbs from "@/shared/components/custom-breadcrumbs";
import Card from "@mui/material/Card";
import { useBoolean } from "@/hooks/use-boolean";
import ConfirmDialog from "@/shared/components/confirm-dialog";
import ProductTagsAddForm from "./tags-add-form";
import { useRouter } from "@/hooks";
import {
  DataGrid,
  GridColDef,
  GridActionsCellItem,
  GridToolbarQuickFilter,
  GridToolbarExport,
  GridToolbarContainer,
  GridRowSelectionModel,
  GridToolbarFilterButton,
} from "@mui/x-data-grid";
import { useDeleteTag, useDeleteTags, useGetTags } from "@/shared/api/tag";
import { CircularProgress, Typography, Alert } from "@mui/material";

export default function ProductTags() {
  const [selectedRowIds, setSelectedRowIds] = useState<GridRowSelectionModel>([]);
  const [successMessage, setSuccessMessage] = useState<string | null>(null);
  const [errorMessage, setErrorMessage] = useState<string | null>(null);
  const [confirmDelete, setConfirmDelete] = useState<{ open: boolean; id?: number }>({ open: false });
  const router = useRouter();
  const confirmRows = useBoolean();

  const { tagsData, tagsLoading, tagsError, tagsMutate } = useGetTags();

  const handleEditRow = useCallback(
    (id: string) => {
      router.push(paths.dashboard.admin.product.editTag(id));
    },
    [router]
  );

  const handleRetry = async () => {
    try {
      tagsMutate();
    } catch (err) {
      setErrorMessage("Erreur lors du rechargement des étiquettes.");
      const timeoutId = setTimeout(() => {
        setErrorMessage(null);
        clearTimeout(timeoutId);
      }, 5000); 
    }
  };

  const handleDeleteRows = async () => {
    try {
      if (selectedRowIds.length > 0) {
        const idsToDelete = selectedRowIds.map((id) => id.toString());
        const response = await useDeleteTags(idsToDelete);
        tagsMutate();
  
        if (response.status === 204) {
          setSelectedRowIds([]);
          setSuccessMessage("Étiquette(s) supprimée(s) avec succès.");
          
        }
      }
    } catch (error) {
      if ((error as any).message  && (error as any).message.includes("constraint")) {
        setErrorMessage("Impossible de supprimer une étiquette liée à un produit.");
      } else {
        setErrorMessage("Erreur lors de la suppression des étiquettes.");
      }
    } finally {
      setTimeout(() => {
        setSuccessMessage(null);
        setErrorMessage(null);
      }, 5000); 
    }
  };
  
  const handleDeleteRow = async (id: number) => {
    try {
      const response = await useDeleteTag(id.toString());
      tagsMutate();
  
      if (response.status === 204) {
        setSelectedRowIds([]);
        setSuccessMessage("Étiquette supprimée avec succès.");
      }
    } catch (error) {
      if ((error as any).message  && (error as any).message.includes("constraint")) {
        setErrorMessage("Impossible de supprimer une étiquette liée à un produit.");
      } else {
        setErrorMessage("Erreur lors de la suppression de l'étiquette.");
      }
    } finally {
      setTimeout(() => {
        setSuccessMessage(null);
        setErrorMessage(null);
      }, 5000); // 5 secondes pour nettoyer les messages
    }
  };

  const columns: GridColDef[] = [
    {
      field: "name",
      headerName: "Nom",
      flex: 1,
      editable: true,
    },
    {
      field: "slug",
      headerName: "Slug",
      flex: 1,
      editable: true,
    },
    {
      field: "description",
      headerName: "Description",
      flex: 1,
      editable: true,
    },
    {
      field: "total",
      headerName: "Total",
      flex: 1,
      editable: true,
    },
    {
      field: "createdAt",
      headerName: "Créé le",
      flex: 1,
      editable: true,
    },
    {
      type: "actions",
      field: "actions",
      headerName: " ",
      align: "right",
      headerAlign: "right",
      width: 60,
      sortable: false,
      filterable: false,
      disableColumnMenu: true,
      getActions: (params) => [
        <GridActionsCellItem
          showInMenu
          icon={<Iconify icon="solar:pen-bold" />}
          label="modifier"
          onClick={() => handleEditRow(params.row.id)}
        />,
        <GridActionsCellItem
          showInMenu
          icon={<Iconify icon="solar:trash-bin-trash-bold" />}
          label="Supprimer"
          onClick={() => setConfirmDelete({ open: true, id: params.row.id })}
          sx={{ color: "error.main" }}
        />,
        <GridActionsCellItem
          showInMenu
          icon={<Iconify icon="bi:eye" />}
          label="voir"
        />,
      ],
    },
  ];

  const rows = tagsData
    .map((tag) => ({
      id: tag.id,
      name: tag.nom,
      slug: tag.slug,
      description: tag.description,
      total: tag.total,
      createdAt: tag.createdAt,
    }))
    .sort((a, b) => new Date(b.createdAt ?? 0).getTime() - new Date(a.createdAt ?? 0).getTime());

  return (
    <Container
      maxWidth="xl"
      sx={{
        flexGrow: 1,
        display: "flex",
        flexDirection: "column",
      }}
    >
      <CustomBreadcrumbs
        heading="Étiquettes"
        links={[
          { name: "Tableau de bord", href: paths.dashboard.root },
          { name: "Product", href: paths.dashboard.admin.product.root },
          { name: "Étiquettes" },
        ]}
        sx={{
          mb: {
            xs: 3,
            md: 5,
          },
        }}
      />

      <Grid container spacing={5}>
        <Grid item xs={4}>
          <ProductTagsAddForm  />
        </Grid>

        <Grid item xs={8}>
          {tagsLoading ? (
            <Stack
              direction="row"
              spacing={2}
              justifyContent="center"
              alignItems="center"
              sx={{ height: '100%', width: '100%' }}
            >
              <CircularProgress />
              <Typography>Chargement des étiquettes...</Typography>
            </Stack>
          ) : tagsError ? (
            <Container>
              <Alert severity="error" sx={{ mb: 2 }}>
                Impossible de charger la liste des étiquettes
              </Alert>
              <Button variant="contained" color="primary" onClick={handleRetry}>
                Réessayer
              </Button>
            </Container>
          ) : (
            <Card variant="outlined">
              <DataGrid
                checkboxSelection
                autoHeight
                getRowHeight={() => "auto"}
                rows={rows}
                columns={columns}
                disableColumnSelector
                disableDensitySelector
                pageSizeOptions={[5, 10, 25]}
                initialState={{
                  pagination: {
                    paginationModel: { pageSize: 10 },
                  },
                }}
                onRowSelectionModelChange={(newSelectionModel) => {
                  setSelectedRowIds(newSelectionModel);
                }}
                slots={{
                  toolbar: () => (
                    <GridToolbarContainer>
                      <GridToolbarFilterButton />
                      <GridToolbarExport />
                      <GridToolbarQuickFilter placeholder="Recherche..."/>
                      <Stack
                        spacing={1}
                        flexGrow={1}
                        direction="row"
                        alignItems="center"
                        justifyContent="flex-end"
                      >
                        {!!selectedRowIds.length && (
                          <Stack>
                            <Button
                              size="small"
                              color="error"
                              startIcon={
                                <Iconify icon="solar:trash-bin-trash-bold" />
                              }
                              onClick={confirmRows.onTrue}
                            >
                              Supprimer ({selectedRowIds.length})
                            </Button>
                            
                          </Stack>
                        )}
                      </Stack>
                    </GridToolbarContainer>
                  ),
                }}
                slotProps={{
                  toolbar: {
                    showQuickFilter: true,
                  },
                }}
              />
              {errorMessage && (
                <Alert severity="error" onClose={() => setErrorMessage(null)}>
                  {errorMessage}
                </Alert>
              )}

              {successMessage && (
                <Alert severity="success" onClose={() => setSuccessMessage(null)}>
                  {successMessage}
                </Alert>
              )}

            </Card>
          )}
          <ConfirmDialog
            open={confirmRows.value}
            onClose={confirmRows.onFalse}
            title="Supprimer"
            content={
              <>
                Êtes-vous sûr de vouloir supprimer{" "}
                <strong> {selectedRowIds.length} </strong> items?
              </>
            }
            action={
              <Button
                variant="contained"
                color="error"
                onClick={() => {
                  handleDeleteRows();
                  confirmRows.onFalse();
                }}
              >
                Supprimer
              </Button>
            }
          />
          <ConfirmDialog
            open={confirmDelete.open}
            onClose={() => setConfirmDelete({ open: false })}
            title="Supprimer"
            content="Êtes-vous sûr de vouloir supprimer cette étiquette?"
            action={
              <Button
                variant="contained"
                color="error"
                onClick={() => {
                  if (confirmDelete.id !== undefined) {
                    handleDeleteRow(confirmDelete.id);
                  }
                  setConfirmDelete({ open: false });
                }}
              >
                Supprimer
              </Button>
            }
          />
        </Grid>
      </Grid>
    </Container>
  );
}
