"use client";
import React, { useState } from "react";
import {
  TextField,
  Button,
  Stack,
  Typography,
  Card,
  Snackbar,
  Alert,
} from "@mui/material";
import * as Yup from "yup";
import { ITag } from "@/shared/types/tag";
import { useAddTag, useGetTags } from "@/shared/api/tag";

// Validation schema using Yup
const validationSchema = Yup.object().shape({
  nom: Yup.string()
    .matches(
      /^[a-zA-Z0-9]+(?:[- ][a-zA-Z0-9]+)*$/,
      "Nom ne doit contenir que des lettres, des chiffres et des tirets."
    )
    .required("Le nom est requis"),
  slug: Yup.string()
    .matches(
      /^[a-zA-Z0-9]+(?:[a-zA-Z0-9- ]*[a-zA-Z0-9]+)*$/,
      "Le slug doit commencer par une lettre ou un chiffre, peut contenir des tirets, et ne doit pas contenir uniquement des caractères spéciaux."
    )
    .required("Le slug est requis"),
});

export default function ProductTagsAddForm() {
  const { tagsMutate } = useGetTags();

  // Centralized form data state
  const [formData, setFormData] = useState<{
    nom: string;
    slug: string;
    description: string;
  }>({
    nom: "",
    slug: "",
    description: "",
  });

  // Centralized form errors state
  const [formErrors, setFormErrors] = useState<{
    nom?: string;
    slug?: string;
  }>({});

  // Local Snackbar state
  const [openSnackbar, setOpenSnackbar] = useState<boolean>(false);
  const [snackbarMessage, setSnackbarMessage] = useState<string>("");
  const [snackbarSeverity, setSnackbarSeverity] = useState<
    "success" | "error" | "warning" | "info"
  >("success");

  // Handler for text input changes
  const handleChange = (
    e: React.ChangeEvent<HTMLInputElement | HTMLTextAreaElement>
  ) => {
    const { name, value } = e.target;
    setFormData((prev) => ({
      ...prev,
      [name]: value,
    }));
    // Clear specific field error when user modifies the input
    setFormErrors((prev) => ({
      ...prev,
      [name]: undefined,
    }));
  };

  // Form submission handler
  const handleOnSubmit = async (event: React.FormEvent<HTMLFormElement>) => {
    event.preventDefault();

    try {
      // Validate form data using Yup
      await validationSchema.validate(formData, { abortEarly: false });

      // Prepare tag request object with description set to null
      const tagRequest: ITag = {
        nom: formData.nom,
        slug: formData.slug,
        description: formData.description,
      };

      // Add tag via API
      await useAddTag(tagRequest);
      // Refresh tags list
      tagsMutate();
      // Show success message
      setSnackbarMessage("L'étiquette a été ajoutée avec succès.");
      setSnackbarSeverity("success");
      setOpenSnackbar(true);
      // Reset form fields
      setFormData({
        nom: "",
        slug: "",
        description: "",
      });
      setFormErrors({});
    } catch (error: any) {
      if (error instanceof Yup.ValidationError) {
        // Handle validation errors
        const errors: { nom?: string; slug?: string } = {};
        error.inner.forEach((err) => {
          if (err.path) {
            errors[err.path as keyof typeof errors] = err.message;
          }
        });
        setFormErrors(errors);
        // Show validation error message
        setSnackbarMessage("Veuillez corriger les erreurs du formulaire.");
        setSnackbarSeverity("error");
        setOpenSnackbar(true);
      } else {
        // Handle API or other errors
        let errorMessage = "L'étiquette n'a pas pu être créée.";
        if (
          (typeof error === "string" &&
            error.includes("A tag with this name already exists")) ||
          (error instanceof Error &&
            error.message.includes("A tag with this name already exists"))
        ) {
          errorMessage = "Le slug doit également être unique.";
        }
        setSnackbarMessage(errorMessage);
        setSnackbarSeverity("error");
        setOpenSnackbar(true);
      }
    }
  };

  // Handler to close the Snackbar
  const handleCloseSnackbar = (
    event?: React.SyntheticEvent | Event,
    reason?: string
  ) => {
    if (reason === "clickaway") {
      return;
    }
    setOpenSnackbar(false);
  };

  return (
    <Card variant="outlined">
      <form onSubmit={handleOnSubmit}>
        <Stack spacing={2} sx={{ p: 3 }}>
          <Typography variant="h6">Ajouter une Étiquette</Typography>

          <TextField
            name="nom"
            label="Nom *"
            InputLabelProps={{ shrink: true }}
            value={formData.nom}
            onChange={handleChange}
            error={Boolean(formErrors.nom)}
            helperText={formErrors.nom}
            fullWidth
          />

          <TextField
            name="slug"
            label="Slug *"
            InputLabelProps={{ shrink: true }}
            value={formData.slug}
            onChange={handleChange}
            error={Boolean(formErrors.slug)}
            helperText={formErrors.slug}
            fullWidth
          />

          <TextField
            name="description"
            label="Description"
            multiline
            rows={4}
            InputLabelProps={{ shrink: true }}
            value={formData.description}
            onChange={handleChange}
          />

          <Button type="submit" variant="contained" color="primary">
            Enregistrer
          </Button>
        </Stack>

        {/* Snackbar for Notifications */}
        <Snackbar
          open={openSnackbar}
          autoHideDuration={6000}
          onClose={handleCloseSnackbar}
          anchorOrigin={{ vertical: "bottom", horizontal: "center" }}
        >
          <Alert
            onClose={handleCloseSnackbar}
            severity={snackbarSeverity}
            sx={{ width: "100%" }}
          >
            {snackbarMessage}
          </Alert>
        </Snackbar>
      </form>
    </Card>
  );
}
