import React, { useState, useEffect } from 'react';
import { useGetTag, useGetTags, useUpdateTag } from '@/shared/api/tag';
import { ITag } from '@/shared/types/tag';
import { TextField, Button, Stack, Typography, Card, Alert, Snackbar } from "@mui/material";

export default function ProductTagsEditForm({ id }: { id: string }) {
  const [nom, setNom] = useState<string>('');
  const [slug, setSlug] = useState<string>('');
  const [description, setDescription] = useState<string>('');
  const [openSnackbar, setOpenSnackbar] = useState<boolean>(false);
  const [snackbarMessage, setSnackbarMessage] = useState<string>("");
  const [showSuccessMessage, setShowSuccessMessage] = useState(false);

  // Use hooks directly
  const { tagData, } = useGetTag(id);
  const { tagsMutate } = useGetTags();

  useEffect(() => {
    if (tagData) {
      setNom(tagData.nom);
      setSlug(tagData.slug);
      setDescription(tagData.description);
    }
  }, [tagData]);

  const validateForm = (): boolean => {
    if (!nom || !slug || !description) {
      setSnackbarMessage("Tous les champs requis doivent être remplis.");
      setOpenSnackbar(true);
      return false;
    }
    return true;
  };

  const handleCloseSnackbar = () => {
    setOpenSnackbar(false);
  };

  const handleOnSubmit = async (event: React.FormEvent<HTMLFormElement>) => {
    event.preventDefault();
    if (!validateForm()) {
      return;
    }
    const tagRequest: ITag = { nom, slug, description };

    try {
      await useUpdateTag(tagRequest, id);
      tagsMutate();
      setShowSuccessMessage(true);
      const timeoutId =setTimeout(() => {setShowSuccessMessage(false)
        clearTimeout(timeoutId)
      }, 5000);
    } catch (error) {
      setSnackbarMessage("l'Étiquette n'a pas pu être modifiée.");
      setOpenSnackbar(true);
      const timeoutId =setTimeout(() => {setOpenSnackbar(false);
        clearTimeout(timeoutId)
      }, 5000);
    }
  };

  return (
    <Card variant="outlined">
      <form onSubmit={handleOnSubmit}>
        <Stack spacing={1.5} sx={{ p: 3 }}>
          <Typography variant="subtitle2">Modifier l'étiquette
          </Typography>
          <TextField name="nom" label="Nom" InputLabelProps={{ shrink: true }} value={nom} onChange={(e) => setNom(e.target.value)} />
          <TextField label="Slug" name="slug" InputLabelProps={{ shrink: true }} value={slug} onChange={(e) => setSlug(e.target.value)} />
          <TextField name="description" label="Description" multiline rows={4} InputLabelProps={{ shrink: true }} value={description} onChange={(e) => setDescription(e.target.value)} />
          <Button type="submit" variant="contained" color="primary">Enregistrer</Button>
        </Stack>
        <Snackbar open={openSnackbar} autoHideDuration={6000} onClose={handleCloseSnackbar}>
          <Alert onClose={handleCloseSnackbar} severity="error">{snackbarMessage}</Alert>
        </Snackbar>
        {showSuccessMessage && <Alert severity="success" onClose={() => setShowSuccessMessage(false)}>l'Étiquette ajoutée avec succès</Alert>}
      </form>
    </Card>
  );
}
