import React, { useEffect } from "react";
import {
  Grid,
  FormControlLabel,
  Switch,
  Typography,
  CardHeader,
  Card,
  Stack,
  Divider,
  Box,
} from "@mui/material";
import LoadingButton from "@mui/lab/LoadingButton";
import { IProductItem } from "@/shared/types/product";
import { Controller, useFormContext, useWatch } from "react-hook-form";

interface FormActionsProps {
  mdUp: boolean;
  isSubmitting: boolean;
  currentProduct?: IProductItem;
  onClose?: () => void;
}

export default function FormActions({
  mdUp,
  isSubmitting,
  currentProduct,
  onClose,
}: FormActionsProps) {
  const { control, setValue } = useFormContext();

  const publishLive = useWatch({ control, name: "publishLive" });
  const publishWeb = useWatch({ control, name: "publishWeb" });

  useEffect(() => {
    if (publishLive || publishWeb) {
      setValue("draft", false);
    } else {
      setValue("draft", true);
    }
  }, [publishLive, publishWeb, setValue]);

  const isValidDate = (date: string | number | Date) =>
    !isNaN(new Date(date).getTime());

  const publicationDate =
    currentProduct?.createdAt && isValidDate(currentProduct.createdAt)
      ? new Date(currentProduct.createdAt).toLocaleString()
      : "N/A";

  const creationDate =
    currentProduct?.createdAt && isValidDate(currentProduct.createdAt)
      ? new Date(currentProduct.createdAt).toLocaleString()
      : "N/A";

  const modificationDate =
    currentProduct?.updatedAt && isValidDate(currentProduct.updatedAt)
      ? new Date(currentProduct.updatedAt).toLocaleString()
      : "N/A";

  return (
    <>
      {mdUp && (
        <Grid item md={4}>
          <Typography variant="h6" sx={{ mb: 0.5 }}>
            Publication
          </Typography>
          <Typography variant="body2" sx={{ color: "text.secondary" }}>
            La publication du produit
          </Typography>
        </Grid>
      )}
      <Grid item xs={12} md={8}>
        <Card>
          {!mdUp && <CardHeader title="Publication" />}
          <Stack spacing={2} sx={{ p: 3 }}>
            <Box display="flex" alignItems="center">
              <Controller
                name="draft"
                control={control}
                defaultValue={currentProduct?.draft ?? false}
                render={({ field }) => (
                  <FormControlLabel
                    control={
                      <Switch
                        {...field}
                        checked={!field.value}
                        disabled
                      />
                    }
                    label="Publier"
                    sx={{ flexGrow: 1, pl: 3 }}
                  />
                )}
              />
            </Box>
            <Divider sx={{ borderStyle: "dashed" }} />
            <Stack spacing={1}>
              <Typography variant="body2">
                <strong>Date de publication:</strong> {publicationDate}
              </Typography>
              <Typography variant="body2">
                <strong>Date de création:</strong> {creationDate}
              </Typography>
              <Typography variant="body2">
                <strong>Date de modification:</strong> {modificationDate}
              </Typography>
            </Stack>
          </Stack>
        </Card>
        <Box sx={{ mt: 2 }}>
          <LoadingButton
            type="submit"
            variant="contained"
            size="large"
            loading={isSubmitting}
            fullWidth
          >
            {!currentProduct
              ? "Créer le produit"
              : "Enregistrer les modifications"}
          </LoadingButton>
        </Box>
        {onClose && (
          <Box sx={{ mt: 2 }}>
            <LoadingButton
              variant="outlined"
              size="large"
              fullWidth
              onClick={onClose}
            >
              Annuler
            </LoadingButton>
          </Box>
        )}
      </Grid>
    </>
  );
}
