import React, { useMemo, useState, useEffect } from "react";
import { useForm, Resolver, Controller, SubmitHandler, FieldErrors, Path } from "react-hook-form";
import { yupResolver } from "@hookform/resolvers/yup";
import * as Yup from "yup";
import { Grid } from "@mui/material";
import { useRouter } from "@/hooks";
import { useResponsive } from "@/hooks/use-responsive";
import FormProvider from "@/shared/components/hook-form";
import ProductDetails from "./ProductDetails";
import ProductProperties from "./ProductProperties";
import ProductCategory from "./ProductCategory";
import FormActions from "./FormActions";
import {
  AuthorizePreOrder,
  IProductItem,
  IVariation,
  ProductType,
} from "@/shared/types/product";
import {
  addVariableProduct,
  updateProduct,
  useAllGetProducts,
  useAddSimpleProduct,
  useUpdateProduct,
  checkUgsUnique,
} from "@/shared/api/product";
import { paths } from "@/routes/paths";
import CustomAlert from "@/shared/components/CustomAlert";
import { logger } from "@/utils/logger";
import { CustomProduct } from "../../supplier/supplier-orders/add-edit-SupplierOrder/order-new-edit-details";


type Props = {
  currentProduct?: IProductItem;
  onClose?: () => void;
  onAddProduct?: (product: CustomProduct) => void;
  defaultSupplierName?: string;
};

export default function ProductNewEditForm({
  currentProduct,
  onClose,
  onAddProduct,
  defaultSupplierName,
}: Props) {
  const router = useRouter();
  const mdUp = useResponsive("up", "md");
  const [mediaIds, setMediaIds] = useState<(number | string)[]>([]);
  const [pathList, setPathList] = useState<(number | string)[]>([]);
  const [mediaId, setMediaId] = useState<number | string>();
  const [path, setPath] = useState<number | string>();
  const [variations, setVariations] = useState<IVariation[]>([]);
  const { revalidate } = useAllGetProducts();
  const { productMutate } = useUpdateProduct(currentProduct?.id!);

  const [alert, setAlert] = useState<{ message: string; type: 'success' | 'error' } | null>(null);

  useEffect(() => {
    if (alert) {
      const timer = setTimeout(() => {
        setAlert(null);
      }, 1000);
      return () => clearTimeout(timer);
    }
  }, [alert]);
  const [isCheckingUgs, setIsCheckingUgs] = useState(false);

  const NewProductSchema = Yup.object().shape({
    ugs: Yup.string()
      .required("UGS est requis")
      .test("is-unique", "UGS déjà utilisé.", async function (value) {
        const { currentProduct } = this.options.context as { currentProduct?: IProductItem };
        if (currentProduct && currentProduct.ugs === value) {
          return true;
        }
        const isUnique = await checkUgsUnique(value || "");
        return isUnique;
      }),
    name: Yup.string().required("Le nom est requis"),
    regularPrice: Yup.number()
      .required("Le tarif régulier est requis")
      .typeError("Le tarif régulier doit être un nombre")
      .moreThan(0, "Le prix doit être strictement positif"),
    promoPrice: Yup.number()
      .nullable()
      .transform((value, originalValue) => (originalValue === "" ? null : value))
      .typeError("Le prix promo doit être un nombre")
      .min(0, "Le prix promo ne peut pas être négatif"),
    description: Yup.string().optional(),
    shortDescription: Yup.string().optional(),
    draft: Yup.boolean().required("Le statut de brouillon est requis"),
    publishWeb: Yup.boolean().optional(),
    publishLive: Yup.boolean().optional(),
    liveProductConfigs: Yup.object().shape({
      feedStockFromWebStock: Yup.boolean().optional(),
      authorizeUpdateQuantity: Yup.boolean().optional(),
    }),

    stockWebConfig: Yup.object().shape({
      quantity: Yup.number()
        .nullable()
        .transform((value, originalValue) => (originalValue === "" ? null : value))
        .typeError("La quantité Web doit être un nombre")
        .min(0, "La quantité Web ne peut pas être négative"),
      thresholdValue: Yup.number()
        .nullable()
        .transform((value, originalValue) => (originalValue === "" ? null : value))
        .typeError("Le seuil de précommande Web doit être un nombre")
        .min(0, "Le seuil de précommande Web ne peut pas être négatif"),
    }),

    stockLiveConfig: Yup.object().shape({
      quantity: Yup.number()
        .nullable()
        .transform((value, originalValue) => (originalValue === "" ? null : value))
        .typeError("La quantité Live doit être un nombre")
        .min(0, "La quantité Live ne peut pas être négative"),
      thresholdValue: Yup.number()
        .nullable()
        .transform((value, originalValue) => (originalValue === "" ? null : value))
        .typeError("Le seuil de précommande Web doit être un nombre")
        .min(0, "Le seuil de précommande Web ne peut pas être négatif"),
    }),

    stockAlert: Yup.number()
      .nullable()
      .typeError("Le seuil de stock doit être un nombre")
      .min(0, "Le seuil de stock ne peut pas être négatif"),
    weight: Yup.number()
      .nullable()
      .typeError("Le poids doit être un nombre")
      .min(0, "Le poids ne peut pas être négatif"),
    lengthP: Yup.number()
      .nullable()
      .typeError("La longueur doit être un nombre")
      .min(0, "La longueur ne peut pas être négative"),
    widthP: Yup.number()
      .nullable()
      .typeError("La largeur doit être un nombre")
      .min(0, "La largeur ne peut pas être négative"),
    heightP: Yup.number()
      .nullable()
      .typeError("La hauteur doit être un nombre")
      .min(0, "La hauteur ne peut pas être négative"),
    variations: Yup.array().of(
      Yup.object().shape({
        regularPrice: Yup.number()
          .required("Le tarif régulier est requis")
          .typeError("Le tarif régulier doit être un nombre")
          .moreThan(0, "Le prix doit être strictement positif"),

        stockWebConfig: Yup.object().shape({
          quantity: Yup.number()
            .nullable()
            .transform((value, originalValue) => (originalValue === "" ? null : value))
            .typeError("La quantité Web doit être un nombre")
            .min(0, "La quantité Web ne peut pas être négative"),
          thresholdValue: Yup.number()
            .nullable()
            .transform((value, originalValue) => (originalValue === "" ? null : value))
            .typeError("Le seuil de précommande Web doit être un nombre")
            .min(0, "Le seuil de précommande Web ne peut pas être négatif"),
        }),

        stockLiveConfig: Yup.object().shape({
          quantity: Yup.number()
            .nullable()
            .transform((value, originalValue) => (originalValue === "" ? null : value))
            .typeError("La quantité Live doit être un nombre")
            .min(0, "La quantité Live ne peut pas être négative"),
          thresholdValue: Yup.number()
            .nullable()
            .transform((value, originalValue) => (originalValue === "" ? null : value))
            .typeError("Le seuil de précommande Web doit être un nombre")
            .min(0, "Le seuil de précommande Web ne peut pas être négatif"),
        }),
      })

    ),

  });

  const defaultValues = useMemo(
    () => ({
      ugs: currentProduct?.ugs || "",
      name: currentProduct?.name || "",
      description: currentProduct?.description || "",
      shortDescription: currentProduct?.shortDescription || "",
      imagesIDs: currentProduct?.imagesIDs || [],
      mainImageID: currentProduct?.mainImageID,
      mainImagePath: currentProduct?.mainImagePath,
      imagesPaths: currentProduct?.imagesPaths,
      productType: currentProduct?.productType || ProductType.SIMPLE,
      draft: currentProduct?.draft,
      material: currentProduct?.material || "",
      categoryIds: currentProduct?.categoryIds,
      tagIds: currentProduct?.tagIds || [],
      variations: currentProduct?.variations || [],
      regularPrice: currentProduct?.regularPrice,
      promoPrice: currentProduct?.promoPrice,
      stockAlert: currentProduct?.stockAlert,
      shippingClassId: currentProduct?.shippingClassId,
      publishWeb: currentProduct?.publishWeb || false,
      publishLive: currentProduct?.publishLive || false,
      weight: currentProduct?.weight,
      lengthP: currentProduct?.lengthP,
      widthP: currentProduct?.widthP,
      heightP: currentProduct?.heightP,
      emplacement: currentProduct?.emplacement || "",
      supplierRef: currentProduct?.supplierRef,
      supplierName: currentProduct?.supplierName || defaultSupplierName || '',
      stockWebConfig: currentProduct?.stockWebConfig || {
        quantity: null,
        isPreOrderAuthorized: AuthorizePreOrder.DO_NOT_AUTHORIZE,
        authorizedConfig: false,
        thresholdValue: null,
      },
      stockLiveConfig: currentProduct?.stockLiveConfig || {
        quantity: null,
        isPreOrderAuthorized: AuthorizePreOrder.AUTHORIZE,
        authorizedConfig: false,
        thresholdValue: null,
      },
      liveProductConfigs: currentProduct?.liveProductConfigs || {
        feedStockFromWebStock: true,
        authorizeUpdateQuantity: true,
      },
      purchasePrice: currentProduct?.purchasePrice,
      secondPurchasePrice: currentProduct?.secondPurchasePrice,
      supplierNameSecond: currentProduct?.supplierNameSecond,
      supplierRefSecond: currentProduct?.supplierRefSecond,
    }),
    [currentProduct, defaultSupplierName]
  );
  const isEditing = Boolean(currentProduct);
  const methods = useForm<IProductItem>({
    resolver: yupResolver(NewProductSchema) as unknown as Resolver<
      IProductItem,
      any
    >,
    defaultValues,
    context: { currentProduct },
    mode: "onChange",
  });

  const {
    reset,
    watch,
    control,
    handleSubmit,
    formState: { isSubmitting },
    setError,
    clearErrors,
    setFocus,
  } = methods;
  const values = watch();

  useEffect(() => {
    if (variations.length > 0) {
      methods.setValue("variations", variations);
    }
  }, [variations, methods]);

  useEffect(() => {
    if (currentProduct) {
      setMediaIds(currentProduct.imagesIDs || []);
      setPathList(currentProduct.imagesPaths || []);
      setMediaId(currentProduct.mainImageID);
      setPath(currentProduct.mainImagePath);
      setVariations(currentProduct.variations || []);
      reset({
        ...defaultValues,
        categoryIds: currentProduct.categoryIds || [],
        tagIds: currentProduct.tagIds || [],
        variations: currentProduct.variations || [],
      });
    }
  }, [currentProduct, reset, defaultValues]);

  const ugsValue = watch("ugs");

  useEffect(() => {
    if (ugsValue === undefined) return;

    const handler = setTimeout(async () => {
      if (ugsValue) {
        if (isEditing && currentProduct && currentProduct.ugs === ugsValue) {
          clearErrors("ugs");
          setIsCheckingUgs(false);
          return;
        }
        setIsCheckingUgs(true);
        const isUnique = await checkUgsUnique(ugsValue);
        setIsCheckingUgs(false);
        if (!isUnique) {
          setError("ugs", {
            type: "manual",
            message: "UGS déjà utilisé.",
          });
        } else {
          clearErrors("ugs");
        }
      }
    }, 300);

    return () => {
      clearTimeout(handler);
    };
  }, [ugsValue, isEditing, currentProduct, setError, clearErrors]);

  const onSubmit: SubmitHandler<IProductItem> = async (data) => {
    try {
      const processedVariations = variations.map(({ id, isNew, ...rest }) => {
        const matchedVariation = data.variations.find((v) => v.id === id);

        const publishWeb = matchedVariation?.publishWeb ?? rest.publishWeb;
        const publishLive = matchedVariation?.publishLive ?? rest.publishLive;

        return {
          ...rest,
          publishWeb,
          publishLive,
          ...(!isNew && { id }),
        };
      });
      const formData = {
        ...data,
        mainImagePath: path ?? "",
        imagesPaths: pathList.map((p) => String(p)),
        mainImageID: mediaId,
        imagesIDs: mediaIds,
        categoryIds: values.categoryIds,
        tagIds: values.tagIds,
        supplierId: data.idSupplier,
        supplierName: data.supplierName,
        variations: processedVariations,
      };
      let response;
      logger("Transformed form data:", formData);
      logger("Current form data:", currentProduct);
      if (isEditing) {
        response = await updateProduct(
          currentProduct!.id!.toString(),
          formData
        );
        productMutate();
        revalidate();
      } else {
        if (formData.productType === ProductType.SIMPLE) {
          response = await useAddSimpleProduct(formData);
        } else {
          response = await addVariableProduct(formData);
        }
        revalidate();
      }
      reset();

      let message = isEditing ? "Mise à jour réussie !" : "Création réussie !";

      if (data.draft === true) {
        message += " Le produit est publié en mode brouillon.";
      } else {
        message += " Le produit est publié en production";
        const publishedOn = [];
        if (data.publishWeb === true) publishedOn.push("le web");
        if (data.publishLive === true) publishedOn.push("le live");
        if (publishedOn.length > 0) {
          message += " sur " + publishedOn.join(" et ") + ".";
        } else {
          message += ", mais n'est ni publié sur le web ni sur le live.";
        }
      }

      setAlert({
        message,
        type: 'success',
      });

      if (onAddProduct && response.id && response.mainImagePath) {
        await onAddProduct({
          id: response.id,
          name: response.name,
          productPrice: response.purchasePrice!,
          isVariable: response.productType == ProductType.VARIABLE,
          mainImagePath: response.mainImagePath,
          type: response.productType,
          quantity:
            (response.stockWebConfig?.quantity || 0) +
            (response.stockLiveConfig?.quantity || 0),
          variations: response.variations ? response.variations.map(
            (v: any) => ({
              id: v.id,
              name: v.name,
              productPrice: v.purchasePrice,
              mainImagePath: v.mainImagePath || response.mainImagePath,
              quantity: (v.stockWebConfig?.quantity || 0) +
                (v.stockLiveConfig?.quantity || 0)
            })
          ) : []
        });
        onClose && onClose();
        return;
      }
      if (onClose) {
        onClose();
      }
      router.push(paths.dashboard.admin.product.root);
    } catch (error: any) {
      setAlert({
        message: error.message,
        type: 'error',
      });
      logger("Error adding product:", error);
    }
  };

  const findFirstErrorFieldName = (
    errors: FieldErrors<IProductItem>
  ): Path<IProductItem> | undefined => {
    for (const [fieldName, errorValue] of Object.entries(errors)) {
      if ((errorValue as any)?.ref) {
        return fieldName as Path<IProductItem>;
      } else if (typeof errorValue === 'object' && errorValue !== null) {
        const nestedFieldName = findFirstErrorFieldName(errorValue as FieldErrors<IProductItem>);
        if (nestedFieldName) {
          return `${fieldName}.${nestedFieldName}` as Path<IProductItem>;
        }
      }
    }
    return undefined;
  };

  const onError = (errors: FieldErrors<IProductItem>) => {
    const errorFields = collectErrorFields(errors);

    setAlert({
      message: `Veuillez remplir les champs suivants : ${errorFields.join(', ')}`,
      type: 'error',
    });

    const firstErrorField = findFirstErrorFieldName(errors);
    if (firstErrorField) {
      setFocus(firstErrorField);
    }
  };

  const collectErrorFields = (errors: FieldErrors<IProductItem>, parentKey = ''): string[] => {
    let errorFields: string[] = [];
    for (const key in errors) {
      if (Object.prototype.hasOwnProperty.call(errors, key)) {
        const error = errors[key as keyof IProductItem];
        const fieldKey = parentKey ? `${parentKey}.${key}` : key;

        if (error?.message) {
          errorFields.push(getFieldLabel(fieldKey));
        } else if (typeof error === 'object' && error !== null) {
          errorFields = errorFields.concat(collectErrorFields(error as FieldErrors<IProductItem>, fieldKey));
        }
      }
    }
    return errorFields;
  };

  const getFieldLabel = (fieldKey: string): string => {
    const fieldLabels: { [key: string]: string } = {
      ugs: 'UGS produit',
      name: 'Nom du produit',
      regularPrice: 'Tarif régulier',

    };
    return fieldLabels[fieldKey] || fieldKey;
  };

  return (
    <>
      {alert && (
        <CustomAlert
          message={alert.message}
          type={alert.type}
          onClose={() => setAlert(null)}
          style={{ top: '20px', right: '20px', zIndex: 3000 }}
        />
      )}
      <FormProvider methods={methods} onSubmit={handleSubmit(onSubmit, onError)}>
        <ProductDetails
          mdUp={mdUp}
          setMediaIds={setMediaIds}
          setMediaId={setMediaId}
          setPath={setPath}
          setPathList={setPathList}
          isEditing={isEditing}
          currentProduct={currentProduct}
          defaultSupplierName={defaultSupplierName}
        />

        <Grid container spacing={3} justifyContent="flex-start">
          <ProductCategory mdUp={mdUp} control={control} />
          <ProductProperties
            mdUp={mdUp}
            control={control}
            values={values}
            setVariations={setVariations}
            currentProduct={currentProduct}
            mainImagePath={path}
            isCheckingUgs={isCheckingUgs}
          />

          <FormActions
            mdUp={mdUp}
            isSubmitting={isSubmitting}
            currentProduct={currentProduct}
            onClose={onClose}
          />
        </Grid>
      </FormProvider>
    </>
  );
}
