import React, { useState } from "react";
import {
  Stack,
  Card,
  Typography,
  Grid,
  CardHeader,
  Divider,
  Checkbox,
  TextField,
  Chip,
  Autocomplete,
} from "@mui/material";
import CheckBoxOutlineBlankIcon from "@mui/icons-material/CheckBoxOutlineBlank";
import CheckBoxIcon from "@mui/icons-material/CheckBox";
import { useGetHierarchicalCategories } from "@/shared/api/product";
import { useGetTags } from "@/shared/api/tag";
import { Controller } from "react-hook-form";

const icon = <CheckBoxOutlineBlankIcon fontSize="small" />;
const checkedIcon = <CheckBoxIcon fontSize="small" />;

interface ProductCategoryProps {
  mdUp: boolean;
  control: any;
}

export default function ProductCategory({
  mdUp,
  control,
}: ProductCategoryProps) {
  const {
    categories,
    isLoading: isLoadingCategories,
    isError: isErrorCategories,
  } = useGetHierarchicalCategories();
  const { tagsData: tags } = useGetTags();
  interface CategoryOption {
    group: string | null;
    classify: string | null;
    label: string;
    id: number;
    isParent: boolean;
  }

  interface TagOption {
    id: string;
    nom: string;
  }
  if (isLoadingCategories) return <div>Loading...</div>;
  if (isErrorCategories) return <div>Error loading categories</div>;

  const flattenCategories = (
    categories: any[],
    parentLabel: string | null = null
  ): CategoryOption[] => {
    return categories.flatMap((category: any) => {
      const label = parentLabel ? ` ${category.nom}` : category.nom;
      const hasChildren = category.children && category.children.length > 0;
      
      const children = hasChildren
        ? flattenCategories(category.children, label)
        : [];
      
      return [
        { 
          group: parentLabel, 
          classify: category.nom, 
          label, 
          id: category.id,
          isParent: hasChildren 
        },
        ...children,
      ];
    });
  };

  const allCategories = flattenCategories(categories);
  const selectableCategoryOptions = allCategories.filter(cat => !cat.isParent);

  return (
    <>
      {mdUp && (
        <Grid item md={4}>
          <Typography variant="h6" sx={{ mb: 0.5 }}>
            Catégories et Étiquettes
          </Typography>
          <Typography variant="body2" sx={{ color: "text.secondary" }}>
            Ajouter des catégories et des étiquettes
          </Typography>
        </Grid>
      )}
      <Grid item xs={12} md={8} sx={{ mb: 3 }}>
        <Card>
          {!mdUp && <CardHeader title="Catégories et Étiquettes" />}
          <Stack spacing={3} sx={{ p: 3 }}>
            {/* Categories */}
            <Controller
              name="categoryIds"
              control={control}
              render={({ field }) => (
                <Autocomplete
                  multiple
                  id="categoryIds"
                  options={selectableCategoryOptions}
                  groupBy={(option) => option.group || ""}
                  disableCloseOnSelect
                  getOptionLabel={(option) =>
                    option.classify || option.group || ""
                  }
                  isOptionEqualToValue={(option, value) =>
                    option.id === value.id
                  }
                  value={selectableCategoryOptions.filter((option) =>
                    field.value?.includes(option.id)
                  )}
                  onChange={(_event, value) =>
                    field.onChange(value.map((v) => v.id))
                  }
                  renderOption={(props, option, { selected }) => (
                    <li {...props}>
                      <Checkbox
                        icon={icon}
                        checkedIcon={checkedIcon}
                        style={{ marginRight: 8 }}
                        checked={selected}
                      />
                      {option.label}
                    </li>
                  )}
                  renderInput={(params) => (
                    <TextField
                      {...params}
                      label="Catégorie"
                      placeholder="Sélectionnez des sous-catégories"
                    />
                  )}
                  renderTags={(selected, getTagProps) =>
                    selected.map((option, index) => (
                      <Chip
                        {...getTagProps({ index })}
                        key={option.id}
                        label={option.label}
                        size="small"
                        color="info"
                        variant="soft"
                      />
                    ))
                  }
                />
              )}
            />
          </Stack>
          <Divider sx={{ borderStyle: "dashed" }} />
          <Stack spacing={3} sx={{ p: 3 }}>
            {/* Tags */}
            <Controller
              name="tagIds"
              control={control}
              render={({ field }) => (
                <Autocomplete
                  multiple
                  id="tagIds"
                  options={tags.map((tag) => ({ id: tag.id!, nom: tag.nom }))}
                  getOptionLabel={(option) => option.nom}
                  isOptionEqualToValue={(option, value) =>
                    option.id === value.id
                  }
                  value={tags
                    .map((tag) => ({ id: tag.id, nom: tag.nom }))
                    .filter((option) => field.value?.includes(option.id))}
                  onChange={(_event, value) =>
                    field.onChange(value.map((v) => v.id))
                  }
                  renderOption={(props, option, { selected }) => (
                    <li {...props}>
                      <Checkbox
                        icon={icon}
                        checkedIcon={checkedIcon}
                        style={{ marginRight: 8 }}
                        checked={selected}
                      />
                      {option.nom}
                    </li>
                  )}
                  renderInput={(params) => (
                    <TextField
                      {...params}
                      label="Étiquettes"
                      placeholder="Étiquettes"
                    />
                  )}
                  renderTags={(selected, getTagProps) =>
                    selected.map((option, index) => (
                      <Chip
                        {...getTagProps({ index })}
                        key={option.id}
                        label={option.nom}
                        size="small"
                        color="info"
                        variant="soft"
                      />
                    ))
                  }
                />
              )}
            />
          </Stack>
          <Divider sx={{ borderStyle: "dashed" }} />
          <Stack spacing={3} sx={{ p: 3 }}>
            <Controller
              name="material"
              control={control}
              defaultValue=""
              render={({ field }) => (
                <TextField {...field} label="Matière" placeholder="Matière" />
              )}
            />

            <Controller
              name="emplacement"
              control={control}
              defaultValue=""
              render={({ field }) => (
                <TextField
                  {...field}
                  label="Emplacement"
                  placeholder="Emplacement"
                />
              )}
            />
          </Stack>
        </Card>
      </Grid>
    </>
  );
}
