import React, { Dispatch, SetStateAction, useEffect, useState, useRef } from "react";
import {
  Stack,
  Card,
  Typography,
  Grid,
  CardHeader,
  Box,
  Divider,
  IconButton,
  Button,
  FormControlLabel,
  Switch,
  TextField,
  Autocomplete,
  CircularProgress,
  Tooltip,
} from "@mui/material";
import { Controller } from "react-hook-form";
import AddIcon from "@mui/icons-material/Add";
import DeleteIcon from "@mui/icons-material/Delete";
import ProductExpedition from "./ProductExpedition";
import Variations from "./ProductVariations";
import { RHFSelect, RHFTextField } from "@/shared/components/hook-form";
import {
  ProductType,
  AuthorizePreOrder,
  IVariation,
  IProductItem,
} from "@/shared/types/product";
import {
  useGetAttributs,
  useGetTermsByAttributeId,
} from "@/shared/api/attribut";
import { fetcher } from "@/shared/api/server";
import LibraryAddCheckSharpIcon from '@mui/icons-material/LibraryAddCheckSharp';

interface ProductPropertiesProps {
  mdUp: boolean;
  control: any;
  values: any;
  setVariations: Dispatch<SetStateAction<IVariation[]>>;
  currentProduct?: IProductItem;
  mainImagePath?: string | number;
  isCheckingUgs: boolean;
}

interface Attribute {
  id?: number;
  attribute: string;
  type: string | null;
  values: string[];
  termIds: {
    termLabelId: number | null;
    termImageId: number | null;
    termColorId: number | null;
    termChoixId: number | null;
  };
}

const PRODUCT_TYPE_OPTIONS = [
  { value: ProductType.SIMPLE, label: "Produit Simple" },
  { value: ProductType.VARIABLE, label: "Produit Variable" },
];

const ALLOW_BACKORDER_OPTIONS = [
  { value: AuthorizePreOrder.DO_NOT_AUTHORIZE, label: "Ne pas autoriser" },
  { value: AuthorizePreOrder.AUTHORIZE, label: "Autoriser" },
  {
    value: AuthorizePreOrder.AUTHORIZE_WITH_CUSTOMER_NOTIFICATION,
    label: "Autoriser, mais avec notification client",
  },
];

export default function ProductProperties({
  mdUp,
  control,
  values,
  setVariations,
  currentProduct,
  mainImagePath,
  isCheckingUgs
}: ProductPropertiesProps) {
  const [productType, setProductType] = useState(ProductType.SIMPLE);
  const [publishOnWeb, setPublishOnWeb] = useState(false);
  const [publishOnLive, setPublishOnLive] = useState(false);
  const [allowPreorderWeb, setAllowPreorderWeb] = useState(false);
  const [allowPreorderLive, setAllowPreorderLive] = useState(false);
  const [backorderOptionWeb, setBackorderOptionWeb] = useState(
    AuthorizePreOrder.DO_NOT_AUTHORIZE
  );
  const [backorderOptionLive, setBackorderOptionLive] = useState(
    AuthorizePreOrder.DO_NOT_AUTHORIZE
  );

  const [attributes, setAttributes] = useState<Attribute[]>([
    {
      attribute: "",
      values: [],
      type: "",
      termIds: {
        termLabelId: null,
        termImageId: null,
        termColorId: null,
        termChoixId: null,
      },
    },
  ]);

  const [showVariations, setShowVariations] = useState(true);

  const [selectedAttributeId, setSelectedAttributeId] = useState<string | null>(
    null
  );

  const { termsData } = useGetTermsByAttributeId(selectedAttributeId || "");
  const { AttributesData } = useGetAttributs();

  const [allAttributeTerms, setAllAttributeTerms] = useState<{
    [key: string]: any[];
  }>({});

  const [attributeTermsMap, setAttributeTermsMap] = useState<{
    [attributeId: number]: any[];
  }>({});

  const [preselectedInitialized, setPreselectedInitialized] = useState(false);

  const [showShippingInfo, setShowShippingInfo] = useState(false);

  const listboxRef = useRef<HTMLUListElement | null>(null);

  const handleSelectAllAttributeValues = (index: number) => {
    const attr = attributes[index];
    const allValues =
      allAttributeTerms[attr.attribute]?.map((term) => term.nom || term.name) ||
      (attributeTermsMap[attr.id!]
        ? attributeTermsMap[attr.id!].map((term) => term.nom || term.name)
        : []);
    handleAttributeValuesChange(index, allValues, attributeTermsMap);
  };

  useEffect(() => {
    if (currentProduct) {
      const initialAttributes = extractAttributesFromVariations(
        currentProduct.variations,
        AttributesData,
        attributeTermsMap
      );
      setAttributes(initialAttributes);
      setProductType(currentProduct.productType);
      setShowVariations(currentProduct.productType === ProductType.VARIABLE);
      setPublishOnWeb(currentProduct.publishWeb || false);
      setPublishOnLive(currentProduct.publishLive || false);
      setVariations(currentProduct.variations || []);
    }
  }, [currentProduct, AttributesData, attributeTermsMap]);

  useEffect(() => {
    if (termsData && selectedAttributeId) {
      const selectedAttribute = AttributesData.find(
        (a) => a.id === parseInt(selectedAttributeId)
      );

      if (selectedAttribute) {
        setAllAttributeTerms((prev) => ({
          ...prev,
          [selectedAttribute.nom]: termsData,
        }));
      }
    }
  }, [termsData, selectedAttributeId]);

  useEffect(() => {
    if (AttributesData && attributes && !preselectedInitialized) {
      attributes.forEach((attr, index) => {
        if (attr.attribute) {
          const selectedAttribute = AttributesData.find(
            (attribute) => attribute.nom === attr.attribute
          );
          if (selectedAttribute) {
            setSelectedAttributeId(selectedAttribute.id!.toString());
            handleAttributeChange(index, selectedAttribute.nom);
          }
        }
      });
      setPreselectedInitialized(true);
    }
  }, [attributes, AttributesData, preselectedInitialized]);

  useEffect(() => {
    if (currentProduct) {
      setVariations(currentProduct.variations || []);
    }
  }, [currentProduct]);

  useEffect(() => {
    if (currentProduct && currentProduct.variations) {
      const attributeIds = new Set<number>();
      currentProduct.variations.forEach((variation) => {
        variation.variationAttributes.forEach((attr) => {
          attributeIds.add(attr.attributeId);
        });
      });
      const attributeIdsToFetch = Array.from(attributeIds).filter(
        (attributeId) => !attributeTermsMap[attributeId]
      );

      if (attributeIdsToFetch.length > 0) {
        fetchTermsForAttributes(attributeIdsToFetch).then((termsMap) => {
          setAttributeTermsMap((prev) => ({
            ...prev,
            ...termsMap,
          }));
        });
      }
    }
  }, [currentProduct]);

  const fetchTermsForAttributes = async (attributeIds: number[]) => {
    const promises = attributeIds.map((attributeId) =>
      fetcher(`/api/stock/attributs/${attributeId}/terms`).then((terms) => ({
        attributeId,
        terms,
      }))
    );

    const results = await Promise.all(promises);
    const termsMap: { [attributeId: number]: any[] } = {};
    results.forEach(({ attributeId, terms }) => {
      termsMap[attributeId] = terms;
    });
    return termsMap;
  };
  const extractAttributesFromVariations = (
    variations: IVariation[],
    AttributesData: any[],
    attributeTermsMap: { [attributeId: number]: any[] }
  ): Attribute[] => {
    const attributesMap: { [key: number]: Attribute } = {};
    const attributePosition = currentProduct?.idPositionMap;
  
    variations.forEach((variation) => {
      variation.variationAttributes.forEach((attr) => {
        const existingAttribute = attributesMap[attr.attributeId];
        const termsData = attributeTermsMap[attr.attributeId] || [];
  
        const termName =
          findTermNameById(attr.termLabelId, termsData) ||
          findTermNameById(attr.termColorId, termsData) ||
          findTermNameById(attr.termImageId, termsData) ||
          findTermNameById(attr.termChoixId, termsData);
  
        if (!existingAttribute) {
          const attributeData = AttributesData.find(
            (attribute) => attribute.id === attr.attributeId
          );
  
          attributesMap[attr.attributeId] = {
            id: attr.attributeId,
            attribute: attributeData ? attributeData.nom : "",
            type: attributeData ? attributeData.type : null,
            values: [termName],
            termIds: {
              termLabelId: attr.termLabelId,
              termImageId: attr.termImageId,
              termColorId: attr.termColorId,
              termChoixId: attr.termChoixId,
            },
          };
        } else {
          if (!existingAttribute.values.includes(termName)) {
            existingAttribute.values.push(termName);
          }
        }
      });
    });
  
    const attributesArray = Object.values(attributesMap);
    if (attributePosition) {
      attributesArray.sort((a, b) => {
        const positionA = attributePosition[a.id as number] ?? Number.MAX_SAFE_INTEGER;
        const positionB = attributePosition[b.id as number] ?? Number.MAX_SAFE_INTEGER;
        return positionA - positionB;
      });
    }
  
    return attributesArray;
  };
  

  const findTermNameById = (
    termId: number | null,
    termsData: any[]
  ): string => {
    if (!termId) {
      return "";
    }

    const term = termsData.find((term) => term.id === termId);
    if (term) {
      return term.nom || term.name;
    }

    return "";
  };
  useEffect(() => {
    if (
      currentProduct &&
      currentProduct.variations &&
      AttributesData &&
      Object.keys(attributeTermsMap).length > 0
    ) {
      const extractedAttributes = extractAttributesFromVariations(
        currentProduct.variations,
        AttributesData,
        attributeTermsMap
      );
      setAttributes(extractedAttributes);
    }
  }, [currentProduct, AttributesData, attributeTermsMap]);

  useEffect(() => {
    if (currentProduct && currentProduct.stockWebConfig) {
      setBackorderOptionWeb(currentProduct.stockWebConfig.isPreOrderAuthorized);
      setAllowPreorderWeb(currentProduct.stockWebConfig.authorizedConfig || false);
    }
  }, [currentProduct]);

  useEffect(() => {
    if (currentProduct && currentProduct.stockLiveConfig) {
      setBackorderOptionLive(currentProduct.stockLiveConfig.isPreOrderAuthorized);
      setAllowPreorderLive(currentProduct.stockLiveConfig.authorizedConfig || false);
    }
  }, [currentProduct]);
  const handleSelectProductType = (
    e: React.ChangeEvent<{ value: unknown }>
  ) => {
    const selectedType = e.target.value as ProductType;
    setProductType(selectedType);
    setShowVariations(selectedType === ProductType.VARIABLE);
  };

  const handleBackorderChangeWeb = (
    event: React.ChangeEvent<{ value: unknown }>
  ) => {
    const selectedValue = event.target.value as AuthorizePreOrder;
    setBackorderOptionWeb(selectedValue);
    setAllowPreorderWeb(selectedValue !== AuthorizePreOrder.DO_NOT_AUTHORIZE);
  };

  const handleBackorderChangeLive = (
    event: React.ChangeEvent<{ value: unknown }>
  ) => {
    const selectedValue = event.target.value as AuthorizePreOrder;
    setBackorderOptionLive(selectedValue);
    setAllowPreorderLive(selectedValue !== AuthorizePreOrder.DO_NOT_AUTHORIZE);
  };

  const handleAddAttribute = () => {
    setAttributes([
      ...attributes,
      {
        attribute: "",
        values: [],
        type: "",
        termIds: {
          termLabelId: null,
          termImageId: null,
          termColorId: null,
          termChoixId: null,
        },
      },
    ]);
  };

  const handleRemoveAttribute = (index: number) => {
    setAttributes(attributes.filter((_, i) => i !== index));
  };

  const handleAttributeChange = (index: number, newValue: string | null) => {
    const newAttributes = [...attributes];

    // Check if the attribute is already selected to avoid unnecessary updates
    if (newAttributes[index].attribute === newValue) return;

    newAttributes[index].attribute = newValue || "";
    newAttributes[index].values = [];

    if (newValue) {
      const selectedAttribute = AttributesData.find(
        (attr) => attr.nom === newValue
      );

      if (selectedAttribute) {
        setSelectedAttributeId(selectedAttribute.id!.toString());
        newAttributes[index].id = selectedAttribute.id;
        newAttributes[index].type = selectedAttribute.type || "";
      } else {
        setSelectedAttributeId(null);
        newAttributes[index].id = undefined;
      }
    } else {
      setSelectedAttributeId(null);
    }

    setAttributes(newAttributes);
  };
  const handleAttributeValuesChange = (
    index: number,
    newValues: string[],
    attributeTermsMap: { [attributeId: number]: any[] }
  ) => {
    const newAttributes = [...attributes];
    newAttributes[index].values = newValues;

    newAttributes[index].termIds = {
      termLabelId: null,
      termImageId: null,
      termColorId: null,
      termChoixId: null,
    };

    newValues.forEach((value) => {
      // Find the term based on the value selected by the user
      const term =
        allAttributeTerms[newAttributes[index].attribute]?.find(
          (term) => term.nom === value
        ) ||
        (newAttributes[index].id &&
          attributeTermsMap[newAttributes[index].id!]?.find(
            (term) => term.nom === value || term.name === value
          ));

      if (term) {
        switch (newAttributes[index].type) {
          case "LABEL":
            newAttributes[index].termIds.termLabelId = term.id;
            break;
          case "IMAGE":
            newAttributes[index].termIds.termImageId = term.id;
            break;
          case "COLOR":
            newAttributes[index].termIds.termColorId = term.id;
            break;
          case "CHOICE":
            newAttributes[index].termIds.termChoixId = term.id;
            break;
          default:
            break;
        }
      }
    });

    setAttributes(newAttributes);
  };

  return (
    <>
      {mdUp && (
        <Grid item md={4}>
          <Typography variant="h6" sx={{ mb: 0.5 }}>
            Propriétés
          </Typography>
          <Typography variant="body2" sx={{ color: "text.secondary" }}>
            Fonctions et attributs supplémentaires...
          </Typography>
        </Grid>
      )}

      <Grid item xs={12} md={8} >
        <Card>
          {!mdUp && <CardHeader title="Properties" />}
          <Stack spacing={3} sx={{ p: 3 }}>
            <Controller
              name="productType"
              control={control}
              defaultValue={ProductType.SIMPLE}
              render={({ field }) => (
                <RHFSelect
                  {...field}
                  native
                  name="productType"
                  label="Type de produit"
                  InputLabelProps={{ shrink: true }}
                  onChange={(e) => {
                    field.onChange(e.target.value);
                    handleSelectProductType(e);
                  }}
                >
                  {PRODUCT_TYPE_OPTIONS.map((option) => (
                    <option key={option.value} value={option.value}>
                      {option.label}
                    </option>
                  ))}
                </RHFSelect>
              )}
            />

<Controller
  name="ugs"
  control={control}
  render={({ field, fieldState: { error } }) => (
    <Box position="relative">
      <RHFTextField
        {...field}
        label="UGS produit"
        error={!!error}
        helperText={error ? error.message : ''}
        disabled={Boolean(currentProduct)}

      />
      {isCheckingUgs && (
        <CircularProgress
          size={24}
          sx={{
            position: "absolute",
            top: "50%",
            right: "10px",
            marginTop: "-12px",
          }}
        />
      )}
    </Box>
  )}
/>

            <Box
              columnGap={2}
              rowGap={3}
              display="grid"
              gridTemplateColumns={{
                xs: "repeat(1, 1fr)",
                md: "repeat(2, 1fr)",
              }}
            >
              <Controller
                name="regularPrice"
                control={control}
                render={({ field }) => (
                  <RHFTextField
                    {...field}
                    label="Tarif régulier"
                    placeholder="0.00"
                   
                    InputLabelProps={{ shrink: true }}
                    onChange={(e) => {
                      const newValue = parseFloat(e.target.value);
                      if (!isNaN(newValue)) {
                        field.onChange(newValue);
                      } else {
                        field.onChange('');
                      }
                    }}
                  />
                )}
              />
              <Controller
                name="promoPrice"
                control={control}
                render={({ field }) => (
                  <RHFTextField
                    {...field}
                    label="Tarif promo"
                    placeholder="0.00"
                    
                    InputLabelProps={{ shrink: true }}
                  />
                )}
              />
            </Box>

            <Controller
              name="stockAlert"
              control={control}
              render={({ field }) => (
                <RHFTextField
                  {...field}
                  label="Seuil de stock faible"
                
                  InputLabelProps={{ shrink: true }}
                  placeholder="0"
                />
              )}
            />

            <Controller
              name="publishWeb"
              control={control}
              defaultValue={false}
              render={({ field }) => (
                <FormControlLabel
                  control={
                    <Switch
                      {...field}
                      checked={field.value}
                      onChange={(e) => {
                        field.onChange(e.target.checked);
                        setPublishOnWeb(e.target.checked);
                      }}
                    />
                  }
                  label="Publier sur le web"
                />
              )}
            />
            {publishOnWeb && (
              <>
                <Controller
                  name="stockWebConfig.quantity"
                  control={control}
                  render={({ field }) => (
                    <RHFTextField
                      {...field}
                      label="Quantité Web"
                     
                      InputLabelProps={{ shrink: true }}
                      placeholder="0"
                    />
                  )}
                />
                <Controller
                  name="stockWebConfig.isPreOrderAuthorized"
                  control={control}
                  defaultValue={AuthorizePreOrder.DO_NOT_AUTHORIZE}
                  render={({ field }) => (
                    <RHFSelect
                      {...field}
                      native
                      label="Autoriser les commandes en réapprovisionnement ?"
                      InputLabelProps={{ shrink: true }}
                      onChange={(e) => {
                        field.onChange(e.target.value);
                        handleBackorderChangeWeb(e);
                      }}
                    >
                      {ALLOW_BACKORDER_OPTIONS.map((option) => (
                        <option key={option.value} value={option.value}>
                          {option.label}
                        </option>
                      ))}
                    </RHFSelect>
                  )}
                />
                {(backorderOptionWeb === AuthorizePreOrder.AUTHORIZE ||
                  backorderOptionWeb ===
                  AuthorizePreOrder.AUTHORIZE_WITH_CUSTOMER_NOTIFICATION) && (
                    <>
                      <Controller
                        name="stockWebConfig.authorizedConfig"
                        control={control}
                        defaultValue={true}
                        render={({ field }) => (
                          <FormControlLabel
                            control={
                              <Switch
                                {...field}
                                checked={field.value}
                                onChange={(e) => {
                                  field.onChange(e.target.checked);
                                  setAllowPreorderWeb(e.target.checked);
                                }}
                              />
                            }
                            label="Définir la limite de précommande"
                          />
                        )}
                      />
                      {allowPreorderWeb && (
                        <Controller
                          name="stockWebConfig.thresholdValue"
                          control={control}
                          defaultValue={0}
                          render={({ field }) => (
                            <RHFTextField
                              {...field}
                              label="Seuil de précommande en Web"
                              
                              InputLabelProps={{ shrink: true }}
                              placeholder="0"
                            />
                          )}
                        />
                      )}
                    </>
                  )}
              </>
            )}

            <Divider sx={{ borderStyle: "dashed" }} />

            <Controller
              name="publishLive"
              control={control}
              defaultValue={false}
              render={({ field }) => (
                <FormControlLabel
                  control={
                    <Switch
                      {...field}
                      checked={field.value}
                      onChange={(e) => {
                        field.onChange(e.target.checked);
                        setPublishOnLive(e.target.checked);
                      }}
                    />
                  }
                  label="Publier sur le Live"
                />
              )}
            />
            {publishOnLive && (
              <>
                <Controller
                  name="stockLiveConfig.quantity"
                  control={control}
                  render={({ field }) => (
                    <RHFTextField
                      {...field}
                      label="Quantité Live"
                     
                      InputLabelProps={{ shrink: true }}
                      placeholder="0"
                    />
                  )}
                />
                <Controller
                  name="stockLiveConfig.isPreOrderAuthorized"
                  control={control}
                  defaultValue={AuthorizePreOrder.DO_NOT_AUTHORIZE}
                  render={({ field }) => (
                    <RHFSelect
                      {...field}
                      native
                      label="Autoriser les commandes en réapprovisionnement ?"
                      InputLabelProps={{ shrink: true }}
                      onChange={(e) => {
                        field.onChange(e.target.value);
                        handleBackorderChangeLive(e);
                      }}
                    >
                      {ALLOW_BACKORDER_OPTIONS.map((option) => (
                        <option key={option.value} value={option.value}>
                          {option.label}
                        </option>
                      ))}
                    </RHFSelect>
                  )}
                />
                {(backorderOptionLive === AuthorizePreOrder.AUTHORIZE ||
                  backorderOptionLive ===
                  AuthorizePreOrder.AUTHORIZE_WITH_CUSTOMER_NOTIFICATION) && (
                    <>
                      <Controller
                        name="stockLiveConfig.authorizedConfig"
                        control={control}
                        defaultValue={true}
                        render={({ field }) => (
                          <FormControlLabel
                            control={
                              <Switch
                                {...field}
                                checked={field.value}
                                onChange={(e) => {
                                  field.onChange(e.target.checked);
                                  setAllowPreorderLive(e.target.checked);
                                }}
                              />
                            }
                            label="Définir la limite de précommande"
                          />
                        )}
                      />
                      {allowPreorderLive && (
                        <Controller
                          name="stockLiveConfig.thresholdValue"
                          control={control}
                          defaultValue={0}
                          render={({ field }) => (
                            <RHFTextField
                              {...field}
                              label="Seuil de précommande Live"
                              
                              InputLabelProps={{ shrink: true }}
                              placeholder="0"
                            />
                          )}
                        />
                      )}
                    </>
                  )}
                <Controller
                  name="liveProductConfigs.authorizeUpdateQuantity"
                  control={control}
                  render={({ field }) => (
                    <FormControlLabel
                      control={
                        <Switch
                          {...field}
                          checked={field.value}
                          onChange={(e) => field.onChange(e.target.checked)}
                        />
                      }
                      label="Autoriser les Modifications"
                    />
                  )}
                />
                <Controller
                  name="liveProductConfigs.feedStockFromWebStock"
                  control={control}
                  render={({ field }) => (
                    <FormControlLabel
                      control={
                        <Switch
                          {...field}
                          checked={field.value}
                          onChange={(e) => field.onChange(e.target.checked)}
                        />
                      }
                      label="Alimenter le stock depuis le stock web"
                    />
                  )}
                />
              </>
            )}

            <Divider sx={{ borderStyle: "dashed" }} />
            <FormControlLabel
              control={
                <Switch
                  checked={showShippingInfo}
                  onChange={(e) => setShowShippingInfo(e.target.checked)}
                />
              }
              label="Afficher les informations sur l'expédition"
            />

            {showShippingInfo && (
              <>
                <Divider sx={{ borderStyle: "dashed" }} />
                <Typography variant="body2">
                  Informations sur l'expédition
                </Typography>
                <ProductExpedition />
              </>
            )}
          </Stack>
        </Card>
      </Grid>
      {productType === ProductType.VARIABLE && (
        <>
          {mdUp && (
            <Grid item md={4}>
              <Typography variant="h6" >
                Configurations des Variations
              </Typography>
              <Typography variant="body2" sx={{ color: "text.secondary" }}>
                variations et attributs ...
              </Typography>
            </Grid>
          )}

          <Grid item xs={12} md={8} >
            <Card>
              {!mdUp && <CardHeader title="variation" />}
              <Stack spacing={3} sx={{ p: 3 }}>
                <Typography variant="body2">Ajout des Attributs</Typography>
                <Stack spacing={2}>
                  <Stack spacing={2}>
                    {attributes.map((attr, index) => (
                      <Box key={index} sx={{ mb: 2 }}>
                        <Box display="flex" alignItems="center" gap={2}>
                          <Autocomplete
                            options={AttributesData.map(
                              (attribute) => attribute.nom
                            )}
                            value={attr.attribute}
                            onChange={(_, newValue) => {
                              handleAttributeChange(index, newValue);
                            }}
                            filterOptions={(options, { inputValue }) => {
                              const lowerInput = inputValue.toLowerCase();
                              const exactMatches: string[] = [];
                              const startsWith: string[] = [];
                              const contains: string[] = [];

                              options.forEach((val) => {
                                const lowerVal = val.toLowerCase();
                                if (lowerVal === lowerInput) {
                                  exactMatches.push(val);
                                } else if (lowerVal.startsWith(lowerInput)) {
                                  startsWith.push(val);
                                } else if (lowerVal.includes(lowerInput)) {
                                  contains.push(val);
                                }
                              });

                              return [...exactMatches, ...startsWith, ...contains];
                            }}
                            renderInput={(params) => (
                              <TextField
                                {...params}
                                label="Sélectionner un attribut"
                                fullWidth
                              />
                            )}
                            sx={{ flexGrow: 1 }}
                          />
                          <IconButton
                            onClick={() => handleRemoveAttribute(index)}
                            color="error"
                            aria-label="delete"
                          >
                            <DeleteIcon />
                          </IconButton>
                        </Box>
                        <Box display="flex" alignItems="center" gap={2}>

                          <Autocomplete
                            multiple
                            options={
                              allAttributeTerms[attr.attribute]?.map(
                                (term) => term.nom || term.name
                              ) ||
                              (attributeTermsMap[attr.id!]
                                ? attributeTermsMap[attr.id!].map(
                                  (term) => term.nom || term.name
                                )
                                : [])
                            }
                            filterOptions={(options, { inputValue }) => {
                              const lowerInput = inputValue.toLowerCase();
                              const exactMatches: string[] = [];
                              const startsWith: string[] = [];
                              const contains: string[] = [];

                              options.forEach((val) => {
                                const lowerVal = val.toLowerCase();
                                if (lowerVal === lowerInput) {
                                  exactMatches.push(val);
                                } else if (lowerVal.startsWith(lowerInput)) {
                                  startsWith.push(val);
                                } else if (lowerVal.includes(lowerInput)) {
                                  contains.push(val);
                                }
                              });

                              setTimeout(() => {
                                if (listboxRef.current) {
                                  listboxRef.current.scrollTop = 0;
                                }
                              }, 0);
                              
                              return [...exactMatches, ...startsWith, ...contains];
                            }}
                            value={attr.values}
                            onChange={(_, newValues) => {
                              handleAttributeValuesChange(
                                index,
                                newValues,
                                attributeTermsMap
                              );
                              if (listboxRef.current) {
                                listboxRef.current.scrollTop = 0;
                              }
                            }}
                            onInputChange={() => {
                              setTimeout(() => {
                                if (listboxRef.current) {
                                  listboxRef.current.scrollTop = 0;
                                }
                              }, 0);
                            }}
                            renderInput={(params) => (
                              <TextField
                                {...params}
                                label="Valeurs d'attribut"
                                fullWidth
                              />
                            )}
                            autoHighlight
                            ListboxProps={{
                              ref: (node) => {
                                listboxRef.current = node as HTMLUListElement;
                              }
                            }}
                            sx={{ mt: 2, flexGrow: 1 }}
                          />
                          <Tooltip title="Tous sélectionner">
                            <IconButton
                              size="small"
                              onClick={() => handleSelectAllAttributeValues(index)}
                              color="primary"
                              aria-label="select all"
                            >
                              <LibraryAddCheckSharpIcon />
                            </IconButton>
                          </Tooltip>
                        </Box>
                      </Box>
                    ))}
                  </Stack>

                  <Button
                    variant="outlined"
                    startIcon={<AddIcon />}
                    onClick={handleAddAttribute}
                    disabled={Boolean(currentProduct)}

                  >
                    Ajouter un attribut
                  </Button>
                  {showVariations &&
                    (attributes.length > 0 ? (
                      <Variations
                        attributes={attributes}
                        control={control}
                        values={values}
                        termsData={termsData}
                        allAttributeTerms={allAttributeTerms}
                        attributeTermsMap={attributeTermsMap}
                        setVariations={setVariations}
                        currentProduct={currentProduct}
                        mainImagePath={
                          typeof mainImagePath === "string" ? mainImagePath : ""
                        }
                      />
                    ) : (
                      <Typography variant="body2" color="textSecondary">
                        No attributes available. Please add an attribute.
                      </Typography>
                    ))}
                </Stack>

              </Stack>
            </Card>
          </Grid>
        </>
      )}
    </>
  );
}
