import React, { useEffect, useState } from "react";
import {
  Stack,
  Typography,
  Button,
  IconButton,
  Box,
  Divider,
  Card,
  Grid,
  Select,
  MenuItem,
  FormControl,
  InputLabel,
  Switch,
  FormControlLabel,
  InputAdornment,
  TextField,
  Dialog,
  DialogTitle,
  DialogContent,
  DialogActions,
  DialogContentText,
} from "@mui/material";
import DeleteIcon from "@mui/icons-material/Delete";
import EditIcon from "@mui/icons-material/Edit";
import { RHFTextField, RHFSelect } from "@/shared/components/hook-form";
import { Controller } from "react-hook-form";
import {
  AuthorizePreOrder,
  IProductItem,
  IVariation,
} from "@/shared/types/product";
import { useGetShippingClasses } from "@/shared/api/shipping";
import { IMedia } from "@/shared/types/media";
import AddMediaButton from "../../media/Add-media-button";

interface Attribute {
  id?: number;
  attribute: string;
  type: string | null;
  values: string[];
  termIds: {
    termLabelId: number | null;
    termImageId: number | null;
    termColorId: number | null;
    termChoixId: number | null;
  };
}

interface VariationsProps {
  attributes: Attribute[];
  control: any;
  values: any;
  termsData: any[];
  setVariations: React.Dispatch<React.SetStateAction<IVariation[]>>;
  allAttributeTerms: { [key: string]: any[] };
  attributeTermsMap: { [attributeId: number]: any[] };
  currentProduct?: IProductItem;
  mainImagePath: string;
}

type Combination = string[];

const generateCombinations = (attributes: Attribute[]): Combination[] => {
  if (attributes.length === 0) return [[]];

  const [first, ...rest] = attributes;
  const restCombinations = generateCombinations(rest);

  return first.values.flatMap((value) =>
    restCombinations.map((combination) => [value, ...combination])
  );
};

const Variations: React.FC<VariationsProps> = ({
  attributes,
  control,
  values,
  termsData,
  setVariations,
  allAttributeTerms,
  currentProduct,
  attributeTermsMap,
  mainImagePath,
}) => {
  const [localVariations, setLocalVariations] = useState<IVariation[]>([]);
  const [editingIndex, setEditingIndex] = useState<number | null>(null);
  const [allowPreorderWeb, setAllowPreorderWeb] = useState(false);
  const [backorderOptionWeb, setBackorderOptionWeb] = useState("");
  const [allowPreorderLive, setAllowPreorderLive] = useState(false);
  const [backorderOptionLive, setBackorderOptionLive] = useState("");
  const { shippingClassData } = useGetShippingClasses();
  const [showWebFields, setShowWebFields] = useState(false);
  const [showLiveFields, setShowLiveFields] = useState(false);
  const [path, setPath] = useState<number | string>();
  const [returnedMainImage, setReturnedMainImage] = useState<
    IMedia[] | undefined
  >();
  const [showShippingInfo, setShowShippingInfo] = useState(false);

  // État pour la recherche par UGS :
  const [searchUGS, setSearchUGS] = useState("");

  // Gère l'état du dialog :
  const [openConfirmation, setOpenConfirmation] = useState(false);
  // Stocke l'index de la variation à supprimer :
  const [variationIndexToRemove, setVariationIndexToRemove] = useState<number | null>(null);

  useEffect(() => {
    if (returnedMainImage && returnedMainImage.length > 0) {
      const mediaId = returnedMainImage[0];
      const firstMediaPath = mediaId.url;
      setPath(firstMediaPath);
    }
  }, [returnedMainImage])

  useEffect(() => {
    if (currentProduct && currentProduct.variations) {
      const formattedVariations = currentProduct.variations.map((variation) => {
        const formattedAttributes = variation.variationAttributes.map(
          (attr) => {
            const attribute = attributes.find(
              (attribute) => attribute.id === attr.attributeId
            );
            const term = attribute?.attribute
              ? allAttributeTerms[attribute.attribute]?.find(
                (t) =>
                  t.id ===
                  (attr.termLabelId ||
                    attr.termColorId ||
                    attr.termImageId ||
                    attr.termChoixId)
              )
              : undefined;

            return {
              ...attr,
              value: term?.nom || term?.name || "",
            };
          }
        );

        return {
          ...variation,
          variationAttributes: formattedAttributes,
        };
      });


      setLocalVariations(formattedVariations);
      setVariations(formattedVariations);
    }

  }, [
    currentProduct,
    attributes,
    allAttributeTerms,
    mainImagePath,

  ]);

  const compareStringsCaseInsensitive = (str1: string, str2: string): boolean =>
    str1.localeCompare(str2, undefined, { sensitivity: 'base' }) === 0;

  const handleGenerateVariations = () => {
    // 1. Récupérer toutes les combinaisons de valeurs possibles à partir des attributs
    const termCombinations = generateCombinations(attributes);
  
    // 2. Parcourir chaque combinaison pour construire la variation correspondante
    const updatedVariations = termCombinations.map((terms) => {
      const combinedSkuParts: string[] = [values.ugs];
      const combinedNameParts: string[] = [values.name];
  
      // 3. Générer les attributs pour cette variation
      const generatedAttributes = attributes.map((attr, i) => {
        const termValue = terms[i]?.trim();
  
        // Recherche du "term" correspondant (ex. pour les couleurs, labels, images, etc.)
        const matchingTerm =
          allAttributeTerms[attr.attribute]?.find((term) =>
            compareStringsCaseInsensitive(term.nom || term.name, termValue)
          ) ||
          (attr.id &&
            attributeTermsMap[attr.id]?.find((term) =>
              compareStringsCaseInsensitive(term.nom || term.name, termValue)
            ));
  
        const termSlug = matchingTerm?.slug || termValue;
  
        combinedSkuParts.push(termSlug);
        combinedNameParts.push(termValue);
  
        return {
          attributeId: attr.id ?? 0,
          type: attr.type,
          termColorId: attr.type === "COLOR" ? matchingTerm?.id : null,
          termLabelId: attr.type === "LABEL" ? matchingTerm?.id : null,
          termImageId: attr.type === "IMAGE" ? matchingTerm?.id : null,
          termChoixId: attr.type === "CHOICE" ? matchingTerm?.id : null,
          value: termValue,
        };
      });
  
      // 4. Construire l'UGS (SKU) et le name en se basant sur la combinaison
      const combinedSku = combinedSkuParts.join("-");
      const combinedName = combinedNameParts.join("-");
  
      
      const existingVariationInCurrentProduct = currentProduct?.variations?.find(
        (v) => v.ugs === combinedSku
      );
  
      if (existingVariationInCurrentProduct) {
        return {
          // On récupère l'id de la variation déjà en base
          id: existingVariationInCurrentProduct.id,
          isNew: false,
          ugs: combinedSku,
          name: combinedName,
          variationAttributes: generatedAttributes,
          image: values.mainImageID,
          mainImagePath: mainImagePath,
          stockWebConfig: values.stockWebConfig,
          stockLiveConfig: values.stockLiveConfig,
          liveProductConfigs: values.liveProductConfigs,
          regularPrice: values.regularPrice,
          promoPrice: values.promoPrice,
          purchasePrice: values.purchasePrice,
          stockAlert: values.stockAlert,
          publishWeb: values.publishWeb,
          publishLive: values.publishLive,
          weight: values.weight,
          lengthP: values.lengthP,
          widthP: values.widthP,
          heightP: values.heightP,
          shippingClassId: values.shippingClassId,
          secondPurchasePrice: values.secondPurchasePrice,
        };
      } else {
        // Nouvelle variation => id = undefined
        return {
          id: undefined,
          isNew: true,
          ugs: combinedSku,
          name: combinedName,
          variationAttributes: generatedAttributes,
          image: values.mainImageID,
          mainImagePath: mainImagePath,
          stockWebConfig: values.stockWebConfig,
          stockLiveConfig: values.stockLiveConfig,
          liveProductConfigs: values.liveProductConfigs,
          regularPrice: values.regularPrice,
          promoPrice: values.promoPrice,
          purchasePrice: values.purchasePrice,
          stockAlert: values.stockAlert,
          publishWeb: values.publishWeb,
          publishLive: values.publishLive,
          weight: values.weight,
          lengthP: values.lengthP,
          widthP: values.widthP,
          heightP: values.heightP,
          shippingClassId: values.shippingClassId,
          secondPurchasePrice: values.secondPurchasePrice,
        };
      }
    });
  
    // 6. (Optionnel) trier le tableau des variations
    updatedVariations.sort((a, b) =>
      a.ugs.localeCompare(b.ugs, undefined, { numeric: true })
    );
  
    // 7. Mettre à jour le state local et l'état parent (via setVariations)
    setLocalVariations(updatedVariations);
    setVariations(updatedVariations);
  };
  
  


  const handleRemoveVariation = (variationIndex: number) => {
    setLocalVariations((prev) => prev.filter((_, i) => i !== variationIndex));
    setVariations((prev) => prev.filter((_, i) => i !== variationIndex));
  };


  const handleDeleteAllVariations = () => {
    setLocalVariations([]);
    setVariations([]);
  };

  const handleEditClick = (index: number) => {
    setEditingIndex((prev) => (prev === index ? null : index));
  };



  const handleBackorderChangeWeb = (
    event: React.ChangeEvent<{ value: unknown }>
  ) => {
    setBackorderOptionWeb(event.target.value as string);
  };

  const handleBackorderChangeLive = (
    event: React.ChangeEvent<{ value: unknown }>
  ) => {
    setBackorderOptionLive(event.target.value as string);
  };

  const handleAttributeChange = (
    variationIndex: number,
    attributeIndex: number,
    value: string
  ) => {
    const newVariations = [...localVariations];
    newVariations[variationIndex].variationAttributes[attributeIndex].value =
      value;
    setLocalVariations(newVariations);
    setVariations(newVariations);
  };

  const handlePublishWebChange = (checked: boolean) => {
    setShowWebFields(checked);
    // Update variations to reflect the change
    const newVariations = localVariations.map(variation => ({
      ...variation,
      publishWeb: checked
    }));
    setLocalVariations(newVariations);
    setVariations(newVariations);
  };

  const handlePublishLiveChange = (checked: boolean) => {
    setShowLiveFields(checked);
    // Update variations to reflect the change
    const newVariations = localVariations.map(variation => ({
      ...variation,
      publishLive: checked
    }));
    setLocalVariations(newVariations);
    setVariations(newVariations);
  };

  const ALLOW_BACKORDER_OPTIONS = [
    { value: AuthorizePreOrder.DO_NOT_AUTHORIZE, label: "Ne pas autoriser" },
    { value: AuthorizePreOrder.AUTHORIZE, label: "Autoriser" },
    {
      value: AuthorizePreOrder.AUTHORIZE_WITH_CUSTOMER_NOTIFICATION,
      label: "Autoriser, mais avec notification client",
    },
  ];

  const handleImageChange = (index: number, newImage: IMedia) => {
    const newVariations = [...localVariations];
    newVariations[index].mainImagePath = newImage.url;
    setLocalVariations(newVariations);
    setVariations(newVariations);
  };

  // On filtre les variations selon la valeur du champ de recherche :
  const filteredVariations = localVariations.filter((variation) =>
    variation.ugs.toLowerCase().includes(searchUGS.toLowerCase())
  );

  // Ouvrir le dialog de confirmation
  const handleRemoveClick = (index: number) => {
    // Only show confirmation dialog if the variation has an existing ID (not undefined)
    if (localVariations[index].id !== undefined) {
      setVariationIndexToRemove(index);
      setOpenConfirmation(true);
    } else {
      // Otherwise, remove immediately
      handleRemoveVariation(index);
    }
  };

  // Confirmer la suppression
  const confirmRemoveVariation = () => {
    if (variationIndexToRemove === null) return;
    // Suppression de la variation concernée
    setLocalVariations((prev) => prev.filter((_, i) => i !== variationIndexToRemove));
    setVariations((prev) => prev.filter((_, i) => i !== variationIndexToRemove));
    // Fermer la popin
    setOpenConfirmation(false);
    setVariationIndexToRemove(null);
  };

  // Annuler la suppression
  const cancelRemoveVariation = () => {
    setOpenConfirmation(false);
    setVariationIndexToRemove(null);
  };

  return (
    
    <div>
      <Divider sx={{ borderStyle: "dashed", mb: 2 }} />
      <Typography variant="subtitle2" sx={{ mb: 2 }}>
        Variations
      </Typography>

      {localVariations.length > 0 ? (
  <Box sx={{ mb: 2 }}>
    <TextField
      label="Rechercher par UGS"
      variant="outlined"
      value={searchUGS}
      onChange={(e) => setSearchUGS(e.target.value)}
      fullWidth
    />
  </Box>
) : (
  <Box>
  </Box>
)}


      <Stack direction="row" alignItems="center" spacing={3} sx={{ mb: 3 }}>
        <Button variant="outlined" onClick={handleGenerateVariations}>
          Générer des variations
        </Button>

        {localVariations.length > 0 && (
          <Button
            variant="outlined"
            color="error"
            onClick={handleDeleteAllVariations}
            disabled={Boolean(currentProduct)}
          >
            Supprimer les variations
          </Button>
        )}
      </Stack>
      {filteredVariations.map((variation, index) => (
        <Box key={variation.id ?? index} mb={2}>
          <Box display="flex" alignItems="center" gap={2}>
            <Typography variant="body2" sx={{ width: 80 }}>
              #{variation.ugs}
            </Typography>

            {variation.variationAttributes.map((attr, i) => (
              <FormControl key={i} sx={{ minWidth: 120 }}>
                {attributes[i] ? (
                  <>
                    <InputLabel>{attributes[i].attribute}</InputLabel>
                    <Select
                      value={attr.value || ""}
                      label={attributes[i].attribute}
                      onChange={(e) =>
                        handleAttributeChange(
                          index,
                          i,
                          e.target.value as string
                        )
                      }
                    >
                      {attributes[i].values.map((value, j) => (
                        <MenuItem key={j} value={value}>
                          {value}
                        </MenuItem>
                      ))}
                    </Select>
                  </>
                ) : (
                  <Typography color="error">
                    Attribute not found for index {i}
                  </Typography>
                )}
              </FormControl>
            ))}

            {/* Quantité Web */}
            <Controller
              name={`variations[${index}].stockWebConfig.quantity`}
              control={control}
              defaultValue={variation.stockWebConfig.quantity || 0}
              render={({ field }) => (
                <RHFTextField
                  {...field}
                  label="Web"
                  type="number"
                  InputLabelProps={{ shrink: true }}
                  placeholder="0"
                  sx={{ width: 80 }} 
                  onChange={(e) => {
                    const newValue = parseInt(e.target.value, 10);
                    field.onChange(newValue);
                    const newVariations = [...localVariations];
                    newVariations[index].stockWebConfig = {
                      ...newVariations[index].stockWebConfig,
                      quantity: newValue,
                    };
                    setLocalVariations(newVariations);
                    setVariations(newVariations);
                  }}
                />
              )}
            />

            {/* Quantité Live */}
            <Controller
              name={`variations[${index}].stockLiveConfig.quantity`}
              control={control}
              defaultValue={variation.stockLiveConfig.quantity || 0}
              render={({ field }) => (
                <RHFTextField
                  {...field}
                  label="Live"
                  type="number"
                  InputLabelProps={{ shrink: true }}
                  placeholder="0"
                  sx={{ width: 80 }}
                  onChange={(e) => {
                    const newValue = parseInt(e.target.value, 10);
                    field.onChange(newValue);
                    const newVariations = [...localVariations];
                    newVariations[index].stockLiveConfig = {
                      ...newVariations[index].stockLiveConfig,
                      quantity: newValue,
                    };
                    setLocalVariations(newVariations);
                    setVariations(newVariations);
                  }}
                />
              )}
            />

            {/* Prix régulier */}
            <Controller
              name={`variations[${index}].regularPrice`}
              control={control}
              defaultValue={variation.regularPrice || 0}
              render={({ field }) => (
                <RHFTextField
                  {...field}
                  label="Prix €"
                  type="number"
                  InputLabelProps={{ shrink: true }}
                  placeholder="0"
                  sx={{ width: 80 }}
                  onChange={(e) => {
                    const newValue = parseFloat(e.target.value);
                    field.onChange(newValue);
                    const newVariations = [...localVariations];
                    newVariations[index].regularPrice = newValue;
                    setLocalVariations(newVariations);
                    setVariations(newVariations);
                  }}
                />
              )}
            />

            <IconButton
              color="primary"
              onClick={() => handleEditClick(index)}
            >
              <EditIcon />
            </IconButton>
            <IconButton
              onClick={() => handleRemoveClick(index)}
              color="secondary"
            >
              <DeleteIcon />
            </IconButton>
          </Box>

          {editingIndex === index && (
            <Grid container item xs={12} sx={{ mt: 2 }}>
              <Card sx={{ width: "100%" }}>
                <Stack spacing={3} sx={{ p: 3 }}>
                  <Controller
                    name={`variations[${index}].ugs`}
                    control={control}
                    render={({ field }) => (
                      <RHFTextField
                        {...field}
                        value={variation.ugs}
                        label="UGS produit"
                      />
                    )}
                  />
                  <Box
                    columnGap={2}
                    rowGap={3}
                    display="grid"
                    gridTemplateColumns={{
                      xs: "repeat(1, 1fr)",
                      md: "repeat(2, 1fr)",
                    }}
                  >
                    <Controller
                      name={`variations[${index}].regularPrice`}
                      control={control}
                      defaultValue={variation.regularPrice}
                      render={({ field, fieldState: { error } }) => (
                        <RHFTextField
                          {...field}
                          label="Tarif régulier"
                          placeholder="0.00"
                          type="number"
                          InputLabelProps={{ shrink: true }}
                          error={!!error}
                          helperText={error ? error.message : ''}
                          onChange={(e) => {
                            const newValue = parseFloat(e.target.value);
                            field.onChange(newValue);
                            const newVariations = [...localVariations];
                            newVariations[index].regularPrice = newValue;
                            setLocalVariations(newVariations);
                            setVariations(newVariations);
                          }}
                        />
                      )}
                    />
                    <Controller
                      name={`variations[${index}].promoPrice`}
                      control={control}
                      defaultValue={variation.promoPrice}
                      render={({ field }) => (
                        <RHFTextField
                          {...field}
                          label="Tarif promo"
                          placeholder="0.00"
                          type="number"
                          InputLabelProps={{ shrink: true }}
                          onChange={(e) => {
                            const newValue = parseFloat(e.target.value);
                            field.onChange(newValue);
                            // Update localVariations and variations
                            const newVariations = [...localVariations];
                            newVariations[index].promoPrice = newValue;
                            setLocalVariations(newVariations);
                            setVariations(newVariations);
                          }}
                        />
                      )}
                    />
                  </Box>

                  <Controller
                    name={`variations[${index}].stockAlert`}
                    control={control}
                    defaultValue={variation.stockAlert}
                    render={({ field }) => (
                      <RHFTextField
                        {...field}
                        label="Seuil de stock faible"
                        type="number"
                        InputLabelProps={{ shrink: true }}
                        onChange={(e) => {
                          const newValue = parseInt(e.target.value, 10);
                          field.onChange(newValue);
                          // Update variations
                          const newVariations = [...localVariations];
                          newVariations[index].stockAlert = newValue;
                          setLocalVariations(newVariations);
                          setVariations(newVariations);
                        }}
                      />
                    )}
                  />

                  <FormControlLabel
                    control={
                      <Switch
                        checked={showShippingInfo}
                        onChange={(e) => setShowShippingInfo(e.target.checked)}
                      />
                    }
                    label="Afficher les informations sur l'expédition dans les variations"
                    sx={{ mb: 3 }}
                  />
                  {showShippingInfo && (
                    <>

                      <Controller
                        name={`variations[${index}].weight`}
                        control={control}
                        defaultValue={variation.weight}
                        render={({ field }) => (
                          <RHFTextField
                            {...field}
                            label="Poids (g)"
                            type="number"
                            InputLabelProps={{ shrink: true }}
                            InputProps={{
                              startAdornment: (
                                <InputAdornment position="start">g</InputAdornment>
                              ),
                            }}
                            onChange={(e) => {
                              const newValue = parseFloat(e.target.value);
                              field.onChange(newValue);
                              // Update variations
                              const newVariations = [...localVariations];
                              newVariations[index].weight = newValue;
                              setLocalVariations(newVariations);
                              setVariations(newVariations);
                            }}
                          />
                        )}
                      />
                      <Box
                        columnGap={2}
                        rowGap={3}
                        display="grid"
                        gridTemplateColumns={{
                          xs: "repeat(1, 1fr)",
                          md: "repeat(3, 1fr)",
                        }}
                      >
                        <Controller
                          name={`variations[${index}].lengthP`}
                          control={control}
                          defaultValue={variation.lengthP}
                          render={({ field }) => (
                            <RHFTextField
                              {...field}
                              label="Longueur"
                              type="number"
                              InputLabelProps={{ shrink: true }}
                              onChange={(e) => {
                                const newValue = parseFloat(e.target.value);
                                field.onChange(newValue);
                                // Update variations
                                const newVariations = [...localVariations];
                                newVariations[index].lengthP = newValue;
                                setLocalVariations(newVariations);
                                setVariations(newVariations);
                              }}
                            />
                          )}
                        />

                        <Controller
                          name={`variations[${index}].widthP`}
                          control={control}
                          defaultValue={variation.widthP}
                          render={({ field }) => (
                            <RHFTextField
                              {...field}
                              label="Largeur"
                              type="number"
                              InputLabelProps={{ shrink: true }}
                              onChange={(e) => {
                                const newValue = parseFloat(e.target.value);
                                field.onChange(newValue);
                                // Update variations
                                const newVariations = [...localVariations];
                                newVariations[index].widthP = newValue;
                                setLocalVariations(newVariations);
                                setVariations(newVariations);
                              }}
                            />
                          )}
                        />

                        <Controller
                          name={`variations[${index}].heightP`}
                          control={control}
                          defaultValue={variation.heightP}
                          render={({ field }) => (
                            <RHFTextField
                              {...field}
                              label="Hauteur"
                              type="number"
                              InputLabelProps={{ shrink: true }}
                              onChange={(e) => {
                                const newValue = parseFloat(e.target.value);
                                field.onChange(newValue);
                                // Update variations
                                const newVariations = [...localVariations];
                                newVariations[index].heightP = newValue;
                                setLocalVariations(newVariations);
                                setVariations(newVariations);
                              }}
                            />
                          )}
                        />
                      </Box>

                      <Controller
                        name={`variations[${index}].shippingClassId`}
                        control={control}
                        defaultValue={variation.shippingClassId}
                        render={({ field }) => (
                          <RHFSelect
                            {...field}
                            label="Classe d'expédition"
                            variant="outlined"
                            onChange={(e) => {
                              const newValue = e.target.value as unknown as number;
                              field.onChange(newValue);
                              // Update variations
                              const newVariations = [...localVariations];
                              newVariations[index].shippingClassId = newValue;
                              setLocalVariations(newVariations);
                              setVariations(newVariations);
                            }}
                          >
                            {shippingClassData.map((shippingClass) => (
                              <MenuItem
                                key={shippingClass.id}
                                value={shippingClass.id}
                              >
                                {shippingClass.name}
                              </MenuItem>
                            ))}
                          </RHFSelect>
                        )}
                      />
                    </>
                  )}
                  <Divider sx={{ borderStyle: "dashed" }} />
                  <Controller
                    name={`variations[${index}].publishWeb`}
                    control={control}
                    defaultValue={variation.publishWeb}
                    render={({ field }) => (
                      <FormControlLabel
                        control={
                          <Switch
                            {...field}
                            checked={field.value || false}
                            onChange={(e) => {
                              const newValue = e.target.checked;
                              field.onChange(newValue);
                              handlePublishWebChange(newValue);
                            }}
                          />
                        }
                        label="Publier sur le web"
                      />
                    )}
                  />

                  {variation.publishWeb && (
                    <>
                      <Controller
                        name={`variations[${index}].stockWebConfig.quantity`}
                        control={control}
                        defaultValue={variation.stockWebConfig.quantity || 0}
                        render={({ field }) => (
                          <RHFTextField
                            {...field}
                            label="Quantité Web"
                            InputLabelProps={{ shrink: true }}
                            placeholder="0"
                            onChange={(e) => {
                              const newValue = parseInt(e.target.value, 10);
                              field.onChange(newValue);
                              const newVariations = [...localVariations];
                              newVariations[index].stockWebConfig = {
                                ...newVariations[index].stockWebConfig,
                                quantity: newValue,
                              };
                              setLocalVariations(newVariations);
                              setVariations(newVariations);
                            }}
                          />
                        )}
                      />
                      <Controller
                        name={`variations[${index}].stockWebConfig.isPreOrderAuthorized`}
                        control={control}
                        defaultValue={AuthorizePreOrder.DO_NOT_AUTHORIZE}
                        render={({ field }) => (
                          <RHFSelect
                            {...field}
                            native
                            value={
                              field.value ||
                              variation.stockWebConfig.isPreOrderAuthorized
                            }
                            label="Autoriser les commandes en réapprovisionnement ?"
                            InputLabelProps={{ shrink: true }}
                            onChange={(e) => {
                              field.onChange(e.target.value);
                              handleBackorderChangeWeb(e);
                            }}
                          >
                            {ALLOW_BACKORDER_OPTIONS.map((option) => (
                              <option key={option.value} value={option.value}>
                                {option.label}
                              </option>
                            ))}
                          </RHFSelect>
                        )}
                      />
                      {(backorderOptionWeb === AuthorizePreOrder.AUTHORIZE ||
                        backorderOptionWeb ===
                        AuthorizePreOrder.AUTHORIZE_WITH_CUSTOMER_NOTIFICATION) && (
                          <>
                            <Controller
                              name={`variations[${index}].stockWebConfig.authorizedConfig`}
                              control={control}
                              defaultValue={true}
                              render={({ field }) => (
                                <FormControlLabel
                                  control={
                                    <Switch
                                      {...field}
                                      checked={
                                        field.value ||
                                        variation.stockWebConfig.authorizedConfig
                                      }
                                      onChange={(e) => {
                                        field.onChange(e.target.checked);
                                        setAllowPreorderWeb(e.target.checked);
                                        // Update variations
                                        const newVariations = [
                                          ...localVariations,
                                        ];
                                        newVariations[index].stockWebConfig = {
                                          ...newVariations[index].stockWebConfig,
                                          authorizedConfig: e.target.checked,
                                        };
                                        setLocalVariations(newVariations);
                                        setVariations(newVariations);
                                      }}

                                    />
                                  }
                                  label="Définir la limite de précommande"
                                />
                              )}
                            />
                            {allowPreorderWeb && (
                              <Controller
                                name={`variations[${index}].stockWebConfig.thresholdValue`}
                                control={control}
                                defaultValue={
                                  variation.stockWebConfig.thresholdValue
                                }
                                render={({ field }) => (
                                  <RHFTextField
                                    {...field}
                                    label="Seuil de précommande en Web"
                                    InputLabelProps={{ shrink: true }}
                                    placeholder="0"
                                    onChange={(e) => {
                                      const newValue = parseInt(
                                        e.target.value,
                                        10
                                      );
                                      field.onChange(newValue);
                                      // Update variations
                                      const newVariations = [...localVariations];
                                      newVariations[index].stockWebConfig = {
                                        ...newVariations[index].stockWebConfig,
                                        thresholdValue: newValue,
                                      };
                                      setLocalVariations(newVariations);
                                      setVariations(newVariations);
                                    }}
                                  />
                                )}
                              />
                            )}
                          </>
                        )}
                    </>
                  )}

                  <Divider sx={{ borderStyle: "dashed" }} />

                  <Controller
                    name={`variations[${index}].publishLive`}
                    control={control}
                    defaultValue={variation.publishLive}
                    render={({ field }) => (
                      <FormControlLabel
                        control={
                          <Switch
                            {...field}
                            checked={field.value || false}
                            onChange={(e) => {
                              const newValue = e.target.checked;
                              field.onChange(newValue);
                              handlePublishLiveChange(newValue);
                            }}
                          />
                        }
                        label="Publier sur le Live"
                      />
                    )}
                  />
                  {variation.publishLive && (
                    <>
                      <Controller
                        name={`variations[${index}].stockLiveConfig.quantity`}
                        control={control}
                        defaultValue={variation.stockLiveConfig.quantity || 0}
                        render={({ field }) => (
                          <RHFTextField
                            {...field}
                            label="Quantité Live"
                            InputLabelProps={{ shrink: true }}
                            placeholder="0"
                            onChange={(e) => {
                              const newValue = parseInt(e.target.value, 10);
                              field.onChange(newValue);
                              // Update variations
                              const newVariations = [...localVariations];
                              newVariations[index].stockLiveConfig = {
                                ...newVariations[index].stockLiveConfig,
                                quantity: newValue,
                              };
                              setLocalVariations(newVariations);
                              setVariations(newVariations);
                            }}
                          />
                        )}
                      />
                      <Controller
                        name={`variations[${index}].stockLiveConfig.isPreOrderAuthorized`}
                        control={control}
                        defaultValue={AuthorizePreOrder.DO_NOT_AUTHORIZE}
                        render={({ field }) => (
                          <RHFSelect
                            {...field}
                            native
                            value={
                              field.value ||
                              variation.stockLiveConfig.isPreOrderAuthorized
                            }
                            label="Autoriser les commandes en réapprovisionnement ?"
                            InputLabelProps={{ shrink: true }}
                            onChange={(e) => {
                              field.onChange(e.target.value);
                              handleBackorderChangeLive(e);
                              // Update variations
                              const newVariations = [...localVariations];
                              newVariations[index].stockLiveConfig = {
                                ...newVariations[index].stockLiveConfig,
                                isPreOrderAuthorized: e.target
                                  .value as AuthorizePreOrder,
                              };
                              setLocalVariations(newVariations);
                              setVariations(newVariations);
                            }}
                          >
                            {ALLOW_BACKORDER_OPTIONS.map((option) => (
                              <option key={option.value} value={option.value}>
                                {option.label}
                              </option>
                            ))}
                          </RHFSelect>
                        )}
                      />
                      {(backorderOptionLive === AuthorizePreOrder.AUTHORIZE ||
                        backorderOptionLive ===
                        AuthorizePreOrder.AUTHORIZE_WITH_CUSTOMER_NOTIFICATION) && (
                          <>
                            <Controller
                              name={`variations[${index}].stockLiveConfig.authorizedConfig`}
                              control={control}
                              defaultValue={true}
                              render={({ field }) => (
                                <FormControlLabel
                                  control={
                                    <Switch
                                      {...field}
                                      checked={
                                        field.value ||
                                        variation.stockLiveConfig.authorizedConfig
                                      }
                                      onChange={(e) => {
                                        field.onChange(e.target.checked);
                                        setAllowPreorderLive(e.target.checked);
                                        // Update variations
                                        const newVariations = [
                                          ...localVariations,
                                        ];
                                        newVariations[index].stockLiveConfig = {
                                          ...newVariations[index].stockLiveConfig,
                                          authorizedConfig: e.target.checked,
                                        };
                                        setLocalVariations(newVariations);
                                        setVariations(newVariations);
                                      }}
                                    />
                                  }
                                  label="Définir la limite de précommande"
                                />
                              )}
                            />
                            {allowPreorderLive && (
                              <Controller
                                name={`variations[${index}].stockLiveConfig.thresholdValue`}
                                control={control}
                                render={({ field }) => (
                                  <RHFTextField
                                    {...field}
                                    value={
                                      field.value ||
                                      variation.stockLiveConfig.thresholdValue
                                    }
                                    label="Seuil de précommande Live"
                                 
                                    InputLabelProps={{ shrink: true }}
                                    placeholder="0"
                                    onChange={(e) => {
                                      const newValue = parseInt(
                                        e.target.value,
                                        10
                                      );
                                      field.onChange(newValue);

                                      const newVariations = [...localVariations];
                                      newVariations[index].stockLiveConfig = {
                                        ...newVariations[index].stockLiveConfig,
                                        thresholdValue: newValue,
                                      };
                                      setLocalVariations(newVariations);
                                      setVariations(newVariations);
                                    }}
                                  />
                                )}
                              />
                            )}
                          </>
                        )}
                      <Controller
                        name={`variations[${index}].liveProductConfigs.authorizeUpdateQuantity`}
                        control={control}
                        render={({ field }) => (
                          <FormControlLabel
                            control={
                              <Switch
                                {...field}
                                checked={
                                  field.value ||
                                  variation.liveProductConfigs
                                    .authorizeUpdateQuantity
                                }
                                onChange={(e) => {
                                  const newValue = e.target.checked;
                                  field.onChange(newValue);
                             
                                  const newVariations = [...localVariations];
                                  newVariations[index].liveProductConfigs = {
                                    ...newVariations[index].liveProductConfigs,
                                    authorizeUpdateQuantity: newValue,
                                  };
                                  setLocalVariations(newVariations);
                                  setVariations(newVariations);
                                }}
                              />
                            }
                            label="Autoriser les Modifications"
                          />
                        )}
                      />
                      <Controller
                        name={`variations[${index}].liveProductConfigs.feedStockFromWebStock`}
                        control={control}
                        render={({ field }) => (
                          <FormControlLabel
                            control={
                              <Switch
                                {...field}
                                checked={
                                  field.value ||
                                  variation.liveProductConfigs
                                    .feedStockFromWebStock
                                }
                                onChange={(e) => {
                                  const newValue = e.target.checked;
                                  field.onChange(newValue);
                                  const newVariations = [...localVariations];
                                  newVariations[index].liveProductConfigs = {
                                    ...newVariations[index].liveProductConfigs,
                                    feedStockFromWebStock: newValue,
                                  };
                                  setLocalVariations(newVariations);
                                  setVariations(newVariations);
                                }}
                              />
                            }
                            label="Alimenter le stock depuis le stock web"
                          />
                        )}
                      />
                    </>
                  )}

                  <Grid container spacing={1} mt={1}>
                    <AddMediaButton
                      variant="outlined"
                      color="success"
                      size="medium"
                      sx={{ mt: 1 }}
                      returnedSingleMedia={returnedMainImage}
                      setReturnedSingleMedia={(media) => {
                        setReturnedMainImage(media);
                        handleImageChange(index, media[0]);
                      }}
                    />
                  </Grid>


                </Stack>
              </Card>
            </Grid>
          )}
        </Box>
      ))}

      {/* Dialog de confirmation de suppression */}
      <Dialog open={openConfirmation} onClose={cancelRemoveVariation}>
        <DialogTitle>Attention</DialogTitle>
        <DialogContent>
          <DialogContentText>
            Attention, les modifications faites peuvent avoir un impact si le
            produit est dans le panier. Voulez-vous confirmer l'enregistrement ?
          </DialogContentText>
        </DialogContent>
        <DialogActions>
          <Button onClick={cancelRemoveVariation} color="inherit">
            Annuler
          </Button>
          <Button onClick={confirmRemoveVariation} color="error" autoFocus>
            Confirmer
          </Button>
        </DialogActions>
      </Dialog>
    </div>
  );
};

export default Variations;


