"use client";
import isEqual from "lodash/isEqual";
import { useState, useEffect, useCallback } from "react";
import axiosInstance, { endpoints } from "@/shared/api/server";

import Stack from "@mui/material/Stack";
import Button from "@mui/material/Button";
import Container from "@mui/material/Container";
import Box from "@mui/material/Box";
import {
  DataGrid,
  GridColDef,
  GridActionsCellItem,
  GridToolbarContainer,
  GridRowSelectionModel,
  GridToolbarColumnsButton,
  GridColumnVisibilityModel,
  GridFilterModel,
  GridSortModel,
} from "@mui/x-data-grid";
import Snackbar from "@mui/material/Snackbar";
import Alert from "@mui/material/Alert";
import Dialog from "@mui/material/Dialog";
import DialogActions from "@mui/material/DialogActions";
import DialogContent from "@mui/material/DialogContent";
import DialogContentText from "@mui/material/DialogContentText";
import TextField from "@mui/material/TextField";

import { paths } from "@/routes/paths";
import { useRouter } from "@/hooks";
import RouterLink from "@/routes/router-link";

import { useBoolean } from "@/hooks/use-boolean";

import {
  useDeleteProduct,
  useDeleteProducts,
  duplicateProduct,
  useFilteredProducts,
  downloadProductZebraFile,
  checkUgsUnique,
} from "@/shared/api/product";

import Iconify from "@/shared/components/iconify";
import { ConfirmDialog } from "@/shared/components/custom-dialog";
import { useSettingsContext } from "@/shared/components/settings";
import CustomBreadcrumbs from "@/shared/components/custom-breadcrumbs";

import {
  IProductListItem,
  IProductTableFilters,
  IProductTableFilterValue,
} from "@/shared/types/product";

import ProductTableToolbar from "./product-table-toolbar";
import ProductTableFiltersResult from "./product-table-filters-result";
import {
  RenderCellStock,
  RenderCellPrice,
  RenderCellPublish,
  RenderCellProduct,
  RenderCellCreatedAt,
  RenderCellPuchasePrice,
} from "./product-table-row";

import ProductSearch from "../../../components/SearchField";
import { frFR } from '@mui/x-data-grid';
import { DialogTitle } from "@mui/material";

const stockOptions = [
  { value: "true", label: "En stock" },
  { value: "false", label: "En rupture de stock" },
];

const publishOptions = [
  { value: "PublierLive", label: "Publié Live" },
  { value: "PublierWeb", label: "Publié Web" },
  { value: "Brouillon", label: "Brouillon" },
];

const defaultFilters: IProductTableFilters = {
  publish: [],
  isInStockWeb: null,
  isInStockLive: null,
  keyword: "",
  isDraft: false,
  isPublishLive: false,
  isPublishWeb: false,
  priceMax: null,
  priceMin: null,
  startDate: null,
  endDate: null,

};
const HIDE_COLUMNS = {
  categoryNames: false,
};

const HIDE_COLUMNS_TOGGLABLE = ["categoryNames", "actions"];

export default function ProductListView() {
  const confirmRows = useBoolean();
  const router = useRouter();
  const settings = useSettingsContext();
  const [tableData, setTableData] = useState<IProductListItem[]>([]);
  const [pageNumb, setPageNumb] = useState(0);
  const [pageSiz, setPageSiz] = useState(10);
  const [selectedRowIds, setSelectedRowIds] = useState<GridRowSelectionModel>([]);
  const [successMessage, setSuccessMessage] = useState<string | null>(null);
  const [errorMessage, setErrorMessage] = useState<string | null>(null);
  const [forceUpdate, setForceUpdate] = useState(0);
  const [columnVisibilityModel, setColumnVisibilityModel] = useState<GridColumnVisibilityModel>(HIDE_COLUMNS);
  const [confirmDeleteRow, setConfirmDeleteRow] = useState<number | null>(null);
  const [searchTerm, setSearchTerm] = useState<string>('');
  const [sortModel, setSortModel] = useState<GridSortModel>([
    { field: "createdAt", sort: "desc" },
  ]);
  
  const defaultFiltersLocalStorage: IProductTableFilters = localStorage.getItem("lastProductsFilters")
  ?
  JSON.parse(localStorage.getItem("lastProductsFilters")!)
  :
  {
    publish: [],
    isInStockWeb: null,
    isInStockLive: null,
    keyword: "",
    isDraft: false,
    isPublishLive: false,
    isPublishWeb: false,
    priceMax: null,
    priceMin: null,
    startDate: null,
    endDate: null,
  };
  
  
  const [filters, setFilters] = useState<IProductTableFilters>(defaultFiltersLocalStorage);
  
    useEffect(() => {
      localStorage.removeItem("lastProductsFilters")
      localStorage.setItem("lastProductsFilters", JSON.stringify(filters));
    }, [filters]);
    
  const sortBy = sortModel.map((item) => item.field);
  const direction: string[] = sortModel.map((item) => (item.sort ?? "desc"));
  const [filterModel, setFilterModel] = useState<GridFilterModel>({
    items: [],
    quickFilterValues: [],
  });
  const {
    products,
    totalProducts,
    totalPages,
    pageNumber,
    pageSize,
    loading,
    revalidate,
  } = useFilteredProducts({
    ...filters,
    page: pageNumb,
    size: pageSiz,
    sortBy,
    direction,
    startDate: filters.startDate,
    endDate: filters.endDate,
  });
  const handlePaginationModelChange = useCallback(
    (newPaginationModel: { page: number; pageSize: number }) => {
      setPageNumb(newPaginationModel.page);
      setPageSiz(newPaginationModel.pageSize);
    },
    []
  );
  
  useEffect(() => {
    if (totalProducts !== 0 && tableData !== products) {
      setTableData(products);
    } else if (totalProducts === 0 && tableData.length !== 0) {
      setTableData([]);
    }
  }, [products, forceUpdate, totalProducts, tableData]);
  
  
  const canReset = !isEqual(defaultFilters, filters);
  
  const handleFilters = useCallback(
    (name: string, value: IProductTableFilterValue) => {
      setFilters((prevState) => ({
        ...prevState,
        [name]: value,
      }));
    },
    []
  );
  
  const handleResetFilters = useCallback(() => {
    setFilters(defaultFilters);
    setFilterModel({
      items: [],
      quickFilterValues: [],
    });
    setSearchTerm('');
  }, []);

  const handleExport = async () => {
    try {
      // Construct query parameters
      const exportParams = {
        priceMin: filters.priceMin,
        priceMax: filters.priceMax,
        sortBy: sortBy.join(','),
        direction: direction.join(','),
        isDraft: filters.isDraft,
        isPublishLive: filters.isPublishLive,
        isPublishWeb: filters.isPublishWeb,
        isInStockWeb: filters.isInStockWeb,
        isInStockLive: filters.isInStockLive,
        keyword: filters.keyword,
        startDate: filters.startDate,
        endDate: filters.endDate,
      };

      // Remove null or undefined values from the query parameters
      const queryString = new URLSearchParams(
        Object.entries(exportParams).reduce((acc, [key, value]) => {
          if (value !== null && value !== undefined && value !== "") {
            acc[key] = value.toString();
          }
          return acc;
        }, {} as Record<string, string>)
      ).toString();


      // Make the API call with query parameters
      const response = await axiosInstance.get(`${endpoints.product.export}?${queryString}`, {
        headers: {
          "Content-Type": "application/json",
        },
      });

      // Handle the response and initiate file download
      const blob = new Blob([response.data], { type: "text/csv" });
      const url = window.URL.createObjectURL(blob);
      const a = document.createElement("a");
      a.href = url;
      a.download = "products.csv";
      a.click();
      window.URL.revokeObjectURL(url);
    } catch (error) {
      console.error(error);
      setErrorMessage(
        typeof error === "string"
          ? error
          : "An error occurred while exporting products."
      );
    }
  };


  const handleSearch = useCallback((searchTerm: string) => {
    handleFilters('keyword', searchTerm);
    revalidate();
  }, [handleFilters, revalidate]);

  const handleDeleteRow = async (id: number) => {
    try {
      const response = await useDeleteProduct(id.toString());
      if (response.status === 200 || response.status === 204) {
        await revalidate();
        setSelectedRowIds([]);
        setSuccessMessage("Produit supprimée avec succès.");
        setForceUpdate((prev) => prev + 1);
        const timeoutId = setTimeout(() => {
          setSuccessMessage(null);
          clearTimeout(timeoutId);
        }, 5000);
      }
    } catch (error: unknown) {
      let errorMessage = "Erreur lors de la suppression du Produit.";
      if (typeof error === "string") {
        errorMessage = error;
      }
      setErrorMessage(errorMessage);
      const timeoutId = setTimeout(() => {
        setErrorMessage(null);
        clearTimeout(timeoutId);
      }, 5000);
    }
  };

  const handleDeleteRows = async () => {
    try {
      if (selectedRowIds.length > 0) {
        const idsToDelete = selectedRowIds.map((id) => id.toString());
        const response = await useDeleteProducts(idsToDelete);
        if (response.status === 200 || response.status === 204) {
          await revalidate();
          setSelectedRowIds([]);
          setSuccessMessage("Produit(s) supprimée(s) avec succès.");
          setForceUpdate((prev) => prev + 1);
          const timeoutId = setTimeout(() => {
            setSuccessMessage(null);
            clearTimeout(timeoutId);
          }, 5000);
        }
      }
    } catch (error: unknown) {
      let errorMessage = "Erreur lors de la suppression des produits.";
      if (typeof error === "string") {
        errorMessage = error;
      }
      setErrorMessage(errorMessage);
      const timeoutId = setTimeout(() => {
        setErrorMessage(null);
        clearTimeout(timeoutId);
      }, 5000);
    }
  };

  const handleEditRow = useCallback(
    (id: string) => {
      router.push(paths.dashboard.admin.product.edit(id));
    },
    [router]
  );

  const handleViewRow = useCallback(
    (id: string) => {
      router.push(paths.dashboard.admin.product.details(id));
    },
    [router]
  );

  const [duplicateDialogOpen, setDuplicateDialogOpen] = useState<boolean>(false);
  const [productToDuplicate, setProductToDuplicate] = useState<string | null>(null);
  const [newUgs, setNewUgs] = useState<string>("");
  const [isCheckingUgs, setIsCheckingUgs] = useState(false);
  const [ugsError, setUgsError] = useState<string | null>(null);

  const handleDuplicateDialogOpen = (id: string) => {
    setProductToDuplicate(id);
    setNewUgs("");
    setDuplicateDialogOpen(true);
  };

  const handleDuplicateDialogClose = () => {
    setDuplicateDialogOpen(false);
    setProductToDuplicate(null);
    setNewUgs("");
    setUgsError(null);
  };

  const handleDuplicateRow = async (id: string) => {
    handleDuplicateDialogOpen(id);
  };

  const handleConfirmDuplicate = async () => {
    if (!productToDuplicate || !newUgs.trim()) {
      setErrorMessage("L'UGS est obligatoire pour dupliquer le produit.");
      return;
    }

    try {
      setIsCheckingUgs(true);
      const isUnique = await checkUgsUnique(newUgs);
      setIsCheckingUgs(false);
      
      if (!isUnique) {
        setUgsError("UGS déjà utilisé. Veuillez en choisir un autre.");
        return;
      }
      
      await duplicateProduct(productToDuplicate, newUgs);
      setSuccessMessage("Produit dupliqué avec succès.");
      revalidate();
      handleDuplicateDialogClose();
      const timeoutId = setTimeout(() => {
        setSuccessMessage(null);
        clearTimeout(timeoutId);
      }, 5000);
    } catch (error) {
      setErrorMessage("Erreur lors de la duplication du produit.");
      const timeoutId = setTimeout(() => {
        setErrorMessage(null);
        clearTimeout(timeoutId);
      }, 5000);
    }
  };

  const handleUgsChange = (e: React.ChangeEvent<HTMLInputElement>) => {
    setNewUgs(e.target.value);
    setUgsError(null); // Clear error when input changes
  };

  const handleDownloadZebraFile = async (id: string) => {
    try {
      await downloadProductZebraFile(id);
    } catch (err) {
      console.error('Error downloading file:', err);
    }
  };

  const columns: GridColDef[] = [
    {
      type: "actions",
      field: "actions",
      headerName: " ",
      align: "right",
      headerAlign: "right",
      width: 60,
      sortable: false,
      filterable: false,
      disableColumnMenu: true,
      getActions: (params) => [
        <GridActionsCellItem
          showInMenu
          icon={<Iconify icon="solar:eye-bold" />}
          label="Prévisualiser"
          onClick={() => handleViewRow(params.row.id)}
        />,
        <GridActionsCellItem
          showInMenu
          icon={<Iconify icon="solar:pen-bold" />}
          label="Modifier"
          onClick={() => handleEditRow(params.row.id)}
        />,
        <GridActionsCellItem
          showInMenu
          icon={<Iconify icon="solar:copy-bold" />}
          label="Dupliquer"
          onClick={() => handleDuplicateRow(params.row.id)}
        />,
        <GridActionsCellItem
          showInMenu
          icon={<Iconify icon="solar:trash-bin-trash-bold" />}
          label="Supprimer"
          onClick={() => setConfirmDeleteRow(params.row.id)}
          sx={{ color: "error.main" }}
        />,
        <GridActionsCellItem
          showInMenu
          icon={<Iconify icon="solar:download-bold" />}
          label="Fichier Zebra"
          onClick={() => handleDownloadZebraFile(params.row.id)}
        />,
      ],
    },
    {
      field: "categoryNames",
      headerName: "Catégorie",
      valueGetter: (params) => params.row.categoryNames.join(", "),
    },
    {
      field: "name",
      headerName: "Produit",
      flex: 1,
      minWidth: 200,
      renderCell: (params) => <RenderCellProduct params={params} />,
    },
    {
      field: "ugs",
      headerName: "UGS",
      width: 100,
    },
    {
      field: "stockWebQuantity",
      headerName: "Stock Web",
      width: 160,
      renderCell: (params) => (
        <RenderCellStock quantity={params.row.stockWebQuantity} />
      ),
    },
    {
      field: "stockLiveQuantity",
      headerName: "Stock Live",
      width: 160,
      renderCell: (params) => (
        <RenderCellStock quantity={params.row.stockLiveQuantity} />
      ),
    },
    {
      field: "regularPrice",
      headerName: "Prix",
      width: 90,
      renderCell: (params) => <RenderCellPrice params={params} />,
    },
    {
      field: "tagNames",
      headerName: "Étiquettes",
      width: 140,
      valueGetter: (params) => {
        return params.row.tagNames.length !== 0
          ? params.row.tagNames.join(", ")
          : "sans étiquette";
      },
    },
    {
      field: "productType",
      headerName: "Type",
      width: 140,
      valueGetter: (params) =>
        params.row.productType === "SIMPLE" ? "Simple" : "Variable",
    },
    {
      field: "createdAt",
      headerName: "Date Création",
      width: 160,
      renderCell: (params) => <RenderCellCreatedAt params={params} />,
    },
    {
      field: "draft",
      headerName: "Publié",
      width: 110,
      renderCell: (params) => <RenderCellPublish params={params} />,
    },
    {
      field: "purchasePrice",
      headerName: "Prix d'achat",
      width: 80,
      renderCell: (params) => <RenderCellPuchasePrice params={params} />,
    },
    {
      field: "supplierName",
      headerName: "Fournisseur",
      width: 100,
    },
    {
      field: "supplierRef",
      headerName: "Référence",
      width: 70,
    },

  ];

  const getTogglableColumns = () =>
    columns
      .filter((column) => !HIDE_COLUMNS_TOGGLABLE.includes(column.field))
      .map((column) => column.field);
  const handleSortModelChange = (newSortModel: GridSortModel) => {
    if (newSortModel.length > 0) {
      const updatedSortModel = [newSortModel[newSortModel.length - 1]];
      setSortModel(updatedSortModel);
      revalidate();
    } else {
      setSortModel([]);
      revalidate();
    }
  };

  return (
    <>
      <Container
        maxWidth={settings.themeStretch ? false : "xl"}
        sx={{
          flexGrow: 1,
          display: "flex",
          flexDirection: "column",
          height: "100vh",
        }}
      >
        <CustomBreadcrumbs
          heading="Liste des produits"
          links={[
            { name: "Tableau de bord", href: paths.dashboard.root },
            {
              name: "Produits",
              href: paths.dashboard.admin.product.root,
            },
            { name: "Lister" },
          ]}
          action={
            <Button
              component={RouterLink}
              href={paths.dashboard.admin.product.new}
              variant="contained"
              startIcon={<Iconify icon="mingcute:add-line" />}
            >
              Ajouter des Produits
            </Button>
          }
          sx={{
            mb: {
              xs: 3,
              md: 5,
            },
          }}
        />

        <Box
          sx={{
            height: "100%",
            width: "100%",
            flexGrow: { md: 1 },
            display: { md: "flex" },
            flexDirection: { md: "column" },
          }}
        >
          <DataGrid
            onSortModelChange={handleSortModelChange}
            sortingOrder={["asc", "desc"]}
            sortModel={sortModel}
            checkboxSelection
            disableRowSelectionOnClick
            rows={tableData}
            columns={columns}
            loading={loading}
            getRowHeight={() => "auto"}
            pageSizeOptions={[5, 10, 25]}
            filterModel={filterModel}
            rowCount={totalProducts}

            onFilterModelChange={(newModel) => {
              setFilterModel(newModel);
            }}
            pagination
            paginationMode="server"
            initialState={{
              pagination: {
                paginationModel: {
                  pageSize: 10,
                  page: pageNumb,
                },
              },
              sorting: {
                sortModel: sortModel,
              },
            }}
            onPaginationModelChange={handlePaginationModelChange}
            onRowSelectionModelChange={(newSelectionModel) => {
              setSelectedRowIds(newSelectionModel);
            }}
            columnVisibilityModel={columnVisibilityModel}
            onColumnVisibilityModelChange={(newModel) =>
              setColumnVisibilityModel(newModel)
            }
            localeText={{
              ...frFR.components.MuiDataGrid.defaultProps.localeText,
              toolbarColumnsLabel: "Colonnes",
              noRowsLabel: "Aucun produit trouvé",
              filterPanelInputPlaceholder: "Valeur à filtrer",
              filterPanelColumns: "Colonnes",
              toolbarExportCSV: "Télécharger au format CSV",
              toolbarExportPrint: "Imprimer",
              filterPanelOperator: "Opérateur",
              filterPanelInputLabel: "Valeur",
            }}
            slots={{
              toolbar: () => (
                <>
                  <GridToolbarContainer>
                    <ProductTableToolbar
                      filters={filters}
                      onFilters={handleFilters}
                      stockOptions={stockOptions}
                      publishOptions={publishOptions}
                    />
                    <ProductSearch onSearch={handleSearch} />
                    <Stack
                      spacing={1}
                      flexGrow={1}
                      direction="row"
                      alignItems="center"
                      justifyContent="flex-end"
                    >
                      {!!selectedRowIds.length && (
                        <Stack>
                          <Button
                            size="small"
                            color="error"
                            startIcon={<Iconify icon="solar:trash-bin-trash-bold" />}
                            onClick={confirmRows.onTrue}
                          >
                            Supprimer ({selectedRowIds.length})
                          </Button>
                        </Stack>
                      )}

                      <GridToolbarColumnsButton />
                      {/* <GridToolbarFilterButton /> */}
                      {(filters.startDate != null || filters.endDate != null) && <Button
                        onClick={handleExport}
                        startIcon={<Iconify icon="material-symbols:download" />}
                      >
                        Exporter
                      </Button>}
                    </Stack>
                  </GridToolbarContainer>
                  {canReset && (
                    <ProductTableFiltersResult
                      filters={filters}
                      onFilters={handleFilters}
                      onResetFilters={handleResetFilters}
                      results={totalProducts}
                      sx={{ p: 2.5, pt: 0 }}
                    />
                  )}
                </>
              ),
            }}
            slotProps={{
              columnsPanel: {
                getTogglableColumns,
              },
            }}
          />
        </Box>
      </Container>

      <ConfirmDialog
        open={confirmRows.value}
        onClose={confirmRows.onFalse}
        title="Supprimer"
        content={
          <>
            Êtes-vous sûr de vouloir supprimer{" "}
            <strong> {selectedRowIds.length} </strong> items?
          </>
        }
        action={
          <Button
            variant="contained"
            color="error"
            onClick={() => {
              handleDeleteRows();
              confirmRows.onFalse();
            }}
          >
            Supprimer
          </Button>
        }
      />

      <ConfirmDialog
        open={confirmDeleteRow !== null}
        onClose={() => setConfirmDeleteRow(null)}
        title="Supprimer"
        content="Êtes-vous sûr de vouloir supprimer cet élément?"
        action={
          <Button
            variant="contained"
            color="error"
            onClick={() => {
              if (confirmDeleteRow !== null) {
                handleDeleteRow(confirmDeleteRow);
                setConfirmDeleteRow(null);
              }
            }}
          >
            Supprimer
          </Button>
        }
      />

      <Dialog open={duplicateDialogOpen} onClose={handleDuplicateDialogClose}>
        <DialogTitle>Dupliquer le produit</DialogTitle>
        <DialogContent>
          <DialogContentText>
            Veuillez entrer un UGS unique pour le produit dupliqué:
          </DialogContentText>
          <TextField
            autoFocus
            margin="dense"
            id="newUgs"
            label="UGS"
            fullWidth
            variant="outlined"
            value={newUgs}
            onChange={handleUgsChange}
            error={!!ugsError}
            helperText={ugsError}
            disabled={isCheckingUgs}
          />
        </DialogContent>
        <DialogActions>
          <Button onClick={handleDuplicateDialogClose}>Annuler</Button>
          <Button 
            onClick={handleConfirmDuplicate} 
            variant="contained"
            disabled={isCheckingUgs || !newUgs.trim()}
          >
            {isCheckingUgs ? 'Vérification...' : 'Dupliquer'}
          </Button>
        </DialogActions>
      </Dialog>

      {successMessage && (
        <Snackbar
          open={true}
          autoHideDuration={5000}
          onClose={() => setSuccessMessage(null)}
          anchorOrigin={{ vertical: "top", horizontal: "right" }}
        >
          <Alert onClose={() => setSuccessMessage(null)} severity="success">
            {successMessage}
          </Alert>
        </Snackbar>
      )}

      {errorMessage && (
        <Snackbar
          open={true}
          autoHideDuration={5000}
          onClose={() => setErrorMessage(null)}
          anchorOrigin={{ vertical: "top", horizontal: "right" }}
        >
          <Alert onClose={() => setErrorMessage(null)} severity="error">
            {errorMessage}
          </Alert>
        </Snackbar>
      )}
    </>
  );
}

// ----------------------------------------------------------------------

