
import { useCallback } from "react";

import Box from "@mui/material/Box";
import Chip from "@mui/material/Chip";
import Paper from "@mui/material/Paper";
import Button from "@mui/material/Button";
import Stack, { StackProps } from "@mui/material/Stack";

import Iconify from "@/shared/components/iconify";

import { IProductTableFilters, IProductTableFilterValue } from "@/shared/types/product";

// ----------------------------------------------------------------------

type Props = StackProps & {
  filters: IProductTableFilters;
  onFilters: (name: string, value: IProductTableFilterValue) => void;
  //
  onResetFilters: VoidFunction;
  //
  results: number;
};

export default function ProductTableFiltersResult({
  filters,
  onFilters,
  onResetFilters,
  results,
  ...other
}: Props) {
 const handleRemovePublish = useCallback(
    (inputValue: string) => {
      if (inputValue === "Brouillon") {
        onFilters("isDraft", false);
      } else if (inputValue === "PublierLive") {
        onFilters("isPublishLive", false);
      } else if (inputValue === "PublierWeb") {
        onFilters("isPublishWeb", false);
      }
    },
    
    [onFilters]
  );
  const handleRemoveStockWeb = useCallback(() => {
    onFilters("isInStockWeb", null); 
  }, [filters.isInStockWeb, onFilters]);

  const handleRemoveStockLive = useCallback(() => {
    onFilters("isInStockLive", null); 
  }, [filters.isInStockLive, onFilters]);
 const handleRemovePrice = useCallback(() => {
    onFilters("priceMin", null);
    onFilters("priceMax", null);
  }, [onFilters]);
    const handleRemoveSearchKeyword = useCallback(() => {
    onFilters("keyword", null);  
  }, [onFilters]);
  
  return (
    <Stack spacing={1.5} {...other}>
      <Box sx={{ typography: "body2" }}>
        <strong>{results}</strong>
        <Box component="span" sx={{ color: "text.secondary", ml: 0.25 }}>
          results found
        </Box>
      </Box>

      <Stack
        flexGrow={1}
        spacing={1}
        direction="row"
        flexWrap="wrap"
        alignItems="center"
      >
       {(filters.isDraft || filters.isPublishLive || filters.isPublishWeb) && (
          <Block label="Publier:">
            {filters.isDraft && (
              <Chip
                key="Brouillon"
                label="Brouillon"
                size="small"
                onDelete={() => handleRemovePublish("Brouillon")}
              />
            )}
            {filters.isPublishLive && (
              <Chip
                key="PublierLive"
                label="Publié Live"
                size="small"
                onDelete={() => handleRemovePublish("PublierLive")}
              />
            )}
            {filters.isPublishWeb && (
              <Chip
                key="PublierWeb"
                label="Publié Web"
                size="small"
                onDelete={() => handleRemovePublish("PublierWeb")}
              />
            )}
          </Block>
        )}
  {filters.keyword && (
              <Block label="Rechercher:">
            <Chip label={filters.keyword} size="small" onDelete={handleRemoveSearchKeyword} />
          </Block>
            )}

        {filters.isInStockWeb !== null && (
          <Block label="Stock Web:">
            <Chip
              key="stock-web"
              label={filters.isInStockWeb ? "En stock" : "En rupture de stock"}
              size="small"
              onDelete={handleRemoveStockWeb}
            />
          </Block>
        )}

        {filters.isInStockLive !== null && (
          <Block label="Stock Live:">
            <Chip
              key="stock-live"
              label={filters.isInStockLive ? "En stock" : "En rupture de stock"}
              size="small"
              onDelete={handleRemoveStockLive}
            />
          </Block>
        )}
{(filters.priceMin || filters.priceMax) && (
          <Block label="Price:">
            {filters.priceMin && (
              <Chip
                key="price-min"
                label={`Min: ${filters.priceMin}`}
                size="small"
                onDelete={handleRemovePrice}
              />
            )}
            {filters.priceMax && (
              <Chip
                key="price-max"
                label={`Max: ${filters.priceMax}`}
                size="small"
                onDelete={handleRemovePrice}
              />
            )}
          </Block>
        )}
        <Button
          color="error"
          onClick={onResetFilters}
          startIcon={<Iconify icon="solar:trash-bin-trash-bold" />}
        >
          Clear
        </Button>
      </Stack>
    </Stack>
  );
}

// ----------------------------------------------------------------------

type BlockProps = StackProps & {
  label: string;
};

function Block({ label, children, sx, ...other }: BlockProps) {
  return (
    <Stack
      component={Paper}
      variant="outlined"
      spacing={1}
      direction="row"
      sx={{
        p: 1,
        borderRadius: 1,
        overflow: "hidden",
        borderStyle: "dashed",
        ...sx,
      }}
      {...other}
    >
      <Box component="span" sx={{ typography: "subtitle2" }}>
        {label}
      </Box>

      <Stack spacing={1} direction="row" flexWrap="wrap">
        {children}
      </Stack>
    </Stack>
  );
}
