import { useState, useCallback, useRef, useEffect } from "react";
import MenuItem from "@mui/material/MenuItem";
import InputLabel from "@mui/material/InputLabel";
import FormControl from "@mui/material/FormControl";
import OutlinedInput from "@mui/material/OutlinedInput";
import Select, { SelectChangeEvent } from "@mui/material/Select";
import Checkbox from "@mui/material/Checkbox";
import Iconify from "@/shared/components/iconify";
import CustomPopover, { usePopover } from "@/shared/components/custom-popover";
import { IProductTableFilters, IProductTableFilterValue } from "@/shared/types/product";
import { Button, TextField } from "@mui/material";
import CustomDateRangePicker from "@/shared/components/custom-date-range-picker";

type Props = {
  filters: IProductTableFilters;
  onFilters: (name: string, value: IProductTableFilterValue) => void;
  stockOptions: { value: string; label: string }[];
  publishOptions: { value: string; label: string }[];
};

export default function ProductTableToolbar({
  filters,
  onFilters,
  stockOptions,
  publishOptions,
}: Props) {
  const popover = usePopover();

  const [isDraft, setIsDraft] = useState<boolean | null>(filters.isDraft || null);
  const [isPublishLive, setIsPublishLive] = useState<boolean | null>(filters.isPublishLive || null);
  const [isPublishWeb, setIsPublishWeb] = useState<boolean | null>(filters.isPublishWeb || null);
  const [isInStockWeb, setIsInStockWeb] = useState<string | null>(
    filters.isInStockWeb === true ? "inStock" : filters.isInStockWeb === false ? "outOfStock" : null
  );
  const [isInStockLive, setIsInStockLive] = useState<string | null>(
    filters.isInStockLive === true ? "inStock" : filters.isInStockLive === false ? "outOfStock" : null
  );
  const [priceMin, setPriceMin] = useState<number | null>(filters.priceMin || null);
  const [priceMax, setPriceMax] = useState<number | null>(filters.priceMax || null);
  const [errorMessageMin, setErrorMessageMin] = useState<string>("");
  const [errorMessageMax, setErrorMessageMax] = useState<string>("");
  const priceMinRef = useRef<HTMLInputElement | null>(null);
  const priceMaxRef = useRef<HTMLInputElement | null>(null);


  const handleChangePublish = useCallback(
    (event: SelectChangeEvent<string[]>) => {
      const { value } = event.target;
      const newPublish = typeof value === "string" ? value.split(",") : value;

      const updatedIsDraft = newPublish.includes("Brouillon");
      const updatedIsPublishLive = newPublish.includes("PublierLive");
      const updatedIsPublishWeb = newPublish.includes("PublierWeb");

      if (updatedIsDraft !== isDraft) onFilters("isDraft", updatedIsDraft);
      if (updatedIsPublishLive !== isPublishLive) onFilters("isPublishLive", updatedIsPublishLive);
      if (updatedIsPublishWeb !== isPublishWeb) onFilters("isPublishWeb", updatedIsPublishWeb);
    },
    [onFilters, isDraft, isPublishLive, isPublishWeb]
  );

  useEffect(() => {
    if (priceMinRef.current && priceMin !== null) {
      priceMinRef.current.focus();
    }
  }, [priceMin]);

  useEffect(() => {
    if (priceMaxRef.current && priceMax !== null) {
      priceMaxRef.current.focus();
    }
  }, [priceMax]);
  const handleChangeStockWeb = useCallback(
    (event: SelectChangeEvent<string>) => {
      const { value } = event.target;
      setIsInStockWeb(value);
      onFilters("isInStockWeb", value === "inStock");
    },
    [onFilters]
  );

  const handleChangeStockLive = useCallback(
    (event: SelectChangeEvent<string>) => {
      const { value } = event.target;
      setIsInStockLive(value);
      onFilters("isInStockLive", value === "inStock");
    },
    [onFilters]
  );
  const handleChangePriceMin = useCallback(
    (event: React.ChangeEvent<HTMLInputElement>) => {
      const newPriceMin = event.target.value ? parseFloat(event.target.value) : null;
      setPriceMin(newPriceMin);

      if (newPriceMin !== null && priceMax !== null && newPriceMin >= priceMax) {
        setErrorMessageMin("Le prix minimum doit être inférieur au prix maximum.");
      } else {
        setErrorMessageMin("");
        onFilters("priceMin", newPriceMin);
      }

      if (priceMinRef.current) {
        priceMinRef.current.focus();
      }
    },
    [onFilters, priceMax]
  );

  const handleChangePriceMax = useCallback(
    (event: React.ChangeEvent<HTMLInputElement>) => {
      const newPriceMax = event.target.value ? parseFloat(event.target.value) : null;
      setPriceMax(newPriceMax);

      if (newPriceMax !== null && priceMin !== null && newPriceMax <= priceMin) {
        setErrorMessageMax("Le prix maximum doit être supérieur au prix minimum.");
      } else {
        setErrorMessageMax("");
        onFilters("priceMax", newPriceMax);
      }

      if (priceMaxRef.current) {
        priceMaxRef.current.focus();
      }
    },
    [onFilters, priceMin]
  );

  

  const [isDateRangeOpen, setIsDateRangeOpen] = useState(false);
  

  const formatDateForBackend = (date: Date | null): string | null => {
    if (!date) return null;
    
    return date.toISOString()
      .replace(/\.\d{3}Z$/, '');
  };
  
  const onChangeStartDate = (newStartDate: Date | null) => {
    if (newStartDate) {
      const formattedDate = formatDateForBackend(newStartDate);
      onFilters("startDate", formattedDate);
    }
  };
  
  const onChangeEndDate = (newEndDate: Date | null) => {
    if (newEndDate) {
      const formattedDate = formatDateForBackend(newEndDate);
      onFilters("endDate", formattedDate);
    }
  };
  
  const closeDateRange = () => {
    setIsDateRangeOpen(!isDateRangeOpen);
  }

  const getLabelForValue = (value: string | null) => {
    return value === "inStock" ? "In Stock" : value === "outOfStock" ? "Out of Stock" : "";
  };
  return (
    <>
      <FormControl sx={{ flexShrink: 0, width: { xs: 1, md: 200 } }}>
        <InputLabel>Stock Web</InputLabel>
        <Select
          value={isInStockWeb || ""}
          onChange={handleChangeStockWeb}
          input={<OutlinedInput label="Stock Web" />}
          renderValue={(selected) => getLabelForValue(selected)}
          sx={{ textTransform: "capitalize" }}
        >
          {["inStock", "outOfStock"].map((option) => (
            <MenuItem key={option} value={option}>
              {option === "inStock" ? "En stock" : "En rupture de stock"}
            </MenuItem>
          ))}
        </Select>
      </FormControl>

      <FormControl sx={{ flexShrink: 0, width: { xs: 1, md: 200 } }}>
        <InputLabel>Stock Live</InputLabel>
        <Select
          value={isInStockLive || ""}
          onChange={handleChangeStockLive}
          input={<OutlinedInput label="Stock Live" />}
          renderValue={(selected) => getLabelForValue(selected)}
          sx={{ textTransform: "capitalize" }}
        >
          {["inStock", "outOfStock"].map((option) => (
            <MenuItem key={option} value={option}>
              {option === "inStock" ? "En stock" : "En rupture de stock"}
            </MenuItem>
          ))}
        </Select>
      </FormControl>

      <FormControl sx={{ flexShrink: 0, width: { xs: 1, md: 200 } }}>
        <InputLabel>Publié</InputLabel>
        <Select
          multiple
          value={["PublierLive", "PublierWeb", "Brouillon"].filter((option) => {
            if (option === "PublierLive") return isPublishLive;
            if (option === "PublierWeb") return isPublishWeb;
            if (option === "Brouillon") return isDraft;
            return false;
          })}
          onChange={handleChangePublish}
          input={<OutlinedInput label="Publier" />}
          renderValue={(selected) => selected.join(", ")}
          sx={{ textTransform: "capitalize" }}
        >
          {publishOptions.map((option) => (
            <MenuItem key={option.value} value={option.value}>
              <Checkbox
                disableRipple
                size="small"
                checked={
                  (option.value === "PublierLive" && isPublishLive === true) ||
                  (option.value === "PublierWeb" && isPublishWeb === true) ||
                  (option.value === "Brouillon" && isDraft === true)
                }
              />
              {option.label}
            </MenuItem>
          ))}
        </Select>
      </FormControl>
      <FormControl sx={{ flexShrink: 0, width: { xs: 1, md: 100 } }}>
        <TextField
          label="Prix Min"
          type="number"
          value={priceMin || ""}
          onChange={handleChangePriceMin}
          inputRef={priceMinRef}
          InputProps={{
            startAdornment: <Iconify icon="mdi:currency-eur" sx={{ mr: 1 }} />,
          }}
          fullWidth
          error={!!errorMessageMin}
        />
      </FormControl>

      <FormControl sx={{ flexShrink: 0, width: { xs: 1, md: 100 } }}>
        <TextField
          label="Prix Max"
          type="number"
          value={priceMax || ""}
          inputRef={priceMaxRef}
          onChange={handleChangePriceMax}
          InputProps={{
            startAdornment: <Iconify icon="mdi:currency-eur" sx={{ mr: 1 }} />,
          }}
          fullWidth
          error={!!errorMessageMax}
        />
      </FormControl>

      <Button
        sx={{
          width: {
            xs: '100%', sm: '50%', md: '25%'
          },
          height: {
            xs: '45px', sm: '50px', md: '53px'
          },
          whiteSpace: 'nowrap',
          overflow: 'hidden',
          textOverflow: 'ellipsis',
          fontSize: {
            xs: '12px', sm: '14px'
          },
          padding: {
            xs: '8px', sm: '12px', md: '16px'
          }
        }}
        variant="outlined"
        startIcon={
          <Iconify
            icon="carbon:calendar"
            sx={{
              mr: { xs: 0.5, sm: 1 },
              fontSize: {
                xs: '18px',
                sm: '20px',
                md: '24px'
              }
            }}
          />
        }
        onClick={() => setIsDateRangeOpen(true)}
      >
        {filters.startDate?.split("T")[0]} {filters.endDate != null && filters.startDate != null && " / "} {filters.endDate?.split("T")[0]}
      </Button>
      <CustomPopover
        open={popover.open}
        onClose={popover.onClose}
        arrow="right-top"
        sx={{ width: 140 }}
      >
        <MenuItem onClick={() => popover.onClose()}>
          <Iconify icon="solar:printer-minimalistic-bold" /> Imprimer
        </MenuItem>
        <MenuItem onClick={() => popover.onClose()}>
          <Iconify icon="solar:import-bold" /> Importer
        </MenuItem>
        <MenuItem onClick={() => popover.onClose()}>
          <Iconify icon="solar:export-bold" /> Exporter
        </MenuItem>
      </CustomPopover>

      <CustomDateRangePicker
        open={isDateRangeOpen}
        onClose={closeDateRange}
        startDate={filters.startDate ? new Date(filters.startDate) : null}
        endDate={filters.endDate ? new Date(filters.endDate) : null}
        onChangeEndDate={(newValue: Date | null) => {
          if (newValue) {
            onChangeEndDate(newValue);
          }
        }}
        onChangeStartDate={(newValue: Date | null) => {
          if (newValue) {
            onChangeStartDate(newValue);
          }
        }}
      />
    </>
  );
}
