// ProductDetailsPage.tsx

"use client";

import { useState, useEffect, useCallback, useMemo } from "react";

import Tab from "@mui/material/Tab";
import Tabs from "@mui/material/Tabs";
import Grid from "@mui/material/Unstable_Grid2";
import Container from "@mui/material/Container";
import Button from "@mui/material/Button";

import { exportProductDetailsToCsv, useGetProduct } from "@/shared/api/product";
import { VariationDetailDTO } from "@/shared/types/product";

import { paths } from "@/routes/paths";
import RouterLink from "@/routes/router-link";

import Iconify from "@/shared/components/iconify";
import EmptyContent from "@/shared/components/empty-content";
import { useSettingsContext } from "@/shared/components/settings";
import { ProductDetailsSkeleton } from "./product-skeleton";
import ProductDetailsToolbar from "./product-details-toolbar";
import ProductDetailsCarousel from "./product-details-carousel";
import ProductDetailsSummary from "./product-details-summary";
import ProductDetailsDescription from "./product-details-description";

type Props = {
  id: string;
};

export default function ProductDetailsPage({ id }: Props) {
  const settings = useSettingsContext();

  const { product, productLoading, productError } = useGetProduct(id);

  const [selectedAttributes, setSelectedAttributes] = useState<{ [key: string]: string }>({});

  const [currentVariation, setCurrentVariation] = useState<VariationDetailDTO | null>(null);

  const [currentTab, setCurrentTab] = useState("description");

  const attributeOptions = useMemo(() => {
    if (product?.productType === "VARIABLE" && product.variations) {
      const options: { [key: string]: Set<string> } = {};

      product.variations.forEach((variation) => {
        variation.attributes.forEach((attr) => {
          if (!options[attr.attributeName]) {
            options[attr.attributeName] = new Set();
          }
          options[attr.attributeName].add(attr.attributeValue);
        });
      });

      const optionsObj: { [key: string]: string[] } = {};
      Object.keys(options).forEach((attrName) => {
        optionsObj[attrName] = Array.from(options[attrName]);
      });

      return optionsObj;
    }
    return {};
  }, [product]);

  useEffect(() => {
    if (
      product?.productType === "VARIABLE" &&
      Object.keys(attributeOptions).length > 0 &&
      Object.keys(selectedAttributes).length === Object.keys(attributeOptions).length
    ) {
      const variation = product.variations.find((variation) =>
        variation.attributes.every(
          (attr) => selectedAttributes[attr.attributeName] === attr.attributeValue
        )
      );
      if (currentVariation?.id !== variation?.id) {
        setCurrentVariation(variation || null);
      }
    } else {
      if (currentVariation !== null) {
        setCurrentVariation(null);
      }
    }
  }, [selectedAttributes, attributeOptions, product, currentVariation]);

  const handleChangeTab = useCallback(
    (event: React.SyntheticEvent, newValue: string) => {
      setCurrentTab(newValue);
    },
    []
  );

  const [publish, setPublish] = useState("");
  const handleChangePublish = useCallback((newValue: string) => {
    setPublish(newValue);
  }, []);
  const handleExport = useCallback(() => {
    if (!product) {
      console.error("Aucun produit disponible pour l'exportation.");
      return;
    }
  
    const filename = `product-${product?.name || 'unknown'}-${product?.ugs || 'unknown'}.csv`;
    exportProductDetailsToCsv(product, filename);
  }, [product]);
  
  if (productLoading) {
    return (
      <Container maxWidth={settings.themeStretch ? false : "lg"}>
        <ProductDetailsSkeleton />
      </Container>
    );
  }

  if (productError) {
    return (
      <Container maxWidth={settings.themeStretch ? false : "lg"}>
        <EmptyContent
          filled
          title={`${productError?.message}`}
          action={
            <Button
              component={RouterLink}
              href={paths.dashboard.admin.product.root}
              startIcon={<Iconify icon="eva:arrow-ios-back-fill" width={16} />}
              sx={{ mt: 3 }}
            >
            Retour à la liste
            </Button>
          }
          sx={{ py: 10 }}
        />
      </Container>
    );
  }

  return (
    <Container maxWidth={settings.themeStretch ? false : "lg"}>
      <ProductDetailsToolbar
        backLink={paths.dashboard.admin.product.root}
        editLink={product ? paths.dashboard.admin.product.edit(`${product.id}`) : ""}
        liveLink={""} 
        publish={publish}
        onChangePublish={handleChangePublish}
        onExport={handleExport}

      />

      <Grid container spacing={{ xs: 3, md: 5, lg: 8 }}>
        <Grid xs={12} md={6} lg={7}>
          {product && <ProductDetailsCarousel product={product} currentVariation={currentVariation} />}
        </Grid>
        <Grid xs={12} md={6} lg={5}>
          {product && (
            <ProductDetailsSummary
              product={product}
            currentVariation={currentVariation}
            attributeOptions={attributeOptions}
            selectedAttributes={selectedAttributes}
            setSelectedAttributes={setSelectedAttributes}
            />
          )}
          </Grid>
        </Grid>

        <Tabs
          value={currentTab}
        onChange={handleChangeTab}
        sx={{
          px: 3,
          boxShadow: (theme) => `inset 0 -2px 0 0 ${theme.palette.grey[500]}08`,
        }}
      >
        <Tab value="description" label="Description" />
      </Tabs>

      {currentTab === "description" && (
        <ProductDetailsDescription description={product?.description || ""} />
      )}
    </Container>
  );
}
