import { useEffect } from "react";
import { useForm } from "react-hook-form";
import Box from "@mui/material/Box";
import Stack from "@mui/material/Stack";
import Typography from "@mui/material/Typography";
import Divider from "@mui/material/Divider";
import Grid from "@mui/material/Unstable_Grid2";
import Label from "@/shared/components/label";
import FormProvider from "@/shared/components/hook-form";
import { fCurrency } from "@/utils/format-number";
import {
  AuthorizePreOrder,
  ProductDetailsDTO,
  VariationDetailDTO,
} from "@/shared/types/product";
import {
  FormControl,
  InputLabel,
  Select,
  MenuItem,
  Card,
  CardContent,
} from "@mui/material";

type Props = {
  product: ProductDetailsDTO;
  currentVariation?: VariationDetailDTO | null;
  attributeOptions?: { [key: string]: string[] };
  selectedAttributes?: { [key: string]: string };
  setSelectedAttributes?: React.Dispatch<
    React.SetStateAction<{ [key: string]: string }>
  >;
};

export default function ProductDetailsSummary({
  product,
  currentVariation,
  attributeOptions = {},
  selectedAttributes = {},
  setSelectedAttributes,
  ...other
}: Props) {
  const {
    id,
    name,
    shortDescription,
    productType,
    categoryNames,
    tagNames,
    material,
    emplacement,
    publishWeb,
    publishLive,
    regularPrice: productRegularPrice,
    stockWebQuantity: productStockWebQuantity,
    stockLiveQuantity: productStockLiveQuantity,
    supplierName,
    supplierRef,
    purchasePrice,
    stockLiveConfig: productstockLiveConfig,
    stockWebConfig: productstockWebConfig,
  } = product;

  const {
    regularPrice = productRegularPrice,
    stockWebQuantity = productStockWebQuantity,
    stockLiveQuantity = productStockLiveQuantity,
    stockLiveConfig = productstockLiveConfig,
    stockWebConfig = productstockWebConfig,
  } = currentVariation || {};

  const defaultValues = {
    id,
    name,
    regularPrice,
    stockWebQuantity,
    stockLiveQuantity,
    publishWeb,
    publishLive,
  };

  // Tableau d’options pour la précommande
  const ALLOW_BACKORDER_OPTIONS = [
    { value: AuthorizePreOrder.DO_NOT_AUTHORIZE, label: "Ne pas autoriser" },
    { value: AuthorizePreOrder.AUTHORIZE, label: "Autoriser" },
    {
      value: AuthorizePreOrder.AUTHORIZE_WITH_CUSTOMER_NOTIFICATION,
      label: "Autoriser, mais avec notification client",
    },
  ];

  const getAuthorizePreOrderLabel = (value: AuthorizePreOrder | undefined) => {
    if (!value) return "N/A";
    const option = ALLOW_BACKORDER_OPTIONS.find((opt) => opt.value === value);
    return option ? option.label : "N/A";
  };

  const methods = useForm({
    defaultValues,
  });

  // ---- Rendus individuels ----

  // Prix du produit
  const renderPrice = (
    <Box sx={{ typography: "h4", fontWeight: "bold" }}>
      {fCurrency(regularPrice)}
    </Box>
  );

  // Description du produit
  const renderSubDescription = (
    <Typography variant="body2" sx={{ color: "text.secondary" }}>
      {shortDescription}
    </Typography>
  );

  // Labels : statut de publication Web / Live
  const renderPublicationStatusLabels = (
    <Stack direction="row" spacing={1}>
      <Label color={publishWeb ? "success" : "error"}>
        {publishWeb ? "Publié sur le web" : "Non publié sur le web"}
      </Label>
      <Label color={publishLive ? "success" : "error"}>
        {publishLive ? "Publié en live" : "Non publié en live"}
      </Label>
    </Stack>
  );

  // Bloc : Quantités de stock
  const renderStockQuantities = (
    <Card variant="outlined">
      <CardContent>
        <Typography variant="subtitle1" sx={{ mb: 2, fontWeight: 600 }}>
          Gestion des Stocks
        </Typography>
        <Grid container spacing={2}>
          <Grid xs={12} md={6}>
            <Typography variant="subtitle2" sx={{ fontWeight: 500 }}>
              Stock Web
            </Typography>
            <Typography variant="body2" sx={{ mt: 1 }}>
              {stockWebQuantity || 0} articles disponibles
            </Typography>

            <Typography variant="body2">
              Préco :
              <strong> {getAuthorizePreOrderLabel(stockWebConfig.isPreOrderAuthorized)}</strong>
            </Typography>

            <Typography variant="body2">
              Seuil préco :{" "}
              <strong>
                {stockWebConfig.authorizedConfig == null
                  ? "N/A"
                  : stockWebConfig.authorizedConfig
                  ? "définie"
                  : "non définie"}
              </strong>
            </Typography>

            <Typography variant="body2">
              Valeur seuil préco :{" "}
              <strong>{stockWebConfig.thresholdValue || 0}</strong>
            </Typography>
          </Grid>

          {/* Stock Live */}
          <Grid xs={12} md={6}>
            <Typography variant="subtitle2" sx={{ fontWeight: 500 }}>
              Stock Live
            </Typography>
            <Typography variant="body2" sx={{ mt: 1 }}>
              {stockLiveQuantity || 0} articles disponibles
            </Typography>

            <Typography variant="body2">
              Préco :{" "}
              <strong>
                {getAuthorizePreOrderLabel(stockLiveConfig.isPreOrderAuthorized)}
              </strong>
            </Typography>

            <Typography variant="body2">
              Seuil préco :{" "}
              <strong>
                {stockLiveConfig.authorizedConfig == null
                  ? "N/A"
                  : stockLiveConfig.authorizedConfig
                  ? "définie"
                  : "non définie"}
              </strong>
            </Typography>

            <Typography variant="body2">
              Valeur seuil préco :{" "}
              <strong>{stockLiveConfig.thresholdValue || 0}</strong>
            </Typography>
          </Grid>
        </Grid>
      </CardContent>
    </Card>
  );

  const renderSupplierDetails = (
    <Card variant="outlined">
      <CardContent>
        <Typography variant="subtitle1" sx={{ mb: 2, fontWeight: 600 }}>
          Informations Fournisseur
        </Typography>
        <Grid container spacing={2}>
          <Grid xs={12} sm={6}>
            <Typography variant="body2" sx={{ fontWeight: 500 }}>
              Fournisseur :
            </Typography>
            <Typography variant="body2" sx={{ color: "text.secondary", mb: 2 }}>
              {supplierName || "N/A"}
            </Typography>
          </Grid>
          <Grid xs={12} sm={6}>
            <Typography variant="body2" sx={{ fontWeight: 500 }}>
              Réf Fournisseur :
            </Typography>
            <Typography variant="body2" sx={{ color: "text.secondary", mb: 2 }}>
              {supplierRef || "N/A"}
            </Typography>
          </Grid>
          <Grid xs={12} sm={6}>
            <Typography variant="body2" sx={{ fontWeight: 500 }}>
              Prix d’achat :
            </Typography>
            <Typography variant="body2" sx={{ color: "text.secondary" }}>
              {fCurrency(purchasePrice) || "N/A"}
            </Typography>
          </Grid>
        </Grid>
      </CardContent>
    </Card>
  );

  const renderProductDetails = (
    <Card variant="outlined">
      <CardContent>
        <Typography variant="subtitle1" sx={{ mb: 2, fontWeight: 600 }}>
          Détails du Produit
        </Typography>
        <Grid container spacing={2}>
          <Grid xs={12} sm={6}>
            <Typography variant="body2" sx={{ fontWeight: 500 }}>
              Matériau :
            </Typography>
            <Typography variant="body2" sx={{ color: "text.secondary", mb: 2 }}>
              {material || "N/A"}
            </Typography>
          </Grid>
          <Grid xs={12} sm={6}>
            <Typography variant="body2" sx={{ fontWeight: 500 }}>
              Emplacement :
            </Typography>
            <Typography variant="body2" sx={{ color: "text.secondary", mb: 2 }}>
              {emplacement || "N/A"}
            </Typography>
          </Grid>
          <Grid xs={12}>
            <Typography variant="body2" sx={{ fontWeight: 500 }}>
              Catégories :
            </Typography>
            <Typography variant="body2" sx={{ color: "text.secondary", mb: 2 }}>
              {categoryNames && categoryNames.length > 0
                ? categoryNames.join(", ")
                : "N/A"}
            </Typography>
          </Grid>
          <Grid xs={12}>
            <Typography variant="body2" sx={{ fontWeight: 500 }}>
              Étiquettes :
            </Typography>
            <Typography variant="body2" sx={{ color: "text.secondary" }}>
              {tagNames && tagNames.length > 0
                ? tagNames.join(", ")
                : "N/A"}
            </Typography>
          </Grid>
        </Grid>
      </CardContent>
    </Card>
  );

  return (
    <FormProvider methods={methods}>
      <Stack spacing={3} sx={{ pt: 3 }} {...other}>
        <Card variant="outlined">
          <CardContent>
            <Stack spacing={2}>
              {renderPublicationStatusLabels}
              <Typography variant="h5" sx={{ fontWeight: "bold" }}>
                {name}
              </Typography>
              {renderPrice}
              {renderSubDescription}
            </Stack>
          </CardContent>
        </Card>

        {productType === "VARIABLE" && attributeOptions && setSelectedAttributes && (
          <Card variant="outlined">
            <CardContent>
              <Typography variant="subtitle1" sx={{ mb: 2, fontWeight: 600 }}>
                Attributs de la variation
              </Typography>
              <Stack spacing={2}>
                {Object.keys(attributeOptions).map((attributeName) => (
                  <FormControl fullWidth key={attributeName}>
                    <InputLabel>{attributeName}</InputLabel>
                    <Select
                      label={attributeName}
                      value={selectedAttributes[attributeName] || ""}
                      onChange={(e) => {
                        setSelectedAttributes((prev) => ({
                          ...prev,
                          [attributeName]: e.target.value,
                        }));
                      }}
                    >
                      {attributeOptions[attributeName].map((value) => (
                        <MenuItem key={value} value={value}>
                          {value}
                        </MenuItem>
                      ))}
                    </Select>
                  </FormControl>
                ))}
              </Stack>
            </CardContent>
          </Card>
        )}

        {renderStockQuantities}

        {renderSupplierDetails}

        {renderProductDetails}
      </Stack>
    </FormProvider>
  );
}
