"use client";

import React, { useState, useCallback, useEffect } from "react";
import isEqual from "lodash/isEqual";
import {
  Card,
  Table,
  Button,
  Container,
  TableBody,
  TableContainer,
  Typography,
} from "@mui/material";

import { useRouter } from "@/hooks";
import { useSnackbar } from "@/shared/components/snackbar";
import { useSettingsContext } from "@/shared/components/settings";
import CustomBreadcrumbs from "@/shared/components/custom-breadcrumbs";
import {
  useTable,
  emptyRows,
  TableNoData,
  getComparator,
  TableEmptyRows,
  TableHeadCustom,
  TablePaginationCustom,
} from "@/shared/components/table";
import Scrollbar from "@/shared/components/scrollbar";
import Iconify from "@/shared/components/iconify";

import MigrationTableToolbar from "./migration-table-toolbar";
import MigrationTableRow from "./migration-table-row";
import MigrationTableFiltersResult from "./migration-table-filters-result";
import {
  IMigration,
  IMigrationTableFilters,
  IMigrationTableFilterValue,
} from "@/shared/types/migration";
import { useGetMigrationProducts, confirmMigration, triggerMigration } from "@/shared/api/product";

import { paths } from "@/routes/paths";

const TABLE_HEAD = [
  { id: "name", label: "Nom du produit" },
  { id: "migrationDate", label: "Date de génération", width: 180 },
  { id: "quantitySold", label: "Quantité vendue", width: 180 },
  { id: "remainingLiveStock", label: "Quantité restante", width: 180 },
  { id: "quantityToMigrate", label: "Quantité à migrer", width: 180 },
];

const defaultFilters: IMigrationTableFilters = {
  name: "",
};

export default function MigrationListView() {
  const { enqueueSnackbar } = useSnackbar();

  const table = useTable();

  const settings = useSettingsContext();

  const router = useRouter();

  const { migrationProducts, loading, error, mutate } = useGetMigrationProducts();

  const [tableData, setTableData] = useState<IMigration[]>([]);

  const [filters, setFilters] = useState<IMigrationTableFilters>(defaultFilters);

  const [triggerLoading, setTriggerLoading] = useState(false);

  useEffect(() => {
    if (migrationProducts) {
      setTableData(migrationProducts);
    }
  }, [migrationProducts]);

  const dataFiltered = applyFilter({
    inputData: tableData,
    comparator: getComparator(table.order, table.orderBy),
    filters,
  });

  const dataInPage = dataFiltered.slice(
    table.page * table.rowsPerPage,
    table.page * table.rowsPerPage + table.rowsPerPage
  );

  const denseHeight = table.dense ? 52 : 72;

  const canReset = !isEqual(defaultFilters, filters);

  const notFound = (!dataFiltered.length && canReset) || !dataFiltered.length;

  const handleFilters = useCallback(
    (name: string, value: IMigrationTableFilterValue) => {
      table.onResetPage();
      setFilters((prevState) => ({
        ...prevState,
        [name]: value,
      }));
    },
    [table]
  );

  const handleResetFilters = useCallback(() => {
    setFilters(defaultFilters);
  }, []);

  const handleUpdateQuantity = useCallback(
    (productId: number, newQuantityToMigrate: number) => {
      setTableData((prevData) =>
        prevData.map((item) =>
          item.productId === productId
            ? { ...item, quantityToMigrate: newQuantityToMigrate }
            : item
        )
      );
    },
    []
  );

  const handleConfirmMigration = async (product: IMigration) => {
    try {
      await confirmMigration([product]);
      setTableData((prevData) =>
        prevData.filter((item) => item.productId !== product.productId)
      );
      enqueueSnackbar("Migration confirmée avec succès!", { variant: "success" });
    } catch (error: any) {
      const errorMessage =
        error.response?.data || "Erreur lors de la confirmation de la migration";
      enqueueSnackbar(errorMessage, {
        variant: "error",
      });
    }
  };

  const handleTriggerMigration = async () => {
    setTriggerLoading(true);
    try {
      await triggerMigration();
      enqueueSnackbar("Migration lancée avec succès!", { variant: "success" });
      await mutate();
    } catch (error: any) {
      const errorMessage =
        error.response?.data || "Erreur lors du lancement de la migration";
      enqueueSnackbar(errorMessage, {
        variant: "error",
      });
    } finally {
      setTriggerLoading(false);
    }
  };

  return (
    <>
      <Container maxWidth={settings.themeStretch ? false : "lg"}>
        <CustomBreadcrumbs
          heading="Listes des migrations"
          links={[
            { name: "Tableau de bord", href: paths.dashboard.root },
            {
              name: "Migration des produits",
              href: paths.dashboard.admin.product.migration,
            },
            { name: "Lister" },
          ]}
          action={
            <Button
              variant="contained"
              startIcon={<Iconify icon="eva:play-circle-outline" />}
              onClick={handleTriggerMigration}
              disabled={triggerLoading}
            >
              {triggerLoading ? "Lancement..." : "Lancer la Migration"}
            </Button>
          }
          sx={{
            mb: { xs: 3, md: 5 },
          }}
        />

        <Typography variant="body1" sx={{ mb: 3 }}>
          Cette interface permet de migrer les produits vendus lors des sessions de live vers le
          stock web. L’administrateur peut confirmer le transfert des quantités restantes du stock
          live vers le stock web.
        </Typography>

        <Card>
          <MigrationTableToolbar filters={filters} onFilters={handleFilters} />

          {canReset && (
            <MigrationTableFiltersResult
              filters={filters}
              onFilters={handleFilters}
              onResetFilters={handleResetFilters}
              results={dataFiltered.length}
              sx={{ p: 2.5, pt: 0 }}
            />
          )}

          <TableContainer sx={{ position: "relative", overflow: "unset" }}>
            <Scrollbar>
              <Table
                size={table.dense ? "small" : "medium"}
                sx={{ minWidth: 960 }}
              >
                <TableHeadCustom
                  order={table.order}
                  orderBy={table.orderBy}
                  headLabel={TABLE_HEAD}
                  rowCount={dataFiltered.length}
                  numSelected={table.selected.length}
                  onSort={table.onSort}
                  //please lemine i want to keep it commented
                  // onSelectAllRows={(checked) =>
                  //   table.onSelectAllRows(
                  //     checked,
                  //     dataFiltered.map((row) => row.productId.toString())
                  //   )
                  // }
                />

                <TableBody>
                  {dataFiltered.map((row) => (
                    <MigrationTableRow
                      key={row.productId}
                      row={row}
                      selected={table.selected.includes(row.productId.toString())}
                      onSelectRow={() => table.onSelectRow(row.productId.toString())}
                      onUpdateQuantity={handleUpdateQuantity}
                      onConfirmMigration={handleConfirmMigration}
                    />
                  ))}

                  <TableEmptyRows
                    height={denseHeight}
                    emptyRows={emptyRows(
                      table.page,
                      table.rowsPerPage,
                      dataFiltered.length
                    )}
                  />

                  <TableNoData notFound={notFound} />
                </TableBody>
              </Table>
            </Scrollbar>
          </TableContainer>

          <TablePaginationCustom
            count={dataFiltered.length}
            page={table.page}
            rowsPerPage={table.rowsPerPage}
            onPageChange={table.onChangePage}
            onRowsPerPageChange={table.onChangeRowsPerPage}
            //
            dense={table.dense}
            onChangeDense={table.onChangeDense}
          />
        </Card>
      </Container>
    </>
  );
}

// ----------------------------------------------------------------------

function applyFilter({
  inputData,
  comparator,
  filters,
}: {
  inputData: IMigration[];
  comparator: (a: any, b: any) => number;
  filters: IMigrationTableFilters;
}) {
  const { name } = filters; 

  let data = [...inputData];

  if (name) {
    data = data.filter((item) =>
      item.name.toLowerCase().includes(name.toLowerCase())
    );
  }

  data.sort(comparator);

  return data;
}
