import React, { useState } from "react";
import TableRow from "@mui/material/TableRow";
import Checkbox from "@mui/material/Checkbox";
import TableCell from "@mui/material/TableCell";
import IconButton from "@mui/material/IconButton";
import ListItemText from "@mui/material/ListItemText";
import Button from "@mui/material/Button";
import { ConfirmDialog } from "@/shared/components/custom-dialog";
import { IMigration } from "@/shared/types/migration";
import AddIcon from "@mui/icons-material/Add";
import RemoveIcon from "@mui/icons-material/Remove";
import { fDate, fTime } from "@/utils/format-time";
import { ClearUrl } from "@/utils/cleanImageUrl";

type Props = {
  selected: boolean;
  row: IMigration;
  onSelectRow: VoidFunction;
  onUpdateQuantity: (productId: number, newQuantity: number) => void;
  onConfirmMigration: (product: IMigration) => Promise<void>;
};

export default function MigrationTableRow({
  row,
  selected,
  onSelectRow,
  onUpdateQuantity,
  onConfirmMigration,
}: Props) {
  const {
    productId,
    name,
    quantitySold,
    remainingLiveStock,
    quantityToMigrate,
    imagePath,
    migrationDate,
  } = row;

  const [quantity, setQuantity] = useState(quantityToMigrate);
  const [openConfirmDialog, setOpenConfirmDialog] = useState(false);
  const [loading, setLoading] = useState(false);

  const handleCheckBoxClick = (e: React.MouseEvent<HTMLButtonElement>) => {
    e.stopPropagation();
    onSelectRow();
  };

  const handleQuantityChange = (newQuantity: number) => {
    const clampedQuantity = Math.max(0, Math.min(remainingLiveStock, newQuantity));
    setQuantity(clampedQuantity);
    onUpdateQuantity(productId, clampedQuantity);
  };

  const handleConfirm = () => {
    setOpenConfirmDialog(true);
  };

  const handleConfirmClose = async (confirmed: boolean) => {
    setOpenConfirmDialog(false);
    if (confirmed) {
      setLoading(true);
      try {
        const updatedProduct = { ...row, quantityToMigrate: quantity };
        await onConfirmMigration(updatedProduct);
      } catch (error) {
        
      } finally {
        setLoading(false);
      }
    }
  };

  const cleanImagePath = ClearUrl(imagePath);

  return (
    <>
      <TableRow hover selected={selected} className="cursor-pointer">
        {/* <TableCell padding="checkbox">
          <Checkbox checked={selected} onClick={handleCheckBoxClick} />
        </TableCell> */}

        <TableCell sx={{ display: "flex", alignItems: "center" }}>
          <img
            src={cleanImagePath}
            alt={name}
            style={{ width: 50, height: 50, marginRight: 10 }}
          />
          <ListItemText
            primary={name}
            primaryTypographyProps={{ variant: "body2" }}
            secondaryTypographyProps={{
              component: "span",
              color: "text.disabled",
            }}
          />
        </TableCell>

        <TableCell sx={{ whiteSpace: "nowrap" }}>
          <ListItemText
            primary={fDate(migrationDate)}
            secondary={fTime(migrationDate)}
            primaryTypographyProps={{ typography: "body2", noWrap: true }}
            secondaryTypographyProps={{
              mt: 0.5,
              component: "span",
              typography: "caption",
            }}
          />
        </TableCell>

        <TableCell sx={{ whiteSpace: "nowrap" }}>{quantitySold}</TableCell>

        <TableCell sx={{ whiteSpace: "nowrap" }}>{remainingLiveStock}</TableCell>

        <TableCell
          sx={{ whiteSpace: "nowrap", display: "flex", alignItems: "center" }}
        >
          <IconButton onClick={() => handleQuantityChange(quantity - 1)}>
            <RemoveIcon />
          </IconButton>
          <input
            type="number"
            value={quantity}
            onChange={(e) => handleQuantityChange(Number(e.target.value))}
            min={0}
            max={remainingLiveStock}
            style={{ width: 50, textAlign: "center" }}
          />
          <IconButton onClick={() => handleQuantityChange(quantity + 1)}>
            <AddIcon />
          </IconButton>
          <Button
            variant="contained"
            color="primary"
            onClick={handleConfirm}
            sx={{ ml: 2 }}
            disabled={loading}
          >
            {loading ? "Confirmation..." : "Confirmer"}
          </Button>
          <ConfirmDialog
            open={openConfirmDialog}
            onClose={() => handleConfirmClose(false)}
            title="Confirmation"
            content="Voulez-vous confirmer la migration?"
            action={
              <Button onClick={() => handleConfirmClose(true)} color="primary">
                Confirmer
              </Button>
            }
          />
        </TableCell>
      </TableRow>
    </>
  );
}
