'use client';

import React, { useState, useMemo, useEffect } from 'react';
import TextField from '@mui/material/TextField';
import { Card, CardHeader, Checkbox, Grid, Stack, InputAdornment } from '@mui/material';
import MenuItem from '@mui/material/MenuItem';
import CustomBreadcrumbs from '@/components/custom-breadcrumbs/custom-breadcrumbs';
import { paths } from '@/routes/paths';
import { useRouter } from '@/hooks';
import { Autocomplete, Chip } from '@mui/material';
import Box from '@mui/material/Box';
import { useGetCategories } from '@/shared/api/categorie';
import { getPromoCodeById, addPromoCode, updatePromoCode, checkPromoCodeExists, checkPromoCodeIfUsed } from '@/shared/api/promoCode';
import { ProductPromoCode, PromoCodeType } from '@/contexts/types/promoCode';
import LoadingButton from '@mui/lab/LoadingButton';
import { useResponsive } from '@/hooks/use-responsive';
import Container from '@mui/material/Container';
import FormProvider from '@/components/hook-form';
import { yupResolver } from '@hookform/resolvers/yup';
import * as Yup from 'yup';
import { Controller, useForm } from 'react-hook-form';
import { useSettingsContext } from "@/components/settings";
import { DateTimePicker } from '@mui/x-date-pickers';
import { enqueueSnackbar, SnackbarProvider } from 'notistack';
import { ProductType } from '@/shared/types/product';
import PaginatedProductsDialog from './PaginatedProductsDialog';
import SelectedProductsList from './SelectedProductsList';
import SelectVariationsModal from './SelectVariationsModal';

export type ProductChoiceItem = {
  id: string;
  name: string;
  ugs: string;
  mainImagePath: string;
}

type Props = {
  id?: string;
  isEdit?: boolean;
  isDuplicate?: boolean;
}

export default function AddEditDuplicatePromoCodeView({ id, isEdit, isDuplicate }: Props) {

  const settings = useSettingsContext();
  const mdUp = useResponsive('up', 'md');
  const router = useRouter();
  const [selectedPromoCodeType, setSelectedPromoCodeType] = useState<PromoCodeType>(PromoCodeType.DISCOUNT_PER_CART);

  const NewPromoCodeSchema = Yup.object().shape({
    code: Yup.string().required('Code est requis'),
    description: Yup.string().notRequired(),
    usageLimit: Yup.number()
      .transform((value, originalValue) => (originalValue === "" ? 0 : value))
      .optional().nullable(),
    usageLimitPerClient: Yup.number()
      .moreThan(0, 'Limite d\'utilisation par client doit être supérieur à 0')
      .required('Limite d\'utilisation par client est requise'),
    minAmount: Yup.number().optional(),
    promoCodeType: Yup.string().required('Type de promo est requis'),
    startDate: Yup.date()
      .transform((value, originalValue) => {
        const today = new Date();
        return originalValue && new Date(originalValue) < today ? today : value;
      })
      .min(new Date(), 'La date de début doit être à jour ou une date future')
      .required('Date de début est requise'),
    expirationDate: Yup.date()
      .min(
        Yup.ref('startDate'),
        'La date d\'expiration doit être supérieure à la date de début'
      )
      .required('Date d\'expiration est requise'),
    uniquePromoCode: Yup.boolean().notRequired(),
    referral: Yup.boolean().notRequired(),
    productsIncludedIds: Yup.array().of(Yup.string()).notRequired(),
    productsExcludedIds: Yup.array().of(Yup.string()).notRequired(),
    variationsIncludedIds: Yup.array().of(Yup.string()).notRequired(),
    variationsExcludedIds: Yup.array().of(Yup.string()).notRequired(),
    categoriesIncludedIds: Yup.array().of(Yup.string()).notRequired(),
    categoriesExcludedIds: Yup.array().of(Yup.string()).notRequired(),
  });

  const SchemaPercentage = Yup.object().shape({
    valueCode: Yup.number()
      .required('La valeur de réduction en pourcentage est requise')
      .moreThan(0, 'La valeur doit être supérieure à 0%')
      .max(100, 'La valeur ne peut pas dépasser 100%')
      .test(
        'max-2-decimals',
        'La valeur ne peut pas avoir plus de 2 chiffres après la virgule',
        (value) => value !== undefined && /^\d+(\.\d{1,2})?$/.test(value.toString())
      ),
  });

  const SchemaDiscount = Yup.object().shape({
    valueCode: Yup.number()
      .required('Le montant de la réduction est requis')
      .moreThan(0, 'La valeur doit être supérieure à 0')
      .test(
        'max-2-decimals',
        'La valeur ne peut pas avoir plus de 2 chiffres après la virgule',
        (value) => value !== undefined && /^\d+(\.\d{1,2})?$/.test(value.toString())
      ),
  });

  const getNewSchema = (promoCodeType: PromoCodeType) => {
    const baseSchema = Yup.object().shape({
      ...NewPromoCodeSchema.fields,
    });

    const startDateField = isEdit && isEdit === true
      ? Yup.date().max(Yup.ref('expirationDate'), 'La date de début doit être inférieure à la date d\'expiration').required('Date de début est requise')
      : Yup.date()
        .transform((value, originalValue) => {
          const today = new Date();
          return originalValue && new Date(originalValue) < today ? today : value;
        })
        .min(new Date(), 'La date de début doit être à jour ou une date future')
        .required('Date de début est requise');

    if (promoCodeType === PromoCodeType.PERCENTAGE_PER_CART) {
      return baseSchema.shape({
        startDate: startDateField,
        ...SchemaPercentage.fields,
      });
    } else if (promoCodeType === PromoCodeType.DISCOUNT_PER_CART || promoCodeType === PromoCodeType.DISCOUNT_PER_PRODUCT) {
      return baseSchema.shape({
        startDate: startDateField,
        ...SchemaDiscount.fields,
      });
    }

    return baseSchema.shape({
      startDate: startDateField,
      ...SchemaPercentage.fields,
    });
  };

  const defaultValues = useMemo(() => ({
    code: '',
    valueCode: 0.00,
    description: '',
    usageLimit: 0,
    usageLimitPerClient: 0,
    minAmount: 0,
    promoCodeType: PromoCodeType.DISCOUNT_PER_CART,
    startDate: new Date(),
    expirationDate: new Date(),
    uniquePromoCode: false,
    productsIncludedIds: [],
    productsExcludedIds: [],
    variationsIncludedIds: [],
    variationsExcludedIds: [],
    categoriesIncludedIds: [],
    categoriesExcludedIds: [],
    referral: false,
  }), []);

  const methods = useForm({
    resolver: yupResolver(getNewSchema(selectedPromoCodeType)),
    defaultValues,
  });

  const {
    watch,
    control,
    setValue,
    handleSubmit,
    formState: { isSubmitting, errors },
  } = methods;

  const values = watch();

  const { promoCodeData } = getPromoCodeById(id!);

  useEffect(() => {
    if (isEdit || isDuplicate) {
      setSelectedPromoCodeType(promoCodeData?.promoCodeType || PromoCodeType.DISCOUNT_PER_CART);
      setValue("code", promoCodeData?.code || '');
      setValue("promoCodeType", promoCodeData?.promoCodeType || PromoCodeType.DISCOUNT_PER_CART);
      setValue("valueCode", promoCodeData?.valueCode || 0);
      setValue("usageLimit", promoCodeData?.usageLimit || 0);
      setValue("usageLimitPerClient", promoCodeData?.usageLimitPerClient || 0);
      setValue("minAmount", promoCodeData?.minAmount || 0);
      setValue("startDate", promoCodeData?.startDate ? new Date(promoCodeData.startDate) : new Date());
      setValue("expirationDate", promoCodeData?.expirationDate ? new Date(promoCodeData.expirationDate) : new Date());
      setValue("description", promoCodeData?.description || '');
      setValue("uniquePromoCode", promoCodeData?.uniquePromoCode || false);
      setValue("productsIncludedIds", promoCodeData?.productsIncludedIds || []);
      setValue("productsExcludedIds", promoCodeData?.productsExcludedIds || []);
      setValue("variationsIncludedIds", promoCodeData?.variationsIncludedIds || []);
      setValue("variationsExcludedIds", promoCodeData?.variationsExcludedIds || []);
      setProductsIncluded(promoCodeData?.productsIncluded || []);
      setProductsExcluded(promoCodeData?.productsExcluded || []);
      setVariationsIncluded(promoCodeData?.variationsIncluded || []);
      setVariationsExcluded(promoCodeData?.variationsExcluded || []);
      setValue("categoriesIncludedIds", promoCodeData?.categoriesIncludedIds || []);
      setValue("categoriesExcludedIds", promoCodeData?.categoriesExcludedIds || []);
      setValue("referral", promoCodeData?.referral || false);
    }
    if (isDuplicate) {
      setValue("code", promoCodeData?.code + "_duplicate");
    }
  }, [isEdit, isDuplicate, promoCodeData]);

  const { categoriesData: categories } = useGetCategories();

  const [productsIncluded, setProductsIncluded] = useState<ProductChoiceItem[]>([]);
  const [productsExcluded, setProductsExcluded] = useState<ProductChoiceItem[]>([]);
  const [variationsIncluded, setVariationsIncluded] = useState<ProductChoiceItem[]>([]);
  const [variationsExcluded, setVariationsExcluded] = useState<ProductChoiceItem[]>([]);
  const [variationsIncludedChoices, setVariationsIncludedChoices] = useState<ProductChoiceItem[]>([]);
  const [variationsExcludedChoices, setVariationsExcludedChoices] = useState<ProductChoiceItem[]>([]);
  const [variationsIncludedModal, setVariationsIncludedModal] = useState(false);
  const [variationsExcludedModal, setVariationsExcludedModal] = useState(false);

  const handleAddProductIncluded = (product: ProductPromoCode) => {
    if (product.productType === ProductType.SIMPLE) {
      const isAlreadyExcluded = productsExcluded.some(p => p.id === product.id);
      if (isAlreadyExcluded) {
        enqueueSnackbar("Le produit est déjà dans la liste des exclus. Veuillez le retirer avant de l'inclure.", { variant: "error" });
        return;
      }
      const isAlreadyIncluded = productsIncluded.some(p => p.id === product.id);
      if (!isAlreadyIncluded) {
        const newProductsIncluded = [...productsIncluded, product];
        setProductsIncluded(newProductsIncluded);
        setValue("productsIncludedIds", newProductsIncluded.map(p => p.id));
      }
    } else {
      const mappedVariations = product.variations.map(variation => ({
        id: variation.id,
        name: variation.name,
        ugs: variation.ugs,
        mainImagePath: variation.mainImagePath,
      }));
      setVariationsIncludedChoices((prev) => [...prev, ...mappedVariations]);
      setVariationsIncludedModal(true);
    }
  };

  const handleAddProductExcluded = (product: ProductPromoCode) => {
    if (product.productType === ProductType.SIMPLE) {
      const isAlreadyIncluded = productsIncluded.some(p => p.id === product.id);
      if (isAlreadyIncluded) {
        enqueueSnackbar("Le produit est déjà dans la liste des inclus. Veuillez le retirer avant de l'exclure.", { variant: "error" });
        return;
      }
      const isAlreadyExcluded = productsExcluded.some(p => p.id === product.id);
      if (!isAlreadyExcluded) {
        const newProductsExcluded = [...productsExcluded, product];
        setProductsExcluded(newProductsExcluded);
        setValue("productsExcludedIds", newProductsExcluded.map(p => p.id));
      }
    } else {
      const mappedVariations = product.variations.map(variation => ({
        id: variation.id,
        name: variation.name,
        ugs: variation.ugs,
        mainImagePath: variation.mainImagePath,
      }));
      setVariationsExcludedChoices((prev) => [...prev, ...mappedVariations]);
      setVariationsExcludedModal(true);
    }
  };

  const handleDeleteProductIncluded = (productId: string) => {
    const newProductsIncluded = productsIncluded.filter(p => p.id !== productId);
    setProductsIncluded(newProductsIncluded);
    setValue("productsIncludedIds", newProductsIncluded.map(p => p.id));
  };

  const handleDeleteProductExcluded = (productId: string) => {
    const newProductsExcluded = productsExcluded.filter(p => p.id !== productId);
    setProductsExcluded(newProductsExcluded);
    setValue("productsExcludedIds", newProductsExcluded.map(p => p.id));
  };

  const handleAddVariationIncluded = (variation: ProductChoiceItem) => {
    const isAlreadyExcluded = variationsExcluded.some(v => v.id === variation.id);
    if (isAlreadyExcluded) {
      enqueueSnackbar("La variation est déjà dans la liste des exclus. Veuillez la retirer avant de l'inclure.", { variant: "error" });
      return;
    }
    const isAlreadyIncluded = variationsIncluded.some(v => v.id === variation.id);
    if (!isAlreadyIncluded) {
      const newVariationsIncluded = [...variationsIncluded, variation];
      setVariationsIncluded(newVariationsIncluded);
      setValue("variationsIncludedIds", newVariationsIncluded.map(v => v.id));
    }
  };

  const handleAddVariationExcluded = (variation: ProductChoiceItem) => {
    const isAlreadyIncluded = variationsIncluded.some(v => v.id === variation.id);
    if (isAlreadyIncluded) {
      enqueueSnackbar("La variation est déjà dans la liste des inclus. Veuillez la retirer avant de l'exclure.", { variant: "error" });
      return;
    }
    const isAlreadyExcluded = variationsExcluded.some(v => v.id === variation.id);
    if (!isAlreadyExcluded) {
      const newVariationsExcluded = [...variationsExcluded, variation];
      setVariationsExcluded(newVariationsExcluded);
      setValue("variationsExcludedIds", newVariationsExcluded.map(v => v.id));
    }
  };

  const handleDeleteVariationIncluded = (variationId: string) => {
    const newVariationsIncluded = variationsIncluded.filter(v => v.id !== variationId);
    setVariationsIncluded(newVariationsIncluded);
    setValue("variationsIncludedIds", newVariationsIncluded.map(v => v.id));
  };

  const handleDeleteVariationExcluded = (variationId: string) => {
    const newVariationsExcluded = variationsExcluded.filter(v => v.id !== variationId);
    setVariationsExcluded(newVariationsExcluded);
    setValue("variationsExcludedIds", newVariationsExcluded.map(v => v.id));
  };

  const onSubmit = handleSubmit(async (data: any) => {
    if (!isEdit) {
      const checkValue = await checkPromoCodeExists(data.code);
      if (checkValue) {
        enqueueSnackbar("Code promo déjà existant", { variant: "error" });
        return;
      }
    }

    if (isEdit) {
      const checkPromoCodeUsage = await checkPromoCodeIfUsed(data.code);
      if (checkPromoCodeUsage) {
        enqueueSnackbar("Ce code promo ne peut pas être modifié car il a déjà été utilisé.", { variant: "error" });
        return;
      }
    }

    data.startDate = new Date(new Date(data.startDate).getTime() - new Date(data.startDate).getTimezoneOffset() * 60000).toISOString();
    data.expirationDate = new Date(new Date(data.expirationDate).getTime() - new Date(data.expirationDate).getTimezoneOffset() * 60000).toISOString();
    if (isEdit && !isDuplicate) {
      await updatePromoCode(id!, { ...data }).then(() => {
        enqueueSnackbar("Code promo modifié avec succès", { variant: "success" });
        router.push(paths.dashboard.admin.promoCode.root);
      }).catch(() => {
        enqueueSnackbar("Échec de la modification du code promo", { variant: "error" });
      });
    } else if (!isEdit && !isDuplicate) {
      await addPromoCode({ ...data }).then(() => {
        enqueueSnackbar("Code promo ajouté avec succès", { variant: "success" });
        router.push(paths.dashboard.admin.promoCode.root);
      }).catch(() => {
        enqueueSnackbar("Échec de l'ajout du code promo", { variant: "error" });
      });
    } else if (isDuplicate && !isEdit) {
      await addPromoCode({ ...data }).then(() => {
        enqueueSnackbar("Code promo dupliqué avec succès", { variant: "success" });
        router.push(paths.dashboard.admin.promoCode.root);
      }).catch(() => {
        enqueueSnackbar("Échec de l'ajout du code promo", { variant: "error" });
      });
    }
  });

  const renderValueCodeField = () => {
    const handleValueChange = (event: any) => {
      const { value } = event.target;
      const formattedValue = parseFloat(value).toFixed(2);
      setValue('valueCode', Number(formattedValue));
    };

    if (values.promoCodeType === PromoCodeType.PERCENTAGE_PER_CART) {
      return (
        <>
          <Controller
            name="valueCode"
            control={control}
            render={({ field }) => (
              <TextField
                {...field}
                id="outlined-value-code"
                label="Pourcentage de réduction (%)"
                type="number"
                placeholder="Saisir le pourcentage de réduction"
                variant="outlined"
                fullWidth
                error={!!errors.valueCode}
                helperText={errors.valueCode?.message}
                onChange={handleValueChange}
                InputProps={{
                  endAdornment: <InputAdornment position="end">%</InputAdornment>,
                }}
                inputProps={{
                  step: 0.01,
                }}
              />
            )}
          />
        </>
      );
    } else if (
      values.promoCodeType === PromoCodeType.DISCOUNT_PER_CART ||
      values.promoCodeType === PromoCodeType.DISCOUNT_PER_PRODUCT
    ) {
      return (
        <Controller
          name="valueCode"
          control={control}
          render={({ field }) => (
            <TextField
              {...field}
              id="outlined-value-code"
              label="Montant de réduction (€)"
              type="number"
              placeholder="0"
              fullWidth
              error={!!errors.valueCode}
              helperText={errors.valueCode?.message}
              value={values.valueCode}
              onChange={handleValueChange}
              InputProps={{
                endAdornment: <InputAdornment position="end">€</InputAdornment>,
              }}
              inputProps={{
                step: 0.01,
              }}
            />
          )}
        />
      );
    }
    return null;
  };


  return (
    <SnackbarProvider>
      <Container maxWidth={settings.themeStretch ? false : "lg"}>
        <Grid md={12}>
          <CustomBreadcrumbs
            heading={isEdit ? "Modifier un code promo: " + promoCodeData?.code : isDuplicate ? "Dupliquer un code promo: " + promoCodeData?.code : "Ajouter un code promo"}
            links={[
              { name: 'Tableau de bord', href: paths.dashboard.root },
              { name: "Codes promo", href: paths.dashboard.admin.promoCode.root },
              { name: isEdit ? "Modifier un code promo" : isDuplicate ? "Dupliquer un code promo" : "Ajouter un code promo" },
            ]}
            rollbackLink={paths.dashboard.admin.promoCode.root}
            sx={{
              mb: { xs: 3, md: 5 },
            }}
          />
        </Grid>

        <FormProvider methods={methods} onSubmit={onSubmit}>
          <Grid container spacing={3}>
            <Grid xs={12} md={12} sx={{ mt: 3 }}>
              <Card>
                <CardHeader title="Informations générales" />
                <Stack spacing={3} sx={{ p: 3 }}>
                  <Grid container spacing={3}>
                    <Grid item xs={12} md={3}>
                      <Controller
                        name="code"
                        control={control}
                        render={({ field }) => (
                          <TextField
                            {...field}
                            label="Code promo"
                            type="search"
                            placeholder="Saisir le code promo"
                            variant="outlined"
                            fullWidth
                            error={!!errors.code}
                            helperText={errors.code?.message}
                            value={field.value || ""}
                            onChange={(e) => {
                              const lowercaseValue = e.target.value.toLowerCase();
                              field.onChange(lowercaseValue);
                            }}
                          />
                        )}
                      />
                    </Grid>
                    <Grid item xs={12} md={3}>
                      <Controller
                        name="promoCodeType"
                        control={control}
                        render={({ field }) => (
                          <TextField
                            {...field}
                            id="outlined-type-code"
                            select
                            label="Type de promo"
                            value={values.promoCodeType}
                            variant="outlined"
                            required
                            name="promoCodeType"
                            fullWidth
                            error={!!errors.promoCodeType}
                            helperText={errors.promoCodeType?.message}
                            onChange={(event) => {
                              const newValue = event.target.value;
                              field.onChange(newValue);
                              setSelectedPromoCodeType(newValue as PromoCodeType);
                            }}
                          >
                            <MenuItem value={PromoCodeType.DISCOUNT_PER_CART}>Remise sur Panier</MenuItem>
                            <MenuItem value={PromoCodeType.DISCOUNT_PER_PRODUCT}>Remise sur Produit</MenuItem>
                            <MenuItem value={PromoCodeType.PERCENTAGE_PER_CART}>Pourcentage sur Panier</MenuItem>
                          </TextField>
                        )}
                      />
                    </Grid>
                    <Grid item xs={12} md={3}>
                      {renderValueCodeField()}
                    </Grid>
                    <Grid item xs={12} md={3}>
                      <Controller
                        name="usageLimitPerClient"
                        control={control}
                        render={({ field }) => (
                          <TextField
                            {...field}
                            id="outlined-usage-limit-per-client"
                            label="Limite d'utilisation par client"
                            type="number"
                            placeholder="0"
                            value={values.usageLimitPerClient}
                            variant="outlined"
                            name="usageLimitPerClient"
                            fullWidth
                            error={!!errors.usageLimitPerClient}
                            helperText={errors.usageLimitPerClient?.message}
                          />
                        )}
                      />
                    </Grid>
                    <Grid item xs={12} md={2}>
                      <Controller
                        name="usageLimit"
                        control={control}
                        render={({ field }) => (
                          <TextField
                            {...field}
                            id="outlined-usage-limit"
                            label="Limite d'utilisation"
                            type="number"
                            placeholder="0"
                            value={values.usageLimit}
                            variant="outlined"
                            name="usageLimit"
                            fullWidth
                            error={!!errors.usageLimit}
                            helperText={errors.usageLimit?.message}
                          />
                        )}
                      />
                    </Grid>
                    <Grid item xs={12} md={2}>
                      <Controller
                        name="minAmount"
                        control={control}
                        render={({ field }) => (
                          <TextField
                            {...field}
                            id="outlined-min-amount"
                            label="Montant minimum"
                            type="number"
                            placeholder="0"
                            value={values.minAmount}
                            variant="outlined"
                            name="minAmount"
                            fullWidth
                            error={!!errors.minAmount}
                            helperText={errors.minAmount?.message}
                          />
                        )}
                      />
                    </Grid>
                    <Grid item xs={12} md={4}>
                      <Controller
                        name="startDate"
                        control={control}
                        render={({ field, fieldState: { error } }) => (
                          <DateTimePicker
                            label="Date et heure de début du code promo"
                            value={field.value}
                            onChange={(newValue) => {
                              field.onChange(newValue);
                            }}
                            disabled={isEdit && isEdit === true}
                            format="yyyy/MM/dd HH:mm:ss"
                            slotProps={{
                              textField: {
                                fullWidth: true,
                                error: !!error,
                                helperText: error?.message,
                              },
                            }}
                          />
                        )}
                      />
                    </Grid>
                    <Grid item xs={12} md={4}>
                      <Controller
                        name="expirationDate"
                        control={control}
                        render={({ field, fieldState: { error } }) => (
                          <DateTimePicker
                            label="Date et heure d'expiration du code promo"
                            value={field.value}
                            onChange={(newValue) => {
                              field.onChange(newValue);
                            }}
                            format="yyyy/MM/dd HH:mm:ss"
                            slotProps={{
                              textField: {
                                fullWidth: true,
                                error: !!error,
                                helperText: error?.message,
                              },
                            }}
                          />
                        )}
                      />
                    </Grid>
                    <Grid item xs={12} md={12}>
                      <Controller
                        name="description"
                        control={control}
                        render={({ field }) => (
                          <TextField
                            {...field}
                            id="outlined-description"
                            label="Description"
                            multiline
                            rows={2}
                            value={values.description}
                            variant="outlined"
                            placeholder="Saisissez votre description ici..."
                            name="description"
                            fullWidth
                            error={!!errors.description}
                            helperText={errors.description?.message}
                          />
                        )}
                      />
                    </Grid>

                    <Grid item xs={12} md={12}>
                      <Box sx={{ display: 'flex', alignItems: 'center' }}>
                        <Controller
                          name="uniquePromoCode"
                          control={control}
                          render={({ field }) => (
                            <>
                              <Checkbox
                                {...field}
                                checked={!!field.value}
                                size="small"
                                onChange={(e) => field.onChange(e.target.checked)}
                              />
                              <span>
                                Cochez cette case si le code promo ne peut être utilisé conjointement avec d’autres codes promo.
                              </span>
                            </>
                          )}
                        />
                      </Box>
                    </Grid>
                    <Grid item xs={12} md={12}>
                      <Box sx={{ display: 'flex', alignItems: 'center' }}>
                        <Controller
                          name="referral"
                          control={control}
                          render={({ field }) => (
                            <>
                              <Checkbox
                                {...field}
                                checked={!!field.value}
                                size="small"
                                onChange={(e) => field.onChange(e.target.checked)}
                              />
                              <span>
                                Cochez cette case si le code promo et de parrainage.
                              </span>
                            </>
                          )}
                        />
                      </Box>
                    </Grid>

                  </Grid>
                </Stack>
              </Card>
            </Grid>

            <Grid xs={12} md={12} sx={{ mt: 3 }}>
              <Card>
                <CardHeader title="Restriction d'usage" />
                <Grid container spacing={3} sx={{ mt: 1, p: 3 }}>
                  <Grid item xs={6} md={6}>
                    <PaginatedProductsDialog title="Choisir les produits inclus" onAdd={handleAddProductIncluded} />
                  </Grid>
                  <Grid item xs={6} md={6}>
                    <PaginatedProductsDialog title="Choisir les produits exclus" onAdd={handleAddProductExcluded} />
                  </Grid>
                </Grid>
                <Stack spacing={3} sx={{ p: 3 }}>
                  <Grid container spacing={3}>
                    <Grid item xs={6} md={3} sx={{ mt: 1 }}>
                      <h3>Produits inclus</h3>
                      <SelectedProductsList selectedProducts={productsIncluded} onRemove={handleDeleteProductIncluded} />
                    </Grid>

                    <Grid item xs={6} md={3} sx={{ mt: 1 }}>
                      <h3>Variations incluses</h3>
                      <SelectedProductsList selectedProducts={variationsIncluded} onRemove={handleDeleteVariationIncluded} />
                    </Grid>

                    <Grid item xs={6} md={3} sx={{ mt: 1 }}>
                      <h3>Produits exclus</h3>
                      <SelectedProductsList selectedProducts={productsExcluded} onRemove={handleDeleteProductExcluded} />
                    </Grid>

                    <Grid item xs={6} md={3} sx={{ mt: 1 }}>
                      <h3>Variations excluses</h3>
                      <SelectedProductsList selectedProducts={variationsExcluded} onRemove={handleDeleteVariationExcluded} />
                    </Grid>

                    <Grid item xs={12} md={6}>
                      <Controller
                        name="categoriesIncludedIds"
                        control={control}
                        render={({ field }) => (
                          <Autocomplete
                            {...field}
                            multiple
                            id="tags-filled-categories"
                            options={categories}
                            getOptionLabel={(option) => option.nom}
                            onChange={(_, selectedCategories) => {
                              const excludedCategories: string[] = values.categoriesExcludedIds as string[];
                              const categoriesToAdd = selectedCategories.filter((category: any) => {
                                if (excludedCategories.includes(category.id)) {
                                  enqueueSnackbar(`La catégorie ${category.nom} est déjà exclue.`, { variant: 'warning' });
                                  return false;
                                }
                                return true;
                              });
                              field.onChange(categoriesToAdd.map((category: any) => category.id));
                            }}
                            value={categories.filter((category: any) => (field.value as any[])?.includes(category.id))}
                            renderTags={(tagValue, getTagProps) =>
                              tagValue.map((option, index) => (
                                <Chip variant="outlined" label={option.nom} {...getTagProps({ index })} key={index} />
                              ))
                            }
                            renderInput={(params) => (
                              <TextField
                                {...params}
                                variant="outlined"
                                label="Catégories inclus"
                                placeholder="Choisir une catégorie"
                                fullWidth
                              />
                            )}
                            renderOption={(props, option) => (
                              <Box component="li" {...props}>
                                <img
                                  loading="lazy"
                                  width="40"
                                  src={option.image ? option.image : ""}
                                  alt={option.nom}
                                  style={{ marginRight: '10px' }}
                                />
                                {`${option.nom}`}
                              </Box>
                            )}
                          />
                        )}
                      />
                    </Grid>

                    <Grid item xs={12} md={6}>
                      <Controller
                        name="categoriesExcludedIds"
                        control={control}
                        render={({ field }) => (
                          <Autocomplete
                            {...field}
                            multiple
                            id="tags-filled-categories-excluded"
                            options={categories}
                            getOptionLabel={(option) => option.nom}
                            onChange={(_, selectedCategories) => {
                              const includedCategories: string[] = values.categoriesIncludedIds as string[];
                              const categoriesToAdd = selectedCategories.filter((category: any) => {
                                if (includedCategories.includes(category.id)) {
                                  enqueueSnackbar(`La catégorie ${category.nom} est déjà inclue.`, { variant: 'warning' });
                                  return false;
                                }
                                return true;
                              });
                              field.onChange(categoriesToAdd.map((category: any) => category.id));
                            }}
                            value={categories.filter((category: any) => (field.value as any[])?.includes(category.id))}
                            renderTags={(tagValue, getTagProps) =>
                              tagValue.map((option, index) => (
                                <Chip variant="outlined" label={option.nom} {...getTagProps({ index })} key={index} />
                              ))
                            }
                            renderInput={(params) => (
                              <TextField
                                {...params}
                                variant="outlined"
                                label="Catégories exclues"
                                placeholder="Choisir une catégorie"
                                fullWidth
                              />
                            )}
                            renderOption={(props, option) => (
                              <Box component="li" {...props}>
                                <img
                                  loading="lazy"
                                  width="40"
                                  src={option.image ? option.image : ""}
                                  alt={option.nom}
                                  style={{ marginRight: '10px' }}
                                />
                                {`${option.nom}`}
                              </Box>
                            )}
                          />
                        )}
                      />
                    </Grid>
                  </Grid>
                </Stack>
              </Card>
            </Grid>

            <SelectVariationsModal
              title="Choisir les variations incluses"
              variationsModal={variationsIncludedModal}
              setVariationsModal={setVariationsIncludedModal}
              variationsChoices={variationsIncludedChoices}
              setVariationsChoices={setVariationsIncludedChoices}
              selectedVariations={variationsIncluded}
              setSelectedVariations={setVariationsIncluded}
              handleAddSelectedVariation={handleAddVariationIncluded}
            />

            <SelectVariationsModal
              title="Choisir les variations exclues"
              variationsModal={variationsExcludedModal}
              setVariationsModal={setVariationsExcludedModal}
              variationsChoices={variationsExcludedChoices}
              setVariationsChoices={setVariationsExcludedChoices}
              selectedVariations={variationsExcluded}
              setSelectedVariations={setVariationsExcluded}
              handleAddSelectedVariation={handleAddVariationExcluded}
            />

            {mdUp && <Grid md={4} />}
            <Grid xs={12} md={8} sx={{ display: 'flex', justifyContent: 'end', py: 4 }}>
              <LoadingButton
                type="submit"
                variant="contained"
                size="large"
                loading={isSubmitting}
                sx={{ ml: 2 }}
              >
                Enregistrer
              </LoadingButton>
            </Grid>
          </Grid>
        </FormProvider>
      </Container>
    </SnackbarProvider>
  );
}
