import React, { useEffect, useState } from 'react';
import {Card, CardContent, CardMedia, Typography, Box, Button, Dialog, Pagination, CircularProgress, Grid, MenuItem, Select, 
  FormControl, SelectChangeEvent,TextField,} from '@mui/material';
import { getPaginatedProductsPromoCodes } from '@/shared/api/promoCode';
import { ProductPromoCode } from '@/contexts/types/promoCode';
import { DEFAULT_IMAGE_URL } from '@/config-global';

interface PaginatedProductsDialogProps {
  onAdd: (product: ProductPromoCode) => void;
  title: string;
}

const PaginatedProductsDialog: React.FC<PaginatedProductsDialogProps> = ({ onAdd, title }) => {
  const [items, setItems] = useState<ProductPromoCode[]>([]);
  const [open, setOpen] = useState(false);
  const [page, setPage] = useState(0);
  const [size, setSize] = useState(12);
  const [keyword, setKeyword] = useState('');

  const { promoCodesData, promoCodesLoading, totalPages } = getPaginatedProductsPromoCodes(page, size, keyword);

  useEffect(() => {
    if (open) {
      setItems(promoCodesData);
    }
  }, [open, promoCodesData]);

  const handlePageChange = (event: React.ChangeEvent<unknown>, value: number) => {
    setPage(value - 1);
  };

  const handleSizeChange = (event: SelectChangeEvent<number>) => {
    setSize(event.target.value as number);
    setPage(0);
  };

  const handleKeywordChange = (event: React.ChangeEvent<HTMLInputElement>) => {
    const value = event.target.value.trim();

    if (value === '') {
      setKeyword('');
      setPage(0);
    } else if (value.length >= 3) {
      setKeyword(value);
      setPage(0);
    }
  };

  const isValidImagePath = (path: string): string => {
    const img = new Image();
    img.src = path;
    return img.complete && img.naturalWidth !== 0 ? path : DEFAULT_IMAGE_URL!;
  };

  return (
    <Box>
      <Button variant="outlined" onClick={() => setOpen(true)} fullWidth>{title}</Button>
      <Dialog open={open} onClose={() => setOpen(false)} maxWidth="md" fullWidth>
        <Typography variant="h6" sx={{ flex: 1, textAlign: 'center', mt: 2 }}>
          {title}
        </Typography>
        <Box
          sx={{
            position: 'relative',
            p: 2,
            display: 'flex',
            justifyContent: 'space-between',
            alignItems: 'center',
            width: '100%',
          }}
        >
          <TextField
            label="Rechercher"
            fullWidth
            sx={{
              flex: 1,
              width: '70%',
            }}
            onChange={handleKeywordChange}
          />
          <FormControl
            sx={{
              minWidth: 80,
              maxWidth: 80,
              mt: 0,
              ml: 2,
            }}
          >
            <Select value={size} onChange={handleSizeChange}>
              {[12, 24, 36].map((sizeOption) => (
                <MenuItem key={sizeOption} value={sizeOption}>
                  {sizeOption}
                </MenuItem>
              ))}
            </Select>
          </FormControl>
        </Box>

        <Box sx={{ p: 2 }}>
          {promoCodesLoading ? (
            <Grid container justifyContent="center">
              <CircularProgress />
            </Grid>
          ) : (
            <>
              <Grid container spacing={2}>
                {items.length === 0 ? (
                  <Grid item xs={12} sx={{ display: 'flex', justifyContent: 'center', marginTop: 2, marginBottom: 2 }}>
                    <Typography variant="body1" color="text.secondary">Aucun produit trouvé</Typography>
                  </Grid>
                ) : (
                  items.map((item) => (
                    <Grid item xs={12} sm={6} md={4} key={item.id}>
                    <Card
                      onClick={() => {
                        setOpen(false);
                        onAdd(item);
                      }}
                      sx={{ cursor: 'pointer', position: 'relative', display: 'flex', flexDirection: 'column', height: '100%' }}
                    >
                      {item.mainImagePath && (
                        <CardMedia
                          component="img"
                          height="140"
                          image={isValidImagePath(item.mainImagePath)}
                          alt={item.name}
                        />
                      )}
                      <CardContent>
                        <Typography variant="h6">{item.name}</Typography>
                        <Typography color="text.secondary">{item.ugs}</Typography>
                      </CardContent>
                    </Card>
                    </Grid>
                  ))
                )}
              </Grid>
              <Pagination
                count={totalPages}
                page={page + 1}
                onChange={handlePageChange}
                color="primary"
                sx={{ display: 'flex', justifyContent: 'center', marginTop: 2 }}
              />
            </>
          )}
        </Box>
      </Dialog>
    </Box>
  );
};

export default PaginatedProductsDialog;
