"use client";

import Card from "@mui/material/Card";
import Container from "@mui/material/Container";
import { useSettingsContext } from "@/shared/components/settings";
import { paths } from "@/routes/paths";
import Label from "@/shared/components/label";
import CustomBreadcrumbs from "@/shared/components/custom-breadcrumbs";
import { useEffect, useState } from "react";
import { Alert, Box, Button, CircularProgress, Divider, Grid, Stack, Typography } from "@mui/material";
import { fDate, fTime } from "@/utils/format-time";
import { useResponsive } from "@/hooks/use-responsive";
import { fCurrency } from "@/utils/format-number";
import { CollectionStatus, ReceiptStatus } from "@/contexts/types/supplierOrder";
import { SnackbarProvider } from "notistack";
import { getPromoCodeTypeColor, PromoCodeItem, PromoCodeType, promoCodeTypeLabels } from "@/contexts/types/promoCode";
import { getPromoCodeById } from "@/shared/api/promoCode";
import PromoCodeCardProductsList from "./SelectedProductsList";
import { ICategory } from "@/shared/types/category";
import ModeEditIcon from '@mui/icons-material/ModeEdit';
import ContentCopyIcon from '@mui/icons-material/ContentCopy';
import { useRouter } from "@/hooks";

type Props = {
    promoCodeId: string;
}

export default function PromoCodeDetailsView({ promoCodeId }: Props) {

    const settings = useSettingsContext();
    const lgUp = useResponsive("up", "lg");
    const mdUp = useResponsive("up", "md");
    const router = useRouter();

    const [promoCode, setPromoCode] = useState<PromoCodeItem>();

    const { promoCodeData, promoCodeLoading, promoCodeError } = getPromoCodeById(promoCodeId);

    useEffect(() => {
        setPromoCode(promoCodeData);
    }, [promoCodeData]);

    const handleEditPromoCode = ((promoCodeId: string) => {
        router.push(paths.dashboard.admin.promoCode.edit(promoCodeId));
    });

    const handleDuplicatePromoCode = ((promoCodeId: string) => {
        router.push(paths.dashboard.admin.promoCode.duplicate(promoCodeId));
    });

    if (promoCodeLoading) return (
        <Stack alignItems="center" justifyContent="center" sx={{ py: 5 }}>
            <CircularProgress />
        </Stack>
    )

    if (promoCodeError) {
        return (
            <Container maxWidth={settings.themeStretch ? false : 'lg'}>
                <Alert severity="error">Problème lors de la récupération des données du code promo. Veuillez réessayer plus tard.</Alert>
            </Container>
        );
    }

    return (
        <SnackbarProvider>
            <Container maxWidth={settings.themeStretch ? false : "lg"}>
                <CustomBreadcrumbs
                    heading={`Détail Code promo: ${promoCode?.code}`}
                    links={[
                        { name: "Tableau de bord", href: paths.dashboard.root },
                        { name: "Codes promos", href: paths.dashboard.admin.promoCode.root },
                        { name: "Détail Code promo" },
                    ]}
                    sx={{
                        mb: { xs: 3, md: 5 },
                    }}
                />

                <Box sx={{ display: 'flex', justifyContent: 'flex-end', mb: 2, mt: 2 }}>
                    <Button variant="outlined" color="primary" sx={{ mr: 2 }} startIcon={<ModeEditIcon />} onClick={() => handleEditPromoCode(promoCodeId)}> Modifier </Button>
                    <Button variant="outlined" color="warning" startIcon={<ContentCopyIcon />} onClick={() => handleDuplicatePromoCode(promoCodeId)}> Dupliquer </Button>
                </Box>

                <Card className="p-3" >


                    <Grid component="div" spacing={{ xs: 4 }} container p={0} py={2} justifyContent={'space-evenly'}>

                        <Grid item lg={2} md={3} sm={4} container justifyContent={mdUp ? 'center' : 'start'} alignItems='center'>
                            <p>
                                <Typography
                                    variant="h6"
                                    sx={{ color: "text.disabled", flexGrow: 1 }}
                                >
                                    Type
                                </Typography>
                                <Label
                                    variant="soft"
                                    color={getPromoCodeTypeColor(promoCode?.promoCodeType || PromoCodeType.DISCOUNT_PER_CART)}
                                >
                                    {promoCode?.promoCodeType
                                        ? promoCodeTypeLabels[promoCode.promoCodeType]
                                        : "Type inconnu"}
                                </Label>
                            </p>
                        </Grid>

                        <Divider orientation={mdUp ? 'vertical' : 'horizontal'} flexItem style={{ borderStyle: 'dashed', margin: "16px 0" }} />

                        <Grid item lg={2} md={2} sm={4} container justifyContent={mdUp ? 'center' : 'start'} alignItems='center'>
                            <p>
                                <Typography
                                    variant="h6"
                                    sx={{ color: "text.disabled", flexGrow: 1 }}
                                >
                                    Valeur
                                </Typography>
                                {promoCode?.promoCodeType === PromoCodeType.DISCOUNT_PER_CART || promoCode?.promoCodeType === PromoCodeType.DISCOUNT_PER_PRODUCT ? (
                                    <span>{fCurrency(promoCode?.valueCode || 0)}</span>
                                ) : promoCode?.promoCodeType === PromoCodeType.PERCENTAGE_PER_CART ? (
                                    <span>{promoCode?.valueCode}%</span>
                                ) : (
                                    <></>
                                )}
                            </p>
                        </Grid>

                        <Divider orientation={mdUp ? 'vertical' : 'horizontal'} flexItem style={{ borderStyle: 'dashed', margin: "16px 0" }} />

                        <Grid item lg={2} md={2} sm={4} container justifyContent={mdUp ? 'center' : 'start'} alignItems='center'>
                            <p>
                                <Typography
                                    variant="h6"
                                    sx={{ color: "text.disabled", flexGrow: 1 }}
                                >
                                    Usage
                                </Typography>
                                <span>{promoCode?.usage}</span>
                            </p>
                        </Grid>

                        <Divider orientation={mdUp ? 'vertical' : 'horizontal'} flexItem style={{ borderStyle: 'dashed', margin: "16px 0" }} />

                        <Grid item lg={2} md={2} sm={6} container justifyContent={mdUp ? 'center' : 'start'} alignItems='center'>
                            <p>
                                <Typography
                                    variant="h6"
                                    sx={{ color: "text.disabled", flexGrow: 1 }}
                                >
                                    Usage limite
                                </Typography>
                                <span>{promoCode?.usageLimit}</span>
                            </p>
                        </Grid>

                        <Divider orientation={mdUp ? 'vertical' : 'horizontal'} flexItem style={{ borderStyle: 'dashed', margin: "16px 0" }} />

                        <Grid item lg={2} md={2} sm={6} container justifyContent={mdUp ? 'center' : 'start'} alignItems='center'>
                            <p>
                                <Typography
                                    variant="h6"
                                    sx={{ color: "text.disabled", flexGrow: 1 }}
                                >
                                    Usage limite par client
                                </Typography>
                                <span>{promoCode?.usageLimitPerClient}</span>
                            </p>
                        </Grid>

                        <Grid item lg={2} md={2} sm={2} container justifyContent={mdUp ? 'center' : 'start'} alignItems='center'>
                            <p>
                                <Typography
                                    variant="h6"
                                    sx={{ color: "text.disabled", flexGrow: 1 }}
                                >
                                    Montant minimum
                                </Typography>
                                <span>{fCurrency(promoCode?.minAmount || 0)}</span>
                            </p>
                        </Grid>

                        <Divider orientation="horizontal" flexItem style={{ borderStyle: 'dashed', margin: "16px 0" }} />

                        <Grid item lg={2} md={5} sm={5} container justifyContent={mdUp ? 'center' : 'start'} alignItems='center'>
                            <p>
                                <Typography
                                    variant="h6"
                                    sx={{ color: "text.disabled", flexGrow: 1 }}
                                >
                                    Date de début
                                </Typography>
                                <span>{fDate(promoCode?.startDate)}</span> - <span>{fTime(promoCode?.startDate)}</span>
                            </p>
                        </Grid>

                        <Divider orientation={mdUp ? 'vertical' : 'horizontal'} flexItem style={{ borderStyle: 'dashed', margin: "16px 0" }} />

                        <Grid item lg={2} md={5} sm={5} container justifyContent={mdUp ? 'center' : 'start'} alignItems='center'>
                            <p>
                                <Typography
                                    variant="h6"
                                    sx={{ color: "text.disabled", flexGrow: 1 }}
                                >
                                    Date d'expiration
                                </Typography>
                                <span>{fDate(promoCode?.expirationDate)}</span> - <span>{fTime(promoCode?.expirationDate)}</span>
                            </p>
                        </Grid>

                        <Divider orientation={mdUp ? 'vertical' : 'horizontal'} hidden={mdUp && (!lgUp)} flexItem style={{ borderStyle: 'dashed', margin: "16px 0" }} />

                    </Grid>
                </Card>

                {promoCode?.description ? (
                    <Card sx={{ p: 3, mt: 2 }}>
                        <p>
                            <Typography variant="h6" sx={{ color: "text.disabled", flexGrow: 1 }}>
                                Description
                            </Typography>
                            <span>{promoCode?.description}</span>
                        </p>
                    </Card>
                ) : (
                    <Card sx={{ p: 3, mt: 2 }}>
                        <p>
                            <Typography variant="h6" sx={{ color: "text.disabled", flexGrow: 1 }}>
                                Aucune description disponible pour le moment.
                            </Typography>
                        </p>
                    </Card>
                )}
                
                <Grid container spacing={2} sx={{ mt: 2 }} >
                    <Grid item xs={6} md={3}>
                        <PromoCodeCardProductsList title="Produits inclus" products={promoCode?.productsIncluded || []} />
                    </Grid>
                    <Grid item xs={6} md={3}>
                        <PromoCodeCardProductsList title="Produits exclus" products={promoCode?.productsExcluded || []} />
                    </Grid>
                    <Grid item xs={6} md={3}>
                        <PromoCodeCardProductsList title="Variations incluses" products={promoCode?.variationsIncluded || []} />
                    </Grid>
                    <Grid item xs={6} md={3}>
                        <PromoCodeCardProductsList title="Variations excluses" products={promoCode?.variationsExcluded || []} />
                    </Grid>
                </Grid>

                <Grid container spacing={2} sx={{ mt: 2 }} >
                    <Grid item xs={6} md={6}>
                        <Typography variant="h6" sx={{ color: "text.disabled", flexGrow: 1 }}>
                            Catégories incluses
                        </Typography>
                        {promoCode?.categoriesIncluded && promoCode?.categoriesIncluded.map((category: ICategory) => (
                            <span key={category.id} style={{ display: 'block', color: 'text.secondary', backgroundColor: 'background.neutral', borderRadius: '5px', padding: '5px', marginTop: '10px', width: '100%' }}>{category.nom}</span>
                        ))}
                    </Grid>
                    <Grid item xs={6} md={6}>
                        <Typography variant="h6" sx={{ color: "text.disabled", flexGrow: 1 }}>
                            Catégories excluses
                        </Typography>
                        {promoCode?.categoriesExcluded && promoCode?.categoriesExcluded.map((category: ICategory) => (
                            <span key={category.id} style={{ display: 'block', color: 'text.secondary', backgroundColor: 'background.neutral', borderRadius: '5px', padding: '5px', marginTop: '10px', width: '100%' }}>{category.nom}</span>
                        ))}
                    </Grid>
                </Grid>
            </Container>
        </SnackbarProvider>
    )
}

const getReceiptStatusColor = (status: ReceiptStatus) => {
    switch (status) {
        case ReceiptStatus.PRODUCT_HAS_BEEN_COLLECTED:
            return "success";
        case ReceiptStatus.PRODUCT_WILL_BE_COLLECTED_LATER:
            return "info";
        case ReceiptStatus.PRODUCT_WILL_NEVER_BE_COLLECTED:
            return "error";
        case ReceiptStatus.PRODUCT_RECEIVED_INCOMPLETELY:
            return "warning";
        default:
            return "default";
    }
};

const getCollecteStatusColor = (status: CollectionStatus) => {
    switch (status) {
        case CollectionStatus.PRODUCT_HAS_BEEN_COLLECTED:
            return "success";
        case CollectionStatus.PRODUCT_WILL_BE_COLLECTED_LATER:
            return "info";
        case CollectionStatus.PRODUCT_WILL_NEVER_BE_COLLECTED:
            return "error";
        case CollectionStatus.PRODUCT_RECEIVED_INCOMPLETELY:
            return "warning";
        default:
            return "default";
    }
};
