"use client";

import { useState, useCallback, useEffect } from "react";
import Tab from "@mui/material/Tab";
import Tabs from "@mui/material/Tabs";
import Card from "@mui/material/Card";
import Table from "@mui/material/Table";
import Stack from "@mui/material/Stack";
import Button from "@mui/material/Button";
import Tooltip from "@mui/material/Tooltip";
import Container from "@mui/material/Container";
import TableBody from "@mui/material/TableBody";
import IconButton from "@mui/material/IconButton";
import { alpha, useTheme } from "@mui/material/styles";
import TableContainer from "@mui/material/TableContainer";
import { paths } from "@/routes/paths";
import RouterLink from "@/routes/router-link";
import { useBoolean } from "@/hooks/use-boolean";
import { isAfter, isBetween } from "@/utils/format-time";
import Label from "@/shared/components/label";
import Iconify from "@/components/iconify";
import Scrollbar from "@/components/scrollbar";
import { ConfirmDialog } from "@/shared/components/custom-dialog";
import { useSettingsContext } from "@/components/settings";
import CustomBreadcrumbs from "@/components/custom-breadcrumbs";
import {useTable,emptyRows,TableNoData,TableEmptyRows,TableHeadCustom,TableSelectedAction,TablePaginationCustom} from "@/shared/components/table";
import { Alert, CircularProgress, Snackbar } from '@mui/material';
import { getPromoCodeTypeColor, PromoCodeItem, PromoCodeTableFilters, PromoCodeTableFilterValue, PromoCodeType, promoCodeTypeLabels } from '@/contexts/types/promoCode';
import { deletePromoCodeById, deletePromoCodesByIDs, getAllPromoCodes } from '@/shared/api/promoCode';
import PromoCodeTableToolbar from './promo-code-table-toolbar';
import PromoCodeTableFiltersResult from './promo-code-table-filters-result';
import PromoCodeTableRow from './promo-code-table-row';
import { useSnackbar } from '@/components/snackbar';
import { LoadingScreen } from '@/shared/components/loading-screen';

const TABLE_HEAD = [
  { id: "code", label: "Code" },
  { id: "valueCode", label: "Valeur du code" },
  { id: "minAmount", label: "Montant minimum" },
  { id: "usage", label: "Usage" },
  { id: "usageLimit", label: "Usage limit" },
  { id: "usageLimitPerClient", label: "Usage limit par client" },
  { id: "expirationDate", label: "Date d'expiration" },
  { id: "promoCodeType", label: "Type" },
  { id: "" },
];

const defaultFilters: PromoCodeTableFilters = {
  name: "",
  promoCodeType: "all",
  startDate: null,
  endDate: null,
  category: [],
};

export default function PromoCodeListView() {
  const { enqueueSnackbar } = useSnackbar();

  const theme = useTheme();

  const settings = useSettingsContext();

  const table = useTable({ defaultOrderBy: "id", defaultOrder: "desc" });

  const confirm = useBoolean();

  const [tableData, setTableData] = useState<PromoCodeItem[]>([]);

  const [filters, setFilters] = useState(defaultFilters);

  const dateError = isAfter(filters.startDate, filters.endDate);

  const { promoCodes, promoCodesLoading, promoCodesError, mutatePromoCodes } = getAllPromoCodes();

  const [openDeleteAlert, setOpenDeleteAlert] = useState(false);
  const [openErrorAlert, setOpenErrorAlert] = useState(false);

  useEffect(() => {
    if (promoCodes && promoCodes.length > 0) {
      setTableData(promoCodes);
    }
  }, [promoCodes]);

  function getComparator<T extends PromoCodeItem>(order: 'asc' | 'desc', orderBy: keyof T) {
    return order === 'desc'
      ? (a: T, b: T) => descendingComparator(a, b, orderBy)
      : (a: T, b: T) => -descendingComparator(a, b, orderBy);
  }

  function descendingComparator<T extends PromoCodeItem>(a: T, b: T, orderBy: keyof T) {
    if (b[orderBy] < a[orderBy]) {
      return -1;
    }
    if (b[orderBy] > a[orderBy]) {
      return 1;
    }
    return 0;
  }

  const dataFiltered = applyFilter({
    inputData: tableData,
    comparator: getComparator(table.order, table.orderBy),
    filters,
    dateError,
  });

  const dataInPage = dataFiltered.slice(
    table.page * table.rowsPerPage,
    table.page * table.rowsPerPage + table.rowsPerPage
  );

  const denseHeight = table.dense ? 56 : 56 + 20;

  const canReset =
    !!filters.name ||
    !!filters.promoCodeType.length ||
    !!filters.category.length ||
    (!!filters.startDate && !!filters.endDate);

  const notFound = (!dataFiltered.length && canReset) || !dataFiltered.length;

  const getPromoCodesLength = (type: PromoCodeType) =>
    tableData.filter((item) => item.promoCodeType === type).length;

  const TABS = [
    { value: "all", label: "Tous", color: "default", count: tableData.length },
    {
      value: PromoCodeType.PERCENTAGE_PER_CART,
      label: promoCodeTypeLabels[PromoCodeType.PERCENTAGE_PER_CART],
      color: getPromoCodeTypeColor(PromoCodeType.PERCENTAGE_PER_CART),
      count: getPromoCodesLength(PromoCodeType.PERCENTAGE_PER_CART),
    },
    {
      value: PromoCodeType.DISCOUNT_PER_PRODUCT,
      label: promoCodeTypeLabels[PromoCodeType.DISCOUNT_PER_PRODUCT],
      color: getPromoCodeTypeColor(PromoCodeType.DISCOUNT_PER_PRODUCT),
      count: getPromoCodesLength(PromoCodeType.DISCOUNT_PER_PRODUCT),
    },
    {
      value: PromoCodeType.DISCOUNT_PER_CART,
      label: promoCodeTypeLabels[PromoCodeType.DISCOUNT_PER_CART],
      color: getPromoCodeTypeColor(PromoCodeType.DISCOUNT_PER_CART),
      count: getPromoCodesLength(PromoCodeType.DISCOUNT_PER_CART),
    },
  ] as const;

  const handleFilters = useCallback(
    (name: string, value: PromoCodeTableFilterValue) => {
      table.onResetPage();
      setFilters((prevState) => ({
        ...prevState,
        [name]: value,
      }));
    },
    [table]
  );

  const handleResetFilters = useCallback(() => {
    setFilters(defaultFilters);
  }, []);

  const handleDeleteRow = useCallback(
    async (id: string) => {
      try {
        const status = await deletePromoCodeById(id);
        if (status === 200) {
          const deleteRow = tableData.filter((row) => row.id !== id);
          setOpenDeleteAlert(true);
          setTableData(deleteRow);
          table.onUpdatePageDeleteRow(dataInPage.length);
          mutatePromoCodes();
        } else {
          setOpenErrorAlert(true);
        }
      } catch (error) {
        setOpenErrorAlert(true);
      }
    },
    [dataInPage.length, enqueueSnackbar, table, tableData]
  );

  const handleDeleteRows = useCallback(async () => {
    try {
      const promoCodesToDelete: string[] = table.selected.map((id) => {
        const promoCode = tableData.find((row) => row.id === id);
        return promoCode ? promoCode.id : null;
      }).filter((id): id is string => id !== null);
      if (promoCodesToDelete.length === 0) {
        setOpenErrorAlert(true);
        return;
      }
      const status = await deletePromoCodesByIDs(promoCodesToDelete);
      if (status === 200) {
        const deleteRows = tableData.filter((row) => !table.selected.includes(row.id!));
        setOpenDeleteAlert(true);
        setTableData(deleteRows);
        table.onUpdatePageDeleteRows({
          totalRowsInPage: dataInPage.length,
          totalRowsFiltered: dataFiltered.length,
        });
        mutatePromoCodes();
      } else {
        setOpenErrorAlert(true);
      }
    } catch (error) {
      setOpenErrorAlert(true);
    }
  }, [
    dataFiltered.length,
    dataInPage.length,
    table,
    tableData,
  ]);

  const handleFilterType = useCallback(
    (event: React.SyntheticEvent, newValue: string) => {
      handleFilters("promoCodeType", newValue);
    },
    [handleFilters]
  );

  if (promoCodesLoading) {
    return (
      <Container maxWidth={settings.themeStretch ? false : 'lg'}>
        <LoadingScreen />
      </Container>
    );
  } 

  if (promoCodesError) {
    return (
      <Container maxWidth={settings.themeStretch ? false : 'lg'}>
        <Alert severity="error">Problème lors de la récupération des données des codes promos. Veuillez réessayer plus tard.</Alert>
      </Container>
    );
  }

  return (
    <>
      <Container maxWidth={settings.themeStretch ? false : "lg"}>
        <CustomBreadcrumbs
          heading="Liste des codes promos"
          links={[
            {
              name: "Tableau de bord",
              href: paths.dashboard.root,
            },
            {
              name: "Liste des codes promos",
              href: paths.dashboard.admin.promoCode.root,
            }
          ]}
          action={
            <Button
              component={RouterLink}
              href={paths.dashboard.admin.promoCode.new}
              variant="contained"
              startIcon={<Iconify icon="mingcute:add-line" />}
            >
              Nouveau Code Promo
            </Button>
          }
          sx={{
            mb: { xs: 3, md: 5 },
          }}
        />

        <Card>
          <Tabs
            value={filters.promoCodeType}
            onChange={handleFilterType}
            sx={{
              px: 2.5,
              boxShadow: `inset 0 -2px 0 0 ${alpha(
                theme.palette.grey[500],
                0.08
              )}`,
            }}
          >
            {TABS.map((tab) => (
              <Tab

                key={tab.value}
                value={tab.value}
                label={tab.label}
                iconPosition="end"
                icon={
                  <Label
                    variant={
                      ((tab.value === "all" || tab.value === filters.promoCodeType) &&
                        "filled") ||
                      "soft"
                    }
                    color={tab.color}
                  >
                    {tab.count}
                  </Label>
                }

              />
            ))}
          </Tabs>

          <PromoCodeTableToolbar
            filters={filters}
            onFilters={handleFilters}
            dateError={dateError}
          />

          {canReset && (
            <PromoCodeTableFiltersResult
              filters={filters}
              onFilters={handleFilters}
              //
              onResetFilters={handleResetFilters}
              //
              results={dataFiltered.length}
              sx={{ p: 2.5, pt: 0 }}
            />
          )}

          <TableContainer sx={{ position: "relative", overflow: "unset" }}>
            <TableSelectedAction
              dense={table.dense}
              numSelected={table.selected.length}
              rowCount={dataFiltered.length}
              onSelectAllRows={(checked) => {
                table.onSelectAllRows(
                  checked,
                  dataFiltered.map((row) => row.id!)
                );
              }}
              action={
                <Stack direction="row">
                  <Tooltip title="Delete">
                    <IconButton color="primary" onClick={confirm.onTrue}>
                      <Iconify icon="solar:trash-bin-trash-bold" />
                    </IconButton>
                  </Tooltip>
                </Stack>
              }
            />
            {
              promoCodesLoading ? (
                <Stack alignItems="center" justifyContent="center" sx={{ py: 5 }}>
                  <CircularProgress />
                </Stack>
              ) : (
                <Scrollbar>
                  <Table
                    size={table.dense ? "small" : "medium"}
                    sx={{ minWidth: 800 }}
                  >
                    <TableHeadCustom
                      order={table.order}
                      orderBy={table.orderBy}
                      headLabel={TABLE_HEAD}
                      rowCount={dataFiltered.length}
                      numSelected={table.selected.length}
                      onSort={table.onSort}
                      onSelectAllRows={(checked) =>
                        table.onSelectAllRows(
                          checked,
                          dataFiltered.map((row) => row.id!)
                        )
                      }
                    />

                    <TableBody>
                      {dataFiltered
                        .slice(
                          table.page * table.rowsPerPage,
                          table.page * table.rowsPerPage + table.rowsPerPage
                        )
                        .map((row) => (
                          <PromoCodeTableRow
                            key={row.id}
                            row={row}
                            selected={table.selected.includes(row.id!)}
                            onSelectRow={() => table.onSelectRow(row.id!)}
                            onDeleteRow={() => handleDeleteRow(row.id!)}
                          />
                        ))}

                      <TableEmptyRows
                        height={denseHeight}
                        emptyRows={emptyRows(
                          table.page,
                          table.rowsPerPage,
                          dataFiltered.length
                        )}
                      />

                      <TableNoData notFound={notFound} />
                    </TableBody>
                  </Table>
                </Scrollbar>
              )
            }

          </TableContainer>

          <TablePaginationCustom
            count={dataFiltered.length}
            page={table.page}
            rowsPerPage={table.rowsPerPage}
            onPageChange={table.onChangePage}
            onRowsPerPageChange={table.onChangeRowsPerPage}
            //
            dense={table.dense}
            onChangeDense={table.onChangeDense}
          />
        </Card>
      </Container>

      <ConfirmDialog
        open={confirm.value}
        onClose={confirm.onFalse}
        title="Supprimer"
        content={
          <>
            Êtes-vous sûr de vouloir supprimer{" "}
            <strong> {table.selected.length} </strong> éléments?
          </>
        }
        action={
          <Button
            variant="contained"
            color="error"
            onClick={() => {
              handleDeleteRows();
              confirm.onFalse();
            }}
          >
            Supprimer
          </Button>
        }
      />
      <Snackbar open={openDeleteAlert} autoHideDuration={3000}
        onClose={() => setOpenDeleteAlert(false)} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
        <Alert
          onClose={() => setOpenDeleteAlert(false)}
          severity="success"
          variant="filled"
          sx={{ width: '100%' }}
        >
          Suppression réussie
        </Alert>
      </Snackbar>
      <Snackbar open={openErrorAlert} autoHideDuration={3000}
        onClose={() => setOpenErrorAlert(false)} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
        <Alert
          onClose={() => setOpenErrorAlert(false)}
          severity="error"
          variant="filled"
          sx={{ width: '100%' }}
        >
          Échec de suppression
        </Alert>
      </Snackbar>
    </>
  );
}

// ----------------------------------------------------------------------

function applyFilter({
  inputData,
  comparator,
  filters,
  dateError,
}: {
  inputData: PromoCodeItem[];
  comparator: (a: any, b: any) => number;
  filters: PromoCodeTableFilters;
  dateError: boolean;
}) {
  const { name, promoCodeType, category, startDate, endDate } = filters;

  const stabilizedThis = inputData.map((el, index) => [el, index] as const);

  stabilizedThis.sort((a, b) => {
    const order = comparator(a[0], b[0]);
    if (order !== 0) return order;
    return a[1] - b[1];
  });

  inputData = stabilizedThis.map((el) => el[0]);

  if (name) {
    inputData = inputData.filter((promoCode) =>
      promoCode.code?.toLowerCase().indexOf(name.toLowerCase()) !== -1 ||
      promoCode.valueCode?.toString().indexOf(name.toLowerCase()) !== -1 ||
      promoCode.minAmount?.toString().indexOf(name.toLowerCase()) !== -1 ||
      promoCode.usage?.toString().indexOf(name.toLowerCase()) !== -1 ||
      promoCode.usageLimit?.toString().indexOf(name.toLowerCase()) !== -1
    );
  }

  if (promoCodeType !== "all") {
    inputData = inputData.filter((promoCode) => promoCode.promoCodeType === promoCodeType);
  }

  if (category.length) {
    inputData = inputData.filter((promoCode) =>
      promoCode.categoriesIncludedIds.some((filterItem) => category.includes(filterItem.toString()))
    );
  }

  if (!dateError) {
    if (startDate && endDate) {
      inputData = inputData.filter((promoCode) =>
        isBetween(promoCode.startDate, startDate, endDate)
      );
    }
  }

  return inputData;
}
