import { useCallback } from "react";
import Box from "@mui/material/Box";
import Chip from "@mui/material/Chip";
import Paper from "@mui/material/Paper";
import Button from "@mui/material/Button";
import Stack, { StackProps } from "@mui/material/Stack";
import Iconify from "@/shared/components/iconify";
import { shortDateLabel } from "@/shared/components/custom-date-range-picker";
import { PromoCodeTableFilterValue, PromoCodeType, promoCodeTypeLabels } from "@/contexts/types/promoCode";
import { PromoCodeTableFilters } from "@/contexts/types/promoCode";

type Props = StackProps & {
  filters: PromoCodeTableFilters;
  onFilters: (name: string, value: PromoCodeTableFilterValue) => void;
  onResetFilters: VoidFunction;
  results: number;
};

export default function PromoCodeTableFiltersResult({
  filters,
  onFilters,
  //
  onResetFilters,
  //
  results,
  ...other
}: Props) {
  const shortLabel = shortDateLabel(filters.startDate, filters.endDate);

  const handleRemoveName = useCallback(() => {
    onFilters("name", "");
  }, [onFilters]);

  const handleRemovePromoCodeType = useCallback(() => {
    onFilters("promoCodeType", "all");
  }, [onFilters]);

  const handleRemoveCategory = useCallback(() => {
    onFilters("category", []);
  }, [onFilters]);

  const handleRemoveDate = useCallback(() => {
    onFilters("startDate", null);
    onFilters("endDate", null);
  }, [onFilters]);

  const hasActiveFilters =
    filters.name ||
    filters.promoCodeType !== "all" ||
    filters.category.length > 0 ||
    (filters.startDate && filters.endDate);

  return (
    <Stack spacing={1.5} {...other}>
      {hasActiveFilters && (
        <>
          {results > 0 && (
            <Box sx={{ typography: "body2" }}>
              <strong>{results}</strong>
              <Box component="span" sx={{ color: "text.secondary", ml: 0.25 }}>
                Résultat trouvé
              </Box>
            </Box>
          )}

          <Stack
            flexGrow={1}
            spacing={1}
            direction="row"
            flexWrap="wrap"
            alignItems="center"
          >
            {filters.promoCodeType !== "all" ? (
              <Block label="Type de code promo:">
                <Chip
                  size="small"
                  label={promoCodeTypeLabels[filters.promoCodeType as PromoCodeType]}
                  onDelete={handleRemovePromoCodeType}
                />
              </Block>
            ) : null}

            {filters.category.length > 0 ? (
              <Block label="Catégorie:">
                <Chip size="small" label={filters.category.join(", ")} onDelete={handleRemoveCategory} />
              </Block>
            ) : null}

            {filters.startDate && filters.endDate ? (
              <Block label="Date:">
                <Chip size="small" label={shortLabel} onDelete={handleRemoveDate} />
              </Block>
            ) : null}

            {!!filters.name ? (
              <Block label="Code promo:">
                <Chip label={filters.name} onDelete={handleRemoveName} />
              </Block>
            ) : null}

            <Button
              color="error"
              onClick={onResetFilters}
              startIcon={<Iconify icon="solar:trash-bin-trash-bold" />}
            >
              Supprimer
            </Button>
          </Stack>
        </>
      )}
    </Stack>
  );
}

// ----------------------------------------------------------------------

type BlockProps = StackProps & {
  label: string;
};

function Block({ label, children, sx, ...other }: BlockProps) {
  return (
    <Stack
      component={Paper}
      variant="outlined"
      spacing={1}
      direction="row"
      sx={{
        p: 1,
        borderRadius: 1,
        overflow: "hidden",
        borderStyle: "dashed",
        ...sx,
      }}
      {...other}
    >
      <Box component="span" sx={{ typography: "subtitle2" }}>
        {label}
      </Box>

      <Stack spacing={1} direction="row" flexWrap="wrap">
        {children}
      </Stack>
    </Stack>
  );
}
