import Button from "@mui/material/Button";
import Divider from "@mui/material/Divider";
import MenuItem from "@mui/material/MenuItem";
import TableRow from "@mui/material/TableRow";
import Checkbox from "@mui/material/Checkbox";
import TableCell from "@mui/material/TableCell";
import IconButton from "@mui/material/IconButton";
import ListItemText from "@mui/material/ListItemText";
import { useBoolean } from "@/hooks/use-boolean";
import { fCurrency } from "@/utils/format-number";
import { fDate, fTime } from "@/utils/format-time";
import Label from "@/shared/components/label";
import Iconify from "@/components/iconify";
import { ConfirmDialog } from "@/shared/components/custom-dialog";
import CustomPopover, { usePopover } from "@/shared/components/custom-popover";
import { useRouter } from "@/hooks";
import { paths } from '@/routes/paths';
import { getPromoCodeTypeColor, PromoCodeItem, PromoCodeType, promoCodeTypeLabels } from "@/contexts/types/promoCode";

type Props = {
  row: PromoCodeItem;
  selected: boolean;
  onSelectRow: VoidFunction;
  onDeleteRow: VoidFunction;
};

export default function PromoCodeTableRow({
  row,
  selected,
  onSelectRow,
  onDeleteRow,
}: Props) {
  const { id, code, valueCode, minAmount, usage, usageLimit, usageLimitPerClient, expirationDate, promoCodeType } = row;

  const confirm = useBoolean();
  const popover = usePopover();
  const router = useRouter();

  const handleShowPromoCodeDetails = ((promoCodeId: string) => {
    router.push(paths.dashboard.admin.promoCode.details(promoCodeId));
  })

  const handleEditPromoCode = ((promoCodeId: string) => {
    router.push(paths.dashboard.admin.promoCode.edit(promoCodeId));
  });

  const handleDuplicatePromoCode = ((promoCodeId: string) => {
    router.push(paths.dashboard.admin.promoCode.duplicate(promoCodeId));
  });

  const valueSymbol = PromoCodeType.DISCOUNT_PER_CART === promoCodeType || PromoCodeType.DISCOUNT_PER_PRODUCT === promoCodeType ? " €"
    : PromoCodeType.PERCENTAGE_PER_CART === promoCodeType
      ? " %"
      : "";

  return (
    <>
      <TableRow hover selected={selected} onClick={() => handleShowPromoCodeDetails(id!)}>
        <TableCell padding="checkbox" onClick={(event) => event.stopPropagation()}>
          <Checkbox checked={selected} onClick={onSelectRow} />
        </TableCell>

        <TableCell align="center">{code}</TableCell>

        <TableCell align="center">{valueCode + valueSymbol}</TableCell>

        <TableCell align="center">{fCurrency(minAmount)}</TableCell>

        <TableCell align="center">{usage}</TableCell>

        <TableCell align="center">{usageLimit}</TableCell>

        <TableCell align="center">{usageLimitPerClient}</TableCell>

        <TableCell align="center">
          <ListItemText
            primary={fDate(expirationDate)}
            secondary={fTime(expirationDate)}
            primaryTypographyProps={{ typography: "body2", noWrap: true }}
            secondaryTypographyProps={{
              mt: 0.5,
              component: "span",
              typography: "caption",
            }}
          />
        </TableCell>

        <TableCell align="center">
          <Label
            variant="soft"
            color={getPromoCodeTypeColor(promoCodeType)}
          >
            {promoCodeTypeLabels[promoCodeType]}
          </Label>
        </TableCell>

        <TableCell align="right" sx={{ px: 1 }}>
          <IconButton
            color={popover.open ? "inherit" : "default"}
            onClick={(e) => {
              popover.onOpen(e);
              e.stopPropagation();
            }}
          >
            <Iconify icon="eva:more-vertical-fill" />
          </IconButton>
        </TableCell>
      </TableRow>

      <CustomPopover
        open={popover.open}
        onClose={popover.onClose}
        arrow="right-top"
        sx={{ width: 160 }}
      >
        <MenuItem
          onClick={() => handleShowPromoCodeDetails(id!)}
        >
          <Iconify icon="solar:eye-bold" />
          Visualiser
        </MenuItem>

        <MenuItem
          onClick={() => handleEditPromoCode(id!)}
        >
          <Iconify icon="solar:pen-bold" />
          Modifier
        </MenuItem>

        <MenuItem
          onClick={() => handleDuplicatePromoCode(id!)}
        >
          <Iconify icon="solar:copy-bold" />
          Dupliquer
        </MenuItem>
        <Divider sx={{ borderStyle: "dashed" }} />

        <MenuItem
          onClick={() => {
            confirm.onTrue();
            popover.onClose();
          }}
          sx={{ color: "error.main" }}
        >
          <Iconify icon="solar:trash-bin-trash-bold" />
          Supprimer
        </MenuItem>
      </CustomPopover>

      <ConfirmDialog
        open={confirm.value}
        onClose={confirm.onFalse}
        title="Supprimer"
        content="Êtes-vous sûr de vouloir supprimer cet élément?"
        action={
          <Button variant="contained" color="error" onClick={onDeleteRow}>
            Supprimer
          </Button>
        }
      />
    </>
  );
}
