"use client";

import { useState, useEffect } from "react";
import Stack from "@mui/material/Stack";
import Card from "@mui/material/Card";
import Box from "@mui/material/Box";
import {
  DataGrid,
  GridColDef,
  GridToolbarExport,
  GridToolbarContainer,
  GridRowSelectionModel,
  GridToolbarQuickFilter,
  GridToolbarColumnsButton,
} from "@mui/x-data-grid";
import { Typography } from "@mui/material";
import { useBoolean } from "@/hooks/use-boolean";
import Iconify from "@/shared/components/iconify";
import { useSnackbar } from "@/shared/components/snackbar";
import { useSettingsContext } from "@/shared/components/settings";
import MenuItem from "@mui/material/MenuItem";
import CustomPopover, { usePopover } from "@/shared/components/custom-popover";
import { ReferralListResponse } from "@/shared/types/referral";
import { getAllReferral } from "@/shared/api/referral";
import { RenderCellGain } from "./Listreferrer-table-row";

// ----------------------------------------------------------------------

export default function ListreferrerView() {
  const { enqueueSnackbar } = useSnackbar();
  const confirmRows = useBoolean();
  const popover = usePopover();

  const [tableData, setTableData] = useState<ReferralListResponse[]>([]);
  const [selectedRowIds, setSelectedRowIds] = useState<GridRowSelectionModel>([]);
  const [loading, setLoading] = useState<boolean>(true);
  const [error, setError] = useState<string | null>(null);

  useEffect(() => {
    const fetchReferrals = async () => {
      setLoading(true);
      setError(null);

      try {
        const data = await getAllReferral();
        setTableData(data); // Mettre à jour avec les données réelles
      } catch (err: any) {
        setError(err.message || "Une erreur s'est produite.");
        enqueueSnackbar("Échec du chargement des données.", { variant: "error" });
      } finally {
        setLoading(false);
      }
    };

    fetchReferrals();
  }, [enqueueSnackbar]);

  const columns: GridColDef[] = [
    { field: "name", headerName: "Nom", flex: 1 },
    { field: "referrerEmail", headerName: "Email", flex: 1 },
    { field: "referrerCode", headerName: "Code de parrainage", flex: 1 },
    { field: "referralTotal", headerName: "Total de parrainage", flex: 1 },
    {
      field: "total",
      headerName: "Gains de Parrainage",
      flex: 1,
      renderCell: (params) => <RenderCellGain params={params} />,
    },
  ];

  return (
    <>
      <Typography variant="h6" gutterBottom sx={{ marginTop: "70px" }}>
        Liste des parrains
      </Typography>

      <Card sx={{ marginTop: "30px" }}>
        <Box
          sx={{
            height: 430,
            width: "100%",
            flexGrow: { md: 1 },
            display: { md: "flex" },
            flexDirection: { md: "column" },
          }}
        >
          <DataGrid
            loading={loading}
            rows={tableData}
            columns={columns}
            checkboxSelection
            disableRowSelectionOnClick
            pageSizeOptions={[5, 10, 25]}
            initialState={{
              pagination: {
                paginationModel: { pageSize: 10 },
              },
            }}
            onRowSelectionModelChange={(newSelectionModel) => {
              setSelectedRowIds(newSelectionModel);
            }}
            localeText={{
              toolbarColumns: "Colonnes", 
              toolbarColumnsLabel:"chercher ",
              toolbarFilters: "Filtrer",
              toolbarExport: "Exporter",
            
            }}
            slots={{
              toolbar: () => (
                <GridToolbarContainer>
                  <GridToolbarQuickFilter placeholder="Recherche..." />
                  <Stack
                    spacing={1}
                    flexGrow={1}
                    direction="row"
                    alignItems="center"
                    justifyContent="flex-end"
                  >
                    <GridToolbarColumnsButton />
                    <GridToolbarExport />
                  </Stack>
                </GridToolbarContainer>
              ),
            }}
          />
        </Box>

        <CustomPopover
          open={popover.open}
          onClose={popover.onClose}
          arrow="right-top"
          sx={{ width: 140 }}
        >
          <MenuItem
            onClick={() => {
              popover.onClose();
            }}
          >
            <Iconify icon="solar:printer-minimalistic-bold" />
            Imprimer
          </MenuItem>
          <MenuItem
            onClick={() => {
              popover.onClose();
            }}
          >
            <Iconify icon="solar:import-bold" />
            Importer
          </MenuItem>
          <MenuItem
            onClick={() => {
              popover.onClose();
            }}
          >
            <Iconify icon="solar:export-bold" />
            Exporter
          </MenuItem>
        </CustomPopover>
      </Card>
    </>
  );
}
