"use client";

import { useState, useEffect } from "react";
import Container from "@mui/material/Container";
import {
  Typography,
  Table,
  TableBody,
  TableCell,
  TableContainer,
  TableHead,
  TableRow,
  Paper,
  Switch,
} from "@mui/material";
import { paths } from "../../../../routes/paths";
import { useSettingsContext } from "../../../components/settings";
import CustomBreadcrumbs from "../../../../components/custom-breadcrumbs";
import { useGetconfig, useUpdateConfig } from "@/shared/api/Review-Config";

// ----------------------------------------------------------------------

export default function ReviewListView() {
  const settings = useSettingsContext();

  const { reviewData, reviewLoading, reviewMutate } = useGetconfig();

  const [preferences, setPreferences] = useState({
    commentsEnabled: true,
    autoApproval: false,
    notificationsEnabled: false,
  });

  // Update preferences state when data is loaded
  useEffect(() => {
    if (reviewData) {
      setPreferences({
        commentsEnabled: reviewData.commentsEnabled || false,
        autoApproval: reviewData.autoApproval || false,
        notificationsEnabled: reviewData.notificationsEnabled || false,
      });
    }
  }, [reviewData]);

  const handlePreferenceChange = async (event: {
    target: { name: any; checked: any };
  }) => {
    const { name, checked } = event.target;

    const updatedPreferences = {
      ...preferences,
      [name]: checked,
    };

    setPreferences(updatedPreferences);

    // Call update function with the modified preferences
    await useUpdateConfig(updatedPreferences);
    reviewMutate(); // Revalidate data after update
  };

  if (reviewLoading) {
    return <Typography>Chargement...</Typography>;
  }

  return (
    <Container maxWidth={settings.themeStretch ? false : "lg"}>
      <CustomBreadcrumbs
        heading="Configuration"
        links={[
          { name: "Tableau de bord", href: paths.dashboard.root },
          {
            name: "Commentaires",
            href: paths.dashboard.admin.order.all_orders,
          },
          { name: "Configuration" },
        ]}
        rollbackLink={paths.dashboard.admin.review.all_reviews} 

        sx={{ mb: { xs: 3, md: 5 } }}
      />

      <TableContainer component={Paper}>
        <Table>
          <TableHead>
            <TableRow>
              <TableCell>Nom</TableCell>
              <TableCell align="center">Activé</TableCell>
              <TableCell>Description</TableCell>
            </TableRow>
          </TableHead>
          <TableBody>
            {/* Comments Enabled Switch */}
            <TableRow key="commentsEnabled">
              <TableCell>Autoriser les commentaires sur le site</TableCell>
              <TableCell align="center">
                <Switch
                  checked={preferences.commentsEnabled}
                  onChange={handlePreferenceChange}
                  name="commentsEnabled"
                  color="primary"
                />
              </TableCell>
              <TableCell>
                Cette option permet d'activer ou de désactiver la fonctionnalité
                de commentaires sur le site. Lorsqu'elle est activée, les
                visiteurs peuvent laisser des commentaires sur les articles. Si
                elle est désactivée, cette fonctionnalité sera masquée pour les
                utilisateurs.
              </TableCell>
            </TableRow>

            {/* Conditionally render other options if commentsEnabled is true */}
            {preferences.commentsEnabled && (
              <>
                <TableRow key="notificationsEnabled">
                  <TableCell>Autoriser les notifications pour admin</TableCell>
                  <TableCell align="center">
                    <Switch
                      checked={preferences.notificationsEnabled}
                      onChange={handlePreferenceChange}
                      name="notificationsEnabled"
                      color="primary"
                    />
                  </TableCell>
                  <TableCell>
                    Cette option permet d'activer ou de désactiver les
                    notifications pour les administrateurs. Lorsqu'elle est
                    activée, les administrateurs recevront des notifications
                    pour de nouveaux commentaires. Si elle est désactivée, les
                    administrateurs ne recevront pas ces notifications.
                  </TableCell>
                </TableRow>

                <TableRow key="autoApproval">
                  <TableCell>
                    Approuver les commentaires automatiquement
                  </TableCell>
                  <TableCell align="center">
                    <Switch
                      checked={preferences.autoApproval}
                      onChange={handlePreferenceChange}
                      name="autoApproval"
                      color="primary"
                    />
                  </TableCell>
                  <TableCell>
                    Si cette option est activée, tous les commentaires laissés
                    par les utilisateurs seront automatiquement approuvés et
                    publiés sur le site sans nécessiter une intervention
                    manuelle de l'administrateur. Cela permet d'accélérer le
                    processus de publication des commentaires.
                  </TableCell>
                </TableRow>
              </>
            )}
          </TableBody>
        </Table>
      </TableContainer>
    </Container>
  );
}
