"use client";

import isEqual from "lodash/isEqual";
import { useState, useEffect, useCallback } from "react";
import Stack from "@mui/material/Stack";
import Button from "@mui/material/Button";
import Card from "@mui/material/Card";
import Box from "@mui/material/Box";
import {
  DataGrid,
  GridColDef,
  GridToolbarContainer,
  GridRowSelectionModel,
  GridToolbarQuickFilter,
  GridToolbarColumnsButton,
  GridColumnVisibilityModel,
} from "@mui/x-data-grid";
import { paths } from "@/routes/paths";
import { useRouter } from "@/hooks";
import { isAfter, isBetween } from '@/utils/format-time';
import EmptyContent from "@/shared/components/empty-content";
import CustomBreadcrumbs from "@/shared/components/custom-breadcrumbs";
import Select, { SelectChangeEvent } from "@mui/material/Select";
import FormControl from "@mui/material/FormControl";
import InputLabel from "@mui/material/InputLabel";
import MenuItem from "@mui/material/MenuItem";
import {
  RenderCellID,
  RenderCellCreatedAt,
  RenderCellCustomer,
  RenderCellAdress,
  RenderCellCountry,
  RenderCellShippingMethod,
  RenderCellStatus,
  RenderCellEtiquette
} from "./orderColissimo-table-row";
import { IOrderColissimoItem, IOrderColissimoStatus, IOrderColissimoTableFilters, IOrderColissimoTableFilterValue } from "@/shared/types/shipping";
import OrderColissimoTableToolbar from "./orderColissimo-table-toolbar";
import OrderColissimoTableFiltersResult from "./orderColissimo-table-filters-result";
import { ShippingMethodType } from "@/contexts/types/Shipping";
import orderColissimoList from "@/shared/_mock/_shipping";

const Country_OPTIONS = [
  { value: "France", label: "France" },
  { value: "Espagne", label: "Espagne" },
];
const ShippingMethod_OPTIONS = [
  { value: ShippingMethodType.ColissimoWithSignature, label: "Colissimo avec signature" },
  { value: ShippingMethodType.ColissimoWithoutSignature, label: "Colissimo sans signature" },
];
const Status_OPTIONS = [
  { value: IOrderColissimoStatus.WaitingForPayment, label: "Attente paiement" },
  { value: IOrderColissimoStatus.PartiallyPaid, label: "Partially-paid" },
  { value: IOrderColissimoStatus.Pending, label: "En attente" },
  { value: IOrderColissimoStatus.InProgress, label: "En cours" },
  { value: IOrderColissimoStatus.Completed, label: "Terminée" },
  { value: IOrderColissimoStatus.Canceled, label: "Annulé" },
  { value: IOrderColissimoStatus.Failed, label: "Échouée" },
  { value: IOrderColissimoStatus.Trach, label: "Trach" },
];

const Etiquette_OPTIONS = [
  { value: "Aller", label: "Aller" },
  { value: "Retour", label: "Retour" },
];

const defaultFilters: IOrderColissimoTableFilters = {
  country: [],
  shippingMethod: [],
  status: [],
  etiquette: [],
  startDate: null,
  endDate: null,
};

const HIDE_COLUMNS = {
  category: false,
};

const HIDE_COLUMNS_TOGGLABLE = ["category", "actions"];

export default function OrderColissimoListView() {

  const router = useRouter();

  const [orders, setOrders] = useState<IOrderColissimoItem[]>(orderColissimoList);

  const [tableData, setTableData] = useState<IOrderColissimoItem[]>(orderColissimoList);

  const [filters, setFilters] = useState(defaultFilters);

  const [selectedRowIds, setSelectedRowIds] = useState<GridRowSelectionModel>(
    []
  );

  const [columnVisibilityModel, setColumnVisibilityModel] =
    useState<GridColumnVisibilityModel>(HIDE_COLUMNS);

  useEffect(() => {
    if (orders.length) {
      setTableData(orders);
    }
  }, [orders]);

  const dateError = isAfter(filters.startDate, filters.endDate);

  const dataFiltered = applyFilter({
    inputData: tableData,
    filters,
    dateError,
  });

  const canReset = !isEqual(defaultFilters, filters);

  const handleFilters = useCallback(
    (name: string, value: IOrderColissimoTableFilterValue) => {
      setFilters((prevState) => ({
        ...prevState,
        [name]: value,
      }));
    },
    []
  );

  const handleResetFilters = useCallback(() => {
    setFilters(defaultFilters);
  }, []);

  const handleViewRow = useCallback(
    (id: string) => {
      router.push(paths.dashboard.admin.product.details(id));
    },
    [router]
  );

  const columns: GridColDef[] = [
    {
      field: "id",
      headerName: "ID",
      flex: 1,
      width: 80,
      filterable: true,
      hideable: false,
      renderCell: (params) => <RenderCellID params={params} />,
    },
    {
      field: "createdAt",
      headerName: "Date Création",
      width: 120,
      hideable: false,
      renderCell: (params) => <RenderCellCreatedAt params={params} />,
    },
    {
      field: "customer",
      headerName: "Client",
      minWidth: 130,
      renderCell: (params) => <RenderCellCustomer params={params} />,
    },
    {
      field: "adress",
      headerName: "Adresse",
      minWidth: 150,
      renderCell: (params) => <RenderCellAdress params={params} />,
    },
    {
      field: "country",
      headerName: "Pays",
      minWidth: 100,
      renderCell: (params) => <RenderCellCountry params={params} />,
    },
    {
      field: "shippingMethod",
      headerName: "Méthode de livraison",
      minWidth: 170,
      renderCell: (params) => <RenderCellShippingMethod params={params} />,
    },
    {
      field: "status",
      headerName: "Status",
      minWidth: 120,
      renderCell: (params) => <RenderCellStatus params={params} />,
    },
    {
      field: "etiquette",
      headerName: "Etiquette (Aller/Retour/Bordereaux)",
      minWidth: 262,
      renderCell: (params) => <RenderCellEtiquette params={params} />,
    },
  ];

  const getTogglableColumns = () =>
    columns
      .filter((column) => !HIDE_COLUMNS_TOGGLABLE.includes(column.field))
      .map((column) => column.field);

  const [actionGroupe, setActionGroupe] = useState("");

  const handleChangeActionGroupe = (event: SelectChangeEvent) => {
    setActionGroupe(event.target.value as string);
  };

  return (
    <>
      <CustomBreadcrumbs
        heading="Liste des commandes"
        links={[
          { name: 'Dashboard', href: paths.dashboard.root, },
          { name: 'Livraison', href: paths.dashboard.admin.shipping.general.root, },
          { name: "Colissimo" },
        ]}
        sx={{
          mb: {
            xs: 3,
            md: 5,
          },
        }}
      />

      <Stack direction={'row'} alignItems="center">
        <Box sx={{ mb: 1, mr: 0.5, width: 250 }}>
          <FormControl fullWidth size="small">
            <InputLabel id="demo-simple-select-label">Actions groupées</InputLabel>
            <Select
              labelId="demo-simple-select-label"
              id="demo-simple-select"
              value={actionGroupe}
              label="Actions groupées"
              onChange={handleChangeActionGroupe}
            >
              <MenuItem value={10} defaultChecked>Actions groupées</MenuItem>
              <MenuItem value={20}>Générer le bordereau</MenuItem>
              <MenuItem value={30}>Télécharger les étiquettes de livraison</MenuItem>
              <MenuItem value={40}>Générer l'étiquette aller</MenuItem>
              <MenuItem value={50}>Générer l'étiquette retour</MenuItem>
              <MenuItem value={60}>Imprimer l'étiquette aller</MenuItem>
              <MenuItem value={70}>Imprimer l'étiquette retour</MenuItem>
              <MenuItem value={80}>Imprimer les étiquettes de livraison</MenuItem>
              <MenuItem value={90}>Supprimer toutes les étiquettes</MenuItem>
            </Select>
          </FormControl>
        </Box>
        <Button variant="outlined" color="success" sx={{ mb: 1 }}>Appliquer</Button>
          <Button variant="outlined" color="success" sx={{ mb: 1, ml: "auto" }}>Mettre à jour les statuts Colissimo</Button>
      </Stack>

      <Card>
        <Box
          sx={{
            width: "100%",
            flexGrow: { md: 1 },
            display: { md: "flex" },
            flexDirection: { md: "column" },
          }}
        >
          <DataGrid
            checkboxSelection
            disableRowSelectionOnClick
            rows={dataFiltered}
            columns={columns}
            getRowHeight={() => "auto"}
            pageSizeOptions={[5, 10, 25]}
            initialState={{
              pagination: {
                paginationModel: { pageSize: 10 },
              },
            }}
            onRowSelectionModelChange={(newSelectionModel) => {
              setSelectedRowIds(newSelectionModel);
            }}
            columnVisibilityModel={columnVisibilityModel}
            onColumnVisibilityModelChange={(newModel) =>
              setColumnVisibilityModel(newModel)
            }
            slots={{
              toolbar: () => (
                <>
                  <GridToolbarContainer>
                    <OrderColissimoTableToolbar
                      filters={filters}
                      onFilters={handleFilters}
                      countryOptions={Country_OPTIONS}
                      shippingMethodOptions={ShippingMethod_OPTIONS}
                      statusOptions={Status_OPTIONS}
                      etiquetteOptions={Etiquette_OPTIONS}
                      dateError={dateError}
                    />

                    <GridToolbarQuickFilter />

                    <Stack
                      spacing={1}
                      flexGrow={1}
                      direction="row"
                      alignItems="center"
                      justifyContent="flex-end"
                    >

                      <GridToolbarColumnsButton />
                    </Stack>
                  </GridToolbarContainer>

                  {canReset && (
                    <OrderColissimoTableFiltersResult
                      filters={filters}
                      onFilters={handleFilters}
                      onResetFilters={handleResetFilters}
                      results={dataFiltered.length}
                      sx={{ p: 2.5, pt: 0 }}
                    />
                  )}
                </>
              ),
              noRowsOverlay: () => <EmptyContent title="Aucune donnée" />,
              noResultsOverlay: () => <EmptyContent title="Aucun résultat trouvé" />,
            }}
            slotProps={{
              columnsPanel: {
                getTogglableColumns,
              },
            }}
          />
        </Box>
      </Card>
    </>
  );
}

// ----------------------------------------------------------------------

function applyFilter({
  inputData,
  filters,
  dateError,

}: {
  inputData: IOrderColissimoItem[];
  filters: IOrderColissimoTableFilters;
  dateError: boolean;
}) {
  const { country, shippingMethod, status, etiquette, startDate, endDate } = filters;

  if (country.length) {
    inputData = inputData.filter((order) =>
      country.includes(order.country)
    );
  }

  if (shippingMethod.length) {
    inputData = inputData.filter((order) =>
      shippingMethod.includes(order.shippingMethod)
    );
  }

  if (status.length) {
    inputData = inputData.filter((order) =>
      status.includes(order.status)
    );
  }

  if (etiquette.length) {
    inputData = inputData.filter((order) =>
      etiquette.includes(order.etiquette)
    );
  }

  if (!dateError) {
    if (startDate && endDate) {
      inputData = inputData.filter((order) => isBetween(order.createdAt, startDate, endDate));
    }
  }

  return inputData;
}
