import { useState, useCallback } from "react";
import MenuItem from "@mui/material/MenuItem";
import Checkbox from "@mui/material/Checkbox";
import Box from "@mui/material/Box";
import InputLabel from "@mui/material/InputLabel";
import FormControl from "@mui/material/FormControl";
import OutlinedInput from "@mui/material/OutlinedInput";
import { DatePicker } from '@mui/x-date-pickers/DatePicker';
import Select, { SelectChangeEvent } from "@mui/material/Select";
import { formHelperTextClasses } from '@mui/material/FormHelperText';
import { IOrderColissimoTableFilters, IOrderColissimoTableFilterValue } from "@/shared/types/shipping";
import { IconButton, Menu, Typography } from '@mui/material';
import { GridFilterAltIcon } from "@mui/x-data-grid";

type Props = {
  filters: IOrderColissimoTableFilters;
  onFilters: (name: string, value: IOrderColissimoTableFilterValue) => void;
  dateError: boolean;
  //
  countryOptions: {
    value: string;
    label: string;
  }[];
  shippingMethodOptions: {
    value: string;
    label: string;
  }[];
  statusOptions: {
    value: string;
    label: string;
  }[];
  etiquetteOptions: {
    value: string;
    label: string;
  }[];
};

export default function OrderColissimoTableToolbar({
  filters,
  onFilters,
  //
  countryOptions,
  shippingMethodOptions,
  statusOptions,
  etiquetteOptions,
  dateError
}: Props) {

  const [country, setCountry] = useState<string[]>(filters.country);

  const [shippingMethod, setShippingMethod] = useState<string[]>(filters.shippingMethod);

  const [status, setStatus] = useState<string[]>(filters.status);

  const [etiquette, setEtiquette] = useState<string[]>(filters.etiquette);

  const handleChangeCountry = useCallback(
    (event: SelectChangeEvent<string[]>) => {
      const {
        target: { value },
      } = event;
      setCountry(typeof value === "string" ? value.split(",") : value);
    },
    []
  );

  const handleChangeShippingMethod = useCallback(
    (event: SelectChangeEvent<string[]>) => {
      const {
        target: { value },
      } = event;
      setShippingMethod(typeof value === "string" ? value.split(",") : value);
    },
    []
  );

  const handleChangeStatus = useCallback(
    (event: SelectChangeEvent<string[]>) => {
      const {
        target: { value },
      } = event;
      setStatus(typeof value === "string" ? value.split(",") : value);
    },
    []
  );

  const handleChangeEtiquette = useCallback(
    (event: SelectChangeEvent<string[]>) => {
      const {
        target: { value },
      } = event;
      setEtiquette(typeof value === "string" ? value.split(",") : value);
    },
    []
  );
  const handleCloseCountry = useCallback(() => {
    onFilters("country", country);
  }, [onFilters, country]);

  const handleCloseShippingMethod = useCallback(() => {
    onFilters("shippingMethod", shippingMethod);
  }, [onFilters, shippingMethod]);

  const handleCloseStatus = useCallback(() => {
    onFilters("status", status);
  }, [onFilters, status]);

  const handleCloseEtiquette = useCallback(() => {
    onFilters("etiquette", etiquette);
  }, [onFilters, etiquette]);

  const handleFilterStartDate = useCallback(
    (newValue: Date | null) => {
      onFilters('startDate', newValue);
    },
    [onFilters]
  );

  const handleFilterEndDate = useCallback(
    (newValue: Date | null) => {
      onFilters('endDate', newValue);
    },
    [onFilters]
  );

  const [anchorEl, setAnchorEl] = useState<HTMLElement | null>(null);

  const handleClick = (event: React.MouseEvent<HTMLButtonElement>) => {
    setAnchorEl(event.currentTarget);
  };

  const handleClose = () => {
    setAnchorEl(null);
  };

  return (
    <>
      <Box>
        <IconButton
          size="small"
          edge="start"
          color="inherit"
          aria-label="menu"
          aria-controls="filter-menu"
          aria-haspopup="true"
          onClick={handleClick}
          sx={{
            borderRadius: 1,
            '&:hover': {
              borderRadius: '10px',
              backgroundColor: 'rgba(0, 0, 0, 0.04)',
            },
          }}
        >
          <GridFilterAltIcon />
          <Typography fontWeight="bold" sx={{ fontSize: "12.5px" }}>Filters</Typography>
        </IconButton>
        <Menu
          id="filter-menu"
          anchorEl={anchorEl}
          open={Boolean(anchorEl)}
          onClose={handleClose}
          MenuListProps={{ onClick: (event) => event.stopPropagation() }}
          anchorOrigin={{
            vertical: 'bottom',
            horizontal: 'left',
          }}
          transformOrigin={{
            vertical: 'top',
            horizontal: 'left',
          }}
        >


          <MenuItem sx={{ display: 'flex', justifyContent: 'space-between' }}>
            <Box sx={{ display: 'grid', gridTemplateColumns: '1fr 1fr', gap: 2, width: '100%' }}>

              <FormControl
                sx={{
                  flexShrink: 0,
                  width: { xs: 1, md: 220 },
                }}
              >
                <InputLabel>Pays</InputLabel>

                <Select
                  multiple
                  value={country}
                  onChange={handleChangeCountry}
                  input={<OutlinedInput label="Pays" />}
                  renderValue={(selected) => selected.map((value) => value).join(", ")}
                  onClose={handleCloseCountry}
                  sx={{ textTransform: "capitalize" }}
                >
                  {countryOptions.map((option) => (
                    <MenuItem key={option.value} value={option.value}>
                      <Checkbox
                        disableRipple
                        size="small"
                        checked={country.includes(option.value)}
                      />
                      {option.label}
                    </MenuItem>
                  ))}
                </Select>
              </FormControl>
              <FormControl
                sx={{
                  flexShrink: 0,
                  width: { xs: 1, md: 220 },
                }}
              >
                <InputLabel>Méthode de livraison</InputLabel>


                <Select
                  multiple
                  value={shippingMethod}
                  onChange={handleChangeShippingMethod}
                  input={<OutlinedInput label="Méthode de livraison" />}
                  renderValue={(selected) => selected.map((value) => value).join(", ")}
                  onClose={handleCloseShippingMethod}
                  sx={{ textTransform: "capitalize" }}
                >
                  {shippingMethodOptions.map((option) => (
                    <MenuItem key={option.value} value={option.value}>
                      <Checkbox
                        disableRipple
                        size="small"
                        checked={shippingMethod.includes(option.value)}
                      />
                      {option.label}
                    </MenuItem>
                  ))}
                </Select>
              </FormControl>
              <FormControl
                sx={{
                  flexShrink: 0,
                  width: { xs: 1, md: 220 },
                }}
              >
                <InputLabel>Status</InputLabel>

                <Select
                  multiple
                  value={status}
                  onChange={handleChangeStatus}
                  input={<OutlinedInput label="Status" />}
                  renderValue={(selected) => selected.map((value) => value).join(", ")}
                  onClose={handleCloseStatus}
                  sx={{ textTransform: "capitalize" }}
                >
                  {statusOptions.map((option) => (
                    <MenuItem key={option.value} value={option.value}>
                      <Checkbox
                        disableRipple
                        size="small"
                        checked={status.includes(option.value)}
                      />
                      {option.label}
                    </MenuItem>
                  ))}
                </Select>
              </FormControl>
              <FormControl
                sx={{
                  flexShrink: 0,
                  width: { xs: 1, md: 220 },
                }}
              >
                <InputLabel>Etiquette</InputLabel>

                <Select
                  multiple
                  value={etiquette}
                  onChange={handleChangeEtiquette}
                  input={<OutlinedInput label="Etiquette" />}
                  renderValue={(selected) => selected.map((value) => value).join(", ")}
                  onClose={handleCloseEtiquette}
                  sx={{ textTransform: "capitalize" }}
                >
                  {etiquetteOptions.map((option) => (
                    <MenuItem key={option.value} value={option.value}>
                      <Checkbox
                        disableRipple
                        size="small"
                        checked={etiquette.includes(option.value)}
                      />
                      {option.label}
                    </MenuItem>
                  ))}
                </Select>
              </FormControl>
              <DatePicker
                label="Date début étiquette aller"
                value={filters.startDate}
                onChange={handleFilterStartDate}
                slotProps={{
                  textField: {
                    fullWidth: true,
                  },
                }}
                sx={{
                  maxWidth: { md: 220 },
                }}
              />
              <DatePicker
                label="Date fin étiquette aller"
                value={filters.endDate}
                onChange={handleFilterEndDate}
                slotProps={{
                  textField: {
                    fullWidth: true,
                    error: dateError,
                    helperText: dateError && 'End date must be later than start date',
                  },
                }}
                sx={{
                  maxWidth: { md: 220 },
                  [`& .${formHelperTextClasses.root}`]: {
                    position: { md: 'absolute' },
                    bottom: { md: -40 },
                  },
                }}
              />
            </Box>
          </MenuItem>
        </Menu>
      </Box>
    </>
  );
}
