"use client";

import isEqual from "lodash/isEqual";
import { useState, useEffect, useCallback } from "react";
import Stack from "@mui/material/Stack";
import Card from "@mui/material/Card";
import Box from "@mui/material/Box";
import {
  DataGrid,
  GridColDef,
  GridToolbarContainer,
  GridRowSelectionModel,
  GridToolbarQuickFilter,
  GridToolbarColumnsButton,
  GridColumnVisibilityModel,
} from "@mui/x-data-grid";
import { paths } from "@/routes/paths";
import { useRouter } from "@/hooks";
import { useBoolean } from "@/hooks/use-boolean";
import { isAfter, isBetween } from "@/utils/format-time";
import { useSnackbar } from "@/shared/components/snackbar";
import EmptyContent from "@/shared/components/empty-content";
import { useSettingsContext } from "@/shared/components/settings";
import CustomBreadcrumbs from "@/shared/components/custom-breadcrumbs";
import { SelectChangeEvent } from "@mui/material/Select";
import {
  RenderCellID,
  RenderCellCreatedAt,
  RenderCellCustomer,
  RenderCellAdress,
  RenderCellCountry,
  RenderCellShippingMethod,
  RenderCellStatus,
  RenderCellTrackingNumber,
} from "./shipment-table-row";
import {
  ShippingMethodType,
  shippingMethodTypeLabels,
} from "@/contexts/types/Shipping";
import { countries } from "@/shared/assets/data";
import {
  Shipment,
  ShipmentStatus,
  shipmentStatusLabels,
  ShipmentTableFilters,
  ShipmentTableFilterValue,
} from "@/contexts/types/shipment";
import ShipmentTableToolbar from "./shipment-table-toolbar";
import ShipmentTableFiltersResult from "./shipment-table-filters-result";
import { getAllShipments } from "@/shared/api/shipment";
import { Alert } from "@mui/material";
import { LoadingScreen } from "@/shared/components/loading-screen";
import { Grid } from "@mui/material";

const Country_OPTIONS = countries
  .filter((country) => country.label && country.code)
  .map((country) => ({
    value: country.label,
    label: country.label,
  }));

const ShippingMethod_OPTIONS = Object.keys(ShippingMethodType).map((key) => ({
  value: ShippingMethodType[key as keyof typeof ShippingMethodType],
  label:
    shippingMethodTypeLabels[
      ShippingMethodType[key as keyof typeof ShippingMethodType]
    ],
}));

const Status_OPTIONS = Object.keys(ShipmentStatus).map((key) => ({
  value: ShipmentStatus[key as keyof typeof ShipmentStatus],
  label:
    shipmentStatusLabels[ShipmentStatus[key as keyof typeof ShipmentStatus]],
}));

const defaultFilters: ShipmentTableFilters = {
  country: [],
  shippingMethod: [],
  status: [],
  startDate: null,
  endDate: null,
};

const HIDE_COLUMNS = {
  category: false,
};

const HIDE_COLUMNS_TOGGLABLE = ["category", "actions"];

export default function ShipmentListView() {
  const { enqueueSnackbar } = useSnackbar();
  const router = useRouter();
  const { shipmentsData, shipmentsError, shipmentsLoading } = getAllShipments();
  const [tableData, setTableData] = useState<Shipment[]>(shipmentsData);
  const [filters, setFilters] = useState(defaultFilters);
  const [selectedRowIds, setSelectedRowIds] = useState<GridRowSelectionModel>(
    []
  );

  const [columnVisibilityModel, setColumnVisibilityModel] =
    useState<GridColumnVisibilityModel>(HIDE_COLUMNS);

  useEffect(() => {
    if (shipmentsData.length) {
      setTableData(shipmentsData);
    }
  }, [shipmentsData]);

  const dateError = isAfter(filters.startDate, filters.endDate);

  const dataFiltered = applyFilter({
    inputData: tableData,
    filters,
    dateError,
  });

  const canReset = !isEqual(defaultFilters, filters);

  const handleFilters = useCallback(
    (name: string, value: ShipmentTableFilterValue) => {
      setFilters((prevState) => ({
        ...prevState,
        [name]: value,
      }));
    },
    []
  );

  const handleResetFilters = useCallback(() => {
    setFilters(defaultFilters);
  }, []);

  const handleViewRow = useCallback(
    (id: string) => {
      router.push(paths.dashboard.admin.shipping.general.shipments.details(id));
    },
    [router]
  );

  const columns: GridColDef[] = [
    {
      field: "id",
      headerName: "ID",
      flex: 1,
      width: 40,
      filterable: true,
      hideable: false,
      renderCell: (params) => <RenderCellID params={params} />,
    },
    {
      field: "createdAt",
      headerName: "Date Création",
      width: 120,
      hideable: false,
      renderCell: (params) => <RenderCellCreatedAt params={params} />,
    },
    {
      field: "customer",
      headerName: "Client",
      minWidth: 190,
      renderCell: (params) => <RenderCellCustomer params={params} />,
    },
    {
      field: "adress",
      headerName: "Adresse",
      minWidth: 180,
      renderCell: (params) => <RenderCellAdress params={params} />,
    },
    {
      field: "country",
      headerName: "Pays",
      minWidth: 100,
      renderCell: (params) => <RenderCellCountry params={params} />,
    },
    {
      field: "shippingMethod",
      headerName: "Méthode de livraison",
      minWidth: 200,
      renderCell: (params) => <RenderCellShippingMethod params={params} />,
    },
    {
      field: "status",
      headerName: "Status",
      minWidth: 110,
      renderCell: (params) => <RenderCellStatus params={params} />,
    },
    {
      field: "trackingNumber",
      headerName: "Numéro de suivi",
      minWidth: 130,
      align: "right",
      renderCell: (params) => <RenderCellTrackingNumber params={params} />,
    },
  ];

  const getTogglableColumns = () =>
    columns
      .filter((column) => !HIDE_COLUMNS_TOGGLABLE.includes(column.field))
      .map((column) => column.field);

  const [actionGroupe, setActionGroupe] = useState("");

  const handleChangeActionGroupe = (event: SelectChangeEvent) => {
    setActionGroupe(event.target.value as string);
  };

  if (shipmentsLoading) {
    return (
      <Grid
        container
        spacing={3}
        sx={{ justifyContent: "center", alignItems: "center", height: "100vh" }}
      >
        <LoadingScreen />
      </Grid>
    );
  }

  if (shipmentsError) {
    return (
      <Grid
        container
        spacing={3}
        sx={{ justifyContent: "center", alignItems: "center", height: "100vh" }}
      >
        <Alert severity="error">
          Problème lors de la récupération des données des expéditions. Veuillez
          réessayer plus tard.
        </Alert>
      </Grid>
    );
  }

  return (
    <>
      <CustomBreadcrumbs
        heading="Liste des expéditions"
        links={[
          { name: "Tableau de bord", href: paths.dashboard.root },
          {
            name: "Livraison",
            href: paths.dashboard.admin.shipping.general.root,
          },
          { name: "Expéditions" },
        ]}
        sx={{
          mb: {
            xs: 3,
            md: 5,
          },
        }}
      />

      <Card>
        <Box
          sx={{
            width: "100%",
            flexGrow: { md: 1 },
            display: { md: "flex" },
            flexDirection: { md: "column" },
          }}
        >
          <DataGrid
            checkboxSelection
            disableRowSelectionOnClick
            rows={dataFiltered}
            columns={columns}
            getRowHeight={() => "auto"}
            pageSizeOptions={[5, 10, 25]}
            initialState={{
              pagination: {
                paginationModel: { pageSize: 10 },
              },
            }}
            onRowSelectionModelChange={(newSelectionModel) => {
              setSelectedRowIds(newSelectionModel);
            }}
            columnVisibilityModel={columnVisibilityModel}
            onColumnVisibilityModelChange={(newModel) =>
              setColumnVisibilityModel(newModel)
            }
            slots={{
              toolbar: () => (
                <>
                  <GridToolbarContainer>
                    <ShipmentTableToolbar
                      filters={filters}
                      onFilters={handleFilters}
                      countryOptions={Country_OPTIONS}
                      shippingMethodOptions={ShippingMethod_OPTIONS}
                      statusOptions={Status_OPTIONS}
                      dateError={dateError}
                    />

                    <GridToolbarQuickFilter />

                    <Stack
                      spacing={1}
                      flexGrow={1}
                      direction="row"
                      alignItems="center"
                      justifyContent="flex-end"
                    >
                      <GridToolbarColumnsButton />
                    </Stack>
                  </GridToolbarContainer>

                  {canReset && (
                    <ShipmentTableFiltersResult
                      filters={filters}
                      onFilters={handleFilters}
                      onResetFilters={handleResetFilters}
                      results={dataFiltered.length}
                      sx={{ p: 2.5, pt: 0 }}
                      shippingMethodOptions={ShippingMethod_OPTIONS}
                      statusOptions={Status_OPTIONS}
                    />
                  )}
                </>
              ),
              noRowsOverlay: () => <EmptyContent title="Aucune donnée" />,
              noResultsOverlay: () => (
                <EmptyContent title="Aucun résultat trouvé" />
              ),
            }}
            slotProps={{
              columnsPanel: {
                getTogglableColumns,
              },
            }}
          />
        </Box>
      </Card>
    </>
  );
}

// ----------------------------------------------------------------------

function applyFilter({
  inputData,
  filters,
  dateError,
}: {
  inputData: Shipment[];
  filters: ShipmentTableFilters;
  dateError: boolean;
}) {
  const { country, shippingMethod, status, startDate, endDate } = filters;

  if (country.length) {
    inputData = inputData.filter((shipment) =>
      country.includes(shipment.shippingCountry)
    );
  }

  if (shippingMethod.length) {
    inputData = inputData.filter((shipment) =>
      shippingMethod.includes(shipment.shippingMethodType)
    );
  }

  if (status.length) {
    inputData = inputData.filter((shipment) =>
      status.includes(shipment.shipmentStatus)
    );
  }

  if (!dateError) {
    if (startDate && endDate) {
      inputData = inputData.filter((shipment) =>
        isBetween(shipment.createdAt, startDate, endDate)
      );
    }
  }

  return inputData;
}
