import { useCallback } from "react";
import Box from "@mui/material/Box";
import Chip from "@mui/material/Chip";
import Paper from "@mui/material/Paper";
import Button from "@mui/material/Button";
import Stack, { StackProps } from "@mui/material/Stack";
import Iconify from "@/shared/components/iconify";
import { ShipmentTableFilters, ShipmentTableFilterValue } from "@/contexts/types/shipment";


type Props = StackProps & {
  filters: ShipmentTableFilters;
  onFilters: (name: string, value: ShipmentTableFilterValue) => void;
  //
  onResetFilters: VoidFunction;
  //
  results: number;
  shippingMethodOptions: {
    value: string;
    label: string;
  }[];
  statusOptions: {
    value: string;
    label: string;
  }[];
};

export default function ShipmentTableFiltersResult({
  filters,
  onFilters,
  //
  onResetFilters,
  //
  results,
  shippingMethodOptions,
  statusOptions,
  ...other
}: Props) {
  const handleRemoveCountry = useCallback(
    (inputValue: string) => {
      const newValue = filters.country.filter((item) => item !== inputValue);

      onFilters("country", newValue);
    },
    [filters.country, onFilters]
  );

  const handleRemoveShippingMethod = useCallback(
    (inputValue: string) => {
      const newValue = filters.shippingMethod.filter((item) => item !== inputValue);

      onFilters("shippingMethod", newValue);
    },
    [filters.shippingMethod, onFilters]
  );

  const handleRemoveStatus = useCallback(
    (inputValue: string) => {
      const newValue = filters.status.filter((item) => item !== inputValue);

      onFilters("status", newValue);
    },
    [filters.status, onFilters]
  );

  return (
    <Stack spacing={1.5} {...other}>
      <Box sx={{ typography: "body2" }}>
        <strong>{results}</strong>
        <Box component="span" sx={{ color: "text.secondary", ml: 0.25 }}>
          results found
        </Box>
      </Box>

      <Stack
        flexGrow={1}
        spacing={1}
        direction="row"
        flexWrap="wrap"
        alignItems="center"
      >
        {!!filters.country.length && (
          <Block label="Pays:">
            {filters.country.map((item) => (
              <Chip
                key={item}
                label={item}
                size="small"
                onDelete={() => handleRemoveCountry(item)}
              />
            ))}
          </Block>
        )}

        {!!filters.shippingMethod.length && (
          <Block label="Méthode d'expédition:">
            {filters.shippingMethod.map((item) => {
              const label = shippingMethodOptions.find((option) => option.value === item)?.label || item;
              return (
                <Chip
                  key={item}
                  label={label}
                  size="small"
                  onDelete={() => handleRemoveShippingMethod(item)}
                />
              );
            })}
          </Block>
        )}

        {!!filters.status.length && (
          <Block label="Status:">
            {filters.status.map((item) => {
              const label = statusOptions.find((option) => option.value === item)?.label || item;
              return (
                <Chip
                  key={item}
                  label={label}
                  size="small"
                  onDelete={() => handleRemoveStatus(item)}
                />
              );
            })}
          </Block>
        )}
        <Button
          color="error"
          onClick={onResetFilters}
          startIcon={<Iconify icon="solar:trash-bin-trash-bold" />}
        >
          Clear
        </Button>
      </Stack>
    </Stack>
  );
}

// ----------------------------------------------------------------------

type BlockProps = StackProps & {
  label: string;
};

function Block({ label, children, sx, ...other }: BlockProps) {
  return (
    <Stack
      component={Paper}
      variant="outlined"
      spacing={1}
      direction="row"
      sx={{
        p: 1,
        borderRadius: 1,
        overflow: "hidden",
        borderStyle: "dashed",
        ...sx,
      }}
      {...other}
    >
      <Box component="span" sx={{ typography: "subtitle2" }}>
        {label}
      </Box>

      <Stack spacing={1} direction="row" flexWrap="wrap">
        {children}
      </Stack>
    </Stack>
  );
}
