import { GridCellParams } from "@mui/x-data-grid";
import ListItemText from "@mui/material/ListItemText";
import { fTime, fDate } from "@/utils/format-time";
import Label from "@/shared/components/label";
import { ShipmentStatus, shipmentStatusLabels } from "@/contexts/types/shipment";
import { useEffect, useState } from "react";
import { IClientItem } from "@/shared/types/user";
import { useGetUserById } from "@/shared/api/user";
import { ShippingMethodType, shippingMethodTypeLabels } from "@/contexts/types/Shipping";

const ShippingMethod_OPTIONS = Object.keys(ShippingMethodType).map((key) => ({
  value: ShippingMethodType[key as keyof typeof ShippingMethodType],
  label: shippingMethodTypeLabels[ShippingMethodType[key as keyof typeof ShippingMethodType]],
}));

const Status_OPTIONS = Object.keys(ShipmentStatus).map((key) => ({
  value: ShipmentStatus[key as keyof typeof ShipmentStatus],
  label: shipmentStatusLabels[ShipmentStatus[key as keyof typeof ShipmentStatus]],
}));

type ParamsProps = {
  params: GridCellParams;
};

export function RenderCellID({ params }: ParamsProps) {
  return <>{params.row.id}</>;
}

export function RenderCellCreatedAt({ params }: ParamsProps) {
  return (
    <ListItemText
      primary={fDate(params.row.createdAt)}
      secondary={fTime(params.row.createdAt)}
      primaryTypographyProps={{ typography: "body2", noWrap: true }}
      secondaryTypographyProps={{
        mt: 0.5,
        component: "span",
        typography: "caption",
      }}
    />
  );
}

export function RenderCellCustomer({ params }: ParamsProps) {
  const { userData, userError } = useGetUserById(Number(params.row.clientId));
  const [clientData, setClientData] = useState<IClientItem | null>(null);

  useEffect(() => {
    if (userData && userData.roles.name === "CLIENT") {
      setClientData(userData as IClientItem);
    }
  }, [userData]);
  return (
    <>
      {clientData ? (
        <ListItemText
          primary={clientData.firstName + ' ' + clientData.lastName}
          secondary={clientData.email}
        />
      ) : (
        <></>
      )}
    </>
  );
}

export function RenderCellAdress({ params }: ParamsProps) {
  return <>{params.row.shippingStreet + ` ` + params.row.shippingHome}</>;
}

export function RenderCellCountry({ params }: ParamsProps) {
  return <>{params.row.shippingCountry}</>;
}

export function RenderCellShippingMethod({ params }: ParamsProps) {
  const shippingMethodLabel = ShippingMethod_OPTIONS.find(
    (option) => option.value === params.row.shippingMethodType
  )?.label || params.row.shippingMethodType;

  return <>{shippingMethodLabel}</>;
}

export function RenderCellStatus({ params }: ParamsProps) {
  const statusLabel = Status_OPTIONS.find(
    (option) => option.value === params.row.shipmentStatus
  )?.label || params.row.shipmentStatus;

  return (
    <Label
      variant="soft"
      color={
        (params.row.shipmentStatus === ShipmentStatus.CANCELED && "error") ||
        (params.row.shipmentStatus === ShipmentStatus.WAITING_FOR_PAYMENT && "info") ||
        (params.row.shipmentStatus === ShipmentStatus.DELIVERED && "success") ||
        (params.row.shipmentStatus === ShipmentStatus.FAILED && "default") ||
        (params.row.shipmentStatus === ShipmentStatus.PENDING && "secondary") ||
        (params.row.shipmentStatus === ShipmentStatus.SHIPPED && "primary") ||
        "default"
      }
    >
      {statusLabel}
    </Label>
  );
}

export function RenderCellTrackingNumber({ params }: ParamsProps) {
  return <>{params.row.trackingNumber}</>;
}