import { useState, useCallback } from "react";
import MenuItem from "@mui/material/MenuItem";
import Checkbox from "@mui/material/Checkbox";
import Box from "@mui/material/Box";
import InputLabel from "@mui/material/InputLabel";
import FormControl from "@mui/material/FormControl";
import OutlinedInput from "@mui/material/OutlinedInput";
import { DatePicker } from '@mui/x-date-pickers/DatePicker';
import Select, { SelectChangeEvent } from "@mui/material/Select";
import { formHelperTextClasses } from '@mui/material/FormHelperText';
import { ShipmentTableFilters, ShipmentTableFilterValue } from "@/contexts/types/shipment";


type Props = {
  filters: ShipmentTableFilters;
  onFilters: (name: string, value: ShipmentTableFilterValue) => void;
  dateError: boolean;
  //
  countryOptions: {
    value: string;
    label: string;
  }[];
  shippingMethodOptions: {
    value: string;
    label: string;
  }[];
  statusOptions: {
    value: string;
    label: string;
  }[];
};

export default function ShipmentTableToolbar({
  filters,
  onFilters,
  //
  countryOptions,
  shippingMethodOptions,
  statusOptions,
  dateError
}: Props) {

  const [country, setCountry] = useState<string[]>(filters.country);

  const [shippingMethod, setShippingMethod] = useState<string[]>(filters.shippingMethod);

  const [status, setStatus] = useState<string[]>(filters.status);

  const handleChangeCountry = useCallback(
    (event: SelectChangeEvent<string[]>) => {
      const {
        target: { value },
      } = event;
      setCountry(typeof value === "string" ? value.split(",") : value);
    },
    []
  );

  const handleChangeShippingMethod = useCallback(
    (event: SelectChangeEvent<string[]>) => {
      const {
        target: { value },
      } = event;
      setShippingMethod(typeof value === "string" ? value.split(",") : value);
    },
    []
  );

  const handleChangeStatus = useCallback(
    (event: SelectChangeEvent<string[]>) => {
      const {
        target: { value },
      } = event;
      setStatus(typeof value === "string" ? value.split(",") : value);
    },
    []
  );

  const handleCloseCountry = useCallback(() => {
    onFilters("country", country);
  }, [onFilters, country]);

  const handleCloseShippingMethod = useCallback(() => {
    onFilters("shippingMethod", shippingMethod);
  }, [onFilters, shippingMethod]);

  const handleCloseStatus = useCallback(() => {
    onFilters("status", status);
  }, [onFilters, status]);

  const handleFilterStartDate = useCallback(
    (newValue: Date | null) => {
      onFilters('startDate', newValue);
    },
    [onFilters]
  );

  const handleFilterEndDate = useCallback(
    (newValue: Date | null) => {
      onFilters('endDate', newValue);
    },
    [onFilters]
  );

  const [anchorEl, setAnchorEl] = useState<HTMLElement | null>(null);

  const handleClick = (event: React.MouseEvent<HTMLButtonElement>) => {
    setAnchorEl(event.currentTarget);
  };

  const handleClose = () => {
    setAnchorEl(null);
  };

  return (
    <>
      <Box>
        <Box
          sx={{
            display: 'grid',
            gridTemplateColumns: 'repeat(5, 1fr)',
            gap: 2,
            width: '100%',
          }}
        >
          <FormControl
            sx={{
              flexShrink: 0,
              width: { xs: 1, md: 220 },
            }}
          >
            <InputLabel>Pays</InputLabel>
            <Select
              multiple
              value={country}
              onChange={handleChangeCountry}
              input={<OutlinedInput label="Pays" />}
              renderValue={(selected) => selected.map((value) => value).join(", ")}
              onClose={handleCloseCountry}
              sx={{ textTransform: "capitalize" }}
            >
              {countryOptions.map((option) => (
                <MenuItem key={option.value} value={option.value}>
                  <Checkbox
                    disableRipple
                    size="small"
                    checked={country.includes(option.value)}
                  />
                  {option.label}
                </MenuItem>
              ))}
            </Select>
          </FormControl>
          <FormControl
            sx={{
              flexShrink: 0,
              width: { xs: 1, md: 220 },
            }}
          >
            <InputLabel>Méthode de livraison</InputLabel>
            <Select
              multiple
              value={shippingMethod}
              onChange={handleChangeShippingMethod}
              input={<OutlinedInput label="Méthode de livraison" />}
              renderValue={(selected) =>
                selected
                  .map((value) => shippingMethodOptions.find((option) => option.value === value)?.label)
                  .join(", ")
              }
              onClose={handleCloseShippingMethod}
              sx={{ textTransform: "capitalize" }}
            >
              {shippingMethodOptions.map((option) => (
                <MenuItem key={option.value} value={option.value}>
                  <Checkbox
                    disableRipple
                    size="small"
                    checked={shippingMethod.includes(option.value)}
                  />
                  {option.label}
                </MenuItem>
              ))}
            </Select>
          </FormControl>
          <FormControl
            sx={{
              flexShrink: 0,
              width: { xs: 1, md: 200 },
            }}
          >
            <InputLabel>Status</InputLabel>
            <Select
              multiple
              value={status}
              onChange={handleChangeStatus}
              input={<OutlinedInput label="Status" />}
              renderValue={(selected) =>
                selected
                  .map((value) => statusOptions.find((option) => option.value === value)?.label)
                  .join(", ")
              }
              onClose={handleCloseStatus}
              sx={{ textTransform: "capitalize" }}
            >
              {statusOptions.map((option) => (
                <MenuItem key={option.value} value={option.value}>
                  <Checkbox
                    disableRipple
                    size="small"
                    checked={status.includes(option.value)}
                  />
                  {option.label}
                </MenuItem>
              ))}
            </Select>
          </FormControl>
          <DatePicker
            label="Date début"
            value={filters.startDate}
            onChange={handleFilterStartDate}
            slotProps={{
              textField: {
                fullWidth: true,
              },
            }}
            sx={{
              maxWidth: { md: 200 },
            }}
          />
          <DatePicker
            label="Date fin"
            value={filters.endDate}
            onChange={handleFilterEndDate}
            slotProps={{
              textField: {
                fullWidth: true,
                error: dateError,
                helperText: dateError && 'La date de fin doit être ultérieure à la date de début.',
              },
            }}
            sx={{
              maxWidth: { md: 200 },
              [`& .${formHelperTextClasses.root}`]: {
                position: { md: 'absolute' },
                bottom: { md: -40 },
              },
            }}
          />
        </Box>
      </Box>
    </>
  );
}
