'use client';

import { useState, useCallback, useEffect } from 'react';
import Card from '@mui/material/Card';
import Table from '@mui/material/Table';
import Stack from '@mui/material/Stack';
import Button from '@mui/material/Button';
import Grid from '@mui/material/Grid';
import Container from '@mui/material/Container';
import TableBody from '@mui/material/TableBody';
import TableContainer from '@mui/material/TableContainer';
import LocalizationProvider from '@/shared/locales/localization-provider';
import { paths } from '@/routes/paths';
import Iconify from '@/shared/components/iconify';
import Scrollbar from '@/shared/components/scrollbar';
import { useSnackbar } from '@/components/snackbar';
import { useSettingsContext } from '@/shared/components/settings';
import CustomBreadcrumbs from '@/components/custom-breadcrumbs';
import { useTable, emptyRows, TableNoData, TablePaginationCustom, TableEmptyRows, TableHeadCustom } from '@/shared/components/table';
import { ShippingClass } from '@/contexts/types/Shipping';
import ShippingClassTableRow from './ShippingClassTableRow';
import * as Yup from 'yup';
import { Controller, useForm } from 'react-hook-form';
import { yupResolver } from '@hookform/resolvers/yup';
import { useMemo } from 'react';
import { Alert, Box, Modal, Snackbar, TextField, Typography } from '@mui/material';
import FormProvider from '@/components/hook-form/form-provider';
import { useUpdateShippingClass, useDeleteShippingClass, useGetShippingClasses, useAddShippingClass } from '@/shared/api/shipping';
import { LoadingScreen } from '@/shared/components/loading-screen';


const TABLE_HEAD = [
    { id: 'name', label: 'Nom', align: 'left', width: '350px' },
    { id: 'code', label: 'Code', align: 'left', width: '350px' },
    { id: 'description', label: 'Description', align: 'left', width: '350px' },
    { id: 'actions', label: 'Actions', align: 'right' },
];


export default function ShippingClassTableView() {
    const { enqueueSnackbar } = useSnackbar();
    const settings = useSettingsContext();
    const table = useTable({ defaultOrderBy: 'title' });
    const { shippingClassData, shippingClassLoading, shippingClassError } = useGetShippingClasses();
    const [tableData, setTableData] = useState<ShippingClass[]>(shippingClassData);

    const denseHeight = table.dense ? 56 : 56 + 20;
    const notFound = (!tableData.length) || !tableData.length;

    const [openAddModal, setOpenAddModal] = useState(false);
    const [openEditModal, setOpenEditModal] = useState(false);
    const [shippingClassEditing, setShippingClassEditing] = useState<ShippingClass>();

    const NewShippingClassSchema = Yup.object().shape({
        name: Yup.string().required('Nom est requis'),
        description: Yup.string().required('Description est requis'),
    });

    const defaultValues = useMemo(() => ({
        name: shippingClassEditing?.name || '',
        description: shippingClassEditing?.description || '',
    }), [shippingClassEditing]);

    const methods = useForm({
        resolver: yupResolver(NewShippingClassSchema),
        defaultValues,
    });

    const {
        reset,
        handleSubmit,
        control
    } = methods;

    useEffect(() => {
        setTableData(shippingClassData);
    }, [shippingClassData])

    useEffect(() => {
        if (shippingClassEditing) {
            reset(defaultValues);
        }
    }, [shippingClassEditing, defaultValues, reset])

    ////////////////////////Show and close Add and Edit Modal//////////////////////////////
    const handleAddClick = () => {
        setShippingClassEditing(undefined);
        methods.reset(defaultValues);
        setOpenAddModal(true);
    };
    const handleEditClick = (id: string) => {
        const shippingClassEdited = tableData.find((c) => c.id === id);
        if (shippingClassEdited) {
            setShippingClassEditing(shippingClassEdited);
            methods.reset(defaultValues);
            setOpenEditModal(true);
        }
    };
    const handleCloseEditModal = () => {
        setOpenEditModal(false);
        setShippingClassEditing(undefined);
        setSelectedShippingClass(null);
        reset();
    };
    const handleCloseAddModal = () => {
        setOpenAddModal(false);
        reset();
    };

    //----------------view details--------------

    const [selectedShippingClass, setSelectedShippingClass] = useState<ShippingClass | null>(null);
    const handelSelectedShippingClass = (shippingClass: any) => {
        setSelectedShippingClass(shippingClass);
    };
    const handleViewRow = useCallback(
        (id: string) => {
            const shippingClass = tableData.find((a) => a.id === id);
            handelSelectedShippingClass(shippingClass);
            setShippingClassEditing(shippingClass);
        },
        [tableData]
    );

    /////////////////////////////CRUD Method///////////////////////////
    const handleAddFormSubmit = (formData: any) => {
        useAddShippingClass(formData)
            .then(() => {
                enqueueSnackbar("Classe d'expédition créée avec succès", { variant: "success" });
                reset();
                setOpenAddModal(false);
                setOpenSaveAlert(true);
            })
            .catch((error) => {
                setOpenErrorAlert(true);
                enqueueSnackbar("Échec de la création de la classe d'expédition", { variant: 'error' });
            })
    };

    const handleEditFormSubmit = (data: any) => {
        if (shippingClassEditing) {
            const formDataEdit = {
                id: shippingClassEditing.id,
                ...data,
            };
            useUpdateShippingClass(shippingClassEditing?.id, formDataEdit).then(() => {
                enqueueSnackbar('Classe d\'expédition modifiée avec succès', { variant: 'success' });
                setOpenEditModal(false);
                reset();
                setOpenSaveAlert(true);
            }).catch((error) => {
                setOpenErrorAlert(true);
                enqueueSnackbar("Échec de la modification de la classe d'expédition", { variant: 'error' });
                console.error(error);
            })
        }
    };

    const handleDeleteRow = useCallback(
        async (id: string) => {
            try {
                await useDeleteShippingClass(id);
                const updatedTableData = tableData.filter((row) => row.id !== id);
                setTableData(updatedTableData);
                table.onUpdatePageDeleteRow(updatedTableData.length);
                setOpenDeleteAlert(true);
            } catch (error) {
                setOpenErrorAlert(true);
                console.error(`Error deleting shipping class with ID ${id}:`, error);
                enqueueSnackbar('Échec de la suppression de la classe d\'expédition', { variant: 'error' });
            }
        },
        [tableData, table, enqueueSnackbar]
    );

    const [openSaveAlert, setOpenSaveAlert] = useState(false);

    const handleCloseSaveAlert = (event?: React.SyntheticEvent | Event, reason?: string) => {
        if (reason === 'clickaway') {
            return;
        }
        setOpenSaveAlert(false);
    };

    const [openDeleteAlert, setOpenDeleteAlert] = useState(false);
    const handleCloseDeleteAlert = (event?: React.SyntheticEvent | Event, reason?: string) => {
        if (reason === 'clickaway') {
            return;
        }
        setOpenDeleteAlert(false);
    };

    const [openErrorAlert, setOpenErrorAlert] = useState(false);
    const handleCloseErrorAlert = (event?: React.SyntheticEvent | Event, reason?: string) => {
        if (reason === 'clickaway') {
            return;
        }
        setOpenErrorAlert(false);
    };

    if (shippingClassLoading) {
        return (
            <Grid container spacing={3} sx={{ justifyContent: 'center', alignItems: 'center', height: '100vh' }}>
                <LoadingScreen />
            </Grid>
        );
    }

    if (shippingClassError) {
        return (
            <Grid container spacing={3} sx={{ justifyContent: 'center', alignItems: 'center', height: '100vh' }}>
                <Alert severity="error">Problème lors de la récupération des données des classes d'expédition. Veuillez réessayer plus tard.</Alert>
            </Grid>
        );
    }

    return (
        <>
            <LocalizationProvider >
                <Container maxWidth={settings.themeStretch ? false : 'lg'}>
                    <Grid container alignItems="center" justifyContent="space-between">
                        <Grid item>
                            <CustomBreadcrumbs
                                heading="Classes d'expédition"
                                links={[
                                    {
                                        name: 'Tableau de bord',
                                        href: paths.dashboard.root,
                                    },
                                    {
                                        name: "Livraison",
                                        href: paths.dashboard.admin.shipping.general.root,
                                    }
                                ]}
                                sx={{
                                    mb: { xs: 3, md: 5 },
                                }}
                            />
                        </Grid>
                        <Grid item>
                            <Stack
                                spacing={3}
                                justifyContent="space-between"
                                alignItems={{ xs: 'flex-end', sm: 'center' }}
                                direction={{ xs: 'column', sm: 'row' }}
                            >
                                <Button
                                    onClick={handleAddClick}
                                    variant="contained"
                                   
                                    startIcon={<Iconify icon="mingcute:add-line" />}
                                >
                                    Ajouter
                                </Button>
                            </Stack>
                        </Grid>
                    </Grid>

                    <Card>
                        <TableContainer sx={{ position: 'relative', overflow: 'unset' }}>
                            <Scrollbar>
                                <Table size={table.dense ? 'small' : 'medium'} sx={{ minWidth: 800 }}>
                                    <TableHeadCustom
                                        order={table.order}
                                        orderBy={table.orderBy}
                                        headLabel={TABLE_HEAD}
                                        rowCount={tableData.length}
                                        numSelected={table.selected.length}
                                        onSort={table.onSort}
                                    />
                                    <TableBody>
                                        {tableData
                                            .slice(
                                                table.page * table.rowsPerPage,
                                                table.page * table.rowsPerPage + table.rowsPerPage
                                            )
                                            .map((row, index) => (
                                                <ShippingClassTableRow
                                                    key={row.id}
                                                    row={row}
                                                    onViewRow={() => handleViewRow(row.id)}
                                                    onEditRow={() => handleEditClick(row.id)}
                                                    onDeleteRow={() => handleDeleteRow(row.id)}
                                                />
                                            ))}

                                        <TableEmptyRows
                                            height={denseHeight}
                                            emptyRows={emptyRows(table.page, table.rowsPerPage, tableData.length)}
                                        />
                                        <TableNoData notFound={notFound} />
                                    </TableBody>
                                </Table>
                            </Scrollbar>
                        </TableContainer>

                        <TablePaginationCustom
                            count={tableData.length}
                            page={table.page}
                            rowsPerPage={table.rowsPerPage}
                            onPageChange={table.onChangePage}
                            onRowsPerPageChange={table.onChangeRowsPerPage}
                            dense={table.dense}
                            onChangeDense={table.onChangeDense}
                        />
                    </Card>
                </Container>
            </LocalizationProvider>

            <Modal open={openAddModal} onClose={handleCloseAddModal}>
                <Box sx={{ position: 'absolute', top: '50%', left: '50%', transform: 'translate(-50%, -50%)', width: 500, bgcolor: 'background.paper', boxShadow: 24, p: 4, borderRadius: 3 }}>
                    <Typography variant="h6" align="center" gutterBottom sx={{ mt: 2, mb: 3 }}>Ajouter une classe d'expédition</Typography>
                    <FormProvider methods={methods} onSubmit={handleSubmit(handleAddFormSubmit)}>
                        <Controller
                            name="name"
                            control={control}
                            render={({ field }) => (
                                <TextField
                                    {...field}
                                    label="Nom"
                                    fullWidth
                                    margin="normal"
                                />
                            )}
                        />
                        <Controller
                            name="description"
                            control={control}
                            render={({ field }) => (
                                <TextField
                                    {...field}
                                    label="Description"
                                    fullWidth
                                    margin="normal"
                                    multiline
                                    rows={4}
                                />
                            )}
                        />
                        <Box sx={{ display: 'flex', justifyContent: 'center', mt: 3 }}>
                            <Button onClick={handleCloseAddModal} sx={{ mt: 2, mb: 2, mr: 1 }} variant="outlined" color="inherit">Annuler</Button>
                            <Button type='submit' sx={{ mt: 2, mb: 2 }} variant="contained" color="inherit">Ajouter</Button>
                        </Box>
                    </FormProvider>
                </Box>
            </Modal >

            <Modal open={openEditModal} onClose={handleCloseEditModal}>
                <Box sx={{ position: 'absolute', top: '50%', left: '50%', transform: 'translate(-50%, -50%)', width: 500, bgcolor: 'background.paper', boxShadow: 24, p: 4, borderRadius: 3 }}>
                    <Typography variant="h6" align="center" gutterBottom sx={{ mt: 2, mb: 3 }}>Modifier une classe d'expédition</Typography>
                    <FormProvider methods={methods} onSubmit={handleSubmit(handleEditFormSubmit)}>
                        <Controller
                            name="name"
                            control={control}
                            render={({ field }) => (
                                <TextField
                                    {...field}
                                    label="Nom"
                                    fullWidth
                                    margin="normal"
                                    defaultValue={shippingClassEditing?.name}
                                />
                            )}
                        />
                        <Controller
                            name="description"
                            control={control}
                            render={({ field }) => (
                                <TextField
                                    {...field}
                                    label="Description"
                                    fullWidth
                                    margin="normal"
                                    multiline
                                    rows={4}
                                    defaultValue={shippingClassEditing?.description}
                                />
                            )}
                        />
                        <Box sx={{ display: 'flex', justifyContent: 'center', mt: 3 }}>
                            <Button onClick={handleCloseEditModal} sx={{ mt: 2, mb: 2, mr: 1 }} variant="outlined" color="inherit">Annuler</Button>
                            <Button type='submit' sx={{ mt: 2, mb: 2 }} variant="contained" color="inherit">Modifier</Button>
                        </Box>
                    </FormProvider>
                </Box>
            </Modal>
            <Snackbar open={openSaveAlert} autoHideDuration={3000} 
                onClose={handleCloseSaveAlert} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
                <Alert
                    onClose={handleCloseSaveAlert}
                    severity="success"
                    variant="filled"
                    sx={{ width: '100%' }}
                >
                    Classe d'expédition enregistrer avec succès
                </Alert>
            </Snackbar>
            <Snackbar open={openDeleteAlert} autoHideDuration={3000} 
                onClose={handleCloseDeleteAlert} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
                <Alert
                    onClose={handleCloseDeleteAlert}
                    severity="success"
                    variant="filled"
                    sx={{ width: '100%' }}
                >
                    Classe d'expédition supprimer avec succès
                </Alert>
            </Snackbar>
            <Snackbar open={openErrorAlert} autoHideDuration={3000}
                onClose={handleCloseErrorAlert} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
                <Alert
                    onClose={handleCloseErrorAlert}
                    severity="error"
                    variant="filled"
                    sx={{ width: '100%' }}
                >
                    Échec de l'enregistrement de la classe d'expédition
                </Alert>
            </Snackbar>
        </>
    );
}
