"use client";

import { useEffect, useState } from 'react';
import * as Yup from 'yup';
import { yupResolver } from '@hookform/resolvers/yup';
import Chip from '@mui/material/Chip';
import Grid from '@mui/material/Unstable_Grid2';
import Card from '@mui/material/Card';
import Stack from '@mui/material/Stack';
import CardHeader from '@mui/material/CardHeader';
import Typography from '@mui/material/Typography';
import Switch from '@mui/material/Switch';
import { AutocompleteChangeReason } from '@mui/material/Autocomplete';
import { useResponsive } from '@/hooks/use-responsive';
import { useSnackbar } from '@/components/snackbar';
import FormProvider, {
    RHFTextField,
    RHFAutocomplete,
} from '@/components/hook-form';
import { paths } from '@/routes/paths';
import { useRouter } from '@/hooks';
import LoadingButton from '@mui/lab/LoadingButton';
import {
    CalculationType,
    DeliveryCostColissimo,
    DeliveryCostMondialRelay,
    DeliveryReductionColissimo,
    MondialRelayMode,
    RequiresFreeShipping,
    ShippingMethodType,
    shippingMethodTypeLabels
} from '@/contexts/types/Shipping';
import DetailsFreeShipping from './DetailsFreeShipping';
import DetailsFlatRate from './DetailsFlatRate';
import DetailsPointOfSale from './DetailsPointOfSale';
import CustomBreadcrumbs from '@/components/custom-breadcrumbs/custom-breadcrumbs';
import {
    useAddColissimo,
    useAddFlatRate,
    useAddFreeShipping,
    useAddMondialRelay,
    useAddPointOfSale,
    useGetShippingMethods
} from '@/shared/api/shipping';
import { Alert, Snackbar } from '@mui/material';
import { Controller, useForm } from 'react-hook-form';
import DetailsColissimo from './DetailsColissimo';
import DetailsMondialRelay from './DetailsMondialRelay';

const flatRateDefaultValues = {
    tvaStatut: false,
    cost: 0,
    costHeavyShippingClass: 0,
    costNoShippingClass: 0,
    calculationType: CalculationType.BY_CLASS,
};

const flatRateValidationSchema = Yup.object().shape({
    tvaStatut: Yup.boolean().required('Le statut TVA est requis'),
    cost: Yup.number().min(0, 'Le coût doit être positif').required(),
    costHeavyShippingClass: Yup.number().min(0, 'Le coût doit être positif'),
    costNoShippingClass: Yup.number().min(0, 'Le coût doit être positif'),
    calculationType: Yup.mixed().oneOf(Object.values(CalculationType)).required(),
});

const freeShippingDefaultValues = {
    requires: RequiresFreeShipping.NOTHING,
    minOrderAmount: 0,
    applyMinOrderRuleBeforePromoCodeDiscount: false,
};

const freeShippingValidationSchema = Yup.object().shape({
    requires: Yup.mixed().oneOf(Object.values(RequiresFreeShipping)).required(),
    minOrderAmount: Yup.number().min(0, 'Le montant minimum de la commande doit être positif').required(),
    applyMinOrderRuleBeforePromoCodeDiscount: Yup.boolean().required(),
});

const mondialRelayDefaultValues = {
    amountFreeShipping: 0,
    nameFreeShipping: '',
    modeMondialRelay: MondialRelayMode.MED,
    deliveryCostMondialRelay: [],
};

const mondialRelayValidationSchema = Yup.object().shape({
    amountFreeShipping: Yup.number().min(0, 'Le montant doit être positif'),
    nameFreeShipping: Yup.string(),
    modeMondialRelay: Yup.mixed().oneOf(Object.values(MondialRelayMode)).required(),
    deliveryCostMondialRelay: Yup.array()
        .of(
            Yup.object().shape({
                weightCart: Yup.number().min(0, 'Le poids doit être positif').required(),
                weightPackaging: Yup.number().min(0, 'Le poids doit être positif'),
                cost: Yup.number().min(0, 'Le coût doit être positif').required(),
            })
        )
        .required('Les coûts de livraison sont requis'),
});

const pointOfSaleDefaultValues = {
    tvaStatut: false,
    cost: 0,
};

const pointOfSaleValidationSchema = Yup.object().shape({
    tvaStatut: Yup.boolean().required('Le statut TVA est requis'),
    cost: Yup.number().min(0, 'Le coût doit être positif').required(),
});

const colissimoDefaultValues = {
    signature: false,
    alwaysFree: false,
    nameFreeShipping: '',
    shippingClassExcluded: [],
    freeShippingClasses: [],
    freeIfOneProductHaveFreeShippingClass: false,
    deliveryCostList: [],
    deliveryReductionList: [],
};

const colissimoValidationSchema = Yup.object().shape({
    signature: Yup.boolean().required('La signature est requise'),
    alwaysFree: Yup.boolean(),
    nameFreeShipping: Yup.string(),
    shippingClassExcluded: Yup.array().of(Yup.string()),
    freeShippingClasses: Yup.array().of(Yup.string()),
    freeIfOneProductHaveFreeShippingClass: Yup.boolean().required('Cette règle est requise'),
    deliveryCostList: Yup.array()
        .of(
            Yup.object().shape({
                id: Yup.string().nullable(),
                fromWeight: Yup.number().nullable().min(0, 'Le poids doit être positif'),
                toWeight: Yup.number().nullable().min(0, 'Le poids doit être positif'),
                fromPrice: Yup.number().nullable().min(0, 'Le prix doit être positif'),
                toPrice: Yup.number().nullable().min(0, 'Le prix doit être positif'),
                shippingClassId: Yup.string().nullable(),
                cost: Yup.number().nullable().min(0, 'Le coût doit être positif'),
            })
        ),
    deliveryReductionList: Yup.array()
        .of(
            Yup.object().shape({
                id: Yup.string().nullable(),
                nbrProducts: Yup.number().nullable().min(0, 'Le nombre de produits doit être positif'),
                percentageReduction: Yup.number().nullable().min(0, 'La réduction en pourcentage doit être positive'),
            })
        )
});

const getDefaultValues = (selectedType: ShippingMethodType) => {
    switch (selectedType) {
        case ShippingMethodType.FreeShipping:
            return {
                ...freeShippingDefaultValues,
                name: '',
                description: '',
                active: false,
                type: ShippingMethodType.FreeShipping,
            };
        case ShippingMethodType.FlatRate:
            return {
                ...flatRateDefaultValues,
                name: '',
                description: '',
                active: false,
                type: ShippingMethodType.FlatRate,
            };
        case ShippingMethodType.MondialRelay:
            return {
                ...mondialRelayDefaultValues,
                name: '',
                description: '',
                active: false,
                type: ShippingMethodType.MondialRelay,
            };
        case ShippingMethodType.PointOfSale:
            return {
                ...pointOfSaleDefaultValues,
                name: '',
                description: '',
                active: false,
                type: ShippingMethodType.PointOfSale,
            };
        case ShippingMethodType.ColissimoWithoutSignature:
        case ShippingMethodType.ColissimoWithSignature:
            return {
                ...colissimoDefaultValues,
                name: '',
                description: '',
                active: false,
                type: selectedType,
            };
        default:
            return {
                name: '',
                description: '',
                active: false,
                type: selectedType,
            };
    }
};

const getValidationSchema = (selectedType: ShippingMethodType) => {
    switch (selectedType) {
        case ShippingMethodType.FreeShipping:
            return Yup.object().shape({
                ...freeShippingValidationSchema.fields,
                name: Yup.string().required('Le nom est requis'),
                description: Yup.string().required('La description est requise'),
                active: Yup.boolean().required("L'activité est requise"),
                type: Yup.mixed().oneOf(Object.values(ShippingMethodType)).required(),
            });
        case ShippingMethodType.FlatRate:
            return Yup.object().shape({
                ...flatRateValidationSchema.fields,
                name: Yup.string().required('Le nom est requis'),
                description: Yup.string().required('La description est requise'),
                active: Yup.boolean().required("L'activité est requise"),
                type: Yup.mixed().oneOf(Object.values(ShippingMethodType)).required(),
            });
        case ShippingMethodType.MondialRelay:
            return Yup.object().shape({
                ...mondialRelayValidationSchema.fields,
                name: Yup.string().required('Le nom est requis'),
                description: Yup.string().required('La description est requise'),
                active: Yup.boolean().required("L'activité est requise"),
                type: Yup.mixed().oneOf(Object.values(ShippingMethodType)).required(),
            });
        case ShippingMethodType.PointOfSale:
            return Yup.object().shape({
                ...pointOfSaleValidationSchema.fields,
                name: Yup.string().required('Le nom est requis'),
                description: Yup.string().required('La description est requise'),
                active: Yup.boolean().required("L'activité est requise"),
                type: Yup.mixed().oneOf(Object.values(ShippingMethodType)).required(),
            });
        case ShippingMethodType.ColissimoWithoutSignature:
        case ShippingMethodType.ColissimoWithSignature:
            return Yup.object().shape({
                ...colissimoValidationSchema.fields,
                name: Yup.string().required('Le nom est requis'),
                description: Yup.string().required('La description est requise'),
                active: Yup.boolean().required("L'activité est requise"),
                type: Yup.mixed().oneOf(Object.values(ShippingMethodType)).required(),
            });
        default:
            return Yup.object().shape({
                name: Yup.string().required('Le nom est requis'),
                description: Yup.string().required('La description est requise'),
                active: Yup.boolean().required("L'activité est requise"),
                type: Yup.mixed().oneOf(Object.values(ShippingMethodType)).required(),
            });
    }
};

export default function CreateShippingMethodView() {
    const mdUp = useResponsive('up', 'md');
    const { enqueueSnackbar } = useSnackbar();
    const router = useRouter();

    const { mutateShippingMethods } = useGetShippingMethods();

    const shippingMethodTypes = Object.values(ShippingMethodType);

    const [selectedType, setSelectedType] = useState(ShippingMethodType.FreeShipping);
    const [selectedValueActive, setSelecetedValueActive] = useState(false);
    const [dataDeliveryCostColissimo, setDataDeliveryCostColissimo] = useState<DeliveryCostColissimo[]>([]);
    const [dataDeliveryReductionColissimo, setDataDeliveryReductionColissimo] = useState<DeliveryReductionColissimo[]>([]);
    const [dataDeliveryCostMondialRelay, setDataDeliveryCostMondialRelay] = useState<DeliveryCostMondialRelay[]>([]);

    const [alertErrorMessage, setAlertErrorMessage] = useState('');
    const [openAlertErrorMessage, setOpenAlertErrorMessage] = useState(false);

    const [isActive, setIsActive] = useState(false);


    const defaultValues = getDefaultValues(selectedType);
    const validationSchema = getValidationSchema(selectedType);

    const methods = useForm({
        resolver: yupResolver(validationSchema),
        defaultValues,
    });

    const {
        watch,
        control,
        setValue,
        handleSubmit,
        formState: { isSubmitting },
        register,
        getValues,
    } = methods;

    const values = watch();

    const [openSaveAlert, setOpenSaveAlert] = useState(false);

    const handleClickSaveAlert = () => {
        setOpenSaveAlert(true);
    };

    const handleCloseSaveAlert = (event?: React.SyntheticEvent | Event, reason?: string) => {
        if (reason === 'clickaway') {
            return;
        }

        setOpenSaveAlert(false);
    };

    const [openErrorAlert, setOpenErrorAlert] = useState(false);

    const handleClickErrorAlert = () => {
        setOpenErrorAlert(true);
    };

    const handleCloseErrorAlert = (event?: React.SyntheticEvent | Event, reason?: string) => {
        if (reason === 'clickaway') {
            return;
        }

        setOpenErrorAlert(false);
    };

    const onSubmit = handleSubmit((data: any) => {
        data.deliveryCostColissimoList = dataDeliveryCostColissimo;
        data.deliveryReductionColissimoList = dataDeliveryReductionColissimo;
        data.deliveryCostMondialRelayList = dataDeliveryCostMondialRelay;
        console.log("mode: " + data.mondialRelayMode)

        switch (selectedType) {
            case ShippingMethodType.FreeShipping:
                useAddFreeShipping(data).then(() => {
                    router.push(paths.dashboard.admin.shipping.general.shippingMethod.shippingMethodList)
                    mutateShippingMethods();
                    setOpenSaveAlert(true);
                    enqueueSnackbar('Méthode de livraison gratuite ajoutée avec succès', { variant: 'success' });
                }).catch((error) => {
                    setOpenErrorAlert(true);
                    enqueueSnackbar(`Erreur lors de l'ajout de la méthode de livraison gratuite : ${error.message}`, { variant: 'error' });
                });
                break;
            case ShippingMethodType.FlatRate:
                useAddFlatRate(data).then(() => {
                    router.push(paths.dashboard.admin.shipping.general.shippingMethod.shippingMethodList)
                    mutateShippingMethods();
                    setOpenSaveAlert(true);
                    enqueueSnackbar('Méthode FlatRate ajoutée avec succès', { variant: 'success' });
                }).catch((error) => {
                    setOpenErrorAlert(true);
                    enqueueSnackbar(`Erreur lors de l'ajout de la méthode FlatRate : ${error.message}`, { variant: 'error' });
                });
                break;
            case ShippingMethodType.MondialRelay:
                if (dataDeliveryCostMondialRelay.length === 0) {
                    setAlertErrorMessage(`Liste des frais d'expédition est vide`);
                    setOpenAlertErrorMessage(true);
                    return;
                }
                useAddMondialRelay(data).then(() => {
                    router.push(paths.dashboard.admin.shipping.general.shippingMethod.shippingMethodList)
                    mutateShippingMethods();
                    setOpenSaveAlert(true);
                    enqueueSnackbar('Méthode MondialRelay ajoutée avec succès', { variant: 'success' });
                }).catch((error) => {
                    setOpenErrorAlert(true);
                    enqueueSnackbar(`Erreur lors de l'ajout de la méthode MondialRelay : ${error.message}`, { variant: 'error' });
                });
                break;
            case ShippingMethodType.PointOfSale:
                useAddPointOfSale(data).then(() => {
                    router.push(paths.dashboard.admin.shipping.general.shippingMethod.shippingMethodList)
                    mutateShippingMethods();
                    setOpenSaveAlert(true);
                    enqueueSnackbar('Méthode PointOfSale ajoutée avec succès', { variant: 'success' });
                }).catch((error) => {
                    setOpenErrorAlert(true);
                    enqueueSnackbar(`Erreur lors de l'ajout de la méthode PointOfSale : ${error.message}`, { variant: 'error' });
                });
                break;
            case ShippingMethodType.ColissimoWithSignature:
                if (dataDeliveryCostColissimo.length === 0) {
                    setAlertErrorMessage(`Liste des frais d'expédition est vide`);
                    setOpenAlertErrorMessage(true);
                    return;
                }
                useAddColissimo(data, true).then(() => {
                    router.push(paths.dashboard.admin.shipping.general.shippingMethod.shippingMethodList)
                    mutateShippingMethods();
                    setOpenSaveAlert(true);
                    enqueueSnackbar('Méthode ColissimoWithSignature ajoutée avec succès', { variant: 'success' });
                }).catch((error) => {
                    setOpenErrorAlert(true);
                    enqueueSnackbar(`Erreur lors de l'ajout de la méthode ColissimoWithSignature : ${error.message}`, { variant: 'error' });
                });
                break;
            case ShippingMethodType.ColissimoWithoutSignature:
                if (dataDeliveryCostColissimo.length === 0) {
                    setAlertErrorMessage(`Liste des frais d'expédition est vide`);
                    setOpenAlertErrorMessage(true);
                    return;
                }
                useAddColissimo(data, false).then(() => {
                    router.push(paths.dashboard.admin.shipping.general.shippingMethod.shippingMethodList)
                    mutateShippingMethods();
                    setOpenSaveAlert(true);
                    enqueueSnackbar('Méthode ColissimoWithoutSignature ajoutée avec succès', { variant: 'success' });
                }).catch((error) => {
                    setOpenErrorAlert(true);
                    enqueueSnackbar(`Erreur lors de l'ajout de la méthode ColissimoWithoutSignature : ${error.message}`, { variant: 'error' });
                });
                break;
            default:
                break;
        }
        router.push(paths.dashboard.admin.shipping.general.root)
    });

    const handleChangeType = (
        event: React.SyntheticEvent,
        value: string | string[],
        reason: AutocompleteChangeReason,
        onChange: any
    ) => {
        const newValue = Array.isArray(value) ? value[0] : value;

        if (Object.values(ShippingMethodType).includes(newValue as ShippingMethodType)) {
            const validatedValue = newValue as ShippingMethodType;
            setSelectedType(validatedValue);
            setIsActive(false);
            setValue('type', validatedValue);
            setValue('active', false);
            methods.reset({
                ...getDefaultValues(validatedValue),
                active: false,
                type: validatedValue,
            });
            onChange?.(validatedValue);
        } else {
            console.error('Invalid shipping method type:', newValue);
        }
    };

    const renderBasic = (
        <>
            <Grid md={12}>
                <CustomBreadcrumbs
                    heading="Ajouter une méthode de livraison"
                    links={[
                        { name: 'Tableau de bord', href: paths.dashboard.root },
                        { name: "Méthodes de livraison", href: paths.dashboard.admin.shipping.general.shippingMethod.shippingMethodList },
                        { name: "Nouvelle méthode de livraison" },
                    ]}
                    sx={{
                        mb: { xs: 3, md: 5 },
                    }}
                />
            </Grid>

            {mdUp && (
                <Grid md={4}>
                    <Typography variant="body2" sx={{ color: 'text.secondary' }}>
                        Saisissez les informations de la nouvelle méthode de livraison : nom de la méthode, son type et d'autres informations sa dépendent de chaque type.
                    </Typography>
                </Grid>
            )}

            <Grid xs={12} md={8}>
                <Card>
                    {!mdUp && <CardHeader title="Nouvelle méthode de livraison" />}

                    <Stack spacing={3} sx={{ p: 3 }}>
                        <Controller
                            name="type"
                            control={control}
                            render={({ field }) => (
                                <RHFAutocomplete
                                    {...field}
                                    label="Type du méthode"
                                    placeholder="Type du méthode"
                                    disableClearable
                                    options={Object.values(ShippingMethodType)}
                                    getOptionLabel={(option) => shippingMethodTypeLabels[option as ShippingMethodType]}
                                    onChange={(event, value, reason) => handleChangeType(event, value, reason, field.onChange)}
                                    value={selectedType}
                                    renderOption={(props, option) => (
                                        <li {...props} key={option}>
                                            {shippingMethodTypeLabels[option as ShippingMethodType]}
                                        </li>
                                    )}
                                    renderTags={(selected, getTagProps) =>
                                        selected.map((option, index) => (
                                            <Chip
                                                {...getTagProps({ index })}
                                                key={option}
                                                label={shippingMethodTypeLabels[option as ShippingMethodType]}
                                                size="small"
                                                color="success"
                                                variant="soft"
                                            />
                                        ))
                                    }
                                />
                            )}
                        />
                        <Controller
                            name="name"
                            control={control}
                            render={({ field }) => (
                                <RHFTextField {...field} label="Nom de la méthode" />
                            )}
                        />
                        <Controller
                            name="description"
                            control={control}
                            render={({ field }) => (
                                <RHFTextField {...field} label="Description" placeholder="Description..." multiline rows={4} />
                            )}
                        />
                        <Controller
                            name="active"
                            control={control}
                            defaultValue={isActive}
                            render={({ field }) => (
                                <div style={{ display: 'flex', alignItems: 'center' }}>
                                    <label>Activé</label>
                                    <Switch
                                        {...field}
                                        checked={isActive}
                                        onChange={(e) => {
                                            setIsActive(e.target.checked);
                                            field.onChange(e.target.checked);
                                        }}
                                    />
                                </div>
                            )}
                        />
                    </Stack>
                </Card>
            </Grid>
            <Snackbar open={openSaveAlert} autoHideDuration={2500}
                onClose={handleCloseSaveAlert} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
                <Alert
                    onClose={handleCloseSaveAlert}
                    severity="success"
                    variant="filled"
                    sx={{ width: '100%' }}
                >
                    Méthode de livraison enregistrer avec succès
                </Alert>
            </Snackbar>
            <Snackbar open={openErrorAlert} autoHideDuration={3000}
                onClose={handleCloseErrorAlert} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
                <Alert
                    onClose={handleCloseErrorAlert}
                    severity="error"
                    variant="filled"
                    sx={{ width: '100%' }}
                >
                    Échec
                </Alert>
            </Snackbar>
            <Snackbar open={openAlertErrorMessage} autoHideDuration={3000}
                onClose={() => setOpenAlertErrorMessage(false)} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
                <Alert
                    onClose={() => setOpenAlertErrorMessage(false)}
                    severity="error"
                    variant="filled"
                    sx={{ width: '100%' }}
                >
                    {alertErrorMessage}
                </Alert>
            </Snackbar>
        </>
    );

    const renderActions = (
        <>
            {mdUp && <Grid md={4} />}
            <Grid xs={12} md={8} sx={{ display: 'flex', justifyContent: 'end', py: 4 }}>
                <LoadingButton
                    type="submit"
                    variant="contained"
                    size="large"
                    loading={isSubmitting}
                    sx={{ ml: 2 }}
                >
                    Enregistrer
                </LoadingButton>
            </Grid>
        </>
    );

    const renderDetails = () => {
        switch (selectedType) {
            case ShippingMethodType.FreeShipping:
                return (
                    <>
                        <DetailsFreeShipping control={control} />
                        {renderActions}
                    </>
                );
            case ShippingMethodType.FlatRate:
                return (
                    <>
                        <DetailsFlatRate control={control} />
                        {renderActions}
                    </>
                );
            case ShippingMethodType.MondialRelay:
                return (
                    <>
                        <DetailsMondialRelay
                            control={control}
                            dataDeliveryCostMondialRelay={dataDeliveryCostMondialRelay}
                            setDataDeliveryCostMondialRelay={setDataDeliveryCostMondialRelay}
                        />
                        {renderActions}
                    </>
                );
            case ShippingMethodType.PointOfSale:
                return (
                    <>
                        <DetailsPointOfSale control={control} />
                        {renderActions}
                    </>
                );
            case ShippingMethodType.ColissimoWithoutSignature:
            case ShippingMethodType.ColissimoWithSignature:
                return (
                    <>
                        <DetailsColissimo
                            control={control}
                            dataListCostDelivery={dataDeliveryCostColissimo}
                            setDataListCostDelivery={setDataDeliveryCostColissimo}
                            dataListReductionDelivery={dataDeliveryReductionColissimo}
                            setDataListReductionDelivery={setDataDeliveryReductionColissimo}
                        />
                        {renderActions}
                    </>
                );
            default:
                return null;
        }
    };

    return (
        <FormProvider methods={methods} onSubmit={onSubmit}>
            <Grid container spacing={3}>
                {renderBasic}
                {renderDetails()}
            </Grid>
        </FormProvider>
    );
}
