"use client"

import { useEffect, useState } from 'react';
import * as Yup from 'yup';
import { Controller, useForm } from 'react-hook-form';
import { yupResolver } from '@hookform/resolvers/yup';
import { useMemo } from 'react';
import Grid from '@mui/material/Unstable_Grid2';
import Card from '@mui/material/Card';
import Stack from '@mui/material/Stack';
import Chip from '@mui/material/Chip';
import Button from '@mui/material/Button';
import CardHeader from '@mui/material/CardHeader';
import Typography from '@mui/material/Typography';
import Switch from '@mui/material/Switch';
import Box from '@mui/material/Box';
import DeleteIcon from '@mui/icons-material/Delete';
import InputAdornment from '@mui/material/InputAdornment';
import Table from '@mui/material/Table';
import TableBody from '@mui/material/TableBody';
import EditIcon from '@mui/icons-material/Edit';
import TableCell from '@mui/material/TableCell';
import AddIcon from '@mui/icons-material/Add';
import TableContainer from '@mui/material/TableContainer';
import TableHead from '@mui/material/TableHead';
import TableRow from '@mui/material/TableRow';
import Paper from '@mui/material/Paper';
import { useResponsive } from '@/hooks/use-responsive';
import { useSnackbar } from '@/components/snackbar';
import FormProvider, {RHFTextField} from '@/components/hook-form';
import { useRouter } from '@/hooks';
import { paths } from '@/routes/paths';
import LoadingButton from '@mui/lab/LoadingButton';
import { DeliveryCostColissimo, DeliveryReductionColissimo } from '@/contexts/types/Shipping'
import { Alert, Autocomplete, Checkbox, Container, IconButton, Modal, Snackbar, TextField } from '@mui/material';
import CustomBreadcrumbs from '@/shared/components/custom-breadcrumbs/custom-breadcrumbs';
import ConfirmDialog from '@/shared/components/confirm-dialog';
import { useBoolean } from '@/hooks';
import { useGetColissimo, useGetShippingClasses, useGetShippingMethods, useUpdateColissimo } from '@/shared/api/shipping';


const modalStyle = {
    position: 'absolute',
    top: '50%',
    left: '50%',
    transform: 'translate(-50%, -50%)',
    width: 600,
    bgcolor: 'background.paper',
    boxShadow: 24,
    p: 4,
    borderRadius: 3,
};

const modalContentStyle = {
    display: 'flex',
    flexDirection: 'column',
    gap: 2,
};

type Props = {
    id: string;
}


export default function EditColissimoWithSignature({ id }: Props) {

    const { shippingClassData } = useGetShippingClasses();

    const getShippingClassById = (id: string): string | null => {
        const shippingClass = shippingClassData.find((shippingClass) => shippingClass.id === id);
        return shippingClass ? shippingClass.name : null;
    }

    const { shippingMethod, mutateShippingMethod } = useGetColissimo(id);
    const { mutateShippingMethods } = useGetShippingMethods();

    const mdUp = useResponsive('up', 'md');
    const router = useRouter();
    const { enqueueSnackbar } = useSnackbar();

    const [isActive, setIsActive] = useState(false);

    const [dataDeliveryCostColissimo, setDataDeliveryCostColissimo] = useState<DeliveryCostColissimo[]>(shippingMethod?.deliveryCostColissimoList || []);
    const [dataDeliveryReductionColissimo, setDataDeliveryReductionColissimo] = useState<DeliveryReductionColissimo[]>(shippingMethod?.deliveryReductionColissimoList || []);

    const NewShippingMethodSchema = Yup.object().shape({
        name: Yup.string().required('Nom est requis'),
        description: Yup.string().required('Description est requis'),
        active: Yup.boolean().required('Le statut actif est requis'),
        signature: Yup.boolean().required('La signature est requise'),
        alwaysFree: Yup.boolean(),
        nameFreeShipping: Yup.string(),
        shippingClassExcludedList: Yup.array(),
        freeShippingClassList: Yup.array(),
        freeIfOneProductHaveFreeShippingClass: Yup.boolean(),
        deliveryCostColissimoList: Yup.array()
            .of(
                Yup.object().shape({
                    id: Yup.string().nullable(),
                    fromWeight: Yup.number().nullable().min(0, 'Le poids doit être positif'),
                    toWeight: Yup.number().nullable().min(0, 'Le poids doit être positif'),
                    fromPrice: Yup.number().nullable().min(0, 'Le prix doit être positif'),
                    toPrice: Yup.number().nullable().min(0, 'Le prix doit être positif'),
                    shippingClassId: Yup.string().nullable(),
                    cost: Yup.number().nullable().min(0, 'Le coût doit être positif'),
                })
            ),
        deliveryReductionColissimoList: Yup.array()
            .of(
                Yup.object().shape({
                    id: Yup.string().nullable(),
                    nbrProducts: Yup.number().nullable().min(0, 'Le nombre de produits doit être positif'),
                    percentageReduction: Yup.number().nullable().min(0, 'La réduction en pourcentage doit être positive'),
                })
            ),
    });

    const defaultValues = useMemo(() => ({
        name: shippingMethod?.name || '',
        description: shippingMethod?.description || '',
        active: shippingMethod?.active || false,
        signature: shippingMethod?.signature || false,
        alwaysFree: shippingMethod?.alwaysFree || false,
        nameFreeShipping: shippingMethod?.nameFreeShipping || '',
        shippingClassExcludedList: shippingMethod?.shippingClassExcludedList || [],
        freeShippingClassList: shippingMethod?.freeShippingClassList || [],
        freeIfOneProductHaveFreeShippingClass: shippingMethod?.freeIfOneProductHaveFreeShippingClass || false,
        deliveryCostColissimoList: shippingMethod?.deliveryCostColissimoList || [],
        deliveryReductionColissimoList: shippingMethod?.deliveryReductionColissimoList || [],
    }), [shippingMethod]);

    const methods = useForm({
        resolver: yupResolver(NewShippingMethodSchema),
        defaultValues,
    });

    const {
        watch,
        control,
        setValue,
        handleSubmit,
        formState: { isSubmitting },
        reset,
    } = methods;

    const values = watch();

    const onSubmit = handleSubmit((data: any) => {
        data.id = id;
        data.deliveryCostColissimoList = dataDeliveryCostColissimo;
        data.deliveryReductionColissimoList = dataDeliveryReductionColissimo;
        useUpdateColissimo(data.id, data).then(() => {
            mutateShippingMethod();
            mutateShippingMethods();
            router.push(paths.dashboard.admin.shipping.general.shippingMethod.shippingMethodList);
            setOpenSaveAlert(true);
        }).catch((error) => {
            setOpenErrorAlert(true);
            enqueueSnackbar(`Erreur lors de l'ajout de la méthode de livraison gratuite : ${error.message}`, { variant: 'error' });
        });
    });

    const [checkedStillFree, setCheckedStillFree] = useState(false);
    const [checkedConditionFree, setCheckedConditionFree] = useState(false);
    const [formDataCost, setFormDataCost] = useState<DeliveryCostColissimo>({
        id: null,
        fromWeight: null,
        toWeight: null,
        fromPrice: null,
        toPrice: null,
        shippingClassId: null,
        cost: null,
    });
    const [formDataReduction, setFormDataReduction] = useState<DeliveryReductionColissimo>({
        id: null,
        nbrProducts: null,
        percentageReduction: null,
    });

    const [openAddCostModal, setOpenAddCostModal] = useState(false);
    const [openEditCostModal, setOpenEditCostModal] = useState(false);
    const [openAddReductionModal, setOpenAddReductionModal] = useState(false);
    const [openEditReductionModal, setOpenEditReductionModal] = useState(false);

    const [selectedFileCost, setSelectedFileCost] = useState<File | null>(null);
    const handleFileCostInputChange = (event: React.ChangeEvent<HTMLInputElement>) => {
        const file = event.target.files?.[0];
        if (file && file.type === 'text/csv') {
            setSelectedFileCost(file);
        } else {
            setSelectedFileCost(null);
        }
    };

    useEffect(() => {
        if (shippingMethod) {
            reset(defaultValues);
            setIsActive(defaultValues.active)
            setDataDeliveryCostColissimo(defaultValues.deliveryCostColissimoList)
            setDataDeliveryReductionColissimo(defaultValues.deliveryReductionColissimoList)
        }
    }, [shippingMethod, defaultValues, reset])

    const confirmDeleteCost = useBoolean();
    const confirmDeleteReduction = useBoolean();

    const handleChangeStillFree = (event: React.ChangeEvent<HTMLInputElement>) => {
        setCheckedStillFree(event.target.checked);
    };

    const handleChangeConditionFree = (event: React.ChangeEvent<HTMLInputElement>) => {
        setCheckedConditionFree(event.target.checked);
    };

    const handleAddCostClick = () => {
        setFormDataCost({
            id: null,
            fromWeight: null,
            toWeight: null,
            fromPrice: null,
            toPrice: null,
            shippingClassId: null,
            cost: null,
        });
        setOpenAddCostModal(true);
    };

    const handleEditCostClick = (index: number) => {
        const row = dataDeliveryCostColissimo[index];
        if (row) {
            setFormDataCost({
                id: row.id,
                fromWeight: row.fromWeight,
                toWeight: row.toWeight,
                fromPrice: row.fromPrice,
                toPrice: row.toPrice,
                shippingClassId: row.shippingClassId,
                cost: row.cost,
            });
        }
        setOpenEditCostModal(true);
    };

    const handleAddReductionClick = () => {
        setFormDataReduction({
            id: null,
            nbrProducts: null,
            percentageReduction: null,
        });
        setOpenAddReductionModal(true);
    };

    const handleEditReductionClick = (index: number) => {
        const row = dataDeliveryReductionColissimo[index];
        if (row) {
            setFormDataReduction({
                id: row.id,
                nbrProducts: row.nbrProducts,
                percentageReduction: row.percentageReduction,
            });
        }
        setOpenEditReductionModal(true);
    };

    const handleSaveCost = () => {
        if (formDataCost.id !== null) {
            const updatedList = dataDeliveryCostColissimo.map((item) => (item.id === formDataCost.id ? formDataCost : item));
            setDataDeliveryCostColissimo(updatedList);
        } else {
            setDataDeliveryCostColissimo([...dataDeliveryCostColissimo, { ...formDataCost, id: Date.now().toString(), checked: false }]);
        }
        setOpenAddCostModal(false);
        setOpenEditCostModal(false);
    };

    const handleSaveReduction = () => {
        if (formDataReduction.id !== null) {
            const updatedList = dataDeliveryReductionColissimo.map((item) => (item.id === formDataReduction.id ? formDataReduction : item));
            setDataDeliveryReductionColissimo(updatedList);
        } else {
            setDataDeliveryReductionColissimo([...dataDeliveryReductionColissimo, { ...formDataReduction, id: Date.now().toString(), checked: false }]);
        }
        setOpenAddReductionModal(false);
        setOpenEditReductionModal(false);
    };

    const [openSaveAlert, setOpenSaveAlert] = useState(false);
    const handleCloseSaveAlert = (event?: React.SyntheticEvent | Event, reason?: string) => {
        if (reason === 'clickaway') {
            return;
        }
        setOpenSaveAlert(false);
    };

    const [openErrorAlert, setOpenErrorAlert] = useState(false);
    const handleCloseErrorAlert = (event?: React.SyntheticEvent | Event, reason?: string) => {
        if (reason === 'clickaway') {
            return;
        }
        setOpenErrorAlert(false);
    };

    const renderAddEditCostModal = (open: boolean, handleClose: () => void, title: string, handleSave: () => void) => (
        <Modal open={open} onClose={handleClose}>
            <Box sx={modalStyle}>
                <Typography variant="h6" align="center" sx={{ mt: 2, mb: 3 }} gutterBottom>
                    {title}
                </Typography>
                <Box sx={modalContentStyle}>
                    <TextField
                        label="À partir du poids (g) *"
                        placeholder="0.00"
                        type="number"
                        sx={{ m: 1, width: '25ch' }}
                        InputLabelProps={{ shrink: true }}
                        InputProps={{ startAdornment: <InputAdornment position="start">g</InputAdornment> }}
                        value={formDataCost.fromWeight ?? ''}
                        onChange={(e) => setFormDataCost({ ...formDataCost, fromWeight: e.target.value ? Number(e.target.value) : null })}
                    />
                    <TextField
                        label="Jusqu’au poids (g) *"
                        placeholder="0.00"
                        type="number"
                        sx={{ m: 1, width: '25ch' }}
                        InputLabelProps={{ shrink: true }}
                        InputProps={{ startAdornment: <InputAdornment position="start">g</InputAdornment> }}
                        value={formDataCost.toWeight ?? ''}
                        onChange={(e) => setFormDataCost({ ...formDataCost, toWeight: e.target.value ? Number(e.target.value) : null })}
                    />
                    <TextField
                        label="À partir du prix du panier (EUR €) *"
                        placeholder="0.00"
                        type="number"
                        sx={{ m: 1, width: '25ch' }}
                        InputLabelProps={{ shrink: true }}
                        InputProps={{ startAdornment: <InputAdornment position="start">€</InputAdornment> }}
                        value={formDataCost.fromPrice ?? ''}
                        onChange={(e) => setFormDataCost({ ...formDataCost, fromPrice: e.target.value ? Number(e.target.value) : null })}
                    />
                    <TextField
                        label="Jusqu’au prix panier (EUR €) *"
                        placeholder="0.00"
                        type="number"
                        sx={{ m: 1, width: '25ch' }}
                        InputLabelProps={{ shrink: true }}
                        InputProps={{ startAdornment: <InputAdornment position="start">€</InputAdornment> }}
                        value={formDataCost.toPrice ?? ''}
                        onChange={(e) => setFormDataCost({ ...formDataCost, toPrice: e.target.value ? Number(e.target.value) : null })}
                    />
                    <Autocomplete
                        options={shippingClassData}
                        getOptionLabel={(option) => option.name}
                        value={shippingClassData.find((sc) => sc.id === formDataCost.shippingClassId) || null}
                        onChange={(_, value) => setFormDataCost({ ...formDataCost, shippingClassId: value?.id || null })}
                        renderInput={(params) => (
                            <TextField
                                {...params}
                                label="Classe d'expédition"
                                sx={{ m: 1, width: '30ch' }}
                                InputLabelProps={{ shrink: true }}
                            />
                        )}
                        renderTags={(selected, getTagProps) =>
                            selected.map((option, index) => (
                                <Chip
                                    {...getTagProps({ index })}
                                    key={option.id}
                                    label={option.name}
                                    size="small"
                                    color="success"
                                    variant="soft"
                                />
                            ))
                        }
                    />
                    <TextField
                        label="Prix (EUR €) *"
                        placeholder="0.00"
                        type="number"
                        sx={{ m: 1, width: '20ch' }}
                        InputLabelProps={{ shrink: true }}
                        InputProps={{ startAdornment: <InputAdornment position="start">€</InputAdornment> }}
                        value={formDataCost.cost ?? ''}
                        onChange={(e) => setFormDataCost({ ...formDataCost, cost: e.target.value ? Number(e.target.value) : null })}
                    />
                    <Box sx={{ display: 'flex', justifyContent: 'center', mt: 4 }}>
                        <Button onClick={handleClose} sx={{ mr: 1 }} variant="outlined" color="inherit">Annuler</Button>
                        <Button onClick={handleSave} variant="contained" color="inherit">{title.split(' ')[0]}</Button>
                    </Box>
                </Box>
            </Box>
        </Modal>
    );

    const renderAddEditReductionModal = (open: boolean, handleClose: () => void, title: string, handleSave: () => void) => (
        <Modal open={open} onClose={handleClose}>
            <Box sx={modalStyle}>
                <Typography variant="h6" align="center" gutterBottom sx={{ mt: 2, mb: 3 }}>{title}</Typography>
                <Box sx={{ width: '80%', ml: 'auto', mr: 'auto' }}>
                    <TextField
                        label="Nombre de produits *"
                        type="number"
                        placeholder="0"
                        sx={{ m: 1 }}
                        InputLabelProps={{ shrink: true }}
                        value={formDataReduction.nbrProducts ?? ''}
                        onChange={(e) => setFormDataReduction({ ...formDataReduction, nbrProducts: e.target.value ? Number(e.target.value) : null })}
                    />
                    <TextField
                        label="Réduction en pourcentage *"
                        placeholder="0.00"
                        type="number"
                        sx={{ m: 1 }}
                        InputLabelProps={{ shrink: true }}
                        InputProps={{ startAdornment: <InputAdornment position="start">%</InputAdornment> }}
                        value={formDataReduction.percentageReduction ?? ''}
                        onChange={(e) => setFormDataReduction({ ...formDataReduction, percentageReduction: e.target.value ? Number(e.target.value) : null })}
                    />
                </Box>
                <Box sx={{ display: 'flex', justifyContent: 'center', mt: 4 }}>
                    <Button onClick={handleClose} sx={{ mr: 1 }} variant="outlined" color="inherit">Annuler</Button>
                    <Button onClick={handleSave} variant="contained" color="inherit">{title.split(' ')[0]}</Button>
                </Box>
            </Box>
        </Modal>
    );

    const handleRowCheckboxChange = (index: number, checked: boolean, list: any[], setList: React.Dispatch<React.SetStateAction<any[]>>) => {
        const updatedList = [...list];
        updatedList[index].checked = checked;
        setList(updatedList);
    };

    const handleDeleteSelectedRows = (list: any[], setList: React.Dispatch<React.SetStateAction<any[]>>) => {
        const updatedList = list.filter((item) => !item.checked);
        setList(updatedList);
        confirmDeleteCost.onFalse();
        confirmDeleteReduction.onFalse();
    };

    const handleDeleteCost = (index: number) => {
        handleRowCheckboxChange(index, true, dataDeliveryCostColissimo, setDataDeliveryCostColissimo);
        confirmDeleteCost.onTrue();
    };

    const handleDeleteReduction = (index: number) => {
        handleRowCheckboxChange(index, true, dataDeliveryReductionColissimo, setDataDeliveryReductionColissimo);
        confirmDeleteReduction.onTrue();
    };

    const renderHead = (
        <Grid md={12}>
            <CustomBreadcrumbs
                heading="Modifier méthode de livraison"
                links={[
                    { name: 'Dashboard', href: paths.dashboard.root },
                    { name: 'Méthodes de livraison', href: paths.dashboard.admin.shipping.general.shippingMethod.shippingMethodList },
                    { name: 'Colissimo avec signature' }
                ]}
                sx={{
                    mb: { xs: 3, md: 5 },
                }}
            />
        </Grid>
    );

    const renderForm = (
        <>
            {mdUp && (
                <Grid md={4}>
                    <Typography variant="h5" sx={{ color: 'text.secondary' }}>
                        Colissimo avec signature
                    </Typography>
                    <Typography variant="body2" sx={{ color: 'text.secondary' }}>
                        Mettre à jour les informations de cette méthode de livraison.
                    </Typography>
                </Grid>
            )}

            <Grid xs={12} md={8}>
                <Card>
                    {!mdUp && <CardHeader title="Nouvelle méthode de livraison" />}

                    <Stack spacing={3} sx={{ p: 3 }}>
                        <Stack spacing={3} sx={{ p: 3 }}>
                            <Controller
                                name="name"
                                control={control}
                                render={({ field }) => (
                                    <RHFTextField {...field} label="Nom de la méthode" />
                                )}
                            />

                            <Controller
                                name="description"
                                control={control}
                                render={({ field }) => (
                                    <RHFTextField {...field} label="Description" placeholder="Description..." multiline rows={4} />
                                )}
                            />

                            <Controller
                                name="active"
                                control={control}
                                defaultValue={isActive}
                                render={({ field }) => (
                                    <div style={{ display: 'flex', alignItems: 'center' }}>
                                        <label>Activé</label>
                                        <Switch
                                            {...field}
                                            checked={isActive}
                                            onChange={(e) => {
                                                setIsActive(e.target.checked);
                                                field.onChange(e.target.checked);
                                            }}
                                        />
                                    </div>
                                )}
                            />

                            <Grid container alignItems="center" spacing={1}>
                                <Controller
                                    name="alwaysFree"
                                    control={control}
                                    defaultValue={checkedStillFree}
                                    render={({ field }) => (
                                        <Checkbox
                                            {...field}
                                            checked={field.value}
                                            onChange={(e) => {
                                                setCheckedStillFree(e.target.checked);
                                                field.onChange(e.target.checked);
                                            }}
                                            color="success"
                                            inputProps={{ 'aria-label': 'primary checkbox' }}
                                        />
                                    )}
                                />
                                <Grid>
                                    <Typography variant="body2">Toujours gratuit ?</Typography>
                                </Grid>
                            </Grid>

                            <Controller
                                name="nameFreeShipping"
                                control={control}
                                defaultValue=""
                                render={({ field }) => <TextField {...field} label="Nom si la livraison est offerte" />}
                            />

                            <Controller
                                name="shippingClassExcludedList"
                                control={control}
                                defaultValue={[]}
                                render={({ field }) => (
                                    <Autocomplete
                                        multiple
                                        disableCloseOnSelect
                                        options={shippingClassData}
                                        getOptionLabel={(option) => option.name}
                                        value={field.value}
                                        onChange={(_, value) => {
                                            field.onChange(value);
                                        }}
                                        isOptionEqualToValue={(option, value) => option.id === value.id}
                                        renderInput={(params) => (
                                            <TextField
                                                {...params}
                                                label="Classes d’expédition exclues"
                                                placeholder="+ Classes d’expédition exclues"
                                            />
                                        )}
                                        renderTags={(selected, getTagProps) =>
                                            selected.map((option, index) => (
                                                <Chip
                                                    {...getTagProps({ index })}
                                                    key={option.id}
                                                    label={option.name}
                                                    size="small"
                                                    color="success"
                                                    variant="soft"
                                                />
                                            ))
                                        }
                                    />
                                )}
                            />
                            <Controller
                                name="freeShippingClassList"
                                control={control}
                                defaultValue={[]}
                                render={({ field }) => (
                                    <Autocomplete
                                        multiple
                                        disableCloseOnSelect
                                        options={shippingClassData}
                                        getOptionLabel={(option) => option.name}
                                        value={field.value}
                                        onChange={(_, value) => {
                                            field.onChange(value);
                                        }}
                                        isOptionEqualToValue={(option, value) => option.id === value.id}
                                        renderInput={(params) => (
                                            <TextField
                                                {...params}
                                                label="Classes d’expédition gratuites"
                                                placeholder="+ Classes d’expédition gratuites"
                                            />
                                        )}
                                        renderTags={(selected, getTagProps) =>
                                            selected.map((option, index) => (
                                                <Chip
                                                    {...getTagProps({ index })}
                                                    key={option.id}
                                                    label={option.name}
                                                    size="small"
                                                    color="success"
                                                    variant="soft"
                                                />
                                            ))
                                        }
                                    />
                                )}
                            />
                            <Grid container alignItems="center" spacing={1}>
                                <Grid md={1} xs={2}>
                                    <Controller
                                        name="freeIfOneProductHaveFreeShippingClass"
                                        control={control}
                                        defaultValue={checkedConditionFree}
                                        render={({ field }) => (
                                            <Checkbox
                                                {...field}
                                                checked={field.value}
                                                onChange={(e) => {
                                                    setCheckedConditionFree(e.target.checked);
                                                    field.onChange(e.target.checked);
                                                }}
                                                color="success"
                                                inputProps={{ 'aria-label': 'primary checkbox' }}
                                            />
                                        )}
                                    />
                                </Grid>
                                <Grid md={11} xs={10}>
                                    <Typography variant="body2">
                                        Gratuit si au moins un des produits du panier a une des classes d’expédition gratuite ci-dessus
                                    </Typography>
                                </Grid>
                            </Grid>
                        </Stack>
                    </Stack>
                </Card>
            </Grid>

            {mdUp && (
                <Grid md={4} sx={{ mt: 4 }}>
                    <Typography variant="h6">
                        Frais de livraison
                    </Typography>
                </Grid>
            )}

            <Grid xs={12} md={8}>
                <Card>
                    {!mdUp && <CardHeader title="Frais de livraison" />}
                    <Container sx={{ mt: 2 }}>
                        <Box sx={{ display: 'flex', alignItems: 'center', gap: 1 }}>
                            <Button
                                variant="outlined"
                                size="small"
                                sx={{ color: '#808080', width: '15%' }}
                                onClick={handleAddCostClick}
                                startIcon={<AddIcon />}
                            >
                                Ajouter
                            </Button>
                            <Button variant="outlined" size="small" sx={{ color: '#808080', width: '15%' }} onClick={() => handleDeleteSelectedRows(dataDeliveryCostColissimo, setDataDeliveryCostColissimo)} startIcon={<DeleteIcon />}>
                                Supprimer
                            </Button>
                        </Box>
                    </Container>
                    <TableContainer component={Paper} sx={{ mt: 1, maxHeight: '500px' }}>
                        <Table sx={{ minWidth: 650 }} aria-label="simple table">
                            <TableHead>
                                <TableRow>
                                    <TableCell align="center">
                                        <Checkbox
                                            indeterminate={dataDeliveryCostColissimo.length > 0 && dataDeliveryCostColissimo.some(item => item.checked)}
                                            checked={dataDeliveryCostColissimo.length > 0 && dataDeliveryCostColissimo.every(item => item.checked)}
                                            onChange={(event) => {
                                                const checked = event.target.checked;
                                                const updatedList = dataDeliveryCostColissimo.map(item => ({ ...item, checked }));
                                                setDataDeliveryCostColissimo(updatedList);
                                            }}
                                        />
                                    </TableCell>
                                    <TableCell align="center">À partir du poids (g)</TableCell>
                                    <TableCell align="center">Jusqu’au poids (g)</TableCell>
                                    <TableCell align="center">À partir du prix du panier (EUR €)</TableCell>
                                    <TableCell align="center">Jusqu’au prix panier (EUR €)</TableCell>
                                    <TableCell align="center">Classe d’expédition</TableCell>
                                    <TableCell align="center" width="100px">
                                        Prix (EUR €)
                                    </TableCell>
                                    <TableCell align="center" width="115px">
                                        Actions
                                    </TableCell>
                                </TableRow>
                            </TableHead>
                            <TableBody>
                                {dataDeliveryCostColissimo.map((row, index) => (
                                    <TableRow key={index} sx={{ '&:last-child td, &:last-child th': { border: 0 } }}>
                                        <TableCell component="th" scope="row" align="center">
                                            <Checkbox
                                                checked={row.checked || false}
                                                onChange={(event) => handleRowCheckboxChange(index, event.target.checked, dataDeliveryCostColissimo, setDataDeliveryCostColissimo)}
                                            />
                                        </TableCell>
                                        <TableCell align="center">{row?.fromWeight} g</TableCell>
                                        <TableCell align="center">{row?.toWeight} g</TableCell>
                                        <TableCell align="center">{row?.fromPrice} €</TableCell>
                                        <TableCell align="center">{row?.toPrice} €</TableCell>
                                        <TableCell align="center">{getShippingClassById(row?.shippingClassId!)}</TableCell>
                                        <TableCell align="center">{row.cost} €</TableCell>
                                        <TableCell align="center">
                                            <Box>
                                                <IconButton onClick={() => handleEditCostClick(index)}>
                                                    <EditIcon sx={{ color: 'green' }} />
                                                </IconButton>
                                                <IconButton onClick={() => handleDeleteCost(index)}>
                                                    <DeleteIcon sx={{ color: 'red' }} />
                                                </IconButton>
                                            </Box>
                                        </TableCell>
                                    </TableRow>
                                ))}
                            </TableBody>
                        </Table>
                    </TableContainer>
                </Card>
            </Grid>

            <ConfirmDialog
                open={confirmDeleteCost.value}
                onClose={confirmDeleteCost.onFalse}
                title="Supprimer frais"
                content="Voulez-vous vraiment supprimer?"
                action={
                    <Button variant="contained" color="error" onClick={() => handleDeleteSelectedRows(dataDeliveryCostColissimo, setDataDeliveryCostColissimo)}>
                        Supprimer
                    </Button>
                }
            />

            {mdUp && (
                <Grid md={4} sx={{ mt: 4 }}>
                    <Typography variant="h6" >
                        Réduction sur la livraison
                    </Typography>
                </Grid>
            )}

            <Grid xs={12} md={8}>
                <Card>
                    {!mdUp && <CardHeader title="Réduction sur la livraison" />}
                    <Stack spacing={3} sx={{ p: 3 }}>
                        <Container sx={{ mt: 2 }}>
                            <Box sx={{ display: 'flex', alignItems: 'center', gap: 1 }}>
                                <Button
                                    variant="outlined"
                                    size="small"
                                    sx={{ color: '#808080', width: '15%' }}
                                    onClick={handleAddReductionClick}
                                    startIcon={<AddIcon />}
                                >
                                    Ajouter
                                </Button>
                                <Button variant="outlined" size="small" sx={{ color: '#808080', width: '15%' }} onClick={() => handleDeleteSelectedRows(dataDeliveryReductionColissimo, setDataDeliveryReductionColissimo)} startIcon={<DeleteIcon />}>
                                    Supprimer
                                </Button>
                            </Box>
                        </Container>
                        <TableContainer component={Paper} sx={{ mt: 1, maxHeight: '435px' }}>
                            <Table sx={{ minWidth: 650 }} aria-label="simple table">
                                <TableHead>
                                    <TableRow>
                                        <TableCell align="center">
                                            <Checkbox
                                                indeterminate={dataDeliveryReductionColissimo.length > 0 && dataDeliveryReductionColissimo.some(item => item.checked)}
                                                checked={dataDeliveryReductionColissimo.length > 0 && dataDeliveryReductionColissimo.every(item => item.checked)}
                                                onChange={(event) => {
                                                    const checked = event.target.checked;
                                                    const updatedList = dataDeliveryReductionColissimo.map(item => ({ ...item, checked }));
                                                    setDataDeliveryReductionColissimo(updatedList);
                                                }}
                                            />
                                        </TableCell>
                                        <TableCell align="center">Nombre de produits</TableCell>
                                        <TableCell align="center">Réduction en pourcentage</TableCell>
                                        <TableCell align="center">Actions</TableCell>
                                    </TableRow>
                                </TableHead>
                                <TableBody>
                                    {dataDeliveryReductionColissimo.map((row, index) => (
                                        <TableRow key={index} sx={{ '&:last-child td, &:last-child th': { border: 0 } }}>
                                            <TableCell component="th" scope="row" align="center">
                                                <Checkbox
                                                    checked={row.checked || false}
                                                    onChange={(event) => handleRowCheckboxChange(index, event.target.checked, dataDeliveryReductionColissimo, setDataDeliveryReductionColissimo)}
                                                />
                                            </TableCell>
                                            <TableCell component="th" scope="row" align="center">
                                                {row.nbrProducts}
                                            </TableCell>
                                            <TableCell align="center">{row.percentageReduction} %</TableCell>
                                            <TableCell align="center">
                                                <Box>
                                                    <IconButton onClick={() => handleEditReductionClick(index)}>
                                                        <EditIcon sx={{ color: 'green' }} />
                                                    </IconButton>
                                                    <IconButton onClick={() => handleDeleteReduction(index)}>
                                                        <DeleteIcon sx={{ color: 'red' }} />
                                                    </IconButton>
                                                </Box>
                                            </TableCell>
                                        </TableRow>
                                    ))}
                                </TableBody>
                            </Table>
                        </TableContainer>
                    </Stack>
                </Card>
            </Grid>

            <ConfirmDialog
                open={confirmDeleteReduction.value}
                onClose={confirmDeleteReduction.onFalse}
                title="Supprimer réduction"
                content="Voulez-vous vraiment supprimer?"
                action={
                    <Button variant="contained" color="error" onClick={() => handleDeleteSelectedRows(dataDeliveryReductionColissimo, setDataDeliveryReductionColissimo)}>
                        Supprimer
                    </Button>
                }
            />

            {renderAddEditCostModal(openAddCostModal, () => setOpenAddCostModal(false), "Ajouter frais de livraison", handleSaveCost)}
            {renderAddEditCostModal(openEditCostModal, () => setOpenEditCostModal(false), "Modifier frais de livraison", handleSaveCost)}
            {renderAddEditReductionModal(openAddReductionModal, () => setOpenAddReductionModal(false), "Ajouter réduction sur la livraison", handleSaveReduction)}
            {renderAddEditReductionModal(openEditReductionModal, () => setOpenEditReductionModal(false), "Modifier réduction sur la livraison", handleSaveReduction)}
        
            <Snackbar open={openSaveAlert} autoHideDuration={3000} 
                onClose={handleCloseSaveAlert} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
                <Alert
                    onClose={handleCloseSaveAlert}
                    severity="success"
                    variant="filled"
                    sx={{ width: '100%' }}
                >
                    Méthode de livraison enregistrer avec succès
                </Alert>
            </Snackbar>
            <Snackbar open={openErrorAlert} autoHideDuration={3000}
                onClose={handleCloseErrorAlert} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
                <Alert
                    onClose={handleCloseErrorAlert}
                    severity="error"
                    variant="filled"
                    sx={{ width: '100%' }}
                >
                    Échec
                </Alert>
            </Snackbar>
        </>
    );

    const renderActions = (
        <>
            {mdUp && <Grid md={4} />}
            <Grid xs={12} md={8} sx={{ display: 'flex', justifyContent: 'end', py: 4 }}>
                <LoadingButton
                    type="submit"
                    variant="contained"
                    size="large"
                    loading={isSubmitting}
                    sx={{ ml: 2 }}
                >Enregistrer</LoadingButton>
            </Grid>
        </>
    );

    return (
        <FormProvider methods={methods} onSubmit={onSubmit}>
            <Grid container spacing={3}>
                {renderHead}
                {renderForm}
                {renderActions}
            </Grid>
        </FormProvider>
    );
}
