"use client";

import * as Yup from 'yup';
import { useForm, Controller } from 'react-hook-form';
import { yupResolver } from '@hookform/resolvers/yup';
import { useEffect, useMemo, useState } from 'react';
import Chip from '@mui/material/Chip';
import Grid from '@mui/material/Unstable_Grid2';
import Card from '@mui/material/Card';
import Stack from '@mui/material/Stack';
import Typography from '@mui/material/Typography';
import { useResponsive } from '@/hooks/use-responsive';
import {
    RHFTextField,
    RHFAutocomplete,
} from '@/components/hook-form';
import { RequiresFreeShipping, requiresFreeShippingLabels } from '@/contexts/types/Shipping';
import Checkbox from '@mui/material/Checkbox';
import CustomBreadcrumbs from '@/components/custom-breadcrumbs/custom-breadcrumbs';
import { useRouter } from '@/hooks';
import { useSnackbar } from 'notistack';
import { paths } from '@/routes/paths';
import LoadingButton from '@mui/lab/LoadingButton';
import FormProvider from '@/components/hook-form/form-provider';
import { Alert, AutocompleteChangeReason, Box, InputAdornment, Snackbar, Switch } from '@mui/material';
import { useGetFreeShipping, useGetShippingMethods, useUpdateFreeShipping } from '@/shared/api/shipping';
import LoadingScreen from '@/shared/components/loading-screen/loading-screen';

type Props = {
    id: string;
};

export default function EditFreeShipping({ id }: Props) {

    const { shippingMethod, shippingMethodLoading, shippingMethodError, mutateShippingMethod } = useGetFreeShipping(id);
    const { mutateShippingMethods } = useGetShippingMethods();
    
    const router = useRouter();
    const mdUp = useResponsive('up', 'md');
    const { enqueueSnackbar } = useSnackbar();

    const NewShippingMethodSchema = Yup.object().shape({
        name: Yup.string().required('Nom est requis'),
        description: Yup.string().required('Description est requis'),
        active: Yup.boolean().required('Le statut actif est requis'),
        requires: Yup.mixed().oneOf(Object.values(RequiresFreeShipping)).required('Requis est requis'),
        minOrderAmount: Yup.number().min(0, 'Le montant minimum de la commande doit être positif'),
        applyMinOrderRuleBeforePromoCodeDiscount: Yup.boolean(),
    });

    const defaultValues = useMemo(() => ({
        name: shippingMethod?.name || '',
        description: shippingMethod?.description || '',
        active: shippingMethod?.active || false,
        requires: shippingMethod?.requires || RequiresFreeShipping.NOTHING,
        minOrderAmount: shippingMethod?.minOrderAmount || 0,
        applyMinOrderRuleBeforePromoCodeDiscount: shippingMethod?.applyMinOrderRuleBeforePromoCodeDiscount || false,
    }), [shippingMethod]);

    const methods = useForm({
        resolver: yupResolver(NewShippingMethodSchema),
        defaultValues,
    });

    const {
        reset,
        watch,
        setValue,
        handleSubmit,
        formState: { isSubmitting },
        control,
        register,
        getValues,
    } = methods;

    const values = watch();

    const [ruleChecked, setRuleChecked] = useState(defaultValues.applyMinOrderRuleBeforePromoCodeDiscount);
    const [selectedRequis, setSelectedRequis] = useState(defaultValues.requires);
    const [isActive, setIsActive] = useState(defaultValues.active);

    useEffect(() => {
        if (shippingMethod) {
            reset(defaultValues);
            setRuleChecked(defaultValues.applyMinOrderRuleBeforePromoCodeDiscount);
            setSelectedRequis(defaultValues.requires);
            setIsActive(defaultValues.active);
        }
    }, [shippingMethod, defaultValues, reset]);

    const onSubmit = handleSubmit((data: any) => {
        data.id = id;
        useUpdateFreeShipping(data.id, data).then(() => {
            setOpenSaveAlert(true);
            router.push(paths.dashboard.admin.shipping.general.root);
            router.refresh;
            mutateShippingMethod();
            mutateShippingMethods();
            enqueueSnackbar('Méthode de livraison gratuite ajoutée avec succès', { variant: 'success' });
        }).catch((error) => {
            setOpenErrorAlert(true);
            enqueueSnackbar(`Erreur lors de l'ajout de la méthode de livraison gratuite : ${error.message}`, { variant: 'error' });
        });
    });

    const handleChangeRule = (event: React.ChangeEvent<HTMLInputElement>) => {
        setRuleChecked(event.target.checked);
        setValue('applyMinOrderRuleBeforePromoCodeDiscount', event.target.checked);
    };

    const handleChangeRequis = (
        event: React.SyntheticEvent,
        value: string | string[],
        reason: AutocompleteChangeReason,
        onChange: any
    ) => {
        const newValue = Array.isArray(value) ? value[0] : value;
        if (Object.values(RequiresFreeShipping).includes(newValue as RequiresFreeShipping)) {
            setSelectedRequis(newValue as RequiresFreeShipping);
            onChange(newValue as RequiresFreeShipping);
        } else {
            console.error('Invalid shipping method type:', newValue);
        }
    };

    const [openSaveAlert, setOpenSaveAlert] = useState(false);

    const handleClickSaveAlert = () => {
        setOpenSaveAlert(true);
    };

    const handleCloseSaveAlert = (event?: React.SyntheticEvent | Event, reason?: string) => {
        if (reason === 'clickaway') {
            return;
        }

        setOpenSaveAlert(false);
    };

    const [openErrorAlert, setOpenErrorAlert] = useState(false);

    const handleClickErrorAlert = () => {
        setOpenErrorAlert(true);
    };

    const handleCloseErrorAlert = (event?: React.SyntheticEvent | Event, reason?: string) => {
        if (reason === 'clickaway') {
            return;
        }

        setOpenErrorAlert(false);
    };

    if (shippingMethodLoading) {
        return <Grid container spacing={3} alignItems="center" justifyContent="center"><Grid md={12}><LoadingScreen /></Grid></Grid>;
    }

    if (shippingMethodError) {
        return <Grid container spacing={3} alignItems="center" justifyContent="center"><Grid md={12}><Alert severity="warning">Problème lors de la récupération de la méthode de livraison. Veuillez réessayer plus tard.</Alert></Grid></Grid>;
    }

    const renderHead = (
        <Grid md={12}>
            <CustomBreadcrumbs
                heading="Modifier méthode de livraison"
                links={[
                    { name: 'Tableau de bord', href: paths.dashboard.root },
                    { name: 'Méthodes de livraison', href: paths.dashboard.admin.shipping.general.shippingMethod.shippingMethodList },
                    { name: shippingMethod?.type },
                ]}
                sx={{
                    mb: { xs: 3, md: 5 },
                }}
            />
        </Grid>
    );

    const renderRequisComponents = (
        <>
            <Controller
                name="minOrderAmount"
                control={methods.control}
                render={({ field }) => (
                    <RHFTextField
                        {...field}
                        label="Montant minimum de commande"
                        placeholder="0.00"
                        type="number"
                        InputLabelProps={{ shrink: true }}
                        InputProps={{
                            startAdornment: (
                                <InputAdornment position="start">
                                    <Box component="span" sx={{ color: 'text.disabled' }}>
                                        €
                                    </Box>
                                </InputAdornment>
                            ),
                        }}
                    />
                )}
            />
            <Grid container alignItems="center" spacing={1}>
                <Grid>
                    <Controller
                        name="applyMinOrderRuleBeforePromoCodeDiscount"
                        control={methods.control}
                        render={({ field }) => (
                            <Checkbox
                                {...field}
                                checked={ruleChecked}
                                onChange={(event) => {
                                    handleChangeRule(event);
                                    field.onChange(event.target.checked);
                                }}

                                inputProps={{ 'aria-label': 'primary checkbox' }}
                            />
                        )}
                    />
                </Grid>
                <Grid>
                    <Typography>Appliquer la règle de commande minimale avant la remise avec code promo</Typography>
                </Grid>
            </Grid>
        </>
    );

    const renderDetails = () => {
        switch (selectedRequis) {
            case RequiresFreeShipping.NOTHING:
            case RequiresFreeShipping.PROMO_CODE:
                return null;
            case RequiresFreeShipping.MINIMUM_ORDER_AMOUNT:
            case RequiresFreeShipping.PROMO_CODE_OR_MINIMUM_ORDER_AMOUNT:
            case RequiresFreeShipping.PROMO_CODE_AND_MINIMUM_ORDER_AMOUNT:
                return <>{renderRequisComponents}</>;
            default:
                return null;
        }
    };

    const renderForm = (
        <>
            {mdUp && (
                <Grid md={4}>
                    <Typography variant="h6" sx={{ mb: 0.5 }}>
                        Livraison gratuite
                    </Typography>
                    <Typography variant="body2" sx={{ color: 'text.secondary' }}>
                        Saisissez les informations de cette méthode de livraison
                    </Typography>
                </Grid>
            )}
            <Grid xs={12} md={8}>
                <Card>
                    <Stack spacing={3} sx={{ p: 3 }}>
                        <Controller
                            name="name"
                            control={control}
                            render={({ field }) => (
                                <RHFTextField {...field} label="Nom de la méthode" />
                            )}
                        />
                        <Controller
                            name="description"
                            control={control}
                            render={({ field }) => (
                                <RHFTextField {...field} label="Description" placeholder="Description..." multiline rows={4} />
                            )}
                        />
                        <Controller
                            name="active"
                            control={control}
                            render={({ field }) => (
                                <div style={{ display: 'flex', alignItems: 'center' }}>
                                    <label>Activé</label>
                                    <Switch
                                        {...field}
                                        checked={isActive}
                                        onChange={(e) => {
                                            setIsActive(e.target.checked);
                                            field.onChange(e.target.checked);
                                        }}
                                    />
                                </div>
                            )}
                        />
                    </Stack>
                </Card>
                <Card sx={{ mt: 3 }}>
                    <Stack spacing={3} sx={{ p: 3 }}>
                        <Controller
                            name="requires"
                            control={control}
                            render={({ field }) => (
                                <RHFAutocomplete
                                    {...field}
                                    label="La livraison gratuite requiert"
                                    placeholder="+ La livraison gratuite requiert"
                                    disableClearable
                                    options={Object.values(RequiresFreeShipping)}
                                    getOptionLabel={(option) => requiresFreeShippingLabels[option as RequiresFreeShipping]}
                                    onChange={(event, value, reason) => handleChangeRequis(event, value, reason, field.onChange)}
                                    value={selectedRequis}
                                    renderOption={(props, option: RequiresFreeShipping) => (
                                        <li {...props} key={option}>
                                            {requiresFreeShippingLabels[option]}
                                        </li>
                                    )}
                                    renderTags={(selected, getTagProps) =>
                                        selected.map((option, index) => (
                                            <Chip
                                                {...getTagProps({ index })}
                                                key={option}
                                                label={requiresFreeShippingLabels[option]}
                                                size="small"
                                                color="info"
                                                variant="soft"
                                            />
                                        ))
                                    }
                                />
                            )}
                        />
                        {renderDetails()}
                    </Stack>
                </Card>
            </Grid>
            <Snackbar open={openSaveAlert} autoHideDuration={2000}
                onClose={handleCloseSaveAlert} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
                <Alert
                    onClose={handleCloseSaveAlert}
                    severity="success"
                    variant="filled"
                    sx={{ width: '100%' }}
                >
                    Méthode de livraison enregistrer avec succès
                </Alert>
            </Snackbar>
            <Snackbar open={openErrorAlert} autoHideDuration={3000}
                onClose={handleCloseErrorAlert} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
                <Alert
                    onClose={handleCloseErrorAlert}
                    severity="error"
                    variant="filled"
                    sx={{ width: '100%' }}
                >
                    Échec
                </Alert>
            </Snackbar>
        </>
    );

    const renderActions = (
        <>
            {mdUp && <Grid md={4} />}
            <Grid xs={12} md={8} sx={{ display: 'flex', justifyContent: 'end', py: 4 }}>
                <LoadingButton
                    type="submit"
                    variant="contained"
                    size="large"
                    loading={isSubmitting}
                    sx={{ ml: 2 }}
                >
                    Enregistrer
                </LoadingButton>
            </Grid>
        </>
    );

    return (
        <FormProvider methods={methods} onSubmit={onSubmit}>
            <Grid container spacing={3}>
                {renderHead}
                {renderForm}
                {renderActions}
            </Grid>
        </FormProvider>
    );
}
