"use client"

import * as Yup from 'yup';
import { Controller, useForm } from 'react-hook-form';
import { yupResolver } from '@hookform/resolvers/yup';
import { useEffect, useMemo } from 'react';
import { useState } from 'react';
import Box from '@mui/material/Box';
import Grid from '@mui/material/Unstable_Grid2';
import Card from '@mui/material/Card';
import Stack from '@mui/material/Stack';
import Typography from '@mui/material/Typography';
import Radio from '@mui/material/Radio';
import RadioGroup from '@mui/material/RadioGroup';
import FormControlLabel from '@mui/material/FormControlLabel';
import FormControl from '@mui/material/FormControl';
import FormLabel from '@mui/material/FormLabel';
import InputAdornment from '@mui/material/InputAdornment';
import { useResponsive } from '@/hooks/use-responsive';
import {RHFTextField} from '@/components/hook-form';
import CustomBreadcrumbs from '@/components/custom-breadcrumbs/custom-breadcrumbs';
import { useRouter } from '@/hooks';
import { useSnackbar } from 'notistack';
import { paths } from '@/routes/paths';
import LoadingButton from '@mui/lab/LoadingButton';
import FormProvider from '@/components/hook-form/form-provider';
import { Alert, Snackbar, Switch } from '@mui/material';
import { useGetPointOfSale, useGetShippingMethods, useUpdatePointOfSale } from '@/shared/api/shipping';
import { LoadingScreen } from '@/shared/components/loading-screen';


type Props = {
    id: string;
}

export default function EditPointOfSale({ id }: Props) {

    const { shippingMethod, shippingMethodLoading, shippingMethodError, mutateShippingMethod } = useGetPointOfSale(id);
    const { mutateShippingMethods } = useGetShippingMethods();

    const router = useRouter();

    const mdUp = useResponsive('up', 'md');
    const { enqueueSnackbar } = useSnackbar();

    const NewShippingMethodSchema = Yup.object().shape({
        name: Yup.string().required('Nom est requis'),
        description: Yup.string().required('Description est requis'),
        active: Yup.boolean().required('Le statut actif est requis'),
        tvaStatut: Yup.boolean().required('Le statut TVA est requis'),
        cost: Yup.number().min(0, 'Le coût doit être positif'),
    });

    const defaultValues = useMemo(() => ({
        name: shippingMethod?.name || '',
        description: shippingMethod?.description || '',
        active: shippingMethod?.active || false,
        tvaStatut: shippingMethod?.tvaStatut || false,
        cost: shippingMethod?.cost || 0,
    }), [shippingMethod]);

    const methods = useForm({
        resolver: yupResolver(NewShippingMethodSchema),
        defaultValues,
    });

    const {
        reset,
        watch,
        setValue,
        handleSubmit,
        formState: { isSubmitting },
        control,
    } = methods;

    useEffect(() => {
        if (shippingMethod) {
            reset(defaultValues);
            setIsActive(defaultValues.active)
        }
    }, [shippingMethod, defaultValues, reset])

    const onSubmit = handleSubmit((data: any) => {
        useUpdatePointOfSale(shippingMethod?.id!, data).then(() => {
            setOpenSaveAlert(true);
            router.push(paths.dashboard.admin.shipping.general.shippingMethod.shippingMethodList);
            mutateShippingMethod();
            mutateShippingMethods();
            enqueueSnackbar('Méthode de livraison gratuite ajoutée avec succès', { variant: 'success' });
        }).catch((error) => {
            setOpenErrorAlert(true);
            enqueueSnackbar(`Erreur lors de l'ajout de la méthode de livraison gratuite : ${error.message}`, { variant: 'error' });
        });
    });

    const [isActive, setIsActive] = useState(false);

    const [openSaveAlert, setOpenSaveAlert] = useState(false);
    const handleCloseSaveAlert = (event?: React.SyntheticEvent | Event, reason?: string) => {
        if (reason === 'clickaway') {
            return;
        }
        setOpenSaveAlert(false);
    };

    const [openErrorAlert, setOpenErrorAlert] = useState(false);
    const handleCloseErrorAlert = (event?: React.SyntheticEvent | Event, reason?: string) => {
        if (reason === 'clickaway') {
            return;
        }
        setOpenErrorAlert(false);
    };

    if (shippingMethodLoading) {
        return <Grid container spacing={3} alignItems="center" justifyContent="center"><Grid md={12}><LoadingScreen /></Grid></Grid>;
    }

    if (shippingMethodError) {
        return <Grid container spacing={3} alignItems="center" justifyContent="center"><Grid md={12}><Alert severity="warning">Problème lors de la récupération de la méthode de livraison. Veuillez réessayer plus tard.</Alert></Grid></Grid>;
    }

    const renderHead = (
        <Grid md={12}>
            <CustomBreadcrumbs
                heading="Modifier méthode de livraison"
                links={[
                    { name: 'Tableau de bord', href: paths.dashboard.root, },
                    { name: "Méthodes de livraison", href: paths.dashboard.admin.shipping.general.shippingMethod.shippingMethodList, },
                    { name: shippingMethod?.type },
                ]}
                sx={{
                    mb: { xs: 3, md: 5 },
                }}
            />
        </Grid>
    );


    const renderForm = (
        <>
            {
                mdUp && (
                    <Grid md={4}>
                        <Typography variant="h6" sx={{ mb: 0.5 }}>
                            Point de vente
                        </Typography>
                        <Typography variant="body2" sx={{ color: 'text.secondary' }}>
                            Saisissez les informations de cette méthode de livraison
                        </Typography>
                    </Grid>
                )
            }
            <Grid xs={12} md={8}>
                <Card>
                    <Stack spacing={3} sx={{ p: 3 }}>
                        <Controller
                            name="name"
                            control={control}
                            render={({ field }) => (
                                <RHFTextField {...field} label="Nom de la méthode" />
                            )}
                        />
                        <Controller
                            name="description"
                            control={control}
                            render={({ field }) => (
                                <RHFTextField {...field} label="Description" placeholder="Description..." multiline rows={4} />
                            )}
                        />
                        <Controller
                            name="active"
                            control={control}
                            defaultValue={isActive}
                            render={({ field }) => (
                                <div style={{ display: 'flex', alignItems: 'center' }}>
                                    <label>Activé</label>
                                    <Switch
                                        {...field}
                                        checked={isActive}
                                        onChange={(e) => {
                                            setIsActive(e.target.checked);
                                            field.onChange(e.target.checked);
                                        }}
                                    />
                                </div>
                            )}
                        />
                    </Stack>
                </Card>

                <Card sx={{ mt: 3 }}>
                    <Stack spacing={3} sx={{ p: 3 }}>
                        <FormControl>
                            <FormLabel id="demo-row-radio-buttons-group-label">État de la TVA</FormLabel>
                            <Controller
                                name="tvaStatut"
                                control={control}
                                defaultValue={false}
                                render={({ field }) => (
                                    <RadioGroup
                                        row
                                        aria-labelledby="demo-row-radio-buttons-group-label"
                                        value={field.value}
                                        onChange={(event) => field.onChange(event.target.value === 'true')}
                                    >
                                        <FormControlLabel value={true} control={<Radio />} label="Taxable" />
                                        <FormControlLabel value={false} control={<Radio />} label="Aucune" />
                                    </RadioGroup>
                                )}
                            />
                        </FormControl>
                        <RHFTextField
                            name="cost"
                            label="Coût"
                            placeholder="0.00"
                            type="number"
                            InputLabelProps={{ shrink: true }}
                            InputProps={{
                                startAdornment: (
                                    <InputAdornment position="start">
                                        <Box component="span" sx={{ color: 'text.disabled' }}>
                                            €
                                        </Box>
                                    </InputAdornment>
                                ),
                            }}
                        />
                    </Stack>
                </Card>
            </Grid>
            <Snackbar open={openSaveAlert} autoHideDuration={2000}
                onClose={handleCloseSaveAlert} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
                <Alert
                    onClose={handleCloseSaveAlert}
                    severity="success"
                    variant="filled"
                    sx={{ width: '100%' }}
                >
                    Méthode de livraison enregistrer avec succès
                </Alert>
            </Snackbar>
            <Snackbar open={openErrorAlert} autoHideDuration={3000}
                onClose={handleCloseErrorAlert} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
                <Alert
                    onClose={handleCloseErrorAlert}
                    severity="error"
                    variant="filled"
                    sx={{ width: '100%' }}
                >
                    Échec
                </Alert>
            </Snackbar>
        </>
    );


    const renderActions = (
        <>
            {mdUp && <Grid md={4} />}
            <Grid xs={12} md={8} sx={{ display: 'flex', justifyContent: 'end', py: 4 }}>
                <LoadingButton
                    type="submit"
                    variant="contained"
                    size="large"
                    loading={isSubmitting}
                    sx={{ ml: 2 }}
                >Enregistrer</LoadingButton>
            </Grid>
        </>
    );


    return (
        <>
            <FormProvider methods={methods} onSubmit={onSubmit}>
                <Grid container spacing={3}>
                    {renderHead}
                    {renderForm}
                    {renderActions}
                </Grid>
            </FormProvider>
        </>
    );
}
