"use client"

import { useEffect, useState } from 'react';
import { Controller, useForm } from 'react-hook-form';
import * as Yup from 'yup';
import { useMemo } from 'react';
import { yupResolver } from '@hookform/resolvers/yup';
import {Box, Button, Card, CardHeader, Checkbox, Grid, IconButton, InputAdornment, Modal, Paper, Stack, Table, TableBody, TableCell,
    TableContainer, TableHead, TableRow, TextField, Typography, Chip, Autocomplete, Switch, Snackbar, Alert} from '@mui/material';
import CustomBreadcrumbs from '@/components/custom-breadcrumbs/custom-breadcrumbs';
import { Add as AddIcon, Delete as DeleteIcon, Edit as EditIcon } from '@mui/icons-material';
import { useResponsive, useBoolean } from '@/hooks';
import { useSnackbar } from '@/components/snackbar';
import ConfirmDialog from '@/shared/components/confirm-dialog';
import { DeliveryCostMondialRelay, MondialRelayMode, MondialRelayModeLabels } from '@/contexts/types/Shipping';
import { RHFTextField } from '@/components/hook-form';
import LoadingButton from '@mui/lab/LoadingButton';
import FormProvider from '@/components/hook-form/form-provider';
import { useRouter } from '@/hooks';
import { paths } from '@/routes/paths';
import { useGetMondialRelay, useGetShippingMethods, useUpdateMondialRelay } from '@/shared/api/shipping';
import { LoadingScreen } from '@/shared/components/loading-screen';



const modalStyle = {
    position: 'absolute',
    top: '50%',
    left: '50%',
    transform: 'translate(-50%, -50%)',
    width: 600,
    bgcolor: 'background.paper',
    boxShadow: 24,
    p: 4,
    borderRadius: 3,
};

const modalContentStyle = {
    display: 'flex',
    flexDirection: 'column',
    gap: 2,
};

type Props = {
    id: string;
}

export default function EditShippingMethodMondialRelay({ id }: Props) {
    const mdUp = useResponsive('up', 'md');
    const { enqueueSnackbar } = useSnackbar();
    const router = useRouter();

    const { shippingMethod, shippingMethodLoading, shippingMethodError, mutateShippingMethod } = useGetMondialRelay(id);
    const { mutateShippingMethods } = useGetShippingMethods();

    const [dataDeliveryCostMondialRelay, setDataDeliveryCostMondialRelay] = useState<DeliveryCostMondialRelay[]>([]);

    const NewShippingMethodSchema = Yup.object().shape({
        name: Yup.string().required('Nom est requis'),
        description: Yup.string().required('Description est requis'),
        active: Yup.boolean().required('Le statut actif est requis'),
        amountFreeShipping: Yup.number().min(0, 'Le montant doit être positif'),
        nameFreeShipping: Yup.string(),
        mondialRelayMode: Yup.mixed().oneOf(Object.values(MondialRelayMode)).required(),
        deliveryCostMondialRelayList: Yup.array()
            .of(
                Yup.object().shape({
                    id: Yup.string().nullable(),
                    weightCart: Yup.number().min(0, 'Le poids doit être positif').nullable(),
                    weightPackaging: Yup.number().min(0, 'Le poids doit être positif').nullable(),
                    cost: Yup.number().min(0, 'Le coût doit être positif').nullable(),
                })
            ).notRequired()
    });

    const defaultValues = useMemo(() => ({
        id: shippingMethod?.id,
        name: shippingMethod?.name || '',
        description: shippingMethod?.description || '',
        active: shippingMethod?.active || false,
        amountFreeShipping: shippingMethod?.amountFreeShipping || 0,
        nameFreeShipping: shippingMethod?.nameFreeShipping || '',
        mondialRelayMode: shippingMethod?.mondialRelayMode || MondialRelayMode.MED,
        deliveryCostMondialRelayList: shippingMethod?.deliveryCostMondialRelayList || [],
    }), [shippingMethod]);

    const methods = useForm({
        resolver: yupResolver(NewShippingMethodSchema),
        defaultValues,
    });

    const {
        watch,
        control,
        setValue,
        handleSubmit,
        formState: { isSubmitting },
        reset,
    } = methods;

    useEffect(() => {
        if (shippingMethod) {
            reset(defaultValues);
            setIsActive(defaultValues.active)
            setDataDeliveryCostMondialRelay(defaultValues.deliveryCostMondialRelayList)
        }
    }, [shippingMethod, defaultValues, reset])

    const values = watch();

    const [openAlertErrorAlert, setOpenAlertErrorAlert] = useState(false);
    const [alertErrorMessage, setAlertErrorMessage] = useState('');

    const onSubmit = handleSubmit((data: any) => {
        data.id = id;
        data.deliveryCostMondialRelayList = dataDeliveryCostMondialRelay;
        if (dataDeliveryCostMondialRelay.length === 0) {
            setOpenAlertErrorAlert(true);
            setAlertErrorMessage('Liste des frais d\'expédition est vide');
            return;
        }
        useUpdateMondialRelay(data.id, data).then(() => {
            setOpenSaveAlert(true);
            router.push(paths.dashboard.admin.shipping.general.shippingMethod.shippingMethodList);
            mutateShippingMethod();
            mutateShippingMethods();
            enqueueSnackbar('Méthode de livraison gratuite ajoutée avec succès', { variant: 'success' });
        }).catch((error) => {
            setOpenErrorAlert(true);
            enqueueSnackbar(`Erreur lors de l'ajout de la méthode de livraison gratuite : ${error.message}`, { variant: 'error' });
        });
    });

    const [formDataCostMondialRelay, setFormDataCostMondialRelay] = useState<DeliveryCostMondialRelay>({
        id: null,
        weightCart: null,
        weightPackaging: null,
        cost: null,
    });

    const [selectedMondialRelayMode, setSelectedMondialRelayMode] = useState<MondialRelayMode>(MondialRelayMode.MED);

    const [openCostModal, setOpenCostModal] = useState(false);
    const [isEdit, setIsEdit] = useState(false);

    const [isActive, setIsActive] = useState(false);

    const handleOpenCostModal = (edit: boolean, index: number | null = null) => {
        setIsEdit(edit);
        if (edit && index !== null) {
            const row = dataDeliveryCostMondialRelay[index];
            if (row) {
                setFormDataCostMondialRelay({
                    id: row.id,
                    weightCart: row.weightCart,
                    weightPackaging: row.weightPackaging,
                    cost: row.cost,
                });
            }
        } else {
            setFormDataCostMondialRelay({
                id: null,
                weightCart: null,
                weightPackaging: null,
                cost: null,
            });
        }
        setOpenCostModal(true);
    };

    const handleSaveCostMondialRelay = () => {
        if (formDataCostMondialRelay.id !== null) {
            const updatedList = dataDeliveryCostMondialRelay.map((item) => (item.id === formDataCostMondialRelay.id ? formDataCostMondialRelay : item));
            setDataDeliveryCostMondialRelay(updatedList);
        } else {
            setDataDeliveryCostMondialRelay([...dataDeliveryCostMondialRelay, { ...formDataCostMondialRelay, id: Date.now().toString() }]);
        }
        setOpenCostModal(false);
    };

    const handleRowCheckboxChange = (index: number, checked: boolean, list: DeliveryCostMondialRelay[], setList: React.Dispatch<React.SetStateAction<DeliveryCostMondialRelay[]>>) => {
        const updatedList = [...list];
        updatedList[index].checked = checked;
        setList(updatedList);
    };

    const handleDeleteSelectedRows = (list: DeliveryCostMondialRelay[], setList: React.Dispatch<React.SetStateAction<DeliveryCostMondialRelay[]>>) => {
        const updatedList = list.filter((item) => !item.checked);
        setList(updatedList);
        confirmDeleteCost.onFalse();
    };

    const confirmDeleteCost = useBoolean();

    const handleDeleteCost = (index: number) => {
        handleRowCheckboxChange(index, true, dataDeliveryCostMondialRelay, setDataDeliveryCostMondialRelay);
        confirmDeleteCost.onTrue();
    };

    const handleChangeMondialRelayMode = (event: any, value: any, reason: any, onChange: any) => {
        setSelectedMondialRelayMode(value);
        onChange(value);
    };

    const [openSaveAlert, setOpenSaveAlert] = useState(false);

    const handleClickSaveAlert = () => {
        setOpenSaveAlert(true);
    };

    const handleCloseSaveAlert = (event?: React.SyntheticEvent | Event, reason?: string) => {
        if (reason === 'clickaway') {
            return;
        }

        setOpenSaveAlert(false);
    };

    const [openErrorAlert, setOpenErrorAlert] = useState(false);

    const handleClickErrorAlert = () => {
        setOpenErrorAlert(true);
    };

    const handleCloseErrorAlert = (event?: React.SyntheticEvent | Event, reason?: string) => {
        if (reason === 'clickaway') {
            return;
        }

        setOpenErrorAlert(false);
    };

    if (shippingMethodLoading) {
        return <Grid container spacing={3} alignItems="center" justifyContent="center"><Grid md={12}><LoadingScreen /></Grid></Grid>;
    }

    if (shippingMethodError) {
        return <Grid container spacing={3} alignItems="center" justifyContent="center"><Grid md={12}><Alert severity="warning">Problème lors de la récupération de la méthode de livraison. Veuillez réessayer plus tard.</Alert></Grid></Grid>;
    }

    const renderHead = (
        <Grid md={12}>
            <CustomBreadcrumbs
                heading="Modifier méthode de livraison"
                links={[
                    { name: 'Tableau de bord', href: paths.dashboard.root },
                    { name: 'Méthodes de livraison', href: paths.dashboard.admin.shipping.general.shippingMethod.shippingMethodList },
                    { name: shippingMethod?.type },
                ]}
                sx={{
                    mb: { xs: 3, md: 5 },
                }}
            />
        </Grid>
    );

    const renderCostModal = (open: boolean, handleClose: () => void, title: string, handleSave: () => void) => (
        <Modal open={open} onClose={handleClose}>
            <Box sx={modalStyle}>
                <Typography variant="h6" align="center" sx={{ mt: 2, mb: 3 }} gutterBottom>
                    {title}
                </Typography>
                <Box sx={modalContentStyle}>
                    <TextField
                        label="Poids du panier jusqu'à (g)"
                        placeholder="0.00"
                        type="number"
                        sx={{ m: 1, width: '25ch' }}
                        InputLabelProps={{ shrink: true }}
                        InputProps={{ startAdornment: <InputAdornment position="start">g</InputAdornment> }}
                        value={formDataCostMondialRelay.weightCart ?? ''}
                        onChange={(e) => setFormDataCostMondialRelay({ ...formDataCostMondialRelay, weightCart: e.target.value ? Number(e.target.value) : null })}
                    />
                    <TextField
                        label="Poids d'emballage (g)"
                        placeholder="0.00"
                        type="number"
                        sx={{ m: 1, width: '25ch' }}
                        InputLabelProps={{ shrink: true }}
                        InputProps={{ startAdornment: <InputAdornment position="start">g</InputAdornment> }}
                        value={formDataCostMondialRelay.weightPackaging ?? ''}
                        onChange={(e) => setFormDataCostMondialRelay({ ...formDataCostMondialRelay, weightPackaging: e.target.value ? Number(e.target.value) : null })}
                    />
                    <TextField
                        label="Prix (EUR €)"
                        placeholder="0.00"
                        type="number"
                        sx={{ m: 1, width: '25ch' }}
                        InputLabelProps={{ shrink: true }}
                        InputProps={{ startAdornment: <InputAdornment position="start">€</InputAdornment> }}
                        value={formDataCostMondialRelay.cost ?? ''}
                        onChange={(e) => setFormDataCostMondialRelay({ ...formDataCostMondialRelay, cost: e.target.value ? Number(e.target.value) : null })}
                    />
                    <Box sx={{ display: 'flex', justifyContent: 'center', mt: 4 }}>
                        <Button onClick={handleClose} sx={{ mr: 1 }} variant="outlined" color="inherit">Annuler</Button>
                        <Button onClick={handleSave} variant="contained" color="inherit">{title.split(' ')[0]}</Button>
                    </Box>
                </Box>
            </Box>
        </Modal>
    );

    const renderForm = (
        <>
            {mdUp && (
                <Grid xs={12} md={4} sx={{ mt: 4 }}>
                    <Typography variant="h6" sx={{ mb: 0.5 }}>
                        Mondial Relay - InPost
                    </Typography>
                    <Typography variant="body2" sx={{ color: 'text.secondary' }}>
                        Saisissez les informations de cette méthode de livraison
                    </Typography>
                </Grid>
            )}

            <Grid xs={12} md={8}>
                <Card>
                    <Stack spacing={3} sx={{ p: 3 }}>
                        <Controller
                            name="name"
                            control={control}
                            render={({ field }) => (
                                <RHFTextField {...field} label="Nom de la méthode" />
                            )}
                        />
                        <Controller
                            name="description"
                            control={control}
                            render={({ field }) => (
                                <RHFTextField {...field} label="Description" placeholder="Description..." multiline rows={4} />
                            )}
                        />
                        <Controller
                            name="active"
                            control={control}
                            defaultValue={isActive}
                            render={({ field }) => (
                                <div style={{ display: 'flex', alignItems: 'center' }}>
                                    <label>Activé</label>
                                    <Switch
                                        {...field}
                                        checked={isActive}
                                        onChange={(e) => {
                                            setIsActive(e.target.checked);
                                            field.onChange(e.target.checked);
                                        }}
                                    />
                                </div>
                            )}
                        />
                    </Stack>
                </Card>
                <Card sx={{ mt: 3 }}>
                    {!mdUp && <CardHeader title="Détails" />}
                    <Stack spacing={3} sx={{ p: 3 }}>
                        <RHFTextField
                            name="amountFreeShipping"
                            label="Offrir la livraison à partir de ce montant (optionnel)"
                            placeholder="0.00"
                            type="number"
                            InputLabelProps={{ shrink: true }}
                            InputProps={{
                                startAdornment: (
                                    <InputAdornment position="start">
                                        <Box component="span" sx={{ color: 'text.disabled' }}>
                                            €
                                        </Box>
                                    </InputAdornment>
                                ),
                            }}
                        />
                        <RHFTextField name="nameFreeShipping" label="Nom si la livraison est offerte" />

                        <Controller
                            name="mondialRelayMode"
                            control={control}
                            defaultValue={MondialRelayMode.MED}
                            render={({ field }) => (
                                <Autocomplete
                                    disabled
                                    {...field}
                                    options={Object.values(MondialRelayMode)}
                                    getOptionLabel={(option) => MondialRelayModeLabels[option as MondialRelayMode]}
                                    onChange={(event, value, reason) => handleChangeMondialRelayMode(event, value, reason, field.onChange)}
                                    value={selectedMondialRelayMode}
                                    renderInput={(params) => <TextField {...params} label="Mode de livraison" placeholder="+ Mode de livraison" />}
                                    renderOption={(props, option) => (
                                        <li {...props} key={option}>
                                            {MondialRelayModeLabels[option as MondialRelayMode]}
                                        </li>
                                    )}
                                    renderTags={(selected, getTagProps) =>
                                        selected.map((option, index) => (
                                            <Chip
                                                {...getTagProps({ index })}
                                                key={option}
                                                label={MondialRelayModeLabels[option as MondialRelayMode]}
                                                size="small"
                                                color="info"
                                                variant="soft"
                                            />
                                        ))
                                    }
                                />
                            )}
                        />
                        <Box sx={{ display: 'flex', alignItems: 'center', gap: 1 }}>
                            <Button
                                variant="outlined"
                                size="small"
                                sx={{ color: '#808080', width: '15%' }}
                                onClick={() => handleOpenCostModal(false)}
                                startIcon={<AddIcon />}
                            >
                                Ajouter
                            </Button>
                            <Button
                                variant="outlined"
                                size="small"
                                sx={{ color: '#808080', width: '15%' }}
                                onClick={() => handleDeleteSelectedRows(dataDeliveryCostMondialRelay, setDataDeliveryCostMondialRelay)}
                                startIcon={<DeleteIcon />}
                            >
                                Supprimer
                            </Button>
                        </Box>
                        <TableContainer component={Paper} sx={{ mt: 1, maxHeight: '500px' }}>
                            <Table sx={{ minWidth: 650 }} aria-label="simple table">
                                <TableHead>
                                    <TableRow>
                                        <TableCell align="center">
                                            <Checkbox
                                                indeterminate={dataDeliveryCostMondialRelay.length > 0 && dataDeliveryCostMondialRelay.some(item => item.checked)}
                                                checked={dataDeliveryCostMondialRelay.length > 0 && dataDeliveryCostMondialRelay.every(item => item.checked)}
                                                onChange={(event) => {
                                                    const checked = event.target.checked;
                                                    const updatedList = dataDeliveryCostMondialRelay.map(item => ({ ...item, checked }));
                                                    setDataDeliveryCostMondialRelay(updatedList);
                                                }}
                                            />
                                        </TableCell>
                                        <TableCell align="center">Poids du panier jusqu'à (g)</TableCell>
                                        <TableCell align="center">Poids d'emballage (g)</TableCell>
                                        <TableCell align="center">Prix (EUR €)</TableCell>
                                        <TableCell align="center">Actions</TableCell>
                                    </TableRow>
                                </TableHead>
                                <TableBody>
                                    {dataDeliveryCostMondialRelay.map((row, index) => (
                                        <TableRow key={index} sx={{ '&:last-child td, &:last-child th': { border: 0 } }}>
                                            <TableCell component="th" scope="row" align="center">
                                                <Checkbox
                                                    checked={row.checked || false}
                                                    onChange={(event) => handleRowCheckboxChange(index, event.target.checked, dataDeliveryCostMondialRelay, setDataDeliveryCostMondialRelay)}
                                                />
                                            </TableCell>
                                            <TableCell align="center">{row.weightCart} g</TableCell>
                                            <TableCell align="center">{row.weightPackaging} g</TableCell>
                                            <TableCell align="center">{row.cost} €</TableCell>
                                            <TableCell align="center">
                                                <Box>
                                                    <IconButton onClick={() => handleOpenCostModal(true, index)}>
                                                        <EditIcon sx={{ color: 'green' }} />
                                                    </IconButton>
                                                    <IconButton onClick={() => handleDeleteCost(index)}>
                                                        <DeleteIcon sx={{ color: 'red' }} />
                                                    </IconButton>
                                                </Box>
                                            </TableCell>
                                        </TableRow>
                                    ))}
                                </TableBody>
                            </Table>
                        </TableContainer>
                    </Stack>
                </Card>
            </Grid>

            <ConfirmDialog
                open={confirmDeleteCost.value}
                onClose={confirmDeleteCost.onFalse}
                title="Supprimer frais"
                content="Voulez-vous vraiment supprimer?"
                action={
                    <Button variant="contained" color="error" onClick={() => handleDeleteSelectedRows(dataDeliveryCostMondialRelay, setDataDeliveryCostMondialRelay)}>
                        Supprimer
                    </Button>
                }
            />

            {renderCostModal(openCostModal, () => setOpenCostModal(false), isEdit ? "Modifier frais de livraison" : "Ajouter frais de livraison", handleSaveCostMondialRelay)}

            <Snackbar open={openSaveAlert} autoHideDuration={2000}
                onClose={handleCloseSaveAlert} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
                <Alert
                    onClose={handleCloseSaveAlert}
                    severity="success"
                    variant="filled"
                    sx={{ width: '100%' }}
                >
                    Méthode de livraison enregistrer avec succès
                </Alert>
            </Snackbar>
            <Snackbar open={openErrorAlert} autoHideDuration={3000}
                onClose={handleCloseErrorAlert} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
                <Alert
                    onClose={handleCloseErrorAlert}
                    severity="error"
                    variant="filled"
                    sx={{ width: '100%' }}
                >
                    Échec
                </Alert>
            </Snackbar>
            <Snackbar open={openAlertErrorAlert} autoHideDuration={3000}
                onClose={() => setOpenAlertErrorAlert(false)} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
                <Alert
                    onClose={() => setOpenAlertErrorAlert(false)}
                    severity="error"
                    variant="filled"
                    sx={{ width: '100%' }}
                >
                    {alertErrorMessage}
                </Alert>
            </Snackbar>
        </>
    );



    const renderActions = (
        <>
            {mdUp && <Grid md={4} />}
            <Grid xs={12} md={8} sx={{ display: 'flex', justifyContent: 'end', py: 4 }}>
                <LoadingButton
                    type="submit"
                    variant="contained"
                    size="large"
                    loading={isSubmitting}
                    sx={{ ml: 2 }}
                >Enregistrer</LoadingButton>
            </Grid>
        </>
    );


    return (
        <>
            <FormProvider methods={methods} onSubmit={onSubmit}>
                <Grid container spacing={3}>
                    {renderHead}
                    {renderForm}
                    {renderActions}
                </Grid>
            </FormProvider>
        </>
    );
}

