'use client';

import { useState, useCallback, useEffect } from 'react';
import Card from '@mui/material/Card';
import Table from '@mui/material/Table';
import Stack from '@mui/material/Stack';
import Button from '@mui/material/Button';
import Grid from '@mui/material/Grid';
import Container from '@mui/material/Container';
import TableBody from '@mui/material/TableBody';
import TableCell from '@mui/material/TableCell';
import TableHead from '@mui/material/TableHead';
import TableContainer from '@mui/material/TableContainer';
import LocalizationProvider from '@/shared/locales/localization-provider';
import { paths } from '@/routes/paths';
import { useRouter } from '@/hooks';
import Iconify from '@/shared/components/iconify';
import Scrollbar from '@/shared/components/scrollbar';
import { useSnackbar } from '@/components/snackbar';
import { useSettingsContext } from '@/shared/components/settings';
import CustomBreadcrumbs from '@/components/custom-breadcrumbs';
import { useTable, emptyRows, TableNoData, TablePaginationCustom, TableEmptyRows, } from '@/shared/components/table';
import { ShippingMethod, ShippingMethodType } from '@/contexts/types/Shipping';
import ShippingMethodTableRow from './ShippingMethodTableRow';
import RouterLink from '@/routes/router-link';
import { useDeleteShippingMethod, useGetShippingMethods } from '@/shared/api/shipping';
import { Alert, Snackbar } from '@mui/material';
import { LoadingScreen } from '@/shared/components/loading-screen';


export default function ShippingMethodTableView() {
    const { enqueueSnackbar } = useSnackbar();
    const settings = useSettingsContext();
    const router = useRouter();
    const table = useTable({ defaultOrderBy: 'title' });
    const { shippingMethodData, shippingMethodLoading, shippingMethodError, mutateShippingMethods } = useGetShippingMethods();
    const [tableData, setTableData] = useState<ShippingMethod[]>(shippingMethodData);

    const denseHeight = table.dense ? 56 : 56 + 20;
    const notFound = (!tableData.length) || !tableData.length;

    useEffect(() => {
        setTableData(shippingMethodData);
    }, [shippingMethodData])

    const handleEditRow = useCallback((id: string, type: ShippingMethodType) => {
        switch (type) {
            case ShippingMethodType.FreeShipping:
                router.push(paths.dashboard.admin.shipping.general.shippingMethod.editFreeShipping(id));
                break;
            case ShippingMethodType.FlatRate:
                router.push(paths.dashboard.admin.shipping.general.shippingMethod.editFlatRate(id));
                break;
            case ShippingMethodType.PointOfSale:
                router.push(paths.dashboard.admin.shipping.general.shippingMethod.editPointOfSale(id));
                break;
            case ShippingMethodType.ColissimoWithoutSignature:
                router.push(paths.dashboard.admin.shipping.general.shippingMethod.editColissimoWithoutSignature(id));
                break;
            case ShippingMethodType.ColissimoWithSignature:
                router.push(paths.dashboard.admin.shipping.general.shippingMethod.editColissimoWithSignature(id));
                break;
            case ShippingMethodType.MondialRelay:
                router.push(paths.dashboard.admin.shipping.general.shippingMethod.editMondialRelay(id));
                break;
            default:
                return null;
        }
    }, [router]);

    const [selectedShippingMethod, setSelectedShippingMethod] = useState<ShippingMethod | null>(null);
    const handelSelectedShippingMethod = (shippingArea: any) => {
        setSelectedShippingMethod(shippingArea);
    };
    const handleViewRow = useCallback(
        (id: string) => {
            const shippingArea = tableData.find((a) => a.id === id);
            handelSelectedShippingMethod(shippingArea)
        },
        [tableData]
    );

    const handleDeleteRow = useCallback(
        async (id: string) => {
            try {
                await useDeleteShippingMethod(id);
                const updatedTableData = tableData.filter((row) => row.id !== id);
                setTableData(updatedTableData);
                table.onUpdatePageDeleteRow(updatedTableData.length);
                setOpenDeleteAlert(true);
                mutateShippingMethods();
            } catch (error) {
                setOpenErrorAlert(true);
                console.error(`Error deleting shipping method with ID ${id}:`, error);
                enqueueSnackbar('Error in deleting shipping method', { variant: 'error' });
            }
        },
        [tableData, table, enqueueSnackbar]
    );

    const [openSaveAlert, setOpenSaveAlert] = useState(false);
    const handleCloseSaveAlert = (event?: React.SyntheticEvent | Event, reason?: string) => {
        if (reason === 'clickaway') {
            return;
        }
        setOpenSaveAlert(false);
    };

    const [openDeleteAlert, setOpenDeleteAlert] = useState(false);
    const handleCloseDeleteAlert = (event?: React.SyntheticEvent | Event, reason?: string) => {
        if (reason === 'clickaway') {
            return;
        }
        setOpenDeleteAlert(false);
    };

    const [openErrorAlert, setOpenErrorAlert] = useState(false);
    const handleCloseErrorAlert = (event?: React.SyntheticEvent | Event, reason?: string) => {
        if (reason === 'clickaway') {
            return;
        }
        setOpenErrorAlert(false);
    };

    if (shippingMethodLoading) {
        return (
            <Grid container spacing={3} sx={{ justifyContent: 'center', alignItems: 'center', height: '100vh' }}>
                <LoadingScreen />
            </Grid>
        );
    }

    if (shippingMethodError) {
        return (
            <Grid container spacing={3} sx={{ justifyContent: 'center', alignItems: 'center', height: '100vh' }}>
                <Alert severity="error">Problème lors de la récupération des données des méthodes de livraison. Veuillez réessayer plus tard.</Alert>
            </Grid>
        );
    }

    return (
        <>
            <LocalizationProvider >
                <Container maxWidth={settings.themeStretch ? false : 'lg'}>
                    <Grid container alignItems="center" justifyContent="space-between">
                        <Grid item>
                            <CustomBreadcrumbs
                                heading="Méthodes de livraison"
                                links={[
                                    {
                                        name: 'Tableau de bord',
                                        href: paths.dashboard.root,
                                    },
                                    {
                                        name: "Livraison",
                                    }
                                ]}
                                sx={{
                                    mb: { xs: 3, md: 5 },
                                }}
                            />
                        </Grid>
                        <Grid item>
                            <Stack
                                spacing={3}
                                justifyContent="space-between"
                                alignItems={{ xs: 'flex-end', sm: 'center' }}
                                direction={{ xs: 'column', sm: 'row' }}
                            >
                                <Button
                                    component={RouterLink}
                                    href={paths.dashboard.admin.shipping.general.shippingMethod.createShippingMethod}
                                    variant="contained"
                                    startIcon={<Iconify icon="mingcute:add-line" />}
                                >
                                    Ajouter
                                </Button>
                            </Stack>
                        </Grid>
                    </Grid>

                    <Card>
                        <TableContainer sx={{ position: 'relative', overflow: 'unset' }}>
                            <TableHead sx={{ width: '100%' }} >
                                <TableCell width={'30%'}>Nom</TableCell>
                                <TableCell width={'20%'}>Activé</TableCell>
                                <TableCell width={'25%'}>Description</TableCell>
                                <TableCell width={'20%'}>Zones d'expédition</TableCell>
                                <TableCell width={'5%'}>Actions</TableCell>
                            </TableHead>

                            <Scrollbar>
                                <Table size={table.dense ? 'small' : 'medium'} sx={{ minWidth: 800 }}>
                                    <TableBody>
                                        {tableData
                                            .slice(
                                                table.page * table.rowsPerPage,
                                                table.page * table.rowsPerPage + table.rowsPerPage
                                            )
                                            .map((row, index) => (
                                                <ShippingMethodTableRow
                                                    key={row.id}
                                                    row={row}
                                                    selected={table.selected.includes(row.id)}
                                                    onSelectRow={() => table.onSelectRow(row.id)}
                                                    onViewRow={() => handleViewRow(row.id)}
                                                    selectedShippingMethod={selectedShippingMethod}
                                                    onEditRow={() => handleEditRow(row.id, row.type)}
                                                    onDeleteRow={() => handleDeleteRow(row.id)}
                                                />
                                            ))}

                                        <TableEmptyRows
                                            height={denseHeight}
                                            emptyRows={emptyRows(table.page, table.rowsPerPage, tableData.length)}
                                        />

                                        <TableNoData notFound={notFound} />
                                    </TableBody>
                                </Table>
                            </Scrollbar>
                        </TableContainer>

                        <TablePaginationCustom
                            count={tableData.length}
                            page={table.page}
                            rowsPerPage={table.rowsPerPage}
                            onPageChange={table.onChangePage}
                            onRowsPerPageChange={table.onChangeRowsPerPage}
                            dense={table.dense}
                            onChangeDense={table.onChangeDense}
                        />
                    </Card>

                </Container>
            </LocalizationProvider>
            <Snackbar open={openSaveAlert} autoHideDuration={3000}
                onClose={handleCloseSaveAlert} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
                <Alert
                    onClose={handleCloseSaveAlert}
                    severity="success"
                    variant="filled"
                    sx={{ width: '100%' }}
                >
                    Méthode de livraison enregistrer avec succès
                </Alert>
            </Snackbar>
            <Snackbar open={openDeleteAlert} autoHideDuration={3000}
                onClose={handleCloseDeleteAlert} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
                <Alert
                    onClose={handleCloseDeleteAlert}
                    severity="success"
                    variant="filled"
                    sx={{ width: '100%' }}
                >
                    Méthode de livraison supprimer avec succès
                </Alert>
            </Snackbar>
            <Snackbar open={openErrorAlert} autoHideDuration={3000}
                onClose={handleCloseErrorAlert} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
                <Alert
                    onClose={handleCloseErrorAlert}
                    severity="error"
                    variant="filled"
                    sx={{ width: '100%' }}
                >
                    Échec
                </Alert>
            </Snackbar>
        </>
    );
}
