import React, { useState, useEffect } from 'react';
import Grid from '@mui/material/Grid';
import { DatePicker } from '@mui/x-date-pickers/DatePicker';
import StatisticsDeliveredOrders from './StatisticsDeliveredOrders';
import CustomBreadcrumbs from '@/components/custom-breadcrumbs/custom-breadcrumbs';
import { paths } from '@/routes/paths';
import { FormControl, InputLabel, Select, MenuItem } from '@mui/material';

// Définition des types
type Commande = {
    id: string;
    codepostal: string;
    dateLivraison: Date;
    nomPays: string;
    codePays: string;
    nomVille: string;
};

type Country = {
    nomPays: string;
    codePays: string;
    villes: string[];
};

export default function ShippingStatisticsView() {
    // Simulated commandes data (should be fetched or initialized properly)
    let commandes: Commande[] = [];

    // Générer les données pour les trois pays
    const countries: Country[] = [
        { nomPays: 'France', codePays: 'FR', villes: ['Paris', 'Marseille', 'Lyon'] },
        { nomPays: 'Italie', codePays: 'IT', villes: ['Rome', 'Milan', 'Naples'] },
        { nomPays: 'Espagne', codePays: 'ES', villes: ['Madrid', 'Barcelona', 'Valencia'] },
    ];

    countries.forEach(country => {
        commandes.push(...generateCountryData(country, 30));
    });

    const getCitiesForCountry = (countryCode: string) => {
        return Array.from(new Set(commandes
            .filter(commande => commande.codePays === countryCode)
            .map(commande => commande.nomVille)));
    };

    // Initialiser la sélection du pays avec la France
    const defaultSelectedCountry = countries[0];
    const defaultSelectedCity = "all";

    const [selectedCountry, setSelectedCountry] = useState<Country>(defaultSelectedCountry);
    const [selectedCity, setSelectedCity] = useState<string>(defaultSelectedCity);
    const [selectedStartDate, setSelectedStartDate] = useState<Date | null>(null);
    const [selectedEndDate, setSelectedEndDate] = useState<Date | null>(null);
    const [cities, setCities] = useState<string[]>(getCitiesForCountry(defaultSelectedCountry.codePays));
    const [uniquePrefixes, setUniquePrefixes] = useState<string[]>([]);
    const [selectedPrefix, setSelectedPrefix] = useState<string>("all");
    const [sourceData, setSourceData] = useState<Commande[]>(commandes);
    const [chartData, setChartData] = useState<Commande[]>([]);

    // Appliquer le filtre par pays au chargement initial de la page
    useEffect(() => {
        const filteredData = sourceData.filter(commande => commande.codePays === selectedCountry.codePays);
        setChartData(filteredData);
        setUniquePrefixes(Array.from(new Set(filteredData.map(commande => commande.codepostal.substr(0, 2)))));
    }, [sourceData, selectedCountry]);

    // Méthode pour filtrer les données
    const applyFilters = () => {
        let filteredData = sourceData;

        // Filtrer par pays
        filteredData = filteredData.filter(commande => commande.codePays === selectedCountry.codePays);

        // Filtrer par ville
        if (selectedCity && selectedCity !== "all") {
            filteredData = filteredData.filter(commande => commande.nomVille === selectedCity);
        }

        // Mettre à jour les préfixes uniques en fonction du filtre actuel
        const updatedPrefixes = Array.from(new Set(filteredData.map(commande => commande.codepostal.substr(0, 2))));
        setUniquePrefixes(updatedPrefixes);

        // Filtrer par préfixe
        if (selectedPrefix && selectedPrefix !== "all") {
            filteredData = filteredData.filter(commande => commande.codepostal.startsWith(selectedPrefix));
        }

        // Filtrer par date
        if (selectedStartDate && selectedEndDate) {
            filteredData = filteredData.filter(commande => {
                const commandDate = new Date(commande.dateLivraison);
                return commandDate >= selectedStartDate && commandDate <= selectedEndDate;
            });
        }

        setChartData(filteredData);
    };

    // Appliquer les filtres de ville, préfixe et date à chaque changement
    useEffect(() => {
        applyFilters();
    }, [selectedCity, selectedPrefix, selectedCountry, selectedStartDate, selectedEndDate]);

    useEffect(() => {
        const newCities = getCitiesForCountry(selectedCountry.codePays);
        setCities(newCities);
        setSelectedCity('all');
    }, [selectedCountry]);

    const handleStartDateChange = (date: Date | null) => {
        setSelectedStartDate(date);
    };

    const handleEndDateChange = (date: Date | null) => {
        setSelectedEndDate(date);
    };

    let commandesParCodePostal: { label: string; value: number }[] = [];
    let countMap: { [codePostal: string]: number } = {};

    chartData.forEach(data => {
        if (countMap[data.codepostal]) {
            countMap[data.codepostal]++;
        } else {
            countMap[data.codepostal] = 1;
        }
    });

    for (let codepostal in countMap) {
        commandesParCodePostal.push({ label: codepostal, value: countMap[codepostal] });
    }

    return (
        <>
            <Grid container spacing={2}>
                <Grid item xs={12} md={12}>
                    <CustomBreadcrumbs
                        heading="Statistiques de livraison"
                        links={[
                            {
                                name: 'Tableau de bord',
                                href: paths.dashboard.root,
                            },
                            {
                                name: "Livraison",
                            }
                        ]}
                        sx={{
                            mb: { xs: 3, md: 5 },
                        }}
                    />
                </Grid>
                <Grid item xs={12} md={12}>
                    <FormControl variant="outlined" sx={{ mb: 2, width: "20%" }}>
                        <InputLabel id="country-label">Pays</InputLabel>
                        <Select
                            labelId="country-label"
                            id="country-select"
                            value={selectedCountry.codePays}
                            onChange={(event) => {
                                const countryCode = event.target.value as string;
                                const newSelectedCountry = countries.find(country => country.codePays === countryCode);
                                setSelectedCountry(newSelectedCountry || defaultSelectedCountry);
                                setSelectedCity("all");
                            }}
                            label="Pays"
                        >
                            {countries.map(country => (
                                <MenuItem key={country.codePays} value={country.codePays}>
                                    {country.nomPays}
                                </MenuItem>
                            ))}
                        </Select>
                    </FormControl>

                    <FormControl variant="outlined" sx={{ mb: 2, ml: 1, width: "20%" }}>
                        <InputLabel id="city-label">Ville</InputLabel>
                        <Select
                            labelId="city-label"
                            id="city-select"
                            value={selectedCity}
                            onChange={(event) => {
                                setSelectedCity(event.target.value as string);
                            }}
                            label="Ville"
                        >
                            <MenuItem key={"all"} value={"all"}>Toutes les villes</MenuItem>
                            {cities.map(ville => (
                                <MenuItem key={ville} value={ville}>
                                    {ville}
                                </MenuItem>
                            ))}
                        </Select>
                    </FormControl>
                    <FormControl variant="outlined" sx={{ mb: 2, ml: 1, width: "20%" }}>
                        <InputLabel id="prefix-label">Préfixe</InputLabel>
                        <Select
                            labelId="prefix-label"
                            id="prefix-select"
                            value={selectedPrefix}
                            onChange={(event) => {
                                setSelectedPrefix(event.target.value as string);
                            }}
                            label="Préfixe"
                        >
                            <MenuItem key={"all"} value={"all"}>
                                Tous les préfixes
                            </MenuItem>
                            {uniquePrefixes.map(prefix => (
                                <MenuItem key={prefix} value={prefix}>
                                    {prefix}
                                </MenuItem>
                            ))}
                        </Select>
                    </FormControl>
                    <DatePicker
                        label="Date de début"
                        slotProps={{
                            textField: {
                                fullWidth: true,
                            },
                        }}
                        sx={{
                            width: "15%", ml: 1
                        }}
                        value={selectedStartDate}
                        onChange={handleStartDateChange}
                    />
                    <DatePicker
                        label="Date de fin"
                        slotProps={{
                            textField: {
                                fullWidth: true,
                            },
                        }}
                        sx={{
                            width: "15%", ml:1
                        }}
                        value={selectedEndDate}
                        onChange={handleEndDateChange}
                    />
                </Grid>
                <Grid xs={12} md={12}>
                    <StatisticsDeliveredOrders
                        title="Commandes livrées par code postal"
                        chart={{
                            series: commandesParCodePostal
                        }}
                    />
                </Grid>
            </Grid>
        </>
    );
}

// Fonction pour générer des données de commandes
function generateCountryData(country: Country, numberOfOrders: number): Commande[] {
    let commandes: Commande[] = [];

    // Générer des commandes pour chaque ville du pays
    country.villes.forEach((nomVille) => {
        let prefixCommun = Math.floor(Math.random() * 90); // Générer un préfixe commun initial

        for (let i = 0; i < numberOfOrders / country.villes.length; i += 2) {
            const randomSuffix1 = Math.floor(Math.random() * 10000); // Suffixe aléatoire pour les 4 derniers chiffres
            const randomSuffix2 = Math.floor(Math.random() * 10000); // Suffixe aléatoire pour les 4 derniers chiffres

            // Générer deux commandes avec le même préfixe
            const codePostal1 = `${prefixCommun}${randomSuffix1}`.padStart(6, '0').slice(0, 6);
            const codePostal2 = `${prefixCommun}${randomSuffix2}`.padStart(6, '0').slice(0, 6);

            let dateLivraison1 = generateRandomDate();
            let dateLivraison2 = generateRandomDate();
            let id1 = Math.floor(Math.random() * 123).toString();
            let id2 = Math.floor(Math.random() * 123).toString();

            commandes.push({ id: id1, codepostal: codePostal1, dateLivraison: dateLivraison1, nomPays: country.nomPays, codePays: country.codePays, nomVille });
            commandes.push({ id: id2, codepostal: codePostal2, dateLivraison: dateLivraison2, nomPays: country.nomPays, codePays: country.codePays, nomVille });

            // Incrémenter le préfixe pour la prochaine paire de codes postaux
            prefixCommun = (prefixCommun + 1) % 100; // Assurer que le préfixe reste dans la plage de 0 à 99
        }
    });

    return commandes;
}

// Fonction pour générer une date aléatoire
function generateRandomDate(): Date {
    let year = 2024;
    let month = Math.floor(Math.random() * 12) + 1;
    let day = Math.floor(Math.random() * 28) + 1;
    return new Date(year, month - 1, day);
}
