'use client'

import { useState, useEffect, useMemo } from 'react';
import * as Yup from 'yup';
import { useForm } from 'react-hook-form';
import { yupResolver } from '@hookform/resolvers/yup';
import Grid from '@mui/material/Unstable_Grid2';
import Card from '@mui/material/Card';
import Stack from '@mui/material/Stack';
import Typography from '@mui/material/Typography';
import { useSnackbar } from '@/components/snackbar';
import LoadingButton from '@mui/lab/LoadingButton';
import Checkbox from '@mui/material/Checkbox';
import FormProvider, {
    RHFEditor,
    RHFTextField,
} from "@/shared/components/hook-form";
import { EmailConfig } from '@/contexts/types/Shipping';
import { useResponsive } from '@/hooks/use-responsive';
import { IMedia } from '@/shared/types/media';
import AddMediaButton from '@/shared/sections/media/Add-media-button';



export default function ConfigEmailView() {

    const { enqueueSnackbar } = useSnackbar();
    const mdUp = useResponsive('up', 'md');

    const ConfigEmailSchema = Yup.object().shape({
        subject: Yup.string(),
        description: Yup.string(),
    });

    const defaultSubject = "Votre commande n°[expedition] a été expédiée.";
    const defaultDescription = `Bonjour [prenom] [nom], <br/><br/>Nous vous informons que votre commande a été expédiée. Vous la recevrez sous peu, mais si vous souhaitez garder un œil dessus, vous pouvez vous rendre sur [lien] pour suivre son trajet. <br/>L'expédition [expedition] sera livrée à l’adresse suivante : [adresse]. <br/><br/>N'hésitez pas à nous contacter pour toute information complémentaire. <br/><br/>Votre service client`;

    const emailConfigValue: EmailConfig = {
        subject: defaultSubject,
        description: defaultDescription
    };

    const defaultValues = useMemo(() => ({
        subject: emailConfigValue.subject,
        description: emailConfigValue.description,
    }), []);

    const methods = useForm({
        resolver: yupResolver(ConfigEmailSchema),
    });

    const {
        reset,
        handleSubmit,
        formState: { isSubmitting },
    } = methods;

    useEffect(() => {
        reset(defaultValues);
        console.log(defaultValues)
    }, [defaultValues, reset])

    const onSubmit = handleSubmit(async (data) => {
        try {
            await new Promise((resolve) => setTimeout(resolve, 500));
            enqueueSnackbar('Success config!', { variant: 'success' });
            console.info('Form Data:', data);
        } catch (error) {
            console.error(error);
            enqueueSnackbar('Failed config.', { variant: 'error' });
        }
    });

    const [checked, setChecked] = useState(true);

    const handleChange = (event: React.ChangeEvent<HTMLInputElement>) => {
        setChecked(event.target.checked);
    };

    const [returnedMultiMedia, setReturnedMultiMedia] = useState<IMedia[] | undefined>();


    const renderBasic = (
        <>
            <Grid xs={12} md={12}>

                <Stack spacing={3}>

                    <Typography variant="h6" sx={{ mb: 0.5 }}>
                        Configuration des emails
                    </Typography>
                    <Grid container alignItems="center" spacing={1} mt={1}>
                        <Grid>
                            <Checkbox
                                name='sendEmailToCustomerWhenCreatingExpedition'
                                checked={checked}
                                onChange={handleChange}
                                
                                inputProps={{ 'aria-label': 'primary checkbox' }}
                            />
                        </Grid>
                        <Grid>
                            <Typography>Envoyer un email au client lors de la création d'une expédition</Typography>
                        </Grid>
                    </Grid>
                    <Typography component="div">
                        <Typography sx={{ mb: 0.5 }} fontSize={14} mt={3}>
                            <strong style={{ fontWeight: 'bold' }}>Shortcodes :</strong>
                        </Typography>
                        <Typography fontSize={13}>
                            <span>[lien] :</span> affichage du lien de suivi du colis <span style={{ fontStyle: 'italic' }}>(obligatoire)</span>
                        </Typography>
                        <Typography fontSize={13}>
                            <span>[expedition] :</span> numéro d'expédition <span style={{ fontStyle: 'italic' }}>(obligatoire)</span>
                        </Typography>
                        <Typography fontSize={13}>
                            <span>[prenom] :</span> Prénom du client
                        </Typography>
                        <Typography fontSize={13}>
                            <span>[nom] :</span> Nom du client
                        </Typography>
                        <Typography fontSize={13}>
                            <span>[adresse] :</span> Adresse de livraison
                        </Typography>
                    </Typography>

                    <Card>
                        <Stack spacing={1.5} p={2}>
                            <Grid container alignItems="center" spacing={1} mt={1}>
                                <Grid md={1}>
                                    <Typography variant="subtitle2">Sujet</Typography>
                                </Grid>
                                <Grid md={11}>
                                    <RHFTextField name="subject" sx={{ width: '100%' }} />
                                </Grid>
                            </Grid>
                            <Grid container spacing={1} mt={1}>
                                <AddMediaButton
                                    multiple
                                    variant="outlined" color="success" 
                                    size="medium" sx={{ mt: 1 }} 
                                    returnedMultiMedia={returnedMultiMedia}
                                    setReturnedMultiMedia={setReturnedMultiMedia}/>
                            </Grid>
                            <RHFEditor simple name="description"
                                onChange={(value) => {
                                    console.log(value);
                                }}
                            />
                        </Stack>
                    </Card>
                </Stack>
            </Grid>
        </>
    );

    const renderActions = (
        <>
            {mdUp && <Grid md={4} />}
            <Grid xs={12} md={8} sx={{ display: 'flex', justifyContent: 'end', py: 4 }}>
                <LoadingButton
                    type="submit"
                    variant="contained"
                    size="large"
                    loading={isSubmitting}
                    sx={{ ml: 2 }}
                >Enregistrer</LoadingButton>

            </Grid>
        </>
    );



    return (
        <FormProvider methods={methods} onSubmit={onSubmit}>
            <Grid container spacing={3}>
                {renderBasic}
                {renderActions}
            </Grid>
        </FormProvider>
    );
}
