import { useState, useCallback } from "react";
import MenuItem from "@mui/material/MenuItem";
import Checkbox from "@mui/material/Checkbox";
import Box from "@mui/material/Box";
import InputLabel from "@mui/material/InputLabel";
import FormControl from "@mui/material/FormControl";
import OutlinedInput from "@mui/material/OutlinedInput";
import { DatePicker } from '@mui/x-date-pickers/DatePicker';
import Select, { SelectChangeEvent } from "@mui/material/Select";
import Iconify from "@/shared/components/iconify";
import { formHelperTextClasses } from '@mui/material/FormHelperText';
import CustomPopover, { usePopover } from "@/shared/components/custom-popover";
import { Toolbar, IconButton, Menu, Typography } from '@mui/material';
import { GridFilterAltIcon } from "@mui/x-data-grid";
import { ExpeditionMondialRelayTableFilters, ExpeditionMondialRelayTableFilterValue } from "@/contexts/types/Shipping";


type Props = {
  filters: ExpeditionMondialRelayTableFilters;
  onFilters: (name: string, value: ExpeditionMondialRelayTableFilterValue) => void;
  dateError: boolean;
  //
  shippingModeOptions: {
    value: string;
    label: string;
  }[];
  statusOptions: {
    value: string;
    label: string;
  }[];
};

export default function ExpeditionMondialRelayTableToolbar({
  filters,
  onFilters,
  //
  shippingModeOptions,
  statusOptions,
  dateError
}: Props) {
  const popover = usePopover();

  const [shippingMode, setShippingMode] = useState<string[]>(filters.shippingMode);

  const [status, setStatus] = useState<string[]>(filters.status);

  const handleChangeShippingMode = useCallback(
    (event: SelectChangeEvent<string[]>) => {
      const {
        target: { value },
      } = event;
      setShippingMode(typeof value === "string" ? value.split(",") : value);
    },
    []
  );

  const handleChangeStatus = useCallback(
    (event: SelectChangeEvent<string[]>) => {
      const {
        target: { value },
      } = event;
      setStatus(typeof value === "string" ? value.split(",") : value);
    },
    []
  );

  const handleCloseShippingMode = useCallback(() => {
    onFilters("shippingMode", shippingMode);
  }, [onFilters, shippingMode]);

  const handleCloseStatus = useCallback(() => {
    onFilters("status", status);
  }, [onFilters, status]);

  const handleFilterStartDate = useCallback(
    (newValue: Date | null) => {
      onFilters('startDate', newValue);
    },
    [onFilters]
  );

  const handleFilterEndDate = useCallback(
    (newValue: Date | null) => {
      onFilters('endDate', newValue);
    },
    [onFilters]
  );

  const [anchorEl, setAnchorEl] = useState<HTMLElement | null>(null);

  const handleClick = (event: React.MouseEvent<HTMLButtonElement>) => {
    setAnchorEl(event.currentTarget);
  };

  const handleClose = () => {
    setAnchorEl(null);
  };

  return (
    <>
      <Box>
        <IconButton
          size="small"
          edge="start"
          color="inherit"
          aria-label="menu"
          aria-controls="filter-menu"
          aria-haspopup="true"
          onClick={handleClick}
          sx={{
            borderRadius: 1, // Pour rendre le bouton carré
            '&:hover': {
              borderRadius: '10px', // Rayon de bordure lors du survol
              backgroundColor: 'rgba(0, 0, 0, 0.04)', // Couleur de survol
            },
          }}
        >
          <GridFilterAltIcon />
          <Typography fontWeight="bold" sx={{ fontSize: "12.5px" }}>Filters</Typography>
        </IconButton>
        <Menu
          id="filter-menu"
          anchorEl={anchorEl}
          open={Boolean(anchorEl)}
          onClose={handleClose}
          MenuListProps={{ onClick: (event) => event.stopPropagation() }} // Empêche la fermeture du menu lors de la sélection dans un select
          anchorOrigin={{
            vertical: 'bottom', // Pour placer le menu en bas du bouton
            horizontal: 'left',
          }}
          transformOrigin={{
            vertical: 'top',
            horizontal: 'left',
          }}
        >


          <MenuItem sx={{ display: 'flex', justifyContent: 'space-between' }}>
            <Box sx={{ display: 'grid', gridTemplateColumns: '1fr 1fr', gap: 2, width: '100%' }}>

              <FormControl
                sx={{
                  flexShrink: 0,
                  width: { xs: 1, md: 220 },
                }}
              >
                <InputLabel>Mode d'expédition</InputLabel>

                <Select
                  multiple
                  value={shippingMode}
                  onChange={handleChangeShippingMode}
                  input={<OutlinedInput label="Mode d'expédition" />}
                  renderValue={(selected) => selected.map((value) => value).join(", ")}
                  onClose={handleCloseShippingMode}
                  sx={{ textTransform: "capitalize" }}
                >
                  {shippingModeOptions.map((option) => (
                    <MenuItem key={option.value} value={option.value}>
                      <Checkbox
                        disableRipple
                        size="small"
                        checked={shippingMode.includes(option.value)}
                      />
                      {option.label}
                    </MenuItem>
                  ))}
                </Select>
              </FormControl>
              <FormControl
                sx={{
                  flexShrink: 0,
                  width: { xs: 1, md: 220 },
                }}
              >
                <InputLabel>Status</InputLabel>

                <Select
                  multiple
                  value={status}
                  onChange={handleChangeStatus}
                  input={<OutlinedInput label="Status" />}
                  renderValue={(selected) => selected.map((value) => value).join(", ")}
                  onClose={handleCloseStatus}
                  sx={{ textTransform: "capitalize" }}
                >
                  {statusOptions.map((option) => (
                    <MenuItem key={option.value} value={option.value}>
                      <Checkbox
                        disableRipple
                        size="small"
                        checked={status.includes(option.value)}
                      />
                      {option.label}
                    </MenuItem>
                  ))}
                </Select>
              </FormControl>
              <DatePicker
                label="Date début"
                value={filters.startDate}
                onChange={handleFilterStartDate}
                slotProps={{
                  textField: {
                    fullWidth: true,
                  },
                }}
                sx={{
                  maxWidth: { md: 220 },
                }}
              />
              <DatePicker
                label="Date fin"
                value={filters.endDate}
                onChange={handleFilterEndDate}
                slotProps={{
                  textField: {
                    fullWidth: true,
                    error: dateError,
                    helperText: dateError && 'End date must be later than start date',
                  },
                }}
                sx={{
                  maxWidth: { md: 220 },
                  [`& .${formHelperTextClasses.root}`]: {
                    position: { md: 'absolute' },
                    bottom: { md: -40 },
                  },
                }}
              />
            </Box>
          </MenuItem>
        </Menu>
      </Box>
      <CustomPopover
        open={popover.open}
        onClose={popover.onClose}
        arrow="right-top"
        sx={{ width: 140 }}
      >
        <MenuItem
          onClick={() => {
            popover.onClose();
          }}
        >
          <Iconify icon="solar:printer-minimalistic-bold" />
          Imprimer
        </MenuItem>

        <MenuItem
          onClick={() => {
            popover.onClose();
          }}
        >
          <Iconify icon="solar:import-bold" />
          Importer
        </MenuItem>

        <MenuItem
          onClick={() => {
            popover.onClose();
          }}
        >
          <Iconify icon="solar:export-bold" />
          Exporter
        </MenuItem>

      </CustomPopover>
    </>
  );
}
