import { useState, useCallback } from "react";
import MenuItem from "@mui/material/MenuItem";
import Checkbox from "@mui/material/Checkbox";
import InputLabel from "@mui/material/InputLabel";
import FormControl from "@mui/material/FormControl";
import OutlinedInput from "@mui/material/OutlinedInput";
import { DatePicker } from '@mui/x-date-pickers/DatePicker';
import Select, { SelectChangeEvent } from "@mui/material/Select";
import { IOrderMondialRelayTableFilters, IOrderMondialRelayTableFilterValue } from "@/contexts/types/Shipping";


type Props = {
  filters: IOrderMondialRelayTableFilters;
  onFilters: (name: string, value: IOrderMondialRelayTableFilterValue) => void;
  //
  shippingModeOptions: {
    value: string;
    label: string;
  }[];
  statusOptions: {
    value: string;
    label: string;
  }[];
};

export default function OrderMondialRelayTableToolbar({
  filters,
  onFilters,
  //
  shippingModeOptions,
  statusOptions,
}: Props) {

  const [shippingMode, setShippingMode] = useState<string[]>(filters.shippingMode);

  const [status, setStatus] = useState<string[]>(filters.status);

  const handleChangeShippingMode = useCallback(
    (event: SelectChangeEvent<string[]>) => {
      const {
        target: { value },
      } = event;
      setShippingMode(typeof value === "string" ? value.split(",") : value);
    },
    []
  );

  const handleChangeStatus = useCallback(
    (event: SelectChangeEvent<string[]>) => {
      const {
        target: { value },
      } = event;
      setStatus(typeof value === "string" ? value.split(",") : value);
    },
    []
  );

  const handleCloseShippingMode = useCallback(() => {
    onFilters("shippingMode", shippingMode);
  }, [onFilters, shippingMode]);

  const handleCloseStatus = useCallback(() => {
    onFilters("status", status);
  }, [onFilters, status]);

  const handleFilterDateCreation = useCallback(
    (newValue: Date | null) => {
      onFilters('dateCreation', newValue);
    },
    [onFilters]
  );


  return (
    <>
      <FormControl
        sx={{
          flexShrink: 0,
          width: { xs: 1, md: 220 },
        }}
      >
        <InputLabel>Mode d'expédition</InputLabel>

        <Select
          multiple
          value={shippingMode}
          onChange={handleChangeShippingMode}
          input={<OutlinedInput label="Mode d'expédition" />}
          renderValue={(selected) => selected.map((value) => value).join(", ")}
          onClose={handleCloseShippingMode}
          sx={{ textTransform: "capitalize" }}
        >
          {shippingModeOptions.map((option) => (
            <MenuItem key={option.value} value={option.value}>
              <Checkbox
                disableRipple
                size="small"
                checked={shippingMode.includes(option.value)}
              />
              {option.label}
            </MenuItem>
          ))}
        </Select>
      </FormControl>
      <FormControl
        sx={{
          flexShrink: 0,
          width: { xs: 1, md: 220 },
        }}
      >
        <InputLabel>Status</InputLabel>

        <Select
          multiple
          value={status}
          onChange={handleChangeStatus}
          input={<OutlinedInput label="Status" />}
          renderValue={(selected) => selected.map((value) => value).join(", ")}
          onClose={handleCloseStatus}
          sx={{ textTransform: "capitalize" }}
        >
          {statusOptions.map((option) => (
            <MenuItem key={option.value} value={option.value}>
              <Checkbox
                disableRipple
                size="small"
                checked={status.includes(option.value)}
              />
              {option.label}
            </MenuItem>
          ))}
        </Select>
      </FormControl>
      <DatePicker
        label="Date création"
        value={filters.dateCreation}
        onChange={handleFilterDateCreation}
        slotProps={{
          textField: {
            fullWidth: true,
          },
        }}
        sx={{
          maxWidth: { md: 220 },
        }}
      />
    </>
  );
}
