import React, { useState } from "react";
import { useForm, FormProvider } from "react-hook-form";
import { yupResolver } from "@hookform/resolvers/yup";
import * as Yup from "yup";
import {
  Grid,
  Card,
  Box,
  Typography,
  Stack,
  Snackbar,
  Alert,
} from "@mui/material";
import LoadingButton from "@mui/lab/LoadingButton";
import { RHFTextField } from "@/shared/components/hook-form";
import { useSnackbar } from "notistack";
import { useAddSupplier } from "@/shared/api/supplier";
import { SaveSupplierRequest } from "@/contexts/types/supplierOrder";

type Props = {
  onSuccess?: () => void;
  isDialog?: boolean;
};

type FormData = {
  companyName: string;
  companyStreet: string;
  companyCity: string;
  companyCountry: string;
  siretNumber?: string | null;
  companyPhoneNumber: string;
  companyEmail: string;
  supplierName: string;
  supplierLastName: string;
  supplierPhoneNumber: string;
  supplierEmail: string;
  supplierZone?: string | null;
};

// Validation Schema avec Yup
const validationSchema = Yup.object({
  companyName: Yup.string().required("Le nom de l'entreprise est requis"),
  companyStreet: Yup.string().required("La rue est requise"),
  companyCity: Yup.string().required("La ville est requise"),
  companyCountry: Yup.string().required("Le pays est requis"),
  siretNumber: Yup.string()
  .test("is-valid-siret", "Le numéro SIRET doit contenir exactement 14 chiffres", function (value) {
    if (value && value.trim() !== "") {
      return /^\d{14}$/.test(value);
    }
    return true;
  })
  .notRequired(),
  companyPhoneNumber: Yup.string().required("Le téléphone est requis"),
  companyEmail: Yup.string()
    .email("Email invalide")
    .required("L'email est requis"),
  supplierName: Yup.string().required("Le nom est requis"),
  supplierLastName: Yup.string().required("Le prénom est requis"),
  supplierPhoneNumber: Yup.string().required("Le téléphone est requis"),
  supplierEmail: Yup.string()
    .email("Email invalide")
    .required("L'email est requis"),
  supplierZone: Yup.string().notRequired(),
});

const FormulaireDivise = ({ onSuccess, isDialog }: Props) => {
  const methods = useForm<FormData>({
    resolver: yupResolver(validationSchema),
    defaultValues: {
      companyName: "",
      companyStreet: "",
      companyCity: "",
      companyCountry: "",
      siretNumber: "",
      companyPhoneNumber: "",
      companyEmail: "",
      supplierName: "",
      supplierLastName: "",
      supplierPhoneNumber: "",
      supplierEmail: "",
      supplierZone: "",
    },
  });

  const {
    reset,
    handleSubmit,
    formState: { isSubmitting },
  } = methods;
  const { enqueueSnackbar } = useSnackbar();

  const [snackbarOpen, setSnackbarOpen] = useState(false);
  const [snackbarMessage, setSnackbarMessage] = useState("");
  const [snackbarSeverity, setSnackbarSeverity] = useState<"success" | "error">(
    "success"
  );

  const onSubmit = handleSubmit(async (data) => {
    try {
      const supplierDto: SaveSupplierRequest = {
        companyName: data.companyName,
        companyEmail: data.companyEmail,
        companyPhoneNumber: data.companyPhoneNumber,
        companyStreet: data.companyStreet,
        companyCity: data.companyCity,
        companyCountry: data.companyCountry,
        siretNumber: Number(data.siretNumber),
        supplierName: data.supplierName,
        supplierLastName: data.supplierLastName,
        supplierPhoneNumber: data.supplierPhoneNumber,
        supplierEmail: data.supplierEmail,
        supplierZone: data?.supplierZone || "",
      };
      await useAddSupplier(supplierDto);
      enqueueSnackbar("Création réussie!", { variant: "success" });
      setSnackbarMessage("Création réussie!");
      setSnackbarSeverity("success");
      setSnackbarOpen(true);
      reset();
      if (isDialog) {
        onSuccess?.();
      }
    } catch (error: any) {
      enqueueSnackbar("Échec de la création!", { variant: "error" });
      setSnackbarMessage(error.message);
      setSnackbarSeverity("error");
      setSnackbarOpen(true);
    }
  });

  const handleSnackbarClose = (
    event?: React.SyntheticEvent | Event,
    reason?: string
  ) => {
    if (reason === "clickaway") {
      return;
    }
    setSnackbarOpen(false);
  };

  return (
    <FormProvider {...methods}>
      <Box component="form" onSubmit={onSubmit}>
        <Grid container spacing={3}>
          <Grid item xs={12} md={6}>
            <Card
              variant="outlined"
              sx={{
                p: 3,
                ...(isDialog && { boxShadow: 0 }),
              }}
            >
              <Typography variant="h6" mb={2}>
                Informations sur l'entreprise
              </Typography>
              <Stack spacing={2}>
                <RHFTextField name="companyName" label="Nom de l'entreprise" />
                <RHFTextField name="companyStreet" label="Rue" />
                <RHFTextField name="companyCity" label="Ville" />
                <RHFTextField name="companyCountry" label="Pays" />
                <RHFTextField name="siretNumber" label="Numéro de SIRET" />
                <RHFTextField name="companyPhoneNumber" label="Téléphone" />
                <RHFTextField name="companyEmail" label="Email" />
              </Stack>
            </Card>
          </Grid>

          <Grid item xs={12} md={6}>
            <Card
              variant="outlined"
              sx={{
                p: 3,
                ...(isDialog && { boxShadow: 0 }),
              }}
            >
              <Typography variant="h6" mb={2}>
                Informations sur le fournisseur
              </Typography>
              <Stack spacing={2}>
                <RHFTextField name="supplierName" label="Nom" />
                <RHFTextField name="supplierLastName" label="Prénom" />
                <RHFTextField name="supplierPhoneNumber" label="Téléphone" />
                <RHFTextField name="supplierEmail" label="Email" />
                <RHFTextField name="supplierZone" label="Zone" />
              </Stack>
            </Card>
          </Grid>
        </Grid>

        <Box sx={{ mt: 3, ...(isDialog && { mb: 3 }) }}>
          <LoadingButton
            type="submit"
            variant="contained"
            size="large"
            fullWidth
            loading={isSubmitting}
          >
            Soumettre
          </LoadingButton>
        </Box>
      </Box>
      <Snackbar
        open={snackbarOpen}
        autoHideDuration={6000}
        onClose={handleSnackbarClose}
        anchorOrigin={{ vertical: "top", horizontal: "right" }}
      >
        <Alert
          onClose={handleSnackbarClose}
          severity={snackbarSeverity}
          sx={{ width: "100%" }}
        >
          {snackbarMessage}
        </Alert>
      </Snackbar>
    </FormProvider>
  );
};

export default FormulaireDivise;
