"use client";
import * as Yup from "yup";
import { useState, useEffect, ChangeEvent } from "react";
import Box from "@mui/material/Box";
import Card from "@mui/material/Card";
import Stack from "@mui/material/Stack";
import Grid from "@mui/material/Unstable_Grid2";
import LoadingButton from "@mui/lab/LoadingButton";
import {
  TextField,
  Autocomplete,
  Typography,
  Snackbar,
  Alert,
  Container,
} from "@mui/material";
import { countries } from "@/shared/assets/data";
import { useSnackbar } from "@/components/snackbar";
import { useGetSupplierById, useUpdateSupplier } from "@/shared/api/supplier";
import CustomBreadcrumbs from "@/components/custom-breadcrumbs";
import { useSettingsContext } from "@/components/settings";
import { paths } from "@/routes/paths";
import { useRouter } from "next/navigation";

type SupplierFormValues = {
  supplierName: string;
  supplierLastName: string;
  supplierEmail: string;
  supplierPhoneNumber: string;
  companyName: string;
  companyStreet: string;
  companyCity: string;
  companyCountry: string;
  siretNumber: string;
  companyPhoneNumber: string;
  companyEmail: string;
};

type Props = {
  id?: string;
};

const validationSchema = Yup.object().shape({
  supplierName: Yup.string().required("Nom requis"),
  supplierLastName: Yup.string().required("Prénom requis"),
  supplierEmail: Yup.string()
    .required("Email requis")
    .email("L'email doit être une adresse email valide"),
  supplierPhoneNumber: Yup.string().required("Numéro de téléphone requis"),
  companyCity: Yup.string().required("Ville requise"),
  companyCountry: Yup.string().required("Pays requis"),
  companyStreet: Yup.string().required("Adresse requise"),
  companyName: Yup.string().required("Nom de l'entreprise requis"),
  siretNumber: Yup.string()
    .matches(
      /^\d{14}$/,
      "Le numéro de SIRET doit contenir exactement 14 chiffres"
    )
    .required("Le numéro de SIRET est requis"),
  companyEmail: Yup.string()
    .required("Email de l'entreprise requis")
    .email("L'email doit être une adresse email valide"),
  companyPhoneNumber: Yup.string().required("Téléphone de l'entreprise requis"),
});

export default function SupplierNewEditForm({ id }: Props) {
  const { enqueueSnackbar } = useSnackbar();
  const supplierId = Array.isArray(id) ? id[0] : id;

  const { supplier } = useGetSupplierById(supplierId);

  const [formValues, setFormValues] = useState<SupplierFormValues>({
    supplierName: supplier?.supplierName || "",
    supplierLastName: supplier?.supplierLastName || "",
    supplierEmail: supplier?.supplierEmail || "",
    supplierPhoneNumber: supplier?.supplierPhoneNumber || "",
    companyName: supplier?.companyName || "",
    companyStreet: supplier?.companyStreet || "",
    companyCity: supplier?.companyCity || "",
    companyCountry: supplier?.companyCountry || "",
    siretNumber: String(supplier?.siretNumber) || "",
    companyPhoneNumber: supplier?.companyPhoneNumber || "",
    companyEmail: supplier?.companyEmail || "",
  });

  const [snackbarOpen, setSnackbarOpen] = useState(false);
  const [snackbarMessage, setSnackbarMessage] = useState("");
  const [snackbarSeverity, setSnackbarSeverity] = useState<"success" | "error">(
    "success"
  );

  useEffect(() => {
    if (supplier) {
      setFormValues({
        supplierName: supplier.supplierName,
        supplierLastName: supplier.supplierLastName,
        supplierEmail: supplier.supplierEmail,
        supplierPhoneNumber: supplier.supplierPhoneNumber,
        companyName: supplier.companyName,
        companyStreet: supplier.companyStreet,
        companyCity: supplier.companyCity,
        companyCountry: supplier.companyCountry,
        siretNumber: String(supplier.siretNumber),
        companyPhoneNumber: supplier.companyPhoneNumber,
        companyEmail: supplier.companyEmail,
      });
    }
  }, [supplier]);

  const handleTextChange = (
    e: React.ChangeEvent<HTMLInputElement | HTMLTextAreaElement>
  ) => {
    const { name, value } = e.target;
    setFormValues((prevValues) => ({
      ...prevValues,
      [name]: value,
    }));
  };

  const handleAutocompleteChange = (
    event: ChangeEvent<unknown>,
    value: string | null
  ) => {
    setFormValues((prevValues) => ({
      ...prevValues,
      companyCountry: value || "",
    }));
  };
  const router = useRouter(); 
  const handleSubmit = async (e: React.FormEvent) => {
    e.preventDefault();
    try {
      const validatedData = await validationSchema.validate(formValues, {
        abortEarly: false,
      });
      const supplierDto = {
        ...validatedData,
        siretNumber: Number(validatedData.siretNumber),
        id: supplierId,
      };
      const updatedSupplier = await useUpdateSupplier(supplierDto);
      if (updatedSupplier.status===200) {
        setSnackbarMessage("Modifications enregistrées avec succès !");
        setSnackbarSeverity("success");
        setSnackbarOpen(true); 
        router.push(paths.dashboard.admin.supplier.root);
      } else {
        throw new Error("Erreur lors de la mise à jour du fournisseur");
      }
    } catch (error) {
      setSnackbarMessage("Une erreur s'est produite");
      setSnackbarSeverity("error");
      setSnackbarOpen(true); 
    }
  };
    const settings = useSettingsContext();

  return (
    <Container maxWidth={settings.themeStretch ? false : "lg"}>
     <CustomBreadcrumbs
            heading="Modifier un fournisseur"
            links={[
              {
                name: "Dashboard",
                href: paths.dashboard.root,
              },
              {
                name: "Fournisseur",
                href: paths.dashboard.admin.supplier.root,
              },
            ]}
            sx={{
              mb: { xs: 3, md: 5 },
            }}
          />
    <form onSubmit={handleSubmit}>
      <Grid container spacing={3}>
        <Grid xs={12} md={6}>
          <Card variant="outlined" sx={{ p: 3 }}>
            <Typography variant="h6" mb={2}>
              Informations sur l'entreprise
            </Typography>
            <Stack spacing={2}>
              <TextField
                name="companyName"
                label="Nom de l'entreprise"
                value={formValues.companyName}
                onChange={handleTextChange}
                fullWidth
              />
              <TextField
                name="companyStreet"
                label="Rue"
                value={formValues.companyStreet}
                onChange={handleTextChange}
                fullWidth
              />
              <TextField
                name="companyCity"
                label="Ville"
                value={formValues.companyCity}
                onChange={handleTextChange}
                fullWidth
              />
              <Autocomplete
                value={formValues.companyCountry}
                onChange={handleAutocompleteChange}
                options={countries.map((country) => country.label)}
                renderInput={(params) => <TextField {...params} label="Pays" />}
                fullWidth
              />

              <TextField
                name="siretNumber"
                label="Numéro de SIRET"
                value={formValues.siretNumber}
                onChange={handleTextChange}
                disabled // Disable the SIRET number input
                fullWidth
              />
              <TextField
                name="companyPhoneNumber"
                label="Téléphone"
                value={formValues.companyPhoneNumber}
                onChange={handleTextChange}
                fullWidth
              />
              <TextField
                name="companyEmail"
                label="Email"
                value={formValues.companyEmail}
                onChange={handleTextChange}
                disabled // Disable the company email input
                fullWidth
              />
            </Stack>
          </Card>
        </Grid>

        <Grid xs={12} md={6}>
          <Card variant="outlined" sx={{ p: 3 }}>
            <Typography variant="h6" mb={2}>
              Informations sur le fournisseur
            </Typography>
            <Stack spacing={2}>
              <TextField
                name="supplierName"
                label="Nom"
                value={formValues.supplierName}
                onChange={handleTextChange}
                fullWidth
              />
              <TextField
                name="supplierLastName"
                label="Prénom"
                value={formValues.supplierLastName}
                onChange={handleTextChange}
                fullWidth
              />
              <TextField
                name="supplierPhoneNumber"
                label="Téléphone"
                value={formValues.supplierPhoneNumber}
                onChange={handleTextChange}
                fullWidth
              />
              <TextField
                name="supplierEmail"
                label="Email"
                value={formValues.supplierEmail}
                onChange={handleTextChange}
                disabled // Disable the supplier email input
                fullWidth
              />
            </Stack>
          </Card>
        </Grid>
      </Grid>

      <LoadingButton
        type="submit"
        variant="contained"
        fullWidth // Added fullWidth to make the button take full width
        sx={{ mt: 3 }}
      >
        Enregistrer les modifications
      </LoadingButton>

      <Snackbar
        open={snackbarOpen}
        autoHideDuration={3000}
        onClose={() => setSnackbarOpen(false)}
        anchorOrigin={{ vertical: 'top', horizontal: 'right' }} 
      >
        <Alert
          onClose={() => setSnackbarOpen(false)}
          severity={snackbarSeverity}
        >
          {snackbarMessage}
        </Alert>
      </Snackbar>
    </form>
    </Container>
  );
}
