"use client";

import isEqual from "lodash/isEqual";
import { useState, useCallback, useEffect } from "react";
import Card from "@mui/material/Card";
import Table from "@mui/material/Table";
import Button from "@mui/material/Button";
import Tooltip from "@mui/material/Tooltip";
import Container from "@mui/material/Container";
import TableBody from "@mui/material/TableBody";
import IconButton from "@mui/material/IconButton";
import TableContainer from "@mui/material/TableContainer";
import { paths } from "@/routes/paths";
import RouterLink from "@/routes/router-link";
import { useBoolean } from "@/hooks/use-boolean";
import Iconify from "@/components/iconify";
import Scrollbar from "@/shared/components/scrollbar";
import { useSnackbar } from "@/shared/components/snackbar";
import { ConfirmDialog } from "@/shared/components/custom-dialog";
import { useSettingsContext } from "@/shared/components/settings";
import CustomBreadcrumbs from "../../../../components/custom-breadcrumbs";
import {
  useTable,
  emptyRows,
  TableNoData,
  getComparator,
  TableEmptyRows,
  TableHeadCustom,
  TableSelectedAction,
  TablePaginationCustom,
} from "@/shared/components/table";
import SupplierTableRow from "./supplier-table-row";
import SupplierTableFiltersResult from "./supplier-table-filters-result";
import CustomModal from "./custom-modal";
import {
  ISupplierItemTableFilters,
  ISupplierItemTableFilterValue,
  SupplierDTO,
} from "@/contexts/types/supplierOrder";
import SupplierTableToolbar from "./SupplierTableToolbar";
import {
  archiveSupplier,
  useGetArchivedSuppliers,
} from "@/shared/api/supplier";
import { useRouter } from "next/navigation";
import { Alert, Snackbar } from "@mui/material";

// ----------------------------------------------------------------------

const TABLE_HEAD = [
  { id: "supplierLastName", label: "Nom", width: 180 },
  { id: "supplierEmail", label: "Email", width: 180 },
  { id: "supplierPhoneNumber", label: "Numéro de téléphone", width: 180 },
  { id: "address", label: "Adresse", width: 200 },
  { id: "", label: "Action", width: 88 },
];

const defaultFilters: ISupplierItemTableFilters = {
  name: "",
};

// ----------------------------------------------------------------------

export default function SupplierArchivedListView() {
  const { enqueueSnackbar } = useSnackbar();

  const table = useTable();

  const settings = useSettingsContext();

  const router = useRouter();

  const confirm = useBoolean();

  const [tableData, setTableData] = useState<SupplierDTO[]>([]);

  const [filters, setFilters] = useState(defaultFilters);

  const [modalOpen, setModalOpen] = useState(false);
  const [snackbarOpen, setSnackbarOpen] = useState(false);
  const [snackbarMessage, setSnackbarMessage] = useState("");
  const [snackbarSeverity, setSnackbarSeverity] = useState<"success" | "error">(
    "success"
  );

  const { suppliersData, suppliersLoading, suppliersError, mutate } =
    useGetArchivedSuppliers();

  useEffect(() => {
    if (suppliersData) {
      setTableData(suppliersData);
    }
  }, [suppliersData]);

  const dataFiltered = applyFilter({
    inputData: tableData,
    comparator: getComparator(table.order, table.orderBy),
    filters,
  });
  const handleUnarchiveRow = useCallback(
    async (id: number) => {
      try {
        const response = await archiveSupplier(id, false); // Désarchiver
        if (response.status === 200) {
          setSnackbarMessage("Fournisseur désarchivé avec succès");
          setSnackbarSeverity("success");
          setSnackbarOpen(true);
          mutate(); 
        } else {
          setSnackbarMessage("Échec de la désarchivation du fournisseur");
          setSnackbarSeverity("error");
          setSnackbarOpen(true);
        }
      } catch (error) {
        setSnackbarMessage("Échec de la désarchivation du fournisseur");
        setSnackbarSeverity("error");
        setSnackbarOpen(true);
        console.error(error);
      }
    },
    [enqueueSnackbar, mutate]
  );

  const dataInPage = dataFiltered.slice(
    table.page * table.rowsPerPage,
    table.page * table.rowsPerPage + table.rowsPerPage
  );

  const denseHeight = table.dense ? 56 : 76;

  const canReset = !isEqual(defaultFilters, filters);

  const notFound = !dataFiltered.length;

  const handleFilters = useCallback(
    (name: string, value: ISupplierItemTableFilterValue) => {
      table.onResetPage();
      setFilters((prevState) => ({
        ...prevState,
        [name]: value,
      }));
    },
    [table]
  );

  const handleResetFilters = useCallback(() => {
    setFilters(defaultFilters);
  }, []);

  const handleDeleteRow = useCallback(
    (id: string) => {
      const updatedData = tableData?.filter((row) => row.id !== id);

      enqueueSnackbar("Suppression réussie !");
      setTableData(updatedData);

      table.onUpdatePageDeleteRow(dataInPage.length);
    },
    [tableData, dataInPage.length, enqueueSnackbar, table]
  );

  const handleDeleteRows = useCallback(() => {
    const updatedData = tableData?.filter(
      (row) => !table.selected.includes(String(row.id))
    );

    enqueueSnackbar("Suppression réussie !");
    setTableData(updatedData);

    table.onUpdatePageDeleteRows({
      totalRowsInPage: dataInPage.length,
      totalRowsFiltered: dataFiltered.length,
    });
  }, [
    dataFiltered.length,
    dataInPage.length,
    table,
    tableData,
    enqueueSnackbar,
  ]);

  const handleEditRow = useCallback(
    (id: string) => {
      router.push(paths.dashboard.admin.supplier.edit(id));
    },
    [router]
  );

  return (
    <>
      <Container maxWidth={settings.themeStretch ? false : "lg"}>
        <CustomBreadcrumbs
          heading="Liste des Fournisseurs archivés"
          links={[
            { name: "Tableau de bord", href: paths.dashboard.root },
            { name: "Fournisseur", href: paths.dashboard.admin.supplier.list },
            { name: "Lister" },
          ]}
          rollbackLink={paths.dashboard.admin.supplier.list} 
 
          sx={{ mb: { xs: 3, md: 5 } }}
        />

        <Card>
          <SupplierTableToolbar filters={filters} onFilters={handleFilters} />

          {canReset && (
            <SupplierTableFiltersResult
              filters={filters}
              onFilters={handleFilters}
              onResetFilters={handleResetFilters}
              results={dataFiltered.length}
              sx={{ p: 2.5, pt: 0 }}
            />
          )}

          <TableContainer sx={{ position: "relative", overflow: "unset" }}>
            <Scrollbar>
              <Table
                size={table.dense ? "small" : "medium"}
                sx={{ minWidth: 960 }}
              >
                <TableHeadCustom
                  order={table.order}
                  orderBy={table.orderBy}
                  headLabel={TABLE_HEAD}
                  rowCount={dataFiltered.length}
                  numSelected={table.selected.length}
                  onSort={table.onSort}
                />

                <TableBody>
                  {dataInPage.map((row) => (
                    <SupplierTableRow
                      key={row.id}
                      row={row}
                      selected={table.selected.includes(String(row.id))}
                      onSelectRow={() => table.onSelectRow(String(row.id))}
                      onDeleteRow={() => handleDeleteRow(row.id)}
                      onEditRow={() => handleEditRow(String(row.id))}
                      onUnarchiveRow={() => handleUnarchiveRow(Number(row.id))}
                    />
                  ))}

                  <TableEmptyRows
                    height={denseHeight}
                    emptyRows={emptyRows(
                      table.page,
                      table.rowsPerPage,
                      dataFiltered.length
                    )}
                  />

                  <TableNoData notFound={notFound} />
                </TableBody>
              </Table>
            </Scrollbar>
          </TableContainer>

          <TablePaginationCustom
            count={dataFiltered.length}
            page={table.page}
            rowsPerPage={table.rowsPerPage}
            onPageChange={table.onChangePage}
            onRowsPerPageChange={table.onChangeRowsPerPage}
            dense={table.dense}
            onChangeDense={table.onChangeDense}
          />
        </Card>
      </Container>

      <CustomModal
        open={modalOpen}
        onClose={() => setModalOpen(false)}
        message="Suppression réussie !"
      />

      <ConfirmDialog
        open={confirm.value}
        onClose={confirm.onFalse}
        title="Suppression"
        content={`Êtes-vous sûr de vouloir supprimer ${table.selected.length} élément(s) ?`}
        action={
          <Button
            variant="contained"
            color="error"
            onClick={() => {
              handleDeleteRows();
              confirm.onFalse();
            }}
          >
            Supprimer
          </Button>
        }
      />
      <Snackbar
        open={snackbarOpen}
        autoHideDuration={3000}
        onClose={() => setSnackbarOpen(false)}
        anchorOrigin={{ vertical: "top", horizontal: "right" }}
      >
        <Alert
          onClose={() => setSnackbarOpen(false)}
          severity={snackbarSeverity}
        >
          {snackbarMessage}
        </Alert>
      </Snackbar>
    </>
  );
}

// ----------------------------------------------------------------------

function applyFilter({
  inputData,
  comparator,
  filters,
}: {
  inputData: SupplierDTO[];
  comparator: (a: SupplierDTO, b: SupplierDTO) => number;
  filters: ISupplierItemTableFilters;
}) {
  const { name } = filters;

  let filteredData = inputData;

  if (name) {
    filteredData = filteredData.filter((supplier) =>
      supplier.supplierName.toLowerCase().includes(name.toLowerCase())
    );
  }

  return filteredData.sort(comparator);
}
