"use client";

import Card from "@mui/material/Card";
import Container from "@mui/material/Container";
import TableContainer from "@mui/material/TableContainer";
import Scrollbar from "@/components/scrollbar";
import Table from "@mui/material/Table";
import {
  useTable,
  emptyRows,
  TableNoData,
  getComparator,
  TableEmptyRows,
  TableHeadCustom,
  TableSelectedAction,
  TablePaginationCustom,
} from "@/shared/components/table";


import { useSettingsContext } from "@/shared/components/settings";
import { paths } from "@/routes/paths";

import Label from "@/shared/components/label";
import CustomBreadcrumbs from "@/shared/components/custom-breadcrumbs";
import { useParams } from "next/navigation";
import { useEffect, useState } from "react";
import { Avatar, Box, Divider, Grid, ListItemText, Stack, TableBody, TableCell, TableRow, Tooltip, Typography } from "@mui/material";
import notFound from "@/app/not-found";
import { _invoices } from "@/shared/_mock";
import { IInvoice, IInvoiceItem } from "@/shared/types/invoice";
import { fDate, fTime } from "@/utils/format-time";
import { useResponsive } from "@/hooks/use-responsive";
import { fCurrency } from "@/utils/format-number";
import CampaignIcon from '@mui/icons-material/Campaign';
import { Circle } from "@mui/icons-material";
import SaleOrderGapModal from "./sale-order-gap-modal";


// -------------------------------------------------------------------------------------------------

const TABLE_HEAD = [
  { id: "service", label: "Produit", alignRight: false },
  { id: "quantity", label: "Quantité", alignRight: false },
  { id: "price", label: "Prix", alignRight: false },
  { id: "total", label: "Total", alignRight: false },
  { id: "sizes", label: "Taille", alignRight: false },
  { id: "colors", label: "Couleur", alignRight: false },
  { id: "statut", label: "Statut de réception", alignRight: false },
];


const ITEMS_STATUS = [
  "Le produit a été récupéré",

  "Le produit sera récupéré plus tard",

  "Le produit ne sera jamais récupéré",

  "Le produit a été reçue de manière incomplète",
]


export default function SaleOrderDetails() {

  const settings = useSettingsContext();
  const params = useParams()
  const [commandData, setCommandData] = useState<IInvoice | null>(null);
  const [gapDeclaration, setGapDeclaration] = useState(false);
  const [declarationData, setDeclarationData] = useState({variation: "", id: "", type: "", productName: ""});
  const lgUp = useResponsive("up", "lg");
  const mdUp = useResponsive("up", "md");

  const getCommandDetails = (id: string) => {
    const command = _invoices.find(invoice => invoice.id === id);
    return command || null;
  };

  const toggleGapDeclaration = (type:string,variation: string,item: IInvoiceItem) => {
    setGapDeclaration(!gapDeclaration);
    setDeclarationData({
      variation: variation,
      id : item.id.toString(),
      productName: item.productName,
      type: type
    })

  }

  useEffect(() => {
    if (params.id) {
      const data = getCommandDetails(params.id as string);
      setCommandData(data);
    }
  }, [params.id]);


  return (
    <Container maxWidth={settings.themeStretch ? false : "lg"}>
      <CustomBreadcrumbs
        heading={`Détail Commande: ${commandData?.invoiceNumber}`}
        links={[
          { name: "Tableau de bord", href: paths.dashboard.root },
          { name: "Commandes fournisseurs", href: paths.dashboard.admin.supplierOrder.root },
          { name: "Détail Commande" },
        ]}
        sx={{
          mb: { xs: 3, md: 5 },
        }}
      />

      <Card className="p-3" >
        <Grid component="div" spacing={{ xs: 4 }} container p={0} py={2} justifyContent={'space-evenly'}>
          <Grid item lg={2} md={5} sm={12} container justifyContent={mdUp ? 'center' : 'start'} alignItems='center'>
            <p>
              <Typography
                variant="h6"
                sx={{ color: "text.disabled", flexGrow: 1 }}
              >
                Date de création
              </Typography>
              <span>{fDate(commandData?.createDate)}</span> - <span>{fTime(commandData?.createDate)}</span>
            </p>
          </Grid>

          <Divider orientation={mdUp ? 'vertical' : 'horizontal'} flexItem style={{ borderStyle: 'dashed', margin: "16px 0" }} />

          <Grid item lg={2} md={5} sm={12} container justifyContent={mdUp ? 'center' : 'start'} alignItems='center'>
            <p>
              <Typography
                variant="h6"
                sx={{ color: "text.disabled", flexGrow: 1 }}
              >
                Date de livraison
              </Typography>
              <span>{fDate(commandData?.dueDate)}</span> - <span>{fTime(commandData?.dueDate)}</span>
            </p>
          </Grid>

          <Divider orientation={mdUp ? 'vertical' : 'horizontal'} hidden={mdUp && (!lgUp)} flexItem style={{ borderStyle: 'dashed', margin: "16px 0" }} />

          <Grid item lg={2} md={3} sm={12} container justifyContent={mdUp ? 'center' : 'start'} alignItems='center'>
            <p>
              <Typography
                variant="h6"
                sx={{ color: "text.disabled", flexGrow: 1 }}
              >
                Montant
              </Typography>
              <span>{fCurrency(commandData?.totalAmount || 0)}</span>
            </p>
          </Grid>
          <Divider orientation={mdUp ? 'vertical' : 'horizontal'} flexItem style={{ borderStyle: 'dashed', margin: "16px 0" }} />

          <Grid item lg={2} md={3} sm={12} container justifyContent={mdUp ? 'center' : 'start'} alignItems='center'>
            <p>
              <Typography
                variant="h6"
                sx={{ color: "text.disabled", flexGrow: 1 }}
              >
                Statut
              </Typography>
              <Label
                variant="soft"
                color={
                  (commandData?.status === "Terminée" && "success") ||
                  (commandData?.status === "En cours" && "warning") ||
                  (commandData?.status === "A faire" && "info") ||
                  "default"
                }
              >
                {commandData?.status}
              </Label>
            </p>

          </Grid>

          <Divider orientation={mdUp ? 'vertical' : 'horizontal'} flexItem style={{ borderStyle: 'dashed', margin: "16px 0" }} />

          <Grid item lg={2} md={3} sm={12} container justifyContent={mdUp ? 'center' : 'start'} alignItems='center'>
            <p>
              <Typography
                variant="h6"
                sx={{ color: "text.disabled", flexGrow: 1 }}
              >
                Commandes effectuées
              </Typography>
              <span>{commandData?.sent}</span>
            </p>
          </Grid>

          <Divider orientation="horizontal" flexItem style={{ borderStyle: 'dashed', margin: "16px 0" }} />

          <Grid item md={5} sm={12} container justifyContent={mdUp ? 'center' : 'start'} alignItems='center'>
            <p>
              <Typography
                variant="h6"
                sx={{ color: "text.disabled", flexGrow: 1 }}
              >
                Fournisseur
              </Typography>
              <div className="flex items-center">
                <Avatar alt={commandData?.invoiceTo.name} sx={{ mr: 1, width: 25, height: 25 }}>
                  {commandData?.invoiceTo.name.charAt(0).toUpperCase()}
                </Avatar>
                <ListItemText
                  disableTypography
                  primary={
                    <Typography variant="body2" noWrap>
                      {commandData?.invoiceTo.name}
                    </Typography>
                  }
                />
              </div>
            </p>
          </Grid>

          <Divider orientation={mdUp ? 'vertical' : 'horizontal'} flexItem style={{ borderStyle: 'dashed', margin: "16px 0" }} />

          <Grid item md={5} sm={12} container justifyContent={mdUp ? 'center' : 'start'} alignItems='center'>
            <p>
              <Typography
                variant="h6"
                sx={{ color: "text.disabled", flexGrow: 1 }}
              >
                Chauffeur
              </Typography>
              <div className="flex items-center">
                <Avatar alt={commandData?.driver} sx={{ mr: 1, width: 25, height: 25 }}>
                  {commandData?.driver.charAt(0).toUpperCase()}
                </Avatar>
                <ListItemText
                  disableTypography
                  primary={
                    <Typography variant="body2" noWrap>
                      {commandData?.driver}
                    </Typography>
                  }
                />
              </div>
            </p>
          </Grid>
        </Grid>

      </Card>




      <Card sx={{ mt: 2 }} >
        <TableContainer sx={{ position: "relative", overflow: "unset" }}>
          <Scrollbar>
            <Table sx={{ minWidth: 800 }}>
              <TableHeadCustom
                headLabel={TABLE_HEAD}

              />

              <TableBody>
                {
                  commandData?.items.map((item: IInvoiceItem, index: number) => (

                    <TableRow
                      key={item.id}
                    >
                      <TableCell>
                        {item.service}
                      </TableCell>
                      <TableCell>
                        {item.quantity}
                      </TableCell>
                      <TableCell>
                        {fCurrency(item.price)}
                      </TableCell>
                      <TableCell>
                        {fCurrency(item.total)}
                      </TableCell>
                      <TableCell>
                        {item.sizes.map((size, index) => (
                          <div className="flex" key={index} onClick={() => toggleGapDeclaration('size',size.value,item)}>
                            <span className="min-w-[18px] mr-2">{size.value}</span>
                            <Tooltip title="Déclarer un écart">
                              <CampaignIcon sx={{ color: 'error.main', cursor: 'pointer' }} />
                            </Tooltip>
                          </div>
                        ))}
                      </TableCell>
                      <TableCell>
                        {item.colors.map((color, index) => (

                          <div key={index} className="flex" onClick={() => toggleGapDeclaration('color',color.value,item)}>
                            <Circle sx={{ color: color.value, mr: '16px' }} />
                            <Tooltip title="Déclarer un écart">
                              <CampaignIcon sx={{ color: 'error.main', cursor: 'pointer' }} />
                            </Tooltip>
                          </div>

                        ))}
                      </TableCell>
                      <TableCell>
                        {
                          commandData?.status === "Terminée" ? (
                            <Label
                              variant="soft"
                              color={
                                (ITEMS_STATUS[index%4] === "Le produit a été récupéré" && "success") ||
                                (ITEMS_STATUS[index%4] === "Le produit sera récupéré plus tard" && "info") ||
                                (ITEMS_STATUS[index%4] === "Le produit ne sera jamais récupéré" && "error") ||
                                (ITEMS_STATUS[index%4] === "Le produit a été reçue de manière incomplète" && "warning") ||
                                "default"
                              }
                            >
                              {ITEMS_STATUS[index%4]}
                            </Label>
                          ) : (
                            <Label
                              variant="soft"
                              color={
                                (commandData?.status === "En cours" && "warning") ||
                                (commandData?.status === "A faire" && "info") ||
                                "default"
                              }
                            >
                              {commandData?.status}
                            </Label>
                          )
                        }
                      </TableCell>
                    </TableRow>
                  ))
                }
              </TableBody>
            </Table>
          </Scrollbar>
        </TableContainer>
      </Card>

      {
        gapDeclaration && (
          <SaleOrderGapModal 
            open={gapDeclaration} 
            onClose= {() => setGapDeclaration(false)}
            idCommand={commandData?.invoiceNumber || ""} 
            idProduct={declarationData.id} 
            productName={declarationData.productName}
            variation={declarationData.variation}
            type={declarationData.type}
            />
        )
      }
    </Container>

  )
}

