'use client'

import { Button, Card, Container, Grid, Stack, Tab, Table, TableBody, TableContainer, Tabs, Tooltip, CircularProgress, Dialog, DialogActions, DialogContent, DialogContentText, DialogTitle, Alert, TableRow, TableCell } from "@mui/material";
import { useSettingsContext } from "@/components/settings";
import CustomBreadcrumbs from "@/components/custom-breadcrumbs";
import { paths } from "@/routes/paths";
import { useCallback, useEffect, useState } from "react";
import { _invoices, INVOICE_SERVICE_OPTIONS } from "@/shared/_mock";
import { useTable } from "@/shared/components/table";
import { alpha, useTheme } from "@mui/material/styles";
import Label from "@/shared/components/label";
import { useSnackbar } from "@/components/snackbar";
import IconButton from "@mui/material/IconButton";
import { emptyRows, TableNoData, TableEmptyRows, TableHeadCustom, TableSelectedAction, TablePaginationCustom } from "@/shared/components/table";
import { isAfter } from "@/utils/format-time";
import Iconify from "@/components/iconify";
import Scrollbar from "@/components/scrollbar";
import { json2csv } from "json-2-csv";
import { useBoolean } from "@/hooks";
import { isSameDay } from "date-fns";
import { useGetAllSuppliers, useGetNotArchivedSuppliers } from "@/shared/api/supplier";
import { getAllSuppliers } from "@/shared/api/supplierOrders";
import { Driver } from "@/contexts/types/driver";
import { filter, isEqual } from "lodash";
import { getAllSaleOrders, useSearchSaleOrders } from "@/shared/api/saleOrders";
import { SaleOrder, SaleOrderPage } from "@/contexts/types/saleOrder";
import { ISaleOrderTableFilters, ISaleOrderTableFilterValue } from "@/shared/types/saleOrder";
import { useGetUserById, UserByEmail } from "@/shared/api/user";
import { IUser } from "@/shared/types/user";
import { deleteSaleOrders, generateSaleOrder } from "@/shared/api/saleOrders";
import SaleOrderTableToolbar from "./sale-order-table-toolbar";
import SaleOrderTableFiltersResult from "./sale-order-table-filters-result";
import SaleOrderTableRow from "./sale-order-table-row";
import { getAllDrivers } from "@/shared/api/supplierOrders";
import { SupplierDTO } from "@/contexts/types/supplierOrder";
import { LoadingScreen } from "@/shared/components/loading-screen";

const TABLE_HEAD = [
  { id: "" },
  { id: "id", label: "N°" },
  { id: "supplierName", label: "Fournisseur", },
  { id: "createdAt", label: "Date de Création" },
  { id: "totalPrice", label: "Montant" },
  { id: "saleOrderItems", label: "Nombre de produits", align: "center" },
  { id: "action", label: "Action", align: "center" },
  { id: "" },
];

const defaultFilters: ISaleOrderTableFilters = {
  name: "",
  service: [],
  status: "all",
  creationDate: null,
};

type DriverItem = {
  id: string;
  name: string;
};

export default function SaleOrderListView() {
  const table = useTable();
  const { enqueueSnackbar } = useSnackbar();
  const [tableData, setTableData] = useState<SaleOrderPage[]>([])
  const [loadingSaleOrder, setLoadingSaleOrder] = useState(false);
  const [openConfirmDialog, setOpenConfirmDialog] = useState(false);
  const [driversList, setDriversList] = useState<DriverItem[]>([]);
  const [suppliersList, setSuppliersList] = useState<SupplierDTO[]>([]);
  const { suppliersData } = useGetNotArchivedSuppliers();
  const { driversData } = getAllDrivers();
  const [userId, setUserId] = useState<string>("");



  const [filters, setFilters] = useState(defaultFilters);
  const {
    saleOrders,
    totalSaleOrders,
    totalPages,
    pageNumber,
    pageSize,
    loading,
    revalidate,
  } = useSearchSaleOrders({
    name: filters.name,
    creationDate: filters.creationDate,
    page: table.page,
    size: table.rowsPerPage,
    sortBy: table.orderBy,
    sortDir: table.order,
  });
  const canReset = !isEqual(defaultFilters, filters);

  const notFound = totalSaleOrders === 0 && canReset;
  const mapDriverToDriverSupplierItem = (driver: Driver): DriverItem => {
    return {
      id: driver.id.toString(),
      name: `${driver.firstName} ${driver.lastName}`.trim()
    }
  }


  useEffect(() => {
    if (suppliersData && suppliersData.length > 0) {
      const suppliersDetails: SupplierDTO[] = suppliersData;
      setSuppliersList(suppliersDetails);
    }
  }, [suppliersData]);

  useEffect(() => {
    if (driversData && driversData.length > 0) {
      const driversDetails: DriverItem[] = driversData.map(mapDriverToDriverSupplierItem);
      setDriversList(driversDetails);
    }
  }, [driversData]);


 
  useEffect(() => {
    if (
      filters.name ||
      filters.creationDate 
    ) {
      revalidate();
    }
  }, [filters, revalidate]);



  const dataInPage = tableData.slice(
    table.page * table.rowsPerPage,
    table.page * table.rowsPerPage + table.rowsPerPage
  );

  const denseHeight = table.dense ? 56 : 76;

  const settings = useSettingsContext();

  const handleFilters = useCallback(
    (name: string, value: ISaleOrderTableFilterValue) => {
      table.onResetPage();
      setFilters((prevState) => ({
        ...prevState,
        [name]: value,
      }));
    },
    [table]
  );

  const handleResetFilters = useCallback(() => {
    setFilters(defaultFilters);
  }, []);

  const handleGenerateSaleOrder = async () => {
    setOpenConfirmDialog(true);
  };

  useEffect(() => {
    if (saleOrders && saleOrders.length > 0) {
      setTableData(saleOrders);
    }
  }, [saleOrders, table.page, table.rowsPerPage]);

  const fetchUserData = async () => {
    const clientEmail = localStorage.getItem("email");
    const response = await UserByEmail(clientEmail!);
    const userData : IUser = response.data;
    if (userData)
      setUserId(userData.id)
  }

  const confirmGenerateSaleOrder = async () => {
    loading;
    setOpenConfirmDialog(false);
    try {
      fetchUserData();
      if (userId != "") {
        await generateSaleOrder(userId);
        revalidate();
        enqueueSnackbar("Commande de vente générée avec succès!", { variant: "success", autoHideDuration: 5000 });
      }
    } catch (error) {
      enqueueSnackbar("Échec de la génération de la commande de vente!", { variant: "error", autoHideDuration: 5000 });
    } finally {
      loading;
    }
  };

  const handleDeleteSelected = () => {
    const remainingData = tableData.filter(
      (row) => !table.selected.includes(row.saleOrder.id)
    );
    setTableData(remainingData);
    table.setSelected([]);
    enqueueSnackbar("Orders deleted successfully!", { variant: "success" });
  };

  const [openConfirmDeleteDialog, setOpenConfirmDeleteDialog] = useState(false);

  const handleOpenConfirmDeleteDialog = () => {
    setOpenConfirmDeleteDialog(true);
  };

  const handleCloseConfirmDeleteDialog = () => {
    setOpenConfirmDeleteDialog(false);
  };

  const handleConfirmDelete = async () => {
    try {
      const selectedOrderIds = table.selected;
      if (selectedOrderIds.length === 0) {
        enqueueSnackbar("No orders selected for deletion!", { variant: "warning" });
        return;
      }
      await deleteSaleOrders(selectedOrderIds);
      enqueueSnackbar("Orders deleted successfully!", { variant: "success" });
      setTableData((prevData) => prevData.filter((order) => !selectedOrderIds.includes(order.saleOrder.id)));
      table.setSelected([]);
      setOpenConfirmDeleteDialog(false);
    } catch (error) {
      enqueueSnackbar("Failed to delete orders!", { variant: "error" });
    }
  };

  const handleDeleteSaleOrder = (saleOrderId: string) => {
    const updatedSaleOrders = tableData.filter((order) => order.saleOrder.id !== saleOrderId);
    setTableData(updatedSaleOrders);
  };

  return (
    <Container maxWidth={settings.themeStretch ? false : "lg"}>
      <CustomBreadcrumbs
        heading="Liste commande préco"
        links={[
          { name: "Tableau de bord", href: paths.dashboard.root },
          { name: "Commande de vente", href: paths.dashboard.admin.saleOrder.root },
          { name: "List" },
        ]}
        sx={{ mb: { xs: 3, md: 5 } }}
      />

      <Card>
        <Grid container spacing={2} alignItems="center" justifyContent="space-between" sx={{ p: 2.5 }}>
          <Grid item xs={12} sm={6} md="auto">
            <Button
              variant="contained"
              color="success"
              startIcon={loading ? <CircularProgress size={20} /> : <Iconify icon="eva:plus-fill" />}
              sx={{ width: "100%", height: "100%" }}
              onClick={handleGenerateSaleOrder}
              disabled={loading}
            >
              {loading ? "Génération... (cela peut prendre du temps)" : "Générer Commande de Vente"}
            </Button>
          </Grid>
        </Grid>

        <SaleOrderTableToolbar
          filters={filters}
          onFilters={handleFilters}
          serviceOptions={INVOICE_SERVICE_OPTIONS.map((option) => option.name)}
        />

        {canReset && (
          <SaleOrderTableFiltersResult
            filters={filters}
            onFilters={handleFilters}
            onResetFilters={handleResetFilters}
            results={totalSaleOrders}
            sx={{ p: 2.5, pt: 0 }}
          />
        )}

        <TableContainer sx={{ position: "relative", overflow: "unset" }}>
          {loadingSaleOrder ? (
            <Stack alignItems="center" justifyContent="center" sx={{ py: 5 }}>
              <CircularProgress />
            </Stack>
          ) : (
            <>
              <TableSelectedAction
                dense={table.dense}
                numSelected={table.selected.length}
                rowCount={totalSaleOrders}
                onSelectAllRows={(checked) => {
                  table.onSelectAllRows(checked, saleOrders.map((row) => row.saleOrder.id));
                }}
                action={
                  <>
                    <IconButton onClick={handleOpenConfirmDeleteDialog}>
                      <Iconify icon="eva:trash-2-outline" />
                    </IconButton>
                  </>
                }
              />

              <Dialog
                open={openConfirmDeleteDialog}
                onClose={handleCloseConfirmDeleteDialog}
              >
                <DialogTitle>Confirmation</DialogTitle>
                <DialogContent>
                  <DialogContentText>
                    Êtes-vous sûr de vouloir supprimer les commandes de vente sélectionnées ?
                  </DialogContentText>
                </DialogContent>
                <DialogActions>
                  <Button onClick={handleCloseConfirmDeleteDialog} color="primary">
                    Annuler
                  </Button>
                  <Button onClick={handleConfirmDelete} color="primary" autoFocus>
                    Supprimer
                  </Button>
                </DialogActions>
              </Dialog>

              <Scrollbar>
                <Table size={table.dense ? "small" : "medium"} sx={{ minWidth: 800 }}>
                  <TableHeadCustom
                    order={table.order}
                    orderBy={table.orderBy}
                    headLabel={TABLE_HEAD}
                    rowCount={totalSaleOrders}
                    numSelected={table.selected.length}
                    onSort={table.onSort}
                  />
                  <TableBody>
                  {loading ? (
                    <TableRow>
                      <TableCell
                        colSpan={TABLE_HEAD.length}
                        style={{ textAlign: "center", padding: "20px" }}
                      >
                        <CircularProgress />
                      </TableCell>
                    </TableRow>
                  ) : (
                    <>
                      {notFound ? (
                        <TableNoData notFound={notFound} />
                      ) : (
                        tableData.map((row) => (
                          <SaleOrderTableRow
                          key={row.saleOrder.id}
                          row={row}
                          selected={table.selected.includes(row.saleOrder.id)}
                          onSelectRow={() => table.onSelectRow(row.saleOrder.id)}
                          onViewRow={() => { }}
                          onEditRow={() => console.log(row.saleOrder.id)}
                          onDeleteRow={() => handleDeleteSaleOrder(row.saleOrder.id)}
                          drivers={driversList}
                          suppliers={suppliersList}
                        />
                        ))
                      )}

                      <TableEmptyRows
                        height={denseHeight}
                        emptyRows={emptyRows(
                          table.page,
                          table.rowsPerPage,
                          totalSaleOrders
                        )}
                      />
                    </>
                  )}
                </TableBody>



                </Table>
              </Scrollbar>
            </>
          )}
        </TableContainer>

        <TablePaginationCustom
            count={totalSaleOrders}
            page={pageNumber}
            rowsPerPage={pageSize}
            onPageChange={table.onChangePage}
            onRowsPerPageChange={table.onChangeRowsPerPage}
            dense={table.dense}
            onChangeDense={table.onChangeDense}
          />
      </Card>

      <Dialog open={openConfirmDialog} onClose={() => setOpenConfirmDialog(false)}>
        <DialogTitle>Confirmer la génération</DialogTitle>
        <DialogContent>
          <DialogContentText>Êtes-vous sûr de vouloir générer les commandes de vente?</DialogContentText>
        </DialogContent>
        <DialogActions>
          <Button onClick={() => setOpenConfirmDialog(false)} color="primary">
            Annuler
          </Button>
          <Button onClick={confirmGenerateSaleOrder} color="primary">
            Générer
          </Button>
        </DialogActions>
      </Dialog>
    </Container>
  );
}

