import React, { useEffect, useState } from 'react';
import { styled } from '@mui/system';
import { Button, IconButton, TableRow, TableCell, Collapse, Stack, Box, Paper, Divider, ListItemText, Dialog, DialogActions, DialogContent, DialogContentText, DialogTitle, MenuItem, TextField, CircularProgress } from '@mui/material';
import Avatar from "@mui/material/Avatar";
import Checkbox from "@mui/material/Checkbox";
import Typography from "@mui/material/Typography";
import { useBoolean } from "@/hooks/use-boolean";
import { fCurrency } from "@/utils/format-number";
import { fDate, fTime } from "@/utils/format-time";
import Iconify from "@/components/iconify";
import CustomPopover, { usePopover } from "@/shared/components/custom-popover";
import { DateTimePicker } from '@mui/x-date-pickers';
import { deleteSaleOrder, deleteSaleOrderItem } from '@/shared/api/saleOrders';
import { enqueueSnackbar } from 'notistack';
import { ConfirmSaleOrderRequest, SaleOrder, SaleOrderItem, SaleOrderPage } from '@/contexts/types/saleOrder';
import { UserByEmail } from '@/shared/api/user';
import { IUser } from '@/shared/types/user';
import { confirmSaleOrderCreation } from '@/shared/api/saleOrders';

type Props = {
  row: SaleOrderPage;
  selected: boolean;
  onSelectRow: VoidFunction;
  onViewRow: VoidFunction;
  onEditRow: VoidFunction;
  onDeleteRow: VoidFunction;
  drivers: any[];
  suppliers: any[];
};

const StyledTextField = styled('input')({
  padding: '0 12px',
  fontSize: '14px',
  height: '36px',
  border: '1px solid #E0E0E0',
  borderRadius: '8px',
  textAlign: 'center',
  width: '60px',
});

export default function SaleOrderTableRow({
  row,
  selected,
  onSelectRow,
  onViewRow,
  onEditRow,
  onDeleteRow,
  drivers,
  suppliers
}: Props) {


  const [showDetails, setShowDetails] = useState(false);
  const [updatedItems, setUpdatedItems] = useState<SaleOrderItem[]>(row.saleOrder.saleOrderItems);
  const [openConfirmDeleteOrderDialog, setOpenConfirmDeleteOrderDialog] = useState(false);
  const [openConfirmDeleteProductDialog, setOpenConfirmDeleteProductDialog] = useState(false);
  const [itemToDelete, setItemToDelete] = useState<string | null>(null);
  const [loadingSaleOrder, setLoadingSaleOrder] = useState(false);
  const [orderToDelete, setOrderToDelete] = useState<string | null>(null);
  const [tableData, setTableData] = useState<SaleOrder[]>([])
  const [openConfirmDialog, setOpenConfirmDialog] = useState(false);
  const [selectedFournisseur, setSelectedFournisseur] = useState("");
  const [selectedChauffeur, setSelectedChauffeur] = useState("");
  const [deliveryDate, setDeliveryDate] = useState<Date | null>(null);
  const confirm = useBoolean();
  const popover = usePopover();
  const [reload, setReload] = useState(false);  // Add this state definition
  const [confirmLoading, setConfirmLoading] = useState(false);
  const [userId, setUserId] = useState<string>("");

  const toggleDetails = () => setShowDetails(!showDetails);

  const handleQuantityChange = (itemId: string, newQuantity: number) => {
    const validQuantity = newQuantity < 1 ? 1 : newQuantity;
    setUpdatedItems((prevItems) =>
      prevItems.map((item) =>
        item.id === itemId
          ? { ...item, quantity: validQuantity, total: validQuantity * item.productPrice }
          : item
      )
    );
  };

  useEffect(() => {
    if (suppliers.length > 0) {
      setSelectedFournisseur(suppliers[0].id);
    }
    if (drivers.length > 0) {
      setSelectedChauffeur(drivers[0].id);
    }
    fetchUserData();
  }, [suppliers, drivers])


  const fetchUserData = async () => {
    const clientEmail = localStorage.getItem("email");
    const response = await UserByEmail(clientEmail!);
    const userData: IUser = response.data;
    if (userData)
      setUserId(userData.id)
  }

  const handleDeleteProduct = (itemId: string) => {
    setItemToDelete(itemId);
    setOpenConfirmDeleteProductDialog(true);
  };


  const confirmDeleteProduct = async () => {
    try {
      await deleteSaleOrderItem(itemToDelete!);
      enqueueSnackbar("Produit supprimé avec succès !!", { variant: "success" });
      const updated = updatedItems.filter((item) => item.id !== itemToDelete);
      setUpdatedItems(updated);
      setOpenConfirmDeleteProductDialog(false);
      if (updated.length === 0) {
        onDeleteRow();
        setOpenConfirmDeleteOrderDialog(false);
      }
    } catch (error) {
      enqueueSnackbar("Échec de la suppression du produit !", { variant: "error" });
    }
  };

  const handleDeleteOrder = (saleOrderId: string) => {
    setOrderToDelete(saleOrderId);
    setOpenConfirmDeleteOrderDialog(true);
  };

  const confirmDeleteOrder = async () => {
    try {
      await deleteSaleOrder(orderToDelete!);
      enqueueSnackbar("La Commande de vente supprimée avec succès !", { variant: "success" });
      onDeleteRow();
      setOrderToDelete(null);
      setOpenConfirmDeleteOrderDialog(false);
      setReload(!reload);
      setOpenConfirmDeleteOrderDialog(false);
    } catch (error) {
      enqueueSnackbar("Échec de la suppression de la commande de vente !", { variant: "error" });
    }
  };

  const handleConfirmOrder = (row: SaleOrder) => {
    setOpenConfirmDialog(true);
  };

  const confirmOrder = async () => {
    if (!selectedFournisseur) {
      enqueueSnackbar("Le fournisseur est requis", { variant: "error" });
      return;
    }

    if (!selectedChauffeur) {
      enqueueSnackbar("Le chauffeur est requis", { variant: "error" });
      return;
    }

    if (!deliveryDate) {
      enqueueSnackbar("La date de livraison est requise", { variant: "error" });
      return;
    }

    if (new Date(deliveryDate) < new Date(row.saleOrder.createdAt)) {
      enqueueSnackbar("La date de livraison doit être supérieure ou égale à la date de création", { variant: "error" });
      return;
    }

    if (!updatedItems || updatedItems.length === 0) {
      enqueueSnackbar("Aucun article n'a été ajouté à la commande", { variant: "error" });
      return;
    }

    setConfirmLoading(true);

    if (!userId) {
      setConfirmLoading(false);
      enqueueSnackbar("Échec de la récupération d'utilisateur !", { variant: "error" });
      return;
    }

    try {
      const confirmData: ConfirmSaleOrderRequest = {
        saleOrderId: row.saleOrder.id,
        confirmedBy: userId,
        supplierId: selectedFournisseur,
        driverId: selectedChauffeur,
        deliveryDate: new Date(new Date(deliveryDate).getTime() - new Date(deliveryDate).getTimezoneOffset() * 60000).toISOString(),
        orderItems: updatedItems.map(item => ({
          productId: item.productId,
          quantity: item.quantity,
        })),
      };

      const response = await confirmSaleOrderCreation(confirmData);

      setOpenConfirmDialog(false);
      onDeleteRow();
      onEditRow();

      enqueueSnackbar("Commande de vente confirmée avec succès !", { variant: "success" });
      enqueueSnackbar("Commande d'achat générée avec succès !", { variant: "success", autoHideDuration: 5000 });

      console.log("Response reçue :", response);
    } catch (error) {
      console.error("Erreur lors de la confirmation de la commande :", error);
      enqueueSnackbar("Échec de la confirmation de la commande de vente !", { variant: "error" });
    } finally {
      setConfirmLoading(false);
    }
  };


  return (
    <>
      <TableRow hover selected={selected} onClick={toggleDetails}>
        <TableCell padding="checkbox" onClick={(event) => event.stopPropagation()}>
          <Checkbox checked={selected} onClick={onSelectRow} />
        </TableCell>

        <TableCell>{row.saleOrder.id}</TableCell>

        <TableCell>
          <div className="flex items-center">
            <Avatar
              alt={row.supplierName}
              sx={{ mr: 1, width: 25, height: 25 }}
            >
              {(row.supplierName?.charAt(0) || "F").toUpperCase()}
            </Avatar>
            <Typography variant="body2" noWrap>
              {row.supplierName || "Fournisseur non trouvé"}
            </Typography>
          </div>
        </TableCell>

        <TableCell>
          <ListItemText
            primary={fDate(row.saleOrder?.createdAt)}
            secondary={fTime(row.saleOrder?.createdAt)}
            primaryTypographyProps={{ typography: "body2", noWrap: true }}
            secondaryTypographyProps={{
              mt: 0.5,
              component: "span",
              typography: "caption",
            }}
          />
        </TableCell>

        <TableCell>{fCurrency(row.saleOrder.totalPrice)}</TableCell>

        <TableCell align="center">{updatedItems.length}</TableCell>

        <TableCell>
          <Button variant="contained" color="primary" size="small" onClick={toggleDetails}>
            Afficher
          </Button>
        </TableCell>

        <TableCell align="right" sx={{ px: 1 }}>
          <IconButton
            color={popover.open ? "inherit" : "default"}
            onClick={(e) => {
              popover.onOpen(e);
              e.stopPropagation();
            }}
          >
            <Iconify icon="eva:more-vertical-fill" />
          </IconButton>
        </TableCell>
      </TableRow>

      <CustomPopover
        open={popover.open}
        onClose={popover.onClose}
        arrow="right-top"
        sx={{ width: 160 }}
      >
        <MenuItem
          onClick={() => {
            handleDeleteOrder(row.saleOrder.id);
            popover.onClose();
          }}
        >
          <Iconify icon="eva:trash-2-outline" />
          Supprimer
        </MenuItem>
      </CustomPopover>

      {showDetails && (
        <TableRow>
          <TableCell sx={{ border: 'none' }} colSpan={10}>
            <Collapse in={showDetails} timeout="auto" unmountOnExit sx={{ bgcolor: 'background.neutral' }}>
              <Stack direction="row" alignItems="center" sx={{ p: 1.5, m: 1.5, bgcolor: 'background.neutral' }}>
                <Typography variant="subtitle2" sx={{ flex: 1 }}>
                  Produits commandés
                </Typography>
                <IconButton size="small" onClick={toggleDetails}>
                  <Iconify icon="eva:arrow-ios-upward-fill" />
                </IconButton>
              </Stack>
              <Stack component={Paper} sx={{ m: 1.5 }}>
                {updatedItems.map((item) => (
                  <Stack
                    key={item.id}
                    direction="row"
                    alignItems="center"
                    justifyContent="space-between"
                    sx={{
                      p: (theme) => theme.spacing(1.5, 2, 1.5, 1.5),
                      '&:not(:last-of-type)': {
                        borderBottom: (theme) => `solid 2px ${theme.palette.background.neutral}`,
                      },
                    }}
                  >
                    <ListItemText
                      primary={item.productName}
                      primaryTypographyProps={{ typography: 'body2' }}
                      secondaryTypographyProps={{
                        component: 'span',
                        color: 'text.disabled',
                        mt: 0.5,
                      }}
                    />
                    <Box sx={{ display: 'flex', alignItems: 'center' }}>
                      <Typography variant="body2" sx={{ minWidth: 80, textAlign: 'right' }}>{fCurrency(item.productPrice)}</Typography>
                      <StyledTextField
                        type="number"
                        value={item.quantity}
                        onChange={(e) => handleQuantityChange(item.id, parseInt(e.target.value, 10) || 1)}
                        min={1}
                      />
                      <Typography variant="body2" sx={{ minWidth: 80, textAlign: 'right' }}>{fCurrency(item.productPrice * item.quantity)}</Typography>
                      <IconButton color="error" onClick={() => handleDeleteProduct(item.id)}>
                        <Iconify icon="eva:trash-2-outline" />
                      </IconButton>
                    </Box>
                  </Stack>
                ))}
              </Stack>
              <Stack>
                <Divider />
                <Stack direction="row" alignItems="center" sx={{ p: 1.5, m: 1.5, bgcolor: 'background.neutral' }}>
                  <Button
                    variant="contained"
                    color="info"
                    size="small"
                    onClick={() => handleConfirmOrder(row.saleOrder)}
                  >
                    Confirmer la commande
                  </Button>
                </Stack>
              </Stack>
            </Collapse>
          </TableCell>
        </TableRow>
      )}

      {/* Confirmation dialog for order details */}
      <Dialog
        open={openConfirmDialog}
        onClose={() => setOpenConfirmDialog(false)}
      >
        <DialogTitle>Confirmer la commande</DialogTitle>
        <DialogContent>
          <DialogContentText>
            Sélectionnez les détails de la commande avant de confirmer.
          </DialogContentText>
          <Stack spacing={3}>
            <TextField
              select
              label="Fournisseur"
              value={selectedFournisseur}
              onChange={(e) => setSelectedFournisseur(e.target.value)}
            >
              {suppliers.map((supplier) => (
                <MenuItem key={supplier.id} value={supplier.id}>
                  {supplier.supplierName} {supplier.supplierLastName}
                </MenuItem>
              ))}
            </TextField>

            {/* Chauffeur Select */}
            <TextField
              select
              label="Chauffeur"
              value={selectedChauffeur}
              onChange={(e) => setSelectedChauffeur(e.target.value)}
            >
              {drivers.map((driver) => (
                <MenuItem key={driver.id} value={driver.id}>
                  {driver.name}
                </MenuItem>
              ))}
            </TextField>

            <DateTimePicker
              label="Date de livraison"
              value={deliveryDate}
              onChange={(newValue) => setDeliveryDate(newValue)}
              format="yyyy/MM/dd HH:mm:ss"
              slotProps={{
                textField: {
                  fullWidth: true,
                },
              }}
            />
          </Stack>
        </DialogContent>
        <DialogActions>
          <Button onClick={() => setOpenConfirmDialog(false)} color="primary">
            Annuler
          </Button>
          <Button onClick={confirmOrder} color="primary">
            {
              confirmLoading ? (
                <CircularProgress size={24} />
              ) : (
                "Confirmer"
              )
            }
          </Button>
        </DialogActions>
      </Dialog>

      {/* Confirmation dialog for deleting a product */}
      <Dialog
        open={openConfirmDeleteProductDialog}
        onClose={() => setOpenConfirmDeleteProductDialog(false)}
      >
        <DialogTitle>Confirmer la suppression du produit</DialogTitle>
        <DialogContent>
          <DialogContentText>Êtes-vous sûr de vouloir supprimer ce produit ?</DialogContentText>
        </DialogContent>
        <DialogActions>
          <Button onClick={() => setOpenConfirmDeleteProductDialog(false)} color="primary">
            Annuler
          </Button>
          <Button onClick={confirmDeleteProduct} color="error">
            Supprimer
          </Button>
        </DialogActions>
      </Dialog>

      {/* Confirmation dialog for deleting the order */}
      <Dialog
        open={openConfirmDeleteOrderDialog}
        onClose={() => setOpenConfirmDeleteOrderDialog(false)}
      >
        <DialogTitle>Confirmer la suppression de la commande</DialogTitle>
        <DialogContent>
          <DialogContentText>Êtes-vous sûr de vouloir supprimer cette commande de vente ?</DialogContentText>
        </DialogContent>
        <DialogActions>
          <Button onClick={() => setOpenConfirmDeleteOrderDialog(false)} color="primary">
            Annuler
          </Button>
          <Button onClick={confirmDeleteOrder} color="error">
            Supprimer
          </Button>
        </DialogActions>
      </Dialog>
    </>
  );
}


