import { useState, useCallback, useEffect } from "react";
import Stack from "@mui/material/Stack";
import Dialog from "@mui/material/Dialog";
import TextField from "@mui/material/TextField";
import Typography from "@mui/material/Typography";
import InputAdornment from "@mui/material/InputAdornment";
import ListItemButton, {
  listItemButtonClasses,
} from "@mui/material/ListItemButton";
import Iconify from "@/components/iconify";
import SearchNotFound from "@/shared/components/search-not-found";

export type DialogItem = {
  id?: string;
  name: string;
  company?: string;
  primary?: boolean;
  fullAddress?: string;
  phoneNumber?: string;
};

type Props = {
  title?: string;
  list: DialogItem[];
  action?: React.ReactNode;
  //
  open: boolean;
  onClose: VoidFunction;
  //
  selected: (selectedId: string) => boolean;
  onSelect: (address:  DialogItem | null) => void;
  defaultSelectedId: string | null;
  setSelectedId: (defaultSelectedId: string) => void;
};

export default function DriverSupplierListDialog({
  title = "Liste",
  list,
  action,
  
  open,
  onClose,
  
  selected,
  onSelect,
  defaultSelectedId,
  setSelectedId
}: Props) {
  const [searchValue, setSearchValue] = useState("");

  useEffect(() => {
    if (list.length > 0) {
      if (!defaultSelectedId) {
        const firstItem = list[0];
        if (firstItem.id) {
          if (firstItem.id !== defaultSelectedId) {
            setSelectedId(firstItem.id);
            onSelect(firstItem);
          }
        }
      } else {
        const defaultItem = list.find((item) => item.id === defaultSelectedId);
        if (defaultItem) {
          onSelect(defaultItem);
        }
      }
    }
  }, [defaultSelectedId, list]);

  const dataFiltered = applyFilter({
    inputData: list,
    query: searchValue,
  });

  const notFound = !dataFiltered.length && !!searchValue;

  const handleSearchItem = useCallback(
    (event: React.ChangeEvent<HTMLInputElement>) => {
      setSearchValue(event.target.value);
    },
    []
  );

  const handleSelectItem = useCallback(
    (item: DialogItem | null) => {
      onSelect(item);
      setSearchValue("");
      if (item?.id) {
        setSelectedId(item?.id)
      }
      onClose();
    },
    [onClose, onSelect]
  );

  const renderList = (
    <Stack
      spacing={0.5}
      sx={{
        p: 0.5,
        maxHeight: 80 * 8,
        overflowX: "hidden",
      }}
    >
      {dataFiltered.map((item) => (
        <Stack
          key={item.id}
          spacing={0.5}
          component={ListItemButton}
          selected={selected(item?.id ?? "") || item.id === defaultSelectedId}
          onClick={() => handleSelectItem(item)}
          sx={{
            py: 1,
            px: 1.5,
            borderRadius: 1,
            flexDirection: "column",
            alignItems: "flex-start",
            [`&.${listItemButtonClasses.selected}`]: {
              bgcolor: "action.selected",
              "&:hover": {
                bgcolor: "action.selected",
              },
            },
          }}
        >

          <Typography variant="subtitle2" sx={{ color: "text.secondary" }}>
            {item.id}
          </Typography>

          <Stack direction="row" alignItems="center" spacing={1}>
            <Typography variant="subtitle2" sx={{ color: "primary.main" }}>
              {item.name}
            </Typography>
          </Stack>

          <Typography variant="body2" sx={{ color: "text.secondary" }}>
            {item.fullAddress}
          </Typography>

          <Typography variant="body2" sx={{ color: "text.secondary" }}>
            {item.phoneNumber}
          </Typography>
        </Stack>
      ))}
    </Stack>
  );

  return (
    <Dialog fullWidth maxWidth="xs" open={open} onClose={onClose}>
      <Stack
        direction="row"
        alignItems="center"
        justifyContent="space-between"
        sx={{ p: 3, pr: 1.5 }}
      >
        <Typography variant="h6"> {title} </Typography>

        {action && action}
      </Stack>

      <Stack sx={{ p: 2, pt: 0 }}>
        <TextField
          value={searchValue}
          onChange={handleSearchItem}
          placeholder="Recherche..."
          InputProps={{
            startAdornment: (
              <InputAdornment position="start">
                <Iconify
                  icon="eva:search-fill"
                  sx={{ color: "text.disabled" }}
                />
              </InputAdornment>
            ),
          }}
        />
      </Stack>

      {notFound ? (
        <SearchNotFound query={searchValue} sx={{ px: 3, pt: 5, pb: 10 }} />
      ) : (
        renderList
      )}
    </Dialog>
  );
}

// ----------------------------------------------------------------------

function applyFilter({
  inputData,
  query,
}: {
  inputData: DialogItem[];
  query: string;
}) {
  if (query) {
    return inputData.filter(
      (item) =>
        item.name.toLowerCase().indexOf(query.toLowerCase()) !== -1 ||
        item.fullAddress?.toLowerCase().indexOf(query.toLowerCase()) !== -1 ||
        item.phoneNumber?.toLowerCase().indexOf(query.toLowerCase()) !== -1 ||
        `${item.id}`.toLowerCase().indexOf(query.toLowerCase()) !== -1
    );
  }

  return inputData;
}
