import React, { useEffect, useState } from 'react';
import {Card, CardContent, CardMedia, Typography, Box, Button, Dialog, Pagination, CircularProgress, Grid, MenuItem, Select, 
  FormControl, SelectChangeEvent, TextField} from '@mui/material';
import { DEFAULT_IMAGE_URL } from '@/config-global';
import { SaveSupplierOrderProductItem, SupplierOrderProduct } from '@/contexts/types/supplierOrder';
import { getAllProductSummaryWithKeyword } from '@/shared/api/supplierOrders';

interface PaginatedProductsDialogProps {
  onAdd: (product: SupplierOrderProduct) => void;
  existingProducts: SaveSupplierOrderProductItem[];
}

const SupplierOrderProductsDialog: React.FC<PaginatedProductsDialogProps> = ({ onAdd, existingProducts }) => {
  const [items, setItems] = useState<SupplierOrderProduct[]>([]);
  const [open, setOpen] = useState(false);
  const [page, setPage] = useState(0);
  const [size, setSize] = useState(12);
  const [keyword, setKeyword] = useState('');

  const { productsData, productsLoading, totalPages } = getAllProductSummaryWithKeyword(page, size, keyword);

  useEffect(() => {
    if (open) {
      setItems(productsData);
    }
    console.log(productsData);
  }, [open, productsData]);

  const handlePageChange = (event: React.ChangeEvent<unknown>, value: number) => {
    setPage(value - 1);
  };

  const handleSizeChange = (event: SelectChangeEvent<number>) => {
    setSize(event.target.value as number);
    setPage(0);
  };

  const handleKeywordChange = (event: React.ChangeEvent<HTMLInputElement>) => {
    const value = event.target.value.trim();

    if (value === '') {
      setKeyword('');
      setPage(0);
    } else if (value.length >= 3) {
      setKeyword(value);
      setPage(0);
    }
  };

  const isValidImagePath = (path: string): string => {
    const img = new Image();
    img.src = path;
    return img.complete && img.naturalWidth !== 0 ? path : DEFAULT_IMAGE_URL!;
  };

  const handleAddProduct = (product: SupplierOrderProduct) => {
    if (existingProducts.some((item: SaveSupplierOrderProductItem) => item.productId === product.id)) {
      return;
    }
    onAdd(product);
    setOpen(false);
  };

  return (
    <Box>
      <Button variant="outlined" onClick={() => setOpen(true)} fullWidth>Choisir un produit existant</Button>
      <Dialog open={open} onClose={() => setOpen(false)} maxWidth="md" fullWidth>
        <Typography variant="h6" sx={{ flex: 1, textAlign: 'center', mt: 2 }}>
          Choisir un produit
        </Typography>
        <Box
          sx={{
            position: 'relative',
            p: 2,
            display: 'flex',
            justifyContent: 'space-between',
            alignItems: 'center',
            width: '100%',
          }}
        >
          <TextField
            label="Rechercher"
            fullWidth
            sx={{
              flex: 1,
              width: '70%',
            }}
            onChange={handleKeywordChange}
          />
          <FormControl
            sx={{
              minWidth: 80,
              maxWidth: 80,
              mt: 0,
              ml: 2,
            }}
          >
            <Select value={size} onChange={handleSizeChange}>
              {[12, 24, 36].map((sizeOption) => (
                <MenuItem key={sizeOption} value={sizeOption}>
                  {sizeOption}
                </MenuItem>
              ))}
            </Select>
          </FormControl>
        </Box>

        <Box sx={{ p: 2 }}>
          {productsLoading ? (
            <Grid container justifyContent="center">
              <CircularProgress />
            </Grid>
          ) : (
            <>
              <Grid container spacing={2}>
                {items.length === 0 ? (
                  <Grid item xs={12} sx={{ display: 'flex', justifyContent: 'center', marginTop: 2, marginBottom: 2 }}>
                    <Typography variant="body1" color="text.secondary">Aucun produit trouvé</Typography>
                  </Grid>
                ) : (
                  items.map((item) => (
                    <Grid item xs={12} sm={6} md={4} key={item.id}>
                      <Card
                        onClick={() => handleAddProduct(item)}
                        sx={{ cursor: 'pointer', position: 'relative', display: 'flex', flexDirection: 'column', height: '100%' }}
                      >
                        {item.mainImagePath && (
                          <CardMedia
                            component="img"
                            height="140"
                            image={isValidImagePath(item.mainImagePath)}
                            alt={item.name}
                          />
                        )}
                        <CardContent>
                          <Typography variant="h6">{item.name}</Typography>
                          <Typography color="text.secondary">{item.ugs}</Typography>
                        </CardContent>
                      </Card>
                    </Grid>
                  ))
                )}
              </Grid>
              <Pagination
                count={totalPages}
                page={page + 1}
                onChange={handlePageChange}
                color="primary"
                sx={{ display: 'flex', justifyContent: 'center', marginTop: 2 }}
              />
            </>
          )}
        </Box>
      </Dialog>
    </Box>
  );
};

export default SupplierOrderProductsDialog;
