import React, { useEffect, useState } from 'react';
import { List, ListItem, ListItemText, Avatar, Typography, Button, Card, Modal, Checkbox, TextField, InputAdornment } from '@mui/material';
import { DEFAULT_IMAGE_URL } from '@/config-global';
import { SaveSupplierOrderProductItem, SupplierOrderProductVariation } from '@/contexts/types/supplierOrder';
import Iconify from '@/components/iconify';

interface SelectVariationsModalProps {
    showVariationsModal: boolean;
    setShowVariationsModal: (value: boolean) => void;
    variationsChoices: SupplierOrderProductVariation[];
    setVariationsChoices: (value: SupplierOrderProductVariation[]) => void;
    selectedItems: SaveSupplierOrderProductItem[];
    setSelectedItems: (value: SaveSupplierOrderProductItem[]) => void;
    handleAddSelectedVariation: (variation: SupplierOrderProductVariation) => void;
}

const SupplierOrderVariationsModal: React.FC<SelectVariationsModalProps> = ({
    showVariationsModal,
    setShowVariationsModal,
    variationsChoices,
    setVariationsChoices,
    selectedItems,
    setSelectedItems,
    handleAddSelectedVariation
}) => {

    const isValidImagePath = (path: string): string => {
        const img = new Image();
        img.src = path;
        return img.complete && img.naturalWidth !== 0 ? path : DEFAULT_IMAGE_URL!;
    };

    const toggleVariationSelection = (variation: SupplierOrderProductVariation) => {
        const isSelected = selectedItems.some((item: SaveSupplierOrderProductItem) => item.productId === variation.id);
        if (isSelected) {
            setSelectedItems(selectedItems.filter((item: SaveSupplierOrderProductItem) => item.productId !== variation.id));
        } else {
            handleAddSelectedVariation(variation);
        }
    };

    const [filtredVariations, setFiltredVariations] = useState<SupplierOrderProductVariation[]>([]);
    const [keyword, setKeyword] = useState<string>("");
    
    useEffect(() => {
        if (keyword !== "") {
            const data = variationsChoices.filter((variation) => 
                variation.name.toLowerCase().includes(keyword.toLowerCase()) || 
                variation.id.toString().includes(keyword)
            );
            setFiltredVariations(data);
        } else {
            setFiltredVariations(variationsChoices);
        }
    }, [keyword, variationsChoices]);    
    
    const onChangeKeyword = (event: React.ChangeEvent<HTMLInputElement | HTMLTextAreaElement>) => {
        setKeyword(event.target.value);
    };
    
    return (
        <Modal
            open={showVariationsModal && variationsChoices && variationsChoices.length > 0}
            onClose={() => {
                setVariationsChoices([]);
                setShowVariationsModal(false);
            }}
            aria-labelledby="modal-modal-title"
            aria-describedby="modal-modal-description"
        >
            <Card sx={{
                position: 'absolute',
                top: '50%',
                left: '50%',
                transform: 'translate(-50%, -50%)',
                maxHeight: '80vh',
                maxWidth: '80vw',
                overflow: 'hidden',
                display: 'flex',
                flexDirection: 'column',
                boxShadow: 24,
                p: 2,
            }}>
                <Typography id="modal-modal-title" variant="h6" component="h2" sx={{ mb: 1, ml: 3 }}>
                    Liste des variations
                </Typography>

                <TextField
                    fullWidth
                    value={keyword}
                    onChange={onChangeKeyword}
                    placeholder="Chercher par code promo, montant minimum, usage, limite d'usage"
                    InputProps={{
                        startAdornment: (
                            <InputAdornment position="start">
                                <Iconify
                                    icon="eva:search-fill"
                                    sx={{ color: "text.disabled" }}
                                />
                            </InputAdornment>
                        ),
                    }}
                />

                <List sx={{ overflowY: 'auto', maxHeight: 'calc(100% - 64px)', p: 2 }}>
                    {filtredVariations?.map((variation: SupplierOrderProductVariation) => {
                        const isChecked = selectedItems.some((item: SaveSupplierOrderProductItem) => item.productId === variation.id);
                        return (
                            <ListItem key={variation.id} onClick={() => toggleVariationSelection(variation)}>
                                <Checkbox
                                    edge="start"
                                    checked={isChecked}
                                    tabIndex={-1}
                                    disableRipple
                                    sx={{ mr: 2 }}
                                />
                                {variation.mainImagePath && (
                                    <Avatar
                                        src={isValidImagePath(variation.mainImagePath)}
                                        alt={variation.name}
                                    />
                                )}
                                <ListItemText
                                    primary={`${variation.name}`}
                                    secondary={`ID: ${variation.id}`}
                                />
                            </ListItem>
                        );
                    })}
                </List>
                <Button
                    onClick={() => {
                        setVariationsChoices([]);
                        setShowVariationsModal(false);
                    }}
                    sx={{
                        backgroundColor: 'black',
                        color: 'white',
                        '&:hover': {
                            backgroundColor: 'darkgray',
                        }
                    }}
                >
                    Fermer
                </Button>
            </Card>
        </Modal>
    );
};

export default SupplierOrderVariationsModal;
