import sum from "lodash/sum";
import { useEffect, useCallback, useState } from "react";
import { useFormContext } from "react-hook-form";
import Box from "@mui/material/Box";
import Stack from "@mui/material/Stack";
import Button from "@mui/material/Button";
import Divider from "@mui/material/Divider";
import Typography from "@mui/material/Typography";
import InputAdornment from "@mui/material/InputAdornment";
import { inputBaseClasses } from "@mui/material/InputBase";
import { fCurrency } from "@/utils/format-number";
import Iconify from "@/components/iconify";
import { RHFTextField } from "@/shared/components/hook-form";
import { Alert, CircularProgress, Grid, ListItemText, Modal, Snackbar } from "@mui/material";
import { useSnackbar } from "@/components/snackbar";
import ProductNewEditForm from "@/shared/sections/product/add-product/MainForm";
import { SaveSupplierOrderProductItem, SupplierOrderProduct, SupplierOrderProductVariation } from "@/contexts/types/supplierOrder";
import { ProductType } from "@/shared/types/product";
import SupplierOrderProductsDialog from "./SupplierOrderProductsDialog";
import SupplierOrderVariationsModal from "./SupplierOrderVariationsModal";


export interface CustomProduct {
  id: any;
  name: any;
  mainImagePath: any;
  type: any;
  productPrice: number;
  isVariable: boolean;
  quantity: number;
  variations?: CustomVariation[];
  comment?: string;
}

export type CustomVariation = {
  id: string;
  name: string;
  mainImagePath: any;
  productPrice: number;
  quantity: number;
  comment?: string;
}

type Props = {
  itemsList: SaveSupplierOrderProductItem[];
  setItemsList?: (itemsList: SaveSupplierOrderProductItem[]) => void;
  supplierName?: string;
}

export default function OrderNewEditDetails({ itemsList, setItemsList, supplierName }: Props) {
  const { enqueueSnackbar } = useSnackbar();
  const { setValue, formState: { errors } } = useFormContext();
  const [originItemList, setOriginItemList] = useState<SaveSupplierOrderProductItem[]>(() => [...itemsList]);
  const [productsData, setProductsData] = useState<CustomProduct[]>([]);
  const [productLoading, setProductLoading] = useState(false);
  const [addNewPoductModal, setAddNewProductModal] = useState(false);
  const [selectProductVariationModal, setSelectProductVariationModal] = useState<boolean>(false);
  const [variationsList, setVariationsList] = useState<SupplierOrderProductVariation[]>([]);
  const [totalPriceSupplierOrder, setTotalPriceSupplierOrder] = useState(0);

  useEffect(() => {
    if (itemsList.length > 0 && originItemList.length === 0) {
      setOriginItemList([...itemsList]);
    }
  }, [itemsList.length]);

  useEffect(() => {
    if (setItemsList && originItemList !== itemsList) {
      setItemsList(originItemList);
    }
  }, [originItemList, setItemsList, itemsList]);

  useEffect(() => {
    const totalPrice = sum(originItemList.map(item => item.totalPrice));
    setTotalPriceSupplierOrder(totalPrice);
    setValue("totalPrice", totalPrice);
  }, [originItemList]);

  const handleAddItem = (newItem: SaveSupplierOrderProductItem) => {
    setOriginItemList((prevItems) => [...prevItems, newItem]);
  };

  const handleRemoveItem = (index: number) => {
    setOriginItemList((prevItems) => prevItems.filter((_, i) => i !== index));
  };

  const handleOpenAddNewProductModal = () => {
    setAddNewProductModal(true);
  }

  const handleAddProduct = useCallback((product: SupplierOrderProduct) => {
    let index = (originItemList?.length ?? 0) > 0 ? originItemList.length - 1 : 0;

    if (product) {
      if (!originItemList || originItemList.length === 0) {
        handleRemoveItem(index);
      } else {
        index++
      }
      if (product.productType == ProductType.VARIABLE) {
        setSelectProductVariationModal(true)
        setVariationsList(product.variations || [])
      } else {
        handleAddItem({
          quantity: 1,
          productPrice: product.purchasePrice!,
          productName: product.name,
          productId: product.id,
          productType: ProductType.SIMPLE,
          totalPrice: product.purchasePrice! * 1,
          comment: "",
        });
      }
    } else {
      handleRemoveItem(index);
    }
  },
    [setValue, originItemList, productsData]
  );

  const handleRemove = (productId: string) => {
    if (originItemList && originItemList.length > 1) {
      setOriginItemList((prevItems) => prevItems.filter(item => item.productId !== productId));
    } else {
      enqueueSnackbar("Vous ne pouvez pas supprimer le dernier produit", { variant: "error" });
    }
  };

  const handleToggleVariation = (variation: SupplierOrderProductVariation) => {
    setOriginItemList((prevItems) => {
      const index = prevItems.findIndex((item) => item.productId === variation.id);

      const updatedItems = index === -1
        ? [
          ...prevItems,
          {
            productName: variation.name,
            productId: variation.id,
            quantity: 1,
            productPrice: variation.purchasePrice,
            totalPrice: variation.purchasePrice * 1,
            productType: ProductType.VARIABLE,
            comment: "",
          },
        ]
        : prevItems.filter((item) => item.productId !== variation.id);

      return updatedItems;
    });
  };

  const handleAddNewProduct = (product: CustomProduct) => {
    if (product.id) {
      const productToAdd: SupplierOrderProduct = {
        id: product.id,
        name: product.name,
        ugs: "",
        mainImagePath: product.mainImagePath,
        productType: ProductType.SIMPLE,
        regularPrice: 0,
        purchasePrice: product.productPrice,
        variations: product.variations?.map((v: CustomVariation) => ({
          id: v.id,
          name: v.name,
          ugs: "",
          mainImagePath: "",
          quantity: v.quantity,
          productId: v.id,
          regularPrice: 0,
          purchasePrice: v.productPrice,
        })) || []
      }
      handleAddProduct(productToAdd)
      setOpenAddAlert(true);
    } else {
      setOpenErrorAlert(true);
    }
  }

  const handleChangeComment = (productId: string, newComment: string) => {
    setOriginItemList((prevItems) =>
      prevItems.map((item) =>
        item.productId === productId ? { ...item, comment: newComment } : item
      )
    );
  };

  const handleChangeQuantity = (event: React.ChangeEvent<HTMLInputElement | HTMLTextAreaElement>, index: number) => {
    const newQuantity = Number(event.target.value);
    const updatedList = originItemList.map((item, i) =>
      i === index ? { ...item, quantity: newQuantity, totalPrice: item.productPrice * newQuantity } : item
    );
    setOriginItemList(updatedList);
  };

  const handleChangePrice = (event: React.ChangeEvent<HTMLInputElement | HTMLTextAreaElement>, index: number) => {
    const newUnitPrice = Number(event.target.value);
    setOriginItemList((prevItems) =>
      prevItems.map((item, i) =>
        i === index ? { ...item, totalPrice: newUnitPrice * item.quantity, productPrice: newUnitPrice } : item
      )
    );
  };

  const [openAddAlert, setOpenAddAlert] = useState(false);
  const handleCloseAddAlert = (event?: React.SyntheticEvent | Event, reason?: string) => {
    if (reason === 'clickaway') {
      return;
    }
    setOpenAddAlert(false);
  };

  const [openErrorAlert, setOpenErrorAlert] = useState(false);
  const handleCloseErrorAlert = (event?: React.SyntheticEvent | Event, reason?: string) => {
    if (reason === 'clickaway') {
      return;
    }
    setOpenErrorAlert(false);
  };

  const renderTotal = (
    <Stack
      spacing={2}
      alignItems="flex-end"
      sx={{ mt: 3, textAlign: "right", typography: "body2" }}
    >

      <Stack direction="row" sx={{ typography: "subtitle1" }}>
        <Box>Total</Box>
        <Box sx={{ width: 160 }}>{fCurrency(totalPriceSupplierOrder) || fCurrency(0)}</Box>
      </Stack>
    </Stack>
  );

  if (productLoading) return (
    <Stack alignItems="center" justifyContent="center" sx={{ py: 5 }}>
      <CircularProgress />
    </Stack>
  )

  return (
    <Box sx={{ p: 3 }}>
      <Typography variant="h6" sx={{ color: "text.disabled", mb: 3 }}>
        Détails :
      </Typography>

      <Stack
        direction={{ xs: "column", md: "row" }}
        spacing={2}
        sx={{ width: 1 }}
      >
        <Grid container spacing={2}>
          <Grid item xs={12} md={6}>
            <SupplierOrderProductsDialog
              onAdd={handleAddProduct}
              existingProducts={originItemList}
            />
          </Grid>

          <Grid item xs={12} md={6}>
            <Button
              fullWidth
              size="small"
              variant="contained"
              onClick={handleOpenAddNewProductModal}
              sx={{ flexShrink: 0 }}
            >
              Ajouter un nouveau produit
            </Button>
          </Grid>
        </Grid>
      </Stack>

      <SupplierOrderVariationsModal
        showVariationsModal={selectProductVariationModal}
        setShowVariationsModal={setSelectProductVariationModal}
        variationsChoices={variationsList}
        setVariationsChoices={setVariationsList}
        selectedItems={originItemList}
        setSelectedItems={setOriginItemList}
        handleAddSelectedVariation={handleToggleVariation}
      />

      <Modal
        className="overflow-y-scroll p-10"
        open={addNewPoductModal}
        onClose={() => setAddNewProductModal(false)}
        aria-labelledby="modal-modal-title"
        aria-describedby="modal-modal-description"
      >
        <div className="bg-white rounded-lg p-4">
          <Typography variant="h4" sx={{ mb: 3 }}>
            Ajouter un nouveau produit
          </Typography>
          <ProductNewEditForm
            onClose={() => setAddNewProductModal(false)}
            onAddProduct={handleAddNewProduct}
            defaultSupplierName={supplierName}
          />
        </div>
      </Modal>

      <Stack
        mt={3}
        divider={<Divider flexItem sx={{ borderStyle: "dashed" }} />}
        spacing={3}
      >
        {
          errors.items && (errors.items as any).message && (
            <Typography color="error" variant="caption">
              {(errors.items as any).message}
            </Typography>
          )
        }

        <Stack spacing={3}>
          {originItemList && originItemList.length > 0 && originItemList.map((product: SaveSupplierOrderProductItem, index: number) => (
            <Stack key={product.id} alignItems="flex-end">
              <Stack
                direction="row"
                spacing={2}
                alignItems="center"
                className="w-full"
              >
                {errors.items && (errors.items as any)[index] && (errors.items as any)[index].productName && (
                  <Typography color="error" variant="caption">
                    {(errors.items as any)[index]?.productName.message}
                  </Typography>
                )}
              </Stack>
              <Stack
                direction={{ xs: "column", md: "row" }}
                alignItems="center"
                className="w-full"
                spacing={0}
              >
                <div className="flex justify-center flex-col gap-2">
                  <div className="flex items-center" key={product.productId}>
                    <ListItemText
                      primary={product.productName}
                      secondary={`ID: ${product.productId}`}
                      sx={{ minWidth: "250px" }}
                    />
                    <Stack direction="row" spacing={1} flexGrow="grow">
                      <RHFTextField
                        size="small"
                        defaultValue={product.comment}
                        name={`items[${index}].comment`}
                        label="Commentaire"
                        onChange={(event) => handleChangeComment(product.productId, event.target.value)}
                        InputLabelProps={{ shrink: true }}
                      />
                      <RHFTextField
                        size="small"
                        type="number"
                        defaultValue={product.quantity}
                        name={`items[${index}].quantity`}
                        label="Quantité"
                        placeholder="0"
                        onChange={(event) => handleChangeQuantity(event, index)}
                        InputLabelProps={{ shrink: true }}
                      />

                      <RHFTextField
                        size="small"
                        type="number"
                        value={product.productPrice}
                        defaultValue={product.productPrice}
                        name={`items[${index}].productPrice`}
                        label="Prix d'achat"
                        placeholder="0.00"
                        onChange={(event) => handleChangePrice(event, index)}
                        InputProps={{
                          startAdornment: (
                            <InputAdornment position="start">
                              <Box
                                sx={{ typography: "subtitle2", color: "text.disabled" }}
                              >
                                €
                              </Box>
                            </InputAdornment>
                          ),
                        }}
                      />

                      <RHFTextField
                        disabled
                        size="small"
                        type="number"
                        defaultValue={product.totalPrice}
                        name={`items[${index}].totalPrice`}
                        label="Total"
                        value={product.quantity * product.productPrice}
                        onChange={(event: any) => handleChangePrice(event, index)}
                        InputProps={{
                          startAdornment: (
                            <InputAdornment position="start">
                              <Box
                                sx={{ typography: "subtitle2", color: "text.disabled" }}
                              >
                                €
                              </Box>
                            </InputAdornment>
                          ),
                        }}
                        sx={{
                          [`& .${inputBaseClasses.input}`]: { textAlign: { md: "right" }, },
                        }}
                      />
                      <Button
                        size="small"
                        color="error"
                        startIcon={<Iconify icon="solar:trash-bin-trash-bold" />}
                        onClick={() => {
                          handleRemove(product.productId)
                        }}
                      >
                      </Button>
                    </Stack>
                  </div>
                </div>
              </Stack>
            </Stack>
          ))}
        </Stack>
      </Stack>

      <Divider sx={{ my: 3, borderStyle: "dashed" }} />

      {renderTotal}

      <Snackbar open={openAddAlert} autoHideDuration={3500}
        onClose={handleCloseAddAlert} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
        <Alert
          onClose={handleCloseAddAlert}
          severity="success"
          variant="filled"
          sx={{ width: '100%' }}
        >
          Produit enregistré avec succés
        </Alert>
      </Snackbar>
      <Snackbar open={openErrorAlert} autoHideDuration={3000}
        onClose={handleCloseErrorAlert} anchorOrigin={{ vertical: 'top', horizontal: 'right' }}>
        <Alert
          onClose={handleCloseErrorAlert}
          severity="error"
          variant="filled"
          sx={{ width: '100%' }}
        >
          Échec de l'enregistrement du produit
        </Alert>
      </Snackbar>
    </Box>

  );
}
