import * as Yup from "yup";
import { useEffect, useMemo, useState } from "react";
import { useForm } from "react-hook-form";
import { yupResolver } from "@hookform/resolvers/yup";
import Card from "@mui/material/Card";
import Stack from "@mui/material/Stack";
import LoadingButton from "@mui/lab/LoadingButton";
import { paths } from "@/routes/paths";
import { useRouter } from "@/hooks";
import { useBoolean } from "@/hooks/use-boolean";
import { _addressBooks } from "@/shared/_mock";
import FormProvider from "@/shared/components/hook-form";
import OrderNewEditDetails from "./order-new-edit-details";
import OrderNewEditAddress from "./Order-new-edit-address";
import OrdereNewEditStatusDate from "./order-new-edit-status-date";
import OrderCreationModalMessage from "./order-creation-modal-message";
import { CollectionStatus, GapType, ReceiptStatus, SaveSupplierOrderProductItem, SaveSupplierOrderRequest, SupplierOrder, 
  SupplierOrderStatus, SupplierOrderType } from "@/contexts/types/supplierOrder";
import { ProductType } from "@/shared/types/product";
import { logger } from "@/utils/logger";
import { createSupplierOrder, updateSupplierOrder } from "@/shared/api/supplierOrders";
import { UserByEmail } from "@/shared/api/user";
import { IUser } from "@/shared/types/user";
import { enqueueSnackbar } from 'notistack';
import { isValid, parseISO } from "date-fns";

type Props = {
  currentSupplierOrder?: SupplierOrder;
  isEdit?: boolean;
};

export default function SupplierOrderNewEditForm({ currentSupplierOrder }: Props) {
  const router = useRouter();
  const loadingSend = useBoolean();
  const [isEdit, setIsEdit] = useState<boolean>(false);
  const [driverId, setDriverId] = useState<string>("");
  const [supplierId, setSupplierId] = useState<string>("");
  const [itemsList, setItemsList] = useState<SaveSupplierOrderProductItem[]>([]);
  const [userId, setUserId] = useState<string>("");
  const [supplierName, setSupplierName] = useState<string>("");

  const fetchUserData = async () => {
    const clientEmail = localStorage.getItem("email");
    const response = await UserByEmail(clientEmail!);
    const userData: IUser = response.data;
    if (userData)
      setUserId(userData.id)
  }

  useEffect(() => {
    fetchUserData();
  }, []);

  useEffect(() => {
    if (currentSupplierOrder?.id) {
      setIsEdit(true);
      const createdAt = currentSupplierOrder.createdAt ? parseISO(currentSupplierOrder.createdAt.toString()) : new Date();
      const deliveryDate = currentSupplierOrder.deliveryDate ? parseISO(currentSupplierOrder.deliveryDate.toString()) : new Date();

      if (createdAt && isValid(createdAt)) {
        setValue("createdAt", createdAt);
      }
      if (deliveryDate && isValid(deliveryDate)) {
        setValue("deliveryDate", deliveryDate);
      }
      setValue("supplierOrderStatus", currentSupplierOrder.supplierOrderStatus!);
      console.log("currentSupplierOrder to edit", currentSupplierOrder);
    }
  }, [currentSupplierOrder]);

  const NewSupplierOrderSchema = Yup.object().shape({
    createdAt: Yup.date()
      .required("Date de création requise")
      .default(() => new Date())
      .max(
        Yup.ref("deliveryDate"),
        "La date de création doit être antérieure à la date de livraison"
      ),
    deliveryDate: Yup.date()
      .min(Yup.ref("createdAt"), "La date de livraison doit être postérieure à la date de création")
      .default(() => new Date()),
    supplierOrderStatus: Yup.mixed<SupplierOrderStatus>().oneOf(Object.values(SupplierOrderStatus)).required(),
    supplierOrderType: Yup.mixed<SupplierOrderType>().oneOf(Object.values(SupplierOrderType)).required(),
    supplierOrderItems: Yup.array().of(
      Yup.object().shape({
        quantity: Yup.number()
          .min(1, "La quantité doit être au moins 1")
          .required("Quantité requise"),
      })
    ),
  });

  const defaultValues = useMemo(() => ({
    createdAt: currentSupplierOrder?.createdAt ?? new Date(),
    deliveryDate: currentSupplierOrder?.deliveryDate ?? new Date(),
    supplierOrderStatus: currentSupplierOrder?.supplierOrderStatus ?? SupplierOrderStatus.TO_BE_COLLECTED,
    supplierOrderType: currentSupplierOrder?.supplierOrderType ?? SupplierOrderType.PURCHASES,
  }), [currentSupplierOrder]);

  const methods = useForm({
    resolver: yupResolver(NewSupplierOrderSchema),
    defaultValues
  });

  const {
    reset,
    setValue,
    handleSubmit,
    formState: { isSubmitting },
    control
  } = methods;

  function transformItemsToSaveFormat(items: any[]): SaveSupplierOrderProductItem[] {
    return items.map((item) => ({
      id: item.id || '',
      productId: item.productId || '',
      productName: item.productName || '',
      productType: item.productType || ProductType.SIMPLE,
      productPrice: item.productPrice || 0,
      quantity: item.quantity || 0,
      totalPrice: item.totalPrice || 0,
      comment: item.comment || '',
      receiptStatus: item.receiptStatus as ReceiptStatus || ReceiptStatus.PRODUCT_WILL_BE_COLLECTED_LATER,
      gapType: item.gapType || GapType.NORMAL,
      gap: item.gap || 0,
    }));
  }

  useEffect(() => {
    if (itemsList && itemsList.length > 0) {
      setItemsList(itemsList);
    }
  }, [itemsList, setItemsList]);

  function validateFormattedData(formattedData: any): boolean {
    const errors: string[] = [];

    if (!formattedData.supplierId) {
      errors.push("ID du fournisseur est requis.");
    }
    if (!formattedData.driverId) {
      errors.push("ID du chauffeur est requis.");
    }
    if (!formattedData.supplierOrderItems || formattedData.supplierOrderItems.length === 0) {
      errors.push("Les éléments de commande sont requis.");
    } else {
      formattedData.supplierOrderItems.forEach((item: any, index: number) => {
        if (!item.productId) {
          errors.push(`Product ID est requis pour l'élément ${index + 1}.`);
        }
        if (!item.productPrice || item.productPrice <= 0) {
          errors.push(`Prix du produit est requis pour l'élément ${index + 1}.`);
        }
        if (!item.productType) {
          errors.push(`Type de produit est requis pour l'élément ${index + 1}.`);
        }
        if (item.quantity == null || item.quantity <= 0) {
          errors.push(`Quantité est requise pour l'élément ${index + 1}.`);
        }
      });
    }

    if (errors.length > 0) {
      errors.forEach(error => {
        enqueueSnackbar(error, { variant: "error" });
      });
      return false;
    }

    return true;
  }

  const handleSubmitForm = handleSubmit(async (data) => {
    try {
      loadingSend.onTrue();
      const formattedData: SaveSupplierOrderRequest = {
        supplierId: supplierId || '',
        driverId: driverId || '',
        createdAt: new Date(new Date(data.createdAt).getTime() - new Date(data.createdAt).getTimezoneOffset() * 60000),
        deliveryDate: new Date(new Date(data.deliveryDate).getTime() - new Date(data.deliveryDate).getTimezoneOffset() * 60000),
        supplierOrderStatus: data.supplierOrderStatus || SupplierOrderStatus.TO_BE_COLLECTED,
        supplierOrderType: data.supplierOrderType || SupplierOrderType.PURCHASES,
        supplierOrderItems: itemsList.map((item: SaveSupplierOrderProductItem) => ({
          id: item.id || '',
          productId: item.productId || '',
          productName: item.productName || '',
          productType: item.productType || ProductType.SIMPLE,
          productPrice: item.productPrice || 0,
          quantity: item.quantity || 0,
          totalPrice: item.totalPrice || 0,
          comment: item.comment || '',
          receiptStatus: item.receiptStatus as ReceiptStatus || ReceiptStatus.PRODUCT_WILL_BE_COLLECTED_LATER,
          collectionStatus: item.collectionStatus as CollectionStatus || CollectionStatus.PRODUCT_WILL_BE_COLLECTED_LATER,
          gapType: item.gapType || GapType.NORMAL,
          gap: item.gap || 0,
        })),
      };

      if (!validateFormattedData(formattedData)) {
        enqueueSnackbar("Échec dans la création de la commande d'achat!", { variant: "error" });
        return;
      }

      if (isEdit) {
        formattedData.id = currentSupplierOrder?.id!;
        await updateSupplierOrder(formattedData).then(() => {
          enqueueSnackbar("Commande d'achat modifiée avec succès!", { variant: "success" });
          router.push(paths.dashboard.admin.supplierOrder.root);
        })
          .catch(() => {
            enqueueSnackbar("Échec de la modification de la commande d'achat!", { variant: "error" });
          });
        logger("updateSupplierOrder", formattedData);
      } else {
        await createSupplierOrder(userId, formattedData).then(() => {
          enqueueSnackbar("Commande d'achat réussie!", { variant: "success" });
          router.push(paths.dashboard.admin.supplierOrder.root);
        })
          .catch(() => {
            enqueueSnackbar("Échec de la création de la commande d'achat!", { variant: "error" });
          });
        logger("createSupplierOrder", formattedData);
      }
    } catch (error) {
      enqueueSnackbar("Échec dans l'enregistrement de la commande d'achat!", { variant: "error" });
    } finally {
      loadingSend.onFalse();
    }
  });


  return (
    <>
      <FormProvider methods={methods}>
        <Card>
          <OrderNewEditAddress
            isEdit={isEdit}
            supplierId={currentSupplierOrder?.supplierId}
            setSupplierId={setSupplierId}
            supplierName={currentSupplierOrder?.supplierName}
            setSupplierName={setSupplierName}
            driverId={currentSupplierOrder?.driverId}
            setDriverId={setDriverId}
          />
          <OrdereNewEditStatusDate />
          <OrderNewEditDetails
            itemsList={transformItemsToSaveFormat(currentSupplierOrder?.supplierOrderItems || [])}
            setItemsList={setItemsList}
            supplierName={supplierName}
          />
        </Card>

        <Stack
          justifyContent="flex-end"
          direction="row"
          spacing={2}
          sx={{ mt: 3 }}
        >
          <LoadingButton
            size="large"
            variant="contained"
            loading={loadingSend.value && methods.formState.isSubmitting}
            onClick={handleSubmitForm}
            type="submit"
          >
            {currentSupplierOrder ? "Mettre à jour" : "Créer"}
          </LoadingButton>
        </Stack>
      </FormProvider>
    </>
  );
}
