"use client";

import Card from "@mui/material/Card";
import Container from "@mui/material/Container";
import TableContainer from "@mui/material/TableContainer";
import Scrollbar from "@/components/scrollbar";
import Table from "@mui/material/Table";
import { TableHeadCustom } from "@/shared/components/table";
import { useSettingsContext } from "@/shared/components/settings";
import { paths } from "@/routes/paths";
import Label from "@/shared/components/label";
import CustomBreadcrumbs from "@/shared/components/custom-breadcrumbs";
import WhatsAppIcon from "@mui/icons-material/WhatsApp";
import { useEffect, useState } from "react";
import {
  Avatar, Divider, FormControl, Grid, InputLabel, ListItemText, MenuItem, Select, Stack, TableBody, TableCell, TableRow,
  Tooltip, Typography, Alert
} from "@mui/material";
import { _invoices } from "@/shared/_mock";
import { fDate, fTime } from "@/utils/format-time";
import { useResponsive } from "@/hooks/use-responsive";
import { fCurrency } from "@/utils/format-number";
import CampaignIcon from "@mui/icons-material/Campaign";
import SupplierOrderGapModal from "./supplier-order-gap-modal";
import {
  CollectionStatus, DiscrepancyDeclaration, GapType, getCollecteStatusColor, ReceiptStatus, receiptStatusLabels, SupplierDTO, SupplierOrder,
  SupplierOrderItem, SupplierOrderStatus, supplierOrderStatusLabels
} from "@/contexts/types/supplierOrder";
import { getSupplierOrderById, saveDiscrepancyDeclaration, updateSupplierOrderItemReceiptStatus } from "@/shared/api/supplierOrders";
import { getDriverById } from "@/shared/api/driver";
import { getSupplierById } from "@/shared/api/supplierOrders";
import { enqueueSnackbar, SnackbarProvider } from "notistack";
import { UserByEmail } from "@/shared/api/user";
import { IUser } from "@/shared/types/user";
import { LoadingScreen } from "@/shared/components/loading-screen";


const TABLE_HEAD = [
  { id: "service", label: "Produit", alignRight: false },
  { id: "quantity", label: "Quantité", alignRight: false },
  { id: "price", label: "Prix", alignRight: false },
  { id: "total", label: "Total", alignRight: false },
  { id: "collecteStatus", label: "Statut de collecte", alignRight: false },
  { id: "ecart", label: "Ecart", alignRight: false },
  { id: "statut", label: "Statut de réception", alignRight: false },
];

type DriverItem = {
  id: string;
  name: string;
  email: string;
};

type Props = {
  supplierOrderId: string;
};

export default function SupplierOrderDetails({ supplierOrderId }: Props) {
  const settings = useSettingsContext();
  const lgUp = useResponsive("up", "lg");
  const mdUp = useResponsive("up", "md");

  const [supplierOrder, setSupplierOrder] = useState<SupplierOrder>();
  const [driver, setDriver] = useState<DriverItem>();
  const [supplier, setSupplier] = useState<SupplierDTO>();

  const { supplierOrderData, supplierOrderLoading, supplierOrderError, mutateSupplierOrder } = getSupplierOrderById(supplierOrderId);

  const [gapDeclaration, setGapDeclaration] = useState(false);
  const [declarationItem, setDeclarationItem] = useState<SupplierOrderItem>();

  const [userId, setUserId] = useState<string>("");

  const fetchUserData = async () => {
    const clientEmail = localStorage.getItem("email");
    const response = await UserByEmail(clientEmail!);
    const userData: IUser = response.data;
    if (userData) setUserId(userData.id);
  };

  useEffect(() => {
    fetchUserData();
  }, []);

  useEffect(() => {
    const loadDriverAndSupplier = async () => {
      if (!supplierOrderData) return;

      try {
        const driverData = await getDriverById(supplierOrderData.driverId!);
        if (driverData) {
          setDriver({
            id: String(driverData.id),
            name: `${driverData.firstName} ${driverData.lastName}`,
            email: driverData.email,
          });
        }

        const supplierData = await getSupplierById(
          Number(supplierOrderData.supplierId!)
        );
        if (supplierData) {
          setSupplier({
            id: supplierData.id,
            companyName: supplierData.companyName,
            companyEmail: supplierData.companyEmail,
            companyPhoneNumber: supplierData.companyPhoneNumber,
            companyStreet: supplierData.companyStreet,
            companyCity: supplierData.companyCity,
            companyCountry: supplierData.companyCountry,
            siretNumber: supplierData.siretNumber,
            supplierName: supplierData.supplierName,
            supplierLastName: supplierData.supplierLastName,
            supplierPhoneNumber: supplierData.supplierPhoneNumber,
            supplierEmail: supplierData.supplierEmail,
          });
        }
      } catch (error) {
        enqueueSnackbar("Failed to fetch driver or supplier", {
          variant: "error",
        });
      }
    };

    loadDriverAndSupplier();
  }, [supplierOrderData]);

  useEffect(() => {
    const enrichSupplierOrder = () => {
      if (supplierOrderData) {
        const enrichedOrder: SupplierOrder = {
          ...supplierOrderData,
          supplierName:
            supplier?.supplierName + " " + supplier?.supplierLastName ||
            "Fournisseur non trouvé",
          driverName: driver?.name || "Chauffeur non trouvé",
          supplierPhoneNumber: supplier?.supplierPhoneNumber,
        };
        setSupplierOrder(enrichedOrder);
      }
    };
    enrichSupplierOrder();
  }, [supplierOrderData, supplier, driver]);

  const toggleGapDeclaration = (item: SupplierOrderItem) => {
    setDeclarationItem(item);
    setGapDeclaration(true);
  };

  const onHandleSaveDeclaration = (
    gapType: GapType,
    gapValue: number,
    deliveryDate: string,
    supplierOrderItemId: string
  ) => {
    const formData: DiscrepancyDeclaration = {
      userId: userId,
      supplierOrderId: supplierOrderId,
      supplierOrderItemId: supplierOrderItemId,
      gapType: gapType,
      declaredGap: gapValue,
      deliveryDate: deliveryDate,
    };
    saveDiscrepancyDeclaration(formData)
      .then((response) => {
        enqueueSnackbar("Ecart déclaré avec succès", { variant: "success" });
        setGapDeclaration(false);
      })
      .catch((error: any) => {
        enqueueSnackbar(error.message, { variant: "error" });
      });
  };

  const onHandleSaveReceiptStatus = (
    itemId: string,
    receiptStatus: ReceiptStatus
  ) => {
    updateSupplierOrderItemReceiptStatus(receiptStatus, itemId)
      .then(() => {
        mutateSupplierOrder();
        enqueueSnackbar(
          `Statut de réception mis à jour avec succès pour le produit ${itemId}`,
          { variant: "success" }
        );
      })
      .catch(() => {
        enqueueSnackbar("Problème lors de la mise à jour du statut de réception", { variant: "error" });
      })
  };

  if (supplierOrderLoading) {
    return (
      <Grid container spacing={3} sx={{ justifyContent: 'center', alignItems: 'center', height: '100vh' }}>
        <LoadingScreen />
      </Grid>
    );
  }

  if (supplierOrderError) {
    return (
      <Grid container spacing={3} sx={{ justifyContent: 'center', alignItems: 'center', height: '100vh' }}>
        <Alert severity="error">Problème lors de la récupération des données de la commande d'achat. Veuillez réessayer plus tard.</Alert>
      </Grid>
    );
  }

  return (
    <SnackbarProvider>
      <Container maxWidth={settings.themeStretch ? false : "lg"}>
        <CustomBreadcrumbs
          heading={`Détail Commande d'achat: ${supplierOrderId}`}
          links={[
            { name: "Tableau de bord", href: paths.dashboard.root },
            {
              name: "Commandes fournisseurs",
              href: paths.dashboard.admin.supplierOrder.root,
            },
            { name: "Détail Commande" },
          ]}
          sx={{
            mb: { xs: 3, md: 5 },
          }}
        />

        <Card className="p-3">
          <Grid
            component="div"
            spacing={{ xs: 4 }}
            container
            p={0}
            py={2}
            justifyContent={"space-evenly"}
          >
            <Grid
              item
              lg={2}
              md={5}
              sm={12}
              container
              justifyContent={mdUp ? "center" : "start"}
              alignItems="center"
            >
              <p>
                <Typography
                  variant="h6"
                  sx={{ color: "text.disabled", flexGrow: 1 }}
                >
                  Date de création
                </Typography>
                <span>{fDate(supplierOrder?.createdAt)}</span> -{" "}
                <span>{fTime(supplierOrder?.createdAt)}</span>
              </p>
            </Grid>

            <Divider
              orientation={mdUp ? "vertical" : "horizontal"}
              flexItem
              style={{ borderStyle: "dashed", margin: "16px 0" }}
            />

            <Grid
              item
              lg={2}
              md={5}
              sm={12}
              container
              justifyContent={mdUp ? "center" : "start"}
              alignItems="center"
            >
              <p>
                <Typography
                  variant="h6"
                  sx={{ color: "text.disabled", flexGrow: 1 }}
                >
                  Date de livraison
                </Typography>
                <span>{fDate(supplierOrder?.deliveryDate)}</span> -{" "}
                <span>{fTime(supplierOrder?.deliveryDate)}</span>
              </p>
            </Grid>

            <Divider
              orientation={mdUp ? "vertical" : "horizontal"}
              hidden={mdUp && !lgUp}
              flexItem
              style={{ borderStyle: "dashed", margin: "16px 0" }}
            />

            <Grid
              item
              lg={2}
              md={3}
              sm={12}
              container
              justifyContent={mdUp ? "center" : "start"}
              alignItems="center"
            >
              <p>
                <Typography
                  variant="h6"
                  sx={{ color: "text.disabled", flexGrow: 1 }}
                >
                  Montant
                </Typography>
                <span>{fCurrency(supplierOrder?.totalPrice || 0)}</span>
              </p>
            </Grid>
            <Divider
              orientation={mdUp ? "vertical" : "horizontal"}
              flexItem
              style={{ borderStyle: "dashed", margin: "16px 0" }}
            />

            <Grid
              item
              lg={2}
              md={3}
              sm={12}
              container
              justifyContent={mdUp ? "center" : "start"}
              alignItems="center"
            >
              <p>
                <Typography
                  variant="h6"
                  sx={{ color: "text.disabled", flexGrow: 1 }}
                >
                  Statut
                </Typography>
                <Label
                  variant="soft"
                  color={
                    (supplierOrder?.supplierOrderStatus ===
                      SupplierOrderStatus.RECEIVED &&
                      "success") ||
                    (supplierOrder?.supplierOrderStatus ===
                      SupplierOrderStatus.DELIVERED &&
                      "warning") ||
                    (supplierOrder?.supplierOrderStatus ===
                      SupplierOrderStatus.TO_BE_COLLECTED &&
                      "info") ||
                    "default"
                  }
                >
                  {supplierOrder?.supplierOrderStatus
                    ? supplierOrderStatusLabels[
                    supplierOrder.supplierOrderStatus
                    ]
                    : "Statut inconnu"}
                </Label>
              </p>
            </Grid>

            <Divider
              orientation={mdUp ? "vertical" : "horizontal"}
              flexItem
              style={{ borderStyle: "dashed", margin: "16px 0" }}
            />

            <Grid
              item
              lg={2}
              md={3}
              sm={12}
              container
              justifyContent={mdUp ? "center" : "start"}
              alignItems="center"
            >
              <p>
                <Typography
                  variant="h6"
                  sx={{ color: "text.disabled", flexGrow: 1 }}
                >
                  Nombre de produits
                </Typography>
                <span>{supplierOrder?.supplierOrderItems.length}</span>
              </p>
            </Grid>

            <Divider
              orientation="horizontal"
              flexItem
              style={{ borderStyle: "dashed", margin: "16px 0" }}
            />

            <Grid
              item
              md={5}
              sm={12}
              container
              justifyContent={mdUp ? "center" : "start"}
              alignItems="center"
            >
              <p>
                <Typography
                  variant="h6"
                  sx={{ color: "text.disabled", flexGrow: 1 }}
                >
                  Fournisseur
                </Typography>
                <div className="flex items-center">
                  <Avatar
                    alt={supplierOrder?.supplierName}
                    sx={{ mr: 1, width: 25, height: 25 }}
                  >
                    {supplierOrder?.supplierName!.charAt(0).toUpperCase()}
                  </Avatar>
                  <ListItemText
                    disableTypography
                    primary={
                      <Typography variant="body2" noWrap>
                        {supplierOrder?.supplierName}
                      </Typography>
                    }
                  />
                </div>
                {supplierOrder?.supplierPhoneNumber && (
                  <div
                    className="flex items-center cursor-pointer hover:bg-gray-100 rounded-lg"
                    style={{ fontSize: "1.2rem" }}
                    onClick={() =>
                      window.open(
                        `https://wa.me/${supplierOrder.supplierPhoneNumber}`,
                        "_blank"
                      )
                    }
                  >
                    <WhatsAppIcon
                      style={{ color: "#25d366", fontSize: "1.7rem" }}
                      aria-label="WhatsApp Icon"
                    />
                    <ListItemText
                      disableTypography
                      primary={
                        <Typography
                          variant="body2"
                          noWrap
                          style={{ marginLeft: "8px" }}
                        >
                          Contacter par WhatsApp
                        </Typography>
                      }
                    />
                  </div>
                )}
              </p>
            </Grid>

            <Divider
              orientation={mdUp ? "vertical" : "horizontal"}
              flexItem
              style={{ borderStyle: "dashed", margin: "16px 0" }}
            />

            <Grid
              item
              md={5}
              sm={12}
              container
              justifyContent={mdUp ? "center" : "start"}
              alignItems="center"
            >
              <p>
                <Typography
                  variant="h6"
                  sx={{ color: "text.disabled", flexGrow: 1 }}
                >
                  Chauffeur
                </Typography>
                <div className="flex items-center">
                  <Avatar
                    alt={supplierOrder?.driverName!.charAt(0).toUpperCase()}
                    sx={{ mr: 1, width: 25, height: 25 }}
                  >
                    {driver?.email.charAt(0).toUpperCase()}
                  </Avatar>
                  <ListItemText
                    disableTypography
                    primary={
                      <Typography variant="body2" noWrap>
                        {supplierOrder?.driverName}
                      </Typography>
                    }
                  />
                </div>
              </p>
            </Grid>
          </Grid>
        </Card>

        <Card sx={{ mt: 2 }}>
          <TableContainer sx={{ position: "relative", overflow: "unset" }}>
            <Scrollbar>
              <Table sx={{ minWidth: 800 }}>
                <TableHeadCustom headLabel={TABLE_HEAD} />

                <TableBody>
                  {supplierOrder?.supplierOrderItems
                    .sort((a, b) => String(a.id).localeCompare(String(b.id)))
                    .map(
                      (item: SupplierOrderItem, index: number) => (
                        <TableRow>
                          <TableCell>
                            <ListItemText primary={`${item.productName}`} />
                          </TableCell>
                          <TableCell>x {item.quantity}</TableCell>
                          <TableCell>{fCurrency(item.productPrice)}</TableCell>
                          <TableCell>{fCurrency(item.totalPrice)}</TableCell>
                          <TableCell>
                            {supplierOrder.supplierOrderStatus ===
                              SupplierOrderStatus.DELIVERED ||
                              supplierOrder.supplierOrderStatus ===
                              SupplierOrderStatus.RECEIVED ? (
                              item.collectionStatus ? (
                                <Label
                                  variant="soft"
                                  color={getCollecteStatusColor(item.collectionStatus)}
                                >
                                  {receiptStatusLabels[item.collectionStatus] ||
                                    "Statut inconnu"}
                                </Label>
                              ) : (
                                <></>
                              )
                            ) : null}
                          </TableCell>
                          <TableCell>
                            <Tooltip title="Déclarer un écart">
                              <CampaignIcon
                                sx={{ color: "error.main", cursor: "pointer" }}
                                onClick={() => toggleGapDeclaration(item)}
                              />
                            </Tooltip>
                          </TableCell>
                          <TableCell>
                            <FormControl
                              fullWidth
                              variant="outlined"
                              sx={{ mb: 2 }}
                            >
                              <InputLabel>Status de réception</InputLabel>
                              <Stack
                                direction="row"
                                spacing={1}
                                alignItems="center"
                              >
                                <Select
                                  defaultValue={item.receiptStatus}
                                  value={item.receiptStatus}
                                  onChange={(event) => {
                                    const newStatus = event.target
                                      .value as ReceiptStatus;
                                    onHandleSaveReceiptStatus(item.id, newStatus);
                                  }}
                                  label="Status de réception"
                                  sx={{ flexGrow: 1 }}
                                >
                                  {Object.values(ReceiptStatus).map((status) => (
                                    <MenuItem key={status} value={status}>
                                      {receiptStatusLabels[status]}
                                    </MenuItem>
                                  ))}
                                </Select>
                              </Stack>
                            </FormControl>
                          </TableCell>
                        </TableRow>
                      )
                    )}
                </TableBody>
              </Table>
            </Scrollbar>
          </TableContainer>
        </Card>

        {supplierOrder?.comment ? (
          <Card sx={{ p: 3, mt: 2 }}>
            <p>
              <Typography
                variant="h6"
                sx={{ color: "text.disabled", flexGrow: 1 }}
              >
                Commentaire du chauffeur
              </Typography>
              <span>{supplierOrder.comment}</span>
            </p>
          </Card>
        ) : (
          <Card sx={{ p: 3, mt: 2 }}>
            <p>
              <Typography
                variant="h6"
                sx={{ color: "text.disabled", flexGrow: 1 }}
              >
                Aucun commentaire du chauffeur n'est disponible pour le moment.
              </Typography>
            </p>
          </Card>
        )}

        {gapDeclaration && (
          <SupplierOrderGapModal
            open={gapDeclaration}
            onClose={() => setGapDeclaration(false)}
            supplierOrderItem={declarationItem!}
            onHandleSaveDeclaration={onHandleSaveDeclaration}
          />
        )}
      </Container>
    </SnackbarProvider>
  );
}

