import { GapType, SupplierOrderItem } from "@/contexts/types/supplierOrder";
import { formatDateForLocalDateTime } from "@/shared/api/saleOrders";
import { Box, Button, MenuItem, Modal, Select, SelectChangeEvent, Stack, TextField, Typography } from "@mui/material";
import { DateTimePicker } from "@mui/x-date-pickers";
import { enqueueSnackbar } from "notistack";
import { useEffect, useState } from "react";


type CustomModalProps = {
    open: boolean;
    onClose: () => void;
    supplierOrderItem: SupplierOrderItem;
    onHandleSaveDeclaration: (gapType: GapType, gapValue: number, deliveryDate: string, supplierOrderItemId: string) => void;
}

export default function SupplierOrderGapModal({ open, onClose, supplierOrderItem, onHandleSaveDeclaration }: CustomModalProps) {

    const [gapType, setGapType] = useState<GapType>(GapType.NORMAL);
    const [gapValue, setGapValue] = useState<number>(0);
    const [deliveryDate, setDeliveryDate] = useState<Date>(new Date());

    useEffect(() => {
        setGapType(supplierOrderItem.gapType || GapType.NORMAL);
        setGapValue(supplierOrderItem.gap || 0);
    }, [supplierOrderItem]);

    const handleGapChange = (event: SelectChangeEvent<string>) => {
        setGapType(event.target.value as GapType);
    };

    const handleGapDetailsChange = (event: React.ChangeEvent<HTMLInputElement>) => {
        setGapValue(Number(event.target.value));
    };

    const handleDateChange = (event: Date | null) => {
        setDeliveryDate(event || new Date());
    };

    const handleSaveDeclaration = () => {
        const today = new Date();
        today.setHours(0, 0, 0, 0);

        let deliveryDateNormalized = new Date(deliveryDate);
        deliveryDateNormalized.setHours(0, 0, 0, 0);

        if (deliveryDateNormalized < today) {
            enqueueSnackbar("Choisissez une date de livraison future", { variant: "error" });
            return;
        }

        if (deliveryDateNormalized.getTime() === today.getTime()) {
            const deliveryTime = new Date(deliveryDate);
            if (deliveryTime.getTime() < today.getTime()) {
                deliveryTime.setMinutes(deliveryTime.getMinutes() + 5);
                setDeliveryDate(deliveryTime);
            }
        }

        const formattedDeliveryDate = formatDateForLocalDateTime(deliveryDate);

        if (gapType === GapType.LOWER) {
            if (gapValue > 0 && gapValue < supplierOrderItem.quantity) {
                onHandleSaveDeclaration(gapType, gapValue, formattedDeliveryDate, supplierOrderItem.id);
            } else {
                enqueueSnackbar("Veuillez entrer une valeur valide pour l'écart inférieur", { variant: "error" });
            }
        } else if (gapType === GapType.NORMAL || gapType === GapType.UPPER) {
            onHandleSaveDeclaration(gapType, 0, formattedDeliveryDate, supplierOrderItem.id);
        }

        onClose();
    };


    return (
        <Modal
            className="flex items-center justify-center"
            open={open}
            onClose={onClose}
            aria-labelledby="modal-title"
            aria-describedby="modal-description"
        >
            <div className="bg-white rounded-lg shadow p-6 flex flex-col justify-center items-center">
                <Typography variant="body2" id="modal-description" sx={{ mb: 2, fontSize: 20 }}>
                    Déclaration des écarts
                </Typography>
                <span>{supplierOrderItem.productId}</span> <span className="font-bold">{supplierOrderItem.productName}</span>
                <span className="font-bold">{supplierOrderItem.productUgs}</span>
                <Box sx={{ width: 300, mb: 2 }}>
                    <Select
                        value={gapType}
                        onChange={handleGapChange}
                        fullWidth
                    >
                        <MenuItem value={GapType.UPPER}>Quantité supérieure</MenuItem>
                        <MenuItem value={GapType.NORMAL}>Quantité normale</MenuItem>
                        <MenuItem value={GapType.LOWER}>Quantité inférieure</MenuItem>
                    </Select>
                </Box>
                {gapType === GapType.LOWER && (
                    <>
                        <Box sx={{ width: 300, mb: 2 }}>
                            <DateTimePicker
                                label="Date de livraison"
                                value={deliveryDate}
                                onChange={handleDateChange}
                                format="yyyy/MM/dd HH:mm:ss"
                                sx={{ width: '100%' }}
                                slotProps={{
                                    textField: {
                                        fullWidth: true,
                                    },
                                }}
                            />
                        </Box>
                        <Box sx={{ width: 300, mb: 2 }}>
                            <TextField
                                label="Détails de l'écart"
                                value={gapValue}
                                onChange={handleGapDetailsChange}
                                fullWidth
                            />
                        </Box>
                    </>
                )}
                <Stack direction="row" spacing={2}>
                    <Button variant="contained" color="error" onClick={onClose}>Annuler</Button>
                    <Button variant="contained" color="info" onClick={handleSaveDeclaration}>Enregistrer</Button>
                </Stack>
            </div>
        </Modal >
    )
}
