"use client";

import { useState, useCallback, useEffect } from "react";
import Tab from "@mui/material/Tab";
import Tabs from "@mui/material/Tabs";
import Card from "@mui/material/Card";
import Table from "@mui/material/Table";
import Stack from "@mui/material/Stack";
import Button from "@mui/material/Button";
import Tooltip from "@mui/material/Tooltip";
import Container from "@mui/material/Container";
import TableBody from "@mui/material/TableBody";
import IconButton from "@mui/material/IconButton";
import { alpha, useTheme } from "@mui/material/styles";
import TableContainer from "@mui/material/TableContainer";
import { paths } from "@/routes/paths";
import { useRouter } from "@/hooks";
import RouterLink from "@/routes/router-link";
import { useBoolean } from "@/hooks/use-boolean";
import { isAfter } from "@/utils/format-time";
import Label from "@/shared/components/label";
import Iconify from "@/components/iconify";
import Scrollbar from "@/components/scrollbar";
import { useSnackbar } from "@/components/snackbar";
import { ConfirmDialog } from "@/shared/components/custom-dialog";
import { useSettingsContext } from "@/components/settings";
import CustomBreadcrumbs from "@/components/custom-breadcrumbs";
import {
  useTable,
  emptyRows,
  TableNoData,
  TableEmptyRows,
  TableHeadCustom,
  TableSelectedAction,
  TablePaginationCustom,
} from "@/shared/components/table";
import { Alert, CircularProgress, Grid, TableCell, TableRow } from '@mui/material';
import {
  ISupplierOrderDetails,
  ISupplierOrderTableFilters, ISupplierOrderTableFilterValue, SupplierDTO,SupplierOrder, SupplierOrderItem,
  SupplierOrderStatus, supplierOrderStatusLabels
} from '@/contexts/types/supplierOrder';
import SupplierOrderTableFiltersResult from './supplier-order-table-filters-result';
import SupplierOrderTableToolbar from './supplier-order-table-toolbar';
import SupplierOrderTableRow from './supplier-order-table-row';
import { deleteMultipleSupplierOrders, deleteSupplierOrder, exportSuppliers, getAllDrivers, getAllSupplierOrders,useSearchSupplierOrder, useSupplierOrderCountsByStatus } from '@/shared/api/supplierOrders';
import { useGetAllSuppliers } from '@/shared/api/supplier';
import { Driver } from '@/contexts/types/driver';
import { LoadingScreen } from '@/shared/components/loading-screen';


const TABLE_HEAD = [
  { id: "id", label: "N°" },
  { id: "supplierName", label: "Fournisseur", },
  { id: "driverName", label: "Chauffeur" },
  { id: "createdAt", label: "Création" },
  { id: "deliveryDate", label: "Date livraison", align: "center" },
  { id: "totalPrice", label: "Montant" },
  { id: "totalItemsQuantity", label: "Nombre d'articles", align: "center" },
  { id: "supplierOrderStatus", label: "Statut" },
  { id: "supplierOrderType", label: "Type" },
  { id: "" },
];
const TABS = [
  { value: "", label: "Tous" },
  { value: "TO_BE_COLLECTED", label: "À collecter" },
  { value: "DELIVERED", label: "Livré" },
  { value: "RECEIVED", label: "Réceptionné" },
];

const defaultFilters: ISupplierOrderTableFilters = {
  driverEmail: "",
  supplierEmail: "",
  status: "",
  startDate: null,
  endDate: null,
  product: "",
};

export default function SupplierOrderListView() {
  const { enqueueSnackbar } = useSnackbar();

  const theme = useTheme();

  const settings = useSettingsContext();

  const router = useRouter();

  const table = useTable({ defaultOrderBy: "id", defaultOrder: "desc" });

  const confirm = useBoolean();

  const [tableData, setTableData] = useState<ISupplierOrderDetails[]>([]);

  const [filters, setFilters] = useState(defaultFilters);
  const { supplierOrderCounts, supplierOrderCountsError } = useSupplierOrderCountsByStatus(filters);
  const [statusCounts, setStatusCounts] = useState<{ [key: string]: number }>({
    TO_BE_COLLECTED: 0,
    DELIVERED: 0,
    RECEIVED: 0,
  });

  const {
    supplierOrders,
    totalSupplierOrders,
    totalPages,
    pageNumber,
    isError,
    isValidating,
    loading,
    pageSize,
    revalidate,
  } = useSearchSupplierOrder({
    driverEmail: filters.driverEmail,
    status: filters.status,
    supplierEmail: filters.supplierEmail,
    startDate: filters.startDate,
    endDate: filters.endDate,
    product: filters.product,
    page: table.page,
    size: table.rowsPerPage
  });

  const dateError = isAfter(filters.startDate as Date, filters.endDate as Date);

  useEffect(() => {
    if (supplierOrders && supplierOrders.length > 0) {
      setTableData(supplierOrders);
    }
  }, [supplierOrders, table.page, table.rowsPerPage]);


  useEffect(() => {
    if (
      filters.driverEmail ||
      filters.supplierEmail ||
      filters.status ||
      filters.product ||
      filters.startDate ||
      filters.endDate
    ) {
      revalidate();
    }
  }, [filters, revalidate]);

  const dataInPage = tableData.slice(
    table.page * table.rowsPerPage,
    table.page * table.rowsPerPage + table.rowsPerPage
  );


  useEffect(() => {
    if (supplierOrderCountsError) {
      console.error("Error fetching status counts:", supplierOrderCountsError);
      return;
    }
    if (Array.isArray(supplierOrderCounts)) {
      const statusObj = supplierOrderCounts.reduce<{ [key: string]: number }>(
        (acc, [status, count]) => {
          acc[status] = count;
          return acc;
        },
        {}
      );
      setStatusCounts(statusObj);
    } else {
      console.error(
        "Expected counts to be an array, but received:",
        supplierOrderCounts
      );
    }
  }, [filters, supplierOrderCounts, supplierOrderCountsError]);
  const denseHeight = table.dense ? 56 : 56 + 20;

  const canReset =
    !!filters.driverEmail ||
    !!filters.product ||
    !!filters.supplierEmail ||
    filters.status !== "" ||
    (!!filters.startDate && !!filters.endDate);

  const notFound = totalSupplierOrders === 0 && canReset;

  const handleFilters = useCallback(
    (name: string, value: ISupplierOrderTableFilterValue) => {
      table.onResetPage();
      setFilters((prevState) => ({
        ...prevState,
        [name]: value,
      }));
    },
    [table]
  );

  const handleResetFilters = useCallback(() => {
    setFilters(defaultFilters);
  }, []);

  const handleDeleteRow = useCallback(
    async (id: string) => {

      const response = await deleteSupplierOrder(id);
      if (response) {
        setTableData(tableData.filter((row) => row.supplierOrderDTO.id.toString() !== id));
        table.onUpdatePageDeleteRows({
          totalRowsInPage: dataInPage.length,
          totalRowsFiltered: totalSupplierOrders,
        });
        enqueueSnackbar("Supprimé avec succès", { variant: "success" });

      }
    },
    [dataInPage.length, enqueueSnackbar, table, tableData]
  );

  const handleDeleteRows = useCallback(async () => {
    const ids = table.selected;
    const response = await deleteMultipleSupplierOrders(ids);
    if (response) {
      const deleteRows = tableData.filter(
        (row) => !table.selected.includes(row.supplierOrderDTO.id.toString())
      );
      enqueueSnackbar("Supprimé avec succès", { variant: "success" });
      setTableData(deleteRows);
      table.onUpdatePageDeleteRows({
        totalRowsInPage: dataInPage.length,
        totalRowsFiltered: totalSupplierOrders,
      });
    }
  }, [
    totalSupplierOrders,
    dataInPage.length,
    enqueueSnackbar,
    table,
    tableData,
  ]);

  const handleEditRow = useCallback(
    (id: string) => {
      router.push(paths.dashboard.admin.supplierOrder.edit(id));
    },
    [router]
  );

  const handleExport = () => {
    exportSuppliers(filters.startDate!, filters.endDate!, filters.supplierEmail!, filters.driverEmail!, filters.product!,filters.status!);
  }

  if (loading) {
    return (
      <Grid container spacing={3} sx={{ justifyContent: 'center', alignItems: 'center', height: '100vh' }}>
        <LoadingScreen />
      </Grid>
    );
  }

  if (isError) {
    return (
      <Grid container spacing={3} sx={{ justifyContent: 'center', alignItems: 'center', height: '100vh' }}>
        <Alert severity="error">Problème lors de la récupération des données des commandes d'achat. Veuillez réessayer plus tard.</Alert>
      </Grid>
    );
  }

  return (
    <>
      <Container maxWidth={settings.themeStretch ? false : "lg"}>
        <CustomBreadcrumbs
          heading="Liste des commandes d'achat"
          links={[
            {
              name: "Tableau de bord",
              href: paths.dashboard.root,
            },
            {
              name: "Liste des commandes",
              href: paths.dashboard.admin.supplierOrder.root,
            }
          ]}
          action={
            <div className="w-full flex md:flex-row flex-col gap-5">
              <Button
                component={RouterLink}
                href={paths.dashboard.admin.supplierOrder.new}
                variant="contained"
                startIcon={<Iconify icon="mingcute:add-line" />}
              >
                Nouvelle Commande
              </Button>
                <Button
                  onClick={handleExport}
                  variant="contained"
                  startIcon={<Iconify icon="mingcute:add-line" />}
                >
                  Export
                </Button>
            </div>
          }
          sx={{
            mb: { xs: 3, md: 5 },
          }}
        />

        <Card>
        <Tabs
              value={filters.status}
              onChange={(event, newValue) => handleFilters("status", newValue)}
              sx={{
                px: 2.5,
                boxShadow: (theme) =>
                  `inset 0 -2px 0 0 ${alpha(theme.palette.grey[500], 0.08)}`,
              }}
            >
        {TABS.map((tab) => {
                const count =
                  statusCounts[tab.value as keyof typeof statusCounts] || 0;
                return (
                  <Tab
                    key={tab.value}
                    iconPosition="end"
                    value={tab.value}
                    label={tab.label}
                    icon={
                      <Label
                        variant={
                          ((tab.value === "" ||
                            tab.value === filters.status) &&
                            "filled") ||
                          "soft"
                        }
                        color={
                          (tab.value === "DELIVERED" && "success") ||
                          (tab.value === "TO_BE_COLLECTED" && "info") ||
                          (tab.value === "RECEIVED" && "warning") ||
                          "default"
                        }
                      >
                        {tab.value === ""
                          ? Object.values(statusCounts).reduce(
                              (acc, count) => acc + count,
                              0
                            )
                          : count}
                      </Label>
                    }
                  />
                );
              })}
            </Tabs>

          <SupplierOrderTableToolbar
            filters={filters}
            onFilters={handleFilters}
            dateError={dateError}
          />

          {canReset && (
            <SupplierOrderTableFiltersResult
              filters={filters}
              onFilters={handleFilters}
              //
              onResetFilters={handleResetFilters}
              //
              results={totalSupplierOrders}
              sx={{ p: 2.5, pt: 0 }}
            />
          )}

          <TableContainer sx={{ position: "relative", overflow: "unset" }}>
            <TableSelectedAction
              dense={table.dense}
              numSelected={table.selected.length}
              rowCount={totalSupplierOrders}
              onSelectAllRows={(checked) => {
                table.onSelectAllRows(
                  checked,
                  tableData.map((row) => row.supplierOrderDTO.id.toString())
                );
              }}
              action={
                <Stack direction="row">
                  <Tooltip title="Delete">
                    <IconButton color="primary" onClick={confirm.onTrue}>
                      <Iconify icon="solar:trash-bin-trash-bold" />
                    </IconButton>
                  </Tooltip>
                </Stack>
              }
            />
            {
              loading ? (
                <Stack alignItems="center" justifyContent="center" sx={{ py: 5 }}>
                  <CircularProgress />
                </Stack>
              ) : (
                <Scrollbar>
                  <Table
                    size={table.dense ? "small" : "medium"}
                    sx={{ minWidth: 800 }}
                  >
                    <TableHeadCustom
                      order={table.order}
                      orderBy={table.orderBy}
                      headLabel={TABLE_HEAD}
                      rowCount={totalSupplierOrders}
                      numSelected={table.selected.length}
                      onSort={table.onSort}
                      onSelectAllRows={(checked) =>
                        table.onSelectAllRows(
                          checked,
                          tableData.map((row) => row.supplierOrderDTO.id.toString())
                        )
                      }
                    />

                <TableBody>
                  {loading ? (
                    <TableRow>
                      <TableCell
                        colSpan={TABLE_HEAD.length}
                        style={{ textAlign: "center", padding: "20px" }}
                      >
                        <CircularProgress />
                      </TableCell>
                    </TableRow>
                  ) : (
                    <>
                      {notFound ? (
                        <TableNoData notFound={notFound} />
                      ) : (
                      tableData
                        .map((row) => (
                          <SupplierOrderTableRow
                            key={row.supplierOrderDTO.id}
                            row={row}
                            selected={table.selected.includes(row.supplierOrderDTO.id.toString())}
                            onSelectRow={() => table.onSelectRow(row.supplierOrderDTO.id.toString())}
                            onViewRow={() => { }}
                            onEditRow={() => handleEditRow(row.supplierOrderDTO.id.toString())}
                            onDeleteRow={() => handleDeleteRow(row.supplierOrderDTO.id.toString())}
                            onMutateSupplierOrders={revalidate}
                            />
                          ))
                        )}
  

                     
                      <TableEmptyRows
                        height={denseHeight}
                        emptyRows={emptyRows(
                          table.page,
                          table.rowsPerPage,
                          totalSupplierOrders
                        )}
                      />
                    </>
                  )}
                    </TableBody>
                  </Table>
                </Scrollbar>
              )
            }

          </TableContainer>

          <TablePaginationCustom
            count={totalSupplierOrders}
            page={pageNumber}
            rowsPerPage={pageSize}
            onPageChange={table.onChangePage}
            onRowsPerPageChange={table.onChangeRowsPerPage}
            dense={table.dense}
            onChangeDense={table.onChangeDense}
          />
        </Card>
      </Container>

      <ConfirmDialog
        open={confirm.value}
        onClose={confirm.onFalse}
        title="Supprimer"
        content={
          <>
            Êtes-vous sûr de vouloir supprimer{" "}
            <strong> {table.selected.length} </strong> éléments?
          </>
        }
        action={
          <Button
            variant="contained"
            color="error"
            onClick={() => {
              handleDeleteRows();
              confirm.onFalse();
            }}
          >
            Supprimer
          </Button>
        }
      />
    </>
  );
}

// ----------------------------------------------------------------------

