import { useCallback } from "react";
import Box from "@mui/material/Box";
import Chip from "@mui/material/Chip";
import Paper from "@mui/material/Paper";
import Button from "@mui/material/Button";
import Stack, { StackProps } from "@mui/material/Stack";
import Iconify from "@/shared/components/iconify";
import { shortDateLabel } from "@/shared/components/custom-date-range-picker";
import { ISupplierOrderTableFilters, ISupplierOrderTableFilterValue } from "@/contexts/types/supplierOrder";


type Props = StackProps & {
  filters: ISupplierOrderTableFilters;
  onFilters: (name: string, value: ISupplierOrderTableFilterValue) => void;
  onResetFilters: VoidFunction;
  results: number;
};

export default function SupplierOrderTableFiltersResult({
  filters,
  onFilters,
  //
  onResetFilters,
  //
  results,
  ...other
}: Props) {
  const shortLabel = shortDateLabel(filters.startDate as Date, filters.endDate as Date);

  const handleRemoveName = useCallback(() => {
    onFilters("supplierEmail", "");
  }, [onFilters]);

  const handleRemoveDriver = useCallback(() => {
    onFilters("driverEmail", "");
  }, [onFilters]);

  const handleRemoveStatus = useCallback(() => {
    onFilters("status", "");
  }, [onFilters]);

  const handleRemoveDate = useCallback(() => {
    onFilters("startDate", null);
    onFilters("endDate", null);
  }, [onFilters]);

  return (
    <Stack spacing={1.5} {...other}>
      <Box sx={{ typography: "body2" }}>
        <strong>{results}</strong>
        <Box component="span" sx={{ color: "text.secondary", ml: 0.25 }}>
          Résultat trouvé
        </Box>
      </Box>

      <Stack
        flexGrow={1}
        spacing={1}
        direction="row"
        flexWrap="wrap"
        alignItems="center"
      >
        {filters.status !== "" && (
          <Block label="Status:">
            <Chip
              size="small"
              label={filters.status}
              onDelete={handleRemoveStatus}
            />
          </Block>
        )}

        {filters.startDate && filters.endDate && (
          <Block label="Date:">
            <Chip size="small" label={shortLabel} onDelete={handleRemoveDate} />
          </Block>
        )}

        {!!filters.supplierEmail && (
          <Block label="Fournisseur:">
            <Chip
              label={filters.supplierEmail}
              size="small"
              onDelete={handleRemoveName}
            />
          </Block>
        )}

        {!!filters.driverEmail && (
          <Block label="Chauffeur:">
            <Chip
              label={filters.driverEmail}
              size="small"
              onDelete={handleRemoveDriver}
            />
          </Block>
        )}

        <Button
          color="error"
          onClick={onResetFilters}
          startIcon={<Iconify icon="solar:trash-bin-trash-bold" />}
        >
          Supprimer
        </Button>
      </Stack>
    </Stack>
  );
}

// ----------------------------------------------------------------------

type BlockProps = StackProps & {
  label: string;
};

function Block({ label, children, sx, ...other }: BlockProps) {
  return (
    <Stack
      component={Paper}
      variant="outlined"
      spacing={1}
      direction="row"
      sx={{
        p: 1,
        borderRadius: 1,
        overflow: "hidden",
        borderStyle: "dashed",
        ...sx,
      }}
      {...other}
    >
      <Box component="span" sx={{ typography: "subtitle2" }}>
        {label}
      </Box>

      <Stack spacing={1} direction="row" flexWrap="wrap">
        {children}
      </Stack>
    </Stack>
  );
}
