import Button from "@mui/material/Button";
import Avatar from "@mui/material/Avatar";
import Divider from "@mui/material/Divider";
import MenuItem from "@mui/material/MenuItem";
import TableRow from "@mui/material/TableRow";
import Checkbox from "@mui/material/Checkbox";
import TableCell from "@mui/material/TableCell";
import IconButton from "@mui/material/IconButton";
import Typography from "@mui/material/Typography";
import ListItemText from "@mui/material/ListItemText";
import Paper from "@mui/material/Paper";
import { useBoolean } from "@/hooks/use-boolean";
import { useEffect, useState } from "react";
import { fCurrency } from "@/utils/format-number";
import { fDate, fTime } from "@/utils/format-time";
import Label from "@/shared/components/label";
import Iconify from "@/components/iconify";
import { ConfirmDialog } from "@/shared/components/custom-dialog";
import CustomPopover, { usePopover } from "@/shared/components/custom-popover";
import { Box, Collapse, Stack } from "@mui/material";
import OrderEditStatusModal from "./supplier-order-edit-status-modal";
import { useRouter } from "@/hooks";
import { getSupplierOrderStatusColor, getSupplierOrderTypeColor,SupplierOrderStatus, supplierOrderTypeLabels, 
  supplierOrderStatusLabels, UpdateSupplierOrderStatus, 
  ISupplierOrderDetails} from "@/contexts/types/supplierOrder";
import { paths } from '@/routes/paths';
import { downloadSupplierOrderZebraFile, updateSupplierOrderStatus } from "@/shared/api/supplierOrders";
import { IUser } from "@/shared/types/user";
import { UserByEmail } from "@/shared/api/user";


type Props = {
  row: ISupplierOrderDetails;
  selected: boolean;
  onSelectRow: VoidFunction;
  onViewRow: VoidFunction;
  onEditRow: VoidFunction;
  onDeleteRow: VoidFunction;
  onMutateSupplierOrders: () => void;
};

export default function SupplierOrderTableRow({
  row,
  selected,
  onSelectRow,
  onViewRow,
  onEditRow,
  onDeleteRow,
  onMutateSupplierOrders,
}: Props) {
  const { supplierOrderDTO , supplierDTO ,driverDTO } = row;

  const confirm = useBoolean();
  const popover = usePopover();
  const router = useRouter();

  const [showDetails, setShowDetails] = useState(false);
  const [openEditStatusModal, setOpenEditStatusModal] = useState(false);
  const [rowStatus, setRowStatus] = useState(row.supplierOrderDTO.supplierOrderStatus);

  const totalItemsQuantity = supplierOrderDTO.supplierOrderItems.reduce((acc, item) => acc + item.quantity, 0);

  const [userId, setUserId] = useState<string>("");

  const fetchUserData = async () => {
    const clientEmail = localStorage.getItem("email");
    const response = await UserByEmail(clientEmail!);
    const userData: IUser = response.data;
    if (userData)
      setUserId(userData.id)
  }

  useEffect(() => {
    fetchUserData();
  }, []);

  const toggleDetails = () => {
    setShowDetails(!showDetails);
  }

  const openEditModalStatus = (() => {
    setOpenEditStatusModal(true);
  })

  const handleShowCommandDetails = (() => {
    router.push(paths.dashboard.admin.supplierOrder.details(row.supplierOrderDTO.id));
  })

  const handleEditSupplierOrder = ((orderId: string) => {
    router.push(paths.dashboard.admin.supplierOrder.edit(orderId));
  });

  const handleDownloadOrderZebraFile = async (id: string) => {
    try {
      await downloadSupplierOrderZebraFile(id);
    } catch (err) {
      console.error('Error downloading file:', err);
    }
  };

  const onUpdateSupplierOrderStatus = async () => {
    const formData: UpdateSupplierOrderStatus = {
      userId: userId,
      supplierOrderId: supplierOrderDTO.id,
      supplierOrderStatus: rowStatus
    };
    await updateSupplierOrderStatus(formData);
    onMutateSupplierOrders();
  };


  return (
    <>
      <TableRow
        hover
        selected={selected}
        onClick={toggleDetails}
        className="cursor-pointer"
        sx={{
          backgroundColor: row.supplierOrderDTO.zebraFileGenerated === true ? "#efdef4" : "inherit",
        }}
      >
        <TableCell padding="checkbox" onClick={(event) => event.stopPropagation()}>
          <Checkbox checked={selected} onClick={onSelectRow} />
        </TableCell>

        <TableCell>{supplierOrderDTO.id}</TableCell>

        <TableCell>
          <div className="flex items-center">
            <Avatar
              alt={supplierDTO?.supplierName}
              sx={{ mr: 1, width: 25, height: 25 }}
            >
              {(supplierDTO?.supplierName?.charAt(0) || "F").toUpperCase()}
            </Avatar>
            <Typography variant="body2" noWrap>
              {supplierDTO?.supplierName || "Fournisseur non trouvé"}
            </Typography>
          </div>
        </TableCell>

        <TableCell>
          <div className="flex items-center">
            <Avatar
              alt={driverDTO?.name}
              sx={{ mr: 1, width: 25, height: 25 }}
            >
              {(driverDTO?.name?.charAt(0) || "F").toUpperCase()}
            </Avatar>
            <Typography variant="body2" noWrap>
              {driverDTO?.name || "Chauffeur non trouvé"}
            </Typography>
          </div>
        </TableCell>

        <TableCell>
          <ListItemText
            primary={fDate(supplierOrderDTO.createdAt)}
            secondary={fTime(supplierOrderDTO.createdAt)}
            primaryTypographyProps={{ typography: "body2", noWrap: true }}
            secondaryTypographyProps={{
              mt: 0.5,
              component: "span",
              typography: "caption",
            }}
          />
        </TableCell>

        <TableCell>
          <ListItemText
            primary={fDate(supplierOrderDTO.deliveryDate)}
            secondary={fTime(supplierOrderDTO.deliveryDate)}
            primaryTypographyProps={{ typography: "body2", noWrap: true }}
            secondaryTypographyProps={{
              mt: 0.5,
              component: "span",
              typography: "caption",
            }}
          />
        </TableCell>

        <TableCell>{fCurrency(supplierOrderDTO.totalPrice)}</TableCell>

        <TableCell align="center">{totalItemsQuantity}</TableCell>

        <TableCell>
          <Label
            variant="soft"
            color={getSupplierOrderStatusColor(supplierOrderDTO.supplierOrderStatus)}
          >
            {supplierOrderStatusLabels[supplierOrderDTO.supplierOrderStatus]}
          </Label>
        </TableCell>

        <TableCell>
          <Label
            variant="soft"
            color={getSupplierOrderTypeColor(supplierOrderDTO.supplierOrderType)}
          >
            {supplierOrderTypeLabels[supplierOrderDTO.supplierOrderType]}
          </Label>
        </TableCell>

        <TableCell align="right" sx={{ px: 1 }}>
          <IconButton
            color={popover.open ? "inherit" : "default"}
            onClick={(e) => {
              popover.onOpen(e);
              e.stopPropagation();
            }}
          >
            <Iconify icon="eva:more-vertical-fill" />
          </IconButton>
        </TableCell>
      </TableRow>

      <CustomPopover
        open={popover.open}
        onClose={popover.onClose}
        arrow="right-top"
        sx={{ width: 160 }}
      >
        <MenuItem
          onClick={() => {
            handleShowCommandDetails();
          }}
        >
          <Iconify icon="solar:eye-bold" />
          Voir
        </MenuItem>

        <MenuItem
          onClick={() => {
            handleEditSupplierOrder(supplierOrderDTO.id);
          }}
          disabled={row.supplierOrderDTO.supplierOrderStatus === SupplierOrderStatus.DELIVERED || row.supplierOrderDTO.supplierOrderStatus === SupplierOrderStatus.RECEIVED}
        >
          <Iconify icon="solar:pen-bold" />
          Modifier
        </MenuItem>

        <MenuItem
          onClick={() => { handleDownloadOrderZebraFile(supplierOrderDTO.id); }}
        >
          <Iconify icon="solar:download-bold" />
          Fichier de Zebra
        </MenuItem>
        <Divider sx={{ borderStyle: "dashed" }} />

        <MenuItem
          onClick={() => {
            confirm.onTrue();
            popover.onClose();
          }}
          sx={{ color: "error.main" }}
        >
          <Iconify icon="solar:trash-bin-trash-bold" />
          Supprimer
        </MenuItem>
      </CustomPopover>

      <ConfirmDialog
        open={confirm.value}
        onClose={confirm.onFalse}
        title="Supprimer"
        content="Êtes-vous sûr de vouloir supprimer cet élément?"
        action={
          <Button variant="contained" color="error" onClick={onDeleteRow}>
            Supprimer
          </Button>
        }
      />
      {
        showDetails && (
          <TableRow>
            <TableCell sx={{ border: 'none' }} colSpan={10}>
              <Collapse in={showDetails} timeout="auto" unmountOnExit sx={{ bgcolor: 'background.neutral' }}>
                <Stack direction="row" alignItems="center" sx={{ p: 1.5, m: 1.5, bgcolor: 'background.neutral' }}>
                  <Typography variant="subtitle2" sx={{ flex: 1 }}>
                    Produits commandés
                  </Typography>
                  <IconButton size="small" onClick={toggleDetails}>
                    <Iconify icon="eva:arrow-ios-upward-fill" />
                  </IconButton>
                </Stack>
                <Stack component={Paper} sx={{ m: 1.5 }}>
                  {(row as ISupplierOrderDetails).supplierOrderDTO.supplierOrderItems.map((item) => (
                    <Stack
                      key={item.productName}
                      direction="row"
                      alignItems="center"
                      sx={{
                        p: (theme) => theme.spacing(1.5, 2, 1.5, 1.5),
                        '&:not(:last-of-type)': {
                          borderBottom: (theme) => `solid 2px ${theme.palette.background.neutral}`,
                        },
                      }}
                    >
                      <ListItemText
                        primary={item.productName}
                        primaryTypographyProps={{ typography: 'body2' }}
                        secondaryTypographyProps={{
                          component: 'span',
                          color: 'text.disabled',
                          mt: 0.5,
                        }}
                      />
                      <Box sx={{ width: 110, textAlign: 'right' }}>
                        {fCurrency(item.productPrice)}
                      </Box>
                      <Box sx={{ width: 110, textAlign: 'right' }}>
                        x {item.quantity}
                      </Box>
                      <Box sx={{ width: 110, textAlign: 'right' }}>
                        {fCurrency(item.totalPrice)}
                      </Box>
                    </Stack>
                  ))}
                </Stack>
                <Stack>
                  <Divider />
                  <Stack direction="row" alignItems="center" sx={{ p: 1.5, m: 1.5, bgcolor: 'background.neutral' }}>
                    <Button
                      variant="contained"
                      color="info"
                      size="small"
                      onClick={() => openEditModalStatus()}>
                      Changer le statut de la commande
                    </Button>
                  </Stack>
                </Stack>
              </Collapse>
            </TableCell>
          </TableRow>
        )
      }
      <OrderEditStatusModal
        open={openEditStatusModal}
        onClose={() => setOpenEditStatusModal(false)}
        message="Modifier le statut de la commande"
        rowStatus={rowStatus}
        setRowStatus={setRowStatus}
        onUpdateSupplierOrderStatus={onUpdateSupplierOrderStatus}
      />
    </>
  );
}
