import React, { useCallback, useState } from "react";
import { useForm } from "react-hook-form";
import { yupResolver } from "@hookform/resolvers/yup";
import * as Yup from "yup";
import Box from "@mui/material/Box";
import Card from "@mui/material/Card";
import Stack from "@mui/material/Stack";
import Grid from "@mui/material/Grid";
import Typography from "@mui/material/Typography";
import LoadingButton from "@mui/lab/LoadingButton";
import { useRouter } from "../../../../hooks/use-router";
import { fData } from "@/utils/format-number";
import { useSnackbar } from "@/components/snackbar";
import FormProvider, {
  RHFTextField,
  RHFUploadAvatar,
  RHFSelect,
} from "../../../components/hook-form";
import {
  InputAdornment,
  IconButton,
  MenuItem,
  FormControlLabel,
  Switch,
  Snackbar,
  Alert,
} from "@mui/material";
import { Add, Refresh, Visibility, VisibilityOff } from "@mui/icons-material";

import { v4 as uuidv4 } from "uuid";
import { saveUser, useGetRoles } from "@/shared/api/user";
import { IRole } from "@/shared/types/user";
type Props = {
  onSuccess?: () => void;
  isDialog?: boolean;
  isDriver?: boolean;
};

type UserFormValues = {
  pseudo: string;
  firstName: string;
  lastName: string;
  phoneNumber: string;
  roles: string;
  email: string;
  password: string;
  imageUrl?: string | null;
  confirmPassword: string;
};

const NewUserSchema = Yup.object().shape({
  roles: Yup.string().required("Le rôle est requis"),
  email: Yup.string()
    .required("L'email est requis")
    .email("L'email doit être une adresse email valide")
    .matches(
      /^[a-z0-9._%+-]+@[a-z0-9.-]+.[a-z]{2,}$/,
      "L'email doit être en minuscule et valide, par exemple: utilisateur@example.com"
    )
    .test(
      "is-lowercase",
      "L'email doit être en minuscule",
      (value) => value === value?.toLowerCase()
    ),
  pseudo: Yup.string().required("Le pseudo est requis"),
  firstName: Yup.string().required("Le prénom est requis"),
  lastName: Yup.string().required("Le nom est requis"),
  phoneNumber: Yup.string()
    .required("Le numéro de téléphone est requis")
    .min(10, "Le numéro de téléphone doit comporter au moins 10 chiffres")
    .max(15, "Le numéro de téléphone doit comporter au maximum 15 chiffres"),
  imageUrl: Yup.string().nullable(),
  password: Yup.string()
    .required("Le mot de passe est requis")
    .min(8, "Le mot de passe doit comporter au moins 8 caractères")
    .matches(
      /[A-Z]/,
      "Le mot de passe doit comporter au moins une lettre majuscule"
    )
    .matches(
      /[a-z]/,
      "Le mot de passe doit comporter au moins une lettre minuscule"
    )
    .matches(/[0-9]/, "Le mot de passe doit comporter au moins un chiffre")
    .matches(
      /[!@#$%^&*(),.?":{}|<>]/,
      "Le mot de passe doit comporter au moins un caractère spécial"
    ),
  confirmPassword: Yup.string()
    .oneOf([Yup.ref("password")], "Les mots de passe doivent correspondre")
    .required("La confirmation du mot de passe est requise"),
});

export type SaveUserItemRequest = {
  pseudo: string;
  firstName: string | null;
  lastName: string | null;
  phoneNumber: string;
  roles: IRole | null;
  email: string;
  password: string;
  status: string | null;
};

export default function UserNewEditForm({
  onSuccess,
  isDialog,
  isDriver,
}: Props) {
  const router = useRouter();
  const { enqueueSnackbar } = useSnackbar();
  const [showPassword, setShowPassword] = useState(false);
  const [showConfirmPassword, setShowConfirmPassword] = useState(false);
  const [isVerified, setIsVerified] = useState(false);
  const [imageFile, setImageFile] = useState<File | null>(null);
  const [snackbarOpen, setSnackbarOpen] = useState(false);
  const [snackbarMessage, setSnackbarMessage] = useState("");
  const [snackbarSeverity, setSnackbarSeverity] = useState<"success" | "error">(
    "success"
  );

  const { roleData } = useGetRoles();
  const handleTogglePasswordVisibility = () => setShowPassword((prev) => !prev);
  const handleToggleConfirmPasswordVisibility = () =>
    setShowConfirmPassword((prev) => !prev);

  const defaultValues: UserFormValues = {
    pseudo: "",
    firstName: "",
    lastName: "",
    phoneNumber: "",
    roles: isDriver ? "DRIVER" : "",
    email: "",
    password: "",
    imageUrl: null,
    confirmPassword: "",
  };

  const methods = useForm<UserFormValues>({
    defaultValues,
    resolver: yupResolver(NewUserSchema),
  });

  const {
    reset,
    watch,
    setValue,
    handleSubmit,
    formState: { isSubmitting },
  } = methods;

  const onSubmit = handleSubmit(async (data: UserFormValues) => {
    try {
      const userItem: SaveUserItemRequest = {
        firstName: data.firstName,
        lastName: data.lastName,
        pseudo: data.pseudo,
        email: data.email,
        password: data.password,
        phoneNumber: data.phoneNumber,
        roles: isDriver
          ? { id: null, name: "DRIVER" } // Force le rôle DRIVER si isDriver est true
          : { id: null, name: data.roles },
        status: "APPROVED",
      };
      const userdtoJson = JSON.stringify(userItem);
      let renamedFile;
      if (imageFile) {
        const uniqueFileName = `${uuidv4()}-${imageFile?.name}`;
        renamedFile = new File([imageFile], uniqueFileName, {
          type: imageFile?.type,
        });
        await saveUser(userdtoJson, renamedFile);
      } else {
        await saveUser(userdtoJson);
      }
      enqueueSnackbar("Création réussie!", { variant: "success" });
      setSnackbarMessage("Création réussie!");
      setSnackbarSeverity("success");
      setSnackbarOpen(true);
      reset();
      if (isDialog) {
        onSuccess?.();
      }
    } catch (error: any) {
      enqueueSnackbar("Échec de la création!", { variant: "error" });
      setSnackbarMessage(error.message);
      setSnackbarSeverity("error");
      setSnackbarOpen(true);
    }
  });

  const handleDrop = useCallback(
    (acceptedFiles: File[]) => {
      const file = acceptedFiles[0];
      setImageFile(file);
      const newFile = Object.assign(file, {
        preview: URL.createObjectURL(file),
      });

      if (file) {
        setValue("imageUrl", newFile.preview, { shouldValidate: true });
      }
    },
    [setValue]
  );

  const handleVerifiedChange = (event: React.ChangeEvent<HTMLInputElement>) => {
    setIsVerified(event.target.checked);
  };

  const handleSnackbarClose = (
    event?: React.SyntheticEvent | Event,
    reason?: string
  ) => {
    if (reason === "clickaway") {
      return;
    }
    setSnackbarOpen(false);
  };

  return (
    <FormProvider methods={methods} onSubmit={onSubmit}>
      <Grid container spacing={3}>
        <Grid item xs={12} md={4}>
          <Card sx={{ pt: 10, pb: 5, px: 3 }}>
            <Box sx={{ mb: 5 }}>
              <RHFUploadAvatar
                name="imageUrl"
                maxSize={3145728}
                onDrop={handleDrop}
                helperText={
                  <Typography
                    variant="caption"
                    sx={{
                      mt: 3,
                      mx: "auto",
                      display: "block",
                      textAlign: "center",
                      color: "text.disabled",
                    }}
                  >
                    Formats autorisés: *.jpeg, *.jpg, *.png
                    <br /> taille max {fData(3145728)}
                  </Typography>
                }
              />
            </Box>
          </Card>
        </Grid>

        <Grid item xs={12} md={8}>
          <Card>
            <Stack spacing={3} sx={{ p: 3 }}>
              <Typography variant="h6">Informations utilisateur</Typography>

              <RHFTextField name="firstName" label="Nom" />
              <RHFTextField name="lastName" label="Prénom" />
              <RHFTextField name="pseudo" label="Pseudo" />
              <RHFTextField name="phoneNumber" label="Numéro de téléphone" />
              <RHFTextField
                name="email"
                label="Email"
                onChange={(e) => {
                  const lowercaseEmail = e.target.value.toLowerCase();
                  setValue("email", lowercaseEmail); // Update the value in the form state
                }}
              />
              <RHFTextField
                name="password"
                label="Mot de passe"
                type={showPassword ? "text" : "password"}
                InputProps={{
                  endAdornment: (
                    <InputAdornment position="end">
                      <IconButton onClick={handleTogglePasswordVisibility}>
                        {showPassword ? <VisibilityOff /> : <Visibility />}
                      </IconButton>
                    </InputAdornment>
                  ),
                }}
              />
              <RHFTextField
                name="confirmPassword"
                label="Confirmation du mot de passe"
                type={showConfirmPassword ? "text" : "password"}
                InputProps={{
                  endAdornment: (
                    <InputAdornment position="end">
                      <IconButton
                        onClick={handleToggleConfirmPasswordVisibility}
                      >
                        {showConfirmPassword ? (
                          <VisibilityOff />
                        ) : (
                          <Visibility />
                        )}
                      </IconButton>
                    </InputAdornment>
                  ),
                }}
              />
              {!isDriver && (
                <RHFSelect
                  name="roles"
                  label="Rôles"
                  placeholder="Sélectionnez un rôle"
                  onChange={(event) => {
                    setValue("roles", event.target.value, {
                      shouldValidate: true,
                    });
                  }}
                >
                  {roleData &&
                    roleData.map((role) => (
                      <MenuItem key={role.id} value={role.name}>
                        {role.value}
                      </MenuItem>
                    ))}
                </RHFSelect>
              )}

              <Stack spacing={3} sx={{ mt: 5 }}>
                <LoadingButton
                  fullWidth
                  size="large"
                  type="submit"
                  variant="contained"
                  loading={isSubmitting}
                >
                  Enregistrer
                </LoadingButton>
              </Stack>
            </Stack>
          </Card>
        </Grid>
      </Grid>
      <Snackbar
        open={snackbarOpen}
        autoHideDuration={6000}
        onClose={handleSnackbarClose}
        anchorOrigin={{ vertical: "top", horizontal: "right" }}
      >
        <Alert
          onClose={handleSnackbarClose}
          severity={snackbarSeverity}
          sx={{ width: "100%" }}
        >
          {snackbarMessage}
        </Alert>
      </Snackbar>
    </FormProvider>
  );
}
