import { useState, useEffect, useMemo } from "react";
import * as Yup from "yup";
import { useForm } from "react-hook-form";
import { yupResolver } from "@hookform/resolvers/yup";

import Box from "@mui/material/Box";
import Button from "@mui/material/Button";
import Dialog from "@mui/material/Dialog";
import MenuItem from "@mui/material/MenuItem";
import LoadingButton from "@mui/lab/LoadingButton";
import DialogTitle from "@mui/material/DialogTitle";
import DialogActions from "@mui/material/DialogActions";
import DialogContent from "@mui/material/DialogContent";

import { useSnackbar } from "@/components/snackbar";
import FormProvider, { RHFSelect, RHFTextField } from "@/components/hook-form";

import { IUser, IRole } from "../../../types/user";
import { updateUser, useGetRoles } from "@/shared/api/user";
import { USER_STATUS_OPTIONS } from "@/shared/_mock";

// ----------------------------------------------------------------------

type Props = {
  open: boolean;
  onClose: VoidFunction;
  currentUser?: IUser;
  onUpdateUser: (updatedUser: IUser) => void;
};

export default function UserQuickEditForm({
  currentUser,
  open,
  onClose,
  onUpdateUser,
}: Props) {
  const { enqueueSnackbar } = useSnackbar();

  const [roleData, setRoleData] = useState<IRole[]>([]);

  const NewUserSchema = Yup.object().shape({
    lastName: Yup.string().required("Name is required"),
    firstName: Yup.string().required("Prénom is required"),
    email: Yup.string()
      .required("Email is required")
      .email("Email must be a valid email address"),
    pseudo: Yup.string().required("Pseudo is required"),
    status: Yup.string().nullable().required("Status is required"),
    role: Yup.string().required("Role is required"), // Validating only the role name selected by the user
  });

  const defaultValues = useMemo(
    () => ({
      lastName: currentUser?.lastName || "",
      firstName: currentUser?.firstName || "",
      email: currentUser?.email || "",
      pseudo: currentUser?.pseudo || "",
      status: currentUser?.status || "",
      role: currentUser?.roles?.name || "",
    }),
    [currentUser]
  );

  const methods = useForm({
    resolver: yupResolver(NewUserSchema),
    defaultValues,
  });

  const {
    reset,
    handleSubmit,
    formState: { isSubmitting },
  } = methods;

  const { roleData: fetchedRoleData } = useGetRoles();

  useEffect(() => {
    if (fetchedRoleData) {
      setRoleData(fetchedRoleData);
    }
  }, [fetchedRoleData]);

  const onSubmit = handleSubmit(async (data) => {
    try {
      const selectedRole = roleData.find((role) => role.name === data.role);

      const uid = currentUser?.uid || 0;
      const userdto: IUser = {
        id: currentUser?.id || "",
        email: data.email,
        pseudo: data.pseudo,
        uid,
        password: currentUser?.password || "",
        phoneNumber: currentUser?.phoneNumber || "",
        firstName: data.firstName,
        lastName: data.lastName || "",
        imageUrl: currentUser?.imageUrl || "",
        roles: selectedRole || { id: null, name: data.role },
        status: data.status,
        activityLog: currentUser?.activityLog || false,
      };

      const result = await updateUser(JSON.stringify(userdto));
      if (result.status === 200) {
        enqueueSnackbar("Update success!", { variant: "success" });
        onUpdateUser(userdto);
        setTimeout(() => {
          onClose();
        }, 1000);
      }
    } catch (error) {
      console.error(error);
      enqueueSnackbar("Update failed!", { variant: "error" });
    }
  });

  return (
    <Dialog
      fullWidth
      maxWidth={false}
      open={open}
      onClose={onClose}
      PaperProps={{
        sx: { maxWidth: 720 },
      }}
    >
      <FormProvider methods={methods} onSubmit={onSubmit}>
        <DialogTitle>Édition rapide</DialogTitle>

        <DialogContent>
          <Box
            rowGap={3}
            columnGap={3}
            display="grid"
            gridTemplateColumns={{
              xs: "repeat(1, 1fr)",
              sm: "repeat(2, 1fr)",
            }}
            sx={{ mt: 2 }}
          >
            <RHFSelect name="status" label="Statut" disabled={defaultValues.role !== "CLIENT"}>
              {USER_STATUS_OPTIONS.map((status) => (
                <MenuItem key={status.value} value={status.value}>
                  {status.label}
                </MenuItem>
              ))}
            </RHFSelect>

            <RHFTextField name="lastName" label="Nom" />
            <RHFTextField name="firstName" label="prénom" />
            <RHFTextField disabled name="email" label="Addresse Email" />
            <RHFTextField name="pseudo" label="Pseudo" />

            <RHFSelect
              name="role"
              label="Rôle"
              defaultValue={defaultValues.role}
              disabled={defaultValues.role === "CLIENT"}
            >
              {roleData
                .filter(
                  (role) =>
                    role.name !== "CLIENT" || defaultValues.role === "CLIENT"
                )
                .map((role) => (
                  <MenuItem key={role.id} value={role.name}>
                    {role.value}
                  </MenuItem>
                ))}
            </RHFSelect>
          </Box>
        </DialogContent>

        <DialogActions>
          <Button variant="outlined" onClick={onClose}>
            Annuler
          </Button>

          <LoadingButton
            type="submit"
            variant="contained"
            loading={isSubmitting}
          >
            Modifier
          </LoadingButton>
        </DialogActions>
      </FormProvider>
    </Dialog>
  );
}
