import { useCallback } from "react";

import Box from "@mui/material/Box";
import Chip from "@mui/material/Chip";
import Paper from "@mui/material/Paper";
import Button from "@mui/material/Button";
import Stack, { StackProps } from "@mui/material/Stack";

import Iconify from "../../../components/iconify";

import { IUserTableFilters, IUserTableFilterValue } from "../../../types/user";
import { translateStatus } from "@/utils/UserStatusTranslate";

// ----------------------------------------------------------------------

type Props = StackProps & {
  filters: IUserTableFilters;
  onFilters: (name: string, value: IUserTableFilterValue) => void;
  //
  onResetFilters: VoidFunction;
  //
  results: number;
};

export default function UserTableFiltersResult({
  filters,
  onFilters,
  //
  onResetFilters,
  //
  results,
  ...other
}: Props) {
  const handleRemoveKeyword = useCallback(() => {
    onFilters("keyword", "");
  }, [onFilters]);

  const handleRemovestartAndEndDate = useCallback(() => {
    onFilters("startDate", "");
    onFilters("endDate", "");
  }, [onFilters]);

  const handleRemoveStatus = useCallback(() => {
    onFilters("status", "all");
  }, [onFilters]);

  const handleRemoveRole = useCallback(
    (inputValue: string) => {
      const newValue = filters.roles.filter((item) => item !== inputValue);

      onFilters("roles", newValue);
    },
    [filters.roles, onFilters]
  );

  const handleRemoveTag = useCallback(
    (inputValue: string) => {
      const newValue = filters.tags.filter((item) => item !== inputValue);

      onFilters("tags", newValue);
    },
    [filters.tags, onFilters]
  );

  return (
    <Stack spacing={1.5} {...other}>
      <Box sx={{ typography: "body2" }}>
        <strong>{results}</strong>
        <Box component="span" sx={{ color: "text.secondary", ml: 0.25 }}>
          résultats trouvés
        </Box>
      </Box>

      <Stack
        flexGrow={1}
        spacing={1}
        direction="row"
        flexWrap="wrap"
        alignItems="center"
      >
        {filters.status !== "" && (
          <Block label="Statut :">
            <Chip
              size="small"
              label={translateStatus(filters.status)}
              onDelete={handleRemoveStatus}
            />
          </Block>
        )}

        {!!filters.roles.length && (
          <Block label="Rôle :">
            {filters.roles.map((item) => (
              <Chip
                key={item}
                label={item}
                size="small"
                onDelete={() => handleRemoveRole(item)}
              />
            ))}
          </Block>
        )}

        {!!filters.tags.length && (
          <Block label="Tag :">
            {filters.tags.map((item) => (
              <Chip
                key={item}
                label={item}
                size="small"
                onDelete={() => handleRemoveTag(item)}
              />
            ))}
          </Block>
        )}

        {!!filters.keyword && (
          <Block label="Mot-clé :">
            <Chip
              label={filters.keyword}
              size="small"
              onDelete={handleRemoveKeyword}
            />
          </Block>
        )}

        {!!filters.startDate && !!filters.endDate && (
          <Block label="Plage de dates :">
            <Chip
              label={`${new Date(
                filters.startDate
              ).toLocaleDateString()} - ${new Date(
                filters.endDate
              ).toLocaleDateString()}`}
              size="small"
              onDelete={handleRemovestartAndEndDate}
            />
          </Block>
        )}

        <Button
          color="error"
          onClick={onResetFilters}
          startIcon={<Iconify icon="solar:trash-bin-trash-bold" />}
        >
          Effacer
        </Button>
      </Stack>
    </Stack>
  );
}

// ----------------------------------------------------------------------

type BlockProps = StackProps & {
  label: string;
};

function Block({ label, children, sx, ...other }: BlockProps) {
  return (
    <Stack
      component={Paper}
      variant="outlined"
      spacing={1}
      direction="row"
      sx={{
        p: 1,
        borderRadius: 1,
        overflow: "hidden",
        borderStyle: "dashed",
        ...sx,
      }}
      {...other}
    >
      <Box component="span" sx={{ typography: "subtitle2" }}>
        {label}
      </Box>

      <Stack spacing={1} direction="row" flexWrap="wrap">
        {children}
      </Stack>
    </Stack>
  );
}
