import React, { useEffect, useState } from "react";
import Button from "@mui/material/Button";
import Tooltip from "@mui/material/Tooltip";
import MenuItem from "@mui/material/MenuItem";
import TableRow from "@mui/material/TableRow";
import Checkbox from "@mui/material/Checkbox";
import TableCell from "@mui/material/TableCell";
import IconButton from "@mui/material/IconButton";
import ListItemText from "@mui/material/ListItemText";
import Avatar from "@mui/material/Avatar";
import { useBoolean } from "@/hooks/use-boolean";
import Label from "../../../components/label";
import Iconify from "@/components/iconify";
import { ConfirmDialog } from "../../../components/custom-dialog";
import CustomPopover, { usePopover } from "../../../components/custom-popover";
import {
  IUser,
  RequestChangeUserStatus,
  RequestLoginActivity,
} from "../../../types/user";
import ExitToAppIcon from "@mui/icons-material/ExitToApp";
import UserQuickEditForm from "./user-quick-edit-form";
import { changeUserStatus, ChangingLogingActivity } from "@/shared/api/user";
import { useRouter } from "next/navigation";
import { fDate } from "@/utils/format-time";
import { translateStatus } from "@/utils/UserStatusTranslate";

type Props = {
  selected: boolean;
  onEditRow: VoidFunction;
  row: IUser;
  onSelectRow: VoidFunction;
  onDeleteRow: VoidFunction;
  onUpdateUser: (updatedUser: IUser) => void;
};

export default function UserTableRow({
  row,
  selected,
  onEditRow,
  onSelectRow,
  onDeleteRow,
  onUpdateUser,
}: Props) {
  const {
    firstName,
    phoneNumber,
    imageUrl,
    activityLog,
    status,
    email,
    pseudo,
    createdAt,
    tags,
    roles,
    uid,
    id,
  } = row;

  const router = useRouter();

  const confirm = useBoolean();
  const quickEdit = useBoolean();
  const popover = usePopover();

  const [userLoginActivity, setUserLoginActivity] = useState(activityLog);

  const handleLogout = async () => {
    const requestLoginActivity: RequestLoginActivity = {
      email: email,
      activityLog: false,
    };
    const result = await ChangingLogingActivity(requestLoginActivity);
    if (result.status == 200) {
      setUserLoginActivity(false);
      popover.onClose();
    }
  };

  const handleDeleteRow = async () => {
    try {
      const requestChangeUserStatus: RequestChangeUserStatus = {
        email: email,
        status: "DELETED",
      };
      const result = await changeUserStatus(requestChangeUserStatus);
      if (result.status === 200) {
        onDeleteRow();
      }
    } catch (error) {
      console.error(
        "Une erreur est survenue lors du changement de statut de l'utilisateur:",
        error
      );
    }
  };

  useEffect(() => {
    setUserLoginActivity(activityLog);
  }, [activityLog]);

  const handleRowClick = () => {
    router.push(`/dashboard/user/detail/${id}`);
  };

  return (
    <>
      <TableRow
        hover
        selected={selected}
        sx={{
          cursor: "pointer",
          backgroundColor:
            status === "DELETED" ? "rgba(255, 0, 0, 0.1)" : "inherit",
        }}
      >
        
        <TableCell onClick={handleRowClick} sx={{ whiteSpace: "nowrap" }}>
         [{uid}]
        </TableCell>
        <TableCell
          onClick={handleRowClick}
          sx={{ display: "flex", alignItems: "center" }}
        >
          <Avatar alt={firstName ?? ""} src={imageUrl ?? ""} sx={{ mr: 2 }} />
          <ListItemText
            primary={firstName}
            secondary={email}
            primaryTypographyProps={{ typography: "body2" }}
            secondaryTypographyProps={{
              component: "span",
              color: "text.disabled",
            }}
          />
        </TableCell>
        <TableCell onClick={handleRowClick} sx={{ whiteSpace: "nowrap" }}>
          {pseudo}
        </TableCell>
        <TableCell onClick={handleRowClick} sx={{ whiteSpace: "nowrap" }}>
          {createdAt ? fDate(createdAt) : "--/--/----"}
        </TableCell>
        <TableCell onClick={handleRowClick} sx={{ whiteSpace: "nowrap" }}>
          {phoneNumber ? phoneNumber : "--"}
        </TableCell>
        <TableCell onClick={handleRowClick} sx={{ whiteSpace: "nowrap" }}>
          {(tags ?? []).length > 0 ? (tags ?? []).join(", ") : "--"}
        </TableCell>
        <TableCell onClick={handleRowClick} sx={{ whiteSpace: "nowrap" }}>
          {roles.value}
        </TableCell>
        <TableCell onClick={handleRowClick}>
          <Label
            variant="soft"
            color={
              status === "APPROVED"
                ? "success"
                : status === "PENDING"
                ? "warning"
                : status === "BANNED"
                ? "error"
                : status === "DELETED"
                ? "default"
                : "default"
            }
          >
            {translateStatus(status)}
          </Label>
        </TableCell>
        <TableCell align="right" sx={{ px: 1, whiteSpace: "nowrap" }}>
          {status !== "DELETED" && (
            <>
              <Tooltip title="Édition rapide" placement="top" arrow>
                <IconButton
                  color={quickEdit.value ? "inherit" : "default"}
                  onClick={quickEdit.onTrue}
                >
                  <Iconify icon="solar:pen-bold" />
                </IconButton>
              </Tooltip>

              <IconButton
                color={popover.open ? "inherit" : "default"}
                onClick={popover.onOpen}
              >
                <Iconify icon="eva:more-vertical-fill" />
              </IconButton>
            </>
          )}
        </TableCell>
      </TableRow>

      <UserQuickEditForm
        currentUser={row}
        open={quickEdit.value}
        onClose={quickEdit.onFalse}
        onUpdateUser={onUpdateUser}
      />

      <CustomPopover
        open={popover.open}
        onClose={popover.onClose}
        arrow="right-top"
        sx={{ width: 180 }}
      >
        <MenuItem
          onClick={() => {
            onEditRow();
            popover.onClose();
          }}
        >
          <Iconify icon="solar:pen-bold" />
          Modifier
        </MenuItem>
        <MenuItem
          onClick={() => {
            confirm.onTrue();
            popover.onClose();
          }}
          sx={{ color: "error.main" }}
        >
          <Iconify icon="solar:trash-bin-trash-bold" />
          Supprimer
        </MenuItem>


        {userLoginActivity && (
          <MenuItem onClick={handleLogout}>
            <ExitToAppIcon />
            Déconnexion
          </MenuItem>
        )}
      </CustomPopover>

      <ConfirmDialog
        open={confirm.value}
        onClose={confirm.onFalse}
        title="Supprimer"
        content="Êtes-vous sûr de vouloir supprimer cet utilisateur ?"
        action={
          <Button
            variant="contained"
            color="error"
            onClick={() => {
              handleDeleteRow(); // This handles the deletion logic
              confirm.onFalse(); // This will close the ConfirmDialog
            }}
          >
            Supprimer
          </Button>
        }
      />
    </>
  );
}
