import { useCallback, useState } from "react";

import Stack from "@mui/material/Stack";
import MenuItem from "@mui/material/MenuItem";
import Checkbox from "@mui/material/Checkbox";
import TextField from "@mui/material/TextField";
import InputLabel from "@mui/material/InputLabel";
import FormControl from "@mui/material/FormControl";
import OutlinedInput from "@mui/material/OutlinedInput";
import InputAdornment from "@mui/material/InputAdornment";
import Select, { SelectChangeEvent } from "@mui/material/Select";

import Iconify from "../../../components/iconify";
import CustomPopover, { usePopover } from "../../../components/custom-popover";
import {
  IRole,
  IUserTableFilters,
  IUserTableFilterValue,
} from "../../../types/user";
import { Card, ListItemText } from "@mui/material";
import CustomDateRangePicker from "@/shared/components/custom-date-range-picker";
import { fDate, formatDateToString } from "@/utils/format-time";

// ----------------------------------------------------------------------

type Props = {
  filters: IUserTableFilters;
  onFilters: (name: string, value: IUserTableFilterValue) => void;
  roleOptions: IRole[];
};

export default function UserTableToolbar({
  filters,
  onFilters,
  roleOptions,
}: Props) {
  const popover = usePopover();

  const tagsList = ["vip", "pub_fb"];

  const handleFilterName = useCallback(
    (event: React.ChangeEvent<HTMLInputElement>) => {
      onFilters("keyword", event.target.value);
    },
    [onFilters]
  );

  const handleFilterRole = useCallback(
    (event: SelectChangeEvent<string[]>) => {
      onFilters(
        "roles",
        typeof event.target.value === "string"
          ? event.target.value.split(",")
          : event.target.value
      );
    },
    [onFilters]
  );

  const handleFilterTag = useCallback(
    (event: SelectChangeEvent<string[]>) => {
      const selectedTags =
        typeof event.target.value === "string"
          ? event.target.value.split(",")
          : event.target.value;
      console.log("Selected Tags:", selectedTags);
      onFilters("tags", selectedTags);
    },
    [onFilters]
  );

  const handleFilterDate = useCallback(
    (event: React.ChangeEvent<HTMLInputElement>) => {
      onFilters("date", event.target.value);
    },
    [onFilters]
  );

  const [isDateRangeOpen, setIsDateRangeOpen] = useState(false);
  const [startDate, setStartDate] = useState<Date | null>(null);
  const [endDate, setEndDate] = useState<Date | null>(null);

  const openDateRange = useCallback(() => {
    setIsDateRangeOpen(true);
  }, []);

  const closeDateRange = useCallback(() => {
    setIsDateRangeOpen(false);
  }, []);

  const onChangeStartDate = (newStartDate: Date | null) => {
    setStartDate(newStartDate);
    if (newStartDate) {
      const formattedDate = formatDateToString(newStartDate);
      onFilters("startDate", formattedDate);
    }
  };

  const onChangeEndDate = (newEndDate: Date | null) => {
    setEndDate(newEndDate);
    if (newEndDate) {
      onFilters("endDate", formatDateToString(newEndDate));
    }
  };

  return (
    <>
      <Stack
        spacing={2}
        alignItems={{ xs: "flex-end", md: "center" }}
        direction={{
          xs: "column",
          md: "row",
        }}
        sx={{
          p: 2.5,
          pr: { xs: 2.5, md: 1 },
        }}
      >
        <FormControl
          sx={{
            flexShrink: 0,
            width: { xs: 1, md: 200 },
          }}
        >
          <InputLabel>Role</InputLabel>

          <Select
            multiple
            value={filters.roles}
            onChange={handleFilterRole}
            input={<OutlinedInput label="Role" />}
            renderValue={(selected) =>
              selected.map((value) => value).join(", ")
            }
            MenuProps={{
              PaperProps: {
                sx: { maxHeight: 240 },
              },
            }}
          >
            {roleOptions.map((option) => (
              <MenuItem key={option.name} value={option.value}>
                <Checkbox
                  disableRipple
                  size="small"
                  checked={filters.roles.includes(option.value ?? "")}
                />
                {option.value}
              </MenuItem>
            ))}
          </Select>
        </FormControl>

        <FormControl
          sx={{
            flexShrink: 0,
            width: { xs: 1, md: 200 },
          }}
        >
          <InputLabel>Tag</InputLabel>

          <Select
            multiple
            value={filters.tags}
            onChange={handleFilterTag}
            input={<OutlinedInput label="Tag" />}
            renderValue={(selected) =>
              selected.map((value) => value).join(", ")
            }
            MenuProps={{
              PaperProps: {
                sx: { maxHeight: 240 },
              },
            }}
          >
            {tagsList.map((option) => (
              <MenuItem key={option} value={option}>
                <Checkbox
                  disableRipple
                  size="small"
                  checked={filters.tags.includes(option ?? "")}
                />
                {option}
              </MenuItem>
            ))}
          </Select>
        </FormControl>

        <FormControl
          sx={{
            flexShrink: 0,
            width: { xs: 1, md: 350 },
          }}
        >
          <Card
            onClick={openDateRange}
            variant="outlined"
            sx={{
              height: 55,
              cursor: "pointer",
              borderRadius: 1,
              p: 3,
              display: "flex",
              boxShadow: "none",
              justifyContent: "space-between",
              alignItems: "center",
            }}
          >
            <Iconify sx={{ mr: 3 }} icon="solar:calendar-date-bold" />
            <ListItemText
              secondary={
                filters.startDate && filters.endDate
                  ? `${fDate(filters.startDate)} - ${fDate(filters.endDate)}`
                  : filters.startDate
                  ? `${fDate(filters.startDate)} - sélectionner la date de fin`
                  : filters.endDate
                  ? `sélectionner la date de début - ${fDate(filters.endDate)}`
                  : "Sélectionner une plage de dates"
              }
              primaryTypographyProps={{
                typography: "body2",
                color: "text.primary",
                mb: 0.5,
              }}
            />
          </Card>

          <CustomDateRangePicker
            open={isDateRangeOpen}
            onClose={closeDateRange}
            startDate={startDate}
            endDate={endDate}
            onChangeStartDate={onChangeStartDate}
            onChangeEndDate={onChangeEndDate}
          />
        </FormControl>

        <Stack
          direction="row"
          alignItems="center"
          spacing={2}
          flexGrow={1}
          sx={{ width: 1 }}
        >
          <TextField
            fullWidth
            value={filters.keyword}
            onChange={handleFilterName}
            placeholder="Recheche..."
            InputProps={{
              startAdornment: (
                <InputAdornment position="start">
                  <Iconify
                    icon="eva:search-fill"
                    sx={{ color: "text.disabled" }}
                  />
                </InputAdornment>
              ),
            }}
          />
        </Stack>
      </Stack>

      <CustomPopover
        open={popover.open}
        onClose={popover.onClose}
        arrow="right-top"
        sx={{ width: 140 }}
      >
        <MenuItem
          onClick={() => {
            popover.onClose();
          }}
        >
          <Iconify icon="solar:printer-minimalistic-bold" />
          Print
        </MenuItem>

        <MenuItem
          onClick={() => {
            popover.onClose();
          }}
        >
          <Iconify icon="solar:export-bold" />
          Export
        </MenuItem>
      </CustomPopover>
    </>
  );
}
