"use client";
import { useEffect, useState } from "react";
import Box from "@mui/material/Box";
import Card from "@mui/material/Card";
import Grid from "@mui/material/Unstable_Grid2";
import TextField from "@mui/material/TextField";
import { Avatar, Container, Typography } from "@mui/material";
import Label from "@/shared/components/label";
import { AddressType, IClientItem, IUser } from "@/shared/types/user";
import CustomBreadcrumbs from "@/components/custom-breadcrumbs/custom-breadcrumbs";
import { paths } from "@/routes/paths";
import { useSettingsContext } from "@/components/settings";
import { useGetUserById } from "@/shared/api/user";

interface UserDetailsProps {
  id: number;
}

export default function UserDetails({ id }: UserDetailsProps) {
  const { userData: user } = useGetUserById(id);

  const translateStatus = (status: string) => {
    switch (status) {
      case "APPROVED":
        return "Approuvé";
      case "PENDING":
        return "En attente";
      case "BANNED":
        return "Banni";
      case "DELETED":
        return "Supprimé";
      default:
        return "Indéfini";
    }
  };

  const settings = useSettingsContext();

  const [clientData, setClientData] = useState<IClientItem | null>(null);
  const [userDataState, setUserDataState] = useState<IUser | null>(null);
  const isClient = user?.roles?.value === "Client";
  useEffect(() => {
    if (user) {
      if (isClient) {
        const client = user as IClientItem;
  
        const billingAddress = client.addresses
          .filter((addr) => addr.addressType === AddressType.BILLING)
          .sort((a, b) => (b.id ?? 0) - (a.id ?? 0))[0];
  
        const shippingAddress = client.addresses
          .filter((addr) => addr.addressType === AddressType.SHIPPING)
          .sort((a, b) => (b.id ?? 0) - (a.id ?? 0))[0];
  
        // Set the first billing and shipping addresses (if they exist)
        setClientData({
          ...client,
          addresses: [
            ...(billingAddress ? [billingAddress] : []),
            ...(shippingAddress ? [shippingAddress] : []),
          ],
        });
        setUserDataState(null);
      } else {
        setUserDataState(user as IUser);
        setClientData(null);
      }
    }
  }, [user]);
  
  return (
    <>
      <Container maxWidth={settings.themeStretch ? false : "lg"}>
        <CustomBreadcrumbs
          heading="Détail"
          links={[
            { name: "Tableau de bord", href: paths.dashboard.root },
            { name: "Utilisateur", href: paths.dashboard.admin.user.all_users },
            { name: "Détail" },
          ]}
          sx={{
            mb: { xs: 3, md: 5 },
          }}
        />
      </Container>
      <Grid container spacing={3}>
        <Grid xs={12} md={4}>
          <Card
            sx={{
              pb: 10,
              px: 3,
              textAlign: "center",
              display: "flex",
              flexDirection: "column",
              alignItems: "center",
            }}
          >
            <Label
              variant="soft"
              color={
                user?.status === "APPROVED"
                  ? "success"
                  : user?.status === "PENDING"
                  ? "warning"
                  : user?.status === "BANNED"
                  ? "error"
                  : "default"
              }
              sx={{ position: "absolute", right: 16, top: 6 }}
            >
              {user?.status && translateStatus(user.status)}
            </Label>
            <Avatar
              alt={user?.firstName}
              src={user?.imageUrl || undefined}
              sx={{ width: 120, height: 120, top: 35 }}
            />
          </Card>
        </Grid>
  
        <Grid xs={12} md={8}>
          <Card sx={{ p: 3 }}>
            <Box rowGap={3} columnGap={2} display="grid">
              {/* User Details */}
              <TextField
                disabled
                name="firstName"
                label="Prénom"
                value={user?.firstName || ""}
                InputProps={{ readOnly: true, style: { color: "#888" } }}
                fullWidth
              />
              <TextField
                disabled
                name="lastName"
                label="Nom de famille"
                value={user?.lastName || ""}
                InputProps={{ readOnly: true, style: { color: "#888" } }}
                fullWidth
              />
              <TextField
                disabled
                name="uid"
                label="UID"
                value={user?.uid || ""}
                InputProps={{ readOnly: true, style: { color: "#888" } }}
                fullWidth
              />
              <TextField
                disabled
                name="email"
                label="Adresse e-mail"
                value={user?.email || ""}
                InputProps={{ readOnly: true, style: { color: "#888" } }}
                fullWidth
              />
              <TextField
                disabled
                name="pseudo"
                label="Pseudo"
                value={user?.pseudo || ""}
                InputProps={{ readOnly: true, style: { color: "#888" } }}
                fullWidth
              />
              <TextField
                disabled
                name="phoneNumber"
                label="Numéro de téléphone"
                value={user?.phoneNumber || ""}
                InputProps={{ readOnly: true, style: { color: "#888" } }}
                fullWidth
              />
              <TextField
                disabled
                name="role"
                label="Rôle"
                value={user?.roles?.value || ""}
                InputProps={{ readOnly: true, style: { color: "#888" } }}
                fullWidth
              />
            </Box>
  
            {isClient && (
              <>
                <Box rowGap={3} columnGap={2} display="grid">
                <Typography variant="subtitle1" sx={{ marginTop: 2 }}>
                    Adresse de Facturation
                  </Typography>
                  {clientData?.addresses[0] ? (
                    <>
                      <TextField
                        label="Prénom"
                        value={clientData?.addresses[0]?.firstName || ""}
                        fullWidth
                        disabled
                      />
                      <TextField
                        label="Nom de famille"
                        value={clientData?.addresses[0]?.lastName || ""}
                        fullWidth
                        disabled
                      />
                      <TextField
                        label="Nom de la société"
                        value={clientData?.addresses[0]?.companyName || ""}
                        fullWidth
                        disabled
                      />
                      <TextField
                        label="Email"
                        value={clientData?.addresses[0]?.email || ""}
                        fullWidth
                        disabled
                      />
                      <TextField
                        label="Numéro de téléphone"
                        value={clientData?.addresses[0]?.phoneNumber || ""}
                        fullWidth
                        disabled
                      />
                      <TextField
                        label="Rue"
                        value={clientData?.addresses[0]?.street || ""}
                        fullWidth
                        disabled
                      />
                      <TextField
                        label="Appartement/Maison"
                        value={clientData?.addresses[0]?.home || ""}
                        fullWidth
                        disabled
                      />
                      <TextField
                        label="Ville"
                        value={clientData?.addresses[0]?.city || ""}
                        fullWidth
                        disabled
                      />
                      <TextField
                        label="Code postal"
                        value={clientData?.addresses[0]?.zipCode || ""}
                        fullWidth
                        disabled
                      />
                      <TextField
                        label="Pays"
                        value={clientData?.addresses[0]?.country || ""}
                        fullWidth
                        disabled
                      />
                    </>
                  ) : (
                    <Typography variant="body2" color="text.secondary" sx={{ mt: -1 }}>
                    Aucune adresse de facturation disponible.
                  </Typography>
                  )}
                </Box>
  
                <Box rowGap={3} columnGap={2} display="grid">
                  <Typography variant="subtitle1" sx={{ marginTop: 2 }}>
                    Adresse de Livraison
                  </Typography>
                  {clientData?.addresses[1] ? (
                    <>
                      <TextField
                        label="Prénom"
                        value={clientData?.addresses[1]?.firstName || ""}
                        fullWidth
                        disabled
                      />
                      <TextField
                        label="Nom de famille"
                        value={clientData?.addresses[1]?.lastName || ""}
                        fullWidth
                        disabled
                      />
                      <TextField
                        label="Nom de la société"
                        value={clientData?.addresses[1]?.companyName || ""}
                        fullWidth
                        disabled
                      />
                      <TextField
                        label="Email"
                        value={clientData?.addresses[1]?.email || ""}
                        fullWidth
                        disabled
                      />
                      <TextField
                        label="Numéro de téléphone"
                        value={clientData?.addresses[1]?.phoneNumber || ""}
                        fullWidth
                        disabled
                      />
                      <TextField
                        label="Rue"
                        value={clientData?.addresses[1]?.street || ""}
                        fullWidth
                        disabled
                      />
                      <TextField
                        label="Appartement/Maison"
                        value={clientData?.addresses[1]?.home || ""}
                        fullWidth
                        disabled
                      />
                      <TextField
                        label="Ville"
                        value={clientData?.addresses[1]?.city || ""}
                        fullWidth
                        disabled
                      />
                      <TextField
                        label="Code postal"
                        value={clientData?.addresses[1]?.zipCode || ""}
                        fullWidth
                        disabled
                      />
                      <TextField
                        label="Pays"
                        value={clientData?.addresses[1]?.country || ""}
                        fullWidth
                        disabled
                      />
                    </>
                  ) : (
                  <Typography variant="body2" color="text.secondary" sx={{ mt: -1 }}>
                    Aucune adresse de livraison disponible.
                  </Typography>
                  )}
                </Box>
              </>
            )}
          </Card>
        </Grid>
      </Grid>
    </>
  );
  
}
