import React, { useState } from 'react';
import { Button, Dialog, DialogActions, DialogContent, DialogTitle, TextField, Snackbar, Alert } from '@mui/material';
import { addRole } from '@/shared/api/user';
import { IRole } from '@/shared/types/user';

type Props = {
  open: boolean;
  handleClose: () => void;
  refreshRoles: () => void;
  setRolesData: React.Dispatch<React.SetStateAction<IRole[]>>;  
};

const AddRoleDialog: React.FC<Props> = ({ open, handleClose, refreshRoles, setRolesData }) => {
  const [roleName, setRoleName] = useState('');
  const [snackbarOpen, setSnackbarOpen] = useState(false);
  const [snackbarMessage, setSnackbarMessage] = useState('');
  const [snackbarSeverity, setSnackbarSeverity] = useState<'success' | 'error'>('success');

  const handleSnackbarOpen = (message: string, severity: 'success' | 'error') => {
    setSnackbarMessage(message);
    setSnackbarSeverity(severity);
    setSnackbarOpen(true);
  };

  const handleSnackbarClose = () => {
    setSnackbarOpen(false);
  };

  const handleSubmit = async (event: React.FormEvent<HTMLFormElement>) => {
    event.preventDefault();

    try {
      await addRole(roleName);
      refreshRoles(); 
      handleSnackbarOpen('Le rôle est ajouté avec succès!', 'success');

      setRolesData((prevRoles) => [
        ...prevRoles, 
        { id: null, name: roleName, value: roleName, rolePermissions: [] } 
      ]);

      setRoleName('');
      handleClose();
    } catch (error) {
      handleSnackbarOpen("Erreur lors de l'ajout du rôle", 'error');
      console.error('Error adding role:', error);
    }
  };

  return (
    <>
      <Dialog
        open={open}
        onClose={handleClose}
        PaperProps={{
          component: 'form',
          onSubmit: handleSubmit,
        }}
      >
        <DialogTitle>Ajouter un nouveau rôle</DialogTitle>
        <DialogContent>
          <TextField
            autoFocus
            margin="dense"
            id="roleName"
            label="Nom du rôle (ID)"
            type="text"
            fullWidth
            value={roleName}
            onChange={(e) => setRoleName(e.target.value)}
          />
        </DialogContent>
        <DialogActions>
          <Button type="submit">Ajouter un rôle</Button>
          <Button onClick={handleClose}>Annuler</Button>
        </DialogActions>
      </Dialog>

      <Snackbar
        open={snackbarOpen}
        autoHideDuration={3000}
        onClose={handleSnackbarClose}
        anchorOrigin={{ vertical: 'top', horizontal: 'right' }} 
      >
        <Alert onClose={handleSnackbarClose} severity={snackbarSeverity} sx={{ width: '100%' }}>
          {snackbarMessage}
        </Alert>
      </Snackbar>
    </>
  );
};

export default AddRoleDialog;
