import { useMemo, useCallback, useState, useEffect } from "react";
import { useForm, Controller } from "react-hook-form";
import { yupResolver } from "@hookform/resolvers/yup";
import * as Yup from "yup";

// MUI Components
import Box from "@mui/material/Box";
import Card from "@mui/material/Card";
import Stack from "@mui/material/Stack";
import Button from "@mui/material/Button";
import Switch from "@mui/material/Switch";
import Grid from "@mui/material/Unstable_Grid2";
import Typography from "@mui/material/Typography";
import LoadingButton from "@mui/lab/LoadingButton";
import FormControlLabel from "@mui/material/FormControlLabel";
import {
  Alert,
  AlertTitle,
  IconButton,
  InputAdornment,
  MenuItem,
  TextField,
} from "@mui/material";
import { Visibility, VisibilityOff } from "@mui/icons-material";

// Custom Components
import { useSnackbar } from "@/components/snackbar";
import FormProvider, {
  RHFTextField,
  RHFUploadAvatar,
  RHFAutocomplete,
  RHFSelect,
} from "../../../components/hook-form";
import Label from "../../../components/label";

// Utilities
import { fData } from "@/utils/format-number";
import { paths } from "@/routes/paths";
import { countries } from "../../../assets/data";
import { useRouter } from "../../../../hooks/use-router";

// Types and API Calls
import {
  AddressType,
  IAddressItem,
  IClientItem,
  IUser,
  UpdatePasswordRequest,
} from "../../../types/user";
import {
  updateClient,
  updateUser,
  UpdateUserPassword,
  useGetRoles,
  useGetUserById,
} from "@/shared/api/user";
import { useParams } from "next/navigation";
type FormValues = {
  billing_firstName?: string;
  billing_lastName?: string;
  billing_companyName?: string;
  billing_email?: string;
  billing_addresscountry?: string;
  billing_street?: string;
  billing_home?: string;
  billing_addresscity?: string;
  billing_zipCode?: string;
  billing_phoneNumber?: number;
  billing_id?: number; // Ensure this is included

  shipping_firstName?: string;
  shipping_lastName?: string;
  shipping_companyName?: string;
  shipping_email?: string;
  shipping_addresscountry?: string;
  shipping_street?: string;
  shipping_home?: string;
  shipping_addresscity?: string;
  shipping_zipCode?: string;
  shipping_phoneNumber?: number;
  shipping_id?: number; // Ensure this is included

  firstName: string;
  lastName: string;
  role: string;
  email?: string;
  status?: string;
  pseudo: string;
  uid?: number;
  phoneNumber: string;
  birthDate?: string;
};

// Helper Functions
const generateStrongPassword = () => {
  const length = 12;
  const charset =
    "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!@#$%^&*()_-+=";
  let password = "";
  for (let i = 0; i < length; ++i) {
    const randomIndex = Math.floor(Math.random() * charset.length);
    password += charset[randomIndex];
  }
  return password;
};

export default function UserNewEditForm() {
  // Hooks
  const router = useRouter();
  const { id } = useParams();
  const { enqueueSnackbar } = useSnackbar();

  // States
  const [newPassword, setNewPassword] = useState("");
  const [resetPassword, setResetPassword] = useState("");
  const [showNewPasswordField, setShowNewPasswordField] = useState(false);
  const [showPassword, setShowPassword] = useState(false);
  const [alertMessage, setAlertMessage] = useState("");
  const [alertSeverity, setAlertSeverity] = useState<
    "success" | "error" | "info" | "warning"
  >("success");
  const [showAlert, setShowAlert] = useState(false);

  const handleCloseAlert = () => setShowAlert(false);

  // Password handling
  const handleTogglePasswordVisibility = useCallback(() => {
    setShowPassword((prev) => !prev);
  }, []);

  const handleNewPasswordClick = useCallback(() => {
    const generatedPassword = generateStrongPassword();
    setNewPassword(generatedPassword);
    setShowPassword(true);
    setShowNewPasswordField(true);
  }, []);

  const handleResetPasswordReset = useCallback(() => {
    setResetPassword("");
  }, []);

  // Schema for form validation (adjusted for required fields in addresses)
  const NewUserSchema = Yup.object().shape({
    firstName: Yup.string().required("Le prénom est requis"),
    lastName: Yup.string().required("Le nom est requis"),
    role: Yup.string().required("Le rôle est requis"),
    pseudo: Yup.string().required("Le pseudo est requis"),
    email: Yup.string().notRequired(),
    status: Yup.string().notRequired(),
    uid: Yup.number().notRequired(),
    birthDate: Yup.string()
      .matches(
        /^\d{4}-\d{2}-\d{2}$/,
        "Invalid date format. Expected YYYY-MM-DD"
      )
      .notRequired(),

    // Billing address fields (all required)
    billing_addresscountry: Yup.string().notRequired(),
    billing_street: Yup.string().notRequired(),
    billing_home: Yup.string().notRequired(),
    billing_addresscity: Yup.string().notRequired(),
    billing_zipCode: Yup.string().notRequired(),
    billing_phoneNumber: Yup.number().notRequired(),
    billing_firstName: Yup.string().notRequired(),
    billing_lastName: Yup.string().notRequired(),
    billing_companyName: Yup.string().notRequired(),
    billing_email: Yup.string().notRequired(),
    shipping_addresscountry: Yup.string().notRequired(),
    shipping_street: Yup.string().notRequired(),
    shipping_home: Yup.string().notRequired(),
    shipping_addresscity: Yup.string().notRequired(),
    shipping_zipCode: Yup.string().notRequired(),
    shipping_phoneNumber: Yup.number().notRequired(),
    shipping_firstName: Yup.string().notRequired(),
    shipping_lastName: Yup.string().notRequired(),
    shipping_companyName: Yup.string().notRequired(),
    shipping_email: Yup.string().notRequired(),
    imageUrl: Yup.mixed().nullable(),
    phoneNumber: Yup.string().required("Le numéro de téléphone est requis"),
    password: Yup.string()
      .notRequired()
  });

  const { userData } = useGetUserById(Number(id));
  const { roleData } = useGetRoles();

  const [clientData, setClientData] = useState<IClientItem | null>(null);
  const [userDataState, setUserDataState] = useState<IUser | null>(null);

  useEffect(() => {
    if (userData && "roles" in userData && userData.roles.name === "CLIENT") {
      const clientData = userData as IClientItem;

      const getMaxIdAddress = (
        addresses: IAddressItem[],
        addressType: string
      ) => {
        return addresses
          ?.filter((addr) => addr.addressType === addressType)
          .sort((a, b) => (b.id ?? 0) - (a.id ?? 0))[0];
      };

      const billingAddress = getMaxIdAddress(
        clientData.addresses,
        AddressType.BILLING
      );
      const shippingAddress = getMaxIdAddress(
        clientData.addresses,
        AddressType.SHIPPING
      );

      const updatedAddresses = [
        ...(billingAddress ? [billingAddress] : []),
        ...(shippingAddress ? [shippingAddress] : []),
      ];

      setClientData({
        ...clientData,
        addresses: updatedAddresses,
      });

      setUserDataState(null);
    } else if (userData) {
      setUserDataState(userData as IUser);
      setClientData(null);
    }
  }, [userData]);

  const defaultValues = useMemo(() => {
    const formatDate = (date: Date | string) =>
      date ? new Date(date).toISOString().slice(0, 10) : "";

    if (clientData) {
      const billingAddress = clientData.addresses.find(
        (addr) => addr.addressType === AddressType.BILLING
      );
      const shippingAddress = clientData.addresses.find(
        (addr) => addr.addressType === AddressType.SHIPPING
      );

      return {
        firstName: clientData.firstName || "",
        lastName: clientData.lastName || "",
        role: clientData.roles?.name || "",
        email: clientData.email || "",
        status: clientData.status || "",
        pseudo: clientData.pseudo || "",
        birthDate: formatDate(clientData.birthDate),
        uid: Number(clientData.uid) || 0,
        password: "",

        // Billing address required fields
        billing_id: billingAddress?.id || "",
        billing_firstName: billingAddress?.firstName || "",
        billing_lastName: billingAddress?.lastName || "",
        billing_companyName: billingAddress?.companyName || "",
        billing_email: billingAddress?.email || "",
        billing_addresscountry: billingAddress?.country || "",
        billing_street: billingAddress?.street || "",
        billing_home: billingAddress?.home || "",
        billing_addresscity: billingAddress?.city || "",
        billing_zipCode: billingAddress?.zipCode || "",
        billing_phoneNumber: billingAddress?.phoneNumber || 0,

        // Shipping address required fields
        shipping_id: shippingAddress?.id || "",
        shipping_firstName: shippingAddress?.firstName || "",
        shipping_lastName: shippingAddress?.lastName || "",
        shipping_companyName: shippingAddress?.companyName || "",
        shipping_email: shippingAddress?.email || "",
        shipping_addresscountry: shippingAddress?.country || "",
        shipping_street: shippingAddress?.street || "",
        shipping_home: shippingAddress?.home || "",
        shipping_addresscity: shippingAddress?.city || "",
        shipping_zipCode: shippingAddress?.zipCode || "",
        shipping_phoneNumber: shippingAddress?.phoneNumber || 0,

        imageUrl: clientData.imageUrl || null,
        phoneNumber: clientData.phoneNumber || "",
      };
    } else if (userDataState) {
      return {
        firstName: userDataState.firstName || "",
        lastName: userDataState.lastName || "",
        role: userDataState.roles?.name || "",
        pseudo: userDataState.pseudo || "",
        email: userDataState.email || "",
        status: userDataState.status || "",
        imageUrl: userDataState.imageUrl || null,
        phoneNumber: userDataState.phoneNumber || "",
        uid: Number(userDataState.uid) || 0,
        password: "",
      };
    } else {
      return {
        firstName: "",
        lastName: "",
        role: "",
        pseudo: "",
        email: "",
        status: "",
        imageUrl: null,
        phoneNumber: "",
        uid: 0,
        password: "",

        billing_firstName: "",
        billing_lastName: "",
        billing_companyName: "",
        billing_email: "",
        billing_addresscountry: "",
        billing_street: "",
        billing_home: "",
        billing_addresscity: "",
        billing_zipCode: "",
        billing_phoneNumber: 0,

        shipping_firstName: "",
        shipping_lastName: "",
        shipping_companyName: "",
        shipping_email: "",
        shipping_addresscountry: "",
        shipping_street: "",
        shipping_home: "",
        shipping_addresscity: "",
        shipping_zipCode: "",
        shipping_phoneNumber: 0,
      };
    }
  }, [clientData, userDataState, userData]);

  const methods = useForm({
    resolver: yupResolver(NewUserSchema),
    defaultValues,
  });
  const {
    reset,
    watch,
    control,
    handleSubmit,
    setValue,
    formState: { isSubmitting },
  } = methods;

  useEffect(() => {
    if (clientData || userDataState) {
      reset(defaultValues);
    }
  }, [clientData, userDataState, reset, defaultValues]);

  const values = watch();
const [imageFile, setImageFile] = useState<File | null>(null);
const handleDrop = useCallback(
  (acceptedFiles: File[]) => {
    if (acceptedFiles.length > 0) {
      const file = acceptedFiles[0];
      if (file.size > 3 * 1024 ) {
        alert("La taille du fichier dépasse la limite de 3 Mo. Veuillez télécharger un fichier plus petit.");
        return;
      }

      const newFile = Object.assign(file, {
        preview: URL.createObjectURL(file),
      });
      setImageFile(file);
      setValue("imageUrl", newFile.preview, { shouldValidate: true });
    }
  },
  [setValue]
);


  const updateUserForm = async () => {
    const { uid, firstName, lastName, email, pseudo, phoneNumber, role } =
      methods.getValues();
    const selectedRole = roleData.find(
      (roleData) => roleData.name === role
    ) || { id: null, name: role };

    const userdto = {
      id: userDataState?.id || "",
      uid: uid || 0,
      firstName: firstName || "",
      lastName: lastName || "",
      pseudo: pseudo || "",
      email: email || "",
      password: userDataState?.password || "",
      phoneNumber: phoneNumber || "",
      roles: selectedRole,
      status: methods.getValues().status || "",
      activityLog: userDataState?.activityLog || false,
    };

    const result = imageFile
      ? await updateUser(JSON.stringify(userdto), imageFile)
      : await updateUser(JSON.stringify(userdto));

    if (result.status === 200) {
      setAlertSeverity("success");
      setAlertMessage("Mise à jour réussie pour l'utilisateur !");
      setShowAlert(true);
    } else {
      setAlertSeverity("error");
      setAlertMessage("La mise à jour de l'utilisateur a échoué !");
      setShowAlert(true);
    }
  };
  const [initialAddresses, setInitialAddresses] = useState<IAddressItem[]>([]);


  useEffect(() => {
    if (clientData) {
      const billingAddress = clientData.addresses.find(
        (addr) => addr.addressType === AddressType.BILLING
      );
      const shippingAddress = clientData.addresses.find(
        (addr) => addr.addressType === AddressType.SHIPPING
      );
      
      setInitialAddresses([
        billingAddress,
        shippingAddress
      ].filter((addr): addr is IAddressItem => addr !== undefined));
    }
  }, [clientData]);


  const updateClientForm = async () => {
    const { billing_id, shipping_id } = methods.getValues() as {
      billing_id?: number;
      shipping_id?: number;
    };
    
    const billingAddress: IAddressItem = {
      id: billing_id || 0,
      country: methods.getValues().billing_addresscountry || "",
      street: methods.getValues().billing_street || "",
      home: methods.getValues().billing_home || "",
      city: methods.getValues().billing_addresscity || "",
      zipCode: methods.getValues().billing_zipCode || "",
      phoneNumber: Number(methods.getValues().billing_phoneNumber),
      firstName: methods.getValues().billing_firstName || "",
      lastName: methods.getValues().billing_lastName ||  "",
      companyName: methods.getValues().billing_companyName || "",
      email: methods.getValues().billing_email || "",
      addressType: AddressType.BILLING,
    };

    const shippingAddress: IAddressItem = {
      id: shipping_id || 0,
      country: methods.getValues().shipping_addresscountry || "",
      street: methods.getValues().shipping_street || "",
      home: methods.getValues().shipping_home || "",
      city: methods.getValues().shipping_addresscity || "",
      zipCode: methods.getValues().shipping_zipCode || "",
      phoneNumber: Number(methods.getValues().shipping_phoneNumber),
      firstName: methods.getValues().shipping_firstName || "",
      lastName: methods.getValues().shipping_lastName || "",
      companyName: methods.getValues().shipping_companyName || "",
      email: methods.getValues().shipping_email  || "",
      addressType: AddressType.SHIPPING,
    };

    const selectedRole = roleData.find(
      (roleData) => roleData.name === methods.getValues().role
    ) || { id: null, name: methods.getValues().role };

    const clientdto: IClientItem = {
      id: clientData?.id || "",
      uid: methods.getValues().uid || 0,
      firstName: methods.getValues().firstName || "",
      lastName: methods.getValues().lastName || "",
      birthDate: new Date(
        methods.getValues().birthDate || Date.now()
      ).toISOString(),
      imageUrl: "",
      pseudo: methods.getValues().pseudo || "",
      email: methods.getValues().email || "",
      password: clientData?.password || "",
      phoneNumber: methods.getValues().phoneNumber || "",
      roles: selectedRole,
      status: methods.getValues().status || "",
      activityLog: clientData?.activityLog || false,
      addresses: [billingAddress, shippingAddress],
      howDidYouHearAboutUs:
        (clientData?.howDidYouHearAboutUs as string) ||
        "OTHER",
    };

    const result = imageFile
      ? await updateClient(clientdto, imageFile) 
      : await updateClient(clientdto);
    if (result.status === 200) {
      setAlertSeverity("success");
      setAlertMessage("Mise à jour réussie pour le client !");
      setShowAlert(true);
    } else {
      setAlertSeverity("error");
      setAlertMessage("La mise à jour du client a échoué !");
      setShowAlert(true);
    }
  };
  const onSaveChangesClick = async () => {
    try {
      const currentValues = methods.getValues();
      const hasChanged = (Object.keys(defaultValues) as Array<keyof typeof defaultValues>).some(
        (key) => (currentValues as any)[key] !== (defaultValues as any)[key]
      );
      
      if (!hasChanged) {
        setAlertSeverity("info");
        setAlertMessage("Aucune modification détectée.");
        setShowAlert(true);
        return;
      }
  
      if (userDataState) {
        await updateUserForm();
      } else if (clientData) {
        await updateClientForm();
      }
    } catch (error) {
      console.error(error);
      setAlertSeverity("error");
      setAlertMessage("La mise à jour a échoué !");
      setShowAlert(true);
    }
  };
  
  const handleNewPasswordSubmit = useCallback(async () => {
    try {
      if (userData || clientData) {
        const updatePasswordRequest: UpdatePasswordRequest = {
          email: userData?.email || clientData?.email || "",
          password: newPassword,
        };
        const response = await UpdateUserPassword(updatePasswordRequest);

        if (response.status === 200) {
          setAlertSeverity("success");
          setAlertMessage("Mot de passe mis à jour avec succès !");
          setShowAlert(true);
          setShowNewPasswordField(false);
        }
      }
    } catch (error) {
      console.error(error);
      setAlertSeverity("error");
      setAlertMessage("Échec de la mise à jour du mot de passe !");
      setShowAlert(true);
    }
  }, [userData, newPassword]);

  useEffect(() => {
    if (showAlert) {
      const timer = setTimeout(() => setShowAlert(false), 3000);
      return () => clearTimeout(timer);
    }
  }, [showAlert]);

  return (
    <FormProvider methods={methods}>
      <Grid container spacing={3}>
        <Grid xs={12} md={4}>
          <Card sx={{ pt: 10, pb: 5, px: 3 }}>
            <Label
              color={
                (values.status === "APPROVED" && "success") ||
                (values.status === "BANNED" && "error") ||
                "warning"
              }
              sx={{ position: "absolute", top: 24, right: 24 }}
            >
              {values.status}
            </Label>
            <Box sx={{ mb: 5 }}>
              <RHFUploadAvatar
                name="imageUrl"
                maxSize={3145728} 
                onDrop={handleDrop}
                helperText={
                  <Typography
                    variant="caption"
                    sx={{
                      mt: 3,
                      mx: "auto",
                      display: "block",
                      textAlign: "center",
                      color: "text.disabled",
                    }}
                  >
                    Formats autorisés : *.jpeg, *.jpg, *.png, *.gif
                    <br /> taille maximale {fData(3145728)}
                  </Typography>
                }
              />
            </Box>
            {userData && userData.roles.name === "CLIENT" && (
            <FormControlLabel
              labelPlacement="start"
              control={
                <Controller
                  name="status"
                  control={control}
                  render={({ field }) => (
                    <Switch
                      {...field}
                      checked={field.value === "BANNED"}
                      onChange={(event) =>
                        field.onChange(
                          event.target.checked ? "BANNED" : "APPROVED"
                        )
                      }
                    />
                  )}
                />
              }
              label={
                <>
                  <Typography variant="subtitle2" sx={{ mb: 0.5 }}>
                    Banni
                  </Typography>
                  <Typography variant="body2" sx={{ color: "text.secondary" }}>
                    Appliquer la désactivation du compte
                  </Typography>
                </>
              }
              sx={{ mx: 0, mb: 3, width: 1, justifyContent: "space-between" }}
            />
            )}
          </Card>
        </Grid>
        <Grid xs={12} md={8}>
          <Card sx={{ p: 3 }}>
            {showAlert && (
              <Alert
                severity={alertSeverity}
                onClose={handleCloseAlert}
                sx={{ mb: 2 }}
              >
                <AlertTitle>
                  {alertSeverity === "success" ? "Succès" : "Erreur"}
                </AlertTitle>
                {alertMessage}
              </Alert>
            )}
            <Box
              rowGap={3}
              columnGap={2}
              display="grid"
              gridTemplateColumns={{
                xs: "repeat(1, 1fr)",
                sm: "repeat(2, 1fr)",
              }}
            >
              <RHFTextField name="firstName" label="Prénom" />
              <RHFTextField name="lastName" label="Nom" />
              <RHFTextField name="pseudo" label="Pseudo" />
              <RHFTextField name="email" label="Adresse Email" disabled />
              <RHFTextField name="uid" label="UID" disabled />
              <RHFTextField name="phoneNumber" label="Numéro de Téléphone" />
              {clientData && (
                <RHFTextField
                  type="date"
                  name="birthDate"
                  label="Date de Naissance"
                  InputLabelProps={{
                    shrink: true,
                  }}
                />
              )}
              <RHFSelect name="role" label="Rôle" disabled={!!clientData}>
                {roleData &&
                  roleData
                    .filter(
                      (option) =>
                        !(clientData === null && option.name === "CLIENT")
                    )
                    .map((option) => (
                      <MenuItem key={option.id} value={option.name}>
                        {option.value}
                      </MenuItem>
                    ))}
              </RHFSelect>
            </Box>
            {clientData && (
              <>
                <Box sx={{ mb: 3 }}>
                  <Typography variant="subtitle1">
                    Adresse de Facturation
                  </Typography>
                  <Box sx={{ mt: 2 }}>
                    <RHFTextField name="billing_firstName" label="Prénom" />
                  </Box>
                  <Box sx={{ mt: 2 }}>
                    <RHFTextField name="billing_lastName" label="Nom" />
                  </Box>
                  <Box sx={{ mt: 2 }}>
                    <RHFTextField name="billing_email" label="Email" />
                  </Box>
                  <Box sx={{ mt: 2 }}>
                    <RHFAutocomplete
                      name="billing_addresscountry"
                      type="country"
                      label="Pays"
                      placeholder="Choisissez un pays"
                      fullWidth
                      options={countries.map((option) => option.label)}
                      getOptionLabel={(option) => option}
                      defaultValue={clientData?.addresses[0]?.country || ""}
                    />
                  </Box>
                  <Box sx={{ mt: 2 }}>
                    <RHFTextField name="billing_street" label="Rue" />
                  </Box>
                  <Box sx={{ mt: 2 }}>
                    <RHFTextField name="billing_home" label="Domicile" />
                  </Box>
                  <Box sx={{ mt: 2 }}>
                    <RHFTextField name="billing_addresscity" label="Ville" />
                  </Box>
                  <Box sx={{ mt: 2 }}>
                    <RHFTextField
                      name="billing_phoneNumber"
                      label="Numéro de télephone"
                    />
                  </Box>
                  <Box sx={{ mt: 2 }}>
                    <RHFTextField name="billing_zipCode" label="Code Postal" />
                  </Box>

                  <Box sx={{ mt: 2 }}>
                    <RHFTextField
                      name="billing_companyName"
                      label="Nom de l'Entreprise"
                    />
                  </Box>
                </Box>

                <Box display="grid" sx={{ mb: 3 }}>
                  <Typography variant="subtitle1">
                    Adresse de Livraison
                  </Typography>
                  <Box sx={{ mt: 2 }}>
                    <RHFTextField name="shipping_firstName" label="Prénom" />
                  </Box>
                  <Box sx={{ mt: 2 }}>
                    <RHFTextField name="shipping_lastName" label="Nom" />
                  </Box>
                  <Box sx={{ mt: 2 }}>
                    <RHFTextField name="shipping_email" label="Email" />
                  </Box>
                  <Box sx={{ mt: 2 }}>
                    <RHFAutocomplete
                      name="shipping_addresscountry"
                      type="country"
                      label="Pays"
                      placeholder="Choisissez un pays"
                      fullWidth
                      options={countries.map((option) => option.label)}
                      getOptionLabel={(option) => option}
                      defaultValue={clientData?.addresses[1]?.country || ""}
                    />
                  </Box>
                  <Box sx={{ mt: 2 }}>
                    <RHFTextField name="shipping_street" label="Rue" />
                  </Box>
                  <Box sx={{ mt: 2 }}>
                    <RHFTextField name="shipping_home" label="Domicile" />
                  </Box>
                  <Box sx={{ mt: 2 }}>
                    <RHFTextField name="shipping_addresscity" label="Ville" />
                  </Box>
                  <Box sx={{ mt: 2 }}>
                    <RHFTextField
                      name="shipping_phoneNumber"
                      label="Numéro de télephone"
                    />
                  </Box>
                  <Box sx={{ mt: 2 }}>
                    <RHFTextField name="shipping_zipCode" label="Code Postal" />
                  </Box>

                  <Box sx={{ mt: 2 }}>
                    <RHFTextField
                      name="shipping_companyName"
                      label="Nom de l'Entreprise"
                    />
                  </Box>
                </Box>
              </>
            )}

            <Box sx={{ mb: 3, mt: 3 }}>
              <Typography variant="subtitle1">Gestion du Compte</Typography>
              <Box sx={{ display: "flex", alignItems: "center", mt: 2 }}>
                <Button variant="contained" onClick={handleNewPasswordClick}>
                  Réinitialiser le mot de passe
                </Button>
              </Box>

              {showNewPasswordField && (
                <Box sx={{ mt: 2, display: "flex", alignItems: "center" }}>
                  <TextField
                    type={showPassword ? "text" : "password"}
                    value={newPassword}
                    onChange={(e) => setNewPassword(e.target.value)}
                    InputProps={{
                      endAdornment: (
                        <InputAdornment position="end">
                          <IconButton onClick={handleTogglePasswordVisibility}>
                            {showPassword ? <VisibilityOff /> : <Visibility />}
                          </IconButton>
                        </InputAdornment>
                      ),
                    }}
                    sx={{ mr: 1 }}
                  />
                  <Box sx={{ marginRight: 1 }} />
                  <Button variant="contained" onClick={handleNewPasswordSubmit}>
                    Réinitialiser
                  </Button>
                  <Box sx={{ marginLeft: 1 }} />
                  <Button
                    variant="contained"
                    onClick={() => setShowNewPasswordField(false)}
                  >
                    Annuler
                  </Button>
                </Box>
              )}
            </Box>

            <Stack alignItems="flex-end" sx={{ mt: 3 }}>
              <LoadingButton
                variant="contained"
                loading={isSubmitting}
                onClick={handleSubmit(onSaveChangesClick)}
              >
                Sauvegarder les modifications
              </LoadingButton>
            </Stack>
          </Card>
        </Grid>
      </Grid>
    </FormProvider>
  );
}
