'use client';
import React, { useState, useCallback, useEffect } from 'react';
import {
  DataGrid,
  GridColDef,
  GridActionsCellItem,
  GridToolbarContainer,
  GridToolbarQuickFilter,
  GridToolbarFilterButton,
  GridToolbarExport,
  GridRowSelectionModel,
} from '@mui/x-data-grid';
import Button from '@mui/material/Button';
import Container from '@mui/material/Container';
import Iconify from '@/shared/components/iconify';
import { ConfirmDialog } from '@/shared/components/custom-dialog';
import { deleteVoucher, duplicateVoucher, sendVoucherEmail, useGetVouchers } from '@/shared/api/voucher';
import CustomBreadcrumbs from '@/components/custom-breadcrumbs';
import { paths } from '@/routes/paths';
import { useRouter } from '@/hooks';
import { useBoolean } from '@/hooks/use-boolean';
import { Card, Stack, IconButton, CircularProgress } from '@mui/material';
import { useSettingsContext } from '@/components/settings';
import EmailIcon from '@mui/icons-material/Email';
import CustomAlert from '@/shared/components/CustomAlert';
import RouterLink from '@/routes/router-link';
import { GridCellParams } from '@mui/x-data-grid';
import { RenderCellAmount, RenderCellCreatedAt, RenderCellExperationDate , RenderCellLegend } from './Voucher-table-row';

type ParamsProps = {
  params: GridCellParams;
};

export default function VoucherListView() {
  const { vouchers, isLoading, mutate } = useGetVouchers();
  const [alert, setAlert] = useState<{ message: string; type: 'success' | 'error' } | null>(null);
  const router = useRouter();
  const [selectedRowIds, setSelectedRowIds] = useState<GridRowSelectionModel>([]);
  const [confirmDeleteOpen, setConfirmDeleteOpen] = useState(false);
  const [confirmDuplicateOpen, setConfirmDuplicateOpen] = useState(false);
  const [voucherIdToDuplicate, setVoucherIdToDuplicate] = useState<number | null>(null);
  const settings = useSettingsContext();
  const confirmRows = useBoolean();

  useEffect(() => {
    if (alert) {
      const timer = setTimeout(() => {
        setAlert(null);
      }, 3000);
      return () => clearTimeout(timer);
    }
  }, [alert]);

  const handleEditRow = useCallback(
    (id: string) => {
      router.push(paths.dashboard.admin.voucher.edit(id));
    },
    [router]
  );

  const handleDelete = async () => {
    try {
      if (selectedRowIds.length > 0) {
        await Promise.all(selectedRowIds.map((id) => deleteVoucher(Number(id))));
        setAlert({ message: 'Suppression réussie !', type: 'success' });
        mutate();
        setSelectedRowIds([]);
      }
    } catch (error) {
      setAlert({ message: 'Erreur lors de la suppression.', type: 'error' });
    } finally {
      setConfirmDeleteOpen(false);
      confirmRows.onFalse();
    }
  };

  const handleDuplicate = async () => {
    if (voucherIdToDuplicate !== null) {
      try {
        await duplicateVoucher(voucherIdToDuplicate);
        setAlert({ message: 'Duplication réussie !', type: 'success' });
        mutate();
      } catch (error) {
        setAlert({ message: 'Erreur lors de la duplication.', type: 'error' });
      } finally {
        setConfirmDuplicateOpen(false);
        setVoucherIdToDuplicate(null);
      }
    }
  };

  const RenderCellVoucheremail = ({ params }: ParamsProps) => {
    const [loading, setLoading] = useState(false);

    const handleSendEmail = async () => {
      setLoading(true);
      try {
        await sendVoucherEmail(params.row.id);
        setAlert({ message: 'Email envoyé avec succès !', type: 'success' });
      } catch (error) {
        setAlert({ message: 'Erreur lors de l\'envoi de l\'email.', type: 'error' });
      } finally {
        setLoading(false);
      }
    };

    return (
      <IconButton
        onClick={handleSendEmail}
        disabled={loading}
      >
        {loading ? <CircularProgress size={24} /> : <EmailIcon sx={{ color: 'blue' }} />} {/* Utilise une couleur personnalisée */}
      </IconButton>
    );
  };

  const columns: GridColDef[] = [
    { field: 'uniqueNumber', headerName: 'Numéro', flex: 1, minWidth: 150 },
    { field: 'email', headerName: 'Email', flex: 1, minWidth: 200 },
    {
      field: 'createdAt',
      headerName: 'Date de création',
      width: 150,
      renderCell: (params) => <RenderCellCreatedAt params={params} />,
    },
    {
      field: 'amount',
      headerName: 'Montant (€)',
      width: 150,
      renderCell: (params) => <RenderCellAmount params={params} />,
    },
    {
      field: 'expirationDate',
      headerName: 'Date d\'expiration',
      width: 150,
      renderCell: (params) => <RenderCellExperationDate params={params} />,
    },
    {
      field: 'legend',
      headerName: 'Legende',
      width: 150,
      renderCell: (params) => <RenderCellLegend params={params} />,
    }, 
    {
      field: "Voucheremail",
      headerName: "Envoyer",
      width: 100,
      filterable: false,
      renderCell: (params) => <RenderCellVoucheremail params={params} />,
    },
    {
      field: 'actions',
      type: 'actions',
      headerName: 'Actions',
      width: 150,
      getActions: (params) => [
        <GridActionsCellItem
          icon={<Iconify icon="eva:edit-outline" />}
          label="Modifier"
          onClick={() => handleEditRow(params.id.toString())}
        />,
        <GridActionsCellItem
          icon={<Iconify icon="eva:copy-outline" />}
          label="Dupliquer"
          onClick={() => {
            setVoucherIdToDuplicate(Number(params.id));
            setConfirmDuplicateOpen(true);
          }}
        />,
        <GridActionsCellItem
          icon={<Iconify icon="eva:trash-2-outline" />}
          label="Supprimer"
          onClick={() => {
            setSelectedRowIds([params.id]);
            setConfirmDeleteOpen(true);
          }}
          sx={{ color: 'error.main' }}
        />,
      ],
    },
  ];

  return (
    <>
      {alert && (
        <CustomAlert
          message={alert.message}
          type={alert.type}
          onClose={() => setAlert(null)}
          style={{ top: '20px', right: '20px', zIndex: 3000 }}
        />
      )}
      <Container
        maxWidth={settings.themeStretch ? false : 'xl'}
        sx={{
          flexGrow: 1,
          display: 'flex',
          flexDirection: 'column',
        }}
      >
        <CustomBreadcrumbs
          heading="Liste des bons d’achat"
          links={[
            { name: 'Tableau de bord', href: paths.dashboard.root },
            { name: 'Bons', href: paths.dashboard.admin.voucher.root },
            { name: 'Liste' },
          ]}
          action={
            <Button
              component={RouterLink}
              href={paths.dashboard.admin.voucher.new}
              variant="contained"
              startIcon={<Iconify icon="mingcute:add-line" />}
            >
              Ajouter un Bon
            </Button>
          }
          sx={{
            mb: {
              xs: 3,
              md: 5,
            },
          }}
        />
        <Card variant="outlined">
          <DataGrid
            autoHeight
            rows={vouchers}
            columns={columns}
            loading={isLoading}
            getRowId={(row) => row.id}
            pageSizeOptions={[5, 10, 25]}
            initialState={{
              pagination: {
                paginationModel: { pageSize: 10 },
              },
            }}
            sortModel={[
              {
                field: 'createdAt',
                sort: 'desc',
              },
            ]}
            localeText={{
              toolbarColumns: "Colonnes",
              toolbarColumnsLabel: "Chercher",
              toolbarFilters: "Filtrer",
              toolbarExport: "Exporter",
              filterPanelOperator: 'Opérateurs',
              filterPanelColumns: 'Colonnes',
              filterPanelInputLabel: 'Valeur',
              filterPanelInputPlaceholder: 'Valeur du filtre',
              filterOperatorContains: 'Contient',
              filterOperatorEquals: 'Égal à',
              filterOperatorStartsWith: 'Commence par',
              filterOperatorEndsWith: 'Finit par',
              filterOperatorIsEmpty: 'Est vide',
              filterOperatorIsNotEmpty: 'N\'est pas vide',
              filterOperatorIsAnyOf: 'Est l\'un de',
              columnMenuLabel: 'Menu',
              columnMenuShowColumns: 'Afficher les colonnes',
              columnMenuManageColumns: 'Gérer les colonnes',
              columnMenuFilter: 'Filtrer',
              columnMenuHideColumn: 'Masquer la colonne',
              columnMenuUnsort: 'Annuler le tri',
              columnMenuSortAsc: 'Tri croissant',
              columnMenuSortDesc: 'Tri décroissant',
              columnsPanelTextFieldLabel: 'Trouver une colonne',
              columnsPanelTextFieldPlaceholder: 'Titre de la colonne',
              columnsPanelDragIconLabel: 'Réorganiser les colonnes',
              columnsPanelShowAllButton: 'Tout afficher',
              columnsPanelHideAllButton: 'Tout masquer',
            }}
            slots={{
              toolbar: () => (
                <GridToolbarContainer>
                  <GridToolbarFilterButton />
                  <GridToolbarExport />
                  <GridToolbarQuickFilter placeholder='Recherche...'/>
                  <Stack
                    spacing={1}
                    flexGrow={1}
                    direction="row"
                    alignItems="center"
                    justifyContent="flex-end"
                  ></Stack>
                </GridToolbarContainer>
              ),
            }}
            slotProps={{
              toolbar: {
                showQuickFilter: true,
                quickFilterProps: { debounceMs: 500 },
              },
            }}
            onRowSelectionModelChange={(newSelection) => {
              setSelectedRowIds(newSelection);
            }}
            rowSelectionModel={selectedRowIds}
          />
        </Card>

        <ConfirmDialog
          open={confirmDeleteOpen}
          onClose={() => {
            setConfirmDeleteOpen(false);
            confirmRows.onFalse();
          }}
          title="Confirmer la Suppression"
          content={`Êtes-vous sûr de vouloir supprimer ${selectedRowIds.length} bon(s) ?`}
          action={
            <Button variant="contained" color="error" onClick={handleDelete}>
              Supprimer
            </Button>
          }
        />

        <ConfirmDialog
          open={confirmDuplicateOpen}
          onClose={() => setConfirmDuplicateOpen(false)}
          title="Confirmer la Duplication"
          content="Êtes-vous sûr de vouloir dupliquer ce bon ?"
          action={
            <Button variant="contained" onClick={handleDuplicate}>
              Dupliquer
            </Button>
          }
        />
      </Container>
    </>
  );
}
