'use client';

import React, { useState } from 'react';
import { useFormik } from 'formik';
import * as Yup from 'yup';
import Typography from '@mui/material/Typography';
import FormControl from '@mui/material/FormControl';
import TextField, { FilledTextFieldProps, OutlinedTextFieldProps, StandardTextFieldProps, TextFieldVariants } from '@mui/material/TextField';
import Button from '@mui/material/Button';
import Checkbox from '@mui/material/Checkbox';
import FormControlLabel from '@mui/material/FormControlLabel';
import { paths } from '@/routes/paths';
import { useSettingsContext } from '@/components/settings';
import { Box, Card } from '@mui/material';
import CustomAlert from '@/shared/components/CustomAlert';
import { useRouter } from '@/hooks';
import { updateVoucher, useGetVoucherById } from '@/shared/api/voucher';
import { logger } from '@/utils/logger';
import { DateTimePicker, LocalizationProvider } from '@mui/x-date-pickers';
import { AdapterDayjs } from '@mui/x-date-pickers/AdapterDayjs';
import dayjs, { Dayjs } from 'dayjs';
import { IVoucher } from '@/shared/types/voucher';



interface VoucherFormValues {
  email: string;
  amount: number;
  legend: string;
  description: string;
  expirationDate: Dayjs | null;
}

export default function EditVoucher({ voucherId }: { voucherId: number }) {
  const settings = useSettingsContext();
  const router = useRouter();
  const [alert, setAlert] = useState<{ message: string; type: 'success' | 'error' } | null>(null);

  const { voucher, isLoading, isError } = useGetVoucherById(voucherId.toString());

  const validationSchema = Yup.object({
    email: Yup.string()
      .email("Format d'email invalide")
      .required("L'email est requis"),
    amount: Yup.number()
      .required("Le montant est requis")
      .min(1, "Le montant doit être d'au moins 1")
      .max(10000, "Le montant ne doit pas dépasser 10 000"),
    legend: Yup.string()
      .required("La légende est requise")
      .max(100, "La légende ne doit pas dépasser 100 caractères"),
    description: Yup.string()
      .max(500, "La description ne doit pas dépasser 500 caractères"),
    expirationDate: Yup.date()
      .required("La date d'expiration est requise")
      .min(new Date(), "La date d'expiration doit être future"),
  });

  const formik = useFormik<VoucherFormValues>({
    initialValues: {
      email: voucher?.email || '',
      amount: voucher?.amount || 1,
      legend: voucher?.legend || '',
      description: voucher?.description || '',
      expirationDate: voucher?.expirationDate ? dayjs(voucher.expirationDate) : null,
    },
    enableReinitialize: true,
    validationSchema,
    onSubmit: async (values) => {
      try {
        const formattedExpirationDate = values.expirationDate
          ? values.expirationDate.format('YYYY-MM-DDTHH:mm:ss')
          : null;

        const dataToSend: IVoucher = {
          email: values.email,
          amount: values.amount,
          legend: values.legend,
          description: values.description,
      
          expirationDate: formattedExpirationDate,
        };

        console.log('Données envoyées au backend:', dataToSend); // Log pour vérification

        const updatedVoucher = await updateVoucher(voucherId, dataToSend);
        logger('Voucher updated:', updatedVoucher);
        formik.resetForm();
        setAlert({
          message: "Bon d'achat mis à jour avec succès !",
          type: 'success',
        });

        router.push(paths.dashboard.admin.voucher.root);
      } catch (error) {
        console.error('Error updating voucher:', error); // Log d'erreur détaillé
        setAlert({
          message: "Erreur lors de la mise à jour du bon d'achat",
          type: 'error',
        });
      }
    },
  });

  if (isLoading) return <Typography>Chargement...</Typography>;
  if (isError) return <Typography>Erreur lors du chargement des données.</Typography>;
  if (!voucher) return <Typography>Bon d'achat non trouvé.</Typography>;

  return (
    <>
      {alert && (
        <CustomAlert
          message={alert.message}
          type={alert.type}
          onClose={() => setAlert(null)}
          style={{ top: '20px', right: '20px', zIndex: 3000 }}
        />
      )}

      <Card variant="outlined" sx={{ padding: 2, maxWidth: 550, ml: 0 }}>
        <Box sx={{ maxWidth: 500 }}>
          <form onSubmit={formik.handleSubmit}>
            <FormControl fullWidth>
              <Typography variant="h5" gutterBottom>
                Modifier le bon d'achat
              </Typography>

              <TextField
                label="Email"
                name="email"
                type="email"
                value={formik.values.email}
                onChange={formik.handleChange}
                onBlur={formik.handleBlur}
                error={formik.touched.email && Boolean(formik.errors.email)}
                helperText={formik.touched.email && formik.errors.email}
                fullWidth
                margin="normal"
              />

              <TextField
                label="Montant (€)"
                name="amount"
                type="number"
                value={formik.values.amount}
                onChange={formik.handleChange}
                onBlur={formik.handleBlur}
                error={formik.touched.amount && Boolean(formik.errors.amount)}
                helperText={formik.touched.amount && formik.errors.amount}
                fullWidth
                margin="normal"
                inputProps={{ min: 1 }}
              />

              <TextField
                label="Légende"
                name="legend"
                type="text"
                value={formik.values.legend}
                onChange={formik.handleChange}
                onBlur={formik.handleBlur}
                error={formik.touched.legend && Boolean(formik.errors.legend)}
                helperText={formik.touched.legend && formik.errors.legend}
                fullWidth
                margin="normal"
              />

              <LocalizationProvider dateAdapter={AdapterDayjs}>
                <DateTimePicker
                  label="Date d'expiration"
                  format="DD/MM/YYYY hh:mm a"
                  value={formik.values.expirationDate}
                  onChange={(value) => formik.setFieldValue('expirationDate', value)}
                  slotProps={{
                    textField: {
                      name: 'expirationDate',
                      onBlur: (event) => {
                        formik.handleBlur(event);
                      },
                      error:
                        formik.touched.expirationDate && Boolean(formik.errors.expirationDate),
                      helperText:
                        formik.touched.expirationDate && formik.errors.expirationDate,
                      fullWidth: true,
                      margin: 'normal',
                    },
                  }}
                />
              </LocalizationProvider>

              <TextField
                label="Description (facultative)"
                name="description"
                multiline
                rows={4}
                value={formik.values.description}
                onChange={formik.handleChange}
                onBlur={formik.handleBlur}
                error={formik.touched.description && Boolean(formik.errors.description)}
                helperText={formik.touched.description && formik.errors.description}
                fullWidth
                margin="normal"
              />

            

              <Button
                type="submit"
                variant="contained"
                color="primary"
                fullWidth
                style={{ marginTop: '20px' }}
              >
                Enregistrer les modifications
              </Button>
            </FormControl>
          </form>
        </Box>
      </Card>
    </>
  );
}
