import { PaymentMethodType } from "@/contexts/types/payment";

export type IPaymentMethod = {
    id: string;
    order: number;
    title: string;
    active: boolean;
    type: string;
    description: string;
}


export interface IBankAccount {
    id?: number;
    accountName: string;
    bankName: string;
    accountNumber: string;
    iban: string;
    swift: string;
    currency?: string;
    order: number;
}



export interface IBankTransferConfig extends IPaymentMethod {
    instructions: string;
    bankAccounts: IBankAccount[];
}

export interface ICreditCardConfig extends IPaymentMethod {
    logos: string[];
    disabledCreditCards: string[];
    vaulting: boolean;
    contingencyFor3DSecure: string;
}


export interface IPayPalConfig extends IPaymentMethod {
    intent: string;
    contingencyFor3DSecure: string;
}

export type IBankTransferConf = {
    enabled: boolean;
    title: string;
    description: string;
    instructions: string;
    bankAccounts: {
        name: string;
        number: string;
        bank: string;
        code: string;
        IBAN: string;
        bic_swift: string;
    }[];
}








//////////////////////paypal connexion//////////////////////

export enum ConnexionMode {
    LIVE = 'LIVE',
    SANDBOX = 'SANDBOX',
}


export type IPaypalConnexionConf = {
    mode: ConnexionMode;
    liveEmail: string;
    liveMerchantId?: string;
    liveClientId: string;
    liveSecretKey: string;
    sandboxEmail: string;
    sandboxMerchantId?: string;
    sandboxClientId: string;
    sandboxSecretKey: string;
    connected: boolean;
}






//////////////////////payplug configuration//////////////////////

export type IPayPlugConfig = IPaymentMethod & {
    force3ds: boolean;
}

export type IPayPlugPaymentConfig = {
    liveSecretKey: string;
    testSecretKey: string;
    payPlugMode: PayPlugMode;
}

export enum PayPlugMode {
    LIVE = 'LIVE',
    SANDBOX = 'SANDBOX',
}

export const PayPlugModeLabels = {
    [PayPlugMode.LIVE]: 'Live',
    [PayPlugMode.SANDBOX]: 'Test',
}


export type PaymentPayPlugResponse = {
    id: string;
    object: string;
    is_live: boolean;
    amount: number;
    amount_refunded: number;
    currency: string;
    created_at: number;
    description: string;
    is_paid: boolean;
    paid_at: number | null;
    is_refunded: boolean;
    is3ds: boolean | null;
    save_card: boolean;
    card: PaymentPayPlugResponse.CardDTO;
    hosted_payment: PaymentPayPlugResponse.HostedPaymentDTO;
    notification: PaymentPayPlugResponse.NotificationDTO;
    metadata: PaymentPayPlugResponse.MetadataDTO;
    failure: any; // Replace `any` with the actual type if known
    installment_plan_id: string | null;
    authorization: any; // Replace `any` with the actual type if known
    refundable_after: any; // Replace `any` with the actual type if known
    refundable_until: any; // Replace `any` with the actual type if known
    integration: any; // Replace `any` with the actual type if known
    customer: PaymentPayPlugResponse.CustomerDTO;
}

export namespace PaymentPayPlugResponse {
    export type CardDTO = {
        last4: string;
        country: string;
        exp_year: number;
        exp_month: number;
        brand: string;
        id: string;
        metadata: any;
    }

    export type HostedPaymentDTO = {
        payment_url: string;
        return_url: string;
        cancel_url: string;
        paid_at: number | null;
        sent_by: string;
    }

    export type NotificationDTO = {
        url: string;
        response_code: number;
    }

    export type MetadataDTO = {
        customer_id: number;
    }

    export type CustomerDTO = {
        first_name: string;
        last_name: string;
        address1: string;
        address2: string;
        postcode: string;
        city: string;
        country: string;
        email: string;
        phone_number: string;
        language: string;
    }
}

export type RefundPayPlugRequest = {
    amount: number;
    metadata: RefundMetadataRequest;
}

export type RefundMetadataRequest = {
    customer_id: string;
    reason: string;
}

export type RefundPayPlugResponse = {
    id: string;
    paymentId: string;
    isLive: boolean;
    amount: number;
    currency: string;
    createdAt: number;
    metadata: RefundMetadataRequest;
}

export function getPaymentMethodNameInFrench(paymentMethod: string): string {
    switch (paymentMethod) {
        case PaymentMethodType.CREDIT_CARD:
            return 'Carte de crédit';
        case PaymentMethodType.BANK_TRANSFER:
            return 'Virement bancaire';
        case PaymentMethodType.PAYPAL:
            return 'PayPal';
        case PaymentMethodType.PAYPLUG:
            return 'PayPlug';
        default:
            return 'Méthode de paiement inconnue';
    }
}
