// ----------------------------------------------------------------------

import { TypeOption } from "./attribut";

export type IProductFilterValue = string | string[] | number | number[];

export type IProductFilters = {
  rating: string;
  gender: string[];
  category: string;
  colors: string[];
  priceRange: number[];
};

// ----------------------------------------------------------------------

export type IProductReviewNewForm = {
  rating: number | null;
  review: string;
  name: string;
  email: string;
};

export type IProductReview = {
  id: string;
  name: string;
  rating: number;
  comment: string;
  helpful: number;
  avatarUrl: string;
  isPurchased: boolean;
  attachments?: string[];
  postedAt: Date;
};

export enum ProductType {
  SIMPLE = "SIMPLE",
  VARIABLE = "VARIABLE",
  GIFT_CARD = "GIFT_CARD"
}

export enum AuthorizePreOrder {
  AUTHORIZE = "AUTHORIZE",
  AUTHORIZE_WITH_CUSTOMER_NOTIFICATION = "AUTHORIZE_WITH_CUSTOMER_NOTIFICATION",
  DO_NOT_AUTHORIZE = "DO_NOT_AUTHORIZE",
}

export interface IProductItem {
  id?: number;
  createdAt: string;
  updatedAt: string;
  ugs: string;
  name: string;
  description?: string;
  shortDescription?: string;
  imagesIDs?: (number | string)[];
  mainImageID?: number | string;
  mainImagePath: number | string;
  imagesPaths: string[];
  productType: ProductType;
  draft: boolean;
  material?: string;
  categoryIds: number[];
  tagIds: number[];
  variations: IVariation[];
  idSupplier?: number;
  regularPrice: number;
  promoPrice?: number;
  purchasePrice?: number;
  stockAlert: number;
  publishWeb: boolean;
  publishLive: boolean;
  weight: number;
  lengthP: number;
  widthP: number;
  heightP: number;
  shippingClassId?: string;
  emplacement?: string;
  stockLiveConfig: stockLiveConfig;
  stockWebConfig: stockWebConfig;
  liveProductConfigs: LiveProductConfig;
  supplierName?: string;
  supplierRef?: string;
  supplierRefSecond? : string;
  supplierNameSecond? : string;
  secondPurchasePrice? : number;
  idPositionMap? : Record<number, number>;
}
export interface IProductFilterParams {
  category?: string;
  name?: string;
  priceMin?: number | null;
  priceMax?: number | null;
  page: number;
  size: number;
  sortBy: string[];
  direction: string[];
  isDraft?: boolean | null;
  isPublishLive?: boolean | null;
  isPublishWeb?: boolean | null;
  isInStockWeb?: boolean | null;
  isInStockLive?: boolean | null;
  keyword?: string;
  startDate?: string | null;
  endDate?: string | null;
}
export interface IProductPage {
  content: IProductListItem[];
  pageable: {
    sort: {
      empty: boolean;
      sorted: boolean;
      unsorted: boolean;
    },
    offset: number;
    pageNumber: number;
    pageSize: number;
    unpaged: boolean;
    paged: boolean;
  },
  totalPages: number,
  totalElements: number,
  last: boolean;
  sort: {
    empty: boolean;
    sorted: boolean;
    unsorted: boolean;
  };
}
export interface IVariation {
  id?: number;
  ugs: string;
  isNew?: boolean;
  name: string;
  image?: number;
  mainImagePath?: string;
  productId?: number;
  stockWebConfig: stockWebConfig;
  stockLiveConfig: stockLiveConfig;
  liveProductConfigs: LiveProductConfig;
  regularPrice: number;
  promoPrice?: number;
  purchasePrice?: number;
  stockAlert: number;
  publishWeb: boolean;
  publishLive: boolean;
  weight: number;
  lengthP: number;
  widthP: number;
  heightP: number;
  shippingClassId?: number;
  variationAttributes: {
    [x: string]: any;
    type: string | null;
    attributeId: number;
    termLabelId: number | null;
    termImageId: number | null;
    termColorId: number | null;
    termChoixId: number | null;
  }[];
  secondPurchasePrice? : number;

}

interface stockLiveConfig {
  quantity: number | null;
  isPreOrderAuthorized: AuthorizePreOrder;
  authorizedConfig: boolean;
  thresholdValue: number | null;
}

interface stockWebConfig {
  quantity: number | null;
  isPreOrderAuthorized: AuthorizePreOrder;
  authorizedConfig: boolean;
  thresholdValue: number | null;
}

interface LiveProductConfig {
  feedStockFromWebStock: boolean;
  authorizeUpdateQuantity: boolean;
}

export type IProductTableFilterValue = string | string[] | Date | null | boolean | number;

export type IProductTableFilters = {
  keyword: string;
  publish: string[];
  isInStockWeb: boolean | null;
  isInStockLive: boolean | null;
  isDraft: boolean | null;
  isPublishLive: boolean | null;
  isPublishWeb: boolean | null;
  priceMin: number | null;
  priceMax: number | null;
  startDate: string | null;
  endDate: string | null;
};
export interface IProductListItem {
  id: number;
  name: string;
  ugs: string;
  regularPrice: number;
  productType: ProductType;
  draft: boolean;
  createdAt: string;
  categoryNames: string[];
  tagNames: string[];
  stockWebQuantity: number;
  stockLiveQuantity: number;
  mainImagePath: string;
  mainImageID: number;
  imagesPaths: string[];
  description: string;
  shortDescription: string;
  publishWeb: boolean;
  publishLive: boolean;
  emplacement: string;
  purchasePrice: number;
  material: string;
  supplierName : string;
  supplierRef : string;
  priceRangeDisplay : string;
  minPrice : number;
  maxPrice : number;
}

export type ProductWithCustomResponse = {
  id: number;
  productName: string;
  ugs:string;
  price: number;
  type:ProductType,
  image:string;
  outOfStock: boolean;
};


/////////////////////// salesession type //////////////////////////////

// Enum pour le type d'attribut
export enum AttributeType {
  LABEL = "LABEL",
  COLOR = "COLOR",
  IMAGE = "IMAGE",
  CHOICE = "CHOICE",
}

// Interface pour les attributs de la variation
export interface IVariationAttribute {
  attributeName: string;
  type: AttributeType;
  attributeValue: string;
}

// Interface pour la configuration de stock
export interface IStockConfig {
  quantity: number;
  isPreOrderAuthorized:
    | "AUTHORIZE"
    | "AUTHORIZE_WITH_CUSTOMER_NOTIFICATION"
    | "DO_NOT_AUTHORIZE";
  authorizedConfig: boolean;
  thresholdValue: number;
}

// Interface pour les configurations en temps réel du produit (live)
export interface ILiveProductConfig {
  feedStockFromWebStock: boolean;
  authorizeUpdateQuantity: boolean;
}

// Interface pour la variation
export interface IVariationItem {
  id: number;
  ugs: string;
  name: string;
  image?: number | string; // l'image est stockée sous forme de numéro dans l'API
  regularPrice: number;
  purchasePrice?: number;
  attributes: IVariationAttribute[]; // Liste des attributs associés
  stockLiveConfig: IStockConfig;
  liveProductConfigs: ILiveProductConfig;
  productId: number; // ID du produit parent
  stockWebConfig: IStockConfig;
}

// Interface pour le produit principal
export interface IProductSessionItem {
  id: number;
  ugs: string;
  name: string;
  createdAt: string;
  mainImagePath: string;
  productType: ProductType;
  regularPrice: number;
  purchasePrice?: number;
  quantitySimpleProduct?: number;
  variations: IVariationItem[]; // Liste des variations si le produit est de type VARIABLE
  stockLiveConfig: IStockConfig; // Configuration de stock en direct (live)
  liveProductConfigs?: ILiveProductConfig | null; // Peut être null dans certaines réponses
  stockWebConfig: IStockConfig;
}

export interface IProductSessionSerchTerm {
  searchTerm : string;
}


/////////////////// Product Details /////////////////

export interface AttributeDetailDTO {
  attributeName: string;
  attributeValue: string;
}

export interface VariationDetailDTO {
  id: number;
  name: string;
  ugs: string;
  mainImagePath: string;
  regularPrice: number;
  stockWebQuantity: number;
  stockLiveQuantity: number;
  publishWeb: boolean;
  publishLive: boolean;
  attributes: AttributeDetailDTO[];
  stockLiveConfig: stockLiveConfig;
  stockWebConfig: stockWebConfig;
}

 export interface ProductDetailsDTO {
  id: number;
  name: string;
  ugs: string;
  regularPrice: number;
  productType: ProductType;
  createdAt: string;
  categoryNames: string[];
  tagNames: string[];
  stockWebQuantity: number;
  stockLiveQuantity: number;
  imagesPaths: string[];
  mainImagePath: string;
  description: string;
  shortDescription: string;
  publishWeb: boolean;
  publishLive: boolean;
  emplacement: string;
  purchasePrice: number;
  material: string;
  isDraft: boolean;
  variations: VariationDetailDTO[];
  supplierName: string;
  supplierRef: string;
  stockLiveConfig: stockLiveConfig;
  stockWebConfig: stockWebConfig;
}

